/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|*  * AArch64 Disassembler                                                    *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#include "llvm/MC/MCInst.h"
#include "llvm/MC/MCSubtargetInfo.h"
#include "llvm/Support/DataTypes.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/LEB128.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/TargetParser/SubtargetFeature.h"
#include <assert.h>

namespace {

// InsnBitWidth is essentially a type trait used by the decoder emitter to query
// the supported bitwidth for a given type. But default, the value is 0, making
// it an invalid type for use as `InsnType` when instantiating the decoder.
// Individual targets are expected to provide specializations for these based
// on their usage.
template <typename T> constexpr uint32_t InsnBitWidth = 0;

static const uint8_t DecoderTable32[94228] = {
  OPC_SwitchField, 26, 3,                    //     0: switch Inst[28:26] {
  0, 221, 134, 1,                            //     3: case 0x0: {
  OPC_SwitchField, 29, 3,                    //     7:  switch Inst[31:29] {
  0, 8,                                      //    10:  case 0x0: {
  OPC_CheckField, 16, 10, 0,                 //    12:   check Inst[25:16] == 0x0
  OPC_Decode, 168, 63, 0,                    //    16:   decode to UDF using decoder 0
                                             //    16:  }
  4, 132, 13,                                //    20:  case 0x4: {
  OPC_SwitchField, 21, 5,                    //    23:   switch Inst[25:21] {
  0, 147, 2,                                 //    26:   case 0x0: {
  OPC_SwitchField, 2, 4,                     //    29:    switch Inst[5:2] {
  0, 87,                                     //    32:    case 0x0: {
  OPC_SwitchField, 9, 8,                     //    34:     switch Inst[16:9] {
  0, 19,                                     //    37:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //    39:      switch Inst[20] {
  0, 6,                                      //    42:      case 0x0: {
  OPC_CheckPredicate, 0,                     //    44:       check predicate 0
  OPC_Decode, 170, 29, 1,                    //    46:       decode to FMOP4A_MZZ_S using decoder 1
                                             //    46:      }
  1, 0,                                      //    50:      case 0x1: {
  OPC_CheckPredicate, 0,                     //    52:       check predicate 0
  OPC_Decode, 164, 29, 2,                    //    54:       decode to FMOP4A_MZ2Z_S using decoder 2
                                             //    54:      }
                                             //    54:      } // switch Inst[20]
                                             //    54:     }
  1, 19,                                     //    58:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //    60:      switch Inst[20] {
  0, 6,                                      //    63:      case 0x0: {
  OPC_CheckPredicate, 0,                     //    65:       check predicate 0
  OPC_Decode, 158, 29, 3,                    //    67:       decode to FMOP4A_M2ZZ_S using decoder 3
                                             //    67:      }
  1, 0,                                      //    71:      case 0x1: {
  OPC_CheckPredicate, 0,                     //    73:       check predicate 0
  OPC_Decode, 152, 29, 4,                    //    75:       decode to FMOP4A_M2Z2Z_S using decoder 4
                                             //    75:      }
                                             //    75:      } // switch Inst[20]
                                             //    75:     }
  64, 19,                                    //    79:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //    81:      switch Inst[20] {
  0, 6,                                      //    84:      case 0x0: {
  OPC_CheckPredicate, 0,                     //    86:       check predicate 0
  OPC_Decode, 178, 50, 1,                    //    88:       decode to SMOP4A_MZZ_BToS using decoder 1
                                             //    88:      }
  1, 0,                                      //    92:      case 0x1: {
  OPC_CheckPredicate, 0,                     //    94:       check predicate 0
  OPC_Decode, 175, 50, 2,                    //    96:       decode to SMOP4A_MZ2Z_BToS using decoder 2
                                             //    96:      }
                                             //    96:      } // switch Inst[20]
                                             //    96:     }
  65, 0,                                     //   100:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //   102:      switch Inst[20] {
  0, 6,                                      //   105:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   107:       check predicate 0
  OPC_Decode, 172, 50, 3,                    //   109:       decode to SMOP4A_M2ZZ_BToS using decoder 3
                                             //   109:      }
  1, 0,                                      //   113:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   115:       check predicate 0
  OPC_Decode, 169, 50, 4,                    //   117:       decode to SMOP4A_M2Z2Z_BToS using decoder 4
                                             //   117:      }
                                             //   117:      } // switch Inst[20]
                                             //   117:     }
                                             //   117:     } // switch Inst[16:9]
                                             //   117:    }
  2, 45,                                     //   121:    case 0x2: {
  OPC_SwitchField, 9, 8,                     //   123:     switch Inst[16:9] {
  64, 19,                                    //   126:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //   128:      switch Inst[20] {
  0, 6,                                      //   131:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   133:       check predicate 0
  OPC_Decode, 179, 50, 1,                    //   135:       decode to SMOP4A_MZZ_HToS using decoder 1
                                             //   135:      }
  1, 0,                                      //   139:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   141:       check predicate 0
  OPC_Decode, 176, 50, 2,                    //   143:       decode to SMOP4A_MZ2Z_HToS using decoder 2
                                             //   143:      }
                                             //   143:      } // switch Inst[20]
                                             //   143:     }
  65, 0,                                     //   147:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //   149:      switch Inst[20] {
  0, 6,                                      //   152:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   154:       check predicate 0
  OPC_Decode, 173, 50, 3,                    //   156:       decode to SMOP4A_M2ZZ_HToS using decoder 3
                                             //   156:      }
  1, 0,                                      //   160:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   162:       check predicate 0
  OPC_Decode, 170, 50, 4,                    //   164:       decode to SMOP4A_M2Z2Z_HToS using decoder 4
                                             //   164:      }
                                             //   164:      } // switch Inst[20]
                                             //   164:     }
                                             //   164:     } // switch Inst[16:9]
                                             //   164:    }
  4, 87,                                     //   168:    case 0x4: {
  OPC_SwitchField, 9, 8,                     //   170:     switch Inst[16:9] {
  0, 19,                                     //   173:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   175:      switch Inst[20] {
  0, 6,                                      //   178:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   180:       check predicate 0
  OPC_Decode, 186, 29, 1,                    //   182:       decode to FMOP4S_MZZ_S using decoder 1
                                             //   182:      }
  1, 0,                                      //   186:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   188:       check predicate 0
  OPC_Decode, 182, 29, 2,                    //   190:       decode to FMOP4S_MZ2Z_S using decoder 2
                                             //   190:      }
                                             //   190:      } // switch Inst[20]
                                             //   190:     }
  1, 19,                                     //   194:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //   196:      switch Inst[20] {
  0, 6,                                      //   199:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   201:       check predicate 0
  OPC_Decode, 178, 29, 3,                    //   203:       decode to FMOP4S_M2ZZ_S using decoder 3
                                             //   203:      }
  1, 0,                                      //   207:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   209:       check predicate 0
  OPC_Decode, 174, 29, 4,                    //   211:       decode to FMOP4S_M2Z2Z_S using decoder 4
                                             //   211:      }
                                             //   211:      } // switch Inst[20]
                                             //   211:     }
  64, 19,                                    //   215:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //   217:      switch Inst[20] {
  0, 6,                                      //   220:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   222:       check predicate 0
  OPC_Decode, 190, 50, 1,                    //   224:       decode to SMOP4S_MZZ_BToS using decoder 1
                                             //   224:      }
  1, 0,                                      //   228:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   230:       check predicate 0
  OPC_Decode, 187, 50, 2,                    //   232:       decode to SMOP4S_MZ2Z_BToS using decoder 2
                                             //   232:      }
                                             //   232:      } // switch Inst[20]
                                             //   232:     }
  65, 0,                                     //   236:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //   238:      switch Inst[20] {
  0, 6,                                      //   241:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   243:       check predicate 0
  OPC_Decode, 184, 50, 3,                    //   245:       decode to SMOP4S_M2ZZ_BToS using decoder 3
                                             //   245:      }
  1, 0,                                      //   249:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   251:       check predicate 0
  OPC_Decode, 181, 50, 4,                    //   253:       decode to SMOP4S_M2Z2Z_BToS using decoder 4
                                             //   253:      }
                                             //   253:      } // switch Inst[20]
                                             //   253:     }
                                             //   253:     } // switch Inst[16:9]
                                             //   253:    }
  6, 0,                                      //   257:    case 0x6: {
  OPC_SwitchField, 9, 8,                     //   259:     switch Inst[16:9] {
  64, 19,                                    //   262:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //   264:      switch Inst[20] {
  0, 6,                                      //   267:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   269:       check predicate 0
  OPC_Decode, 191, 50, 1,                    //   271:       decode to SMOP4S_MZZ_HToS using decoder 1
                                             //   271:      }
  1, 0,                                      //   275:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   277:       check predicate 0
  OPC_Decode, 188, 50, 2,                    //   279:       decode to SMOP4S_MZ2Z_HToS using decoder 2
                                             //   279:      }
                                             //   279:      } // switch Inst[20]
                                             //   279:     }
  65, 0,                                     //   283:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //   285:      switch Inst[20] {
  0, 6,                                      //   288:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   290:       check predicate 0
  OPC_Decode, 185, 50, 3,                    //   292:       decode to SMOP4S_M2ZZ_HToS using decoder 3
                                             //   292:      }
  1, 0,                                      //   296:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   298:       check predicate 0
  OPC_Decode, 182, 50, 4,                    //   300:       decode to SMOP4S_M2Z2Z_HToS using decoder 4
                                             //   300:      }
                                             //   300:      } // switch Inst[20]
                                             //   300:     }
                                             //   300:     } // switch Inst[16:9]
                                             //   300:    }
                                             //   300:    } // switch Inst[5:2]
                                             //   300:   }
  1, 207, 1,                                 //   304:   case 0x1: {
  OPC_SwitchField, 9, 8,                     //   307:    switch Inst[16:9] {
  0, 53,                                     //   310:    case 0x0: {
  OPC_SwitchField, 2, 4,                     //   312:     switch Inst[5:2] {
  0, 19,                                     //   315:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   317:      switch Inst[20] {
  0, 6,                                      //   320:      case 0x0: {
  OPC_CheckPredicate, 1,                     //   322:       check predicate 1
  OPC_Decode, 166, 29, 1,                    //   324:       decode to FMOP4A_MZZ_BtoS using decoder 1
                                             //   324:      }
  1, 0,                                      //   328:      case 0x1: {
  OPC_CheckPredicate, 1,                     //   330:       check predicate 1
  OPC_Decode, 160, 29, 2,                    //   332:       decode to FMOP4A_MZ2Z_BtoS using decoder 2
                                             //   332:      }
                                             //   332:      } // switch Inst[20]
                                             //   332:     }
  2, 0,                                      //   336:     case 0x2: {
  OPC_SwitchField, 20, 1,                    //   338:      switch Inst[20] {
  0, 10,                                     //   341:      case 0x0: {
  OPC_CheckPredicate, 2,                     //   343:       check predicate 2
  OPC_CheckField, 1, 1, 0,                   //   345:       check Inst[1] == 0x0
  OPC_Decode, 165, 29, 5,                    //   349:       decode to FMOP4A_MZZ_BtoH using decoder 5
                                             //   349:      }
  1, 0,                                      //   353:      case 0x1: {
  OPC_CheckPredicate, 2,                     //   355:       check predicate 2
  OPC_CheckField, 1, 1, 0,                   //   357:       check Inst[1] == 0x0
  OPC_Decode, 159, 29, 6,                    //   361:       decode to FMOP4A_MZ2Z_BtoH using decoder 6
                                             //   361:      }
                                             //   361:      } // switch Inst[20]
                                             //   361:     }
                                             //   361:     } // switch Inst[5:2]
                                             //   361:    }
  1, 53,                                     //   365:    case 0x1: {
  OPC_SwitchField, 2, 4,                     //   367:     switch Inst[5:2] {
  0, 19,                                     //   370:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   372:      switch Inst[20] {
  0, 6,                                      //   375:      case 0x0: {
  OPC_CheckPredicate, 1,                     //   377:       check predicate 1
  OPC_Decode, 154, 29, 3,                    //   379:       decode to FMOP4A_M2ZZ_BtoS using decoder 3
                                             //   379:      }
  1, 0,                                      //   383:      case 0x1: {
  OPC_CheckPredicate, 1,                     //   385:       check predicate 1
  OPC_Decode, 148, 29, 4,                    //   387:       decode to FMOP4A_M2Z2Z_BtoS using decoder 4
                                             //   387:      }
                                             //   387:      } // switch Inst[20]
                                             //   387:     }
  2, 0,                                      //   391:     case 0x2: {
  OPC_SwitchField, 20, 1,                    //   393:      switch Inst[20] {
  0, 10,                                     //   396:      case 0x0: {
  OPC_CheckPredicate, 2,                     //   398:       check predicate 2
  OPC_CheckField, 1, 1, 0,                   //   400:       check Inst[1] == 0x0
  OPC_Decode, 153, 29, 7,                    //   404:       decode to FMOP4A_M2ZZ_BtoH using decoder 7
                                             //   404:      }
  1, 0,                                      //   408:      case 0x1: {
  OPC_CheckPredicate, 2,                     //   410:       check predicate 2
  OPC_CheckField, 1, 1, 0,                   //   412:       check Inst[1] == 0x0
  OPC_Decode, 147, 29, 8,                    //   416:       decode to FMOP4A_M2Z2Z_BtoH using decoder 8
                                             //   416:      }
                                             //   416:      } // switch Inst[20]
                                             //   416:     }
                                             //   416:     } // switch Inst[5:2]
                                             //   416:    }
  64, 45,                                    //   420:    case 0x40: {
  OPC_SwitchField, 2, 4,                     //   422:     switch Inst[5:2] {
  0, 19,                                     //   425:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   427:      switch Inst[20] {
  0, 6,                                      //   430:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   432:       check predicate 0
  OPC_Decode, 218, 60, 1,                    //   434:       decode to SUMOP4A_MZZ_BToS using decoder 1
                                             //   434:      }
  1, 0,                                      //   438:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   440:       check predicate 0
  OPC_Decode, 216, 60, 2,                    //   442:       decode to SUMOP4A_MZ2Z_BToS using decoder 2
                                             //   442:      }
                                             //   442:      } // switch Inst[20]
                                             //   442:     }
  4, 0,                                      //   446:     case 0x4: {
  OPC_SwitchField, 20, 1,                    //   448:      switch Inst[20] {
  0, 6,                                      //   451:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   453:       check predicate 0
  OPC_Decode, 226, 60, 1,                    //   455:       decode to SUMOP4S_MZZ_BToS using decoder 1
                                             //   455:      }
  1, 0,                                      //   459:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   461:       check predicate 0
  OPC_Decode, 224, 60, 2,                    //   463:       decode to SUMOP4S_MZ2Z_BToS using decoder 2
                                             //   463:      }
                                             //   463:      } // switch Inst[20]
                                             //   463:     }
                                             //   463:     } // switch Inst[5:2]
                                             //   463:    }
  65, 0,                                     //   467:    case 0x41: {
  OPC_SwitchField, 2, 4,                     //   469:     switch Inst[5:2] {
  0, 19,                                     //   472:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   474:      switch Inst[20] {
  0, 6,                                      //   477:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   479:       check predicate 0
  OPC_Decode, 214, 60, 3,                    //   481:       decode to SUMOP4A_M2ZZ_BToS using decoder 3
                                             //   481:      }
  1, 0,                                      //   485:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   487:       check predicate 0
  OPC_Decode, 212, 60, 4,                    //   489:       decode to SUMOP4A_M2Z2Z_BToS using decoder 4
                                             //   489:      }
                                             //   489:      } // switch Inst[20]
                                             //   489:     }
  4, 0,                                      //   493:     case 0x4: {
  OPC_SwitchField, 20, 1,                    //   495:      switch Inst[20] {
  0, 6,                                      //   498:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   500:       check predicate 0
  OPC_Decode, 222, 60, 3,                    //   502:       decode to SUMOP4S_M2ZZ_BToS using decoder 3
                                             //   502:      }
  1, 0,                                      //   506:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   508:       check predicate 0
  OPC_Decode, 220, 60, 4,                    //   510:       decode to SUMOP4S_M2Z2Z_BToS using decoder 4
                                             //   510:      }
                                             //   510:      } // switch Inst[20]
                                             //   510:     }
                                             //   510:     } // switch Inst[5:2]
                                             //   510:    }
                                             //   510:    } // switch Inst[16:9]
                                             //   510:   }
  2, 36,                                     //   514:   case 0x2: {
  OPC_SwitchField, 2, 2,                     //   516:    switch Inst[3:2] {
  0, 19,                                     //   519:    case 0x0: {
  OPC_SwitchField, 13, 3,                    //   521:     switch Inst[15:13] {
  0, 6,                                      //   524:     case 0x0: {
  OPC_CheckPredicate, 3,                     //   526:      check predicate 3
  OPC_Decode, 199, 32, 9,                    //   528:      decode to FTMOPA_M2ZZZI_StoS using decoder 9
                                             //   528:     }
  4, 0,                                      //   532:     case 0x4: {
  OPC_CheckPredicate, 3,                     //   534:      check predicate 3
  OPC_Decode, 225, 58, 9,                    //   536:      decode to STMOPA_M2ZZZI_BtoS using decoder 9
                                             //   536:     }
                                             //   536:     } // switch Inst[15:13]
                                             //   536:    }
  2, 0,                                      //   540:    case 0x2: {
  OPC_CheckPredicate, 3,                     //   542:     check predicate 3
  OPC_CheckField, 13, 3, 4,                  //   544:     check Inst[15:13] == 0x4
  OPC_Decode, 226, 58, 9,                    //   548:     decode to STMOPA_M2ZZZI_HtoS using decoder 9
                                             //   548:    }
                                             //   548:    } // switch Inst[3:2]
                                             //   548:   }
  3, 40,                                     //   552:   case 0x3: {
  OPC_SwitchField, 2, 2,                     //   554:    switch Inst[3:2] {
  0, 19,                                     //   557:    case 0x0: {
  OPC_SwitchField, 13, 3,                    //   559:     switch Inst[15:13] {
  0, 6,                                      //   562:     case 0x0: {
  OPC_CheckPredicate, 4,                     //   564:      check predicate 4
  OPC_Decode, 196, 32, 9,                    //   566:      decode to FTMOPA_M2ZZZI_BtoS using decoder 9
                                             //   566:     }
  4, 0,                                      //   570:     case 0x4: {
  OPC_CheckPredicate, 3,                     //   572:      check predicate 3
  OPC_Decode, 131, 61, 9,                    //   574:      decode to SUTMOPA_M2ZZZI_BtoS using decoder 9
                                             //   574:     }
                                             //   574:     } // switch Inst[15:13]
                                             //   574:    }
  2, 0,                                      //   578:    case 0x2: {
  OPC_CheckPredicate, 5,                     //   580:     check predicate 5
  OPC_CheckField, 13, 3, 0,                  //   582:     check Inst[15:13] == 0x0
  OPC_CheckField, 1, 1, 0,                   //   586:     check Inst[1] == 0x0
  OPC_Decode, 195, 32, 10,                   //   590:     decode to FTMOPA_M2ZZZI_BtoH using decoder 10
                                             //   590:    }
                                             //   590:    } // switch Inst[3:2]
                                             //   590:   }
  4, 35,                                     //   594:   case 0x4: {
  OPC_SwitchField, 2, 3,                     //   596:    switch Inst[4:2] {
  0, 6,                                      //   599:    case 0x0: {
  OPC_CheckPredicate, 6,                     //   601:     check predicate 6
  OPC_Decode, 192, 29, 11,                   //   603:     decode to FMOPA_MPPZZ_S using decoder 11
                                             //   603:    }
  2, 6,                                      //   607:    case 0x2: {
  OPC_CheckPredicate, 7,                     //   609:     check predicate 7
  OPC_Decode, 160, 16, 11,                   //   611:     decode to BMOPA_MPPZZ_S using decoder 11
                                             //   611:    }
  4, 6,                                      //   615:    case 0x4: {
  OPC_CheckPredicate, 6,                     //   617:     check predicate 6
  OPC_Decode, 196, 29, 11,                   //   619:     decode to FMOPS_MPPZZ_S using decoder 11
                                             //   619:    }
  6, 0,                                      //   623:    case 0x6: {
  OPC_CheckPredicate, 7,                     //   625:     check predicate 7
  OPC_Decode, 161, 16, 11,                   //   627:     decode to BMOPS_MPPZZ_S using decoder 11
                                             //   627:    }
                                             //   627:    } // switch Inst[4:2]
                                             //   627:   }
  5, 23,                                     //   631:   case 0x5: {
  OPC_SwitchField, 2, 3,                     //   633:    switch Inst[4:2] {
  0, 6,                                      //   636:    case 0x0: {
  OPC_CheckPredicate, 8,                     //   638:     check predicate 8
  OPC_Decode, 189, 29, 11,                   //   640:     decode to FMOPA_MPPZZ_BtoS using decoder 11
                                             //   640:    }
  2, 0,                                      //   644:    case 0x2: {
  OPC_CheckPredicate, 9,                     //   646:     check predicate 9
  OPC_CheckField, 1, 1, 0,                   //   648:     check Inst[1] == 0x0
  OPC_Decode, 188, 29, 12,                   //   652:     decode to FMOPA_MPPZZ_BtoH using decoder 12
                                             //   652:    }
                                             //   652:    } // switch Inst[4:2]
                                             //   652:   }
  6, 145, 1,                                 //   656:   case 0x6: {
  OPC_SwitchField, 3, 2,                     //   659:    switch Inst[4:3] {
  0, 6,                                      //   662:    case 0x0: {
  OPC_CheckPredicate, 10,                    //   664:     check predicate 10
  OPC_Decode, 190, 29, 13,                   //   666:     decode to FMOPA_MPPZZ_D using decoder 13
                                             //   666:    }
  1, 61,                                     //   670:    case 0x1: {
  OPC_SwitchField, 9, 8,                     //   672:     switch Inst[16:9] {
  0, 27,                                     //   675:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   677:      switch Inst[20] {
  0, 10,                                     //   680:      case 0x0: {
  OPC_CheckPredicate, 11,                    //   682:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   684:       check Inst[5] == 0x0
  OPC_Decode, 167, 29, 14,                   //   688:       decode to FMOP4A_MZZ_D using decoder 14
                                             //   688:      }
  1, 0,                                      //   692:      case 0x1: {
  OPC_CheckPredicate, 11,                    //   694:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   696:       check Inst[5] == 0x0
  OPC_Decode, 161, 29, 15,                   //   700:       decode to FMOP4A_MZ2Z_D using decoder 15
                                             //   700:      }
                                             //   700:      } // switch Inst[20]
                                             //   700:     }
  1, 0,                                      //   704:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //   706:      switch Inst[20] {
  0, 10,                                     //   709:      case 0x0: {
  OPC_CheckPredicate, 11,                    //   711:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   713:       check Inst[5] == 0x0
  OPC_Decode, 155, 29, 16,                   //   717:       decode to FMOP4A_M2ZZ_D using decoder 16
                                             //   717:      }
  1, 0,                                      //   721:      case 0x1: {
  OPC_CheckPredicate, 11,                    //   723:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   725:       check Inst[5] == 0x0
  OPC_Decode, 149, 29, 17,                   //   729:       decode to FMOP4A_M2Z2Z_D using decoder 17
                                             //   729:      }
                                             //   729:      } // switch Inst[20]
                                             //   729:     }
                                             //   729:     } // switch Inst[16:9]
                                             //   729:    }
  2, 6,                                      //   733:    case 0x2: {
  OPC_CheckPredicate, 10,                    //   735:     check predicate 10
  OPC_Decode, 194, 29, 13,                   //   737:     decode to FMOPS_MPPZZ_D using decoder 13
                                             //   737:    }
  3, 0,                                      //   741:    case 0x3: {
  OPC_SwitchField, 9, 8,                     //   743:     switch Inst[16:9] {
  0, 27,                                     //   746:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   748:      switch Inst[20] {
  0, 10,                                     //   751:      case 0x0: {
  OPC_CheckPredicate, 11,                    //   753:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   755:       check Inst[5] == 0x0
  OPC_Decode, 183, 29, 14,                   //   759:       decode to FMOP4S_MZZ_D using decoder 14
                                             //   759:      }
  1, 0,                                      //   763:      case 0x1: {
  OPC_CheckPredicate, 11,                    //   765:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   767:       check Inst[5] == 0x0
  OPC_Decode, 179, 29, 15,                   //   771:       decode to FMOP4S_MZ2Z_D using decoder 15
                                             //   771:      }
                                             //   771:      } // switch Inst[20]
                                             //   771:     }
  1, 0,                                      //   775:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //   777:      switch Inst[20] {
  0, 10,                                     //   780:      case 0x0: {
  OPC_CheckPredicate, 11,                    //   782:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   784:       check Inst[5] == 0x0
  OPC_Decode, 175, 29, 16,                   //   788:       decode to FMOP4S_M2ZZ_D using decoder 16
                                             //   788:      }
  1, 0,                                      //   792:      case 0x1: {
  OPC_CheckPredicate, 11,                    //   794:       check predicate 11
  OPC_CheckField, 5, 1, 0,                   //   796:       check Inst[5] == 0x0
  OPC_Decode, 171, 29, 17,                   //   800:       decode to FMOP4S_M2Z2Z_D using decoder 17
                                             //   800:      }
                                             //   800:      } // switch Inst[20]
                                             //   800:     }
                                             //   800:     } // switch Inst[16:9]
                                             //   800:    }
                                             //   800:    } // switch Inst[4:3]
                                             //   800:   }
  8, 135, 3,                                 //   804:   case 0x8: {
  OPC_SwitchField, 2, 4,                     //   807:    switch Inst[5:2] {
  0, 87,                                     //   810:    case 0x0: {
  OPC_SwitchField, 9, 8,                     //   812:     switch Inst[16:9] {
  0, 19,                                     //   815:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   817:      switch Inst[20] {
  0, 6,                                      //   820:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   822:       check predicate 0
  OPC_Decode, 223, 15, 1,                    //   824:       decode to BFMOP4A_MZZ_S using decoder 1
                                             //   824:      }
  1, 0,                                      //   828:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   830:       check predicate 0
  OPC_Decode, 221, 15, 2,                    //   832:       decode to BFMOP4A_MZ2Z_S using decoder 2
                                             //   832:      }
                                             //   832:      } // switch Inst[20]
                                             //   832:     }
  1, 19,                                     //   836:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //   838:      switch Inst[20] {
  0, 6,                                      //   841:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   843:       check predicate 0
  OPC_Decode, 219, 15, 3,                    //   845:       decode to BFMOP4A_M2ZZ_S using decoder 3
                                             //   845:      }
  1, 0,                                      //   849:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   851:       check predicate 0
  OPC_Decode, 217, 15, 4,                    //   853:       decode to BFMOP4A_M2Z2Z_S using decoder 4
                                             //   853:      }
                                             //   853:      } // switch Inst[20]
                                             //   853:     }
  64, 19,                                    //   857:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //   859:      switch Inst[20] {
  0, 6,                                      //   862:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   864:       check predicate 0
  OPC_Decode, 199, 68, 1,                    //   866:       decode to USMOP4A_MZZ_BToS using decoder 1
                                             //   866:      }
  1, 0,                                      //   870:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   872:       check predicate 0
  OPC_Decode, 197, 68, 2,                    //   874:       decode to USMOP4A_MZ2Z_BToS using decoder 2
                                             //   874:      }
                                             //   874:      } // switch Inst[20]
                                             //   874:     }
  65, 0,                                     //   878:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //   880:      switch Inst[20] {
  0, 6,                                      //   883:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   885:       check predicate 0
  OPC_Decode, 195, 68, 3,                    //   887:       decode to USMOP4A_M2ZZ_BToS using decoder 3
                                             //   887:      }
  1, 0,                                      //   891:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   893:       check predicate 0
  OPC_Decode, 193, 68, 4,                    //   895:       decode to USMOP4A_M2Z2Z_BToS using decoder 4
                                             //   895:      }
                                             //   895:      } // switch Inst[20]
                                             //   895:     }
                                             //   895:     } // switch Inst[16:9]
                                             //   895:    }
  2, 103,                                    //   899:    case 0x2: {
  OPC_SwitchField, 9, 8,                     //   901:     switch Inst[16:9] {
  0, 27,                                     //   904:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //   906:      switch Inst[20] {
  0, 10,                                     //   909:      case 0x0: {
  OPC_CheckPredicate, 12,                    //   911:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //   913:       check Inst[1] == 0x0
  OPC_Decode, 168, 29, 5,                    //   917:       decode to FMOP4A_MZZ_H using decoder 5
                                             //   917:      }
  1, 0,                                      //   921:      case 0x1: {
  OPC_CheckPredicate, 12,                    //   923:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //   925:       check Inst[1] == 0x0
  OPC_Decode, 162, 29, 6,                    //   929:       decode to FMOP4A_MZ2Z_H using decoder 6
                                             //   929:      }
                                             //   929:      } // switch Inst[20]
                                             //   929:     }
  1, 27,                                     //   933:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //   935:      switch Inst[20] {
  0, 10,                                     //   938:      case 0x0: {
  OPC_CheckPredicate, 12,                    //   940:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //   942:       check Inst[1] == 0x0
  OPC_Decode, 156, 29, 7,                    //   946:       decode to FMOP4A_M2ZZ_H using decoder 7
                                             //   946:      }
  1, 0,                                      //   950:      case 0x1: {
  OPC_CheckPredicate, 12,                    //   952:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //   954:       check Inst[1] == 0x0
  OPC_Decode, 150, 29, 8,                    //   958:       decode to FMOP4A_M2Z2Z_H using decoder 8
                                             //   958:      }
                                             //   958:      } // switch Inst[20]
                                             //   958:     }
  64, 19,                                    //   962:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //   964:      switch Inst[20] {
  0, 6,                                      //   967:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   969:       check predicate 0
  OPC_Decode, 188, 65, 1,                    //   971:       decode to UMOP4A_MZZ_HToS using decoder 1
                                             //   971:      }
  1, 0,                                      //   975:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   977:       check predicate 0
  OPC_Decode, 185, 65, 2,                    //   979:       decode to UMOP4A_MZ2Z_HToS using decoder 2
                                             //   979:      }
                                             //   979:      } // switch Inst[20]
                                             //   979:     }
  65, 0,                                     //   983:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //   985:      switch Inst[20] {
  0, 6,                                      //   988:      case 0x0: {
  OPC_CheckPredicate, 0,                     //   990:       check predicate 0
  OPC_Decode, 182, 65, 3,                    //   992:       decode to UMOP4A_M2ZZ_HToS using decoder 3
                                             //   992:      }
  1, 0,                                      //   996:      case 0x1: {
  OPC_CheckPredicate, 0,                     //   998:       check predicate 0
  OPC_Decode, 179, 65, 4,                    //  1000:       decode to UMOP4A_M2Z2Z_HToS using decoder 4
                                             //  1000:      }
                                             //  1000:      } // switch Inst[20]
                                             //  1000:     }
                                             //  1000:     } // switch Inst[16:9]
                                             //  1000:    }
  4, 87,                                     //  1004:    case 0x4: {
  OPC_SwitchField, 9, 8,                     //  1006:     switch Inst[16:9] {
  0, 19,                                     //  1009:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  1011:      switch Inst[20] {
  0, 6,                                      //  1014:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1016:       check predicate 0
  OPC_Decode, 231, 15, 1,                    //  1018:       decode to BFMOP4S_MZZ_S using decoder 1
                                             //  1018:      }
  1, 0,                                      //  1022:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1024:       check predicate 0
  OPC_Decode, 229, 15, 2,                    //  1026:       decode to BFMOP4S_MZ2Z_S using decoder 2
                                             //  1026:      }
                                             //  1026:      } // switch Inst[20]
                                             //  1026:     }
  1, 19,                                     //  1030:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  1032:      switch Inst[20] {
  0, 6,                                      //  1035:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1037:       check predicate 0
  OPC_Decode, 227, 15, 3,                    //  1039:       decode to BFMOP4S_M2ZZ_S using decoder 3
                                             //  1039:      }
  1, 0,                                      //  1043:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1045:       check predicate 0
  OPC_Decode, 225, 15, 4,                    //  1047:       decode to BFMOP4S_M2Z2Z_S using decoder 4
                                             //  1047:      }
                                             //  1047:      } // switch Inst[20]
                                             //  1047:     }
  64, 19,                                    //  1051:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //  1053:      switch Inst[20] {
  0, 6,                                      //  1056:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1058:       check predicate 0
  OPC_Decode, 207, 68, 1,                    //  1060:       decode to USMOP4S_MZZ_BToS using decoder 1
                                             //  1060:      }
  1, 0,                                      //  1064:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1066:       check predicate 0
  OPC_Decode, 205, 68, 2,                    //  1068:       decode to USMOP4S_MZ2Z_BToS using decoder 2
                                             //  1068:      }
                                             //  1068:      } // switch Inst[20]
                                             //  1068:     }
  65, 0,                                     //  1072:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //  1074:      switch Inst[20] {
  0, 6,                                      //  1077:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1079:       check predicate 0
  OPC_Decode, 203, 68, 3,                    //  1081:       decode to USMOP4S_M2ZZ_BToS using decoder 3
                                             //  1081:      }
  1, 0,                                      //  1085:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1087:       check predicate 0
  OPC_Decode, 201, 68, 4,                    //  1089:       decode to USMOP4S_M2Z2Z_BToS using decoder 4
                                             //  1089:      }
                                             //  1089:      } // switch Inst[20]
                                             //  1089:     }
                                             //  1089:     } // switch Inst[16:9]
                                             //  1089:    }
  6, 0,                                      //  1093:    case 0x6: {
  OPC_SwitchField, 9, 8,                     //  1095:     switch Inst[16:9] {
  0, 27,                                     //  1098:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  1100:      switch Inst[20] {
  0, 10,                                     //  1103:      case 0x0: {
  OPC_CheckPredicate, 12,                    //  1105:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //  1107:       check Inst[1] == 0x0
  OPC_Decode, 184, 29, 5,                    //  1111:       decode to FMOP4S_MZZ_H using decoder 5
                                             //  1111:      }
  1, 0,                                      //  1115:      case 0x1: {
  OPC_CheckPredicate, 12,                    //  1117:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //  1119:       check Inst[1] == 0x0
  OPC_Decode, 180, 29, 6,                    //  1123:       decode to FMOP4S_MZ2Z_H using decoder 6
                                             //  1123:      }
                                             //  1123:      } // switch Inst[20]
                                             //  1123:     }
  1, 27,                                     //  1127:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  1129:      switch Inst[20] {
  0, 10,                                     //  1132:      case 0x0: {
  OPC_CheckPredicate, 12,                    //  1134:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //  1136:       check Inst[1] == 0x0
  OPC_Decode, 176, 29, 7,                    //  1140:       decode to FMOP4S_M2ZZ_H using decoder 7
                                             //  1140:      }
  1, 0,                                      //  1144:      case 0x1: {
  OPC_CheckPredicate, 12,                    //  1146:       check predicate 12
  OPC_CheckField, 1, 1, 0,                   //  1148:       check Inst[1] == 0x0
  OPC_Decode, 172, 29, 8,                    //  1152:       decode to FMOP4S_M2Z2Z_H using decoder 8
                                             //  1152:      }
                                             //  1152:      } // switch Inst[20]
                                             //  1152:     }
  64, 19,                                    //  1156:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //  1158:      switch Inst[20] {
  0, 6,                                      //  1161:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1163:       check predicate 0
  OPC_Decode, 200, 65, 1,                    //  1165:       decode to UMOP4S_MZZ_HToS using decoder 1
                                             //  1165:      }
  1, 0,                                      //  1169:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1171:       check predicate 0
  OPC_Decode, 197, 65, 2,                    //  1173:       decode to UMOP4S_MZ2Z_HToS using decoder 2
                                             //  1173:      }
                                             //  1173:      } // switch Inst[20]
                                             //  1173:     }
  65, 0,                                     //  1177:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //  1179:      switch Inst[20] {
  0, 6,                                      //  1182:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1184:       check predicate 0
  OPC_Decode, 194, 65, 3,                    //  1186:       decode to UMOP4S_M2ZZ_HToS using decoder 3
                                             //  1186:      }
  1, 0,                                      //  1190:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1192:       check predicate 0
  OPC_Decode, 191, 65, 4,                    //  1194:       decode to UMOP4S_M2Z2Z_HToS using decoder 4
                                             //  1194:      }
                                             //  1194:      } // switch Inst[20]
                                             //  1194:     }
                                             //  1194:     } // switch Inst[16:9]
                                             //  1194:    }
                                             //  1194:    } // switch Inst[5:2]
                                             //  1194:   }
  9, 179, 2,                                 //  1198:   case 0x9: {
  OPC_SwitchField, 2, 4,                     //  1201:    switch Inst[5:2] {
  0, 87,                                     //  1204:    case 0x0: {
  OPC_SwitchField, 9, 8,                     //  1206:     switch Inst[16:9] {
  0, 19,                                     //  1209:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  1211:      switch Inst[20] {
  0, 6,                                      //  1214:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1216:       check predicate 0
  OPC_Decode, 169, 29, 1,                    //  1218:       decode to FMOP4A_MZZ_HtoS using decoder 1
                                             //  1218:      }
  1, 0,                                      //  1222:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1224:       check predicate 0
  OPC_Decode, 163, 29, 2,                    //  1226:       decode to FMOP4A_MZ2Z_HtoS using decoder 2
                                             //  1226:      }
                                             //  1226:      } // switch Inst[20]
                                             //  1226:     }
  1, 19,                                     //  1230:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  1232:      switch Inst[20] {
  0, 6,                                      //  1235:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1237:       check predicate 0
  OPC_Decode, 157, 29, 3,                    //  1239:       decode to FMOP4A_M2ZZ_HtoS using decoder 3
                                             //  1239:      }
  1, 0,                                      //  1243:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1245:       check predicate 0
  OPC_Decode, 151, 29, 4,                    //  1247:       decode to FMOP4A_M2Z2Z_HtoS using decoder 4
                                             //  1247:      }
                                             //  1247:      } // switch Inst[20]
                                             //  1247:     }
  64, 19,                                    //  1251:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //  1253:      switch Inst[20] {
  0, 6,                                      //  1256:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1258:       check predicate 0
  OPC_Decode, 187, 65, 1,                    //  1260:       decode to UMOP4A_MZZ_BToS using decoder 1
                                             //  1260:      }
  1, 0,                                      //  1264:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1266:       check predicate 0
  OPC_Decode, 184, 65, 2,                    //  1268:       decode to UMOP4A_MZ2Z_BToS using decoder 2
                                             //  1268:      }
                                             //  1268:      } // switch Inst[20]
                                             //  1268:     }
  65, 0,                                     //  1272:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //  1274:      switch Inst[20] {
  0, 6,                                      //  1277:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1279:       check predicate 0
  OPC_Decode, 181, 65, 3,                    //  1281:       decode to UMOP4A_M2ZZ_BToS using decoder 3
                                             //  1281:      }
  1, 0,                                      //  1285:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1287:       check predicate 0
  OPC_Decode, 178, 65, 4,                    //  1289:       decode to UMOP4A_M2Z2Z_BToS using decoder 4
                                             //  1289:      }
                                             //  1289:      } // switch Inst[20]
                                             //  1289:     }
                                             //  1289:     } // switch Inst[16:9]
                                             //  1289:    }
  2, 61,                                     //  1293:    case 0x2: {
  OPC_SwitchField, 9, 8,                     //  1295:     switch Inst[16:9] {
  0, 27,                                     //  1298:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  1300:      switch Inst[20] {
  0, 10,                                     //  1303:      case 0x0: {
  OPC_CheckPredicate, 13,                    //  1305:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1307:       check Inst[1] == 0x0
  OPC_Decode, 222, 15, 5,                    //  1311:       decode to BFMOP4A_MZZ_H using decoder 5
                                             //  1311:      }
  1, 0,                                      //  1315:      case 0x1: {
  OPC_CheckPredicate, 13,                    //  1317:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1319:       check Inst[1] == 0x0
  OPC_Decode, 220, 15, 6,                    //  1323:       decode to BFMOP4A_MZ2Z_H using decoder 6
                                             //  1323:      }
                                             //  1323:      } // switch Inst[20]
                                             //  1323:     }
  1, 0,                                      //  1327:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  1329:      switch Inst[20] {
  0, 10,                                     //  1332:      case 0x0: {
  OPC_CheckPredicate, 13,                    //  1334:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1336:       check Inst[1] == 0x0
  OPC_Decode, 218, 15, 7,                    //  1340:       decode to BFMOP4A_M2ZZ_H using decoder 7
                                             //  1340:      }
  1, 0,                                      //  1344:      case 0x1: {
  OPC_CheckPredicate, 13,                    //  1346:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1348:       check Inst[1] == 0x0
  OPC_Decode, 216, 15, 8,                    //  1352:       decode to BFMOP4A_M2Z2Z_H using decoder 8
                                             //  1352:      }
                                             //  1352:      } // switch Inst[20]
                                             //  1352:     }
                                             //  1352:     } // switch Inst[16:9]
                                             //  1352:    }
  4, 87,                                     //  1356:    case 0x4: {
  OPC_SwitchField, 9, 8,                     //  1358:     switch Inst[16:9] {
  0, 19,                                     //  1361:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  1363:      switch Inst[20] {
  0, 6,                                      //  1366:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1368:       check predicate 0
  OPC_Decode, 185, 29, 1,                    //  1370:       decode to FMOP4S_MZZ_HtoS using decoder 1
                                             //  1370:      }
  1, 0,                                      //  1374:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1376:       check predicate 0
  OPC_Decode, 181, 29, 2,                    //  1378:       decode to FMOP4S_MZ2Z_HtoS using decoder 2
                                             //  1378:      }
                                             //  1378:      } // switch Inst[20]
                                             //  1378:     }
  1, 19,                                     //  1382:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  1384:      switch Inst[20] {
  0, 6,                                      //  1387:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1389:       check predicate 0
  OPC_Decode, 177, 29, 3,                    //  1391:       decode to FMOP4S_M2ZZ_HtoS using decoder 3
                                             //  1391:      }
  1, 0,                                      //  1395:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1397:       check predicate 0
  OPC_Decode, 173, 29, 4,                    //  1399:       decode to FMOP4S_M2Z2Z_HtoS using decoder 4
                                             //  1399:      }
                                             //  1399:      } // switch Inst[20]
                                             //  1399:     }
  64, 19,                                    //  1403:     case 0x40: {
  OPC_SwitchField, 20, 1,                    //  1405:      switch Inst[20] {
  0, 6,                                      //  1408:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1410:       check predicate 0
  OPC_Decode, 199, 65, 1,                    //  1412:       decode to UMOP4S_MZZ_BToS using decoder 1
                                             //  1412:      }
  1, 0,                                      //  1416:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1418:       check predicate 0
  OPC_Decode, 196, 65, 2,                    //  1420:       decode to UMOP4S_MZ2Z_BToS using decoder 2
                                             //  1420:      }
                                             //  1420:      } // switch Inst[20]
                                             //  1420:     }
  65, 0,                                     //  1424:     case 0x41: {
  OPC_SwitchField, 20, 1,                    //  1426:      switch Inst[20] {
  0, 6,                                      //  1429:      case 0x0: {
  OPC_CheckPredicate, 0,                     //  1431:       check predicate 0
  OPC_Decode, 193, 65, 3,                    //  1433:       decode to UMOP4S_M2ZZ_BToS using decoder 3
                                             //  1433:      }
  1, 0,                                      //  1437:      case 0x1: {
  OPC_CheckPredicate, 0,                     //  1439:       check predicate 0
  OPC_Decode, 190, 65, 4,                    //  1441:       decode to UMOP4S_M2Z2Z_BToS using decoder 4
                                             //  1441:      }
                                             //  1441:      } // switch Inst[20]
                                             //  1441:     }
                                             //  1441:     } // switch Inst[16:9]
                                             //  1441:    }
  6, 0,                                      //  1445:    case 0x6: {
  OPC_SwitchField, 9, 8,                     //  1447:     switch Inst[16:9] {
  0, 27,                                     //  1450:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  1452:      switch Inst[20] {
  0, 10,                                     //  1455:      case 0x0: {
  OPC_CheckPredicate, 13,                    //  1457:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1459:       check Inst[1] == 0x0
  OPC_Decode, 230, 15, 5,                    //  1463:       decode to BFMOP4S_MZZ_H using decoder 5
                                             //  1463:      }
  1, 0,                                      //  1467:      case 0x1: {
  OPC_CheckPredicate, 13,                    //  1469:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1471:       check Inst[1] == 0x0
  OPC_Decode, 228, 15, 6,                    //  1475:       decode to BFMOP4S_MZ2Z_H using decoder 6
                                             //  1475:      }
                                             //  1475:      } // switch Inst[20]
                                             //  1475:     }
  1, 0,                                      //  1479:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  1481:      switch Inst[20] {
  0, 10,                                     //  1484:      case 0x0: {
  OPC_CheckPredicate, 13,                    //  1486:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1488:       check Inst[1] == 0x0
  OPC_Decode, 226, 15, 7,                    //  1492:       decode to BFMOP4S_M2ZZ_H using decoder 7
                                             //  1492:      }
  1, 0,                                      //  1496:      case 0x1: {
  OPC_CheckPredicate, 13,                    //  1498:       check predicate 13
  OPC_CheckField, 1, 1, 0,                   //  1500:       check Inst[1] == 0x0
  OPC_Decode, 224, 15, 8,                    //  1504:       decode to BFMOP4S_M2Z2Z_H using decoder 8
                                             //  1504:      }
                                             //  1504:      } // switch Inst[20]
                                             //  1504:     }
                                             //  1504:     } // switch Inst[16:9]
                                             //  1504:    }
                                             //  1504:    } // switch Inst[5:2]
                                             //  1504:   }
  10, 49,                                    //  1508:   case 0xa: {
  OPC_SwitchField, 2, 2,                     //  1510:    switch Inst[3:2] {
  0, 19,                                     //  1513:    case 0x0: {
  OPC_SwitchField, 13, 3,                    //  1515:     switch Inst[15:13] {
  0, 6,                                      //  1518:     case 0x0: {
  OPC_CheckPredicate, 3,                     //  1520:      check predicate 3
  OPC_Decode, 255, 15, 9,                    //  1522:      decode to BFTMOPA_M2ZZZI_HtoS using decoder 9
                                             //  1522:     }
  4, 0,                                      //  1526:     case 0x4: {
  OPC_CheckPredicate, 3,                     //  1528:      check predicate 3
  OPC_Decode, 240, 68, 9,                    //  1530:      decode to USTMOPA_M2ZZZI_BtoS using decoder 9
                                             //  1530:     }
                                             //  1530:     } // switch Inst[15:13]
                                             //  1530:    }
  2, 0,                                      //  1534:    case 0x2: {
  OPC_SwitchField, 13, 3,                    //  1536:     switch Inst[15:13] {
  0, 10,                                     //  1539:     case 0x0: {
  OPC_CheckPredicate, 14,                    //  1541:      check predicate 14
  OPC_CheckField, 1, 1, 0,                   //  1543:      check Inst[1] == 0x0
  OPC_Decode, 197, 32, 10,                   //  1547:      decode to FTMOPA_M2ZZZI_HtoH using decoder 10
                                             //  1547:     }
  4, 0,                                      //  1551:     case 0x4: {
  OPC_CheckPredicate, 3,                     //  1553:      check predicate 3
  OPC_Decode, 139, 69, 9,                    //  1555:      decode to UTMOPA_M2ZZZI_HtoS using decoder 9
                                             //  1555:     }
                                             //  1555:     } // switch Inst[15:13]
                                             //  1555:    }
                                             //  1555:    } // switch Inst[3:2]
                                             //  1555:   }
  11, 40,                                    //  1559:   case 0xb: {
  OPC_SwitchField, 2, 2,                     //  1561:    switch Inst[3:2] {
  0, 19,                                     //  1564:    case 0x0: {
  OPC_SwitchField, 13, 3,                    //  1566:     switch Inst[15:13] {
  0, 6,                                      //  1569:     case 0x0: {
  OPC_CheckPredicate, 3,                     //  1571:      check predicate 3
  OPC_Decode, 198, 32, 9,                    //  1573:      decode to FTMOPA_M2ZZZI_HtoS using decoder 9
                                             //  1573:     }
  4, 0,                                      //  1577:     case 0x4: {
  OPC_CheckPredicate, 3,                     //  1579:      check predicate 3
  OPC_Decode, 138, 69, 9,                    //  1581:      decode to UTMOPA_M2ZZZI_BtoS using decoder 9
                                             //  1581:     }
                                             //  1581:     } // switch Inst[15:13]
                                             //  1581:    }
  2, 0,                                      //  1585:    case 0x2: {
  OPC_CheckPredicate, 15,                    //  1587:     check predicate 15
  OPC_CheckField, 13, 3, 0,                  //  1589:     check Inst[15:13] == 0x0
  OPC_CheckField, 1, 1, 0,                   //  1593:     check Inst[1] == 0x0
  OPC_Decode, 254, 15, 10,                   //  1597:     decode to BFTMOPA_M2ZZZI_HtoH using decoder 10
                                             //  1597:    }
                                             //  1597:    } // switch Inst[3:2]
                                             //  1597:   }
  12, 43,                                    //  1601:   case 0xc: {
  OPC_SwitchField, 2, 3,                     //  1603:    switch Inst[4:2] {
  0, 6,                                      //  1606:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  1608:     check predicate 6
  OPC_Decode, 232, 15, 11,                   //  1610:     decode to BFMOPA_MPPZZ using decoder 11
                                             //  1610:    }
  2, 10,                                     //  1614:    case 0x2: {
  OPC_CheckPredicate, 16,                    //  1616:     check predicate 16
  OPC_CheckField, 1, 1, 0,                   //  1618:     check Inst[1] == 0x0
  OPC_Decode, 191, 29, 12,                   //  1622:     decode to FMOPA_MPPZZ_H using decoder 12
                                             //  1622:    }
  4, 6,                                      //  1626:    case 0x4: {
  OPC_CheckPredicate, 6,                     //  1628:     check predicate 6
  OPC_Decode, 234, 15, 11,                   //  1630:     decode to BFMOPS_MPPZZ using decoder 11
                                             //  1630:    }
  6, 0,                                      //  1634:    case 0x6: {
  OPC_CheckPredicate, 16,                    //  1636:     check predicate 16
  OPC_CheckField, 1, 1, 0,                   //  1638:     check Inst[1] == 0x0
  OPC_Decode, 195, 29, 12,                   //  1642:     decode to FMOPS_MPPZZ_H using decoder 12
                                             //  1642:    }
                                             //  1642:    } // switch Inst[4:2]
                                             //  1642:   }
  13, 0,                                     //  1646:   case 0xd: {
  OPC_SwitchField, 2, 3,                     //  1648:    switch Inst[4:2] {
  0, 6,                                      //  1651:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  1653:     check predicate 6
  OPC_Decode, 187, 29, 11,                   //  1655:     decode to FMOPAL_MPPZZ using decoder 11
                                             //  1655:    }
  2, 10,                                     //  1659:    case 0x2: {
  OPC_CheckPredicate, 17,                    //  1661:     check predicate 17
  OPC_CheckField, 1, 1, 0,                   //  1663:     check Inst[1] == 0x0
  OPC_Decode, 233, 15, 12,                   //  1667:     decode to BFMOPA_MPPZZ_H using decoder 12
                                             //  1667:    }
  4, 6,                                      //  1671:    case 0x4: {
  OPC_CheckPredicate, 6,                     //  1673:     check predicate 6
  OPC_Decode, 193, 29, 11,                   //  1675:     decode to FMOPSL_MPPZZ using decoder 11
                                             //  1675:    }
  6, 0,                                      //  1679:    case 0x6: {
  OPC_CheckPredicate, 17,                    //  1681:     check predicate 17
  OPC_CheckField, 1, 1, 0,                   //  1683:     check Inst[1] == 0x0
  OPC_Decode, 235, 15, 12,                   //  1687:     decode to BFMOPS_MPPZZ_H using decoder 12
                                             //  1687:    }
                                             //  1687:    } // switch Inst[4:2]
                                             //  1687:   }
                                             //  1687:   } // switch Inst[25:21]
                                             //  1687:  }
  5, 182, 18,                                //  1691:  case 0x5: {
  OPC_SwitchField, 21, 5,                    //  1694:   switch Inst[25:21] {
  0, 171, 1,                                 //  1697:   case 0x0: {
  OPC_SwitchField, 13, 3,                    //  1700:    switch Inst[15:13] {
  0, 19,                                     //  1703:    case 0x0: {
  OPC_SwitchField, 0, 1,                     //  1705:     switch Inst[0] {
  0, 6,                                      //  1708:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1710:      check predicate 18
  OPC_Decode, 178, 34, 18,                   //  1712:      decode to LD1B_2Z using decoder 18
                                             //  1712:     }
  1, 0,                                      //  1716:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1718:      check predicate 18
  OPC_Decode, 222, 38, 18,                   //  1720:      decode to LDNT1B_2Z using decoder 18
                                             //  1720:     }
                                             //  1720:     } // switch Inst[0]
                                             //  1720:    }
  1, 19,                                     //  1724:    case 0x1: {
  OPC_SwitchField, 0, 1,                     //  1726:     switch Inst[0] {
  0, 6,                                      //  1729:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1731:      check predicate 18
  OPC_Decode, 222, 34, 18,                   //  1733:      decode to LD1H_2Z using decoder 18
                                             //  1733:     }
  1, 0,                                      //  1737:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1739:      check predicate 18
  OPC_Decode, 245, 38, 18,                   //  1741:      decode to LDNT1H_2Z using decoder 18
                                             //  1741:     }
                                             //  1741:     } // switch Inst[0]
                                             //  1741:    }
  2, 19,                                     //  1745:    case 0x2: {
  OPC_SwitchField, 0, 1,                     //  1747:     switch Inst[0] {
  0, 6,                                      //  1750:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1752:      check predicate 18
  OPC_Decode, 216, 35, 18,                   //  1754:      decode to LD1W_2Z using decoder 18
                                             //  1754:     }
  1, 0,                                      //  1758:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1760:      check predicate 18
  OPC_Decode, 134, 39, 18,                   //  1762:      decode to LDNT1W_2Z using decoder 18
                                             //  1762:     }
                                             //  1762:     } // switch Inst[0]
                                             //  1762:    }
  3, 19,                                     //  1766:    case 0x3: {
  OPC_SwitchField, 0, 1,                     //  1768:     switch Inst[0] {
  0, 6,                                      //  1771:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1773:      check predicate 18
  OPC_Decode, 194, 34, 18,                   //  1775:      decode to LD1D_2Z using decoder 18
                                             //  1775:     }
  1, 0,                                      //  1779:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1781:      check predicate 18
  OPC_Decode, 234, 38, 18,                   //  1783:      decode to LDNT1D_2Z using decoder 18
                                             //  1783:     }
                                             //  1783:     } // switch Inst[0]
                                             //  1783:    }
  4, 19,                                     //  1787:    case 0x4: {
  OPC_SwitchField, 0, 2,                     //  1789:     switch Inst[1:0] {
  0, 6,                                      //  1792:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1794:      check predicate 18
  OPC_Decode, 182, 34, 19,                   //  1796:      decode to LD1B_4Z using decoder 19
                                             //  1796:     }
  1, 0,                                      //  1800:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1802:      check predicate 18
  OPC_Decode, 226, 38, 19,                   //  1804:      decode to LDNT1B_4Z using decoder 19
                                             //  1804:     }
                                             //  1804:     } // switch Inst[1:0]
                                             //  1804:    }
  5, 19,                                     //  1808:    case 0x5: {
  OPC_SwitchField, 0, 2,                     //  1810:     switch Inst[1:0] {
  0, 6,                                      //  1813:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1815:      check predicate 18
  OPC_Decode, 226, 34, 19,                   //  1817:      decode to LD1H_4Z using decoder 19
                                             //  1817:     }
  1, 0,                                      //  1821:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1823:      check predicate 18
  OPC_Decode, 249, 38, 19,                   //  1825:      decode to LDNT1H_4Z using decoder 19
                                             //  1825:     }
                                             //  1825:     } // switch Inst[1:0]
                                             //  1825:    }
  6, 19,                                     //  1829:    case 0x6: {
  OPC_SwitchField, 0, 2,                     //  1831:     switch Inst[1:0] {
  0, 6,                                      //  1834:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1836:      check predicate 18
  OPC_Decode, 220, 35, 19,                   //  1838:      decode to LD1W_4Z using decoder 19
                                             //  1838:     }
  1, 0,                                      //  1842:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1844:      check predicate 18
  OPC_Decode, 138, 39, 19,                   //  1846:      decode to LDNT1W_4Z using decoder 19
                                             //  1846:     }
                                             //  1846:     } // switch Inst[1:0]
                                             //  1846:    }
  7, 0,                                      //  1850:    case 0x7: {
  OPC_SwitchField, 0, 2,                     //  1852:     switch Inst[1:0] {
  0, 6,                                      //  1855:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1857:      check predicate 18
  OPC_Decode, 198, 34, 19,                   //  1859:      decode to LD1D_4Z using decoder 19
                                             //  1859:     }
  1, 0,                                      //  1863:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1865:      check predicate 18
  OPC_Decode, 238, 38, 19,                   //  1867:      decode to LDNT1D_4Z using decoder 19
                                             //  1867:     }
                                             //  1867:     } // switch Inst[1:0]
                                             //  1867:    }
                                             //  1867:    } // switch Inst[15:13]
                                             //  1867:   }
  1, 171, 1,                                 //  1871:   case 0x1: {
  OPC_SwitchField, 13, 3,                    //  1874:    switch Inst[15:13] {
  0, 19,                                     //  1877:    case 0x0: {
  OPC_SwitchField, 0, 1,                     //  1879:     switch Inst[0] {
  0, 6,                                      //  1882:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1884:      check predicate 18
  OPC_Decode, 161, 56, 18,                   //  1886:      decode to ST1B_2Z using decoder 18
                                             //  1886:     }
  1, 0,                                      //  1890:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1892:      check predicate 18
  OPC_Decode, 232, 58, 18,                   //  1894:      decode to STNT1B_2Z using decoder 18
                                             //  1894:     }
                                             //  1894:     } // switch Inst[0]
                                             //  1894:    }
  1, 19,                                     //  1898:    case 0x1: {
  OPC_SwitchField, 0, 1,                     //  1900:     switch Inst[0] {
  0, 6,                                      //  1903:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1905:      check predicate 18
  OPC_Decode, 205, 56, 18,                   //  1907:      decode to ST1H_2Z using decoder 18
                                             //  1907:     }
  1, 0,                                      //  1911:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1913:      check predicate 18
  OPC_Decode, 255, 58, 18,                   //  1915:      decode to STNT1H_2Z using decoder 18
                                             //  1915:     }
                                             //  1915:     } // switch Inst[0]
                                             //  1915:    }
  2, 19,                                     //  1919:    case 0x2: {
  OPC_SwitchField, 0, 1,                     //  1921:     switch Inst[0] {
  0, 6,                                      //  1924:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1926:      check predicate 18
  OPC_Decode, 139, 57, 18,                   //  1928:      decode to ST1W_2Z using decoder 18
                                             //  1928:     }
  1, 0,                                      //  1932:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1934:      check predicate 18
  OPC_Decode, 139, 59, 18,                   //  1936:      decode to STNT1W_2Z using decoder 18
                                             //  1936:     }
                                             //  1936:     } // switch Inst[0]
                                             //  1936:    }
  3, 19,                                     //  1940:    case 0x3: {
  OPC_SwitchField, 0, 1,                     //  1942:     switch Inst[0] {
  0, 6,                                      //  1945:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1947:      check predicate 18
  OPC_Decode, 177, 56, 18,                   //  1949:      decode to ST1D_2Z using decoder 18
                                             //  1949:     }
  1, 0,                                      //  1953:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1955:      check predicate 18
  OPC_Decode, 244, 58, 18,                   //  1957:      decode to STNT1D_2Z using decoder 18
                                             //  1957:     }
                                             //  1957:     } // switch Inst[0]
                                             //  1957:    }
  4, 19,                                     //  1961:    case 0x4: {
  OPC_SwitchField, 0, 2,                     //  1963:     switch Inst[1:0] {
  0, 6,                                      //  1966:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1968:      check predicate 18
  OPC_Decode, 165, 56, 19,                   //  1970:      decode to ST1B_4Z using decoder 19
                                             //  1970:     }
  1, 0,                                      //  1974:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1976:      check predicate 18
  OPC_Decode, 236, 58, 19,                   //  1978:      decode to STNT1B_4Z using decoder 19
                                             //  1978:     }
                                             //  1978:     } // switch Inst[1:0]
                                             //  1978:    }
  5, 19,                                     //  1982:    case 0x5: {
  OPC_SwitchField, 0, 2,                     //  1984:     switch Inst[1:0] {
  0, 6,                                      //  1987:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  1989:      check predicate 18
  OPC_Decode, 209, 56, 19,                   //  1991:      decode to ST1H_4Z using decoder 19
                                             //  1991:     }
  1, 0,                                      //  1995:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  1997:      check predicate 18
  OPC_Decode, 131, 59, 19,                   //  1999:      decode to STNT1H_4Z using decoder 19
                                             //  1999:     }
                                             //  1999:     } // switch Inst[1:0]
                                             //  1999:    }
  6, 19,                                     //  2003:    case 0x6: {
  OPC_SwitchField, 0, 2,                     //  2005:     switch Inst[1:0] {
  0, 6,                                      //  2008:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2010:      check predicate 18
  OPC_Decode, 143, 57, 19,                   //  2012:      decode to ST1W_4Z using decoder 19
                                             //  2012:     }
  1, 0,                                      //  2016:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2018:      check predicate 18
  OPC_Decode, 143, 59, 19,                   //  2020:      decode to STNT1W_4Z using decoder 19
                                             //  2020:     }
                                             //  2020:     } // switch Inst[1:0]
                                             //  2020:    }
  7, 0,                                      //  2024:    case 0x7: {
  OPC_SwitchField, 0, 2,                     //  2026:     switch Inst[1:0] {
  0, 6,                                      //  2029:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2031:      check predicate 18
  OPC_Decode, 181, 56, 19,                   //  2033:      decode to ST1D_4Z using decoder 19
                                             //  2033:     }
  1, 0,                                      //  2037:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2039:      check predicate 18
  OPC_Decode, 248, 58, 19,                   //  2041:      decode to STNT1D_4Z using decoder 19
                                             //  2041:     }
                                             //  2041:     } // switch Inst[1:0]
                                             //  2041:    }
                                             //  2041:    } // switch Inst[15:13]
                                             //  2041:   }
  2, 235, 1,                                 //  2045:   case 0x2: {
  OPC_SwitchField, 13, 3,                    //  2048:    switch Inst[15:13] {
  0, 27,                                     //  2051:    case 0x0: {
  OPC_SwitchField, 0, 1,                     //  2053:     switch Inst[0] {
  0, 10,                                     //  2056:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2058:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2060:      check Inst[20] == 0x0
  OPC_Decode, 179, 34, 20,                   //  2064:      decode to LD1B_2Z_IMM using decoder 20
                                             //  2064:     }
  1, 0,                                      //  2068:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2070:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2072:      check Inst[20] == 0x0
  OPC_Decode, 223, 38, 20,                   //  2076:      decode to LDNT1B_2Z_IMM using decoder 20
                                             //  2076:     }
                                             //  2076:     } // switch Inst[0]
                                             //  2076:    }
  1, 27,                                     //  2080:    case 0x1: {
  OPC_SwitchField, 0, 1,                     //  2082:     switch Inst[0] {
  0, 10,                                     //  2085:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2087:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2089:      check Inst[20] == 0x0
  OPC_Decode, 223, 34, 20,                   //  2093:      decode to LD1H_2Z_IMM using decoder 20
                                             //  2093:     }
  1, 0,                                      //  2097:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2099:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2101:      check Inst[20] == 0x0
  OPC_Decode, 246, 38, 20,                   //  2105:      decode to LDNT1H_2Z_IMM using decoder 20
                                             //  2105:     }
                                             //  2105:     } // switch Inst[0]
                                             //  2105:    }
  2, 27,                                     //  2109:    case 0x2: {
  OPC_SwitchField, 0, 1,                     //  2111:     switch Inst[0] {
  0, 10,                                     //  2114:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2116:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2118:      check Inst[20] == 0x0
  OPC_Decode, 217, 35, 20,                   //  2122:      decode to LD1W_2Z_IMM using decoder 20
                                             //  2122:     }
  1, 0,                                      //  2126:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2128:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2130:      check Inst[20] == 0x0
  OPC_Decode, 135, 39, 20,                   //  2134:      decode to LDNT1W_2Z_IMM using decoder 20
                                             //  2134:     }
                                             //  2134:     } // switch Inst[0]
                                             //  2134:    }
  3, 27,                                     //  2138:    case 0x3: {
  OPC_SwitchField, 0, 1,                     //  2140:     switch Inst[0] {
  0, 10,                                     //  2143:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2145:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2147:      check Inst[20] == 0x0
  OPC_Decode, 195, 34, 20,                   //  2151:      decode to LD1D_2Z_IMM using decoder 20
                                             //  2151:     }
  1, 0,                                      //  2155:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2157:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2159:      check Inst[20] == 0x0
  OPC_Decode, 235, 38, 20,                   //  2163:      decode to LDNT1D_2Z_IMM using decoder 20
                                             //  2163:     }
                                             //  2163:     } // switch Inst[0]
                                             //  2163:    }
  4, 27,                                     //  2167:    case 0x4: {
  OPC_SwitchField, 0, 2,                     //  2169:     switch Inst[1:0] {
  0, 10,                                     //  2172:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2174:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2176:      check Inst[20] == 0x0
  OPC_Decode, 183, 34, 21,                   //  2180:      decode to LD1B_4Z_IMM using decoder 21
                                             //  2180:     }
  1, 0,                                      //  2184:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2186:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2188:      check Inst[20] == 0x0
  OPC_Decode, 227, 38, 21,                   //  2192:      decode to LDNT1B_4Z_IMM using decoder 21
                                             //  2192:     }
                                             //  2192:     } // switch Inst[1:0]
                                             //  2192:    }
  5, 27,                                     //  2196:    case 0x5: {
  OPC_SwitchField, 0, 2,                     //  2198:     switch Inst[1:0] {
  0, 10,                                     //  2201:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2203:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2205:      check Inst[20] == 0x0
  OPC_Decode, 227, 34, 21,                   //  2209:      decode to LD1H_4Z_IMM using decoder 21
                                             //  2209:     }
  1, 0,                                      //  2213:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2215:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2217:      check Inst[20] == 0x0
  OPC_Decode, 250, 38, 21,                   //  2221:      decode to LDNT1H_4Z_IMM using decoder 21
                                             //  2221:     }
                                             //  2221:     } // switch Inst[1:0]
                                             //  2221:    }
  6, 27,                                     //  2225:    case 0x6: {
  OPC_SwitchField, 0, 2,                     //  2227:     switch Inst[1:0] {
  0, 10,                                     //  2230:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2232:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2234:      check Inst[20] == 0x0
  OPC_Decode, 221, 35, 21,                   //  2238:      decode to LD1W_4Z_IMM using decoder 21
                                             //  2238:     }
  1, 0,                                      //  2242:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2244:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2246:      check Inst[20] == 0x0
  OPC_Decode, 139, 39, 21,                   //  2250:      decode to LDNT1W_4Z_IMM using decoder 21
                                             //  2250:     }
                                             //  2250:     } // switch Inst[1:0]
                                             //  2250:    }
  7, 0,                                      //  2254:    case 0x7: {
  OPC_SwitchField, 0, 2,                     //  2256:     switch Inst[1:0] {
  0, 10,                                     //  2259:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2261:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2263:      check Inst[20] == 0x0
  OPC_Decode, 199, 34, 21,                   //  2267:      decode to LD1D_4Z_IMM using decoder 21
                                             //  2267:     }
  1, 0,                                      //  2271:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2273:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2275:      check Inst[20] == 0x0
  OPC_Decode, 239, 38, 21,                   //  2279:      decode to LDNT1D_4Z_IMM using decoder 21
                                             //  2279:     }
                                             //  2279:     } // switch Inst[1:0]
                                             //  2279:    }
                                             //  2279:    } // switch Inst[15:13]
                                             //  2279:   }
  3, 235, 1,                                 //  2283:   case 0x3: {
  OPC_SwitchField, 13, 3,                    //  2286:    switch Inst[15:13] {
  0, 27,                                     //  2289:    case 0x0: {
  OPC_SwitchField, 0, 1,                     //  2291:     switch Inst[0] {
  0, 10,                                     //  2294:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2296:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2298:      check Inst[20] == 0x0
  OPC_Decode, 162, 56, 20,                   //  2302:      decode to ST1B_2Z_IMM using decoder 20
                                             //  2302:     }
  1, 0,                                      //  2306:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2308:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2310:      check Inst[20] == 0x0
  OPC_Decode, 233, 58, 20,                   //  2314:      decode to STNT1B_2Z_IMM using decoder 20
                                             //  2314:     }
                                             //  2314:     } // switch Inst[0]
                                             //  2314:    }
  1, 27,                                     //  2318:    case 0x1: {
  OPC_SwitchField, 0, 1,                     //  2320:     switch Inst[0] {
  0, 10,                                     //  2323:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2325:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2327:      check Inst[20] == 0x0
  OPC_Decode, 206, 56, 20,                   //  2331:      decode to ST1H_2Z_IMM using decoder 20
                                             //  2331:     }
  1, 0,                                      //  2335:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2337:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2339:      check Inst[20] == 0x0
  OPC_Decode, 128, 59, 20,                   //  2343:      decode to STNT1H_2Z_IMM using decoder 20
                                             //  2343:     }
                                             //  2343:     } // switch Inst[0]
                                             //  2343:    }
  2, 27,                                     //  2347:    case 0x2: {
  OPC_SwitchField, 0, 1,                     //  2349:     switch Inst[0] {
  0, 10,                                     //  2352:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2354:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2356:      check Inst[20] == 0x0
  OPC_Decode, 140, 57, 20,                   //  2360:      decode to ST1W_2Z_IMM using decoder 20
                                             //  2360:     }
  1, 0,                                      //  2364:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2366:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2368:      check Inst[20] == 0x0
  OPC_Decode, 140, 59, 20,                   //  2372:      decode to STNT1W_2Z_IMM using decoder 20
                                             //  2372:     }
                                             //  2372:     } // switch Inst[0]
                                             //  2372:    }
  3, 27,                                     //  2376:    case 0x3: {
  OPC_SwitchField, 0, 1,                     //  2378:     switch Inst[0] {
  0, 10,                                     //  2381:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2383:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2385:      check Inst[20] == 0x0
  OPC_Decode, 178, 56, 20,                   //  2389:      decode to ST1D_2Z_IMM using decoder 20
                                             //  2389:     }
  1, 0,                                      //  2393:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2395:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2397:      check Inst[20] == 0x0
  OPC_Decode, 245, 58, 20,                   //  2401:      decode to STNT1D_2Z_IMM using decoder 20
                                             //  2401:     }
                                             //  2401:     } // switch Inst[0]
                                             //  2401:    }
  4, 27,                                     //  2405:    case 0x4: {
  OPC_SwitchField, 0, 2,                     //  2407:     switch Inst[1:0] {
  0, 10,                                     //  2410:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2412:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2414:      check Inst[20] == 0x0
  OPC_Decode, 166, 56, 21,                   //  2418:      decode to ST1B_4Z_IMM using decoder 21
                                             //  2418:     }
  1, 0,                                      //  2422:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2424:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2426:      check Inst[20] == 0x0
  OPC_Decode, 237, 58, 21,                   //  2430:      decode to STNT1B_4Z_IMM using decoder 21
                                             //  2430:     }
                                             //  2430:     } // switch Inst[1:0]
                                             //  2430:    }
  5, 27,                                     //  2434:    case 0x5: {
  OPC_SwitchField, 0, 2,                     //  2436:     switch Inst[1:0] {
  0, 10,                                     //  2439:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2441:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2443:      check Inst[20] == 0x0
  OPC_Decode, 210, 56, 21,                   //  2447:      decode to ST1H_4Z_IMM using decoder 21
                                             //  2447:     }
  1, 0,                                      //  2451:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2453:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2455:      check Inst[20] == 0x0
  OPC_Decode, 132, 59, 21,                   //  2459:      decode to STNT1H_4Z_IMM using decoder 21
                                             //  2459:     }
                                             //  2459:     } // switch Inst[1:0]
                                             //  2459:    }
  6, 27,                                     //  2463:    case 0x6: {
  OPC_SwitchField, 0, 2,                     //  2465:     switch Inst[1:0] {
  0, 10,                                     //  2468:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2470:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2472:      check Inst[20] == 0x0
  OPC_Decode, 144, 57, 21,                   //  2476:      decode to ST1W_4Z_IMM using decoder 21
                                             //  2476:     }
  1, 0,                                      //  2480:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2482:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2484:      check Inst[20] == 0x0
  OPC_Decode, 144, 59, 21,                   //  2488:      decode to STNT1W_4Z_IMM using decoder 21
                                             //  2488:     }
                                             //  2488:     } // switch Inst[1:0]
                                             //  2488:    }
  7, 0,                                      //  2492:    case 0x7: {
  OPC_SwitchField, 0, 2,                     //  2494:     switch Inst[1:0] {
  0, 10,                                     //  2497:     case 0x0: {
  OPC_CheckPredicate, 18,                    //  2499:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2501:      check Inst[20] == 0x0
  OPC_Decode, 182, 56, 21,                   //  2505:      decode to ST1D_4Z_IMM using decoder 21
                                             //  2505:     }
  1, 0,                                      //  2509:     case 0x1: {
  OPC_CheckPredicate, 18,                    //  2511:      check predicate 18
  OPC_CheckField, 20, 1, 0,                  //  2513:      check Inst[20] == 0x0
  OPC_Decode, 249, 58, 21,                   //  2517:      decode to STNT1D_4Z_IMM using decoder 21
                                             //  2517:     }
                                             //  2517:     } // switch Inst[1:0]
                                             //  2517:    }
                                             //  2517:    } // switch Inst[15:13]
                                             //  2517:   }
  4, 35,                                     //  2521:   case 0x4: {
  OPC_SwitchField, 2, 3,                     //  2523:    switch Inst[4:2] {
  0, 6,                                      //  2526:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  2528:     check predicate 6
  OPC_Decode, 195, 50, 11,                   //  2530:     decode to SMOPA_MPPZZ_S using decoder 11
                                             //  2530:    }
  2, 6,                                      //  2534:    case 0x2: {
  OPC_CheckPredicate, 7,                     //  2536:     check predicate 7
  OPC_Decode, 194, 50, 11,                   //  2538:     decode to SMOPA_MPPZZ_HtoS using decoder 11
                                             //  2538:    }
  4, 6,                                      //  2542:    case 0x4: {
  OPC_CheckPredicate, 6,                     //  2544:     check predicate 6
  OPC_Decode, 198, 50, 11,                   //  2546:     decode to SMOPS_MPPZZ_S using decoder 11
                                             //  2546:    }
  6, 0,                                      //  2550:    case 0x6: {
  OPC_CheckPredicate, 7,                     //  2552:     check predicate 7
  OPC_Decode, 197, 50, 11,                   //  2554:     decode to SMOPS_MPPZZ_HtoS using decoder 11
                                             //  2554:    }
                                             //  2554:    } // switch Inst[4:2]
                                             //  2554:   }
  5, 19,                                     //  2558:   case 0x5: {
  OPC_SwitchField, 2, 3,                     //  2560:    switch Inst[4:2] {
  0, 6,                                      //  2563:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  2565:     check predicate 6
  OPC_Decode, 229, 60, 11,                   //  2567:     decode to SUMOPA_MPPZZ_S using decoder 11
                                             //  2567:    }
  4, 0,                                      //  2571:    case 0x4: {
  OPC_CheckPredicate, 6,                     //  2573:     check predicate 6
  OPC_Decode, 231, 60, 11,                   //  2575:     decode to SUMOPS_MPPZZ_S using decoder 11
                                             //  2575:    }
                                             //  2575:    } // switch Inst[4:2]
                                             //  2575:   }
  6, 145, 1,                                 //  2579:   case 0x6: {
  OPC_SwitchField, 3, 2,                     //  2582:    switch Inst[4:3] {
  0, 6,                                      //  2585:    case 0x0: {
  OPC_CheckPredicate, 19,                    //  2587:     check predicate 19
  OPC_Decode, 193, 50, 13,                   //  2589:     decode to SMOPA_MPPZZ_D using decoder 13
                                             //  2589:    }
  1, 61,                                     //  2593:    case 0x1: {
  OPC_SwitchField, 9, 8,                     //  2595:     switch Inst[16:9] {
  0, 27,                                     //  2598:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  2600:      switch Inst[20] {
  0, 10,                                     //  2603:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2605:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2607:       check Inst[5] == 0x0
  OPC_Decode, 180, 50, 14,                   //  2611:       decode to SMOP4A_MZZ_HtoD using decoder 14
                                             //  2611:      }
  1, 0,                                      //  2615:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2617:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2619:       check Inst[5] == 0x0
  OPC_Decode, 177, 50, 15,                   //  2623:       decode to SMOP4A_MZ2Z_HtoD using decoder 15
                                             //  2623:      }
                                             //  2623:      } // switch Inst[20]
                                             //  2623:     }
  1, 0,                                      //  2627:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  2629:      switch Inst[20] {
  0, 10,                                     //  2632:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2634:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2636:       check Inst[5] == 0x0
  OPC_Decode, 174, 50, 16,                   //  2640:       decode to SMOP4A_M2ZZ_HtoD using decoder 16
                                             //  2640:      }
  1, 0,                                      //  2644:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2646:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2648:       check Inst[5] == 0x0
  OPC_Decode, 171, 50, 17,                   //  2652:       decode to SMOP4A_M2Z2Z_HtoD using decoder 17
                                             //  2652:      }
                                             //  2652:      } // switch Inst[20]
                                             //  2652:     }
                                             //  2652:     } // switch Inst[16:9]
                                             //  2652:    }
  2, 6,                                      //  2656:    case 0x2: {
  OPC_CheckPredicate, 19,                    //  2658:     check predicate 19
  OPC_Decode, 196, 50, 13,                   //  2660:     decode to SMOPS_MPPZZ_D using decoder 13
                                             //  2660:    }
  3, 0,                                      //  2664:    case 0x3: {
  OPC_SwitchField, 9, 8,                     //  2666:     switch Inst[16:9] {
  0, 27,                                     //  2669:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  2671:      switch Inst[20] {
  0, 10,                                     //  2674:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2676:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2678:       check Inst[5] == 0x0
  OPC_Decode, 192, 50, 14,                   //  2682:       decode to SMOP4S_MZZ_HtoD using decoder 14
                                             //  2682:      }
  1, 0,                                      //  2686:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2688:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2690:       check Inst[5] == 0x0
  OPC_Decode, 189, 50, 15,                   //  2694:       decode to SMOP4S_MZ2Z_HtoD using decoder 15
                                             //  2694:      }
                                             //  2694:      } // switch Inst[20]
                                             //  2694:     }
  1, 0,                                      //  2698:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  2700:      switch Inst[20] {
  0, 10,                                     //  2703:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2705:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2707:       check Inst[5] == 0x0
  OPC_Decode, 186, 50, 16,                   //  2711:       decode to SMOP4S_M2ZZ_HtoD using decoder 16
                                             //  2711:      }
  1, 0,                                      //  2715:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2717:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2719:       check Inst[5] == 0x0
  OPC_Decode, 183, 50, 17,                   //  2723:       decode to SMOP4S_M2Z2Z_HtoD using decoder 17
                                             //  2723:      }
                                             //  2723:      } // switch Inst[20]
                                             //  2723:     }
                                             //  2723:     } // switch Inst[16:9]
                                             //  2723:    }
                                             //  2723:    } // switch Inst[4:3]
                                             //  2723:   }
  7, 145, 1,                                 //  2727:   case 0x7: {
  OPC_SwitchField, 3, 2,                     //  2730:    switch Inst[4:3] {
  0, 6,                                      //  2733:    case 0x0: {
  OPC_CheckPredicate, 19,                    //  2735:     check predicate 19
  OPC_Decode, 228, 60, 13,                   //  2737:     decode to SUMOPA_MPPZZ_D using decoder 13
                                             //  2737:    }
  1, 61,                                     //  2741:    case 0x1: {
  OPC_SwitchField, 9, 8,                     //  2743:     switch Inst[16:9] {
  0, 27,                                     //  2746:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  2748:      switch Inst[20] {
  0, 10,                                     //  2751:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2753:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2755:       check Inst[5] == 0x0
  OPC_Decode, 219, 60, 14,                   //  2759:       decode to SUMOP4A_MZZ_HtoD using decoder 14
                                             //  2759:      }
  1, 0,                                      //  2763:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2765:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2767:       check Inst[5] == 0x0
  OPC_Decode, 217, 60, 15,                   //  2771:       decode to SUMOP4A_MZ2Z_HtoD using decoder 15
                                             //  2771:      }
                                             //  2771:      } // switch Inst[20]
                                             //  2771:     }
  1, 0,                                      //  2775:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  2777:      switch Inst[20] {
  0, 10,                                     //  2780:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2782:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2784:       check Inst[5] == 0x0
  OPC_Decode, 215, 60, 16,                   //  2788:       decode to SUMOP4A_M2ZZ_HtoD using decoder 16
                                             //  2788:      }
  1, 0,                                      //  2792:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2794:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2796:       check Inst[5] == 0x0
  OPC_Decode, 213, 60, 17,                   //  2800:       decode to SUMOP4A_M2Z2Z_HtoD using decoder 17
                                             //  2800:      }
                                             //  2800:      } // switch Inst[20]
                                             //  2800:     }
                                             //  2800:     } // switch Inst[16:9]
                                             //  2800:    }
  2, 6,                                      //  2804:    case 0x2: {
  OPC_CheckPredicate, 19,                    //  2806:     check predicate 19
  OPC_Decode, 230, 60, 13,                   //  2808:     decode to SUMOPS_MPPZZ_D using decoder 13
                                             //  2808:    }
  3, 0,                                      //  2812:    case 0x3: {
  OPC_SwitchField, 9, 8,                     //  2814:     switch Inst[16:9] {
  0, 27,                                     //  2817:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  2819:      switch Inst[20] {
  0, 10,                                     //  2822:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2824:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2826:       check Inst[5] == 0x0
  OPC_Decode, 227, 60, 14,                   //  2830:       decode to SUMOP4S_MZZ_HtoD using decoder 14
                                             //  2830:      }
  1, 0,                                      //  2834:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2836:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2838:       check Inst[5] == 0x0
  OPC_Decode, 225, 60, 15,                   //  2842:       decode to SUMOP4S_MZ2Z_HtoD using decoder 15
                                             //  2842:      }
                                             //  2842:      } // switch Inst[20]
                                             //  2842:     }
  1, 0,                                      //  2846:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  2848:      switch Inst[20] {
  0, 10,                                     //  2851:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  2853:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2855:       check Inst[5] == 0x0
  OPC_Decode, 223, 60, 16,                   //  2859:       decode to SUMOP4S_M2ZZ_HtoD using decoder 16
                                             //  2859:      }
  1, 0,                                      //  2863:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  2865:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  2867:       check Inst[5] == 0x0
  OPC_Decode, 221, 60, 17,                   //  2871:       decode to SUMOP4S_M2Z2Z_HtoD using decoder 17
                                             //  2871:      }
                                             //  2871:      } // switch Inst[20]
                                             //  2871:     }
                                             //  2871:     } // switch Inst[16:9]
                                             //  2871:    }
                                             //  2871:    } // switch Inst[4:3]
                                             //  2871:   }
  8, 171, 1,                                 //  2875:   case 0x8: {
  OPC_SwitchField, 13, 3,                    //  2878:    switch Inst[15:13] {
  0, 19,                                     //  2881:    case 0x0: {
  OPC_SwitchField, 3, 1,                     //  2883:     switch Inst[3] {
  0, 6,                                      //  2886:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  2888:      check predicate 7
  OPC_Decode, 180, 34, 22,                   //  2890:      decode to LD1B_2Z_STRIDED using decoder 22
                                             //  2890:     }
  1, 0,                                      //  2894:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  2896:      check predicate 7
  OPC_Decode, 224, 38, 22,                   //  2898:      decode to LDNT1B_2Z_STRIDED using decoder 22
                                             //  2898:     }
                                             //  2898:     } // switch Inst[3]
                                             //  2898:    }
  1, 19,                                     //  2902:    case 0x1: {
  OPC_SwitchField, 3, 1,                     //  2904:     switch Inst[3] {
  0, 6,                                      //  2907:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  2909:      check predicate 7
  OPC_Decode, 224, 34, 22,                   //  2911:      decode to LD1H_2Z_STRIDED using decoder 22
                                             //  2911:     }
  1, 0,                                      //  2915:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  2917:      check predicate 7
  OPC_Decode, 247, 38, 22,                   //  2919:      decode to LDNT1H_2Z_STRIDED using decoder 22
                                             //  2919:     }
                                             //  2919:     } // switch Inst[3]
                                             //  2919:    }
  2, 19,                                     //  2923:    case 0x2: {
  OPC_SwitchField, 3, 1,                     //  2925:     switch Inst[3] {
  0, 6,                                      //  2928:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  2930:      check predicate 7
  OPC_Decode, 218, 35, 22,                   //  2932:      decode to LD1W_2Z_STRIDED using decoder 22
                                             //  2932:     }
  1, 0,                                      //  2936:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  2938:      check predicate 7
  OPC_Decode, 136, 39, 22,                   //  2940:      decode to LDNT1W_2Z_STRIDED using decoder 22
                                             //  2940:     }
                                             //  2940:     } // switch Inst[3]
                                             //  2940:    }
  3, 19,                                     //  2944:    case 0x3: {
  OPC_SwitchField, 3, 1,                     //  2946:     switch Inst[3] {
  0, 6,                                      //  2949:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  2951:      check predicate 7
  OPC_Decode, 196, 34, 22,                   //  2953:      decode to LD1D_2Z_STRIDED using decoder 22
                                             //  2953:     }
  1, 0,                                      //  2957:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  2959:      check predicate 7
  OPC_Decode, 236, 38, 22,                   //  2961:      decode to LDNT1D_2Z_STRIDED using decoder 22
                                             //  2961:     }
                                             //  2961:     } // switch Inst[3]
                                             //  2961:    }
  4, 19,                                     //  2965:    case 0x4: {
  OPC_SwitchField, 2, 2,                     //  2967:     switch Inst[3:2] {
  0, 6,                                      //  2970:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  2972:      check predicate 7
  OPC_Decode, 184, 34, 23,                   //  2974:      decode to LD1B_4Z_STRIDED using decoder 23
                                             //  2974:     }
  2, 0,                                      //  2978:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  2980:      check predicate 7
  OPC_Decode, 228, 38, 23,                   //  2982:      decode to LDNT1B_4Z_STRIDED using decoder 23
                                             //  2982:     }
                                             //  2982:     } // switch Inst[3:2]
                                             //  2982:    }
  5, 19,                                     //  2986:    case 0x5: {
  OPC_SwitchField, 2, 2,                     //  2988:     switch Inst[3:2] {
  0, 6,                                      //  2991:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  2993:      check predicate 7
  OPC_Decode, 228, 34, 23,                   //  2995:      decode to LD1H_4Z_STRIDED using decoder 23
                                             //  2995:     }
  2, 0,                                      //  2999:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3001:      check predicate 7
  OPC_Decode, 251, 38, 23,                   //  3003:      decode to LDNT1H_4Z_STRIDED using decoder 23
                                             //  3003:     }
                                             //  3003:     } // switch Inst[3:2]
                                             //  3003:    }
  6, 19,                                     //  3007:    case 0x6: {
  OPC_SwitchField, 2, 2,                     //  3009:     switch Inst[3:2] {
  0, 6,                                      //  3012:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3014:      check predicate 7
  OPC_Decode, 222, 35, 23,                   //  3016:      decode to LD1W_4Z_STRIDED using decoder 23
                                             //  3016:     }
  2, 0,                                      //  3020:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3022:      check predicate 7
  OPC_Decode, 140, 39, 23,                   //  3024:      decode to LDNT1W_4Z_STRIDED using decoder 23
                                             //  3024:     }
                                             //  3024:     } // switch Inst[3:2]
                                             //  3024:    }
  7, 0,                                      //  3028:    case 0x7: {
  OPC_SwitchField, 2, 2,                     //  3030:     switch Inst[3:2] {
  0, 6,                                      //  3033:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3035:      check predicate 7
  OPC_Decode, 200, 34, 23,                   //  3037:      decode to LD1D_4Z_STRIDED using decoder 23
                                             //  3037:     }
  2, 0,                                      //  3041:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3043:      check predicate 7
  OPC_Decode, 240, 38, 23,                   //  3045:      decode to LDNT1D_4Z_STRIDED using decoder 23
                                             //  3045:     }
                                             //  3045:     } // switch Inst[3:2]
                                             //  3045:    }
                                             //  3045:    } // switch Inst[15:13]
                                             //  3045:   }
  9, 171, 1,                                 //  3049:   case 0x9: {
  OPC_SwitchField, 13, 3,                    //  3052:    switch Inst[15:13] {
  0, 19,                                     //  3055:    case 0x0: {
  OPC_SwitchField, 3, 1,                     //  3057:     switch Inst[3] {
  0, 6,                                      //  3060:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3062:      check predicate 7
  OPC_Decode, 163, 56, 22,                   //  3064:      decode to ST1B_2Z_STRIDED using decoder 22
                                             //  3064:     }
  1, 0,                                      //  3068:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3070:      check predicate 7
  OPC_Decode, 234, 58, 22,                   //  3072:      decode to STNT1B_2Z_STRIDED using decoder 22
                                             //  3072:     }
                                             //  3072:     } // switch Inst[3]
                                             //  3072:    }
  1, 19,                                     //  3076:    case 0x1: {
  OPC_SwitchField, 3, 1,                     //  3078:     switch Inst[3] {
  0, 6,                                      //  3081:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3083:      check predicate 7
  OPC_Decode, 207, 56, 22,                   //  3085:      decode to ST1H_2Z_STRIDED using decoder 22
                                             //  3085:     }
  1, 0,                                      //  3089:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3091:      check predicate 7
  OPC_Decode, 129, 59, 22,                   //  3093:      decode to STNT1H_2Z_STRIDED using decoder 22
                                             //  3093:     }
                                             //  3093:     } // switch Inst[3]
                                             //  3093:    }
  2, 19,                                     //  3097:    case 0x2: {
  OPC_SwitchField, 3, 1,                     //  3099:     switch Inst[3] {
  0, 6,                                      //  3102:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3104:      check predicate 7
  OPC_Decode, 141, 57, 22,                   //  3106:      decode to ST1W_2Z_STRIDED using decoder 22
                                             //  3106:     }
  1, 0,                                      //  3110:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3112:      check predicate 7
  OPC_Decode, 141, 59, 22,                   //  3114:      decode to STNT1W_2Z_STRIDED using decoder 22
                                             //  3114:     }
                                             //  3114:     } // switch Inst[3]
                                             //  3114:    }
  3, 19,                                     //  3118:    case 0x3: {
  OPC_SwitchField, 3, 1,                     //  3120:     switch Inst[3] {
  0, 6,                                      //  3123:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3125:      check predicate 7
  OPC_Decode, 179, 56, 22,                   //  3127:      decode to ST1D_2Z_STRIDED using decoder 22
                                             //  3127:     }
  1, 0,                                      //  3131:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3133:      check predicate 7
  OPC_Decode, 246, 58, 22,                   //  3135:      decode to STNT1D_2Z_STRIDED using decoder 22
                                             //  3135:     }
                                             //  3135:     } // switch Inst[3]
                                             //  3135:    }
  4, 19,                                     //  3139:    case 0x4: {
  OPC_SwitchField, 2, 2,                     //  3141:     switch Inst[3:2] {
  0, 6,                                      //  3144:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3146:      check predicate 7
  OPC_Decode, 167, 56, 23,                   //  3148:      decode to ST1B_4Z_STRIDED using decoder 23
                                             //  3148:     }
  2, 0,                                      //  3152:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3154:      check predicate 7
  OPC_Decode, 238, 58, 23,                   //  3156:      decode to STNT1B_4Z_STRIDED using decoder 23
                                             //  3156:     }
                                             //  3156:     } // switch Inst[3:2]
                                             //  3156:    }
  5, 19,                                     //  3160:    case 0x5: {
  OPC_SwitchField, 2, 2,                     //  3162:     switch Inst[3:2] {
  0, 6,                                      //  3165:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3167:      check predicate 7
  OPC_Decode, 211, 56, 23,                   //  3169:      decode to ST1H_4Z_STRIDED using decoder 23
                                             //  3169:     }
  2, 0,                                      //  3173:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3175:      check predicate 7
  OPC_Decode, 133, 59, 23,                   //  3177:      decode to STNT1H_4Z_STRIDED using decoder 23
                                             //  3177:     }
                                             //  3177:     } // switch Inst[3:2]
                                             //  3177:    }
  6, 19,                                     //  3181:    case 0x6: {
  OPC_SwitchField, 2, 2,                     //  3183:     switch Inst[3:2] {
  0, 6,                                      //  3186:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3188:      check predicate 7
  OPC_Decode, 145, 57, 23,                   //  3190:      decode to ST1W_4Z_STRIDED using decoder 23
                                             //  3190:     }
  2, 0,                                      //  3194:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3196:      check predicate 7
  OPC_Decode, 145, 59, 23,                   //  3198:      decode to STNT1W_4Z_STRIDED using decoder 23
                                             //  3198:     }
                                             //  3198:     } // switch Inst[3:2]
                                             //  3198:    }
  7, 0,                                      //  3202:    case 0x7: {
  OPC_SwitchField, 2, 2,                     //  3204:     switch Inst[3:2] {
  0, 6,                                      //  3207:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3209:      check predicate 7
  OPC_Decode, 183, 56, 23,                   //  3211:      decode to ST1D_4Z_STRIDED using decoder 23
                                             //  3211:     }
  2, 0,                                      //  3215:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3217:      check predicate 7
  OPC_Decode, 250, 58, 23,                   //  3219:      decode to STNT1D_4Z_STRIDED using decoder 23
                                             //  3219:     }
                                             //  3219:     } // switch Inst[3:2]
                                             //  3219:    }
                                             //  3219:    } // switch Inst[15:13]
                                             //  3219:   }
  10, 235, 1,                                //  3223:   case 0xa: {
  OPC_SwitchField, 13, 3,                    //  3226:    switch Inst[15:13] {
  0, 27,                                     //  3229:    case 0x0: {
  OPC_SwitchField, 3, 1,                     //  3231:     switch Inst[3] {
  0, 10,                                     //  3234:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3236:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3238:      check Inst[20] == 0x0
  OPC_Decode, 181, 34, 24,                   //  3242:      decode to LD1B_2Z_STRIDED_IMM using decoder 24
                                             //  3242:     }
  1, 0,                                      //  3246:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3248:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3250:      check Inst[20] == 0x0
  OPC_Decode, 225, 38, 24,                   //  3254:      decode to LDNT1B_2Z_STRIDED_IMM using decoder 24
                                             //  3254:     }
                                             //  3254:     } // switch Inst[3]
                                             //  3254:    }
  1, 27,                                     //  3258:    case 0x1: {
  OPC_SwitchField, 3, 1,                     //  3260:     switch Inst[3] {
  0, 10,                                     //  3263:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3265:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3267:      check Inst[20] == 0x0
  OPC_Decode, 225, 34, 24,                   //  3271:      decode to LD1H_2Z_STRIDED_IMM using decoder 24
                                             //  3271:     }
  1, 0,                                      //  3275:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3277:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3279:      check Inst[20] == 0x0
  OPC_Decode, 248, 38, 24,                   //  3283:      decode to LDNT1H_2Z_STRIDED_IMM using decoder 24
                                             //  3283:     }
                                             //  3283:     } // switch Inst[3]
                                             //  3283:    }
  2, 27,                                     //  3287:    case 0x2: {
  OPC_SwitchField, 3, 1,                     //  3289:     switch Inst[3] {
  0, 10,                                     //  3292:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3294:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3296:      check Inst[20] == 0x0
  OPC_Decode, 219, 35, 24,                   //  3300:      decode to LD1W_2Z_STRIDED_IMM using decoder 24
                                             //  3300:     }
  1, 0,                                      //  3304:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3306:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3308:      check Inst[20] == 0x0
  OPC_Decode, 137, 39, 24,                   //  3312:      decode to LDNT1W_2Z_STRIDED_IMM using decoder 24
                                             //  3312:     }
                                             //  3312:     } // switch Inst[3]
                                             //  3312:    }
  3, 27,                                     //  3316:    case 0x3: {
  OPC_SwitchField, 3, 1,                     //  3318:     switch Inst[3] {
  0, 10,                                     //  3321:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3323:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3325:      check Inst[20] == 0x0
  OPC_Decode, 197, 34, 24,                   //  3329:      decode to LD1D_2Z_STRIDED_IMM using decoder 24
                                             //  3329:     }
  1, 0,                                      //  3333:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3335:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3337:      check Inst[20] == 0x0
  OPC_Decode, 237, 38, 24,                   //  3341:      decode to LDNT1D_2Z_STRIDED_IMM using decoder 24
                                             //  3341:     }
                                             //  3341:     } // switch Inst[3]
                                             //  3341:    }
  4, 27,                                     //  3345:    case 0x4: {
  OPC_SwitchField, 2, 2,                     //  3347:     switch Inst[3:2] {
  0, 10,                                     //  3350:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3352:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3354:      check Inst[20] == 0x0
  OPC_Decode, 185, 34, 25,                   //  3358:      decode to LD1B_4Z_STRIDED_IMM using decoder 25
                                             //  3358:     }
  2, 0,                                      //  3362:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3364:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3366:      check Inst[20] == 0x0
  OPC_Decode, 229, 38, 25,                   //  3370:      decode to LDNT1B_4Z_STRIDED_IMM using decoder 25
                                             //  3370:     }
                                             //  3370:     } // switch Inst[3:2]
                                             //  3370:    }
  5, 27,                                     //  3374:    case 0x5: {
  OPC_SwitchField, 2, 2,                     //  3376:     switch Inst[3:2] {
  0, 10,                                     //  3379:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3381:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3383:      check Inst[20] == 0x0
  OPC_Decode, 229, 34, 25,                   //  3387:      decode to LD1H_4Z_STRIDED_IMM using decoder 25
                                             //  3387:     }
  2, 0,                                      //  3391:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3393:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3395:      check Inst[20] == 0x0
  OPC_Decode, 252, 38, 25,                   //  3399:      decode to LDNT1H_4Z_STRIDED_IMM using decoder 25
                                             //  3399:     }
                                             //  3399:     } // switch Inst[3:2]
                                             //  3399:    }
  6, 27,                                     //  3403:    case 0x6: {
  OPC_SwitchField, 2, 2,                     //  3405:     switch Inst[3:2] {
  0, 10,                                     //  3408:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3410:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3412:      check Inst[20] == 0x0
  OPC_Decode, 223, 35, 25,                   //  3416:      decode to LD1W_4Z_STRIDED_IMM using decoder 25
                                             //  3416:     }
  2, 0,                                      //  3420:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3422:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3424:      check Inst[20] == 0x0
  OPC_Decode, 141, 39, 25,                   //  3428:      decode to LDNT1W_4Z_STRIDED_IMM using decoder 25
                                             //  3428:     }
                                             //  3428:     } // switch Inst[3:2]
                                             //  3428:    }
  7, 0,                                      //  3432:    case 0x7: {
  OPC_SwitchField, 2, 2,                     //  3434:     switch Inst[3:2] {
  0, 10,                                     //  3437:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3439:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3441:      check Inst[20] == 0x0
  OPC_Decode, 201, 34, 25,                   //  3445:      decode to LD1D_4Z_STRIDED_IMM using decoder 25
                                             //  3445:     }
  2, 0,                                      //  3449:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3451:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3453:      check Inst[20] == 0x0
  OPC_Decode, 241, 38, 25,                   //  3457:      decode to LDNT1D_4Z_STRIDED_IMM using decoder 25
                                             //  3457:     }
                                             //  3457:     } // switch Inst[3:2]
                                             //  3457:    }
                                             //  3457:    } // switch Inst[15:13]
                                             //  3457:   }
  11, 235, 1,                                //  3461:   case 0xb: {
  OPC_SwitchField, 13, 3,                    //  3464:    switch Inst[15:13] {
  0, 27,                                     //  3467:    case 0x0: {
  OPC_SwitchField, 3, 1,                     //  3469:     switch Inst[3] {
  0, 10,                                     //  3472:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3474:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3476:      check Inst[20] == 0x0
  OPC_Decode, 164, 56, 24,                   //  3480:      decode to ST1B_2Z_STRIDED_IMM using decoder 24
                                             //  3480:     }
  1, 0,                                      //  3484:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3486:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3488:      check Inst[20] == 0x0
  OPC_Decode, 235, 58, 24,                   //  3492:      decode to STNT1B_2Z_STRIDED_IMM using decoder 24
                                             //  3492:     }
                                             //  3492:     } // switch Inst[3]
                                             //  3492:    }
  1, 27,                                     //  3496:    case 0x1: {
  OPC_SwitchField, 3, 1,                     //  3498:     switch Inst[3] {
  0, 10,                                     //  3501:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3503:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3505:      check Inst[20] == 0x0
  OPC_Decode, 208, 56, 24,                   //  3509:      decode to ST1H_2Z_STRIDED_IMM using decoder 24
                                             //  3509:     }
  1, 0,                                      //  3513:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3515:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3517:      check Inst[20] == 0x0
  OPC_Decode, 130, 59, 24,                   //  3521:      decode to STNT1H_2Z_STRIDED_IMM using decoder 24
                                             //  3521:     }
                                             //  3521:     } // switch Inst[3]
                                             //  3521:    }
  2, 27,                                     //  3525:    case 0x2: {
  OPC_SwitchField, 3, 1,                     //  3527:     switch Inst[3] {
  0, 10,                                     //  3530:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3532:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3534:      check Inst[20] == 0x0
  OPC_Decode, 142, 57, 24,                   //  3538:      decode to ST1W_2Z_STRIDED_IMM using decoder 24
                                             //  3538:     }
  1, 0,                                      //  3542:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3544:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3546:      check Inst[20] == 0x0
  OPC_Decode, 142, 59, 24,                   //  3550:      decode to STNT1W_2Z_STRIDED_IMM using decoder 24
                                             //  3550:     }
                                             //  3550:     } // switch Inst[3]
                                             //  3550:    }
  3, 27,                                     //  3554:    case 0x3: {
  OPC_SwitchField, 3, 1,                     //  3556:     switch Inst[3] {
  0, 10,                                     //  3559:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3561:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3563:      check Inst[20] == 0x0
  OPC_Decode, 180, 56, 24,                   //  3567:      decode to ST1D_2Z_STRIDED_IMM using decoder 24
                                             //  3567:     }
  1, 0,                                      //  3571:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  3573:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3575:      check Inst[20] == 0x0
  OPC_Decode, 247, 58, 24,                   //  3579:      decode to STNT1D_2Z_STRIDED_IMM using decoder 24
                                             //  3579:     }
                                             //  3579:     } // switch Inst[3]
                                             //  3579:    }
  4, 27,                                     //  3583:    case 0x4: {
  OPC_SwitchField, 2, 2,                     //  3585:     switch Inst[3:2] {
  0, 10,                                     //  3588:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3590:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3592:      check Inst[20] == 0x0
  OPC_Decode, 168, 56, 25,                   //  3596:      decode to ST1B_4Z_STRIDED_IMM using decoder 25
                                             //  3596:     }
  2, 0,                                      //  3600:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3602:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3604:      check Inst[20] == 0x0
  OPC_Decode, 239, 58, 25,                   //  3608:      decode to STNT1B_4Z_STRIDED_IMM using decoder 25
                                             //  3608:     }
                                             //  3608:     } // switch Inst[3:2]
                                             //  3608:    }
  5, 27,                                     //  3612:    case 0x5: {
  OPC_SwitchField, 2, 2,                     //  3614:     switch Inst[3:2] {
  0, 10,                                     //  3617:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3619:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3621:      check Inst[20] == 0x0
  OPC_Decode, 212, 56, 25,                   //  3625:      decode to ST1H_4Z_STRIDED_IMM using decoder 25
                                             //  3625:     }
  2, 0,                                      //  3629:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3631:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3633:      check Inst[20] == 0x0
  OPC_Decode, 134, 59, 25,                   //  3637:      decode to STNT1H_4Z_STRIDED_IMM using decoder 25
                                             //  3637:     }
                                             //  3637:     } // switch Inst[3:2]
                                             //  3637:    }
  6, 27,                                     //  3641:    case 0x6: {
  OPC_SwitchField, 2, 2,                     //  3643:     switch Inst[3:2] {
  0, 10,                                     //  3646:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3648:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3650:      check Inst[20] == 0x0
  OPC_Decode, 146, 57, 25,                   //  3654:      decode to ST1W_4Z_STRIDED_IMM using decoder 25
                                             //  3654:     }
  2, 0,                                      //  3658:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3660:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3662:      check Inst[20] == 0x0
  OPC_Decode, 146, 59, 25,                   //  3666:      decode to STNT1W_4Z_STRIDED_IMM using decoder 25
                                             //  3666:     }
                                             //  3666:     } // switch Inst[3:2]
                                             //  3666:    }
  7, 0,                                      //  3670:    case 0x7: {
  OPC_SwitchField, 2, 2,                     //  3672:     switch Inst[3:2] {
  0, 10,                                     //  3675:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  3677:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3679:      check Inst[20] == 0x0
  OPC_Decode, 184, 56, 25,                   //  3683:      decode to ST1D_4Z_STRIDED_IMM using decoder 25
                                             //  3683:     }
  2, 0,                                      //  3687:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  3689:      check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  3691:      check Inst[20] == 0x0
  OPC_Decode, 251, 58, 25,                   //  3695:      decode to STNT1D_4Z_STRIDED_IMM using decoder 25
                                             //  3695:     }
                                             //  3695:     } // switch Inst[3:2]
                                             //  3695:    }
                                             //  3695:    } // switch Inst[15:13]
                                             //  3695:   }
  12, 35,                                    //  3699:   case 0xc: {
  OPC_SwitchField, 2, 3,                     //  3701:    switch Inst[4:2] {
  0, 6,                                      //  3704:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  3706:     check predicate 6
  OPC_Decode, 210, 68, 11,                   //  3708:     decode to USMOPA_MPPZZ_S using decoder 11
                                             //  3708:    }
  2, 6,                                      //  3712:    case 0x2: {
  OPC_CheckPredicate, 7,                     //  3714:     check predicate 7
  OPC_Decode, 203, 65, 11,                   //  3716:     decode to UMOPA_MPPZZ_HtoS using decoder 11
                                             //  3716:    }
  4, 6,                                      //  3720:    case 0x4: {
  OPC_CheckPredicate, 6,                     //  3722:     check predicate 6
  OPC_Decode, 212, 68, 11,                   //  3724:     decode to USMOPS_MPPZZ_S using decoder 11
                                             //  3724:    }
  6, 0,                                      //  3728:    case 0x6: {
  OPC_CheckPredicate, 7,                     //  3730:     check predicate 7
  OPC_Decode, 206, 65, 11,                   //  3732:     decode to UMOPS_MPPZZ_HtoS using decoder 11
                                             //  3732:    }
                                             //  3732:    } // switch Inst[4:2]
                                             //  3732:   }
  13, 19,                                    //  3736:   case 0xd: {
  OPC_SwitchField, 2, 3,                     //  3738:    switch Inst[4:2] {
  0, 6,                                      //  3741:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  3743:     check predicate 6
  OPC_Decode, 204, 65, 11,                   //  3745:     decode to UMOPA_MPPZZ_S using decoder 11
                                             //  3745:    }
  4, 0,                                      //  3749:    case 0x4: {
  OPC_CheckPredicate, 6,                     //  3751:     check predicate 6
  OPC_Decode, 207, 65, 11,                   //  3753:     decode to UMOPS_MPPZZ_S using decoder 11
                                             //  3753:    }
                                             //  3753:    } // switch Inst[4:2]
                                             //  3753:   }
  14, 145, 1,                                //  3757:   case 0xe: {
  OPC_SwitchField, 3, 2,                     //  3760:    switch Inst[4:3] {
  0, 6,                                      //  3763:    case 0x0: {
  OPC_CheckPredicate, 19,                    //  3765:     check predicate 19
  OPC_Decode, 209, 68, 13,                   //  3767:     decode to USMOPA_MPPZZ_D using decoder 13
                                             //  3767:    }
  1, 61,                                     //  3771:    case 0x1: {
  OPC_SwitchField, 9, 8,                     //  3773:     switch Inst[16:9] {
  0, 27,                                     //  3776:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  3778:      switch Inst[20] {
  0, 10,                                     //  3781:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  3783:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3785:       check Inst[5] == 0x0
  OPC_Decode, 200, 68, 14,                   //  3789:       decode to USMOP4A_MZZ_HtoD using decoder 14
                                             //  3789:      }
  1, 0,                                      //  3793:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  3795:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3797:       check Inst[5] == 0x0
  OPC_Decode, 198, 68, 15,                   //  3801:       decode to USMOP4A_MZ2Z_HtoD using decoder 15
                                             //  3801:      }
                                             //  3801:      } // switch Inst[20]
                                             //  3801:     }
  1, 0,                                      //  3805:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  3807:      switch Inst[20] {
  0, 10,                                     //  3810:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  3812:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3814:       check Inst[5] == 0x0
  OPC_Decode, 196, 68, 16,                   //  3818:       decode to USMOP4A_M2ZZ_HtoD using decoder 16
                                             //  3818:      }
  1, 0,                                      //  3822:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  3824:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3826:       check Inst[5] == 0x0
  OPC_Decode, 194, 68, 17,                   //  3830:       decode to USMOP4A_M2Z2Z_HtoD using decoder 17
                                             //  3830:      }
                                             //  3830:      } // switch Inst[20]
                                             //  3830:     }
                                             //  3830:     } // switch Inst[16:9]
                                             //  3830:    }
  2, 6,                                      //  3834:    case 0x2: {
  OPC_CheckPredicate, 19,                    //  3836:     check predicate 19
  OPC_Decode, 211, 68, 13,                   //  3838:     decode to USMOPS_MPPZZ_D using decoder 13
                                             //  3838:    }
  3, 0,                                      //  3842:    case 0x3: {
  OPC_SwitchField, 9, 8,                     //  3844:     switch Inst[16:9] {
  0, 27,                                     //  3847:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  3849:      switch Inst[20] {
  0, 10,                                     //  3852:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  3854:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3856:       check Inst[5] == 0x0
  OPC_Decode, 208, 68, 14,                   //  3860:       decode to USMOP4S_MZZ_HtoD using decoder 14
                                             //  3860:      }
  1, 0,                                      //  3864:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  3866:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3868:       check Inst[5] == 0x0
  OPC_Decode, 206, 68, 15,                   //  3872:       decode to USMOP4S_MZ2Z_HtoD using decoder 15
                                             //  3872:      }
                                             //  3872:      } // switch Inst[20]
                                             //  3872:     }
  1, 0,                                      //  3876:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  3878:      switch Inst[20] {
  0, 10,                                     //  3881:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  3883:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3885:       check Inst[5] == 0x0
  OPC_Decode, 204, 68, 16,                   //  3889:       decode to USMOP4S_M2ZZ_HtoD using decoder 16
                                             //  3889:      }
  1, 0,                                      //  3893:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  3895:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3897:       check Inst[5] == 0x0
  OPC_Decode, 202, 68, 17,                   //  3901:       decode to USMOP4S_M2Z2Z_HtoD using decoder 17
                                             //  3901:      }
                                             //  3901:      } // switch Inst[20]
                                             //  3901:     }
                                             //  3901:     } // switch Inst[16:9]
                                             //  3901:    }
                                             //  3901:    } // switch Inst[4:3]
                                             //  3901:   }
  15, 0,                                     //  3905:   case 0xf: {
  OPC_SwitchField, 3, 2,                     //  3907:    switch Inst[4:3] {
  0, 6,                                      //  3910:    case 0x0: {
  OPC_CheckPredicate, 19,                    //  3912:     check predicate 19
  OPC_Decode, 202, 65, 13,                   //  3914:     decode to UMOPA_MPPZZ_D using decoder 13
                                             //  3914:    }
  1, 61,                                     //  3918:    case 0x1: {
  OPC_SwitchField, 9, 8,                     //  3920:     switch Inst[16:9] {
  0, 27,                                     //  3923:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  3925:      switch Inst[20] {
  0, 10,                                     //  3928:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  3930:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3932:       check Inst[5] == 0x0
  OPC_Decode, 189, 65, 14,                   //  3936:       decode to UMOP4A_MZZ_HtoD using decoder 14
                                             //  3936:      }
  1, 0,                                      //  3940:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  3942:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3944:       check Inst[5] == 0x0
  OPC_Decode, 186, 65, 15,                   //  3948:       decode to UMOP4A_MZ2Z_HtoD using decoder 15
                                             //  3948:      }
                                             //  3948:      } // switch Inst[20]
                                             //  3948:     }
  1, 0,                                      //  3952:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  3954:      switch Inst[20] {
  0, 10,                                     //  3957:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  3959:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3961:       check Inst[5] == 0x0
  OPC_Decode, 183, 65, 16,                   //  3965:       decode to UMOP4A_M2ZZ_HtoD using decoder 16
                                             //  3965:      }
  1, 0,                                      //  3969:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  3971:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  3973:       check Inst[5] == 0x0
  OPC_Decode, 180, 65, 17,                   //  3977:       decode to UMOP4A_M2Z2Z_HtoD using decoder 17
                                             //  3977:      }
                                             //  3977:      } // switch Inst[20]
                                             //  3977:     }
                                             //  3977:     } // switch Inst[16:9]
                                             //  3977:    }
  2, 6,                                      //  3981:    case 0x2: {
  OPC_CheckPredicate, 19,                    //  3983:     check predicate 19
  OPC_Decode, 205, 65, 13,                   //  3985:     decode to UMOPS_MPPZZ_D using decoder 13
                                             //  3985:    }
  3, 0,                                      //  3989:    case 0x3: {
  OPC_SwitchField, 9, 8,                     //  3991:     switch Inst[16:9] {
  0, 27,                                     //  3994:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  3996:      switch Inst[20] {
  0, 10,                                     //  3999:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  4001:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  4003:       check Inst[5] == 0x0
  OPC_Decode, 201, 65, 14,                   //  4007:       decode to UMOP4S_MZZ_HtoD using decoder 14
                                             //  4007:      }
  1, 0,                                      //  4011:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  4013:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  4015:       check Inst[5] == 0x0
  OPC_Decode, 198, 65, 15,                   //  4019:       decode to UMOP4S_MZ2Z_HtoD using decoder 15
                                             //  4019:      }
                                             //  4019:      } // switch Inst[20]
                                             //  4019:     }
  1, 0,                                      //  4023:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  4025:      switch Inst[20] {
  0, 10,                                     //  4028:      case 0x0: {
  OPC_CheckPredicate, 20,                    //  4030:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  4032:       check Inst[5] == 0x0
  OPC_Decode, 195, 65, 16,                   //  4036:       decode to UMOP4S_M2ZZ_HtoD using decoder 16
                                             //  4036:      }
  1, 0,                                      //  4040:      case 0x1: {
  OPC_CheckPredicate, 20,                    //  4042:       check predicate 20
  OPC_CheckField, 5, 1, 0,                   //  4044:       check Inst[5] == 0x0
  OPC_Decode, 192, 65, 17,                   //  4048:       decode to UMOP4S_M2Z2Z_HtoD using decoder 17
                                             //  4048:      }
                                             //  4048:      } // switch Inst[20]
                                             //  4048:     }
                                             //  4048:     } // switch Inst[16:9]
                                             //  4048:    }
                                             //  4048:    } // switch Inst[4:3]
                                             //  4048:   }
                                             //  4048:   } // switch Inst[25:21]
                                             //  4048:  }
  6, 132, 100,                               //  4052:  case 0x6: {
  OPC_SwitchField, 23, 3,                    //  4055:   switch Inst[25:23] {
  0, 243, 5,                                 //  4058:   case 0x0: {
  OPC_SwitchField, 15, 8,                    //  4061:    switch Inst[22:15] {
  0, 10,                                     //  4064:    case 0x0: {
  OPC_CheckPredicate, 6,                     //  4066:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  4068:     check Inst[4] == 0x0
  OPC_Decode, 129, 34, 26,                   //  4072:     decode to INSERT_MXIPZ_H_B using decoder 26
                                             //  4072:    }
  1, 10,                                     //  4076:    case 0x1: {
  OPC_CheckPredicate, 6,                     //  4078:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  4080:     check Inst[4] == 0x0
  OPC_Decode, 134, 34, 26,                   //  4084:     decode to INSERT_MXIPZ_V_B using decoder 26
                                             //  4084:    }
  4, 23,                                     //  4088:    case 0x4: {
  OPC_SwitchField, 9, 1,                     //  4090:     switch Inst[9] {
  0, 6,                                      //  4093:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  4095:      check predicate 6
  OPC_Decode, 152, 21, 27,                   //  4097:      decode to EXTRACT_ZPMXI_H_B using decoder 27
                                             //  4097:     }
  1, 0,                                      //  4101:     case 0x1: {
  OPC_CheckPredicate, 21,                    //  4103:      check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  4105:      check Inst[12:10] == 0x0
  OPC_Decode, 168, 42, 28,                   //  4109:      decode to MOVAZ_ZMI_H_B using decoder 28
                                             //  4109:     }
                                             //  4109:     } // switch Inst[9]
                                             //  4109:    }
  5, 23,                                     //  4113:    case 0x5: {
  OPC_SwitchField, 9, 1,                     //  4115:     switch Inst[9] {
  0, 6,                                      //  4118:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  4120:      check predicate 6
  OPC_Decode, 157, 21, 27,                   //  4122:      decode to EXTRACT_ZPMXI_V_B using decoder 27
                                             //  4122:     }
  1, 0,                                      //  4126:     case 0x1: {
  OPC_CheckPredicate, 21,                    //  4128:      check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  4130:      check Inst[12:10] == 0x0
  OPC_Decode, 173, 42, 28,                   //  4134:      decode to MOVAZ_ZMI_V_B using decoder 28
                                             //  4134:     }
                                             //  4134:     } // switch Inst[9]
                                             //  4134:    }
  8, 51,                                     //  4138:    case 0x8: {
  OPC_SwitchField, 10, 3,                    //  4140:     switch Inst[12:10] {
  0, 10,                                     //  4143:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4145:      check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4147:      check Inst[5:3] == 0x0
  OPC_Decode, 194, 42, 29,                   //  4151:      decode to MOVA_MXI2Z_H_B using decoder 29
                                             //  4151:     }
  1, 10,                                     //  4155:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  4157:      check predicate 7
  OPC_CheckField, 2, 5, 0,                   //  4159:      check Inst[6:2] == 0x0
  OPC_Decode, 202, 42, 30,                   //  4163:      decode to MOVA_MXI4Z_H_B using decoder 30
                                             //  4163:     }
  2, 10,                                     //  4167:     case 0x2: {
  OPC_CheckPredicate, 7,                     //  4169:      check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4171:      check Inst[5:3] == 0x0
  OPC_Decode, 211, 42, 31,                   //  4175:      decode to MOVA_VG2_MXI2Z using decoder 31
                                             //  4175:     }
  3, 0,                                      //  4179:     case 0x3: {
  OPC_CheckPredicate, 7,                     //  4181:      check predicate 7
  OPC_CheckField, 3, 4, 0,                   //  4183:      check Inst[6:3] == 0x0
  OPC_Decode, 213, 42, 32,                   //  4187:      decode to MOVA_VG4_MXI4Z using decoder 32
                                             //  4187:     }
                                             //  4187:     } // switch Inst[12:10]
                                             //  4187:    }
  9, 27,                                     //  4191:    case 0x9: {
  OPC_SwitchField, 10, 3,                    //  4193:     switch Inst[12:10] {
  0, 10,                                     //  4196:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4198:      check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4200:      check Inst[5:3] == 0x0
  OPC_Decode, 198, 42, 29,                   //  4204:      decode to MOVA_MXI2Z_V_B using decoder 29
                                             //  4204:     }
  1, 0,                                      //  4208:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  4210:      check predicate 7
  OPC_CheckField, 2, 5, 0,                   //  4212:      check Inst[6:2] == 0x0
  OPC_Decode, 206, 42, 30,                   //  4216:      decode to MOVA_MXI4Z_V_B using decoder 30
                                             //  4216:     }
                                             //  4216:     } // switch Inst[12:10]
                                             //  4216:    }
  12, 107,                                   //  4220:    case 0xc: {
  OPC_SwitchField, 8, 5,                     //  4222:     switch Inst[12:8] {
  0, 10,                                     //  4225:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4227:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  4229:      check Inst[0] == 0x0
  OPC_Decode, 178, 42, 33,                   //  4233:      decode to MOVA_2ZMXI_H_B using decoder 33
                                             //  4233:     }
  2, 10,                                     //  4237:     case 0x2: {
  OPC_CheckPredicate, 21,                    //  4239:      check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  4241:      check Inst[0] == 0x0
  OPC_Decode, 150, 42, 34,                   //  4245:      decode to MOVAZ_2ZMI_H_B using decoder 34
                                             //  4245:     }
  4, 14,                                     //  4249:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  4251:      check predicate 7
  OPC_CheckField, 7, 1, 0,                   //  4253:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4257:      check Inst[1:0] == 0x0
  OPC_Decode, 186, 42, 35,                   //  4261:      decode to MOVA_4ZMXI_H_B using decoder 35
                                             //  4261:     }
  6, 14,                                     //  4265:     case 0x6: {
  OPC_CheckPredicate, 21,                    //  4267:      check predicate 21
  OPC_CheckField, 7, 1, 0,                   //  4269:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4273:      check Inst[1:0] == 0x0
  OPC_Decode, 158, 42, 36,                   //  4277:      decode to MOVAZ_4ZMI_H_B using decoder 36
                                             //  4277:     }
  8, 10,                                     //  4281:     case 0x8: {
  OPC_CheckPredicate, 7,                     //  4283:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  4285:      check Inst[0] == 0x0
  OPC_Decode, 210, 42, 37,                   //  4289:      decode to MOVA_VG2_2ZMXI using decoder 37
                                             //  4289:     }
  10, 10,                                    //  4293:     case 0xa: {
  OPC_CheckPredicate, 21,                    //  4295:      check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  4297:      check Inst[0] == 0x0
  OPC_Decode, 166, 42, 38,                   //  4301:      decode to MOVAZ_VG2_2ZMXI using decoder 38
                                             //  4301:     }
  12, 10,                                    //  4305:     case 0xc: {
  OPC_CheckPredicate, 7,                     //  4307:      check predicate 7
  OPC_CheckField, 0, 2, 0,                   //  4309:      check Inst[1:0] == 0x0
  OPC_Decode, 212, 42, 39,                   //  4313:      decode to MOVA_VG4_4ZMXI using decoder 39
                                             //  4313:     }
  14, 0,                                     //  4317:     case 0xe: {
  OPC_CheckPredicate, 21,                    //  4319:      check predicate 21
  OPC_CheckField, 0, 2, 0,                   //  4321:      check Inst[1:0] == 0x0
  OPC_Decode, 167, 42, 40,                   //  4325:      decode to MOVAZ_VG4_4ZMXI using decoder 40
                                             //  4325:     }
                                             //  4325:     } // switch Inst[12:8]
                                             //  4325:    }
  13, 59,                                    //  4329:    case 0xd: {
  OPC_SwitchField, 8, 5,                     //  4331:     switch Inst[12:8] {
  0, 10,                                     //  4334:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4336:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  4338:      check Inst[0] == 0x0
  OPC_Decode, 182, 42, 33,                   //  4342:      decode to MOVA_2ZMXI_V_B using decoder 33
                                             //  4342:     }
  2, 10,                                     //  4346:     case 0x2: {
  OPC_CheckPredicate, 21,                    //  4348:      check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  4350:      check Inst[0] == 0x0
  OPC_Decode, 154, 42, 34,                   //  4354:      decode to MOVAZ_2ZMI_V_B using decoder 34
                                             //  4354:     }
  4, 14,                                     //  4358:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  4360:      check predicate 7
  OPC_CheckField, 7, 1, 0,                   //  4362:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4366:      check Inst[1:0] == 0x0
  OPC_Decode, 190, 42, 35,                   //  4370:      decode to MOVA_4ZMXI_V_B using decoder 35
                                             //  4370:     }
  6, 0,                                      //  4374:     case 0x6: {
  OPC_CheckPredicate, 21,                    //  4376:      check predicate 21
  OPC_CheckField, 7, 1, 0,                   //  4378:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4382:      check Inst[1:0] == 0x0
  OPC_Decode, 162, 42, 36,                   //  4386:      decode to MOVAZ_4ZMI_V_B using decoder 36
                                             //  4386:     }
                                             //  4386:     } // switch Inst[12:8]
                                             //  4386:    }
  16, 10,                                    //  4390:    case 0x10: {
  OPC_CheckPredicate, 6,                     //  4392:     check predicate 6
  OPC_CheckField, 8, 7, 0,                   //  4394:     check Inst[14:8] == 0x0
  OPC_Decode, 243, 70, 41,                   //  4398:     decode to ZERO_M using decoder 41
                                             //  4398:    }
  24, 10,                                    //  4402:    case 0x18: {
  OPC_CheckPredicate, 21,                    //  4404:     check predicate 21
  OPC_CheckField, 3, 10, 0,                  //  4406:     check Inst[12:3] == 0x0
  OPC_Decode, 248, 70, 42,                   //  4410:     decode to ZERO_MXI_VG2_Z using decoder 42
                                             //  4410:    }
  25, 10,                                    //  4414:    case 0x19: {
  OPC_CheckPredicate, 21,                    //  4416:     check predicate 21
  OPC_CheckField, 3, 10, 0,                  //  4418:     check Inst[12:3] == 0x0
  OPC_Decode, 244, 70, 42,                   //  4422:     decode to ZERO_MXI_2Z using decoder 42
                                             //  4422:    }
  26, 10,                                    //  4426:    case 0x1a: {
  OPC_CheckPredicate, 21,                    //  4428:     check predicate 21
  OPC_CheckField, 2, 11, 0,                  //  4430:     check Inst[12:2] == 0x0
  OPC_Decode, 246, 70, 43,                   //  4434:     decode to ZERO_MXI_VG2_2Z using decoder 43
                                             //  4434:    }
  27, 10,                                    //  4438:    case 0x1b: {
  OPC_CheckPredicate, 21,                    //  4440:     check predicate 21
  OPC_CheckField, 2, 11, 0,                  //  4442:     check Inst[12:2] == 0x0
  OPC_Decode, 249, 70, 43,                   //  4446:     decode to ZERO_MXI_VG4_2Z using decoder 43
                                             //  4446:    }
  28, 10,                                    //  4450:    case 0x1c: {
  OPC_CheckPredicate, 21,                    //  4452:     check predicate 21
  OPC_CheckField, 3, 10, 0,                  //  4454:     check Inst[12:3] == 0x0
  OPC_Decode, 251, 70, 42,                   //  4458:     decode to ZERO_MXI_VG4_Z using decoder 42
                                             //  4458:    }
  29, 10,                                    //  4462:    case 0x1d: {
  OPC_CheckPredicate, 21,                    //  4464:     check predicate 21
  OPC_CheckField, 2, 11, 0,                  //  4466:     check Inst[12:2] == 0x0
  OPC_Decode, 245, 70, 43,                   //  4470:     decode to ZERO_MXI_4Z using decoder 43
                                             //  4470:    }
  30, 10,                                    //  4474:    case 0x1e: {
  OPC_CheckPredicate, 21,                    //  4476:     check predicate 21
  OPC_CheckField, 1, 12, 0,                  //  4478:     check Inst[12:1] == 0x0
  OPC_Decode, 247, 70, 44,                   //  4482:     decode to ZERO_MXI_VG2_4Z using decoder 44
                                             //  4482:    }
  31, 10,                                    //  4486:    case 0x1f: {
  OPC_CheckPredicate, 21,                    //  4488:     check predicate 21
  OPC_CheckField, 1, 12, 0,                  //  4490:     check Inst[12:1] == 0x0
  OPC_Decode, 250, 70, 44,                   //  4494:     decode to ZERO_MXI_VG4_4Z using decoder 44
                                             //  4494:    }
  128, 1, 10,                                //  4498:    case 0x80: {
  OPC_CheckPredicate, 6,                     //  4501:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  4503:     check Inst[4] == 0x0
  OPC_Decode, 131, 34, 45,                   //  4507:     decode to INSERT_MXIPZ_H_H using decoder 45
                                             //  4507:    }
  129, 1, 10,                                //  4511:    case 0x81: {
  OPC_CheckPredicate, 6,                     //  4514:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  4516:     check Inst[4] == 0x0
  OPC_Decode, 136, 34, 45,                   //  4520:     decode to INSERT_MXIPZ_V_H using decoder 45
                                             //  4520:    }
  132, 1, 23,                                //  4524:    case 0x84: {
  OPC_SwitchField, 9, 1,                     //  4527:     switch Inst[9] {
  0, 6,                                      //  4530:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  4532:      check predicate 6
  OPC_Decode, 154, 21, 46,                   //  4534:      decode to EXTRACT_ZPMXI_H_H using decoder 46
                                             //  4534:     }
  1, 0,                                      //  4538:     case 0x1: {
  OPC_CheckPredicate, 21,                    //  4540:      check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  4542:      check Inst[12:10] == 0x0
  OPC_Decode, 170, 42, 47,                   //  4546:      decode to MOVAZ_ZMI_H_H using decoder 47
                                             //  4546:     }
                                             //  4546:     } // switch Inst[9]
                                             //  4546:    }
  133, 1, 23,                                //  4550:    case 0x85: {
  OPC_SwitchField, 9, 1,                     //  4553:     switch Inst[9] {
  0, 6,                                      //  4556:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  4558:      check predicate 6
  OPC_Decode, 159, 21, 46,                   //  4560:      decode to EXTRACT_ZPMXI_V_H using decoder 46
                                             //  4560:     }
  1, 0,                                      //  4564:     case 0x1: {
  OPC_CheckPredicate, 21,                    //  4566:      check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  4568:      check Inst[12:10] == 0x0
  OPC_Decode, 175, 42, 47,                   //  4572:      decode to MOVAZ_ZMI_V_H using decoder 47
                                             //  4572:     }
                                             //  4572:     } // switch Inst[9]
                                             //  4572:    }
  136, 1, 27,                                //  4576:    case 0x88: {
  OPC_SwitchField, 10, 3,                    //  4579:     switch Inst[12:10] {
  0, 10,                                     //  4582:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4584:      check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4586:      check Inst[5:3] == 0x0
  OPC_Decode, 196, 42, 48,                   //  4590:      decode to MOVA_MXI2Z_H_H using decoder 48
                                             //  4590:     }
  1, 0,                                      //  4594:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  4596:      check predicate 7
  OPC_CheckField, 2, 5, 0,                   //  4598:      check Inst[6:2] == 0x0
  OPC_Decode, 204, 42, 49,                   //  4602:      decode to MOVA_MXI4Z_H_H using decoder 49
                                             //  4602:     }
                                             //  4602:     } // switch Inst[12:10]
                                             //  4602:    }
  137, 1, 27,                                //  4606:    case 0x89: {
  OPC_SwitchField, 10, 3,                    //  4609:     switch Inst[12:10] {
  0, 10,                                     //  4612:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4614:      check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4616:      check Inst[5:3] == 0x0
  OPC_Decode, 200, 42, 48,                   //  4620:      decode to MOVA_MXI2Z_V_H using decoder 48
                                             //  4620:     }
  1, 0,                                      //  4624:     case 0x1: {
  OPC_CheckPredicate, 7,                     //  4626:      check predicate 7
  OPC_CheckField, 2, 5, 0,                   //  4628:      check Inst[6:2] == 0x0
  OPC_Decode, 208, 42, 49,                   //  4632:      decode to MOVA_MXI4Z_V_H using decoder 49
                                             //  4632:     }
                                             //  4632:     } // switch Inst[12:10]
                                             //  4632:    }
  140, 1, 59,                                //  4636:    case 0x8c: {
  OPC_SwitchField, 8, 5,                     //  4639:     switch Inst[12:8] {
  0, 10,                                     //  4642:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4644:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  4646:      check Inst[0] == 0x0
  OPC_Decode, 180, 42, 50,                   //  4650:      decode to MOVA_2ZMXI_H_H using decoder 50
                                             //  4650:     }
  2, 10,                                     //  4654:     case 0x2: {
  OPC_CheckPredicate, 21,                    //  4656:      check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  4658:      check Inst[0] == 0x0
  OPC_Decode, 152, 42, 51,                   //  4662:      decode to MOVAZ_2ZMI_H_H using decoder 51
                                             //  4662:     }
  4, 14,                                     //  4666:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  4668:      check predicate 7
  OPC_CheckField, 7, 1, 0,                   //  4670:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4674:      check Inst[1:0] == 0x0
  OPC_Decode, 188, 42, 52,                   //  4678:      decode to MOVA_4ZMXI_H_H using decoder 52
                                             //  4678:     }
  6, 0,                                      //  4682:     case 0x6: {
  OPC_CheckPredicate, 21,                    //  4684:      check predicate 21
  OPC_CheckField, 7, 1, 0,                   //  4686:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4690:      check Inst[1:0] == 0x0
  OPC_Decode, 160, 42, 53,                   //  4694:      decode to MOVAZ_4ZMI_H_H using decoder 53
                                             //  4694:     }
                                             //  4694:     } // switch Inst[12:8]
                                             //  4694:    }
  141, 1, 59,                                //  4698:    case 0x8d: {
  OPC_SwitchField, 8, 5,                     //  4701:     switch Inst[12:8] {
  0, 10,                                     //  4704:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  4706:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  4708:      check Inst[0] == 0x0
  OPC_Decode, 184, 42, 50,                   //  4712:      decode to MOVA_2ZMXI_V_H using decoder 50
                                             //  4712:     }
  2, 10,                                     //  4716:     case 0x2: {
  OPC_CheckPredicate, 21,                    //  4718:      check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  4720:      check Inst[0] == 0x0
  OPC_Decode, 156, 42, 51,                   //  4724:      decode to MOVAZ_2ZMI_V_H using decoder 51
                                             //  4724:     }
  4, 14,                                     //  4728:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  4730:      check predicate 7
  OPC_CheckField, 7, 1, 0,                   //  4732:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4736:      check Inst[1:0] == 0x0
  OPC_Decode, 192, 42, 52,                   //  4740:      decode to MOVA_4ZMXI_V_H using decoder 52
                                             //  4740:     }
  6, 0,                                      //  4744:     case 0x6: {
  OPC_CheckPredicate, 21,                    //  4746:      check predicate 21
  OPC_CheckField, 7, 1, 0,                   //  4748:      check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  4752:      check Inst[1:0] == 0x0
  OPC_Decode, 164, 42, 53,                   //  4756:      decode to MOVAZ_4ZMI_V_H using decoder 53
                                             //  4756:     }
                                             //  4756:     } // switch Inst[12:8]
                                             //  4756:    }
  144, 1, 10,                                //  4760:    case 0x90: {
  OPC_CheckPredicate, 7,                     //  4763:     check predicate 7
  OPC_CheckField, 0, 15, 1,                  //  4765:     check Inst[14:0] == 0x1
  OPC_Decode, 252, 70, 54,                   //  4769:     decode to ZERO_T using decoder 54
                                             //  4769:    }
  152, 1, 10,                                //  4773:    case 0x98: {
  OPC_CheckPredicate, 7,                     //  4776:     check predicate 7
  OPC_CheckField, 5, 7, 31,                  //  4778:     check Inst[11:5] == 0x1f
  OPC_Decode, 239, 42, 55,                   //  4782:     decode to MOVT_XTI using decoder 55
                                             //  4782:    }
  156, 1, 10,                                //  4786:    case 0x9c: {
  OPC_CheckPredicate, 7,                     //  4789:     check predicate 7
  OPC_CheckField, 5, 7, 31,                  //  4791:     check Inst[11:5] == 0x1f
  OPC_Decode, 237, 42, 56,                   //  4795:     decode to MOVT_TIX using decoder 56
                                             //  4795:    }
  158, 1, 0,                                 //  4799:    case 0x9e: {
  OPC_CheckPredicate, 22,                    //  4802:     check predicate 22
  OPC_CheckField, 14, 1, 0,                  //  4804:     check Inst[14] == 0x0
  OPC_CheckField, 5, 7, 31,                  //  4808:     check Inst[11:5] == 0x1f
  OPC_Decode, 238, 42, 57,                   //  4812:     decode to MOVT_TIZ using decoder 57
                                             //  4812:    }
                                             //  4812:    } // switch Inst[22:15]
                                             //  4812:   }
  1, 212, 8,                                 //  4816:   case 0x1: {
  OPC_SwitchField, 18, 5,                    //  4819:    switch Inst[22:18] {
  0, 77,                                     //  4822:    case 0x0: {
  OPC_SwitchField, 15, 3,                    //  4824:     switch Inst[17:15] {
  0, 10,                                     //  4827:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  4829:      check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  4831:      check Inst[4] == 0x0
  OPC_Decode, 133, 34, 58,                   //  4835:      decode to INSERT_MXIPZ_H_S using decoder 58
                                             //  4835:     }
  1, 10,                                     //  4839:     case 0x1: {
  OPC_CheckPredicate, 6,                     //  4841:      check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  4843:      check Inst[4] == 0x0
  OPC_Decode, 138, 34, 58,                   //  4847:      decode to INSERT_MXIPZ_V_S using decoder 58
                                             //  4847:     }
  4, 23,                                     //  4851:     case 0x4: {
  OPC_SwitchField, 9, 1,                     //  4853:      switch Inst[9] {
  0, 6,                                      //  4856:      case 0x0: {
  OPC_CheckPredicate, 6,                     //  4858:       check predicate 6
  OPC_Decode, 156, 21, 59,                   //  4860:       decode to EXTRACT_ZPMXI_H_S using decoder 59
                                             //  4860:      }
  1, 0,                                      //  4864:      case 0x1: {
  OPC_CheckPredicate, 21,                    //  4866:       check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  4868:       check Inst[12:10] == 0x0
  OPC_Decode, 172, 42, 60,                   //  4872:       decode to MOVAZ_ZMI_H_S using decoder 60
                                             //  4872:      }
                                             //  4872:      } // switch Inst[9]
                                             //  4872:     }
  5, 0,                                      //  4876:     case 0x5: {
  OPC_SwitchField, 9, 1,                     //  4878:      switch Inst[9] {
  0, 6,                                      //  4881:      case 0x0: {
  OPC_CheckPredicate, 6,                     //  4883:       check predicate 6
  OPC_Decode, 161, 21, 59,                   //  4885:       decode to EXTRACT_ZPMXI_V_S using decoder 59
                                             //  4885:      }
  1, 0,                                      //  4889:      case 0x1: {
  OPC_CheckPredicate, 21,                    //  4891:       check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  4893:       check Inst[12:10] == 0x0
  OPC_Decode, 177, 42, 60,                   //  4897:       decode to MOVAZ_ZMI_V_S using decoder 60
                                             //  4897:      }
                                             //  4897:      } // switch Inst[9]
                                             //  4897:     }
                                             //  4897:     } // switch Inst[17:15]
                                             //  4897:    }
  1, 183, 1,                                 //  4901:    case 0x1: {
  OPC_SwitchField, 15, 3,                    //  4904:     switch Inst[17:15] {
  0, 27,                                     //  4907:     case 0x0: {
  OPC_SwitchField, 10, 3,                    //  4909:      switch Inst[12:10] {
  0, 10,                                     //  4912:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  4914:       check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4916:       check Inst[5:3] == 0x0
  OPC_Decode, 197, 42, 61,                   //  4920:       decode to MOVA_MXI2Z_H_S using decoder 61
                                             //  4920:      }
  1, 0,                                      //  4924:      case 0x1: {
  OPC_CheckPredicate, 7,                     //  4926:       check predicate 7
  OPC_CheckField, 2, 5, 0,                   //  4928:       check Inst[6:2] == 0x0
  OPC_Decode, 205, 42, 62,                   //  4932:       decode to MOVA_MXI4Z_H_S using decoder 62
                                             //  4932:      }
                                             //  4932:      } // switch Inst[12:10]
                                             //  4932:     }
  1, 27,                                     //  4936:     case 0x1: {
  OPC_SwitchField, 10, 3,                    //  4938:      switch Inst[12:10] {
  0, 10,                                     //  4941:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  4943:       check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  4945:       check Inst[5:3] == 0x0
  OPC_Decode, 201, 42, 61,                   //  4949:       decode to MOVA_MXI2Z_V_S using decoder 61
                                             //  4949:      }
  1, 0,                                      //  4953:      case 0x1: {
  OPC_CheckPredicate, 7,                     //  4955:       check predicate 7
  OPC_CheckField, 2, 5, 0,                   //  4957:       check Inst[6:2] == 0x0
  OPC_Decode, 209, 42, 62,                   //  4961:       decode to MOVA_MXI4Z_V_S using decoder 62
                                             //  4961:      }
                                             //  4961:      } // switch Inst[12:10]
                                             //  4961:     }
  4, 59,                                     //  4965:     case 0x4: {
  OPC_SwitchField, 8, 5,                     //  4967:      switch Inst[12:8] {
  0, 10,                                     //  4970:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  4972:       check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  4974:       check Inst[0] == 0x0
  OPC_Decode, 181, 42, 63,                   //  4978:       decode to MOVA_2ZMXI_H_S using decoder 63
                                             //  4978:      }
  2, 10,                                     //  4982:      case 0x2: {
  OPC_CheckPredicate, 21,                    //  4984:       check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  4986:       check Inst[0] == 0x0
  OPC_Decode, 153, 42, 64,                   //  4990:       decode to MOVAZ_2ZMI_H_S using decoder 64
                                             //  4990:      }
  4, 14,                                     //  4994:      case 0x4: {
  OPC_CheckPredicate, 7,                     //  4996:       check predicate 7
  OPC_CheckField, 7, 1, 0,                   //  4998:       check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  5002:       check Inst[1:0] == 0x0
  OPC_Decode, 189, 42, 65,                   //  5006:       decode to MOVA_4ZMXI_H_S using decoder 65
                                             //  5006:      }
  6, 0,                                      //  5010:      case 0x6: {
  OPC_CheckPredicate, 21,                    //  5012:       check predicate 21
  OPC_CheckField, 7, 1, 0,                   //  5014:       check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  5018:       check Inst[1:0] == 0x0
  OPC_Decode, 161, 42, 66,                   //  5022:       decode to MOVAZ_4ZMI_H_S using decoder 66
                                             //  5022:      }
                                             //  5022:      } // switch Inst[12:8]
                                             //  5022:     }
  5, 0,                                      //  5026:     case 0x5: {
  OPC_SwitchField, 8, 5,                     //  5028:      switch Inst[12:8] {
  0, 10,                                     //  5031:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  5033:       check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  5035:       check Inst[0] == 0x0
  OPC_Decode, 185, 42, 63,                   //  5039:       decode to MOVA_2ZMXI_V_S using decoder 63
                                             //  5039:      }
  2, 10,                                     //  5043:      case 0x2: {
  OPC_CheckPredicate, 21,                    //  5045:       check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  5047:       check Inst[0] == 0x0
  OPC_Decode, 157, 42, 64,                   //  5051:       decode to MOVAZ_2ZMI_V_S using decoder 64
                                             //  5051:      }
  4, 14,                                     //  5055:      case 0x4: {
  OPC_CheckPredicate, 7,                     //  5057:       check predicate 7
  OPC_CheckField, 7, 1, 0,                   //  5059:       check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  5063:       check Inst[1:0] == 0x0
  OPC_Decode, 193, 42, 65,                   //  5067:       decode to MOVA_4ZMXI_V_S using decoder 65
                                             //  5067:      }
  6, 0,                                      //  5071:      case 0x6: {
  OPC_CheckPredicate, 21,                    //  5073:       check predicate 21
  OPC_CheckField, 7, 1, 0,                   //  5075:       check Inst[7] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  5079:       check Inst[1:0] == 0x0
  OPC_Decode, 165, 42, 66,                   //  5083:       decode to MOVAZ_4ZMI_V_S using decoder 66
                                             //  5083:      }
                                             //  5083:      } // switch Inst[12:8]
                                             //  5083:     }
                                             //  5083:     } // switch Inst[17:15]
                                             //  5083:    }
  2, 128, 1,                                 //  5087:    case 0x2: {
  OPC_SwitchField, 10, 5,                    //  5090:     switch Inst[14:10] {
  0, 35,                                     //  5093:     case 0x0: {
  OPC_SwitchField, 15, 3,                    //  5095:      switch Inst[17:15] {
  4, 14,                                     //  5098:      case 0x4: {
  OPC_CheckPredicate, 23,                    //  5100:       check predicate 23
  OPC_CheckField, 5, 2, 0,                   //  5102:       check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  5106:       check Inst[1:0] == 0x0
  OPC_Decode, 223, 41, 67,                   //  5110:       decode to LUTI6_4ZT3Z using decoder 67
                                             //  5110:      }
  6, 0,                                      //  5114:      case 0x6: {
  OPC_CheckPredicate, 22,                    //  5116:       check predicate 22
  OPC_CheckField, 5, 1, 0,                   //  5118:       check Inst[5] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  5122:       check Inst[1:0] == 0x0
  OPC_Decode, 211, 41, 68,                   //  5126:       decode to LUTI4_4ZZT2Z using decoder 68
                                             //  5126:      }
                                             //  5126:      } // switch Inst[17:15]
                                             //  5126:     }
  4, 18,                                     //  5130:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  5132:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5134:      check Inst[17] == 0x1
  OPC_CheckField, 15, 1, 1,                  //  5138:      check Inst[15] == 0x1
  OPC_CheckField, 0, 2, 0,                   //  5142:      check Inst[1:0] == 0x0
  OPC_Decode, 209, 41, 69,                   //  5146:      decode to LUTI4_4ZTZI_H using decoder 69
                                             //  5146:     }
  8, 18,                                     //  5150:     case 0x8: {
  OPC_CheckPredicate, 7,                     //  5152:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5154:      check Inst[17] == 0x1
  OPC_CheckField, 15, 1, 1,                  //  5158:      check Inst[15] == 0x1
  OPC_CheckField, 0, 2, 0,                   //  5162:      check Inst[1:0] == 0x0
  OPC_Decode, 210, 41, 69,                   //  5166:      decode to LUTI4_4ZTZI_S using decoder 69
                                             //  5166:     }
  16, 14,                                    //  5170:     case 0x10: {
  OPC_CheckPredicate, 7,                     //  5172:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5174:      check Inst[17] == 0x1
  OPC_CheckField, 0, 1, 0,                   //  5178:      check Inst[0] == 0x0
  OPC_Decode, 206, 41, 70,                   //  5182:      decode to LUTI4_2ZTZI_B using decoder 70
                                             //  5182:     }
  20, 14,                                    //  5186:     case 0x14: {
  OPC_CheckPredicate, 7,                     //  5188:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5190:      check Inst[17] == 0x1
  OPC_CheckField, 0, 1, 0,                   //  5194:      check Inst[0] == 0x0
  OPC_Decode, 207, 41, 70,                   //  5198:      decode to LUTI4_2ZTZI_H using decoder 70
                                             //  5198:     }
  24, 0,                                     //  5202:     case 0x18: {
  OPC_CheckPredicate, 7,                     //  5204:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5206:      check Inst[17] == 0x1
  OPC_CheckField, 0, 1, 0,                   //  5210:      check Inst[0] == 0x0
  OPC_Decode, 208, 41, 70,                   //  5214:      decode to LUTI4_2ZTZI_S using decoder 70
                                             //  5214:     }
                                             //  5214:     } // switch Inst[14:10]
                                             //  5214:    }
  3, 87,                                     //  5218:    case 0x3: {
  OPC_SwitchField, 10, 5,                    //  5220:     switch Inst[14:10] {
  0, 14,                                     //  5223:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  5225:      check predicate 7
  OPC_CheckField, 15, 1, 1,                  //  5227:      check Inst[15] == 0x1
  OPC_CheckField, 0, 2, 0,                   //  5231:      check Inst[1:0] == 0x0
  OPC_Decode, 194, 41, 71,                   //  5235:      decode to LUTI2_4ZTZI_B using decoder 71
                                             //  5235:     }
  4, 14,                                     //  5239:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  5241:      check predicate 7
  OPC_CheckField, 15, 1, 1,                  //  5243:      check Inst[15] == 0x1
  OPC_CheckField, 0, 2, 0,                   //  5247:      check Inst[1:0] == 0x0
  OPC_Decode, 195, 41, 71,                   //  5251:      decode to LUTI2_4ZTZI_H using decoder 71
                                             //  5251:     }
  8, 14,                                     //  5255:     case 0x8: {
  OPC_CheckPredicate, 7,                     //  5257:      check predicate 7
  OPC_CheckField, 15, 1, 1,                  //  5259:      check Inst[15] == 0x1
  OPC_CheckField, 0, 2, 0,                   //  5263:      check Inst[1:0] == 0x0
  OPC_Decode, 196, 41, 71,                   //  5267:      decode to LUTI2_4ZTZI_S using decoder 71
                                             //  5267:     }
  16, 10,                                    //  5271:     case 0x10: {
  OPC_CheckPredicate, 7,                     //  5273:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  5275:      check Inst[0] == 0x0
  OPC_Decode, 191, 41, 72,                   //  5279:      decode to LUTI2_2ZTZI_B using decoder 72
                                             //  5279:     }
  20, 10,                                    //  5283:     case 0x14: {
  OPC_CheckPredicate, 7,                     //  5285:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  5287:      check Inst[0] == 0x0
  OPC_Decode, 192, 41, 72,                   //  5291:      decode to LUTI2_2ZTZI_H using decoder 72
                                             //  5291:     }
  24, 0,                                     //  5295:     case 0x18: {
  OPC_CheckPredicate, 7,                     //  5297:      check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  5299:      check Inst[0] == 0x0
  OPC_Decode, 193, 41, 72,                   //  5303:      decode to LUTI2_2ZTZI_S using decoder 72
                                             //  5303:     }
                                             //  5303:     } // switch Inst[14:10]
                                             //  5303:    }
  4, 27,                                     //  5307:    case 0x4: {
  OPC_SwitchField, 16, 2,                    //  5309:     switch Inst[17:16] {
  0, 10,                                     //  5312:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  5314:      check predicate 6
  OPC_CheckField, 2, 3, 0,                   //  5316:      check Inst[4:2] == 0x0
  OPC_Decode, 136, 13, 73,                   //  5320:      decode to ADDHA_MPPZ_S using decoder 73
                                             //  5320:     }
  1, 0,                                      //  5324:     case 0x1: {
  OPC_CheckPredicate, 6,                     //  5326:      check predicate 6
  OPC_CheckField, 2, 3, 0,                   //  5328:      check Inst[4:2] == 0x0
  OPC_Decode, 185, 13, 73,                   //  5332:      decode to ADDVA_MPPZ_S using decoder 73
                                             //  5332:     }
                                             //  5332:     } // switch Inst[17:16]
                                             //  5332:    }
  6, 92,                                     //  5336:    case 0x6: {
  OPC_SwitchField, 10, 5,                    //  5338:     switch Inst[14:10] {
  0, 35,                                     //  5341:     case 0x0: {
  OPC_SwitchField, 15, 3,                    //  5343:      switch Inst[17:15] {
  4, 14,                                     //  5346:      case 0x4: {
  OPC_CheckPredicate, 23,                    //  5348:       check predicate 23
  OPC_CheckField, 5, 2, 0,                   //  5350:       check Inst[6:5] == 0x0
  OPC_CheckField, 2, 2, 0,                   //  5354:       check Inst[3:2] == 0x0
  OPC_Decode, 225, 41, 74,                   //  5358:       decode to LUTI6_S_4ZT3Z using decoder 74
                                             //  5358:      }
  6, 0,                                      //  5362:      case 0x6: {
  OPC_CheckPredicate, 24,                    //  5364:       check predicate 24
  OPC_CheckField, 5, 1, 0,                   //  5366:       check Inst[5] == 0x0
  OPC_CheckField, 2, 2, 0,                   //  5370:       check Inst[3:2] == 0x0
  OPC_Decode, 215, 41, 75,                   //  5374:       decode to LUTI4_S_4ZZT2Z using decoder 75
                                             //  5374:      }
                                             //  5374:      } // switch Inst[17:15]
                                             //  5374:     }
  4, 18,                                     //  5378:     case 0x4: {
  OPC_CheckPredicate, 21,                    //  5380:      check predicate 21
  OPC_CheckField, 17, 1, 1,                  //  5382:      check Inst[17] == 0x1
  OPC_CheckField, 15, 1, 1,                  //  5386:      check Inst[15] == 0x1
  OPC_CheckField, 2, 2, 0,                   //  5390:      check Inst[3:2] == 0x0
  OPC_Decode, 214, 41, 76,                   //  5394:      decode to LUTI4_S_4ZTZI_H using decoder 76
                                             //  5394:     }
  16, 14,                                    //  5398:     case 0x10: {
  OPC_CheckPredicate, 21,                    //  5400:      check predicate 21
  OPC_CheckField, 17, 1, 1,                  //  5402:      check Inst[17] == 0x1
  OPC_CheckField, 3, 1, 0,                   //  5406:      check Inst[3] == 0x0
  OPC_Decode, 212, 41, 77,                   //  5410:      decode to LUTI4_S_2ZTZI_B using decoder 77
                                             //  5410:     }
  20, 0,                                     //  5414:     case 0x14: {
  OPC_CheckPredicate, 21,                    //  5416:      check predicate 21
  OPC_CheckField, 17, 1, 1,                  //  5418:      check Inst[17] == 0x1
  OPC_CheckField, 3, 1, 0,                   //  5422:      check Inst[3] == 0x0
  OPC_Decode, 213, 41, 77,                   //  5426:      decode to LUTI4_S_2ZTZI_H using decoder 77
                                             //  5426:     }
                                             //  5426:     } // switch Inst[14:10]
                                             //  5426:    }
  7, 59,                                     //  5430:    case 0x7: {
  OPC_SwitchField, 10, 5,                    //  5432:     switch Inst[14:10] {
  0, 14,                                     //  5435:     case 0x0: {
  OPC_CheckPredicate, 21,                    //  5437:      check predicate 21
  OPC_CheckField, 15, 1, 1,                  //  5439:      check Inst[15] == 0x1
  OPC_CheckField, 2, 2, 0,                   //  5443:      check Inst[3:2] == 0x0
  OPC_Decode, 199, 41, 78,                   //  5447:      decode to LUTI2_S_4ZTZI_B using decoder 78
                                             //  5447:     }
  4, 14,                                     //  5451:     case 0x4: {
  OPC_CheckPredicate, 21,                    //  5453:      check predicate 21
  OPC_CheckField, 15, 1, 1,                  //  5455:      check Inst[15] == 0x1
  OPC_CheckField, 2, 2, 0,                   //  5459:      check Inst[3:2] == 0x0
  OPC_Decode, 200, 41, 78,                   //  5463:      decode to LUTI2_S_4ZTZI_H using decoder 78
                                             //  5463:     }
  16, 10,                                    //  5467:     case 0x10: {
  OPC_CheckPredicate, 21,                    //  5469:      check predicate 21
  OPC_CheckField, 3, 1, 0,                   //  5471:      check Inst[3] == 0x0
  OPC_Decode, 197, 41, 79,                   //  5475:      decode to LUTI2_S_2ZTZI_B using decoder 79
                                             //  5475:     }
  20, 0,                                     //  5479:     case 0x14: {
  OPC_CheckPredicate, 21,                    //  5481:      check predicate 21
  OPC_CheckField, 3, 1, 0,                   //  5483:      check Inst[3] == 0x0
  OPC_Decode, 198, 41, 79,                   //  5487:      decode to LUTI2_S_2ZTZI_H using decoder 79
                                             //  5487:     }
                                             //  5487:     } // switch Inst[14:10]
                                             //  5487:    }
  16, 151, 1,                                //  5491:    case 0x10: {
  OPC_SwitchField, 15, 3,                    //  5494:     switch Inst[17:15] {
  0, 10,                                     //  5497:     case 0x0: {
  OPC_CheckPredicate, 6,                     //  5499:      check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  5501:      check Inst[4] == 0x0
  OPC_Decode, 130, 34, 80,                   //  5505:      decode to INSERT_MXIPZ_H_D using decoder 80
                                             //  5505:     }
  1, 10,                                     //  5509:     case 0x1: {
  OPC_CheckPredicate, 6,                     //  5511:      check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  5513:      check Inst[4] == 0x0
  OPC_Decode, 135, 34, 80,                   //  5517:      decode to INSERT_MXIPZ_V_D using decoder 80
                                             //  5517:     }
  2, 10,                                     //  5521:     case 0x2: {
  OPC_CheckPredicate, 6,                     //  5523:      check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  5525:      check Inst[4] == 0x0
  OPC_Decode, 132, 34, 81,                   //  5529:      decode to INSERT_MXIPZ_H_Q using decoder 81
                                             //  5529:     }
  3, 10,                                     //  5533:     case 0x3: {
  OPC_CheckPredicate, 6,                     //  5535:      check predicate 6
  OPC_CheckField, 4, 1, 0,                   //  5537:      check Inst[4] == 0x0
  OPC_Decode, 137, 34, 81,                   //  5541:      decode to INSERT_MXIPZ_V_Q using decoder 81
                                             //  5541:     }
  4, 23,                                     //  5545:     case 0x4: {
  OPC_SwitchField, 9, 1,                     //  5547:      switch Inst[9] {
  0, 6,                                      //  5550:      case 0x0: {
  OPC_CheckPredicate, 6,                     //  5552:       check predicate 6
  OPC_Decode, 153, 21, 82,                   //  5554:       decode to EXTRACT_ZPMXI_H_D using decoder 82
                                             //  5554:      }
  1, 0,                                      //  5558:      case 0x1: {
  OPC_CheckPredicate, 21,                    //  5560:       check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  5562:       check Inst[12:10] == 0x0
  OPC_Decode, 169, 42, 83,                   //  5566:       decode to MOVAZ_ZMI_H_D using decoder 83
                                             //  5566:      }
                                             //  5566:      } // switch Inst[9]
                                             //  5566:     }
  5, 23,                                     //  5570:     case 0x5: {
  OPC_SwitchField, 9, 1,                     //  5572:      switch Inst[9] {
  0, 6,                                      //  5575:      case 0x0: {
  OPC_CheckPredicate, 6,                     //  5577:       check predicate 6
  OPC_Decode, 158, 21, 82,                   //  5579:       decode to EXTRACT_ZPMXI_V_D using decoder 82
                                             //  5579:      }
  1, 0,                                      //  5583:      case 0x1: {
  OPC_CheckPredicate, 21,                    //  5585:       check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  5587:       check Inst[12:10] == 0x0
  OPC_Decode, 174, 42, 83,                   //  5591:       decode to MOVAZ_ZMI_V_D using decoder 83
                                             //  5591:      }
                                             //  5591:      } // switch Inst[9]
                                             //  5591:     }
  6, 23,                                     //  5595:     case 0x6: {
  OPC_SwitchField, 9, 1,                     //  5597:      switch Inst[9] {
  0, 6,                                      //  5600:      case 0x0: {
  OPC_CheckPredicate, 6,                     //  5602:       check predicate 6
  OPC_Decode, 155, 21, 84,                   //  5604:       decode to EXTRACT_ZPMXI_H_Q using decoder 84
                                             //  5604:      }
  1, 0,                                      //  5608:      case 0x1: {
  OPC_CheckPredicate, 21,                    //  5610:       check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  5612:       check Inst[12:10] == 0x0
  OPC_Decode, 171, 42, 85,                   //  5616:       decode to MOVAZ_ZMI_H_Q using decoder 85
                                             //  5616:      }
                                             //  5616:      } // switch Inst[9]
                                             //  5616:     }
  7, 0,                                      //  5620:     case 0x7: {
  OPC_SwitchField, 9, 1,                     //  5622:      switch Inst[9] {
  0, 6,                                      //  5625:      case 0x0: {
  OPC_CheckPredicate, 6,                     //  5627:       check predicate 6
  OPC_Decode, 160, 21, 84,                   //  5629:       decode to EXTRACT_ZPMXI_V_Q using decoder 84
                                             //  5629:      }
  1, 0,                                      //  5633:      case 0x1: {
  OPC_CheckPredicate, 21,                    //  5635:       check predicate 21
  OPC_CheckField, 10, 3, 0,                  //  5637:       check Inst[12:10] == 0x0
  OPC_Decode, 176, 42, 85,                   //  5641:       decode to MOVAZ_ZMI_V_Q using decoder 85
                                             //  5641:      }
                                             //  5641:      } // switch Inst[9]
                                             //  5641:     }
                                             //  5641:     } // switch Inst[17:15]
                                             //  5641:    }
  17, 167, 1,                                //  5645:    case 0x11: {
  OPC_SwitchField, 15, 3,                    //  5648:     switch Inst[17:15] {
  0, 27,                                     //  5651:     case 0x0: {
  OPC_SwitchField, 10, 3,                    //  5653:      switch Inst[12:10] {
  0, 10,                                     //  5656:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  5658:       check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  5660:       check Inst[5:3] == 0x0
  OPC_Decode, 195, 42, 86,                   //  5664:       decode to MOVA_MXI2Z_H_D using decoder 86
                                             //  5664:      }
  1, 0,                                      //  5668:      case 0x1: {
  OPC_CheckPredicate, 7,                     //  5670:       check predicate 7
  OPC_CheckField, 3, 4, 0,                   //  5672:       check Inst[6:3] == 0x0
  OPC_Decode, 203, 42, 87,                   //  5676:       decode to MOVA_MXI4Z_H_D using decoder 87
                                             //  5676:      }
                                             //  5676:      } // switch Inst[12:10]
                                             //  5676:     }
  1, 27,                                     //  5680:     case 0x1: {
  OPC_SwitchField, 10, 3,                    //  5682:      switch Inst[12:10] {
  0, 10,                                     //  5685:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  5687:       check predicate 7
  OPC_CheckField, 3, 3, 0,                   //  5689:       check Inst[5:3] == 0x0
  OPC_Decode, 199, 42, 86,                   //  5693:       decode to MOVA_MXI2Z_V_D using decoder 86
                                             //  5693:      }
  1, 0,                                      //  5697:      case 0x1: {
  OPC_CheckPredicate, 7,                     //  5699:       check predicate 7
  OPC_CheckField, 3, 4, 0,                   //  5701:       check Inst[6:3] == 0x0
  OPC_Decode, 207, 42, 87,                   //  5705:       decode to MOVA_MXI4Z_V_D using decoder 87
                                             //  5705:      }
                                             //  5705:      } // switch Inst[12:10]
                                             //  5705:     }
  4, 51,                                     //  5709:     case 0x4: {
  OPC_SwitchField, 8, 5,                     //  5711:      switch Inst[12:8] {
  0, 10,                                     //  5714:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  5716:       check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  5718:       check Inst[0] == 0x0
  OPC_Decode, 179, 42, 88,                   //  5722:       decode to MOVA_2ZMXI_H_D using decoder 88
                                             //  5722:      }
  2, 10,                                     //  5726:      case 0x2: {
  OPC_CheckPredicate, 21,                    //  5728:       check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  5730:       check Inst[0] == 0x0
  OPC_Decode, 151, 42, 89,                   //  5734:       decode to MOVAZ_2ZMI_H_D using decoder 89
                                             //  5734:      }
  4, 10,                                     //  5738:      case 0x4: {
  OPC_CheckPredicate, 7,                     //  5740:       check predicate 7
  OPC_CheckField, 0, 2, 0,                   //  5742:       check Inst[1:0] == 0x0
  OPC_Decode, 187, 42, 90,                   //  5746:       decode to MOVA_4ZMXI_H_D using decoder 90
                                             //  5746:      }
  6, 0,                                      //  5750:      case 0x6: {
  OPC_CheckPredicate, 21,                    //  5752:       check predicate 21
  OPC_CheckField, 0, 2, 0,                   //  5754:       check Inst[1:0] == 0x0
  OPC_Decode, 159, 42, 91,                   //  5758:       decode to MOVAZ_4ZMI_H_D using decoder 91
                                             //  5758:      }
                                             //  5758:      } // switch Inst[12:8]
                                             //  5758:     }
  5, 0,                                      //  5762:     case 0x5: {
  OPC_SwitchField, 8, 5,                     //  5764:      switch Inst[12:8] {
  0, 10,                                     //  5767:      case 0x0: {
  OPC_CheckPredicate, 7,                     //  5769:       check predicate 7
  OPC_CheckField, 0, 1, 0,                   //  5771:       check Inst[0] == 0x0
  OPC_Decode, 183, 42, 88,                   //  5775:       decode to MOVA_2ZMXI_V_D using decoder 88
                                             //  5775:      }
  2, 10,                                     //  5779:      case 0x2: {
  OPC_CheckPredicate, 21,                    //  5781:       check predicate 21
  OPC_CheckField, 0, 1, 0,                   //  5783:       check Inst[0] == 0x0
  OPC_Decode, 155, 42, 89,                   //  5787:       decode to MOVAZ_2ZMI_V_D using decoder 89
                                             //  5787:      }
  4, 10,                                     //  5791:      case 0x4: {
  OPC_CheckPredicate, 7,                     //  5793:       check predicate 7
  OPC_CheckField, 0, 2, 0,                   //  5795:       check Inst[1:0] == 0x0
  OPC_Decode, 191, 42, 90,                   //  5799:       decode to MOVA_4ZMXI_V_D using decoder 90
                                             //  5799:      }
  6, 0,                                      //  5803:      case 0x6: {
  OPC_CheckPredicate, 21,                    //  5805:       check predicate 21
  OPC_CheckField, 0, 2, 0,                   //  5807:       check Inst[1:0] == 0x0
  OPC_Decode, 163, 42, 91,                   //  5811:       decode to MOVAZ_4ZMI_V_D using decoder 91
                                             //  5811:      }
                                             //  5811:      } // switch Inst[12:8]
                                             //  5811:     }
                                             //  5811:     } // switch Inst[17:15]
                                             //  5811:    }
  18, 52,                                    //  5815:    case 0x12: {
  OPC_SwitchField, 10, 4,                    //  5817:     switch Inst[13:10] {
  0, 23,                                     //  5820:     case 0x0: {
  OPC_SwitchField, 17, 1,                    //  5822:      switch Inst[17] {
  0, 10,                                     //  5825:      case 0x0: {
  OPC_CheckPredicate, 23,                    //  5827:       check predicate 23
  OPC_CheckField, 14, 3, 1,                  //  5829:       check Inst[16:14] == 0x1
  OPC_Decode, 228, 41, 92,                   //  5833:       decode to LUTI6_ZTZ using decoder 92
                                             //  5833:      }
  1, 0,                                      //  5837:      case 0x1: {
  OPC_CheckPredicate, 7,                     //  5839:       check predicate 7
  OPC_Decode, 217, 41, 93,                   //  5841:       decode to LUTI4_ZTZI_B using decoder 93
                                             //  5841:      }
                                             //  5841:      } // switch Inst[17]
                                             //  5841:     }
  4, 10,                                     //  5845:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  5847:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5849:      check Inst[17] == 0x1
  OPC_Decode, 218, 41, 93,                   //  5853:      decode to LUTI4_ZTZI_H using decoder 93
                                             //  5853:     }
  8, 0,                                      //  5857:     case 0x8: {
  OPC_CheckPredicate, 7,                     //  5859:      check predicate 7
  OPC_CheckField, 17, 1, 1,                  //  5861:      check Inst[17] == 0x1
  OPC_Decode, 219, 41, 93,                   //  5865:      decode to LUTI4_ZTZI_S using decoder 93
                                             //  5865:     }
                                             //  5865:     } // switch Inst[13:10]
                                             //  5865:    }
  19, 27,                                    //  5869:    case 0x13: {
  OPC_SwitchField, 10, 4,                    //  5871:     switch Inst[13:10] {
  0, 6,                                      //  5874:     case 0x0: {
  OPC_CheckPredicate, 7,                     //  5876:      check predicate 7
  OPC_Decode, 201, 41, 94,                   //  5878:      decode to LUTI2_ZTZI_B using decoder 94
                                             //  5878:     }
  4, 6,                                      //  5882:     case 0x4: {
  OPC_CheckPredicate, 7,                     //  5884:      check predicate 7
  OPC_Decode, 202, 41, 94,                   //  5886:      decode to LUTI2_ZTZI_H using decoder 94
                                             //  5886:     }
  8, 0,                                      //  5890:     case 0x8: {
  OPC_CheckPredicate, 7,                     //  5892:      check predicate 7
  OPC_Decode, 203, 41, 94,                   //  5894:      decode to LUTI2_ZTZI_S using decoder 94
                                             //  5894:     }
                                             //  5894:     } // switch Inst[13:10]
                                             //  5894:    }
  20, 0,                                     //  5898:    case 0x14: {
  OPC_SwitchField, 16, 2,                    //  5900:     switch Inst[17:16] {
  0, 10,                                     //  5903:     case 0x0: {
  OPC_CheckPredicate, 19,                    //  5905:      check predicate 19
  OPC_CheckField, 3, 2, 0,                   //  5907:      check Inst[4:3] == 0x0
  OPC_Decode, 135, 13, 95,                   //  5911:      decode to ADDHA_MPPZ_D using decoder 95
                                             //  5911:     }
  1, 0,                                      //  5915:     case 0x1: {
  OPC_CheckPredicate, 19,                    //  5917:      check predicate 19
  OPC_CheckField, 3, 2, 0,                   //  5919:      check Inst[4:3] == 0x0
  OPC_Decode, 184, 13, 95,                   //  5923:      decode to ADDVA_MPPZ_D using decoder 95
                                             //  5923:     }
                                             //  5923:     } // switch Inst[17:16]
                                             //  5923:    }
                                             //  5923:    } // switch Inst[22:18]
                                             //  5923:   }
  2, 214, 40,                                //  5927:   case 0x2: {
  OPC_SwitchField, 21, 1,                    //  5930:    switch Inst[21] {
  0, 241, 6,                                 //  5933:    case 0x0: {
  OPC_SwitchField, 4, 1,                     //  5936:     switch Inst[4] {
  0, 192, 3,                                 //  5939:     case 0x0: {
  OPC_SwitchField, 20, 1,                    //  5942:      switch Inst[20] {
  0, 48,                                     //  5945:      case 0x0: {
  OPC_SwitchField, 2, 2,                     //  5947:       switch Inst[3:2] {
  0, 19,                                     //  5950:       case 0x0: {
  OPC_SwitchField, 22, 1,                    //  5952:        switch Inst[22] {
  0, 6,                                      //  5955:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  5957:         check predicate 7
  OPC_Decode, 212, 49, 96,                   //  5959:         decode to SMLALL_MZZI_BtoS using decoder 96
                                             //  5959:        }
  1, 0,                                      //  5963:        case 0x1: {
  OPC_CheckPredicate, 8,                     //  5965:         check predicate 8
  OPC_Decode, 137, 28, 96,                   //  5967:         decode to FMLALL_MZZI_BtoS using decoder 96
                                             //  5967:        }
                                             //  5967:        } // switch Inst[22]
                                             //  5967:       }
  1, 10,                                     //  5971:       case 0x1: {
  OPC_CheckPredicate, 7,                     //  5973:        check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  5975:        check Inst[22] == 0x0
  OPC_Decode, 183, 68, 96,                   //  5979:        decode to USMLALL_MZZI_BtoS using decoder 96
                                             //  5979:       }
  2, 0,                                      //  5983:       case 0x2: {
  OPC_CheckPredicate, 7,                     //  5985:        check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  5987:        check Inst[22] == 0x0
  OPC_Decode, 128, 50, 96,                   //  5991:        decode to SMLSLL_MZZI_BtoS using decoder 96
                                             //  5991:       }
                                             //  5991:       } // switch Inst[3:2]
                                             //  5991:      }
  1, 0,                                      //  5995:      case 0x1: {
  OPC_SwitchField, 5, 1,                     //  5997:       switch Inst[5] {
  0, 217, 1,                                 //  6000:       case 0x0: {
  OPC_SwitchField, 12, 1,                    //  6003:        switch Inst[12] {
  0, 122,                                    //  6006:        case 0x0: {
  OPC_SwitchField, 3, 1,                     //  6008:         switch Inst[3] {
  0, 62,                                     //  6011:         case 0x0: {
  OPC_SwitchField, 15, 1,                    //  6013:          switch Inst[15] {
  0, 19,                                     //  6016:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6018:           switch Inst[22] {
  0, 6,                                      //  6021:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6023:            check predicate 7
  OPC_Decode, 218, 49, 97,                   //  6025:            decode to SMLALL_VG2_M2ZZI_BtoS using decoder 97
                                             //  6025:           }
  1, 0,                                      //  6029:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6031:            check predicate 7
  OPC_Decode, 176, 28, 98,                   //  6033:            decode to FMLA_VG2_M2ZZI_S using decoder 98
                                             //  6033:           }
                                             //  6033:           } // switch Inst[22]
                                             //  6033:          }
  1, 0,                                      //  6037:          case 0x1: {
  OPC_SwitchField, 6, 1,                     //  6039:           switch Inst[6] {
  0, 19,                                     //  6042:           case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6044:            switch Inst[22] {
  0, 6,                                      //  6047:            case 0x0: {
  OPC_CheckPredicate, 7,                     //  6049:             check predicate 7
  OPC_Decode, 224, 49, 99,                   //  6051:             decode to SMLALL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6051:            }
  1, 0,                                      //  6055:            case 0x1: {
  OPC_CheckPredicate, 7,                     //  6057:             check predicate 7
  OPC_Decode, 185, 28, 100,                  //  6059:             decode to FMLA_VG4_M4ZZI_S using decoder 100
                                             //  6059:            }
                                             //  6059:            } // switch Inst[22]
                                             //  6059:           }
  1, 0,                                      //  6063:           case 0x1: {
  OPC_CheckPredicate, 8,                     //  6065:            check predicate 8
  OPC_CheckField, 22, 1, 0,                  //  6067:            check Inst[22] == 0x0
  OPC_Decode, 143, 28, 99,                   //  6071:            decode to FMLALL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6071:           }
                                             //  6071:           } // switch Inst[6]
                                             //  6071:          }
                                             //  6071:          } // switch Inst[15]
                                             //  6071:         }
  1, 0,                                      //  6075:         case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6077:          switch Inst[15] {
  0, 19,                                     //  6080:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6082:           switch Inst[22] {
  0, 6,                                      //  6085:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6087:            check predicate 7
  OPC_Decode, 134, 50, 97,                   //  6089:            decode to SMLSLL_VG2_M2ZZI_BtoS using decoder 97
                                             //  6089:           }
  1, 0,                                      //  6093:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6095:            check predicate 7
  OPC_Decode, 209, 32, 98,                   //  6097:            decode to FVDOT_VG2_M2ZZI_HtoS using decoder 98
                                             //  6097:           }
                                             //  6097:           } // switch Inst[22]
                                             //  6097:          }
  1, 0,                                      //  6101:          case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6103:           switch Inst[22] {
  0, 10,                                     //  6106:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6108:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6110:            check Inst[6] == 0x0
  OPC_Decode, 140, 50, 99,                   //  6114:            decode to SMLSLL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6114:           }
  1, 0,                                      //  6118:           case 0x1: {
  OPC_CheckPredicate, 8,                     //  6120:            check predicate 8
  OPC_CheckField, 6, 1, 0,                   //  6122:            check Inst[6] == 0x0
  OPC_Decode, 137, 26, 100,                  //  6126:            decode to FDOT_VG4_M4ZZI_BtoS using decoder 100
                                             //  6126:           }
                                             //  6126:           } // switch Inst[22]
                                             //  6126:          }
                                             //  6126:          } // switch Inst[15]
                                             //  6126:         }
                                             //  6126:         } // switch Inst[3]
                                             //  6126:        }
  1, 0,                                      //  6130:        case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6132:         switch Inst[15] {
  0, 32,                                     //  6135:         case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6137:          switch Inst[22] {
  0, 6,                                      //  6140:          case 0x0: {
  OPC_CheckPredicate, 16,                    //  6142:           check predicate 16
  OPC_Decode, 175, 28, 101,                  //  6144:           decode to FMLA_VG2_M2ZZI_H using decoder 101
                                             //  6144:          }
  1, 0,                                      //  6148:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6150:           switch Inst[3] {
  0, 6,                                      //  6153:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6155:            check predicate 7
  OPC_Decode, 185, 47, 98,                   //  6157:            decode to SDOT_VG2_M2ZZI_HToS using decoder 98
                                             //  6157:           }
  1, 0,                                      //  6161:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6163:            check predicate 7
  OPC_Decode, 129, 26, 98,                   //  6165:            decode to FDOT_VG2_M2ZZI_HtoS using decoder 98
                                             //  6165:           }
                                             //  6165:           } // switch Inst[3]
                                             //  6165:          }
                                             //  6165:          } // switch Inst[22]
                                             //  6165:         }
  1, 0,                                      //  6169:         case 0x1: {
  OPC_SwitchField, 6, 1,                     //  6171:          switch Inst[6] {
  0, 32,                                     //  6174:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6176:           switch Inst[22] {
  0, 6,                                      //  6179:           case 0x0: {
  OPC_CheckPredicate, 16,                    //  6181:            check predicate 16
  OPC_Decode, 184, 28, 102,                  //  6183:            decode to FMLA_VG4_M4ZZI_H using decoder 102
                                             //  6183:           }
  1, 0,                                      //  6187:           case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6189:            switch Inst[3] {
  0, 6,                                      //  6192:            case 0x0: {
  OPC_CheckPredicate, 7,                     //  6194:             check predicate 7
  OPC_Decode, 194, 47, 100,                  //  6196:             decode to SDOT_VG4_M4ZZI_HToS using decoder 100
                                             //  6196:            }
  1, 0,                                      //  6200:            case 0x1: {
  OPC_CheckPredicate, 7,                     //  6202:             check predicate 7
  OPC_Decode, 138, 26, 100,                  //  6204:             decode to FDOT_VG4_M4ZZI_HtoS using decoder 100
                                             //  6204:            }
                                             //  6204:            } // switch Inst[3]
                                             //  6204:           }
                                             //  6204:           } // switch Inst[22]
                                             //  6204:          }
  1, 0,                                      //  6208:          case 0x1: {
  OPC_CheckPredicate, 9,                     //  6210:           check predicate 9
  OPC_CheckField, 22, 1, 0,                  //  6212:           check Inst[22] == 0x0
  OPC_Decode, 136, 26, 102,                  //  6216:           decode to FDOT_VG4_M4ZZI_BtoH using decoder 102
                                             //  6216:          }
                                             //  6216:          } // switch Inst[6]
                                             //  6216:         }
                                             //  6216:         } // switch Inst[15]
                                             //  6216:        }
                                             //  6216:        } // switch Inst[12]
                                             //  6216:       }
  1, 0,                                      //  6220:       case 0x1: {
  OPC_SwitchField, 12, 1,                    //  6222:        switch Inst[12] {
  0, 78,                                     //  6225:        case 0x0: {
  OPC_SwitchField, 3, 1,                     //  6227:         switch Inst[3] {
  0, 53,                                     //  6230:         case 0x0: {
  OPC_SwitchField, 15, 1,                    //  6232:          switch Inst[15] {
  0, 19,                                     //  6235:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6237:           switch Inst[22] {
  0, 6,                                      //  6240:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6242:            check predicate 7
  OPC_Decode, 186, 68, 97,                   //  6244:            decode to USMLALL_VG2_M2ZZI_BtoS using decoder 97
                                             //  6244:           }
  1, 0,                                      //  6248:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6250:            check predicate 7
  OPC_Decode, 134, 61, 98,                   //  6252:            decode to SVDOT_VG2_M2ZZI_HtoS using decoder 98
                                             //  6252:           }
                                             //  6252:           } // switch Inst[22]
                                             //  6252:          }
  1, 0,                                      //  6256:          case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6258:           switch Inst[22] {
  0, 10,                                     //  6261:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6263:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6265:            check Inst[6] == 0x0
  OPC_Decode, 189, 68, 99,                   //  6269:            decode to USMLALL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6269:           }
  1, 0,                                      //  6273:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6275:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6277:            check Inst[6] == 0x0
  OPC_Decode, 135, 61, 100,                  //  6281:            decode to SVDOT_VG4_M4ZZI_BtoS using decoder 100
                                             //  6281:           }
                                             //  6281:           } // switch Inst[22]
                                             //  6281:          }
                                             //  6281:          } // switch Inst[15]
                                             //  6281:         }
  1, 0,                                      //  6285:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  6287:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  //  6289:          check Inst[22] == 0x1
  OPC_CheckField, 15, 1, 1,                  //  6293:          check Inst[15] == 0x1
  OPC_CheckField, 6, 1, 0,                   //  6297:          check Inst[6] == 0x0
  OPC_Decode, 137, 69, 100,                  //  6301:          decode to USVDOT_VG4_M4ZZI_BToS using decoder 100
                                             //  6301:         }
                                             //  6301:         } // switch Inst[3]
                                             //  6301:        }
  1, 0,                                      //  6305:        case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6307:         switch Inst[15] {
  0, 32,                                     //  6310:         case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6312:          switch Inst[22] {
  0, 6,                                      //  6315:          case 0x0: {
  OPC_CheckPredicate, 17,                    //  6317:           check predicate 17
  OPC_Decode, 187, 15, 101,                  //  6319:           decode to BFMLA_VG2_M2ZZI using decoder 101
                                             //  6319:          }
  1, 0,                                      //  6323:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6325:           switch Inst[3] {
  0, 6,                                      //  6328:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6330:            check predicate 7
  OPC_Decode, 184, 47, 98,                   //  6332:            decode to SDOT_VG2_M2ZZI_BToS using decoder 98
                                             //  6332:           }
  1, 0,                                      //  6336:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6338:            check predicate 7
  OPC_Decode, 144, 68, 98,                   //  6340:            decode to USDOT_VG2_M2ZZI_BToS using decoder 98
                                             //  6340:           }
                                             //  6340:           } // switch Inst[3]
                                             //  6340:          }
                                             //  6340:          } // switch Inst[22]
                                             //  6340:         }
  1, 0,                                      //  6344:         case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6346:          switch Inst[22] {
  0, 10,                                     //  6349:          case 0x0: {
  OPC_CheckPredicate, 17,                    //  6351:           check predicate 17
  OPC_CheckField, 6, 1, 0,                   //  6353:           check Inst[6] == 0x0
  OPC_Decode, 190, 15, 102,                  //  6357:           decode to BFMLA_VG4_M4ZZI using decoder 102
                                             //  6357:          }
  1, 0,                                      //  6361:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6363:           switch Inst[3] {
  0, 10,                                     //  6366:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6368:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6370:            check Inst[6] == 0x0
  OPC_Decode, 193, 47, 100,                  //  6374:            decode to SDOT_VG4_M4ZZI_BToS using decoder 100
                                             //  6374:           }
  1, 0,                                      //  6378:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6380:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6382:            check Inst[6] == 0x0
  OPC_Decode, 147, 68, 100,                  //  6386:            decode to USDOT_VG4_M4ZZI_BToS using decoder 100
                                             //  6386:           }
                                             //  6386:           } // switch Inst[3]
                                             //  6386:          }
                                             //  6386:          } // switch Inst[22]
                                             //  6386:         }
                                             //  6386:         } // switch Inst[15]
                                             //  6386:        }
                                             //  6386:        } // switch Inst[12]
                                             //  6386:       }
                                             //  6386:       } // switch Inst[5]
                                             //  6386:      }
                                             //  6386:      } // switch Inst[20]
                                             //  6386:     }
  1, 0,                                      //  6390:     case 0x1: {
  OPC_SwitchField, 20, 1,                    //  6392:      switch Inst[20] {
  0, 39,                                     //  6395:      case 0x0: {
  OPC_SwitchField, 2, 2,                     //  6397:       switch Inst[3:2] {
  0, 10,                                     //  6400:       case 0x0: {
  OPC_CheckPredicate, 7,                     //  6402:        check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  6404:        check Inst[22] == 0x0
  OPC_Decode, 221, 64, 96,                   //  6408:        decode to UMLALL_MZZI_BtoS using decoder 96
                                             //  6408:       }
  1, 10,                                     //  6412:       case 0x1: {
  OPC_CheckPredicate, 7,                     //  6414:        check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  6416:        check Inst[22] == 0x0
  OPC_Decode, 207, 60, 96,                   //  6420:        decode to SUMLALL_MZZI_BtoS using decoder 96
                                             //  6420:       }
  2, 0,                                      //  6424:       case 0x2: {
  OPC_CheckPredicate, 7,                     //  6426:        check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  6428:        check Inst[22] == 0x0
  OPC_Decode, 137, 65, 96,                   //  6432:        decode to UMLSLL_MZZI_BtoS using decoder 96
                                             //  6432:       }
                                             //  6432:       } // switch Inst[3:2]
                                             //  6432:      }
  1, 0,                                      //  6436:      case 0x1: {
  OPC_SwitchField, 5, 1,                     //  6438:       switch Inst[5] {
  0, 190, 1,                                 //  6441:       case 0x0: {
  OPC_SwitchField, 12, 1,                    //  6444:        switch Inst[12] {
  0, 100,                                    //  6447:        case 0x0: {
  OPC_SwitchField, 3, 1,                     //  6449:         switch Inst[3] {
  0, 53,                                     //  6452:         case 0x0: {
  OPC_SwitchField, 15, 1,                    //  6454:          switch Inst[15] {
  0, 19,                                     //  6457:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6459:           switch Inst[22] {
  0, 6,                                      //  6462:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6464:            check predicate 7
  OPC_Decode, 227, 64, 97,                   //  6466:            decode to UMLALL_VG2_M2ZZI_BtoS using decoder 97
                                             //  6466:           }
  1, 0,                                      //  6470:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6472:            check predicate 7
  OPC_Decode, 234, 28, 98,                   //  6474:            decode to FMLS_VG2_M2ZZI_S using decoder 98
                                             //  6474:           }
                                             //  6474:           } // switch Inst[22]
                                             //  6474:          }
  1, 0,                                      //  6478:          case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6480:           switch Inst[22] {
  0, 10,                                     //  6483:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6485:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6487:            check Inst[6] == 0x0
  OPC_Decode, 233, 64, 99,                   //  6491:            decode to UMLALL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6491:           }
  1, 0,                                      //  6495:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6497:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6499:            check Inst[6] == 0x0
  OPC_Decode, 243, 28, 100,                  //  6503:            decode to FMLS_VG4_M4ZZI_S using decoder 100
                                             //  6503:           }
                                             //  6503:           } // switch Inst[22]
                                             //  6503:          }
                                             //  6503:          } // switch Inst[15]
                                             //  6503:         }
  1, 0,                                      //  6507:         case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6509:          switch Inst[15] {
  0, 19,                                     //  6512:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6514:           switch Inst[22] {
  0, 6,                                      //  6517:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6519:            check predicate 7
  OPC_Decode, 143, 65, 97,                   //  6521:            decode to UMLSLL_VG2_M2ZZI_BtoS using decoder 97
                                             //  6521:           }
  1, 0,                                      //  6525:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6527:            check predicate 7
  OPC_Decode, 128, 16, 98,                   //  6529:            decode to BFVDOT_VG2_M2ZZI_HtoS using decoder 98
                                             //  6529:           }
                                             //  6529:           } // switch Inst[22]
                                             //  6529:          }
  1, 0,                                      //  6533:          case 0x1: {
  OPC_CheckPredicate, 7,                     //  6535:           check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  6537:           check Inst[22] == 0x0
  OPC_CheckField, 6, 1, 0,                   //  6541:           check Inst[6] == 0x0
  OPC_Decode, 149, 65, 99,                   //  6545:           decode to UMLSLL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6545:          }
                                             //  6545:          } // switch Inst[15]
                                             //  6545:         }
                                             //  6545:         } // switch Inst[3]
                                             //  6545:        }
  1, 0,                                      //  6549:        case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6551:         switch Inst[15] {
  0, 32,                                     //  6554:         case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6556:          switch Inst[22] {
  0, 6,                                      //  6559:          case 0x0: {
  OPC_CheckPredicate, 16,                    //  6561:           check predicate 16
  OPC_Decode, 233, 28, 101,                  //  6563:           decode to FMLS_VG2_M2ZZI_H using decoder 101
                                             //  6563:          }
  1, 0,                                      //  6567:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6569:           switch Inst[3] {
  0, 6,                                      //  6572:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6574:            check predicate 7
  OPC_Decode, 179, 63, 98,                   //  6576:            decode to UDOT_VG2_M2ZZI_HToS using decoder 98
                                             //  6576:           }
  1, 0,                                      //  6580:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6582:            check predicate 7
  OPC_Decode, 140, 15, 98,                   //  6584:            decode to BFDOT_VG2_M2ZZI_HtoS using decoder 98
                                             //  6584:           }
                                             //  6584:           } // switch Inst[3]
                                             //  6584:          }
                                             //  6584:          } // switch Inst[22]
                                             //  6584:         }
  1, 0,                                      //  6588:         case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6590:          switch Inst[22] {
  0, 10,                                     //  6593:          case 0x0: {
  OPC_CheckPredicate, 16,                    //  6595:           check predicate 16
  OPC_CheckField, 6, 1, 0,                   //  6597:           check Inst[6] == 0x0
  OPC_Decode, 242, 28, 102,                  //  6601:           decode to FMLS_VG4_M4ZZI_H using decoder 102
                                             //  6601:          }
  1, 0,                                      //  6605:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6607:           switch Inst[3] {
  0, 10,                                     //  6610:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6612:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6614:            check Inst[6] == 0x0
  OPC_Decode, 188, 63, 100,                  //  6618:            decode to UDOT_VG4_M4ZZI_HToS using decoder 100
                                             //  6618:           }
  1, 0,                                      //  6622:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6624:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6626:            check Inst[6] == 0x0
  OPC_Decode, 143, 15, 100,                  //  6630:            decode to BFDOT_VG4_M4ZZI_HtoS using decoder 100
                                             //  6630:           }
                                             //  6630:           } // switch Inst[3]
                                             //  6630:          }
                                             //  6630:          } // switch Inst[22]
                                             //  6630:         }
                                             //  6630:         } // switch Inst[15]
                                             //  6630:        }
                                             //  6630:        } // switch Inst[12]
                                             //  6630:       }
  1, 0,                                      //  6634:       case 0x1: {
  OPC_SwitchField, 12, 1,                    //  6636:        switch Inst[12] {
  0, 91,                                     //  6639:        case 0x0: {
  OPC_SwitchField, 3, 1,                     //  6641:         switch Inst[3] {
  0, 53,                                     //  6644:         case 0x0: {
  OPC_SwitchField, 15, 1,                    //  6646:          switch Inst[15] {
  0, 19,                                     //  6649:          case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6651:           switch Inst[22] {
  0, 6,                                      //  6654:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6656:            check predicate 7
  OPC_Decode, 208, 60, 97,                   //  6658:            decode to SUMLALL_VG2_M2ZZI_BtoS using decoder 97
                                             //  6658:           }
  1, 0,                                      //  6662:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6664:            check predicate 7
  OPC_Decode, 152, 69, 98,                   //  6666:            decode to UVDOT_VG2_M2ZZI_HtoS using decoder 98
                                             //  6666:           }
                                             //  6666:           } // switch Inst[22]
                                             //  6666:          }
  1, 0,                                      //  6670:          case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6672:           switch Inst[22] {
  0, 10,                                     //  6675:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6677:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6679:            check Inst[6] == 0x0
  OPC_Decode, 210, 60, 99,                   //  6683:            decode to SUMLALL_VG4_M4ZZI_BtoS using decoder 99
                                             //  6683:           }
  1, 0,                                      //  6687:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6689:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6691:            check Inst[6] == 0x0
  OPC_Decode, 153, 69, 100,                  //  6695:            decode to UVDOT_VG4_M4ZZI_BtoS using decoder 100
                                             //  6695:           }
                                             //  6695:           } // switch Inst[22]
                                             //  6695:          }
                                             //  6695:          } // switch Inst[15]
                                             //  6695:         }
  1, 0,                                      //  6699:         case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6701:          switch Inst[15] {
  0, 10,                                     //  6704:          case 0x0: {
  OPC_CheckPredicate, 8,                     //  6706:           check predicate 8
  OPC_CheckField, 22, 1, 1,                  //  6708:           check Inst[22] == 0x1
  OPC_Decode, 128, 26, 98,                   //  6712:           decode to FDOT_VG2_M2ZZI_BtoS using decoder 98
                                             //  6712:          }
  1, 0,                                      //  6716:          case 0x1: {
  OPC_CheckPredicate, 7,                     //  6718:           check predicate 7
  OPC_CheckField, 22, 1, 1,                  //  6720:           check Inst[22] == 0x1
  OPC_CheckField, 6, 1, 0,                   //  6724:           check Inst[6] == 0x0
  OPC_Decode, 132, 61, 100,                  //  6728:           decode to SUVDOT_VG4_M4ZZI_BToS using decoder 100
                                             //  6728:          }
                                             //  6728:          } // switch Inst[15]
                                             //  6728:         }
                                             //  6728:         } // switch Inst[3]
                                             //  6728:        }
  1, 0,                                      //  6732:        case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6734:         switch Inst[15] {
  0, 32,                                     //  6737:         case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6739:          switch Inst[22] {
  0, 6,                                      //  6742:          case 0x0: {
  OPC_CheckPredicate, 17,                    //  6744:           check predicate 17
  OPC_Decode, 207, 15, 101,                  //  6746:           decode to BFMLS_VG2_M2ZZI using decoder 101
                                             //  6746:          }
  1, 0,                                      //  6750:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6752:           switch Inst[3] {
  0, 6,                                      //  6755:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6757:            check predicate 7
  OPC_Decode, 178, 63, 98,                   //  6759:            decode to UDOT_VG2_M2ZZI_BToS using decoder 98
                                             //  6759:           }
  1, 0,                                      //  6763:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6765:            check predicate 7
  OPC_Decode, 200, 60, 98,                   //  6767:            decode to SUDOT_VG2_M2ZZI_BToS using decoder 98
                                             //  6767:           }
                                             //  6767:           } // switch Inst[3]
                                             //  6767:          }
                                             //  6767:          } // switch Inst[22]
                                             //  6767:         }
  1, 0,                                      //  6771:         case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6773:          switch Inst[22] {
  0, 10,                                     //  6776:          case 0x0: {
  OPC_CheckPredicate, 17,                    //  6778:           check predicate 17
  OPC_CheckField, 6, 1, 0,                   //  6780:           check Inst[6] == 0x0
  OPC_Decode, 210, 15, 102,                  //  6784:           decode to BFMLS_VG4_M4ZZI using decoder 102
                                             //  6784:          }
  1, 0,                                      //  6788:          case 0x1: {
  OPC_SwitchField, 3, 1,                     //  6790:           switch Inst[3] {
  0, 10,                                     //  6793:           case 0x0: {
  OPC_CheckPredicate, 7,                     //  6795:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6797:            check Inst[6] == 0x0
  OPC_Decode, 187, 63, 100,                  //  6801:            decode to UDOT_VG4_M4ZZI_BtoS using decoder 100
                                             //  6801:           }
  1, 0,                                      //  6805:           case 0x1: {
  OPC_CheckPredicate, 7,                     //  6807:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   //  6809:            check Inst[6] == 0x0
  OPC_Decode, 202, 60, 100,                  //  6813:            decode to SUDOT_VG4_M4ZZI_BToS using decoder 100
                                             //  6813:           }
                                             //  6813:           } // switch Inst[3]
                                             //  6813:          }
                                             //  6813:          } // switch Inst[22]
                                             //  6813:         }
                                             //  6813:         } // switch Inst[15]
                                             //  6813:        }
                                             //  6813:        } // switch Inst[12]
                                             //  6813:       }
                                             //  6813:       } // switch Inst[5]
                                             //  6813:      }
                                             //  6813:      } // switch Inst[20]
                                             //  6813:     }
                                             //  6813:     } // switch Inst[4]
                                             //  6813:    }
  1, 0,                                      //  6817:    case 0x1: {
  OPC_SwitchField, 15, 1,                    //  6819:     switch Inst[15] {
  0, 241, 10,                                //  6822:     case 0x0: {
  OPC_SwitchField, 10, 3,                    //  6825:      switch Inst[12:10] {
  0, 150, 2,                                 //  6828:      case 0x0: {
  OPC_SwitchField, 1, 4,                     //  6831:       switch Inst[4:1] {
  0, 45,                                     //  6834:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  6836:        switch Inst[20] {
  0, 19,                                     //  6839:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6841:         switch Inst[22] {
  0, 6,                                      //  6844:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  6846:          check predicate 7
  OPC_Decode, 220, 49, 103,                  //  6848:          decode to SMLALL_VG2_M2ZZ_BtoS using decoder 103
                                             //  6848:         }
  1, 0,                                      //  6852:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  6854:          check predicate 25
  OPC_Decode, 221, 49, 103,                  //  6856:          decode to SMLALL_VG2_M2ZZ_HtoD using decoder 103
                                             //  6856:         }
                                             //  6856:         } // switch Inst[22]
                                             //  6856:        }
  1, 0,                                      //  6860:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6862:         switch Inst[22] {
  0, 6,                                      //  6865:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  6867:          check predicate 7
  OPC_Decode, 226, 49, 104,                  //  6869:          decode to SMLALL_VG4_M4ZZ_BtoS using decoder 104
                                             //  6869:         }
  1, 0,                                      //  6873:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  6875:          check predicate 25
  OPC_Decode, 227, 49, 104,                  //  6877:          decode to SMLALL_VG4_M4ZZ_HtoD using decoder 104
                                             //  6877:         }
                                             //  6877:         } // switch Inst[22]
                                             //  6877:        }
                                             //  6877:        } // switch Inst[20]
                                             //  6877:       }
  1, 27,                                     //  6881:       case 0x1: {
  OPC_SwitchField, 20, 1,                    //  6883:        switch Inst[20] {
  0, 10,                                     //  6886:        case 0x0: {
  OPC_CheckPredicate, 8,                     //  6888:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  //  6890:         check Inst[22] == 0x0
  OPC_Decode, 141, 28, 103,                  //  6894:         decode to FMLALL_VG2_M2ZZ_BtoS using decoder 103
                                             //  6894:        }
  1, 0,                                      //  6898:        case 0x1: {
  OPC_CheckPredicate, 8,                     //  6900:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  //  6902:         check Inst[22] == 0x0
  OPC_Decode, 144, 28, 104,                  //  6906:         decode to FMLALL_VG4_M4ZZ_BtoS using decoder 104
                                             //  6906:        }
                                             //  6906:        } // switch Inst[20]
                                             //  6906:       }
  2, 27,                                     //  6910:       case 0x2: {
  OPC_SwitchField, 20, 1,                    //  6912:        switch Inst[20] {
  0, 10,                                     //  6915:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  6917:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  6919:         check Inst[22] == 0x0
  OPC_Decode, 187, 68, 103,                  //  6923:         decode to USMLALL_VG2_M2ZZ_BtoS using decoder 103
                                             //  6923:        }
  1, 0,                                      //  6927:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  6929:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  6931:         check Inst[22] == 0x0
  OPC_Decode, 190, 68, 104,                  //  6935:         decode to USMLALL_VG4_M4ZZ_BtoS using decoder 104
                                             //  6935:        }
                                             //  6935:        } // switch Inst[20]
                                             //  6935:       }
  4, 45,                                     //  6939:       case 0x4: {
  OPC_SwitchField, 20, 1,                    //  6941:        switch Inst[20] {
  0, 19,                                     //  6944:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6946:         switch Inst[22] {
  0, 6,                                      //  6949:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  6951:          check predicate 7
  OPC_Decode, 136, 50, 103,                  //  6953:          decode to SMLSLL_VG2_M2ZZ_BtoS using decoder 103
                                             //  6953:         }
  1, 0,                                      //  6957:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  6959:          check predicate 25
  OPC_Decode, 137, 50, 103,                  //  6961:          decode to SMLSLL_VG2_M2ZZ_HtoD using decoder 103
                                             //  6961:         }
                                             //  6961:         } // switch Inst[22]
                                             //  6961:        }
  1, 0,                                      //  6965:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  6967:         switch Inst[22] {
  0, 6,                                      //  6970:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  6972:          check predicate 7
  OPC_Decode, 142, 50, 104,                  //  6974:          decode to SMLSLL_VG4_M4ZZ_BtoS using decoder 104
                                             //  6974:         }
  1, 0,                                      //  6978:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  6980:          check predicate 25
  OPC_Decode, 143, 50, 104,                  //  6982:          decode to SMLSLL_VG4_M4ZZ_HtoD using decoder 104
                                             //  6982:         }
                                             //  6982:         } // switch Inst[22]
                                             //  6982:        }
                                             //  6982:        } // switch Inst[20]
                                             //  6982:       }
  8, 45,                                     //  6986:       case 0x8: {
  OPC_SwitchField, 20, 1,                    //  6988:        switch Inst[20] {
  0, 19,                                     //  6991:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  6993:         switch Inst[22] {
  0, 6,                                      //  6996:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  6998:          check predicate 7
  OPC_Decode, 229, 64, 103,                  //  7000:          decode to UMLALL_VG2_M2ZZ_BtoS using decoder 103
                                             //  7000:         }
  1, 0,                                      //  7004:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7006:          check predicate 25
  OPC_Decode, 230, 64, 103,                  //  7008:          decode to UMLALL_VG2_M2ZZ_HtoD using decoder 103
                                             //  7008:         }
                                             //  7008:         } // switch Inst[22]
                                             //  7008:        }
  1, 0,                                      //  7012:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7014:         switch Inst[22] {
  0, 6,                                      //  7017:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7019:          check predicate 7
  OPC_Decode, 235, 64, 104,                  //  7021:          decode to UMLALL_VG4_M4ZZ_BtoS using decoder 104
                                             //  7021:         }
  1, 0,                                      //  7025:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7027:          check predicate 25
  OPC_Decode, 236, 64, 104,                  //  7029:          decode to UMLALL_VG4_M4ZZ_HtoD using decoder 104
                                             //  7029:         }
                                             //  7029:         } // switch Inst[22]
                                             //  7029:        }
                                             //  7029:        } // switch Inst[20]
                                             //  7029:       }
  10, 27,                                    //  7033:       case 0xa: {
  OPC_SwitchField, 20, 1,                    //  7035:        switch Inst[20] {
  0, 10,                                     //  7038:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7040:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7042:         check Inst[22] == 0x0
  OPC_Decode, 209, 60, 103,                  //  7046:         decode to SUMLALL_VG2_M2ZZ_BtoS using decoder 103
                                             //  7046:        }
  1, 0,                                      //  7050:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  7052:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7054:         check Inst[22] == 0x0
  OPC_Decode, 211, 60, 104,                  //  7058:         decode to SUMLALL_VG4_M4ZZ_BtoS using decoder 104
                                             //  7058:        }
                                             //  7058:        } // switch Inst[20]
                                             //  7058:       }
  12, 0,                                     //  7062:       case 0xc: {
  OPC_SwitchField, 20, 1,                    //  7064:        switch Inst[20] {
  0, 19,                                     //  7067:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7069:         switch Inst[22] {
  0, 6,                                      //  7072:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7074:          check predicate 7
  OPC_Decode, 145, 65, 103,                  //  7076:          decode to UMLSLL_VG2_M2ZZ_BtoS using decoder 103
                                             //  7076:         }
  1, 0,                                      //  7080:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7082:          check predicate 25
  OPC_Decode, 146, 65, 103,                  //  7084:          decode to UMLSLL_VG2_M2ZZ_HtoD using decoder 103
                                             //  7084:         }
                                             //  7084:         } // switch Inst[22]
                                             //  7084:        }
  1, 0,                                      //  7088:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7090:         switch Inst[22] {
  0, 6,                                      //  7093:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7095:          check predicate 7
  OPC_Decode, 151, 65, 104,                  //  7097:          decode to UMLSLL_VG4_M4ZZ_BtoS using decoder 104
                                             //  7097:         }
  1, 0,                                      //  7101:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7103:          check predicate 25
  OPC_Decode, 152, 65, 104,                  //  7105:          decode to UMLSLL_VG4_M4ZZ_HtoD using decoder 104
                                             //  7105:         }
                                             //  7105:         } // switch Inst[22]
                                             //  7105:        }
                                             //  7105:        } // switch Inst[20]
                                             //  7105:       }
                                             //  7105:       } // switch Inst[4:1]
                                             //  7105:      }
  1, 144, 1,                                 //  7109:      case 0x1: {
  OPC_SwitchField, 2, 3,                     //  7112:       switch Inst[4:2] {
  0, 36,                                     //  7115:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  7117:        switch Inst[20] {
  0, 19,                                     //  7120:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7122:         switch Inst[22] {
  0, 6,                                      //  7125:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7127:          check predicate 7
  OPC_Decode, 214, 49, 105,                  //  7129:          decode to SMLALL_MZZ_BtoS using decoder 105
                                             //  7129:         }
  1, 0,                                      //  7133:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7135:          check predicate 25
  OPC_Decode, 215, 49, 105,                  //  7137:          decode to SMLALL_MZZ_HtoD using decoder 105
                                             //  7137:         }
                                             //  7137:         } // switch Inst[22]
                                             //  7137:        }
  1, 0,                                      //  7141:        case 0x1: {
  OPC_CheckPredicate, 8,                     //  7143:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  //  7145:         check Inst[22] == 0x0
  OPC_Decode, 138, 28, 105,                  //  7149:         decode to FMLALL_MZZ_BtoS using decoder 105
                                             //  7149:        }
                                             //  7149:        } // switch Inst[20]
                                             //  7149:       }
  1, 14,                                     //  7153:       case 0x1: {
  OPC_CheckPredicate, 7,                     //  7155:        check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7157:        check Inst[22] == 0x0
  OPC_CheckField, 20, 1, 0,                  //  7161:        check Inst[20] == 0x0
  OPC_Decode, 184, 68, 105,                  //  7165:        decode to USMLALL_MZZ_BtoS using decoder 105
                                             //  7165:       }
  2, 27,                                     //  7169:       case 0x2: {
  OPC_SwitchField, 22, 1,                    //  7171:        switch Inst[22] {
  0, 10,                                     //  7174:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7176:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7178:         check Inst[20] == 0x0
  OPC_Decode, 130, 50, 105,                  //  7182:         decode to SMLSLL_MZZ_BtoS using decoder 105
                                             //  7182:        }
  1, 0,                                      //  7186:        case 0x1: {
  OPC_CheckPredicate, 25,                    //  7188:         check predicate 25
  OPC_CheckField, 20, 1, 0,                  //  7190:         check Inst[20] == 0x0
  OPC_Decode, 131, 50, 105,                  //  7194:         decode to SMLSLL_MZZ_HtoD using decoder 105
                                             //  7194:        }
                                             //  7194:        } // switch Inst[22]
                                             //  7194:       }
  4, 27,                                     //  7198:       case 0x4: {
  OPC_SwitchField, 22, 1,                    //  7200:        switch Inst[22] {
  0, 10,                                     //  7203:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7205:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7207:         check Inst[20] == 0x0
  OPC_Decode, 223, 64, 105,                  //  7211:         decode to UMLALL_MZZ_BtoS using decoder 105
                                             //  7211:        }
  1, 0,                                      //  7215:        case 0x1: {
  OPC_CheckPredicate, 25,                    //  7217:         check predicate 25
  OPC_CheckField, 20, 1, 0,                  //  7219:         check Inst[20] == 0x0
  OPC_Decode, 224, 64, 105,                  //  7223:         decode to UMLALL_MZZ_HtoD using decoder 105
                                             //  7223:        }
                                             //  7223:        } // switch Inst[22]
                                             //  7223:       }
  6, 0,                                      //  7227:       case 0x6: {
  OPC_SwitchField, 22, 1,                    //  7229:        switch Inst[22] {
  0, 10,                                     //  7232:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7234:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7236:         check Inst[20] == 0x0
  OPC_Decode, 139, 65, 105,                  //  7240:         decode to UMLSLL_MZZ_BtoS using decoder 105
                                             //  7240:        }
  1, 0,                                      //  7244:        case 0x1: {
  OPC_CheckPredicate, 25,                    //  7246:         check predicate 25
  OPC_CheckField, 20, 1, 0,                  //  7248:         check Inst[20] == 0x0
  OPC_Decode, 140, 65, 105,                  //  7252:         decode to UMLSLL_MZZ_HtoD using decoder 105
                                             //  7252:        }
                                             //  7252:        } // switch Inst[22]
                                             //  7252:       }
                                             //  7252:       } // switch Inst[4:2]
                                             //  7252:      }
  2, 220, 1,                                 //  7256:      case 0x2: {
  OPC_SwitchField, 2, 3,                     //  7259:       switch Inst[4:2] {
  0, 45,                                     //  7262:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  7264:        switch Inst[20] {
  0, 19,                                     //  7267:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7269:         switch Inst[22] {
  0, 6,                                      //  7272:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7274:          check predicate 7
  OPC_Decode, 159, 28, 106,                  //  7276:          decode to FMLAL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7276:         }
  1, 0,                                      //  7280:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7282:          check predicate 7
  OPC_Decode, 237, 49, 106,                  //  7284:          decode to SMLAL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7284:         }
                                             //  7284:         } // switch Inst[22]
                                             //  7284:        }
  1, 0,                                      //  7288:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7290:         switch Inst[22] {
  0, 6,                                      //  7293:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7295:          check predicate 7
  OPC_Decode, 166, 28, 107,                  //  7297:          decode to FMLAL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7297:         }
  1, 0,                                      //  7301:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7303:          check predicate 7
  OPC_Decode, 240, 49, 107,                  //  7305:          decode to SMLAL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7305:         }
                                             //  7305:         } // switch Inst[22]
                                             //  7305:        }
                                             //  7305:        } // switch Inst[20]
                                             //  7305:       }
  1, 27,                                     //  7309:       case 0x1: {
  OPC_SwitchField, 20, 1,                    //  7311:        switch Inst[20] {
  0, 10,                                     //  7314:        case 0x0: {
  OPC_CheckPredicate, 9,                     //  7316:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  //  7318:         check Inst[22] == 0x0
  OPC_Decode, 158, 28, 106,                  //  7322:         decode to FMLAL_VG2_M2ZZ_BtoH using decoder 106
                                             //  7322:        }
  1, 0,                                      //  7326:        case 0x1: {
  OPC_CheckPredicate, 9,                     //  7328:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  //  7330:         check Inst[22] == 0x0
  OPC_Decode, 165, 28, 107,                  //  7334:         decode to FMLAL_VG4_M4ZZ_BtoH using decoder 107
                                             //  7334:        }
                                             //  7334:        } // switch Inst[20]
                                             //  7334:       }
  2, 45,                                     //  7338:       case 0x2: {
  OPC_SwitchField, 20, 1,                    //  7340:        switch Inst[20] {
  0, 19,                                     //  7343:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7345:         switch Inst[22] {
  0, 6,                                      //  7348:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7350:          check predicate 7
  OPC_Decode, 221, 28, 106,                  //  7352:          decode to FMLSL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7352:         }
  1, 0,                                      //  7356:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7358:          check predicate 7
  OPC_Decode, 153, 50, 106,                  //  7360:          decode to SMLSL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7360:         }
                                             //  7360:         } // switch Inst[22]
                                             //  7360:        }
  1, 0,                                      //  7364:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7366:         switch Inst[22] {
  0, 6,                                      //  7369:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7371:          check predicate 7
  OPC_Decode, 224, 28, 107,                  //  7373:          decode to FMLSL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7373:         }
  1, 0,                                      //  7377:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7379:          check predicate 7
  OPC_Decode, 156, 50, 107,                  //  7381:          decode to SMLSL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7381:         }
                                             //  7381:         } // switch Inst[22]
                                             //  7381:        }
                                             //  7381:        } // switch Inst[20]
                                             //  7381:       }
  4, 45,                                     //  7385:       case 0x4: {
  OPC_SwitchField, 20, 1,                    //  7387:        switch Inst[20] {
  0, 19,                                     //  7390:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7392:         switch Inst[22] {
  0, 6,                                      //  7395:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7397:          check predicate 7
  OPC_Decode, 181, 15, 106,                  //  7399:          decode to BFMLAL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7399:         }
  1, 0,                                      //  7403:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7405:          check predicate 7
  OPC_Decode, 246, 64, 106,                  //  7407:          decode to UMLAL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7407:         }
                                             //  7407:         } // switch Inst[22]
                                             //  7407:        }
  1, 0,                                      //  7411:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7413:         switch Inst[22] {
  0, 6,                                      //  7416:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7418:          check predicate 7
  OPC_Decode, 184, 15, 107,                  //  7420:          decode to BFMLAL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7420:         }
  1, 0,                                      //  7424:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7426:          check predicate 7
  OPC_Decode, 249, 64, 107,                  //  7428:          decode to UMLAL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7428:         }
                                             //  7428:         } // switch Inst[22]
                                             //  7428:        }
                                             //  7428:        } // switch Inst[20]
                                             //  7428:       }
  6, 0,                                      //  7432:       case 0x6: {
  OPC_SwitchField, 20, 1,                    //  7434:        switch Inst[20] {
  0, 19,                                     //  7437:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7439:         switch Inst[22] {
  0, 6,                                      //  7442:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7444:          check predicate 7
  OPC_Decode, 201, 15, 106,                  //  7446:          decode to BFMLSL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7446:         }
  1, 0,                                      //  7450:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7452:          check predicate 7
  OPC_Decode, 162, 65, 106,                  //  7454:          decode to UMLSL_VG2_M2ZZ_HtoS using decoder 106
                                             //  7454:         }
                                             //  7454:         } // switch Inst[22]
                                             //  7454:        }
  1, 0,                                      //  7458:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7460:         switch Inst[22] {
  0, 6,                                      //  7463:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7465:          check predicate 7
  OPC_Decode, 204, 15, 107,                  //  7467:          decode to BFMLSL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7467:         }
  1, 0,                                      //  7471:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7473:          check predicate 7
  OPC_Decode, 165, 65, 107,                  //  7475:          decode to UMLSL_VG4_M4ZZ_HtoS using decoder 107
                                             //  7475:         }
                                             //  7475:         } // switch Inst[22]
                                             //  7475:        }
                                             //  7475:        } // switch Inst[20]
                                             //  7475:       }
                                             //  7475:       } // switch Inst[4:2]
                                             //  7475:      }
  3, 128, 1,                                 //  7479:      case 0x3: {
  OPC_SwitchField, 3, 2,                     //  7482:       switch Inst[4:3] {
  0, 36,                                     //  7485:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  7487:        switch Inst[20] {
  0, 19,                                     //  7490:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7492:         switch Inst[22] {
  0, 6,                                      //  7495:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7497:          check predicate 7
  OPC_Decode, 153, 28, 108,                  //  7499:          decode to FMLAL_MZZ_HtoS using decoder 108
                                             //  7499:         }
  1, 0,                                      //  7503:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7505:          check predicate 7
  OPC_Decode, 234, 49, 108,                  //  7507:          decode to SMLAL_MZZ_HtoS using decoder 108
                                             //  7507:         }
                                             //  7507:         } // switch Inst[22]
                                             //  7507:        }
  1, 0,                                      //  7511:        case 0x1: {
  OPC_CheckPredicate, 9,                     //  7513:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  //  7515:         check Inst[22] == 0x0
  OPC_Decode, 160, 28, 108,                  //  7519:         decode to FMLAL_VG2_MZZ_BtoH using decoder 108
                                             //  7519:        }
                                             //  7519:        } // switch Inst[20]
                                             //  7519:       }
  1, 27,                                     //  7523:       case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7525:        switch Inst[22] {
  0, 10,                                     //  7528:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7530:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7532:         check Inst[20] == 0x0
  OPC_Decode, 218, 28, 108,                  //  7536:         decode to FMLSL_MZZ_HtoS using decoder 108
                                             //  7536:        }
  1, 0,                                      //  7540:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  7542:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7544:         check Inst[20] == 0x0
  OPC_Decode, 150, 50, 108,                  //  7548:         decode to SMLSL_MZZ_HtoS using decoder 108
                                             //  7548:        }
                                             //  7548:        } // switch Inst[22]
                                             //  7548:       }
  2, 27,                                     //  7552:       case 0x2: {
  OPC_SwitchField, 22, 1,                    //  7554:        switch Inst[22] {
  0, 10,                                     //  7557:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7559:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7561:         check Inst[20] == 0x0
  OPC_Decode, 178, 15, 108,                  //  7565:         decode to BFMLAL_MZZ_HtoS using decoder 108
                                             //  7565:        }
  1, 0,                                      //  7569:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  7571:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7573:         check Inst[20] == 0x0
  OPC_Decode, 243, 64, 108,                  //  7577:         decode to UMLAL_MZZ_HtoS using decoder 108
                                             //  7577:        }
                                             //  7577:        } // switch Inst[22]
                                             //  7577:       }
  3, 0,                                      //  7581:       case 0x3: {
  OPC_SwitchField, 22, 1,                    //  7583:        switch Inst[22] {
  0, 10,                                     //  7586:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7588:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7590:         check Inst[20] == 0x0
  OPC_Decode, 198, 15, 108,                  //  7594:         decode to BFMLSL_MZZ_HtoS using decoder 108
                                             //  7594:        }
  1, 0,                                      //  7598:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  7600:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  7602:         check Inst[20] == 0x0
  OPC_Decode, 159, 65, 108,                  //  7606:         decode to UMLSL_MZZ_HtoS using decoder 108
                                             //  7606:        }
                                             //  7606:        } // switch Inst[22]
                                             //  7606:       }
                                             //  7606:       } // switch Inst[4:3]
                                             //  7606:      }
  4, 119,                                    //  7610:      case 0x4: {
  OPC_SwitchField, 3, 2,                     //  7612:       switch Inst[4:3] {
  0, 27,                                     //  7615:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  7617:        switch Inst[20] {
  0, 10,                                     //  7620:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7622:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7624:         check Inst[22] == 0x0
  OPC_Decode, 132, 26, 109,                  //  7628:         decode to FDOT_VG2_M2ZZ_HtoS using decoder 109
                                             //  7628:        }
  1, 0,                                      //  7632:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  7634:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7636:         check Inst[22] == 0x0
  OPC_Decode, 141, 26, 110,                  //  7640:         decode to FDOT_VG4_M4ZZ_HtoS using decoder 110
                                             //  7640:        }
                                             //  7640:        } // switch Inst[20]
                                             //  7640:       }
  1, 27,                                     //  7644:       case 0x1: {
  OPC_SwitchField, 20, 1,                    //  7646:        switch Inst[20] {
  0, 10,                                     //  7649:        case 0x0: {
  OPC_CheckPredicate, 9,                     //  7651:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  //  7653:         check Inst[22] == 0x0
  OPC_Decode, 130, 26, 109,                  //  7657:         decode to FDOT_VG2_M2ZZ_BtoH using decoder 109
                                             //  7657:        }
  1, 0,                                      //  7661:        case 0x1: {
  OPC_CheckPredicate, 9,                     //  7663:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  //  7665:         check Inst[22] == 0x0
  OPC_Decode, 139, 26, 110,                  //  7669:         decode to FDOT_VG4_M4ZZ_BtoH using decoder 110
                                             //  7669:        }
                                             //  7669:        } // switch Inst[20]
                                             //  7669:       }
  2, 27,                                     //  7673:       case 0x2: {
  OPC_SwitchField, 20, 1,                    //  7675:        switch Inst[20] {
  0, 10,                                     //  7678:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  7680:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7682:         check Inst[22] == 0x0
  OPC_Decode, 141, 15, 109,                  //  7686:         decode to BFDOT_VG2_M2ZZ_HtoS using decoder 109
                                             //  7686:        }
  1, 0,                                      //  7690:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  7692:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  //  7694:         check Inst[22] == 0x0
  OPC_Decode, 144, 15, 110,                  //  7698:         decode to BFDOT_VG4_M4ZZ_HtoS using decoder 110
                                             //  7698:        }
                                             //  7698:        } // switch Inst[20]
                                             //  7698:       }
  3, 0,                                      //  7702:       case 0x3: {
  OPC_SwitchField, 20, 1,                    //  7704:        switch Inst[20] {
  0, 10,                                     //  7707:        case 0x0: {
  OPC_CheckPredicate, 8,                     //  7709:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  //  7711:         check Inst[22] == 0x0
  OPC_Decode, 131, 26, 109,                  //  7715:         decode to FDOT_VG2_M2ZZ_BtoS using decoder 109
                                             //  7715:        }
  1, 0,                                      //  7719:        case 0x1: {
  OPC_CheckPredicate, 8,                     //  7721:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  //  7723:         check Inst[22] == 0x0
  OPC_Decode, 140, 26, 110,                  //  7727:         decode to FDOT_VG4_M4ZZ_BtoS using decoder 110
                                             //  7727:        }
                                             //  7727:        } // switch Inst[20]
                                             //  7727:       }
                                             //  7727:       } // switch Inst[4:3]
                                             //  7727:      }
  5, 191, 1,                                 //  7731:      case 0x5: {
  OPC_SwitchField, 3, 2,                     //  7734:       switch Inst[4:3] {
  0, 45,                                     //  7737:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  7739:        switch Inst[20] {
  0, 19,                                     //  7742:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7744:         switch Inst[22] {
  0, 6,                                      //  7747:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7749:          check predicate 7
  OPC_Decode, 187, 47, 109,                  //  7751:          decode to SDOT_VG2_M2ZZ_BtoS using decoder 109
                                             //  7751:         }
  1, 0,                                      //  7755:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7757:          check predicate 25
  OPC_Decode, 188, 47, 109,                  //  7759:          decode to SDOT_VG2_M2ZZ_HtoD using decoder 109
                                             //  7759:         }
                                             //  7759:         } // switch Inst[22]
                                             //  7759:        }
  1, 0,                                      //  7763:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7765:         switch Inst[22] {
  0, 6,                                      //  7768:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7770:          check predicate 7
  OPC_Decode, 196, 47, 110,                  //  7772:          decode to SDOT_VG4_M4ZZ_BtoS using decoder 110
                                             //  7772:         }
  1, 0,                                      //  7776:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7778:          check predicate 25
  OPC_Decode, 197, 47, 110,                  //  7780:          decode to SDOT_VG4_M4ZZ_HtoD using decoder 110
                                             //  7780:         }
                                             //  7780:         } // switch Inst[22]
                                             //  7780:        }
                                             //  7780:        } // switch Inst[20]
                                             //  7780:       }
  1, 45,                                     //  7784:       case 0x1: {
  OPC_SwitchField, 20, 1,                    //  7786:        switch Inst[20] {
  0, 19,                                     //  7789:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7791:         switch Inst[22] {
  0, 6,                                      //  7794:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7796:          check predicate 7
  OPC_Decode, 145, 68, 109,                  //  7798:          decode to USDOT_VG2_M2ZZ_BToS using decoder 109
                                             //  7798:         }
  1, 0,                                      //  7802:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7804:          check predicate 7
  OPC_Decode, 189, 47, 109,                  //  7806:          decode to SDOT_VG2_M2ZZ_HtoS using decoder 109
                                             //  7806:         }
                                             //  7806:         } // switch Inst[22]
                                             //  7806:        }
  1, 0,                                      //  7810:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7812:         switch Inst[22] {
  0, 6,                                      //  7815:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7817:          check predicate 7
  OPC_Decode, 148, 68, 110,                  //  7819:          decode to USDOT_VG4_M4ZZ_BToS using decoder 110
                                             //  7819:         }
  1, 0,                                      //  7823:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7825:          check predicate 7
  OPC_Decode, 198, 47, 110,                  //  7827:          decode to SDOT_VG4_M4ZZ_HtoS using decoder 110
                                             //  7827:         }
                                             //  7827:         } // switch Inst[22]
                                             //  7827:        }
                                             //  7827:        } // switch Inst[20]
                                             //  7827:       }
  2, 45,                                     //  7831:       case 0x2: {
  OPC_SwitchField, 20, 1,                    //  7833:        switch Inst[20] {
  0, 19,                                     //  7836:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7838:         switch Inst[22] {
  0, 6,                                      //  7841:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7843:          check predicate 7
  OPC_Decode, 181, 63, 109,                  //  7845:          decode to UDOT_VG2_M2ZZ_BtoS using decoder 109
                                             //  7845:         }
  1, 0,                                      //  7849:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7851:          check predicate 25
  OPC_Decode, 182, 63, 109,                  //  7853:          decode to UDOT_VG2_M2ZZ_HtoD using decoder 109
                                             //  7853:         }
                                             //  7853:         } // switch Inst[22]
                                             //  7853:        }
  1, 0,                                      //  7857:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7859:         switch Inst[22] {
  0, 6,                                      //  7862:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7864:          check predicate 7
  OPC_Decode, 190, 63, 110,                  //  7866:          decode to UDOT_VG4_M4ZZ_BtoS using decoder 110
                                             //  7866:         }
  1, 0,                                      //  7870:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  7872:          check predicate 25
  OPC_Decode, 191, 63, 110,                  //  7874:          decode to UDOT_VG4_M4ZZ_HtoD using decoder 110
                                             //  7874:         }
                                             //  7874:         } // switch Inst[22]
                                             //  7874:        }
                                             //  7874:        } // switch Inst[20]
                                             //  7874:       }
  3, 0,                                      //  7878:       case 0x3: {
  OPC_SwitchField, 20, 1,                    //  7880:        switch Inst[20] {
  0, 19,                                     //  7883:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7885:         switch Inst[22] {
  0, 6,                                      //  7888:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7890:          check predicate 7
  OPC_Decode, 201, 60, 109,                  //  7892:          decode to SUDOT_VG2_M2ZZ_BToS using decoder 109
                                             //  7892:         }
  1, 0,                                      //  7896:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7898:          check predicate 7
  OPC_Decode, 183, 63, 109,                  //  7900:          decode to UDOT_VG2_M2ZZ_HtoS using decoder 109
                                             //  7900:         }
                                             //  7900:         } // switch Inst[22]
                                             //  7900:        }
  1, 0,                                      //  7904:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7906:         switch Inst[22] {
  0, 6,                                      //  7909:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7911:          check predicate 7
  OPC_Decode, 203, 60, 110,                  //  7913:          decode to SUDOT_VG4_M4ZZ_BToS using decoder 110
                                             //  7913:         }
  1, 0,                                      //  7917:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  7919:          check predicate 7
  OPC_Decode, 192, 63, 110,                  //  7921:          decode to UDOT_VG4_M4ZZ_HtoS using decoder 110
                                             //  7921:         }
                                             //  7921:         } // switch Inst[22]
                                             //  7921:        }
                                             //  7921:        } // switch Inst[20]
                                             //  7921:       }
                                             //  7921:       } // switch Inst[4:3]
                                             //  7921:      }
  6, 191, 1,                                 //  7925:      case 0x6: {
  OPC_SwitchField, 3, 2,                     //  7928:       switch Inst[4:3] {
  0, 45,                                     //  7931:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  7933:        switch Inst[20] {
  0, 19,                                     //  7936:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7938:         switch Inst[22] {
  0, 6,                                      //  7941:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7943:          check predicate 7
  OPC_Decode, 179, 28, 109,                  //  7945:          decode to FMLA_VG2_M2ZZ_S using decoder 109
                                             //  7945:         }
  1, 0,                                      //  7949:         case 0x1: {
  OPC_CheckPredicate, 26,                    //  7951:          check predicate 26
  OPC_Decode, 177, 28, 109,                  //  7953:          decode to FMLA_VG2_M2ZZ_D using decoder 109
                                             //  7953:         }
                                             //  7953:         } // switch Inst[22]
                                             //  7953:        }
  1, 0,                                      //  7957:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  7959:         switch Inst[22] {
  0, 6,                                      //  7962:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7964:          check predicate 7
  OPC_Decode, 188, 28, 110,                  //  7966:          decode to FMLA_VG4_M4ZZ_S using decoder 110
                                             //  7966:         }
  1, 0,                                      //  7970:         case 0x1: {
  OPC_CheckPredicate, 26,                    //  7972:          check predicate 26
  OPC_Decode, 186, 28, 110,                  //  7974:          decode to FMLA_VG4_M4ZZ_D using decoder 110
                                             //  7974:         }
                                             //  7974:         } // switch Inst[22]
                                             //  7974:        }
                                             //  7974:        } // switch Inst[20]
                                             //  7974:       }
  1, 45,                                     //  7978:       case 0x1: {
  OPC_SwitchField, 20, 1,                    //  7980:        switch Inst[20] {
  0, 19,                                     //  7983:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  7985:         switch Inst[22] {
  0, 6,                                      //  7988:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  7990:          check predicate 7
  OPC_Decode, 237, 28, 109,                  //  7992:          decode to FMLS_VG2_M2ZZ_S using decoder 109
                                             //  7992:         }
  1, 0,                                      //  7996:         case 0x1: {
  OPC_CheckPredicate, 26,                    //  7998:          check predicate 26
  OPC_Decode, 235, 28, 109,                  //  8000:          decode to FMLS_VG2_M2ZZ_D using decoder 109
                                             //  8000:         }
                                             //  8000:         } // switch Inst[22]
                                             //  8000:        }
  1, 0,                                      //  8004:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8006:         switch Inst[22] {
  0, 6,                                      //  8009:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8011:          check predicate 7
  OPC_Decode, 246, 28, 110,                  //  8013:          decode to FMLS_VG4_M4ZZ_S using decoder 110
                                             //  8013:         }
  1, 0,                                      //  8017:         case 0x1: {
  OPC_CheckPredicate, 26,                    //  8019:          check predicate 26
  OPC_Decode, 244, 28, 110,                  //  8021:          decode to FMLS_VG4_M4ZZ_D using decoder 110
                                             //  8021:         }
                                             //  8021:         } // switch Inst[22]
                                             //  8021:        }
                                             //  8021:        } // switch Inst[20]
                                             //  8021:       }
  2, 45,                                     //  8025:       case 0x2: {
  OPC_SwitchField, 20, 1,                    //  8027:        switch Inst[20] {
  0, 19,                                     //  8030:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8032:         switch Inst[22] {
  0, 6,                                      //  8035:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8037:          check predicate 7
  OPC_Decode, 206, 13, 109,                  //  8039:          decode to ADD_VG2_M2ZZ_S using decoder 109
                                             //  8039:         }
  1, 0,                                      //  8043:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  8045:          check predicate 25
  OPC_Decode, 205, 13, 109,                  //  8047:          decode to ADD_VG2_M2ZZ_D using decoder 109
                                             //  8047:         }
                                             //  8047:         } // switch Inst[22]
                                             //  8047:        }
  1, 0,                                      //  8051:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8053:         switch Inst[22] {
  0, 6,                                      //  8056:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8058:          check predicate 7
  OPC_Decode, 216, 13, 110,                  //  8060:          decode to ADD_VG4_M4ZZ_S using decoder 110
                                             //  8060:         }
  1, 0,                                      //  8064:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  8066:          check predicate 25
  OPC_Decode, 215, 13, 110,                  //  8068:          decode to ADD_VG4_M4ZZ_D using decoder 110
                                             //  8068:         }
                                             //  8068:         } // switch Inst[22]
                                             //  8068:        }
                                             //  8068:        } // switch Inst[20]
                                             //  8068:       }
  3, 0,                                      //  8072:       case 0x3: {
  OPC_SwitchField, 20, 1,                    //  8074:        switch Inst[20] {
  0, 19,                                     //  8077:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8079:         switch Inst[22] {
  0, 6,                                      //  8082:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8084:          check predicate 7
  OPC_Decode, 169, 60, 109,                  //  8086:          decode to SUB_VG2_M2ZZ_S using decoder 109
                                             //  8086:         }
  1, 0,                                      //  8090:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  8092:          check predicate 25
  OPC_Decode, 168, 60, 109,                  //  8094:          decode to SUB_VG2_M2ZZ_D using decoder 109
                                             //  8094:         }
                                             //  8094:         } // switch Inst[22]
                                             //  8094:        }
  1, 0,                                      //  8098:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8100:         switch Inst[22] {
  0, 6,                                      //  8103:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8105:          check predicate 7
  OPC_Decode, 175, 60, 110,                  //  8107:          decode to SUB_VG4_M4ZZ_S using decoder 110
                                             //  8107:         }
  1, 0,                                      //  8111:         case 0x1: {
  OPC_CheckPredicate, 25,                    //  8113:          check predicate 25
  OPC_Decode, 174, 60, 110,                  //  8115:          decode to SUB_VG4_M4ZZ_D using decoder 110
                                             //  8115:         }
                                             //  8115:         } // switch Inst[22]
                                             //  8115:        }
                                             //  8115:        } // switch Inst[20]
                                             //  8115:       }
                                             //  8115:       } // switch Inst[4:3]
                                             //  8115:      }
  7, 0,                                      //  8119:      case 0x7: {
  OPC_SwitchField, 3, 2,                     //  8121:       switch Inst[4:3] {
  0, 45,                                     //  8124:       case 0x0: {
  OPC_SwitchField, 20, 1,                    //  8126:        switch Inst[20] {
  0, 19,                                     //  8129:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8131:         switch Inst[22] {
  0, 6,                                      //  8134:         case 0x0: {
  OPC_CheckPredicate, 16,                    //  8136:          check predicate 16
  OPC_Decode, 178, 28, 109,                  //  8138:          decode to FMLA_VG2_M2ZZ_H using decoder 109
                                             //  8138:         }
  1, 0,                                      //  8142:         case 0x1: {
  OPC_CheckPredicate, 17,                    //  8144:          check predicate 17
  OPC_Decode, 186, 15, 109,                  //  8146:          decode to BFMLA_VG2_M2ZZ using decoder 109
                                             //  8146:         }
                                             //  8146:         } // switch Inst[22]
                                             //  8146:        }
  1, 0,                                      //  8150:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8152:         switch Inst[22] {
  0, 6,                                      //  8155:         case 0x0: {
  OPC_CheckPredicate, 16,                    //  8157:          check predicate 16
  OPC_Decode, 187, 28, 110,                  //  8159:          decode to FMLA_VG4_M4ZZ_H using decoder 110
                                             //  8159:         }
  1, 0,                                      //  8163:         case 0x1: {
  OPC_CheckPredicate, 17,                    //  8165:          check predicate 17
  OPC_Decode, 189, 15, 110,                  //  8167:          decode to BFMLA_VG4_M4ZZ using decoder 110
                                             //  8167:         }
                                             //  8167:         } // switch Inst[22]
                                             //  8167:        }
                                             //  8167:        } // switch Inst[20]
                                             //  8167:       }
  1, 0,                                      //  8171:       case 0x1: {
  OPC_SwitchField, 20, 1,                    //  8173:        switch Inst[20] {
  0, 19,                                     //  8176:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8178:         switch Inst[22] {
  0, 6,                                      //  8181:         case 0x0: {
  OPC_CheckPredicate, 16,                    //  8183:          check predicate 16
  OPC_Decode, 236, 28, 109,                  //  8185:          decode to FMLS_VG2_M2ZZ_H using decoder 109
                                             //  8185:         }
  1, 0,                                      //  8189:         case 0x1: {
  OPC_CheckPredicate, 17,                    //  8191:          check predicate 17
  OPC_Decode, 206, 15, 109,                  //  8193:          decode to BFMLS_VG2_M2ZZ using decoder 109
                                             //  8193:         }
                                             //  8193:         } // switch Inst[22]
                                             //  8193:        }
  1, 0,                                      //  8197:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8199:         switch Inst[22] {
  0, 6,                                      //  8202:         case 0x0: {
  OPC_CheckPredicate, 16,                    //  8204:          check predicate 16
  OPC_Decode, 245, 28, 110,                  //  8206:          decode to FMLS_VG4_M4ZZ_H using decoder 110
                                             //  8206:         }
  1, 0,                                      //  8210:         case 0x1: {
  OPC_CheckPredicate, 17,                    //  8212:          check predicate 17
  OPC_Decode, 209, 15, 110,                  //  8214:          decode to BFMLS_VG4_M4ZZ using decoder 110
                                             //  8214:         }
                                             //  8214:         } // switch Inst[22]
                                             //  8214:        }
                                             //  8214:        } // switch Inst[20]
                                             //  8214:       }
                                             //  8214:       } // switch Inst[4:3]
                                             //  8214:      }
                                             //  8214:      } // switch Inst[12:10]
                                             //  8214:     }
  1, 0,                                      //  8218:     case 0x1: {
  OPC_SwitchField, 13, 2,                    //  8220:      switch Inst[14:13] {
  0, 85,                                     //  8223:      case 0x0: {
  OPC_SwitchField, 16, 1,                    //  8225:       switch Inst[16] {
  0, 35,                                     //  8228:       case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8230:        switch Inst[22] {
  0, 14,                                     //  8233:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  8235:         check predicate 7
  OPC_CheckField, 5, 1, 0,                   //  8237:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8241:         check Inst[0] == 0x0
  OPC_Decode, 212, 47, 111,                  //  8245:         decode to SEL_VG2_2ZC2Z2Z_B using decoder 111
                                             //  8245:        }
  1, 0,                                      //  8249:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  8251:         check predicate 7
  OPC_CheckField, 5, 1, 0,                   //  8253:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8257:         check Inst[0] == 0x0
  OPC_Decode, 214, 47, 111,                  //  8261:         decode to SEL_VG2_2ZC2Z2Z_H using decoder 111
                                             //  8261:        }
                                             //  8261:        } // switch Inst[22]
                                             //  8261:       }
  1, 0,                                      //  8265:       case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8267:        switch Inst[22] {
  0, 18,                                     //  8270:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  8272:         check predicate 7
  OPC_CheckField, 17, 1, 0,                  //  8274:         check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   //  8278:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  8282:         check Inst[1:0] == 0x0
  OPC_Decode, 216, 47, 112,                  //  8286:         decode to SEL_VG4_4ZC4Z4Z_B using decoder 112
                                             //  8286:        }
  1, 0,                                      //  8290:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  8292:         check predicate 7
  OPC_CheckField, 17, 1, 0,                  //  8294:         check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   //  8298:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  8302:         check Inst[1:0] == 0x0
  OPC_Decode, 218, 47, 112,                  //  8306:         decode to SEL_VG4_4ZC4Z4Z_H using decoder 112
                                             //  8306:        }
                                             //  8306:        } // switch Inst[22]
                                             //  8306:       }
                                             //  8306:       } // switch Inst[16]
                                             //  8306:      }
  1, 187, 13,                                //  8310:      case 0x1: {
  OPC_SwitchField, 5, 8,                     //  8313:       switch Inst[12:5] {
  0, 61,                                     //  8316:       case 0x0: {
  OPC_SwitchField, 0, 1,                     //  8318:        switch Inst[0] {
  0, 27,                                     //  8321:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8323:         switch Inst[22] {
  0, 10,                                     //  8326:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8328:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8330:          check Inst[20] == 0x0
  OPC_Decode, 251, 48, 113,                  //  8334:          decode to SMAX_VG2_2ZZ_B using decoder 113
                                             //  8334:         }
  1, 0,                                      //  8338:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8340:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8342:          check Inst[20] == 0x0
  OPC_Decode, 253, 48, 113,                  //  8346:          decode to SMAX_VG2_2ZZ_H using decoder 113
                                             //  8346:         }
                                             //  8346:         } // switch Inst[22]
                                             //  8346:        }
  1, 0,                                      //  8350:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8352:         switch Inst[22] {
  0, 10,                                     //  8355:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8357:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8359:          check Inst[20] == 0x0
  OPC_Decode, 133, 64, 113,                  //  8363:          decode to UMAX_VG2_2ZZ_B using decoder 113
                                             //  8363:         }
  1, 0,                                      //  8367:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8369:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8371:          check Inst[20] == 0x0
  OPC_Decode, 135, 64, 113,                  //  8375:          decode to UMAX_VG2_2ZZ_H using decoder 113
                                             //  8375:         }
                                             //  8375:         } // switch Inst[22]
                                             //  8375:        }
                                             //  8375:        } // switch Inst[0]
                                             //  8375:       }
  1, 61,                                     //  8379:       case 0x1: {
  OPC_SwitchField, 0, 1,                     //  8381:        switch Inst[0] {
  0, 27,                                     //  8384:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8386:         switch Inst[22] {
  0, 10,                                     //  8389:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8391:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8393:          check Inst[20] == 0x0
  OPC_Decode, 181, 49, 113,                  //  8397:          decode to SMIN_VG2_2ZZ_B using decoder 113
                                             //  8397:         }
  1, 0,                                      //  8401:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8403:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8405:          check Inst[20] == 0x0
  OPC_Decode, 183, 49, 113,                  //  8409:          decode to SMIN_VG2_2ZZ_H using decoder 113
                                             //  8409:         }
                                             //  8409:         } // switch Inst[22]
                                             //  8409:        }
  1, 0,                                      //  8413:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8415:         switch Inst[22] {
  0, 10,                                     //  8418:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8420:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8422:          check Inst[20] == 0x0
  OPC_Decode, 190, 64, 113,                  //  8426:          decode to UMIN_VG2_2ZZ_B using decoder 113
                                             //  8426:         }
  1, 0,                                      //  8430:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8432:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8434:          check Inst[20] == 0x0
  OPC_Decode, 192, 64, 113,                  //  8438:          decode to UMIN_VG2_2ZZ_H using decoder 113
                                             //  8438:         }
                                             //  8438:         } // switch Inst[22]
                                             //  8438:        }
                                             //  8438:        } // switch Inst[0]
                                             //  8438:       }
  8, 61,                                     //  8442:       case 0x8: {
  OPC_SwitchField, 0, 1,                     //  8444:        switch Inst[0] {
  0, 27,                                     //  8447:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8449:         switch Inst[22] {
  0, 10,                                     //  8452:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8454:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8456:          check Inst[20] == 0x0
  OPC_Decode, 155, 15, 113,                  //  8460:          decode to BFMAX_VG2_2ZZ_H using decoder 113
                                             //  8460:         }
  1, 0,                                      //  8464:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8466:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8468:          check Inst[20] == 0x0
  OPC_Decode, 128, 27, 113,                  //  8472:          decode to FMAX_VG2_2ZZ_H using decoder 113
                                             //  8472:         }
                                             //  8472:         } // switch Inst[22]
                                             //  8472:        }
  1, 0,                                      //  8476:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8478:         switch Inst[22] {
  0, 10,                                     //  8481:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8483:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8485:          check Inst[20] == 0x0
  OPC_Decode, 165, 15, 113,                  //  8489:          decode to BFMIN_VG2_2ZZ_H using decoder 113
                                             //  8489:         }
  1, 0,                                      //  8493:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8495:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8497:          check Inst[20] == 0x0
  OPC_Decode, 220, 27, 113,                  //  8501:          decode to FMIN_VG2_2ZZ_H using decoder 113
                                             //  8501:         }
                                             //  8501:         } // switch Inst[22]
                                             //  8501:        }
                                             //  8501:        } // switch Inst[0]
                                             //  8501:       }
  9, 61,                                     //  8505:       case 0x9: {
  OPC_SwitchField, 0, 1,                     //  8507:        switch Inst[0] {
  0, 27,                                     //  8510:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8512:         switch Inst[22] {
  0, 10,                                     //  8515:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8517:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8519:          check Inst[20] == 0x0
  OPC_Decode, 150, 15, 113,                  //  8523:          decode to BFMAXNM_VG2_2ZZ_H using decoder 113
                                             //  8523:         }
  1, 0,                                      //  8527:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8529:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8531:          check Inst[20] == 0x0
  OPC_Decode, 212, 26, 113,                  //  8535:          decode to FMAXNM_VG2_2ZZ_H using decoder 113
                                             //  8535:         }
                                             //  8535:         } // switch Inst[22]
                                             //  8535:        }
  1, 0,                                      //  8539:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8541:         switch Inst[22] {
  0, 10,                                     //  8544:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8546:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8548:          check Inst[20] == 0x0
  OPC_Decode, 160, 15, 113,                  //  8552:          decode to BFMINNM_VG2_2ZZ_H using decoder 113
                                             //  8552:         }
  1, 0,                                      //  8556:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8558:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8560:          check Inst[20] == 0x0
  OPC_Decode, 176, 27, 113,                  //  8564:          decode to FMINNM_VG2_2ZZ_H using decoder 113
                                             //  8564:         }
                                             //  8564:         } // switch Inst[22]
                                             //  8564:        }
                                             //  8564:        } // switch Inst[0]
                                             //  8564:       }
  12, 35,                                    //  8568:       case 0xc: {
  OPC_SwitchField, 22, 1,                    //  8570:        switch Inst[22] {
  0, 14,                                     //  8573:        case 0x0: {
  OPC_CheckPredicate, 28,                    //  8575:         check predicate 28
  OPC_CheckField, 20, 1, 0,                  //  8577:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8581:         check Inst[0] == 0x0
  OPC_Decode, 246, 15, 113,                  //  8585:         decode to BFSCALE_2ZZ using decoder 113
                                             //  8585:        }
  1, 0,                                      //  8589:        case 0x1: {
  OPC_CheckPredicate, 29,                    //  8591:         check predicate 29
  OPC_CheckField, 20, 1, 0,                  //  8593:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8597:         check Inst[0] == 0x0
  OPC_Decode, 133, 32, 113,                  //  8601:         decode to FSCALE_2ZZ_H using decoder 113
                                             //  8601:        }
                                             //  8601:        } // switch Inst[22]
                                             //  8601:       }
  17, 61,                                    //  8605:       case 0x11: {
  OPC_SwitchField, 0, 1,                     //  8607:        switch Inst[0] {
  0, 27,                                     //  8610:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8612:         switch Inst[22] {
  0, 10,                                     //  8615:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8617:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8619:          check Inst[20] == 0x0
  OPC_Decode, 131, 55, 113,                  //  8623:          decode to SRSHL_VG2_2ZZ_B using decoder 113
                                             //  8623:         }
  1, 0,                                      //  8627:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8629:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8631:          check Inst[20] == 0x0
  OPC_Decode, 133, 55, 113,                  //  8635:          decode to SRSHL_VG2_2ZZ_H using decoder 113
                                             //  8635:         }
                                             //  8635:         } // switch Inst[22]
                                             //  8635:        }
  1, 0,                                      //  8639:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8641:         switch Inst[22] {
  0, 10,                                     //  8644:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8646:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8648:          check Inst[20] == 0x0
  OPC_Decode, 219, 67, 113,                  //  8652:          decode to URSHL_VG2_2ZZ_B using decoder 113
                                             //  8652:         }
  1, 0,                                      //  8656:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8658:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8660:          check Inst[20] == 0x0
  OPC_Decode, 221, 67, 113,                  //  8664:          decode to URSHL_VG2_2ZZ_H using decoder 113
                                             //  8664:         }
                                             //  8664:         } // switch Inst[22]
                                             //  8664:        }
                                             //  8664:        } // switch Inst[0]
                                             //  8664:       }
  24, 35,                                    //  8668:       case 0x18: {
  OPC_SwitchField, 22, 1,                    //  8670:        switch Inst[22] {
  0, 14,                                     //  8673:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  8675:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8677:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8681:         check Inst[0] == 0x0
  OPC_Decode, 199, 13, 113,                  //  8685:         decode to ADD_VG2_2ZZ_B using decoder 113
                                             //  8685:        }
  1, 0,                                      //  8689:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  8691:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8693:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8697:         check Inst[0] == 0x0
  OPC_Decode, 201, 13, 113,                  //  8701:         decode to ADD_VG2_2ZZ_H using decoder 113
                                             //  8701:        }
                                             //  8701:        } // switch Inst[22]
                                             //  8701:       }
  32, 35,                                    //  8705:       case 0x20: {
  OPC_SwitchField, 22, 1,                    //  8707:        switch Inst[22] {
  0, 14,                                     //  8710:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  8712:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8714:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8718:         check Inst[0] == 0x0
  OPC_Decode, 253, 51, 113,                  //  8722:         decode to SQDMULH_VG2_2ZZ_B using decoder 113
                                             //  8722:        }
  1, 0,                                      //  8726:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  8728:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8730:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  8734:         check Inst[0] == 0x0
  OPC_Decode, 255, 51, 113,                  //  8738:         decode to SQDMULH_VG2_2ZZ_H using decoder 113
                                             //  8738:        }
                                             //  8738:        } // switch Inst[22]
                                             //  8738:       }
  64, 61,                                    //  8742:       case 0x40: {
  OPC_SwitchField, 0, 2,                     //  8744:        switch Inst[1:0] {
  0, 27,                                     //  8747:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8749:         switch Inst[22] {
  0, 10,                                     //  8752:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8754:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8756:          check Inst[20] == 0x0
  OPC_Decode, 131, 49, 114,                  //  8760:          decode to SMAX_VG4_4ZZ_B using decoder 114
                                             //  8760:         }
  1, 0,                                      //  8764:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8766:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8768:          check Inst[20] == 0x0
  OPC_Decode, 133, 49, 114,                  //  8772:          decode to SMAX_VG4_4ZZ_H using decoder 114
                                             //  8772:         }
                                             //  8772:         } // switch Inst[22]
                                             //  8772:        }
  1, 0,                                      //  8776:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8778:         switch Inst[22] {
  0, 10,                                     //  8781:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8783:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8785:          check Inst[20] == 0x0
  OPC_Decode, 141, 64, 114,                  //  8789:          decode to UMAX_VG4_4ZZ_B using decoder 114
                                             //  8789:         }
  1, 0,                                      //  8793:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8795:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8797:          check Inst[20] == 0x0
  OPC_Decode, 143, 64, 114,                  //  8801:          decode to UMAX_VG4_4ZZ_H using decoder 114
                                             //  8801:         }
                                             //  8801:         } // switch Inst[22]
                                             //  8801:        }
                                             //  8801:        } // switch Inst[1:0]
                                             //  8801:       }
  65, 61,                                    //  8805:       case 0x41: {
  OPC_SwitchField, 0, 2,                     //  8807:        switch Inst[1:0] {
  0, 27,                                     //  8810:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8812:         switch Inst[22] {
  0, 10,                                     //  8815:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8817:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8819:          check Inst[20] == 0x0
  OPC_Decode, 189, 49, 114,                  //  8823:          decode to SMIN_VG4_4ZZ_B using decoder 114
                                             //  8823:         }
  1, 0,                                      //  8827:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8829:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8831:          check Inst[20] == 0x0
  OPC_Decode, 191, 49, 114,                  //  8835:          decode to SMIN_VG4_4ZZ_H using decoder 114
                                             //  8835:         }
                                             //  8835:         } // switch Inst[22]
                                             //  8835:        }
  1, 0,                                      //  8839:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8841:         switch Inst[22] {
  0, 10,                                     //  8844:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  8846:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8848:          check Inst[20] == 0x0
  OPC_Decode, 198, 64, 114,                  //  8852:          decode to UMIN_VG4_4ZZ_B using decoder 114
                                             //  8852:         }
  1, 0,                                      //  8856:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8858:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8860:          check Inst[20] == 0x0
  OPC_Decode, 200, 64, 114,                  //  8864:          decode to UMIN_VG4_4ZZ_H using decoder 114
                                             //  8864:         }
                                             //  8864:         } // switch Inst[22]
                                             //  8864:        }
                                             //  8864:        } // switch Inst[1:0]
                                             //  8864:       }
  72, 61,                                    //  8868:       case 0x48: {
  OPC_SwitchField, 0, 2,                     //  8870:        switch Inst[1:0] {
  0, 27,                                     //  8873:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8875:         switch Inst[22] {
  0, 10,                                     //  8878:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8880:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8882:          check Inst[20] == 0x0
  OPC_Decode, 157, 15, 114,                  //  8886:          decode to BFMAX_VG4_4ZZ_H using decoder 114
                                             //  8886:         }
  1, 0,                                      //  8890:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8892:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8894:          check Inst[20] == 0x0
  OPC_Decode, 134, 27, 114,                  //  8898:          decode to FMAX_VG4_4ZZ_H using decoder 114
                                             //  8898:         }
                                             //  8898:         } // switch Inst[22]
                                             //  8898:        }
  1, 0,                                      //  8902:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8904:         switch Inst[22] {
  0, 10,                                     //  8907:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8909:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8911:          check Inst[20] == 0x0
  OPC_Decode, 167, 15, 114,                  //  8915:          decode to BFMIN_VG4_4ZZ_H using decoder 114
                                             //  8915:         }
  1, 0,                                      //  8919:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8921:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8923:          check Inst[20] == 0x0
  OPC_Decode, 226, 27, 114,                  //  8927:          decode to FMIN_VG4_4ZZ_H using decoder 114
                                             //  8927:         }
                                             //  8927:         } // switch Inst[22]
                                             //  8927:        }
                                             //  8927:        } // switch Inst[1:0]
                                             //  8927:       }
  73, 61,                                    //  8931:       case 0x49: {
  OPC_SwitchField, 0, 2,                     //  8933:        switch Inst[1:0] {
  0, 27,                                     //  8936:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  8938:         switch Inst[22] {
  0, 10,                                     //  8941:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8943:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8945:          check Inst[20] == 0x0
  OPC_Decode, 152, 15, 114,                  //  8949:          decode to BFMAXNM_VG4_4ZZ_H using decoder 114
                                             //  8949:         }
  1, 0,                                      //  8953:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8955:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8957:          check Inst[20] == 0x0
  OPC_Decode, 218, 26, 114,                  //  8961:          decode to FMAXNM_VG4_4ZZ_H using decoder 114
                                             //  8961:         }
                                             //  8961:         } // switch Inst[22]
                                             //  8961:        }
  1, 0,                                      //  8965:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  8967:         switch Inst[22] {
  0, 10,                                     //  8970:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  8972:          check predicate 27
  OPC_CheckField, 20, 1, 0,                  //  8974:          check Inst[20] == 0x0
  OPC_Decode, 162, 15, 114,                  //  8978:          decode to BFMINNM_VG4_4ZZ_H using decoder 114
                                             //  8978:         }
  1, 0,                                      //  8982:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  8984:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  8986:          check Inst[20] == 0x0
  OPC_Decode, 182, 27, 114,                  //  8990:          decode to FMINNM_VG4_4ZZ_H using decoder 114
                                             //  8990:         }
                                             //  8990:         } // switch Inst[22]
                                             //  8990:        }
                                             //  8990:        } // switch Inst[1:0]
                                             //  8990:       }
  76, 35,                                    //  8994:       case 0x4c: {
  OPC_SwitchField, 22, 1,                    //  8996:        switch Inst[22] {
  0, 14,                                     //  8999:        case 0x0: {
  OPC_CheckPredicate, 28,                    //  9001:         check predicate 28
  OPC_CheckField, 20, 1, 0,                  //  9003:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9007:         check Inst[1:0] == 0x0
  OPC_Decode, 248, 15, 114,                  //  9011:         decode to BFSCALE_4ZZ using decoder 114
                                             //  9011:        }
  1, 0,                                      //  9015:        case 0x1: {
  OPC_CheckPredicate, 29,                    //  9017:         check predicate 29
  OPC_CheckField, 20, 1, 0,                  //  9019:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9023:         check Inst[1:0] == 0x0
  OPC_Decode, 139, 32, 114,                  //  9027:         decode to FSCALE_4ZZ_H using decoder 114
                                             //  9027:        }
                                             //  9027:        } // switch Inst[22]
                                             //  9027:       }
  81, 61,                                    //  9031:       case 0x51: {
  OPC_SwitchField, 0, 2,                     //  9033:        switch Inst[1:0] {
  0, 27,                                     //  9036:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9038:         switch Inst[22] {
  0, 10,                                     //  9041:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9043:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9045:          check Inst[20] == 0x0
  OPC_Decode, 139, 55, 114,                  //  9049:          decode to SRSHL_VG4_4ZZ_B using decoder 114
                                             //  9049:         }
  1, 0,                                      //  9053:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9055:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9057:          check Inst[20] == 0x0
  OPC_Decode, 141, 55, 114,                  //  9061:          decode to SRSHL_VG4_4ZZ_H using decoder 114
                                             //  9061:         }
                                             //  9061:         } // switch Inst[22]
                                             //  9061:        }
  1, 0,                                      //  9065:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9067:         switch Inst[22] {
  0, 10,                                     //  9070:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9072:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9074:          check Inst[20] == 0x0
  OPC_Decode, 227, 67, 114,                  //  9078:          decode to URSHL_VG4_4ZZ_B using decoder 114
                                             //  9078:         }
  1, 0,                                      //  9082:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9084:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9086:          check Inst[20] == 0x0
  OPC_Decode, 229, 67, 114,                  //  9090:          decode to URSHL_VG4_4ZZ_H using decoder 114
                                             //  9090:         }
                                             //  9090:         } // switch Inst[22]
                                             //  9090:        }
                                             //  9090:        } // switch Inst[1:0]
                                             //  9090:       }
  88, 35,                                    //  9094:       case 0x58: {
  OPC_SwitchField, 22, 1,                    //  9096:        switch Inst[22] {
  0, 14,                                     //  9099:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  9101:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9103:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9107:         check Inst[1:0] == 0x0
  OPC_Decode, 209, 13, 114,                  //  9111:         decode to ADD_VG4_4ZZ_B using decoder 114
                                             //  9111:        }
  1, 0,                                      //  9115:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  9117:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9119:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9123:         check Inst[1:0] == 0x0
  OPC_Decode, 211, 13, 114,                  //  9127:         decode to ADD_VG4_4ZZ_H using decoder 114
                                             //  9127:        }
                                             //  9127:        } // switch Inst[22]
                                             //  9127:       }
  96, 35,                                    //  9131:       case 0x60: {
  OPC_SwitchField, 22, 1,                    //  9133:        switch Inst[22] {
  0, 14,                                     //  9136:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  9138:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9140:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9144:         check Inst[1:0] == 0x0
  OPC_Decode, 133, 52, 114,                  //  9148:         decode to SQDMULH_VG4_4ZZ_B using decoder 114
                                             //  9148:        }
  1, 0,                                      //  9152:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  9154:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  //  9156:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9160:         check Inst[1:0] == 0x0
  OPC_Decode, 135, 52, 114,                  //  9164:         decode to SQDMULH_VG4_4ZZ_H using decoder 114
                                             //  9164:        }
                                             //  9164:        } // switch Inst[22]
                                             //  9164:       }
  128, 1, 61,                                //  9168:       case 0x80: {
  OPC_SwitchField, 0, 1,                     //  9171:        switch Inst[0] {
  0, 27,                                     //  9174:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9176:         switch Inst[22] {
  0, 10,                                     //  9179:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9181:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9183:          check Inst[16] == 0x0
  OPC_Decode, 247, 48, 115,                  //  9187:          decode to SMAX_VG2_2Z2Z_B using decoder 115
                                             //  9187:         }
  1, 0,                                      //  9191:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9193:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9195:          check Inst[16] == 0x0
  OPC_Decode, 249, 48, 115,                  //  9199:          decode to SMAX_VG2_2Z2Z_H using decoder 115
                                             //  9199:         }
                                             //  9199:         } // switch Inst[22]
                                             //  9199:        }
  1, 0,                                      //  9203:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9205:         switch Inst[22] {
  0, 10,                                     //  9208:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9210:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9212:          check Inst[16] == 0x0
  OPC_Decode, 129, 64, 115,                  //  9216:          decode to UMAX_VG2_2Z2Z_B using decoder 115
                                             //  9216:         }
  1, 0,                                      //  9220:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9222:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9224:          check Inst[16] == 0x0
  OPC_Decode, 131, 64, 115,                  //  9228:          decode to UMAX_VG2_2Z2Z_H using decoder 115
                                             //  9228:         }
                                             //  9228:         } // switch Inst[22]
                                             //  9228:        }
                                             //  9228:        } // switch Inst[0]
                                             //  9228:       }
  129, 1, 61,                                //  9232:       case 0x81: {
  OPC_SwitchField, 0, 1,                     //  9235:        switch Inst[0] {
  0, 27,                                     //  9238:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9240:         switch Inst[22] {
  0, 10,                                     //  9243:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9245:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9247:          check Inst[16] == 0x0
  OPC_Decode, 177, 49, 115,                  //  9251:          decode to SMIN_VG2_2Z2Z_B using decoder 115
                                             //  9251:         }
  1, 0,                                      //  9255:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9257:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9259:          check Inst[16] == 0x0
  OPC_Decode, 179, 49, 115,                  //  9263:          decode to SMIN_VG2_2Z2Z_H using decoder 115
                                             //  9263:         }
                                             //  9263:         } // switch Inst[22]
                                             //  9263:        }
  1, 0,                                      //  9267:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9269:         switch Inst[22] {
  0, 10,                                     //  9272:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9274:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9276:          check Inst[16] == 0x0
  OPC_Decode, 186, 64, 115,                  //  9280:          decode to UMIN_VG2_2Z2Z_B using decoder 115
                                             //  9280:         }
  1, 0,                                      //  9284:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9286:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9288:          check Inst[16] == 0x0
  OPC_Decode, 188, 64, 115,                  //  9292:          decode to UMIN_VG2_2Z2Z_H using decoder 115
                                             //  9292:         }
                                             //  9292:         } // switch Inst[22]
                                             //  9292:        }
                                             //  9292:        } // switch Inst[0]
                                             //  9292:       }
  136, 1, 61,                                //  9296:       case 0x88: {
  OPC_SwitchField, 0, 1,                     //  9299:        switch Inst[0] {
  0, 27,                                     //  9302:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9304:         switch Inst[22] {
  0, 10,                                     //  9307:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9309:          check predicate 27
  OPC_CheckField, 16, 1, 0,                  //  9311:          check Inst[16] == 0x0
  OPC_Decode, 154, 15, 115,                  //  9315:          decode to BFMAX_VG2_2Z2Z_H using decoder 115
                                             //  9315:         }
  1, 0,                                      //  9319:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9321:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9323:          check Inst[16] == 0x0
  OPC_Decode, 253, 26, 115,                  //  9327:          decode to FMAX_VG2_2Z2Z_H using decoder 115
                                             //  9327:         }
                                             //  9327:         } // switch Inst[22]
                                             //  9327:        }
  1, 0,                                      //  9331:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9333:         switch Inst[22] {
  0, 10,                                     //  9336:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9338:          check predicate 27
  OPC_CheckField, 16, 1, 0,                  //  9340:          check Inst[16] == 0x0
  OPC_Decode, 164, 15, 115,                  //  9344:          decode to BFMIN_VG2_2Z2Z_H using decoder 115
                                             //  9344:         }
  1, 0,                                      //  9348:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9350:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9352:          check Inst[16] == 0x0
  OPC_Decode, 217, 27, 115,                  //  9356:          decode to FMIN_VG2_2Z2Z_H using decoder 115
                                             //  9356:         }
                                             //  9356:         } // switch Inst[22]
                                             //  9356:        }
                                             //  9356:        } // switch Inst[0]
                                             //  9356:       }
  137, 1, 61,                                //  9360:       case 0x89: {
  OPC_SwitchField, 0, 1,                     //  9363:        switch Inst[0] {
  0, 27,                                     //  9366:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9368:         switch Inst[22] {
  0, 10,                                     //  9371:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9373:          check predicate 27
  OPC_CheckField, 16, 1, 0,                  //  9375:          check Inst[16] == 0x0
  OPC_Decode, 149, 15, 115,                  //  9379:          decode to BFMAXNM_VG2_2Z2Z_H using decoder 115
                                             //  9379:         }
  1, 0,                                      //  9383:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9385:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9387:          check Inst[16] == 0x0
  OPC_Decode, 209, 26, 115,                  //  9391:          decode to FMAXNM_VG2_2Z2Z_H using decoder 115
                                             //  9391:         }
                                             //  9391:         } // switch Inst[22]
                                             //  9391:        }
  1, 0,                                      //  9395:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9397:         switch Inst[22] {
  0, 10,                                     //  9400:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9402:          check predicate 27
  OPC_CheckField, 16, 1, 0,                  //  9404:          check Inst[16] == 0x0
  OPC_Decode, 159, 15, 115,                  //  9408:          decode to BFMINNM_VG2_2Z2Z_H using decoder 115
                                             //  9408:         }
  1, 0,                                      //  9412:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9414:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9416:          check Inst[16] == 0x0
  OPC_Decode, 173, 27, 115,                  //  9420:          decode to FMINNM_VG2_2Z2Z_H using decoder 115
                                             //  9420:         }
                                             //  9420:         } // switch Inst[22]
                                             //  9420:        }
                                             //  9420:        } // switch Inst[0]
                                             //  9420:       }
  138, 1, 35,                                //  9424:       case 0x8a: {
  OPC_SwitchField, 0, 1,                     //  9427:        switch Inst[0] {
  0, 14,                                     //  9430:        case 0x0: {
  OPC_CheckPredicate, 30,                    //  9432:         check predicate 30
  OPC_CheckField, 22, 1, 1,                  //  9434:         check Inst[22] == 0x1
  OPC_CheckField, 16, 1, 0,                  //  9438:         check Inst[16] == 0x0
  OPC_Decode, 143, 22, 115,                  //  9442:         decode to FAMAX_2Z2Z_H using decoder 115
                                             //  9442:        }
  1, 0,                                      //  9446:        case 0x1: {
  OPC_CheckPredicate, 30,                    //  9448:         check predicate 30
  OPC_CheckField, 22, 1, 1,                  //  9450:         check Inst[22] == 0x1
  OPC_CheckField, 16, 1, 0,                  //  9454:         check Inst[16] == 0x0
  OPC_Decode, 157, 22, 115,                  //  9458:         decode to FAMIN_2Z2Z_H using decoder 115
                                             //  9458:        }
                                             //  9458:        } // switch Inst[0]
                                             //  9458:       }
  140, 1, 35,                                //  9462:       case 0x8c: {
  OPC_SwitchField, 22, 1,                    //  9465:        switch Inst[22] {
  0, 14,                                     //  9468:        case 0x0: {
  OPC_CheckPredicate, 28,                    //  9470:         check predicate 28
  OPC_CheckField, 16, 1, 0,                  //  9472:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  9476:         check Inst[0] == 0x0
  OPC_Decode, 245, 15, 115,                  //  9480:         decode to BFSCALE_2Z2Z using decoder 115
                                             //  9480:        }
  1, 0,                                      //  9484:        case 0x1: {
  OPC_CheckPredicate, 29,                    //  9486:         check predicate 29
  OPC_CheckField, 16, 1, 0,                  //  9488:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  9492:         check Inst[0] == 0x0
  OPC_Decode, 130, 32, 115,                  //  9496:         decode to FSCALE_2Z2Z_H using decoder 115
                                             //  9496:        }
                                             //  9496:        } // switch Inst[22]
                                             //  9496:       }
  145, 1, 61,                                //  9500:       case 0x91: {
  OPC_SwitchField, 0, 1,                     //  9503:        switch Inst[0] {
  0, 27,                                     //  9506:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9508:         switch Inst[22] {
  0, 10,                                     //  9511:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9513:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9515:          check Inst[16] == 0x0
  OPC_Decode, 255, 54, 115,                  //  9519:          decode to SRSHL_VG2_2Z2Z_B using decoder 115
                                             //  9519:         }
  1, 0,                                      //  9523:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9525:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9527:          check Inst[16] == 0x0
  OPC_Decode, 129, 55, 115,                  //  9531:          decode to SRSHL_VG2_2Z2Z_H using decoder 115
                                             //  9531:         }
                                             //  9531:         } // switch Inst[22]
                                             //  9531:        }
  1, 0,                                      //  9535:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9537:         switch Inst[22] {
  0, 10,                                     //  9540:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9542:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9544:          check Inst[16] == 0x0
  OPC_Decode, 215, 67, 115,                  //  9548:          decode to URSHL_VG2_2Z2Z_B using decoder 115
                                             //  9548:         }
  1, 0,                                      //  9552:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9554:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9556:          check Inst[16] == 0x0
  OPC_Decode, 217, 67, 115,                  //  9560:          decode to URSHL_VG2_2Z2Z_H using decoder 115
                                             //  9560:         }
                                             //  9560:         } // switch Inst[22]
                                             //  9560:        }
                                             //  9560:        } // switch Inst[0]
                                             //  9560:       }
  160, 1, 35,                                //  9564:       case 0xa0: {
  OPC_SwitchField, 22, 1,                    //  9567:        switch Inst[22] {
  0, 14,                                     //  9570:        case 0x0: {
  OPC_CheckPredicate, 7,                     //  9572:         check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9574:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  9578:         check Inst[0] == 0x0
  OPC_Decode, 249, 51, 115,                  //  9582:         decode to SQDMULH_VG2_2Z2Z_B using decoder 115
                                             //  9582:        }
  1, 0,                                      //  9586:        case 0x1: {
  OPC_CheckPredicate, 7,                     //  9588:         check predicate 7
  OPC_CheckField, 16, 1, 0,                  //  9590:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   //  9594:         check Inst[0] == 0x0
  OPC_Decode, 251, 51, 115,                  //  9598:         decode to SQDMULH_VG2_2Z2Z_H using decoder 115
                                             //  9598:        }
                                             //  9598:        } // switch Inst[22]
                                             //  9598:       }
  192, 1, 61,                                //  9602:       case 0xc0: {
  OPC_SwitchField, 0, 2,                     //  9605:        switch Inst[1:0] {
  0, 27,                                     //  9608:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9610:         switch Inst[22] {
  0, 10,                                     //  9613:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9615:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9617:          check Inst[17:16] == 0x0
  OPC_Decode, 255, 48, 116,                  //  9621:          decode to SMAX_VG4_4Z4Z_B using decoder 116
                                             //  9621:         }
  1, 0,                                      //  9625:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9627:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9629:          check Inst[17:16] == 0x0
  OPC_Decode, 129, 49, 116,                  //  9633:          decode to SMAX_VG4_4Z4Z_H using decoder 116
                                             //  9633:         }
                                             //  9633:         } // switch Inst[22]
                                             //  9633:        }
  1, 0,                                      //  9637:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9639:         switch Inst[22] {
  0, 10,                                     //  9642:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9644:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9646:          check Inst[17:16] == 0x0
  OPC_Decode, 137, 64, 116,                  //  9650:          decode to UMAX_VG4_4Z4Z_B using decoder 116
                                             //  9650:         }
  1, 0,                                      //  9654:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9656:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9658:          check Inst[17:16] == 0x0
  OPC_Decode, 139, 64, 116,                  //  9662:          decode to UMAX_VG4_4Z4Z_H using decoder 116
                                             //  9662:         }
                                             //  9662:         } // switch Inst[22]
                                             //  9662:        }
                                             //  9662:        } // switch Inst[1:0]
                                             //  9662:       }
  193, 1, 61,                                //  9666:       case 0xc1: {
  OPC_SwitchField, 0, 2,                     //  9669:        switch Inst[1:0] {
  0, 27,                                     //  9672:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9674:         switch Inst[22] {
  0, 10,                                     //  9677:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9679:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9681:          check Inst[17:16] == 0x0
  OPC_Decode, 185, 49, 116,                  //  9685:          decode to SMIN_VG4_4Z4Z_B using decoder 116
                                             //  9685:         }
  1, 0,                                      //  9689:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9691:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9693:          check Inst[17:16] == 0x0
  OPC_Decode, 187, 49, 116,                  //  9697:          decode to SMIN_VG4_4Z4Z_H using decoder 116
                                             //  9697:         }
                                             //  9697:         } // switch Inst[22]
                                             //  9697:        }
  1, 0,                                      //  9701:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9703:         switch Inst[22] {
  0, 10,                                     //  9706:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9708:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9710:          check Inst[17:16] == 0x0
  OPC_Decode, 194, 64, 116,                  //  9714:          decode to UMIN_VG4_4Z4Z_B using decoder 116
                                             //  9714:         }
  1, 0,                                      //  9718:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9720:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9722:          check Inst[17:16] == 0x0
  OPC_Decode, 196, 64, 116,                  //  9726:          decode to UMIN_VG4_4Z4Z_H using decoder 116
                                             //  9726:         }
                                             //  9726:         } // switch Inst[22]
                                             //  9726:        }
                                             //  9726:        } // switch Inst[1:0]
                                             //  9726:       }
  200, 1, 61,                                //  9730:       case 0xc8: {
  OPC_SwitchField, 0, 2,                     //  9733:        switch Inst[1:0] {
  0, 27,                                     //  9736:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9738:         switch Inst[22] {
  0, 10,                                     //  9741:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9743:          check predicate 27
  OPC_CheckField, 16, 2, 0,                  //  9745:          check Inst[17:16] == 0x0
  OPC_Decode, 156, 15, 116,                  //  9749:          decode to BFMAX_VG4_4Z2Z_H using decoder 116
                                             //  9749:         }
  1, 0,                                      //  9753:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9755:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9757:          check Inst[17:16] == 0x0
  OPC_Decode, 131, 27, 116,                  //  9761:          decode to FMAX_VG4_4Z4Z_H using decoder 116
                                             //  9761:         }
                                             //  9761:         } // switch Inst[22]
                                             //  9761:        }
  1, 0,                                      //  9765:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9767:         switch Inst[22] {
  0, 10,                                     //  9770:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9772:          check predicate 27
  OPC_CheckField, 16, 2, 0,                  //  9774:          check Inst[17:16] == 0x0
  OPC_Decode, 166, 15, 116,                  //  9778:          decode to BFMIN_VG4_4Z2Z_H using decoder 116
                                             //  9778:         }
  1, 0,                                      //  9782:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9784:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9786:          check Inst[17:16] == 0x0
  OPC_Decode, 223, 27, 116,                  //  9790:          decode to FMIN_VG4_4Z4Z_H using decoder 116
                                             //  9790:         }
                                             //  9790:         } // switch Inst[22]
                                             //  9790:        }
                                             //  9790:        } // switch Inst[1:0]
                                             //  9790:       }
  201, 1, 61,                                //  9794:       case 0xc9: {
  OPC_SwitchField, 0, 2,                     //  9797:        switch Inst[1:0] {
  0, 27,                                     //  9800:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9802:         switch Inst[22] {
  0, 10,                                     //  9805:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9807:          check predicate 27
  OPC_CheckField, 16, 2, 0,                  //  9809:          check Inst[17:16] == 0x0
  OPC_Decode, 151, 15, 116,                  //  9813:          decode to BFMAXNM_VG4_4Z2Z_H using decoder 116
                                             //  9813:         }
  1, 0,                                      //  9817:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9819:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9821:          check Inst[17:16] == 0x0
  OPC_Decode, 215, 26, 116,                  //  9825:          decode to FMAXNM_VG4_4Z4Z_H using decoder 116
                                             //  9825:         }
                                             //  9825:         } // switch Inst[22]
                                             //  9825:        }
  1, 0,                                      //  9829:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9831:         switch Inst[22] {
  0, 10,                                     //  9834:         case 0x0: {
  OPC_CheckPredicate, 27,                    //  9836:          check predicate 27
  OPC_CheckField, 16, 2, 0,                  //  9838:          check Inst[17:16] == 0x0
  OPC_Decode, 161, 15, 116,                  //  9842:          decode to BFMINNM_VG4_4Z2Z_H using decoder 116
                                             //  9842:         }
  1, 0,                                      //  9846:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9848:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9850:          check Inst[17:16] == 0x0
  OPC_Decode, 179, 27, 116,                  //  9854:          decode to FMINNM_VG4_4Z4Z_H using decoder 116
                                             //  9854:         }
                                             //  9854:         } // switch Inst[22]
                                             //  9854:        }
                                             //  9854:        } // switch Inst[1:0]
                                             //  9854:       }
  202, 1, 35,                                //  9858:       case 0xca: {
  OPC_SwitchField, 0, 2,                     //  9861:        switch Inst[1:0] {
  0, 14,                                     //  9864:        case 0x0: {
  OPC_CheckPredicate, 30,                    //  9866:         check predicate 30
  OPC_CheckField, 22, 1, 1,                  //  9868:         check Inst[22] == 0x1
  OPC_CheckField, 16, 2, 0,                  //  9872:         check Inst[17:16] == 0x0
  OPC_Decode, 146, 22, 116,                  //  9876:         decode to FAMAX_4Z4Z_H using decoder 116
                                             //  9876:        }
  1, 0,                                      //  9880:        case 0x1: {
  OPC_CheckPredicate, 30,                    //  9882:         check predicate 30
  OPC_CheckField, 22, 1, 1,                  //  9884:         check Inst[22] == 0x1
  OPC_CheckField, 16, 2, 0,                  //  9888:         check Inst[17:16] == 0x0
  OPC_Decode, 160, 22, 116,                  //  9892:         decode to FAMIN_4Z4Z_H using decoder 116
                                             //  9892:        }
                                             //  9892:        } // switch Inst[1:0]
                                             //  9892:       }
  204, 1, 35,                                //  9896:       case 0xcc: {
  OPC_SwitchField, 22, 1,                    //  9899:        switch Inst[22] {
  0, 14,                                     //  9902:        case 0x0: {
  OPC_CheckPredicate, 28,                    //  9904:         check predicate 28
  OPC_CheckField, 16, 2, 0,                  //  9906:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9910:         check Inst[1:0] == 0x0
  OPC_Decode, 247, 15, 116,                  //  9914:         decode to BFSCALE_4Z4Z using decoder 116
                                             //  9914:        }
  1, 0,                                      //  9918:        case 0x1: {
  OPC_CheckPredicate, 29,                    //  9920:         check predicate 29
  OPC_CheckField, 16, 2, 0,                  //  9922:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   //  9926:         check Inst[1:0] == 0x0
  OPC_Decode, 136, 32, 116,                  //  9930:         decode to FSCALE_4Z4Z_H using decoder 116
                                             //  9930:        }
                                             //  9930:        } // switch Inst[22]
                                             //  9930:       }
  209, 1, 61,                                //  9934:       case 0xd1: {
  OPC_SwitchField, 0, 2,                     //  9937:        switch Inst[1:0] {
  0, 27,                                     //  9940:        case 0x0: {
  OPC_SwitchField, 22, 1,                    //  9942:         switch Inst[22] {
  0, 10,                                     //  9945:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9947:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9949:          check Inst[17:16] == 0x0
  OPC_Decode, 135, 55, 116,                  //  9953:          decode to SRSHL_VG4_4Z4Z_B using decoder 116
                                             //  9953:         }
  1, 0,                                      //  9957:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9959:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9961:          check Inst[17:16] == 0x0
  OPC_Decode, 137, 55, 116,                  //  9965:          decode to SRSHL_VG4_4Z4Z_H using decoder 116
                                             //  9965:         }
                                             //  9965:         } // switch Inst[22]
                                             //  9965:        }
  1, 0,                                      //  9969:        case 0x1: {
  OPC_SwitchField, 22, 1,                    //  9971:         switch Inst[22] {
  0, 10,                                     //  9974:         case 0x0: {
  OPC_CheckPredicate, 7,                     //  9976:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9978:          check Inst[17:16] == 0x0
  OPC_Decode, 223, 67, 116,                  //  9982:          decode to URSHL_VG4_4Z4Z_B using decoder 116
                                             //  9982:         }
  1, 0,                                      //  9986:         case 0x1: {
  OPC_CheckPredicate, 7,                     //  9988:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  //  9990:          check Inst[17:16] == 0x0
  OPC_Decode, 225, 67, 116,                  //  9994:          decode to URSHL_VG4_4Z4Z_H using decoder 116
                                             //  9994:         }
                                             //  9994:         } // switch Inst[22]
                                             //  9994:        }
                                             //  9994:        } // switch Inst[1:0]
                                             //  9994:       }
  224, 1, 0,                                 //  9998:       case 0xe0: {
  OPC_SwitchField, 22, 1,                    // 10001:        switch Inst[22] {
  0, 14,                                     // 10004:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 10006:         check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 10008:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10012:         check Inst[1:0] == 0x0
  OPC_Decode, 129, 52, 116,                  // 10016:         decode to SQDMULH_VG4_4Z4Z_B using decoder 116
                                             // 10016:        }
  1, 0,                                      // 10020:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 10022:         check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 10024:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10028:         check Inst[1:0] == 0x0
  OPC_Decode, 131, 52, 116,                  // 10032:         decode to SQDMULH_VG4_4Z4Z_H using decoder 116
                                             // 10032:        }
                                             // 10032:        } // switch Inst[22]
                                             // 10032:       }
                                             // 10032:       } // switch Inst[12:5]
                                             // 10032:      }
  2, 185, 2,                                 // 10036:      case 0x2: {
  OPC_SwitchField, 10, 3,                    // 10039:       switch Inst[12:10] {
  0, 27,                                     // 10042:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10044:        switch Inst[22] {
  0, 10,                                     // 10047:        case 0x0: {
  OPC_CheckPredicate, 27,                    // 10049:         check predicate 27
  OPC_CheckField, 0, 1, 0,                   // 10051:         check Inst[0] == 0x0
  OPC_Decode, 253, 14, 117,                  // 10055:         decode to BFCLAMP_VG2_2ZZZ_H using decoder 117
                                             // 10055:        }
  1, 0,                                      // 10059:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 10061:         check predicate 7
  OPC_CheckField, 0, 1, 0,                   // 10063:         check Inst[0] == 0x0
  OPC_Decode, 185, 22, 117,                  // 10067:         decode to FCLAMP_VG2_2Z2Z_H using decoder 117
                                             // 10067:        }
                                             // 10067:        } // switch Inst[22]
                                             // 10067:       }
  1, 45,                                     // 10071:       case 0x1: {
  OPC_SwitchField, 0, 1,                     // 10073:        switch Inst[0] {
  0, 19,                                     // 10076:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10078:         switch Inst[22] {
  0, 6,                                      // 10081:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10083:          check predicate 7
  OPC_Decode, 237, 46, 117,                  // 10085:          decode to SCLAMP_VG2_2Z2Z_B using decoder 117
                                             // 10085:         }
  1, 0,                                      // 10089:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10091:          check predicate 7
  OPC_Decode, 239, 46, 117,                  // 10093:          decode to SCLAMP_VG2_2Z2Z_H using decoder 117
                                             // 10093:         }
                                             // 10093:         } // switch Inst[22]
                                             // 10093:        }
  1, 0,                                      // 10097:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 10099:         switch Inst[22] {
  0, 6,                                      // 10102:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10104:          check predicate 7
  OPC_Decode, 230, 62, 117,                  // 10106:          decode to UCLAMP_VG2_2Z2Z_B using decoder 117
                                             // 10106:         }
  1, 0,                                      // 10110:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10112:          check predicate 7
  OPC_Decode, 232, 62, 117,                  // 10114:          decode to UCLAMP_VG2_2Z2Z_H using decoder 117
                                             // 10114:         }
                                             // 10114:         } // switch Inst[22]
                                             // 10114:        }
                                             // 10114:        } // switch Inst[0]
                                             // 10114:       }
  2, 27,                                     // 10118:       case 0x2: {
  OPC_SwitchField, 22, 1,                    // 10120:        switch Inst[22] {
  0, 10,                                     // 10123:        case 0x0: {
  OPC_CheckPredicate, 27,                    // 10125:         check predicate 27
  OPC_CheckField, 0, 2, 0,                   // 10127:         check Inst[1:0] == 0x0
  OPC_Decode, 254, 14, 118,                  // 10131:         decode to BFCLAMP_VG4_4ZZZ_H using decoder 118
                                             // 10131:        }
  1, 0,                                      // 10135:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 10137:         check predicate 7
  OPC_CheckField, 0, 2, 0,                   // 10139:         check Inst[1:0] == 0x0
  OPC_Decode, 188, 22, 118,                  // 10143:         decode to FCLAMP_VG4_4Z4Z_H using decoder 118
                                             // 10143:        }
                                             // 10143:        } // switch Inst[22]
                                             // 10143:       }
  3, 45,                                     // 10147:       case 0x3: {
  OPC_SwitchField, 0, 2,                     // 10149:        switch Inst[1:0] {
  0, 19,                                     // 10152:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10154:         switch Inst[22] {
  0, 6,                                      // 10157:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10159:          check predicate 7
  OPC_Decode, 241, 46, 118,                  // 10161:          decode to SCLAMP_VG4_4Z4Z_B using decoder 118
                                             // 10161:         }
  1, 0,                                      // 10165:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10167:          check predicate 7
  OPC_Decode, 243, 46, 118,                  // 10169:          decode to SCLAMP_VG4_4Z4Z_H using decoder 118
                                             // 10169:         }
                                             // 10169:         } // switch Inst[22]
                                             // 10169:        }
  1, 0,                                      // 10173:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 10175:         switch Inst[22] {
  0, 6,                                      // 10178:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10180:          check predicate 7
  OPC_Decode, 234, 62, 118,                  // 10182:          decode to UCLAMP_VG4_4Z4Z_B using decoder 118
                                             // 10182:         }
  1, 0,                                      // 10186:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10188:          check predicate 7
  OPC_Decode, 236, 62, 118,                  // 10190:          decode to UCLAMP_VG4_4Z4Z_H using decoder 118
                                             // 10190:         }
                                             // 10190:         } // switch Inst[22]
                                             // 10190:        }
                                             // 10190:        } // switch Inst[1:0]
                                             // 10190:       }
  4, 45,                                     // 10194:       case 0x4: {
  OPC_SwitchField, 0, 1,                     // 10196:        switch Inst[0] {
  0, 19,                                     // 10199:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10201:         switch Inst[22] {
  0, 6,                                      // 10204:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10206:          check predicate 7
  OPC_Decode, 165, 71, 119,                  // 10208:          decode to ZIP_VG2_2ZZZ_B using decoder 119
                                             // 10208:         }
  1, 0,                                      // 10212:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10214:          check predicate 7
  OPC_Decode, 167, 71, 119,                  // 10216:          decode to ZIP_VG2_2ZZZ_H using decoder 119
                                             // 10216:         }
                                             // 10216:         } // switch Inst[22]
                                             // 10216:        }
  1, 0,                                      // 10220:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 10222:         switch Inst[22] {
  0, 6,                                      // 10225:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10227:          check predicate 7
  OPC_Decode, 207, 69, 119,                  // 10229:          decode to UZP_VG2_2ZZZ_B using decoder 119
                                             // 10229:         }
  1, 0,                                      // 10233:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10235:          check predicate 7
  OPC_Decode, 209, 69, 119,                  // 10237:          decode to UZP_VG2_2ZZZ_H using decoder 119
                                             // 10237:         }
                                             // 10237:         } // switch Inst[22]
                                             // 10237:        }
                                             // 10237:        } // switch Inst[0]
                                             // 10237:       }
  5, 27,                                     // 10241:       case 0x5: {
  OPC_SwitchField, 0, 1,                     // 10243:        switch Inst[0] {
  0, 10,                                     // 10246:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 10248:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10250:         check Inst[22] == 0x0
  OPC_Decode, 168, 71, 119,                  // 10254:         decode to ZIP_VG2_2ZZZ_Q using decoder 119
                                             // 10254:        }
  1, 0,                                      // 10258:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 10260:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10262:         check Inst[22] == 0x0
  OPC_Decode, 210, 69, 119,                  // 10266:         decode to UZP_VG2_2ZZZ_Q using decoder 119
                                             // 10266:        }
                                             // 10266:        } // switch Inst[0]
                                             // 10266:       }
  6, 39,                                     // 10270:       case 0x6: {
  OPC_SwitchField, 5, 2,                     // 10272:        switch Inst[6:5] {
  0, 10,                                     // 10275:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 10277:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10279:         check Inst[22] == 0x1
  OPC_Decode, 215, 53, 120,                  // 10283:         decode to SQRSHR_VG4_Z4ZI_B using decoder 120
                                             // 10283:        }
  1, 10,                                     // 10287:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 10289:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10291:         check Inst[22] == 0x1
  OPC_Decode, 230, 66, 120,                  // 10295:         decode to UQRSHR_VG4_Z4ZI_B using decoder 120
                                             // 10295:        }
  2, 0,                                      // 10299:        case 0x2: {
  OPC_CheckPredicate, 7,                     // 10301:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10303:         check Inst[22] == 0x1
  OPC_Decode, 212, 53, 120,                  // 10307:         decode to SQRSHRU_VG4_Z4ZI_B using decoder 120
                                             // 10307:        }
                                             // 10307:        } // switch Inst[6:5]
                                             // 10307:       }
  7, 0,                                      // 10311:       case 0x7: {
  OPC_SwitchField, 5, 2,                     // 10313:        switch Inst[6:5] {
  0, 10,                                     // 10316:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 10318:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10320:         check Inst[22] == 0x1
  OPC_Decode, 179, 53, 120,                  // 10324:         decode to SQRSHRN_VG4_Z4ZI_B using decoder 120
                                             // 10324:        }
  1, 10,                                     // 10328:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 10330:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10332:         check Inst[22] == 0x1
  OPC_Decode, 216, 66, 120,                  // 10336:         decode to UQRSHRN_VG4_Z4ZI_B using decoder 120
                                             // 10336:        }
  2, 0,                                      // 10340:        case 0x2: {
  OPC_CheckPredicate, 7,                     // 10342:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10344:         check Inst[22] == 0x1
  OPC_Decode, 198, 53, 120,                  // 10348:         decode to SQRSHRUN_VG4_Z4ZI_B using decoder 120
                                             // 10348:        }
                                             // 10348:        } // switch Inst[6:5]
                                             // 10348:       }
                                             // 10348:       } // switch Inst[12:10]
                                             // 10348:      }
  3, 0,                                      // 10352:      case 0x3: {
  OPC_SwitchField, 10, 3,                    // 10354:       switch Inst[12:10] {
  0, 194, 4,                                 // 10357:       case 0x0: {
  OPC_SwitchField, 16, 5,                    // 10360:        switch Inst[20:16] {
  0, 45,                                     // 10363:        case 0x0: {
  OPC_SwitchField, 5, 1,                     // 10365:         switch Inst[5] {
  0, 19,                                     // 10368:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10370:          switch Inst[22] {
  0, 6,                                      // 10373:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10375:           check predicate 7
  OPC_Decode, 224, 25, 121,                  // 10377:           decode to FCVT_Z2Z_StoH using decoder 121
                                             // 10377:          }
  1, 0,                                      // 10381:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10383:           check predicate 7
  OPC_Decode, 136, 15, 121,                  // 10385:           decode to BFCVT_Z2Z_StoH using decoder 121
                                             // 10385:          }
                                             // 10385:          } // switch Inst[22]
                                             // 10385:         }
  1, 0,                                      // 10389:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 10391:          switch Inst[22] {
  0, 6,                                      // 10394:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10396:           check predicate 7
  OPC_Decode, 197, 24, 121,                  // 10398:           decode to FCVTN_Z2Z_StoH using decoder 121
                                             // 10398:          }
  1, 0,                                      // 10402:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10404:           check predicate 7
  OPC_Decode, 134, 15, 121,                  // 10406:           decode to BFCVTN_Z2Z_StoH using decoder 121
                                             // 10406:          }
                                             // 10406:          } // switch Inst[22]
                                             // 10406:         }
                                             // 10406:         } // switch Inst[5]
                                             // 10406:        }
  1, 35,                                     // 10410:        case 0x1: {
  OPC_SwitchField, 5, 1,                     // 10412:         switch Inst[5] {
  0, 14,                                     // 10415:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10417:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10419:          check Inst[22] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 10423:          check Inst[0] == 0x0
  OPC_Decode, 139, 25, 122,                  // 10427:          decode to FCVTZS_2Z2Z_StoS using decoder 122
                                             // 10427:         }
  1, 0,                                      // 10431:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10433:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10435:          check Inst[22] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 10439:          check Inst[0] == 0x0
  OPC_Decode, 190, 25, 122,                  // 10443:          decode to FCVTZU_2Z2Z_StoS using decoder 122
                                             // 10443:         }
                                             // 10443:         } // switch Inst[5]
                                             // 10443:        }
  2, 35,                                     // 10447:        case 0x2: {
  OPC_SwitchField, 5, 1,                     // 10449:         switch Inst[5] {
  0, 14,                                     // 10452:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10454:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10456:          check Inst[22] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 10460:          check Inst[0] == 0x0
  OPC_Decode, 140, 47, 122,                  // 10464:          decode to SCVTF_2Z2Z_StoS using decoder 122
                                             // 10464:         }
  1, 0,                                      // 10468:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10470:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10472:          check Inst[22] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 10476:          check Inst[0] == 0x0
  OPC_Decode, 133, 63, 122,                  // 10480:          decode to UCVTF_2Z2Z_StoS using decoder 122
                                             // 10480:         }
                                             // 10480:         } // switch Inst[5]
                                             // 10480:        }
  3, 36,                                     // 10484:        case 0x3: {
  OPC_SwitchField, 5, 1,                     // 10486:         switch Inst[5] {
  0, 19,                                     // 10489:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10491:          switch Inst[22] {
  0, 6,                                      // 10494:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10496:           check predicate 7
  OPC_Decode, 174, 51, 121,                  // 10498:           decode to SQCVT_Z2Z_StoH using decoder 121
                                             // 10498:          }
  1, 0,                                      // 10502:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10504:           check predicate 7
  OPC_Decode, 171, 51, 121,                  // 10506:           decode to SQCVTU_Z2Z_StoH using decoder 121
                                             // 10506:          }
                                             // 10506:          } // switch Inst[22]
                                             // 10506:         }
  1, 0,                                      // 10510:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10512:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10514:          check Inst[22] == 0x0
  OPC_Decode, 144, 66, 121,                  // 10518:          decode to UQCVT_Z2Z_StoH using decoder 121
                                             // 10518:         }
                                             // 10518:         } // switch Inst[5]
                                             // 10518:        }
  4, 27,                                     // 10522:        case 0x4: {
  OPC_SwitchField, 22, 1,                    // 10524:         switch Inst[22] {
  0, 10,                                     // 10527:         case 0x0: {
  OPC_CheckPredicate, 29,                    // 10529:          check predicate 29
  OPC_CheckField, 5, 1, 0,                   // 10531:          check Inst[5] == 0x0
  OPC_Decode, 223, 25, 121,                  // 10535:          decode to FCVT_Z2Z_HtoB using decoder 121
                                             // 10535:         }
  1, 0,                                      // 10539:         case 0x1: {
  OPC_CheckPredicate, 29,                    // 10541:          check predicate 29
  OPC_CheckField, 5, 1, 0,                   // 10543:          check Inst[5] == 0x0
  OPC_Decode, 135, 15, 121,                  // 10547:          decode to BFCVT_Z2Z_HtoB using decoder 121
                                             // 10547:         }
                                             // 10547:         } // switch Inst[22]
                                             // 10547:        }
  5, 27,                                     // 10551:        case 0x5: {
  OPC_SwitchField, 0, 1,                     // 10553:         switch Inst[0] {
  0, 10,                                     // 10556:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10558:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10560:          check Inst[22] == 0x1
  OPC_Decode, 239, 60, 123,                  // 10564:          decode to SUNPK_VG2_2ZZ_H using decoder 123
                                             // 10564:         }
  1, 0,                                      // 10568:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10570:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10572:          check Inst[22] == 0x1
  OPC_Decode, 147, 69, 123,                  // 10576:          decode to UUNPK_VG2_2ZZ_H using decoder 123
                                             // 10576:         }
                                             // 10576:         } // switch Inst[0]
                                             // 10576:        }
  6, 45,                                     // 10580:        case 0x6: {
  OPC_SwitchField, 0, 1,                     // 10582:         switch Inst[0] {
  0, 19,                                     // 10585:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10587:          switch Inst[22] {
  0, 6,                                      // 10590:          case 0x0: {
  OPC_CheckPredicate, 29,                    // 10592:           check predicate 29
  OPC_Decode, 172, 21, 123,                  // 10594:           decode to F1CVT_2ZZ_BtoH using decoder 123
                                             // 10594:          }
  1, 0,                                      // 10598:          case 0x1: {
  OPC_CheckPredicate, 29,                    // 10600:           check predicate 29
  OPC_Decode, 241, 14, 123,                  // 10602:           decode to BF1CVT_2ZZ_BtoH using decoder 123
                                             // 10602:          }
                                             // 10602:          } // switch Inst[22]
                                             // 10602:         }
  1, 0,                                      // 10606:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 10608:          switch Inst[22] {
  0, 6,                                      // 10611:          case 0x0: {
  OPC_CheckPredicate, 29,                    // 10613:           check predicate 29
  OPC_Decode, 171, 21, 123,                  // 10615:           decode to F1CVTL_2ZZ_BtoH using decoder 123
                                             // 10615:          }
  1, 0,                                      // 10619:          case 0x1: {
  OPC_CheckPredicate, 29,                    // 10621:           check predicate 29
  OPC_Decode, 240, 14, 123,                  // 10623:           decode to BF1CVTL_2ZZ_BtoH using decoder 123
                                             // 10623:          }
                                             // 10623:          } // switch Inst[22]
                                             // 10623:         }
                                             // 10623:         } // switch Inst[0]
                                             // 10623:        }
  17, 35,                                    // 10627:        case 0x11: {
  OPC_SwitchField, 5, 2,                     // 10629:         switch Inst[6:5] {
  0, 14,                                     // 10632:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10634:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10636:          check Inst[22] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10640:          check Inst[1:0] == 0x0
  OPC_Decode, 140, 25, 124,                  // 10644:          decode to FCVTZS_4Z4Z_StoS using decoder 124
                                             // 10644:         }
  1, 0,                                      // 10648:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10650:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10652:          check Inst[22] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10656:          check Inst[1:0] == 0x0
  OPC_Decode, 191, 25, 124,                  // 10660:          decode to FCVTZU_4Z4Z_StoS using decoder 124
                                             // 10660:         }
                                             // 10660:         } // switch Inst[6:5]
                                             // 10660:        }
  18, 35,                                    // 10664:        case 0x12: {
  OPC_SwitchField, 5, 2,                     // 10666:         switch Inst[6:5] {
  0, 14,                                     // 10669:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10671:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10673:          check Inst[22] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10677:          check Inst[1:0] == 0x0
  OPC_Decode, 141, 47, 124,                  // 10681:          decode to SCVTF_4Z4Z_StoS using decoder 124
                                             // 10681:         }
  1, 0,                                      // 10685:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10687:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10689:          check Inst[22] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10693:          check Inst[1:0] == 0x0
  OPC_Decode, 134, 63, 124,                  // 10697:          decode to UCVTF_4Z4Z_StoS using decoder 124
                                             // 10697:         }
                                             // 10697:         } // switch Inst[6:5]
                                             // 10697:        }
  19, 69,                                    // 10701:        case 0x13: {
  OPC_SwitchField, 5, 2,                     // 10703:         switch Inst[6:5] {
  0, 19,                                     // 10706:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10708:          switch Inst[22] {
  0, 6,                                      // 10711:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10713:           check predicate 7
  OPC_Decode, 176, 51, 125,                  // 10715:           decode to SQCVT_Z4Z_StoB using decoder 125
                                             // 10715:          }
  1, 0,                                      // 10719:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10721:           check predicate 7
  OPC_Decode, 173, 51, 125,                  // 10723:           decode to SQCVTU_Z4Z_StoB using decoder 125
                                             // 10723:          }
                                             // 10723:          } // switch Inst[22]
                                             // 10723:         }
  1, 10,                                     // 10727:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10729:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10731:          check Inst[22] == 0x0
  OPC_Decode, 146, 66, 125,                  // 10735:          decode to UQCVT_Z4Z_StoB using decoder 125
                                             // 10735:         }
  2, 19,                                     // 10739:         case 0x2: {
  OPC_SwitchField, 22, 1,                    // 10741:          switch Inst[22] {
  0, 6,                                      // 10744:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10746:           check predicate 7
  OPC_Decode, 167, 51, 125,                  // 10748:           decode to SQCVTN_Z4Z_StoB using decoder 125
                                             // 10748:          }
  1, 0,                                      // 10752:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10754:           check predicate 7
  OPC_Decode, 170, 51, 125,                  // 10756:           decode to SQCVTUN_Z4Z_StoB using decoder 125
                                             // 10756:          }
                                             // 10756:          } // switch Inst[22]
                                             // 10756:         }
  3, 0,                                      // 10760:         case 0x3: {
  OPC_CheckPredicate, 7,                     // 10762:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10764:          check Inst[22] == 0x0
  OPC_Decode, 143, 66, 125,                  // 10768:          decode to UQCVTN_Z4Z_StoB using decoder 125
                                             // 10768:         }
                                             // 10768:         } // switch Inst[6:5]
                                             // 10768:        }
  20, 27,                                    // 10772:        case 0x14: {
  OPC_SwitchField, 5, 2,                     // 10774:         switch Inst[6:5] {
  0, 10,                                     // 10777:         case 0x0: {
  OPC_CheckPredicate, 29,                    // 10779:          check predicate 29
  OPC_CheckField, 22, 1, 0,                  // 10781:          check Inst[22] == 0x0
  OPC_Decode, 225, 25, 125,                  // 10785:          decode to FCVT_Z4Z_StoB using decoder 125
                                             // 10785:         }
  1, 0,                                      // 10789:         case 0x1: {
  OPC_CheckPredicate, 29,                    // 10791:          check predicate 29
  OPC_CheckField, 22, 1, 0,                  // 10793:          check Inst[22] == 0x0
  OPC_Decode, 198, 24, 125,                  // 10797:          decode to FCVTN_Z4Z_StoB using decoder 125
                                             // 10797:         }
                                             // 10797:         } // switch Inst[6:5]
                                             // 10797:        }
  21, 35,                                    // 10801:        case 0x15: {
  OPC_SwitchField, 0, 2,                     // 10803:         switch Inst[1:0] {
  0, 14,                                     // 10806:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10808:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10810:          check Inst[22] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 10814:          check Inst[5] == 0x0
  OPC_Decode, 242, 60, 126,                  // 10818:          decode to SUNPK_VG4_4Z2Z_H using decoder 126
                                             // 10818:         }
  1, 0,                                      // 10822:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 10824:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 10826:          check Inst[22] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 10830:          check Inst[5] == 0x0
  OPC_Decode, 150, 69, 126,                  // 10834:          decode to UUNPK_VG4_4Z2Z_H using decoder 126
                                             // 10834:         }
                                             // 10834:         } // switch Inst[1:0]
                                             // 10834:        }
  22, 61,                                    // 10838:        case 0x16: {
  OPC_SwitchField, 0, 2,                     // 10840:         switch Inst[1:0] {
  0, 27,                                     // 10843:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10845:          switch Inst[22] {
  0, 10,                                     // 10848:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10850:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 10852:           check Inst[6:5] == 0x0
  OPC_Decode, 170, 71, 124,                  // 10856:           decode to ZIP_VG4_4Z4Z_B using decoder 124
                                             // 10856:          }
  1, 0,                                      // 10860:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10862:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 10864:           check Inst[6:5] == 0x0
  OPC_Decode, 172, 71, 124,                  // 10868:           decode to ZIP_VG4_4Z4Z_H using decoder 124
                                             // 10868:          }
                                             // 10868:          } // switch Inst[22]
                                             // 10868:         }
  2, 0,                                      // 10872:         case 0x2: {
  OPC_SwitchField, 22, 1,                    // 10874:          switch Inst[22] {
  0, 10,                                     // 10877:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 10879:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 10881:           check Inst[6:5] == 0x0
  OPC_Decode, 212, 69, 124,                  // 10885:           decode to UZP_VG4_4Z4Z_B using decoder 124
                                             // 10885:          }
  1, 0,                                      // 10889:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 10891:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 10893:           check Inst[6:5] == 0x0
  OPC_Decode, 214, 69, 124,                  // 10897:           decode to UZP_VG4_4Z4Z_H using decoder 124
                                             // 10897:          }
                                             // 10897:          } // switch Inst[22]
                                             // 10897:         }
                                             // 10897:         } // switch Inst[1:0]
                                             // 10897:        }
  23, 0,                                     // 10901:        case 0x17: {
  OPC_SwitchField, 0, 2,                     // 10903:         switch Inst[1:0] {
  0, 14,                                     // 10906:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 10908:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10910:          check Inst[22] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 10914:          check Inst[6:5] == 0x0
  OPC_Decode, 173, 71, 124,                  // 10918:          decode to ZIP_VG4_4Z4Z_Q using decoder 124
                                             // 10918:         }
  2, 0,                                      // 10922:         case 0x2: {
  OPC_CheckPredicate, 7,                     // 10924:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 10926:          check Inst[22] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 10930:          check Inst[6:5] == 0x0
  OPC_Decode, 215, 69, 124,                  // 10934:          decode to UZP_VG4_4Z4Z_Q using decoder 124
                                             // 10934:         }
                                             // 10934:         } // switch Inst[1:0]
                                             // 10934:        }
                                             // 10934:        } // switch Inst[20:16]
                                             // 10934:       }
  1, 87,                                     // 10938:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 10940:        switch Inst[16] {
  0, 35,                                     // 10943:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 10945:         switch Inst[22] {
  0, 14,                                     // 10948:         case 0x0: {
  OPC_CheckPredicate, 28,                    // 10950:          check predicate 28
  OPC_CheckField, 5, 1, 0,                   // 10952:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 10956:          check Inst[0] == 0x0
  OPC_Decode, 236, 15, 127,                  // 10960:          decode to BFMUL_2Z2Z using decoder 127
                                             // 10960:         }
  1, 0,                                      // 10964:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 10966:          check predicate 31
  OPC_CheckField, 5, 1, 0,                   // 10968:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 10972:          check Inst[0] == 0x0
  OPC_Decode, 247, 29, 127,                  // 10976:          decode to FMUL_2Z2Z_H using decoder 127
                                             // 10976:         }
                                             // 10976:         } // switch Inst[22]
                                             // 10976:        }
  1, 0,                                      // 10980:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 10982:         switch Inst[22] {
  0, 19,                                     // 10985:         case 0x0: {
  OPC_CheckPredicate, 28,                    // 10987:          check predicate 28
  OPC_CheckField, 17, 1, 0,                  // 10989:          check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 10993:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 10997:          check Inst[1:0] == 0x0
  OPC_Decode, 238, 15, 128, 1,               // 11001:          decode to BFMUL_4Z4Z using decoder 128
                                             // 11001:         }
  1, 0,                                      // 11006:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 11008:          check predicate 31
  OPC_CheckField, 17, 1, 0,                  // 11010:          check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 11014:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 11018:          check Inst[1:0] == 0x0
  OPC_Decode, 253, 29, 128, 1,               // 11022:          decode to FMUL_4Z4Z_H using decoder 128
                                             // 11022:         }
                                             // 11022:         } // switch Inst[22]
                                             // 11022:        }
                                             // 11022:        } // switch Inst[16]
                                             // 11022:       }
  2, 81,                                     // 11027:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 11029:        switch Inst[16] {
  0, 37,                                     // 11032:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11034:         switch Inst[22] {
  0, 15,                                     // 11037:         case 0x0: {
  OPC_CheckPredicate, 28,                    // 11039:          check predicate 28
  OPC_CheckField, 5, 1, 0,                   // 11041:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 11045:          check Inst[0] == 0x0
  OPC_Decode, 237, 15, 129, 1,               // 11049:          decode to BFMUL_2ZZ using decoder 129
                                             // 11049:         }
  1, 0,                                      // 11054:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 11056:          check predicate 31
  OPC_CheckField, 5, 1, 0,                   // 11058:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 11062:          check Inst[0] == 0x0
  OPC_Decode, 250, 29, 129, 1,               // 11066:          decode to FMUL_2ZZ_H using decoder 129
                                             // 11066:         }
                                             // 11066:         } // switch Inst[22]
                                             // 11066:        }
  1, 0,                                      // 11071:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11073:         switch Inst[22] {
  0, 15,                                     // 11076:         case 0x0: {
  OPC_CheckPredicate, 28,                    // 11078:          check predicate 28
  OPC_CheckField, 5, 2, 0,                   // 11080:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 11084:          check Inst[1:0] == 0x0
  OPC_Decode, 239, 15, 130, 1,               // 11088:          decode to BFMUL_4ZZ using decoder 130
                                             // 11088:         }
  1, 0,                                      // 11093:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 11095:          check predicate 31
  OPC_CheckField, 5, 2, 0,                   // 11097:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 11101:          check Inst[1:0] == 0x0
  OPC_Decode, 128, 30, 130, 1,               // 11105:          decode to FMUL_4ZZ_H using decoder 130
                                             // 11105:         }
                                             // 11105:         } // switch Inst[22]
                                             // 11105:        }
                                             // 11105:        } // switch Inst[16]
                                             // 11105:       }
  5, 11,                                     // 11110:       case 0x5: {
  OPC_CheckPredicate, 23,                    // 11112:        check predicate 23
  OPC_CheckField, 0, 2, 0,                   // 11114:        check Inst[1:0] == 0x0
  OPC_Decode, 222, 41, 131, 1,               // 11118:        decode to LUTI6_4Z2Z2ZI using decoder 131
                                             // 11118:       }
  7, 0,                                      // 11123:       case 0x7: {
  OPC_CheckPredicate, 23,                    // 11125:        check predicate 23
  OPC_CheckField, 2, 2, 0,                   // 11127:        check Inst[3:2] == 0x0
  OPC_Decode, 224, 41, 132, 1,               // 11131:        decode to LUTI6_S_4Z2Z2ZI using decoder 132
                                             // 11131:       }
                                             // 11131:       } // switch Inst[12:10]
                                             // 11131:      }
                                             // 11131:      } // switch Inst[14:13]
                                             // 11131:     }
                                             // 11131:     } // switch Inst[15]
                                             // 11131:    }
                                             // 11131:    } // switch Inst[21]
                                             // 11131:   }
  3, 0,                                      // 11136:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 11138:    switch Inst[21] {
  0, 139, 7,                                 // 11141:    case 0x0: {
  OPC_SwitchField, 4, 1,                     // 11144:     switch Inst[4] {
  0, 214, 3,                                 // 11147:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 11150:      switch Inst[12] {
  0, 246, 1,                                 // 11153:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 11156:       switch Inst[20] {
  0, 35,                                     // 11159:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11161:        switch Inst[22] {
  0, 21,                                     // 11164:        case 0x0: {
  OPC_SwitchField, 2, 2,                     // 11166:         switch Inst[3:2] {
  0, 7,                                      // 11169:         case 0x0: {
  OPC_CheckPredicate, 25,                    // 11171:          check predicate 25
  OPC_Decode, 213, 49, 133, 1,               // 11173:          decode to SMLALL_MZZI_HtoD using decoder 133
                                             // 11173:         }
  2, 0,                                      // 11178:         case 0x2: {
  OPC_CheckPredicate, 25,                    // 11180:          check predicate 25
  OPC_Decode, 129, 50, 133, 1,               // 11182:          decode to SMLSLL_MZZI_HtoD using decoder 133
                                             // 11182:         }
                                             // 11182:         } // switch Inst[3:2]
                                             // 11182:        }
  1, 0,                                      // 11187:        case 0x1: {
  OPC_CheckPredicate, 9,                     // 11189:         check predicate 9
  OPC_Decode, 151, 28, 134, 1,               // 11191:         decode to FMLAL_MZZI_BtoH using decoder 134
                                             // 11191:        }
                                             // 11191:        } // switch Inst[22]
                                             // 11191:       }
  1, 0,                                      // 11196:       case 0x1: {
  OPC_SwitchField, 5, 1,                     // 11198:        switch Inst[5] {
  0, 165, 1,                                 // 11201:        case 0x0: {
  OPC_SwitchField, 11, 1,                    // 11204:         switch Inst[11] {
  0, 121,                                    // 11207:         case 0x0: {
  OPC_SwitchField, 3, 1,                     // 11209:          switch Inst[3] {
  0, 57,                                     // 11212:          case 0x0: {
  OPC_SwitchField, 15, 1,                    // 11214:           switch Inst[15] {
  0, 21,                                     // 11217:           case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11219:            switch Inst[22] {
  0, 7,                                      // 11222:            case 0x0: {
  OPC_CheckPredicate, 25,                    // 11224:             check predicate 25
  OPC_Decode, 219, 49, 135, 1,               // 11226:             decode to SMLALL_VG2_M2ZZI_HtoD using decoder 135
                                             // 11226:            }
  1, 0,                                      // 11231:            case 0x1: {
  OPC_CheckPredicate, 26,                    // 11233:             check predicate 26
  OPC_Decode, 174, 28, 136, 1,               // 11235:             decode to FMLA_VG2_M2ZZI_D using decoder 136
                                             // 11235:            }
                                             // 11235:            } // switch Inst[22]
                                             // 11235:           }
  1, 0,                                      // 11240:           case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11242:            switch Inst[22] {
  0, 11,                                     // 11245:            case 0x0: {
  OPC_CheckPredicate, 25,                    // 11247:             check predicate 25
  OPC_CheckField, 6, 1, 0,                   // 11249:             check Inst[6] == 0x0
  OPC_Decode, 225, 49, 137, 1,               // 11253:             decode to SMLALL_VG4_M4ZZI_HtoD using decoder 137
                                             // 11253:            }
  1, 0,                                      // 11258:            case 0x1: {
  OPC_CheckPredicate, 26,                    // 11260:             check predicate 26
  OPC_CheckField, 6, 1, 0,                   // 11262:             check Inst[6] == 0x0
  OPC_Decode, 183, 28, 138, 1,               // 11266:             decode to FMLA_VG4_M4ZZI_D using decoder 138
                                             // 11266:            }
                                             // 11266:            } // switch Inst[22]
                                             // 11266:           }
                                             // 11266:           } // switch Inst[15]
                                             // 11266:          }
  1, 0,                                      // 11271:          case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11273:           switch Inst[15] {
  0, 21,                                     // 11276:           case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11278:            switch Inst[22] {
  0, 7,                                      // 11281:            case 0x0: {
  OPC_CheckPredicate, 25,                    // 11283:             check predicate 25
  OPC_Decode, 135, 50, 135, 1,               // 11285:             decode to SMLSLL_VG2_M2ZZI_HtoD using decoder 135
                                             // 11285:            }
  1, 0,                                      // 11290:            case 0x1: {
  OPC_CheckPredicate, 25,                    // 11292:             check predicate 25
  OPC_Decode, 186, 47, 136, 1,               // 11294:             decode to SDOT_VG2_M2ZZI_HtoD using decoder 136
                                             // 11294:            }
                                             // 11294:            } // switch Inst[22]
                                             // 11294:           }
  1, 0,                                      // 11299:           case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11301:            switch Inst[22] {
  0, 11,                                     // 11304:            case 0x0: {
  OPC_CheckPredicate, 25,                    // 11306:             check predicate 25
  OPC_CheckField, 6, 1, 0,                   // 11308:             check Inst[6] == 0x0
  OPC_Decode, 141, 50, 137, 1,               // 11312:             decode to SMLSLL_VG4_M4ZZI_HtoD using decoder 137
                                             // 11312:            }
  1, 0,                                      // 11317:            case 0x1: {
  OPC_CheckPredicate, 25,                    // 11319:             check predicate 25
  OPC_CheckField, 6, 1, 0,                   // 11321:             check Inst[6] == 0x0
  OPC_Decode, 195, 47, 138, 1,               // 11325:             decode to SDOT_VG4_M4ZZI_HtoD using decoder 138
                                             // 11325:            }
                                             // 11325:            } // switch Inst[22]
                                             // 11325:           }
                                             // 11325:           } // switch Inst[15]
                                             // 11325:          }
                                             // 11325:          } // switch Inst[3]
                                             // 11325:         }
  1, 0,                                      // 11330:         case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11332:          switch Inst[15] {
  0, 11,                                     // 11335:          case 0x0: {
  OPC_CheckPredicate, 8,                     // 11337:           check predicate 8
  OPC_CheckField, 22, 1, 1,                  // 11339:           check Inst[22] == 0x1
  OPC_Decode, 206, 32, 139, 1,               // 11343:           decode to FVDOTB_VG4_M2ZZI_BtoS using decoder 139
                                             // 11343:          }
  1, 0,                                      // 11348:          case 0x1: {
  OPC_CheckPredicate, 25,                    // 11350:           check predicate 25
  OPC_CheckField, 22, 1, 1,                  // 11352:           check Inst[22] == 0x1
  OPC_CheckField, 6, 1, 0,                   // 11356:           check Inst[6] == 0x0
  OPC_CheckField, 3, 1, 1,                   // 11360:           check Inst[3] == 0x1
  OPC_Decode, 136, 61, 138, 1,               // 11364:           decode to SVDOT_VG4_M4ZZI_HtoD using decoder 138
                                             // 11364:          }
                                             // 11364:          } // switch Inst[15]
                                             // 11364:         }
                                             // 11364:         } // switch Inst[11]
                                             // 11364:        }
  1, 0,                                      // 11369:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11371:         switch Inst[22] {
  0, 14,                                     // 11374:         case 0x0: {
  OPC_CheckPredicate, 8,                     // 11376:          check predicate 8
  OPC_CheckField, 15, 1, 0,                  // 11378:          check Inst[15] == 0x0
  OPC_CheckField, 3, 1, 0,                   // 11382:          check Inst[3] == 0x0
  OPC_Decode, 140, 28, 97,                   // 11386:          decode to FMLALL_VG2_M2ZZI_BtoS using decoder 97
                                             // 11386:         }
  1, 0,                                      // 11390:         case 0x1: {
  OPC_CheckPredicate, 9,                     // 11392:          check predicate 9
  OPC_CheckField, 15, 1, 0,                  // 11394:          check Inst[15] == 0x0
  OPC_Decode, 255, 25, 101,                  // 11398:          decode to FDOT_VG2_M2ZZI_BtoH using decoder 101
                                             // 11398:         }
                                             // 11398:         } // switch Inst[22]
                                             // 11398:        }
                                             // 11398:        } // switch Inst[5]
                                             // 11398:       }
                                             // 11398:       } // switch Inst[20]
                                             // 11398:      }
  1, 0,                                      // 11402:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 11404:       switch Inst[20] {
  0, 49,                                     // 11407:       case 0x0: {
  OPC_SwitchField, 3, 1,                     // 11409:        switch Inst[3] {
  0, 21,                                     // 11412:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11414:         switch Inst[22] {
  0, 7,                                      // 11417:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 11419:          check predicate 7
  OPC_Decode, 152, 28, 140, 1,               // 11421:          decode to FMLAL_MZZI_HtoS using decoder 140
                                             // 11421:         }
  1, 0,                                      // 11426:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 11428:          check predicate 7
  OPC_Decode, 233, 49, 140, 1,               // 11430:          decode to SMLAL_MZZI_HtoS using decoder 140
                                             // 11430:         }
                                             // 11430:         } // switch Inst[22]
                                             // 11430:        }
  1, 0,                                      // 11435:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11437:         switch Inst[22] {
  0, 7,                                      // 11440:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 11442:          check predicate 7
  OPC_Decode, 217, 28, 140, 1,               // 11444:          decode to FMLSL_MZZI_HtoS using decoder 140
                                             // 11444:         }
  1, 0,                                      // 11449:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 11451:          check predicate 7
  OPC_Decode, 149, 50, 140, 1,               // 11453:          decode to SMLSL_MZZI_HtoS using decoder 140
                                             // 11453:         }
                                             // 11453:         } // switch Inst[22]
                                             // 11453:        }
                                             // 11453:        } // switch Inst[3]
                                             // 11453:       }
  1, 0,                                      // 11458:       case 0x1: {
  OPC_SwitchField, 5, 1,                     // 11460:        switch Inst[5] {
  0, 121,                                    // 11463:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 11465:         switch Inst[3] {
  0, 57,                                     // 11468:         case 0x0: {
  OPC_SwitchField, 15, 1,                    // 11470:          switch Inst[15] {
  0, 21,                                     // 11473:          case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11475:           switch Inst[22] {
  0, 7,                                      // 11478:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11480:            check predicate 7
  OPC_Decode, 157, 28, 141, 1,               // 11482:            decode to FMLAL_VG2_M2ZZI_HtoS using decoder 141
                                             // 11482:           }
  1, 0,                                      // 11487:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11489:            check predicate 7
  OPC_Decode, 236, 49, 141, 1,               // 11491:            decode to SMLAL_VG2_M2ZZI_S using decoder 141
                                             // 11491:           }
                                             // 11491:           } // switch Inst[22]
                                             // 11491:          }
  1, 0,                                      // 11496:          case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11498:           switch Inst[22] {
  0, 11,                                     // 11501:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11503:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 11505:            check Inst[6] == 0x0
  OPC_Decode, 164, 28, 142, 1,               // 11509:            decode to FMLAL_VG4_M4ZZI_HtoS using decoder 142
                                             // 11509:           }
  1, 0,                                      // 11514:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11516:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 11518:            check Inst[6] == 0x0
  OPC_Decode, 239, 49, 142, 1,               // 11522:            decode to SMLAL_VG4_M4ZZI_HtoS using decoder 142
                                             // 11522:           }
                                             // 11522:           } // switch Inst[22]
                                             // 11522:          }
                                             // 11522:          } // switch Inst[15]
                                             // 11522:         }
  1, 0,                                      // 11527:         case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11529:          switch Inst[15] {
  0, 21,                                     // 11532:          case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11534:           switch Inst[22] {
  0, 7,                                      // 11537:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11539:            check predicate 7
  OPC_Decode, 220, 28, 141, 1,               // 11541:            decode to FMLSL_VG2_M2ZZI_HtoS using decoder 141
                                             // 11541:           }
  1, 0,                                      // 11546:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11548:            check predicate 7
  OPC_Decode, 152, 50, 141, 1,               // 11550:            decode to SMLSL_VG2_M2ZZI_S using decoder 141
                                             // 11550:           }
                                             // 11550:           } // switch Inst[22]
                                             // 11550:          }
  1, 0,                                      // 11555:          case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11557:           switch Inst[22] {
  0, 11,                                     // 11560:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11562:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 11564:            check Inst[6] == 0x0
  OPC_Decode, 223, 28, 142, 1,               // 11568:            decode to FMLSL_VG4_M4ZZI_HtoS using decoder 142
                                             // 11568:           }
  1, 0,                                      // 11573:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11575:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 11577:            check Inst[6] == 0x0
  OPC_Decode, 155, 50, 142, 1,               // 11581:            decode to SMLSL_VG4_M4ZZI_HtoS using decoder 142
                                             // 11581:           }
                                             // 11581:           } // switch Inst[22]
                                             // 11581:          }
                                             // 11581:          } // switch Inst[15]
                                             // 11581:         }
                                             // 11581:         } // switch Inst[3]
                                             // 11581:        }
  1, 0,                                      // 11586:        case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11588:         switch Inst[15] {
  0, 10,                                     // 11591:         case 0x0: {
  OPC_CheckPredicate, 9,                     // 11593:          check predicate 9
  OPC_CheckField, 22, 1, 1,                  // 11595:          check Inst[22] == 0x1
  OPC_Decode, 208, 32, 101,                  // 11599:          decode to FVDOT_VG2_M2ZZI_BtoH using decoder 101
                                             // 11599:         }
  1, 0,                                      // 11603:         case 0x1: {
  OPC_CheckPredicate, 9,                     // 11605:          check predicate 9
  OPC_CheckField, 22, 1, 0,                  // 11607:          check Inst[22] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 11611:          check Inst[6] == 0x0
  OPC_Decode, 163, 28, 143, 1,               // 11615:          decode to FMLAL_VG4_M4ZZI_BtoH using decoder 143
                                             // 11615:         }
                                             // 11615:         } // switch Inst[15]
                                             // 11615:        }
                                             // 11615:        } // switch Inst[5]
                                             // 11615:       }
                                             // 11615:       } // switch Inst[20]
                                             // 11615:      }
                                             // 11615:      } // switch Inst[12]
                                             // 11615:     }
  1, 0,                                      // 11620:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 11622:      switch Inst[12] {
  0, 222, 1,                                 // 11625:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 11628:       switch Inst[20] {
  0, 29,                                     // 11631:       case 0x0: {
  OPC_SwitchField, 2, 2,                     // 11633:        switch Inst[3:2] {
  0, 11,                                     // 11636:        case 0x0: {
  OPC_CheckPredicate, 25,                    // 11638:         check predicate 25
  OPC_CheckField, 22, 1, 0,                  // 11640:         check Inst[22] == 0x0
  OPC_Decode, 222, 64, 133, 1,               // 11644:         decode to UMLALL_MZZI_HtoD using decoder 133
                                             // 11644:        }
  2, 0,                                      // 11649:        case 0x2: {
  OPC_CheckPredicate, 25,                    // 11651:         check predicate 25
  OPC_CheckField, 22, 1, 0,                  // 11653:         check Inst[22] == 0x0
  OPC_Decode, 138, 65, 133, 1,               // 11657:         decode to UMLSLL_MZZI_HtoD using decoder 133
                                             // 11657:        }
                                             // 11657:        } // switch Inst[3:2]
                                             // 11657:       }
  1, 0,                                      // 11662:       case 0x1: {
  OPC_SwitchField, 11, 1,                    // 11664:        switch Inst[11] {
  0, 137, 1,                                 // 11667:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 11670:         switch Inst[3] {
  0, 65,                                     // 11673:         case 0x0: {
  OPC_SwitchField, 15, 1,                    // 11675:          switch Inst[15] {
  0, 29,                                     // 11678:          case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11680:           switch Inst[22] {
  0, 11,                                     // 11683:           case 0x0: {
  OPC_CheckPredicate, 25,                    // 11685:            check predicate 25
  OPC_CheckField, 5, 1, 0,                   // 11687:            check Inst[5] == 0x0
  OPC_Decode, 228, 64, 135, 1,               // 11691:            decode to UMLALL_VG2_M2ZZI_HtoD using decoder 135
                                             // 11691:           }
  1, 0,                                      // 11696:           case 0x1: {
  OPC_CheckPredicate, 26,                    // 11698:            check predicate 26
  OPC_CheckField, 5, 1, 0,                   // 11700:            check Inst[5] == 0x0
  OPC_Decode, 232, 28, 136, 1,               // 11704:            decode to FMLS_VG2_M2ZZI_D using decoder 136
                                             // 11704:           }
                                             // 11704:           } // switch Inst[22]
                                             // 11704:          }
  1, 0,                                      // 11709:          case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11711:           switch Inst[22] {
  0, 11,                                     // 11714:           case 0x0: {
  OPC_CheckPredicate, 25,                    // 11716:            check predicate 25
  OPC_CheckField, 5, 2, 0,                   // 11718:            check Inst[6:5] == 0x0
  OPC_Decode, 234, 64, 137, 1,               // 11722:            decode to UMLALL_VG4_M4ZZI_HtoD using decoder 137
                                             // 11722:           }
  1, 0,                                      // 11727:           case 0x1: {
  OPC_CheckPredicate, 26,                    // 11729:            check predicate 26
  OPC_CheckField, 5, 2, 0,                   // 11731:            check Inst[6:5] == 0x0
  OPC_Decode, 241, 28, 138, 1,               // 11735:            decode to FMLS_VG4_M4ZZI_D using decoder 138
                                             // 11735:           }
                                             // 11735:           } // switch Inst[22]
                                             // 11735:          }
                                             // 11735:          } // switch Inst[15]
                                             // 11735:         }
  1, 0,                                      // 11740:         case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11742:          switch Inst[15] {
  0, 29,                                     // 11745:          case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11747:           switch Inst[22] {
  0, 11,                                     // 11750:           case 0x0: {
  OPC_CheckPredicate, 25,                    // 11752:            check predicate 25
  OPC_CheckField, 5, 1, 0,                   // 11754:            check Inst[5] == 0x0
  OPC_Decode, 144, 65, 135, 1,               // 11758:            decode to UMLSLL_VG2_M2ZZI_HtoD using decoder 135
                                             // 11758:           }
  1, 0,                                      // 11763:           case 0x1: {
  OPC_CheckPredicate, 25,                    // 11765:            check predicate 25
  OPC_CheckField, 5, 1, 0,                   // 11767:            check Inst[5] == 0x0
  OPC_Decode, 180, 63, 136, 1,               // 11771:            decode to UDOT_VG2_M2ZZI_HtoD using decoder 136
                                             // 11771:           }
                                             // 11771:           } // switch Inst[22]
                                             // 11771:          }
  1, 0,                                      // 11776:          case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11778:           switch Inst[22] {
  0, 11,                                     // 11781:           case 0x0: {
  OPC_CheckPredicate, 25,                    // 11783:            check predicate 25
  OPC_CheckField, 5, 2, 0,                   // 11785:            check Inst[6:5] == 0x0
  OPC_Decode, 150, 65, 137, 1,               // 11789:            decode to UMLSLL_VG4_M4ZZI_HtoD using decoder 137
                                             // 11789:           }
  1, 0,                                      // 11794:           case 0x1: {
  OPC_CheckPredicate, 25,                    // 11796:            check predicate 25
  OPC_CheckField, 5, 2, 0,                   // 11798:            check Inst[6:5] == 0x0
  OPC_Decode, 189, 63, 138, 1,               // 11802:            decode to UDOT_VG4_M4ZZI_HtoD using decoder 138
                                             // 11802:           }
                                             // 11802:           } // switch Inst[22]
                                             // 11802:          }
                                             // 11802:          } // switch Inst[15]
                                             // 11802:         }
                                             // 11802:         } // switch Inst[3]
                                             // 11802:        }
  1, 0,                                      // 11807:        case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11809:         switch Inst[15] {
  0, 15,                                     // 11812:         case 0x0: {
  OPC_CheckPredicate, 8,                     // 11814:          check predicate 8
  OPC_CheckField, 22, 1, 1,                  // 11816:          check Inst[22] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 11820:          check Inst[5] == 0x0
  OPC_Decode, 207, 32, 139, 1,               // 11824:          decode to FVDOTT_VG4_M2ZZI_BtoS using decoder 139
                                             // 11824:         }
  1, 0,                                      // 11829:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 11831:          check predicate 25
  OPC_CheckField, 22, 1, 1,                  // 11833:          check Inst[22] == 0x1
  OPC_CheckField, 5, 2, 0,                   // 11837:          check Inst[6:5] == 0x0
  OPC_CheckField, 3, 1, 1,                   // 11841:          check Inst[3] == 0x1
  OPC_Decode, 154, 69, 138, 1,               // 11845:          decode to UVDOT_VG4_M4ZZI_HtoD using decoder 138
                                             // 11845:         }
                                             // 11845:         } // switch Inst[15]
                                             // 11845:        }
                                             // 11845:        } // switch Inst[11]
                                             // 11845:       }
                                             // 11845:       } // switch Inst[20]
                                             // 11845:      }
  1, 0,                                      // 11850:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 11852:       switch Inst[20] {
  0, 49,                                     // 11855:       case 0x0: {
  OPC_SwitchField, 3, 1,                     // 11857:        switch Inst[3] {
  0, 21,                                     // 11860:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11862:         switch Inst[22] {
  0, 7,                                      // 11865:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 11867:          check predicate 7
  OPC_Decode, 177, 15, 140, 1,               // 11869:          decode to BFMLAL_MZZI_HtoS using decoder 140
                                             // 11869:         }
  1, 0,                                      // 11874:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 11876:          check predicate 7
  OPC_Decode, 242, 64, 140, 1,               // 11878:          decode to UMLAL_MZZI_HtoS using decoder 140
                                             // 11878:         }
                                             // 11878:         } // switch Inst[22]
                                             // 11878:        }
  1, 0,                                      // 11883:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11885:         switch Inst[22] {
  0, 7,                                      // 11888:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 11890:          check predicate 7
  OPC_Decode, 197, 15, 140, 1,               // 11892:          decode to BFMLSL_MZZI_HtoS using decoder 140
                                             // 11892:         }
  1, 0,                                      // 11897:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 11899:          check predicate 7
  OPC_Decode, 158, 65, 140, 1,               // 11901:          decode to UMLSL_MZZI_HtoS using decoder 140
                                             // 11901:         }
                                             // 11901:         } // switch Inst[22]
                                             // 11901:        }
                                             // 11901:        } // switch Inst[3]
                                             // 11901:       }
  1, 0,                                      // 11906:       case 0x1: {
  OPC_SwitchField, 5, 1,                     // 11908:        switch Inst[5] {
  0, 121,                                    // 11911:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 11913:         switch Inst[3] {
  0, 57,                                     // 11916:         case 0x0: {
  OPC_SwitchField, 15, 1,                    // 11918:          switch Inst[15] {
  0, 21,                                     // 11921:          case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11923:           switch Inst[22] {
  0, 7,                                      // 11926:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11928:            check predicate 7
  OPC_Decode, 180, 15, 141, 1,               // 11930:            decode to BFMLAL_VG2_M2ZZI_HtoS using decoder 141
                                             // 11930:           }
  1, 0,                                      // 11935:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11937:            check predicate 7
  OPC_Decode, 245, 64, 141, 1,               // 11939:            decode to UMLAL_VG2_M2ZZI_S using decoder 141
                                             // 11939:           }
                                             // 11939:           } // switch Inst[22]
                                             // 11939:          }
  1, 0,                                      // 11944:          case 0x1: {
  OPC_SwitchField, 22, 1,                    // 11946:           switch Inst[22] {
  0, 11,                                     // 11949:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11951:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 11953:            check Inst[6] == 0x0
  OPC_Decode, 183, 15, 142, 1,               // 11957:            decode to BFMLAL_VG4_M4ZZI_HtoS using decoder 142
                                             // 11957:           }
  1, 0,                                      // 11962:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11964:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 11966:            check Inst[6] == 0x0
  OPC_Decode, 248, 64, 142, 1,               // 11970:            decode to UMLAL_VG4_M4ZZI_HtoS using decoder 142
                                             // 11970:           }
                                             // 11970:           } // switch Inst[22]
                                             // 11970:          }
                                             // 11970:          } // switch Inst[15]
                                             // 11970:         }
  1, 0,                                      // 11975:         case 0x1: {
  OPC_SwitchField, 15, 1,                    // 11977:          switch Inst[15] {
  0, 21,                                     // 11980:          case 0x0: {
  OPC_SwitchField, 22, 1,                    // 11982:           switch Inst[22] {
  0, 7,                                      // 11985:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 11987:            check predicate 7
  OPC_Decode, 200, 15, 141, 1,               // 11989:            decode to BFMLSL_VG2_M2ZZI_HtoS using decoder 141
                                             // 11989:           }
  1, 0,                                      // 11994:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 11996:            check predicate 7
  OPC_Decode, 161, 65, 141, 1,               // 11998:            decode to UMLSL_VG2_M2ZZI_S using decoder 141
                                             // 11998:           }
                                             // 11998:           } // switch Inst[22]
                                             // 11998:          }
  1, 0,                                      // 12003:          case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12005:           switch Inst[22] {
  0, 11,                                     // 12008:           case 0x0: {
  OPC_CheckPredicate, 7,                     // 12010:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 12012:            check Inst[6] == 0x0
  OPC_Decode, 203, 15, 142, 1,               // 12016:            decode to BFMLSL_VG4_M4ZZI_HtoS using decoder 142
                                             // 12016:           }
  1, 0,                                      // 12021:           case 0x1: {
  OPC_CheckPredicate, 7,                     // 12023:            check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 12025:            check Inst[6] == 0x0
  OPC_Decode, 164, 65, 142, 1,               // 12029:            decode to UMLSL_VG4_M4ZZI_HtoS using decoder 142
                                             // 12029:           }
                                             // 12029:           } // switch Inst[22]
                                             // 12029:          }
                                             // 12029:          } // switch Inst[15]
                                             // 12029:         }
                                             // 12029:         } // switch Inst[3]
                                             // 12029:        }
  1, 0,                                      // 12034:        case 0x1: {
  OPC_CheckPredicate, 9,                     // 12036:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  // 12038:         check Inst[22] == 0x0
  OPC_CheckField, 15, 1, 0,                  // 12042:         check Inst[15] == 0x0
  OPC_Decode, 156, 28, 144, 1,               // 12046:         decode to FMLAL_VG2_M2ZZI_BtoH using decoder 144
                                             // 12046:        }
                                             // 12046:        } // switch Inst[5]
                                             // 12046:       }
                                             // 12046:       } // switch Inst[20]
                                             // 12046:      }
                                             // 12046:      } // switch Inst[12]
                                             // 12046:     }
                                             // 12046:     } // switch Inst[4]
                                             // 12046:    }
  1, 0,                                      // 12051:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 12053:     switch Inst[15] {
  0, 153, 15,                                // 12056:     case 0x0: {
  OPC_SwitchField, 3, 3,                     // 12059:      switch Inst[5:3] {
  0, 219, 3,                                 // 12062:      case 0x0: {
  OPC_SwitchField, 10, 3,                    // 12065:       switch Inst[12:10] {
  0, 109,                                    // 12068:       case 0x0: {
  OPC_SwitchField, 1, 2,                     // 12070:        switch Inst[2:1] {
  0, 65,                                     // 12073:        case 0x0: {
  OPC_SwitchField, 16, 1,                    // 12075:         switch Inst[16] {
  0, 21,                                     // 12078:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12080:          switch Inst[22] {
  0, 7,                                      // 12083:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 12085:           check predicate 7
  OPC_Decode, 216, 49, 145, 1,               // 12087:           decode to SMLALL_VG2_M2Z2Z_BtoS using decoder 145
                                             // 12087:          }
  1, 0,                                      // 12092:          case 0x1: {
  OPC_CheckPredicate, 25,                    // 12094:           check predicate 25
  OPC_Decode, 217, 49, 145, 1,               // 12096:           decode to SMLALL_VG2_M2Z2Z_HtoD using decoder 145
                                             // 12096:          }
                                             // 12096:          } // switch Inst[22]
                                             // 12096:         }
  1, 0,                                      // 12101:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12103:          switch Inst[22] {
  0, 15,                                     // 12106:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 12108:           check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12110:           check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12114:           check Inst[6] == 0x0
  OPC_Decode, 222, 49, 146, 1,               // 12118:           decode to SMLALL_VG4_M4Z4Z_BtoS using decoder 146
                                             // 12118:          }
  1, 0,                                      // 12123:          case 0x1: {
  OPC_CheckPredicate, 25,                    // 12125:           check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 12127:           check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12131:           check Inst[6] == 0x0
  OPC_Decode, 223, 49, 146, 1,               // 12135:           decode to SMLALL_VG4_M4Z4Z_HtoD using decoder 146
                                             // 12135:          }
                                             // 12135:          } // switch Inst[22]
                                             // 12135:         }
                                             // 12135:         } // switch Inst[16]
                                             // 12135:        }
  2, 0,                                      // 12140:        case 0x2: {
  OPC_SwitchField, 16, 1,                    // 12142:         switch Inst[16] {
  0, 11,                                     // 12145:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12147:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 12149:          check Inst[22] == 0x0
  OPC_Decode, 185, 68, 145, 1,               // 12153:          decode to USMLALL_VG2_M2Z2Z_BtoS using decoder 145
                                             // 12153:         }
  1, 0,                                      // 12158:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12160:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 12162:          check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 12166:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12170:          check Inst[6] == 0x0
  OPC_Decode, 188, 68, 146, 1,               // 12174:          decode to USMLALL_VG4_M4Z4Z_BtoS using decoder 146
                                             // 12174:         }
                                             // 12174:         } // switch Inst[16]
                                             // 12174:        }
                                             // 12174:        } // switch Inst[2:1]
                                             // 12174:       }
  2, 81,                                     // 12179:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 12181:        switch Inst[16] {
  0, 29,                                     // 12184:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12186:         switch Inst[22] {
  0, 11,                                     // 12189:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12191:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 12193:          check Inst[2] == 0x0
  OPC_Decode, 155, 28, 147, 1,               // 12197:          decode to FMLAL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 12197:         }
  1, 0,                                      // 12202:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12204:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 12206:          check Inst[2] == 0x0
  OPC_Decode, 235, 49, 147, 1,               // 12210:          decode to SMLAL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 12210:         }
                                             // 12210:         } // switch Inst[22]
                                             // 12210:        }
  1, 0,                                      // 12215:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12217:         switch Inst[22] {
  0, 19,                                     // 12220:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12222:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12224:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12228:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 12232:          check Inst[2] == 0x0
  OPC_Decode, 162, 28, 148, 1,               // 12236:          decode to FMLAL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 12236:         }
  1, 0,                                      // 12241:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12243:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12245:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12249:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 12253:          check Inst[2] == 0x0
  OPC_Decode, 238, 49, 148, 1,               // 12257:          decode to SMLAL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 12257:         }
                                             // 12257:         } // switch Inst[22]
                                             // 12257:        }
                                             // 12257:        } // switch Inst[16]
                                             // 12257:       }
  4, 37,                                     // 12262:       case 0x4: {
  OPC_SwitchField, 16, 1,                    // 12264:        switch Inst[16] {
  0, 11,                                     // 12267:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 12269:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 12271:         check Inst[22] == 0x0
  OPC_Decode, 254, 25, 149, 1,               // 12275:         decode to FDOT_VG2_M2Z2Z_HtoS using decoder 149
                                             // 12275:        }
  1, 0,                                      // 12280:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 12282:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 12284:         check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 12288:         check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12292:         check Inst[6] == 0x0
  OPC_Decode, 135, 26, 150, 1,               // 12296:         decode to FDOT_VG4_M4Z4Z_HtoS using decoder 150
                                             // 12296:        }
                                             // 12296:        } // switch Inst[16]
                                             // 12296:       }
  5, 65,                                     // 12301:       case 0x5: {
  OPC_SwitchField, 16, 1,                    // 12303:        switch Inst[16] {
  0, 21,                                     // 12306:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12308:         switch Inst[22] {
  0, 7,                                      // 12311:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12313:          check predicate 7
  OPC_Decode, 181, 47, 149, 1,               // 12315:          decode to SDOT_VG2_M2Z2Z_BtoS using decoder 149
                                             // 12315:         }
  1, 0,                                      // 12320:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 12322:          check predicate 25
  OPC_Decode, 182, 47, 149, 1,               // 12324:          decode to SDOT_VG2_M2Z2Z_HtoD using decoder 149
                                             // 12324:         }
                                             // 12324:         } // switch Inst[22]
                                             // 12324:        }
  1, 0,                                      // 12329:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12331:         switch Inst[22] {
  0, 15,                                     // 12334:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12336:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12338:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12342:          check Inst[6] == 0x0
  OPC_Decode, 190, 47, 150, 1,               // 12346:          decode to SDOT_VG4_M4Z4Z_BtoS using decoder 150
                                             // 12346:         }
  1, 0,                                      // 12351:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 12353:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 12355:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12359:          check Inst[6] == 0x0
  OPC_Decode, 191, 47, 150, 1,               // 12363:          decode to SDOT_VG4_M4Z4Z_HtoD using decoder 150
                                             // 12363:         }
                                             // 12363:         } // switch Inst[22]
                                             // 12363:        }
                                             // 12363:        } // switch Inst[16]
                                             // 12363:       }
  6, 65,                                     // 12368:       case 0x6: {
  OPC_SwitchField, 16, 1,                    // 12370:        switch Inst[16] {
  0, 21,                                     // 12373:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12375:         switch Inst[22] {
  0, 7,                                      // 12378:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12380:          check predicate 7
  OPC_Decode, 173, 28, 149, 1,               // 12382:          decode to FMLA_VG2_M2Z2Z_S using decoder 149
                                             // 12382:         }
  1, 0,                                      // 12387:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12389:          check predicate 26
  OPC_Decode, 171, 28, 149, 1,               // 12391:          decode to FMLA_VG2_M2Z2Z_D using decoder 149
                                             // 12391:         }
                                             // 12391:         } // switch Inst[22]
                                             // 12391:        }
  1, 0,                                      // 12396:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12398:         switch Inst[22] {
  0, 15,                                     // 12401:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12403:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12405:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12409:          check Inst[6] == 0x0
  OPC_Decode, 182, 28, 150, 1,               // 12413:          decode to FMLA_VG4_M4Z4Z_S using decoder 150
                                             // 12413:         }
  1, 0,                                      // 12418:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12420:          check predicate 26
  OPC_CheckField, 17, 1, 0,                  // 12422:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12426:          check Inst[6] == 0x0
  OPC_Decode, 180, 28, 150, 1,               // 12430:          decode to FMLA_VG4_M4Z4Z_D using decoder 150
                                             // 12430:         }
                                             // 12430:         } // switch Inst[22]
                                             // 12430:        }
                                             // 12430:        } // switch Inst[16]
                                             // 12430:       }
  7, 0,                                      // 12435:       case 0x7: {
  OPC_SwitchField, 16, 5,                    // 12437:        switch Inst[20:16] {
  0, 19,                                     // 12440:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12442:         switch Inst[22] {
  0, 6,                                      // 12445:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12447:          check predicate 7
  OPC_Decode, 252, 21, 31,                   // 12449:          decode to FADD_VG2_M2Z_S using decoder 31
                                             // 12449:         }
  1, 0,                                      // 12453:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12455:          check predicate 26
  OPC_Decode, 250, 21, 31,                   // 12457:          decode to FADD_VG2_M2Z_D using decoder 31
                                             // 12457:         }
                                             // 12457:         } // switch Inst[22]
                                             // 12457:        }
  1, 27,                                     // 12461:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12463:         switch Inst[22] {
  0, 10,                                     // 12466:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12468:          check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 12470:          check Inst[6] == 0x0
  OPC_Decode, 255, 21, 32,                   // 12474:          decode to FADD_VG4_M4Z_S using decoder 32
                                             // 12474:         }
  1, 0,                                      // 12478:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12480:          check predicate 26
  OPC_CheckField, 6, 1, 0,                   // 12482:          check Inst[6] == 0x0
  OPC_Decode, 253, 21, 32,                   // 12486:          decode to FADD_VG4_M4Z_D using decoder 32
                                             // 12486:         }
                                             // 12486:         } // switch Inst[22]
                                             // 12486:        }
  4, 19,                                     // 12490:        case 0x4: {
  OPC_SwitchField, 22, 1,                    // 12492:         switch Inst[22] {
  0, 6,                                      // 12495:         case 0x0: {
  OPC_CheckPredicate, 32,                    // 12497:          check predicate 32
  OPC_Decode, 251, 21, 31,                   // 12499:          decode to FADD_VG2_M2Z_H using decoder 31
                                             // 12499:         }
  1, 0,                                      // 12503:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 12505:          check predicate 17
  OPC_Decode, 249, 14, 31,                   // 12507:          decode to BFADD_VG2_M2Z_H using decoder 31
                                             // 12507:         }
                                             // 12507:         } // switch Inst[22]
                                             // 12507:        }
  5, 0,                                      // 12511:        case 0x5: {
  OPC_SwitchField, 22, 1,                    // 12513:         switch Inst[22] {
  0, 10,                                     // 12516:         case 0x0: {
  OPC_CheckPredicate, 32,                    // 12518:          check predicate 32
  OPC_CheckField, 6, 1, 0,                   // 12520:          check Inst[6] == 0x0
  OPC_Decode, 254, 21, 32,                   // 12524:          decode to FADD_VG4_M4Z_H using decoder 32
                                             // 12524:         }
  1, 0,                                      // 12528:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 12530:          check predicate 17
  OPC_CheckField, 6, 1, 0,                   // 12532:          check Inst[6] == 0x0
  OPC_Decode, 250, 14, 32,                   // 12536:          decode to BFADD_VG4_M4Z_H using decoder 32
                                             // 12536:         }
                                             // 12536:         } // switch Inst[22]
                                             // 12536:        }
                                             // 12536:        } // switch Inst[20:16]
                                             // 12536:       }
                                             // 12536:       } // switch Inst[12:10]
                                             // 12536:      }
  1, 219, 3,                                 // 12540:      case 0x1: {
  OPC_SwitchField, 10, 3,                    // 12543:       switch Inst[12:10] {
  0, 81,                                     // 12546:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 12548:        switch Inst[16] {
  0, 29,                                     // 12551:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12553:         switch Inst[22] {
  0, 11,                                     // 12556:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12558:          check predicate 7
  OPC_CheckField, 1, 2, 0,                   // 12560:          check Inst[2:1] == 0x0
  OPC_Decode, 132, 50, 145, 1,               // 12564:          decode to SMLSLL_VG2_M2Z2Z_BtoS using decoder 145
                                             // 12564:         }
  1, 0,                                      // 12569:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 12571:          check predicate 25
  OPC_CheckField, 1, 2, 0,                   // 12573:          check Inst[2:1] == 0x0
  OPC_Decode, 133, 50, 145, 1,               // 12577:          decode to SMLSLL_VG2_M2Z2Z_HtoD using decoder 145
                                             // 12577:         }
                                             // 12577:         } // switch Inst[22]
                                             // 12577:        }
  1, 0,                                      // 12582:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12584:         switch Inst[22] {
  0, 19,                                     // 12587:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12589:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12591:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12595:          check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 12599:          check Inst[2:1] == 0x0
  OPC_Decode, 138, 50, 146, 1,               // 12603:          decode to SMLSLL_VG4_M4Z4Z_BtoS using decoder 146
                                             // 12603:         }
  1, 0,                                      // 12608:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 12610:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 12612:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12616:          check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 12620:          check Inst[2:1] == 0x0
  OPC_Decode, 139, 50, 146, 1,               // 12624:          decode to SMLSLL_VG4_M4Z4Z_HtoD using decoder 146
                                             // 12624:         }
                                             // 12624:         } // switch Inst[22]
                                             // 12624:        }
                                             // 12624:        } // switch Inst[16]
                                             // 12624:       }
  2, 81,                                     // 12629:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 12631:        switch Inst[16] {
  0, 29,                                     // 12634:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12636:         switch Inst[22] {
  0, 11,                                     // 12639:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12641:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 12643:          check Inst[2] == 0x0
  OPC_Decode, 219, 28, 147, 1,               // 12647:          decode to FMLSL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 12647:         }
  1, 0,                                      // 12652:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12654:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 12656:          check Inst[2] == 0x0
  OPC_Decode, 151, 50, 147, 1,               // 12660:          decode to SMLSL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 12660:         }
                                             // 12660:         } // switch Inst[22]
                                             // 12660:        }
  1, 0,                                      // 12665:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12667:         switch Inst[22] {
  0, 19,                                     // 12670:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12672:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12674:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12678:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 12682:          check Inst[2] == 0x0
  OPC_Decode, 222, 28, 148, 1,               // 12686:          decode to FMLSL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 12686:         }
  1, 0,                                      // 12691:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12693:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12695:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12699:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 12703:          check Inst[2] == 0x0
  OPC_Decode, 154, 50, 148, 1,               // 12707:          decode to SMLSL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 12707:         }
                                             // 12707:         } // switch Inst[22]
                                             // 12707:        }
                                             // 12707:        } // switch Inst[16]
                                             // 12707:       }
  4, 65,                                     // 12712:       case 0x4: {
  OPC_SwitchField, 16, 1,                    // 12714:        switch Inst[16] {
  0, 21,                                     // 12717:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12719:         switch Inst[22] {
  0, 7,                                      // 12722:         case 0x0: {
  OPC_CheckPredicate, 16,                    // 12724:          check predicate 16
  OPC_Decode, 172, 28, 149, 1,               // 12726:          decode to FMLA_VG2_M2Z2Z_H using decoder 149
                                             // 12726:         }
  1, 0,                                      // 12731:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 12733:          check predicate 17
  OPC_Decode, 185, 15, 149, 1,               // 12735:          decode to BFMLA_VG2_M2Z2Z using decoder 149
                                             // 12735:         }
                                             // 12735:         } // switch Inst[22]
                                             // 12735:        }
  1, 0,                                      // 12740:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12742:         switch Inst[22] {
  0, 15,                                     // 12745:         case 0x0: {
  OPC_CheckPredicate, 16,                    // 12747:          check predicate 16
  OPC_CheckField, 17, 1, 0,                  // 12749:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12753:          check Inst[6] == 0x0
  OPC_Decode, 181, 28, 150, 1,               // 12757:          decode to FMLA_VG4_M4Z4Z_H using decoder 150
                                             // 12757:         }
  1, 0,                                      // 12762:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 12764:          check predicate 17
  OPC_CheckField, 17, 1, 0,                  // 12766:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12770:          check Inst[6] == 0x0
  OPC_Decode, 188, 15, 150, 1,               // 12774:          decode to BFMLA_VG4_M4Z4Z using decoder 150
                                             // 12774:         }
                                             // 12774:         } // switch Inst[22]
                                             // 12774:        }
                                             // 12774:        } // switch Inst[16]
                                             // 12774:       }
  5, 65,                                     // 12779:       case 0x5: {
  OPC_SwitchField, 16, 1,                    // 12781:        switch Inst[16] {
  0, 21,                                     // 12784:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12786:         switch Inst[22] {
  0, 7,                                      // 12789:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12791:          check predicate 7
  OPC_Decode, 143, 68, 149, 1,               // 12793:          decode to USDOT_VG2_M2Z2Z_BToS using decoder 149
                                             // 12793:         }
  1, 0,                                      // 12798:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12800:          check predicate 7
  OPC_Decode, 183, 47, 149, 1,               // 12802:          decode to SDOT_VG2_M2Z2Z_HtoS using decoder 149
                                             // 12802:         }
                                             // 12802:         } // switch Inst[22]
                                             // 12802:        }
  1, 0,                                      // 12807:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12809:         switch Inst[22] {
  0, 15,                                     // 12812:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12814:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12816:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12820:          check Inst[6] == 0x0
  OPC_Decode, 146, 68, 150, 1,               // 12824:          decode to USDOT_VG4_M4Z4Z_BToS using decoder 150
                                             // 12824:         }
  1, 0,                                      // 12829:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 12831:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12833:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12837:          check Inst[6] == 0x0
  OPC_Decode, 192, 47, 150, 1,               // 12841:          decode to SDOT_VG4_M4Z4Z_HtoS using decoder 150
                                             // 12841:         }
                                             // 12841:         } // switch Inst[22]
                                             // 12841:        }
                                             // 12841:        } // switch Inst[16]
                                             // 12841:       }
  6, 65,                                     // 12846:       case 0x6: {
  OPC_SwitchField, 16, 1,                    // 12848:        switch Inst[16] {
  0, 21,                                     // 12851:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12853:         switch Inst[22] {
  0, 7,                                      // 12856:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12858:          check predicate 7
  OPC_Decode, 231, 28, 149, 1,               // 12860:          decode to FMLS_VG2_M2Z2Z_S using decoder 149
                                             // 12860:         }
  1, 0,                                      // 12865:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12867:          check predicate 26
  OPC_Decode, 229, 28, 149, 1,               // 12869:          decode to FMLS_VG2_M2Z2Z_D using decoder 149
                                             // 12869:         }
                                             // 12869:         } // switch Inst[22]
                                             // 12869:        }
  1, 0,                                      // 12874:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12876:         switch Inst[22] {
  0, 15,                                     // 12879:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12881:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 12883:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12887:          check Inst[6] == 0x0
  OPC_Decode, 240, 28, 150, 1,               // 12891:          decode to FMLS_VG4_M4Z4Z_S using decoder 150
                                             // 12891:         }
  1, 0,                                      // 12896:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12898:          check predicate 26
  OPC_CheckField, 17, 1, 0,                  // 12900:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 12904:          check Inst[6] == 0x0
  OPC_Decode, 238, 28, 150, 1,               // 12908:          decode to FMLS_VG4_M4Z4Z_D using decoder 150
                                             // 12908:         }
                                             // 12908:         } // switch Inst[22]
                                             // 12908:        }
                                             // 12908:        } // switch Inst[16]
                                             // 12908:       }
  7, 0,                                      // 12913:       case 0x7: {
  OPC_SwitchField, 16, 5,                    // 12915:        switch Inst[20:16] {
  0, 19,                                     // 12918:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 12920:         switch Inst[22] {
  0, 6,                                      // 12923:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12925:          check predicate 7
  OPC_Decode, 174, 32, 31,                   // 12927:          decode to FSUB_VG2_M2Z_S using decoder 31
                                             // 12927:         }
  1, 0,                                      // 12931:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12933:          check predicate 26
  OPC_Decode, 172, 32, 31,                   // 12935:          decode to FSUB_VG2_M2Z_D using decoder 31
                                             // 12935:         }
                                             // 12935:         } // switch Inst[22]
                                             // 12935:        }
  1, 27,                                     // 12939:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 12941:         switch Inst[22] {
  0, 10,                                     // 12944:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 12946:          check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 12948:          check Inst[6] == 0x0
  OPC_Decode, 177, 32, 32,                   // 12952:          decode to FSUB_VG4_M4Z_S using decoder 32
                                             // 12952:         }
  1, 0,                                      // 12956:         case 0x1: {
  OPC_CheckPredicate, 26,                    // 12958:          check predicate 26
  OPC_CheckField, 6, 1, 0,                   // 12960:          check Inst[6] == 0x0
  OPC_Decode, 175, 32, 32,                   // 12964:          decode to FSUB_VG4_M4Z_D using decoder 32
                                             // 12964:         }
                                             // 12964:         } // switch Inst[22]
                                             // 12964:        }
  4, 19,                                     // 12968:        case 0x4: {
  OPC_SwitchField, 22, 1,                    // 12970:         switch Inst[22] {
  0, 6,                                      // 12973:         case 0x0: {
  OPC_CheckPredicate, 32,                    // 12975:          check predicate 32
  OPC_Decode, 173, 32, 31,                   // 12977:          decode to FSUB_VG2_M2Z_H using decoder 31
                                             // 12977:         }
  1, 0,                                      // 12981:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 12983:          check predicate 17
  OPC_Decode, 250, 15, 31,                   // 12985:          decode to BFSUB_VG2_M2Z_H using decoder 31
                                             // 12985:         }
                                             // 12985:         } // switch Inst[22]
                                             // 12985:        }
  5, 0,                                      // 12989:        case 0x5: {
  OPC_SwitchField, 22, 1,                    // 12991:         switch Inst[22] {
  0, 10,                                     // 12994:         case 0x0: {
  OPC_CheckPredicate, 32,                    // 12996:          check predicate 32
  OPC_CheckField, 6, 1, 0,                   // 12998:          check Inst[6] == 0x0
  OPC_Decode, 176, 32, 32,                   // 13002:          decode to FSUB_VG4_M4Z_H using decoder 32
                                             // 13002:         }
  1, 0,                                      // 13006:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 13008:          check predicate 17
  OPC_CheckField, 6, 1, 0,                   // 13010:          check Inst[6] == 0x0
  OPC_Decode, 251, 15, 32,                   // 13014:          decode to BFSUB_VG4_M4Z_H using decoder 32
                                             // 13014:         }
                                             // 13014:         } // switch Inst[22]
                                             // 13014:        }
                                             // 13014:        } // switch Inst[20:16]
                                             // 13014:       }
                                             // 13014:       } // switch Inst[12:10]
                                             // 13014:      }
  2, 141, 3,                                 // 13018:      case 0x2: {
  OPC_SwitchField, 10, 3,                    // 13021:       switch Inst[12:10] {
  0, 81,                                     // 13024:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 13026:        switch Inst[16] {
  0, 29,                                     // 13029:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13031:         switch Inst[22] {
  0, 11,                                     // 13034:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13036:          check predicate 7
  OPC_CheckField, 1, 2, 0,                   // 13038:          check Inst[2:1] == 0x0
  OPC_Decode, 225, 64, 145, 1,               // 13042:          decode to UMLALL_VG2_M2Z2Z_BtoS using decoder 145
                                             // 13042:         }
  1, 0,                                      // 13047:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13049:          check predicate 25
  OPC_CheckField, 1, 2, 0,                   // 13051:          check Inst[2:1] == 0x0
  OPC_Decode, 226, 64, 145, 1,               // 13055:          decode to UMLALL_VG2_M2Z2Z_HtoD using decoder 145
                                             // 13055:         }
                                             // 13055:         } // switch Inst[22]
                                             // 13055:        }
  1, 0,                                      // 13060:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13062:         switch Inst[22] {
  0, 19,                                     // 13065:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13067:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13069:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13073:          check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 13077:          check Inst[2:1] == 0x0
  OPC_Decode, 231, 64, 146, 1,               // 13081:          decode to UMLALL_VG4_M4Z4Z_BtoS using decoder 146
                                             // 13081:         }
  1, 0,                                      // 13086:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13088:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 13090:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13094:          check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 13098:          check Inst[2:1] == 0x0
  OPC_Decode, 232, 64, 146, 1,               // 13102:          decode to UMLALL_VG4_M4Z4Z_HtoD using decoder 146
                                             // 13102:         }
                                             // 13102:         } // switch Inst[22]
                                             // 13102:        }
                                             // 13102:        } // switch Inst[16]
                                             // 13102:       }
  2, 81,                                     // 13107:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 13109:        switch Inst[16] {
  0, 29,                                     // 13112:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13114:         switch Inst[22] {
  0, 11,                                     // 13117:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13119:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 13121:          check Inst[2] == 0x0
  OPC_Decode, 179, 15, 147, 1,               // 13125:          decode to BFMLAL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 13125:         }
  1, 0,                                      // 13130:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 13132:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 13134:          check Inst[2] == 0x0
  OPC_Decode, 244, 64, 147, 1,               // 13138:          decode to UMLAL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 13138:         }
                                             // 13138:         } // switch Inst[22]
                                             // 13138:        }
  1, 0,                                      // 13143:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13145:         switch Inst[22] {
  0, 19,                                     // 13148:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13150:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13152:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13156:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 13160:          check Inst[2] == 0x0
  OPC_Decode, 182, 15, 148, 1,               // 13164:          decode to BFMLAL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 13164:         }
  1, 0,                                      // 13169:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 13171:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13173:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13177:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 13181:          check Inst[2] == 0x0
  OPC_Decode, 247, 64, 148, 1,               // 13185:          decode to UMLAL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 13185:         }
                                             // 13185:         } // switch Inst[22]
                                             // 13185:        }
                                             // 13185:        } // switch Inst[16]
                                             // 13185:       }
  4, 37,                                     // 13190:       case 0x4: {
  OPC_SwitchField, 16, 1,                    // 13192:        switch Inst[16] {
  0, 11,                                     // 13195:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 13197:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 13199:         check Inst[22] == 0x0
  OPC_Decode, 139, 15, 149, 1,               // 13203:         decode to BFDOT_VG2_M2Z2Z_HtoS using decoder 149
                                             // 13203:        }
  1, 0,                                      // 13208:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 13210:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 13212:         check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 13216:         check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13220:         check Inst[6] == 0x0
  OPC_Decode, 142, 15, 150, 1,               // 13224:         decode to BFDOT_VG4_M4Z4Z_HtoS using decoder 150
                                             // 13224:        }
                                             // 13224:        } // switch Inst[16]
                                             // 13224:       }
  5, 65,                                     // 13229:       case 0x5: {
  OPC_SwitchField, 16, 1,                    // 13231:        switch Inst[16] {
  0, 21,                                     // 13234:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13236:         switch Inst[22] {
  0, 7,                                      // 13239:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13241:          check predicate 7
  OPC_Decode, 175, 63, 149, 1,               // 13243:          decode to UDOT_VG2_M2Z2Z_BtoS using decoder 149
                                             // 13243:         }
  1, 0,                                      // 13248:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13250:          check predicate 25
  OPC_Decode, 176, 63, 149, 1,               // 13252:          decode to UDOT_VG2_M2Z2Z_HtoD using decoder 149
                                             // 13252:         }
                                             // 13252:         } // switch Inst[22]
                                             // 13252:        }
  1, 0,                                      // 13257:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13259:         switch Inst[22] {
  0, 15,                                     // 13262:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13264:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13266:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13270:          check Inst[6] == 0x0
  OPC_Decode, 184, 63, 150, 1,               // 13274:          decode to UDOT_VG4_M4Z4Z_BtoS using decoder 150
                                             // 13274:         }
  1, 0,                                      // 13279:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13281:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 13283:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13287:          check Inst[6] == 0x0
  OPC_Decode, 185, 63, 150, 1,               // 13291:          decode to UDOT_VG4_M4Z4Z_HtoD using decoder 150
                                             // 13291:         }
                                             // 13291:         } // switch Inst[22]
                                             // 13291:        }
                                             // 13291:        } // switch Inst[16]
                                             // 13291:       }
  6, 65,                                     // 13296:       case 0x6: {
  OPC_SwitchField, 16, 1,                    // 13298:        switch Inst[16] {
  0, 21,                                     // 13301:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13303:         switch Inst[22] {
  0, 7,                                      // 13306:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13308:          check predicate 7
  OPC_Decode, 204, 13, 149, 1,               // 13310:          decode to ADD_VG2_M2Z2Z_S using decoder 149
                                             // 13310:         }
  1, 0,                                      // 13315:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13317:          check predicate 25
  OPC_Decode, 203, 13, 149, 1,               // 13319:          decode to ADD_VG2_M2Z2Z_D using decoder 149
                                             // 13319:         }
                                             // 13319:         } // switch Inst[22]
                                             // 13319:        }
  1, 0,                                      // 13324:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13326:         switch Inst[22] {
  0, 15,                                     // 13329:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13331:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13333:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13337:          check Inst[6] == 0x0
  OPC_Decode, 214, 13, 150, 1,               // 13341:          decode to ADD_VG4_M4Z4Z_S using decoder 150
                                             // 13341:         }
  1, 0,                                      // 13346:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13348:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 13350:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13354:          check Inst[6] == 0x0
  OPC_Decode, 213, 13, 150, 1,               // 13358:          decode to ADD_VG4_M4Z4Z_D using decoder 150
                                             // 13358:         }
                                             // 13358:         } // switch Inst[22]
                                             // 13358:        }
                                             // 13358:        } // switch Inst[16]
                                             // 13358:       }
  7, 0,                                      // 13363:       case 0x7: {
  OPC_SwitchField, 16, 5,                    // 13365:        switch Inst[20:16] {
  0, 19,                                     // 13368:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13370:         switch Inst[22] {
  0, 6,                                      // 13373:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13375:          check predicate 7
  OPC_Decode, 208, 13, 31,                   // 13377:          decode to ADD_VG2_M2Z_S using decoder 31
                                             // 13377:         }
  1, 0,                                      // 13381:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13383:          check predicate 25
  OPC_Decode, 207, 13, 31,                   // 13385:          decode to ADD_VG2_M2Z_D using decoder 31
                                             // 13385:         }
                                             // 13385:         } // switch Inst[22]
                                             // 13385:        }
  1, 0,                                      // 13389:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13391:         switch Inst[22] {
  0, 10,                                     // 13394:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13396:          check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 13398:          check Inst[6] == 0x0
  OPC_Decode, 218, 13, 32,                   // 13402:          decode to ADD_VG4_M4Z_S using decoder 32
                                             // 13402:         }
  1, 0,                                      // 13406:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13408:          check predicate 25
  OPC_CheckField, 6, 1, 0,                   // 13410:          check Inst[6] == 0x0
  OPC_Decode, 217, 13, 32,                   // 13414:          decode to ADD_VG4_M4Z_D using decoder 32
                                             // 13414:         }
                                             // 13414:         } // switch Inst[22]
                                             // 13414:        }
                                             // 13414:        } // switch Inst[20:16]
                                             // 13414:       }
                                             // 13414:       } // switch Inst[12:10]
                                             // 13414:      }
  3, 141, 3,                                 // 13418:      case 0x3: {
  OPC_SwitchField, 10, 3,                    // 13421:       switch Inst[12:10] {
  0, 81,                                     // 13424:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 13426:        switch Inst[16] {
  0, 29,                                     // 13429:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13431:         switch Inst[22] {
  0, 11,                                     // 13434:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13436:          check predicate 7
  OPC_CheckField, 1, 2, 0,                   // 13438:          check Inst[2:1] == 0x0
  OPC_Decode, 141, 65, 145, 1,               // 13442:          decode to UMLSLL_VG2_M2Z2Z_BtoS using decoder 145
                                             // 13442:         }
  1, 0,                                      // 13447:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13449:          check predicate 25
  OPC_CheckField, 1, 2, 0,                   // 13451:          check Inst[2:1] == 0x0
  OPC_Decode, 142, 65, 145, 1,               // 13455:          decode to UMLSLL_VG2_M2Z2Z_HtoD using decoder 145
                                             // 13455:         }
                                             // 13455:         } // switch Inst[22]
                                             // 13455:        }
  1, 0,                                      // 13460:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13462:         switch Inst[22] {
  0, 19,                                     // 13465:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13467:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13469:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13473:          check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 13477:          check Inst[2:1] == 0x0
  OPC_Decode, 147, 65, 146, 1,               // 13481:          decode to UMLSLL_VG4_M4Z4Z_BtoS using decoder 146
                                             // 13481:         }
  1, 0,                                      // 13486:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13488:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 13490:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13494:          check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 13498:          check Inst[2:1] == 0x0
  OPC_Decode, 148, 65, 146, 1,               // 13502:          decode to UMLSLL_VG4_M4Z4Z_HtoD using decoder 146
                                             // 13502:         }
                                             // 13502:         } // switch Inst[22]
                                             // 13502:        }
                                             // 13502:        } // switch Inst[16]
                                             // 13502:       }
  2, 81,                                     // 13507:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 13509:        switch Inst[16] {
  0, 29,                                     // 13512:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13514:         switch Inst[22] {
  0, 11,                                     // 13517:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13519:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 13521:          check Inst[2] == 0x0
  OPC_Decode, 199, 15, 147, 1,               // 13525:          decode to BFMLSL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 13525:         }
  1, 0,                                      // 13530:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 13532:          check predicate 7
  OPC_CheckField, 2, 1, 0,                   // 13534:          check Inst[2] == 0x0
  OPC_Decode, 160, 65, 147, 1,               // 13538:          decode to UMLSL_VG2_M2Z2Z_HtoS using decoder 147
                                             // 13538:         }
                                             // 13538:         } // switch Inst[22]
                                             // 13538:        }
  1, 0,                                      // 13543:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13545:         switch Inst[22] {
  0, 19,                                     // 13548:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13550:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13552:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13556:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 13560:          check Inst[2] == 0x0
  OPC_Decode, 202, 15, 148, 1,               // 13564:          decode to BFMLSL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 13564:         }
  1, 0,                                      // 13569:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 13571:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13573:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13577:          check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 13581:          check Inst[2] == 0x0
  OPC_Decode, 163, 65, 148, 1,               // 13585:          decode to UMLSL_VG4_M4Z4Z_HtoS using decoder 148
                                             // 13585:         }
                                             // 13585:         } // switch Inst[22]
                                             // 13585:        }
                                             // 13585:        } // switch Inst[16]
                                             // 13585:       }
  4, 65,                                     // 13590:       case 0x4: {
  OPC_SwitchField, 16, 1,                    // 13592:        switch Inst[16] {
  0, 21,                                     // 13595:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13597:         switch Inst[22] {
  0, 7,                                      // 13600:         case 0x0: {
  OPC_CheckPredicate, 16,                    // 13602:          check predicate 16
  OPC_Decode, 230, 28, 149, 1,               // 13604:          decode to FMLS_VG2_M2Z2Z_H using decoder 149
                                             // 13604:         }
  1, 0,                                      // 13609:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 13611:          check predicate 17
  OPC_Decode, 205, 15, 149, 1,               // 13613:          decode to BFMLS_VG2_M2Z2Z using decoder 149
                                             // 13613:         }
                                             // 13613:         } // switch Inst[22]
                                             // 13613:        }
  1, 0,                                      // 13618:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13620:         switch Inst[22] {
  0, 15,                                     // 13623:         case 0x0: {
  OPC_CheckPredicate, 16,                    // 13625:          check predicate 16
  OPC_CheckField, 17, 1, 0,                  // 13627:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13631:          check Inst[6] == 0x0
  OPC_Decode, 239, 28, 150, 1,               // 13635:          decode to FMLS_VG4_M4Z4Z_H using decoder 150
                                             // 13635:         }
  1, 0,                                      // 13640:         case 0x1: {
  OPC_CheckPredicate, 17,                    // 13642:          check predicate 17
  OPC_CheckField, 17, 1, 0,                  // 13644:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13648:          check Inst[6] == 0x0
  OPC_Decode, 208, 15, 150, 1,               // 13652:          decode to BFMLS_VG4_M4Z4Z using decoder 150
                                             // 13652:         }
                                             // 13652:         } // switch Inst[22]
                                             // 13652:        }
                                             // 13652:        } // switch Inst[16]
                                             // 13652:       }
  5, 37,                                     // 13657:       case 0x5: {
  OPC_SwitchField, 16, 1,                    // 13659:        switch Inst[16] {
  0, 11,                                     // 13662:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 13664:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 13666:         check Inst[22] == 0x1
  OPC_Decode, 177, 63, 149, 1,               // 13670:         decode to UDOT_VG2_M2Z2Z_HtoS using decoder 149
                                             // 13670:        }
  1, 0,                                      // 13675:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 13677:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 13679:         check Inst[22] == 0x1
  OPC_CheckField, 17, 1, 0,                  // 13683:         check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13687:         check Inst[6] == 0x0
  OPC_Decode, 186, 63, 150, 1,               // 13691:         decode to UDOT_VG4_M4Z4Z_HtoS using decoder 150
                                             // 13691:        }
                                             // 13691:        } // switch Inst[16]
                                             // 13691:       }
  6, 65,                                     // 13696:       case 0x6: {
  OPC_SwitchField, 16, 1,                    // 13698:        switch Inst[16] {
  0, 21,                                     // 13701:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13703:         switch Inst[22] {
  0, 7,                                      // 13706:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13708:          check predicate 7
  OPC_Decode, 167, 60, 149, 1,               // 13710:          decode to SUB_VG2_M2Z2Z_S using decoder 149
                                             // 13710:         }
  1, 0,                                      // 13715:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13717:          check predicate 25
  OPC_Decode, 166, 60, 149, 1,               // 13719:          decode to SUB_VG2_M2Z2Z_D using decoder 149
                                             // 13719:         }
                                             // 13719:         } // switch Inst[22]
                                             // 13719:        }
  1, 0,                                      // 13724:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13726:         switch Inst[22] {
  0, 15,                                     // 13729:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13731:          check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 13733:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13737:          check Inst[6] == 0x0
  OPC_Decode, 173, 60, 150, 1,               // 13741:          decode to SUB_VG4_M4Z4Z_S using decoder 150
                                             // 13741:         }
  1, 0,                                      // 13746:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13748:          check predicate 25
  OPC_CheckField, 17, 1, 0,                  // 13750:          check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13754:          check Inst[6] == 0x0
  OPC_Decode, 172, 60, 150, 1,               // 13758:          decode to SUB_VG4_M4Z4Z_D using decoder 150
                                             // 13758:         }
                                             // 13758:         } // switch Inst[22]
                                             // 13758:        }
                                             // 13758:        } // switch Inst[16]
                                             // 13758:       }
  7, 0,                                      // 13763:       case 0x7: {
  OPC_SwitchField, 16, 5,                    // 13765:        switch Inst[20:16] {
  0, 19,                                     // 13768:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 13770:         switch Inst[22] {
  0, 6,                                      // 13773:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13775:          check predicate 7
  OPC_Decode, 171, 60, 31,                   // 13777:          decode to SUB_VG2_M2Z_S using decoder 31
                                             // 13777:         }
  1, 0,                                      // 13781:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13783:          check predicate 25
  OPC_Decode, 170, 60, 31,                   // 13785:          decode to SUB_VG2_M2Z_D using decoder 31
                                             // 13785:         }
                                             // 13785:         } // switch Inst[22]
                                             // 13785:        }
  1, 0,                                      // 13789:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 13791:         switch Inst[22] {
  0, 10,                                     // 13794:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 13796:          check predicate 7
  OPC_CheckField, 6, 1, 0,                   // 13798:          check Inst[6] == 0x0
  OPC_Decode, 177, 60, 32,                   // 13802:          decode to SUB_VG4_M4Z_S using decoder 32
                                             // 13802:         }
  1, 0,                                      // 13806:         case 0x1: {
  OPC_CheckPredicate, 25,                    // 13808:          check predicate 25
  OPC_CheckField, 6, 1, 0,                   // 13810:          check Inst[6] == 0x0
  OPC_Decode, 176, 60, 32,                   // 13814:          decode to SUB_VG4_M4Z_D using decoder 32
                                             // 13814:         }
                                             // 13814:         } // switch Inst[22]
                                             // 13814:        }
                                             // 13814:        } // switch Inst[20:16]
                                             // 13814:       }
                                             // 13814:       } // switch Inst[12:10]
                                             // 13814:      }
  4, 136, 1,                                 // 13818:      case 0x4: {
  OPC_SwitchField, 10, 3,                    // 13821:       switch Inst[12:10] {
  0, 45,                                     // 13824:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 13826:        switch Inst[16] {
  0, 15,                                     // 13829:        case 0x0: {
  OPC_CheckPredicate, 8,                     // 13831:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  // 13833:         check Inst[22] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 13837:         check Inst[2:1] == 0x0
  OPC_Decode, 139, 28, 145, 1,               // 13841:         decode to FMLALL_VG2_M2Z2Z_BtoS using decoder 145
                                             // 13841:        }
  1, 0,                                      // 13846:        case 0x1: {
  OPC_CheckPredicate, 8,                     // 13848:         check predicate 8
  OPC_CheckField, 22, 1, 0,                  // 13850:         check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 13854:         check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13858:         check Inst[6] == 0x0
  OPC_CheckField, 1, 2, 0,                   // 13862:         check Inst[2:1] == 0x0
  OPC_Decode, 142, 28, 146, 1,               // 13866:         decode to FMLALL_VG4_M4Z4Z_BtoS using decoder 146
                                             // 13866:        }
                                             // 13866:        } // switch Inst[16]
                                             // 13866:       }
  2, 45,                                     // 13871:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 13873:        switch Inst[16] {
  0, 15,                                     // 13876:        case 0x0: {
  OPC_CheckPredicate, 9,                     // 13878:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  // 13880:         check Inst[22] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 13884:         check Inst[2] == 0x0
  OPC_Decode, 154, 28, 147, 1,               // 13888:         decode to FMLAL_VG2_M2Z2Z_BtoH using decoder 147
                                             // 13888:        }
  1, 0,                                      // 13893:        case 0x1: {
  OPC_CheckPredicate, 9,                     // 13895:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  // 13897:         check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 13901:         check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13905:         check Inst[6] == 0x0
  OPC_CheckField, 2, 1, 0,                   // 13909:         check Inst[2] == 0x0
  OPC_Decode, 161, 28, 148, 1,               // 13913:         decode to FMLAL_VG4_M4Z4Z_BtoH using decoder 148
                                             // 13913:        }
                                             // 13913:        } // switch Inst[16]
                                             // 13913:       }
  4, 0,                                      // 13918:       case 0x4: {
  OPC_SwitchField, 16, 1,                    // 13920:        switch Inst[16] {
  0, 11,                                     // 13923:        case 0x0: {
  OPC_CheckPredicate, 9,                     // 13925:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  // 13927:         check Inst[22] == 0x0
  OPC_Decode, 252, 25, 149, 1,               // 13931:         decode to FDOT_VG2_M2Z2Z_BtoH using decoder 149
                                             // 13931:        }
  1, 0,                                      // 13936:        case 0x1: {
  OPC_CheckPredicate, 9,                     // 13938:         check predicate 9
  OPC_CheckField, 22, 1, 0,                  // 13940:         check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 13944:         check Inst[17] == 0x0
  OPC_CheckField, 6, 1, 0,                   // 13948:         check Inst[6] == 0x0
  OPC_Decode, 133, 26, 150, 1,               // 13952:         decode to FDOT_VG4_M4Z4Z_BtoH using decoder 150
                                             // 13952:        }
                                             // 13952:        } // switch Inst[16]
                                             // 13952:       }
                                             // 13952:       } // switch Inst[12:10]
                                             // 13952:      }
  6, 0,                                      // 13957:      case 0x6: {
  OPC_SwitchField, 16, 1,                    // 13959:       switch Inst[16] {
  0, 15,                                     // 13962:       case 0x0: {
  OPC_CheckPredicate, 8,                     // 13964:        check predicate 8
  OPC_CheckField, 22, 1, 0,                  // 13966:        check Inst[22] == 0x0
  OPC_CheckField, 10, 3, 4,                  // 13970:        check Inst[12:10] == 0x4
  OPC_Decode, 253, 25, 149, 1,               // 13974:        decode to FDOT_VG2_M2Z2Z_BtoS using decoder 149
                                             // 13974:       }
  1, 0,                                      // 13979:       case 0x1: {
  OPC_CheckPredicate, 8,                     // 13981:        check predicate 8
  OPC_CheckField, 22, 1, 0,                  // 13983:        check Inst[22] == 0x0
  OPC_CheckField, 17, 1, 0,                  // 13987:        check Inst[17] == 0x0
  OPC_CheckField, 10, 3, 4,                  // 13991:        check Inst[12:10] == 0x4
  OPC_CheckField, 6, 1, 0,                   // 13995:        check Inst[6] == 0x0
  OPC_Decode, 134, 26, 150, 1,               // 13999:        decode to FDOT_VG4_M4Z4Z_BtoS using decoder 150
                                             // 13999:       }
                                             // 13999:       } // switch Inst[16]
                                             // 13999:      }
                                             // 13999:      } // switch Inst[5:3]
                                             // 13999:     }
  1, 0,                                      // 14004:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 14006:      switch Inst[14:13] {
  0, 85,                                     // 14009:      case 0x0: {
  OPC_SwitchField, 16, 1,                    // 14011:       switch Inst[16] {
  0, 35,                                     // 14014:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14016:        switch Inst[22] {
  0, 14,                                     // 14019:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 14021:         check predicate 7
  OPC_CheckField, 5, 1, 0,                   // 14023:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14027:         check Inst[0] == 0x0
  OPC_Decode, 215, 47, 111,                  // 14031:         decode to SEL_VG2_2ZC2Z2Z_S using decoder 111
                                             // 14031:        }
  1, 0,                                      // 14035:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 14037:         check predicate 7
  OPC_CheckField, 5, 1, 0,                   // 14039:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14043:         check Inst[0] == 0x0
  OPC_Decode, 213, 47, 111,                  // 14047:         decode to SEL_VG2_2ZC2Z2Z_D using decoder 111
                                             // 14047:        }
                                             // 14047:        } // switch Inst[22]
                                             // 14047:       }
  1, 0,                                      // 14051:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14053:        switch Inst[22] {
  0, 18,                                     // 14056:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 14058:         check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 14060:         check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 14064:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14068:         check Inst[1:0] == 0x0
  OPC_Decode, 219, 47, 112,                  // 14072:         decode to SEL_VG4_4ZC4Z4Z_S using decoder 112
                                             // 14072:        }
  1, 0,                                      // 14076:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 14078:         check predicate 7
  OPC_CheckField, 17, 1, 0,                  // 14080:         check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 14084:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14088:         check Inst[1:0] == 0x0
  OPC_Decode, 217, 47, 112,                  // 14092:         decode to SEL_VG4_4ZC4Z4Z_D using decoder 112
                                             // 14092:        }
                                             // 14092:        } // switch Inst[22]
                                             // 14092:       }
                                             // 14092:       } // switch Inst[16]
                                             // 14092:      }
  1, 239, 13,                                // 14096:      case 0x1: {
  OPC_SwitchField, 5, 8,                     // 14099:       switch Inst[12:5] {
  0, 61,                                     // 14102:       case 0x0: {
  OPC_SwitchField, 0, 1,                     // 14104:        switch Inst[0] {
  0, 27,                                     // 14107:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14109:         switch Inst[22] {
  0, 10,                                     // 14112:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14114:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14116:          check Inst[20] == 0x0
  OPC_Decode, 254, 48, 113,                  // 14120:          decode to SMAX_VG2_2ZZ_S using decoder 113
                                             // 14120:         }
  1, 0,                                      // 14124:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14126:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14128:          check Inst[20] == 0x0
  OPC_Decode, 252, 48, 113,                  // 14132:          decode to SMAX_VG2_2ZZ_D using decoder 113
                                             // 14132:         }
                                             // 14132:         } // switch Inst[22]
                                             // 14132:        }
  1, 0,                                      // 14136:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14138:         switch Inst[22] {
  0, 10,                                     // 14141:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14143:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14145:          check Inst[20] == 0x0
  OPC_Decode, 136, 64, 113,                  // 14149:          decode to UMAX_VG2_2ZZ_S using decoder 113
                                             // 14149:         }
  1, 0,                                      // 14153:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14155:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14157:          check Inst[20] == 0x0
  OPC_Decode, 134, 64, 113,                  // 14161:          decode to UMAX_VG2_2ZZ_D using decoder 113
                                             // 14161:         }
                                             // 14161:         } // switch Inst[22]
                                             // 14161:        }
                                             // 14161:        } // switch Inst[0]
                                             // 14161:       }
  1, 61,                                     // 14165:       case 0x1: {
  OPC_SwitchField, 0, 1,                     // 14167:        switch Inst[0] {
  0, 27,                                     // 14170:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14172:         switch Inst[22] {
  0, 10,                                     // 14175:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14177:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14179:          check Inst[20] == 0x0
  OPC_Decode, 184, 49, 113,                  // 14183:          decode to SMIN_VG2_2ZZ_S using decoder 113
                                             // 14183:         }
  1, 0,                                      // 14187:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14189:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14191:          check Inst[20] == 0x0
  OPC_Decode, 182, 49, 113,                  // 14195:          decode to SMIN_VG2_2ZZ_D using decoder 113
                                             // 14195:         }
                                             // 14195:         } // switch Inst[22]
                                             // 14195:        }
  1, 0,                                      // 14199:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14201:         switch Inst[22] {
  0, 10,                                     // 14204:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14206:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14208:          check Inst[20] == 0x0
  OPC_Decode, 193, 64, 113,                  // 14212:          decode to UMIN_VG2_2ZZ_S using decoder 113
                                             // 14212:         }
  1, 0,                                      // 14216:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14218:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14220:          check Inst[20] == 0x0
  OPC_Decode, 191, 64, 113,                  // 14224:          decode to UMIN_VG2_2ZZ_D using decoder 113
                                             // 14224:         }
                                             // 14224:         } // switch Inst[22]
                                             // 14224:        }
                                             // 14224:        } // switch Inst[0]
                                             // 14224:       }
  8, 61,                                     // 14228:       case 0x8: {
  OPC_SwitchField, 0, 1,                     // 14230:        switch Inst[0] {
  0, 27,                                     // 14233:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14235:         switch Inst[22] {
  0, 10,                                     // 14238:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14240:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14242:          check Inst[20] == 0x0
  OPC_Decode, 129, 27, 113,                  // 14246:          decode to FMAX_VG2_2ZZ_S using decoder 113
                                             // 14246:         }
  1, 0,                                      // 14250:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14252:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14254:          check Inst[20] == 0x0
  OPC_Decode, 255, 26, 113,                  // 14258:          decode to FMAX_VG2_2ZZ_D using decoder 113
                                             // 14258:         }
                                             // 14258:         } // switch Inst[22]
                                             // 14258:        }
  1, 0,                                      // 14262:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14264:         switch Inst[22] {
  0, 10,                                     // 14267:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14269:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14271:          check Inst[20] == 0x0
  OPC_Decode, 221, 27, 113,                  // 14275:          decode to FMIN_VG2_2ZZ_S using decoder 113
                                             // 14275:         }
  1, 0,                                      // 14279:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14281:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14283:          check Inst[20] == 0x0
  OPC_Decode, 219, 27, 113,                  // 14287:          decode to FMIN_VG2_2ZZ_D using decoder 113
                                             // 14287:         }
                                             // 14287:         } // switch Inst[22]
                                             // 14287:        }
                                             // 14287:        } // switch Inst[0]
                                             // 14287:       }
  9, 61,                                     // 14291:       case 0x9: {
  OPC_SwitchField, 0, 1,                     // 14293:        switch Inst[0] {
  0, 27,                                     // 14296:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14298:         switch Inst[22] {
  0, 10,                                     // 14301:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14303:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14305:          check Inst[20] == 0x0
  OPC_Decode, 213, 26, 113,                  // 14309:          decode to FMAXNM_VG2_2ZZ_S using decoder 113
                                             // 14309:         }
  1, 0,                                      // 14313:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14315:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14317:          check Inst[20] == 0x0
  OPC_Decode, 211, 26, 113,                  // 14321:          decode to FMAXNM_VG2_2ZZ_D using decoder 113
                                             // 14321:         }
                                             // 14321:         } // switch Inst[22]
                                             // 14321:        }
  1, 0,                                      // 14325:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14327:         switch Inst[22] {
  0, 10,                                     // 14330:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14332:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14334:          check Inst[20] == 0x0
  OPC_Decode, 177, 27, 113,                  // 14338:          decode to FMINNM_VG2_2ZZ_S using decoder 113
                                             // 14338:         }
  1, 0,                                      // 14342:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14344:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14346:          check Inst[20] == 0x0
  OPC_Decode, 175, 27, 113,                  // 14350:          decode to FMINNM_VG2_2ZZ_D using decoder 113
                                             // 14350:         }
                                             // 14350:         } // switch Inst[22]
                                             // 14350:        }
                                             // 14350:        } // switch Inst[0]
                                             // 14350:       }
  12, 35,                                    // 14354:       case 0xc: {
  OPC_SwitchField, 22, 1,                    // 14356:        switch Inst[22] {
  0, 14,                                     // 14359:        case 0x0: {
  OPC_CheckPredicate, 29,                    // 14361:         check predicate 29
  OPC_CheckField, 20, 1, 0,                  // 14363:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14367:         check Inst[0] == 0x0
  OPC_Decode, 134, 32, 113,                  // 14371:         decode to FSCALE_2ZZ_S using decoder 113
                                             // 14371:        }
  1, 0,                                      // 14375:        case 0x1: {
  OPC_CheckPredicate, 29,                    // 14377:         check predicate 29
  OPC_CheckField, 20, 1, 0,                  // 14379:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14383:         check Inst[0] == 0x0
  OPC_Decode, 132, 32, 113,                  // 14387:         decode to FSCALE_2ZZ_D using decoder 113
                                             // 14387:        }
                                             // 14387:        } // switch Inst[22]
                                             // 14387:       }
  17, 61,                                    // 14391:       case 0x11: {
  OPC_SwitchField, 0, 1,                     // 14393:        switch Inst[0] {
  0, 27,                                     // 14396:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14398:         switch Inst[22] {
  0, 10,                                     // 14401:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14403:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14405:          check Inst[20] == 0x0
  OPC_Decode, 134, 55, 113,                  // 14409:          decode to SRSHL_VG2_2ZZ_S using decoder 113
                                             // 14409:         }
  1, 0,                                      // 14413:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14415:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14417:          check Inst[20] == 0x0
  OPC_Decode, 132, 55, 113,                  // 14421:          decode to SRSHL_VG2_2ZZ_D using decoder 113
                                             // 14421:         }
                                             // 14421:         } // switch Inst[22]
                                             // 14421:        }
  1, 0,                                      // 14425:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14427:         switch Inst[22] {
  0, 10,                                     // 14430:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14432:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14434:          check Inst[20] == 0x0
  OPC_Decode, 222, 67, 113,                  // 14438:          decode to URSHL_VG2_2ZZ_S using decoder 113
                                             // 14438:         }
  1, 0,                                      // 14442:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14444:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14446:          check Inst[20] == 0x0
  OPC_Decode, 220, 67, 113,                  // 14450:          decode to URSHL_VG2_2ZZ_D using decoder 113
                                             // 14450:         }
                                             // 14450:         } // switch Inst[22]
                                             // 14450:        }
                                             // 14450:        } // switch Inst[0]
                                             // 14450:       }
  24, 35,                                    // 14454:       case 0x18: {
  OPC_SwitchField, 22, 1,                    // 14456:        switch Inst[22] {
  0, 14,                                     // 14459:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 14461:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14463:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14467:         check Inst[0] == 0x0
  OPC_Decode, 202, 13, 113,                  // 14471:         decode to ADD_VG2_2ZZ_S using decoder 113
                                             // 14471:        }
  1, 0,                                      // 14475:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 14477:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14479:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14483:         check Inst[0] == 0x0
  OPC_Decode, 200, 13, 113,                  // 14487:         decode to ADD_VG2_2ZZ_D using decoder 113
                                             // 14487:        }
                                             // 14487:        } // switch Inst[22]
                                             // 14487:       }
  32, 35,                                    // 14491:       case 0x20: {
  OPC_SwitchField, 22, 1,                    // 14493:        switch Inst[22] {
  0, 14,                                     // 14496:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 14498:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14500:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14504:         check Inst[0] == 0x0
  OPC_Decode, 128, 52, 113,                  // 14508:         decode to SQDMULH_VG2_2ZZ_S using decoder 113
                                             // 14508:        }
  1, 0,                                      // 14512:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 14514:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14516:         check Inst[20] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 14520:         check Inst[0] == 0x0
  OPC_Decode, 254, 51, 113,                  // 14524:         decode to SQDMULH_VG2_2ZZ_D using decoder 113
                                             // 14524:        }
                                             // 14524:        } // switch Inst[22]
                                             // 14524:       }
  64, 61,                                    // 14528:       case 0x40: {
  OPC_SwitchField, 0, 2,                     // 14530:        switch Inst[1:0] {
  0, 27,                                     // 14533:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14535:         switch Inst[22] {
  0, 10,                                     // 14538:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14540:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14542:          check Inst[20] == 0x0
  OPC_Decode, 134, 49, 114,                  // 14546:          decode to SMAX_VG4_4ZZ_S using decoder 114
                                             // 14546:         }
  1, 0,                                      // 14550:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14552:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14554:          check Inst[20] == 0x0
  OPC_Decode, 132, 49, 114,                  // 14558:          decode to SMAX_VG4_4ZZ_D using decoder 114
                                             // 14558:         }
                                             // 14558:         } // switch Inst[22]
                                             // 14558:        }
  1, 0,                                      // 14562:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14564:         switch Inst[22] {
  0, 10,                                     // 14567:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14569:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14571:          check Inst[20] == 0x0
  OPC_Decode, 144, 64, 114,                  // 14575:          decode to UMAX_VG4_4ZZ_S using decoder 114
                                             // 14575:         }
  1, 0,                                      // 14579:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14581:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14583:          check Inst[20] == 0x0
  OPC_Decode, 142, 64, 114,                  // 14587:          decode to UMAX_VG4_4ZZ_D using decoder 114
                                             // 14587:         }
                                             // 14587:         } // switch Inst[22]
                                             // 14587:        }
                                             // 14587:        } // switch Inst[1:0]
                                             // 14587:       }
  65, 61,                                    // 14591:       case 0x41: {
  OPC_SwitchField, 0, 2,                     // 14593:        switch Inst[1:0] {
  0, 27,                                     // 14596:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14598:         switch Inst[22] {
  0, 10,                                     // 14601:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14603:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14605:          check Inst[20] == 0x0
  OPC_Decode, 192, 49, 114,                  // 14609:          decode to SMIN_VG4_4ZZ_S using decoder 114
                                             // 14609:         }
  1, 0,                                      // 14613:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14615:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14617:          check Inst[20] == 0x0
  OPC_Decode, 190, 49, 114,                  // 14621:          decode to SMIN_VG4_4ZZ_D using decoder 114
                                             // 14621:         }
                                             // 14621:         } // switch Inst[22]
                                             // 14621:        }
  1, 0,                                      // 14625:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14627:         switch Inst[22] {
  0, 10,                                     // 14630:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14632:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14634:          check Inst[20] == 0x0
  OPC_Decode, 201, 64, 114,                  // 14638:          decode to UMIN_VG4_4ZZ_S using decoder 114
                                             // 14638:         }
  1, 0,                                      // 14642:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14644:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14646:          check Inst[20] == 0x0
  OPC_Decode, 199, 64, 114,                  // 14650:          decode to UMIN_VG4_4ZZ_D using decoder 114
                                             // 14650:         }
                                             // 14650:         } // switch Inst[22]
                                             // 14650:        }
                                             // 14650:        } // switch Inst[1:0]
                                             // 14650:       }
  72, 61,                                    // 14654:       case 0x48: {
  OPC_SwitchField, 0, 2,                     // 14656:        switch Inst[1:0] {
  0, 27,                                     // 14659:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14661:         switch Inst[22] {
  0, 10,                                     // 14664:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14666:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14668:          check Inst[20] == 0x0
  OPC_Decode, 135, 27, 114,                  // 14672:          decode to FMAX_VG4_4ZZ_S using decoder 114
                                             // 14672:         }
  1, 0,                                      // 14676:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14678:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14680:          check Inst[20] == 0x0
  OPC_Decode, 133, 27, 114,                  // 14684:          decode to FMAX_VG4_4ZZ_D using decoder 114
                                             // 14684:         }
                                             // 14684:         } // switch Inst[22]
                                             // 14684:        }
  1, 0,                                      // 14688:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14690:         switch Inst[22] {
  0, 10,                                     // 14693:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14695:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14697:          check Inst[20] == 0x0
  OPC_Decode, 227, 27, 114,                  // 14701:          decode to FMIN_VG4_4ZZ_S using decoder 114
                                             // 14701:         }
  1, 0,                                      // 14705:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14707:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14709:          check Inst[20] == 0x0
  OPC_Decode, 225, 27, 114,                  // 14713:          decode to FMIN_VG4_4ZZ_D using decoder 114
                                             // 14713:         }
                                             // 14713:         } // switch Inst[22]
                                             // 14713:        }
                                             // 14713:        } // switch Inst[1:0]
                                             // 14713:       }
  73, 61,                                    // 14717:       case 0x49: {
  OPC_SwitchField, 0, 2,                     // 14719:        switch Inst[1:0] {
  0, 27,                                     // 14722:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14724:         switch Inst[22] {
  0, 10,                                     // 14727:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14729:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14731:          check Inst[20] == 0x0
  OPC_Decode, 219, 26, 114,                  // 14735:          decode to FMAXNM_VG4_4ZZ_S using decoder 114
                                             // 14735:         }
  1, 0,                                      // 14739:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14741:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14743:          check Inst[20] == 0x0
  OPC_Decode, 217, 26, 114,                  // 14747:          decode to FMAXNM_VG4_4ZZ_D using decoder 114
                                             // 14747:         }
                                             // 14747:         } // switch Inst[22]
                                             // 14747:        }
  1, 0,                                      // 14751:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14753:         switch Inst[22] {
  0, 10,                                     // 14756:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14758:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14760:          check Inst[20] == 0x0
  OPC_Decode, 183, 27, 114,                  // 14764:          decode to FMINNM_VG4_4ZZ_S using decoder 114
                                             // 14764:         }
  1, 0,                                      // 14768:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14770:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14772:          check Inst[20] == 0x0
  OPC_Decode, 181, 27, 114,                  // 14776:          decode to FMINNM_VG4_4ZZ_D using decoder 114
                                             // 14776:         }
                                             // 14776:         } // switch Inst[22]
                                             // 14776:        }
                                             // 14776:        } // switch Inst[1:0]
                                             // 14776:       }
  76, 35,                                    // 14780:       case 0x4c: {
  OPC_SwitchField, 22, 1,                    // 14782:        switch Inst[22] {
  0, 14,                                     // 14785:        case 0x0: {
  OPC_CheckPredicate, 29,                    // 14787:         check predicate 29
  OPC_CheckField, 20, 1, 0,                  // 14789:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14793:         check Inst[1:0] == 0x0
  OPC_Decode, 140, 32, 114,                  // 14797:         decode to FSCALE_4ZZ_S using decoder 114
                                             // 14797:        }
  1, 0,                                      // 14801:        case 0x1: {
  OPC_CheckPredicate, 29,                    // 14803:         check predicate 29
  OPC_CheckField, 20, 1, 0,                  // 14805:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14809:         check Inst[1:0] == 0x0
  OPC_Decode, 138, 32, 114,                  // 14813:         decode to FSCALE_4ZZ_D using decoder 114
                                             // 14813:        }
                                             // 14813:        } // switch Inst[22]
                                             // 14813:       }
  81, 61,                                    // 14817:       case 0x51: {
  OPC_SwitchField, 0, 2,                     // 14819:        switch Inst[1:0] {
  0, 27,                                     // 14822:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14824:         switch Inst[22] {
  0, 10,                                     // 14827:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14829:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14831:          check Inst[20] == 0x0
  OPC_Decode, 142, 55, 114,                  // 14835:          decode to SRSHL_VG4_4ZZ_S using decoder 114
                                             // 14835:         }
  1, 0,                                      // 14839:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14841:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14843:          check Inst[20] == 0x0
  OPC_Decode, 140, 55, 114,                  // 14847:          decode to SRSHL_VG4_4ZZ_D using decoder 114
                                             // 14847:         }
                                             // 14847:         } // switch Inst[22]
                                             // 14847:        }
  1, 0,                                      // 14851:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14853:         switch Inst[22] {
  0, 10,                                     // 14856:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14858:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14860:          check Inst[20] == 0x0
  OPC_Decode, 230, 67, 114,                  // 14864:          decode to URSHL_VG4_4ZZ_S using decoder 114
                                             // 14864:         }
  1, 0,                                      // 14868:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14870:          check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14872:          check Inst[20] == 0x0
  OPC_Decode, 228, 67, 114,                  // 14876:          decode to URSHL_VG4_4ZZ_D using decoder 114
                                             // 14876:         }
                                             // 14876:         } // switch Inst[22]
                                             // 14876:        }
                                             // 14876:        } // switch Inst[1:0]
                                             // 14876:       }
  88, 35,                                    // 14880:       case 0x58: {
  OPC_SwitchField, 22, 1,                    // 14882:        switch Inst[22] {
  0, 14,                                     // 14885:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 14887:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14889:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14893:         check Inst[1:0] == 0x0
  OPC_Decode, 212, 13, 114,                  // 14897:         decode to ADD_VG4_4ZZ_S using decoder 114
                                             // 14897:        }
  1, 0,                                      // 14901:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 14903:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14905:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14909:         check Inst[1:0] == 0x0
  OPC_Decode, 210, 13, 114,                  // 14913:         decode to ADD_VG4_4ZZ_D using decoder 114
                                             // 14913:        }
                                             // 14913:        } // switch Inst[22]
                                             // 14913:       }
  96, 35,                                    // 14917:       case 0x60: {
  OPC_SwitchField, 22, 1,                    // 14919:        switch Inst[22] {
  0, 14,                                     // 14922:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 14924:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14926:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14930:         check Inst[1:0] == 0x0
  OPC_Decode, 136, 52, 114,                  // 14934:         decode to SQDMULH_VG4_4ZZ_S using decoder 114
                                             // 14934:        }
  1, 0,                                      // 14938:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 14940:         check predicate 7
  OPC_CheckField, 20, 1, 0,                  // 14942:         check Inst[20] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 14946:         check Inst[1:0] == 0x0
  OPC_Decode, 134, 52, 114,                  // 14950:         decode to SQDMULH_VG4_4ZZ_D using decoder 114
                                             // 14950:        }
                                             // 14950:        } // switch Inst[22]
                                             // 14950:       }
  128, 1, 61,                                // 14954:       case 0x80: {
  OPC_SwitchField, 0, 1,                     // 14957:        switch Inst[0] {
  0, 27,                                     // 14960:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 14962:         switch Inst[22] {
  0, 10,                                     // 14965:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14967:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 14969:          check Inst[16] == 0x0
  OPC_Decode, 250, 48, 115,                  // 14973:          decode to SMAX_VG2_2Z2Z_S using decoder 115
                                             // 14973:         }
  1, 0,                                      // 14977:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 14979:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 14981:          check Inst[16] == 0x0
  OPC_Decode, 248, 48, 115,                  // 14985:          decode to SMAX_VG2_2Z2Z_D using decoder 115
                                             // 14985:         }
                                             // 14985:         } // switch Inst[22]
                                             // 14985:        }
  1, 0,                                      // 14989:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 14991:         switch Inst[22] {
  0, 10,                                     // 14994:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 14996:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 14998:          check Inst[16] == 0x0
  OPC_Decode, 132, 64, 115,                  // 15002:          decode to UMAX_VG2_2Z2Z_S using decoder 115
                                             // 15002:         }
  1, 0,                                      // 15006:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15008:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15010:          check Inst[16] == 0x0
  OPC_Decode, 130, 64, 115,                  // 15014:          decode to UMAX_VG2_2Z2Z_D using decoder 115
                                             // 15014:         }
                                             // 15014:         } // switch Inst[22]
                                             // 15014:        }
                                             // 15014:        } // switch Inst[0]
                                             // 15014:       }
  129, 1, 61,                                // 15018:       case 0x81: {
  OPC_SwitchField, 0, 1,                     // 15021:        switch Inst[0] {
  0, 27,                                     // 15024:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15026:         switch Inst[22] {
  0, 10,                                     // 15029:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15031:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15033:          check Inst[16] == 0x0
  OPC_Decode, 180, 49, 115,                  // 15037:          decode to SMIN_VG2_2Z2Z_S using decoder 115
                                             // 15037:         }
  1, 0,                                      // 15041:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15043:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15045:          check Inst[16] == 0x0
  OPC_Decode, 178, 49, 115,                  // 15049:          decode to SMIN_VG2_2Z2Z_D using decoder 115
                                             // 15049:         }
                                             // 15049:         } // switch Inst[22]
                                             // 15049:        }
  1, 0,                                      // 15053:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15055:         switch Inst[22] {
  0, 10,                                     // 15058:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15060:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15062:          check Inst[16] == 0x0
  OPC_Decode, 189, 64, 115,                  // 15066:          decode to UMIN_VG2_2Z2Z_S using decoder 115
                                             // 15066:         }
  1, 0,                                      // 15070:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15072:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15074:          check Inst[16] == 0x0
  OPC_Decode, 187, 64, 115,                  // 15078:          decode to UMIN_VG2_2Z2Z_D using decoder 115
                                             // 15078:         }
                                             // 15078:         } // switch Inst[22]
                                             // 15078:        }
                                             // 15078:        } // switch Inst[0]
                                             // 15078:       }
  136, 1, 61,                                // 15082:       case 0x88: {
  OPC_SwitchField, 0, 1,                     // 15085:        switch Inst[0] {
  0, 27,                                     // 15088:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15090:         switch Inst[22] {
  0, 10,                                     // 15093:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15095:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15097:          check Inst[16] == 0x0
  OPC_Decode, 254, 26, 115,                  // 15101:          decode to FMAX_VG2_2Z2Z_S using decoder 115
                                             // 15101:         }
  1, 0,                                      // 15105:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15107:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15109:          check Inst[16] == 0x0
  OPC_Decode, 252, 26, 115,                  // 15113:          decode to FMAX_VG2_2Z2Z_D using decoder 115
                                             // 15113:         }
                                             // 15113:         } // switch Inst[22]
                                             // 15113:        }
  1, 0,                                      // 15117:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15119:         switch Inst[22] {
  0, 10,                                     // 15122:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15124:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15126:          check Inst[16] == 0x0
  OPC_Decode, 218, 27, 115,                  // 15130:          decode to FMIN_VG2_2Z2Z_S using decoder 115
                                             // 15130:         }
  1, 0,                                      // 15134:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15136:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15138:          check Inst[16] == 0x0
  OPC_Decode, 216, 27, 115,                  // 15142:          decode to FMIN_VG2_2Z2Z_D using decoder 115
                                             // 15142:         }
                                             // 15142:         } // switch Inst[22]
                                             // 15142:        }
                                             // 15142:        } // switch Inst[0]
                                             // 15142:       }
  137, 1, 61,                                // 15146:       case 0x89: {
  OPC_SwitchField, 0, 1,                     // 15149:        switch Inst[0] {
  0, 27,                                     // 15152:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15154:         switch Inst[22] {
  0, 10,                                     // 15157:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15159:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15161:          check Inst[16] == 0x0
  OPC_Decode, 210, 26, 115,                  // 15165:          decode to FMAXNM_VG2_2Z2Z_S using decoder 115
                                             // 15165:         }
  1, 0,                                      // 15169:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15171:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15173:          check Inst[16] == 0x0
  OPC_Decode, 208, 26, 115,                  // 15177:          decode to FMAXNM_VG2_2Z2Z_D using decoder 115
                                             // 15177:         }
                                             // 15177:         } // switch Inst[22]
                                             // 15177:        }
  1, 0,                                      // 15181:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15183:         switch Inst[22] {
  0, 10,                                     // 15186:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15188:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15190:          check Inst[16] == 0x0
  OPC_Decode, 174, 27, 115,                  // 15194:          decode to FMINNM_VG2_2Z2Z_S using decoder 115
                                             // 15194:         }
  1, 0,                                      // 15198:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15200:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15202:          check Inst[16] == 0x0
  OPC_Decode, 172, 27, 115,                  // 15206:          decode to FMINNM_VG2_2Z2Z_D using decoder 115
                                             // 15206:         }
                                             // 15206:         } // switch Inst[22]
                                             // 15206:        }
                                             // 15206:        } // switch Inst[0]
                                             // 15206:       }
  138, 1, 61,                                // 15210:       case 0x8a: {
  OPC_SwitchField, 0, 1,                     // 15213:        switch Inst[0] {
  0, 27,                                     // 15216:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15218:         switch Inst[22] {
  0, 10,                                     // 15221:         case 0x0: {
  OPC_CheckPredicate, 30,                    // 15223:          check predicate 30
  OPC_CheckField, 16, 1, 0,                  // 15225:          check Inst[16] == 0x0
  OPC_Decode, 144, 22, 115,                  // 15229:          decode to FAMAX_2Z2Z_S using decoder 115
                                             // 15229:         }
  1, 0,                                      // 15233:         case 0x1: {
  OPC_CheckPredicate, 30,                    // 15235:          check predicate 30
  OPC_CheckField, 16, 1, 0,                  // 15237:          check Inst[16] == 0x0
  OPC_Decode, 142, 22, 115,                  // 15241:          decode to FAMAX_2Z2Z_D using decoder 115
                                             // 15241:         }
                                             // 15241:         } // switch Inst[22]
                                             // 15241:        }
  1, 0,                                      // 15245:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15247:         switch Inst[22] {
  0, 10,                                     // 15250:         case 0x0: {
  OPC_CheckPredicate, 30,                    // 15252:          check predicate 30
  OPC_CheckField, 16, 1, 0,                  // 15254:          check Inst[16] == 0x0
  OPC_Decode, 158, 22, 115,                  // 15258:          decode to FAMIN_2Z2Z_S using decoder 115
                                             // 15258:         }
  1, 0,                                      // 15262:         case 0x1: {
  OPC_CheckPredicate, 30,                    // 15264:          check predicate 30
  OPC_CheckField, 16, 1, 0,                  // 15266:          check Inst[16] == 0x0
  OPC_Decode, 156, 22, 115,                  // 15270:          decode to FAMIN_2Z2Z_D using decoder 115
                                             // 15270:         }
                                             // 15270:         } // switch Inst[22]
                                             // 15270:        }
                                             // 15270:        } // switch Inst[0]
                                             // 15270:       }
  140, 1, 35,                                // 15274:       case 0x8c: {
  OPC_SwitchField, 22, 1,                    // 15277:        switch Inst[22] {
  0, 14,                                     // 15280:        case 0x0: {
  OPC_CheckPredicate, 29,                    // 15282:         check predicate 29
  OPC_CheckField, 16, 1, 0,                  // 15284:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 15288:         check Inst[0] == 0x0
  OPC_Decode, 131, 32, 115,                  // 15292:         decode to FSCALE_2Z2Z_S using decoder 115
                                             // 15292:        }
  1, 0,                                      // 15296:        case 0x1: {
  OPC_CheckPredicate, 29,                    // 15298:         check predicate 29
  OPC_CheckField, 16, 1, 0,                  // 15300:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 15304:         check Inst[0] == 0x0
  OPC_Decode, 129, 32, 115,                  // 15308:         decode to FSCALE_2Z2Z_D using decoder 115
                                             // 15308:        }
                                             // 15308:        } // switch Inst[22]
                                             // 15308:       }
  145, 1, 61,                                // 15312:       case 0x91: {
  OPC_SwitchField, 0, 1,                     // 15315:        switch Inst[0] {
  0, 27,                                     // 15318:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15320:         switch Inst[22] {
  0, 10,                                     // 15323:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15325:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15327:          check Inst[16] == 0x0
  OPC_Decode, 130, 55, 115,                  // 15331:          decode to SRSHL_VG2_2Z2Z_S using decoder 115
                                             // 15331:         }
  1, 0,                                      // 15335:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15337:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15339:          check Inst[16] == 0x0
  OPC_Decode, 128, 55, 115,                  // 15343:          decode to SRSHL_VG2_2Z2Z_D using decoder 115
                                             // 15343:         }
                                             // 15343:         } // switch Inst[22]
                                             // 15343:        }
  1, 0,                                      // 15347:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15349:         switch Inst[22] {
  0, 10,                                     // 15352:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15354:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15356:          check Inst[16] == 0x0
  OPC_Decode, 218, 67, 115,                  // 15360:          decode to URSHL_VG2_2Z2Z_S using decoder 115
                                             // 15360:         }
  1, 0,                                      // 15364:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15366:          check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15368:          check Inst[16] == 0x0
  OPC_Decode, 216, 67, 115,                  // 15372:          decode to URSHL_VG2_2Z2Z_D using decoder 115
                                             // 15372:         }
                                             // 15372:         } // switch Inst[22]
                                             // 15372:        }
                                             // 15372:        } // switch Inst[0]
                                             // 15372:       }
  160, 1, 35,                                // 15376:       case 0xa0: {
  OPC_SwitchField, 22, 1,                    // 15379:        switch Inst[22] {
  0, 14,                                     // 15382:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 15384:         check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15386:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 15390:         check Inst[0] == 0x0
  OPC_Decode, 252, 51, 115,                  // 15394:         decode to SQDMULH_VG2_2Z2Z_S using decoder 115
                                             // 15394:        }
  1, 0,                                      // 15398:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 15400:         check predicate 7
  OPC_CheckField, 16, 1, 0,                  // 15402:         check Inst[16] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 15406:         check Inst[0] == 0x0
  OPC_Decode, 250, 51, 115,                  // 15410:         decode to SQDMULH_VG2_2Z2Z_D using decoder 115
                                             // 15410:        }
                                             // 15410:        } // switch Inst[22]
                                             // 15410:       }
  192, 1, 61,                                // 15414:       case 0xc0: {
  OPC_SwitchField, 0, 2,                     // 15417:        switch Inst[1:0] {
  0, 27,                                     // 15420:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15422:         switch Inst[22] {
  0, 10,                                     // 15425:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15427:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15429:          check Inst[17:16] == 0x0
  OPC_Decode, 130, 49, 116,                  // 15433:          decode to SMAX_VG4_4Z4Z_S using decoder 116
                                             // 15433:         }
  1, 0,                                      // 15437:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15439:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15441:          check Inst[17:16] == 0x0
  OPC_Decode, 128, 49, 116,                  // 15445:          decode to SMAX_VG4_4Z4Z_D using decoder 116
                                             // 15445:         }
                                             // 15445:         } // switch Inst[22]
                                             // 15445:        }
  1, 0,                                      // 15449:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15451:         switch Inst[22] {
  0, 10,                                     // 15454:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15456:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15458:          check Inst[17:16] == 0x0
  OPC_Decode, 140, 64, 116,                  // 15462:          decode to UMAX_VG4_4Z4Z_S using decoder 116
                                             // 15462:         }
  1, 0,                                      // 15466:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15468:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15470:          check Inst[17:16] == 0x0
  OPC_Decode, 138, 64, 116,                  // 15474:          decode to UMAX_VG4_4Z4Z_D using decoder 116
                                             // 15474:         }
                                             // 15474:         } // switch Inst[22]
                                             // 15474:        }
                                             // 15474:        } // switch Inst[1:0]
                                             // 15474:       }
  193, 1, 61,                                // 15478:       case 0xc1: {
  OPC_SwitchField, 0, 2,                     // 15481:        switch Inst[1:0] {
  0, 27,                                     // 15484:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15486:         switch Inst[22] {
  0, 10,                                     // 15489:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15491:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15493:          check Inst[17:16] == 0x0
  OPC_Decode, 188, 49, 116,                  // 15497:          decode to SMIN_VG4_4Z4Z_S using decoder 116
                                             // 15497:         }
  1, 0,                                      // 15501:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15503:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15505:          check Inst[17:16] == 0x0
  OPC_Decode, 186, 49, 116,                  // 15509:          decode to SMIN_VG4_4Z4Z_D using decoder 116
                                             // 15509:         }
                                             // 15509:         } // switch Inst[22]
                                             // 15509:        }
  1, 0,                                      // 15513:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15515:         switch Inst[22] {
  0, 10,                                     // 15518:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15520:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15522:          check Inst[17:16] == 0x0
  OPC_Decode, 197, 64, 116,                  // 15526:          decode to UMIN_VG4_4Z4Z_S using decoder 116
                                             // 15526:         }
  1, 0,                                      // 15530:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15532:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15534:          check Inst[17:16] == 0x0
  OPC_Decode, 195, 64, 116,                  // 15538:          decode to UMIN_VG4_4Z4Z_D using decoder 116
                                             // 15538:         }
                                             // 15538:         } // switch Inst[22]
                                             // 15538:        }
                                             // 15538:        } // switch Inst[1:0]
                                             // 15538:       }
  200, 1, 61,                                // 15542:       case 0xc8: {
  OPC_SwitchField, 0, 2,                     // 15545:        switch Inst[1:0] {
  0, 27,                                     // 15548:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15550:         switch Inst[22] {
  0, 10,                                     // 15553:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15555:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15557:          check Inst[17:16] == 0x0
  OPC_Decode, 132, 27, 116,                  // 15561:          decode to FMAX_VG4_4Z4Z_S using decoder 116
                                             // 15561:         }
  1, 0,                                      // 15565:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15567:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15569:          check Inst[17:16] == 0x0
  OPC_Decode, 130, 27, 116,                  // 15573:          decode to FMAX_VG4_4Z4Z_D using decoder 116
                                             // 15573:         }
                                             // 15573:         } // switch Inst[22]
                                             // 15573:        }
  1, 0,                                      // 15577:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15579:         switch Inst[22] {
  0, 10,                                     // 15582:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15584:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15586:          check Inst[17:16] == 0x0
  OPC_Decode, 224, 27, 116,                  // 15590:          decode to FMIN_VG4_4Z4Z_S using decoder 116
                                             // 15590:         }
  1, 0,                                      // 15594:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15596:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15598:          check Inst[17:16] == 0x0
  OPC_Decode, 222, 27, 116,                  // 15602:          decode to FMIN_VG4_4Z4Z_D using decoder 116
                                             // 15602:         }
                                             // 15602:         } // switch Inst[22]
                                             // 15602:        }
                                             // 15602:        } // switch Inst[1:0]
                                             // 15602:       }
  201, 1, 61,                                // 15606:       case 0xc9: {
  OPC_SwitchField, 0, 2,                     // 15609:        switch Inst[1:0] {
  0, 27,                                     // 15612:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15614:         switch Inst[22] {
  0, 10,                                     // 15617:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15619:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15621:          check Inst[17:16] == 0x0
  OPC_Decode, 216, 26, 116,                  // 15625:          decode to FMAXNM_VG4_4Z4Z_S using decoder 116
                                             // 15625:         }
  1, 0,                                      // 15629:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15631:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15633:          check Inst[17:16] == 0x0
  OPC_Decode, 214, 26, 116,                  // 15637:          decode to FMAXNM_VG4_4Z4Z_D using decoder 116
                                             // 15637:         }
                                             // 15637:         } // switch Inst[22]
                                             // 15637:        }
  1, 0,                                      // 15641:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15643:         switch Inst[22] {
  0, 10,                                     // 15646:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15648:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15650:          check Inst[17:16] == 0x0
  OPC_Decode, 180, 27, 116,                  // 15654:          decode to FMINNM_VG4_4Z4Z_S using decoder 116
                                             // 15654:         }
  1, 0,                                      // 15658:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15660:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15662:          check Inst[17:16] == 0x0
  OPC_Decode, 178, 27, 116,                  // 15666:          decode to FMINNM_VG4_4Z4Z_D using decoder 116
                                             // 15666:         }
                                             // 15666:         } // switch Inst[22]
                                             // 15666:        }
                                             // 15666:        } // switch Inst[1:0]
                                             // 15666:       }
  202, 1, 61,                                // 15670:       case 0xca: {
  OPC_SwitchField, 0, 2,                     // 15673:        switch Inst[1:0] {
  0, 27,                                     // 15676:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15678:         switch Inst[22] {
  0, 10,                                     // 15681:         case 0x0: {
  OPC_CheckPredicate, 30,                    // 15683:          check predicate 30
  OPC_CheckField, 16, 2, 0,                  // 15685:          check Inst[17:16] == 0x0
  OPC_Decode, 147, 22, 116,                  // 15689:          decode to FAMAX_4Z4Z_S using decoder 116
                                             // 15689:         }
  1, 0,                                      // 15693:         case 0x1: {
  OPC_CheckPredicate, 30,                    // 15695:          check predicate 30
  OPC_CheckField, 16, 2, 0,                  // 15697:          check Inst[17:16] == 0x0
  OPC_Decode, 145, 22, 116,                  // 15701:          decode to FAMAX_4Z4Z_D using decoder 116
                                             // 15701:         }
                                             // 15701:         } // switch Inst[22]
                                             // 15701:        }
  1, 0,                                      // 15705:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15707:         switch Inst[22] {
  0, 10,                                     // 15710:         case 0x0: {
  OPC_CheckPredicate, 30,                    // 15712:          check predicate 30
  OPC_CheckField, 16, 2, 0,                  // 15714:          check Inst[17:16] == 0x0
  OPC_Decode, 161, 22, 116,                  // 15718:          decode to FAMIN_4Z4Z_S using decoder 116
                                             // 15718:         }
  1, 0,                                      // 15722:         case 0x1: {
  OPC_CheckPredicate, 30,                    // 15724:          check predicate 30
  OPC_CheckField, 16, 2, 0,                  // 15726:          check Inst[17:16] == 0x0
  OPC_Decode, 159, 22, 116,                  // 15730:          decode to FAMIN_4Z4Z_D using decoder 116
                                             // 15730:         }
                                             // 15730:         } // switch Inst[22]
                                             // 15730:        }
                                             // 15730:        } // switch Inst[1:0]
                                             // 15730:       }
  204, 1, 35,                                // 15734:       case 0xcc: {
  OPC_SwitchField, 22, 1,                    // 15737:        switch Inst[22] {
  0, 14,                                     // 15740:        case 0x0: {
  OPC_CheckPredicate, 29,                    // 15742:         check predicate 29
  OPC_CheckField, 16, 2, 0,                  // 15744:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 15748:         check Inst[1:0] == 0x0
  OPC_Decode, 137, 32, 116,                  // 15752:         decode to FSCALE_4Z4Z_S using decoder 116
                                             // 15752:        }
  1, 0,                                      // 15756:        case 0x1: {
  OPC_CheckPredicate, 29,                    // 15758:         check predicate 29
  OPC_CheckField, 16, 2, 0,                  // 15760:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 15764:         check Inst[1:0] == 0x0
  OPC_Decode, 135, 32, 116,                  // 15768:         decode to FSCALE_4Z4Z_D using decoder 116
                                             // 15768:        }
                                             // 15768:        } // switch Inst[22]
                                             // 15768:       }
  209, 1, 61,                                // 15772:       case 0xd1: {
  OPC_SwitchField, 0, 2,                     // 15775:        switch Inst[1:0] {
  0, 27,                                     // 15778:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15780:         switch Inst[22] {
  0, 10,                                     // 15783:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15785:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15787:          check Inst[17:16] == 0x0
  OPC_Decode, 138, 55, 116,                  // 15791:          decode to SRSHL_VG4_4Z4Z_S using decoder 116
                                             // 15791:         }
  1, 0,                                      // 15795:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15797:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15799:          check Inst[17:16] == 0x0
  OPC_Decode, 136, 55, 116,                  // 15803:          decode to SRSHL_VG4_4Z4Z_D using decoder 116
                                             // 15803:         }
                                             // 15803:         } // switch Inst[22]
                                             // 15803:        }
  1, 0,                                      // 15807:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15809:         switch Inst[22] {
  0, 10,                                     // 15812:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15814:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15816:          check Inst[17:16] == 0x0
  OPC_Decode, 226, 67, 116,                  // 15820:          decode to URSHL_VG4_4Z4Z_S using decoder 116
                                             // 15820:         }
  1, 0,                                      // 15824:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15826:          check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15828:          check Inst[17:16] == 0x0
  OPC_Decode, 224, 67, 116,                  // 15832:          decode to URSHL_VG4_4Z4Z_D using decoder 116
                                             // 15832:         }
                                             // 15832:         } // switch Inst[22]
                                             // 15832:        }
                                             // 15832:        } // switch Inst[1:0]
                                             // 15832:       }
  224, 1, 0,                                 // 15836:       case 0xe0: {
  OPC_SwitchField, 22, 1,                    // 15839:        switch Inst[22] {
  0, 14,                                     // 15842:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 15844:         check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15846:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 15850:         check Inst[1:0] == 0x0
  OPC_Decode, 132, 52, 116,                  // 15854:         decode to SQDMULH_VG4_4Z4Z_S using decoder 116
                                             // 15854:        }
  1, 0,                                      // 15858:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 15860:         check predicate 7
  OPC_CheckField, 16, 2, 0,                  // 15862:         check Inst[17:16] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 15866:         check Inst[1:0] == 0x0
  OPC_Decode, 130, 52, 116,                  // 15870:         decode to SQDMULH_VG4_4Z4Z_D using decoder 116
                                             // 15870:        }
                                             // 15870:        } // switch Inst[22]
                                             // 15870:       }
                                             // 15870:       } // switch Inst[12:5]
                                             // 15870:      }
  2, 191, 2,                                 // 15874:      case 0x2: {
  OPC_SwitchField, 10, 3,                    // 15877:       switch Inst[12:10] {
  0, 27,                                     // 15880:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15882:        switch Inst[22] {
  0, 10,                                     // 15885:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 15887:         check predicate 7
  OPC_CheckField, 0, 1, 0,                   // 15889:         check Inst[0] == 0x0
  OPC_Decode, 186, 22, 117,                  // 15893:         decode to FCLAMP_VG2_2Z2Z_S using decoder 117
                                             // 15893:        }
  1, 0,                                      // 15897:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 15899:         check predicate 7
  OPC_CheckField, 0, 1, 0,                   // 15901:         check Inst[0] == 0x0
  OPC_Decode, 184, 22, 117,                  // 15905:         decode to FCLAMP_VG2_2Z2Z_D using decoder 117
                                             // 15905:        }
                                             // 15905:        } // switch Inst[22]
                                             // 15905:       }
  1, 45,                                     // 15909:       case 0x1: {
  OPC_SwitchField, 0, 1,                     // 15911:        switch Inst[0] {
  0, 19,                                     // 15914:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15916:         switch Inst[22] {
  0, 6,                                      // 15919:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15921:          check predicate 7
  OPC_Decode, 240, 46, 117,                  // 15923:          decode to SCLAMP_VG2_2Z2Z_S using decoder 117
                                             // 15923:         }
  1, 0,                                      // 15927:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15929:          check predicate 7
  OPC_Decode, 238, 46, 117,                  // 15931:          decode to SCLAMP_VG2_2Z2Z_D using decoder 117
                                             // 15931:         }
                                             // 15931:         } // switch Inst[22]
                                             // 15931:        }
  1, 0,                                      // 15935:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 15937:         switch Inst[22] {
  0, 6,                                      // 15940:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15942:          check predicate 7
  OPC_Decode, 233, 62, 117,                  // 15944:          decode to UCLAMP_VG2_2Z2Z_S using decoder 117
                                             // 15944:         }
  1, 0,                                      // 15948:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 15950:          check predicate 7
  OPC_Decode, 231, 62, 117,                  // 15952:          decode to UCLAMP_VG2_2Z2Z_D using decoder 117
                                             // 15952:         }
                                             // 15952:         } // switch Inst[22]
                                             // 15952:        }
                                             // 15952:        } // switch Inst[0]
                                             // 15952:       }
  2, 27,                                     // 15956:       case 0x2: {
  OPC_SwitchField, 22, 1,                    // 15958:        switch Inst[22] {
  0, 10,                                     // 15961:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 15963:         check predicate 7
  OPC_CheckField, 0, 2, 0,                   // 15965:         check Inst[1:0] == 0x0
  OPC_Decode, 189, 22, 118,                  // 15969:         decode to FCLAMP_VG4_4Z4Z_S using decoder 118
                                             // 15969:        }
  1, 0,                                      // 15973:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 15975:         check predicate 7
  OPC_CheckField, 0, 2, 0,                   // 15977:         check Inst[1:0] == 0x0
  OPC_Decode, 187, 22, 118,                  // 15981:         decode to FCLAMP_VG4_4Z4Z_D using decoder 118
                                             // 15981:        }
                                             // 15981:        } // switch Inst[22]
                                             // 15981:       }
  3, 45,                                     // 15985:       case 0x3: {
  OPC_SwitchField, 0, 2,                     // 15987:        switch Inst[1:0] {
  0, 19,                                     // 15990:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 15992:         switch Inst[22] {
  0, 6,                                      // 15995:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 15997:          check predicate 7
  OPC_Decode, 244, 46, 118,                  // 15999:          decode to SCLAMP_VG4_4Z4Z_S using decoder 118
                                             // 15999:         }
  1, 0,                                      // 16003:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 16005:          check predicate 7
  OPC_Decode, 242, 46, 118,                  // 16007:          decode to SCLAMP_VG4_4Z4Z_D using decoder 118
                                             // 16007:         }
                                             // 16007:         } // switch Inst[22]
                                             // 16007:        }
  1, 0,                                      // 16011:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16013:         switch Inst[22] {
  0, 6,                                      // 16016:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 16018:          check predicate 7
  OPC_Decode, 237, 62, 118,                  // 16020:          decode to UCLAMP_VG4_4Z4Z_S using decoder 118
                                             // 16020:         }
  1, 0,                                      // 16024:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 16026:          check predicate 7
  OPC_Decode, 235, 62, 118,                  // 16028:          decode to UCLAMP_VG4_4Z4Z_D using decoder 118
                                             // 16028:         }
                                             // 16028:         } // switch Inst[22]
                                             // 16028:        }
                                             // 16028:        } // switch Inst[1:0]
                                             // 16028:       }
  4, 45,                                     // 16032:       case 0x4: {
  OPC_SwitchField, 0, 1,                     // 16034:        switch Inst[0] {
  0, 19,                                     // 16037:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16039:         switch Inst[22] {
  0, 6,                                      // 16042:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 16044:          check predicate 7
  OPC_Decode, 169, 71, 119,                  // 16046:          decode to ZIP_VG2_2ZZZ_S using decoder 119
                                             // 16046:         }
  1, 0,                                      // 16050:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 16052:          check predicate 7
  OPC_Decode, 166, 71, 119,                  // 16054:          decode to ZIP_VG2_2ZZZ_D using decoder 119
                                             // 16054:         }
                                             // 16054:         } // switch Inst[22]
                                             // 16054:        }
  1, 0,                                      // 16058:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16060:         switch Inst[22] {
  0, 6,                                      // 16063:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 16065:          check predicate 7
  OPC_Decode, 211, 69, 119,                  // 16067:          decode to UZP_VG2_2ZZZ_S using decoder 119
                                             // 16067:         }
  1, 0,                                      // 16071:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 16073:          check predicate 7
  OPC_Decode, 208, 69, 119,                  // 16075:          decode to UZP_VG2_2ZZZ_D using decoder 119
                                             // 16075:         }
                                             // 16075:         } // switch Inst[22]
                                             // 16075:        }
                                             // 16075:        } // switch Inst[0]
                                             // 16075:       }
  5, 51,                                     // 16079:       case 0x5: {
  OPC_SwitchField, 5, 1,                     // 16081:        switch Inst[5] {
  0, 29,                                     // 16084:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 16086:         switch Inst[20] {
  0, 11,                                     // 16089:         case 0x0: {
  OPC_CheckPredicate, 7,                     // 16091:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 16093:          check Inst[22] == 0x1
  OPC_Decode, 214, 53, 151, 1,               // 16097:          decode to SQRSHR_VG2_Z2ZI_H using decoder 151
                                             // 16097:         }
  1, 0,                                      // 16102:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 16104:          check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 16106:          check Inst[22] == 0x1
  OPC_Decode, 211, 53, 151, 1,               // 16110:          decode to SQRSHRU_VG2_Z2ZI_H using decoder 151
                                             // 16110:         }
                                             // 16110:         } // switch Inst[20]
                                             // 16110:        }
  1, 0,                                      // 16115:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 16117:         check predicate 7
  OPC_CheckField, 22, 1, 1,                  // 16119:         check Inst[22] == 0x1
  OPC_CheckField, 20, 1, 0,                  // 16123:         check Inst[20] == 0x0
  OPC_Decode, 229, 66, 151, 1,               // 16127:         decode to UQRSHR_VG2_Z2ZI_H using decoder 151
                                             // 16127:        }
                                             // 16127:        } // switch Inst[5]
                                             // 16127:       }
  6, 30,                                     // 16132:       case 0x6: {
  OPC_SwitchField, 5, 2,                     // 16134:        switch Inst[6:5] {
  0, 7,                                      // 16137:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 16139:         check predicate 7
  OPC_Decode, 216, 53, 152, 1,               // 16141:         decode to SQRSHR_VG4_Z4ZI_H using decoder 152
                                             // 16141:        }
  1, 7,                                      // 16146:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 16148:         check predicate 7
  OPC_Decode, 231, 66, 152, 1,               // 16150:         decode to UQRSHR_VG4_Z4ZI_H using decoder 152
                                             // 16150:        }
  2, 0,                                      // 16155:        case 0x2: {
  OPC_CheckPredicate, 7,                     // 16157:         check predicate 7
  OPC_Decode, 213, 53, 152, 1,               // 16159:         decode to SQRSHRU_VG4_Z4ZI_H using decoder 152
                                             // 16159:        }
                                             // 16159:        } // switch Inst[6:5]
                                             // 16159:       }
  7, 0,                                      // 16164:       case 0x7: {
  OPC_SwitchField, 5, 2,                     // 16166:        switch Inst[6:5] {
  0, 7,                                      // 16169:        case 0x0: {
  OPC_CheckPredicate, 7,                     // 16171:         check predicate 7
  OPC_Decode, 180, 53, 152, 1,               // 16173:         decode to SQRSHRN_VG4_Z4ZI_H using decoder 152
                                             // 16173:        }
  1, 7,                                      // 16178:        case 0x1: {
  OPC_CheckPredicate, 7,                     // 16180:         check predicate 7
  OPC_Decode, 217, 66, 152, 1,               // 16182:         decode to UQRSHRN_VG4_Z4ZI_H using decoder 152
                                             // 16182:        }
  2, 0,                                      // 16187:        case 0x2: {
  OPC_CheckPredicate, 7,                     // 16189:         check predicate 7
  OPC_Decode, 199, 53, 152, 1,               // 16191:         decode to SQRSHRUN_VG4_Z4ZI_H using decoder 152
                                             // 16191:        }
                                             // 16191:        } // switch Inst[6:5]
                                             // 16191:       }
                                             // 16191:       } // switch Inst[12:10]
                                             // 16191:      }
  3, 0,                                      // 16196:      case 0x3: {
  OPC_SwitchField, 10, 3,                    // 16198:       switch Inst[12:10] {
  0, 227, 3,                                 // 16201:       case 0x0: {
  OPC_SwitchField, 16, 5,                    // 16204:        switch Inst[20:16] {
  0, 27,                                     // 16207:        case 0x0: {
  OPC_SwitchField, 0, 1,                     // 16209:         switch Inst[0] {
  0, 10,                                     // 16212:         case 0x0: {
  OPC_CheckPredicate, 16,                    // 16214:          check predicate 16
  OPC_CheckField, 22, 1, 0,                  // 16216:          check Inst[22] == 0x0
  OPC_Decode, 222, 25, 123,                  // 16220:          decode to FCVT_2ZZ_H_S using decoder 123
                                             // 16220:         }
  1, 0,                                      // 16224:         case 0x1: {
  OPC_CheckPredicate, 16,                    // 16226:          check predicate 16
  OPC_CheckField, 22, 1, 0,                  // 16228:          check Inst[22] == 0x0
  OPC_Decode, 237, 23, 123,                  // 16232:          decode to FCVTL_2ZZ_H_S using decoder 123
                                             // 16232:         }
                                             // 16232:         } // switch Inst[0]
                                             // 16232:        }
  5, 45,                                     // 16236:        case 0x5: {
  OPC_SwitchField, 0, 1,                     // 16238:         switch Inst[0] {
  0, 19,                                     // 16241:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16243:          switch Inst[22] {
  0, 6,                                      // 16246:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16248:           check predicate 7
  OPC_Decode, 240, 60, 123,                  // 16250:           decode to SUNPK_VG2_2ZZ_S using decoder 123
                                             // 16250:          }
  1, 0,                                      // 16254:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16256:           check predicate 7
  OPC_Decode, 238, 60, 123,                  // 16258:           decode to SUNPK_VG2_2ZZ_D using decoder 123
                                             // 16258:          }
                                             // 16258:          } // switch Inst[22]
                                             // 16258:         }
  1, 0,                                      // 16262:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16264:          switch Inst[22] {
  0, 6,                                      // 16267:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16269:           check predicate 7
  OPC_Decode, 148, 69, 123,                  // 16271:           decode to UUNPK_VG2_2ZZ_S using decoder 123
                                             // 16271:          }
  1, 0,                                      // 16275:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16277:           check predicate 7
  OPC_Decode, 146, 69, 123,                  // 16279:           decode to UUNPK_VG2_2ZZ_D using decoder 123
                                             // 16279:          }
                                             // 16279:          } // switch Inst[22]
                                             // 16279:         }
                                             // 16279:         } // switch Inst[0]
                                             // 16279:        }
  6, 45,                                     // 16283:        case 0x6: {
  OPC_SwitchField, 0, 1,                     // 16285:         switch Inst[0] {
  0, 19,                                     // 16288:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16290:          switch Inst[22] {
  0, 6,                                      // 16293:          case 0x0: {
  OPC_CheckPredicate, 29,                    // 16295:           check predicate 29
  OPC_Decode, 178, 21, 123,                  // 16297:           decode to F2CVT_2ZZ_BtoH using decoder 123
                                             // 16297:          }
  1, 0,                                      // 16301:          case 0x1: {
  OPC_CheckPredicate, 29,                    // 16303:           check predicate 29
  OPC_Decode, 247, 14, 123,                  // 16305:           decode to BF2CVT_2ZZ_BtoH using decoder 123
                                             // 16305:          }
                                             // 16305:          } // switch Inst[22]
                                             // 16305:         }
  1, 0,                                      // 16309:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16311:          switch Inst[22] {
  0, 6,                                      // 16314:          case 0x0: {
  OPC_CheckPredicate, 29,                    // 16316:           check predicate 29
  OPC_Decode, 177, 21, 123,                  // 16318:           decode to F2CVTL_2ZZ_BtoH using decoder 123
                                             // 16318:          }
  1, 0,                                      // 16322:          case 0x1: {
  OPC_CheckPredicate, 29,                    // 16324:           check predicate 29
  OPC_Decode, 246, 14, 123,                  // 16326:           decode to BF2CVTL_2ZZ_BtoH using decoder 123
                                             // 16326:          }
                                             // 16326:          } // switch Inst[22]
                                             // 16326:         }
                                             // 16326:         } // switch Inst[0]
                                             // 16326:        }
  8, 18,                                     // 16330:        case 0x8: {
  OPC_CheckPredicate, 7,                     // 16332:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16334:         check Inst[22] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 16338:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16342:         check Inst[0] == 0x0
  OPC_Decode, 178, 31, 122,                  // 16346:         decode to FRINTN_2Z2Z_S using decoder 122
                                             // 16346:        }
  9, 18,                                     // 16350:        case 0x9: {
  OPC_CheckPredicate, 7,                     // 16352:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16354:         check Inst[22] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 16358:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16362:         check Inst[0] == 0x0
  OPC_Decode, 194, 31, 122,                  // 16366:         decode to FRINTP_2Z2Z_S using decoder 122
                                             // 16366:        }
  10, 18,                                    // 16370:        case 0xa: {
  OPC_CheckPredicate, 7,                     // 16372:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16374:         check Inst[22] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 16378:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16382:         check Inst[0] == 0x0
  OPC_Decode, 162, 31, 122,                  // 16386:         decode to FRINTM_2Z2Z_S using decoder 122
                                             // 16386:        }
  12, 18,                                    // 16390:        case 0xc: {
  OPC_CheckPredicate, 7,                     // 16392:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16394:         check Inst[22] == 0x0
  OPC_CheckField, 5, 1, 0,                   // 16398:         check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16402:         check Inst[0] == 0x0
  OPC_Decode, 132, 31, 122,                  // 16406:         decode to FRINTA_2Z2Z_S using decoder 122
                                             // 16406:        }
  19, 69,                                    // 16410:        case 0x13: {
  OPC_SwitchField, 5, 2,                     // 16412:         switch Inst[6:5] {
  0, 19,                                     // 16415:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16417:          switch Inst[22] {
  0, 6,                                      // 16420:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16422:           check predicate 7
  OPC_Decode, 175, 51, 125,                  // 16424:           decode to SQCVT_Z4Z_DtoH using decoder 125
                                             // 16424:          }
  1, 0,                                      // 16428:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16430:           check predicate 7
  OPC_Decode, 172, 51, 125,                  // 16432:           decode to SQCVTU_Z4Z_DtoH using decoder 125
                                             // 16432:          }
                                             // 16432:          } // switch Inst[22]
                                             // 16432:         }
  1, 10,                                     // 16436:         case 0x1: {
  OPC_CheckPredicate, 7,                     // 16438:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16440:          check Inst[22] == 0x0
  OPC_Decode, 145, 66, 125,                  // 16444:          decode to UQCVT_Z4Z_DtoH using decoder 125
                                             // 16444:         }
  2, 19,                                     // 16448:         case 0x2: {
  OPC_SwitchField, 22, 1,                    // 16450:          switch Inst[22] {
  0, 6,                                      // 16453:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16455:           check predicate 7
  OPC_Decode, 166, 51, 125,                  // 16457:           decode to SQCVTN_Z4Z_DtoH using decoder 125
                                             // 16457:          }
  1, 0,                                      // 16461:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16463:           check predicate 7
  OPC_Decode, 169, 51, 125,                  // 16465:           decode to SQCVTUN_Z4Z_DtoH using decoder 125
                                             // 16465:          }
                                             // 16465:          } // switch Inst[22]
                                             // 16465:         }
  3, 0,                                      // 16469:         case 0x3: {
  OPC_CheckPredicate, 7,                     // 16471:          check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16473:          check Inst[22] == 0x0
  OPC_Decode, 142, 66, 125,                  // 16477:          decode to UQCVTN_Z4Z_DtoH using decoder 125
                                             // 16477:         }
                                             // 16477:         } // switch Inst[6:5]
                                             // 16477:        }
  21, 61,                                    // 16481:        case 0x15: {
  OPC_SwitchField, 0, 2,                     // 16483:         switch Inst[1:0] {
  0, 27,                                     // 16486:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16488:          switch Inst[22] {
  0, 10,                                     // 16491:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16493:           check predicate 7
  OPC_CheckField, 5, 1, 0,                   // 16495:           check Inst[5] == 0x0
  OPC_Decode, 243, 60, 126,                  // 16499:           decode to SUNPK_VG4_4Z2Z_S using decoder 126
                                             // 16499:          }
  1, 0,                                      // 16503:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16505:           check predicate 7
  OPC_CheckField, 5, 1, 0,                   // 16507:           check Inst[5] == 0x0
  OPC_Decode, 241, 60, 126,                  // 16511:           decode to SUNPK_VG4_4Z2Z_D using decoder 126
                                             // 16511:          }
                                             // 16511:          } // switch Inst[22]
                                             // 16511:         }
  1, 0,                                      // 16515:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16517:          switch Inst[22] {
  0, 10,                                     // 16520:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16522:           check predicate 7
  OPC_CheckField, 5, 1, 0,                   // 16524:           check Inst[5] == 0x0
  OPC_Decode, 151, 69, 126,                  // 16528:           decode to UUNPK_VG4_4Z2Z_S using decoder 126
                                             // 16528:          }
  1, 0,                                      // 16532:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16534:           check predicate 7
  OPC_CheckField, 5, 1, 0,                   // 16536:           check Inst[5] == 0x0
  OPC_Decode, 149, 69, 126,                  // 16540:           decode to UUNPK_VG4_4Z2Z_D using decoder 126
                                             // 16540:          }
                                             // 16540:          } // switch Inst[22]
                                             // 16540:         }
                                             // 16540:         } // switch Inst[1:0]
                                             // 16540:        }
  22, 61,                                    // 16544:        case 0x16: {
  OPC_SwitchField, 0, 2,                     // 16546:         switch Inst[1:0] {
  0, 27,                                     // 16549:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16551:          switch Inst[22] {
  0, 10,                                     // 16554:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16556:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 16558:           check Inst[6:5] == 0x0
  OPC_Decode, 174, 71, 124,                  // 16562:           decode to ZIP_VG4_4Z4Z_S using decoder 124
                                             // 16562:          }
  1, 0,                                      // 16566:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16568:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 16570:           check Inst[6:5] == 0x0
  OPC_Decode, 171, 71, 124,                  // 16574:           decode to ZIP_VG4_4Z4Z_D using decoder 124
                                             // 16574:          }
                                             // 16574:          } // switch Inst[22]
                                             // 16574:         }
  2, 0,                                      // 16578:         case 0x2: {
  OPC_SwitchField, 22, 1,                    // 16580:          switch Inst[22] {
  0, 10,                                     // 16583:          case 0x0: {
  OPC_CheckPredicate, 7,                     // 16585:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 16587:           check Inst[6:5] == 0x0
  OPC_Decode, 216, 69, 124,                  // 16591:           decode to UZP_VG4_4Z4Z_S using decoder 124
                                             // 16591:          }
  1, 0,                                      // 16595:          case 0x1: {
  OPC_CheckPredicate, 7,                     // 16597:           check predicate 7
  OPC_CheckField, 5, 2, 0,                   // 16599:           check Inst[6:5] == 0x0
  OPC_Decode, 213, 69, 124,                  // 16603:           decode to UZP_VG4_4Z4Z_D using decoder 124
                                             // 16603:          }
                                             // 16603:          } // switch Inst[22]
                                             // 16603:         }
                                             // 16603:         } // switch Inst[1:0]
                                             // 16603:        }
  24, 18,                                    // 16607:        case 0x18: {
  OPC_CheckPredicate, 7,                     // 16609:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16611:         check Inst[22] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 16615:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16619:         check Inst[1:0] == 0x0
  OPC_Decode, 179, 31, 124,                  // 16623:         decode to FRINTN_4Z4Z_S using decoder 124
                                             // 16623:        }
  25, 18,                                    // 16627:        case 0x19: {
  OPC_CheckPredicate, 7,                     // 16629:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16631:         check Inst[22] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 16635:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16639:         check Inst[1:0] == 0x0
  OPC_Decode, 195, 31, 124,                  // 16643:         decode to FRINTP_4Z4Z_S using decoder 124
                                             // 16643:        }
  26, 18,                                    // 16647:        case 0x1a: {
  OPC_CheckPredicate, 7,                     // 16649:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16651:         check Inst[22] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 16655:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16659:         check Inst[1:0] == 0x0
  OPC_Decode, 163, 31, 124,                  // 16663:         decode to FRINTM_4Z4Z_S using decoder 124
                                             // 16663:        }
  28, 0,                                     // 16667:        case 0x1c: {
  OPC_CheckPredicate, 7,                     // 16669:         check predicate 7
  OPC_CheckField, 22, 1, 0,                  // 16671:         check Inst[22] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 16675:         check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16679:         check Inst[1:0] == 0x0
  OPC_Decode, 133, 31, 124,                  // 16683:         decode to FRINTA_4Z4Z_S using decoder 124
                                             // 16683:        }
                                             // 16683:        } // switch Inst[20:16]
                                             // 16683:       }
  1, 87,                                     // 16687:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 16689:        switch Inst[16] {
  0, 35,                                     // 16692:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16694:         switch Inst[22] {
  0, 14,                                     // 16697:         case 0x0: {
  OPC_CheckPredicate, 31,                    // 16699:          check predicate 31
  OPC_CheckField, 5, 1, 0,                   // 16701:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16705:          check Inst[0] == 0x0
  OPC_Decode, 248, 29, 127,                  // 16709:          decode to FMUL_2Z2Z_S using decoder 127
                                             // 16709:         }
  1, 0,                                      // 16713:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 16715:          check predicate 31
  OPC_CheckField, 5, 1, 0,                   // 16717:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16721:          check Inst[0] == 0x0
  OPC_Decode, 246, 29, 127,                  // 16725:          decode to FMUL_2Z2Z_D using decoder 127
                                             // 16725:         }
                                             // 16725:         } // switch Inst[22]
                                             // 16725:        }
  1, 0,                                      // 16729:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16731:         switch Inst[22] {
  0, 19,                                     // 16734:         case 0x0: {
  OPC_CheckPredicate, 31,                    // 16736:          check predicate 31
  OPC_CheckField, 17, 1, 0,                  // 16738:          check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 16742:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16746:          check Inst[1:0] == 0x0
  OPC_Decode, 254, 29, 128, 1,               // 16750:          decode to FMUL_4Z4Z_S using decoder 128
                                             // 16750:         }
  1, 0,                                      // 16755:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 16757:          check predicate 31
  OPC_CheckField, 17, 1, 0,                  // 16759:          check Inst[17] == 0x0
  OPC_CheckField, 5, 2, 0,                   // 16763:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16767:          check Inst[1:0] == 0x0
  OPC_Decode, 252, 29, 128, 1,               // 16771:          decode to FMUL_4Z4Z_D using decoder 128
                                             // 16771:         }
                                             // 16771:         } // switch Inst[22]
                                             // 16771:        }
                                             // 16771:        } // switch Inst[16]
                                             // 16771:       }
  2, 0,                                      // 16776:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 16778:        switch Inst[16] {
  0, 37,                                     // 16781:        case 0x0: {
  OPC_SwitchField, 22, 1,                    // 16783:         switch Inst[22] {
  0, 15,                                     // 16786:         case 0x0: {
  OPC_CheckPredicate, 31,                    // 16788:          check predicate 31
  OPC_CheckField, 5, 1, 0,                   // 16790:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16794:          check Inst[0] == 0x0
  OPC_Decode, 251, 29, 129, 1,               // 16798:          decode to FMUL_2ZZ_S using decoder 129
                                             // 16798:         }
  1, 0,                                      // 16803:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 16805:          check predicate 31
  OPC_CheckField, 5, 1, 0,                   // 16807:          check Inst[5] == 0x0
  OPC_CheckField, 0, 1, 0,                   // 16811:          check Inst[0] == 0x0
  OPC_Decode, 249, 29, 129, 1,               // 16815:          decode to FMUL_2ZZ_D using decoder 129
                                             // 16815:         }
                                             // 16815:         } // switch Inst[22]
                                             // 16815:        }
  1, 0,                                      // 16820:        case 0x1: {
  OPC_SwitchField, 22, 1,                    // 16822:         switch Inst[22] {
  0, 15,                                     // 16825:         case 0x0: {
  OPC_CheckPredicate, 31,                    // 16827:          check predicate 31
  OPC_CheckField, 5, 2, 0,                   // 16829:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16833:          check Inst[1:0] == 0x0
  OPC_Decode, 129, 30, 130, 1,               // 16837:          decode to FMUL_4ZZ_S using decoder 130
                                             // 16837:         }
  1, 0,                                      // 16842:         case 0x1: {
  OPC_CheckPredicate, 31,                    // 16844:          check predicate 31
  OPC_CheckField, 5, 2, 0,                   // 16846:          check Inst[6:5] == 0x0
  OPC_CheckField, 0, 2, 0,                   // 16850:          check Inst[1:0] == 0x0
  OPC_Decode, 255, 29, 130, 1,               // 16854:          decode to FMUL_4ZZ_D using decoder 130
                                             // 16854:         }
                                             // 16854:         } // switch Inst[22]
                                             // 16854:        }
                                             // 16854:        } // switch Inst[16]
                                             // 16854:       }
                                             // 16854:       } // switch Inst[12:10]
                                             // 16854:      }
                                             // 16854:      } // switch Inst[14:13]
                                             // 16854:     }
                                             // 16854:     } // switch Inst[15]
                                             // 16854:    }
                                             // 16854:    } // switch Inst[21]
                                             // 16854:   }
                                             // 16854:   } // switch Inst[25:23]
                                             // 16854:  }
  7, 0,                                      // 16859:  case 0x7: {
  OPC_SwitchField, 21, 5,                    // 16861:   switch Inst[25:21] {
  0, 29,                                     // 16864:   case 0x0: {
  OPC_SwitchField, 15, 1,                    // 16866:    switch Inst[15] {
  0, 11,                                     // 16869:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 16871:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16873:     check Inst[4] == 0x0
  OPC_Decode, 229, 35, 153, 1,               // 16877:     decode to LD1_MXIPXX_H_B using decoder 153
                                             // 16877:    }
  1, 0,                                      // 16882:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 16884:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16886:     check Inst[4] == 0x0
  OPC_Decode, 234, 35, 153, 1,               // 16890:     decode to LD1_MXIPXX_V_B using decoder 153
                                             // 16890:    }
                                             // 16890:    } // switch Inst[15]
                                             // 16890:   }
  1, 29,                                     // 16895:   case 0x1: {
  OPC_SwitchField, 15, 1,                    // 16897:    switch Inst[15] {
  0, 11,                                     // 16900:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 16902:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16904:     check Inst[4] == 0x0
  OPC_Decode, 152, 57, 153, 1,               // 16908:     decode to ST1_MXIPXX_H_B using decoder 153
                                             // 16908:    }
  1, 0,                                      // 16913:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 16915:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16917:     check Inst[4] == 0x0
  OPC_Decode, 157, 57, 153, 1,               // 16921:     decode to ST1_MXIPXX_V_B using decoder 153
                                             // 16921:    }
                                             // 16921:    } // switch Inst[15]
                                             // 16921:   }
  2, 29,                                     // 16926:   case 0x2: {
  OPC_SwitchField, 15, 1,                    // 16928:    switch Inst[15] {
  0, 11,                                     // 16931:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 16933:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16935:     check Inst[4] == 0x0
  OPC_Decode, 231, 35, 154, 1,               // 16939:     decode to LD1_MXIPXX_H_H using decoder 154
                                             // 16939:    }
  1, 0,                                      // 16944:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 16946:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16948:     check Inst[4] == 0x0
  OPC_Decode, 236, 35, 154, 1,               // 16952:     decode to LD1_MXIPXX_V_H using decoder 154
                                             // 16952:    }
                                             // 16952:    } // switch Inst[15]
                                             // 16952:   }
  3, 29,                                     // 16957:   case 0x3: {
  OPC_SwitchField, 15, 1,                    // 16959:    switch Inst[15] {
  0, 11,                                     // 16962:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 16964:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16966:     check Inst[4] == 0x0
  OPC_Decode, 154, 57, 154, 1,               // 16970:     decode to ST1_MXIPXX_H_H using decoder 154
                                             // 16970:    }
  1, 0,                                      // 16975:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 16977:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16979:     check Inst[4] == 0x0
  OPC_Decode, 159, 57, 154, 1,               // 16983:     decode to ST1_MXIPXX_V_H using decoder 154
                                             // 16983:    }
                                             // 16983:    } // switch Inst[15]
                                             // 16983:   }
  4, 29,                                     // 16988:   case 0x4: {
  OPC_SwitchField, 15, 1,                    // 16990:    switch Inst[15] {
  0, 11,                                     // 16993:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 16995:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 16997:     check Inst[4] == 0x0
  OPC_Decode, 233, 35, 155, 1,               // 17001:     decode to LD1_MXIPXX_H_S using decoder 155
                                             // 17001:    }
  1, 0,                                      // 17006:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 17008:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17010:     check Inst[4] == 0x0
  OPC_Decode, 238, 35, 155, 1,               // 17014:     decode to LD1_MXIPXX_V_S using decoder 155
                                             // 17014:    }
                                             // 17014:    } // switch Inst[15]
                                             // 17014:   }
  5, 29,                                     // 17019:   case 0x5: {
  OPC_SwitchField, 15, 1,                    // 17021:    switch Inst[15] {
  0, 11,                                     // 17024:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17026:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17028:     check Inst[4] == 0x0
  OPC_Decode, 156, 57, 155, 1,               // 17032:     decode to ST1_MXIPXX_H_S using decoder 155
                                             // 17032:    }
  1, 0,                                      // 17037:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 17039:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17041:     check Inst[4] == 0x0
  OPC_Decode, 161, 57, 155, 1,               // 17045:     decode to ST1_MXIPXX_V_S using decoder 155
                                             // 17045:    }
                                             // 17045:    } // switch Inst[15]
                                             // 17045:   }
  6, 29,                                     // 17050:   case 0x6: {
  OPC_SwitchField, 15, 1,                    // 17052:    switch Inst[15] {
  0, 11,                                     // 17055:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17057:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17059:     check Inst[4] == 0x0
  OPC_Decode, 230, 35, 156, 1,               // 17063:     decode to LD1_MXIPXX_H_D using decoder 156
                                             // 17063:    }
  1, 0,                                      // 17068:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 17070:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17072:     check Inst[4] == 0x0
  OPC_Decode, 235, 35, 156, 1,               // 17076:     decode to LD1_MXIPXX_V_D using decoder 156
                                             // 17076:    }
                                             // 17076:    } // switch Inst[15]
                                             // 17076:   }
  7, 29,                                     // 17081:   case 0x7: {
  OPC_SwitchField, 15, 1,                    // 17083:    switch Inst[15] {
  0, 11,                                     // 17086:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17088:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17090:     check Inst[4] == 0x0
  OPC_Decode, 153, 57, 156, 1,               // 17094:     decode to ST1_MXIPXX_H_D using decoder 156
                                             // 17094:    }
  1, 0,                                      // 17099:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 17101:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17103:     check Inst[4] == 0x0
  OPC_Decode, 158, 57, 156, 1,               // 17107:     decode to ST1_MXIPXX_V_D using decoder 156
                                             // 17107:    }
                                             // 17107:    } // switch Inst[15]
                                             // 17107:   }
  8, 37,                                     // 17112:   case 0x8: {
  OPC_SwitchField, 15, 6,                    // 17114:    switch Inst[20:15] {
  0, 15,                                     // 17117:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17119:     check predicate 6
  OPC_CheckField, 10, 3, 0,                  // 17121:     check Inst[12:10] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 17125:     check Inst[4] == 0x0
  OPC_Decode, 246, 39, 157, 1,               // 17129:     decode to LDR_ZA using decoder 157
                                             // 17129:    }
  63, 0,                                     // 17134:    case 0x3f: {
  OPC_CheckPredicate, 7,                     // 17136:     check predicate 7
  OPC_CheckField, 10, 5, 0,                  // 17138:     check Inst[14:10] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 17142:     check Inst[4:0] == 0x0
  OPC_Decode, 245, 39, 158, 1,               // 17146:     decode to LDR_TX using decoder 158
                                             // 17146:    }
                                             // 17146:    } // switch Inst[20:15]
                                             // 17146:   }
  9, 37,                                     // 17151:   case 0x9: {
  OPC_SwitchField, 15, 6,                    // 17153:    switch Inst[20:15] {
  0, 15,                                     // 17156:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17158:     check predicate 6
  OPC_CheckField, 10, 3, 0,                  // 17160:     check Inst[12:10] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 17164:     check Inst[4] == 0x0
  OPC_Decode, 213, 59, 157, 1,               // 17168:     decode to STR_ZA using decoder 157
                                             // 17168:    }
  63, 0,                                     // 17173:    case 0x3f: {
  OPC_CheckPredicate, 7,                     // 17175:     check predicate 7
  OPC_CheckField, 10, 5, 0,                  // 17177:     check Inst[14:10] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 17181:     check Inst[4:0] == 0x0
  OPC_Decode, 212, 59, 158, 1,               // 17185:     decode to STR_TX using decoder 158
                                             // 17185:    }
                                             // 17185:    } // switch Inst[20:15]
                                             // 17185:   }
  14, 29,                                    // 17190:   case 0xe: {
  OPC_SwitchField, 15, 1,                    // 17192:    switch Inst[15] {
  0, 11,                                     // 17195:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17197:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17199:     check Inst[4] == 0x0
  OPC_Decode, 232, 35, 159, 1,               // 17203:     decode to LD1_MXIPXX_H_Q using decoder 159
                                             // 17203:    }
  1, 0,                                      // 17208:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 17210:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17212:     check Inst[4] == 0x0
  OPC_Decode, 237, 35, 159, 1,               // 17216:     decode to LD1_MXIPXX_V_Q using decoder 159
                                             // 17216:    }
                                             // 17216:    } // switch Inst[15]
                                             // 17216:   }
  15, 0,                                     // 17221:   case 0xf: {
  OPC_SwitchField, 15, 1,                    // 17223:    switch Inst[15] {
  0, 11,                                     // 17226:    case 0x0: {
  OPC_CheckPredicate, 6,                     // 17228:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17230:     check Inst[4] == 0x0
  OPC_Decode, 155, 57, 159, 1,               // 17234:     decode to ST1_MXIPXX_H_Q using decoder 159
                                             // 17234:    }
  1, 0,                                      // 17239:    case 0x1: {
  OPC_CheckPredicate, 6,                     // 17241:     check predicate 6
  OPC_CheckField, 4, 1, 0,                   // 17243:     check Inst[4] == 0x0
  OPC_Decode, 160, 57, 159, 1,               // 17247:     decode to ST1_MXIPXX_V_Q using decoder 159
                                             // 17247:    }
                                             // 17247:    } // switch Inst[15]
                                             // 17247:   }
                                             // 17247:   } // switch Inst[25:21]
                                             // 17247:  }
                                             // 17247:  } // switch Inst[31:29]
                                             // 17247: }
  1, 225, 255, 1,                            // 17252: case 0x1: {
  OPC_SwitchField, 29, 3,                    // 17256:  switch Inst[31:29] {
  0, 166, 74,                                // 17259:  case 0x0: {
  OPC_SwitchField, 21, 1,                    // 17262:   switch Inst[21] {
  0, 129, 32,                                // 17265:   case 0x0: {
  OPC_SwitchField, 23, 3,                    // 17268:    switch Inst[25:23] {
  0, 134, 15,                                // 17271:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 17274:     switch Inst[15:13] {
  0, 243, 2,                                 // 17277:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 17280:      switch Inst[20:16] {
  0, 21,                                     // 17283:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 17285:       switch Inst[22] {
  0, 7,                                      // 17288:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17290:        check predicate 33
  OPC_Decode, 223, 13, 160, 1,               // 17292:        decode to ADD_ZPmZ_B using decoder 160
                                             // 17292:       }
  1, 0,                                      // 17297:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17299:        check predicate 33
  OPC_Decode, 226, 13, 160, 1,               // 17301:        decode to ADD_ZPmZ_H using decoder 160
                                             // 17301:       }
                                             // 17301:       } // switch Inst[22]
                                             // 17301:      }
  1, 21,                                     // 17306:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 17308:       switch Inst[22] {
  0, 7,                                      // 17311:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17313:        check predicate 33
  OPC_Decode, 182, 60, 160, 1,               // 17315:        decode to SUB_ZPmZ_B using decoder 160
                                             // 17315:       }
  1, 0,                                      // 17320:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17322:        check predicate 33
  OPC_Decode, 185, 60, 160, 1,               // 17324:        decode to SUB_ZPmZ_H using decoder 160
                                             // 17324:       }
                                             // 17324:       } // switch Inst[22]
                                             // 17324:      }
  3, 21,                                     // 17329:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 17331:       switch Inst[22] {
  0, 7,                                      // 17334:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17336:        check predicate 33
  OPC_Decode, 148, 60, 160, 1,               // 17338:        decode to SUBR_ZPmZ_B using decoder 160
                                             // 17338:       }
  1, 0,                                      // 17343:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17345:        check predicate 33
  OPC_Decode, 150, 60, 160, 1,               // 17347:        decode to SUBR_ZPmZ_H using decoder 160
                                             // 17347:       }
                                             // 17347:       } // switch Inst[22]
                                             // 17347:      }
  8, 21,                                     // 17352:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 17354:       switch Inst[22] {
  0, 7,                                      // 17357:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17359:        check predicate 33
  OPC_Decode, 139, 49, 160, 1,               // 17361:        decode to SMAX_ZPmZ_B using decoder 160
                                             // 17361:       }
  1, 0,                                      // 17366:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17368:        check predicate 33
  OPC_Decode, 141, 49, 160, 1,               // 17370:        decode to SMAX_ZPmZ_H using decoder 160
                                             // 17370:       }
                                             // 17370:       } // switch Inst[22]
                                             // 17370:      }
  9, 21,                                     // 17375:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 17377:       switch Inst[22] {
  0, 7,                                      // 17380:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17382:        check predicate 33
  OPC_Decode, 149, 64, 160, 1,               // 17384:        decode to UMAX_ZPmZ_B using decoder 160
                                             // 17384:       }
  1, 0,                                      // 17389:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17391:        check predicate 33
  OPC_Decode, 151, 64, 160, 1,               // 17393:        decode to UMAX_ZPmZ_H using decoder 160
                                             // 17393:       }
                                             // 17393:       } // switch Inst[22]
                                             // 17393:      }
  10, 21,                                    // 17398:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 17400:       switch Inst[22] {
  0, 7,                                      // 17403:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17405:        check predicate 33
  OPC_Decode, 197, 49, 160, 1,               // 17407:        decode to SMIN_ZPmZ_B using decoder 160
                                             // 17407:       }
  1, 0,                                      // 17412:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17414:        check predicate 33
  OPC_Decode, 199, 49, 160, 1,               // 17416:        decode to SMIN_ZPmZ_H using decoder 160
                                             // 17416:       }
                                             // 17416:       } // switch Inst[22]
                                             // 17416:      }
  11, 21,                                    // 17421:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 17423:       switch Inst[22] {
  0, 7,                                      // 17426:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17428:        check predicate 33
  OPC_Decode, 206, 64, 160, 1,               // 17430:        decode to UMIN_ZPmZ_B using decoder 160
                                             // 17430:       }
  1, 0,                                      // 17435:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17437:        check predicate 33
  OPC_Decode, 208, 64, 160, 1,               // 17439:        decode to UMIN_ZPmZ_H using decoder 160
                                             // 17439:       }
                                             // 17439:       } // switch Inst[22]
                                             // 17439:      }
  12, 21,                                    // 17444:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 17446:       switch Inst[22] {
  0, 7,                                      // 17449:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17451:        check predicate 33
  OPC_Decode, 166, 46, 160, 1,               // 17453:        decode to SABD_ZPmZ_B using decoder 160
                                             // 17453:       }
  1, 0,                                      // 17458:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17460:        check predicate 33
  OPC_Decode, 168, 46, 160, 1,               // 17462:        decode to SABD_ZPmZ_H using decoder 160
                                             // 17462:       }
                                             // 17462:       } // switch Inst[22]
                                             // 17462:      }
  13, 21,                                    // 17467:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 17469:       switch Inst[22] {
  0, 7,                                      // 17472:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17474:        check predicate 33
  OPC_Decode, 170, 62, 160, 1,               // 17476:        decode to UABD_ZPmZ_B using decoder 160
                                             // 17476:       }
  1, 0,                                      // 17481:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17483:        check predicate 33
  OPC_Decode, 172, 62, 160, 1,               // 17485:        decode to UABD_ZPmZ_H using decoder 160
                                             // 17485:       }
                                             // 17485:       } // switch Inst[22]
                                             // 17485:      }
  16, 21,                                    // 17490:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 17492:       switch Inst[22] {
  0, 7,                                      // 17495:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17497:        check predicate 33
  OPC_Decode, 132, 43, 160, 1,               // 17499:        decode to MUL_ZPmZ_B using decoder 160
                                             // 17499:       }
  1, 0,                                      // 17504:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17506:        check predicate 33
  OPC_Decode, 134, 43, 160, 1,               // 17508:        decode to MUL_ZPmZ_H using decoder 160
                                             // 17508:       }
                                             // 17508:       } // switch Inst[22]
                                             // 17508:      }
  18, 21,                                    // 17513:      case 0x12: {
  OPC_SwitchField, 22, 1,                    // 17515:       switch Inst[22] {
  0, 7,                                      // 17518:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17520:        check predicate 33
  OPC_Decode, 210, 50, 160, 1,               // 17522:        decode to SMULH_ZPmZ_B using decoder 160
                                             // 17522:       }
  1, 0,                                      // 17527:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17529:        check predicate 33
  OPC_Decode, 212, 50, 160, 1,               // 17531:        decode to SMULH_ZPmZ_H using decoder 160
                                             // 17531:       }
                                             // 17531:       } // switch Inst[22]
                                             // 17531:      }
  19, 21,                                    // 17536:      case 0x13: {
  OPC_SwitchField, 22, 1,                    // 17538:       switch Inst[22] {
  0, 7,                                      // 17541:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17543:        check predicate 33
  OPC_Decode, 217, 65, 160, 1,               // 17545:        decode to UMULH_ZPmZ_B using decoder 160
                                             // 17545:       }
  1, 0,                                      // 17550:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17552:        check predicate 33
  OPC_Decode, 219, 65, 160, 1,               // 17554:        decode to UMULH_ZPmZ_H using decoder 160
                                             // 17554:       }
                                             // 17554:       } // switch Inst[22]
                                             // 17554:      }
  24, 21,                                    // 17559:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 17561:       switch Inst[22] {
  0, 7,                                      // 17564:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17566:        check predicate 33
  OPC_Decode, 210, 43, 160, 1,               // 17568:        decode to ORR_ZPmZ_B using decoder 160
                                             // 17568:       }
  1, 0,                                      // 17573:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17575:        check predicate 33
  OPC_Decode, 212, 43, 160, 1,               // 17577:        decode to ORR_ZPmZ_H using decoder 160
                                             // 17577:       }
                                             // 17577:       } // switch Inst[22]
                                             // 17577:      }
  25, 21,                                    // 17582:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 17584:       switch Inst[22] {
  0, 7,                                      // 17587:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17589:        check predicate 33
  OPC_Decode, 137, 21, 160, 1,               // 17591:        decode to EOR_ZPmZ_B using decoder 160
                                             // 17591:       }
  1, 0,                                      // 17596:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17598:        check predicate 33
  OPC_Decode, 139, 21, 160, 1,               // 17600:        decode to EOR_ZPmZ_H using decoder 160
                                             // 17600:       }
                                             // 17600:       } // switch Inst[22]
                                             // 17600:      }
  26, 21,                                    // 17605:      case 0x1a: {
  OPC_SwitchField, 22, 1,                    // 17607:       switch Inst[22] {
  0, 7,                                      // 17610:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17612:        check predicate 33
  OPC_Decode, 166, 14, 160, 1,               // 17614:        decode to AND_ZPmZ_B using decoder 160
                                             // 17614:       }
  1, 0,                                      // 17619:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17621:        check predicate 33
  OPC_Decode, 168, 14, 160, 1,               // 17623:        decode to AND_ZPmZ_H using decoder 160
                                             // 17623:       }
                                             // 17623:       } // switch Inst[22]
                                             // 17623:      }
  27, 0,                                     // 17628:      case 0x1b: {
  OPC_SwitchField, 22, 1,                    // 17630:       switch Inst[22] {
  0, 7,                                      // 17633:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17635:        check predicate 33
  OPC_Decode, 139, 16, 160, 1,               // 17637:        decode to BIC_ZPmZ_B using decoder 160
                                             // 17637:       }
  1, 0,                                      // 17642:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17644:        check predicate 33
  OPC_Decode, 141, 16, 160, 1,               // 17646:        decode to BIC_ZPmZ_H using decoder 160
                                             // 17646:       }
                                             // 17646:       } // switch Inst[22]
                                             // 17646:      }
                                             // 17646:      } // switch Inst[20:16]
                                             // 17646:     }
  1, 184, 3,                                 // 17651:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 17654:      switch Inst[20:16] {
  0, 21,                                     // 17657:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 17659:       switch Inst[22] {
  0, 7,                                      // 17662:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17664:        check predicate 33
  OPC_Decode, 211, 46, 161, 1,               // 17666:        decode to SADDV_VPZ_B using decoder 161
                                             // 17666:       }
  1, 0,                                      // 17671:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17673:        check predicate 33
  OPC_Decode, 212, 46, 161, 1,               // 17675:        decode to SADDV_VPZ_H using decoder 161
                                             // 17675:       }
                                             // 17675:       } // switch Inst[22]
                                             // 17675:      }
  1, 21,                                     // 17680:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 17682:       switch Inst[22] {
  0, 7,                                      // 17685:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17687:        check predicate 33
  OPC_Decode, 212, 62, 161, 1,               // 17689:        decode to UADDV_VPZ_B using decoder 161
                                             // 17689:       }
  1, 0,                                      // 17694:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17696:        check predicate 33
  OPC_Decode, 214, 62, 161, 1,               // 17698:        decode to UADDV_VPZ_H using decoder 161
                                             // 17698:       }
                                             // 17698:       } // switch Inst[22]
                                             // 17698:      }
  5, 21,                                     // 17703:      case 0x5: {
  OPC_SwitchField, 22, 1,                    // 17705:       switch Inst[22] {
  0, 7,                                      // 17708:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 17710:        check predicate 34
  OPC_Decode, 167, 13, 162, 1,               // 17712:        decode to ADDQV_VPZ_B using decoder 162
                                             // 17712:       }
  1, 0,                                      // 17717:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 17719:        check predicate 34
  OPC_Decode, 169, 13, 162, 1,               // 17721:        decode to ADDQV_VPZ_H using decoder 162
                                             // 17721:       }
                                             // 17721:       } // switch Inst[22]
                                             // 17721:      }
  8, 21,                                     // 17726:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 17728:       switch Inst[22] {
  0, 7,                                      // 17731:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17733:        check predicate 33
  OPC_Decode, 234, 48, 161, 1,               // 17735:        decode to SMAXV_VPZ_B using decoder 161
                                             // 17735:       }
  1, 0,                                      // 17740:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17742:        check predicate 33
  OPC_Decode, 236, 48, 161, 1,               // 17744:        decode to SMAXV_VPZ_H using decoder 161
                                             // 17744:       }
                                             // 17744:       } // switch Inst[22]
                                             // 17744:      }
  9, 21,                                     // 17749:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 17751:       switch Inst[22] {
  0, 7,                                      // 17754:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17756:        check predicate 33
  OPC_Decode, 244, 63, 161, 1,               // 17758:        decode to UMAXV_VPZ_B using decoder 161
                                             // 17758:       }
  1, 0,                                      // 17763:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17765:        check predicate 33
  OPC_Decode, 246, 63, 161, 1,               // 17767:        decode to UMAXV_VPZ_H using decoder 161
                                             // 17767:       }
                                             // 17767:       } // switch Inst[22]
                                             // 17767:      }
  10, 21,                                    // 17772:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 17774:       switch Inst[22] {
  0, 7,                                      // 17777:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17779:        check predicate 33
  OPC_Decode, 164, 49, 161, 1,               // 17781:        decode to SMINV_VPZ_B using decoder 161
                                             // 17781:       }
  1, 0,                                      // 17786:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17788:        check predicate 33
  OPC_Decode, 166, 49, 161, 1,               // 17790:        decode to SMINV_VPZ_H using decoder 161
                                             // 17790:       }
                                             // 17790:       } // switch Inst[22]
                                             // 17790:      }
  11, 21,                                    // 17795:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 17797:       switch Inst[22] {
  0, 7,                                      // 17800:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17802:        check predicate 33
  OPC_Decode, 173, 64, 161, 1,               // 17804:        decode to UMINV_VPZ_B using decoder 161
                                             // 17804:       }
  1, 0,                                      // 17809:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17811:        check predicate 33
  OPC_Decode, 175, 64, 161, 1,               // 17813:        decode to UMINV_VPZ_H using decoder 161
                                             // 17813:       }
                                             // 17813:       } // switch Inst[22]
                                             // 17813:      }
  12, 21,                                    // 17818:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 17820:       switch Inst[22] {
  0, 7,                                      // 17823:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 17825:        check predicate 34
  OPC_Decode, 230, 48, 162, 1,               // 17827:        decode to SMAXQV_VPZ_B using decoder 162
                                             // 17827:       }
  1, 0,                                      // 17832:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 17834:        check predicate 34
  OPC_Decode, 232, 48, 162, 1,               // 17836:        decode to SMAXQV_VPZ_H using decoder 162
                                             // 17836:       }
                                             // 17836:       } // switch Inst[22]
                                             // 17836:      }
  13, 21,                                    // 17841:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 17843:       switch Inst[22] {
  0, 7,                                      // 17846:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 17848:        check predicate 34
  OPC_Decode, 240, 63, 162, 1,               // 17850:        decode to UMAXQV_VPZ_B using decoder 162
                                             // 17850:       }
  1, 0,                                      // 17855:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 17857:        check predicate 34
  OPC_Decode, 242, 63, 162, 1,               // 17859:        decode to UMAXQV_VPZ_H using decoder 162
                                             // 17859:       }
                                             // 17859:       } // switch Inst[22]
                                             // 17859:      }
  14, 21,                                    // 17864:      case 0xe: {
  OPC_SwitchField, 22, 1,                    // 17866:       switch Inst[22] {
  0, 7,                                      // 17869:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 17871:        check predicate 34
  OPC_Decode, 160, 49, 162, 1,               // 17873:        decode to SMINQV_VPZ_B using decoder 162
                                             // 17873:       }
  1, 0,                                      // 17878:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 17880:        check predicate 34
  OPC_Decode, 162, 49, 162, 1,               // 17882:        decode to SMINQV_VPZ_H using decoder 162
                                             // 17882:       }
                                             // 17882:       } // switch Inst[22]
                                             // 17882:      }
  15, 21,                                    // 17887:      case 0xf: {
  OPC_SwitchField, 22, 1,                    // 17889:       switch Inst[22] {
  0, 7,                                      // 17892:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 17894:        check predicate 34
  OPC_Decode, 169, 64, 162, 1,               // 17896:        decode to UMINQV_VPZ_B using decoder 162
                                             // 17896:       }
  1, 0,                                      // 17901:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 17903:        check predicate 34
  OPC_Decode, 171, 64, 162, 1,               // 17905:        decode to UMINQV_VPZ_H using decoder 162
                                             // 17905:       }
                                             // 17905:       } // switch Inst[22]
                                             // 17905:      }
  16, 21,                                    // 17910:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 17912:       switch Inst[22] {
  0, 7,                                      // 17915:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17917:        check predicate 33
  OPC_Decode, 232, 42, 161, 1,               // 17919:        decode to MOVPRFX_ZPzZ_B using decoder 161
                                             // 17919:       }
  1, 0,                                      // 17924:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17926:        check predicate 33
  OPC_Decode, 234, 42, 161, 1,               // 17928:        decode to MOVPRFX_ZPzZ_H using decoder 161
                                             // 17928:       }
                                             // 17928:       } // switch Inst[22]
                                             // 17928:      }
  17, 21,                                    // 17933:      case 0x11: {
  OPC_SwitchField, 22, 1,                    // 17935:       switch Inst[22] {
  0, 7,                                      // 17938:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17940:        check predicate 33
  OPC_Decode, 228, 42, 163, 1,               // 17942:        decode to MOVPRFX_ZPmZ_B using decoder 163
                                             // 17942:       }
  1, 0,                                      // 17947:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17949:        check predicate 33
  OPC_Decode, 230, 42, 163, 1,               // 17951:        decode to MOVPRFX_ZPmZ_H using decoder 163
                                             // 17951:       }
                                             // 17951:       } // switch Inst[22]
                                             // 17951:      }
  24, 21,                                    // 17956:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 17958:       switch Inst[22] {
  0, 7,                                      // 17961:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17963:        check predicate 33
  OPC_Decode, 221, 43, 161, 1,               // 17965:        decode to ORV_VPZ_B using decoder 161
                                             // 17965:       }
  1, 0,                                      // 17970:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17972:        check predicate 33
  OPC_Decode, 223, 43, 161, 1,               // 17974:        decode to ORV_VPZ_H using decoder 161
                                             // 17974:       }
                                             // 17974:       } // switch Inst[22]
                                             // 17974:      }
  25, 21,                                    // 17979:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 17981:       switch Inst[22] {
  0, 7,                                      // 17984:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 17986:        check predicate 33
  OPC_Decode, 255, 20, 161, 1,               // 17988:        decode to EORV_VPZ_B using decoder 161
                                             // 17988:       }
  1, 0,                                      // 17993:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 17995:        check predicate 33
  OPC_Decode, 129, 21, 161, 1,               // 17997:        decode to EORV_VPZ_H using decoder 161
                                             // 17997:       }
                                             // 17997:       } // switch Inst[22]
                                             // 17997:      }
  26, 21,                                    // 18002:      case 0x1a: {
  OPC_SwitchField, 22, 1,                    // 18004:       switch Inst[22] {
  0, 7,                                      // 18007:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18009:        check predicate 33
  OPC_Decode, 156, 14, 161, 1,               // 18011:        decode to ANDV_VPZ_B using decoder 161
                                             // 18011:       }
  1, 0,                                      // 18016:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18018:        check predicate 33
  OPC_Decode, 158, 14, 161, 1,               // 18020:        decode to ANDV_VPZ_H using decoder 161
                                             // 18020:       }
                                             // 18020:       } // switch Inst[22]
                                             // 18020:      }
  28, 21,                                    // 18025:      case 0x1c: {
  OPC_SwitchField, 22, 1,                    // 18027:       switch Inst[22] {
  0, 7,                                      // 18030:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 18032:        check predicate 34
  OPC_Decode, 199, 43, 162, 1,               // 18034:        decode to ORQV_VPZ_B using decoder 162
                                             // 18034:       }
  1, 0,                                      // 18039:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 18041:        check predicate 34
  OPC_Decode, 201, 43, 162, 1,               // 18043:        decode to ORQV_VPZ_H using decoder 162
                                             // 18043:       }
                                             // 18043:       } // switch Inst[22]
                                             // 18043:      }
  29, 21,                                    // 18048:      case 0x1d: {
  OPC_SwitchField, 22, 1,                    // 18050:       switch Inst[22] {
  0, 7,                                      // 18053:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 18055:        check predicate 34
  OPC_Decode, 246, 20, 162, 1,               // 18057:        decode to EORQV_VPZ_B using decoder 162
                                             // 18057:       }
  1, 0,                                      // 18062:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 18064:        check predicate 34
  OPC_Decode, 248, 20, 162, 1,               // 18066:        decode to EORQV_VPZ_H using decoder 162
                                             // 18066:       }
                                             // 18066:       } // switch Inst[22]
                                             // 18066:      }
  30, 0,                                     // 18071:      case 0x1e: {
  OPC_SwitchField, 22, 1,                    // 18073:       switch Inst[22] {
  0, 7,                                      // 18076:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 18078:        check predicate 34
  OPC_Decode, 147, 14, 162, 1,               // 18080:        decode to ANDQV_VPZ_B using decoder 162
                                             // 18080:       }
  1, 0,                                      // 18085:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 18087:        check predicate 34
  OPC_Decode, 149, 14, 162, 1,               // 18089:        decode to ANDQV_VPZ_H using decoder 162
                                             // 18089:       }
                                             // 18089:       } // switch Inst[22]
                                             // 18089:      }
                                             // 18089:      } // switch Inst[20:16]
                                             // 18089:     }
  2, 21,                                     // 18094:     case 0x2: {
  OPC_SwitchField, 22, 1,                    // 18096:      switch Inst[22] {
  0, 7,                                      // 18099:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 18101:       check predicate 33
  OPC_Decode, 240, 41, 164, 1,               // 18103:       decode to MLA_ZPmZZ_B using decoder 164
                                             // 18103:      }
  1, 0,                                      // 18108:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 18110:       check predicate 33
  OPC_Decode, 242, 41, 164, 1,               // 18112:       decode to MLA_ZPmZZ_H using decoder 164
                                             // 18112:      }
                                             // 18112:      } // switch Inst[22]
                                             // 18112:     }
  3, 21,                                     // 18117:     case 0x3: {
  OPC_SwitchField, 22, 1,                    // 18119:      switch Inst[22] {
  0, 7,                                      // 18122:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 18124:       check predicate 33
  OPC_Decode, 129, 42, 164, 1,               // 18126:       decode to MLS_ZPmZZ_B using decoder 164
                                             // 18126:      }
  1, 0,                                      // 18131:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 18133:       check predicate 33
  OPC_Decode, 131, 42, 164, 1,               // 18135:       decode to MLS_ZPmZZ_H using decoder 164
                                             // 18135:      }
                                             // 18135:      } // switch Inst[22]
                                             // 18135:     }
  4, 195, 4,                                 // 18140:     case 0x4: {
  OPC_SwitchField, 16, 5,                    // 18143:      switch Inst[20:16] {
  0, 39,                                     // 18146:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 18148:       switch Inst[22] {
  0, 25,                                     // 18151:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18153:        switch Inst[9] {
  0, 11,                                     // 18156:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 18158:         check predicate 33
  OPC_CheckField, 8, 1, 1,                   // 18160:         check Inst[8] == 0x1
  OPC_Decode, 190, 14, 165, 1,               // 18164:         decode to ASR_ZPmI_B using decoder 165
                                             // 18164:        }
  1, 0,                                      // 18169:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 18171:         check predicate 33
  OPC_Decode, 192, 14, 166, 1,               // 18173:         decode to ASR_ZPmI_H using decoder 166
                                             // 18173:        }
                                             // 18173:        } // switch Inst[9]
                                             // 18173:       }
  1, 0,                                      // 18178:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18180:        check predicate 33
  OPC_Decode, 193, 14, 167, 1,               // 18182:        decode to ASR_ZPmI_S using decoder 167
                                             // 18182:       }
                                             // 18182:       } // switch Inst[22]
                                             // 18182:      }
  1, 39,                                     // 18187:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 18189:       switch Inst[22] {
  0, 25,                                     // 18192:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18194:        switch Inst[9] {
  0, 11,                                     // 18197:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 18199:         check predicate 33
  OPC_CheckField, 8, 1, 1,                   // 18201:         check Inst[8] == 0x1
  OPC_Decode, 175, 41, 165, 1,               // 18205:         decode to LSR_ZPmI_B using decoder 165
                                             // 18205:        }
  1, 0,                                      // 18210:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 18212:         check predicate 33
  OPC_Decode, 177, 41, 166, 1,               // 18214:         decode to LSR_ZPmI_H using decoder 166
                                             // 18214:        }
                                             // 18214:        } // switch Inst[9]
                                             // 18214:       }
  1, 0,                                      // 18219:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18221:        check predicate 33
  OPC_Decode, 178, 41, 167, 1,               // 18223:        decode to LSR_ZPmI_S using decoder 167
                                             // 18223:       }
                                             // 18223:       } // switch Inst[22]
                                             // 18223:      }
  3, 39,                                     // 18228:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 18230:       switch Inst[22] {
  0, 25,                                     // 18233:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18235:        switch Inst[9] {
  0, 11,                                     // 18238:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 18240:         check predicate 33
  OPC_CheckField, 8, 1, 1,                   // 18242:         check Inst[8] == 0x1
  OPC_Decode, 151, 41, 168, 1,               // 18246:         decode to LSL_ZPmI_B using decoder 168
                                             // 18246:        }
  1, 0,                                      // 18251:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 18253:         check predicate 33
  OPC_Decode, 153, 41, 169, 1,               // 18255:         decode to LSL_ZPmI_H using decoder 169
                                             // 18255:        }
                                             // 18255:        } // switch Inst[9]
                                             // 18255:       }
  1, 0,                                      // 18260:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18262:        check predicate 33
  OPC_Decode, 154, 41, 170, 1,               // 18264:        decode to LSL_ZPmI_S using decoder 170
                                             // 18264:       }
                                             // 18264:       } // switch Inst[22]
                                             // 18264:      }
  4, 39,                                     // 18269:      case 0x4: {
  OPC_SwitchField, 22, 1,                    // 18271:       switch Inst[22] {
  0, 25,                                     // 18274:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18276:        switch Inst[9] {
  0, 11,                                     // 18279:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 18281:         check predicate 33
  OPC_CheckField, 8, 1, 1,                   // 18283:         check Inst[8] == 0x1
  OPC_Decode, 174, 14, 165, 1,               // 18287:         decode to ASRD_ZPmI_B using decoder 165
                                             // 18287:        }
  1, 0,                                      // 18292:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 18294:         check predicate 33
  OPC_Decode, 176, 14, 166, 1,               // 18296:         decode to ASRD_ZPmI_H using decoder 166
                                             // 18296:        }
                                             // 18296:        } // switch Inst[9]
                                             // 18296:       }
  1, 0,                                      // 18301:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18303:        check predicate 33
  OPC_Decode, 177, 14, 167, 1,               // 18305:        decode to ASRD_ZPmI_S using decoder 167
                                             // 18305:       }
                                             // 18305:       } // switch Inst[22]
                                             // 18305:      }
  6, 39,                                     // 18310:      case 0x6: {
  OPC_SwitchField, 22, 1,                    // 18312:       switch Inst[22] {
  0, 25,                                     // 18315:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18317:        switch Inst[9] {
  0, 11,                                     // 18320:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 18322:         check predicate 35
  OPC_CheckField, 8, 1, 1,                   // 18324:         check Inst[8] == 0x1
  OPC_Decode, 236, 53, 168, 1,               // 18328:         decode to SQSHL_ZPmI_B using decoder 168
                                             // 18328:        }
  1, 0,                                      // 18333:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 18335:         check predicate 35
  OPC_Decode, 238, 53, 169, 1,               // 18337:         decode to SQSHL_ZPmI_H using decoder 169
                                             // 18337:        }
                                             // 18337:        } // switch Inst[9]
                                             // 18337:       }
  1, 0,                                      // 18342:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 18344:        check predicate 35
  OPC_Decode, 239, 53, 170, 1,               // 18346:        decode to SQSHL_ZPmI_S using decoder 170
                                             // 18346:       }
                                             // 18346:       } // switch Inst[22]
                                             // 18346:      }
  7, 39,                                     // 18351:      case 0x7: {
  OPC_SwitchField, 22, 1,                    // 18353:       switch Inst[22] {
  0, 25,                                     // 18356:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18358:        switch Inst[9] {
  0, 11,                                     // 18361:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 18363:         check predicate 35
  OPC_CheckField, 8, 1, 1,                   // 18365:         check Inst[8] == 0x1
  OPC_Decode, 236, 66, 168, 1,               // 18369:         decode to UQSHL_ZPmI_B using decoder 168
                                             // 18369:        }
  1, 0,                                      // 18374:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 18376:         check predicate 35
  OPC_Decode, 238, 66, 169, 1,               // 18378:         decode to UQSHL_ZPmI_H using decoder 169
                                             // 18378:        }
                                             // 18378:        } // switch Inst[9]
                                             // 18378:       }
  1, 0,                                      // 18383:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 18385:        check predicate 35
  OPC_Decode, 239, 66, 170, 1,               // 18387:        decode to UQSHL_ZPmI_S using decoder 170
                                             // 18387:       }
                                             // 18387:       } // switch Inst[22]
                                             // 18387:      }
  12, 39,                                    // 18392:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 18394:       switch Inst[22] {
  0, 25,                                     // 18397:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18399:        switch Inst[9] {
  0, 11,                                     // 18402:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 18404:         check predicate 35
  OPC_CheckField, 8, 1, 1,                   // 18406:         check Inst[8] == 0x1
  OPC_Decode, 155, 55, 165, 1,               // 18410:         decode to SRSHR_ZPmI_B using decoder 165
                                             // 18410:        }
  1, 0,                                      // 18415:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 18417:         check predicate 35
  OPC_Decode, 157, 55, 166, 1,               // 18419:         decode to SRSHR_ZPmI_H using decoder 166
                                             // 18419:        }
                                             // 18419:        } // switch Inst[9]
                                             // 18419:       }
  1, 0,                                      // 18424:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 18426:        check predicate 35
  OPC_Decode, 158, 55, 167, 1,               // 18428:        decode to SRSHR_ZPmI_S using decoder 167
                                             // 18428:       }
                                             // 18428:       } // switch Inst[22]
                                             // 18428:      }
  13, 39,                                    // 18433:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 18435:       switch Inst[22] {
  0, 25,                                     // 18438:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18440:        switch Inst[9] {
  0, 11,                                     // 18443:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 18445:         check predicate 35
  OPC_CheckField, 8, 1, 1,                   // 18447:         check Inst[8] == 0x1
  OPC_Decode, 243, 67, 165, 1,               // 18451:         decode to URSHR_ZPmI_B using decoder 165
                                             // 18451:        }
  1, 0,                                      // 18456:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 18458:         check predicate 35
  OPC_Decode, 245, 67, 166, 1,               // 18460:         decode to URSHR_ZPmI_H using decoder 166
                                             // 18460:        }
                                             // 18460:        } // switch Inst[9]
                                             // 18460:       }
  1, 0,                                      // 18465:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 18467:        check predicate 35
  OPC_Decode, 246, 67, 167, 1,               // 18469:        decode to URSHR_ZPmI_S using decoder 167
                                             // 18469:       }
                                             // 18469:       } // switch Inst[22]
                                             // 18469:      }
  15, 39,                                    // 18474:      case 0xf: {
  OPC_SwitchField, 22, 1,                    // 18476:       switch Inst[22] {
  0, 25,                                     // 18479:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 18481:        switch Inst[9] {
  0, 11,                                     // 18484:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 18486:         check predicate 35
  OPC_CheckField, 8, 1, 1,                   // 18488:         check Inst[8] == 0x1
  OPC_Decode, 221, 53, 168, 1,               // 18492:         decode to SQSHLU_ZPmI_B using decoder 168
                                             // 18492:        }
  1, 0,                                      // 18497:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 18499:         check predicate 35
  OPC_Decode, 223, 53, 169, 1,               // 18501:         decode to SQSHLU_ZPmI_H using decoder 169
                                             // 18501:        }
                                             // 18501:        } // switch Inst[9]
                                             // 18501:       }
  1, 0,                                      // 18506:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 18508:        check predicate 35
  OPC_Decode, 224, 53, 170, 1,               // 18510:        decode to SQSHLU_ZPmI_S using decoder 170
                                             // 18510:       }
                                             // 18510:       } // switch Inst[22]
                                             // 18510:      }
  16, 21,                                    // 18515:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 18517:       switch Inst[22] {
  0, 7,                                      // 18520:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18522:        check predicate 33
  OPC_Decode, 194, 14, 160, 1,               // 18524:        decode to ASR_ZPmZ_B using decoder 160
                                             // 18524:       }
  1, 0,                                      // 18529:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18531:        check predicate 33
  OPC_Decode, 196, 14, 160, 1,               // 18533:        decode to ASR_ZPmZ_H using decoder 160
                                             // 18533:       }
                                             // 18533:       } // switch Inst[22]
                                             // 18533:      }
  17, 21,                                    // 18538:      case 0x11: {
  OPC_SwitchField, 22, 1,                    // 18540:       switch Inst[22] {
  0, 7,                                      // 18543:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18545:        check predicate 33
  OPC_Decode, 179, 41, 160, 1,               // 18547:        decode to LSR_ZPmZ_B using decoder 160
                                             // 18547:       }
  1, 0,                                      // 18552:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18554:        check predicate 33
  OPC_Decode, 181, 41, 160, 1,               // 18556:        decode to LSR_ZPmZ_H using decoder 160
                                             // 18556:       }
                                             // 18556:       } // switch Inst[22]
                                             // 18556:      }
  19, 21,                                    // 18561:      case 0x13: {
  OPC_SwitchField, 22, 1,                    // 18563:       switch Inst[22] {
  0, 7,                                      // 18566:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18568:        check predicate 33
  OPC_Decode, 155, 41, 160, 1,               // 18570:        decode to LSL_ZPmZ_B using decoder 160
                                             // 18570:       }
  1, 0,                                      // 18575:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18577:        check predicate 33
  OPC_Decode, 157, 41, 160, 1,               // 18579:        decode to LSL_ZPmZ_H using decoder 160
                                             // 18579:       }
                                             // 18579:       } // switch Inst[22]
                                             // 18579:      }
  20, 21,                                    // 18584:      case 0x14: {
  OPC_SwitchField, 22, 1,                    // 18586:       switch Inst[22] {
  0, 7,                                      // 18589:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18591:        check predicate 33
  OPC_Decode, 178, 14, 160, 1,               // 18593:        decode to ASRR_ZPmZ_B using decoder 160
                                             // 18593:       }
  1, 0,                                      // 18598:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18600:        check predicate 33
  OPC_Decode, 180, 14, 160, 1,               // 18602:        decode to ASRR_ZPmZ_H using decoder 160
                                             // 18602:       }
                                             // 18602:       } // switch Inst[22]
                                             // 18602:      }
  21, 21,                                    // 18607:      case 0x15: {
  OPC_SwitchField, 22, 1,                    // 18609:       switch Inst[22] {
  0, 7,                                      // 18612:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18614:        check predicate 33
  OPC_Decode, 163, 41, 160, 1,               // 18616:        decode to LSRR_ZPmZ_B using decoder 160
                                             // 18616:       }
  1, 0,                                      // 18621:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18623:        check predicate 33
  OPC_Decode, 165, 41, 160, 1,               // 18625:        decode to LSRR_ZPmZ_H using decoder 160
                                             // 18625:       }
                                             // 18625:       } // switch Inst[22]
                                             // 18625:      }
  23, 21,                                    // 18630:      case 0x17: {
  OPC_SwitchField, 22, 1,                    // 18632:       switch Inst[22] {
  0, 7,                                      // 18635:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18637:        check predicate 33
  OPC_Decode, 139, 41, 160, 1,               // 18639:        decode to LSLR_ZPmZ_B using decoder 160
                                             // 18639:       }
  1, 0,                                      // 18644:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18646:        check predicate 33
  OPC_Decode, 141, 41, 160, 1,               // 18648:        decode to LSLR_ZPmZ_H using decoder 160
                                             // 18648:       }
                                             // 18648:       } // switch Inst[22]
                                             // 18648:      }
  24, 21,                                    // 18653:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 18655:       switch Inst[22] {
  0, 7,                                      // 18658:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18660:        check predicate 33
  OPC_Decode, 184, 14, 160, 1,               // 18662:        decode to ASR_WIDE_ZPmZ_B using decoder 160
                                             // 18662:       }
  1, 0,                                      // 18667:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18669:        check predicate 33
  OPC_Decode, 185, 14, 160, 1,               // 18671:        decode to ASR_WIDE_ZPmZ_H using decoder 160
                                             // 18671:       }
                                             // 18671:       } // switch Inst[22]
                                             // 18671:      }
  25, 21,                                    // 18676:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 18678:       switch Inst[22] {
  0, 7,                                      // 18681:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18683:        check predicate 33
  OPC_Decode, 169, 41, 160, 1,               // 18685:        decode to LSR_WIDE_ZPmZ_B using decoder 160
                                             // 18685:       }
  1, 0,                                      // 18690:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18692:        check predicate 33
  OPC_Decode, 170, 41, 160, 1,               // 18694:        decode to LSR_WIDE_ZPmZ_H using decoder 160
                                             // 18694:       }
                                             // 18694:       } // switch Inst[22]
                                             // 18694:      }
  27, 0,                                     // 18699:      case 0x1b: {
  OPC_SwitchField, 22, 1,                    // 18701:       switch Inst[22] {
  0, 7,                                      // 18704:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18706:        check predicate 33
  OPC_Decode, 145, 41, 160, 1,               // 18708:        decode to LSL_WIDE_ZPmZ_B using decoder 160
                                             // 18708:       }
  1, 0,                                      // 18713:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18715:        check predicate 33
  OPC_Decode, 146, 41, 160, 1,               // 18717:        decode to LSL_WIDE_ZPmZ_H using decoder 160
                                             // 18717:       }
                                             // 18717:       } // switch Inst[22]
                                             // 18717:      }
                                             // 18717:      } // switch Inst[20:16]
                                             // 18717:     }
  5, 173, 3,                                 // 18722:     case 0x5: {
  OPC_SwitchField, 16, 5,                    // 18725:      switch Inst[20:16] {
  0, 11,                                     // 18728:      case 0x0: {
  OPC_CheckPredicate, 36,                    // 18730:       check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 18732:       check Inst[22] == 0x1
  OPC_Decode, 169, 61, 161, 1,               // 18736:       decode to SXTB_ZPzZ_H using decoder 161
                                             // 18736:      }
  1, 11,                                     // 18741:      case 0x1: {
  OPC_CheckPredicate, 36,                    // 18743:       check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 18745:       check Inst[22] == 0x1
  OPC_Decode, 159, 69, 161, 1,               // 18749:       decode to UXTB_ZPzZ_H using decoder 161
                                             // 18749:      }
  6, 21,                                     // 18754:      case 0x6: {
  OPC_SwitchField, 22, 1,                    // 18756:       switch Inst[22] {
  0, 7,                                      // 18759:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18761:        check predicate 36
  OPC_Decode, 242, 12, 161, 1,               // 18763:        decode to ABS_ZPzZ_B using decoder 161
                                             // 18763:       }
  1, 0,                                      // 18768:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18770:        check predicate 36
  OPC_Decode, 244, 12, 161, 1,               // 18772:        decode to ABS_ZPzZ_H using decoder 161
                                             // 18772:       }
                                             // 18772:       } // switch Inst[22]
                                             // 18772:      }
  7, 21,                                     // 18777:      case 0x7: {
  OPC_SwitchField, 22, 1,                    // 18779:       switch Inst[22] {
  0, 7,                                      // 18782:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18784:        check predicate 36
  OPC_Decode, 166, 43, 161, 1,               // 18786:        decode to NEG_ZPzZ_B using decoder 161
                                             // 18786:       }
  1, 0,                                      // 18791:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18793:        check predicate 36
  OPC_Decode, 168, 43, 161, 1,               // 18795:        decode to NEG_ZPzZ_H using decoder 161
                                             // 18795:       }
                                             // 18795:       } // switch Inst[22]
                                             // 18795:      }
  8, 21,                                     // 18800:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 18802:       switch Inst[22] {
  0, 7,                                      // 18805:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18807:        check predicate 36
  OPC_Decode, 181, 17, 161, 1,               // 18809:        decode to CLS_ZPzZ_B using decoder 161
                                             // 18809:       }
  1, 0,                                      // 18814:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18816:        check predicate 36
  OPC_Decode, 183, 17, 161, 1,               // 18818:        decode to CLS_ZPzZ_H using decoder 161
                                             // 18818:       }
                                             // 18818:       } // switch Inst[22]
                                             // 18818:      }
  9, 21,                                     // 18823:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 18825:       switch Inst[22] {
  0, 7,                                      // 18828:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18830:        check predicate 36
  OPC_Decode, 197, 17, 161, 1,               // 18832:        decode to CLZ_ZPzZ_B using decoder 161
                                             // 18832:       }
  1, 0,                                      // 18837:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18839:        check predicate 36
  OPC_Decode, 199, 17, 161, 1,               // 18841:        decode to CLZ_ZPzZ_H using decoder 161
                                             // 18841:       }
                                             // 18841:       } // switch Inst[22]
                                             // 18841:      }
  10, 21,                                    // 18846:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 18848:       switch Inst[22] {
  0, 7,                                      // 18851:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18853:        check predicate 36
  OPC_Decode, 165, 19, 161, 1,               // 18855:        decode to CNT_ZPzZ_B using decoder 161
                                             // 18855:       }
  1, 0,                                      // 18860:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18862:        check predicate 36
  OPC_Decode, 167, 19, 161, 1,               // 18864:        decode to CNT_ZPzZ_H using decoder 161
                                             // 18864:       }
                                             // 18864:       } // switch Inst[22]
                                             // 18864:      }
  11, 21,                                    // 18869:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 18871:       switch Inst[22] {
  0, 7,                                      // 18874:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18876:        check predicate 36
  OPC_Decode, 143, 19, 161, 1,               // 18878:        decode to CNOT_ZPzZ_B using decoder 161
                                             // 18878:       }
  1, 0,                                      // 18883:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18885:        check predicate 36
  OPC_Decode, 145, 19, 161, 1,               // 18887:        decode to CNOT_ZPzZ_H using decoder 161
                                             // 18887:       }
                                             // 18887:       } // switch Inst[22]
                                             // 18887:      }
  12, 11,                                    // 18892:      case 0xc: {
  OPC_CheckPredicate, 36,                    // 18894:       check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 18896:       check Inst[22] == 0x1
  OPC_Decode, 198, 21, 161, 1,               // 18900:       decode to FABS_ZPzZ_H using decoder 161
                                             // 18900:      }
  13, 11,                                    // 18905:      case 0xd: {
  OPC_CheckPredicate, 36,                    // 18907:       check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 18909:       check Inst[22] == 0x1
  OPC_Decode, 162, 30, 161, 1,               // 18913:       decode to FNEG_ZPzZ_H using decoder 161
                                             // 18913:      }
  14, 21,                                    // 18918:      case 0xe: {
  OPC_SwitchField, 22, 1,                    // 18920:       switch Inst[22] {
  0, 7,                                      // 18923:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 18925:        check predicate 36
  OPC_Decode, 187, 43, 161, 1,               // 18927:        decode to NOT_ZPzZ_B using decoder 161
                                             // 18927:       }
  1, 0,                                      // 18932:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 18934:        check predicate 36
  OPC_Decode, 189, 43, 161, 1,               // 18936:        decode to NOT_ZPzZ_H using decoder 161
                                             // 18936:       }
                                             // 18936:       } // switch Inst[22]
                                             // 18936:      }
  16, 11,                                    // 18941:      case 0x10: {
  OPC_CheckPredicate, 33,                    // 18943:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 18945:       check Inst[22] == 0x1
  OPC_Decode, 166, 61, 163, 1,               // 18949:       decode to SXTB_ZPmZ_H using decoder 163
                                             // 18949:      }
  17, 11,                                    // 18954:      case 0x11: {
  OPC_CheckPredicate, 33,                    // 18956:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 18958:       check Inst[22] == 0x1
  OPC_Decode, 156, 69, 163, 1,               // 18962:       decode to UXTB_ZPmZ_H using decoder 163
                                             // 18962:      }
  22, 21,                                    // 18967:      case 0x16: {
  OPC_SwitchField, 22, 1,                    // 18969:       switch Inst[22] {
  0, 7,                                      // 18972:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18974:        check predicate 33
  OPC_Decode, 238, 12, 163, 1,               // 18976:        decode to ABS_ZPmZ_B using decoder 163
                                             // 18976:       }
  1, 0,                                      // 18981:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 18983:        check predicate 33
  OPC_Decode, 240, 12, 163, 1,               // 18985:        decode to ABS_ZPmZ_H using decoder 163
                                             // 18985:       }
                                             // 18985:       } // switch Inst[22]
                                             // 18985:      }
  23, 21,                                    // 18990:      case 0x17: {
  OPC_SwitchField, 22, 1,                    // 18992:       switch Inst[22] {
  0, 7,                                      // 18995:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 18997:        check predicate 33
  OPC_Decode, 162, 43, 163, 1,               // 18999:        decode to NEG_ZPmZ_B using decoder 163
                                             // 18999:       }
  1, 0,                                      // 19004:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19006:        check predicate 33
  OPC_Decode, 164, 43, 163, 1,               // 19008:        decode to NEG_ZPmZ_H using decoder 163
                                             // 19008:       }
                                             // 19008:       } // switch Inst[22]
                                             // 19008:      }
  24, 21,                                    // 19013:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 19015:       switch Inst[22] {
  0, 7,                                      // 19018:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19020:        check predicate 33
  OPC_Decode, 177, 17, 163, 1,               // 19022:        decode to CLS_ZPmZ_B using decoder 163
                                             // 19022:       }
  1, 0,                                      // 19027:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19029:        check predicate 33
  OPC_Decode, 179, 17, 163, 1,               // 19031:        decode to CLS_ZPmZ_H using decoder 163
                                             // 19031:       }
                                             // 19031:       } // switch Inst[22]
                                             // 19031:      }
  25, 21,                                    // 19036:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 19038:       switch Inst[22] {
  0, 7,                                      // 19041:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19043:        check predicate 33
  OPC_Decode, 193, 17, 163, 1,               // 19045:        decode to CLZ_ZPmZ_B using decoder 163
                                             // 19045:       }
  1, 0,                                      // 19050:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19052:        check predicate 33
  OPC_Decode, 195, 17, 163, 1,               // 19054:        decode to CLZ_ZPmZ_H using decoder 163
                                             // 19054:       }
                                             // 19054:       } // switch Inst[22]
                                             // 19054:      }
  26, 21,                                    // 19059:      case 0x1a: {
  OPC_SwitchField, 22, 1,                    // 19061:       switch Inst[22] {
  0, 7,                                      // 19064:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19066:        check predicate 33
  OPC_Decode, 161, 19, 163, 1,               // 19068:        decode to CNT_ZPmZ_B using decoder 163
                                             // 19068:       }
  1, 0,                                      // 19073:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19075:        check predicate 33
  OPC_Decode, 163, 19, 163, 1,               // 19077:        decode to CNT_ZPmZ_H using decoder 163
                                             // 19077:       }
                                             // 19077:       } // switch Inst[22]
                                             // 19077:      }
  27, 21,                                    // 19082:      case 0x1b: {
  OPC_SwitchField, 22, 1,                    // 19084:       switch Inst[22] {
  0, 7,                                      // 19087:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19089:        check predicate 33
  OPC_Decode, 139, 19, 163, 1,               // 19091:        decode to CNOT_ZPmZ_B using decoder 163
                                             // 19091:       }
  1, 0,                                      // 19096:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19098:        check predicate 33
  OPC_Decode, 141, 19, 163, 1,               // 19100:        decode to CNOT_ZPmZ_H using decoder 163
                                             // 19100:       }
                                             // 19100:       } // switch Inst[22]
                                             // 19100:      }
  28, 11,                                    // 19105:      case 0x1c: {
  OPC_CheckPredicate, 33,                    // 19107:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 19109:       check Inst[22] == 0x1
  OPC_Decode, 195, 21, 163, 1,               // 19113:       decode to FABS_ZPmZ_H using decoder 163
                                             // 19113:      }
  29, 11,                                    // 19118:      case 0x1d: {
  OPC_CheckPredicate, 33,                    // 19120:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 19122:       check Inst[22] == 0x1
  OPC_Decode, 159, 30, 163, 1,               // 19126:       decode to FNEG_ZPmZ_H using decoder 163
                                             // 19126:      }
  30, 0,                                     // 19131:      case 0x1e: {
  OPC_SwitchField, 22, 1,                    // 19133:       switch Inst[22] {
  0, 7,                                      // 19136:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19138:        check predicate 33
  OPC_Decode, 183, 43, 163, 1,               // 19140:        decode to NOT_ZPmZ_B using decoder 163
                                             // 19140:       }
  1, 0,                                      // 19145:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19147:        check predicate 33
  OPC_Decode, 185, 43, 163, 1,               // 19149:        decode to NOT_ZPmZ_H using decoder 163
                                             // 19149:       }
                                             // 19149:       } // switch Inst[22]
                                             // 19149:      }
                                             // 19149:      } // switch Inst[20:16]
                                             // 19149:     }
  6, 21,                                     // 19154:     case 0x6: {
  OPC_SwitchField, 22, 1,                    // 19156:      switch Inst[22] {
  0, 7,                                      // 19159:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 19161:       check predicate 33
  OPC_Decode, 233, 41, 171, 1,               // 19163:       decode to MAD_ZPmZZ_B using decoder 171
                                             // 19163:      }
  1, 0,                                      // 19168:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 19170:       check predicate 33
  OPC_Decode, 235, 41, 171, 1,               // 19172:       decode to MAD_ZPmZZ_H using decoder 171
                                             // 19172:      }
                                             // 19172:      } // switch Inst[22]
                                             // 19172:     }
  7, 0,                                      // 19177:     case 0x7: {
  OPC_SwitchField, 22, 1,                    // 19179:      switch Inst[22] {
  0, 7,                                      // 19182:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 19184:       check predicate 33
  OPC_Decode, 244, 42, 171, 1,               // 19186:       decode to MSB_ZPmZZ_B using decoder 171
                                             // 19186:      }
  1, 0,                                      // 19191:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 19193:       check predicate 33
  OPC_Decode, 246, 42, 171, 1,               // 19195:       decode to MSB_ZPmZZ_H using decoder 171
                                             // 19195:      }
                                             // 19195:      } // switch Inst[22]
                                             // 19195:     }
                                             // 19195:     } // switch Inst[15:13]
                                             // 19195:    }
  1, 148, 15,                                // 19200:    case 0x1: {
  OPC_SwitchField, 13, 3,                    // 19203:     switch Inst[15:13] {
  0, 233, 3,                                 // 19206:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 19209:      switch Inst[20:16] {
  0, 21,                                     // 19212:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 19214:       switch Inst[22] {
  0, 7,                                      // 19217:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19219:        check predicate 33
  OPC_Decode, 227, 13, 160, 1,               // 19221:        decode to ADD_ZPmZ_S using decoder 160
                                             // 19221:       }
  1, 0,                                      // 19226:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19228:        check predicate 33
  OPC_Decode, 225, 13, 160, 1,               // 19230:        decode to ADD_ZPmZ_D using decoder 160
                                             // 19230:       }
                                             // 19230:       } // switch Inst[22]
                                             // 19230:      }
  1, 21,                                     // 19235:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 19237:       switch Inst[22] {
  0, 7,                                      // 19240:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19242:        check predicate 33
  OPC_Decode, 186, 60, 160, 1,               // 19244:        decode to SUB_ZPmZ_S using decoder 160
                                             // 19244:       }
  1, 0,                                      // 19249:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19251:        check predicate 33
  OPC_Decode, 184, 60, 160, 1,               // 19253:        decode to SUB_ZPmZ_D using decoder 160
                                             // 19253:       }
                                             // 19253:       } // switch Inst[22]
                                             // 19253:      }
  3, 21,                                     // 19258:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 19260:       switch Inst[22] {
  0, 7,                                      // 19263:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19265:        check predicate 33
  OPC_Decode, 151, 60, 160, 1,               // 19267:        decode to SUBR_ZPmZ_S using decoder 160
                                             // 19267:       }
  1, 0,                                      // 19272:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19274:        check predicate 33
  OPC_Decode, 149, 60, 160, 1,               // 19276:        decode to SUBR_ZPmZ_D using decoder 160
                                             // 19276:       }
                                             // 19276:       } // switch Inst[22]
                                             // 19276:      }
  4, 11,                                     // 19281:      case 0x4: {
  OPC_CheckPredicate, 37,                    // 19283:       check predicate 37
  OPC_CheckField, 22, 1, 1,                  // 19285:       check Inst[22] == 0x1
  OPC_Decode, 224, 13, 160, 1,               // 19289:       decode to ADD_ZPmZ_CPA using decoder 160
                                             // 19289:      }
  5, 11,                                     // 19294:      case 0x5: {
  OPC_CheckPredicate, 37,                    // 19296:       check predicate 37
  OPC_CheckField, 22, 1, 1,                  // 19298:       check Inst[22] == 0x1
  OPC_Decode, 183, 60, 160, 1,               // 19302:       decode to SUB_ZPmZ_CPA using decoder 160
                                             // 19302:      }
  8, 21,                                     // 19307:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 19309:       switch Inst[22] {
  0, 7,                                      // 19312:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19314:        check predicate 33
  OPC_Decode, 142, 49, 160, 1,               // 19316:        decode to SMAX_ZPmZ_S using decoder 160
                                             // 19316:       }
  1, 0,                                      // 19321:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19323:        check predicate 33
  OPC_Decode, 140, 49, 160, 1,               // 19325:        decode to SMAX_ZPmZ_D using decoder 160
                                             // 19325:       }
                                             // 19325:       } // switch Inst[22]
                                             // 19325:      }
  9, 21,                                     // 19330:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 19332:       switch Inst[22] {
  0, 7,                                      // 19335:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19337:        check predicate 33
  OPC_Decode, 152, 64, 160, 1,               // 19339:        decode to UMAX_ZPmZ_S using decoder 160
                                             // 19339:       }
  1, 0,                                      // 19344:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19346:        check predicate 33
  OPC_Decode, 150, 64, 160, 1,               // 19348:        decode to UMAX_ZPmZ_D using decoder 160
                                             // 19348:       }
                                             // 19348:       } // switch Inst[22]
                                             // 19348:      }
  10, 21,                                    // 19353:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 19355:       switch Inst[22] {
  0, 7,                                      // 19358:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19360:        check predicate 33
  OPC_Decode, 200, 49, 160, 1,               // 19362:        decode to SMIN_ZPmZ_S using decoder 160
                                             // 19362:       }
  1, 0,                                      // 19367:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19369:        check predicate 33
  OPC_Decode, 198, 49, 160, 1,               // 19371:        decode to SMIN_ZPmZ_D using decoder 160
                                             // 19371:       }
                                             // 19371:       } // switch Inst[22]
                                             // 19371:      }
  11, 21,                                    // 19376:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 19378:       switch Inst[22] {
  0, 7,                                      // 19381:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19383:        check predicate 33
  OPC_Decode, 209, 64, 160, 1,               // 19385:        decode to UMIN_ZPmZ_S using decoder 160
                                             // 19385:       }
  1, 0,                                      // 19390:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19392:        check predicate 33
  OPC_Decode, 207, 64, 160, 1,               // 19394:        decode to UMIN_ZPmZ_D using decoder 160
                                             // 19394:       }
                                             // 19394:       } // switch Inst[22]
                                             // 19394:      }
  12, 21,                                    // 19399:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 19401:       switch Inst[22] {
  0, 7,                                      // 19404:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19406:        check predicate 33
  OPC_Decode, 169, 46, 160, 1,               // 19408:        decode to SABD_ZPmZ_S using decoder 160
                                             // 19408:       }
  1, 0,                                      // 19413:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19415:        check predicate 33
  OPC_Decode, 167, 46, 160, 1,               // 19417:        decode to SABD_ZPmZ_D using decoder 160
                                             // 19417:       }
                                             // 19417:       } // switch Inst[22]
                                             // 19417:      }
  13, 21,                                    // 19422:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 19424:       switch Inst[22] {
  0, 7,                                      // 19427:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19429:        check predicate 33
  OPC_Decode, 173, 62, 160, 1,               // 19431:        decode to UABD_ZPmZ_S using decoder 160
                                             // 19431:       }
  1, 0,                                      // 19436:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19438:        check predicate 33
  OPC_Decode, 171, 62, 160, 1,               // 19440:        decode to UABD_ZPmZ_D using decoder 160
                                             // 19440:       }
                                             // 19440:       } // switch Inst[22]
                                             // 19440:      }
  16, 21,                                    // 19445:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 19447:       switch Inst[22] {
  0, 7,                                      // 19450:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19452:        check predicate 33
  OPC_Decode, 135, 43, 160, 1,               // 19454:        decode to MUL_ZPmZ_S using decoder 160
                                             // 19454:       }
  1, 0,                                      // 19459:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19461:        check predicate 33
  OPC_Decode, 133, 43, 160, 1,               // 19463:        decode to MUL_ZPmZ_D using decoder 160
                                             // 19463:       }
                                             // 19463:       } // switch Inst[22]
                                             // 19463:      }
  18, 21,                                    // 19468:      case 0x12: {
  OPC_SwitchField, 22, 1,                    // 19470:       switch Inst[22] {
  0, 7,                                      // 19473:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19475:        check predicate 33
  OPC_Decode, 213, 50, 160, 1,               // 19477:        decode to SMULH_ZPmZ_S using decoder 160
                                             // 19477:       }
  1, 0,                                      // 19482:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19484:        check predicate 33
  OPC_Decode, 211, 50, 160, 1,               // 19486:        decode to SMULH_ZPmZ_D using decoder 160
                                             // 19486:       }
                                             // 19486:       } // switch Inst[22]
                                             // 19486:      }
  19, 21,                                    // 19491:      case 0x13: {
  OPC_SwitchField, 22, 1,                    // 19493:       switch Inst[22] {
  0, 7,                                      // 19496:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19498:        check predicate 33
  OPC_Decode, 220, 65, 160, 1,               // 19500:        decode to UMULH_ZPmZ_S using decoder 160
                                             // 19500:       }
  1, 0,                                      // 19505:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19507:        check predicate 33
  OPC_Decode, 218, 65, 160, 1,               // 19509:        decode to UMULH_ZPmZ_D using decoder 160
                                             // 19509:       }
                                             // 19509:       } // switch Inst[22]
                                             // 19509:      }
  20, 21,                                    // 19514:      case 0x14: {
  OPC_SwitchField, 22, 1,                    // 19516:       switch Inst[22] {
  0, 7,                                      // 19519:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19521:        check predicate 33
  OPC_Decode, 180, 47, 160, 1,               // 19523:        decode to SDIV_ZPmZ_S using decoder 160
                                             // 19523:       }
  1, 0,                                      // 19528:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19530:        check predicate 33
  OPC_Decode, 179, 47, 160, 1,               // 19532:        decode to SDIV_ZPmZ_D using decoder 160
                                             // 19532:       }
                                             // 19532:       } // switch Inst[22]
                                             // 19532:      }
  21, 21,                                    // 19537:      case 0x15: {
  OPC_SwitchField, 22, 1,                    // 19539:       switch Inst[22] {
  0, 7,                                      // 19542:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19544:        check predicate 33
  OPC_Decode, 174, 63, 160, 1,               // 19546:        decode to UDIV_ZPmZ_S using decoder 160
                                             // 19546:       }
  1, 0,                                      // 19551:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19553:        check predicate 33
  OPC_Decode, 173, 63, 160, 1,               // 19555:        decode to UDIV_ZPmZ_D using decoder 160
                                             // 19555:       }
                                             // 19555:       } // switch Inst[22]
                                             // 19555:      }
  22, 21,                                    // 19560:      case 0x16: {
  OPC_SwitchField, 22, 1,                    // 19562:       switch Inst[22] {
  0, 7,                                      // 19565:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19567:        check predicate 33
  OPC_Decode, 176, 47, 160, 1,               // 19569:        decode to SDIVR_ZPmZ_S using decoder 160
                                             // 19569:       }
  1, 0,                                      // 19574:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19576:        check predicate 33
  OPC_Decode, 175, 47, 160, 1,               // 19578:        decode to SDIVR_ZPmZ_D using decoder 160
                                             // 19578:       }
                                             // 19578:       } // switch Inst[22]
                                             // 19578:      }
  23, 21,                                    // 19583:      case 0x17: {
  OPC_SwitchField, 22, 1,                    // 19585:       switch Inst[22] {
  0, 7,                                      // 19588:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19590:        check predicate 33
  OPC_Decode, 170, 63, 160, 1,               // 19592:        decode to UDIVR_ZPmZ_S using decoder 160
                                             // 19592:       }
  1, 0,                                      // 19597:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19599:        check predicate 33
  OPC_Decode, 169, 63, 160, 1,               // 19601:        decode to UDIVR_ZPmZ_D using decoder 160
                                             // 19601:       }
                                             // 19601:       } // switch Inst[22]
                                             // 19601:      }
  24, 21,                                    // 19606:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 19608:       switch Inst[22] {
  0, 7,                                      // 19611:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19613:        check predicate 33
  OPC_Decode, 213, 43, 160, 1,               // 19615:        decode to ORR_ZPmZ_S using decoder 160
                                             // 19615:       }
  1, 0,                                      // 19620:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19622:        check predicate 33
  OPC_Decode, 211, 43, 160, 1,               // 19624:        decode to ORR_ZPmZ_D using decoder 160
                                             // 19624:       }
                                             // 19624:       } // switch Inst[22]
                                             // 19624:      }
  25, 21,                                    // 19629:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 19631:       switch Inst[22] {
  0, 7,                                      // 19634:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19636:        check predicate 33
  OPC_Decode, 140, 21, 160, 1,               // 19638:        decode to EOR_ZPmZ_S using decoder 160
                                             // 19638:       }
  1, 0,                                      // 19643:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19645:        check predicate 33
  OPC_Decode, 138, 21, 160, 1,               // 19647:        decode to EOR_ZPmZ_D using decoder 160
                                             // 19647:       }
                                             // 19647:       } // switch Inst[22]
                                             // 19647:      }
  26, 21,                                    // 19652:      case 0x1a: {
  OPC_SwitchField, 22, 1,                    // 19654:       switch Inst[22] {
  0, 7,                                      // 19657:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19659:        check predicate 33
  OPC_Decode, 169, 14, 160, 1,               // 19661:        decode to AND_ZPmZ_S using decoder 160
                                             // 19661:       }
  1, 0,                                      // 19666:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19668:        check predicate 33
  OPC_Decode, 167, 14, 160, 1,               // 19670:        decode to AND_ZPmZ_D using decoder 160
                                             // 19670:       }
                                             // 19670:       } // switch Inst[22]
                                             // 19670:      }
  27, 0,                                     // 19675:      case 0x1b: {
  OPC_SwitchField, 22, 1,                    // 19677:       switch Inst[22] {
  0, 7,                                      // 19680:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19682:        check predicate 33
  OPC_Decode, 142, 16, 160, 1,               // 19684:        decode to BIC_ZPmZ_S using decoder 160
                                             // 19684:       }
  1, 0,                                      // 19689:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19691:        check predicate 33
  OPC_Decode, 140, 16, 160, 1,               // 19693:        decode to BIC_ZPmZ_D using decoder 160
                                             // 19693:       }
                                             // 19693:       } // switch Inst[22]
                                             // 19693:      }
                                             // 19693:      } // switch Inst[20:16]
                                             // 19693:     }
  1, 174, 3,                                 // 19698:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 19701:      switch Inst[20:16] {
  0, 11,                                     // 19704:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 19706:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 19708:       check Inst[22] == 0x0
  OPC_Decode, 213, 46, 161, 1,               // 19712:       decode to SADDV_VPZ_S using decoder 161
                                             // 19712:      }
  1, 21,                                     // 19717:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 19719:       switch Inst[22] {
  0, 7,                                      // 19722:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19724:        check predicate 33
  OPC_Decode, 215, 62, 161, 1,               // 19726:        decode to UADDV_VPZ_S using decoder 161
                                             // 19726:       }
  1, 0,                                      // 19731:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19733:        check predicate 33
  OPC_Decode, 213, 62, 161, 1,               // 19735:        decode to UADDV_VPZ_D using decoder 161
                                             // 19735:       }
                                             // 19735:       } // switch Inst[22]
                                             // 19735:      }
  5, 21,                                     // 19740:      case 0x5: {
  OPC_SwitchField, 22, 1,                    // 19742:       switch Inst[22] {
  0, 7,                                      // 19745:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 19747:        check predicate 34
  OPC_Decode, 170, 13, 162, 1,               // 19749:        decode to ADDQV_VPZ_S using decoder 162
                                             // 19749:       }
  1, 0,                                      // 19754:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 19756:        check predicate 34
  OPC_Decode, 168, 13, 162, 1,               // 19758:        decode to ADDQV_VPZ_D using decoder 162
                                             // 19758:       }
                                             // 19758:       } // switch Inst[22]
                                             // 19758:      }
  8, 21,                                     // 19763:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 19765:       switch Inst[22] {
  0, 7,                                      // 19768:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19770:        check predicate 33
  OPC_Decode, 237, 48, 161, 1,               // 19772:        decode to SMAXV_VPZ_S using decoder 161
                                             // 19772:       }
  1, 0,                                      // 19777:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19779:        check predicate 33
  OPC_Decode, 235, 48, 161, 1,               // 19781:        decode to SMAXV_VPZ_D using decoder 161
                                             // 19781:       }
                                             // 19781:       } // switch Inst[22]
                                             // 19781:      }
  9, 21,                                     // 19786:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 19788:       switch Inst[22] {
  0, 7,                                      // 19791:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19793:        check predicate 33
  OPC_Decode, 247, 63, 161, 1,               // 19795:        decode to UMAXV_VPZ_S using decoder 161
                                             // 19795:       }
  1, 0,                                      // 19800:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19802:        check predicate 33
  OPC_Decode, 245, 63, 161, 1,               // 19804:        decode to UMAXV_VPZ_D using decoder 161
                                             // 19804:       }
                                             // 19804:       } // switch Inst[22]
                                             // 19804:      }
  10, 21,                                    // 19809:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 19811:       switch Inst[22] {
  0, 7,                                      // 19814:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19816:        check predicate 33
  OPC_Decode, 167, 49, 161, 1,               // 19818:        decode to SMINV_VPZ_S using decoder 161
                                             // 19818:       }
  1, 0,                                      // 19823:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19825:        check predicate 33
  OPC_Decode, 165, 49, 161, 1,               // 19827:        decode to SMINV_VPZ_D using decoder 161
                                             // 19827:       }
                                             // 19827:       } // switch Inst[22]
                                             // 19827:      }
  11, 21,                                    // 19832:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 19834:       switch Inst[22] {
  0, 7,                                      // 19837:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19839:        check predicate 33
  OPC_Decode, 176, 64, 161, 1,               // 19841:        decode to UMINV_VPZ_S using decoder 161
                                             // 19841:       }
  1, 0,                                      // 19846:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19848:        check predicate 33
  OPC_Decode, 174, 64, 161, 1,               // 19850:        decode to UMINV_VPZ_D using decoder 161
                                             // 19850:       }
                                             // 19850:       } // switch Inst[22]
                                             // 19850:      }
  12, 21,                                    // 19855:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 19857:       switch Inst[22] {
  0, 7,                                      // 19860:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 19862:        check predicate 34
  OPC_Decode, 233, 48, 162, 1,               // 19864:        decode to SMAXQV_VPZ_S using decoder 162
                                             // 19864:       }
  1, 0,                                      // 19869:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 19871:        check predicate 34
  OPC_Decode, 231, 48, 162, 1,               // 19873:        decode to SMAXQV_VPZ_D using decoder 162
                                             // 19873:       }
                                             // 19873:       } // switch Inst[22]
                                             // 19873:      }
  13, 21,                                    // 19878:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 19880:       switch Inst[22] {
  0, 7,                                      // 19883:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 19885:        check predicate 34
  OPC_Decode, 243, 63, 162, 1,               // 19887:        decode to UMAXQV_VPZ_S using decoder 162
                                             // 19887:       }
  1, 0,                                      // 19892:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 19894:        check predicate 34
  OPC_Decode, 241, 63, 162, 1,               // 19896:        decode to UMAXQV_VPZ_D using decoder 162
                                             // 19896:       }
                                             // 19896:       } // switch Inst[22]
                                             // 19896:      }
  14, 21,                                    // 19901:      case 0xe: {
  OPC_SwitchField, 22, 1,                    // 19903:       switch Inst[22] {
  0, 7,                                      // 19906:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 19908:        check predicate 34
  OPC_Decode, 163, 49, 162, 1,               // 19910:        decode to SMINQV_VPZ_S using decoder 162
                                             // 19910:       }
  1, 0,                                      // 19915:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 19917:        check predicate 34
  OPC_Decode, 161, 49, 162, 1,               // 19919:        decode to SMINQV_VPZ_D using decoder 162
                                             // 19919:       }
                                             // 19919:       } // switch Inst[22]
                                             // 19919:      }
  15, 21,                                    // 19924:      case 0xf: {
  OPC_SwitchField, 22, 1,                    // 19926:       switch Inst[22] {
  0, 7,                                      // 19929:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 19931:        check predicate 34
  OPC_Decode, 172, 64, 162, 1,               // 19933:        decode to UMINQV_VPZ_S using decoder 162
                                             // 19933:       }
  1, 0,                                      // 19938:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 19940:        check predicate 34
  OPC_Decode, 170, 64, 162, 1,               // 19942:        decode to UMINQV_VPZ_D using decoder 162
                                             // 19942:       }
                                             // 19942:       } // switch Inst[22]
                                             // 19942:      }
  16, 21,                                    // 19947:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 19949:       switch Inst[22] {
  0, 7,                                      // 19952:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19954:        check predicate 33
  OPC_Decode, 235, 42, 161, 1,               // 19956:        decode to MOVPRFX_ZPzZ_S using decoder 161
                                             // 19956:       }
  1, 0,                                      // 19961:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19963:        check predicate 33
  OPC_Decode, 233, 42, 161, 1,               // 19965:        decode to MOVPRFX_ZPzZ_D using decoder 161
                                             // 19965:       }
                                             // 19965:       } // switch Inst[22]
                                             // 19965:      }
  17, 21,                                    // 19970:      case 0x11: {
  OPC_SwitchField, 22, 1,                    // 19972:       switch Inst[22] {
  0, 7,                                      // 19975:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 19977:        check predicate 33
  OPC_Decode, 231, 42, 163, 1,               // 19979:        decode to MOVPRFX_ZPmZ_S using decoder 163
                                             // 19979:       }
  1, 0,                                      // 19984:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 19986:        check predicate 33
  OPC_Decode, 229, 42, 163, 1,               // 19988:        decode to MOVPRFX_ZPmZ_D using decoder 163
                                             // 19988:       }
                                             // 19988:       } // switch Inst[22]
                                             // 19988:      }
  24, 21,                                    // 19993:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 19995:       switch Inst[22] {
  0, 7,                                      // 19998:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20000:        check predicate 33
  OPC_Decode, 224, 43, 161, 1,               // 20002:        decode to ORV_VPZ_S using decoder 161
                                             // 20002:       }
  1, 0,                                      // 20007:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20009:        check predicate 33
  OPC_Decode, 222, 43, 161, 1,               // 20011:        decode to ORV_VPZ_D using decoder 161
                                             // 20011:       }
                                             // 20011:       } // switch Inst[22]
                                             // 20011:      }
  25, 21,                                    // 20016:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 20018:       switch Inst[22] {
  0, 7,                                      // 20021:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20023:        check predicate 33
  OPC_Decode, 130, 21, 161, 1,               // 20025:        decode to EORV_VPZ_S using decoder 161
                                             // 20025:       }
  1, 0,                                      // 20030:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20032:        check predicate 33
  OPC_Decode, 128, 21, 161, 1,               // 20034:        decode to EORV_VPZ_D using decoder 161
                                             // 20034:       }
                                             // 20034:       } // switch Inst[22]
                                             // 20034:      }
  26, 21,                                    // 20039:      case 0x1a: {
  OPC_SwitchField, 22, 1,                    // 20041:       switch Inst[22] {
  0, 7,                                      // 20044:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20046:        check predicate 33
  OPC_Decode, 159, 14, 161, 1,               // 20048:        decode to ANDV_VPZ_S using decoder 161
                                             // 20048:       }
  1, 0,                                      // 20053:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20055:        check predicate 33
  OPC_Decode, 157, 14, 161, 1,               // 20057:        decode to ANDV_VPZ_D using decoder 161
                                             // 20057:       }
                                             // 20057:       } // switch Inst[22]
                                             // 20057:      }
  28, 21,                                    // 20062:      case 0x1c: {
  OPC_SwitchField, 22, 1,                    // 20064:       switch Inst[22] {
  0, 7,                                      // 20067:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 20069:        check predicate 34
  OPC_Decode, 202, 43, 162, 1,               // 20071:        decode to ORQV_VPZ_S using decoder 162
                                             // 20071:       }
  1, 0,                                      // 20076:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 20078:        check predicate 34
  OPC_Decode, 200, 43, 162, 1,               // 20080:        decode to ORQV_VPZ_D using decoder 162
                                             // 20080:       }
                                             // 20080:       } // switch Inst[22]
                                             // 20080:      }
  29, 21,                                    // 20085:      case 0x1d: {
  OPC_SwitchField, 22, 1,                    // 20087:       switch Inst[22] {
  0, 7,                                      // 20090:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 20092:        check predicate 34
  OPC_Decode, 249, 20, 162, 1,               // 20094:        decode to EORQV_VPZ_S using decoder 162
                                             // 20094:       }
  1, 0,                                      // 20099:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 20101:        check predicate 34
  OPC_Decode, 247, 20, 162, 1,               // 20103:        decode to EORQV_VPZ_D using decoder 162
                                             // 20103:       }
                                             // 20103:       } // switch Inst[22]
                                             // 20103:      }
  30, 0,                                     // 20108:      case 0x1e: {
  OPC_SwitchField, 22, 1,                    // 20110:       switch Inst[22] {
  0, 7,                                      // 20113:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 20115:        check predicate 34
  OPC_Decode, 150, 14, 162, 1,               // 20117:        decode to ANDQV_VPZ_S using decoder 162
                                             // 20117:       }
  1, 0,                                      // 20122:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 20124:        check predicate 34
  OPC_Decode, 148, 14, 162, 1,               // 20126:        decode to ANDQV_VPZ_D using decoder 162
                                             // 20126:       }
                                             // 20126:       } // switch Inst[22]
                                             // 20126:      }
                                             // 20126:      } // switch Inst[20:16]
                                             // 20126:     }
  2, 21,                                     // 20131:     case 0x2: {
  OPC_SwitchField, 22, 1,                    // 20133:      switch Inst[22] {
  0, 7,                                      // 20136:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 20138:       check predicate 33
  OPC_Decode, 243, 41, 164, 1,               // 20140:       decode to MLA_ZPmZZ_S using decoder 164
                                             // 20140:      }
  1, 0,                                      // 20145:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 20147:       check predicate 33
  OPC_Decode, 241, 41, 164, 1,               // 20149:       decode to MLA_ZPmZZ_D using decoder 164
                                             // 20149:      }
                                             // 20149:      } // switch Inst[22]
                                             // 20149:     }
  3, 21,                                     // 20154:     case 0x3: {
  OPC_SwitchField, 22, 1,                    // 20156:      switch Inst[22] {
  0, 7,                                      // 20159:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 20161:       check predicate 33
  OPC_Decode, 132, 42, 164, 1,               // 20163:       decode to MLS_ZPmZZ_S using decoder 164
                                             // 20163:      }
  1, 0,                                      // 20168:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 20170:       check predicate 33
  OPC_Decode, 130, 42, 164, 1,               // 20172:       decode to MLS_ZPmZZ_D using decoder 164
                                             // 20172:      }
                                             // 20172:      } // switch Inst[22]
                                             // 20172:     }
  4, 133, 2,                                 // 20177:     case 0x4: {
  OPC_SwitchField, 16, 5,                    // 20180:      switch Inst[20:16] {
  0, 7,                                      // 20183:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 20185:       check predicate 33
  OPC_Decode, 191, 14, 172, 1,               // 20187:       decode to ASR_ZPmI_D using decoder 172
                                             // 20187:      }
  1, 7,                                      // 20192:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 20194:       check predicate 33
  OPC_Decode, 176, 41, 172, 1,               // 20196:       decode to LSR_ZPmI_D using decoder 172
                                             // 20196:      }
  3, 7,                                      // 20201:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 20203:       check predicate 33
  OPC_Decode, 152, 41, 173, 1,               // 20205:       decode to LSL_ZPmI_D using decoder 173
                                             // 20205:      }
  4, 7,                                      // 20210:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 20212:       check predicate 33
  OPC_Decode, 175, 14, 172, 1,               // 20214:       decode to ASRD_ZPmI_D using decoder 172
                                             // 20214:      }
  6, 7,                                      // 20219:      case 0x6: {
  OPC_CheckPredicate, 35,                    // 20221:       check predicate 35
  OPC_Decode, 237, 53, 173, 1,               // 20223:       decode to SQSHL_ZPmI_D using decoder 173
                                             // 20223:      }
  7, 7,                                      // 20228:      case 0x7: {
  OPC_CheckPredicate, 35,                    // 20230:       check predicate 35
  OPC_Decode, 237, 66, 173, 1,               // 20232:       decode to UQSHL_ZPmI_D using decoder 173
                                             // 20232:      }
  12, 7,                                     // 20237:      case 0xc: {
  OPC_CheckPredicate, 35,                    // 20239:       check predicate 35
  OPC_Decode, 156, 55, 172, 1,               // 20241:       decode to SRSHR_ZPmI_D using decoder 172
                                             // 20241:      }
  13, 7,                                     // 20246:      case 0xd: {
  OPC_CheckPredicate, 35,                    // 20248:       check predicate 35
  OPC_Decode, 244, 67, 172, 1,               // 20250:       decode to URSHR_ZPmI_D using decoder 172
                                             // 20250:      }
  15, 7,                                     // 20255:      case 0xf: {
  OPC_CheckPredicate, 35,                    // 20257:       check predicate 35
  OPC_Decode, 222, 53, 173, 1,               // 20259:       decode to SQSHLU_ZPmI_D using decoder 173
                                             // 20259:      }
  16, 21,                                    // 20264:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 20266:       switch Inst[22] {
  0, 7,                                      // 20269:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20271:        check predicate 33
  OPC_Decode, 197, 14, 160, 1,               // 20273:        decode to ASR_ZPmZ_S using decoder 160
                                             // 20273:       }
  1, 0,                                      // 20278:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20280:        check predicate 33
  OPC_Decode, 195, 14, 160, 1,               // 20282:        decode to ASR_ZPmZ_D using decoder 160
                                             // 20282:       }
                                             // 20282:       } // switch Inst[22]
                                             // 20282:      }
  17, 21,                                    // 20287:      case 0x11: {
  OPC_SwitchField, 22, 1,                    // 20289:       switch Inst[22] {
  0, 7,                                      // 20292:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20294:        check predicate 33
  OPC_Decode, 182, 41, 160, 1,               // 20296:        decode to LSR_ZPmZ_S using decoder 160
                                             // 20296:       }
  1, 0,                                      // 20301:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20303:        check predicate 33
  OPC_Decode, 180, 41, 160, 1,               // 20305:        decode to LSR_ZPmZ_D using decoder 160
                                             // 20305:       }
                                             // 20305:       } // switch Inst[22]
                                             // 20305:      }
  19, 21,                                    // 20310:      case 0x13: {
  OPC_SwitchField, 22, 1,                    // 20312:       switch Inst[22] {
  0, 7,                                      // 20315:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20317:        check predicate 33
  OPC_Decode, 158, 41, 160, 1,               // 20319:        decode to LSL_ZPmZ_S using decoder 160
                                             // 20319:       }
  1, 0,                                      // 20324:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20326:        check predicate 33
  OPC_Decode, 156, 41, 160, 1,               // 20328:        decode to LSL_ZPmZ_D using decoder 160
                                             // 20328:       }
                                             // 20328:       } // switch Inst[22]
                                             // 20328:      }
  20, 21,                                    // 20333:      case 0x14: {
  OPC_SwitchField, 22, 1,                    // 20335:       switch Inst[22] {
  0, 7,                                      // 20338:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20340:        check predicate 33
  OPC_Decode, 181, 14, 160, 1,               // 20342:        decode to ASRR_ZPmZ_S using decoder 160
                                             // 20342:       }
  1, 0,                                      // 20347:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20349:        check predicate 33
  OPC_Decode, 179, 14, 160, 1,               // 20351:        decode to ASRR_ZPmZ_D using decoder 160
                                             // 20351:       }
                                             // 20351:       } // switch Inst[22]
                                             // 20351:      }
  21, 21,                                    // 20356:      case 0x15: {
  OPC_SwitchField, 22, 1,                    // 20358:       switch Inst[22] {
  0, 7,                                      // 20361:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20363:        check predicate 33
  OPC_Decode, 166, 41, 160, 1,               // 20365:        decode to LSRR_ZPmZ_S using decoder 160
                                             // 20365:       }
  1, 0,                                      // 20370:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20372:        check predicate 33
  OPC_Decode, 164, 41, 160, 1,               // 20374:        decode to LSRR_ZPmZ_D using decoder 160
                                             // 20374:       }
                                             // 20374:       } // switch Inst[22]
                                             // 20374:      }
  23, 21,                                    // 20379:      case 0x17: {
  OPC_SwitchField, 22, 1,                    // 20381:       switch Inst[22] {
  0, 7,                                      // 20384:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20386:        check predicate 33
  OPC_Decode, 142, 41, 160, 1,               // 20388:        decode to LSLR_ZPmZ_S using decoder 160
                                             // 20388:       }
  1, 0,                                      // 20393:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20395:        check predicate 33
  OPC_Decode, 140, 41, 160, 1,               // 20397:        decode to LSLR_ZPmZ_D using decoder 160
                                             // 20397:       }
                                             // 20397:       } // switch Inst[22]
                                             // 20397:      }
  24, 11,                                    // 20402:      case 0x18: {
  OPC_CheckPredicate, 33,                    // 20404:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 20406:       check Inst[22] == 0x0
  OPC_Decode, 186, 14, 160, 1,               // 20410:       decode to ASR_WIDE_ZPmZ_S using decoder 160
                                             // 20410:      }
  25, 11,                                    // 20415:      case 0x19: {
  OPC_CheckPredicate, 33,                    // 20417:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 20419:       check Inst[22] == 0x0
  OPC_Decode, 171, 41, 160, 1,               // 20423:       decode to LSR_WIDE_ZPmZ_S using decoder 160
                                             // 20423:      }
  27, 0,                                     // 20428:      case 0x1b: {
  OPC_CheckPredicate, 33,                    // 20430:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 20432:       check Inst[22] == 0x0
  OPC_Decode, 147, 41, 160, 1,               // 20436:       decode to LSL_WIDE_ZPmZ_S using decoder 160
                                             // 20436:      }
                                             // 20436:      } // switch Inst[20:16]
                                             // 20436:     }
  5, 141, 5,                                 // 20441:     case 0x5: {
  OPC_SwitchField, 16, 5,                    // 20444:      switch Inst[20:16] {
  0, 21,                                     // 20447:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 20449:       switch Inst[22] {
  0, 7,                                      // 20452:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20454:        check predicate 36
  OPC_Decode, 170, 61, 161, 1,               // 20456:        decode to SXTB_ZPzZ_S using decoder 161
                                             // 20456:       }
  1, 0,                                      // 20461:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20463:        check predicate 36
  OPC_Decode, 168, 61, 161, 1,               // 20465:        decode to SXTB_ZPzZ_D using decoder 161
                                             // 20465:       }
                                             // 20465:       } // switch Inst[22]
                                             // 20465:      }
  1, 21,                                     // 20470:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 20472:       switch Inst[22] {
  0, 7,                                      // 20475:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20477:        check predicate 36
  OPC_Decode, 160, 69, 161, 1,               // 20479:        decode to UXTB_ZPzZ_S using decoder 161
                                             // 20479:       }
  1, 0,                                      // 20484:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20486:        check predicate 36
  OPC_Decode, 158, 69, 161, 1,               // 20488:        decode to UXTB_ZPzZ_D using decoder 161
                                             // 20488:       }
                                             // 20488:       } // switch Inst[22]
                                             // 20488:      }
  2, 21,                                     // 20493:      case 0x2: {
  OPC_SwitchField, 22, 1,                    // 20495:       switch Inst[22] {
  0, 7,                                      // 20498:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20500:        check predicate 36
  OPC_Decode, 174, 61, 161, 1,               // 20502:        decode to SXTH_ZPzZ_S using decoder 161
                                             // 20502:       }
  1, 0,                                      // 20507:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20509:        check predicate 36
  OPC_Decode, 173, 61, 161, 1,               // 20511:        decode to SXTH_ZPzZ_D using decoder 161
                                             // 20511:       }
                                             // 20511:       } // switch Inst[22]
                                             // 20511:      }
  3, 21,                                     // 20516:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 20518:       switch Inst[22] {
  0, 7,                                      // 20521:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20523:        check predicate 36
  OPC_Decode, 164, 69, 161, 1,               // 20525:        decode to UXTH_ZPzZ_S using decoder 161
                                             // 20525:       }
  1, 0,                                      // 20530:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20532:        check predicate 36
  OPC_Decode, 163, 69, 161, 1,               // 20534:        decode to UXTH_ZPzZ_D using decoder 161
                                             // 20534:       }
                                             // 20534:       } // switch Inst[22]
                                             // 20534:      }
  4, 11,                                     // 20539:      case 0x4: {
  OPC_CheckPredicate, 36,                    // 20541:       check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 20543:       check Inst[22] == 0x1
  OPC_Decode, 176, 61, 161, 1,               // 20547:       decode to SXTW_ZPzZ_D using decoder 161
                                             // 20547:      }
  5, 11,                                     // 20552:      case 0x5: {
  OPC_CheckPredicate, 36,                    // 20554:       check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 20556:       check Inst[22] == 0x1
  OPC_Decode, 166, 69, 161, 1,               // 20560:       decode to UXTW_ZPzZ_D using decoder 161
                                             // 20560:      }
  6, 21,                                     // 20565:      case 0x6: {
  OPC_SwitchField, 22, 1,                    // 20567:       switch Inst[22] {
  0, 7,                                      // 20570:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20572:        check predicate 36
  OPC_Decode, 245, 12, 161, 1,               // 20574:        decode to ABS_ZPzZ_S using decoder 161
                                             // 20574:       }
  1, 0,                                      // 20579:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20581:        check predicate 36
  OPC_Decode, 243, 12, 161, 1,               // 20583:        decode to ABS_ZPzZ_D using decoder 161
                                             // 20583:       }
                                             // 20583:       } // switch Inst[22]
                                             // 20583:      }
  7, 21,                                     // 20588:      case 0x7: {
  OPC_SwitchField, 22, 1,                    // 20590:       switch Inst[22] {
  0, 7,                                      // 20593:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20595:        check predicate 36
  OPC_Decode, 169, 43, 161, 1,               // 20597:        decode to NEG_ZPzZ_S using decoder 161
                                             // 20597:       }
  1, 0,                                      // 20602:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20604:        check predicate 36
  OPC_Decode, 167, 43, 161, 1,               // 20606:        decode to NEG_ZPzZ_D using decoder 161
                                             // 20606:       }
                                             // 20606:       } // switch Inst[22]
                                             // 20606:      }
  8, 21,                                     // 20611:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 20613:       switch Inst[22] {
  0, 7,                                      // 20616:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20618:        check predicate 36
  OPC_Decode, 184, 17, 161, 1,               // 20620:        decode to CLS_ZPzZ_S using decoder 161
                                             // 20620:       }
  1, 0,                                      // 20625:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20627:        check predicate 36
  OPC_Decode, 182, 17, 161, 1,               // 20629:        decode to CLS_ZPzZ_D using decoder 161
                                             // 20629:       }
                                             // 20629:       } // switch Inst[22]
                                             // 20629:      }
  9, 21,                                     // 20634:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 20636:       switch Inst[22] {
  0, 7,                                      // 20639:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20641:        check predicate 36
  OPC_Decode, 200, 17, 161, 1,               // 20643:        decode to CLZ_ZPzZ_S using decoder 161
                                             // 20643:       }
  1, 0,                                      // 20648:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20650:        check predicate 36
  OPC_Decode, 198, 17, 161, 1,               // 20652:        decode to CLZ_ZPzZ_D using decoder 161
                                             // 20652:       }
                                             // 20652:       } // switch Inst[22]
                                             // 20652:      }
  10, 21,                                    // 20657:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 20659:       switch Inst[22] {
  0, 7,                                      // 20662:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20664:        check predicate 36
  OPC_Decode, 168, 19, 161, 1,               // 20666:        decode to CNT_ZPzZ_S using decoder 161
                                             // 20666:       }
  1, 0,                                      // 20671:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20673:        check predicate 36
  OPC_Decode, 166, 19, 161, 1,               // 20675:        decode to CNT_ZPzZ_D using decoder 161
                                             // 20675:       }
                                             // 20675:       } // switch Inst[22]
                                             // 20675:      }
  11, 21,                                    // 20680:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 20682:       switch Inst[22] {
  0, 7,                                      // 20685:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20687:        check predicate 36
  OPC_Decode, 146, 19, 161, 1,               // 20689:        decode to CNOT_ZPzZ_S using decoder 161
                                             // 20689:       }
  1, 0,                                      // 20694:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20696:        check predicate 36
  OPC_Decode, 144, 19, 161, 1,               // 20698:        decode to CNOT_ZPzZ_D using decoder 161
                                             // 20698:       }
                                             // 20698:       } // switch Inst[22]
                                             // 20698:      }
  12, 21,                                    // 20703:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 20705:       switch Inst[22] {
  0, 7,                                      // 20708:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20710:        check predicate 36
  OPC_Decode, 199, 21, 161, 1,               // 20712:        decode to FABS_ZPzZ_S using decoder 161
                                             // 20712:       }
  1, 0,                                      // 20717:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20719:        check predicate 36
  OPC_Decode, 197, 21, 161, 1,               // 20721:        decode to FABS_ZPzZ_D using decoder 161
                                             // 20721:       }
                                             // 20721:       } // switch Inst[22]
                                             // 20721:      }
  13, 21,                                    // 20726:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 20728:       switch Inst[22] {
  0, 7,                                      // 20731:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20733:        check predicate 36
  OPC_Decode, 163, 30, 161, 1,               // 20735:        decode to FNEG_ZPzZ_S using decoder 161
                                             // 20735:       }
  1, 0,                                      // 20740:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20742:        check predicate 36
  OPC_Decode, 161, 30, 161, 1,               // 20744:        decode to FNEG_ZPzZ_D using decoder 161
                                             // 20744:       }
                                             // 20744:       } // switch Inst[22]
                                             // 20744:      }
  14, 21,                                    // 20749:      case 0xe: {
  OPC_SwitchField, 22, 1,                    // 20751:       switch Inst[22] {
  0, 7,                                      // 20754:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 20756:        check predicate 36
  OPC_Decode, 190, 43, 161, 1,               // 20758:        decode to NOT_ZPzZ_S using decoder 161
                                             // 20758:       }
  1, 0,                                      // 20763:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 20765:        check predicate 36
  OPC_Decode, 188, 43, 161, 1,               // 20767:        decode to NOT_ZPzZ_D using decoder 161
                                             // 20767:       }
                                             // 20767:       } // switch Inst[22]
                                             // 20767:      }
  16, 21,                                    // 20772:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 20774:       switch Inst[22] {
  0, 7,                                      // 20777:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20779:        check predicate 33
  OPC_Decode, 167, 61, 163, 1,               // 20781:        decode to SXTB_ZPmZ_S using decoder 163
                                             // 20781:       }
  1, 0,                                      // 20786:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20788:        check predicate 33
  OPC_Decode, 165, 61, 163, 1,               // 20790:        decode to SXTB_ZPmZ_D using decoder 163
                                             // 20790:       }
                                             // 20790:       } // switch Inst[22]
                                             // 20790:      }
  17, 21,                                    // 20795:      case 0x11: {
  OPC_SwitchField, 22, 1,                    // 20797:       switch Inst[22] {
  0, 7,                                      // 20800:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20802:        check predicate 33
  OPC_Decode, 157, 69, 163, 1,               // 20804:        decode to UXTB_ZPmZ_S using decoder 163
                                             // 20804:       }
  1, 0,                                      // 20809:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20811:        check predicate 33
  OPC_Decode, 155, 69, 163, 1,               // 20813:        decode to UXTB_ZPmZ_D using decoder 163
                                             // 20813:       }
                                             // 20813:       } // switch Inst[22]
                                             // 20813:      }
  18, 21,                                    // 20818:      case 0x12: {
  OPC_SwitchField, 22, 1,                    // 20820:       switch Inst[22] {
  0, 7,                                      // 20823:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20825:        check predicate 33
  OPC_Decode, 172, 61, 163, 1,               // 20827:        decode to SXTH_ZPmZ_S using decoder 163
                                             // 20827:       }
  1, 0,                                      // 20832:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20834:        check predicate 33
  OPC_Decode, 171, 61, 163, 1,               // 20836:        decode to SXTH_ZPmZ_D using decoder 163
                                             // 20836:       }
                                             // 20836:       } // switch Inst[22]
                                             // 20836:      }
  19, 21,                                    // 20841:      case 0x13: {
  OPC_SwitchField, 22, 1,                    // 20843:       switch Inst[22] {
  0, 7,                                      // 20846:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20848:        check predicate 33
  OPC_Decode, 162, 69, 163, 1,               // 20850:        decode to UXTH_ZPmZ_S using decoder 163
                                             // 20850:       }
  1, 0,                                      // 20855:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20857:        check predicate 33
  OPC_Decode, 161, 69, 163, 1,               // 20859:        decode to UXTH_ZPmZ_D using decoder 163
                                             // 20859:       }
                                             // 20859:       } // switch Inst[22]
                                             // 20859:      }
  20, 11,                                    // 20864:      case 0x14: {
  OPC_CheckPredicate, 33,                    // 20866:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 20868:       check Inst[22] == 0x1
  OPC_Decode, 175, 61, 163, 1,               // 20872:       decode to SXTW_ZPmZ_D using decoder 163
                                             // 20872:      }
  21, 11,                                    // 20877:      case 0x15: {
  OPC_CheckPredicate, 33,                    // 20879:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 20881:       check Inst[22] == 0x1
  OPC_Decode, 165, 69, 163, 1,               // 20885:       decode to UXTW_ZPmZ_D using decoder 163
                                             // 20885:      }
  22, 21,                                    // 20890:      case 0x16: {
  OPC_SwitchField, 22, 1,                    // 20892:       switch Inst[22] {
  0, 7,                                      // 20895:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20897:        check predicate 33
  OPC_Decode, 241, 12, 163, 1,               // 20899:        decode to ABS_ZPmZ_S using decoder 163
                                             // 20899:       }
  1, 0,                                      // 20904:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20906:        check predicate 33
  OPC_Decode, 239, 12, 163, 1,               // 20908:        decode to ABS_ZPmZ_D using decoder 163
                                             // 20908:       }
                                             // 20908:       } // switch Inst[22]
                                             // 20908:      }
  23, 21,                                    // 20913:      case 0x17: {
  OPC_SwitchField, 22, 1,                    // 20915:       switch Inst[22] {
  0, 7,                                      // 20918:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20920:        check predicate 33
  OPC_Decode, 165, 43, 163, 1,               // 20922:        decode to NEG_ZPmZ_S using decoder 163
                                             // 20922:       }
  1, 0,                                      // 20927:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20929:        check predicate 33
  OPC_Decode, 163, 43, 163, 1,               // 20931:        decode to NEG_ZPmZ_D using decoder 163
                                             // 20931:       }
                                             // 20931:       } // switch Inst[22]
                                             // 20931:      }
  24, 21,                                    // 20936:      case 0x18: {
  OPC_SwitchField, 22, 1,                    // 20938:       switch Inst[22] {
  0, 7,                                      // 20941:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20943:        check predicate 33
  OPC_Decode, 180, 17, 163, 1,               // 20945:        decode to CLS_ZPmZ_S using decoder 163
                                             // 20945:       }
  1, 0,                                      // 20950:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20952:        check predicate 33
  OPC_Decode, 178, 17, 163, 1,               // 20954:        decode to CLS_ZPmZ_D using decoder 163
                                             // 20954:       }
                                             // 20954:       } // switch Inst[22]
                                             // 20954:      }
  25, 21,                                    // 20959:      case 0x19: {
  OPC_SwitchField, 22, 1,                    // 20961:       switch Inst[22] {
  0, 7,                                      // 20964:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20966:        check predicate 33
  OPC_Decode, 196, 17, 163, 1,               // 20968:        decode to CLZ_ZPmZ_S using decoder 163
                                             // 20968:       }
  1, 0,                                      // 20973:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20975:        check predicate 33
  OPC_Decode, 194, 17, 163, 1,               // 20977:        decode to CLZ_ZPmZ_D using decoder 163
                                             // 20977:       }
                                             // 20977:       } // switch Inst[22]
                                             // 20977:      }
  26, 21,                                    // 20982:      case 0x1a: {
  OPC_SwitchField, 22, 1,                    // 20984:       switch Inst[22] {
  0, 7,                                      // 20987:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 20989:        check predicate 33
  OPC_Decode, 164, 19, 163, 1,               // 20991:        decode to CNT_ZPmZ_S using decoder 163
                                             // 20991:       }
  1, 0,                                      // 20996:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 20998:        check predicate 33
  OPC_Decode, 162, 19, 163, 1,               // 21000:        decode to CNT_ZPmZ_D using decoder 163
                                             // 21000:       }
                                             // 21000:       } // switch Inst[22]
                                             // 21000:      }
  27, 21,                                    // 21005:      case 0x1b: {
  OPC_SwitchField, 22, 1,                    // 21007:       switch Inst[22] {
  0, 7,                                      // 21010:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21012:        check predicate 33
  OPC_Decode, 142, 19, 163, 1,               // 21014:        decode to CNOT_ZPmZ_S using decoder 163
                                             // 21014:       }
  1, 0,                                      // 21019:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21021:        check predicate 33
  OPC_Decode, 140, 19, 163, 1,               // 21023:        decode to CNOT_ZPmZ_D using decoder 163
                                             // 21023:       }
                                             // 21023:       } // switch Inst[22]
                                             // 21023:      }
  28, 21,                                    // 21028:      case 0x1c: {
  OPC_SwitchField, 22, 1,                    // 21030:       switch Inst[22] {
  0, 7,                                      // 21033:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21035:        check predicate 33
  OPC_Decode, 196, 21, 163, 1,               // 21037:        decode to FABS_ZPmZ_S using decoder 163
                                             // 21037:       }
  1, 0,                                      // 21042:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21044:        check predicate 33
  OPC_Decode, 194, 21, 163, 1,               // 21046:        decode to FABS_ZPmZ_D using decoder 163
                                             // 21046:       }
                                             // 21046:       } // switch Inst[22]
                                             // 21046:      }
  29, 21,                                    // 21051:      case 0x1d: {
  OPC_SwitchField, 22, 1,                    // 21053:       switch Inst[22] {
  0, 7,                                      // 21056:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21058:        check predicate 33
  OPC_Decode, 160, 30, 163, 1,               // 21060:        decode to FNEG_ZPmZ_S using decoder 163
                                             // 21060:       }
  1, 0,                                      // 21065:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21067:        check predicate 33
  OPC_Decode, 158, 30, 163, 1,               // 21069:        decode to FNEG_ZPmZ_D using decoder 163
                                             // 21069:       }
                                             // 21069:       } // switch Inst[22]
                                             // 21069:      }
  30, 0,                                     // 21074:      case 0x1e: {
  OPC_SwitchField, 22, 1,                    // 21076:       switch Inst[22] {
  0, 7,                                      // 21079:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21081:        check predicate 33
  OPC_Decode, 186, 43, 163, 1,               // 21083:        decode to NOT_ZPmZ_S using decoder 163
                                             // 21083:       }
  1, 0,                                      // 21088:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21090:        check predicate 33
  OPC_Decode, 184, 43, 163, 1,               // 21092:        decode to NOT_ZPmZ_D using decoder 163
                                             // 21092:       }
                                             // 21092:       } // switch Inst[22]
                                             // 21092:      }
                                             // 21092:      } // switch Inst[20:16]
                                             // 21092:     }
  6, 21,                                     // 21097:     case 0x6: {
  OPC_SwitchField, 22, 1,                    // 21099:      switch Inst[22] {
  0, 7,                                      // 21102:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 21104:       check predicate 33
  OPC_Decode, 236, 41, 171, 1,               // 21106:       decode to MAD_ZPmZZ_S using decoder 171
                                             // 21106:      }
  1, 0,                                      // 21111:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 21113:       check predicate 33
  OPC_Decode, 234, 41, 171, 1,               // 21115:       decode to MAD_ZPmZZ_D using decoder 171
                                             // 21115:      }
                                             // 21115:      } // switch Inst[22]
                                             // 21115:     }
  7, 0,                                      // 21120:     case 0x7: {
  OPC_SwitchField, 22, 1,                    // 21122:      switch Inst[22] {
  0, 7,                                      // 21125:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 21127:       check predicate 33
  OPC_Decode, 247, 42, 171, 1,               // 21129:       decode to MSB_ZPmZZ_S using decoder 171
                                             // 21129:      }
  1, 0,                                      // 21134:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 21136:       check predicate 33
  OPC_Decode, 245, 42, 171, 1,               // 21138:       decode to MSB_ZPmZZ_D using decoder 171
                                             // 21138:      }
                                             // 21138:      } // switch Inst[22]
                                             // 21138:     }
                                             // 21138:     } // switch Inst[15:13]
                                             // 21138:    }
  2, 102,                                    // 21143:    case 0x2: {
  OPC_SwitchField, 20, 1,                    // 21145:     switch Inst[20] {
  0, 29,                                     // 21148:     case 0x0: {
  OPC_SwitchField, 22, 1,                    // 21150:      switch Inst[22] {
  0, 11,                                     // 21153:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 21155:       check predicate 33
  OPC_CheckField, 18, 2, 0,                  // 21157:       check Inst[19:18] == 0x0
  OPC_Decode, 209, 43, 174, 1,               // 21161:       decode to ORR_ZI using decoder 174
                                             // 21161:      }
  1, 0,                                      // 21166:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 21168:       check predicate 33
  OPC_CheckField, 18, 2, 0,                  // 21170:       check Inst[19:18] == 0x0
  OPC_Decode, 136, 21, 174, 1,               // 21174:       decode to EOR_ZI using decoder 174
                                             // 21174:      }
                                             // 21174:      } // switch Inst[22]
                                             // 21174:     }
  1, 0,                                      // 21179:     case 0x1: {
  OPC_SwitchField, 14, 2,                    // 21181:      switch Inst[15:14] {
  0, 21,                                     // 21184:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 21186:       switch Inst[22] {
  0, 7,                                      // 21189:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21191:        check predicate 33
  OPC_Decode, 155, 20, 175, 1,               // 21193:        decode to CPY_ZPzI_B using decoder 175
                                             // 21193:       }
  1, 0,                                      // 21198:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21200:        check predicate 33
  OPC_Decode, 157, 20, 176, 1,               // 21202:        decode to CPY_ZPzI_H using decoder 176
                                             // 21202:       }
                                             // 21202:       } // switch Inst[22]
                                             // 21202:      }
  1, 21,                                     // 21207:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 21209:       switch Inst[22] {
  0, 7,                                      // 21212:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21214:        check predicate 33
  OPC_Decode, 143, 20, 177, 1,               // 21216:        decode to CPY_ZPmI_B using decoder 177
                                             // 21216:       }
  1, 0,                                      // 21221:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21223:        check predicate 33
  OPC_Decode, 145, 20, 178, 1,               // 21225:        decode to CPY_ZPmI_H using decoder 178
                                             // 21225:       }
                                             // 21225:       } // switch Inst[22]
                                             // 21225:      }
  3, 0,                                      // 21230:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 21232:       check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 21234:       check Inst[22] == 0x1
  OPC_CheckField, 13, 1, 0,                  // 21238:       check Inst[13] == 0x0
  OPC_Decode, 188, 23, 179, 1,               // 21242:       decode to FCPY_ZPmI_H using decoder 179
                                             // 21242:      }
                                             // 21242:      } // switch Inst[15:14]
                                             // 21242:     }
                                             // 21242:     } // switch Inst[20]
                                             // 21242:    }
  3, 0,                                      // 21247:    case 0x3: {
  OPC_SwitchField, 20, 1,                    // 21249:     switch Inst[20] {
  0, 29,                                     // 21252:     case 0x0: {
  OPC_SwitchField, 22, 1,                    // 21254:      switch Inst[22] {
  0, 11,                                     // 21257:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 21259:       check predicate 33
  OPC_CheckField, 18, 2, 0,                  // 21261:       check Inst[19:18] == 0x0
  OPC_Decode, 165, 14, 174, 1,               // 21265:       decode to AND_ZI using decoder 174
                                             // 21265:      }
  1, 0,                                      // 21270:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 21272:       check predicate 33
  OPC_CheckField, 18, 2, 0,                  // 21274:       check Inst[19:18] == 0x0
  OPC_Decode, 202, 20, 174, 1,               // 21278:       decode to DUPM_ZI using decoder 174
                                             // 21278:      }
                                             // 21278:      } // switch Inst[22]
                                             // 21278:     }
  1, 0,                                      // 21283:     case 0x1: {
  OPC_SwitchField, 14, 2,                    // 21285:      switch Inst[15:14] {
  0, 21,                                     // 21288:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 21290:       switch Inst[22] {
  0, 7,                                      // 21293:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21295:        check predicate 33
  OPC_Decode, 158, 20, 180, 1,               // 21297:        decode to CPY_ZPzI_S using decoder 180
                                             // 21297:       }
  1, 0,                                      // 21302:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21304:        check predicate 33
  OPC_Decode, 156, 20, 181, 1,               // 21306:        decode to CPY_ZPzI_D using decoder 181
                                             // 21306:       }
                                             // 21306:       } // switch Inst[22]
                                             // 21306:      }
  1, 21,                                     // 21311:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 21313:       switch Inst[22] {
  0, 7,                                      // 21316:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21318:        check predicate 33
  OPC_Decode, 146, 20, 182, 1,               // 21320:        decode to CPY_ZPmI_S using decoder 182
                                             // 21320:       }
  1, 0,                                      // 21325:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21327:        check predicate 33
  OPC_Decode, 144, 20, 183, 1,               // 21329:        decode to CPY_ZPmI_D using decoder 183
                                             // 21329:       }
                                             // 21329:       } // switch Inst[22]
                                             // 21329:      }
  3, 0,                                      // 21334:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 21336:       switch Inst[22] {
  0, 11,                                     // 21339:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21341:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 21343:        check Inst[13] == 0x0
  OPC_Decode, 189, 23, 179, 1,               // 21347:        decode to FCPY_ZPmI_S using decoder 179
                                             // 21347:       }
  1, 0,                                      // 21352:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21354:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 21356:        check Inst[13] == 0x0
  OPC_Decode, 187, 23, 179, 1,               // 21360:        decode to FCPY_ZPmI_D using decoder 179
                                             // 21360:       }
                                             // 21360:       } // switch Inst[22]
                                             // 21360:      }
                                             // 21360:      } // switch Inst[15:14]
                                             // 21360:     }
                                             // 21360:     } // switch Inst[20]
                                             // 21360:    }
                                             // 21360:    } // switch Inst[25:23]
                                             // 21360:   }
  1, 0,                                      // 21365:   case 0x1: {
  OPC_SwitchField, 14, 2,                    // 21367:    switch Inst[15:14] {
  0, 202, 10,                                // 21370:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 21373:     switch Inst[13] {
  0, 206, 2,                                 // 21376:     case 0x0: {
  OPC_SwitchField, 22, 4,                    // 21379:      switch Inst[25:22] {
  0, 57,                                     // 21382:      case 0x0: {
  OPC_SwitchField, 10, 3,                    // 21384:       switch Inst[12:10] {
  0, 7,                                      // 21387:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21389:        check predicate 33
  OPC_Decode, 228, 13, 184, 1,               // 21391:        decode to ADD_ZZZ_B using decoder 184
                                             // 21391:       }
  1, 7,                                      // 21396:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21398:        check predicate 33
  OPC_Decode, 187, 60, 184, 1,               // 21400:        decode to SUB_ZZZ_B using decoder 184
                                             // 21400:       }
  4, 7,                                      // 21405:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 21407:        check predicate 33
  OPC_Decode, 146, 51, 184, 1,               // 21409:        decode to SQADD_ZZZ_B using decoder 184
                                             // 21409:       }
  5, 7,                                      // 21414:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 21416:        check predicate 33
  OPC_Decode, 254, 65, 184, 1,               // 21418:        decode to UQADD_ZZZ_B using decoder 184
                                             // 21418:       }
  6, 7,                                      // 21423:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 21425:        check predicate 33
  OPC_Decode, 184, 54, 184, 1,               // 21427:        decode to SQSUB_ZZZ_B using decoder 184
                                             // 21427:       }
  7, 0,                                      // 21432:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 21434:        check predicate 33
  OPC_Decode, 167, 67, 184, 1,               // 21436:        decode to UQSUB_ZZZ_B using decoder 184
                                             // 21436:       }
                                             // 21436:       } // switch Inst[12:10]
                                             // 21436:      }
  1, 57,                                     // 21441:      case 0x1: {
  OPC_SwitchField, 10, 3,                    // 21443:       switch Inst[12:10] {
  0, 7,                                      // 21446:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21448:        check predicate 33
  OPC_Decode, 231, 13, 184, 1,               // 21450:        decode to ADD_ZZZ_H using decoder 184
                                             // 21450:       }
  1, 7,                                      // 21455:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21457:        check predicate 33
  OPC_Decode, 190, 60, 184, 1,               // 21459:        decode to SUB_ZZZ_H using decoder 184
                                             // 21459:       }
  4, 7,                                      // 21464:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 21466:        check predicate 33
  OPC_Decode, 148, 51, 184, 1,               // 21468:        decode to SQADD_ZZZ_H using decoder 184
                                             // 21468:       }
  5, 7,                                      // 21473:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 21475:        check predicate 33
  OPC_Decode, 128, 66, 184, 1,               // 21477:        decode to UQADD_ZZZ_H using decoder 184
                                             // 21477:       }
  6, 7,                                      // 21482:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 21484:        check predicate 33
  OPC_Decode, 186, 54, 184, 1,               // 21486:        decode to SQSUB_ZZZ_H using decoder 184
                                             // 21486:       }
  7, 0,                                      // 21491:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 21493:        check predicate 33
  OPC_Decode, 169, 67, 184, 1,               // 21495:        decode to UQSUB_ZZZ_H using decoder 184
                                             // 21495:       }
                                             // 21495:       } // switch Inst[12:10]
                                             // 21495:      }
  2, 57,                                     // 21500:      case 0x2: {
  OPC_SwitchField, 10, 3,                    // 21502:       switch Inst[12:10] {
  0, 7,                                      // 21505:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21507:        check predicate 33
  OPC_Decode, 232, 13, 184, 1,               // 21509:        decode to ADD_ZZZ_S using decoder 184
                                             // 21509:       }
  1, 7,                                      // 21514:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21516:        check predicate 33
  OPC_Decode, 191, 60, 184, 1,               // 21518:        decode to SUB_ZZZ_S using decoder 184
                                             // 21518:       }
  4, 7,                                      // 21523:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 21525:        check predicate 33
  OPC_Decode, 149, 51, 184, 1,               // 21527:        decode to SQADD_ZZZ_S using decoder 184
                                             // 21527:       }
  5, 7,                                      // 21532:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 21534:        check predicate 33
  OPC_Decode, 129, 66, 184, 1,               // 21536:        decode to UQADD_ZZZ_S using decoder 184
                                             // 21536:       }
  6, 7,                                      // 21541:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 21543:        check predicate 33
  OPC_Decode, 187, 54, 184, 1,               // 21545:        decode to SQSUB_ZZZ_S using decoder 184
                                             // 21545:       }
  7, 0,                                      // 21550:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 21552:        check predicate 33
  OPC_Decode, 170, 67, 184, 1,               // 21554:        decode to UQSUB_ZZZ_S using decoder 184
                                             // 21554:       }
                                             // 21554:       } // switch Inst[12:10]
                                             // 21554:      }
  3, 75,                                     // 21559:      case 0x3: {
  OPC_SwitchField, 10, 3,                    // 21561:       switch Inst[12:10] {
  0, 7,                                      // 21564:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21566:        check predicate 33
  OPC_Decode, 230, 13, 184, 1,               // 21568:        decode to ADD_ZZZ_D using decoder 184
                                             // 21568:       }
  1, 7,                                      // 21573:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21575:        check predicate 33
  OPC_Decode, 189, 60, 184, 1,               // 21577:        decode to SUB_ZZZ_D using decoder 184
                                             // 21577:       }
  2, 7,                                      // 21582:       case 0x2: {
  OPC_CheckPredicate, 37,                    // 21584:        check predicate 37
  OPC_Decode, 229, 13, 184, 1,               // 21586:        decode to ADD_ZZZ_CPA using decoder 184
                                             // 21586:       }
  3, 7,                                      // 21591:       case 0x3: {
  OPC_CheckPredicate, 37,                    // 21593:        check predicate 37
  OPC_Decode, 188, 60, 184, 1,               // 21595:        decode to SUB_ZZZ_CPA using decoder 184
                                             // 21595:       }
  4, 7,                                      // 21600:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 21602:        check predicate 33
  OPC_Decode, 147, 51, 184, 1,               // 21604:        decode to SQADD_ZZZ_D using decoder 184
                                             // 21604:       }
  5, 7,                                      // 21609:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 21611:        check predicate 33
  OPC_Decode, 255, 65, 184, 1,               // 21613:        decode to UQADD_ZZZ_D using decoder 184
                                             // 21613:       }
  6, 7,                                      // 21618:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 21620:        check predicate 33
  OPC_Decode, 185, 54, 184, 1,               // 21622:        decode to SQSUB_ZZZ_D using decoder 184
                                             // 21622:       }
  7, 0,                                      // 21627:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 21629:        check predicate 33
  OPC_Decode, 168, 67, 184, 1,               // 21631:        decode to UQSUB_ZZZ_D using decoder 184
                                             // 21631:       }
                                             // 21631:       } // switch Inst[12:10]
                                             // 21631:      }
  4, 7,                                      // 21636:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 21638:       check predicate 33
  OPC_Decode, 164, 21, 185, 1,               // 21640:       decode to EXT_ZZI using decoder 185
                                             // 21640:      }
  5, 7,                                      // 21645:      case 0x5: {
  OPC_CheckPredicate, 35,                    // 21647:       check predicate 35
  OPC_Decode, 165, 21, 186, 1,               // 21649:       decode to EXT_ZZI_B using decoder 186
                                             // 21649:      }
  6, 0,                                      // 21654:      case 0x6: {
  OPC_SwitchField, 10, 3,                    // 21656:       switch Inst[12:10] {
  0, 7,                                      // 21659:       case 0x0: {
  OPC_CheckPredicate, 38,                    // 21661:        check predicate 38
  OPC_Decode, 132, 71, 184, 1,               // 21663:        decode to ZIP1_ZZZ_Q using decoder 184
                                             // 21663:       }
  1, 7,                                      // 21668:       case 0x1: {
  OPC_CheckPredicate, 38,                    // 21670:        check predicate 38
  OPC_Decode, 148, 71, 184, 1,               // 21672:        decode to ZIP2_ZZZ_Q using decoder 184
                                             // 21672:       }
  2, 7,                                      // 21677:       case 0x2: {
  OPC_CheckPredicate, 38,                    // 21679:        check predicate 38
  OPC_Decode, 174, 69, 184, 1,               // 21681:        decode to UZP1_ZZZ_Q using decoder 184
                                             // 21681:       }
  3, 7,                                      // 21686:       case 0x3: {
  OPC_CheckPredicate, 38,                    // 21688:        check predicate 38
  OPC_Decode, 190, 69, 184, 1,               // 21690:        decode to UZP2_ZZZ_Q using decoder 184
                                             // 21690:       }
  6, 7,                                      // 21695:       case 0x6: {
  OPC_CheckPredicate, 38,                    // 21697:        check predicate 38
  OPC_Decode, 235, 61, 184, 1,               // 21699:        decode to TRN1_ZZZ_Q using decoder 184
                                             // 21699:       }
  7, 0,                                      // 21704:       case 0x7: {
  OPC_CheckPredicate, 38,                    // 21706:        check predicate 38
  OPC_Decode, 251, 61, 184, 1,               // 21708:        decode to TRN2_ZZZ_Q using decoder 184
                                             // 21708:       }
                                             // 21708:       } // switch Inst[12:10]
                                             // 21708:      }
                                             // 21708:      } // switch Inst[25:22]
                                             // 21708:     }
  1, 0,                                      // 21713:     case 0x1: {
  OPC_SwitchField, 10, 3,                    // 21715:      switch Inst[12:10] {
  0, 87,                                     // 21718:      case 0x0: {
  OPC_SwitchField, 16, 1,                    // 21720:       switch Inst[16] {
  0, 69,                                     // 21723:       case 0x0: {
  OPC_SwitchField, 17, 1,                    // 21725:        switch Inst[17] {
  0, 51,                                     // 21728:        case 0x0: {
  OPC_SwitchField, 18, 1,                    // 21730:         switch Inst[18] {
  0, 33,                                     // 21733:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 21735:          switch Inst[19] {
  0, 15,                                     // 21738:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 21740:           check predicate 33
  OPC_CheckField, 24, 2, 1,                  // 21742:           check Inst[25:24] == 0x1
  OPC_CheckField, 20, 1, 1,                  // 21746:           check Inst[20] == 0x1
  OPC_Decode, 218, 20, 187, 1,               // 21750:           decode to DUP_ZZI_Q using decoder 187
                                             // 21750:          }
  1, 0,                                      // 21755:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 21757:           check predicate 33
  OPC_CheckField, 24, 2, 1,                  // 21759:           check Inst[25:24] == 0x1
  OPC_Decode, 216, 20, 188, 1,               // 21763:           decode to DUP_ZZI_D using decoder 188
                                             // 21763:          }
                                             // 21763:          } // switch Inst[19]
                                             // 21763:         }
  1, 0,                                      // 21768:         case 0x1: {
  OPC_CheckPredicate, 33,                    // 21770:          check predicate 33
  OPC_CheckField, 24, 2, 1,                  // 21772:          check Inst[25:24] == 0x1
  OPC_Decode, 219, 20, 189, 1,               // 21776:          decode to DUP_ZZI_S using decoder 189
                                             // 21776:         }
                                             // 21776:         } // switch Inst[18]
                                             // 21776:        }
  1, 0,                                      // 21781:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 21783:         check predicate 33
  OPC_CheckField, 24, 2, 1,                  // 21785:         check Inst[25:24] == 0x1
  OPC_Decode, 217, 20, 190, 1,               // 21789:         decode to DUP_ZZI_H using decoder 190
                                             // 21789:        }
                                             // 21789:        } // switch Inst[17]
                                             // 21789:       }
  1, 0,                                      // 21794:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21796:        check predicate 33
  OPC_CheckField, 24, 2, 1,                  // 21798:        check Inst[25:24] == 0x1
  OPC_Decode, 215, 20, 191, 1,               // 21802:        decode to DUP_ZZI_B using decoder 191
                                             // 21802:       }
                                             // 21802:       } // switch Inst[16]
                                             // 21802:      }
  1, 71,                                     // 21807:      case 0x1: {
  OPC_SwitchField, 22, 4,                    // 21809:       switch Inst[25:22] {
  4, 53,                                     // 21812:       case 0x4: {
  OPC_SwitchField, 16, 1,                    // 21814:        switch Inst[16] {
  0, 39,                                     // 21817:        case 0x0: {
  OPC_SwitchField, 17, 1,                    // 21819:         switch Inst[17] {
  0, 25,                                     // 21822:         case 0x0: {
  OPC_SwitchField, 18, 1,                    // 21824:          switch Inst[18] {
  0, 11,                                     // 21827:          case 0x0: {
  OPC_CheckPredicate, 34,                    // 21829:           check predicate 34
  OPC_CheckField, 19, 1, 1,                  // 21831:           check Inst[19] == 0x1
  OPC_Decode, 204, 20, 192, 1,               // 21835:           decode to DUPQ_ZZI_D using decoder 192
                                             // 21835:          }
  1, 0,                                      // 21840:          case 0x1: {
  OPC_CheckPredicate, 34,                    // 21842:           check predicate 34
  OPC_Decode, 206, 20, 193, 1,               // 21844:           decode to DUPQ_ZZI_S using decoder 193
                                             // 21844:          }
                                             // 21844:          } // switch Inst[18]
                                             // 21844:         }
  1, 0,                                      // 21849:         case 0x1: {
  OPC_CheckPredicate, 34,                    // 21851:          check predicate 34
  OPC_Decode, 205, 20, 194, 1,               // 21853:          decode to DUPQ_ZZI_H using decoder 194
                                             // 21853:         }
                                             // 21853:         } // switch Inst[17]
                                             // 21853:        }
  1, 0,                                      // 21858:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 21860:         check predicate 34
  OPC_Decode, 203, 20, 195, 1,               // 21862:         decode to DUPQ_ZZI_B using decoder 195
                                             // 21862:        }
                                             // 21862:        } // switch Inst[16]
                                             // 21862:       }
  5, 0,                                      // 21867:       case 0x5: {
  OPC_CheckPredicate, 34,                    // 21869:        check predicate 34
  OPC_CheckField, 20, 1, 0,                  // 21871:        check Inst[20] == 0x0
  OPC_Decode, 151, 21, 196, 1,               // 21875:        decode to EXTQ_ZZI using decoder 196
                                             // 21875:       }
                                             // 21875:       } // switch Inst[25:22]
                                             // 21875:      }
  2, 39,                                     // 21880:      case 0x2: {
  OPC_SwitchField, 22, 4,                    // 21882:       switch Inst[25:22] {
  4, 7,                                      // 21885:       case 0x4: {
  OPC_CheckPredicate, 35,                    // 21887:        check predicate 35
  OPC_Decode, 185, 61, 197, 1,               // 21889:        decode to TBL_ZZZZ_B using decoder 197
                                             // 21889:       }
  5, 7,                                      // 21894:       case 0x5: {
  OPC_CheckPredicate, 35,                    // 21896:        check predicate 35
  OPC_Decode, 187, 61, 197, 1,               // 21898:        decode to TBL_ZZZZ_H using decoder 197
                                             // 21898:       }
  6, 7,                                      // 21903:       case 0x6: {
  OPC_CheckPredicate, 35,                    // 21905:        check predicate 35
  OPC_Decode, 188, 61, 197, 1,               // 21907:        decode to TBL_ZZZZ_S using decoder 197
                                             // 21907:       }
  7, 0,                                      // 21912:       case 0x7: {
  OPC_CheckPredicate, 35,                    // 21914:        check predicate 35
  OPC_Decode, 186, 61, 197, 1,               // 21916:        decode to TBL_ZZZZ_D using decoder 197
                                             // 21916:       }
                                             // 21916:       } // switch Inst[25:22]
                                             // 21916:      }
  3, 39,                                     // 21921:      case 0x3: {
  OPC_SwitchField, 22, 4,                    // 21923:       switch Inst[25:22] {
  4, 7,                                      // 21926:       case 0x4: {
  OPC_CheckPredicate, 35,                    // 21928:        check predicate 35
  OPC_Decode, 207, 61, 198, 1,               // 21930:        decode to TBX_ZZZ_B using decoder 198
                                             // 21930:       }
  5, 7,                                      // 21935:       case 0x5: {
  OPC_CheckPredicate, 35,                    // 21937:        check predicate 35
  OPC_Decode, 209, 61, 198, 1,               // 21939:        decode to TBX_ZZZ_H using decoder 198
                                             // 21939:       }
  6, 7,                                      // 21944:       case 0x6: {
  OPC_CheckPredicate, 35,                    // 21946:        check predicate 35
  OPC_Decode, 210, 61, 198, 1,               // 21948:        decode to TBX_ZZZ_S using decoder 198
                                             // 21948:       }
  7, 0,                                      // 21953:       case 0x7: {
  OPC_CheckPredicate, 35,                    // 21955:        check predicate 35
  OPC_Decode, 208, 61, 198, 1,               // 21957:        decode to TBX_ZZZ_D using decoder 198
                                             // 21957:       }
                                             // 21957:       } // switch Inst[25:22]
                                             // 21957:      }
  4, 75,                                     // 21962:      case 0x4: {
  OPC_SwitchField, 22, 4,                    // 21964:       switch Inst[25:22] {
  0, 7,                                      // 21967:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 21969:        check predicate 33
  OPC_Decode, 170, 14, 184, 1,               // 21971:        decode to AND_ZZZ using decoder 184
                                             // 21971:       }
  1, 7,                                      // 21976:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 21978:        check predicate 33
  OPC_Decode, 214, 43, 184, 1,               // 21980:        decode to ORR_ZZZ using decoder 184
                                             // 21980:       }
  2, 7,                                      // 21985:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 21987:        check predicate 33
  OPC_Decode, 141, 21, 184, 1,               // 21989:        decode to EOR_ZZZ using decoder 184
                                             // 21989:       }
  3, 7,                                      // 21994:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 21996:        check predicate 33
  OPC_Decode, 143, 16, 184, 1,               // 21998:        decode to BIC_ZZZ using decoder 184
                                             // 21998:       }
  4, 7,                                      // 22003:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 22005:        check predicate 33
  OPC_Decode, 189, 61, 184, 1,               // 22007:        decode to TBL_ZZZ_B using decoder 184
                                             // 22007:       }
  5, 7,                                      // 22012:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 22014:        check predicate 33
  OPC_Decode, 191, 61, 184, 1,               // 22016:        decode to TBL_ZZZ_H using decoder 184
                                             // 22016:       }
  6, 7,                                      // 22021:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 22023:        check predicate 33
  OPC_Decode, 192, 61, 184, 1,               // 22025:        decode to TBL_ZZZ_S using decoder 184
                                             // 22025:       }
  7, 0,                                      // 22030:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 22032:        check predicate 33
  OPC_Decode, 190, 61, 184, 1,               // 22034:        decode to TBL_ZZZ_D using decoder 184
                                             // 22034:       }
                                             // 22034:       } // switch Inst[25:22]
                                             // 22034:      }
  5, 99,                                     // 22039:      case 0x5: {
  OPC_SwitchField, 23, 3,                    // 22041:       switch Inst[25:23] {
  0, 39,                                     // 22044:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 22046:        switch Inst[22] {
  0, 25,                                     // 22049:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 22051:         switch Inst[20] {
  0, 11,                                     // 22054:         case 0x0: {
  OPC_CheckPredicate, 35,                    // 22056:          check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 22058:          check Inst[19] == 0x1
  OPC_Decode, 230, 70, 199, 1,               // 22062:          decode to XAR_ZZZI_B using decoder 199
                                             // 22062:         }
  1, 0,                                      // 22067:         case 0x1: {
  OPC_CheckPredicate, 35,                    // 22069:          check predicate 35
  OPC_Decode, 232, 70, 200, 1,               // 22071:          decode to XAR_ZZZI_H using decoder 200
                                             // 22071:         }
                                             // 22071:         } // switch Inst[20]
                                             // 22071:        }
  1, 0,                                      // 22076:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 22078:         check predicate 35
  OPC_Decode, 233, 70, 201, 1,               // 22080:         decode to XAR_ZZZI_S using decoder 201
                                             // 22080:        }
                                             // 22080:        } // switch Inst[22]
                                             // 22080:       }
  1, 7,                                      // 22085:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 22087:        check predicate 35
  OPC_Decode, 231, 70, 202, 1,               // 22089:        decode to XAR_ZZZI_D using decoder 202
                                             // 22089:       }
  2, 21,                                     // 22094:       case 0x2: {
  OPC_SwitchField, 22, 1,                    // 22096:        switch Inst[22] {
  0, 7,                                      // 22099:        case 0x0: {
  OPC_CheckPredicate, 34,                    // 22101:         check predicate 34
  OPC_Decode, 203, 61, 198, 1,               // 22103:         decode to TBXQ_ZZZ_B using decoder 198
                                             // 22103:        }
  1, 0,                                      // 22108:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 22110:         check predicate 34
  OPC_Decode, 205, 61, 198, 1,               // 22112:         decode to TBXQ_ZZZ_H using decoder 198
                                             // 22112:        }
                                             // 22112:        } // switch Inst[22]
                                             // 22112:       }
  3, 0,                                      // 22117:       case 0x3: {
  OPC_SwitchField, 22, 1,                    // 22119:        switch Inst[22] {
  0, 7,                                      // 22122:        case 0x0: {
  OPC_CheckPredicate, 34,                    // 22124:         check predicate 34
  OPC_Decode, 206, 61, 198, 1,               // 22126:         decode to TBXQ_ZZZ_S using decoder 198
                                             // 22126:        }
  1, 0,                                      // 22131:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 22133:         check predicate 34
  OPC_Decode, 204, 61, 198, 1,               // 22135:         decode to TBXQ_ZZZ_D using decoder 198
                                             // 22135:        }
                                             // 22135:        } // switch Inst[22]
                                             // 22135:       }
                                             // 22135:       } // switch Inst[25:23]
                                             // 22135:      }
  6, 159, 4,                                 // 22140:      case 0x6: {
  OPC_SwitchField, 23, 3,                    // 22143:       switch Inst[25:23] {
  0, 21,                                     // 22146:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 22148:        switch Inst[22] {
  0, 7,                                      // 22151:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 22153:         check predicate 35
  OPC_Decode, 241, 20, 203, 1,               // 22155:         decode to EOR3_ZZZZ using decoder 203
                                             // 22155:        }
  1, 0,                                      // 22160:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 22162:         check predicate 35
  OPC_Decode, 225, 14, 203, 1,               // 22164:         decode to BCAX_ZZZZ using decoder 203
                                             // 22164:        }
                                             // 22164:        } // switch Inst[22]
                                             // 22164:       }
  2, 148, 2,                                 // 22169:       case 0x2: {
  OPC_SwitchField, 19, 2,                    // 22172:        switch Inst[20:19] {
  0, 49,                                     // 22175:        case 0x0: {
  OPC_SwitchField, 16, 3,                    // 22177:         switch Inst[18:16] {
  0, 21,                                     // 22180:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 22182:          switch Inst[22] {
  0, 7,                                      // 22185:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22187:           check predicate 33
  OPC_Decode, 211, 20, 204, 1,               // 22189:           decode to DUP_ZR_B using decoder 204
                                             // 22189:          }
  1, 0,                                      // 22194:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22196:           check predicate 33
  OPC_Decode, 213, 20, 204, 1,               // 22198:           decode to DUP_ZR_H using decoder 204
                                             // 22198:          }
                                             // 22198:          } // switch Inst[22]
                                             // 22198:         }
  4, 0,                                      // 22203:         case 0x4: {
  OPC_SwitchField, 22, 1,                    // 22205:          switch Inst[22] {
  0, 7,                                      // 22208:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22210:           check predicate 33
  OPC_Decode, 139, 34, 205, 1,               // 22212:           decode to INSR_ZR_B using decoder 205
                                             // 22212:          }
  1, 0,                                      // 22217:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22219:           check predicate 33
  OPC_Decode, 141, 34, 205, 1,               // 22221:           decode to INSR_ZR_H using decoder 205
                                             // 22221:          }
                                             // 22221:          } // switch Inst[22]
                                             // 22221:         }
                                             // 22221:         } // switch Inst[18:16]
                                             // 22221:        }
  1, 109,                                    // 22226:        case 0x1: {
  OPC_SwitchField, 16, 1,                    // 22228:         switch Inst[16] {
  0, 51,                                     // 22231:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 22233:          switch Inst[22] {
  0, 33,                                     // 22236:          case 0x0: {
  OPC_SwitchField, 18, 1,                    // 22238:           switch Inst[18] {
  0, 15,                                     // 22241:           case 0x0: {
  OPC_CheckPredicate, 34,                    // 22243:            check predicate 34
  OPC_CheckField, 17, 1, 1,                  // 22245:            check Inst[17] == 0x1
  OPC_CheckField, 4, 1, 0,                   // 22249:            check Inst[4] == 0x0
  OPC_Decode, 130, 44, 206, 1,               // 22253:            decode to PMOV_PZI_B using decoder 206
                                             // 22253:           }
  1, 0,                                      // 22258:           case 0x1: {
  OPC_CheckPredicate, 34,                    // 22260:            check predicate 34
  OPC_CheckField, 4, 1, 0,                   // 22262:            check Inst[4] == 0x0
  OPC_Decode, 132, 44, 207, 1,               // 22266:            decode to PMOV_PZI_H using decoder 207
                                             // 22266:           }
                                             // 22266:           } // switch Inst[18]
                                             // 22266:          }
  1, 0,                                      // 22271:          case 0x1: {
  OPC_CheckPredicate, 34,                    // 22273:           check predicate 34
  OPC_CheckField, 4, 1, 0,                   // 22275:           check Inst[4] == 0x0
  OPC_Decode, 133, 44, 208, 1,               // 22279:           decode to PMOV_PZI_S using decoder 208
                                             // 22279:          }
                                             // 22279:          } // switch Inst[22]
                                             // 22279:         }
  1, 0,                                      // 22284:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 22286:          switch Inst[22] {
  0, 33,                                     // 22289:          case 0x0: {
  OPC_SwitchField, 18, 1,                    // 22291:           switch Inst[18] {
  0, 15,                                     // 22294:           case 0x0: {
  OPC_CheckPredicate, 34,                    // 22296:            check predicate 34
  OPC_CheckField, 17, 1, 1,                  // 22298:            check Inst[17] == 0x1
  OPC_CheckField, 9, 1, 0,                   // 22302:            check Inst[9] == 0x0
  OPC_Decode, 134, 44, 209, 1,               // 22306:            decode to PMOV_ZIP_B using decoder 209
                                             // 22306:           }
  1, 0,                                      // 22311:           case 0x1: {
  OPC_CheckPredicate, 34,                    // 22313:            check predicate 34
  OPC_CheckField, 9, 1, 0,                   // 22315:            check Inst[9] == 0x0
  OPC_Decode, 136, 44, 210, 1,               // 22319:            decode to PMOV_ZIP_H using decoder 210
                                             // 22319:           }
                                             // 22319:           } // switch Inst[18]
                                             // 22319:          }
  1, 0,                                      // 22324:          case 0x1: {
  OPC_CheckPredicate, 34,                    // 22326:           check predicate 34
  OPC_CheckField, 9, 1, 0,                   // 22328:           check Inst[9] == 0x0
  OPC_Decode, 137, 44, 211, 1,               // 22332:           decode to PMOV_ZIP_S using decoder 211
                                             // 22332:          }
                                             // 22332:          } // switch Inst[22]
                                             // 22332:         }
                                             // 22332:         } // switch Inst[16]
                                             // 22332:        }
  2, 78,                                     // 22337:        case 0x2: {
  OPC_SwitchField, 16, 3,                    // 22339:         switch Inst[18:16] {
  0, 11,                                     // 22342:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 22344:          check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 22346:          check Inst[22] == 0x1
  OPC_Decode, 236, 60, 212, 1,               // 22350:          decode to SUNPKLO_ZZ_H using decoder 212
                                             // 22350:         }
  1, 11,                                     // 22355:         case 0x1: {
  OPC_CheckPredicate, 33,                    // 22357:          check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 22359:          check Inst[22] == 0x1
  OPC_Decode, 233, 60, 212, 1,               // 22363:          decode to SUNPKHI_ZZ_H using decoder 212
                                             // 22363:         }
  2, 11,                                     // 22368:         case 0x2: {
  OPC_CheckPredicate, 33,                    // 22370:          check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 22372:          check Inst[22] == 0x1
  OPC_Decode, 144, 69, 212, 1,               // 22376:          decode to UUNPKLO_ZZ_H using decoder 212
                                             // 22376:         }
  3, 11,                                     // 22381:         case 0x3: {
  OPC_CheckPredicate, 33,                    // 22383:          check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 22385:          check Inst[22] == 0x1
  OPC_Decode, 141, 69, 212, 1,               // 22389:          decode to UUNPKHI_ZZ_H using decoder 212
                                             // 22389:         }
  4, 0,                                      // 22394:         case 0x4: {
  OPC_SwitchField, 22, 1,                    // 22396:          switch Inst[22] {
  0, 7,                                      // 22399:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22401:           check predicate 33
  OPC_Decode, 143, 34, 213, 1,               // 22403:           decode to INSR_ZV_B using decoder 213
                                             // 22403:          }
  1, 0,                                      // 22408:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22410:           check predicate 33
  OPC_Decode, 145, 34, 213, 1,               // 22412:           decode to INSR_ZV_H using decoder 213
                                             // 22412:          }
                                             // 22412:          } // switch Inst[22]
                                             // 22412:         }
                                             // 22412:         } // switch Inst[18:16]
                                             // 22412:        }
  3, 0,                                      // 22417:        case 0x3: {
  OPC_SwitchField, 22, 1,                    // 22419:         switch Inst[22] {
  0, 11,                                     // 22422:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 22424:          check predicate 33
  OPC_CheckField, 16, 3, 0,                  // 22426:          check Inst[18:16] == 0x0
  OPC_Decode, 225, 45, 212, 1,               // 22430:          decode to REV_ZZ_B using decoder 212
                                             // 22430:         }
  1, 0,                                      // 22435:         case 0x1: {
  OPC_CheckPredicate, 33,                    // 22437:          check predicate 33
  OPC_CheckField, 16, 3, 0,                  // 22439:          check Inst[18:16] == 0x0
  OPC_Decode, 227, 45, 212, 1,               // 22443:          decode to REV_ZZ_H using decoder 212
                                             // 22443:         }
                                             // 22443:         } // switch Inst[22]
                                             // 22443:        }
                                             // 22443:        } // switch Inst[20:19]
                                             // 22443:       }
  3, 0,                                      // 22448:       case 0x3: {
  OPC_SwitchField, 19, 2,                    // 22450:        switch Inst[20:19] {
  0, 49,                                     // 22453:        case 0x0: {
  OPC_SwitchField, 16, 3,                    // 22455:         switch Inst[18:16] {
  0, 21,                                     // 22458:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 22460:          switch Inst[22] {
  0, 7,                                      // 22463:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22465:           check predicate 33
  OPC_Decode, 214, 20, 204, 1,               // 22467:           decode to DUP_ZR_S using decoder 204
                                             // 22467:          }
  1, 0,                                      // 22472:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22474:           check predicate 33
  OPC_Decode, 212, 20, 214, 1,               // 22476:           decode to DUP_ZR_D using decoder 214
                                             // 22476:          }
                                             // 22476:          } // switch Inst[22]
                                             // 22476:         }
  4, 0,                                      // 22481:         case 0x4: {
  OPC_SwitchField, 22, 1,                    // 22483:          switch Inst[22] {
  0, 7,                                      // 22486:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22488:           check predicate 33
  OPC_Decode, 142, 34, 205, 1,               // 22490:           decode to INSR_ZR_S using decoder 205
                                             // 22490:          }
  1, 0,                                      // 22495:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22497:           check predicate 33
  OPC_Decode, 140, 34, 215, 1,               // 22499:           decode to INSR_ZR_D using decoder 215
                                             // 22499:          }
                                             // 22499:          } // switch Inst[22]
                                             // 22499:         }
                                             // 22499:         } // switch Inst[18:16]
                                             // 22499:        }
  1, 29,                                     // 22504:        case 0x1: {
  OPC_SwitchField, 16, 1,                    // 22506:         switch Inst[16] {
  0, 11,                                     // 22509:         case 0x0: {
  OPC_CheckPredicate, 34,                    // 22511:          check predicate 34
  OPC_CheckField, 4, 1, 0,                   // 22513:          check Inst[4] == 0x0
  OPC_Decode, 131, 44, 216, 1,               // 22517:          decode to PMOV_PZI_D using decoder 216
                                             // 22517:         }
  1, 0,                                      // 22522:         case 0x1: {
  OPC_CheckPredicate, 34,                    // 22524:          check predicate 34
  OPC_CheckField, 9, 1, 0,                   // 22526:          check Inst[9] == 0x0
  OPC_Decode, 135, 44, 217, 1,               // 22530:          decode to PMOV_ZIP_D using decoder 217
                                             // 22530:         }
                                             // 22530:         } // switch Inst[16]
                                             // 22530:        }
  2, 118,                                    // 22535:        case 0x2: {
  OPC_SwitchField, 16, 3,                    // 22537:         switch Inst[18:16] {
  0, 21,                                     // 22540:         case 0x0: {
  OPC_SwitchField, 22, 1,                    // 22542:          switch Inst[22] {
  0, 7,                                      // 22545:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22547:           check predicate 33
  OPC_Decode, 237, 60, 212, 1,               // 22549:           decode to SUNPKLO_ZZ_S using decoder 212
                                             // 22549:          }
  1, 0,                                      // 22554:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22556:           check predicate 33
  OPC_Decode, 235, 60, 212, 1,               // 22558:           decode to SUNPKLO_ZZ_D using decoder 212
                                             // 22558:          }
                                             // 22558:          } // switch Inst[22]
                                             // 22558:         }
  1, 21,                                     // 22563:         case 0x1: {
  OPC_SwitchField, 22, 1,                    // 22565:          switch Inst[22] {
  0, 7,                                      // 22568:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22570:           check predicate 33
  OPC_Decode, 234, 60, 212, 1,               // 22572:           decode to SUNPKHI_ZZ_S using decoder 212
                                             // 22572:          }
  1, 0,                                      // 22577:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22579:           check predicate 33
  OPC_Decode, 232, 60, 212, 1,               // 22581:           decode to SUNPKHI_ZZ_D using decoder 212
                                             // 22581:          }
                                             // 22581:          } // switch Inst[22]
                                             // 22581:         }
  2, 21,                                     // 22586:         case 0x2: {
  OPC_SwitchField, 22, 1,                    // 22588:          switch Inst[22] {
  0, 7,                                      // 22591:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22593:           check predicate 33
  OPC_Decode, 145, 69, 212, 1,               // 22595:           decode to UUNPKLO_ZZ_S using decoder 212
                                             // 22595:          }
  1, 0,                                      // 22600:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22602:           check predicate 33
  OPC_Decode, 143, 69, 212, 1,               // 22604:           decode to UUNPKLO_ZZ_D using decoder 212
                                             // 22604:          }
                                             // 22604:          } // switch Inst[22]
                                             // 22604:         }
  3, 21,                                     // 22609:         case 0x3: {
  OPC_SwitchField, 22, 1,                    // 22611:          switch Inst[22] {
  0, 7,                                      // 22614:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22616:           check predicate 33
  OPC_Decode, 142, 69, 212, 1,               // 22618:           decode to UUNPKHI_ZZ_S using decoder 212
                                             // 22618:          }
  1, 0,                                      // 22623:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22625:           check predicate 33
  OPC_Decode, 140, 69, 212, 1,               // 22627:           decode to UUNPKHI_ZZ_D using decoder 212
                                             // 22627:          }
                                             // 22627:          } // switch Inst[22]
                                             // 22627:         }
  4, 0,                                      // 22632:         case 0x4: {
  OPC_SwitchField, 22, 1,                    // 22634:          switch Inst[22] {
  0, 7,                                      // 22637:          case 0x0: {
  OPC_CheckPredicate, 33,                    // 22639:           check predicate 33
  OPC_Decode, 146, 34, 213, 1,               // 22641:           decode to INSR_ZV_S using decoder 213
                                             // 22641:          }
  1, 0,                                      // 22646:          case 0x1: {
  OPC_CheckPredicate, 33,                    // 22648:           check predicate 33
  OPC_Decode, 144, 34, 213, 1,               // 22650:           decode to INSR_ZV_D using decoder 213
                                             // 22650:          }
                                             // 22650:          } // switch Inst[22]
                                             // 22650:         }
                                             // 22650:         } // switch Inst[18:16]
                                             // 22650:        }
  3, 0,                                      // 22655:        case 0x3: {
  OPC_SwitchField, 22, 1,                    // 22657:         switch Inst[22] {
  0, 11,                                     // 22660:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 22662:          check predicate 33
  OPC_CheckField, 16, 3, 0,                  // 22664:          check Inst[18:16] == 0x0
  OPC_Decode, 228, 45, 212, 1,               // 22668:          decode to REV_ZZ_S using decoder 212
                                             // 22668:         }
  1, 0,                                      // 22673:         case 0x1: {
  OPC_CheckPredicate, 33,                    // 22675:          check predicate 33
  OPC_CheckField, 16, 3, 0,                  // 22677:          check Inst[18:16] == 0x0
  OPC_Decode, 226, 45, 212, 1,               // 22681:          decode to REV_ZZ_D using decoder 212
                                             // 22681:         }
                                             // 22681:         } // switch Inst[22]
                                             // 22681:        }
                                             // 22681:        } // switch Inst[20:19]
                                             // 22681:       }
                                             // 22681:       } // switch Inst[25:23]
                                             // 22681:      }
  7, 0,                                      // 22686:      case 0x7: {
  OPC_SwitchField, 22, 4,                    // 22688:       switch Inst[25:22] {
  0, 7,                                      // 22691:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 22693:        check predicate 35
  OPC_Decode, 184, 16, 203, 1,               // 22695:        decode to BSL_ZZZZ using decoder 203
                                             // 22695:       }
  1, 7,                                      // 22700:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 22702:        check predicate 35
  OPC_Decode, 182, 16, 203, 1,               // 22704:        decode to BSL1N_ZZZZ using decoder 203
                                             // 22704:       }
  2, 7,                                      // 22709:       case 0x2: {
  OPC_CheckPredicate, 35,                    // 22711:        check predicate 35
  OPC_Decode, 183, 16, 203, 1,               // 22713:        decode to BSL2N_ZZZZ using decoder 203
                                             // 22713:       }
  3, 0,                                      // 22718:       case 0x3: {
  OPC_CheckPredicate, 35,                    // 22720:        check predicate 35
  OPC_Decode, 161, 43, 203, 1,               // 22722:        decode to NBSL_ZZZZ using decoder 203
                                             // 22722:       }
                                             // 22722:       } // switch Inst[25:22]
                                             // 22722:      }
                                             // 22722:      } // switch Inst[12:10]
                                             // 22722:     }
                                             // 22722:     } // switch Inst[13]
                                             // 22722:    }
  1, 196, 11,                                // 22727:    case 0x1: {
  OPC_SwitchField, 11, 3,                    // 22730:     switch Inst[13:11] {
  0, 222, 2,                                 // 22733:     case 0x0: {
  OPC_SwitchField, 22, 4,                    // 22736:      switch Inst[25:22] {
  0, 21,                                     // 22739:      case 0x0: {
  OPC_SwitchField, 10, 1,                    // 22741:       switch Inst[10] {
  0, 7,                                      // 22744:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 22746:        check predicate 33
  OPC_Decode, 241, 33, 218, 1,               // 22748:        decode to INDEX_II_B using decoder 218
                                             // 22748:       }
  1, 0,                                      // 22753:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 22755:        check predicate 33
  OPC_Decode, 249, 33, 219, 1,               // 22757:        decode to INDEX_RI_B using decoder 219
                                             // 22757:       }
                                             // 22757:       } // switch Inst[10]
                                             // 22757:      }
  1, 21,                                     // 22762:      case 0x1: {
  OPC_SwitchField, 10, 1,                    // 22764:       switch Inst[10] {
  0, 7,                                      // 22767:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 22769:        check predicate 33
  OPC_Decode, 243, 33, 218, 1,               // 22771:        decode to INDEX_II_H using decoder 218
                                             // 22771:       }
  1, 0,                                      // 22776:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 22778:        check predicate 33
  OPC_Decode, 251, 33, 219, 1,               // 22780:        decode to INDEX_RI_H using decoder 219
                                             // 22780:       }
                                             // 22780:       } // switch Inst[10]
                                             // 22780:      }
  2, 21,                                     // 22785:      case 0x2: {
  OPC_SwitchField, 10, 1,                    // 22787:       switch Inst[10] {
  0, 7,                                      // 22790:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 22792:        check predicate 33
  OPC_Decode, 244, 33, 218, 1,               // 22794:        decode to INDEX_II_S using decoder 218
                                             // 22794:       }
  1, 0,                                      // 22799:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 22801:        check predicate 33
  OPC_Decode, 252, 33, 219, 1,               // 22803:        decode to INDEX_RI_S using decoder 219
                                             // 22803:       }
                                             // 22803:       } // switch Inst[10]
                                             // 22803:      }
  3, 21,                                     // 22808:      case 0x3: {
  OPC_SwitchField, 10, 1,                    // 22810:       switch Inst[10] {
  0, 7,                                      // 22813:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 22815:        check predicate 33
  OPC_Decode, 242, 33, 218, 1,               // 22817:        decode to INDEX_II_D using decoder 218
                                             // 22817:       }
  1, 0,                                      // 22822:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 22824:        check predicate 33
  OPC_Decode, 250, 33, 220, 1,               // 22826:        decode to INDEX_RI_D using decoder 220
                                             // 22826:       }
                                             // 22826:       } // switch Inst[10]
                                             // 22826:      }
  4, 82,                                     // 22831:      case 0x4: {
  OPC_SwitchField, 9, 2,                     // 22833:       switch Inst[10:9] {
  0, 60,                                     // 22836:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 22838:        switch Inst[20] {
  0, 11,                                     // 22841:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 22843:         check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 22845:         check Inst[4] == 0x0
  OPC_Decode, 253, 70, 221, 1,               // 22849:         decode to ZIP1_PPP_B using decoder 221
                                             // 22849:        }
  1, 0,                                      // 22854:        case 0x1: {
  OPC_SwitchField, 16, 4,                    // 22856:         switch Inst[19:16] {
  0, 11,                                     // 22859:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 22861:          check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 22863:          check Inst[4] == 0x0
  OPC_Decode, 215, 44, 222, 1,               // 22867:          decode to PUNPKLO_PP using decoder 222
                                             // 22867:         }
  1, 11,                                     // 22872:         case 0x1: {
  OPC_CheckPredicate, 33,                    // 22874:          check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 22876:          check Inst[4] == 0x0
  OPC_Decode, 214, 44, 222, 1,               // 22880:          decode to PUNPKHI_PP using decoder 222
                                             // 22880:         }
  4, 0,                                      // 22885:         case 0x4: {
  OPC_CheckPredicate, 33,                    // 22887:          check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 22889:          check Inst[4] == 0x0
  OPC_Decode, 221, 45, 222, 1,               // 22893:          decode to REV_PP_B using decoder 222
                                             // 22893:         }
                                             // 22893:         } // switch Inst[19:16]
                                             // 22893:        }
                                             // 22893:        } // switch Inst[20]
                                             // 22893:       }
  2, 0,                                      // 22898:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 22900:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 22902:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 22906:        check Inst[4] == 0x0
  OPC_Decode, 141, 71, 221, 1,               // 22910:        decode to ZIP2_PPP_B using decoder 221
                                             // 22910:       }
                                             // 22910:       } // switch Inst[10:9]
                                             // 22910:      }
  5, 55,                                     // 22915:      case 0x5: {
  OPC_SwitchField, 9, 2,                     // 22917:       switch Inst[10:9] {
  0, 33,                                     // 22920:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 22922:        switch Inst[20] {
  0, 11,                                     // 22925:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 22927:         check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 22929:         check Inst[4] == 0x0
  OPC_Decode, 255, 70, 221, 1,               // 22933:         decode to ZIP1_PPP_H using decoder 221
                                             // 22933:        }
  1, 0,                                      // 22938:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 22940:         check predicate 33
  OPC_CheckField, 16, 4, 4,                  // 22942:         check Inst[19:16] == 0x4
  OPC_CheckField, 4, 1, 0,                   // 22946:         check Inst[4] == 0x0
  OPC_Decode, 223, 45, 222, 1,               // 22950:         decode to REV_PP_H using decoder 222
                                             // 22950:        }
                                             // 22950:        } // switch Inst[20]
                                             // 22950:       }
  2, 0,                                      // 22955:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 22957:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 22959:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 22963:        check Inst[4] == 0x0
  OPC_Decode, 143, 71, 221, 1,               // 22967:        decode to ZIP2_PPP_H using decoder 221
                                             // 22967:       }
                                             // 22967:       } // switch Inst[10:9]
                                             // 22967:      }
  6, 55,                                     // 22972:      case 0x6: {
  OPC_SwitchField, 9, 2,                     // 22974:       switch Inst[10:9] {
  0, 33,                                     // 22977:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 22979:        switch Inst[20] {
  0, 11,                                     // 22982:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 22984:         check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 22986:         check Inst[4] == 0x0
  OPC_Decode, 128, 71, 221, 1,               // 22990:         decode to ZIP1_PPP_S using decoder 221
                                             // 22990:        }
  1, 0,                                      // 22995:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 22997:         check predicate 33
  OPC_CheckField, 16, 4, 4,                  // 22999:         check Inst[19:16] == 0x4
  OPC_CheckField, 4, 1, 0,                   // 23003:         check Inst[4] == 0x0
  OPC_Decode, 224, 45, 222, 1,               // 23007:         decode to REV_PP_S using decoder 222
                                             // 23007:        }
                                             // 23007:        } // switch Inst[20]
                                             // 23007:       }
  2, 0,                                      // 23012:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23014:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23016:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23020:        check Inst[4] == 0x0
  OPC_Decode, 144, 71, 221, 1,               // 23024:        decode to ZIP2_PPP_S using decoder 221
                                             // 23024:       }
                                             // 23024:       } // switch Inst[10:9]
                                             // 23024:      }
  7, 0,                                      // 23029:      case 0x7: {
  OPC_SwitchField, 9, 2,                     // 23031:       switch Inst[10:9] {
  0, 33,                                     // 23034:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 23036:        switch Inst[20] {
  0, 11,                                     // 23039:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 23041:         check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 23043:         check Inst[4] == 0x0
  OPC_Decode, 254, 70, 221, 1,               // 23047:         decode to ZIP1_PPP_D using decoder 221
                                             // 23047:        }
  1, 0,                                      // 23052:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 23054:         check predicate 33
  OPC_CheckField, 16, 4, 4,                  // 23056:         check Inst[19:16] == 0x4
  OPC_CheckField, 4, 1, 0,                   // 23060:         check Inst[4] == 0x0
  OPC_Decode, 222, 45, 222, 1,               // 23064:         decode to REV_PP_D using decoder 222
                                             // 23064:        }
                                             // 23064:        } // switch Inst[20]
                                             // 23064:       }
  2, 0,                                      // 23069:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23071:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23073:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23077:        check Inst[4] == 0x0
  OPC_Decode, 142, 71, 221, 1,               // 23081:        decode to ZIP2_PPP_D using decoder 221
                                             // 23081:       }
                                             // 23081:       } // switch Inst[10:9]
                                             // 23081:      }
                                             // 23081:      } // switch Inst[25:22]
                                             // 23081:     }
  1, 251, 1,                                 // 23086:     case 0x1: {
  OPC_SwitchField, 22, 4,                    // 23089:      switch Inst[25:22] {
  0, 21,                                     // 23092:      case 0x0: {
  OPC_SwitchField, 10, 1,                    // 23094:       switch Inst[10] {
  0, 7,                                      // 23097:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23099:        check predicate 33
  OPC_Decode, 245, 33, 223, 1,               // 23101:        decode to INDEX_IR_B using decoder 223
                                             // 23101:       }
  1, 0,                                      // 23106:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23108:        check predicate 33
  OPC_Decode, 253, 33, 224, 1,               // 23110:        decode to INDEX_RR_B using decoder 224
                                             // 23110:       }
                                             // 23110:       } // switch Inst[10]
                                             // 23110:      }
  1, 21,                                     // 23115:      case 0x1: {
  OPC_SwitchField, 10, 1,                    // 23117:       switch Inst[10] {
  0, 7,                                      // 23120:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23122:        check predicate 33
  OPC_Decode, 247, 33, 223, 1,               // 23124:        decode to INDEX_IR_H using decoder 223
                                             // 23124:       }
  1, 0,                                      // 23129:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23131:        check predicate 33
  OPC_Decode, 255, 33, 224, 1,               // 23133:        decode to INDEX_RR_H using decoder 224
                                             // 23133:       }
                                             // 23133:       } // switch Inst[10]
                                             // 23133:      }
  2, 21,                                     // 23138:      case 0x2: {
  OPC_SwitchField, 10, 1,                    // 23140:       switch Inst[10] {
  0, 7,                                      // 23143:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23145:        check predicate 33
  OPC_Decode, 248, 33, 223, 1,               // 23147:        decode to INDEX_IR_S using decoder 223
                                             // 23147:       }
  1, 0,                                      // 23152:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23154:        check predicate 33
  OPC_Decode, 128, 34, 224, 1,               // 23156:        decode to INDEX_RR_S using decoder 224
                                             // 23156:       }
                                             // 23156:       } // switch Inst[10]
                                             // 23156:      }
  3, 21,                                     // 23161:      case 0x3: {
  OPC_SwitchField, 10, 1,                    // 23163:       switch Inst[10] {
  0, 7,                                      // 23166:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23168:        check predicate 33
  OPC_Decode, 246, 33, 225, 1,               // 23170:        decode to INDEX_IR_D using decoder 225
                                             // 23170:       }
  1, 0,                                      // 23175:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23177:        check predicate 33
  OPC_Decode, 254, 33, 226, 1,               // 23179:        decode to INDEX_RR_D using decoder 226
                                             // 23179:       }
                                             // 23179:       } // switch Inst[10]
                                             // 23179:      }
  4, 37,                                     // 23184:      case 0x4: {
  OPC_SwitchField, 9, 2,                     // 23186:       switch Inst[10:9] {
  0, 15,                                     // 23189:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23191:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23193:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23197:        check Inst[4] == 0x0
  OPC_Decode, 167, 69, 221, 1,               // 23201:        decode to UZP1_PPP_B using decoder 221
                                             // 23201:       }
  2, 0,                                      // 23206:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23208:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23210:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23214:        check Inst[4] == 0x0
  OPC_Decode, 183, 69, 221, 1,               // 23218:        decode to UZP2_PPP_B using decoder 221
                                             // 23218:       }
                                             // 23218:       } // switch Inst[10:9]
                                             // 23218:      }
  5, 37,                                     // 23223:      case 0x5: {
  OPC_SwitchField, 9, 2,                     // 23225:       switch Inst[10:9] {
  0, 15,                                     // 23228:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23230:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23232:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23236:        check Inst[4] == 0x0
  OPC_Decode, 169, 69, 221, 1,               // 23240:        decode to UZP1_PPP_H using decoder 221
                                             // 23240:       }
  2, 0,                                      // 23245:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23247:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23249:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23253:        check Inst[4] == 0x0
  OPC_Decode, 185, 69, 221, 1,               // 23257:        decode to UZP2_PPP_H using decoder 221
                                             // 23257:       }
                                             // 23257:       } // switch Inst[10:9]
                                             // 23257:      }
  6, 37,                                     // 23262:      case 0x6: {
  OPC_SwitchField, 9, 2,                     // 23264:       switch Inst[10:9] {
  0, 15,                                     // 23267:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23269:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23271:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23275:        check Inst[4] == 0x0
  OPC_Decode, 170, 69, 221, 1,               // 23279:        decode to UZP1_PPP_S using decoder 221
                                             // 23279:       }
  2, 0,                                      // 23284:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23286:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23288:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23292:        check Inst[4] == 0x0
  OPC_Decode, 186, 69, 221, 1,               // 23296:        decode to UZP2_PPP_S using decoder 221
                                             // 23296:       }
                                             // 23296:       } // switch Inst[10:9]
                                             // 23296:      }
  7, 0,                                      // 23301:      case 0x7: {
  OPC_SwitchField, 9, 2,                     // 23303:       switch Inst[10:9] {
  0, 15,                                     // 23306:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23308:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23310:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23314:        check Inst[4] == 0x0
  OPC_Decode, 168, 69, 221, 1,               // 23318:        decode to UZP1_PPP_D using decoder 221
                                             // 23318:       }
  2, 0,                                      // 23323:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23325:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23327:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23331:        check Inst[4] == 0x0
  OPC_Decode, 184, 69, 221, 1,               // 23335:        decode to UZP2_PPP_D using decoder 221
                                             // 23335:       }
                                             // 23335:       } // switch Inst[10:9]
                                             // 23335:      }
                                             // 23335:      } // switch Inst[25:22]
                                             // 23335:     }
  2, 190, 1,                                 // 23340:     case 0x2: {
  OPC_SwitchField, 22, 4,                    // 23343:      switch Inst[25:22] {
  0, 7,                                      // 23346:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 23348:       check predicate 33
  OPC_Decode, 186, 13, 227, 1,               // 23350:       decode to ADDVL_XXI using decoder 227
                                             // 23350:      }
  1, 7,                                      // 23355:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 23357:       check predicate 33
  OPC_Decode, 149, 13, 227, 1,               // 23359:       decode to ADDPL_XXI using decoder 227
                                             // 23359:      }
  2, 11,                                     // 23364:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 23366:       check predicate 33
  OPC_CheckField, 16, 5, 31,                 // 23368:       check Inst[20:16] == 0x1f
  OPC_Decode, 182, 45, 228, 1,               // 23372:       decode to RDVLI_XI using decoder 228
                                             // 23372:      }
  4, 37,                                     // 23377:      case 0x4: {
  OPC_SwitchField, 9, 2,                     // 23379:       switch Inst[10:9] {
  0, 15,                                     // 23382:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23384:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23386:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23390:        check Inst[4] == 0x0
  OPC_Decode, 228, 61, 221, 1,               // 23394:        decode to TRN1_PPP_B using decoder 221
                                             // 23394:       }
  2, 0,                                      // 23399:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23401:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23403:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23407:        check Inst[4] == 0x0
  OPC_Decode, 244, 61, 221, 1,               // 23411:        decode to TRN2_PPP_B using decoder 221
                                             // 23411:       }
                                             // 23411:       } // switch Inst[10:9]
                                             // 23411:      }
  5, 37,                                     // 23416:      case 0x5: {
  OPC_SwitchField, 9, 2,                     // 23418:       switch Inst[10:9] {
  0, 15,                                     // 23421:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23423:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23425:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23429:        check Inst[4] == 0x0
  OPC_Decode, 230, 61, 221, 1,               // 23433:        decode to TRN1_PPP_H using decoder 221
                                             // 23433:       }
  2, 0,                                      // 23438:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23440:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23442:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23446:        check Inst[4] == 0x0
  OPC_Decode, 246, 61, 221, 1,               // 23450:        decode to TRN2_PPP_H using decoder 221
                                             // 23450:       }
                                             // 23450:       } // switch Inst[10:9]
                                             // 23450:      }
  6, 37,                                     // 23455:      case 0x6: {
  OPC_SwitchField, 9, 2,                     // 23457:       switch Inst[10:9] {
  0, 15,                                     // 23460:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23462:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23464:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23468:        check Inst[4] == 0x0
  OPC_Decode, 231, 61, 221, 1,               // 23472:        decode to TRN1_PPP_S using decoder 221
                                             // 23472:       }
  2, 0,                                      // 23477:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23479:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23481:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23485:        check Inst[4] == 0x0
  OPC_Decode, 247, 61, 221, 1,               // 23489:        decode to TRN2_PPP_S using decoder 221
                                             // 23489:       }
                                             // 23489:       } // switch Inst[10:9]
                                             // 23489:      }
  7, 0,                                      // 23494:      case 0x7: {
  OPC_SwitchField, 9, 2,                     // 23496:       switch Inst[10:9] {
  0, 15,                                     // 23499:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23501:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23503:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23507:        check Inst[4] == 0x0
  OPC_Decode, 229, 61, 221, 1,               // 23511:        decode to TRN1_PPP_D using decoder 221
                                             // 23511:       }
  2, 0,                                      // 23516:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 23518:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 23520:        check Inst[20] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 23524:        check Inst[4] == 0x0
  OPC_Decode, 245, 61, 221, 1,               // 23528:        decode to TRN2_PPP_D using decoder 221
                                             // 23528:       }
                                             // 23528:       } // switch Inst[10:9]
                                             // 23528:      }
                                             // 23528:      } // switch Inst[25:22]
                                             // 23528:     }
  3, 34,                                     // 23533:     case 0x3: {
  OPC_SwitchField, 22, 4,                    // 23535:      switch Inst[25:22] {
  0, 7,                                      // 23538:      case 0x0: {
  OPC_CheckPredicate, 6,                     // 23540:       check predicate 6
  OPC_Decode, 176, 13, 227, 1,               // 23542:       decode to ADDSVL_XXI using decoder 227
                                             // 23542:      }
  1, 7,                                      // 23547:      case 0x1: {
  OPC_CheckPredicate, 6,                     // 23549:       check predicate 6
  OPC_Decode, 171, 13, 227, 1,               // 23551:       decode to ADDSPL_XXI using decoder 227
                                             // 23551:      }
  2, 0,                                      // 23556:      case 0x2: {
  OPC_CheckPredicate, 6,                     // 23558:       check predicate 6
  OPC_CheckField, 16, 5, 31,                 // 23560:       check Inst[20:16] == 0x1f
  OPC_Decode, 181, 45, 228, 1,               // 23564:       decode to RDSVLI_XI using decoder 228
                                             // 23564:      }
                                             // 23564:      } // switch Inst[25:22]
                                             // 23564:     }
  4, 157, 1,                                 // 23569:     case 0x4: {
  OPC_SwitchField, 22, 4,                    // 23572:      switch Inst[25:22] {
  0, 21,                                     // 23575:      case 0x0: {
  OPC_SwitchField, 10, 1,                    // 23577:       switch Inst[10] {
  0, 7,                                      // 23580:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23582:        check predicate 35
  OPC_Decode, 139, 43, 184, 1,               // 23584:        decode to MUL_ZZZ_B using decoder 184
                                             // 23584:       }
  1, 0,                                      // 23589:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23591:        check predicate 35
  OPC_Decode, 149, 44, 184, 1,               // 23593:        decode to PMUL_ZZZ_B using decoder 184
                                             // 23593:       }
                                             // 23593:       } // switch Inst[10]
                                             // 23593:      }
  1, 11,                                     // 23598:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 23600:       check predicate 35
  OPC_CheckField, 10, 1, 0,                  // 23602:       check Inst[10] == 0x0
  OPC_Decode, 141, 43, 184, 1,               // 23606:       decode to MUL_ZZZ_H using decoder 184
                                             // 23606:      }
  2, 11,                                     // 23611:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 23613:       check predicate 35
  OPC_CheckField, 10, 1, 0,                  // 23615:       check Inst[10] == 0x0
  OPC_Decode, 142, 43, 184, 1,               // 23619:       decode to MUL_ZZZ_S using decoder 184
                                             // 23619:      }
  3, 11,                                     // 23624:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 23626:       check predicate 35
  OPC_CheckField, 10, 1, 0,                  // 23628:       check Inst[10] == 0x0
  OPC_Decode, 140, 43, 184, 1,               // 23632:       decode to MUL_ZZZ_D using decoder 184
                                             // 23632:      }
  4, 21,                                     // 23637:      case 0x4: {
  OPC_SwitchField, 10, 1,                    // 23639:       switch Inst[10] {
  0, 7,                                      // 23642:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23644:        check predicate 33
  OPC_Decode, 129, 71, 184, 1,               // 23646:        decode to ZIP1_ZZZ_B using decoder 184
                                             // 23646:       }
  1, 0,                                      // 23651:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23653:        check predicate 33
  OPC_Decode, 145, 71, 184, 1,               // 23655:        decode to ZIP2_ZZZ_B using decoder 184
                                             // 23655:       }
                                             // 23655:       } // switch Inst[10]
                                             // 23655:      }
  5, 21,                                     // 23660:      case 0x5: {
  OPC_SwitchField, 10, 1,                    // 23662:       switch Inst[10] {
  0, 7,                                      // 23665:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23667:        check predicate 33
  OPC_Decode, 131, 71, 184, 1,               // 23669:        decode to ZIP1_ZZZ_H using decoder 184
                                             // 23669:       }
  1, 0,                                      // 23674:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23676:        check predicate 33
  OPC_Decode, 147, 71, 184, 1,               // 23678:        decode to ZIP2_ZZZ_H using decoder 184
                                             // 23678:       }
                                             // 23678:       } // switch Inst[10]
                                             // 23678:      }
  6, 21,                                     // 23683:      case 0x6: {
  OPC_SwitchField, 10, 1,                    // 23685:       switch Inst[10] {
  0, 7,                                      // 23688:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23690:        check predicate 33
  OPC_Decode, 133, 71, 184, 1,               // 23692:        decode to ZIP1_ZZZ_S using decoder 184
                                             // 23692:       }
  1, 0,                                      // 23697:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23699:        check predicate 33
  OPC_Decode, 149, 71, 184, 1,               // 23701:        decode to ZIP2_ZZZ_S using decoder 184
                                             // 23701:       }
                                             // 23701:       } // switch Inst[10]
                                             // 23701:      }
  7, 0,                                      // 23706:      case 0x7: {
  OPC_SwitchField, 10, 1,                    // 23708:       switch Inst[10] {
  0, 7,                                      // 23711:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23713:        check predicate 33
  OPC_Decode, 130, 71, 184, 1,               // 23715:        decode to ZIP1_ZZZ_D using decoder 184
                                             // 23715:       }
  1, 0,                                      // 23720:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23722:        check predicate 33
  OPC_Decode, 146, 71, 184, 1,               // 23724:        decode to ZIP2_ZZZ_D using decoder 184
                                             // 23724:       }
                                             // 23724:       } // switch Inst[10]
                                             // 23724:      }
                                             // 23724:      } // switch Inst[25:22]
                                             // 23724:     }
  5, 187, 1,                                 // 23729:     case 0x5: {
  OPC_SwitchField, 22, 4,                    // 23732:      switch Inst[25:22] {
  0, 21,                                     // 23735:      case 0x0: {
  OPC_SwitchField, 10, 1,                    // 23737:       switch Inst[10] {
  0, 7,                                      // 23740:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23742:        check predicate 35
  OPC_Decode, 214, 50, 184, 1,               // 23744:        decode to SMULH_ZZZ_B using decoder 184
                                             // 23744:       }
  1, 0,                                      // 23749:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23751:        check predicate 35
  OPC_Decode, 221, 65, 184, 1,               // 23753:        decode to UMULH_ZZZ_B using decoder 184
                                             // 23753:       }
                                             // 23753:       } // switch Inst[10]
                                             // 23753:      }
  1, 21,                                     // 23758:      case 0x1: {
  OPC_SwitchField, 10, 1,                    // 23760:       switch Inst[10] {
  0, 7,                                      // 23763:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23765:        check predicate 35
  OPC_Decode, 216, 50, 184, 1,               // 23767:        decode to SMULH_ZZZ_H using decoder 184
                                             // 23767:       }
  1, 0,                                      // 23772:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23774:        check predicate 35
  OPC_Decode, 223, 65, 184, 1,               // 23776:        decode to UMULH_ZZZ_H using decoder 184
                                             // 23776:       }
                                             // 23776:       } // switch Inst[10]
                                             // 23776:      }
  2, 21,                                     // 23781:      case 0x2: {
  OPC_SwitchField, 10, 1,                    // 23783:       switch Inst[10] {
  0, 7,                                      // 23786:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23788:        check predicate 35
  OPC_Decode, 217, 50, 184, 1,               // 23790:        decode to SMULH_ZZZ_S using decoder 184
                                             // 23790:       }
  1, 0,                                      // 23795:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23797:        check predicate 35
  OPC_Decode, 224, 65, 184, 1,               // 23799:        decode to UMULH_ZZZ_S using decoder 184
                                             // 23799:       }
                                             // 23799:       } // switch Inst[10]
                                             // 23799:      }
  3, 21,                                     // 23804:      case 0x3: {
  OPC_SwitchField, 10, 1,                    // 23806:       switch Inst[10] {
  0, 7,                                      // 23809:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23811:        check predicate 35
  OPC_Decode, 215, 50, 184, 1,               // 23813:        decode to SMULH_ZZZ_D using decoder 184
                                             // 23813:       }
  1, 0,                                      // 23818:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23820:        check predicate 35
  OPC_Decode, 222, 65, 184, 1,               // 23822:        decode to UMULH_ZZZ_D using decoder 184
                                             // 23822:       }
                                             // 23822:       } // switch Inst[10]
                                             // 23822:      }
  4, 21,                                     // 23827:      case 0x4: {
  OPC_SwitchField, 10, 1,                    // 23829:       switch Inst[10] {
  0, 7,                                      // 23832:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23834:        check predicate 33
  OPC_Decode, 171, 69, 184, 1,               // 23836:        decode to UZP1_ZZZ_B using decoder 184
                                             // 23836:       }
  1, 0,                                      // 23841:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23843:        check predicate 33
  OPC_Decode, 187, 69, 184, 1,               // 23845:        decode to UZP2_ZZZ_B using decoder 184
                                             // 23845:       }
                                             // 23845:       } // switch Inst[10]
                                             // 23845:      }
  5, 21,                                     // 23850:      case 0x5: {
  OPC_SwitchField, 10, 1,                    // 23852:       switch Inst[10] {
  0, 7,                                      // 23855:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23857:        check predicate 33
  OPC_Decode, 173, 69, 184, 1,               // 23859:        decode to UZP1_ZZZ_H using decoder 184
                                             // 23859:       }
  1, 0,                                      // 23864:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23866:        check predicate 33
  OPC_Decode, 189, 69, 184, 1,               // 23868:        decode to UZP2_ZZZ_H using decoder 184
                                             // 23868:       }
                                             // 23868:       } // switch Inst[10]
                                             // 23868:      }
  6, 21,                                     // 23873:      case 0x6: {
  OPC_SwitchField, 10, 1,                    // 23875:       switch Inst[10] {
  0, 7,                                      // 23878:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23880:        check predicate 33
  OPC_Decode, 175, 69, 184, 1,               // 23882:        decode to UZP1_ZZZ_S using decoder 184
                                             // 23882:       }
  1, 0,                                      // 23887:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23889:        check predicate 33
  OPC_Decode, 191, 69, 184, 1,               // 23891:        decode to UZP2_ZZZ_S using decoder 184
                                             // 23891:       }
                                             // 23891:       } // switch Inst[10]
                                             // 23891:      }
  7, 0,                                      // 23896:      case 0x7: {
  OPC_SwitchField, 10, 1,                    // 23898:       switch Inst[10] {
  0, 7,                                      // 23901:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 23903:        check predicate 33
  OPC_Decode, 172, 69, 184, 1,               // 23905:        decode to UZP1_ZZZ_D using decoder 184
                                             // 23905:       }
  1, 0,                                      // 23910:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 23912:        check predicate 33
  OPC_Decode, 188, 69, 184, 1,               // 23914:        decode to UZP2_ZZZ_D using decoder 184
                                             // 23914:       }
                                             // 23914:       } // switch Inst[10]
                                             // 23914:      }
                                             // 23914:      } // switch Inst[25:22]
                                             // 23914:     }
  6, 187, 1,                                 // 23919:     case 0x6: {
  OPC_SwitchField, 22, 4,                    // 23922:      switch Inst[25:22] {
  0, 21,                                     // 23925:      case 0x0: {
  OPC_SwitchField, 10, 1,                    // 23927:       switch Inst[10] {
  0, 7,                                      // 23930:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23932:        check predicate 35
  OPC_Decode, 140, 52, 184, 1,               // 23934:        decode to SQDMULH_ZZZ_B using decoder 184
                                             // 23934:       }
  1, 0,                                      // 23939:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23941:        check predicate 35
  OPC_Decode, 138, 53, 184, 1,               // 23943:        decode to SQRDMULH_ZZZ_B using decoder 184
                                             // 23943:       }
                                             // 23943:       } // switch Inst[10]
                                             // 23943:      }
  1, 21,                                     // 23948:      case 0x1: {
  OPC_SwitchField, 10, 1,                    // 23950:       switch Inst[10] {
  0, 7,                                      // 23953:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23955:        check predicate 35
  OPC_Decode, 142, 52, 184, 1,               // 23957:        decode to SQDMULH_ZZZ_H using decoder 184
                                             // 23957:       }
  1, 0,                                      // 23962:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23964:        check predicate 35
  OPC_Decode, 140, 53, 184, 1,               // 23966:        decode to SQRDMULH_ZZZ_H using decoder 184
                                             // 23966:       }
                                             // 23966:       } // switch Inst[10]
                                             // 23966:      }
  2, 21,                                     // 23971:      case 0x2: {
  OPC_SwitchField, 10, 1,                    // 23973:       switch Inst[10] {
  0, 7,                                      // 23976:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 23978:        check predicate 35
  OPC_Decode, 143, 52, 184, 1,               // 23980:        decode to SQDMULH_ZZZ_S using decoder 184
                                             // 23980:       }
  1, 0,                                      // 23985:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 23987:        check predicate 35
  OPC_Decode, 141, 53, 184, 1,               // 23989:        decode to SQRDMULH_ZZZ_S using decoder 184
                                             // 23989:       }
                                             // 23989:       } // switch Inst[10]
                                             // 23989:      }
  3, 21,                                     // 23994:      case 0x3: {
  OPC_SwitchField, 10, 1,                    // 23996:       switch Inst[10] {
  0, 7,                                      // 23999:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 24001:        check predicate 35
  OPC_Decode, 141, 52, 184, 1,               // 24003:        decode to SQDMULH_ZZZ_D using decoder 184
                                             // 24003:       }
  1, 0,                                      // 24008:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 24010:        check predicate 35
  OPC_Decode, 139, 53, 184, 1,               // 24012:        decode to SQRDMULH_ZZZ_D using decoder 184
                                             // 24012:       }
                                             // 24012:       } // switch Inst[10]
                                             // 24012:      }
  4, 21,                                     // 24017:      case 0x4: {
  OPC_SwitchField, 10, 1,                    // 24019:       switch Inst[10] {
  0, 7,                                      // 24022:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24024:        check predicate 33
  OPC_Decode, 232, 61, 184, 1,               // 24026:        decode to TRN1_ZZZ_B using decoder 184
                                             // 24026:       }
  1, 0,                                      // 24031:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24033:        check predicate 33
  OPC_Decode, 248, 61, 184, 1,               // 24035:        decode to TRN2_ZZZ_B using decoder 184
                                             // 24035:       }
                                             // 24035:       } // switch Inst[10]
                                             // 24035:      }
  5, 21,                                     // 24040:      case 0x5: {
  OPC_SwitchField, 10, 1,                    // 24042:       switch Inst[10] {
  0, 7,                                      // 24045:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24047:        check predicate 33
  OPC_Decode, 234, 61, 184, 1,               // 24049:        decode to TRN1_ZZZ_H using decoder 184
                                             // 24049:       }
  1, 0,                                      // 24054:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24056:        check predicate 33
  OPC_Decode, 250, 61, 184, 1,               // 24058:        decode to TRN2_ZZZ_H using decoder 184
                                             // 24058:       }
                                             // 24058:       } // switch Inst[10]
                                             // 24058:      }
  6, 21,                                     // 24063:      case 0x6: {
  OPC_SwitchField, 10, 1,                    // 24065:       switch Inst[10] {
  0, 7,                                      // 24068:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24070:        check predicate 33
  OPC_Decode, 236, 61, 184, 1,               // 24072:        decode to TRN1_ZZZ_S using decoder 184
                                             // 24072:       }
  1, 0,                                      // 24077:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24079:        check predicate 33
  OPC_Decode, 252, 61, 184, 1,               // 24081:        decode to TRN2_ZZZ_S using decoder 184
                                             // 24081:       }
                                             // 24081:       } // switch Inst[10]
                                             // 24081:      }
  7, 0,                                      // 24086:      case 0x7: {
  OPC_SwitchField, 10, 1,                    // 24088:       switch Inst[10] {
  0, 7,                                      // 24091:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24093:        check predicate 33
  OPC_Decode, 233, 61, 184, 1,               // 24095:        decode to TRN1_ZZZ_D using decoder 184
                                             // 24095:       }
  1, 0,                                      // 24100:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24102:        check predicate 33
  OPC_Decode, 249, 61, 184, 1,               // 24104:        decode to TRN2_ZZZ_D using decoder 184
                                             // 24104:       }
                                             // 24104:       } // switch Inst[10]
                                             // 24104:      }
                                             // 24104:      } // switch Inst[25:22]
                                             // 24104:     }
  7, 0,                                      // 24109:     case 0x7: {
  OPC_SwitchField, 22, 4,                    // 24111:      switch Inst[25:22] {
  0, 21,                                     // 24114:      case 0x0: {
  OPC_SwitchField, 10, 1,                    // 24116:       switch Inst[10] {
  0, 7,                                      // 24119:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 24121:        check predicate 39
  OPC_Decode, 163, 13, 184, 1,               // 24123:        decode to ADDQP_ZZZ_B using decoder 184
                                             // 24123:       }
  1, 0,                                      // 24128:       case 0x1: {
  OPC_CheckPredicate, 39,                    // 24130:        check predicate 39
  OPC_Decode, 172, 13, 184, 1,               // 24132:        decode to ADDSUBP_ZZZ_B using decoder 184
                                             // 24132:       }
                                             // 24132:       } // switch Inst[10]
                                             // 24132:      }
  1, 21,                                     // 24137:      case 0x1: {
  OPC_SwitchField, 10, 1,                    // 24139:       switch Inst[10] {
  0, 7,                                      // 24142:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 24144:        check predicate 39
  OPC_Decode, 165, 13, 184, 1,               // 24146:        decode to ADDQP_ZZZ_H using decoder 184
                                             // 24146:       }
  1, 0,                                      // 24151:       case 0x1: {
  OPC_CheckPredicate, 39,                    // 24153:        check predicate 39
  OPC_Decode, 174, 13, 184, 1,               // 24155:        decode to ADDSUBP_ZZZ_H using decoder 184
                                             // 24155:       }
                                             // 24155:       } // switch Inst[10]
                                             // 24155:      }
  2, 21,                                     // 24160:      case 0x2: {
  OPC_SwitchField, 10, 1,                    // 24162:       switch Inst[10] {
  0, 7,                                      // 24165:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 24167:        check predicate 39
  OPC_Decode, 166, 13, 184, 1,               // 24169:        decode to ADDQP_ZZZ_S using decoder 184
                                             // 24169:       }
  1, 0,                                      // 24174:       case 0x1: {
  OPC_CheckPredicate, 39,                    // 24176:        check predicate 39
  OPC_Decode, 175, 13, 184, 1,               // 24178:        decode to ADDSUBP_ZZZ_S using decoder 184
                                             // 24178:       }
                                             // 24178:       } // switch Inst[10]
                                             // 24178:      }
  3, 0,                                      // 24183:      case 0x3: {
  OPC_SwitchField, 10, 1,                    // 24185:       switch Inst[10] {
  0, 7,                                      // 24188:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 24190:        check predicate 39
  OPC_Decode, 164, 13, 184, 1,               // 24192:        decode to ADDQP_ZZZ_D using decoder 184
                                             // 24192:       }
  1, 0,                                      // 24197:       case 0x1: {
  OPC_CheckPredicate, 39,                    // 24199:        check predicate 39
  OPC_Decode, 173, 13, 184, 1,               // 24201:        decode to ADDSUBP_ZZZ_D using decoder 184
                                             // 24201:       }
                                             // 24201:       } // switch Inst[10]
                                             // 24201:      }
                                             // 24201:      } // switch Inst[25:22]
                                             // 24201:     }
                                             // 24201:     } // switch Inst[13:11]
                                             // 24201:    }
  2, 234, 13,                                // 24206:    case 0x2: {
  OPC_SwitchField, 23, 3,                    // 24209:     switch Inst[25:23] {
  0, 206, 2,                                 // 24212:     case 0x0: {
  OPC_SwitchField, 10, 4,                    // 24215:      switch Inst[13:10] {
  0, 21,                                     // 24218:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 24220:       switch Inst[22] {
  0, 7,                                      // 24223:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24225:        check predicate 33
  OPC_Decode, 187, 14, 184, 1,               // 24227:        decode to ASR_WIDE_ZZZ_B using decoder 184
                                             // 24227:       }
  1, 0,                                      // 24232:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24234:        check predicate 33
  OPC_Decode, 188, 14, 184, 1,               // 24236:        decode to ASR_WIDE_ZZZ_H using decoder 184
                                             // 24236:       }
                                             // 24236:       } // switch Inst[22]
                                             // 24236:      }
  1, 21,                                     // 24241:      case 0x1: {
  OPC_SwitchField, 22, 1,                    // 24243:       switch Inst[22] {
  0, 7,                                      // 24246:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24248:        check predicate 33
  OPC_Decode, 172, 41, 184, 1,               // 24250:        decode to LSR_WIDE_ZZZ_B using decoder 184
                                             // 24250:       }
  1, 0,                                      // 24255:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24257:        check predicate 33
  OPC_Decode, 173, 41, 184, 1,               // 24259:        decode to LSR_WIDE_ZZZ_H using decoder 184
                                             // 24259:       }
                                             // 24259:       } // switch Inst[22]
                                             // 24259:      }
  3, 21,                                     // 24264:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 24266:       switch Inst[22] {
  0, 7,                                      // 24269:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24271:        check predicate 33
  OPC_Decode, 148, 41, 184, 1,               // 24273:        decode to LSL_WIDE_ZZZ_B using decoder 184
                                             // 24273:       }
  1, 0,                                      // 24278:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24280:        check predicate 33
  OPC_Decode, 149, 41, 184, 1,               // 24282:        decode to LSL_WIDE_ZZZ_H using decoder 184
                                             // 24282:       }
                                             // 24282:       } // switch Inst[22]
                                             // 24282:      }
  4, 39,                                     // 24287:      case 0x4: {
  OPC_SwitchField, 22, 1,                    // 24289:       switch Inst[22] {
  0, 25,                                     // 24292:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 24294:        switch Inst[20] {
  0, 11,                                     // 24297:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24299:         check predicate 33
  OPC_CheckField, 19, 1, 1,                  // 24301:         check Inst[19] == 0x1
  OPC_Decode, 198, 14, 229, 1,               // 24305:         decode to ASR_ZZI_B using decoder 229
                                             // 24305:        }
  1, 0,                                      // 24310:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24312:         check predicate 33
  OPC_Decode, 200, 14, 230, 1,               // 24314:         decode to ASR_ZZI_H using decoder 230
                                             // 24314:        }
                                             // 24314:        } // switch Inst[20]
                                             // 24314:       }
  1, 0,                                      // 24319:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24321:        check predicate 33
  OPC_Decode, 201, 14, 231, 1,               // 24323:        decode to ASR_ZZI_S using decoder 231
                                             // 24323:       }
                                             // 24323:       } // switch Inst[22]
                                             // 24323:      }
  5, 39,                                     // 24328:      case 0x5: {
  OPC_SwitchField, 22, 1,                    // 24330:       switch Inst[22] {
  0, 25,                                     // 24333:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 24335:        switch Inst[20] {
  0, 11,                                     // 24338:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24340:         check predicate 33
  OPC_CheckField, 19, 1, 1,                  // 24342:         check Inst[19] == 0x1
  OPC_Decode, 183, 41, 229, 1,               // 24346:         decode to LSR_ZZI_B using decoder 229
                                             // 24346:        }
  1, 0,                                      // 24351:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24353:         check predicate 33
  OPC_Decode, 185, 41, 230, 1,               // 24355:         decode to LSR_ZZI_H using decoder 230
                                             // 24355:        }
                                             // 24355:        } // switch Inst[20]
                                             // 24355:       }
  1, 0,                                      // 24360:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24362:        check predicate 33
  OPC_Decode, 186, 41, 231, 1,               // 24364:        decode to LSR_ZZI_S using decoder 231
                                             // 24364:       }
                                             // 24364:       } // switch Inst[22]
                                             // 24364:      }
  7, 39,                                     // 24369:      case 0x7: {
  OPC_SwitchField, 22, 1,                    // 24371:       switch Inst[22] {
  0, 25,                                     // 24374:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 24376:        switch Inst[20] {
  0, 11,                                     // 24379:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24381:         check predicate 33
  OPC_CheckField, 19, 1, 1,                  // 24383:         check Inst[19] == 0x1
  OPC_Decode, 159, 41, 232, 1,               // 24387:         decode to LSL_ZZI_B using decoder 232
                                             // 24387:        }
  1, 0,                                      // 24392:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24394:         check predicate 33
  OPC_Decode, 161, 41, 233, 1,               // 24396:         decode to LSL_ZZI_H using decoder 233
                                             // 24396:        }
                                             // 24396:        } // switch Inst[20]
                                             // 24396:       }
  1, 0,                                      // 24401:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24403:        check predicate 33
  OPC_Decode, 162, 41, 234, 1,               // 24405:        decode to LSL_ZZI_S using decoder 234
                                             // 24405:       }
                                             // 24405:       } // switch Inst[22]
                                             // 24405:      }
  8, 21,                                     // 24410:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 24412:       switch Inst[22] {
  0, 7,                                      // 24415:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24417:        check predicate 40
  OPC_Decode, 251, 13, 184, 1,               // 24419:        decode to ADR_SXTW_ZZZ_D_0 using decoder 184
                                             // 24419:       }
  1, 0,                                      // 24424:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24426:        check predicate 40
  OPC_Decode, 255, 13, 184, 1,               // 24428:        decode to ADR_UXTW_ZZZ_D_0 using decoder 184
                                             // 24428:       }
                                             // 24428:       } // switch Inst[22]
                                             // 24428:      }
  9, 21,                                     // 24433:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 24435:       switch Inst[22] {
  0, 7,                                      // 24438:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24440:        check predicate 40
  OPC_Decode, 252, 13, 184, 1,               // 24442:        decode to ADR_SXTW_ZZZ_D_1 using decoder 184
                                             // 24442:       }
  1, 0,                                      // 24447:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24449:        check predicate 40
  OPC_Decode, 128, 14, 184, 1,               // 24451:        decode to ADR_UXTW_ZZZ_D_1 using decoder 184
                                             // 24451:       }
                                             // 24451:       } // switch Inst[22]
                                             // 24451:      }
  10, 21,                                    // 24456:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 24458:       switch Inst[22] {
  0, 7,                                      // 24461:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24463:        check predicate 40
  OPC_Decode, 253, 13, 184, 1,               // 24465:        decode to ADR_SXTW_ZZZ_D_2 using decoder 184
                                             // 24465:       }
  1, 0,                                      // 24470:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24472:        check predicate 40
  OPC_Decode, 129, 14, 184, 1,               // 24474:        decode to ADR_UXTW_ZZZ_D_2 using decoder 184
                                             // 24474:       }
                                             // 24474:       } // switch Inst[22]
                                             // 24474:      }
  11, 21,                                    // 24479:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 24481:       switch Inst[22] {
  0, 7,                                      // 24484:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24486:        check predicate 40
  OPC_Decode, 254, 13, 184, 1,               // 24488:        decode to ADR_SXTW_ZZZ_D_3 using decoder 184
                                             // 24488:       }
  1, 0,                                      // 24493:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24495:        check predicate 40
  OPC_Decode, 130, 14, 184, 1,               // 24497:        decode to ADR_UXTW_ZZZ_D_3 using decoder 184
                                             // 24497:       }
                                             // 24497:       } // switch Inst[22]
                                             // 24497:      }
  12, 11,                                    // 24502:      case 0xc: {
  OPC_CheckPredicate, 40,                    // 24504:       check predicate 40
  OPC_CheckField, 22, 1, 1,                  // 24506:       check Inst[22] == 0x1
  OPC_Decode, 204, 32, 184, 1,               // 24510:       decode to FTSSEL_ZZZ_H using decoder 184
                                             // 24510:      }
  14, 15,                                    // 24515:      case 0xe: {
  OPC_CheckPredicate, 41,                    // 24517:       check predicate 41
  OPC_CheckField, 22, 1, 1,                  // 24519:       check Inst[22] == 0x1
  OPC_CheckField, 16, 5, 0,                  // 24523:       check Inst[20:16] == 0x0
  OPC_Decode, 164, 26, 212, 1,               // 24527:       decode to FEXPA_ZZ_H using decoder 212
                                             // 24527:      }
  15, 0,                                     // 24532:      case 0xf: {
  OPC_CheckPredicate, 33,                    // 24534:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 24536:       check Inst[22] == 0x0
  OPC_CheckField, 16, 5, 0,                  // 24540:       check Inst[20:16] == 0x0
  OPC_Decode, 236, 42, 212, 1,               // 24544:       decode to MOVPRFX_ZZ using decoder 212
                                             // 24544:      }
                                             // 24544:      } // switch Inst[13:10]
                                             // 24544:     }
  1, 215, 1,                                 // 24549:     case 0x1: {
  OPC_SwitchField, 10, 4,                    // 24552:      switch Inst[13:10] {
  0, 11,                                     // 24555:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 24557:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 24559:       check Inst[22] == 0x0
  OPC_Decode, 189, 14, 184, 1,               // 24563:       decode to ASR_WIDE_ZZZ_S using decoder 184
                                             // 24563:      }
  1, 11,                                     // 24568:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 24570:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 24572:       check Inst[22] == 0x0
  OPC_Decode, 174, 41, 184, 1,               // 24576:       decode to LSR_WIDE_ZZZ_S using decoder 184
                                             // 24576:      }
  3, 11,                                     // 24581:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 24583:       check predicate 33
  OPC_CheckField, 22, 1, 0,                  // 24585:       check Inst[22] == 0x0
  OPC_Decode, 150, 41, 184, 1,               // 24589:       decode to LSL_WIDE_ZZZ_S using decoder 184
                                             // 24589:      }
  4, 7,                                      // 24594:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 24596:       check predicate 33
  OPC_Decode, 199, 14, 235, 1,               // 24598:       decode to ASR_ZZI_D using decoder 235
                                             // 24598:      }
  5, 7,                                      // 24603:      case 0x5: {
  OPC_CheckPredicate, 33,                    // 24605:       check predicate 33
  OPC_Decode, 184, 41, 235, 1,               // 24607:       decode to LSR_ZZI_D using decoder 235
                                             // 24607:      }
  7, 7,                                      // 24612:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 24614:       check predicate 33
  OPC_Decode, 160, 41, 236, 1,               // 24616:       decode to LSL_ZZI_D using decoder 236
                                             // 24616:      }
  8, 21,                                     // 24621:      case 0x8: {
  OPC_SwitchField, 22, 1,                    // 24623:       switch Inst[22] {
  0, 7,                                      // 24626:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24628:        check predicate 40
  OPC_Decode, 247, 13, 184, 1,               // 24630:        decode to ADR_LSL_ZZZ_S_0 using decoder 184
                                             // 24630:       }
  1, 0,                                      // 24635:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24637:        check predicate 40
  OPC_Decode, 243, 13, 184, 1,               // 24639:        decode to ADR_LSL_ZZZ_D_0 using decoder 184
                                             // 24639:       }
                                             // 24639:       } // switch Inst[22]
                                             // 24639:      }
  9, 21,                                     // 24644:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 24646:       switch Inst[22] {
  0, 7,                                      // 24649:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24651:        check predicate 40
  OPC_Decode, 248, 13, 184, 1,               // 24653:        decode to ADR_LSL_ZZZ_S_1 using decoder 184
                                             // 24653:       }
  1, 0,                                      // 24658:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24660:        check predicate 40
  OPC_Decode, 244, 13, 184, 1,               // 24662:        decode to ADR_LSL_ZZZ_D_1 using decoder 184
                                             // 24662:       }
                                             // 24662:       } // switch Inst[22]
                                             // 24662:      }
  10, 21,                                    // 24667:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 24669:       switch Inst[22] {
  0, 7,                                      // 24672:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24674:        check predicate 40
  OPC_Decode, 249, 13, 184, 1,               // 24676:        decode to ADR_LSL_ZZZ_S_2 using decoder 184
                                             // 24676:       }
  1, 0,                                      // 24681:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24683:        check predicate 40
  OPC_Decode, 245, 13, 184, 1,               // 24685:        decode to ADR_LSL_ZZZ_D_2 using decoder 184
                                             // 24685:       }
                                             // 24685:       } // switch Inst[22]
                                             // 24685:      }
  11, 21,                                    // 24690:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 24692:       switch Inst[22] {
  0, 7,                                      // 24695:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24697:        check predicate 40
  OPC_Decode, 250, 13, 184, 1,               // 24699:        decode to ADR_LSL_ZZZ_S_3 using decoder 184
                                             // 24699:       }
  1, 0,                                      // 24704:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24706:        check predicate 40
  OPC_Decode, 246, 13, 184, 1,               // 24708:        decode to ADR_LSL_ZZZ_D_3 using decoder 184
                                             // 24708:       }
                                             // 24708:       } // switch Inst[22]
                                             // 24708:      }
  12, 21,                                    // 24713:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 24715:       switch Inst[22] {
  0, 7,                                      // 24718:       case 0x0: {
  OPC_CheckPredicate, 40,                    // 24720:        check predicate 40
  OPC_Decode, 205, 32, 184, 1,               // 24722:        decode to FTSSEL_ZZZ_S using decoder 184
                                             // 24722:       }
  1, 0,                                      // 24727:       case 0x1: {
  OPC_CheckPredicate, 40,                    // 24729:        check predicate 40
  OPC_Decode, 203, 32, 184, 1,               // 24731:        decode to FTSSEL_ZZZ_D using decoder 184
                                             // 24731:       }
                                             // 24731:       } // switch Inst[22]
                                             // 24731:      }
  14, 0,                                     // 24736:      case 0xe: {
  OPC_SwitchField, 22, 1,                    // 24738:       switch Inst[22] {
  0, 11,                                     // 24741:       case 0x0: {
  OPC_CheckPredicate, 41,                    // 24743:        check predicate 41
  OPC_CheckField, 16, 5, 0,                  // 24745:        check Inst[20:16] == 0x0
  OPC_Decode, 165, 26, 212, 1,               // 24749:        decode to FEXPA_ZZ_S using decoder 212
                                             // 24749:       }
  1, 0,                                      // 24754:       case 0x1: {
  OPC_CheckPredicate, 41,                    // 24756:        check predicate 41
  OPC_CheckField, 16, 5, 0,                  // 24758:        check Inst[20:16] == 0x0
  OPC_Decode, 163, 26, 212, 1,               // 24762:        decode to FEXPA_ZZ_D using decoder 212
                                             // 24762:       }
                                             // 24762:       } // switch Inst[22]
                                             // 24762:      }
                                             // 24762:      } // switch Inst[13:10]
                                             // 24762:     }
  2, 184, 4,                                 // 24767:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 24770:      switch Inst[20:16] {
  0, 49,                                     // 24773:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 24775:       switch Inst[13] {
  0, 21,                                     // 24778:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 24780:        switch Inst[22] {
  0, 7,                                      // 24783:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24785:         check predicate 33
  OPC_Decode, 151, 20, 237, 1,               // 24787:         decode to CPY_ZPmV_B using decoder 237
                                             // 24787:        }
  1, 0,                                      // 24792:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24794:         check predicate 33
  OPC_Decode, 153, 20, 238, 1,               // 24796:         decode to CPY_ZPmV_H using decoder 238
                                             // 24796:        }
                                             // 24796:        } // switch Inst[22]
                                             // 24796:       }
  1, 0,                                      // 24801:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 24803:        switch Inst[22] {
  0, 7,                                      // 24806:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24808:         check predicate 33
  OPC_Decode, 157, 34, 239, 1,               // 24810:         decode to LASTA_RPZ_B using decoder 239
                                             // 24810:        }
  1, 0,                                      // 24815:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24817:         check predicate 33
  OPC_Decode, 159, 34, 239, 1,               // 24819:         decode to LASTA_RPZ_H using decoder 239
                                             // 24819:        }
                                             // 24819:        } // switch Inst[22]
                                             // 24819:       }
                                             // 24819:       } // switch Inst[13]
                                             // 24819:      }
  1, 49,                                     // 24824:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 24826:       switch Inst[13] {
  0, 21,                                     // 24829:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 24831:        switch Inst[22] {
  0, 7,                                      // 24834:        case 0x0: {
  OPC_CheckPredicate, 42,                    // 24836:         check predicate 42
  OPC_Decode, 171, 19, 161, 1,               // 24838:         decode to COMPACT_ZPZ_B using decoder 161
                                             // 24838:        }
  1, 0,                                      // 24843:        case 0x1: {
  OPC_CheckPredicate, 42,                    // 24845:         check predicate 42
  OPC_Decode, 173, 19, 161, 1,               // 24847:         decode to COMPACT_ZPZ_H using decoder 161
                                             // 24847:        }
                                             // 24847:        } // switch Inst[22]
                                             // 24847:       }
  1, 0,                                      // 24852:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 24854:        switch Inst[22] {
  0, 7,                                      // 24857:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24859:         check predicate 33
  OPC_Decode, 165, 34, 239, 1,               // 24861:         decode to LASTB_RPZ_B using decoder 239
                                             // 24861:        }
  1, 0,                                      // 24866:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24868:         check predicate 33
  OPC_Decode, 167, 34, 239, 1,               // 24870:         decode to LASTB_RPZ_H using decoder 239
                                             // 24870:        }
                                             // 24870:        } // switch Inst[22]
                                             // 24870:       }
                                             // 24870:       } // switch Inst[13]
                                             // 24870:      }
  2, 29,                                     // 24875:      case 0x2: {
  OPC_SwitchField, 22, 1,                    // 24877:       switch Inst[22] {
  0, 11,                                     // 24880:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24882:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 24884:        check Inst[13] == 0x0
  OPC_Decode, 161, 34, 240, 1,               // 24888:        decode to LASTA_VPZ_B using decoder 240
                                             // 24888:       }
  1, 0,                                      // 24893:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24895:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 24897:        check Inst[13] == 0x0
  OPC_Decode, 163, 34, 241, 1,               // 24901:        decode to LASTA_VPZ_H using decoder 241
                                             // 24901:       }
                                             // 24901:       } // switch Inst[22]
                                             // 24901:      }
  3, 29,                                     // 24906:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 24908:       switch Inst[22] {
  0, 11,                                     // 24911:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24913:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 24915:        check Inst[13] == 0x0
  OPC_Decode, 169, 34, 240, 1,               // 24919:        decode to LASTB_VPZ_B using decoder 240
                                             // 24919:       }
  1, 0,                                      // 24924:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 24926:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 24928:        check Inst[13] == 0x0
  OPC_Decode, 171, 34, 241, 1,               // 24932:        decode to LASTB_VPZ_H using decoder 241
                                             // 24932:       }
                                             // 24932:       } // switch Inst[22]
                                             // 24932:      }
  4, 29,                                     // 24937:      case 0x4: {
  OPC_SwitchField, 13, 1,                    // 24939:       switch Inst[13] {
  0, 11,                                     // 24942:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 24944:        check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 24946:        check Inst[22] == 0x1
  OPC_Decode, 206, 45, 163, 1,               // 24950:        decode to REVB_ZPmZ_H using decoder 163
                                             // 24950:       }
  1, 0,                                      // 24955:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 24957:        check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 24959:        check Inst[22] == 0x1
  OPC_Decode, 209, 45, 161, 1,               // 24963:        decode to REVB_ZPzZ_H using decoder 161
                                             // 24963:       }
                                             // 24963:       } // switch Inst[13]
                                             // 24963:      }
  7, 49,                                     // 24968:      case 0x7: {
  OPC_SwitchField, 13, 1,                    // 24970:       switch Inst[13] {
  0, 21,                                     // 24973:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 24975:        switch Inst[22] {
  0, 7,                                      // 24978:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 24980:         check predicate 33
  OPC_Decode, 232, 44, 163, 1,               // 24982:         decode to RBIT_ZPmZ_B using decoder 163
                                             // 24982:        }
  1, 0,                                      // 24987:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 24989:         check predicate 33
  OPC_Decode, 234, 44, 163, 1,               // 24991:         decode to RBIT_ZPmZ_H using decoder 163
                                             // 24991:        }
                                             // 24991:        } // switch Inst[22]
                                             // 24991:       }
  1, 0,                                      // 24996:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 24998:        switch Inst[22] {
  0, 7,                                      // 25001:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 25003:         check predicate 36
  OPC_Decode, 236, 44, 161, 1,               // 25005:         decode to RBIT_ZPzZ_B using decoder 161
                                             // 25005:        }
  1, 0,                                      // 25010:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 25012:         check predicate 36
  OPC_Decode, 238, 44, 161, 1,               // 25014:         decode to RBIT_ZPzZ_H using decoder 161
                                             // 25014:        }
                                             // 25014:        } // switch Inst[22]
                                             // 25014:       }
                                             // 25014:       } // switch Inst[13]
                                             // 25014:      }
  8, 49,                                     // 25019:      case 0x8: {
  OPC_SwitchField, 13, 1,                    // 25021:       switch Inst[13] {
  0, 21,                                     // 25024:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25026:        switch Inst[22] {
  0, 7,                                      // 25029:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25031:         check predicate 33
  OPC_Decode, 158, 17, 160, 1,               // 25033:         decode to CLASTA_ZPZ_B using decoder 160
                                             // 25033:        }
  1, 0,                                      // 25038:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25040:         check predicate 33
  OPC_Decode, 160, 17, 160, 1,               // 25042:         decode to CLASTA_ZPZ_H using decoder 160
                                             // 25042:        }
                                             // 25042:        } // switch Inst[22]
                                             // 25042:       }
  1, 0,                                      // 25047:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25049:        switch Inst[22] {
  0, 7,                                      // 25052:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25054:         check predicate 33
  OPC_Decode, 147, 20, 242, 1,               // 25056:         decode to CPY_ZPmR_B using decoder 242
                                             // 25056:        }
  1, 0,                                      // 25061:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25063:         check predicate 33
  OPC_Decode, 149, 20, 242, 1,               // 25065:         decode to CPY_ZPmR_H using decoder 242
                                             // 25065:        }
                                             // 25065:        } // switch Inst[22]
                                             // 25065:       }
                                             // 25065:       } // switch Inst[13]
                                             // 25065:      }
  9, 29,                                     // 25070:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 25072:       switch Inst[22] {
  0, 11,                                     // 25075:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25077:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25079:        check Inst[13] == 0x0
  OPC_Decode, 170, 17, 160, 1,               // 25083:        decode to CLASTB_ZPZ_B using decoder 160
                                             // 25083:       }
  1, 0,                                      // 25088:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25090:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25092:        check Inst[13] == 0x0
  OPC_Decode, 172, 17, 160, 1,               // 25096:        decode to CLASTB_ZPZ_H using decoder 160
                                             // 25096:       }
                                             // 25096:       } // switch Inst[22]
                                             // 25096:      }
  10, 29,                                    // 25101:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 25103:       switch Inst[22] {
  0, 11,                                     // 25106:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25108:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25110:        check Inst[13] == 0x0
  OPC_Decode, 154, 17, 243, 1,               // 25114:        decode to CLASTA_VPZ_B using decoder 243
                                             // 25114:       }
  1, 0,                                      // 25119:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25121:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25123:        check Inst[13] == 0x0
  OPC_Decode, 156, 17, 244, 1,               // 25127:        decode to CLASTA_VPZ_H using decoder 244
                                             // 25127:       }
                                             // 25127:       } // switch Inst[22]
                                             // 25127:      }
  11, 29,                                    // 25132:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 25134:       switch Inst[22] {
  0, 11,                                     // 25137:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25139:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25141:        check Inst[13] == 0x0
  OPC_Decode, 166, 17, 243, 1,               // 25145:        decode to CLASTB_VPZ_B using decoder 243
                                             // 25145:       }
  1, 0,                                      // 25150:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25152:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25154:        check Inst[13] == 0x0
  OPC_Decode, 168, 17, 244, 1,               // 25158:        decode to CLASTB_VPZ_H using decoder 244
                                             // 25158:       }
                                             // 25158:       } // switch Inst[22]
                                             // 25158:      }
  12, 29,                                    // 25163:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 25165:       switch Inst[22] {
  0, 11,                                     // 25168:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25170:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25172:        check Inst[13] == 0x0
  OPC_Decode, 243, 50, 160, 1,               // 25176:        decode to SPLICE_ZPZ_B using decoder 160
                                             // 25176:       }
  1, 0,                                      // 25181:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25183:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25185:        check Inst[13] == 0x0
  OPC_Decode, 245, 50, 160, 1,               // 25189:        decode to SPLICE_ZPZ_H using decoder 160
                                             // 25189:       }
                                             // 25189:       } // switch Inst[22]
                                             // 25189:      }
  13, 29,                                    // 25194:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 25196:       switch Inst[22] {
  0, 11,                                     // 25199:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 25201:        check predicate 35
  OPC_CheckField, 13, 1, 0,                  // 25203:        check Inst[13] == 0x0
  OPC_Decode, 239, 50, 245, 1,               // 25207:        decode to SPLICE_ZPZZ_B using decoder 245
                                             // 25207:       }
  1, 0,                                      // 25212:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 25214:        check predicate 35
  OPC_CheckField, 13, 1, 0,                  // 25216:        check Inst[13] == 0x0
  OPC_Decode, 241, 50, 245, 1,               // 25220:        decode to SPLICE_ZPZZ_H using decoder 245
                                             // 25220:       }
                                             // 25220:       } // switch Inst[22]
                                             // 25220:      }
  14, 29,                                    // 25225:      case 0xe: {
  OPC_SwitchField, 13, 1,                    // 25227:       switch Inst[13] {
  0, 11,                                     // 25230:       case 0x0: {
  OPC_CheckPredicate, 43,                    // 25232:        check predicate 43
  OPC_CheckField, 22, 1, 0,                  // 25234:        check Inst[22] == 0x0
  OPC_Decode, 211, 45, 163, 1,               // 25238:        decode to REVD_ZPmZ using decoder 163
                                             // 25238:       }
  1, 0,                                      // 25243:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 25245:        check predicate 36
  OPC_CheckField, 22, 1, 0,                  // 25247:        check Inst[22] == 0x0
  OPC_Decode, 212, 45, 161, 1,               // 25251:        decode to REVD_ZPzZ using decoder 161
                                             // 25251:       }
                                             // 25251:       } // switch Inst[13]
                                             // 25251:      }
  16, 29,                                    // 25256:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 25258:       switch Inst[22] {
  0, 11,                                     // 25261:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25263:        check predicate 33
  OPC_CheckField, 13, 1, 1,                  // 25265:        check Inst[13] == 0x1
  OPC_Decode, 150, 17, 246, 1,               // 25269:        decode to CLASTA_RPZ_B using decoder 246
                                             // 25269:       }
  1, 0,                                      // 25274:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25276:        check predicate 33
  OPC_CheckField, 13, 1, 1,                  // 25278:        check Inst[13] == 0x1
  OPC_Decode, 152, 17, 246, 1,               // 25282:        decode to CLASTA_RPZ_H using decoder 246
                                             // 25282:       }
                                             // 25282:       } // switch Inst[22]
                                             // 25282:      }
  17, 0,                                     // 25287:      case 0x11: {
  OPC_SwitchField, 13, 1,                    // 25289:       switch Inst[13] {
  0, 21,                                     // 25292:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25294:        switch Inst[22] {
  0, 7,                                      // 25297:        case 0x0: {
  OPC_CheckPredicate, 42,                    // 25299:         check predicate 42
  OPC_Decode, 147, 21, 161, 1,               // 25301:         decode to EXPAND_ZPZ_B using decoder 161
                                             // 25301:        }
  1, 0,                                      // 25306:        case 0x1: {
  OPC_CheckPredicate, 42,                    // 25308:         check predicate 42
  OPC_Decode, 149, 21, 161, 1,               // 25310:         decode to EXPAND_ZPZ_H using decoder 161
                                             // 25310:        }
                                             // 25310:        } // switch Inst[22]
                                             // 25310:       }
  1, 0,                                      // 25315:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25317:        switch Inst[22] {
  0, 7,                                      // 25320:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25322:         check predicate 33
  OPC_Decode, 162, 17, 246, 1,               // 25324:         decode to CLASTB_RPZ_B using decoder 246
                                             // 25324:        }
  1, 0,                                      // 25329:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25331:         check predicate 33
  OPC_Decode, 164, 17, 246, 1,               // 25333:         decode to CLASTB_RPZ_H using decoder 246
                                             // 25333:        }
                                             // 25333:        } // switch Inst[22]
                                             // 25333:       }
                                             // 25333:       } // switch Inst[13]
                                             // 25333:      }
                                             // 25333:      } // switch Inst[20:16]
                                             // 25333:     }
  3, 0,                                      // 25338:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 25340:      switch Inst[20:16] {
  0, 49,                                     // 25343:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 25345:       switch Inst[13] {
  0, 21,                                     // 25348:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25350:        switch Inst[22] {
  0, 7,                                      // 25353:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25355:         check predicate 33
  OPC_Decode, 154, 20, 247, 1,               // 25357:         decode to CPY_ZPmV_S using decoder 247
                                             // 25357:        }
  1, 0,                                      // 25362:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25364:         check predicate 33
  OPC_Decode, 152, 20, 248, 1,               // 25366:         decode to CPY_ZPmV_D using decoder 248
                                             // 25366:        }
                                             // 25366:        } // switch Inst[22]
                                             // 25366:       }
  1, 0,                                      // 25371:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25373:        switch Inst[22] {
  0, 7,                                      // 25376:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25378:         check predicate 33
  OPC_Decode, 160, 34, 239, 1,               // 25380:         decode to LASTA_RPZ_S using decoder 239
                                             // 25380:        }
  1, 0,                                      // 25385:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25387:         check predicate 33
  OPC_Decode, 158, 34, 249, 1,               // 25389:         decode to LASTA_RPZ_D using decoder 249
                                             // 25389:        }
                                             // 25389:        } // switch Inst[22]
                                             // 25389:       }
                                             // 25389:       } // switch Inst[13]
                                             // 25389:      }
  1, 49,                                     // 25394:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 25396:       switch Inst[13] {
  0, 21,                                     // 25399:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25401:        switch Inst[22] {
  0, 7,                                      // 25404:        case 0x0: {
  OPC_CheckPredicate, 44,                    // 25406:         check predicate 44
  OPC_Decode, 174, 19, 161, 1,               // 25408:         decode to COMPACT_ZPZ_S using decoder 161
                                             // 25408:        }
  1, 0,                                      // 25413:        case 0x1: {
  OPC_CheckPredicate, 44,                    // 25415:         check predicate 44
  OPC_Decode, 172, 19, 161, 1,               // 25417:         decode to COMPACT_ZPZ_D using decoder 161
                                             // 25417:        }
                                             // 25417:        } // switch Inst[22]
                                             // 25417:       }
  1, 0,                                      // 25422:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25424:        switch Inst[22] {
  0, 7,                                      // 25427:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25429:         check predicate 33
  OPC_Decode, 168, 34, 239, 1,               // 25431:         decode to LASTB_RPZ_S using decoder 239
                                             // 25431:        }
  1, 0,                                      // 25436:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25438:         check predicate 33
  OPC_Decode, 166, 34, 249, 1,               // 25440:         decode to LASTB_RPZ_D using decoder 249
                                             // 25440:        }
                                             // 25440:        } // switch Inst[22]
                                             // 25440:       }
                                             // 25440:       } // switch Inst[13]
                                             // 25440:      }
  2, 29,                                     // 25445:      case 0x2: {
  OPC_SwitchField, 22, 1,                    // 25447:       switch Inst[22] {
  0, 11,                                     // 25450:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25452:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25454:        check Inst[13] == 0x0
  OPC_Decode, 164, 34, 250, 1,               // 25458:        decode to LASTA_VPZ_S using decoder 250
                                             // 25458:       }
  1, 0,                                      // 25463:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25465:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25467:        check Inst[13] == 0x0
  OPC_Decode, 162, 34, 251, 1,               // 25471:        decode to LASTA_VPZ_D using decoder 251
                                             // 25471:       }
                                             // 25471:       } // switch Inst[22]
                                             // 25471:      }
  3, 29,                                     // 25476:      case 0x3: {
  OPC_SwitchField, 22, 1,                    // 25478:       switch Inst[22] {
  0, 11,                                     // 25481:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25483:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25485:        check Inst[13] == 0x0
  OPC_Decode, 172, 34, 250, 1,               // 25489:        decode to LASTB_VPZ_S using decoder 250
                                             // 25489:       }
  1, 0,                                      // 25494:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25496:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25498:        check Inst[13] == 0x0
  OPC_Decode, 170, 34, 251, 1,               // 25502:        decode to LASTB_VPZ_D using decoder 251
                                             // 25502:       }
                                             // 25502:       } // switch Inst[22]
                                             // 25502:      }
  4, 49,                                     // 25507:      case 0x4: {
  OPC_SwitchField, 13, 1,                    // 25509:       switch Inst[13] {
  0, 21,                                     // 25512:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25514:        switch Inst[22] {
  0, 7,                                      // 25517:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25519:         check predicate 33
  OPC_Decode, 207, 45, 163, 1,               // 25521:         decode to REVB_ZPmZ_S using decoder 163
                                             // 25521:        }
  1, 0,                                      // 25526:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25528:         check predicate 33
  OPC_Decode, 205, 45, 163, 1,               // 25530:         decode to REVB_ZPmZ_D using decoder 163
                                             // 25530:        }
                                             // 25530:        } // switch Inst[22]
                                             // 25530:       }
  1, 0,                                      // 25535:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25537:        switch Inst[22] {
  0, 7,                                      // 25540:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 25542:         check predicate 36
  OPC_Decode, 210, 45, 161, 1,               // 25544:         decode to REVB_ZPzZ_S using decoder 161
                                             // 25544:        }
  1, 0,                                      // 25549:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 25551:         check predicate 36
  OPC_Decode, 208, 45, 161, 1,               // 25553:         decode to REVB_ZPzZ_D using decoder 161
                                             // 25553:        }
                                             // 25553:        } // switch Inst[22]
                                             // 25553:       }
                                             // 25553:       } // switch Inst[13]
                                             // 25553:      }
  5, 49,                                     // 25558:      case 0x5: {
  OPC_SwitchField, 13, 1,                    // 25560:       switch Inst[13] {
  0, 21,                                     // 25563:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25565:        switch Inst[22] {
  0, 7,                                      // 25568:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25570:         check predicate 33
  OPC_Decode, 214, 45, 163, 1,               // 25572:         decode to REVH_ZPmZ_S using decoder 163
                                             // 25572:        }
  1, 0,                                      // 25577:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25579:         check predicate 33
  OPC_Decode, 213, 45, 163, 1,               // 25581:         decode to REVH_ZPmZ_D using decoder 163
                                             // 25581:        }
                                             // 25581:        } // switch Inst[22]
                                             // 25581:       }
  1, 0,                                      // 25586:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25588:        switch Inst[22] {
  0, 7,                                      // 25591:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 25593:         check predicate 36
  OPC_Decode, 216, 45, 161, 1,               // 25595:         decode to REVH_ZPzZ_S using decoder 161
                                             // 25595:        }
  1, 0,                                      // 25600:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 25602:         check predicate 36
  OPC_Decode, 215, 45, 161, 1,               // 25604:         decode to REVH_ZPzZ_D using decoder 161
                                             // 25604:        }
                                             // 25604:        } // switch Inst[22]
                                             // 25604:       }
                                             // 25604:       } // switch Inst[13]
                                             // 25604:      }
  6, 29,                                     // 25609:      case 0x6: {
  OPC_SwitchField, 13, 1,                    // 25611:       switch Inst[13] {
  0, 11,                                     // 25614:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25616:        check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 25618:        check Inst[22] == 0x1
  OPC_Decode, 217, 45, 163, 1,               // 25622:        decode to REVW_ZPmZ_D using decoder 163
                                             // 25622:       }
  1, 0,                                      // 25627:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 25629:        check predicate 36
  OPC_CheckField, 22, 1, 1,                  // 25631:        check Inst[22] == 0x1
  OPC_Decode, 218, 45, 161, 1,               // 25635:        decode to REVW_ZPzZ_D using decoder 161
                                             // 25635:       }
                                             // 25635:       } // switch Inst[13]
                                             // 25635:      }
  7, 49,                                     // 25640:      case 0x7: {
  OPC_SwitchField, 13, 1,                    // 25642:       switch Inst[13] {
  0, 21,                                     // 25645:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25647:        switch Inst[22] {
  0, 7,                                      // 25650:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25652:         check predicate 33
  OPC_Decode, 235, 44, 163, 1,               // 25654:         decode to RBIT_ZPmZ_S using decoder 163
                                             // 25654:        }
  1, 0,                                      // 25659:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25661:         check predicate 33
  OPC_Decode, 233, 44, 163, 1,               // 25663:         decode to RBIT_ZPmZ_D using decoder 163
                                             // 25663:        }
                                             // 25663:        } // switch Inst[22]
                                             // 25663:       }
  1, 0,                                      // 25668:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25670:        switch Inst[22] {
  0, 7,                                      // 25673:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 25675:         check predicate 36
  OPC_Decode, 239, 44, 161, 1,               // 25677:         decode to RBIT_ZPzZ_S using decoder 161
                                             // 25677:        }
  1, 0,                                      // 25682:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 25684:         check predicate 36
  OPC_Decode, 237, 44, 161, 1,               // 25686:         decode to RBIT_ZPzZ_D using decoder 161
                                             // 25686:        }
                                             // 25686:        } // switch Inst[22]
                                             // 25686:       }
                                             // 25686:       } // switch Inst[13]
                                             // 25686:      }
  8, 49,                                     // 25691:      case 0x8: {
  OPC_SwitchField, 13, 1,                    // 25693:       switch Inst[13] {
  0, 21,                                     // 25696:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25698:        switch Inst[22] {
  0, 7,                                      // 25701:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25703:         check predicate 33
  OPC_Decode, 161, 17, 160, 1,               // 25705:         decode to CLASTA_ZPZ_S using decoder 160
                                             // 25705:        }
  1, 0,                                      // 25710:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25712:         check predicate 33
  OPC_Decode, 159, 17, 160, 1,               // 25714:         decode to CLASTA_ZPZ_D using decoder 160
                                             // 25714:        }
                                             // 25714:        } // switch Inst[22]
                                             // 25714:       }
  1, 0,                                      // 25719:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25721:        switch Inst[22] {
  0, 7,                                      // 25724:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25726:         check predicate 33
  OPC_Decode, 150, 20, 242, 1,               // 25728:         decode to CPY_ZPmR_S using decoder 242
                                             // 25728:        }
  1, 0,                                      // 25733:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25735:         check predicate 33
  OPC_Decode, 148, 20, 252, 1,               // 25737:         decode to CPY_ZPmR_D using decoder 252
                                             // 25737:        }
                                             // 25737:        } // switch Inst[22]
                                             // 25737:       }
                                             // 25737:       } // switch Inst[13]
                                             // 25737:      }
  9, 29,                                     // 25742:      case 0x9: {
  OPC_SwitchField, 22, 1,                    // 25744:       switch Inst[22] {
  0, 11,                                     // 25747:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25749:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25751:        check Inst[13] == 0x0
  OPC_Decode, 173, 17, 160, 1,               // 25755:        decode to CLASTB_ZPZ_S using decoder 160
                                             // 25755:       }
  1, 0,                                      // 25760:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25762:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25764:        check Inst[13] == 0x0
  OPC_Decode, 171, 17, 160, 1,               // 25768:        decode to CLASTB_ZPZ_D using decoder 160
                                             // 25768:       }
                                             // 25768:       } // switch Inst[22]
                                             // 25768:      }
  10, 29,                                    // 25773:      case 0xa: {
  OPC_SwitchField, 22, 1,                    // 25775:       switch Inst[22] {
  0, 11,                                     // 25778:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25780:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25782:        check Inst[13] == 0x0
  OPC_Decode, 157, 17, 253, 1,               // 25786:        decode to CLASTA_VPZ_S using decoder 253
                                             // 25786:       }
  1, 0,                                      // 25791:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25793:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25795:        check Inst[13] == 0x0
  OPC_Decode, 155, 17, 254, 1,               // 25799:        decode to CLASTA_VPZ_D using decoder 254
                                             // 25799:       }
                                             // 25799:       } // switch Inst[22]
                                             // 25799:      }
  11, 29,                                    // 25804:      case 0xb: {
  OPC_SwitchField, 22, 1,                    // 25806:       switch Inst[22] {
  0, 11,                                     // 25809:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25811:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25813:        check Inst[13] == 0x0
  OPC_Decode, 169, 17, 253, 1,               // 25817:        decode to CLASTB_VPZ_S using decoder 253
                                             // 25817:       }
  1, 0,                                      // 25822:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25824:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25826:        check Inst[13] == 0x0
  OPC_Decode, 167, 17, 254, 1,               // 25830:        decode to CLASTB_VPZ_D using decoder 254
                                             // 25830:       }
                                             // 25830:       } // switch Inst[22]
                                             // 25830:      }
  12, 29,                                    // 25835:      case 0xc: {
  OPC_SwitchField, 22, 1,                    // 25837:       switch Inst[22] {
  0, 11,                                     // 25840:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25842:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25844:        check Inst[13] == 0x0
  OPC_Decode, 246, 50, 160, 1,               // 25848:        decode to SPLICE_ZPZ_S using decoder 160
                                             // 25848:       }
  1, 0,                                      // 25853:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25855:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 25857:        check Inst[13] == 0x0
  OPC_Decode, 244, 50, 160, 1,               // 25861:        decode to SPLICE_ZPZ_D using decoder 160
                                             // 25861:       }
                                             // 25861:       } // switch Inst[22]
                                             // 25861:      }
  13, 29,                                    // 25866:      case 0xd: {
  OPC_SwitchField, 22, 1,                    // 25868:       switch Inst[22] {
  0, 11,                                     // 25871:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 25873:        check predicate 35
  OPC_CheckField, 13, 1, 0,                  // 25875:        check Inst[13] == 0x0
  OPC_Decode, 242, 50, 245, 1,               // 25879:        decode to SPLICE_ZPZZ_S using decoder 245
                                             // 25879:       }
  1, 0,                                      // 25884:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 25886:        check predicate 35
  OPC_CheckField, 13, 1, 0,                  // 25888:        check Inst[13] == 0x0
  OPC_Decode, 240, 50, 245, 1,               // 25892:        decode to SPLICE_ZPZZ_D using decoder 245
                                             // 25892:       }
                                             // 25892:       } // switch Inst[22]
                                             // 25892:      }
  16, 29,                                    // 25897:      case 0x10: {
  OPC_SwitchField, 22, 1,                    // 25899:       switch Inst[22] {
  0, 11,                                     // 25902:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25904:        check predicate 33
  OPC_CheckField, 13, 1, 1,                  // 25906:        check Inst[13] == 0x1
  OPC_Decode, 153, 17, 246, 1,               // 25910:        decode to CLASTA_RPZ_S using decoder 246
                                             // 25910:       }
  1, 0,                                      // 25915:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 25917:        check predicate 33
  OPC_CheckField, 13, 1, 1,                  // 25919:        check Inst[13] == 0x1
  OPC_Decode, 151, 17, 255, 1,               // 25923:        decode to CLASTA_RPZ_D using decoder 255
                                             // 25923:       }
                                             // 25923:       } // switch Inst[22]
                                             // 25923:      }
  17, 0,                                     // 25928:      case 0x11: {
  OPC_SwitchField, 13, 1,                    // 25930:       switch Inst[13] {
  0, 21,                                     // 25933:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 25935:        switch Inst[22] {
  0, 7,                                      // 25938:        case 0x0: {
  OPC_CheckPredicate, 42,                    // 25940:         check predicate 42
  OPC_Decode, 150, 21, 161, 1,               // 25942:         decode to EXPAND_ZPZ_S using decoder 161
                                             // 25942:        }
  1, 0,                                      // 25947:        case 0x1: {
  OPC_CheckPredicate, 42,                    // 25949:         check predicate 42
  OPC_Decode, 148, 21, 161, 1,               // 25951:         decode to EXPAND_ZPZ_D using decoder 161
                                             // 25951:        }
                                             // 25951:        } // switch Inst[22]
                                             // 25951:       }
  1, 0,                                      // 25956:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 25958:        switch Inst[22] {
  0, 7,                                      // 25961:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 25963:         check predicate 33
  OPC_Decode, 165, 17, 246, 1,               // 25965:         decode to CLASTB_RPZ_S using decoder 246
                                             // 25965:        }
  1, 0,                                      // 25970:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 25972:         check predicate 33
  OPC_Decode, 163, 17, 255, 1,               // 25974:         decode to CLASTB_RPZ_D using decoder 255
                                             // 25974:        }
                                             // 25974:        } // switch Inst[22]
                                             // 25974:       }
                                             // 25974:       } // switch Inst[13]
                                             // 25974:      }
                                             // 25974:      } // switch Inst[20:16]
                                             // 25974:     }
                                             // 25974:     } // switch Inst[25:23]
                                             // 25974:    }
  3, 0,                                      // 25979:    case 0x3: {
  OPC_SwitchField, 22, 4,                    // 25981:     switch Inst[25:22] {
  0, 131, 1,                                 // 25984:     case 0x0: {
  OPC_SwitchField, 10, 4,                    // 25987:      switch Inst[13:10] {
  8, 21,                                     // 25990:      case 0x8: {
  OPC_SwitchField, 20, 1,                    // 25992:       switch Inst[20] {
  0, 7,                                      // 25995:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 25997:        check predicate 33
  OPC_Decode, 147, 19, 128, 2,               // 25999:        decode to CNTB_XPiI using decoder 256
                                             // 25999:       }
  1, 0,                                      // 26004:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26006:        check predicate 33
  OPC_Decode, 227, 33, 129, 2,               // 26008:        decode to INCB_XPiI using decoder 257
                                             // 26008:       }
                                             // 26008:       } // switch Inst[20]
                                             // 26008:      }
  9, 11,                                     // 26013:      case 0x9: {
  OPC_CheckPredicate, 33,                    // 26015:       check predicate 33
  OPC_CheckField, 20, 1, 1,                  // 26017:       check Inst[20] == 0x1
  OPC_Decode, 184, 20, 129, 2,               // 26021:       decode to DECB_XPiI using decoder 257
                                             // 26021:      }
  12, 21,                                    // 26026:      case 0xc: {
  OPC_SwitchField, 20, 1,                    // 26028:       switch Inst[20] {
  0, 7,                                      // 26031:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26033:        check predicate 33
  OPC_Decode, 179, 52, 129, 2,               // 26035:        decode to SQINCB_XPiWdI using decoder 257
                                             // 26035:       }
  1, 0,                                      // 26040:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26042:        check predicate 33
  OPC_Decode, 178, 52, 129, 2,               // 26044:        decode to SQINCB_XPiI using decoder 257
                                             // 26044:       }
                                             // 26044:       } // switch Inst[20]
                                             // 26044:      }
  13, 21,                                    // 26049:      case 0xd: {
  OPC_SwitchField, 20, 1,                    // 26051:       switch Inst[20] {
  0, 7,                                      // 26054:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26056:        check predicate 33
  OPC_Decode, 169, 66, 130, 2,               // 26058:        decode to UQINCB_WPiI using decoder 258
                                             // 26058:       }
  1, 0,                                      // 26063:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26065:        check predicate 33
  OPC_Decode, 170, 66, 129, 2,               // 26067:        decode to UQINCB_XPiI using decoder 257
                                             // 26067:       }
                                             // 26067:       } // switch Inst[20]
                                             // 26067:      }
  14, 21,                                    // 26072:      case 0xe: {
  OPC_SwitchField, 20, 1,                    // 26074:       switch Inst[20] {
  0, 7,                                      // 26077:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26079:        check predicate 33
  OPC_Decode, 178, 51, 129, 2,               // 26081:        decode to SQDECB_XPiWdI using decoder 257
                                             // 26081:       }
  1, 0,                                      // 26086:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26088:        check predicate 33
  OPC_Decode, 177, 51, 129, 2,               // 26090:        decode to SQDECB_XPiI using decoder 257
                                             // 26090:       }
                                             // 26090:       } // switch Inst[20]
                                             // 26090:      }
  15, 0,                                     // 26095:      case 0xf: {
  OPC_SwitchField, 20, 1,                    // 26097:       switch Inst[20] {
  0, 7,                                      // 26100:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26102:        check predicate 33
  OPC_Decode, 147, 66, 130, 2,               // 26104:        decode to UQDECB_WPiI using decoder 258
                                             // 26104:       }
  1, 0,                                      // 26109:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26111:        check predicate 33
  OPC_Decode, 148, 66, 129, 2,               // 26113:        decode to UQDECB_XPiI using decoder 257
                                             // 26113:       }
                                             // 26113:       } // switch Inst[20]
                                             // 26113:      }
                                             // 26113:      } // switch Inst[13:10]
                                             // 26113:     }
  1, 203, 1,                                 // 26118:     case 0x1: {
  OPC_SwitchField, 10, 4,                    // 26121:      switch Inst[13:10] {
  0, 21,                                     // 26124:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 26126:       switch Inst[20] {
  0, 7,                                      // 26129:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26131:        check predicate 33
  OPC_Decode, 185, 52, 131, 2,               // 26133:        decode to SQINCH_ZPiI using decoder 259
                                             // 26133:       }
  1, 0,                                      // 26138:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26140:        check predicate 33
  OPC_Decode, 231, 33, 131, 2,               // 26142:        decode to INCH_ZPiI using decoder 259
                                             // 26142:       }
                                             // 26142:       } // switch Inst[20]
                                             // 26142:      }
  1, 21,                                     // 26147:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 26149:       switch Inst[20] {
  0, 7,                                      // 26152:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26154:        check predicate 33
  OPC_Decode, 176, 66, 131, 2,               // 26156:        decode to UQINCH_ZPiI using decoder 259
                                             // 26156:       }
  1, 0,                                      // 26161:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26163:        check predicate 33
  OPC_Decode, 188, 20, 131, 2,               // 26165:        decode to DECH_ZPiI using decoder 259
                                             // 26165:       }
                                             // 26165:       } // switch Inst[20]
                                             // 26165:      }
  2, 11,                                     // 26170:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 26172:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 26174:       check Inst[20] == 0x0
  OPC_Decode, 184, 51, 131, 2,               // 26178:       decode to SQDECH_ZPiI using decoder 259
                                             // 26178:      }
  3, 11,                                     // 26183:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 26185:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 26187:       check Inst[20] == 0x0
  OPC_Decode, 154, 66, 131, 2,               // 26191:       decode to UQDECH_ZPiI using decoder 259
                                             // 26191:      }
  8, 21,                                     // 26196:      case 0x8: {
  OPC_SwitchField, 20, 1,                    // 26198:       switch Inst[20] {
  0, 7,                                      // 26201:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26203:        check predicate 33
  OPC_Decode, 149, 19, 128, 2,               // 26205:        decode to CNTH_XPiI using decoder 256
                                             // 26205:       }
  1, 0,                                      // 26210:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26212:        check predicate 33
  OPC_Decode, 230, 33, 129, 2,               // 26214:        decode to INCH_XPiI using decoder 257
                                             // 26214:       }
                                             // 26214:       } // switch Inst[20]
                                             // 26214:      }
  9, 11,                                     // 26219:      case 0x9: {
  OPC_CheckPredicate, 33,                    // 26221:       check predicate 33
  OPC_CheckField, 20, 1, 1,                  // 26223:       check Inst[20] == 0x1
  OPC_Decode, 187, 20, 129, 2,               // 26227:       decode to DECH_XPiI using decoder 257
                                             // 26227:      }
  12, 21,                                    // 26232:      case 0xc: {
  OPC_SwitchField, 20, 1,                    // 26234:       switch Inst[20] {
  0, 7,                                      // 26237:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26239:        check predicate 33
  OPC_Decode, 184, 52, 129, 2,               // 26241:        decode to SQINCH_XPiWdI using decoder 257
                                             // 26241:       }
  1, 0,                                      // 26246:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26248:        check predicate 33
  OPC_Decode, 183, 52, 129, 2,               // 26250:        decode to SQINCH_XPiI using decoder 257
                                             // 26250:       }
                                             // 26250:       } // switch Inst[20]
                                             // 26250:      }
  13, 21,                                    // 26255:      case 0xd: {
  OPC_SwitchField, 20, 1,                    // 26257:       switch Inst[20] {
  0, 7,                                      // 26260:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26262:        check predicate 33
  OPC_Decode, 174, 66, 130, 2,               // 26264:        decode to UQINCH_WPiI using decoder 258
                                             // 26264:       }
  1, 0,                                      // 26269:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26271:        check predicate 33
  OPC_Decode, 175, 66, 129, 2,               // 26273:        decode to UQINCH_XPiI using decoder 257
                                             // 26273:       }
                                             // 26273:       } // switch Inst[20]
                                             // 26273:      }
  14, 21,                                    // 26278:      case 0xe: {
  OPC_SwitchField, 20, 1,                    // 26280:       switch Inst[20] {
  0, 7,                                      // 26283:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26285:        check predicate 33
  OPC_Decode, 183, 51, 129, 2,               // 26287:        decode to SQDECH_XPiWdI using decoder 257
                                             // 26287:       }
  1, 0,                                      // 26292:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26294:        check predicate 33
  OPC_Decode, 182, 51, 129, 2,               // 26296:        decode to SQDECH_XPiI using decoder 257
                                             // 26296:       }
                                             // 26296:       } // switch Inst[20]
                                             // 26296:      }
  15, 0,                                     // 26301:      case 0xf: {
  OPC_SwitchField, 20, 1,                    // 26303:       switch Inst[20] {
  0, 7,                                      // 26306:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26308:        check predicate 33
  OPC_Decode, 152, 66, 130, 2,               // 26310:        decode to UQDECH_WPiI using decoder 258
                                             // 26310:       }
  1, 0,                                      // 26315:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26317:        check predicate 33
  OPC_Decode, 153, 66, 129, 2,               // 26319:        decode to UQDECH_XPiI using decoder 257
                                             // 26319:       }
                                             // 26319:       } // switch Inst[20]
                                             // 26319:      }
                                             // 26319:      } // switch Inst[13:10]
                                             // 26319:     }
  2, 203, 1,                                 // 26324:     case 0x2: {
  OPC_SwitchField, 10, 4,                    // 26327:      switch Inst[13:10] {
  0, 21,                                     // 26330:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 26332:       switch Inst[20] {
  0, 7,                                      // 26335:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26337:        check predicate 33
  OPC_Decode, 199, 52, 131, 2,               // 26339:        decode to SQINCW_ZPiI using decoder 259
                                             // 26339:       }
  1, 0,                                      // 26344:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26346:        check predicate 33
  OPC_Decode, 240, 33, 131, 2,               // 26348:        decode to INCW_ZPiI using decoder 259
                                             // 26348:       }
                                             // 26348:       } // switch Inst[20]
                                             // 26348:      }
  1, 21,                                     // 26353:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 26355:       switch Inst[20] {
  0, 7,                                      // 26358:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26360:        check predicate 33
  OPC_Decode, 190, 66, 131, 2,               // 26362:        decode to UQINCW_ZPiI using decoder 259
                                             // 26362:       }
  1, 0,                                      // 26367:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26369:        check predicate 33
  OPC_Decode, 197, 20, 131, 2,               // 26371:        decode to DECW_ZPiI using decoder 259
                                             // 26371:       }
                                             // 26371:       } // switch Inst[20]
                                             // 26371:      }
  2, 11,                                     // 26376:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 26378:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 26380:       check Inst[20] == 0x0
  OPC_Decode, 198, 51, 131, 2,               // 26384:       decode to SQDECW_ZPiI using decoder 259
                                             // 26384:      }
  3, 11,                                     // 26389:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 26391:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 26393:       check Inst[20] == 0x0
  OPC_Decode, 168, 66, 131, 2,               // 26397:       decode to UQDECW_ZPiI using decoder 259
                                             // 26397:      }
  8, 21,                                     // 26402:      case 0x8: {
  OPC_SwitchField, 20, 1,                    // 26404:       switch Inst[20] {
  0, 7,                                      // 26407:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26409:        check predicate 33
  OPC_Decode, 158, 19, 128, 2,               // 26411:        decode to CNTW_XPiI using decoder 256
                                             // 26411:       }
  1, 0,                                      // 26416:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26418:        check predicate 33
  OPC_Decode, 239, 33, 129, 2,               // 26420:        decode to INCW_XPiI using decoder 257
                                             // 26420:       }
                                             // 26420:       } // switch Inst[20]
                                             // 26420:      }
  9, 11,                                     // 26425:      case 0x9: {
  OPC_CheckPredicate, 33,                    // 26427:       check predicate 33
  OPC_CheckField, 20, 1, 1,                  // 26429:       check Inst[20] == 0x1
  OPC_Decode, 196, 20, 129, 2,               // 26433:       decode to DECW_XPiI using decoder 257
                                             // 26433:      }
  12, 21,                                    // 26438:      case 0xc: {
  OPC_SwitchField, 20, 1,                    // 26440:       switch Inst[20] {
  0, 7,                                      // 26443:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26445:        check predicate 33
  OPC_Decode, 198, 52, 129, 2,               // 26447:        decode to SQINCW_XPiWdI using decoder 257
                                             // 26447:       }
  1, 0,                                      // 26452:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26454:        check predicate 33
  OPC_Decode, 197, 52, 129, 2,               // 26456:        decode to SQINCW_XPiI using decoder 257
                                             // 26456:       }
                                             // 26456:       } // switch Inst[20]
                                             // 26456:      }
  13, 21,                                    // 26461:      case 0xd: {
  OPC_SwitchField, 20, 1,                    // 26463:       switch Inst[20] {
  0, 7,                                      // 26466:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26468:        check predicate 33
  OPC_Decode, 188, 66, 130, 2,               // 26470:        decode to UQINCW_WPiI using decoder 258
                                             // 26470:       }
  1, 0,                                      // 26475:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26477:        check predicate 33
  OPC_Decode, 189, 66, 129, 2,               // 26479:        decode to UQINCW_XPiI using decoder 257
                                             // 26479:       }
                                             // 26479:       } // switch Inst[20]
                                             // 26479:      }
  14, 21,                                    // 26484:      case 0xe: {
  OPC_SwitchField, 20, 1,                    // 26486:       switch Inst[20] {
  0, 7,                                      // 26489:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26491:        check predicate 33
  OPC_Decode, 197, 51, 129, 2,               // 26493:        decode to SQDECW_XPiWdI using decoder 257
                                             // 26493:       }
  1, 0,                                      // 26498:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26500:        check predicate 33
  OPC_Decode, 196, 51, 129, 2,               // 26502:        decode to SQDECW_XPiI using decoder 257
                                             // 26502:       }
                                             // 26502:       } // switch Inst[20]
                                             // 26502:      }
  15, 0,                                     // 26507:      case 0xf: {
  OPC_SwitchField, 20, 1,                    // 26509:       switch Inst[20] {
  0, 7,                                      // 26512:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26514:        check predicate 33
  OPC_Decode, 166, 66, 130, 2,               // 26516:        decode to UQDECW_WPiI using decoder 258
                                             // 26516:       }
  1, 0,                                      // 26521:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26523:        check predicate 33
  OPC_Decode, 167, 66, 129, 2,               // 26525:        decode to UQDECW_XPiI using decoder 257
                                             // 26525:       }
                                             // 26525:       } // switch Inst[20]
                                             // 26525:      }
                                             // 26525:      } // switch Inst[13:10]
                                             // 26525:     }
  3, 203, 1,                                 // 26530:     case 0x3: {
  OPC_SwitchField, 10, 4,                    // 26533:      switch Inst[13:10] {
  0, 21,                                     // 26536:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 26538:       switch Inst[20] {
  0, 7,                                      // 26541:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26543:        check predicate 33
  OPC_Decode, 182, 52, 131, 2,               // 26545:        decode to SQINCD_ZPiI using decoder 259
                                             // 26545:       }
  1, 0,                                      // 26550:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26552:        check predicate 33
  OPC_Decode, 229, 33, 131, 2,               // 26554:        decode to INCD_ZPiI using decoder 259
                                             // 26554:       }
                                             // 26554:       } // switch Inst[20]
                                             // 26554:      }
  1, 21,                                     // 26559:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 26561:       switch Inst[20] {
  0, 7,                                      // 26564:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26566:        check predicate 33
  OPC_Decode, 173, 66, 131, 2,               // 26568:        decode to UQINCD_ZPiI using decoder 259
                                             // 26568:       }
  1, 0,                                      // 26573:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26575:        check predicate 33
  OPC_Decode, 186, 20, 131, 2,               // 26577:        decode to DECD_ZPiI using decoder 259
                                             // 26577:       }
                                             // 26577:       } // switch Inst[20]
                                             // 26577:      }
  2, 11,                                     // 26582:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 26584:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 26586:       check Inst[20] == 0x0
  OPC_Decode, 181, 51, 131, 2,               // 26590:       decode to SQDECD_ZPiI using decoder 259
                                             // 26590:      }
  3, 11,                                     // 26595:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 26597:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 26599:       check Inst[20] == 0x0
  OPC_Decode, 151, 66, 131, 2,               // 26603:       decode to UQDECD_ZPiI using decoder 259
                                             // 26603:      }
  8, 21,                                     // 26608:      case 0x8: {
  OPC_SwitchField, 20, 1,                    // 26610:       switch Inst[20] {
  0, 7,                                      // 26613:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26615:        check predicate 33
  OPC_Decode, 148, 19, 128, 2,               // 26617:        decode to CNTD_XPiI using decoder 256
                                             // 26617:       }
  1, 0,                                      // 26622:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26624:        check predicate 33
  OPC_Decode, 228, 33, 129, 2,               // 26626:        decode to INCD_XPiI using decoder 257
                                             // 26626:       }
                                             // 26626:       } // switch Inst[20]
                                             // 26626:      }
  9, 11,                                     // 26631:      case 0x9: {
  OPC_CheckPredicate, 33,                    // 26633:       check predicate 33
  OPC_CheckField, 20, 1, 1,                  // 26635:       check Inst[20] == 0x1
  OPC_Decode, 185, 20, 129, 2,               // 26639:       decode to DECD_XPiI using decoder 257
                                             // 26639:      }
  12, 21,                                    // 26644:      case 0xc: {
  OPC_SwitchField, 20, 1,                    // 26646:       switch Inst[20] {
  0, 7,                                      // 26649:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26651:        check predicate 33
  OPC_Decode, 181, 52, 129, 2,               // 26653:        decode to SQINCD_XPiWdI using decoder 257
                                             // 26653:       }
  1, 0,                                      // 26658:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26660:        check predicate 33
  OPC_Decode, 180, 52, 129, 2,               // 26662:        decode to SQINCD_XPiI using decoder 257
                                             // 26662:       }
                                             // 26662:       } // switch Inst[20]
                                             // 26662:      }
  13, 21,                                    // 26667:      case 0xd: {
  OPC_SwitchField, 20, 1,                    // 26669:       switch Inst[20] {
  0, 7,                                      // 26672:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26674:        check predicate 33
  OPC_Decode, 171, 66, 130, 2,               // 26676:        decode to UQINCD_WPiI using decoder 258
                                             // 26676:       }
  1, 0,                                      // 26681:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26683:        check predicate 33
  OPC_Decode, 172, 66, 129, 2,               // 26685:        decode to UQINCD_XPiI using decoder 257
                                             // 26685:       }
                                             // 26685:       } // switch Inst[20]
                                             // 26685:      }
  14, 21,                                    // 26690:      case 0xe: {
  OPC_SwitchField, 20, 1,                    // 26692:       switch Inst[20] {
  0, 7,                                      // 26695:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26697:        check predicate 33
  OPC_Decode, 180, 51, 129, 2,               // 26699:        decode to SQDECD_XPiWdI using decoder 257
                                             // 26699:       }
  1, 0,                                      // 26704:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26706:        check predicate 33
  OPC_Decode, 179, 51, 129, 2,               // 26708:        decode to SQDECD_XPiI using decoder 257
                                             // 26708:       }
                                             // 26708:       } // switch Inst[20]
                                             // 26708:      }
  15, 0,                                     // 26713:      case 0xf: {
  OPC_SwitchField, 20, 1,                    // 26715:       switch Inst[20] {
  0, 7,                                      // 26718:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 26720:        check predicate 33
  OPC_Decode, 149, 66, 130, 2,               // 26722:        decode to UQDECD_WPiI using decoder 258
                                             // 26722:       }
  1, 0,                                      // 26727:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 26729:        check predicate 33
  OPC_Decode, 150, 66, 129, 2,               // 26731:        decode to UQDECD_XPiI using decoder 257
                                             // 26731:       }
                                             // 26731:       } // switch Inst[20]
                                             // 26731:      }
                                             // 26731:      } // switch Inst[13:10]
                                             // 26731:     }
  4, 7,                                      // 26736:     case 0x4: {
  OPC_CheckPredicate, 33,                    // 26738:      check predicate 33
  OPC_Decode, 220, 47, 132, 2,               // 26740:      decode to SEL_ZPZZ_B using decoder 260
                                             // 26740:     }
  5, 7,                                      // 26745:     case 0x5: {
  OPC_CheckPredicate, 33,                    // 26747:      check predicate 33
  OPC_Decode, 222, 47, 132, 2,               // 26749:      decode to SEL_ZPZZ_H using decoder 260
                                             // 26749:     }
  6, 7,                                      // 26754:     case 0x6: {
  OPC_CheckPredicate, 33,                    // 26756:      check predicate 33
  OPC_Decode, 223, 47, 132, 2,               // 26758:      decode to SEL_ZPZZ_S using decoder 260
                                             // 26758:     }
  7, 0,                                      // 26763:     case 0x7: {
  OPC_CheckPredicate, 33,                    // 26765:      check predicate 33
  OPC_Decode, 221, 47, 132, 2,               // 26767:      decode to SEL_ZPZZ_D using decoder 260
                                             // 26767:     }
                                             // 26767:     } // switch Inst[25:22]
                                             // 26767:    }
                                             // 26767:    } // switch Inst[15:14]
                                             // 26767:   }
                                             // 26767:   } // switch Inst[21]
                                             // 26767:  }
  1, 133, 44,                                // 26772:  case 0x1: {
  OPC_SwitchField, 21, 1,                    // 26775:   switch Inst[21] {
  0, 196, 13,                                // 26778:   case 0x0: {
  OPC_SwitchField, 22, 4,                    // 26781:    switch Inst[25:22] {
  0, 187, 1,                                 // 26784:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 26787:     switch Inst[15:13] {
  0, 21,                                     // 26790:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 26792:      switch Inst[4] {
  0, 7,                                      // 26795:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26797:       check predicate 33
  OPC_Decode, 213, 18, 133, 2,               // 26799:       decode to CMPHS_PPzZZ_B using decoder 261
                                             // 26799:      }
  1, 0,                                      // 26804:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26806:       check predicate 33
  OPC_Decode, 202, 18, 133, 2,               // 26808:       decode to CMPHI_PPzZZ_B using decoder 261
                                             // 26808:      }
                                             // 26808:      } // switch Inst[4]
                                             // 26808:     }
  1, 21,                                     // 26813:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 26815:      switch Inst[4] {
  0, 7,                                      // 26818:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26820:       check predicate 33
  OPC_Decode, 173, 18, 133, 2,               // 26822:       decode to CMPEQ_WIDE_PPzZZ_B using decoder 261
                                             // 26822:      }
  1, 0,                                      // 26827:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26829:       check predicate 33
  OPC_Decode, 128, 19, 133, 2,               // 26831:       decode to CMPNE_WIDE_PPzZZ_B using decoder 261
                                             // 26831:      }
                                             // 26831:      } // switch Inst[4]
                                             // 26831:     }
  2, 21,                                     // 26836:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 26838:      switch Inst[4] {
  0, 7,                                      // 26841:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26843:       check predicate 33
  OPC_Decode, 184, 18, 133, 2,               // 26845:       decode to CMPGE_WIDE_PPzZZ_B using decoder 261
                                             // 26845:      }
  1, 0,                                      // 26850:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26852:       check predicate 33
  OPC_Decode, 195, 18, 133, 2,               // 26854:       decode to CMPGT_WIDE_PPzZZ_B using decoder 261
                                             // 26854:      }
                                             // 26854:      } // switch Inst[4]
                                             // 26854:     }
  3, 21,                                     // 26859:     case 0x3: {
  OPC_SwitchField, 4, 1,                     // 26861:      switch Inst[4] {
  0, 7,                                      // 26864:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26866:       check predicate 33
  OPC_Decode, 245, 18, 133, 2,               // 26868:       decode to CMPLT_WIDE_PPzZZ_B using decoder 261
                                             // 26868:      }
  1, 0,                                      // 26873:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26875:       check predicate 33
  OPC_Decode, 224, 18, 133, 2,               // 26877:       decode to CMPLE_WIDE_PPzZZ_B using decoder 261
                                             // 26877:      }
                                             // 26877:      } // switch Inst[4]
                                             // 26877:     }
  4, 21,                                     // 26882:     case 0x4: {
  OPC_SwitchField, 4, 1,                     // 26884:      switch Inst[4] {
  0, 7,                                      // 26887:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26889:       check predicate 33
  OPC_Decode, 180, 18, 133, 2,               // 26891:       decode to CMPGE_PPzZZ_B using decoder 261
                                             // 26891:      }
  1, 0,                                      // 26896:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26898:       check predicate 33
  OPC_Decode, 191, 18, 133, 2,               // 26900:       decode to CMPGT_PPzZZ_B using decoder 261
                                             // 26900:      }
                                             // 26900:      } // switch Inst[4]
                                             // 26900:     }
  5, 21,                                     // 26905:     case 0x5: {
  OPC_SwitchField, 4, 1,                     // 26907:      switch Inst[4] {
  0, 7,                                      // 26910:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26912:       check predicate 33
  OPC_Decode, 169, 18, 133, 2,               // 26914:       decode to CMPEQ_PPzZZ_B using decoder 261
                                             // 26914:      }
  1, 0,                                      // 26919:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26921:       check predicate 33
  OPC_Decode, 252, 18, 133, 2,               // 26923:       decode to CMPNE_PPzZZ_B using decoder 261
                                             // 26923:      }
                                             // 26923:      } // switch Inst[4]
                                             // 26923:     }
  6, 21,                                     // 26928:     case 0x6: {
  OPC_SwitchField, 4, 1,                     // 26930:      switch Inst[4] {
  0, 7,                                      // 26933:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26935:       check predicate 33
  OPC_Decode, 217, 18, 133, 2,               // 26937:       decode to CMPHS_WIDE_PPzZZ_B using decoder 261
                                             // 26937:      }
  1, 0,                                      // 26942:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26944:       check predicate 33
  OPC_Decode, 206, 18, 133, 2,               // 26946:       decode to CMPHI_WIDE_PPzZZ_B using decoder 261
                                             // 26946:      }
                                             // 26946:      } // switch Inst[4]
                                             // 26946:     }
  7, 0,                                      // 26951:     case 0x7: {
  OPC_SwitchField, 4, 1,                     // 26953:      switch Inst[4] {
  0, 7,                                      // 26956:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26958:       check predicate 33
  OPC_Decode, 231, 18, 133, 2,               // 26960:       decode to CMPLO_WIDE_PPzZZ_B using decoder 261
                                             // 26960:      }
  1, 0,                                      // 26965:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26967:       check predicate 33
  OPC_Decode, 238, 18, 133, 2,               // 26969:       decode to CMPLS_WIDE_PPzZZ_B using decoder 261
                                             // 26969:      }
                                             // 26969:      } // switch Inst[4]
                                             // 26969:     }
                                             // 26969:     } // switch Inst[15:13]
                                             // 26969:    }
  1, 187, 1,                                 // 26974:    case 0x1: {
  OPC_SwitchField, 13, 3,                    // 26977:     switch Inst[15:13] {
  0, 21,                                     // 26980:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 26982:      switch Inst[4] {
  0, 7,                                      // 26985:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 26987:       check predicate 33
  OPC_Decode, 215, 18, 133, 2,               // 26989:       decode to CMPHS_PPzZZ_H using decoder 261
                                             // 26989:      }
  1, 0,                                      // 26994:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 26996:       check predicate 33
  OPC_Decode, 204, 18, 133, 2,               // 26998:       decode to CMPHI_PPzZZ_H using decoder 261
                                             // 26998:      }
                                             // 26998:      } // switch Inst[4]
                                             // 26998:     }
  1, 21,                                     // 27003:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 27005:      switch Inst[4] {
  0, 7,                                      // 27008:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27010:       check predicate 33
  OPC_Decode, 174, 18, 133, 2,               // 27012:       decode to CMPEQ_WIDE_PPzZZ_H using decoder 261
                                             // 27012:      }
  1, 0,                                      // 27017:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27019:       check predicate 33
  OPC_Decode, 129, 19, 133, 2,               // 27021:       decode to CMPNE_WIDE_PPzZZ_H using decoder 261
                                             // 27021:      }
                                             // 27021:      } // switch Inst[4]
                                             // 27021:     }
  2, 21,                                     // 27026:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 27028:      switch Inst[4] {
  0, 7,                                      // 27031:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27033:       check predicate 33
  OPC_Decode, 185, 18, 133, 2,               // 27035:       decode to CMPGE_WIDE_PPzZZ_H using decoder 261
                                             // 27035:      }
  1, 0,                                      // 27040:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27042:       check predicate 33
  OPC_Decode, 196, 18, 133, 2,               // 27044:       decode to CMPGT_WIDE_PPzZZ_H using decoder 261
                                             // 27044:      }
                                             // 27044:      } // switch Inst[4]
                                             // 27044:     }
  3, 21,                                     // 27049:     case 0x3: {
  OPC_SwitchField, 4, 1,                     // 27051:      switch Inst[4] {
  0, 7,                                      // 27054:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27056:       check predicate 33
  OPC_Decode, 246, 18, 133, 2,               // 27058:       decode to CMPLT_WIDE_PPzZZ_H using decoder 261
                                             // 27058:      }
  1, 0,                                      // 27063:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27065:       check predicate 33
  OPC_Decode, 225, 18, 133, 2,               // 27067:       decode to CMPLE_WIDE_PPzZZ_H using decoder 261
                                             // 27067:      }
                                             // 27067:      } // switch Inst[4]
                                             // 27067:     }
  4, 21,                                     // 27072:     case 0x4: {
  OPC_SwitchField, 4, 1,                     // 27074:      switch Inst[4] {
  0, 7,                                      // 27077:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27079:       check predicate 33
  OPC_Decode, 182, 18, 133, 2,               // 27081:       decode to CMPGE_PPzZZ_H using decoder 261
                                             // 27081:      }
  1, 0,                                      // 27086:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27088:       check predicate 33
  OPC_Decode, 193, 18, 133, 2,               // 27090:       decode to CMPGT_PPzZZ_H using decoder 261
                                             // 27090:      }
                                             // 27090:      } // switch Inst[4]
                                             // 27090:     }
  5, 21,                                     // 27095:     case 0x5: {
  OPC_SwitchField, 4, 1,                     // 27097:      switch Inst[4] {
  0, 7,                                      // 27100:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27102:       check predicate 33
  OPC_Decode, 171, 18, 133, 2,               // 27104:       decode to CMPEQ_PPzZZ_H using decoder 261
                                             // 27104:      }
  1, 0,                                      // 27109:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27111:       check predicate 33
  OPC_Decode, 254, 18, 133, 2,               // 27113:       decode to CMPNE_PPzZZ_H using decoder 261
                                             // 27113:      }
                                             // 27113:      } // switch Inst[4]
                                             // 27113:     }
  6, 21,                                     // 27118:     case 0x6: {
  OPC_SwitchField, 4, 1,                     // 27120:      switch Inst[4] {
  0, 7,                                      // 27123:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27125:       check predicate 33
  OPC_Decode, 218, 18, 133, 2,               // 27127:       decode to CMPHS_WIDE_PPzZZ_H using decoder 261
                                             // 27127:      }
  1, 0,                                      // 27132:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27134:       check predicate 33
  OPC_Decode, 207, 18, 133, 2,               // 27136:       decode to CMPHI_WIDE_PPzZZ_H using decoder 261
                                             // 27136:      }
                                             // 27136:      } // switch Inst[4]
                                             // 27136:     }
  7, 0,                                      // 27141:     case 0x7: {
  OPC_SwitchField, 4, 1,                     // 27143:      switch Inst[4] {
  0, 7,                                      // 27146:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27148:       check predicate 33
  OPC_Decode, 232, 18, 133, 2,               // 27150:       decode to CMPLO_WIDE_PPzZZ_H using decoder 261
                                             // 27150:      }
  1, 0,                                      // 27155:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27157:       check predicate 33
  OPC_Decode, 239, 18, 133, 2,               // 27159:       decode to CMPLS_WIDE_PPzZZ_H using decoder 261
                                             // 27159:      }
                                             // 27159:      } // switch Inst[4]
                                             // 27159:     }
                                             // 27159:     } // switch Inst[15:13]
                                             // 27159:    }
  2, 187, 1,                                 // 27164:    case 0x2: {
  OPC_SwitchField, 13, 3,                    // 27167:     switch Inst[15:13] {
  0, 21,                                     // 27170:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 27172:      switch Inst[4] {
  0, 7,                                      // 27175:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27177:       check predicate 33
  OPC_Decode, 216, 18, 133, 2,               // 27179:       decode to CMPHS_PPzZZ_S using decoder 261
                                             // 27179:      }
  1, 0,                                      // 27184:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27186:       check predicate 33
  OPC_Decode, 205, 18, 133, 2,               // 27188:       decode to CMPHI_PPzZZ_S using decoder 261
                                             // 27188:      }
                                             // 27188:      } // switch Inst[4]
                                             // 27188:     }
  1, 21,                                     // 27193:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 27195:      switch Inst[4] {
  0, 7,                                      // 27198:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27200:       check predicate 33
  OPC_Decode, 175, 18, 133, 2,               // 27202:       decode to CMPEQ_WIDE_PPzZZ_S using decoder 261
                                             // 27202:      }
  1, 0,                                      // 27207:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27209:       check predicate 33
  OPC_Decode, 130, 19, 133, 2,               // 27211:       decode to CMPNE_WIDE_PPzZZ_S using decoder 261
                                             // 27211:      }
                                             // 27211:      } // switch Inst[4]
                                             // 27211:     }
  2, 21,                                     // 27216:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 27218:      switch Inst[4] {
  0, 7,                                      // 27221:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27223:       check predicate 33
  OPC_Decode, 186, 18, 133, 2,               // 27225:       decode to CMPGE_WIDE_PPzZZ_S using decoder 261
                                             // 27225:      }
  1, 0,                                      // 27230:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27232:       check predicate 33
  OPC_Decode, 197, 18, 133, 2,               // 27234:       decode to CMPGT_WIDE_PPzZZ_S using decoder 261
                                             // 27234:      }
                                             // 27234:      } // switch Inst[4]
                                             // 27234:     }
  3, 21,                                     // 27239:     case 0x3: {
  OPC_SwitchField, 4, 1,                     // 27241:      switch Inst[4] {
  0, 7,                                      // 27244:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27246:       check predicate 33
  OPC_Decode, 247, 18, 133, 2,               // 27248:       decode to CMPLT_WIDE_PPzZZ_S using decoder 261
                                             // 27248:      }
  1, 0,                                      // 27253:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27255:       check predicate 33
  OPC_Decode, 226, 18, 133, 2,               // 27257:       decode to CMPLE_WIDE_PPzZZ_S using decoder 261
                                             // 27257:      }
                                             // 27257:      } // switch Inst[4]
                                             // 27257:     }
  4, 21,                                     // 27262:     case 0x4: {
  OPC_SwitchField, 4, 1,                     // 27264:      switch Inst[4] {
  0, 7,                                      // 27267:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27269:       check predicate 33
  OPC_Decode, 183, 18, 133, 2,               // 27271:       decode to CMPGE_PPzZZ_S using decoder 261
                                             // 27271:      }
  1, 0,                                      // 27276:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27278:       check predicate 33
  OPC_Decode, 194, 18, 133, 2,               // 27280:       decode to CMPGT_PPzZZ_S using decoder 261
                                             // 27280:      }
                                             // 27280:      } // switch Inst[4]
                                             // 27280:     }
  5, 21,                                     // 27285:     case 0x5: {
  OPC_SwitchField, 4, 1,                     // 27287:      switch Inst[4] {
  0, 7,                                      // 27290:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27292:       check predicate 33
  OPC_Decode, 172, 18, 133, 2,               // 27294:       decode to CMPEQ_PPzZZ_S using decoder 261
                                             // 27294:      }
  1, 0,                                      // 27299:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27301:       check predicate 33
  OPC_Decode, 255, 18, 133, 2,               // 27303:       decode to CMPNE_PPzZZ_S using decoder 261
                                             // 27303:      }
                                             // 27303:      } // switch Inst[4]
                                             // 27303:     }
  6, 21,                                     // 27308:     case 0x6: {
  OPC_SwitchField, 4, 1,                     // 27310:      switch Inst[4] {
  0, 7,                                      // 27313:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27315:       check predicate 33
  OPC_Decode, 219, 18, 133, 2,               // 27317:       decode to CMPHS_WIDE_PPzZZ_S using decoder 261
                                             // 27317:      }
  1, 0,                                      // 27322:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27324:       check predicate 33
  OPC_Decode, 208, 18, 133, 2,               // 27326:       decode to CMPHI_WIDE_PPzZZ_S using decoder 261
                                             // 27326:      }
                                             // 27326:      } // switch Inst[4]
                                             // 27326:     }
  7, 0,                                      // 27331:     case 0x7: {
  OPC_SwitchField, 4, 1,                     // 27333:      switch Inst[4] {
  0, 7,                                      // 27336:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27338:       check predicate 33
  OPC_Decode, 233, 18, 133, 2,               // 27340:       decode to CMPLO_WIDE_PPzZZ_S using decoder 261
                                             // 27340:      }
  1, 0,                                      // 27345:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27347:       check predicate 33
  OPC_Decode, 240, 18, 133, 2,               // 27349:       decode to CMPLS_WIDE_PPzZZ_S using decoder 261
                                             // 27349:      }
                                             // 27349:      } // switch Inst[4]
                                             // 27349:     }
                                             // 27349:     } // switch Inst[15:13]
                                             // 27349:    }
  3, 72,                                     // 27354:    case 0x3: {
  OPC_SwitchField, 13, 3,                    // 27356:     switch Inst[15:13] {
  0, 21,                                     // 27359:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 27361:      switch Inst[4] {
  0, 7,                                      // 27364:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27366:       check predicate 33
  OPC_Decode, 214, 18, 133, 2,               // 27368:       decode to CMPHS_PPzZZ_D using decoder 261
                                             // 27368:      }
  1, 0,                                      // 27373:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27375:       check predicate 33
  OPC_Decode, 203, 18, 133, 2,               // 27377:       decode to CMPHI_PPzZZ_D using decoder 261
                                             // 27377:      }
                                             // 27377:      } // switch Inst[4]
                                             // 27377:     }
  4, 21,                                     // 27382:     case 0x4: {
  OPC_SwitchField, 4, 1,                     // 27384:      switch Inst[4] {
  0, 7,                                      // 27387:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27389:       check predicate 33
  OPC_Decode, 181, 18, 133, 2,               // 27391:       decode to CMPGE_PPzZZ_D using decoder 261
                                             // 27391:      }
  1, 0,                                      // 27396:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27398:       check predicate 33
  OPC_Decode, 192, 18, 133, 2,               // 27400:       decode to CMPGT_PPzZZ_D using decoder 261
                                             // 27400:      }
                                             // 27400:      } // switch Inst[4]
                                             // 27400:     }
  5, 0,                                      // 27405:     case 0x5: {
  OPC_SwitchField, 4, 1,                     // 27407:      switch Inst[4] {
  0, 7,                                      // 27410:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27412:       check predicate 33
  OPC_Decode, 170, 18, 133, 2,               // 27414:       decode to CMPEQ_PPzZZ_D using decoder 261
                                             // 27414:      }
  1, 0,                                      // 27419:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27421:       check predicate 33
  OPC_Decode, 253, 18, 133, 2,               // 27423:       decode to CMPNE_PPzZZ_D using decoder 261
                                             // 27423:      }
                                             // 27423:      } // switch Inst[4]
                                             // 27423:     }
                                             // 27423:     } // switch Inst[15:13]
                                             // 27423:    }
  4, 195, 2,                                 // 27428:    case 0x4: {
  OPC_SwitchField, 14, 2,                    // 27431:     switch Inst[15:14] {
  0, 49,                                     // 27434:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 27436:      switch Inst[4] {
  0, 21,                                     // 27439:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 27441:       switch Inst[13] {
  0, 7,                                      // 27444:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 27446:        check predicate 33
  OPC_Decode, 176, 18, 134, 2,               // 27448:        decode to CMPGE_PPzZI_B using decoder 262
                                             // 27448:       }
  1, 0,                                      // 27453:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27455:        check predicate 33
  OPC_Decode, 241, 18, 134, 2,               // 27457:        decode to CMPLT_PPzZI_B using decoder 262
                                             // 27457:       }
                                             // 27457:       } // switch Inst[13]
                                             // 27457:      }
  1, 0,                                      // 27462:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 27464:       switch Inst[13] {
  0, 7,                                      // 27467:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 27469:        check predicate 33
  OPC_Decode, 187, 18, 134, 2,               // 27471:        decode to CMPGT_PPzZI_B using decoder 262
                                             // 27471:       }
  1, 0,                                      // 27476:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27478:        check predicate 33
  OPC_Decode, 220, 18, 134, 2,               // 27480:        decode to CMPLE_PPzZI_B using decoder 262
                                             // 27480:       }
                                             // 27480:       } // switch Inst[13]
                                             // 27480:      }
                                             // 27480:      } // switch Inst[4]
                                             // 27480:     }
  1, 103,                                    // 27485:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 27487:      switch Inst[4] {
  0, 53,                                     // 27490:      case 0x0: {
  OPC_SwitchField, 9, 1,                     // 27492:       switch Inst[9] {
  0, 35,                                     // 27495:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 27497:        switch Inst[20] {
  0, 7,                                      // 27500:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 27502:         check predicate 33
  OPC_Decode, 164, 14, 135, 2,               // 27504:         decode to AND_PPzPP using decoder 263
                                             // 27504:        }
  1, 0,                                      // 27509:        case 0x1: {
  OPC_SwitchField, 16, 4,                    // 27511:         switch Inst[19:16] {
  0, 7,                                      // 27514:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 27516:          check predicate 33
  OPC_Decode, 172, 16, 136, 2,               // 27518:          decode to BRKA_PPzP using decoder 264
                                             // 27518:         }
  8, 0,                                      // 27523:         case 0x8: {
  OPC_CheckPredicate, 33,                    // 27525:          check predicate 33
  OPC_Decode, 177, 16, 137, 2,               // 27527:          decode to BRKN_PPzP using decoder 265
                                             // 27527:         }
                                             // 27527:         } // switch Inst[19:16]
                                             // 27527:        }
                                             // 27527:        } // switch Inst[20]
                                             // 27527:       }
  1, 0,                                      // 27532:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27534:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 27536:        check Inst[20] == 0x0
  OPC_Decode, 135, 21, 135, 2,               // 27540:        decode to EOR_PPzPP using decoder 263
                                             // 27540:       }
                                             // 27540:       } // switch Inst[9]
                                             // 27540:      }
  1, 0,                                      // 27545:      case 0x1: {
  OPC_SwitchField, 9, 1,                     // 27547:       switch Inst[9] {
  0, 25,                                     // 27550:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 27552:        switch Inst[20] {
  0, 7,                                      // 27555:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 27557:         check predicate 33
  OPC_Decode, 138, 16, 135, 2,               // 27559:         decode to BIC_PPzPP using decoder 263
                                             // 27559:        }
  1, 0,                                      // 27564:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 27566:         check predicate 33
  OPC_CheckField, 16, 4, 0,                  // 27568:         check Inst[19:16] == 0x0
  OPC_Decode, 171, 16, 138, 2,               // 27572:         decode to BRKA_PPmP using decoder 266
                                             // 27572:        }
                                             // 27572:        } // switch Inst[20]
                                             // 27572:       }
  1, 0,                                      // 27577:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27579:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 27581:        check Inst[20] == 0x0
  OPC_Decode, 211, 47, 135, 2,               // 27585:        decode to SEL_PPPP using decoder 263
                                             // 27585:       }
                                             // 27585:       } // switch Inst[9]
                                             // 27585:      }
                                             // 27585:      } // switch Inst[4]
                                             // 27585:     }
  2, 29,                                     // 27590:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 27592:      switch Inst[4] {
  0, 11,                                     // 27595:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27597:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 27599:       check Inst[13] == 0x0
  OPC_Decode, 165, 18, 134, 2,               // 27603:       decode to CMPEQ_PPzZI_B using decoder 262
                                             // 27603:      }
  1, 0,                                      // 27608:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27610:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 27612:       check Inst[13] == 0x0
  OPC_Decode, 248, 18, 134, 2,               // 27616:       decode to CMPNE_PPzZI_B using decoder 262
                                             // 27616:      }
                                             // 27616:      } // switch Inst[4]
                                             // 27616:     }
  3, 0,                                      // 27621:     case 0x3: {
  OPC_SwitchField, 4, 1,                     // 27623:      switch Inst[4] {
  0, 109,                                    // 27626:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 27628:       switch Inst[20] {
  0, 11,                                     // 27631:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 27633:        check predicate 33
  OPC_CheckField, 9, 1, 0,                   // 27635:        check Inst[9] == 0x0
  OPC_Decode, 179, 16, 139, 2,               // 27639:        decode to BRKPA_PPzPP using decoder 267
                                             // 27639:       }
  1, 0,                                      // 27644:       case 0x1: {
  OPC_SwitchField, 10, 4,                    // 27646:        switch Inst[13:10] {
  1, 15,                                     // 27649:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 27651:         check predicate 33
  OPC_CheckField, 16, 4, 9,                  // 27653:         check Inst[19:16] == 0x9
  OPC_CheckField, 9, 1, 0,                   // 27657:         check Inst[9] == 0x0
  OPC_Decode, 152, 44, 140, 2,               // 27661:         decode to PNEXT_B using decoder 268
                                             // 27661:        }
  8, 21,                                     // 27666:        case 0x8: {
  OPC_SwitchField, 16, 4,                    // 27668:         switch Inst[19:16] {
  8, 7,                                      // 27671:         case 0x8: {
  OPC_CheckPredicate, 33,                    // 27673:          check predicate 33
  OPC_Decode, 206, 44, 141, 2,               // 27675:          decode to PTRUE_B using decoder 269
                                             // 27675:         }
  9, 0,                                      // 27680:         case 0x9: {
  OPC_CheckPredicate, 33,                    // 27682:          check predicate 33
  OPC_Decode, 202, 44, 141, 2,               // 27684:          decode to PTRUES_B using decoder 269
                                             // 27684:         }
                                             // 27684:         } // switch Inst[19:16]
                                             // 27684:        }
  9, 15,                                     // 27689:        case 0x9: {
  OPC_CheckPredicate, 33,                    // 27691:         check predicate 33
  OPC_CheckField, 16, 4, 8,                  // 27693:         check Inst[19:16] == 0x8
  OPC_CheckField, 5, 5, 0,                   // 27697:         check Inst[9:5] == 0x0
  OPC_Decode, 255, 43, 142, 2,               // 27701:         decode to PFALSE using decoder 270
                                             // 27701:        }
  12, 0,                                     // 27706:        case 0xc: {
  OPC_SwitchField, 16, 4,                    // 27708:         switch Inst[19:16] {
  8, 11,                                     // 27711:         case 0x8: {
  OPC_CheckPredicate, 40,                    // 27713:          check predicate 40
  OPC_CheckField, 9, 1, 0,                   // 27715:          check Inst[9] == 0x0
  OPC_Decode, 180, 45, 222, 1,               // 27719:          decode to RDFFR_PPz using decoder 222
                                             // 27719:         }
  9, 0,                                      // 27724:         case 0x9: {
  OPC_CheckPredicate, 40,                    // 27726:          check predicate 40
  OPC_CheckField, 5, 5, 0,                   // 27728:          check Inst[9:5] == 0x0
  OPC_Decode, 179, 45, 143, 2,               // 27732:          decode to RDFFR_P using decoder 271
                                             // 27732:         }
                                             // 27732:         } // switch Inst[19:16]
                                             // 27732:        }
                                             // 27732:        } // switch Inst[13:10]
                                             // 27732:       }
                                             // 27732:       } // switch Inst[20]
                                             // 27732:      }
  1, 0,                                      // 27737:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27739:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 27741:       check Inst[20] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 27745:       check Inst[9] == 0x0
  OPC_Decode, 181, 16, 139, 2,               // 27749:       decode to BRKPB_PPzPP using decoder 267
                                             // 27749:      }
                                             // 27749:      } // switch Inst[4]
                                             // 27749:     }
                                             // 27749:     } // switch Inst[15:14]
                                             // 27749:    }
  5, 163, 2,                                 // 27754:    case 0x5: {
  OPC_SwitchField, 14, 2,                    // 27757:     switch Inst[15:14] {
  0, 49,                                     // 27760:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 27762:      switch Inst[4] {
  0, 21,                                     // 27765:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 27767:       switch Inst[13] {
  0, 7,                                      // 27770:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 27772:        check predicate 33
  OPC_Decode, 178, 18, 134, 2,               // 27774:        decode to CMPGE_PPzZI_H using decoder 262
                                             // 27774:       }
  1, 0,                                      // 27779:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27781:        check predicate 33
  OPC_Decode, 243, 18, 134, 2,               // 27783:        decode to CMPLT_PPzZI_H using decoder 262
                                             // 27783:       }
                                             // 27783:       } // switch Inst[13]
                                             // 27783:      }
  1, 0,                                      // 27788:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 27790:       switch Inst[13] {
  0, 7,                                      // 27793:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 27795:        check predicate 33
  OPC_Decode, 189, 18, 134, 2,               // 27797:        decode to CMPGT_PPzZI_H using decoder 262
                                             // 27797:       }
  1, 0,                                      // 27802:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27804:        check predicate 33
  OPC_Decode, 222, 18, 134, 2,               // 27806:        decode to CMPLE_PPzZI_H using decoder 262
                                             // 27806:       }
                                             // 27806:       } // switch Inst[13]
                                             // 27806:      }
                                             // 27806:      } // switch Inst[4]
                                             // 27806:     }
  1, 75,                                     // 27811:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 27813:      switch Inst[4] {
  0, 53,                                     // 27816:      case 0x0: {
  OPC_SwitchField, 9, 1,                     // 27818:       switch Inst[9] {
  0, 35,                                     // 27821:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 27823:        switch Inst[20] {
  0, 7,                                      // 27826:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 27828:         check predicate 33
  OPC_Decode, 155, 14, 135, 2,               // 27830:         decode to ANDS_PPzPP using decoder 263
                                             // 27830:        }
  1, 0,                                      // 27835:        case 0x1: {
  OPC_SwitchField, 16, 4,                    // 27837:         switch Inst[19:16] {
  0, 7,                                      // 27840:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 27842:          check predicate 33
  OPC_Decode, 170, 16, 136, 2,               // 27844:          decode to BRKAS_PPzP using decoder 264
                                             // 27844:         }
  8, 0,                                      // 27849:         case 0x8: {
  OPC_CheckPredicate, 33,                    // 27851:          check predicate 33
  OPC_Decode, 176, 16, 137, 2,               // 27853:          decode to BRKNS_PPzP using decoder 265
                                             // 27853:         }
                                             // 27853:         } // switch Inst[19:16]
                                             // 27853:        }
                                             // 27853:        } // switch Inst[20]
                                             // 27853:       }
  1, 0,                                      // 27858:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 27860:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 27862:        check Inst[20] == 0x0
  OPC_Decode, 250, 20, 135, 2,               // 27866:        decode to EORS_PPzPP using decoder 263
                                             // 27866:       }
                                             // 27866:       } // switch Inst[9]
                                             // 27866:      }
  1, 0,                                      // 27871:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27873:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 27875:       check Inst[20] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 27879:       check Inst[9] == 0x0
  OPC_Decode, 135, 16, 135, 2,               // 27883:       decode to BICS_PPzPP using decoder 263
                                             // 27883:      }
                                             // 27883:      } // switch Inst[4]
                                             // 27883:     }
  2, 29,                                     // 27888:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 27890:      switch Inst[4] {
  0, 11,                                     // 27893:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 27895:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 27897:       check Inst[13] == 0x0
  OPC_Decode, 167, 18, 134, 2,               // 27901:       decode to CMPEQ_PPzZI_H using decoder 262
                                             // 27901:      }
  1, 0,                                      // 27906:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 27908:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 27910:       check Inst[13] == 0x0
  OPC_Decode, 250, 18, 134, 2,               // 27914:       decode to CMPNE_PPzZI_H using decoder 262
                                             // 27914:      }
                                             // 27914:      } // switch Inst[4]
                                             // 27914:     }
  3, 0,                                      // 27919:     case 0x3: {
  OPC_SwitchField, 4, 1,                     // 27921:      switch Inst[4] {
  0, 105,                                    // 27924:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 27926:       switch Inst[20] {
  0, 11,                                     // 27929:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 27931:        check predicate 33
  OPC_CheckField, 9, 1, 0,                   // 27933:        check Inst[9] == 0x0
  OPC_Decode, 178, 16, 139, 2,               // 27937:        decode to BRKPAS_PPzPP using decoder 267
                                             // 27937:       }
  1, 0,                                      // 27942:       case 0x1: {
  OPC_SwitchField, 16, 4,                    // 27944:        switch Inst[19:16] {
  0, 15,                                     // 27947:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 27949:         check predicate 33
  OPC_CheckField, 9, 1, 0,                   // 27951:         check Inst[9] == 0x0
  OPC_CheckField, 0, 4, 0,                   // 27955:         check Inst[3:0] == 0x0
  OPC_Decode, 201, 44, 144, 2,               // 27959:         decode to PTEST_PP using decoder 272
                                             // 27959:        }
  8, 38,                                     // 27964:        case 0x8: {
  OPC_SwitchField, 10, 4,                    // 27966:         switch Inst[13:10] {
  0, 11,                                     // 27969:         case 0x0: {
  OPC_CheckPredicate, 33,                    // 27971:          check predicate 33
  OPC_CheckField, 9, 1, 0,                   // 27973:          check Inst[9] == 0x0
  OPC_Decode, 128, 44, 140, 2,               // 27977:          decode to PFIRST_B using decoder 268
                                             // 27977:         }
  8, 7,                                      // 27982:         case 0x8: {
  OPC_CheckPredicate, 33,                    // 27984:          check predicate 33
  OPC_Decode, 212, 44, 141, 2,               // 27986:          decode to PTRUE_H using decoder 269
                                             // 27986:         }
  12, 0,                                     // 27991:         case 0xc: {
  OPC_CheckPredicate, 40,                    // 27993:          check predicate 40
  OPC_CheckField, 9, 1, 0,                   // 27995:          check Inst[9] == 0x0
  OPC_Decode, 178, 45, 222, 1,               // 27999:          decode to RDFFRS_PPz using decoder 222
                                             // 27999:         }
                                             // 27999:         } // switch Inst[13:10]
                                             // 27999:        }
  9, 0,                                      // 28004:        case 0x9: {
  OPC_SwitchField, 10, 4,                    // 28006:         switch Inst[13:10] {
  1, 11,                                     // 28009:         case 0x1: {
  OPC_CheckPredicate, 33,                    // 28011:          check predicate 33
  OPC_CheckField, 9, 1, 0,                   // 28013:          check Inst[9] == 0x0
  OPC_Decode, 154, 44, 140, 2,               // 28017:          decode to PNEXT_H using decoder 268
                                             // 28017:         }
  8, 0,                                      // 28022:         case 0x8: {
  OPC_CheckPredicate, 33,                    // 28024:          check predicate 33
  OPC_Decode, 204, 44, 141, 2,               // 28026:          decode to PTRUES_H using decoder 269
                                             // 28026:         }
                                             // 28026:         } // switch Inst[13:10]
                                             // 28026:        }
                                             // 28026:        } // switch Inst[19:16]
                                             // 28026:       }
                                             // 28026:       } // switch Inst[20]
                                             // 28026:      }
  1, 0,                                      // 28031:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 28033:       check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 28035:       check Inst[20] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 28039:       check Inst[9] == 0x0
  OPC_Decode, 180, 16, 139, 2,               // 28043:       decode to BRKPBS_PPzPP using decoder 267
                                             // 28043:      }
                                             // 28043:      } // switch Inst[4]
                                             // 28043:     }
                                             // 28043:     } // switch Inst[15:14]
                                             // 28043:    }
  6, 237, 1,                                 // 28048:    case 0x6: {
  OPC_SwitchField, 14, 2,                    // 28051:     switch Inst[15:14] {
  0, 49,                                     // 28054:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 28056:      switch Inst[4] {
  0, 21,                                     // 28059:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 28061:       switch Inst[13] {
  0, 7,                                      // 28064:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28066:        check predicate 33
  OPC_Decode, 179, 18, 134, 2,               // 28068:        decode to CMPGE_PPzZI_S using decoder 262
                                             // 28068:       }
  1, 0,                                      // 28073:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28075:        check predicate 33
  OPC_Decode, 244, 18, 134, 2,               // 28077:        decode to CMPLT_PPzZI_S using decoder 262
                                             // 28077:       }
                                             // 28077:       } // switch Inst[13]
                                             // 28077:      }
  1, 0,                                      // 28082:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 28084:       switch Inst[13] {
  0, 7,                                      // 28087:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28089:        check predicate 33
  OPC_Decode, 190, 18, 134, 2,               // 28091:        decode to CMPGT_PPzZI_S using decoder 262
                                             // 28091:       }
  1, 0,                                      // 28096:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28098:        check predicate 33
  OPC_Decode, 223, 18, 134, 2,               // 28100:        decode to CMPLE_PPzZI_S using decoder 262
                                             // 28100:       }
                                             // 28100:       } // switch Inst[13]
                                             // 28100:      }
                                             // 28100:      } // switch Inst[4]
                                             // 28100:     }
  1, 93,                                     // 28105:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 28107:      switch Inst[4] {
  0, 43,                                     // 28110:      case 0x0: {
  OPC_SwitchField, 9, 1,                     // 28112:       switch Inst[9] {
  0, 25,                                     // 28115:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 28117:        switch Inst[20] {
  0, 7,                                      // 28120:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28122:         check predicate 33
  OPC_Decode, 208, 43, 135, 2,               // 28124:         decode to ORR_PPzPP using decoder 263
                                             // 28124:        }
  1, 0,                                      // 28129:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28131:         check predicate 33
  OPC_CheckField, 16, 4, 0,                  // 28133:         check Inst[19:16] == 0x0
  OPC_Decode, 175, 16, 136, 2,               // 28137:         decode to BRKB_PPzP using decoder 264
                                             // 28137:        }
                                             // 28137:        } // switch Inst[20]
                                             // 28137:       }
  1, 0,                                      // 28142:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28144:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 28146:        check Inst[20] == 0x0
  OPC_Decode, 182, 43, 135, 2,               // 28150:        decode to NOR_PPzPP using decoder 263
                                             // 28150:       }
                                             // 28150:       } // switch Inst[9]
                                             // 28150:      }
  1, 0,                                      // 28155:      case 0x1: {
  OPC_SwitchField, 9, 1,                     // 28157:       switch Inst[9] {
  0, 25,                                     // 28160:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 28162:        switch Inst[20] {
  0, 7,                                      // 28165:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28167:         check predicate 33
  OPC_Decode, 196, 43, 135, 2,               // 28169:         decode to ORN_PPzPP using decoder 263
                                             // 28169:        }
  1, 0,                                      // 28174:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28176:         check predicate 33
  OPC_CheckField, 16, 4, 0,                  // 28178:         check Inst[19:16] == 0x0
  OPC_Decode, 174, 16, 138, 2,               // 28182:         decode to BRKB_PPmP using decoder 266
                                             // 28182:        }
                                             // 28182:        } // switch Inst[20]
                                             // 28182:       }
  1, 0,                                      // 28187:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28189:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 28191:        check Inst[20] == 0x0
  OPC_Decode, 160, 43, 135, 2,               // 28195:        decode to NAND_PPzPP using decoder 263
                                             // 28195:       }
                                             // 28195:       } // switch Inst[9]
                                             // 28195:      }
                                             // 28195:      } // switch Inst[4]
                                             // 28195:     }
  2, 29,                                     // 28200:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 28202:      switch Inst[4] {
  0, 11,                                     // 28205:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 28207:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 28209:       check Inst[13] == 0x0
  OPC_Decode, 168, 18, 134, 2,               // 28213:       decode to CMPEQ_PPzZI_S using decoder 262
                                             // 28213:      }
  1, 0,                                      // 28218:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 28220:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 28222:       check Inst[13] == 0x0
  OPC_Decode, 251, 18, 134, 2,               // 28226:       decode to CMPNE_PPzZI_S using decoder 262
                                             // 28226:      }
                                             // 28226:      } // switch Inst[4]
                                             // 28226:     }
  3, 0,                                      // 28231:     case 0x3: {
  OPC_SwitchField, 10, 4,                    // 28233:      switch Inst[13:10] {
  1, 19,                                     // 28236:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 28238:       check predicate 33
  OPC_CheckField, 16, 5, 25,                 // 28240:       check Inst[20:16] == 0x19
  OPC_CheckField, 9, 1, 0,                   // 28244:       check Inst[9] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 28248:       check Inst[4] == 0x0
  OPC_Decode, 155, 44, 140, 2,               // 28252:       decode to PNEXT_S using decoder 268
                                             // 28252:      }
  8, 0,                                      // 28257:      case 0x8: {
  OPC_SwitchField, 16, 5,                    // 28259:       switch Inst[20:16] {
  24, 11,                                    // 28262:       case 0x18: {
  OPC_CheckPredicate, 33,                    // 28264:        check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 28266:        check Inst[4] == 0x0
  OPC_Decode, 213, 44, 141, 2,               // 28270:        decode to PTRUE_S using decoder 269
                                             // 28270:       }
  25, 0,                                     // 28275:       case 0x19: {
  OPC_CheckPredicate, 33,                    // 28277:        check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 28279:        check Inst[4] == 0x0
  OPC_Decode, 205, 44, 141, 2,               // 28283:        decode to PTRUES_S using decoder 269
                                             // 28283:       }
                                             // 28283:       } // switch Inst[20:16]
                                             // 28283:      }
                                             // 28283:      } // switch Inst[13:10]
                                             // 28283:     }
                                             // 28283:     } // switch Inst[15:14]
                                             // 28283:    }
  7, 0,                                      // 28288:    case 0x7: {
  OPC_SwitchField, 14, 2,                    // 28290:     switch Inst[15:14] {
  0, 49,                                     // 28293:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 28295:      switch Inst[4] {
  0, 21,                                     // 28298:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 28300:       switch Inst[13] {
  0, 7,                                      // 28303:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28305:        check predicate 33
  OPC_Decode, 177, 18, 134, 2,               // 28307:        decode to CMPGE_PPzZI_D using decoder 262
                                             // 28307:       }
  1, 0,                                      // 28312:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28314:        check predicate 33
  OPC_Decode, 242, 18, 134, 2,               // 28316:        decode to CMPLT_PPzZI_D using decoder 262
                                             // 28316:       }
                                             // 28316:       } // switch Inst[13]
                                             // 28316:      }
  1, 0,                                      // 28321:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 28323:       switch Inst[13] {
  0, 7,                                      // 28326:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28328:        check predicate 33
  OPC_Decode, 188, 18, 134, 2,               // 28330:        decode to CMPGT_PPzZI_D using decoder 262
                                             // 28330:       }
  1, 0,                                      // 28335:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28337:        check predicate 33
  OPC_Decode, 221, 18, 134, 2,               // 28339:        decode to CMPLE_PPzZI_D using decoder 262
                                             // 28339:       }
                                             // 28339:       } // switch Inst[13]
                                             // 28339:      }
                                             // 28339:      } // switch Inst[4]
                                             // 28339:     }
  1, 79,                                     // 28344:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 28346:      switch Inst[4] {
  0, 43,                                     // 28349:      case 0x0: {
  OPC_SwitchField, 9, 1,                     // 28351:       switch Inst[9] {
  0, 25,                                     // 28354:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 28356:        switch Inst[20] {
  0, 7,                                      // 28359:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28361:         check predicate 33
  OPC_Decode, 203, 43, 135, 2,               // 28363:         decode to ORRS_PPzPP using decoder 263
                                             // 28363:        }
  1, 0,                                      // 28368:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28370:         check predicate 33
  OPC_CheckField, 16, 4, 0,                  // 28372:         check Inst[19:16] == 0x0
  OPC_Decode, 173, 16, 136, 2,               // 28376:         decode to BRKBS_PPzP using decoder 264
                                             // 28376:        }
                                             // 28376:        } // switch Inst[20]
                                             // 28376:       }
  1, 0,                                      // 28381:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28383:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 28385:        check Inst[20] == 0x0
  OPC_Decode, 181, 43, 135, 2,               // 28389:        decode to NORS_PPzPP using decoder 263
                                             // 28389:       }
                                             // 28389:       } // switch Inst[9]
                                             // 28389:      }
  1, 0,                                      // 28394:      case 0x1: {
  OPC_SwitchField, 9, 1,                     // 28396:       switch Inst[9] {
  0, 11,                                     // 28399:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28401:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 28403:        check Inst[20] == 0x0
  OPC_Decode, 193, 43, 135, 2,               // 28407:        decode to ORNS_PPzPP using decoder 263
                                             // 28407:       }
  1, 0,                                      // 28412:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28414:        check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 28416:        check Inst[20] == 0x0
  OPC_Decode, 159, 43, 135, 2,               // 28420:        decode to NANDS_PPzPP using decoder 263
                                             // 28420:       }
                                             // 28420:       } // switch Inst[9]
                                             // 28420:      }
                                             // 28420:      } // switch Inst[4]
                                             // 28420:     }
  2, 29,                                     // 28425:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 28427:      switch Inst[4] {
  0, 11,                                     // 28430:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 28432:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 28434:       check Inst[13] == 0x0
  OPC_Decode, 166, 18, 134, 2,               // 28438:       decode to CMPEQ_PPzZI_D using decoder 262
                                             // 28438:      }
  1, 0,                                      // 28443:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 28445:       check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 28447:       check Inst[13] == 0x0
  OPC_Decode, 249, 18, 134, 2,               // 28451:       decode to CMPNE_PPzZI_D using decoder 262
                                             // 28451:      }
                                             // 28451:      } // switch Inst[4]
                                             // 28451:     }
  3, 0,                                      // 28456:     case 0x3: {
  OPC_SwitchField, 10, 4,                    // 28458:      switch Inst[13:10] {
  1, 19,                                     // 28461:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 28463:       check predicate 33
  OPC_CheckField, 16, 5, 25,                 // 28465:       check Inst[20:16] == 0x19
  OPC_CheckField, 9, 1, 0,                   // 28469:       check Inst[9] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 28473:       check Inst[4] == 0x0
  OPC_Decode, 153, 44, 140, 2,               // 28477:       decode to PNEXT_D using decoder 268
                                             // 28477:      }
  8, 0,                                      // 28482:      case 0x8: {
  OPC_SwitchField, 16, 5,                    // 28484:       switch Inst[20:16] {
  24, 11,                                    // 28487:       case 0x18: {
  OPC_CheckPredicate, 33,                    // 28489:        check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 28491:        check Inst[4] == 0x0
  OPC_Decode, 211, 44, 141, 2,               // 28495:        decode to PTRUE_D using decoder 269
                                             // 28495:       }
  25, 0,                                     // 28500:       case 0x19: {
  OPC_CheckPredicate, 33,                    // 28502:        check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 28504:        check Inst[4] == 0x0
  OPC_Decode, 203, 44, 141, 2,               // 28508:        decode to PTRUES_D using decoder 269
                                             // 28508:       }
                                             // 28508:       } // switch Inst[20:16]
                                             // 28508:      }
                                             // 28508:      } // switch Inst[13:10]
                                             // 28508:     }
                                             // 28508:     } // switch Inst[15:14]
                                             // 28508:    }
                                             // 28508:    } // switch Inst[25:22]
                                             // 28508:   }
  1, 0,                                      // 28513:   case 0x1: {
  OPC_SwitchField, 24, 2,                    // 28515:    switch Inst[25:24] {
  0, 207, 1,                                 // 28518:    case 0x0: {
  OPC_SwitchField, 22, 2,                    // 28521:     switch Inst[23:22] {
  0, 49,                                     // 28524:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 28526:      switch Inst[4] {
  0, 21,                                     // 28529:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 28531:       switch Inst[13] {
  0, 7,                                      // 28534:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28536:        check predicate 33
  OPC_Decode, 209, 18, 145, 2,               // 28538:        decode to CMPHS_PPzZI_B using decoder 273
                                             // 28538:       }
  1, 0,                                      // 28543:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28545:        check predicate 33
  OPC_Decode, 227, 18, 145, 2,               // 28547:        decode to CMPLO_PPzZI_B using decoder 273
                                             // 28547:       }
                                             // 28547:       } // switch Inst[13]
                                             // 28547:      }
  1, 0,                                      // 28552:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 28554:       switch Inst[13] {
  0, 7,                                      // 28557:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28559:        check predicate 33
  OPC_Decode, 198, 18, 145, 2,               // 28561:        decode to CMPHI_PPzZI_B using decoder 273
                                             // 28561:       }
  1, 0,                                      // 28566:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28568:        check predicate 33
  OPC_Decode, 234, 18, 145, 2,               // 28570:        decode to CMPLS_PPzZI_B using decoder 273
                                             // 28570:       }
                                             // 28570:       } // switch Inst[13]
                                             // 28570:      }
                                             // 28570:      } // switch Inst[4]
                                             // 28570:     }
  1, 49,                                     // 28575:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 28577:      switch Inst[4] {
  0, 21,                                     // 28580:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 28582:       switch Inst[13] {
  0, 7,                                      // 28585:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28587:        check predicate 33
  OPC_Decode, 211, 18, 145, 2,               // 28589:        decode to CMPHS_PPzZI_H using decoder 273
                                             // 28589:       }
  1, 0,                                      // 28594:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28596:        check predicate 33
  OPC_Decode, 229, 18, 145, 2,               // 28598:        decode to CMPLO_PPzZI_H using decoder 273
                                             // 28598:       }
                                             // 28598:       } // switch Inst[13]
                                             // 28598:      }
  1, 0,                                      // 28603:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 28605:       switch Inst[13] {
  0, 7,                                      // 28608:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28610:        check predicate 33
  OPC_Decode, 200, 18, 145, 2,               // 28612:        decode to CMPHI_PPzZI_H using decoder 273
                                             // 28612:       }
  1, 0,                                      // 28617:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28619:        check predicate 33
  OPC_Decode, 236, 18, 145, 2,               // 28621:        decode to CMPLS_PPzZI_H using decoder 273
                                             // 28621:       }
                                             // 28621:       } // switch Inst[13]
                                             // 28621:      }
                                             // 28621:      } // switch Inst[4]
                                             // 28621:     }
  2, 49,                                     // 28626:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 28628:      switch Inst[4] {
  0, 21,                                     // 28631:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 28633:       switch Inst[13] {
  0, 7,                                      // 28636:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28638:        check predicate 33
  OPC_Decode, 212, 18, 145, 2,               // 28640:        decode to CMPHS_PPzZI_S using decoder 273
                                             // 28640:       }
  1, 0,                                      // 28645:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28647:        check predicate 33
  OPC_Decode, 230, 18, 145, 2,               // 28649:        decode to CMPLO_PPzZI_S using decoder 273
                                             // 28649:       }
                                             // 28649:       } // switch Inst[13]
                                             // 28649:      }
  1, 0,                                      // 28654:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 28656:       switch Inst[13] {
  0, 7,                                      // 28659:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28661:        check predicate 33
  OPC_Decode, 201, 18, 145, 2,               // 28663:        decode to CMPHI_PPzZI_S using decoder 273
                                             // 28663:       }
  1, 0,                                      // 28668:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28670:        check predicate 33
  OPC_Decode, 237, 18, 145, 2,               // 28672:        decode to CMPLS_PPzZI_S using decoder 273
                                             // 28672:       }
                                             // 28672:       } // switch Inst[13]
                                             // 28672:      }
                                             // 28672:      } // switch Inst[4]
                                             // 28672:     }
  3, 0,                                      // 28677:     case 0x3: {
  OPC_SwitchField, 4, 1,                     // 28679:      switch Inst[4] {
  0, 21,                                     // 28682:      case 0x0: {
  OPC_SwitchField, 13, 1,                    // 28684:       switch Inst[13] {
  0, 7,                                      // 28687:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28689:        check predicate 33
  OPC_Decode, 210, 18, 145, 2,               // 28691:        decode to CMPHS_PPzZI_D using decoder 273
                                             // 28691:       }
  1, 0,                                      // 28696:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28698:        check predicate 33
  OPC_Decode, 228, 18, 145, 2,               // 28700:        decode to CMPLO_PPzZI_D using decoder 273
                                             // 28700:       }
                                             // 28700:       } // switch Inst[13]
                                             // 28700:      }
  1, 0,                                      // 28705:      case 0x1: {
  OPC_SwitchField, 13, 1,                    // 28707:       switch Inst[13] {
  0, 7,                                      // 28710:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 28712:        check predicate 33
  OPC_Decode, 199, 18, 145, 2,               // 28714:        decode to CMPHI_PPzZI_D using decoder 273
                                             // 28714:       }
  1, 0,                                      // 28719:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 28721:        check predicate 33
  OPC_Decode, 235, 18, 145, 2,               // 28723:        decode to CMPLS_PPzZI_D using decoder 273
                                             // 28723:       }
                                             // 28723:       } // switch Inst[13]
                                             // 28723:      }
                                             // 28723:      } // switch Inst[4]
                                             // 28723:     }
                                             // 28723:     } // switch Inst[23:22]
                                             // 28723:    }
  1, 0,                                      // 28728:    case 0x1: {
  OPC_SwitchField, 14, 2,                    // 28730:     switch Inst[15:14] {
  0, 159, 7,                                 // 28733:     case 0x0: {
  OPC_SwitchField, 10, 4,                    // 28736:      switch Inst[13:10] {
  0, 95,                                     // 28739:      case 0x0: {
  OPC_SwitchField, 22, 2,                    // 28741:       switch Inst[23:22] {
  0, 21,                                     // 28744:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 28746:        switch Inst[4] {
  0, 7,                                      // 28749:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28751:         check predicate 35
  OPC_Decode, 227, 69, 146, 2,               // 28753:         decode to WHILEGE_PWW_B using decoder 274
                                             // 28753:        }
  1, 0,                                      // 28758:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 28760:         check predicate 35
  OPC_Decode, 243, 69, 146, 2,               // 28762:         decode to WHILEGT_PWW_B using decoder 274
                                             // 28762:        }
                                             // 28762:        } // switch Inst[4]
                                             // 28762:       }
  1, 21,                                     // 28767:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 28769:        switch Inst[4] {
  0, 7,                                      // 28772:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28774:         check predicate 35
  OPC_Decode, 229, 69, 146, 2,               // 28776:         decode to WHILEGE_PWW_H using decoder 274
                                             // 28776:        }
  1, 0,                                      // 28781:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 28783:         check predicate 35
  OPC_Decode, 245, 69, 146, 2,               // 28785:         decode to WHILEGT_PWW_H using decoder 274
                                             // 28785:        }
                                             // 28785:        } // switch Inst[4]
                                             // 28785:       }
  2, 21,                                     // 28790:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 28792:        switch Inst[4] {
  0, 7,                                      // 28795:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28797:         check predicate 35
  OPC_Decode, 230, 69, 146, 2,               // 28799:         decode to WHILEGE_PWW_S using decoder 274
                                             // 28799:        }
  1, 0,                                      // 28804:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 28806:         check predicate 35
  OPC_Decode, 246, 69, 146, 2,               // 28808:         decode to WHILEGT_PWW_S using decoder 274
                                             // 28808:        }
                                             // 28808:        } // switch Inst[4]
                                             // 28808:       }
  3, 0,                                      // 28813:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 28815:        switch Inst[4] {
  0, 7,                                      // 28818:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28820:         check predicate 35
  OPC_Decode, 228, 69, 146, 2,               // 28822:         decode to WHILEGE_PWW_D using decoder 274
                                             // 28822:        }
  1, 0,                                      // 28827:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 28829:         check predicate 35
  OPC_Decode, 244, 69, 146, 2,               // 28831:         decode to WHILEGT_PWW_D using decoder 274
                                             // 28831:        }
                                             // 28831:        } // switch Inst[4]
                                             // 28831:       }
                                             // 28831:       } // switch Inst[23:22]
                                             // 28831:      }
  1, 95,                                     // 28836:      case 0x1: {
  OPC_SwitchField, 22, 2,                    // 28838:       switch Inst[23:22] {
  0, 21,                                     // 28841:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 28843:        switch Inst[4] {
  0, 7,                                      // 28846:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28848:         check predicate 33
  OPC_Decode, 211, 70, 146, 2,               // 28850:         decode to WHILELT_PWW_B using decoder 274
                                             // 28850:        }
  1, 0,                                      // 28855:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28857:         check predicate 33
  OPC_Decode, 163, 70, 146, 2,               // 28859:         decode to WHILELE_PWW_B using decoder 274
                                             // 28859:        }
                                             // 28859:        } // switch Inst[4]
                                             // 28859:       }
  1, 21,                                     // 28864:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 28866:        switch Inst[4] {
  0, 7,                                      // 28869:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28871:         check predicate 33
  OPC_Decode, 213, 70, 146, 2,               // 28873:         decode to WHILELT_PWW_H using decoder 274
                                             // 28873:        }
  1, 0,                                      // 28878:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28880:         check predicate 33
  OPC_Decode, 165, 70, 146, 2,               // 28882:         decode to WHILELE_PWW_H using decoder 274
                                             // 28882:        }
                                             // 28882:        } // switch Inst[4]
                                             // 28882:       }
  2, 21,                                     // 28887:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 28889:        switch Inst[4] {
  0, 7,                                      // 28892:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28894:         check predicate 33
  OPC_Decode, 214, 70, 146, 2,               // 28896:         decode to WHILELT_PWW_S using decoder 274
                                             // 28896:        }
  1, 0,                                      // 28901:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28903:         check predicate 33
  OPC_Decode, 166, 70, 146, 2,               // 28905:         decode to WHILELE_PWW_S using decoder 274
                                             // 28905:        }
                                             // 28905:        } // switch Inst[4]
                                             // 28905:       }
  3, 0,                                      // 28910:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 28912:        switch Inst[4] {
  0, 7,                                      // 28915:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 28917:         check predicate 33
  OPC_Decode, 212, 70, 146, 2,               // 28919:         decode to WHILELT_PWW_D using decoder 274
                                             // 28919:        }
  1, 0,                                      // 28924:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 28926:         check predicate 33
  OPC_Decode, 164, 70, 146, 2,               // 28928:         decode to WHILELE_PWW_D using decoder 274
                                             // 28928:        }
                                             // 28928:        } // switch Inst[4]
                                             // 28928:       }
                                             // 28928:       } // switch Inst[23:22]
                                             // 28928:      }
  2, 95,                                     // 28933:      case 0x2: {
  OPC_SwitchField, 22, 2,                    // 28935:       switch Inst[23:22] {
  0, 21,                                     // 28938:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 28940:        switch Inst[4] {
  0, 7,                                      // 28943:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28945:         check predicate 35
  OPC_Decode, 147, 70, 146, 2,               // 28947:         decode to WHILEHS_PWW_B using decoder 274
                                             // 28947:        }
  1, 0,                                      // 28952:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 28954:         check predicate 35
  OPC_Decode, 131, 70, 146, 2,               // 28956:         decode to WHILEHI_PWW_B using decoder 274
                                             // 28956:        }
                                             // 28956:        } // switch Inst[4]
                                             // 28956:       }
  1, 21,                                     // 28961:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 28963:        switch Inst[4] {
  0, 7,                                      // 28966:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28968:         check predicate 35
  OPC_Decode, 149, 70, 146, 2,               // 28970:         decode to WHILEHS_PWW_H using decoder 274
                                             // 28970:        }
  1, 0,                                      // 28975:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 28977:         check predicate 35
  OPC_Decode, 133, 70, 146, 2,               // 28979:         decode to WHILEHI_PWW_H using decoder 274
                                             // 28979:        }
                                             // 28979:        } // switch Inst[4]
                                             // 28979:       }
  2, 21,                                     // 28984:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 28986:        switch Inst[4] {
  0, 7,                                      // 28989:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 28991:         check predicate 35
  OPC_Decode, 150, 70, 146, 2,               // 28993:         decode to WHILEHS_PWW_S using decoder 274
                                             // 28993:        }
  1, 0,                                      // 28998:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29000:         check predicate 35
  OPC_Decode, 134, 70, 146, 2,               // 29002:         decode to WHILEHI_PWW_S using decoder 274
                                             // 29002:        }
                                             // 29002:        } // switch Inst[4]
                                             // 29002:       }
  3, 0,                                      // 29007:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29009:        switch Inst[4] {
  0, 7,                                      // 29012:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29014:         check predicate 35
  OPC_Decode, 148, 70, 146, 2,               // 29016:         decode to WHILEHS_PWW_D using decoder 274
                                             // 29016:        }
  1, 0,                                      // 29021:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29023:         check predicate 35
  OPC_Decode, 132, 70, 146, 2,               // 29025:         decode to WHILEHI_PWW_D using decoder 274
                                             // 29025:        }
                                             // 29025:        } // switch Inst[4]
                                             // 29025:       }
                                             // 29025:       } // switch Inst[23:22]
                                             // 29025:      }
  3, 95,                                     // 29030:      case 0x3: {
  OPC_SwitchField, 22, 2,                    // 29032:       switch Inst[23:22] {
  0, 21,                                     // 29035:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 29037:        switch Inst[4] {
  0, 7,                                      // 29040:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29042:         check predicate 33
  OPC_Decode, 179, 70, 146, 2,               // 29044:         decode to WHILELO_PWW_B using decoder 274
                                             // 29044:        }
  1, 0,                                      // 29049:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29051:         check predicate 33
  OPC_Decode, 195, 70, 146, 2,               // 29053:         decode to WHILELS_PWW_B using decoder 274
                                             // 29053:        }
                                             // 29053:        } // switch Inst[4]
                                             // 29053:       }
  1, 21,                                     // 29058:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29060:        switch Inst[4] {
  0, 7,                                      // 29063:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29065:         check predicate 33
  OPC_Decode, 181, 70, 146, 2,               // 29067:         decode to WHILELO_PWW_H using decoder 274
                                             // 29067:        }
  1, 0,                                      // 29072:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29074:         check predicate 33
  OPC_Decode, 197, 70, 146, 2,               // 29076:         decode to WHILELS_PWW_H using decoder 274
                                             // 29076:        }
                                             // 29076:        } // switch Inst[4]
                                             // 29076:       }
  2, 21,                                     // 29081:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 29083:        switch Inst[4] {
  0, 7,                                      // 29086:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29088:         check predicate 33
  OPC_Decode, 182, 70, 146, 2,               // 29090:         decode to WHILELO_PWW_S using decoder 274
                                             // 29090:        }
  1, 0,                                      // 29095:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29097:         check predicate 33
  OPC_Decode, 198, 70, 146, 2,               // 29099:         decode to WHILELS_PWW_S using decoder 274
                                             // 29099:        }
                                             // 29099:        } // switch Inst[4]
                                             // 29099:       }
  3, 0,                                      // 29104:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29106:        switch Inst[4] {
  0, 7,                                      // 29109:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29111:         check predicate 33
  OPC_Decode, 180, 70, 146, 2,               // 29113:         decode to WHILELO_PWW_D using decoder 274
                                             // 29113:        }
  1, 0,                                      // 29118:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29120:         check predicate 33
  OPC_Decode, 196, 70, 146, 2,               // 29122:         decode to WHILELS_PWW_D using decoder 274
                                             // 29122:        }
                                             // 29122:        } // switch Inst[4]
                                             // 29122:       }
                                             // 29122:       } // switch Inst[23:22]
                                             // 29122:      }
  4, 95,                                     // 29127:      case 0x4: {
  OPC_SwitchField, 22, 2,                    // 29129:       switch Inst[23:22] {
  0, 21,                                     // 29132:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 29134:        switch Inst[4] {
  0, 7,                                      // 29137:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29139:         check predicate 35
  OPC_Decode, 231, 69, 147, 2,               // 29141:         decode to WHILEGE_PXX_B using decoder 275
                                             // 29141:        }
  1, 0,                                      // 29146:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29148:         check predicate 35
  OPC_Decode, 247, 69, 147, 2,               // 29150:         decode to WHILEGT_PXX_B using decoder 275
                                             // 29150:        }
                                             // 29150:        } // switch Inst[4]
                                             // 29150:       }
  1, 21,                                     // 29155:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29157:        switch Inst[4] {
  0, 7,                                      // 29160:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29162:         check predicate 35
  OPC_Decode, 233, 69, 147, 2,               // 29164:         decode to WHILEGE_PXX_H using decoder 275
                                             // 29164:        }
  1, 0,                                      // 29169:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29171:         check predicate 35
  OPC_Decode, 249, 69, 147, 2,               // 29173:         decode to WHILEGT_PXX_H using decoder 275
                                             // 29173:        }
                                             // 29173:        } // switch Inst[4]
                                             // 29173:       }
  2, 21,                                     // 29178:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 29180:        switch Inst[4] {
  0, 7,                                      // 29183:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29185:         check predicate 35
  OPC_Decode, 234, 69, 147, 2,               // 29187:         decode to WHILEGE_PXX_S using decoder 275
                                             // 29187:        }
  1, 0,                                      // 29192:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29194:         check predicate 35
  OPC_Decode, 250, 69, 147, 2,               // 29196:         decode to WHILEGT_PXX_S using decoder 275
                                             // 29196:        }
                                             // 29196:        } // switch Inst[4]
                                             // 29196:       }
  3, 0,                                      // 29201:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29203:        switch Inst[4] {
  0, 7,                                      // 29206:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29208:         check predicate 35
  OPC_Decode, 232, 69, 147, 2,               // 29210:         decode to WHILEGE_PXX_D using decoder 275
                                             // 29210:        }
  1, 0,                                      // 29215:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29217:         check predicate 35
  OPC_Decode, 248, 69, 147, 2,               // 29219:         decode to WHILEGT_PXX_D using decoder 275
                                             // 29219:        }
                                             // 29219:        } // switch Inst[4]
                                             // 29219:       }
                                             // 29219:       } // switch Inst[23:22]
                                             // 29219:      }
  5, 95,                                     // 29224:      case 0x5: {
  OPC_SwitchField, 22, 2,                    // 29226:       switch Inst[23:22] {
  0, 21,                                     // 29229:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 29231:        switch Inst[4] {
  0, 7,                                      // 29234:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29236:         check predicate 33
  OPC_Decode, 215, 70, 147, 2,               // 29238:         decode to WHILELT_PXX_B using decoder 275
                                             // 29238:        }
  1, 0,                                      // 29243:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29245:         check predicate 33
  OPC_Decode, 167, 70, 147, 2,               // 29247:         decode to WHILELE_PXX_B using decoder 275
                                             // 29247:        }
                                             // 29247:        } // switch Inst[4]
                                             // 29247:       }
  1, 21,                                     // 29252:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29254:        switch Inst[4] {
  0, 7,                                      // 29257:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29259:         check predicate 33
  OPC_Decode, 217, 70, 147, 2,               // 29261:         decode to WHILELT_PXX_H using decoder 275
                                             // 29261:        }
  1, 0,                                      // 29266:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29268:         check predicate 33
  OPC_Decode, 169, 70, 147, 2,               // 29270:         decode to WHILELE_PXX_H using decoder 275
                                             // 29270:        }
                                             // 29270:        } // switch Inst[4]
                                             // 29270:       }
  2, 21,                                     // 29275:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 29277:        switch Inst[4] {
  0, 7,                                      // 29280:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29282:         check predicate 33
  OPC_Decode, 218, 70, 147, 2,               // 29284:         decode to WHILELT_PXX_S using decoder 275
                                             // 29284:        }
  1, 0,                                      // 29289:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29291:         check predicate 33
  OPC_Decode, 170, 70, 147, 2,               // 29293:         decode to WHILELE_PXX_S using decoder 275
                                             // 29293:        }
                                             // 29293:        } // switch Inst[4]
                                             // 29293:       }
  3, 0,                                      // 29298:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29300:        switch Inst[4] {
  0, 7,                                      // 29303:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29305:         check predicate 33
  OPC_Decode, 216, 70, 147, 2,               // 29307:         decode to WHILELT_PXX_D using decoder 275
                                             // 29307:        }
  1, 0,                                      // 29312:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29314:         check predicate 33
  OPC_Decode, 168, 70, 147, 2,               // 29316:         decode to WHILELE_PXX_D using decoder 275
                                             // 29316:        }
                                             // 29316:        } // switch Inst[4]
                                             // 29316:       }
                                             // 29316:       } // switch Inst[23:22]
                                             // 29316:      }
  6, 95,                                     // 29321:      case 0x6: {
  OPC_SwitchField, 22, 2,                    // 29323:       switch Inst[23:22] {
  0, 21,                                     // 29326:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 29328:        switch Inst[4] {
  0, 7,                                      // 29331:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29333:         check predicate 35
  OPC_Decode, 151, 70, 147, 2,               // 29335:         decode to WHILEHS_PXX_B using decoder 275
                                             // 29335:        }
  1, 0,                                      // 29340:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29342:         check predicate 35
  OPC_Decode, 135, 70, 147, 2,               // 29344:         decode to WHILEHI_PXX_B using decoder 275
                                             // 29344:        }
                                             // 29344:        } // switch Inst[4]
                                             // 29344:       }
  1, 21,                                     // 29349:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29351:        switch Inst[4] {
  0, 7,                                      // 29354:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29356:         check predicate 35
  OPC_Decode, 153, 70, 147, 2,               // 29358:         decode to WHILEHS_PXX_H using decoder 275
                                             // 29358:        }
  1, 0,                                      // 29363:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29365:         check predicate 35
  OPC_Decode, 137, 70, 147, 2,               // 29367:         decode to WHILEHI_PXX_H using decoder 275
                                             // 29367:        }
                                             // 29367:        } // switch Inst[4]
                                             // 29367:       }
  2, 21,                                     // 29372:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 29374:        switch Inst[4] {
  0, 7,                                      // 29377:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29379:         check predicate 35
  OPC_Decode, 154, 70, 147, 2,               // 29381:         decode to WHILEHS_PXX_S using decoder 275
                                             // 29381:        }
  1, 0,                                      // 29386:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29388:         check predicate 35
  OPC_Decode, 138, 70, 147, 2,               // 29390:         decode to WHILEHI_PXX_S using decoder 275
                                             // 29390:        }
                                             // 29390:        } // switch Inst[4]
                                             // 29390:       }
  3, 0,                                      // 29395:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29397:        switch Inst[4] {
  0, 7,                                      // 29400:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29402:         check predicate 35
  OPC_Decode, 152, 70, 147, 2,               // 29404:         decode to WHILEHS_PXX_D using decoder 275
                                             // 29404:        }
  1, 0,                                      // 29409:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29411:         check predicate 35
  OPC_Decode, 136, 70, 147, 2,               // 29413:         decode to WHILEHI_PXX_D using decoder 275
                                             // 29413:        }
                                             // 29413:        } // switch Inst[4]
                                             // 29413:       }
                                             // 29413:       } // switch Inst[23:22]
                                             // 29413:      }
  7, 95,                                     // 29418:      case 0x7: {
  OPC_SwitchField, 22, 2,                    // 29420:       switch Inst[23:22] {
  0, 21,                                     // 29423:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 29425:        switch Inst[4] {
  0, 7,                                      // 29428:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29430:         check predicate 33
  OPC_Decode, 183, 70, 147, 2,               // 29432:         decode to WHILELO_PXX_B using decoder 275
                                             // 29432:        }
  1, 0,                                      // 29437:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29439:         check predicate 33
  OPC_Decode, 199, 70, 147, 2,               // 29441:         decode to WHILELS_PXX_B using decoder 275
                                             // 29441:        }
                                             // 29441:        } // switch Inst[4]
                                             // 29441:       }
  1, 21,                                     // 29446:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29448:        switch Inst[4] {
  0, 7,                                      // 29451:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29453:         check predicate 33
  OPC_Decode, 185, 70, 147, 2,               // 29455:         decode to WHILELO_PXX_H using decoder 275
                                             // 29455:        }
  1, 0,                                      // 29460:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29462:         check predicate 33
  OPC_Decode, 201, 70, 147, 2,               // 29464:         decode to WHILELS_PXX_H using decoder 275
                                             // 29464:        }
                                             // 29464:        } // switch Inst[4]
                                             // 29464:       }
  2, 21,                                     // 29469:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 29471:        switch Inst[4] {
  0, 7,                                      // 29474:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29476:         check predicate 33
  OPC_Decode, 186, 70, 147, 2,               // 29478:         decode to WHILELO_PXX_S using decoder 275
                                             // 29478:        }
  1, 0,                                      // 29483:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29485:         check predicate 33
  OPC_Decode, 202, 70, 147, 2,               // 29487:         decode to WHILELS_PXX_S using decoder 275
                                             // 29487:        }
                                             // 29487:        } // switch Inst[4]
                                             // 29487:       }
  3, 0,                                      // 29492:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29494:        switch Inst[4] {
  0, 7,                                      // 29497:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 29499:         check predicate 33
  OPC_Decode, 184, 70, 147, 2,               // 29501:         decode to WHILELO_PXX_D using decoder 275
                                             // 29501:        }
  1, 0,                                      // 29506:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 29508:         check predicate 33
  OPC_Decode, 200, 70, 147, 2,               // 29510:         decode to WHILELS_PXX_D using decoder 275
                                             // 29510:        }
                                             // 29510:        } // switch Inst[4]
                                             // 29510:       }
                                             // 29510:       } // switch Inst[23:22]
                                             // 29510:      }
  8, 49,                                     // 29515:      case 0x8: {
  OPC_SwitchField, 0, 5,                     // 29517:       switch Inst[4:0] {
  0, 21,                                     // 29520:       case 0x0: {
  OPC_SwitchField, 22, 2,                    // 29522:        switch Inst[23:22] {
  2, 7,                                      // 29525:        case 0x2: {
  OPC_CheckPredicate, 33,                    // 29527:         check predicate 33
  OPC_Decode, 175, 20, 148, 2,               // 29529:         decode to CTERMEQ_WW using decoder 276
                                             // 29529:        }
  3, 0,                                      // 29534:        case 0x3: {
  OPC_CheckPredicate, 33,                    // 29536:         check predicate 33
  OPC_Decode, 176, 20, 149, 2,               // 29538:         decode to CTERMEQ_XX using decoder 277
                                             // 29538:        }
                                             // 29538:        } // switch Inst[23:22]
                                             // 29538:       }
  16, 0,                                     // 29543:       case 0x10: {
  OPC_SwitchField, 22, 2,                    // 29545:        switch Inst[23:22] {
  2, 7,                                      // 29548:        case 0x2: {
  OPC_CheckPredicate, 33,                    // 29550:         check predicate 33
  OPC_Decode, 177, 20, 148, 2,               // 29552:         decode to CTERMNE_WW using decoder 276
                                             // 29552:        }
  3, 0,                                      // 29557:        case 0x3: {
  OPC_CheckPredicate, 33,                    // 29559:         check predicate 33
  OPC_Decode, 178, 20, 149, 2,               // 29561:         decode to CTERMNE_XX using decoder 277
                                             // 29561:        }
                                             // 29561:        } // switch Inst[23:22]
                                             // 29561:       }
                                             // 29561:       } // switch Inst[4:0]
                                             // 29561:      }
  12, 0,                                     // 29566:      case 0xc: {
  OPC_SwitchField, 22, 2,                    // 29568:       switch Inst[23:22] {
  0, 21,                                     // 29571:       case 0x0: {
  OPC_SwitchField, 4, 1,                     // 29573:        switch Inst[4] {
  0, 7,                                      // 29576:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29578:         check predicate 35
  OPC_Decode, 223, 70, 147, 2,               // 29580:         decode to WHILEWR_PXX_B using decoder 275
                                             // 29580:        }
  1, 0,                                      // 29585:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29587:         check predicate 35
  OPC_Decode, 219, 70, 147, 2,               // 29589:         decode to WHILERW_PXX_B using decoder 275
                                             // 29589:        }
                                             // 29589:        } // switch Inst[4]
                                             // 29589:       }
  1, 21,                                     // 29594:       case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29596:        switch Inst[4] {
  0, 7,                                      // 29599:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29601:         check predicate 35
  OPC_Decode, 225, 70, 147, 2,               // 29603:         decode to WHILEWR_PXX_H using decoder 275
                                             // 29603:        }
  1, 0,                                      // 29608:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29610:         check predicate 35
  OPC_Decode, 221, 70, 147, 2,               // 29612:         decode to WHILERW_PXX_H using decoder 275
                                             // 29612:        }
                                             // 29612:        } // switch Inst[4]
                                             // 29612:       }
  2, 21,                                     // 29617:       case 0x2: {
  OPC_SwitchField, 4, 1,                     // 29619:        switch Inst[4] {
  0, 7,                                      // 29622:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29624:         check predicate 35
  OPC_Decode, 226, 70, 147, 2,               // 29626:         decode to WHILEWR_PXX_S using decoder 275
                                             // 29626:        }
  1, 0,                                      // 29631:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29633:         check predicate 35
  OPC_Decode, 222, 70, 147, 2,               // 29635:         decode to WHILERW_PXX_S using decoder 275
                                             // 29635:        }
                                             // 29635:        } // switch Inst[4]
                                             // 29635:       }
  3, 0,                                      // 29640:       case 0x3: {
  OPC_SwitchField, 4, 1,                     // 29642:        switch Inst[4] {
  0, 7,                                      // 29645:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 29647:         check predicate 35
  OPC_Decode, 224, 70, 147, 2,               // 29649:         decode to WHILEWR_PXX_D using decoder 275
                                             // 29649:        }
  1, 0,                                      // 29654:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 29656:         check predicate 35
  OPC_Decode, 220, 70, 147, 2,               // 29658:         decode to WHILERW_PXX_D using decoder 275
                                             // 29658:        }
                                             // 29658:        } // switch Inst[4]
                                             // 29658:       }
                                             // 29658:       } // switch Inst[23:22]
                                             // 29658:      }
                                             // 29658:      } // switch Inst[13:10]
                                             // 29658:     }
  1, 130, 9,                                 // 29663:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 29666:      switch Inst[4] {
  0, 69,                                     // 29669:      case 0x0: {
  OPC_SwitchField, 18, 1,                    // 29671:       switch Inst[18] {
  0, 51,                                     // 29674:       case 0x0: {
  OPC_SwitchField, 19, 1,                    // 29676:        switch Inst[19] {
  0, 33,                                     // 29679:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 29681:         switch Inst[20] {
  0, 15,                                     // 29684:         case 0x0: {
  OPC_CheckPredicate, 43,                    // 29686:          check predicate 43
  OPC_CheckField, 22, 1, 1,                  // 29688:          check Inst[22] == 0x1
  OPC_CheckField, 9, 1, 0,                   // 29692:          check Inst[9] == 0x0
  OPC_Decode, 198, 44, 150, 2,               // 29696:          decode to PSEL_PPPRI_D using decoder 278
                                             // 29696:         }
  1, 0,                                      // 29701:         case 0x1: {
  OPC_CheckPredicate, 43,                    // 29703:          check predicate 43
  OPC_CheckField, 9, 1, 0,                   // 29705:          check Inst[9] == 0x0
  OPC_Decode, 200, 44, 151, 2,               // 29709:          decode to PSEL_PPPRI_S using decoder 279
                                             // 29709:         }
                                             // 29709:         } // switch Inst[20]
                                             // 29709:        }
  1, 0,                                      // 29714:        case 0x1: {
  OPC_CheckPredicate, 43,                    // 29716:         check predicate 43
  OPC_CheckField, 9, 1, 0,                   // 29718:         check Inst[9] == 0x0
  OPC_Decode, 199, 44, 152, 2,               // 29722:         decode to PSEL_PPPRI_H using decoder 280
                                             // 29722:        }
                                             // 29722:        } // switch Inst[19]
                                             // 29722:       }
  1, 0,                                      // 29727:       case 0x1: {
  OPC_CheckPredicate, 43,                    // 29729:        check predicate 43
  OPC_CheckField, 9, 1, 0,                   // 29731:        check Inst[9] == 0x0
  OPC_Decode, 197, 44, 153, 2,               // 29735:        decode to PSEL_PPPRI_B using decoder 281
                                             // 29735:       }
                                             // 29735:       } // switch Inst[18]
                                             // 29735:      }
  1, 0,                                      // 29740:      case 0x1: {
  OPC_SwitchField, 10, 3,                    // 29742:       switch Inst[12:10] {
  0, 95,                                     // 29745:       case 0x0: {
  OPC_SwitchField, 22, 2,                    // 29747:        switch Inst[23:22] {
  0, 21,                                     // 29750:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 29752:         switch Inst[3] {
  0, 7,                                      // 29755:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29757:          check predicate 18
  OPC_Decode, 223, 69, 154, 2,               // 29759:          decode to WHILEGE_CXX_B using decoder 282
                                             // 29759:         }
  1, 0,                                      // 29764:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29766:          check predicate 18
  OPC_Decode, 239, 69, 154, 2,               // 29768:          decode to WHILEGT_CXX_B using decoder 282
                                             // 29768:         }
                                             // 29768:         } // switch Inst[3]
                                             // 29768:        }
  1, 21,                                     // 29773:        case 0x1: {
  OPC_SwitchField, 3, 1,                     // 29775:         switch Inst[3] {
  0, 7,                                      // 29778:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29780:          check predicate 18
  OPC_Decode, 225, 69, 154, 2,               // 29782:          decode to WHILEGE_CXX_H using decoder 282
                                             // 29782:         }
  1, 0,                                      // 29787:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29789:          check predicate 18
  OPC_Decode, 241, 69, 154, 2,               // 29791:          decode to WHILEGT_CXX_H using decoder 282
                                             // 29791:         }
                                             // 29791:         } // switch Inst[3]
                                             // 29791:        }
  2, 21,                                     // 29796:        case 0x2: {
  OPC_SwitchField, 3, 1,                     // 29798:         switch Inst[3] {
  0, 7,                                      // 29801:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29803:          check predicate 18
  OPC_Decode, 226, 69, 154, 2,               // 29805:          decode to WHILEGE_CXX_S using decoder 282
                                             // 29805:         }
  1, 0,                                      // 29810:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29812:          check predicate 18
  OPC_Decode, 242, 69, 154, 2,               // 29814:          decode to WHILEGT_CXX_S using decoder 282
                                             // 29814:         }
                                             // 29814:         } // switch Inst[3]
                                             // 29814:        }
  3, 0,                                      // 29819:        case 0x3: {
  OPC_SwitchField, 3, 1,                     // 29821:         switch Inst[3] {
  0, 7,                                      // 29824:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29826:          check predicate 18
  OPC_Decode, 224, 69, 154, 2,               // 29828:          decode to WHILEGE_CXX_D using decoder 282
                                             // 29828:         }
  1, 0,                                      // 29833:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29835:          check predicate 18
  OPC_Decode, 240, 69, 154, 2,               // 29837:          decode to WHILEGT_CXX_D using decoder 282
                                             // 29837:         }
                                             // 29837:         } // switch Inst[3]
                                             // 29837:        }
                                             // 29837:        } // switch Inst[23:22]
                                             // 29837:       }
  1, 95,                                     // 29842:       case 0x1: {
  OPC_SwitchField, 22, 2,                    // 29844:        switch Inst[23:22] {
  0, 21,                                     // 29847:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 29849:         switch Inst[3] {
  0, 7,                                      // 29852:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29854:          check predicate 18
  OPC_Decode, 207, 70, 154, 2,               // 29856:          decode to WHILELT_CXX_B using decoder 282
                                             // 29856:         }
  1, 0,                                      // 29861:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29863:          check predicate 18
  OPC_Decode, 159, 70, 154, 2,               // 29865:          decode to WHILELE_CXX_B using decoder 282
                                             // 29865:         }
                                             // 29865:         } // switch Inst[3]
                                             // 29865:        }
  1, 21,                                     // 29870:        case 0x1: {
  OPC_SwitchField, 3, 1,                     // 29872:         switch Inst[3] {
  0, 7,                                      // 29875:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29877:          check predicate 18
  OPC_Decode, 209, 70, 154, 2,               // 29879:          decode to WHILELT_CXX_H using decoder 282
                                             // 29879:         }
  1, 0,                                      // 29884:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29886:          check predicate 18
  OPC_Decode, 161, 70, 154, 2,               // 29888:          decode to WHILELE_CXX_H using decoder 282
                                             // 29888:         }
                                             // 29888:         } // switch Inst[3]
                                             // 29888:        }
  2, 21,                                     // 29893:        case 0x2: {
  OPC_SwitchField, 3, 1,                     // 29895:         switch Inst[3] {
  0, 7,                                      // 29898:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29900:          check predicate 18
  OPC_Decode, 210, 70, 154, 2,               // 29902:          decode to WHILELT_CXX_S using decoder 282
                                             // 29902:         }
  1, 0,                                      // 29907:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29909:          check predicate 18
  OPC_Decode, 162, 70, 154, 2,               // 29911:          decode to WHILELE_CXX_S using decoder 282
                                             // 29911:         }
                                             // 29911:         } // switch Inst[3]
                                             // 29911:        }
  3, 0,                                      // 29916:        case 0x3: {
  OPC_SwitchField, 3, 1,                     // 29918:         switch Inst[3] {
  0, 7,                                      // 29921:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29923:          check predicate 18
  OPC_Decode, 208, 70, 154, 2,               // 29925:          decode to WHILELT_CXX_D using decoder 282
                                             // 29925:         }
  1, 0,                                      // 29930:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29932:          check predicate 18
  OPC_Decode, 160, 70, 154, 2,               // 29934:          decode to WHILELE_CXX_D using decoder 282
                                             // 29934:         }
                                             // 29934:         } // switch Inst[3]
                                             // 29934:        }
                                             // 29934:        } // switch Inst[23:22]
                                             // 29934:       }
  2, 95,                                     // 29939:       case 0x2: {
  OPC_SwitchField, 22, 2,                    // 29941:        switch Inst[23:22] {
  0, 21,                                     // 29944:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 29946:         switch Inst[3] {
  0, 7,                                      // 29949:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29951:          check predicate 18
  OPC_Decode, 143, 70, 154, 2,               // 29953:          decode to WHILEHS_CXX_B using decoder 282
                                             // 29953:         }
  1, 0,                                      // 29958:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29960:          check predicate 18
  OPC_Decode, 255, 69, 154, 2,               // 29962:          decode to WHILEHI_CXX_B using decoder 282
                                             // 29962:         }
                                             // 29962:         } // switch Inst[3]
                                             // 29962:        }
  1, 21,                                     // 29967:        case 0x1: {
  OPC_SwitchField, 3, 1,                     // 29969:         switch Inst[3] {
  0, 7,                                      // 29972:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29974:          check predicate 18
  OPC_Decode, 145, 70, 154, 2,               // 29976:          decode to WHILEHS_CXX_H using decoder 282
                                             // 29976:         }
  1, 0,                                      // 29981:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 29983:          check predicate 18
  OPC_Decode, 129, 70, 154, 2,               // 29985:          decode to WHILEHI_CXX_H using decoder 282
                                             // 29985:         }
                                             // 29985:         } // switch Inst[3]
                                             // 29985:        }
  2, 21,                                     // 29990:        case 0x2: {
  OPC_SwitchField, 3, 1,                     // 29992:         switch Inst[3] {
  0, 7,                                      // 29995:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 29997:          check predicate 18
  OPC_Decode, 146, 70, 154, 2,               // 29999:          decode to WHILEHS_CXX_S using decoder 282
                                             // 29999:         }
  1, 0,                                      // 30004:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30006:          check predicate 18
  OPC_Decode, 130, 70, 154, 2,               // 30008:          decode to WHILEHI_CXX_S using decoder 282
                                             // 30008:         }
                                             // 30008:         } // switch Inst[3]
                                             // 30008:        }
  3, 0,                                      // 30013:        case 0x3: {
  OPC_SwitchField, 3, 1,                     // 30015:         switch Inst[3] {
  0, 7,                                      // 30018:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30020:          check predicate 18
  OPC_Decode, 144, 70, 154, 2,               // 30022:          decode to WHILEHS_CXX_D using decoder 282
                                             // 30022:         }
  1, 0,                                      // 30027:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30029:          check predicate 18
  OPC_Decode, 128, 70, 154, 2,               // 30031:          decode to WHILEHI_CXX_D using decoder 282
                                             // 30031:         }
                                             // 30031:         } // switch Inst[3]
                                             // 30031:        }
                                             // 30031:        } // switch Inst[23:22]
                                             // 30031:       }
  3, 95,                                     // 30036:       case 0x3: {
  OPC_SwitchField, 22, 2,                    // 30038:        switch Inst[23:22] {
  0, 21,                                     // 30041:        case 0x0: {
  OPC_SwitchField, 3, 1,                     // 30043:         switch Inst[3] {
  0, 7,                                      // 30046:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30048:          check predicate 18
  OPC_Decode, 175, 70, 154, 2,               // 30050:          decode to WHILELO_CXX_B using decoder 282
                                             // 30050:         }
  1, 0,                                      // 30055:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30057:          check predicate 18
  OPC_Decode, 191, 70, 154, 2,               // 30059:          decode to WHILELS_CXX_B using decoder 282
                                             // 30059:         }
                                             // 30059:         } // switch Inst[3]
                                             // 30059:        }
  1, 21,                                     // 30064:        case 0x1: {
  OPC_SwitchField, 3, 1,                     // 30066:         switch Inst[3] {
  0, 7,                                      // 30069:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30071:          check predicate 18
  OPC_Decode, 177, 70, 154, 2,               // 30073:          decode to WHILELO_CXX_H using decoder 282
                                             // 30073:         }
  1, 0,                                      // 30078:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30080:          check predicate 18
  OPC_Decode, 193, 70, 154, 2,               // 30082:          decode to WHILELS_CXX_H using decoder 282
                                             // 30082:         }
                                             // 30082:         } // switch Inst[3]
                                             // 30082:        }
  2, 21,                                     // 30087:        case 0x2: {
  OPC_SwitchField, 3, 1,                     // 30089:         switch Inst[3] {
  0, 7,                                      // 30092:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30094:          check predicate 18
  OPC_Decode, 178, 70, 154, 2,               // 30096:          decode to WHILELO_CXX_S using decoder 282
                                             // 30096:         }
  1, 0,                                      // 30101:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30103:          check predicate 18
  OPC_Decode, 194, 70, 154, 2,               // 30105:          decode to WHILELS_CXX_S using decoder 282
                                             // 30105:         }
                                             // 30105:         } // switch Inst[3]
                                             // 30105:        }
  3, 0,                                      // 30110:        case 0x3: {
  OPC_SwitchField, 3, 1,                     // 30112:         switch Inst[3] {
  0, 7,                                      // 30115:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30117:          check predicate 18
  OPC_Decode, 176, 70, 154, 2,               // 30119:          decode to WHILELO_CXX_D using decoder 282
                                             // 30119:         }
  1, 0,                                      // 30124:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30126:          check predicate 18
  OPC_Decode, 192, 70, 154, 2,               // 30128:          decode to WHILELS_CXX_D using decoder 282
                                             // 30128:         }
                                             // 30128:         } // switch Inst[3]
                                             // 30128:        }
                                             // 30128:        } // switch Inst[23:22]
                                             // 30128:       }
  4, 167, 1,                                 // 30133:       case 0x4: {
  OPC_SwitchField, 22, 2,                    // 30136:        switch Inst[23:22] {
  0, 39,                                     // 30139:        case 0x0: {
  OPC_SwitchField, 13, 1,                    // 30141:         switch Inst[13] {
  0, 21,                                     // 30144:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30146:          switch Inst[0] {
  0, 7,                                      // 30149:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30151:           check predicate 18
  OPC_Decode, 219, 69, 155, 2,               // 30153:           decode to WHILEGE_2PXX_B using decoder 283
                                             // 30153:          }
  1, 0,                                      // 30158:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30160:           check predicate 18
  OPC_Decode, 235, 69, 155, 2,               // 30162:           decode to WHILEGT_2PXX_B using decoder 283
                                             // 30162:          }
                                             // 30162:          } // switch Inst[0]
                                             // 30162:         }
  1, 0,                                      // 30167:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30169:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30171:          check Inst[20:16] == 0x0
  OPC_Decode, 251, 43, 156, 2,               // 30175:          decode to PEXT_PCI_B using decoder 284
                                             // 30175:         }
                                             // 30175:         } // switch Inst[13]
                                             // 30175:        }
  1, 39,                                     // 30180:        case 0x1: {
  OPC_SwitchField, 13, 1,                    // 30182:         switch Inst[13] {
  0, 21,                                     // 30185:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30187:          switch Inst[0] {
  0, 7,                                      // 30190:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30192:           check predicate 18
  OPC_Decode, 221, 69, 155, 2,               // 30194:           decode to WHILEGE_2PXX_H using decoder 283
                                             // 30194:          }
  1, 0,                                      // 30199:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30201:           check predicate 18
  OPC_Decode, 237, 69, 155, 2,               // 30203:           decode to WHILEGT_2PXX_H using decoder 283
                                             // 30203:          }
                                             // 30203:          } // switch Inst[0]
                                             // 30203:         }
  1, 0,                                      // 30208:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30210:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30212:          check Inst[20:16] == 0x0
  OPC_Decode, 253, 43, 156, 2,               // 30216:          decode to PEXT_PCI_H using decoder 284
                                             // 30216:         }
                                             // 30216:         } // switch Inst[13]
                                             // 30216:        }
  2, 39,                                     // 30221:        case 0x2: {
  OPC_SwitchField, 13, 1,                    // 30223:         switch Inst[13] {
  0, 21,                                     // 30226:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30228:          switch Inst[0] {
  0, 7,                                      // 30231:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30233:           check predicate 18
  OPC_Decode, 222, 69, 155, 2,               // 30235:           decode to WHILEGE_2PXX_S using decoder 283
                                             // 30235:          }
  1, 0,                                      // 30240:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30242:           check predicate 18
  OPC_Decode, 238, 69, 155, 2,               // 30244:           decode to WHILEGT_2PXX_S using decoder 283
                                             // 30244:          }
                                             // 30244:          } // switch Inst[0]
                                             // 30244:         }
  1, 0,                                      // 30249:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30251:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30253:          check Inst[20:16] == 0x0
  OPC_Decode, 254, 43, 156, 2,               // 30257:          decode to PEXT_PCI_S using decoder 284
                                             // 30257:         }
                                             // 30257:         } // switch Inst[13]
                                             // 30257:        }
  3, 0,                                      // 30262:        case 0x3: {
  OPC_SwitchField, 13, 1,                    // 30264:         switch Inst[13] {
  0, 21,                                     // 30267:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30269:          switch Inst[0] {
  0, 7,                                      // 30272:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30274:           check predicate 18
  OPC_Decode, 220, 69, 155, 2,               // 30276:           decode to WHILEGE_2PXX_D using decoder 283
                                             // 30276:          }
  1, 0,                                      // 30281:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30283:           check predicate 18
  OPC_Decode, 236, 69, 155, 2,               // 30285:           decode to WHILEGT_2PXX_D using decoder 283
                                             // 30285:          }
                                             // 30285:          } // switch Inst[0]
                                             // 30285:         }
  1, 0,                                      // 30290:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30292:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30294:          check Inst[20:16] == 0x0
  OPC_Decode, 252, 43, 156, 2,               // 30298:          decode to PEXT_PCI_D using decoder 284
                                             // 30298:         }
                                             // 30298:         } // switch Inst[13]
                                             // 30298:        }
                                             // 30298:        } // switch Inst[23:22]
                                             // 30298:       }
  5, 183, 1,                                 // 30303:       case 0x5: {
  OPC_SwitchField, 22, 2,                    // 30306:        switch Inst[23:22] {
  0, 43,                                     // 30309:        case 0x0: {
  OPC_SwitchField, 13, 1,                    // 30311:         switch Inst[13] {
  0, 21,                                     // 30314:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30316:          switch Inst[0] {
  0, 7,                                      // 30319:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30321:           check predicate 18
  OPC_Decode, 203, 70, 155, 2,               // 30323:           decode to WHILELT_2PXX_B using decoder 283
                                             // 30323:          }
  1, 0,                                      // 30328:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30330:           check predicate 18
  OPC_Decode, 155, 70, 155, 2,               // 30332:           decode to WHILELE_2PXX_B using decoder 283
                                             // 30332:          }
                                             // 30332:          } // switch Inst[0]
                                             // 30332:         }
  1, 0,                                      // 30337:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30339:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30341:          check Inst[20:16] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 30345:          check Inst[9] == 0x0
  OPC_Decode, 247, 43, 157, 2,               // 30349:          decode to PEXT_2PCI_B using decoder 285
                                             // 30349:         }
                                             // 30349:         } // switch Inst[13]
                                             // 30349:        }
  1, 43,                                     // 30354:        case 0x1: {
  OPC_SwitchField, 13, 1,                    // 30356:         switch Inst[13] {
  0, 21,                                     // 30359:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30361:          switch Inst[0] {
  0, 7,                                      // 30364:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30366:           check predicate 18
  OPC_Decode, 205, 70, 155, 2,               // 30368:           decode to WHILELT_2PXX_H using decoder 283
                                             // 30368:          }
  1, 0,                                      // 30373:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30375:           check predicate 18
  OPC_Decode, 157, 70, 155, 2,               // 30377:           decode to WHILELE_2PXX_H using decoder 283
                                             // 30377:          }
                                             // 30377:          } // switch Inst[0]
                                             // 30377:         }
  1, 0,                                      // 30382:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30384:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30386:          check Inst[20:16] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 30390:          check Inst[9] == 0x0
  OPC_Decode, 249, 43, 157, 2,               // 30394:          decode to PEXT_2PCI_H using decoder 285
                                             // 30394:         }
                                             // 30394:         } // switch Inst[13]
                                             // 30394:        }
  2, 43,                                     // 30399:        case 0x2: {
  OPC_SwitchField, 13, 1,                    // 30401:         switch Inst[13] {
  0, 21,                                     // 30404:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30406:          switch Inst[0] {
  0, 7,                                      // 30409:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30411:           check predicate 18
  OPC_Decode, 206, 70, 155, 2,               // 30413:           decode to WHILELT_2PXX_S using decoder 283
                                             // 30413:          }
  1, 0,                                      // 30418:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30420:           check predicate 18
  OPC_Decode, 158, 70, 155, 2,               // 30422:           decode to WHILELE_2PXX_S using decoder 283
                                             // 30422:          }
                                             // 30422:          } // switch Inst[0]
                                             // 30422:         }
  1, 0,                                      // 30427:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30429:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30431:          check Inst[20:16] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 30435:          check Inst[9] == 0x0
  OPC_Decode, 250, 43, 157, 2,               // 30439:          decode to PEXT_2PCI_S using decoder 285
                                             // 30439:         }
                                             // 30439:         } // switch Inst[13]
                                             // 30439:        }
  3, 0,                                      // 30444:        case 0x3: {
  OPC_SwitchField, 13, 1,                    // 30446:         switch Inst[13] {
  0, 21,                                     // 30449:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30451:          switch Inst[0] {
  0, 7,                                      // 30454:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30456:           check predicate 18
  OPC_Decode, 204, 70, 155, 2,               // 30458:           decode to WHILELT_2PXX_D using decoder 283
                                             // 30458:          }
  1, 0,                                      // 30463:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30465:           check predicate 18
  OPC_Decode, 156, 70, 155, 2,               // 30467:           decode to WHILELE_2PXX_D using decoder 283
                                             // 30467:          }
                                             // 30467:          } // switch Inst[0]
                                             // 30467:         }
  1, 0,                                      // 30472:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30474:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30476:          check Inst[20:16] == 0x0
  OPC_CheckField, 9, 1, 0,                   // 30480:          check Inst[9] == 0x0
  OPC_Decode, 248, 43, 157, 2,               // 30484:          decode to PEXT_2PCI_D using decoder 285
                                             // 30484:         }
                                             // 30484:         } // switch Inst[13]
                                             // 30484:        }
                                             // 30484:        } // switch Inst[23:22]
                                             // 30484:       }
  6, 199, 1,                                 // 30489:       case 0x6: {
  OPC_SwitchField, 22, 2,                    // 30492:        switch Inst[23:22] {
  0, 47,                                     // 30495:        case 0x0: {
  OPC_SwitchField, 13, 1,                    // 30497:         switch Inst[13] {
  0, 21,                                     // 30500:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30502:          switch Inst[0] {
  0, 7,                                      // 30505:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30507:           check predicate 18
  OPC_Decode, 139, 70, 155, 2,               // 30509:           decode to WHILEHS_2PXX_B using decoder 283
                                             // 30509:          }
  1, 0,                                      // 30514:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30516:           check predicate 18
  OPC_Decode, 251, 69, 155, 2,               // 30518:           decode to WHILEHI_2PXX_B using decoder 283
                                             // 30518:          }
                                             // 30518:          } // switch Inst[0]
                                             // 30518:         }
  1, 0,                                      // 30523:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30525:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30527:          check Inst[20:16] == 0x0
  OPC_CheckField, 5, 5, 0,                   // 30531:          check Inst[9:5] == 0x0
  OPC_CheckField, 3, 1, 0,                   // 30535:          check Inst[3] == 0x0
  OPC_Decode, 207, 44, 158, 2,               // 30539:          decode to PTRUE_C_B using decoder 286
                                             // 30539:         }
                                             // 30539:         } // switch Inst[13]
                                             // 30539:        }
  1, 47,                                     // 30544:        case 0x1: {
  OPC_SwitchField, 13, 1,                    // 30546:         switch Inst[13] {
  0, 21,                                     // 30549:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30551:          switch Inst[0] {
  0, 7,                                      // 30554:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30556:           check predicate 18
  OPC_Decode, 141, 70, 155, 2,               // 30558:           decode to WHILEHS_2PXX_H using decoder 283
                                             // 30558:          }
  1, 0,                                      // 30563:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30565:           check predicate 18
  OPC_Decode, 253, 69, 155, 2,               // 30567:           decode to WHILEHI_2PXX_H using decoder 283
                                             // 30567:          }
                                             // 30567:          } // switch Inst[0]
                                             // 30567:         }
  1, 0,                                      // 30572:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30574:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30576:          check Inst[20:16] == 0x0
  OPC_CheckField, 5, 5, 0,                   // 30580:          check Inst[9:5] == 0x0
  OPC_CheckField, 3, 1, 0,                   // 30584:          check Inst[3] == 0x0
  OPC_Decode, 209, 44, 158, 2,               // 30588:          decode to PTRUE_C_H using decoder 286
                                             // 30588:         }
                                             // 30588:         } // switch Inst[13]
                                             // 30588:        }
  2, 47,                                     // 30593:        case 0x2: {
  OPC_SwitchField, 13, 1,                    // 30595:         switch Inst[13] {
  0, 21,                                     // 30598:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30600:          switch Inst[0] {
  0, 7,                                      // 30603:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30605:           check predicate 18
  OPC_Decode, 142, 70, 155, 2,               // 30607:           decode to WHILEHS_2PXX_S using decoder 283
                                             // 30607:          }
  1, 0,                                      // 30612:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30614:           check predicate 18
  OPC_Decode, 254, 69, 155, 2,               // 30616:           decode to WHILEHI_2PXX_S using decoder 283
                                             // 30616:          }
                                             // 30616:          } // switch Inst[0]
                                             // 30616:         }
  1, 0,                                      // 30621:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30623:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30625:          check Inst[20:16] == 0x0
  OPC_CheckField, 5, 5, 0,                   // 30629:          check Inst[9:5] == 0x0
  OPC_CheckField, 3, 1, 0,                   // 30633:          check Inst[3] == 0x0
  OPC_Decode, 210, 44, 158, 2,               // 30637:          decode to PTRUE_C_S using decoder 286
                                             // 30637:         }
                                             // 30637:         } // switch Inst[13]
                                             // 30637:        }
  3, 0,                                      // 30642:        case 0x3: {
  OPC_SwitchField, 13, 1,                    // 30644:         switch Inst[13] {
  0, 21,                                     // 30647:         case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30649:          switch Inst[0] {
  0, 7,                                      // 30652:          case 0x0: {
  OPC_CheckPredicate, 18,                    // 30654:           check predicate 18
  OPC_Decode, 140, 70, 155, 2,               // 30656:           decode to WHILEHS_2PXX_D using decoder 283
                                             // 30656:          }
  1, 0,                                      // 30661:          case 0x1: {
  OPC_CheckPredicate, 18,                    // 30663:           check predicate 18
  OPC_Decode, 252, 69, 155, 2,               // 30665:           decode to WHILEHI_2PXX_D using decoder 283
                                             // 30665:          }
                                             // 30665:          } // switch Inst[0]
                                             // 30665:         }
  1, 0,                                      // 30670:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30672:          check predicate 18
  OPC_CheckField, 16, 5, 0,                  // 30674:          check Inst[20:16] == 0x0
  OPC_CheckField, 5, 5, 0,                   // 30678:          check Inst[9:5] == 0x0
  OPC_CheckField, 3, 1, 0,                   // 30682:          check Inst[3] == 0x0
  OPC_Decode, 208, 44, 158, 2,               // 30686:          decode to PTRUE_C_D using decoder 286
                                             // 30686:         }
                                             // 30686:         } // switch Inst[13]
                                             // 30686:        }
                                             // 30686:        } // switch Inst[23:22]
                                             // 30686:       }
  7, 0,                                      // 30691:       case 0x7: {
  OPC_SwitchField, 22, 2,                    // 30693:        switch Inst[23:22] {
  0, 29,                                     // 30696:        case 0x0: {
  OPC_SwitchField, 0, 1,                     // 30698:         switch Inst[0] {
  0, 11,                                     // 30701:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30703:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30705:          check Inst[13] == 0x0
  OPC_Decode, 171, 70, 155, 2,               // 30709:          decode to WHILELO_2PXX_B using decoder 283
                                             // 30709:         }
  1, 0,                                      // 30714:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30716:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30718:          check Inst[13] == 0x0
  OPC_Decode, 187, 70, 155, 2,               // 30722:          decode to WHILELS_2PXX_B using decoder 283
                                             // 30722:         }
                                             // 30722:         } // switch Inst[0]
                                             // 30722:        }
  1, 29,                                     // 30727:        case 0x1: {
  OPC_SwitchField, 0, 1,                     // 30729:         switch Inst[0] {
  0, 11,                                     // 30732:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30734:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30736:          check Inst[13] == 0x0
  OPC_Decode, 173, 70, 155, 2,               // 30740:          decode to WHILELO_2PXX_H using decoder 283
                                             // 30740:         }
  1, 0,                                      // 30745:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30747:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30749:          check Inst[13] == 0x0
  OPC_Decode, 189, 70, 155, 2,               // 30753:          decode to WHILELS_2PXX_H using decoder 283
                                             // 30753:         }
                                             // 30753:         } // switch Inst[0]
                                             // 30753:        }
  2, 29,                                     // 30758:        case 0x2: {
  OPC_SwitchField, 0, 1,                     // 30760:         switch Inst[0] {
  0, 11,                                     // 30763:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30765:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30767:          check Inst[13] == 0x0
  OPC_Decode, 174, 70, 155, 2,               // 30771:          decode to WHILELO_2PXX_S using decoder 283
                                             // 30771:         }
  1, 0,                                      // 30776:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30778:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30780:          check Inst[13] == 0x0
  OPC_Decode, 190, 70, 155, 2,               // 30784:          decode to WHILELS_2PXX_S using decoder 283
                                             // 30784:         }
                                             // 30784:         } // switch Inst[0]
                                             // 30784:        }
  3, 0,                                      // 30789:        case 0x3: {
  OPC_SwitchField, 0, 1,                     // 30791:         switch Inst[0] {
  0, 11,                                     // 30794:         case 0x0: {
  OPC_CheckPredicate, 18,                    // 30796:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30798:          check Inst[13] == 0x0
  OPC_Decode, 172, 70, 155, 2,               // 30802:          decode to WHILELO_2PXX_D using decoder 283
                                             // 30802:         }
  1, 0,                                      // 30807:         case 0x1: {
  OPC_CheckPredicate, 18,                    // 30809:          check predicate 18
  OPC_CheckField, 13, 1, 0,                  // 30811:          check Inst[13] == 0x0
  OPC_Decode, 188, 70, 155, 2,               // 30815:          decode to WHILELS_2PXX_D using decoder 283
                                             // 30815:         }
                                             // 30815:         } // switch Inst[0]
                                             // 30815:        }
                                             // 30815:        } // switch Inst[23:22]
                                             // 30815:       }
                                             // 30815:       } // switch Inst[12:10]
                                             // 30815:      }
                                             // 30815:      } // switch Inst[4]
                                             // 30815:     }
  2, 159, 7,                                 // 30820:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 30823:      switch Inst[20:16] {
  0, 111,                                    // 30826:      case 0x0: {
  OPC_SwitchField, 22, 2,                    // 30828:       switch Inst[23:22] {
  0, 25,                                     // 30831:       case 0x0: {
  OPC_SwitchField, 9, 1,                     // 30833:        switch Inst[9] {
  0, 7,                                      // 30836:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 30838:         check predicate 33
  OPC_Decode, 154, 19, 159, 2,               // 30840:         decode to CNTP_XPP_B using decoder 287
                                             // 30840:        }
  1, 0,                                      // 30845:        case 0x1: {
  OPC_CheckPredicate, 18,                    // 30847:         check predicate 18
  OPC_CheckField, 11, 3, 0,                  // 30849:         check Inst[13:11] == 0x0
  OPC_Decode, 150, 19, 160, 2,               // 30853:         decode to CNTP_XCI_B using decoder 288
                                             // 30853:        }
                                             // 30853:        } // switch Inst[9]
                                             // 30853:       }
  1, 25,                                     // 30858:       case 0x1: {
  OPC_SwitchField, 9, 1,                     // 30860:        switch Inst[9] {
  0, 7,                                      // 30863:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 30865:         check predicate 33
  OPC_Decode, 156, 19, 159, 2,               // 30867:         decode to CNTP_XPP_H using decoder 287
                                             // 30867:        }
  1, 0,                                      // 30872:        case 0x1: {
  OPC_CheckPredicate, 18,                    // 30874:         check predicate 18
  OPC_CheckField, 11, 3, 0,                  // 30876:         check Inst[13:11] == 0x0
  OPC_Decode, 152, 19, 160, 2,               // 30880:         decode to CNTP_XCI_H using decoder 288
                                             // 30880:        }
                                             // 30880:        } // switch Inst[9]
                                             // 30880:       }
  2, 25,                                     // 30885:       case 0x2: {
  OPC_SwitchField, 9, 1,                     // 30887:        switch Inst[9] {
  0, 7,                                      // 30890:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 30892:         check predicate 33
  OPC_Decode, 157, 19, 159, 2,               // 30894:         decode to CNTP_XPP_S using decoder 287
                                             // 30894:        }
  1, 0,                                      // 30899:        case 0x1: {
  OPC_CheckPredicate, 18,                    // 30901:         check predicate 18
  OPC_CheckField, 11, 3, 0,                  // 30903:         check Inst[13:11] == 0x0
  OPC_Decode, 153, 19, 160, 2,               // 30907:         decode to CNTP_XCI_S using decoder 288
                                             // 30907:        }
                                             // 30907:        } // switch Inst[9]
                                             // 30907:       }
  3, 0,                                      // 30912:       case 0x3: {
  OPC_SwitchField, 9, 1,                     // 30914:        switch Inst[9] {
  0, 7,                                      // 30917:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 30919:         check predicate 33
  OPC_Decode, 155, 19, 159, 2,               // 30921:         decode to CNTP_XPP_D using decoder 287
                                             // 30921:        }
  1, 0,                                      // 30926:        case 0x1: {
  OPC_CheckPredicate, 18,                    // 30928:         check predicate 18
  OPC_CheckField, 11, 3, 0,                  // 30930:         check Inst[13:11] == 0x0
  OPC_Decode, 151, 19, 160, 2,               // 30934:         decode to CNTP_XCI_D using decoder 288
                                             // 30934:        }
                                             // 30934:        } // switch Inst[9]
                                             // 30934:       }
                                             // 30934:       } // switch Inst[23:22]
                                             // 30934:      }
  1, 55,                                     // 30939:      case 0x1: {
  OPC_SwitchField, 22, 2,                    // 30941:       switch Inst[23:22] {
  0, 11,                                     // 30944:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 30946:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 30948:        check Inst[9] == 0x0
  OPC_Decode, 166, 26, 159, 2,               // 30952:        decode to FIRSTP_XPP_B using decoder 287
                                             // 30952:       }
  1, 11,                                     // 30957:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 30959:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 30961:        check Inst[9] == 0x0
  OPC_Decode, 168, 26, 159, 2,               // 30965:        decode to FIRSTP_XPP_H using decoder 287
                                             // 30965:       }
  2, 11,                                     // 30970:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 30972:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 30974:        check Inst[9] == 0x0
  OPC_Decode, 169, 26, 159, 2,               // 30978:        decode to FIRSTP_XPP_S using decoder 287
                                             // 30978:       }
  3, 0,                                      // 30983:       case 0x3: {
  OPC_CheckPredicate, 36,                    // 30985:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 30987:        check Inst[9] == 0x0
  OPC_Decode, 167, 26, 159, 2,               // 30991:        decode to FIRSTP_XPP_D using decoder 287
                                             // 30991:       }
                                             // 30991:       } // switch Inst[23:22]
                                             // 30991:      }
  2, 55,                                     // 30996:      case 0x2: {
  OPC_SwitchField, 22, 2,                    // 30998:       switch Inst[23:22] {
  0, 11,                                     // 31001:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 31003:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 31005:        check Inst[9] == 0x0
  OPC_Decode, 173, 34, 159, 2,               // 31009:        decode to LASTP_XPP_B using decoder 287
                                             // 31009:       }
  1, 11,                                     // 31014:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 31016:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 31018:        check Inst[9] == 0x0
  OPC_Decode, 175, 34, 159, 2,               // 31022:        decode to LASTP_XPP_H using decoder 287
                                             // 31022:       }
  2, 11,                                     // 31027:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 31029:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 31031:        check Inst[9] == 0x0
  OPC_Decode, 176, 34, 159, 2,               // 31035:        decode to LASTP_XPP_S using decoder 287
                                             // 31035:       }
  3, 0,                                      // 31040:       case 0x3: {
  OPC_CheckPredicate, 36,                    // 31042:        check predicate 36
  OPC_CheckField, 9, 1, 0,                   // 31044:        check Inst[9] == 0x0
  OPC_Decode, 174, 34, 159, 2,               // 31048:        decode to LASTP_XPP_D using decoder 287
                                             // 31048:       }
                                             // 31048:       } // switch Inst[23:22]
                                             // 31048:      }
  8, 134, 1,                                 // 31053:      case 0x8: {
  OPC_SwitchField, 9, 5,                     // 31056:       switch Inst[13:9] {
  0, 30,                                     // 31059:       case 0x0: {
  OPC_SwitchField, 22, 2,                    // 31061:        switch Inst[23:22] {
  1, 7,                                      // 31064:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 31066:         check predicate 33
  OPC_Decode, 195, 52, 161, 2,               // 31068:         decode to SQINCP_ZP_H using decoder 289
                                             // 31068:        }
  2, 7,                                      // 31073:        case 0x2: {
  OPC_CheckPredicate, 33,                    // 31075:         check predicate 33
  OPC_Decode, 196, 52, 161, 2,               // 31077:         decode to SQINCP_ZP_S using decoder 289
                                             // 31077:        }
  3, 0,                                      // 31082:        case 0x3: {
  OPC_CheckPredicate, 33,                    // 31084:         check predicate 33
  OPC_Decode, 194, 52, 161, 2,               // 31086:         decode to SQINCP_ZP_D using decoder 289
                                             // 31086:        }
                                             // 31086:        } // switch Inst[23:22]
                                             // 31086:       }
  4, 39,                                     // 31091:       case 0x4: {
  OPC_SwitchField, 22, 2,                    // 31093:        switch Inst[23:22] {
  0, 7,                                      // 31096:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31098:         check predicate 33
  OPC_Decode, 186, 52, 162, 2,               // 31100:         decode to SQINCP_XPWd_B using decoder 290
                                             // 31100:        }
  1, 7,                                      // 31105:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 31107:         check predicate 33
  OPC_Decode, 188, 52, 162, 2,               // 31109:         decode to SQINCP_XPWd_H using decoder 290
                                             // 31109:        }
  2, 7,                                      // 31114:        case 0x2: {
  OPC_CheckPredicate, 33,                    // 31116:         check predicate 33
  OPC_Decode, 189, 52, 162, 2,               // 31118:         decode to SQINCP_XPWd_S using decoder 290
                                             // 31118:        }
  3, 0,                                      // 31123:        case 0x3: {
  OPC_CheckPredicate, 33,                    // 31125:         check predicate 33
  OPC_Decode, 187, 52, 162, 2,               // 31127:         decode to SQINCP_XPWd_D using decoder 290
                                             // 31127:        }
                                             // 31127:        } // switch Inst[23:22]
                                             // 31127:       }
  6, 39,                                     // 31132:       case 0x6: {
  OPC_SwitchField, 22, 2,                    // 31134:        switch Inst[23:22] {
  0, 7,                                      // 31137:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31139:         check predicate 33
  OPC_Decode, 190, 52, 162, 2,               // 31141:         decode to SQINCP_XP_B using decoder 290
                                             // 31141:        }
  1, 7,                                      // 31146:        case 0x1: {
  OPC_CheckPredicate, 33,                    // 31148:         check predicate 33
  OPC_Decode, 192, 52, 162, 2,               // 31150:         decode to SQINCP_XP_H using decoder 290
                                             // 31150:        }
  2, 7,                                      // 31155:        case 0x2: {
  OPC_CheckPredicate, 33,                    // 31157:         check predicate 33
  OPC_Decode, 193, 52, 162, 2,               // 31159:         decode to SQINCP_XP_S using decoder 290
                                             // 31159:        }
  3, 0,                                      // 31164:        case 0x3: {
  OPC_CheckPredicate, 33,                    // 31166:         check predicate 33
  OPC_Decode, 191, 52, 162, 2,               // 31168:         decode to SQINCP_XP_D using decoder 290
                                             // 31168:        }
                                             // 31168:        } // switch Inst[23:22]
                                             // 31168:       }
  8, 0,                                      // 31173:       case 0x8: {
  OPC_CheckPredicate, 40,                    // 31175:        check predicate 40
  OPC_CheckField, 22, 2, 0,                  // 31177:        check Inst[23:22] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 31181:        check Inst[4:0] == 0x0
  OPC_Decode, 227, 70, 163, 2,               // 31185:        decode to WRFFR using decoder 291
                                             // 31185:       }
                                             // 31185:       } // switch Inst[13:9]
                                             // 31185:      }
  9, 122,                                    // 31190:      case 0x9: {
  OPC_SwitchField, 22, 2,                    // 31192:       switch Inst[23:22] {
  0, 21,                                     // 31195:       case 0x0: {
  OPC_SwitchField, 9, 5,                     // 31197:        switch Inst[13:9] {
  4, 7,                                      // 31200:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31202:         check predicate 33
  OPC_Decode, 177, 66, 164, 2,               // 31204:         decode to UQINCP_WP_B using decoder 292
                                             // 31204:        }
  6, 0,                                      // 31209:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31211:         check predicate 33
  OPC_Decode, 181, 66, 162, 2,               // 31213:         decode to UQINCP_XP_B using decoder 290
                                             // 31213:        }
                                             // 31213:        } // switch Inst[13:9]
                                             // 31213:       }
  1, 30,                                     // 31218:       case 0x1: {
  OPC_SwitchField, 9, 5,                     // 31220:        switch Inst[13:9] {
  0, 7,                                      // 31223:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31225:         check predicate 33
  OPC_Decode, 186, 66, 161, 2,               // 31227:         decode to UQINCP_ZP_H using decoder 289
                                             // 31227:        }
  4, 7,                                      // 31232:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31234:         check predicate 33
  OPC_Decode, 179, 66, 164, 2,               // 31236:         decode to UQINCP_WP_H using decoder 292
                                             // 31236:        }
  6, 0,                                      // 31241:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31243:         check predicate 33
  OPC_Decode, 183, 66, 162, 2,               // 31245:         decode to UQINCP_XP_H using decoder 290
                                             // 31245:        }
                                             // 31245:        } // switch Inst[13:9]
                                             // 31245:       }
  2, 30,                                     // 31250:       case 0x2: {
  OPC_SwitchField, 9, 5,                     // 31252:        switch Inst[13:9] {
  0, 7,                                      // 31255:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31257:         check predicate 33
  OPC_Decode, 187, 66, 161, 2,               // 31259:         decode to UQINCP_ZP_S using decoder 289
                                             // 31259:        }
  4, 7,                                      // 31264:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31266:         check predicate 33
  OPC_Decode, 180, 66, 164, 2,               // 31268:         decode to UQINCP_WP_S using decoder 292
                                             // 31268:        }
  6, 0,                                      // 31273:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31275:         check predicate 33
  OPC_Decode, 184, 66, 162, 2,               // 31277:         decode to UQINCP_XP_S using decoder 290
                                             // 31277:        }
                                             // 31277:        } // switch Inst[13:9]
                                             // 31277:       }
  3, 0,                                      // 31282:       case 0x3: {
  OPC_SwitchField, 9, 5,                     // 31284:        switch Inst[13:9] {
  0, 7,                                      // 31287:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31289:         check predicate 33
  OPC_Decode, 185, 66, 161, 2,               // 31291:         decode to UQINCP_ZP_D using decoder 289
                                             // 31291:        }
  4, 7,                                      // 31296:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31298:         check predicate 33
  OPC_Decode, 178, 66, 164, 2,               // 31300:         decode to UQINCP_WP_D using decoder 292
                                             // 31300:        }
  6, 0,                                      // 31305:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31307:         check predicate 33
  OPC_Decode, 182, 66, 162, 2,               // 31309:         decode to UQINCP_XP_D using decoder 290
                                             // 31309:        }
                                             // 31309:        } // switch Inst[13:9]
                                             // 31309:       }
                                             // 31309:       } // switch Inst[23:22]
                                             // 31309:      }
  10, 122,                                   // 31314:      case 0xa: {
  OPC_SwitchField, 22, 2,                    // 31316:       switch Inst[23:22] {
  0, 21,                                     // 31319:       case 0x0: {
  OPC_SwitchField, 9, 5,                     // 31321:        switch Inst[13:9] {
  4, 7,                                      // 31324:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31326:         check predicate 33
  OPC_Decode, 185, 51, 162, 2,               // 31328:         decode to SQDECP_XPWd_B using decoder 290
                                             // 31328:        }
  6, 0,                                      // 31333:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31335:         check predicate 33
  OPC_Decode, 189, 51, 162, 2,               // 31337:         decode to SQDECP_XP_B using decoder 290
                                             // 31337:        }
                                             // 31337:        } // switch Inst[13:9]
                                             // 31337:       }
  1, 30,                                     // 31342:       case 0x1: {
  OPC_SwitchField, 9, 5,                     // 31344:        switch Inst[13:9] {
  0, 7,                                      // 31347:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31349:         check predicate 33
  OPC_Decode, 194, 51, 161, 2,               // 31351:         decode to SQDECP_ZP_H using decoder 289
                                             // 31351:        }
  4, 7,                                      // 31356:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31358:         check predicate 33
  OPC_Decode, 187, 51, 162, 2,               // 31360:         decode to SQDECP_XPWd_H using decoder 290
                                             // 31360:        }
  6, 0,                                      // 31365:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31367:         check predicate 33
  OPC_Decode, 191, 51, 162, 2,               // 31369:         decode to SQDECP_XP_H using decoder 290
                                             // 31369:        }
                                             // 31369:        } // switch Inst[13:9]
                                             // 31369:       }
  2, 30,                                     // 31374:       case 0x2: {
  OPC_SwitchField, 9, 5,                     // 31376:        switch Inst[13:9] {
  0, 7,                                      // 31379:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31381:         check predicate 33
  OPC_Decode, 195, 51, 161, 2,               // 31383:         decode to SQDECP_ZP_S using decoder 289
                                             // 31383:        }
  4, 7,                                      // 31388:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31390:         check predicate 33
  OPC_Decode, 188, 51, 162, 2,               // 31392:         decode to SQDECP_XPWd_S using decoder 290
                                             // 31392:        }
  6, 0,                                      // 31397:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31399:         check predicate 33
  OPC_Decode, 192, 51, 162, 2,               // 31401:         decode to SQDECP_XP_S using decoder 290
                                             // 31401:        }
                                             // 31401:        } // switch Inst[13:9]
                                             // 31401:       }
  3, 0,                                      // 31406:       case 0x3: {
  OPC_SwitchField, 9, 5,                     // 31408:        switch Inst[13:9] {
  0, 7,                                      // 31411:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31413:         check predicate 33
  OPC_Decode, 193, 51, 161, 2,               // 31415:         decode to SQDECP_ZP_D using decoder 289
                                             // 31415:        }
  4, 7,                                      // 31420:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31422:         check predicate 33
  OPC_Decode, 186, 51, 162, 2,               // 31424:         decode to SQDECP_XPWd_D using decoder 290
                                             // 31424:        }
  6, 0,                                      // 31429:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31431:         check predicate 33
  OPC_Decode, 190, 51, 162, 2,               // 31433:         decode to SQDECP_XP_D using decoder 290
                                             // 31433:        }
                                             // 31433:        } // switch Inst[13:9]
                                             // 31433:       }
                                             // 31433:       } // switch Inst[23:22]
                                             // 31433:      }
  11, 122,                                   // 31438:      case 0xb: {
  OPC_SwitchField, 22, 2,                    // 31440:       switch Inst[23:22] {
  0, 21,                                     // 31443:       case 0x0: {
  OPC_SwitchField, 9, 5,                     // 31445:        switch Inst[13:9] {
  4, 7,                                      // 31448:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31450:         check predicate 33
  OPC_Decode, 155, 66, 164, 2,               // 31452:         decode to UQDECP_WP_B using decoder 292
                                             // 31452:        }
  6, 0,                                      // 31457:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31459:         check predicate 33
  OPC_Decode, 159, 66, 162, 2,               // 31461:         decode to UQDECP_XP_B using decoder 290
                                             // 31461:        }
                                             // 31461:        } // switch Inst[13:9]
                                             // 31461:       }
  1, 30,                                     // 31466:       case 0x1: {
  OPC_SwitchField, 9, 5,                     // 31468:        switch Inst[13:9] {
  0, 7,                                      // 31471:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31473:         check predicate 33
  OPC_Decode, 164, 66, 161, 2,               // 31475:         decode to UQDECP_ZP_H using decoder 289
                                             // 31475:        }
  4, 7,                                      // 31480:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31482:         check predicate 33
  OPC_Decode, 157, 66, 164, 2,               // 31484:         decode to UQDECP_WP_H using decoder 292
                                             // 31484:        }
  6, 0,                                      // 31489:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31491:         check predicate 33
  OPC_Decode, 161, 66, 162, 2,               // 31493:         decode to UQDECP_XP_H using decoder 290
                                             // 31493:        }
                                             // 31493:        } // switch Inst[13:9]
                                             // 31493:       }
  2, 30,                                     // 31498:       case 0x2: {
  OPC_SwitchField, 9, 5,                     // 31500:        switch Inst[13:9] {
  0, 7,                                      // 31503:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31505:         check predicate 33
  OPC_Decode, 165, 66, 161, 2,               // 31507:         decode to UQDECP_ZP_S using decoder 289
                                             // 31507:        }
  4, 7,                                      // 31512:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31514:         check predicate 33
  OPC_Decode, 158, 66, 164, 2,               // 31516:         decode to UQDECP_WP_S using decoder 292
                                             // 31516:        }
  6, 0,                                      // 31521:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31523:         check predicate 33
  OPC_Decode, 162, 66, 162, 2,               // 31525:         decode to UQDECP_XP_S using decoder 290
                                             // 31525:        }
                                             // 31525:        } // switch Inst[13:9]
                                             // 31525:       }
  3, 0,                                      // 31530:       case 0x3: {
  OPC_SwitchField, 9, 5,                     // 31532:        switch Inst[13:9] {
  0, 7,                                      // 31535:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31537:         check predicate 33
  OPC_Decode, 163, 66, 161, 2,               // 31539:         decode to UQDECP_ZP_D using decoder 289
                                             // 31539:        }
  4, 7,                                      // 31544:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31546:         check predicate 33
  OPC_Decode, 156, 66, 164, 2,               // 31548:         decode to UQDECP_WP_D using decoder 292
                                             // 31548:        }
  6, 0,                                      // 31553:        case 0x6: {
  OPC_CheckPredicate, 33,                    // 31555:         check predicate 33
  OPC_Decode, 160, 66, 162, 2,               // 31557:         decode to UQDECP_XP_D using decoder 290
                                             // 31557:        }
                                             // 31557:        } // switch Inst[13:9]
                                             // 31557:       }
                                             // 31557:       } // switch Inst[23:22]
                                             // 31557:      }
  12, 99,                                    // 31562:      case 0xc: {
  OPC_SwitchField, 22, 2,                    // 31564:       switch Inst[23:22] {
  0, 25,                                     // 31567:       case 0x0: {
  OPC_SwitchField, 9, 5,                     // 31569:        switch Inst[13:9] {
  4, 7,                                      // 31572:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31574:         check predicate 33
  OPC_Decode, 232, 33, 162, 2,               // 31576:         decode to INCP_XP_B using decoder 290
                                             // 31576:        }
  8, 0,                                      // 31581:        case 0x8: {
  OPC_CheckPredicate, 40,                    // 31583:         check predicate 40
  OPC_CheckField, 0, 9, 0,                   // 31585:         check Inst[8:0] == 0x0
  OPC_Decode, 230, 47, 165, 2,               // 31589:         decode to SETFFR using decoder 293
                                             // 31589:        }
                                             // 31589:        } // switch Inst[13:9]
                                             // 31589:       }
  1, 21,                                     // 31594:       case 0x1: {
  OPC_SwitchField, 9, 5,                     // 31596:        switch Inst[13:9] {
  0, 7,                                      // 31599:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31601:         check predicate 33
  OPC_Decode, 237, 33, 161, 2,               // 31603:         decode to INCP_ZP_H using decoder 289
                                             // 31603:        }
  4, 0,                                      // 31608:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31610:         check predicate 33
  OPC_Decode, 234, 33, 162, 2,               // 31612:         decode to INCP_XP_H using decoder 290
                                             // 31612:        }
                                             // 31612:        } // switch Inst[13:9]
                                             // 31612:       }
  2, 21,                                     // 31617:       case 0x2: {
  OPC_SwitchField, 9, 5,                     // 31619:        switch Inst[13:9] {
  0, 7,                                      // 31622:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31624:         check predicate 33
  OPC_Decode, 238, 33, 161, 2,               // 31626:         decode to INCP_ZP_S using decoder 289
                                             // 31626:        }
  4, 0,                                      // 31631:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31633:         check predicate 33
  OPC_Decode, 235, 33, 162, 2,               // 31635:         decode to INCP_XP_S using decoder 290
                                             // 31635:        }
                                             // 31635:        } // switch Inst[13:9]
                                             // 31635:       }
  3, 0,                                      // 31640:       case 0x3: {
  OPC_SwitchField, 9, 5,                     // 31642:        switch Inst[13:9] {
  0, 7,                                      // 31645:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31647:         check predicate 33
  OPC_Decode, 236, 33, 161, 2,               // 31649:         decode to INCP_ZP_D using decoder 289
                                             // 31649:        }
  4, 0,                                      // 31654:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31656:         check predicate 33
  OPC_Decode, 233, 33, 162, 2,               // 31658:         decode to INCP_XP_D using decoder 290
                                             // 31658:        }
                                             // 31658:        } // switch Inst[13:9]
                                             // 31658:       }
                                             // 31658:       } // switch Inst[23:22]
                                             // 31658:      }
  13, 0,                                     // 31663:      case 0xd: {
  OPC_SwitchField, 22, 2,                    // 31665:       switch Inst[23:22] {
  0, 11,                                     // 31668:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31670:        check predicate 33
  OPC_CheckField, 9, 5, 4,                   // 31672:        check Inst[13:9] == 0x4
  OPC_Decode, 189, 20, 162, 2,               // 31676:        decode to DECP_XP_B using decoder 290
                                             // 31676:       }
  1, 21,                                     // 31681:       case 0x1: {
  OPC_SwitchField, 9, 5,                     // 31683:        switch Inst[13:9] {
  0, 7,                                      // 31686:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31688:         check predicate 33
  OPC_Decode, 194, 20, 161, 2,               // 31690:         decode to DECP_ZP_H using decoder 289
                                             // 31690:        }
  4, 0,                                      // 31695:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31697:         check predicate 33
  OPC_Decode, 191, 20, 162, 2,               // 31699:         decode to DECP_XP_H using decoder 290
                                             // 31699:        }
                                             // 31699:        } // switch Inst[13:9]
                                             // 31699:       }
  2, 21,                                     // 31704:       case 0x2: {
  OPC_SwitchField, 9, 5,                     // 31706:        switch Inst[13:9] {
  0, 7,                                      // 31709:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31711:         check predicate 33
  OPC_Decode, 195, 20, 161, 2,               // 31713:         decode to DECP_ZP_S using decoder 289
                                             // 31713:        }
  4, 0,                                      // 31718:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31720:         check predicate 33
  OPC_Decode, 192, 20, 162, 2,               // 31722:         decode to DECP_XP_S using decoder 290
                                             // 31722:        }
                                             // 31722:        } // switch Inst[13:9]
                                             // 31722:       }
  3, 0,                                      // 31727:       case 0x3: {
  OPC_SwitchField, 9, 5,                     // 31729:        switch Inst[13:9] {
  0, 7,                                      // 31732:        case 0x0: {
  OPC_CheckPredicate, 33,                    // 31734:         check predicate 33
  OPC_Decode, 193, 20, 161, 2,               // 31736:         decode to DECP_ZP_D using decoder 289
                                             // 31736:        }
  4, 0,                                      // 31741:        case 0x4: {
  OPC_CheckPredicate, 33,                    // 31743:         check predicate 33
  OPC_Decode, 190, 20, 162, 2,               // 31745:         decode to DECP_XP_D using decoder 290
                                             // 31745:        }
                                             // 31745:        } // switch Inst[13:9]
                                             // 31745:       }
                                             // 31745:       } // switch Inst[23:22]
                                             // 31745:      }
                                             // 31745:      } // switch Inst[20:16]
                                             // 31745:     }
  3, 0,                                      // 31750:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 31752:      switch Inst[20:16] {
  0, 39,                                     // 31755:      case 0x0: {
  OPC_SwitchField, 22, 2,                    // 31757:       switch Inst[23:22] {
  0, 7,                                      // 31760:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31762:        check predicate 33
  OPC_Decode, 219, 13, 166, 2,               // 31764:        decode to ADD_ZI_B using decoder 294
                                             // 31764:       }
  1, 7,                                      // 31769:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 31771:        check predicate 33
  OPC_Decode, 221, 13, 167, 2,               // 31773:        decode to ADD_ZI_H using decoder 295
                                             // 31773:       }
  2, 7,                                      // 31778:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 31780:        check predicate 33
  OPC_Decode, 222, 13, 168, 2,               // 31782:        decode to ADD_ZI_S using decoder 296
                                             // 31782:       }
  3, 0,                                      // 31787:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 31789:        check predicate 33
  OPC_Decode, 220, 13, 169, 2,               // 31791:        decode to ADD_ZI_D using decoder 297
                                             // 31791:       }
                                             // 31791:       } // switch Inst[23:22]
                                             // 31791:      }
  1, 39,                                     // 31796:      case 0x1: {
  OPC_SwitchField, 22, 2,                    // 31798:       switch Inst[23:22] {
  0, 7,                                      // 31801:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31803:        check predicate 33
  OPC_Decode, 178, 60, 166, 2,               // 31805:        decode to SUB_ZI_B using decoder 294
                                             // 31805:       }
  1, 7,                                      // 31810:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 31812:        check predicate 33
  OPC_Decode, 180, 60, 167, 2,               // 31814:        decode to SUB_ZI_H using decoder 295
                                             // 31814:       }
  2, 7,                                      // 31819:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 31821:        check predicate 33
  OPC_Decode, 181, 60, 168, 2,               // 31823:        decode to SUB_ZI_S using decoder 296
                                             // 31823:       }
  3, 0,                                      // 31828:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 31830:        check predicate 33
  OPC_Decode, 179, 60, 169, 2,               // 31832:        decode to SUB_ZI_D using decoder 297
                                             // 31832:       }
                                             // 31832:       } // switch Inst[23:22]
                                             // 31832:      }
  3, 39,                                     // 31837:      case 0x3: {
  OPC_SwitchField, 22, 2,                    // 31839:       switch Inst[23:22] {
  0, 7,                                      // 31842:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31844:        check predicate 33
  OPC_Decode, 144, 60, 166, 2,               // 31846:        decode to SUBR_ZI_B using decoder 294
                                             // 31846:       }
  1, 7,                                      // 31851:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 31853:        check predicate 33
  OPC_Decode, 146, 60, 167, 2,               // 31855:        decode to SUBR_ZI_H using decoder 295
                                             // 31855:       }
  2, 7,                                      // 31860:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 31862:        check predicate 33
  OPC_Decode, 147, 60, 168, 2,               // 31864:        decode to SUBR_ZI_S using decoder 296
                                             // 31864:       }
  3, 0,                                      // 31869:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 31871:        check predicate 33
  OPC_Decode, 145, 60, 169, 2,               // 31873:        decode to SUBR_ZI_D using decoder 297
                                             // 31873:       }
                                             // 31873:       } // switch Inst[23:22]
                                             // 31873:      }
  4, 39,                                     // 31878:      case 0x4: {
  OPC_SwitchField, 22, 2,                    // 31880:       switch Inst[23:22] {
  0, 7,                                      // 31883:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31885:        check predicate 33
  OPC_Decode, 138, 51, 166, 2,               // 31887:        decode to SQADD_ZI_B using decoder 294
                                             // 31887:       }
  1, 7,                                      // 31892:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 31894:        check predicate 33
  OPC_Decode, 140, 51, 167, 2,               // 31896:        decode to SQADD_ZI_H using decoder 295
                                             // 31896:       }
  2, 7,                                      // 31901:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 31903:        check predicate 33
  OPC_Decode, 141, 51, 168, 2,               // 31905:        decode to SQADD_ZI_S using decoder 296
                                             // 31905:       }
  3, 0,                                      // 31910:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 31912:        check predicate 33
  OPC_Decode, 139, 51, 169, 2,               // 31914:        decode to SQADD_ZI_D using decoder 297
                                             // 31914:       }
                                             // 31914:       } // switch Inst[23:22]
                                             // 31914:      }
  5, 39,                                     // 31919:      case 0x5: {
  OPC_SwitchField, 22, 2,                    // 31921:       switch Inst[23:22] {
  0, 7,                                      // 31924:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31926:        check predicate 33
  OPC_Decode, 246, 65, 166, 2,               // 31928:        decode to UQADD_ZI_B using decoder 294
                                             // 31928:       }
  1, 7,                                      // 31933:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 31935:        check predicate 33
  OPC_Decode, 248, 65, 167, 2,               // 31937:        decode to UQADD_ZI_H using decoder 295
                                             // 31937:       }
  2, 7,                                      // 31942:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 31944:        check predicate 33
  OPC_Decode, 249, 65, 168, 2,               // 31946:        decode to UQADD_ZI_S using decoder 296
                                             // 31946:       }
  3, 0,                                      // 31951:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 31953:        check predicate 33
  OPC_Decode, 247, 65, 169, 2,               // 31955:        decode to UQADD_ZI_D using decoder 297
                                             // 31955:       }
                                             // 31955:       } // switch Inst[23:22]
                                             // 31955:      }
  6, 39,                                     // 31960:      case 0x6: {
  OPC_SwitchField, 22, 2,                    // 31962:       switch Inst[23:22] {
  0, 7,                                      // 31965:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 31967:        check predicate 33
  OPC_Decode, 176, 54, 166, 2,               // 31969:        decode to SQSUB_ZI_B using decoder 294
                                             // 31969:       }
  1, 7,                                      // 31974:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 31976:        check predicate 33
  OPC_Decode, 178, 54, 167, 2,               // 31978:        decode to SQSUB_ZI_H using decoder 295
                                             // 31978:       }
  2, 7,                                      // 31983:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 31985:        check predicate 33
  OPC_Decode, 179, 54, 168, 2,               // 31987:        decode to SQSUB_ZI_S using decoder 296
                                             // 31987:       }
  3, 0,                                      // 31992:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 31994:        check predicate 33
  OPC_Decode, 177, 54, 169, 2,               // 31996:        decode to SQSUB_ZI_D using decoder 297
                                             // 31996:       }
                                             // 31996:       } // switch Inst[23:22]
                                             // 31996:      }
  7, 39,                                     // 32001:      case 0x7: {
  OPC_SwitchField, 22, 2,                    // 32003:       switch Inst[23:22] {
  0, 7,                                      // 32006:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32008:        check predicate 33
  OPC_Decode, 159, 67, 166, 2,               // 32010:        decode to UQSUB_ZI_B using decoder 294
                                             // 32010:       }
  1, 7,                                      // 32015:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32017:        check predicate 33
  OPC_Decode, 161, 67, 167, 2,               // 32019:        decode to UQSUB_ZI_H using decoder 295
                                             // 32019:       }
  2, 7,                                      // 32024:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32026:        check predicate 33
  OPC_Decode, 162, 67, 168, 2,               // 32028:        decode to UQSUB_ZI_S using decoder 296
                                             // 32028:       }
  3, 0,                                      // 32033:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32035:        check predicate 33
  OPC_Decode, 160, 67, 169, 2,               // 32037:        decode to UQSUB_ZI_D using decoder 297
                                             // 32037:       }
                                             // 32037:       } // switch Inst[23:22]
                                             // 32037:      }
  8, 55,                                     // 32042:      case 0x8: {
  OPC_SwitchField, 22, 2,                    // 32044:       switch Inst[23:22] {
  0, 11,                                     // 32047:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32049:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32051:        check Inst[13] == 0x0
  OPC_Decode, 135, 49, 170, 2,               // 32055:        decode to SMAX_ZI_B using decoder 298
                                             // 32055:       }
  1, 11,                                     // 32060:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32062:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32064:        check Inst[13] == 0x0
  OPC_Decode, 137, 49, 170, 2,               // 32068:        decode to SMAX_ZI_H using decoder 298
                                             // 32068:       }
  2, 11,                                     // 32073:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32075:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32077:        check Inst[13] == 0x0
  OPC_Decode, 138, 49, 170, 2,               // 32081:        decode to SMAX_ZI_S using decoder 298
                                             // 32081:       }
  3, 0,                                      // 32086:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32088:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32090:        check Inst[13] == 0x0
  OPC_Decode, 136, 49, 170, 2,               // 32094:        decode to SMAX_ZI_D using decoder 298
                                             // 32094:       }
                                             // 32094:       } // switch Inst[23:22]
                                             // 32094:      }
  9, 55,                                     // 32099:      case 0x9: {
  OPC_SwitchField, 22, 2,                    // 32101:       switch Inst[23:22] {
  0, 11,                                     // 32104:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32106:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32108:        check Inst[13] == 0x0
  OPC_Decode, 145, 64, 171, 2,               // 32112:        decode to UMAX_ZI_B using decoder 299
                                             // 32112:       }
  1, 11,                                     // 32117:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32119:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32121:        check Inst[13] == 0x0
  OPC_Decode, 147, 64, 171, 2,               // 32125:        decode to UMAX_ZI_H using decoder 299
                                             // 32125:       }
  2, 11,                                     // 32130:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32132:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32134:        check Inst[13] == 0x0
  OPC_Decode, 148, 64, 171, 2,               // 32138:        decode to UMAX_ZI_S using decoder 299
                                             // 32138:       }
  3, 0,                                      // 32143:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32145:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32147:        check Inst[13] == 0x0
  OPC_Decode, 146, 64, 171, 2,               // 32151:        decode to UMAX_ZI_D using decoder 299
                                             // 32151:       }
                                             // 32151:       } // switch Inst[23:22]
                                             // 32151:      }
  10, 55,                                    // 32156:      case 0xa: {
  OPC_SwitchField, 22, 2,                    // 32158:       switch Inst[23:22] {
  0, 11,                                     // 32161:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32163:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32165:        check Inst[13] == 0x0
  OPC_Decode, 193, 49, 170, 2,               // 32169:        decode to SMIN_ZI_B using decoder 298
                                             // 32169:       }
  1, 11,                                     // 32174:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32176:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32178:        check Inst[13] == 0x0
  OPC_Decode, 195, 49, 170, 2,               // 32182:        decode to SMIN_ZI_H using decoder 298
                                             // 32182:       }
  2, 11,                                     // 32187:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32189:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32191:        check Inst[13] == 0x0
  OPC_Decode, 196, 49, 170, 2,               // 32195:        decode to SMIN_ZI_S using decoder 298
                                             // 32195:       }
  3, 0,                                      // 32200:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32202:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32204:        check Inst[13] == 0x0
  OPC_Decode, 194, 49, 170, 2,               // 32208:        decode to SMIN_ZI_D using decoder 298
                                             // 32208:       }
                                             // 32208:       } // switch Inst[23:22]
                                             // 32208:      }
  11, 55,                                    // 32213:      case 0xb: {
  OPC_SwitchField, 22, 2,                    // 32215:       switch Inst[23:22] {
  0, 11,                                     // 32218:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32220:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32222:        check Inst[13] == 0x0
  OPC_Decode, 202, 64, 171, 2,               // 32226:        decode to UMIN_ZI_B using decoder 299
                                             // 32226:       }
  1, 11,                                     // 32231:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32233:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32235:        check Inst[13] == 0x0
  OPC_Decode, 204, 64, 171, 2,               // 32239:        decode to UMIN_ZI_H using decoder 299
                                             // 32239:       }
  2, 11,                                     // 32244:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32246:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32248:        check Inst[13] == 0x0
  OPC_Decode, 205, 64, 171, 2,               // 32252:        decode to UMIN_ZI_S using decoder 299
                                             // 32252:       }
  3, 0,                                      // 32257:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32259:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32261:        check Inst[13] == 0x0
  OPC_Decode, 203, 64, 171, 2,               // 32265:        decode to UMIN_ZI_D using decoder 299
                                             // 32265:       }
                                             // 32265:       } // switch Inst[23:22]
                                             // 32265:      }
  16, 55,                                    // 32270:      case 0x10: {
  OPC_SwitchField, 22, 2,                    // 32272:       switch Inst[23:22] {
  0, 11,                                     // 32275:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32277:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32279:        check Inst[13] == 0x0
  OPC_Decode, 128, 43, 170, 2,               // 32283:        decode to MUL_ZI_B using decoder 298
                                             // 32283:       }
  1, 11,                                     // 32288:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32290:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32292:        check Inst[13] == 0x0
  OPC_Decode, 130, 43, 170, 2,               // 32296:        decode to MUL_ZI_H using decoder 298
                                             // 32296:       }
  2, 11,                                     // 32301:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32303:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32305:        check Inst[13] == 0x0
  OPC_Decode, 131, 43, 170, 2,               // 32309:        decode to MUL_ZI_S using decoder 298
                                             // 32309:       }
  3, 0,                                      // 32314:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32316:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32318:        check Inst[13] == 0x0
  OPC_Decode, 129, 43, 170, 2,               // 32322:        decode to MUL_ZI_D using decoder 298
                                             // 32322:       }
                                             // 32322:       } // switch Inst[23:22]
                                             // 32322:      }
  24, 39,                                    // 32327:      case 0x18: {
  OPC_SwitchField, 22, 2,                    // 32329:       switch Inst[23:22] {
  0, 7,                                      // 32332:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32334:        check predicate 33
  OPC_Decode, 207, 20, 172, 2,               // 32336:        decode to DUP_ZI_B using decoder 300
                                             // 32336:       }
  1, 7,                                      // 32341:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32343:        check predicate 33
  OPC_Decode, 209, 20, 173, 2,               // 32345:        decode to DUP_ZI_H using decoder 301
                                             // 32345:       }
  2, 7,                                      // 32350:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32352:        check predicate 33
  OPC_Decode, 210, 20, 174, 2,               // 32354:        decode to DUP_ZI_S using decoder 302
                                             // 32354:       }
  3, 0,                                      // 32359:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32361:        check predicate 33
  OPC_Decode, 208, 20, 175, 2,               // 32363:        decode to DUP_ZI_D using decoder 303
                                             // 32363:       }
                                             // 32363:       } // switch Inst[23:22]
                                             // 32363:      }
  25, 0,                                     // 32368:      case 0x19: {
  OPC_SwitchField, 22, 2,                    // 32370:       switch Inst[23:22] {
  1, 11,                                     // 32373:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32375:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32377:        check Inst[13] == 0x0
  OPC_Decode, 161, 26, 176, 2,               // 32381:        decode to FDUP_ZI_H using decoder 304
                                             // 32381:       }
  2, 11,                                     // 32386:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 32388:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32390:        check Inst[13] == 0x0
  OPC_Decode, 162, 26, 176, 2,               // 32394:        decode to FDUP_ZI_S using decoder 304
                                             // 32394:       }
  3, 0,                                      // 32399:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 32401:        check predicate 33
  OPC_CheckField, 13, 1, 0,                  // 32403:        check Inst[13] == 0x0
  OPC_Decode, 160, 26, 176, 2,               // 32407:        decode to FDUP_ZI_D using decoder 304
                                             // 32407:       }
                                             // 32407:       } // switch Inst[23:22]
                                             // 32407:      }
                                             // 32407:      } // switch Inst[20:16]
                                             // 32407:     }
                                             // 32407:     } // switch Inst[15:14]
                                             // 32407:    }
                                             // 32407:    } // switch Inst[25:24]
                                             // 32407:   }
                                             // 32407:   } // switch Inst[21]
                                             // 32407:  }
  2, 247, 57,                                // 32412:  case 0x2: {
  OPC_SwitchField, 13, 3,                    // 32415:   switch Inst[15:13] {
  0, 199, 8,                                 // 32418:   case 0x0: {
  OPC_SwitchField, 23, 3,                    // 32421:    switch Inst[25:23] {
  0, 137, 1,                                 // 32424:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 32427:     switch Inst[12:10] {
  0, 25,                                     // 32430:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 32432:      switch Inst[21] {
  0, 11,                                     // 32435:      case 0x0: {
  OPC_CheckPredicate, 39,                    // 32437:       check predicate 39
  OPC_CheckField, 22, 1, 1,                  // 32439:       check Inst[22] == 0x1
  OPC_Decode, 203, 47, 198, 1,               // 32443:       decode to SDOT_ZZZ_BtoH using decoder 198
                                             // 32443:      }
  1, 0,                                      // 32448:      case 0x1: {
  OPC_CheckPredicate, 39,                    // 32450:       check predicate 39
  OPC_Decode, 199, 47, 177, 2,               // 32452:       decode to SDOT_ZZZI_BtoH using decoder 305
                                             // 32452:      }
                                             // 32452:      } // switch Inst[21]
                                             // 32452:     }
  1, 25,                                     // 32457:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 32459:      switch Inst[21] {
  0, 11,                                     // 32462:      case 0x0: {
  OPC_CheckPredicate, 39,                    // 32464:       check predicate 39
  OPC_CheckField, 22, 1, 1,                  // 32466:       check Inst[22] == 0x1
  OPC_Decode, 197, 63, 198, 1,               // 32470:       decode to UDOT_ZZZ_BtoH using decoder 198
                                             // 32470:      }
  1, 0,                                      // 32475:      case 0x1: {
  OPC_CheckPredicate, 39,                    // 32477:       check predicate 39
  OPC_Decode, 193, 63, 177, 2,               // 32479:       decode to UDOT_ZZZI_BtoH using decoder 305
                                             // 32479:      }
                                             // 32479:      } // switch Inst[21]
                                             // 32479:     }
  2, 25,                                     // 32484:     case 0x2: {
  OPC_SwitchField, 21, 1,                    // 32486:      switch Inst[21] {
  0, 11,                                     // 32489:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 32491:       check predicate 35
  OPC_CheckField, 22, 1, 1,                  // 32493:       check Inst[22] == 0x1
  OPC_Decode, 200, 51, 198, 1,               // 32497:       decode to SQDMLALBT_ZZZ_H using decoder 198
                                             // 32497:      }
  1, 0,                                      // 32502:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 32504:       check predicate 35
  OPC_Decode, 245, 41, 177, 2,               // 32506:       decode to MLA_ZZZI_H using decoder 305
                                             // 32506:      }
                                             // 32506:      } // switch Inst[21]
                                             // 32506:     }
  3, 25,                                     // 32511:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 32513:      switch Inst[21] {
  0, 11,                                     // 32516:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 32518:       check predicate 35
  OPC_CheckField, 22, 1, 1,                  // 32520:       check Inst[22] == 0x1
  OPC_Decode, 225, 51, 198, 1,               // 32524:       decode to SQDMLSLBT_ZZZ_H using decoder 198
                                             // 32524:      }
  1, 0,                                      // 32529:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 32531:       check predicate 35
  OPC_Decode, 134, 42, 177, 2,               // 32533:       decode to MLS_ZZZI_H using decoder 305
                                             // 32533:      }
                                             // 32533:      } // switch Inst[21]
                                             // 32533:     }
  4, 11,                                     // 32538:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 32540:      check predicate 35
  OPC_CheckField, 21, 1, 1,                  // 32542:      check Inst[21] == 0x1
  OPC_Decode, 226, 52, 177, 2,               // 32546:      decode to SQRDMLAH_ZZZI_H using decoder 305
                                             // 32546:     }
  5, 0,                                      // 32551:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 32553:      check predicate 35
  OPC_CheckField, 21, 1, 1,                  // 32555:      check Inst[21] == 0x1
  OPC_Decode, 245, 52, 177, 2,               // 32559:      decode to SQRDMLSH_ZZZI_H using decoder 305
                                             // 32559:     }
                                             // 32559:     } // switch Inst[12:10]
                                             // 32559:    }
  1, 249, 1,                                 // 32564:    case 0x1: {
  OPC_SwitchField, 21, 2,                    // 32567:     switch Inst[22:21] {
  0, 53,                                     // 32570:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 32572:      switch Inst[12] {
  0, 39,                                     // 32575:      case 0x0: {
  OPC_SwitchField, 10, 2,                    // 32577:       switch Inst[11:10] {
  0, 7,                                      // 32580:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32582:        check predicate 33
  OPC_Decode, 204, 47, 198, 1,               // 32584:        decode to SDOT_ZZZ_BtoS using decoder 198
                                             // 32584:       }
  1, 7,                                      // 32589:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32591:        check predicate 33
  OPC_Decode, 198, 63, 198, 1,               // 32593:        decode to UDOT_ZZZ_BtoS using decoder 198
                                             // 32593:       }
  2, 7,                                      // 32598:       case 0x2: {
  OPC_CheckPredicate, 35,                    // 32600:        check predicate 35
  OPC_Decode, 201, 51, 198, 1,               // 32602:        decode to SQDMLALBT_ZZZ_S using decoder 198
                                             // 32602:       }
  3, 0,                                      // 32607:       case 0x3: {
  OPC_CheckPredicate, 35,                    // 32609:        check predicate 35
  OPC_Decode, 226, 51, 198, 1,               // 32611:        decode to SQDMLSLBT_ZZZ_S using decoder 198
                                             // 32611:       }
                                             // 32611:       } // switch Inst[11:10]
                                             // 32611:      }
  1, 0,                                      // 32616:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 32618:       check predicate 35
  OPC_Decode, 147, 17, 178, 2,               // 32620:       decode to CDOT_ZZZ_S using decoder 306
                                             // 32620:      }
                                             // 32620:      } // switch Inst[12]
                                             // 32620:     }
  1, 75,                                     // 32625:     case 0x1: {
  OPC_SwitchField, 10, 3,                    // 32627:      switch Inst[12:10] {
  0, 7,                                      // 32630:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 32632:       check predicate 33
  OPC_Decode, 200, 47, 179, 2,               // 32634:       decode to SDOT_ZZZI_BtoS using decoder 307
                                             // 32634:      }
  1, 7,                                      // 32639:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 32641:       check predicate 33
  OPC_Decode, 194, 63, 179, 2,               // 32643:       decode to UDOT_ZZZI_BtoS using decoder 307
                                             // 32643:      }
  2, 7,                                      // 32648:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 32650:       check predicate 35
  OPC_Decode, 246, 41, 179, 2,               // 32652:       decode to MLA_ZZZI_S using decoder 307
                                             // 32652:      }
  3, 7,                                      // 32657:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 32659:       check predicate 35
  OPC_Decode, 135, 42, 179, 2,               // 32661:       decode to MLS_ZZZI_S using decoder 307
                                             // 32661:      }
  4, 7,                                      // 32666:      case 0x4: {
  OPC_CheckPredicate, 35,                    // 32668:       check predicate 35
  OPC_Decode, 227, 52, 179, 2,               // 32670:       decode to SQRDMLAH_ZZZI_S using decoder 307
                                             // 32670:      }
  5, 7,                                      // 32675:      case 0x5: {
  OPC_CheckPredicate, 35,                    // 32677:       check predicate 35
  OPC_Decode, 246, 52, 179, 2,               // 32679:       decode to SQRDMLSH_ZZZI_S using decoder 307
                                             // 32679:      }
  6, 7,                                      // 32684:      case 0x6: {
  OPC_CheckPredicate, 45,                    // 32686:       check predicate 45
  OPC_Decode, 150, 68, 179, 2,               // 32688:       decode to USDOT_ZZZI using decoder 307
                                             // 32688:      }
  7, 0,                                      // 32693:      case 0x7: {
  OPC_CheckPredicate, 45,                    // 32695:       check predicate 45
  OPC_Decode, 204, 60, 179, 2,               // 32697:       decode to SUDOT_ZZZI using decoder 307
                                             // 32697:      }
                                             // 32697:      } // switch Inst[12:10]
                                             // 32697:     }
  2, 53,                                     // 32702:     case 0x2: {
  OPC_SwitchField, 12, 1,                    // 32704:      switch Inst[12] {
  0, 39,                                     // 32707:      case 0x0: {
  OPC_SwitchField, 10, 2,                    // 32709:       switch Inst[11:10] {
  0, 7,                                      // 32712:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 32714:        check predicate 33
  OPC_Decode, 205, 47, 198, 1,               // 32716:        decode to SDOT_ZZZ_HtoD using decoder 198
                                             // 32716:       }
  1, 7,                                      // 32721:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 32723:        check predicate 33
  OPC_Decode, 199, 63, 198, 1,               // 32725:        decode to UDOT_ZZZ_HtoD using decoder 198
                                             // 32725:       }
  2, 7,                                      // 32730:       case 0x2: {
  OPC_CheckPredicate, 35,                    // 32732:        check predicate 35
  OPC_Decode, 199, 51, 198, 1,               // 32734:        decode to SQDMLALBT_ZZZ_D using decoder 198
                                             // 32734:       }
  3, 0,                                      // 32739:       case 0x3: {
  OPC_CheckPredicate, 35,                    // 32741:        check predicate 35
  OPC_Decode, 224, 51, 198, 1,               // 32743:        decode to SQDMLSLBT_ZZZ_D using decoder 198
                                             // 32743:       }
                                             // 32743:       } // switch Inst[11:10]
                                             // 32743:      }
  1, 0,                                      // 32748:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 32750:       check predicate 35
  OPC_Decode, 146, 17, 178, 2,               // 32752:       decode to CDOT_ZZZ_D using decoder 306
                                             // 32752:      }
                                             // 32752:      } // switch Inst[12]
                                             // 32752:     }
  3, 0,                                      // 32757:     case 0x3: {
  OPC_SwitchField, 10, 3,                    // 32759:      switch Inst[12:10] {
  0, 7,                                      // 32762:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 32764:       check predicate 33
  OPC_Decode, 201, 47, 180, 2,               // 32766:       decode to SDOT_ZZZI_HtoD using decoder 308
                                             // 32766:      }
  1, 7,                                      // 32771:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 32773:       check predicate 33
  OPC_Decode, 195, 63, 180, 2,               // 32775:       decode to UDOT_ZZZI_HtoD using decoder 308
                                             // 32775:      }
  2, 7,                                      // 32780:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 32782:       check predicate 35
  OPC_Decode, 244, 41, 180, 2,               // 32784:       decode to MLA_ZZZI_D using decoder 308
                                             // 32784:      }
  3, 7,                                      // 32789:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 32791:       check predicate 35
  OPC_Decode, 133, 42, 180, 2,               // 32793:       decode to MLS_ZZZI_D using decoder 308
                                             // 32793:      }
  4, 7,                                      // 32798:      case 0x4: {
  OPC_CheckPredicate, 35,                    // 32800:       check predicate 35
  OPC_Decode, 225, 52, 180, 2,               // 32802:       decode to SQRDMLAH_ZZZI_D using decoder 308
                                             // 32802:      }
  5, 0,                                      // 32807:      case 0x5: {
  OPC_CheckPredicate, 35,                    // 32809:       check predicate 35
  OPC_Decode, 244, 52, 180, 2,               // 32811:       decode to SQRDMLSH_ZZZI_D using decoder 308
                                             // 32811:      }
                                             // 32811:      } // switch Inst[12:10]
                                             // 32811:     }
                                             // 32811:     } // switch Inst[22:21]
                                             // 32811:    }
  2, 147, 3,                                 // 32816:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 32819:     switch Inst[12:10] {
  0, 48,                                     // 32822:     case 0x0: {
  OPC_SwitchField, 21, 2,                    // 32824:      switch Inst[22:21] {
  1, 25,                                     // 32827:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 32829:       switch Inst[20] {
  0, 11,                                     // 32832:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 32834:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 32836:        check Inst[19] == 0x1
  OPC_Decode, 155, 54, 229, 1,               // 32840:        decode to SQSHRUNB_ZZI_B using decoder 229
                                             // 32840:       }
  1, 0,                                      // 32845:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 32847:        check predicate 35
  OPC_Decode, 156, 54, 230, 1,               // 32849:        decode to SQSHRUNB_ZZI_H using decoder 230
                                             // 32849:       }
                                             // 32849:       } // switch Inst[20]
                                             // 32849:      }
  2, 7,                                      // 32854:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 32856:       check predicate 35
  OPC_Decode, 189, 46, 184, 1,               // 32858:       decode to SADDLB_ZZZ_H using decoder 184
                                             // 32858:      }
  3, 0,                                      // 32863:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 32865:       check predicate 35
  OPC_Decode, 157, 54, 231, 1,               // 32867:       decode to SQSHRUNB_ZZI_S using decoder 231
                                             // 32867:      }
                                             // 32867:      } // switch Inst[22:21]
                                             // 32867:     }
  1, 48,                                     // 32872:     case 0x1: {
  OPC_SwitchField, 21, 2,                    // 32874:      switch Inst[22:21] {
  1, 25,                                     // 32877:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 32879:       switch Inst[20] {
  0, 11,                                     // 32882:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 32884:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 32886:        check Inst[19] == 0x1
  OPC_Decode, 158, 54, 199, 1,               // 32890:        decode to SQSHRUNT_ZZI_B using decoder 199
                                             // 32890:       }
  1, 0,                                      // 32895:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 32897:        check predicate 35
  OPC_Decode, 159, 54, 200, 1,               // 32899:        decode to SQSHRUNT_ZZI_H using decoder 200
                                             // 32899:       }
                                             // 32899:       } // switch Inst[20]
                                             // 32899:      }
  2, 7,                                      // 32904:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 32906:       check predicate 35
  OPC_Decode, 198, 46, 184, 1,               // 32908:       decode to SADDLT_ZZZ_H using decoder 184
                                             // 32908:      }
  3, 0,                                      // 32913:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 32915:       check predicate 35
  OPC_Decode, 160, 54, 201, 1,               // 32917:       decode to SQSHRUNT_ZZI_S using decoder 201
                                             // 32917:      }
                                             // 32917:      } // switch Inst[22:21]
                                             // 32917:     }
  2, 48,                                     // 32922:     case 0x2: {
  OPC_SwitchField, 21, 2,                    // 32924:      switch Inst[22:21] {
  1, 25,                                     // 32927:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 32929:       switch Inst[20] {
  0, 11,                                     // 32932:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 32934:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 32936:        check Inst[19] == 0x1
  OPC_Decode, 192, 53, 229, 1,               // 32940:        decode to SQRSHRUNB_ZZI_B using decoder 229
                                             // 32940:       }
  1, 0,                                      // 32945:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 32947:        check predicate 35
  OPC_Decode, 193, 53, 230, 1,               // 32949:        decode to SQRSHRUNB_ZZI_H using decoder 230
                                             // 32949:       }
                                             // 32949:       } // switch Inst[20]
                                             // 32949:      }
  2, 7,                                      // 32954:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 32956:       check predicate 35
  OPC_Decode, 190, 62, 184, 1,               // 32958:       decode to UADDLB_ZZZ_H using decoder 184
                                             // 32958:      }
  3, 0,                                      // 32963:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 32965:       check predicate 35
  OPC_Decode, 194, 53, 231, 1,               // 32967:       decode to SQRSHRUNB_ZZI_S using decoder 231
                                             // 32967:      }
                                             // 32967:      } // switch Inst[22:21]
                                             // 32967:     }
  3, 48,                                     // 32972:     case 0x3: {
  OPC_SwitchField, 21, 2,                    // 32974:      switch Inst[22:21] {
  1, 25,                                     // 32977:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 32979:       switch Inst[20] {
  0, 11,                                     // 32982:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 32984:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 32986:        check Inst[19] == 0x1
  OPC_Decode, 195, 53, 199, 1,               // 32990:        decode to SQRSHRUNT_ZZI_B using decoder 199
                                             // 32990:       }
  1, 0,                                      // 32995:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 32997:        check predicate 35
  OPC_Decode, 196, 53, 200, 1,               // 32999:        decode to SQRSHRUNT_ZZI_H using decoder 200
                                             // 32999:       }
                                             // 32999:       } // switch Inst[20]
                                             // 32999:      }
  2, 7,                                      // 33004:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33006:       check predicate 35
  OPC_Decode, 199, 62, 184, 1,               // 33008:       decode to UADDLT_ZZZ_H using decoder 184
                                             // 33008:      }
  3, 0,                                      // 33013:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 33015:       check predicate 35
  OPC_Decode, 197, 53, 201, 1,               // 33017:       decode to SQRSHRUNT_ZZI_S using decoder 201
                                             // 33017:      }
                                             // 33017:      } // switch Inst[22:21]
                                             // 33017:     }
  4, 48,                                     // 33022:     case 0x4: {
  OPC_SwitchField, 21, 2,                    // 33024:      switch Inst[22:21] {
  1, 25,                                     // 33027:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33029:       switch Inst[20] {
  0, 11,                                     // 33032:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 33034:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33036:        check Inst[19] == 0x1
  OPC_Decode, 169, 48, 229, 1,               // 33040:        decode to SHRNB_ZZI_B using decoder 229
                                             // 33040:       }
  1, 0,                                      // 33045:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 33047:        check predicate 35
  OPC_Decode, 170, 48, 230, 1,               // 33049:        decode to SHRNB_ZZI_H using decoder 230
                                             // 33049:       }
                                             // 33049:       } // switch Inst[20]
                                             // 33049:      }
  2, 7,                                      // 33054:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33056:       check predicate 35
  OPC_Decode, 134, 56, 184, 1,               // 33058:       decode to SSUBLB_ZZZ_H using decoder 184
                                             // 33058:      }
  3, 0,                                      // 33063:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 33065:       check predicate 35
  OPC_Decode, 171, 48, 231, 1,               // 33067:       decode to SHRNB_ZZI_S using decoder 231
                                             // 33067:      }
                                             // 33067:      } // switch Inst[22:21]
                                             // 33067:     }
  5, 48,                                     // 33072:     case 0x5: {
  OPC_SwitchField, 21, 2,                    // 33074:      switch Inst[22:21] {
  1, 25,                                     // 33077:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33079:       switch Inst[20] {
  0, 11,                                     // 33082:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 33084:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33086:        check Inst[19] == 0x1
  OPC_Decode, 172, 48, 199, 1,               // 33090:        decode to SHRNT_ZZI_B using decoder 199
                                             // 33090:       }
  1, 0,                                      // 33095:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 33097:        check predicate 35
  OPC_Decode, 173, 48, 200, 1,               // 33099:        decode to SHRNT_ZZI_H using decoder 200
                                             // 33099:       }
                                             // 33099:       } // switch Inst[20]
                                             // 33099:      }
  2, 7,                                      // 33104:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33106:       check predicate 35
  OPC_Decode, 140, 56, 184, 1,               // 33108:       decode to SSUBLT_ZZZ_H using decoder 184
                                             // 33108:      }
  3, 0,                                      // 33113:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 33115:       check predicate 35
  OPC_Decode, 174, 48, 201, 1,               // 33117:       decode to SHRNT_ZZI_S using decoder 201
                                             // 33117:      }
                                             // 33117:      } // switch Inst[22:21]
                                             // 33117:     }
  6, 48,                                     // 33122:     case 0x6: {
  OPC_SwitchField, 21, 2,                    // 33124:      switch Inst[22:21] {
  1, 25,                                     // 33127:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33129:       switch Inst[20] {
  0, 11,                                     // 33132:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 33134:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33136:        check Inst[19] == 0x1
  OPC_Decode, 233, 45, 229, 1,               // 33140:        decode to RSHRNB_ZZI_B using decoder 229
                                             // 33140:       }
  1, 0,                                      // 33145:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 33147:        check predicate 35
  OPC_Decode, 234, 45, 230, 1,               // 33149:        decode to RSHRNB_ZZI_H using decoder 230
                                             // 33149:       }
                                             // 33149:       } // switch Inst[20]
                                             // 33149:      }
  2, 7,                                      // 33154:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33156:       check predicate 35
  OPC_Decode, 242, 68, 184, 1,               // 33158:       decode to USUBLB_ZZZ_H using decoder 184
                                             // 33158:      }
  3, 0,                                      // 33163:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 33165:       check predicate 35
  OPC_Decode, 235, 45, 231, 1,               // 33167:       decode to RSHRNB_ZZI_S using decoder 231
                                             // 33167:      }
                                             // 33167:      } // switch Inst[22:21]
                                             // 33167:     }
  7, 0,                                      // 33172:     case 0x7: {
  OPC_SwitchField, 21, 2,                    // 33174:      switch Inst[22:21] {
  1, 25,                                     // 33177:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33179:       switch Inst[20] {
  0, 11,                                     // 33182:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 33184:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33186:        check Inst[19] == 0x1
  OPC_Decode, 236, 45, 199, 1,               // 33190:        decode to RSHRNT_ZZI_B using decoder 199
                                             // 33190:       }
  1, 0,                                      // 33195:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 33197:        check predicate 35
  OPC_Decode, 237, 45, 200, 1,               // 33199:        decode to RSHRNT_ZZI_H using decoder 200
                                             // 33199:       }
                                             // 33199:       } // switch Inst[20]
                                             // 33199:      }
  2, 7,                                      // 33204:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33206:       check predicate 35
  OPC_Decode, 245, 68, 184, 1,               // 33208:       decode to USUBLT_ZZZ_H using decoder 184
                                             // 33208:      }
  3, 0,                                      // 33213:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 33215:       check predicate 35
  OPC_Decode, 238, 45, 201, 1,               // 33217:       decode to RSHRNT_ZZI_S using decoder 201
                                             // 33217:      }
                                             // 33217:      } // switch Inst[22:21]
                                             // 33217:     }
                                             // 33217:     } // switch Inst[12:10]
                                             // 33217:    }
  3, 0,                                      // 33222:    case 0x3: {
  OPC_SwitchField, 10, 3,                    // 33224:     switch Inst[12:10] {
  0, 56,                                     // 33227:     case 0x0: {
  OPC_SwitchField, 21, 2,                    // 33229:      switch Inst[22:21] {
  0, 7,                                      // 33232:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33234:       check predicate 35
  OPC_Decode, 190, 46, 184, 1,               // 33236:       decode to SADDLB_ZZZ_S using decoder 184
                                             // 33236:      }
  1, 33,                                     // 33241:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33243:       switch Inst[20] {
  0, 15,                                     // 33246:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 33248:        check predicate 39
  OPC_CheckField, 19, 1, 1,                  // 33250:        check Inst[19] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 33254:        check Inst[5] == 0x0
  OPC_Decode, 144, 54, 181, 2,               // 33258:        decode to SQSHRN_Z2ZI_HtoB using decoder 309
                                             // 33258:       }
  1, 0,                                      // 33263:       case 0x1: {
  OPC_CheckPredicate, 39,                    // 33265:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 33267:        check Inst[5] == 0x0
  OPC_Decode, 145, 54, 151, 1,               // 33271:        decode to SQSHRN_Z2ZI_StoH using decoder 151
                                             // 33271:       }
                                             // 33271:       } // switch Inst[20]
                                             // 33271:      }
  2, 0,                                      // 33276:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33278:       check predicate 35
  OPC_Decode, 188, 46, 184, 1,               // 33280:       decode to SADDLB_ZZZ_D using decoder 184
                                             // 33280:      }
                                             // 33280:      } // switch Inst[22:21]
                                             // 33280:     }
  1, 21,                                     // 33285:     case 0x1: {
  OPC_SwitchField, 21, 2,                    // 33287:      switch Inst[22:21] {
  0, 7,                                      // 33290:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33292:       check predicate 35
  OPC_Decode, 199, 46, 184, 1,               // 33294:       decode to SADDLT_ZZZ_S using decoder 184
                                             // 33294:      }
  2, 0,                                      // 33299:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33301:       check predicate 35
  OPC_Decode, 197, 46, 184, 1,               // 33303:       decode to SADDLT_ZZZ_D using decoder 184
                                             // 33303:      }
                                             // 33303:      } // switch Inst[22:21]
                                             // 33303:     }
  2, 56,                                     // 33308:     case 0x2: {
  OPC_SwitchField, 21, 2,                    // 33310:      switch Inst[22:21] {
  0, 7,                                      // 33313:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33315:       check predicate 35
  OPC_Decode, 191, 62, 184, 1,               // 33317:       decode to UADDLB_ZZZ_S using decoder 184
                                             // 33317:      }
  1, 33,                                     // 33322:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33324:       switch Inst[20] {
  0, 15,                                     // 33327:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 33329:        check predicate 39
  OPC_CheckField, 19, 1, 1,                  // 33331:        check Inst[19] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 33335:        check Inst[5] == 0x0
  OPC_Decode, 200, 53, 181, 2,               // 33339:        decode to SQRSHRUN_Z2ZI_HtoB using decoder 309
                                             // 33339:       }
  1, 0,                                      // 33344:       case 0x1: {
  OPC_CheckPredicate, 18,                    // 33346:        check predicate 18
  OPC_CheckField, 5, 1, 0,                   // 33348:        check Inst[5] == 0x0
  OPC_Decode, 201, 53, 151, 1,               // 33352:        decode to SQRSHRUN_Z2ZI_StoH using decoder 151
                                             // 33352:       }
                                             // 33352:       } // switch Inst[20]
                                             // 33352:      }
  2, 0,                                      // 33357:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33359:       check predicate 35
  OPC_Decode, 189, 62, 184, 1,               // 33361:       decode to UADDLB_ZZZ_D using decoder 184
                                             // 33361:      }
                                             // 33361:      } // switch Inst[22:21]
                                             // 33361:     }
  3, 21,                                     // 33366:     case 0x3: {
  OPC_SwitchField, 21, 2,                    // 33368:      switch Inst[22:21] {
  0, 7,                                      // 33371:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33373:       check predicate 35
  OPC_Decode, 200, 62, 184, 1,               // 33375:       decode to UADDLT_ZZZ_S using decoder 184
                                             // 33375:      }
  2, 0,                                      // 33380:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33382:       check predicate 35
  OPC_Decode, 198, 62, 184, 1,               // 33384:       decode to UADDLT_ZZZ_D using decoder 184
                                             // 33384:      }
                                             // 33384:      } // switch Inst[22:21]
                                             // 33384:     }
  4, 56,                                     // 33389:     case 0x4: {
  OPC_SwitchField, 21, 2,                    // 33391:      switch Inst[22:21] {
  0, 7,                                      // 33394:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33396:       check predicate 35
  OPC_Decode, 135, 56, 184, 1,               // 33398:       decode to SSUBLB_ZZZ_S using decoder 184
                                             // 33398:      }
  1, 33,                                     // 33403:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33405:       switch Inst[20] {
  0, 15,                                     // 33408:       case 0x0: {
  OPC_CheckPredicate, 39,                    // 33410:        check predicate 39
  OPC_CheckField, 19, 1, 1,                  // 33412:        check Inst[19] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 33416:        check Inst[5] == 0x0
  OPC_Decode, 144, 67, 181, 2,               // 33420:        decode to UQSHRN_Z2ZI_HtoB using decoder 309
                                             // 33420:       }
  1, 0,                                      // 33425:       case 0x1: {
  OPC_CheckPredicate, 39,                    // 33427:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 33429:        check Inst[5] == 0x0
  OPC_Decode, 145, 67, 151, 1,               // 33433:        decode to UQSHRN_Z2ZI_StoH using decoder 151
                                             // 33433:       }
                                             // 33433:       } // switch Inst[20]
                                             // 33433:      }
  2, 0,                                      // 33438:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33440:       check predicate 35
  OPC_Decode, 133, 56, 184, 1,               // 33442:       decode to SSUBLB_ZZZ_D using decoder 184
                                             // 33442:      }
                                             // 33442:      } // switch Inst[22:21]
                                             // 33442:     }
  5, 21,                                     // 33447:     case 0x5: {
  OPC_SwitchField, 21, 2,                    // 33449:      switch Inst[22:21] {
  0, 7,                                      // 33452:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33454:       check predicate 35
  OPC_Decode, 141, 56, 184, 1,               // 33456:       decode to SSUBLT_ZZZ_S using decoder 184
                                             // 33456:      }
  2, 0,                                      // 33461:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33463:       check predicate 35
  OPC_Decode, 139, 56, 184, 1,               // 33465:       decode to SSUBLT_ZZZ_D using decoder 184
                                             // 33465:      }
                                             // 33465:      } // switch Inst[22:21]
                                             // 33465:     }
  6, 21,                                     // 33470:     case 0x6: {
  OPC_SwitchField, 21, 2,                    // 33472:      switch Inst[22:21] {
  0, 7,                                      // 33475:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33477:       check predicate 35
  OPC_Decode, 243, 68, 184, 1,               // 33479:       decode to USUBLB_ZZZ_S using decoder 184
                                             // 33479:      }
  2, 0,                                      // 33484:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33486:       check predicate 35
  OPC_Decode, 241, 68, 184, 1,               // 33488:       decode to USUBLB_ZZZ_D using decoder 184
                                             // 33488:      }
                                             // 33488:      } // switch Inst[22:21]
                                             // 33488:     }
  7, 0,                                      // 33493:     case 0x7: {
  OPC_SwitchField, 21, 2,                    // 33495:      switch Inst[22:21] {
  0, 7,                                      // 33498:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33500:       check predicate 35
  OPC_Decode, 246, 68, 184, 1,               // 33502:       decode to USUBLT_ZZZ_S using decoder 184
                                             // 33502:      }
  2, 0,                                      // 33507:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 33509:       check predicate 35
  OPC_Decode, 244, 68, 184, 1,               // 33511:       decode to USUBLT_ZZZ_D using decoder 184
                                             // 33511:      }
                                             // 33511:      } // switch Inst[22:21]
                                             // 33511:     }
                                             // 33511:     } // switch Inst[12:10]
                                             // 33511:    }
                                             // 33511:    } // switch Inst[25:23]
                                             // 33511:   }
  1, 217, 5,                                 // 33516:   case 0x1: {
  OPC_SwitchField, 21, 5,                    // 33519:    switch Inst[25:21] {
  0, 21,                                     // 33522:    case 0x0: {
  OPC_SwitchField, 12, 1,                    // 33524:     switch Inst[12] {
  0, 7,                                      // 33527:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 33529:      check predicate 35
  OPC_Decode, 145, 18, 178, 2,               // 33531:      decode to CMLA_ZZZ_B using decoder 306
                                             // 33531:     }
  1, 0,                                      // 33536:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 33538:      check predicate 35
  OPC_Decode, 221, 52, 178, 2,               // 33540:      decode to SQRDCMLAH_ZZZ_B using decoder 306
                                             // 33540:     }
                                             // 33540:     } // switch Inst[12]
                                             // 33540:    }
  2, 21,                                     // 33545:    case 0x2: {
  OPC_SwitchField, 12, 1,                    // 33547:     switch Inst[12] {
  0, 7,                                      // 33550:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 33552:      check predicate 35
  OPC_Decode, 147, 18, 178, 2,               // 33554:      decode to CMLA_ZZZ_H using decoder 306
                                             // 33554:     }
  1, 0,                                      // 33559:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 33561:      check predicate 35
  OPC_Decode, 223, 52, 178, 2,               // 33563:      decode to SQRDCMLAH_ZZZ_H using decoder 306
                                             // 33563:     }
                                             // 33563:     } // switch Inst[12]
                                             // 33563:    }
  4, 21,                                     // 33568:    case 0x4: {
  OPC_SwitchField, 12, 1,                    // 33570:     switch Inst[12] {
  0, 7,                                      // 33573:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 33575:      check predicate 35
  OPC_Decode, 148, 18, 178, 2,               // 33577:      decode to CMLA_ZZZ_S using decoder 306
                                             // 33577:     }
  1, 0,                                      // 33582:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 33584:      check predicate 35
  OPC_Decode, 224, 52, 178, 2,               // 33586:      decode to SQRDCMLAH_ZZZ_S using decoder 306
                                             // 33586:     }
                                             // 33586:     } // switch Inst[12]
                                             // 33586:    }
  5, 49,                                     // 33591:    case 0x5: {
  OPC_SwitchField, 10, 1,                    // 33593:     switch Inst[10] {
  0, 21,                                     // 33596:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 33598:      switch Inst[12] {
  0, 7,                                      // 33601:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33603:       check predicate 35
  OPC_Decode, 203, 51, 182, 2,               // 33605:       decode to SQDMLALB_ZZZI_S using decoder 310
                                             // 33605:      }
  1, 0,                                      // 33610:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33612:       check predicate 35
  OPC_Decode, 228, 51, 182, 2,               // 33614:       decode to SQDMLSLB_ZZZI_S using decoder 310
                                             // 33614:      }
                                             // 33614:      } // switch Inst[12]
                                             // 33614:     }
  1, 0,                                      // 33619:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 33621:      switch Inst[12] {
  0, 7,                                      // 33624:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33626:       check predicate 35
  OPC_Decode, 208, 51, 182, 2,               // 33628:       decode to SQDMLALT_ZZZI_S using decoder 310
                                             // 33628:      }
  1, 0,                                      // 33633:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33635:       check predicate 35
  OPC_Decode, 233, 51, 182, 2,               // 33637:       decode to SQDMLSLT_ZZZI_S using decoder 310
                                             // 33637:      }
                                             // 33637:      } // switch Inst[12]
                                             // 33637:     }
                                             // 33637:     } // switch Inst[10]
                                             // 33637:    }
  6, 21,                                     // 33642:    case 0x6: {
  OPC_SwitchField, 12, 1,                    // 33644:     switch Inst[12] {
  0, 7,                                      // 33647:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 33649:      check predicate 35
  OPC_Decode, 146, 18, 178, 2,               // 33651:      decode to CMLA_ZZZ_D using decoder 306
                                             // 33651:     }
  1, 0,                                      // 33656:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 33658:      check predicate 35
  OPC_Decode, 222, 52, 178, 2,               // 33660:      decode to SQRDCMLAH_ZZZ_D using decoder 306
                                             // 33660:     }
                                             // 33660:     } // switch Inst[12]
                                             // 33660:    }
  7, 49,                                     // 33665:    case 0x7: {
  OPC_SwitchField, 10, 1,                    // 33667:     switch Inst[10] {
  0, 21,                                     // 33670:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 33672:      switch Inst[12] {
  0, 7,                                      // 33675:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33677:       check predicate 35
  OPC_Decode, 202, 51, 183, 2,               // 33679:       decode to SQDMLALB_ZZZI_D using decoder 311
                                             // 33679:      }
  1, 0,                                      // 33684:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33686:       check predicate 35
  OPC_Decode, 227, 51, 183, 2,               // 33688:       decode to SQDMLSLB_ZZZI_D using decoder 311
                                             // 33688:      }
                                             // 33688:      } // switch Inst[12]
                                             // 33688:     }
  1, 0,                                      // 33693:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 33695:      switch Inst[12] {
  0, 7,                                      // 33698:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33700:       check predicate 35
  OPC_Decode, 207, 51, 183, 2,               // 33702:       decode to SQDMLALT_ZZZI_D using decoder 311
                                             // 33702:      }
  1, 0,                                      // 33707:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33709:       check predicate 35
  OPC_Decode, 232, 51, 183, 2,               // 33711:       decode to SQDMLSLT_ZZZI_D using decoder 311
                                             // 33711:      }
                                             // 33711:      } // switch Inst[12]
                                             // 33711:     }
                                             // 33711:     } // switch Inst[10]
                                             // 33711:    }
  9, 219, 1,                                 // 33716:    case 0x9: {
  OPC_SwitchField, 10, 3,                    // 33719:     switch Inst[12:10] {
  0, 25,                                     // 33722:     case 0x0: {
  OPC_SwitchField, 20, 1,                    // 33724:      switch Inst[20] {
  0, 11,                                     // 33727:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33729:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33731:       check Inst[19] == 0x1
  OPC_Decode, 138, 54, 229, 1,               // 33735:       decode to SQSHRNB_ZZI_B using decoder 229
                                             // 33735:      }
  1, 0,                                      // 33740:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33742:       check predicate 35
  OPC_Decode, 139, 54, 230, 1,               // 33744:       decode to SQSHRNB_ZZI_H using decoder 230
                                             // 33744:      }
                                             // 33744:      } // switch Inst[20]
                                             // 33744:     }
  1, 25,                                     // 33749:     case 0x1: {
  OPC_SwitchField, 20, 1,                    // 33751:      switch Inst[20] {
  0, 11,                                     // 33754:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33756:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33758:       check Inst[19] == 0x1
  OPC_Decode, 141, 54, 199, 1,               // 33762:       decode to SQSHRNT_ZZI_B using decoder 199
                                             // 33762:      }
  1, 0,                                      // 33767:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33769:       check predicate 35
  OPC_Decode, 142, 54, 200, 1,               // 33771:       decode to SQSHRNT_ZZI_H using decoder 200
                                             // 33771:      }
                                             // 33771:      } // switch Inst[20]
                                             // 33771:     }
  2, 25,                                     // 33776:     case 0x2: {
  OPC_SwitchField, 20, 1,                    // 33778:      switch Inst[20] {
  0, 11,                                     // 33781:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33783:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33785:       check Inst[19] == 0x1
  OPC_Decode, 173, 53, 229, 1,               // 33789:       decode to SQRSHRNB_ZZI_B using decoder 229
                                             // 33789:      }
  1, 0,                                      // 33794:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33796:       check predicate 35
  OPC_Decode, 174, 53, 230, 1,               // 33798:       decode to SQRSHRNB_ZZI_H using decoder 230
                                             // 33798:      }
                                             // 33798:      } // switch Inst[20]
                                             // 33798:     }
  3, 25,                                     // 33803:     case 0x3: {
  OPC_SwitchField, 20, 1,                    // 33805:      switch Inst[20] {
  0, 11,                                     // 33808:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33810:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33812:       check Inst[19] == 0x1
  OPC_Decode, 176, 53, 199, 1,               // 33816:       decode to SQRSHRNT_ZZI_B using decoder 199
                                             // 33816:      }
  1, 0,                                      // 33821:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33823:       check predicate 35
  OPC_Decode, 177, 53, 200, 1,               // 33825:       decode to SQRSHRNT_ZZI_H using decoder 200
                                             // 33825:      }
                                             // 33825:      } // switch Inst[20]
                                             // 33825:     }
  4, 25,                                     // 33830:     case 0x4: {
  OPC_SwitchField, 20, 1,                    // 33832:      switch Inst[20] {
  0, 11,                                     // 33835:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33837:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33839:       check Inst[19] == 0x1
  OPC_Decode, 138, 67, 229, 1,               // 33843:       decode to UQSHRNB_ZZI_B using decoder 229
                                             // 33843:      }
  1, 0,                                      // 33848:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33850:       check predicate 35
  OPC_Decode, 139, 67, 230, 1,               // 33852:       decode to UQSHRNB_ZZI_H using decoder 230
                                             // 33852:      }
                                             // 33852:      } // switch Inst[20]
                                             // 33852:     }
  5, 25,                                     // 33857:     case 0x5: {
  OPC_SwitchField, 20, 1,                    // 33859:      switch Inst[20] {
  0, 11,                                     // 33862:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33864:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33866:       check Inst[19] == 0x1
  OPC_Decode, 141, 67, 199, 1,               // 33870:       decode to UQSHRNT_ZZI_B using decoder 199
                                             // 33870:      }
  1, 0,                                      // 33875:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33877:       check predicate 35
  OPC_Decode, 142, 67, 200, 1,               // 33879:       decode to UQSHRNT_ZZI_H using decoder 200
                                             // 33879:      }
                                             // 33879:      } // switch Inst[20]
                                             // 33879:     }
  6, 25,                                     // 33884:     case 0x6: {
  OPC_SwitchField, 20, 1,                    // 33886:      switch Inst[20] {
  0, 11,                                     // 33889:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33891:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33893:       check Inst[19] == 0x1
  OPC_Decode, 210, 66, 229, 1,               // 33897:       decode to UQRSHRNB_ZZI_B using decoder 229
                                             // 33897:      }
  1, 0,                                      // 33902:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33904:       check predicate 35
  OPC_Decode, 211, 66, 230, 1,               // 33906:       decode to UQRSHRNB_ZZI_H using decoder 230
                                             // 33906:      }
                                             // 33906:      } // switch Inst[20]
                                             // 33906:     }
  7, 0,                                      // 33911:     case 0x7: {
  OPC_SwitchField, 20, 1,                    // 33913:      switch Inst[20] {
  0, 11,                                     // 33916:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 33918:       check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 33920:       check Inst[19] == 0x1
  OPC_Decode, 213, 66, 199, 1,               // 33924:       decode to UQRSHRNT_ZZI_B using decoder 199
                                             // 33924:      }
  1, 0,                                      // 33929:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 33931:       check predicate 35
  OPC_Decode, 214, 66, 200, 1,               // 33933:       decode to UQRSHRNT_ZZI_H using decoder 200
                                             // 33933:      }
                                             // 33933:      } // switch Inst[20]
                                             // 33933:     }
                                             // 33933:     } // switch Inst[12:10]
                                             // 33933:    }
  10, 39,                                    // 33938:    case 0xa: {
  OPC_SwitchField, 10, 3,                    // 33940:     switch Inst[12:10] {
  4, 7,                                      // 33943:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 33945:      check predicate 35
  OPC_Decode, 155, 46, 184, 1,               // 33947:      decode to SABDLB_ZZZ_H using decoder 184
                                             // 33947:     }
  5, 7,                                      // 33952:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 33954:      check predicate 35
  OPC_Decode, 158, 46, 184, 1,               // 33956:      decode to SABDLT_ZZZ_H using decoder 184
                                             // 33956:     }
  6, 7,                                      // 33961:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 33963:      check predicate 35
  OPC_Decode, 159, 62, 184, 1,               // 33965:      decode to UABDLB_ZZZ_H using decoder 184
                                             // 33965:     }
  7, 0,                                      // 33970:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 33972:      check predicate 35
  OPC_Decode, 162, 62, 184, 1,               // 33974:      decode to UABDLT_ZZZ_H using decoder 184
                                             // 33974:     }
                                             // 33974:     } // switch Inst[12:10]
                                             // 33974:    }
  11, 75,                                    // 33979:    case 0xb: {
  OPC_SwitchField, 10, 3,                    // 33981:     switch Inst[12:10] {
  0, 7,                                      // 33984:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 33986:      check predicate 35
  OPC_Decode, 140, 54, 231, 1,               // 33988:      decode to SQSHRNB_ZZI_S using decoder 231
                                             // 33988:     }
  1, 7,                                      // 33993:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 33995:      check predicate 35
  OPC_Decode, 143, 54, 201, 1,               // 33997:      decode to SQSHRNT_ZZI_S using decoder 201
                                             // 33997:     }
  2, 7,                                      // 34002:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34004:      check predicate 35
  OPC_Decode, 175, 53, 231, 1,               // 34006:      decode to SQRSHRNB_ZZI_S using decoder 231
                                             // 34006:     }
  3, 7,                                      // 34011:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34013:      check predicate 35
  OPC_Decode, 178, 53, 201, 1,               // 34015:      decode to SQRSHRNT_ZZI_S using decoder 201
                                             // 34015:     }
  4, 7,                                      // 34020:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34022:      check predicate 35
  OPC_Decode, 140, 67, 231, 1,               // 34024:      decode to UQSHRNB_ZZI_S using decoder 231
                                             // 34024:     }
  5, 7,                                      // 34029:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34031:      check predicate 35
  OPC_Decode, 143, 67, 201, 1,               // 34033:      decode to UQSHRNT_ZZI_S using decoder 201
                                             // 34033:     }
  6, 7,                                      // 34038:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34040:      check predicate 35
  OPC_Decode, 212, 66, 231, 1,               // 34042:      decode to UQRSHRNB_ZZI_S using decoder 231
                                             // 34042:     }
  7, 0,                                      // 34047:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34049:      check predicate 35
  OPC_Decode, 215, 66, 201, 1,               // 34051:      decode to UQRSHRNT_ZZI_S using decoder 201
                                             // 34051:     }
                                             // 34051:     } // switch Inst[12:10]
                                             // 34051:    }
  12, 39,                                    // 34056:    case 0xc: {
  OPC_SwitchField, 10, 3,                    // 34058:     switch Inst[12:10] {
  4, 7,                                      // 34061:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34063:      check predicate 35
  OPC_Decode, 156, 46, 184, 1,               // 34065:      decode to SABDLB_ZZZ_S using decoder 184
                                             // 34065:     }
  5, 7,                                      // 34070:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34072:      check predicate 35
  OPC_Decode, 159, 46, 184, 1,               // 34074:      decode to SABDLT_ZZZ_S using decoder 184
                                             // 34074:     }
  6, 7,                                      // 34079:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34081:      check predicate 35
  OPC_Decode, 160, 62, 184, 1,               // 34083:      decode to UABDLB_ZZZ_S using decoder 184
                                             // 34083:     }
  7, 0,                                      // 34088:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34090:      check predicate 35
  OPC_Decode, 163, 62, 184, 1,               // 34092:      decode to UABDLT_ZZZ_S using decoder 184
                                             // 34092:     }
                                             // 34092:     } // switch Inst[12:10]
                                             // 34092:    }
  13, 108,                                   // 34097:    case 0xd: {
  OPC_SwitchField, 10, 3,                    // 34099:     switch Inst[12:10] {
  0, 33,                                     // 34102:     case 0x0: {
  OPC_SwitchField, 20, 1,                    // 34104:      switch Inst[20] {
  0, 15,                                     // 34107:      case 0x0: {
  OPC_CheckPredicate, 39,                    // 34109:       check predicate 39
  OPC_CheckField, 19, 1, 1,                  // 34111:       check Inst[19] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 34115:       check Inst[5] == 0x0
  OPC_Decode, 161, 54, 181, 2,               // 34119:       decode to SQSHRUN_Z2ZI_HtoB using decoder 309
                                             // 34119:      }
  1, 0,                                      // 34124:      case 0x1: {
  OPC_CheckPredicate, 39,                    // 34126:       check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 34128:       check Inst[5] == 0x0
  OPC_Decode, 162, 54, 151, 1,               // 34132:       decode to SQSHRUN_Z2ZI_StoH using decoder 151
                                             // 34132:      }
                                             // 34132:      } // switch Inst[20]
                                             // 34132:     }
  2, 33,                                     // 34137:     case 0x2: {
  OPC_SwitchField, 20, 1,                    // 34139:      switch Inst[20] {
  0, 15,                                     // 34142:      case 0x0: {
  OPC_CheckPredicate, 39,                    // 34144:       check predicate 39
  OPC_CheckField, 19, 1, 1,                  // 34146:       check Inst[19] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 34150:       check Inst[5] == 0x0
  OPC_Decode, 181, 53, 181, 2,               // 34154:       decode to SQRSHRN_Z2ZI_HtoB using decoder 309
                                             // 34154:      }
  1, 0,                                      // 34159:      case 0x1: {
  OPC_CheckPredicate, 18,                    // 34161:       check predicate 18
  OPC_CheckField, 5, 1, 0,                   // 34163:       check Inst[5] == 0x0
  OPC_Decode, 182, 53, 151, 1,               // 34167:       decode to SQRSHRN_Z2ZI_StoH using decoder 151
                                             // 34167:      }
                                             // 34167:      } // switch Inst[20]
                                             // 34167:     }
  6, 0,                                      // 34172:     case 0x6: {
  OPC_SwitchField, 20, 1,                    // 34174:      switch Inst[20] {
  0, 15,                                     // 34177:      case 0x0: {
  OPC_CheckPredicate, 39,                    // 34179:       check predicate 39
  OPC_CheckField, 19, 1, 1,                  // 34181:       check Inst[19] == 0x1
  OPC_CheckField, 5, 1, 0,                   // 34185:       check Inst[5] == 0x0
  OPC_Decode, 218, 66, 181, 2,               // 34189:       decode to UQRSHRN_Z2ZI_HtoB using decoder 309
                                             // 34189:      }
  1, 0,                                      // 34194:      case 0x1: {
  OPC_CheckPredicate, 18,                    // 34196:       check predicate 18
  OPC_CheckField, 5, 1, 0,                   // 34198:       check Inst[5] == 0x0
  OPC_Decode, 219, 66, 151, 1,               // 34202:       decode to UQRSHRN_Z2ZI_StoH using decoder 151
                                             // 34202:      }
                                             // 34202:      } // switch Inst[20]
                                             // 34202:     }
                                             // 34202:     } // switch Inst[12:10]
                                             // 34202:    }
  14, 0,                                     // 34207:    case 0xe: {
  OPC_SwitchField, 10, 3,                    // 34209:     switch Inst[12:10] {
  4, 7,                                      // 34212:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34214:      check predicate 35
  OPC_Decode, 154, 46, 184, 1,               // 34216:      decode to SABDLB_ZZZ_D using decoder 184
                                             // 34216:     }
  5, 7,                                      // 34221:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34223:      check predicate 35
  OPC_Decode, 157, 46, 184, 1,               // 34225:      decode to SABDLT_ZZZ_D using decoder 184
                                             // 34225:     }
  6, 7,                                      // 34230:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34232:      check predicate 35
  OPC_Decode, 158, 62, 184, 1,               // 34234:      decode to UABDLB_ZZZ_D using decoder 184
                                             // 34234:     }
  7, 0,                                      // 34239:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34241:      check predicate 35
  OPC_Decode, 161, 62, 184, 1,               // 34243:      decode to UABDLT_ZZZ_D using decoder 184
                                             // 34243:     }
                                             // 34243:     } // switch Inst[12:10]
                                             // 34243:    }
                                             // 34243:    } // switch Inst[25:21]
                                             // 34243:   }
  2, 242, 5,                                 // 34248:   case 0x2: {
  OPC_SwitchField, 21, 5,                    // 34251:    switch Inst[25:21] {
  2, 75,                                     // 34254:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 34256:     switch Inst[12:10] {
  0, 7,                                      // 34259:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34261:      check predicate 35
  OPC_Decode, 210, 49, 198, 1,               // 34263:      decode to SMLALB_ZZZ_H using decoder 198
                                             // 34263:     }
  1, 7,                                      // 34268:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34270:      check predicate 35
  OPC_Decode, 231, 49, 198, 1,               // 34272:      decode to SMLALT_ZZZ_H using decoder 198
                                             // 34272:     }
  2, 7,                                      // 34277:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34279:      check predicate 35
  OPC_Decode, 219, 64, 198, 1,               // 34281:      decode to UMLALB_ZZZ_H using decoder 198
                                             // 34281:     }
  3, 7,                                      // 34286:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34288:      check predicate 35
  OPC_Decode, 240, 64, 198, 1,               // 34290:      decode to UMLALT_ZZZ_H using decoder 198
                                             // 34290:     }
  4, 7,                                      // 34295:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34297:      check predicate 35
  OPC_Decode, 254, 49, 198, 1,               // 34299:      decode to SMLSLB_ZZZ_H using decoder 198
                                             // 34299:     }
  5, 7,                                      // 34304:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34306:      check predicate 35
  OPC_Decode, 147, 50, 198, 1,               // 34308:      decode to SMLSLT_ZZZ_H using decoder 198
                                             // 34308:     }
  6, 7,                                      // 34313:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34315:      check predicate 35
  OPC_Decode, 135, 65, 198, 1,               // 34317:      decode to UMLSLB_ZZZ_H using decoder 198
                                             // 34317:     }
  7, 0,                                      // 34322:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34324:      check predicate 35
  OPC_Decode, 156, 65, 198, 1,               // 34326:      decode to UMLSLT_ZZZ_H using decoder 198
                                             // 34326:     }
                                             // 34326:     } // switch Inst[12:10]
                                             // 34326:    }
  4, 75,                                     // 34331:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 34333:     switch Inst[12:10] {
  0, 7,                                      // 34336:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34338:      check predicate 35
  OPC_Decode, 211, 49, 198, 1,               // 34340:      decode to SMLALB_ZZZ_S using decoder 198
                                             // 34340:     }
  1, 7,                                      // 34345:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34347:      check predicate 35
  OPC_Decode, 232, 49, 198, 1,               // 34349:      decode to SMLALT_ZZZ_S using decoder 198
                                             // 34349:     }
  2, 7,                                      // 34354:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34356:      check predicate 35
  OPC_Decode, 220, 64, 198, 1,               // 34358:      decode to UMLALB_ZZZ_S using decoder 198
                                             // 34358:     }
  3, 7,                                      // 34363:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34365:      check predicate 35
  OPC_Decode, 241, 64, 198, 1,               // 34367:      decode to UMLALT_ZZZ_S using decoder 198
                                             // 34367:     }
  4, 7,                                      // 34372:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34374:      check predicate 35
  OPC_Decode, 255, 49, 198, 1,               // 34376:      decode to SMLSLB_ZZZ_S using decoder 198
                                             // 34376:     }
  5, 7,                                      // 34381:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34383:      check predicate 35
  OPC_Decode, 148, 50, 198, 1,               // 34385:      decode to SMLSLT_ZZZ_S using decoder 198
                                             // 34385:     }
  6, 7,                                      // 34390:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34392:      check predicate 35
  OPC_Decode, 136, 65, 198, 1,               // 34394:      decode to UMLSLB_ZZZ_S using decoder 198
                                             // 34394:     }
  7, 0,                                      // 34399:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34401:      check predicate 35
  OPC_Decode, 157, 65, 198, 1,               // 34403:      decode to UMLSLT_ZZZ_S using decoder 198
                                             // 34403:     }
                                             // 34403:     } // switch Inst[12:10]
                                             // 34403:    }
  5, 11,                                     // 34408:    case 0x5: {
  OPC_CheckPredicate, 35,                    // 34410:     check predicate 35
  OPC_CheckField, 12, 1, 0,                  // 34412:     check Inst[12] == 0x0
  OPC_Decode, 145, 17, 184, 2,               // 34416:     decode to CDOT_ZZZI_S using decoder 312
                                             // 34416:    }
  6, 75,                                     // 34421:    case 0x6: {
  OPC_SwitchField, 10, 3,                    // 34423:     switch Inst[12:10] {
  0, 7,                                      // 34426:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34428:      check predicate 35
  OPC_Decode, 209, 49, 198, 1,               // 34430:      decode to SMLALB_ZZZ_D using decoder 198
                                             // 34430:     }
  1, 7,                                      // 34435:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34437:      check predicate 35
  OPC_Decode, 230, 49, 198, 1,               // 34439:      decode to SMLALT_ZZZ_D using decoder 198
                                             // 34439:     }
  2, 7,                                      // 34444:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34446:      check predicate 35
  OPC_Decode, 218, 64, 198, 1,               // 34448:      decode to UMLALB_ZZZ_D using decoder 198
                                             // 34448:     }
  3, 7,                                      // 34453:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34455:      check predicate 35
  OPC_Decode, 239, 64, 198, 1,               // 34457:      decode to UMLALT_ZZZ_D using decoder 198
                                             // 34457:     }
  4, 7,                                      // 34462:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34464:      check predicate 35
  OPC_Decode, 253, 49, 198, 1,               // 34466:      decode to SMLSLB_ZZZ_D using decoder 198
                                             // 34466:     }
  5, 7,                                      // 34471:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34473:      check predicate 35
  OPC_Decode, 146, 50, 198, 1,               // 34475:      decode to SMLSLT_ZZZ_D using decoder 198
                                             // 34475:     }
  6, 7,                                      // 34480:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34482:      check predicate 35
  OPC_Decode, 134, 65, 198, 1,               // 34484:      decode to UMLSLB_ZZZ_D using decoder 198
                                             // 34484:     }
  7, 0,                                      // 34489:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34491:      check predicate 35
  OPC_Decode, 155, 65, 198, 1,               // 34493:      decode to UMLSLT_ZZZ_D using decoder 198
                                             // 34493:     }
                                             // 34493:     } // switch Inst[12:10]
                                             // 34493:    }
  7, 11,                                     // 34498:    case 0x7: {
  OPC_CheckPredicate, 35,                    // 34500:     check predicate 35
  OPC_CheckField, 12, 1, 0,                  // 34502:     check Inst[12] == 0x0
  OPC_Decode, 144, 17, 185, 2,               // 34506:     decode to CDOT_ZZZI_D using decoder 313
                                             // 34506:    }
  9, 177, 1,                                 // 34511:    case 0x9: {
  OPC_SwitchField, 10, 3,                    // 34514:     switch Inst[12:10] {
  0, 33,                                     // 34517:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 34519:      switch Inst[20:16] {
  8, 7,                                      // 34522:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 34524:       check predicate 35
  OPC_Decode, 199, 54, 212, 1,               // 34526:       decode to SQXTNB_ZZ_B using decoder 212
                                             // 34526:      }
  16, 7,                                     // 34531:      case 0x10: {
  OPC_CheckPredicate, 35,                    // 34533:       check predicate 35
  OPC_Decode, 200, 54, 212, 1,               // 34535:       decode to SQXTNB_ZZ_H using decoder 212
                                             // 34535:      }
  17, 0,                                     // 34540:      case 0x11: {
  OPC_CheckPredicate, 18,                    // 34542:       check predicate 18
  OPC_CheckField, 5, 1, 0,                   // 34544:       check Inst[5] == 0x0
  OPC_Decode, 165, 51, 121,                  // 34548:       decode to SQCVTN_Z2Z_StoH using decoder 121
                                             // 34548:      }
                                             // 34548:      } // switch Inst[20:16]
                                             // 34548:     }
  1, 21,                                     // 34552:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 34554:      switch Inst[20:16] {
  8, 7,                                      // 34557:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 34559:       check predicate 35
  OPC_Decode, 202, 54, 213, 1,               // 34561:       decode to SQXTNT_ZZ_B using decoder 213
                                             // 34561:      }
  16, 0,                                     // 34566:      case 0x10: {
  OPC_CheckPredicate, 35,                    // 34568:       check predicate 35
  OPC_Decode, 203, 54, 213, 1,               // 34570:       decode to SQXTNT_ZZ_H using decoder 213
                                             // 34570:      }
                                             // 34570:      } // switch Inst[20:16]
                                             // 34570:     }
  2, 33,                                     // 34575:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 34577:      switch Inst[20:16] {
  8, 7,                                      // 34580:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 34582:       check predicate 35
  OPC_Decode, 182, 67, 212, 1,               // 34584:       decode to UQXTNB_ZZ_B using decoder 212
                                             // 34584:      }
  16, 7,                                     // 34589:      case 0x10: {
  OPC_CheckPredicate, 35,                    // 34591:       check predicate 35
  OPC_Decode, 183, 67, 212, 1,               // 34593:       decode to UQXTNB_ZZ_H using decoder 212
                                             // 34593:      }
  17, 0,                                     // 34598:      case 0x11: {
  OPC_CheckPredicate, 18,                    // 34600:       check predicate 18
  OPC_CheckField, 5, 1, 0,                   // 34602:       check Inst[5] == 0x0
  OPC_Decode, 141, 66, 121,                  // 34606:       decode to UQCVTN_Z2Z_StoH using decoder 121
                                             // 34606:      }
                                             // 34606:      } // switch Inst[20:16]
                                             // 34606:     }
  3, 21,                                     // 34610:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 34612:      switch Inst[20:16] {
  8, 7,                                      // 34615:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 34617:       check predicate 35
  OPC_Decode, 185, 67, 213, 1,               // 34619:       decode to UQXTNT_ZZ_B using decoder 213
                                             // 34619:      }
  16, 0,                                     // 34624:      case 0x10: {
  OPC_CheckPredicate, 35,                    // 34626:       check predicate 35
  OPC_Decode, 186, 67, 213, 1,               // 34628:       decode to UQXTNT_ZZ_H using decoder 213
                                             // 34628:      }
                                             // 34628:      } // switch Inst[20:16]
                                             // 34628:     }
  4, 33,                                     // 34633:     case 0x4: {
  OPC_SwitchField, 16, 5,                    // 34635:      switch Inst[20:16] {
  8, 7,                                      // 34638:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 34640:       check predicate 35
  OPC_Decode, 214, 54, 212, 1,               // 34642:       decode to SQXTUNB_ZZ_B using decoder 212
                                             // 34642:      }
  16, 7,                                     // 34647:      case 0x10: {
  OPC_CheckPredicate, 35,                    // 34649:       check predicate 35
  OPC_Decode, 215, 54, 212, 1,               // 34651:       decode to SQXTUNB_ZZ_H using decoder 212
                                             // 34651:      }
  17, 0,                                     // 34656:      case 0x11: {
  OPC_CheckPredicate, 18,                    // 34658:       check predicate 18
  OPC_CheckField, 5, 1, 0,                   // 34660:       check Inst[5] == 0x0
  OPC_Decode, 168, 51, 121,                  // 34664:       decode to SQCVTUN_Z2Z_StoH using decoder 121
                                             // 34664:      }
                                             // 34664:      } // switch Inst[20:16]
                                             // 34664:     }
  5, 0,                                      // 34668:     case 0x5: {
  OPC_SwitchField, 16, 5,                    // 34670:      switch Inst[20:16] {
  8, 7,                                      // 34673:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 34675:       check predicate 35
  OPC_Decode, 217, 54, 213, 1,               // 34677:       decode to SQXTUNT_ZZ_B using decoder 213
                                             // 34677:      }
  16, 0,                                     // 34682:      case 0x10: {
  OPC_CheckPredicate, 35,                    // 34684:       check predicate 35
  OPC_Decode, 218, 54, 213, 1,               // 34686:       decode to SQXTUNT_ZZ_H using decoder 213
                                             // 34686:      }
                                             // 34686:      } // switch Inst[20:16]
                                             // 34686:     }
                                             // 34686:     } // switch Inst[12:10]
                                             // 34686:    }
  10, 75,                                    // 34691:    case 0xa: {
  OPC_SwitchField, 10, 3,                    // 34693:     switch Inst[12:10] {
  0, 7,                                      // 34696:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34698:      check predicate 35
  OPC_Decode, 215, 46, 184, 1,               // 34700:      decode to SADDWB_ZZZ_H using decoder 184
                                             // 34700:     }
  1, 7,                                      // 34705:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34707:      check predicate 35
  OPC_Decode, 218, 46, 184, 1,               // 34709:      decode to SADDWT_ZZZ_H using decoder 184
                                             // 34709:     }
  2, 7,                                      // 34714:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34716:      check predicate 35
  OPC_Decode, 217, 62, 184, 1,               // 34718:      decode to UADDWB_ZZZ_H using decoder 184
                                             // 34718:     }
  3, 7,                                      // 34723:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34725:      check predicate 35
  OPC_Decode, 220, 62, 184, 1,               // 34727:      decode to UADDWT_ZZZ_H using decoder 184
                                             // 34727:     }
  4, 7,                                      // 34732:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34734:      check predicate 35
  OPC_Decode, 149, 56, 184, 1,               // 34736:      decode to SSUBWB_ZZZ_H using decoder 184
                                             // 34736:     }
  5, 7,                                      // 34741:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34743:      check predicate 35
  OPC_Decode, 152, 56, 184, 1,               // 34745:      decode to SSUBWT_ZZZ_H using decoder 184
                                             // 34745:     }
  6, 7,                                      // 34750:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34752:      check predicate 35
  OPC_Decode, 254, 68, 184, 1,               // 34754:      decode to USUBWB_ZZZ_H using decoder 184
                                             // 34754:     }
  7, 0,                                      // 34759:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34761:      check predicate 35
  OPC_Decode, 129, 69, 184, 1,               // 34763:      decode to USUBWT_ZZZ_H using decoder 184
                                             // 34763:     }
                                             // 34763:     } // switch Inst[12:10]
                                             // 34763:    }
  11, 81,                                    // 34768:    case 0xb: {
  OPC_SwitchField, 10, 3,                    // 34770:     switch Inst[12:10] {
  0, 11,                                     // 34773:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34775:      check predicate 35
  OPC_CheckField, 16, 5, 0,                  // 34777:      check Inst[20:16] == 0x0
  OPC_Decode, 201, 54, 212, 1,               // 34781:      decode to SQXTNB_ZZ_S using decoder 212
                                             // 34781:     }
  1, 11,                                     // 34786:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34788:      check predicate 35
  OPC_CheckField, 16, 5, 0,                  // 34790:      check Inst[20:16] == 0x0
  OPC_Decode, 204, 54, 213, 1,               // 34794:      decode to SQXTNT_ZZ_S using decoder 213
                                             // 34794:     }
  2, 11,                                     // 34799:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34801:      check predicate 35
  OPC_CheckField, 16, 5, 0,                  // 34803:      check Inst[20:16] == 0x0
  OPC_Decode, 184, 67, 212, 1,               // 34807:      decode to UQXTNB_ZZ_S using decoder 212
                                             // 34807:     }
  3, 11,                                     // 34812:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34814:      check predicate 35
  OPC_CheckField, 16, 5, 0,                  // 34816:      check Inst[20:16] == 0x0
  OPC_Decode, 187, 67, 213, 1,               // 34820:      decode to UQXTNT_ZZ_S using decoder 213
                                             // 34820:     }
  4, 11,                                     // 34825:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34827:      check predicate 35
  OPC_CheckField, 16, 5, 0,                  // 34829:      check Inst[20:16] == 0x0
  OPC_Decode, 216, 54, 212, 1,               // 34833:      decode to SQXTUNB_ZZ_S using decoder 212
                                             // 34833:     }
  5, 0,                                      // 34838:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34840:      check predicate 35
  OPC_CheckField, 16, 5, 0,                  // 34842:      check Inst[20:16] == 0x0
  OPC_Decode, 219, 54, 213, 1,               // 34846:      decode to SQXTUNT_ZZ_S using decoder 213
                                             // 34846:     }
                                             // 34846:     } // switch Inst[12:10]
                                             // 34846:    }
  12, 75,                                    // 34851:    case 0xc: {
  OPC_SwitchField, 10, 3,                    // 34853:     switch Inst[12:10] {
  0, 7,                                      // 34856:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34858:      check predicate 35
  OPC_Decode, 216, 46, 184, 1,               // 34860:      decode to SADDWB_ZZZ_S using decoder 184
                                             // 34860:     }
  1, 7,                                      // 34865:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34867:      check predicate 35
  OPC_Decode, 219, 46, 184, 1,               // 34869:      decode to SADDWT_ZZZ_S using decoder 184
                                             // 34869:     }
  2, 7,                                      // 34874:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34876:      check predicate 35
  OPC_Decode, 218, 62, 184, 1,               // 34878:      decode to UADDWB_ZZZ_S using decoder 184
                                             // 34878:     }
  3, 7,                                      // 34883:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34885:      check predicate 35
  OPC_Decode, 221, 62, 184, 1,               // 34887:      decode to UADDWT_ZZZ_S using decoder 184
                                             // 34887:     }
  4, 7,                                      // 34892:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34894:      check predicate 35
  OPC_Decode, 150, 56, 184, 1,               // 34896:      decode to SSUBWB_ZZZ_S using decoder 184
                                             // 34896:     }
  5, 7,                                      // 34901:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34903:      check predicate 35
  OPC_Decode, 153, 56, 184, 1,               // 34905:      decode to SSUBWT_ZZZ_S using decoder 184
                                             // 34905:     }
  6, 7,                                      // 34910:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34912:      check predicate 35
  OPC_Decode, 255, 68, 184, 1,               // 34914:      decode to USUBWB_ZZZ_S using decoder 184
                                             // 34914:     }
  7, 0,                                      // 34919:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34921:      check predicate 35
  OPC_Decode, 130, 69, 184, 1,               // 34923:      decode to USUBWT_ZZZ_S using decoder 184
                                             // 34923:     }
                                             // 34923:     } // switch Inst[12:10]
                                             // 34923:    }
  14, 0,                                     // 34928:    case 0xe: {
  OPC_SwitchField, 10, 3,                    // 34930:     switch Inst[12:10] {
  0, 7,                                      // 34933:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 34935:      check predicate 35
  OPC_Decode, 214, 46, 184, 1,               // 34937:      decode to SADDWB_ZZZ_D using decoder 184
                                             // 34937:     }
  1, 7,                                      // 34942:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 34944:      check predicate 35
  OPC_Decode, 217, 46, 184, 1,               // 34946:      decode to SADDWT_ZZZ_D using decoder 184
                                             // 34946:     }
  2, 7,                                      // 34951:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 34953:      check predicate 35
  OPC_Decode, 216, 62, 184, 1,               // 34955:      decode to UADDWB_ZZZ_D using decoder 184
                                             // 34955:     }
  3, 7,                                      // 34960:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 34962:      check predicate 35
  OPC_Decode, 219, 62, 184, 1,               // 34964:      decode to UADDWT_ZZZ_D using decoder 184
                                             // 34964:     }
  4, 7,                                      // 34969:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 34971:      check predicate 35
  OPC_Decode, 148, 56, 184, 1,               // 34973:      decode to SSUBWB_ZZZ_D using decoder 184
                                             // 34973:     }
  5, 7,                                      // 34978:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 34980:      check predicate 35
  OPC_Decode, 151, 56, 184, 1,               // 34982:      decode to SSUBWT_ZZZ_D using decoder 184
                                             // 34982:     }
  6, 7,                                      // 34987:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 34989:      check predicate 35
  OPC_Decode, 253, 68, 184, 1,               // 34991:      decode to USUBWB_ZZZ_D using decoder 184
                                             // 34991:     }
  7, 0,                                      // 34996:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 34998:      check predicate 35
  OPC_Decode, 128, 69, 184, 1,               // 35000:      decode to USUBWT_ZZZ_D using decoder 184
                                             // 35000:     }
                                             // 35000:     } // switch Inst[12:10]
                                             // 35000:    }
                                             // 35000:    } // switch Inst[25:21]
                                             // 35000:   }
  3, 213, 5,                                 // 35005:   case 0x3: {
  OPC_SwitchField, 21, 5,                    // 35008:    switch Inst[25:21] {
  0, 21,                                     // 35011:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 35013:     switch Inst[12:10] {
  4, 7,                                      // 35016:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35018:      check predicate 35
  OPC_Decode, 228, 52, 198, 1,               // 35020:      decode to SQRDMLAH_ZZZ_B using decoder 198
                                             // 35020:     }
  5, 0,                                      // 35025:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35027:      check predicate 35
  OPC_Decode, 247, 52, 198, 1,               // 35029:      decode to SQRDMLSH_ZZZ_B using decoder 198
                                             // 35029:     }
                                             // 35029:     } // switch Inst[12:10]
                                             // 35029:    }
  2, 57,                                     // 35034:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 35036:     switch Inst[12:10] {
  0, 7,                                      // 35039:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35041:      check predicate 35
  OPC_Decode, 205, 51, 198, 1,               // 35043:      decode to SQDMLALB_ZZZ_H using decoder 198
                                             // 35043:     }
  1, 7,                                      // 35048:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35050:      check predicate 35
  OPC_Decode, 210, 51, 198, 1,               // 35052:      decode to SQDMLALT_ZZZ_H using decoder 198
                                             // 35052:     }
  2, 7,                                      // 35057:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35059:      check predicate 35
  OPC_Decode, 230, 51, 198, 1,               // 35061:      decode to SQDMLSLB_ZZZ_H using decoder 198
                                             // 35061:     }
  3, 7,                                      // 35066:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35068:      check predicate 35
  OPC_Decode, 235, 51, 198, 1,               // 35070:      decode to SQDMLSLT_ZZZ_H using decoder 198
                                             // 35070:     }
  4, 7,                                      // 35075:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35077:      check predicate 35
  OPC_Decode, 230, 52, 198, 1,               // 35079:      decode to SQRDMLAH_ZZZ_H using decoder 198
                                             // 35079:     }
  5, 0,                                      // 35084:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35086:      check predicate 35
  OPC_Decode, 249, 52, 198, 1,               // 35088:      decode to SQRDMLSH_ZZZ_H using decoder 198
                                             // 35088:     }
                                             // 35088:     } // switch Inst[12:10]
                                             // 35088:    }
  4, 66,                                     // 35093:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 35095:     switch Inst[12:10] {
  0, 7,                                      // 35098:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35100:      check predicate 35
  OPC_Decode, 206, 51, 198, 1,               // 35102:      decode to SQDMLALB_ZZZ_S using decoder 198
                                             // 35102:     }
  1, 7,                                      // 35107:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35109:      check predicate 35
  OPC_Decode, 211, 51, 198, 1,               // 35111:      decode to SQDMLALT_ZZZ_S using decoder 198
                                             // 35111:     }
  2, 7,                                      // 35116:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35118:      check predicate 35
  OPC_Decode, 231, 51, 198, 1,               // 35120:      decode to SQDMLSLB_ZZZ_S using decoder 198
                                             // 35120:     }
  3, 7,                                      // 35125:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35127:      check predicate 35
  OPC_Decode, 236, 51, 198, 1,               // 35129:      decode to SQDMLSLT_ZZZ_S using decoder 198
                                             // 35129:     }
  4, 7,                                      // 35134:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35136:      check predicate 35
  OPC_Decode, 231, 52, 198, 1,               // 35138:      decode to SQRDMLAH_ZZZ_S using decoder 198
                                             // 35138:     }
  5, 7,                                      // 35143:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35145:      check predicate 35
  OPC_Decode, 250, 52, 198, 1,               // 35147:      decode to SQRDMLSH_ZZZ_S using decoder 198
                                             // 35147:     }
  6, 0,                                      // 35152:     case 0x6: {
  OPC_CheckPredicate, 45,                    // 35154:      check predicate 45
  OPC_Decode, 149, 68, 198, 1,               // 35156:      decode to USDOT_ZZZ using decoder 198
                                             // 35156:     }
                                             // 35156:     } // switch Inst[12:10]
                                             // 35156:    }
  5, 21,                                     // 35161:    case 0x5: {
  OPC_SwitchField, 12, 1,                    // 35163:     switch Inst[12] {
  0, 7,                                      // 35166:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35168:      check predicate 35
  OPC_Decode, 143, 18, 184, 2,               // 35170:      decode to CMLA_ZZZI_H using decoder 312
                                             // 35170:     }
  1, 0,                                      // 35175:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35177:      check predicate 35
  OPC_Decode, 219, 52, 184, 2,               // 35179:      decode to SQRDCMLAH_ZZZI_H using decoder 312
                                             // 35179:     }
                                             // 35179:     } // switch Inst[12]
                                             // 35179:    }
  6, 57,                                     // 35184:    case 0x6: {
  OPC_SwitchField, 10, 3,                    // 35186:     switch Inst[12:10] {
  0, 7,                                      // 35189:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35191:      check predicate 35
  OPC_Decode, 204, 51, 198, 1,               // 35193:      decode to SQDMLALB_ZZZ_D using decoder 198
                                             // 35193:     }
  1, 7,                                      // 35198:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35200:      check predicate 35
  OPC_Decode, 209, 51, 198, 1,               // 35202:      decode to SQDMLALT_ZZZ_D using decoder 198
                                             // 35202:     }
  2, 7,                                      // 35207:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35209:      check predicate 35
  OPC_Decode, 229, 51, 198, 1,               // 35211:      decode to SQDMLSLB_ZZZ_D using decoder 198
                                             // 35211:     }
  3, 7,                                      // 35216:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35218:      check predicate 35
  OPC_Decode, 234, 51, 198, 1,               // 35220:      decode to SQDMLSLT_ZZZ_D using decoder 198
                                             // 35220:     }
  4, 7,                                      // 35225:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35227:      check predicate 35
  OPC_Decode, 229, 52, 198, 1,               // 35229:      decode to SQRDMLAH_ZZZ_D using decoder 198
                                             // 35229:     }
  5, 0,                                      // 35234:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35236:      check predicate 35
  OPC_Decode, 248, 52, 198, 1,               // 35238:      decode to SQRDMLSH_ZZZ_D using decoder 198
                                             // 35238:     }
                                             // 35238:     } // switch Inst[12:10]
                                             // 35238:    }
  7, 21,                                     // 35243:    case 0x7: {
  OPC_SwitchField, 12, 1,                    // 35245:     switch Inst[12] {
  0, 7,                                      // 35248:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35250:      check predicate 35
  OPC_Decode, 144, 18, 185, 2,               // 35252:      decode to CMLA_ZZZI_S using decoder 313
                                             // 35252:     }
  1, 0,                                      // 35257:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35259:      check predicate 35
  OPC_Decode, 220, 52, 185, 2,               // 35261:      decode to SQRDCMLAH_ZZZI_S using decoder 313
                                             // 35261:     }
                                             // 35261:     } // switch Inst[12]
                                             // 35261:    }
  8, 21,                                     // 35266:    case 0x8: {
  OPC_SwitchField, 10, 3,                    // 35268:     switch Inst[12:10] {
  2, 7,                                      // 35271:     case 0x2: {
  OPC_CheckPredicate, 46,                    // 35273:      check predicate 46
  OPC_Decode, 140, 44, 184, 1,               // 35275:      decode to PMULLB_ZZZ_Q using decoder 184
                                             // 35275:     }
  3, 0,                                      // 35280:     case 0x3: {
  OPC_CheckPredicate, 46,                    // 35282:      check predicate 46
  OPC_Decode, 143, 44, 184, 1,               // 35284:      decode to PMULLT_ZZZ_Q using decoder 184
                                             // 35284:     }
                                             // 35284:     } // switch Inst[12:10]
                                             // 35284:    }
  10, 75,                                    // 35289:    case 0xa: {
  OPC_SwitchField, 10, 3,                    // 35291:     switch Inst[12:10] {
  0, 7,                                      // 35294:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35296:      check predicate 35
  OPC_Decode, 159, 52, 184, 1,               // 35298:      decode to SQDMULLB_ZZZ_H using decoder 184
                                             // 35298:     }
  1, 7,                                      // 35303:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35305:      check predicate 35
  OPC_Decode, 164, 52, 184, 1,               // 35307:      decode to SQDMULLT_ZZZ_H using decoder 184
                                             // 35307:     }
  2, 7,                                      // 35312:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35314:      check predicate 35
  OPC_Decode, 139, 44, 184, 1,               // 35316:      decode to PMULLB_ZZZ_H using decoder 184
                                             // 35316:     }
  3, 7,                                      // 35321:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35323:      check predicate 35
  OPC_Decode, 142, 44, 184, 1,               // 35325:      decode to PMULLT_ZZZ_H using decoder 184
                                             // 35325:     }
  4, 7,                                      // 35330:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35332:      check predicate 35
  OPC_Decode, 222, 50, 184, 1,               // 35334:      decode to SMULLB_ZZZ_H using decoder 184
                                             // 35334:     }
  5, 7,                                      // 35339:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35341:      check predicate 35
  OPC_Decode, 227, 50, 184, 1,               // 35343:      decode to SMULLT_ZZZ_H using decoder 184
                                             // 35343:     }
  6, 7,                                      // 35348:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35350:      check predicate 35
  OPC_Decode, 229, 65, 184, 1,               // 35352:      decode to UMULLB_ZZZ_H using decoder 184
                                             // 35352:     }
  7, 0,                                      // 35357:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35359:      check predicate 35
  OPC_Decode, 234, 65, 184, 1,               // 35361:      decode to UMULLT_ZZZ_H using decoder 184
                                             // 35361:     }
                                             // 35361:     } // switch Inst[12:10]
                                             // 35361:    }
  11, 75,                                    // 35366:    case 0xb: {
  OPC_SwitchField, 10, 3,                    // 35368:     switch Inst[12:10] {
  0, 7,                                      // 35371:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35373:      check predicate 35
  OPC_Decode, 137, 13, 184, 1,               // 35375:      decode to ADDHNB_ZZZ_B using decoder 184
                                             // 35375:     }
  1, 7,                                      // 35380:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35382:      check predicate 35
  OPC_Decode, 140, 13, 198, 1,               // 35384:      decode to ADDHNT_ZZZ_B using decoder 198
                                             // 35384:     }
  2, 7,                                      // 35389:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35391:      check predicate 35
  OPC_Decode, 216, 44, 184, 1,               // 35393:      decode to RADDHNB_ZZZ_B using decoder 184
                                             // 35393:     }
  3, 7,                                      // 35398:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35400:      check predicate 35
  OPC_Decode, 219, 44, 198, 1,               // 35402:      decode to RADDHNT_ZZZ_B using decoder 198
                                             // 35402:     }
  4, 7,                                      // 35407:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35409:      check predicate 35
  OPC_Decode, 253, 59, 184, 1,               // 35411:      decode to SUBHNB_ZZZ_B using decoder 184
                                             // 35411:     }
  5, 7,                                      // 35416:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35418:      check predicate 35
  OPC_Decode, 128, 60, 198, 1,               // 35420:      decode to SUBHNT_ZZZ_B using decoder 198
                                             // 35420:     }
  6, 7,                                      // 35425:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35427:      check predicate 35
  OPC_Decode, 245, 45, 184, 1,               // 35429:      decode to RSUBHNB_ZZZ_B using decoder 184
                                             // 35429:     }
  7, 0,                                      // 35434:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35436:      check predicate 35
  OPC_Decode, 248, 45, 198, 1,               // 35438:      decode to RSUBHNT_ZZZ_B using decoder 198
                                             // 35438:     }
                                             // 35438:     } // switch Inst[12:10]
                                             // 35438:    }
  12, 57,                                    // 35443:    case 0xc: {
  OPC_SwitchField, 10, 3,                    // 35445:     switch Inst[12:10] {
  0, 7,                                      // 35448:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35450:      check predicate 35
  OPC_Decode, 160, 52, 184, 1,               // 35452:      decode to SQDMULLB_ZZZ_S using decoder 184
                                             // 35452:     }
  1, 7,                                      // 35457:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35459:      check predicate 35
  OPC_Decode, 165, 52, 184, 1,               // 35461:      decode to SQDMULLT_ZZZ_S using decoder 184
                                             // 35461:     }
  4, 7,                                      // 35466:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35468:      check predicate 35
  OPC_Decode, 223, 50, 184, 1,               // 35470:      decode to SMULLB_ZZZ_S using decoder 184
                                             // 35470:     }
  5, 7,                                      // 35475:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35477:      check predicate 35
  OPC_Decode, 228, 50, 184, 1,               // 35479:      decode to SMULLT_ZZZ_S using decoder 184
                                             // 35479:     }
  6, 7,                                      // 35484:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35486:      check predicate 35
  OPC_Decode, 230, 65, 184, 1,               // 35488:      decode to UMULLB_ZZZ_S using decoder 184
                                             // 35488:     }
  7, 0,                                      // 35493:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35495:      check predicate 35
  OPC_Decode, 235, 65, 184, 1,               // 35497:      decode to UMULLT_ZZZ_S using decoder 184
                                             // 35497:     }
                                             // 35497:     } // switch Inst[12:10]
                                             // 35497:    }
  13, 75,                                    // 35502:    case 0xd: {
  OPC_SwitchField, 10, 3,                    // 35504:     switch Inst[12:10] {
  0, 7,                                      // 35507:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35509:      check predicate 35
  OPC_Decode, 138, 13, 184, 1,               // 35511:      decode to ADDHNB_ZZZ_H using decoder 184
                                             // 35511:     }
  1, 7,                                      // 35516:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35518:      check predicate 35
  OPC_Decode, 141, 13, 198, 1,               // 35520:      decode to ADDHNT_ZZZ_H using decoder 198
                                             // 35520:     }
  2, 7,                                      // 35525:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35527:      check predicate 35
  OPC_Decode, 217, 44, 184, 1,               // 35529:      decode to RADDHNB_ZZZ_H using decoder 184
                                             // 35529:     }
  3, 7,                                      // 35534:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35536:      check predicate 35
  OPC_Decode, 220, 44, 198, 1,               // 35538:      decode to RADDHNT_ZZZ_H using decoder 198
                                             // 35538:     }
  4, 7,                                      // 35543:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35545:      check predicate 35
  OPC_Decode, 254, 59, 184, 1,               // 35547:      decode to SUBHNB_ZZZ_H using decoder 184
                                             // 35547:     }
  5, 7,                                      // 35552:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35554:      check predicate 35
  OPC_Decode, 129, 60, 198, 1,               // 35556:      decode to SUBHNT_ZZZ_H using decoder 198
                                             // 35556:     }
  6, 7,                                      // 35561:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35563:      check predicate 35
  OPC_Decode, 246, 45, 184, 1,               // 35565:      decode to RSUBHNB_ZZZ_H using decoder 184
                                             // 35565:     }
  7, 0,                                      // 35570:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35572:      check predicate 35
  OPC_Decode, 249, 45, 198, 1,               // 35574:      decode to RSUBHNT_ZZZ_H using decoder 198
                                             // 35574:     }
                                             // 35574:     } // switch Inst[12:10]
                                             // 35574:    }
  14, 75,                                    // 35579:    case 0xe: {
  OPC_SwitchField, 10, 3,                    // 35581:     switch Inst[12:10] {
  0, 7,                                      // 35584:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35586:      check predicate 35
  OPC_Decode, 158, 52, 184, 1,               // 35588:      decode to SQDMULLB_ZZZ_D using decoder 184
                                             // 35588:     }
  1, 7,                                      // 35593:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35595:      check predicate 35
  OPC_Decode, 163, 52, 184, 1,               // 35597:      decode to SQDMULLT_ZZZ_D using decoder 184
                                             // 35597:     }
  2, 7,                                      // 35602:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35604:      check predicate 35
  OPC_Decode, 138, 44, 184, 1,               // 35606:      decode to PMULLB_ZZZ_D using decoder 184
                                             // 35606:     }
  3, 7,                                      // 35611:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35613:      check predicate 35
  OPC_Decode, 141, 44, 184, 1,               // 35615:      decode to PMULLT_ZZZ_D using decoder 184
                                             // 35615:     }
  4, 7,                                      // 35620:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35622:      check predicate 35
  OPC_Decode, 221, 50, 184, 1,               // 35624:      decode to SMULLB_ZZZ_D using decoder 184
                                             // 35624:     }
  5, 7,                                      // 35629:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35631:      check predicate 35
  OPC_Decode, 226, 50, 184, 1,               // 35633:      decode to SMULLT_ZZZ_D using decoder 184
                                             // 35633:     }
  6, 7,                                      // 35638:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35640:      check predicate 35
  OPC_Decode, 228, 65, 184, 1,               // 35642:      decode to UMULLB_ZZZ_D using decoder 184
                                             // 35642:     }
  7, 0,                                      // 35647:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35649:      check predicate 35
  OPC_Decode, 233, 65, 184, 1,               // 35651:      decode to UMULLT_ZZZ_D using decoder 184
                                             // 35651:     }
                                             // 35651:     } // switch Inst[12:10]
                                             // 35651:    }
  15, 0,                                     // 35656:    case 0xf: {
  OPC_SwitchField, 10, 3,                    // 35658:     switch Inst[12:10] {
  0, 7,                                      // 35661:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 35663:      check predicate 35
  OPC_Decode, 139, 13, 184, 1,               // 35665:      decode to ADDHNB_ZZZ_S using decoder 184
                                             // 35665:     }
  1, 7,                                      // 35670:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 35672:      check predicate 35
  OPC_Decode, 142, 13, 198, 1,               // 35674:      decode to ADDHNT_ZZZ_S using decoder 198
                                             // 35674:     }
  2, 7,                                      // 35679:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35681:      check predicate 35
  OPC_Decode, 218, 44, 184, 1,               // 35683:      decode to RADDHNB_ZZZ_S using decoder 184
                                             // 35683:     }
  3, 7,                                      // 35688:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35690:      check predicate 35
  OPC_Decode, 221, 44, 198, 1,               // 35692:      decode to RADDHNT_ZZZ_S using decoder 198
                                             // 35692:     }
  4, 7,                                      // 35697:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 35699:      check predicate 35
  OPC_Decode, 255, 59, 184, 1,               // 35701:      decode to SUBHNB_ZZZ_S using decoder 184
                                             // 35701:     }
  5, 7,                                      // 35706:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 35708:      check predicate 35
  OPC_Decode, 130, 60, 198, 1,               // 35710:      decode to SUBHNT_ZZZ_S using decoder 198
                                             // 35710:     }
  6, 7,                                      // 35715:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35717:      check predicate 35
  OPC_Decode, 247, 45, 184, 1,               // 35719:      decode to RSUBHNB_ZZZ_S using decoder 184
                                             // 35719:     }
  7, 0,                                      // 35724:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35726:      check predicate 35
  OPC_Decode, 250, 45, 198, 1,               // 35728:      decode to RSUBHNT_ZZZ_S using decoder 198
                                             // 35728:     }
                                             // 35728:     } // switch Inst[12:10]
                                             // 35728:    }
                                             // 35728:    } // switch Inst[25:21]
                                             // 35728:   }
  4, 231, 10,                                // 35733:   case 0x4: {
  OPC_SwitchField, 21, 5,                    // 35736:    switch Inst[25:21] {
  0, 255, 1,                                 // 35739:    case 0x0: {
  OPC_SwitchField, 16, 5,                    // 35742:     switch Inst[20:16] {
  2, 7,                                      // 35745:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 35747:      check predicate 35
  OPC_Decode, 143, 55, 160, 1,               // 35749:      decode to SRSHL_ZPmZ_B using decoder 160
                                             // 35749:     }
  3, 7,                                      // 35754:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 35756:      check predicate 35
  OPC_Decode, 231, 67, 160, 1,               // 35758:      decode to URSHL_ZPmZ_B using decoder 160
                                             // 35758:     }
  6, 7,                                      // 35763:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 35765:      check predicate 35
  OPC_Decode, 251, 54, 160, 1,               // 35767:      decode to SRSHLR_ZPmZ_B using decoder 160
                                             // 35767:     }
  7, 7,                                      // 35772:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 35774:      check predicate 35
  OPC_Decode, 211, 67, 160, 1,               // 35776:      decode to URSHLR_ZPmZ_B using decoder 160
                                             // 35776:     }
  8, 7,                                      // 35781:     case 0x8: {
  OPC_CheckPredicate, 35,                    // 35783:      check predicate 35
  OPC_Decode, 240, 53, 160, 1,               // 35785:      decode to SQSHL_ZPmZ_B using decoder 160
                                             // 35785:     }
  9, 7,                                      // 35790:     case 0x9: {
  OPC_CheckPredicate, 35,                    // 35792:      check predicate 35
  OPC_Decode, 240, 66, 160, 1,               // 35794:      decode to UQSHL_ZPmZ_B using decoder 160
                                             // 35794:     }
  10, 7,                                     // 35799:     case 0xa: {
  OPC_CheckPredicate, 35,                    // 35801:      check predicate 35
  OPC_Decode, 158, 53, 160, 1,               // 35803:      decode to SQRSHL_ZPmZ_B using decoder 160
                                             // 35803:     }
  11, 7,                                     // 35808:     case 0xb: {
  OPC_CheckPredicate, 35,                    // 35810:      check predicate 35
  OPC_Decode, 195, 66, 160, 1,               // 35812:      decode to UQRSHL_ZPmZ_B using decoder 160
                                             // 35812:     }
  12, 7,                                     // 35817:     case 0xc: {
  OPC_CheckPredicate, 35,                    // 35819:      check predicate 35
  OPC_Decode, 217, 53, 160, 1,               // 35821:      decode to SQSHLR_ZPmZ_B using decoder 160
                                             // 35821:     }
  13, 7,                                     // 35826:     case 0xd: {
  OPC_CheckPredicate, 35,                    // 35828:      check predicate 35
  OPC_Decode, 232, 66, 160, 1,               // 35830:      decode to UQSHLR_ZPmZ_B using decoder 160
                                             // 35830:     }
  14, 7,                                     // 35835:     case 0xe: {
  OPC_CheckPredicate, 35,                    // 35837:      check predicate 35
  OPC_Decode, 154, 53, 160, 1,               // 35839:      decode to SQRSHLR_ZPmZ_B using decoder 160
                                             // 35839:     }
  15, 7,                                     // 35844:     case 0xf: {
  OPC_CheckPredicate, 35,                    // 35846:      check predicate 35
  OPC_Decode, 191, 66, 160, 1,               // 35848:      decode to UQRSHLR_ZPmZ_B using decoder 160
                                             // 35848:     }
  16, 7,                                     // 35853:     case 0x10: {
  OPC_CheckPredicate, 35,                    // 35855:      check predicate 35
  OPC_Decode, 145, 48, 160, 1,               // 35857:      decode to SHADD_ZPmZ_B using decoder 160
                                             // 35857:     }
  17, 7,                                     // 35862:     case 0x11: {
  OPC_CheckPredicate, 35,                    // 35864:      check predicate 35
  OPC_Decode, 205, 63, 160, 1,               // 35866:      decode to UHADD_ZPmZ_B using decoder 160
                                             // 35866:     }
  18, 7,                                     // 35871:     case 0x12: {
  OPC_CheckPredicate, 35,                    // 35873:      check predicate 35
  OPC_Decode, 185, 48, 160, 1,               // 35875:      decode to SHSUB_ZPmZ_B using decoder 160
                                             // 35875:     }
  19, 7,                                     // 35880:     case 0x13: {
  OPC_CheckPredicate, 35,                    // 35882:      check predicate 35
  OPC_Decode, 219, 63, 160, 1,               // 35884:      decode to UHSUB_ZPmZ_B using decoder 160
                                             // 35884:     }
  20, 7,                                     // 35889:     case 0x14: {
  OPC_CheckPredicate, 35,                    // 35891:      check predicate 35
  OPC_Decode, 229, 54, 160, 1,               // 35893:      decode to SRHADD_ZPmZ_B using decoder 160
                                             // 35893:     }
  21, 7,                                     // 35898:     case 0x15: {
  OPC_CheckPredicate, 35,                    // 35900:      check predicate 35
  OPC_Decode, 201, 67, 160, 1,               // 35902:      decode to URHADD_ZPmZ_B using decoder 160
                                             // 35902:     }
  22, 7,                                     // 35907:     case 0x16: {
  OPC_CheckPredicate, 35,                    // 35909:      check predicate 35
  OPC_Decode, 181, 48, 160, 1,               // 35911:      decode to SHSUBR_ZPmZ_B using decoder 160
                                             // 35911:     }
  23, 7,                                     // 35916:     case 0x17: {
  OPC_CheckPredicate, 35,                    // 35918:      check predicate 35
  OPC_Decode, 215, 63, 160, 1,               // 35920:      decode to UHSUBR_ZPmZ_B using decoder 160
                                             // 35920:     }
  24, 7,                                     // 35925:     case 0x18: {
  OPC_CheckPredicate, 35,                    // 35927:      check predicate 35
  OPC_Decode, 142, 51, 160, 1,               // 35929:      decode to SQADD_ZPmZ_B using decoder 160
                                             // 35929:     }
  25, 7,                                     // 35934:     case 0x19: {
  OPC_CheckPredicate, 35,                    // 35936:      check predicate 35
  OPC_Decode, 250, 65, 160, 1,               // 35938:      decode to UQADD_ZPmZ_B using decoder 160
                                             // 35938:     }
  26, 7,                                     // 35943:     case 0x1a: {
  OPC_CheckPredicate, 35,                    // 35945:      check predicate 35
  OPC_Decode, 180, 54, 160, 1,               // 35947:      decode to SQSUB_ZPmZ_B using decoder 160
                                             // 35947:     }
  27, 7,                                     // 35952:     case 0x1b: {
  OPC_CheckPredicate, 35,                    // 35954:      check predicate 35
  OPC_Decode, 163, 67, 160, 1,               // 35956:      decode to UQSUB_ZPmZ_B using decoder 160
                                             // 35956:     }
  28, 7,                                     // 35961:     case 0x1c: {
  OPC_CheckPredicate, 35,                    // 35963:      check predicate 35
  OPC_Decode, 244, 60, 160, 1,               // 35965:      decode to SUQADD_ZPmZ_B using decoder 160
                                             // 35965:     }
  29, 7,                                     // 35970:     case 0x1d: {
  OPC_CheckPredicate, 35,                    // 35972:      check predicate 35
  OPC_Decode, 213, 68, 160, 1,               // 35974:      decode to USQADD_ZPmZ_B using decoder 160
                                             // 35974:     }
  30, 7,                                     // 35979:     case 0x1e: {
  OPC_CheckPredicate, 35,                    // 35981:      check predicate 35
  OPC_Decode, 172, 54, 160, 1,               // 35983:      decode to SQSUBR_ZPmZ_B using decoder 160
                                             // 35983:     }
  31, 0,                                     // 35988:     case 0x1f: {
  OPC_CheckPredicate, 35,                    // 35990:      check predicate 35
  OPC_Decode, 155, 67, 160, 1,               // 35992:      decode to UQSUBR_ZPmZ_B using decoder 160
                                             // 35992:     }
                                             // 35992:     } // switch Inst[20:16]
                                             // 35992:    }
  2, 255, 1,                                 // 35997:    case 0x2: {
  OPC_SwitchField, 16, 5,                    // 36000:     switch Inst[20:16] {
  2, 7,                                      // 36003:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 36005:      check predicate 35
  OPC_Decode, 145, 55, 160, 1,               // 36007:      decode to SRSHL_ZPmZ_H using decoder 160
                                             // 36007:     }
  3, 7,                                      // 36012:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 36014:      check predicate 35
  OPC_Decode, 233, 67, 160, 1,               // 36016:      decode to URSHL_ZPmZ_H using decoder 160
                                             // 36016:     }
  6, 7,                                      // 36021:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 36023:      check predicate 35
  OPC_Decode, 253, 54, 160, 1,               // 36025:      decode to SRSHLR_ZPmZ_H using decoder 160
                                             // 36025:     }
  7, 7,                                      // 36030:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 36032:      check predicate 35
  OPC_Decode, 213, 67, 160, 1,               // 36034:      decode to URSHLR_ZPmZ_H using decoder 160
                                             // 36034:     }
  8, 7,                                      // 36039:     case 0x8: {
  OPC_CheckPredicate, 35,                    // 36041:      check predicate 35
  OPC_Decode, 242, 53, 160, 1,               // 36043:      decode to SQSHL_ZPmZ_H using decoder 160
                                             // 36043:     }
  9, 7,                                      // 36048:     case 0x9: {
  OPC_CheckPredicate, 35,                    // 36050:      check predicate 35
  OPC_Decode, 242, 66, 160, 1,               // 36052:      decode to UQSHL_ZPmZ_H using decoder 160
                                             // 36052:     }
  10, 7,                                     // 36057:     case 0xa: {
  OPC_CheckPredicate, 35,                    // 36059:      check predicate 35
  OPC_Decode, 160, 53, 160, 1,               // 36061:      decode to SQRSHL_ZPmZ_H using decoder 160
                                             // 36061:     }
  11, 7,                                     // 36066:     case 0xb: {
  OPC_CheckPredicate, 35,                    // 36068:      check predicate 35
  OPC_Decode, 197, 66, 160, 1,               // 36070:      decode to UQRSHL_ZPmZ_H using decoder 160
                                             // 36070:     }
  12, 7,                                     // 36075:     case 0xc: {
  OPC_CheckPredicate, 35,                    // 36077:      check predicate 35
  OPC_Decode, 219, 53, 160, 1,               // 36079:      decode to SQSHLR_ZPmZ_H using decoder 160
                                             // 36079:     }
  13, 7,                                     // 36084:     case 0xd: {
  OPC_CheckPredicate, 35,                    // 36086:      check predicate 35
  OPC_Decode, 234, 66, 160, 1,               // 36088:      decode to UQSHLR_ZPmZ_H using decoder 160
                                             // 36088:     }
  14, 7,                                     // 36093:     case 0xe: {
  OPC_CheckPredicate, 35,                    // 36095:      check predicate 35
  OPC_Decode, 156, 53, 160, 1,               // 36097:      decode to SQRSHLR_ZPmZ_H using decoder 160
                                             // 36097:     }
  15, 7,                                     // 36102:     case 0xf: {
  OPC_CheckPredicate, 35,                    // 36104:      check predicate 35
  OPC_Decode, 193, 66, 160, 1,               // 36106:      decode to UQRSHLR_ZPmZ_H using decoder 160
                                             // 36106:     }
  16, 7,                                     // 36111:     case 0x10: {
  OPC_CheckPredicate, 35,                    // 36113:      check predicate 35
  OPC_Decode, 147, 48, 160, 1,               // 36115:      decode to SHADD_ZPmZ_H using decoder 160
                                             // 36115:     }
  17, 7,                                     // 36120:     case 0x11: {
  OPC_CheckPredicate, 35,                    // 36122:      check predicate 35
  OPC_Decode, 207, 63, 160, 1,               // 36124:      decode to UHADD_ZPmZ_H using decoder 160
                                             // 36124:     }
  18, 7,                                     // 36129:     case 0x12: {
  OPC_CheckPredicate, 35,                    // 36131:      check predicate 35
  OPC_Decode, 187, 48, 160, 1,               // 36133:      decode to SHSUB_ZPmZ_H using decoder 160
                                             // 36133:     }
  19, 7,                                     // 36138:     case 0x13: {
  OPC_CheckPredicate, 35,                    // 36140:      check predicate 35
  OPC_Decode, 221, 63, 160, 1,               // 36142:      decode to UHSUB_ZPmZ_H using decoder 160
                                             // 36142:     }
  20, 7,                                     // 36147:     case 0x14: {
  OPC_CheckPredicate, 35,                    // 36149:      check predicate 35
  OPC_Decode, 231, 54, 160, 1,               // 36151:      decode to SRHADD_ZPmZ_H using decoder 160
                                             // 36151:     }
  21, 7,                                     // 36156:     case 0x15: {
  OPC_CheckPredicate, 35,                    // 36158:      check predicate 35
  OPC_Decode, 203, 67, 160, 1,               // 36160:      decode to URHADD_ZPmZ_H using decoder 160
                                             // 36160:     }
  22, 7,                                     // 36165:     case 0x16: {
  OPC_CheckPredicate, 35,                    // 36167:      check predicate 35
  OPC_Decode, 183, 48, 160, 1,               // 36169:      decode to SHSUBR_ZPmZ_H using decoder 160
                                             // 36169:     }
  23, 7,                                     // 36174:     case 0x17: {
  OPC_CheckPredicate, 35,                    // 36176:      check predicate 35
  OPC_Decode, 217, 63, 160, 1,               // 36178:      decode to UHSUBR_ZPmZ_H using decoder 160
                                             // 36178:     }
  24, 7,                                     // 36183:     case 0x18: {
  OPC_CheckPredicate, 35,                    // 36185:      check predicate 35
  OPC_Decode, 144, 51, 160, 1,               // 36187:      decode to SQADD_ZPmZ_H using decoder 160
                                             // 36187:     }
  25, 7,                                     // 36192:     case 0x19: {
  OPC_CheckPredicate, 35,                    // 36194:      check predicate 35
  OPC_Decode, 252, 65, 160, 1,               // 36196:      decode to UQADD_ZPmZ_H using decoder 160
                                             // 36196:     }
  26, 7,                                     // 36201:     case 0x1a: {
  OPC_CheckPredicate, 35,                    // 36203:      check predicate 35
  OPC_Decode, 182, 54, 160, 1,               // 36205:      decode to SQSUB_ZPmZ_H using decoder 160
                                             // 36205:     }
  27, 7,                                     // 36210:     case 0x1b: {
  OPC_CheckPredicate, 35,                    // 36212:      check predicate 35
  OPC_Decode, 165, 67, 160, 1,               // 36214:      decode to UQSUB_ZPmZ_H using decoder 160
                                             // 36214:     }
  28, 7,                                     // 36219:     case 0x1c: {
  OPC_CheckPredicate, 35,                    // 36221:      check predicate 35
  OPC_Decode, 246, 60, 160, 1,               // 36223:      decode to SUQADD_ZPmZ_H using decoder 160
                                             // 36223:     }
  29, 7,                                     // 36228:     case 0x1d: {
  OPC_CheckPredicate, 35,                    // 36230:      check predicate 35
  OPC_Decode, 215, 68, 160, 1,               // 36232:      decode to USQADD_ZPmZ_H using decoder 160
                                             // 36232:     }
  30, 7,                                     // 36237:     case 0x1e: {
  OPC_CheckPredicate, 35,                    // 36239:      check predicate 35
  OPC_Decode, 174, 54, 160, 1,               // 36241:      decode to SQSUBR_ZPmZ_H using decoder 160
                                             // 36241:     }
  31, 0,                                     // 36246:     case 0x1f: {
  OPC_CheckPredicate, 35,                    // 36248:      check predicate 35
  OPC_Decode, 157, 67, 160, 1,               // 36250:      decode to UQSUBR_ZPmZ_H using decoder 160
                                             // 36250:     }
                                             // 36250:     } // switch Inst[20:16]
                                             // 36250:    }
  4, 255, 1,                                 // 36255:    case 0x4: {
  OPC_SwitchField, 16, 5,                    // 36258:     switch Inst[20:16] {
  2, 7,                                      // 36261:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 36263:      check predicate 35
  OPC_Decode, 146, 55, 160, 1,               // 36265:      decode to SRSHL_ZPmZ_S using decoder 160
                                             // 36265:     }
  3, 7,                                      // 36270:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 36272:      check predicate 35
  OPC_Decode, 234, 67, 160, 1,               // 36274:      decode to URSHL_ZPmZ_S using decoder 160
                                             // 36274:     }
  6, 7,                                      // 36279:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 36281:      check predicate 35
  OPC_Decode, 254, 54, 160, 1,               // 36283:      decode to SRSHLR_ZPmZ_S using decoder 160
                                             // 36283:     }
  7, 7,                                      // 36288:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 36290:      check predicate 35
  OPC_Decode, 214, 67, 160, 1,               // 36292:      decode to URSHLR_ZPmZ_S using decoder 160
                                             // 36292:     }
  8, 7,                                      // 36297:     case 0x8: {
  OPC_CheckPredicate, 35,                    // 36299:      check predicate 35
  OPC_Decode, 243, 53, 160, 1,               // 36301:      decode to SQSHL_ZPmZ_S using decoder 160
                                             // 36301:     }
  9, 7,                                      // 36306:     case 0x9: {
  OPC_CheckPredicate, 35,                    // 36308:      check predicate 35
  OPC_Decode, 243, 66, 160, 1,               // 36310:      decode to UQSHL_ZPmZ_S using decoder 160
                                             // 36310:     }
  10, 7,                                     // 36315:     case 0xa: {
  OPC_CheckPredicate, 35,                    // 36317:      check predicate 35
  OPC_Decode, 161, 53, 160, 1,               // 36319:      decode to SQRSHL_ZPmZ_S using decoder 160
                                             // 36319:     }
  11, 7,                                     // 36324:     case 0xb: {
  OPC_CheckPredicate, 35,                    // 36326:      check predicate 35
  OPC_Decode, 198, 66, 160, 1,               // 36328:      decode to UQRSHL_ZPmZ_S using decoder 160
                                             // 36328:     }
  12, 7,                                     // 36333:     case 0xc: {
  OPC_CheckPredicate, 35,                    // 36335:      check predicate 35
  OPC_Decode, 220, 53, 160, 1,               // 36337:      decode to SQSHLR_ZPmZ_S using decoder 160
                                             // 36337:     }
  13, 7,                                     // 36342:     case 0xd: {
  OPC_CheckPredicate, 35,                    // 36344:      check predicate 35
  OPC_Decode, 235, 66, 160, 1,               // 36346:      decode to UQSHLR_ZPmZ_S using decoder 160
                                             // 36346:     }
  14, 7,                                     // 36351:     case 0xe: {
  OPC_CheckPredicate, 35,                    // 36353:      check predicate 35
  OPC_Decode, 157, 53, 160, 1,               // 36355:      decode to SQRSHLR_ZPmZ_S using decoder 160
                                             // 36355:     }
  15, 7,                                     // 36360:     case 0xf: {
  OPC_CheckPredicate, 35,                    // 36362:      check predicate 35
  OPC_Decode, 194, 66, 160, 1,               // 36364:      decode to UQRSHLR_ZPmZ_S using decoder 160
                                             // 36364:     }
  16, 7,                                     // 36369:     case 0x10: {
  OPC_CheckPredicate, 35,                    // 36371:      check predicate 35
  OPC_Decode, 148, 48, 160, 1,               // 36373:      decode to SHADD_ZPmZ_S using decoder 160
                                             // 36373:     }
  17, 7,                                     // 36378:     case 0x11: {
  OPC_CheckPredicate, 35,                    // 36380:      check predicate 35
  OPC_Decode, 208, 63, 160, 1,               // 36382:      decode to UHADD_ZPmZ_S using decoder 160
                                             // 36382:     }
  18, 7,                                     // 36387:     case 0x12: {
  OPC_CheckPredicate, 35,                    // 36389:      check predicate 35
  OPC_Decode, 188, 48, 160, 1,               // 36391:      decode to SHSUB_ZPmZ_S using decoder 160
                                             // 36391:     }
  19, 7,                                     // 36396:     case 0x13: {
  OPC_CheckPredicate, 35,                    // 36398:      check predicate 35
  OPC_Decode, 222, 63, 160, 1,               // 36400:      decode to UHSUB_ZPmZ_S using decoder 160
                                             // 36400:     }
  20, 7,                                     // 36405:     case 0x14: {
  OPC_CheckPredicate, 35,                    // 36407:      check predicate 35
  OPC_Decode, 232, 54, 160, 1,               // 36409:      decode to SRHADD_ZPmZ_S using decoder 160
                                             // 36409:     }
  21, 7,                                     // 36414:     case 0x15: {
  OPC_CheckPredicate, 35,                    // 36416:      check predicate 35
  OPC_Decode, 204, 67, 160, 1,               // 36418:      decode to URHADD_ZPmZ_S using decoder 160
                                             // 36418:     }
  22, 7,                                     // 36423:     case 0x16: {
  OPC_CheckPredicate, 35,                    // 36425:      check predicate 35
  OPC_Decode, 184, 48, 160, 1,               // 36427:      decode to SHSUBR_ZPmZ_S using decoder 160
                                             // 36427:     }
  23, 7,                                     // 36432:     case 0x17: {
  OPC_CheckPredicate, 35,                    // 36434:      check predicate 35
  OPC_Decode, 218, 63, 160, 1,               // 36436:      decode to UHSUBR_ZPmZ_S using decoder 160
                                             // 36436:     }
  24, 7,                                     // 36441:     case 0x18: {
  OPC_CheckPredicate, 35,                    // 36443:      check predicate 35
  OPC_Decode, 145, 51, 160, 1,               // 36445:      decode to SQADD_ZPmZ_S using decoder 160
                                             // 36445:     }
  25, 7,                                     // 36450:     case 0x19: {
  OPC_CheckPredicate, 35,                    // 36452:      check predicate 35
  OPC_Decode, 253, 65, 160, 1,               // 36454:      decode to UQADD_ZPmZ_S using decoder 160
                                             // 36454:     }
  26, 7,                                     // 36459:     case 0x1a: {
  OPC_CheckPredicate, 35,                    // 36461:      check predicate 35
  OPC_Decode, 183, 54, 160, 1,               // 36463:      decode to SQSUB_ZPmZ_S using decoder 160
                                             // 36463:     }
  27, 7,                                     // 36468:     case 0x1b: {
  OPC_CheckPredicate, 35,                    // 36470:      check predicate 35
  OPC_Decode, 166, 67, 160, 1,               // 36472:      decode to UQSUB_ZPmZ_S using decoder 160
                                             // 36472:     }
  28, 7,                                     // 36477:     case 0x1c: {
  OPC_CheckPredicate, 35,                    // 36479:      check predicate 35
  OPC_Decode, 247, 60, 160, 1,               // 36481:      decode to SUQADD_ZPmZ_S using decoder 160
                                             // 36481:     }
  29, 7,                                     // 36486:     case 0x1d: {
  OPC_CheckPredicate, 35,                    // 36488:      check predicate 35
  OPC_Decode, 216, 68, 160, 1,               // 36490:      decode to USQADD_ZPmZ_S using decoder 160
                                             // 36490:     }
  30, 7,                                     // 36495:     case 0x1e: {
  OPC_CheckPredicate, 35,                    // 36497:      check predicate 35
  OPC_Decode, 175, 54, 160, 1,               // 36499:      decode to SQSUBR_ZPmZ_S using decoder 160
                                             // 36499:     }
  31, 0,                                     // 36504:     case 0x1f: {
  OPC_CheckPredicate, 35,                    // 36506:      check predicate 35
  OPC_Decode, 158, 67, 160, 1,               // 36508:      decode to UQSUBR_ZPmZ_S using decoder 160
                                             // 36508:     }
                                             // 36508:     } // switch Inst[20:16]
                                             // 36508:    }
  5, 49,                                     // 36513:    case 0x5: {
  OPC_SwitchField, 10, 1,                    // 36515:     switch Inst[10] {
  0, 21,                                     // 36518:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 36520:      switch Inst[12] {
  0, 7,                                      // 36523:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 36525:       check predicate 35
  OPC_Decode, 208, 49, 182, 2,               // 36527:       decode to SMLALB_ZZZI_S using decoder 310
                                             // 36527:      }
  1, 0,                                      // 36532:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 36534:       check predicate 35
  OPC_Decode, 217, 64, 182, 2,               // 36536:       decode to UMLALB_ZZZI_S using decoder 310
                                             // 36536:      }
                                             // 36536:      } // switch Inst[12]
                                             // 36536:     }
  1, 0,                                      // 36541:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 36543:      switch Inst[12] {
  0, 7,                                      // 36546:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 36548:       check predicate 35
  OPC_Decode, 229, 49, 182, 2,               // 36550:       decode to SMLALT_ZZZI_S using decoder 310
                                             // 36550:      }
  1, 0,                                      // 36555:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 36557:       check predicate 35
  OPC_Decode, 238, 64, 182, 2,               // 36559:       decode to UMLALT_ZZZI_S using decoder 310
                                             // 36559:      }
                                             // 36559:      } // switch Inst[12]
                                             // 36559:     }
                                             // 36559:     } // switch Inst[10]
                                             // 36559:    }
  6, 255, 1,                                 // 36564:    case 0x6: {
  OPC_SwitchField, 16, 5,                    // 36567:     switch Inst[20:16] {
  2, 7,                                      // 36570:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 36572:      check predicate 35
  OPC_Decode, 144, 55, 160, 1,               // 36574:      decode to SRSHL_ZPmZ_D using decoder 160
                                             // 36574:     }
  3, 7,                                      // 36579:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 36581:      check predicate 35
  OPC_Decode, 232, 67, 160, 1,               // 36583:      decode to URSHL_ZPmZ_D using decoder 160
                                             // 36583:     }
  6, 7,                                      // 36588:     case 0x6: {
  OPC_CheckPredicate, 35,                    // 36590:      check predicate 35
  OPC_Decode, 252, 54, 160, 1,               // 36592:      decode to SRSHLR_ZPmZ_D using decoder 160
                                             // 36592:     }
  7, 7,                                      // 36597:     case 0x7: {
  OPC_CheckPredicate, 35,                    // 36599:      check predicate 35
  OPC_Decode, 212, 67, 160, 1,               // 36601:      decode to URSHLR_ZPmZ_D using decoder 160
                                             // 36601:     }
  8, 7,                                      // 36606:     case 0x8: {
  OPC_CheckPredicate, 35,                    // 36608:      check predicate 35
  OPC_Decode, 241, 53, 160, 1,               // 36610:      decode to SQSHL_ZPmZ_D using decoder 160
                                             // 36610:     }
  9, 7,                                      // 36615:     case 0x9: {
  OPC_CheckPredicate, 35,                    // 36617:      check predicate 35
  OPC_Decode, 241, 66, 160, 1,               // 36619:      decode to UQSHL_ZPmZ_D using decoder 160
                                             // 36619:     }
  10, 7,                                     // 36624:     case 0xa: {
  OPC_CheckPredicate, 35,                    // 36626:      check predicate 35
  OPC_Decode, 159, 53, 160, 1,               // 36628:      decode to SQRSHL_ZPmZ_D using decoder 160
                                             // 36628:     }
  11, 7,                                     // 36633:     case 0xb: {
  OPC_CheckPredicate, 35,                    // 36635:      check predicate 35
  OPC_Decode, 196, 66, 160, 1,               // 36637:      decode to UQRSHL_ZPmZ_D using decoder 160
                                             // 36637:     }
  12, 7,                                     // 36642:     case 0xc: {
  OPC_CheckPredicate, 35,                    // 36644:      check predicate 35
  OPC_Decode, 218, 53, 160, 1,               // 36646:      decode to SQSHLR_ZPmZ_D using decoder 160
                                             // 36646:     }
  13, 7,                                     // 36651:     case 0xd: {
  OPC_CheckPredicate, 35,                    // 36653:      check predicate 35
  OPC_Decode, 233, 66, 160, 1,               // 36655:      decode to UQSHLR_ZPmZ_D using decoder 160
                                             // 36655:     }
  14, 7,                                     // 36660:     case 0xe: {
  OPC_CheckPredicate, 35,                    // 36662:      check predicate 35
  OPC_Decode, 155, 53, 160, 1,               // 36664:      decode to SQRSHLR_ZPmZ_D using decoder 160
                                             // 36664:     }
  15, 7,                                     // 36669:     case 0xf: {
  OPC_CheckPredicate, 35,                    // 36671:      check predicate 35
  OPC_Decode, 192, 66, 160, 1,               // 36673:      decode to UQRSHLR_ZPmZ_D using decoder 160
                                             // 36673:     }
  16, 7,                                     // 36678:     case 0x10: {
  OPC_CheckPredicate, 35,                    // 36680:      check predicate 35
  OPC_Decode, 146, 48, 160, 1,               // 36682:      decode to SHADD_ZPmZ_D using decoder 160
                                             // 36682:     }
  17, 7,                                     // 36687:     case 0x11: {
  OPC_CheckPredicate, 35,                    // 36689:      check predicate 35
  OPC_Decode, 206, 63, 160, 1,               // 36691:      decode to UHADD_ZPmZ_D using decoder 160
                                             // 36691:     }
  18, 7,                                     // 36696:     case 0x12: {
  OPC_CheckPredicate, 35,                    // 36698:      check predicate 35
  OPC_Decode, 186, 48, 160, 1,               // 36700:      decode to SHSUB_ZPmZ_D using decoder 160
                                             // 36700:     }
  19, 7,                                     // 36705:     case 0x13: {
  OPC_CheckPredicate, 35,                    // 36707:      check predicate 35
  OPC_Decode, 220, 63, 160, 1,               // 36709:      decode to UHSUB_ZPmZ_D using decoder 160
                                             // 36709:     }
  20, 7,                                     // 36714:     case 0x14: {
  OPC_CheckPredicate, 35,                    // 36716:      check predicate 35
  OPC_Decode, 230, 54, 160, 1,               // 36718:      decode to SRHADD_ZPmZ_D using decoder 160
                                             // 36718:     }
  21, 7,                                     // 36723:     case 0x15: {
  OPC_CheckPredicate, 35,                    // 36725:      check predicate 35
  OPC_Decode, 202, 67, 160, 1,               // 36727:      decode to URHADD_ZPmZ_D using decoder 160
                                             // 36727:     }
  22, 7,                                     // 36732:     case 0x16: {
  OPC_CheckPredicate, 35,                    // 36734:      check predicate 35
  OPC_Decode, 182, 48, 160, 1,               // 36736:      decode to SHSUBR_ZPmZ_D using decoder 160
                                             // 36736:     }
  23, 7,                                     // 36741:     case 0x17: {
  OPC_CheckPredicate, 35,                    // 36743:      check predicate 35
  OPC_Decode, 216, 63, 160, 1,               // 36745:      decode to UHSUBR_ZPmZ_D using decoder 160
                                             // 36745:     }
  24, 7,                                     // 36750:     case 0x18: {
  OPC_CheckPredicate, 35,                    // 36752:      check predicate 35
  OPC_Decode, 143, 51, 160, 1,               // 36754:      decode to SQADD_ZPmZ_D using decoder 160
                                             // 36754:     }
  25, 7,                                     // 36759:     case 0x19: {
  OPC_CheckPredicate, 35,                    // 36761:      check predicate 35
  OPC_Decode, 251, 65, 160, 1,               // 36763:      decode to UQADD_ZPmZ_D using decoder 160
                                             // 36763:     }
  26, 7,                                     // 36768:     case 0x1a: {
  OPC_CheckPredicate, 35,                    // 36770:      check predicate 35
  OPC_Decode, 181, 54, 160, 1,               // 36772:      decode to SQSUB_ZPmZ_D using decoder 160
                                             // 36772:     }
  27, 7,                                     // 36777:     case 0x1b: {
  OPC_CheckPredicate, 35,                    // 36779:      check predicate 35
  OPC_Decode, 164, 67, 160, 1,               // 36781:      decode to UQSUB_ZPmZ_D using decoder 160
                                             // 36781:     }
  28, 7,                                     // 36786:     case 0x1c: {
  OPC_CheckPredicate, 35,                    // 36788:      check predicate 35
  OPC_Decode, 245, 60, 160, 1,               // 36790:      decode to SUQADD_ZPmZ_D using decoder 160
                                             // 36790:     }
  29, 7,                                     // 36795:     case 0x1d: {
  OPC_CheckPredicate, 35,                    // 36797:      check predicate 35
  OPC_Decode, 214, 68, 160, 1,               // 36799:      decode to USQADD_ZPmZ_D using decoder 160
                                             // 36799:     }
  30, 7,                                     // 36804:     case 0x1e: {
  OPC_CheckPredicate, 35,                    // 36806:      check predicate 35
  OPC_Decode, 173, 54, 160, 1,               // 36808:      decode to SQSUBR_ZPmZ_D using decoder 160
                                             // 36808:     }
  31, 0,                                     // 36813:     case 0x1f: {
  OPC_CheckPredicate, 35,                    // 36815:      check predicate 35
  OPC_Decode, 156, 67, 160, 1,               // 36817:      decode to UQSUBR_ZPmZ_D using decoder 160
                                             // 36817:     }
                                             // 36817:     } // switch Inst[20:16]
                                             // 36817:    }
  7, 49,                                     // 36822:    case 0x7: {
  OPC_SwitchField, 10, 1,                    // 36824:     switch Inst[10] {
  0, 21,                                     // 36827:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 36829:      switch Inst[12] {
  0, 7,                                      // 36832:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 36834:       check predicate 35
  OPC_Decode, 207, 49, 183, 2,               // 36836:       decode to SMLALB_ZZZI_D using decoder 311
                                             // 36836:      }
  1, 0,                                      // 36841:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 36843:       check predicate 35
  OPC_Decode, 216, 64, 183, 2,               // 36845:       decode to UMLALB_ZZZI_D using decoder 311
                                             // 36845:      }
                                             // 36845:      } // switch Inst[12]
                                             // 36845:     }
  1, 0,                                      // 36850:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 36852:      switch Inst[12] {
  0, 7,                                      // 36855:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 36857:       check predicate 35
  OPC_Decode, 228, 49, 183, 2,               // 36859:       decode to SMLALT_ZZZI_D using decoder 311
                                             // 36859:      }
  1, 0,                                      // 36864:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 36866:       check predicate 35
  OPC_Decode, 237, 64, 183, 2,               // 36868:       decode to UMLALT_ZZZI_D using decoder 311
                                             // 36868:      }
                                             // 36868:      } // switch Inst[12]
                                             // 36868:     }
                                             // 36868:     } // switch Inst[10]
                                             // 36868:    }
  8, 30,                                     // 36873:    case 0x8: {
  OPC_SwitchField, 10, 3,                    // 36875:     switch Inst[12:10] {
  4, 7,                                      // 36878:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 36880:      check predicate 35
  OPC_Decode, 242, 20, 198, 1,               // 36882:      decode to EORBT_ZZZ_B using decoder 198
                                             // 36882:     }
  5, 7,                                      // 36887:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 36889:      check predicate 35
  OPC_Decode, 251, 20, 198, 1,               // 36891:      decode to EORTB_ZZZ_B using decoder 198
                                             // 36891:     }
  6, 0,                                      // 36896:     case 0x6: {
  OPC_CheckPredicate, 47,                    // 36898:      check predicate 47
  OPC_Decode, 168, 50, 198, 1,               // 36900:      decode to SMMLA_ZZZ using decoder 198
                                             // 36900:     }
                                             // 36900:     } // switch Inst[12:10]
                                             // 36900:    }
  9, 21,                                     // 36905:    case 0x9: {
  OPC_SwitchField, 4, 1,                     // 36907:     switch Inst[4] {
  0, 7,                                      // 36910:     case 0x0: {
  OPC_CheckPredicate, 48,                    // 36912:      check predicate 48
  OPC_Decode, 237, 41, 133, 2,               // 36914:      decode to MATCH_PPzZZ_B using decoder 261
                                             // 36914:     }
  1, 0,                                      // 36919:     case 0x1: {
  OPC_CheckPredicate, 48,                    // 36921:      check predicate 48
  OPC_Decode, 178, 43, 133, 2,               // 36923:      decode to NMATCH_PPzZZ_B using decoder 261
                                             // 36923:     }
                                             // 36923:     } // switch Inst[4]
                                             // 36923:    }
  10, 48,                                    // 36928:    case 0xa: {
  OPC_SwitchField, 10, 3,                    // 36930:     switch Inst[12:10] {
  0, 7,                                      // 36933:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 36935:      check predicate 35
  OPC_Decode, 186, 46, 184, 1,               // 36937:      decode to SADDLBT_ZZZ_H using decoder 184
                                             // 36937:     }
  2, 7,                                      // 36942:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 36944:      check predicate 35
  OPC_Decode, 131, 56, 184, 1,               // 36946:      decode to SSUBLBT_ZZZ_H using decoder 184
                                             // 36946:     }
  3, 7,                                      // 36951:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 36953:      check predicate 35
  OPC_Decode, 137, 56, 184, 1,               // 36955:      decode to SSUBLTB_ZZZ_H using decoder 184
                                             // 36955:     }
  4, 7,                                      // 36960:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 36962:      check predicate 35
  OPC_Decode, 244, 20, 198, 1,               // 36964:      decode to EORBT_ZZZ_H using decoder 198
                                             // 36964:     }
  5, 0,                                      // 36969:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 36971:      check predicate 35
  OPC_Decode, 253, 20, 198, 1,               // 36973:      decode to EORTB_ZZZ_H using decoder 198
                                             // 36973:     }
                                             // 36973:     } // switch Inst[12:10]
                                             // 36973:    }
  11, 21,                                    // 36978:    case 0xb: {
  OPC_SwitchField, 4, 1,                     // 36980:     switch Inst[4] {
  0, 7,                                      // 36983:     case 0x0: {
  OPC_CheckPredicate, 48,                    // 36985:      check predicate 48
  OPC_Decode, 238, 41, 133, 2,               // 36987:      decode to MATCH_PPzZZ_H using decoder 261
                                             // 36987:     }
  1, 0,                                      // 36992:     case 0x1: {
  OPC_CheckPredicate, 48,                    // 36994:      check predicate 48
  OPC_Decode, 179, 43, 133, 2,               // 36996:      decode to NMATCH_PPzZZ_H using decoder 261
                                             // 36996:     }
                                             // 36996:     } // switch Inst[4]
                                             // 36996:    }
  12, 57,                                    // 37001:    case 0xc: {
  OPC_SwitchField, 10, 3,                    // 37003:     switch Inst[12:10] {
  0, 7,                                      // 37006:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 37008:      check predicate 35
  OPC_Decode, 187, 46, 184, 1,               // 37010:      decode to SADDLBT_ZZZ_S using decoder 184
                                             // 37010:     }
  2, 7,                                      // 37015:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 37017:      check predicate 35
  OPC_Decode, 132, 56, 184, 1,               // 37019:      decode to SSUBLBT_ZZZ_S using decoder 184
                                             // 37019:     }
  3, 7,                                      // 37024:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 37026:      check predicate 35
  OPC_Decode, 138, 56, 184, 1,               // 37028:      decode to SSUBLTB_ZZZ_S using decoder 184
                                             // 37028:     }
  4, 7,                                      // 37033:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 37035:      check predicate 35
  OPC_Decode, 245, 20, 198, 1,               // 37037:      decode to EORBT_ZZZ_S using decoder 198
                                             // 37037:     }
  5, 7,                                      // 37042:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 37044:      check predicate 35
  OPC_Decode, 254, 20, 198, 1,               // 37046:      decode to EORTB_ZZZ_S using decoder 198
                                             // 37046:     }
  6, 0,                                      // 37051:     case 0x6: {
  OPC_CheckPredicate, 47,                    // 37053:      check predicate 47
  OPC_Decode, 192, 68, 198, 1,               // 37055:      decode to USMMLA_ZZZ using decoder 198
                                             // 37055:     }
                                             // 37055:     } // switch Inst[12:10]
                                             // 37055:    }
  14, 0,                                     // 37060:    case 0xe: {
  OPC_SwitchField, 10, 3,                    // 37062:     switch Inst[12:10] {
  0, 7,                                      // 37065:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 37067:      check predicate 35
  OPC_Decode, 185, 46, 184, 1,               // 37069:      decode to SADDLBT_ZZZ_D using decoder 184
                                             // 37069:     }
  2, 7,                                      // 37074:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 37076:      check predicate 35
  OPC_Decode, 130, 56, 184, 1,               // 37078:      decode to SSUBLBT_ZZZ_D using decoder 184
                                             // 37078:     }
  3, 7,                                      // 37083:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 37085:      check predicate 35
  OPC_Decode, 136, 56, 184, 1,               // 37087:      decode to SSUBLTB_ZZZ_D using decoder 184
                                             // 37087:     }
  4, 7,                                      // 37092:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 37094:      check predicate 35
  OPC_Decode, 243, 20, 198, 1,               // 37096:      decode to EORBT_ZZZ_D using decoder 198
                                             // 37096:     }
  5, 7,                                      // 37101:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 37103:      check predicate 35
  OPC_Decode, 252, 20, 198, 1,               // 37105:      decode to EORTB_ZZZ_D using decoder 198
                                             // 37105:     }
  6, 0,                                      // 37110:     case 0x6: {
  OPC_CheckPredicate, 47,                    // 37112:      check predicate 47
  OPC_Decode, 177, 65, 198, 1,               // 37114:      decode to UMMLA_ZZZ using decoder 198
                                             // 37114:     }
                                             // 37114:     } // switch Inst[12:10]
                                             // 37114:    }
                                             // 37114:    } // switch Inst[25:21]
                                             // 37114:   }
  5, 216, 7,                                 // 37119:   case 0x5: {
  OPC_SwitchField, 21, 1,                    // 37122:    switch Inst[21] {
  0, 235, 5,                                 // 37125:    case 0x0: {
  OPC_SwitchField, 22, 4,                    // 37128:     switch Inst[25:22] {
  0, 93,                                     // 37131:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 37133:      switch Inst[20:16] {
  8, 7,                                      // 37136:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 37138:       check predicate 35
  OPC_Decode, 247, 50, 163, 1,               // 37140:       decode to SQABS_ZPmZ_B using decoder 163
                                             // 37140:      }
  9, 7,                                      // 37145:      case 0x9: {
  OPC_CheckPredicate, 35,                    // 37147:       check predicate 35
  OPC_Decode, 200, 52, 163, 1,               // 37149:       decode to SQNEG_ZPmZ_B using decoder 163
                                             // 37149:      }
  10, 7,                                     // 37154:      case 0xa: {
  OPC_CheckPredicate, 36,                    // 37156:       check predicate 36
  OPC_Decode, 251, 50, 161, 1,               // 37158:       decode to SQABS_ZPzZ_B using decoder 161
                                             // 37158:      }
  11, 7,                                     // 37163:      case 0xb: {
  OPC_CheckPredicate, 36,                    // 37165:       check predicate 36
  OPC_Decode, 204, 52, 161, 1,               // 37167:       decode to SQNEG_ZPzZ_B using decoder 161
                                             // 37167:      }
  16, 7,                                     // 37172:      case 0x10: {
  OPC_CheckPredicate, 39,                    // 37174:       check predicate 39
  OPC_Decode, 140, 60, 160, 1,               // 37176:       decode to SUBP_ZPmZZ_B using decoder 160
                                             // 37176:      }
  17, 7,                                     // 37181:      case 0x11: {
  OPC_CheckPredicate, 35,                    // 37183:       check predicate 35
  OPC_Decode, 151, 13, 160, 1,               // 37185:       decode to ADDP_ZPmZ_B using decoder 160
                                             // 37185:      }
  20, 7,                                     // 37190:      case 0x14: {
  OPC_CheckPredicate, 35,                    // 37192:       check predicate 35
  OPC_Decode, 220, 48, 160, 1,               // 37194:       decode to SMAXP_ZPmZ_B using decoder 160
                                             // 37194:      }
  21, 7,                                     // 37199:      case 0x15: {
  OPC_CheckPredicate, 35,                    // 37201:       check predicate 35
  OPC_Decode, 230, 63, 160, 1,               // 37203:       decode to UMAXP_ZPmZ_B using decoder 160
                                             // 37203:      }
  22, 7,                                     // 37208:      case 0x16: {
  OPC_CheckPredicate, 35,                    // 37210:       check predicate 35
  OPC_Decode, 150, 49, 160, 1,               // 37212:       decode to SMINP_ZPmZ_B using decoder 160
                                             // 37212:      }
  23, 0,                                     // 37217:      case 0x17: {
  OPC_CheckPredicate, 35,                    // 37219:       check predicate 35
  OPC_Decode, 159, 64, 160, 1,               // 37221:       decode to UMINP_ZPmZ_B using decoder 160
                                             // 37221:      }
                                             // 37221:      } // switch Inst[20:16]
                                             // 37221:     }
  1, 111,                                    // 37226:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 37228:      switch Inst[20:16] {
  4, 7,                                      // 37231:      case 0x4: {
  OPC_CheckPredicate, 35,                    // 37233:       check predicate 35
  OPC_Decode, 177, 46, 160, 1,               // 37235:       decode to SADALP_ZPmZ_H using decoder 160
                                             // 37235:      }
  5, 7,                                      // 37240:      case 0x5: {
  OPC_CheckPredicate, 35,                    // 37242:       check predicate 35
  OPC_Decode, 181, 62, 160, 1,               // 37244:       decode to UADALP_ZPmZ_H using decoder 160
                                             // 37244:      }
  8, 7,                                      // 37249:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 37251:       check predicate 35
  OPC_Decode, 249, 50, 163, 1,               // 37253:       decode to SQABS_ZPmZ_H using decoder 163
                                             // 37253:      }
  9, 7,                                      // 37258:      case 0x9: {
  OPC_CheckPredicate, 35,                    // 37260:       check predicate 35
  OPC_Decode, 202, 52, 163, 1,               // 37262:       decode to SQNEG_ZPmZ_H using decoder 163
                                             // 37262:      }
  10, 7,                                     // 37267:      case 0xa: {
  OPC_CheckPredicate, 36,                    // 37269:       check predicate 36
  OPC_Decode, 253, 50, 161, 1,               // 37271:       decode to SQABS_ZPzZ_H using decoder 161
                                             // 37271:      }
  11, 7,                                     // 37276:      case 0xb: {
  OPC_CheckPredicate, 36,                    // 37278:       check predicate 36
  OPC_Decode, 206, 52, 161, 1,               // 37280:       decode to SQNEG_ZPzZ_H using decoder 161
                                             // 37280:      }
  16, 7,                                     // 37285:      case 0x10: {
  OPC_CheckPredicate, 39,                    // 37287:       check predicate 39
  OPC_Decode, 142, 60, 160, 1,               // 37289:       decode to SUBP_ZPmZZ_H using decoder 160
                                             // 37289:      }
  17, 7,                                     // 37294:      case 0x11: {
  OPC_CheckPredicate, 35,                    // 37296:       check predicate 35
  OPC_Decode, 153, 13, 160, 1,               // 37298:       decode to ADDP_ZPmZ_H using decoder 160
                                             // 37298:      }
  20, 7,                                     // 37303:      case 0x14: {
  OPC_CheckPredicate, 35,                    // 37305:       check predicate 35
  OPC_Decode, 222, 48, 160, 1,               // 37307:       decode to SMAXP_ZPmZ_H using decoder 160
                                             // 37307:      }
  21, 7,                                     // 37312:      case 0x15: {
  OPC_CheckPredicate, 35,                    // 37314:       check predicate 35
  OPC_Decode, 232, 63, 160, 1,               // 37316:       decode to UMAXP_ZPmZ_H using decoder 160
                                             // 37316:      }
  22, 7,                                     // 37321:      case 0x16: {
  OPC_CheckPredicate, 35,                    // 37323:       check predicate 35
  OPC_Decode, 152, 49, 160, 1,               // 37325:       decode to SMINP_ZPmZ_H using decoder 160
                                             // 37325:      }
  23, 0,                                     // 37330:      case 0x17: {
  OPC_CheckPredicate, 35,                    // 37332:       check predicate 35
  OPC_Decode, 161, 64, 160, 1,               // 37334:       decode to UMINP_ZPmZ_H using decoder 160
                                             // 37334:      }
                                             // 37334:      } // switch Inst[20:16]
                                             // 37334:     }
  2, 147, 1,                                 // 37339:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 37342:      switch Inst[20:16] {
  0, 7,                                      // 37345:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 37347:       check predicate 35
  OPC_Decode, 197, 67, 163, 1,               // 37349:       decode to URECPE_ZPmZ_S using decoder 163
                                             // 37349:      }
  1, 7,                                      // 37354:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 37356:       check predicate 35
  OPC_Decode, 255, 67, 163, 1,               // 37358:       decode to URSQRTE_ZPmZ_S using decoder 163
                                             // 37358:      }
  2, 7,                                      // 37363:      case 0x2: {
  OPC_CheckPredicate, 36,                    // 37365:       check predicate 36
  OPC_Decode, 198, 67, 161, 1,               // 37367:       decode to URECPE_ZPzZ_S using decoder 161
                                             // 37367:      }
  3, 7,                                      // 37372:      case 0x3: {
  OPC_CheckPredicate, 36,                    // 37374:       check predicate 36
  OPC_Decode, 128, 68, 161, 1,               // 37376:       decode to URSQRTE_ZPzZ_S using decoder 161
                                             // 37376:      }
  4, 7,                                      // 37381:      case 0x4: {
  OPC_CheckPredicate, 35,                    // 37383:       check predicate 35
  OPC_Decode, 178, 46, 160, 1,               // 37385:       decode to SADALP_ZPmZ_S using decoder 160
                                             // 37385:      }
  5, 7,                                      // 37390:      case 0x5: {
  OPC_CheckPredicate, 35,                    // 37392:       check predicate 35
  OPC_Decode, 182, 62, 160, 1,               // 37394:       decode to UADALP_ZPmZ_S using decoder 160
                                             // 37394:      }
  8, 7,                                      // 37399:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 37401:       check predicate 35
  OPC_Decode, 250, 50, 163, 1,               // 37403:       decode to SQABS_ZPmZ_S using decoder 163
                                             // 37403:      }
  9, 7,                                      // 37408:      case 0x9: {
  OPC_CheckPredicate, 35,                    // 37410:       check predicate 35
  OPC_Decode, 203, 52, 163, 1,               // 37412:       decode to SQNEG_ZPmZ_S using decoder 163
                                             // 37412:      }
  10, 7,                                     // 37417:      case 0xa: {
  OPC_CheckPredicate, 36,                    // 37419:       check predicate 36
  OPC_Decode, 254, 50, 161, 1,               // 37421:       decode to SQABS_ZPzZ_S using decoder 161
                                             // 37421:      }
  11, 7,                                     // 37426:      case 0xb: {
  OPC_CheckPredicate, 36,                    // 37428:       check predicate 36
  OPC_Decode, 207, 52, 161, 1,               // 37430:       decode to SQNEG_ZPzZ_S using decoder 161
                                             // 37430:      }
  16, 7,                                     // 37435:      case 0x10: {
  OPC_CheckPredicate, 39,                    // 37437:       check predicate 39
  OPC_Decode, 143, 60, 160, 1,               // 37439:       decode to SUBP_ZPmZZ_S using decoder 160
                                             // 37439:      }
  17, 7,                                     // 37444:      case 0x11: {
  OPC_CheckPredicate, 35,                    // 37446:       check predicate 35
  OPC_Decode, 154, 13, 160, 1,               // 37448:       decode to ADDP_ZPmZ_S using decoder 160
                                             // 37448:      }
  20, 7,                                     // 37453:      case 0x14: {
  OPC_CheckPredicate, 35,                    // 37455:       check predicate 35
  OPC_Decode, 223, 48, 160, 1,               // 37457:       decode to SMAXP_ZPmZ_S using decoder 160
                                             // 37457:      }
  21, 7,                                     // 37462:      case 0x15: {
  OPC_CheckPredicate, 35,                    // 37464:       check predicate 35
  OPC_Decode, 233, 63, 160, 1,               // 37466:       decode to UMAXP_ZPmZ_S using decoder 160
                                             // 37466:      }
  22, 7,                                     // 37471:      case 0x16: {
  OPC_CheckPredicate, 35,                    // 37473:       check predicate 35
  OPC_Decode, 153, 49, 160, 1,               // 37475:       decode to SMINP_ZPmZ_S using decoder 160
                                             // 37475:      }
  23, 0,                                     // 37480:      case 0x17: {
  OPC_CheckPredicate, 35,                    // 37482:       check predicate 35
  OPC_Decode, 162, 64, 160, 1,               // 37484:       decode to UMINP_ZPmZ_S using decoder 160
                                             // 37484:      }
                                             // 37484:      } // switch Inst[20:16]
                                             // 37484:     }
  3, 111,                                    // 37489:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 37491:      switch Inst[20:16] {
  4, 7,                                      // 37494:      case 0x4: {
  OPC_CheckPredicate, 35,                    // 37496:       check predicate 35
  OPC_Decode, 176, 46, 160, 1,               // 37498:       decode to SADALP_ZPmZ_D using decoder 160
                                             // 37498:      }
  5, 7,                                      // 37503:      case 0x5: {
  OPC_CheckPredicate, 35,                    // 37505:       check predicate 35
  OPC_Decode, 180, 62, 160, 1,               // 37507:       decode to UADALP_ZPmZ_D using decoder 160
                                             // 37507:      }
  8, 7,                                      // 37512:      case 0x8: {
  OPC_CheckPredicate, 35,                    // 37514:       check predicate 35
  OPC_Decode, 248, 50, 163, 1,               // 37516:       decode to SQABS_ZPmZ_D using decoder 163
                                             // 37516:      }
  9, 7,                                      // 37521:      case 0x9: {
  OPC_CheckPredicate, 35,                    // 37523:       check predicate 35
  OPC_Decode, 201, 52, 163, 1,               // 37525:       decode to SQNEG_ZPmZ_D using decoder 163
                                             // 37525:      }
  10, 7,                                     // 37530:      case 0xa: {
  OPC_CheckPredicate, 36,                    // 37532:       check predicate 36
  OPC_Decode, 252, 50, 161, 1,               // 37534:       decode to SQABS_ZPzZ_D using decoder 161
                                             // 37534:      }
  11, 7,                                     // 37539:      case 0xb: {
  OPC_CheckPredicate, 36,                    // 37541:       check predicate 36
  OPC_Decode, 205, 52, 161, 1,               // 37543:       decode to SQNEG_ZPzZ_D using decoder 161
                                             // 37543:      }
  16, 7,                                     // 37548:      case 0x10: {
  OPC_CheckPredicate, 39,                    // 37550:       check predicate 39
  OPC_Decode, 141, 60, 160, 1,               // 37552:       decode to SUBP_ZPmZZ_D using decoder 160
                                             // 37552:      }
  17, 7,                                     // 37557:      case 0x11: {
  OPC_CheckPredicate, 35,                    // 37559:       check predicate 35
  OPC_Decode, 152, 13, 160, 1,               // 37561:       decode to ADDP_ZPmZ_D using decoder 160
                                             // 37561:      }
  20, 7,                                     // 37566:      case 0x14: {
  OPC_CheckPredicate, 35,                    // 37568:       check predicate 35
  OPC_Decode, 221, 48, 160, 1,               // 37570:       decode to SMAXP_ZPmZ_D using decoder 160
                                             // 37570:      }
  21, 7,                                     // 37575:      case 0x15: {
  OPC_CheckPredicate, 35,                    // 37577:       check predicate 35
  OPC_Decode, 231, 63, 160, 1,               // 37579:       decode to UMAXP_ZPmZ_D using decoder 160
                                             // 37579:      }
  22, 7,                                     // 37584:      case 0x16: {
  OPC_CheckPredicate, 35,                    // 37586:       check predicate 35
  OPC_Decode, 151, 49, 160, 1,               // 37588:       decode to SMINP_ZPmZ_D using decoder 160
                                             // 37588:      }
  23, 0,                                     // 37593:      case 0x17: {
  OPC_CheckPredicate, 35,                    // 37595:       check predicate 35
  OPC_Decode, 160, 64, 160, 1,               // 37597:       decode to UMINP_ZPmZ_D using decoder 160
                                             // 37597:      }
                                             // 37597:      } // switch Inst[20:16]
                                             // 37597:     }
  4, 138, 1,                                 // 37602:     case 0x4: {
  OPC_SwitchField, 10, 3,                    // 37605:      switch Inst[12:10] {
  0, 25,                                     // 37608:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 37610:       switch Inst[20] {
  0, 11,                                     // 37613:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 37615:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 37617:        check Inst[19] == 0x1
  OPC_Decode, 180, 55, 232, 1,               // 37621:        decode to SSHLLB_ZZI_H using decoder 232
                                             // 37621:       }
  1, 0,                                      // 37626:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 37628:        check predicate 35
  OPC_Decode, 181, 55, 233, 1,               // 37630:        decode to SSHLLB_ZZI_S using decoder 233
                                             // 37630:       }
                                             // 37630:       } // switch Inst[20]
                                             // 37630:      }
  1, 25,                                     // 37635:      case 0x1: {
  OPC_SwitchField, 20, 1,                    // 37637:       switch Inst[20] {
  0, 11,                                     // 37640:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 37642:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 37644:        check Inst[19] == 0x1
  OPC_Decode, 183, 55, 232, 1,               // 37648:        decode to SSHLLT_ZZI_H using decoder 232
                                             // 37648:       }
  1, 0,                                      // 37653:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 37655:        check predicate 35
  OPC_Decode, 184, 55, 233, 1,               // 37657:        decode to SSHLLT_ZZI_S using decoder 233
                                             // 37657:       }
                                             // 37657:       } // switch Inst[20]
                                             // 37657:      }
  2, 25,                                     // 37662:      case 0x2: {
  OPC_SwitchField, 20, 1,                    // 37664:       switch Inst[20] {
  0, 11,                                     // 37667:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 37669:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 37671:        check Inst[19] == 0x1
  OPC_Decode, 156, 68, 232, 1,               // 37675:        decode to USHLLB_ZZI_H using decoder 232
                                             // 37675:       }
  1, 0,                                      // 37680:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 37682:        check predicate 35
  OPC_Decode, 157, 68, 233, 1,               // 37684:        decode to USHLLB_ZZI_S using decoder 233
                                             // 37684:       }
                                             // 37684:       } // switch Inst[20]
                                             // 37684:      }
  3, 25,                                     // 37689:      case 0x3: {
  OPC_SwitchField, 20, 1,                    // 37691:       switch Inst[20] {
  0, 11,                                     // 37694:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 37696:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 37698:        check Inst[19] == 0x1
  OPC_Decode, 159, 68, 232, 1,               // 37702:        decode to USHLLT_ZZI_H using decoder 232
                                             // 37702:       }
  1, 0,                                      // 37707:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 37709:        check predicate 35
  OPC_Decode, 160, 68, 233, 1,               // 37711:        decode to USHLLT_ZZI_S using decoder 233
                                             // 37711:       }
                                             // 37711:       } // switch Inst[20]
                                             // 37711:      }
  4, 7,                                      // 37716:      case 0x4: {
  OPC_CheckPredicate, 49,                    // 37718:       check predicate 49
  OPC_Decode, 231, 14, 184, 1,               // 37720:       decode to BEXT_ZZZ_B using decoder 184
                                             // 37720:      }
  5, 7,                                      // 37725:      case 0x5: {
  OPC_CheckPredicate, 49,                    // 37727:       check predicate 49
  OPC_Decode, 227, 14, 184, 1,               // 37729:       decode to BDEP_ZZZ_B using decoder 184
                                             // 37729:      }
  6, 0,                                      // 37734:      case 0x6: {
  OPC_CheckPredicate, 49,                    // 37736:       check predicate 49
  OPC_Decode, 129, 16, 184, 1,               // 37738:       decode to BGRP_ZZZ_B using decoder 184
                                             // 37738:      }
                                             // 37738:      } // switch Inst[12:10]
                                             // 37738:     }
  5, 66,                                     // 37743:     case 0x5: {
  OPC_SwitchField, 10, 3,                    // 37745:      switch Inst[12:10] {
  0, 7,                                      // 37748:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 37750:       check predicate 35
  OPC_Decode, 179, 55, 234, 1,               // 37752:       decode to SSHLLB_ZZI_D using decoder 234
                                             // 37752:      }
  1, 7,                                      // 37757:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 37759:       check predicate 35
  OPC_Decode, 182, 55, 234, 1,               // 37761:       decode to SSHLLT_ZZI_D using decoder 234
                                             // 37761:      }
  2, 7,                                      // 37766:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 37768:       check predicate 35
  OPC_Decode, 155, 68, 234, 1,               // 37770:       decode to USHLLB_ZZI_D using decoder 234
                                             // 37770:      }
  3, 7,                                      // 37775:      case 0x3: {
  OPC_CheckPredicate, 35,                    // 37777:       check predicate 35
  OPC_Decode, 158, 68, 234, 1,               // 37779:       decode to USHLLT_ZZI_D using decoder 234
                                             // 37779:      }
  4, 7,                                      // 37784:      case 0x4: {
  OPC_CheckPredicate, 49,                    // 37786:       check predicate 49
  OPC_Decode, 233, 14, 184, 1,               // 37788:       decode to BEXT_ZZZ_H using decoder 184
                                             // 37788:      }
  5, 7,                                      // 37793:      case 0x5: {
  OPC_CheckPredicate, 49,                    // 37795:       check predicate 49
  OPC_Decode, 229, 14, 184, 1,               // 37797:       decode to BDEP_ZZZ_H using decoder 184
                                             // 37797:      }
  6, 0,                                      // 37802:      case 0x6: {
  OPC_CheckPredicate, 49,                    // 37804:       check predicate 49
  OPC_Decode, 131, 16, 184, 1,               // 37806:       decode to BGRP_ZZZ_H using decoder 184
                                             // 37806:      }
                                             // 37806:      } // switch Inst[12:10]
                                             // 37806:     }
  6, 30,                                     // 37811:     case 0x6: {
  OPC_SwitchField, 10, 3,                    // 37813:      switch Inst[12:10] {
  4, 7,                                      // 37816:      case 0x4: {
  OPC_CheckPredicate, 49,                    // 37818:       check predicate 49
  OPC_Decode, 234, 14, 184, 1,               // 37820:       decode to BEXT_ZZZ_S using decoder 184
                                             // 37820:      }
  5, 7,                                      // 37825:      case 0x5: {
  OPC_CheckPredicate, 49,                    // 37827:       check predicate 49
  OPC_Decode, 230, 14, 184, 1,               // 37829:       decode to BDEP_ZZZ_S using decoder 184
                                             // 37829:      }
  6, 0,                                      // 37834:      case 0x6: {
  OPC_CheckPredicate, 49,                    // 37836:       check predicate 49
  OPC_Decode, 132, 16, 184, 1,               // 37838:       decode to BGRP_ZZZ_S using decoder 184
                                             // 37838:      }
                                             // 37838:      } // switch Inst[12:10]
                                             // 37838:     }
  7, 0,                                      // 37843:     case 0x7: {
  OPC_SwitchField, 10, 3,                    // 37845:      switch Inst[12:10] {
  4, 7,                                      // 37848:      case 0x4: {
  OPC_CheckPredicate, 49,                    // 37850:       check predicate 49
  OPC_Decode, 232, 14, 184, 1,               // 37852:       decode to BEXT_ZZZ_D using decoder 184
                                             // 37852:      }
  5, 7,                                      // 37857:      case 0x5: {
  OPC_CheckPredicate, 49,                    // 37859:       check predicate 49
  OPC_Decode, 228, 14, 184, 1,               // 37861:       decode to BDEP_ZZZ_D using decoder 184
                                             // 37861:      }
  6, 0,                                      // 37866:      case 0x6: {
  OPC_CheckPredicate, 49,                    // 37868:       check predicate 49
  OPC_Decode, 130, 16, 184, 1,               // 37870:       decode to BGRP_ZZZ_D using decoder 184
                                             // 37870:      }
                                             // 37870:      } // switch Inst[12:10]
                                             // 37870:     }
                                             // 37870:     } // switch Inst[25:22]
                                             // 37870:    }
  1, 0,                                      // 37875:    case 0x1: {
  OPC_SwitchField, 10, 1,                    // 37877:     switch Inst[10] {
  0, 95,                                     // 37880:     case 0x0: {
  OPC_SwitchField, 24, 2,                    // 37882:      switch Inst[25:24] {
  0, 49,                                     // 37885:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 37887:       switch Inst[12] {
  0, 21,                                     // 37890:       case 0x0: {
  OPC_SwitchField, 22, 2,                    // 37892:        switch Inst[23:22] {
  2, 7,                                      // 37895:        case 0x2: {
  OPC_CheckPredicate, 35,                    // 37897:         check predicate 35
  OPC_Decode, 252, 49, 182, 2,               // 37899:         decode to SMLSLB_ZZZI_S using decoder 310
                                             // 37899:        }
  3, 0,                                      // 37904:        case 0x3: {
  OPC_CheckPredicate, 35,                    // 37906:         check predicate 35
  OPC_Decode, 251, 49, 183, 2,               // 37908:         decode to SMLSLB_ZZZI_D using decoder 311
                                             // 37908:        }
                                             // 37908:        } // switch Inst[23:22]
                                             // 37908:       }
  1, 0,                                      // 37913:       case 0x1: {
  OPC_SwitchField, 22, 2,                    // 37915:        switch Inst[23:22] {
  2, 7,                                      // 37918:        case 0x2: {
  OPC_CheckPredicate, 35,                    // 37920:         check predicate 35
  OPC_Decode, 133, 65, 182, 2,               // 37922:         decode to UMLSLB_ZZZI_S using decoder 310
                                             // 37922:        }
  3, 0,                                      // 37927:        case 0x3: {
  OPC_CheckPredicate, 35,                    // 37929:         check predicate 35
  OPC_Decode, 132, 65, 183, 2,               // 37931:         decode to UMLSLB_ZZZI_D using decoder 311
                                             // 37931:        }
                                             // 37931:        } // switch Inst[23:22]
                                             // 37931:       }
                                             // 37931:       } // switch Inst[12]
                                             // 37931:      }
  1, 0,                                      // 37936:      case 0x1: {
  OPC_SwitchField, 11, 1,                    // 37938:       switch Inst[11] {
  0, 25,                                     // 37941:       case 0x0: {
  OPC_SwitchField, 12, 1,                    // 37943:        switch Inst[12] {
  0, 11,                                     // 37946:        case 0x0: {
  OPC_CheckPredicate, 48,                    // 37948:         check predicate 48
  OPC_CheckField, 22, 2, 0,                  // 37950:         check Inst[23:22] == 0x0
  OPC_Decode, 224, 33, 184, 1,               // 37954:         decode to HISTSEG_ZZZ using decoder 184
                                             // 37954:        }
  1, 0,                                      // 37959:        case 0x1: {
  OPC_CheckPredicate, 50,                    // 37961:         check predicate 50
  OPC_Decode, 204, 41, 186, 2,               // 37963:         decode to LUTI2_ZZZI_B using decoder 314
                                             // 37963:        }
                                             // 37963:        } // switch Inst[12]
                                             // 37963:       }
  1, 0,                                      // 37968:       case 0x1: {
  OPC_CheckPredicate, 50,                    // 37970:        check predicate 50
  OPC_Decode, 205, 41, 187, 2,               // 37972:        decode to LUTI2_ZZZI_H using decoder 315
                                             // 37972:       }
                                             // 37972:       } // switch Inst[11]
                                             // 37972:      }
                                             // 37972:      } // switch Inst[25:24]
                                             // 37972:     }
  1, 0,                                      // 37977:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 37979:      switch Inst[12] {
  0, 71,                                     // 37982:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 37984:       switch Inst[22] {
  0, 25,                                     // 37987:       case 0x0: {
  OPC_SwitchField, 23, 3,                    // 37989:        switch Inst[25:23] {
  1, 7,                                      // 37992:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 37994:         check predicate 35
  OPC_Decode, 145, 50, 182, 2,               // 37996:         decode to SMLSLT_ZZZI_S using decoder 310
                                             // 37996:        }
  2, 0,                                      // 38001:        case 0x2: {
  OPC_CheckPredicate, 51,                    // 38003:         check predicate 51
  OPC_CheckField, 11, 1, 1,                  // 38005:         check Inst[11] == 0x1
  OPC_Decode, 226, 41, 197, 1,               // 38009:         decode to LUTI6_Z2ZZ using decoder 197
                                             // 38009:        }
                                             // 38009:        } // switch Inst[25:23]
                                             // 38009:       }
  1, 0,                                      // 38014:       case 0x1: {
  OPC_SwitchField, 24, 2,                    // 38016:        switch Inst[25:24] {
  0, 11,                                     // 38019:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 38021:         check predicate 35
  OPC_CheckField, 23, 1, 1,                  // 38023:         check Inst[23] == 0x1
  OPC_Decode, 144, 50, 183, 2,               // 38027:         decode to SMLSLT_ZZZI_D using decoder 311
                                             // 38027:        }
  1, 0,                                      // 38032:        case 0x1: {
  OPC_SwitchField, 11, 1,                    // 38034:         switch Inst[11] {
  0, 7,                                      // 38037:         case 0x0: {
  OPC_CheckPredicate, 50,                    // 38039:          check predicate 50
  OPC_Decode, 220, 41, 188, 2,               // 38041:          decode to LUTI4_ZZZI_B using decoder 316
                                             // 38041:         }
  1, 0,                                      // 38046:         case 0x1: {
  OPC_CheckPredicate, 39,                    // 38048:          check predicate 39
  OPC_Decode, 227, 41, 189, 2,               // 38050:          decode to LUTI6_Z2ZZI_H using decoder 317
                                             // 38050:         }
                                             // 38050:         } // switch Inst[11]
                                             // 38050:        }
                                             // 38050:        } // switch Inst[25:24]
                                             // 38050:       }
                                             // 38050:       } // switch Inst[22]
                                             // 38050:      }
  1, 0,                                      // 38055:      case 0x1: {
  OPC_SwitchField, 24, 2,                    // 38057:       switch Inst[25:24] {
  0, 21,                                     // 38060:       case 0x0: {
  OPC_SwitchField, 22, 2,                    // 38062:        switch Inst[23:22] {
  2, 7,                                      // 38065:        case 0x2: {
  OPC_CheckPredicate, 35,                    // 38067:         check predicate 35
  OPC_Decode, 154, 65, 182, 2,               // 38069:         decode to UMLSLT_ZZZI_S using decoder 310
                                             // 38069:        }
  3, 0,                                      // 38074:        case 0x3: {
  OPC_CheckPredicate, 35,                    // 38076:         check predicate 35
  OPC_Decode, 153, 65, 183, 2,               // 38078:         decode to UMLSLT_ZZZI_D using decoder 311
                                             // 38078:        }
                                             // 38078:        } // switch Inst[23:22]
                                             // 38078:       }
  1, 0,                                      // 38083:       case 0x1: {
  OPC_SwitchField, 11, 1,                    // 38085:        switch Inst[11] {
  0, 7,                                      // 38088:        case 0x0: {
  OPC_CheckPredicate, 50,                    // 38090:         check predicate 50
  OPC_Decode, 216, 41, 190, 2,               // 38092:         decode to LUTI4_Z2ZZI using decoder 318
                                             // 38092:        }
  1, 0,                                      // 38097:        case 0x1: {
  OPC_CheckPredicate, 50,                    // 38099:         check predicate 50
  OPC_Decode, 221, 41, 186, 2,               // 38101:         decode to LUTI4_ZZZI_H using decoder 314
                                             // 38101:        }
                                             // 38101:        } // switch Inst[11]
                                             // 38101:       }
                                             // 38101:       } // switch Inst[25:24]
                                             // 38101:      }
                                             // 38101:      } // switch Inst[12]
                                             // 38101:     }
                                             // 38101:     } // switch Inst[10]
                                             // 38101:    }
                                             // 38101:    } // switch Inst[21]
                                             // 38101:   }
  6, 153, 5,                                 // 38106:   case 0x6: {
  OPC_SwitchField, 21, 5,                    // 38109:    switch Inst[25:21] {
  0, 39,                                     // 38112:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 38114:     switch Inst[12:10] {
  0, 7,                                      // 38117:     case 0x0: {
  OPC_CheckPredicate, 43,                    // 38119:      check predicate 43
  OPC_Decode, 245, 46, 198, 1,               // 38121:      decode to SCLAMP_ZZZ_B using decoder 198
                                             // 38121:     }
  1, 7,                                      // 38126:     case 0x1: {
  OPC_CheckPredicate, 43,                    // 38128:      check predicate 43
  OPC_Decode, 238, 62, 198, 1,               // 38130:      decode to UCLAMP_ZZZ_B using decoder 198
                                             // 38130:     }
  2, 7,                                      // 38135:     case 0x2: {
  OPC_CheckPredicate, 18,                    // 38137:      check predicate 18
  OPC_Decode, 206, 47, 198, 1,               // 38139:      decode to SDOT_ZZZ_HtoS using decoder 198
                                             // 38139:     }
  3, 0,                                      // 38144:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 38146:      check predicate 18
  OPC_Decode, 200, 63, 198, 1,               // 38148:      decode to UDOT_ZZZ_HtoS using decoder 198
                                             // 38148:     }
                                             // 38148:     } // switch Inst[12:10]
                                             // 38148:    }
  2, 39,                                     // 38153:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 38155:     switch Inst[12:10] {
  0, 7,                                      // 38158:     case 0x0: {
  OPC_CheckPredicate, 43,                    // 38160:      check predicate 43
  OPC_Decode, 247, 46, 198, 1,               // 38162:      decode to SCLAMP_ZZZ_H using decoder 198
                                             // 38162:     }
  1, 7,                                      // 38167:     case 0x1: {
  OPC_CheckPredicate, 43,                    // 38169:      check predicate 43
  OPC_Decode, 240, 62, 198, 1,               // 38171:      decode to UCLAMP_ZZZ_H using decoder 198
                                             // 38171:     }
  5, 7,                                      // 38176:     case 0x5: {
  OPC_CheckPredicate, 39,                    // 38178:      check predicate 39
  OPC_Decode, 135, 46, 198, 1,               // 38180:      decode to SABAL_ZZZ_BtoH using decoder 198
                                             // 38180:     }
  7, 0,                                      // 38185:     case 0x7: {
  OPC_CheckPredicate, 39,                    // 38187:      check predicate 39
  OPC_Decode, 139, 62, 198, 1,               // 38189:      decode to UABAL_ZZZ_BtoH using decoder 198
                                             // 38189:     }
                                             // 38189:     } // switch Inst[12:10]
                                             // 38189:    }
  4, 57,                                     // 38194:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 38196:     switch Inst[12:10] {
  0, 7,                                      // 38199:     case 0x0: {
  OPC_CheckPredicate, 43,                    // 38201:      check predicate 43
  OPC_Decode, 248, 46, 198, 1,               // 38203:      decode to SCLAMP_ZZZ_S using decoder 198
                                             // 38203:     }
  1, 7,                                      // 38208:     case 0x1: {
  OPC_CheckPredicate, 43,                    // 38210:      check predicate 43
  OPC_Decode, 241, 62, 198, 1,               // 38212:      decode to UCLAMP_ZZZ_S using decoder 198
                                             // 38212:     }
  2, 7,                                      // 38217:     case 0x2: {
  OPC_CheckPredicate, 18,                    // 38219:      check predicate 18
  OPC_Decode, 202, 47, 179, 2,               // 38221:      decode to SDOT_ZZZI_HtoS using decoder 307
                                             // 38221:     }
  3, 7,                                      // 38226:     case 0x3: {
  OPC_CheckPredicate, 18,                    // 38228:      check predicate 18
  OPC_Decode, 196, 63, 179, 2,               // 38230:      decode to UDOT_ZZZI_HtoS using decoder 307
                                             // 38230:     }
  5, 7,                                      // 38235:     case 0x5: {
  OPC_CheckPredicate, 39,                    // 38237:      check predicate 39
  OPC_Decode, 136, 46, 198, 1,               // 38239:      decode to SABAL_ZZZ_HtoS using decoder 198
                                             // 38239:     }
  7, 0,                                      // 38244:     case 0x7: {
  OPC_CheckPredicate, 39,                    // 38246:      check predicate 39
  OPC_Decode, 140, 62, 198, 1,               // 38248:      decode to UABAL_ZZZ_HtoS using decoder 198
                                             // 38248:     }
                                             // 38248:     } // switch Inst[12:10]
                                             // 38248:    }
  5, 49,                                     // 38253:    case 0x5: {
  OPC_SwitchField, 10, 1,                    // 38255:     switch Inst[10] {
  0, 21,                                     // 38258:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 38260:      switch Inst[12] {
  0, 7,                                      // 38263:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38265:       check predicate 35
  OPC_Decode, 220, 50, 191, 2,               // 38267:       decode to SMULLB_ZZZI_S using decoder 319
                                             // 38267:      }
  1, 0,                                      // 38272:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38274:       check predicate 35
  OPC_Decode, 227, 65, 191, 2,               // 38276:       decode to UMULLB_ZZZI_S using decoder 319
                                             // 38276:      }
                                             // 38276:      } // switch Inst[12]
                                             // 38276:     }
  1, 0,                                      // 38281:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 38283:      switch Inst[12] {
  0, 7,                                      // 38286:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38288:       check predicate 35
  OPC_Decode, 225, 50, 191, 2,               // 38290:       decode to SMULLT_ZZZI_S using decoder 319
                                             // 38290:      }
  1, 0,                                      // 38295:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38297:       check predicate 35
  OPC_Decode, 232, 65, 191, 2,               // 38299:       decode to UMULLT_ZZZI_S using decoder 319
                                             // 38299:      }
                                             // 38299:      } // switch Inst[12]
                                             // 38299:     }
                                             // 38299:     } // switch Inst[10]
                                             // 38299:    }
  6, 57,                                     // 38304:    case 0x6: {
  OPC_SwitchField, 10, 3,                    // 38306:     switch Inst[12:10] {
  0, 7,                                      // 38309:     case 0x0: {
  OPC_CheckPredicate, 43,                    // 38311:      check predicate 43
  OPC_Decode, 246, 46, 198, 1,               // 38313:      decode to SCLAMP_ZZZ_D using decoder 198
                                             // 38313:     }
  1, 7,                                      // 38318:     case 0x1: {
  OPC_CheckPredicate, 43,                    // 38320:      check predicate 43
  OPC_Decode, 239, 62, 198, 1,               // 38322:      decode to UCLAMP_ZZZ_D using decoder 198
                                             // 38322:     }
  4, 7,                                      // 38327:     case 0x4: {
  OPC_CheckPredicate, 37,                    // 38329:      check predicate 37
  OPC_Decode, 239, 41, 198, 1,               // 38331:      decode to MLA_CPA using decoder 198
                                             // 38331:     }
  5, 7,                                      // 38336:     case 0x5: {
  OPC_CheckPredicate, 39,                    // 38338:      check predicate 39
  OPC_Decode, 137, 46, 198, 1,               // 38340:      decode to SABAL_ZZZ_StoD using decoder 198
                                             // 38340:     }
  6, 7,                                      // 38345:     case 0x6: {
  OPC_CheckPredicate, 37,                    // 38347:      check predicate 37
  OPC_Decode, 232, 41, 203, 1,               // 38349:      decode to MAD_CPA using decoder 203
                                             // 38349:     }
  7, 0,                                      // 38354:     case 0x7: {
  OPC_CheckPredicate, 39,                    // 38356:      check predicate 39
  OPC_Decode, 141, 62, 198, 1,               // 38358:      decode to UABAL_ZZZ_StoD using decoder 198
                                             // 38358:     }
                                             // 38358:     } // switch Inst[12:10]
                                             // 38358:    }
  7, 49,                                     // 38363:    case 0x7: {
  OPC_SwitchField, 10, 1,                    // 38365:     switch Inst[10] {
  0, 21,                                     // 38368:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 38370:      switch Inst[12] {
  0, 7,                                      // 38373:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38375:       check predicate 35
  OPC_Decode, 219, 50, 192, 2,               // 38377:       decode to SMULLB_ZZZI_D using decoder 320
                                             // 38377:      }
  1, 0,                                      // 38382:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38384:       check predicate 35
  OPC_Decode, 226, 65, 192, 2,               // 38386:       decode to UMULLB_ZZZI_D using decoder 320
                                             // 38386:      }
                                             // 38386:      } // switch Inst[12]
                                             // 38386:     }
  1, 0,                                      // 38391:     case 0x1: {
  OPC_SwitchField, 12, 1,                    // 38393:      switch Inst[12] {
  0, 7,                                      // 38396:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38398:       check predicate 35
  OPC_Decode, 224, 50, 192, 2,               // 38400:       decode to SMULLT_ZZZI_D using decoder 320
                                             // 38400:      }
  1, 0,                                      // 38405:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38407:       check predicate 35
  OPC_Decode, 231, 65, 192, 2,               // 38409:       decode to UMULLT_ZZZI_D using decoder 320
                                             // 38409:      }
                                             // 38409:      } // switch Inst[12]
                                             // 38409:     }
                                             // 38409:     } // switch Inst[10]
                                             // 38409:    }
  8, 49,                                     // 38414:    case 0x8: {
  OPC_SwitchField, 11, 2,                    // 38416:     switch Inst[12:11] {
  2, 21,                                     // 38419:     case 0x2: {
  OPC_SwitchField, 10, 1,                    // 38421:      switch Inst[10] {
  0, 7,                                      // 38424:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38426:       check predicate 35
  OPC_Decode, 255, 12, 198, 1,               // 38428:       decode to ADCLB_ZZZ_S using decoder 198
                                             // 38428:      }
  1, 0,                                      // 38433:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38435:       check predicate 35
  OPC_Decode, 129, 13, 198, 1,               // 38437:       decode to ADCLT_ZZZ_S using decoder 198
                                             // 38437:      }
                                             // 38437:      } // switch Inst[10]
                                             // 38437:     }
  3, 0,                                      // 38442:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 38444:      switch Inst[20:16] {
  0, 7,                                      // 38447:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38449:       check predicate 35
  OPC_Decode, 188, 16, 193, 2,               // 38451:       decode to CADD_ZZI_B using decoder 321
                                             // 38451:      }
  1, 0,                                      // 38456:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38458:       check predicate 35
  OPC_Decode, 161, 51, 193, 2,               // 38460:       decode to SQCADD_ZZI_B using decoder 321
                                             // 38460:      }
                                             // 38460:      } // switch Inst[20:16]
                                             // 38460:     }
                                             // 38460:     } // switch Inst[12:11]
                                             // 38460:    }
  10, 95,                                    // 38465:    case 0xa: {
  OPC_SwitchField, 11, 2,                    // 38467:     switch Inst[12:11] {
  0, 21,                                     // 38470:     case 0x0: {
  OPC_SwitchField, 10, 1,                    // 38472:      switch Inst[10] {
  0, 7,                                      // 38475:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38477:       check predicate 35
  OPC_Decode, 130, 46, 198, 1,               // 38479:       decode to SABALB_ZZZ_H using decoder 198
                                             // 38479:      }
  1, 0,                                      // 38484:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38486:       check predicate 35
  OPC_Decode, 133, 46, 198, 1,               // 38488:       decode to SABALT_ZZZ_H using decoder 198
                                             // 38488:      }
                                             // 38488:      } // switch Inst[10]
                                             // 38488:     }
  1, 21,                                     // 38493:     case 0x1: {
  OPC_SwitchField, 10, 1,                    // 38495:      switch Inst[10] {
  0, 7,                                      // 38498:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38500:       check predicate 35
  OPC_Decode, 134, 62, 198, 1,               // 38502:       decode to UABALB_ZZZ_H using decoder 198
                                             // 38502:      }
  1, 0,                                      // 38507:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38509:       check predicate 35
  OPC_Decode, 137, 62, 198, 1,               // 38511:       decode to UABALT_ZZZ_H using decoder 198
                                             // 38511:      }
                                             // 38511:      } // switch Inst[10]
                                             // 38511:     }
  2, 21,                                     // 38516:     case 0x2: {
  OPC_SwitchField, 10, 1,                    // 38518:      switch Inst[10] {
  0, 7,                                      // 38521:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38523:       check predicate 35
  OPC_Decode, 254, 12, 198, 1,               // 38525:       decode to ADCLB_ZZZ_D using decoder 198
                                             // 38525:      }
  1, 0,                                      // 38530:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38532:       check predicate 35
  OPC_Decode, 128, 13, 198, 1,               // 38534:       decode to ADCLT_ZZZ_D using decoder 198
                                             // 38534:      }
                                             // 38534:      } // switch Inst[10]
                                             // 38534:     }
  3, 0,                                      // 38539:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 38541:      switch Inst[20:16] {
  0, 7,                                      // 38544:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38546:       check predicate 35
  OPC_Decode, 190, 16, 193, 2,               // 38548:       decode to CADD_ZZI_H using decoder 321
                                             // 38548:      }
  1, 0,                                      // 38553:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38555:       check predicate 35
  OPC_Decode, 163, 51, 193, 2,               // 38557:       decode to SQCADD_ZZI_H using decoder 321
                                             // 38557:      }
                                             // 38557:      } // switch Inst[20:16]
                                             // 38557:     }
                                             // 38557:     } // switch Inst[12:11]
                                             // 38557:    }
  12, 95,                                    // 38562:    case 0xc: {
  OPC_SwitchField, 11, 2,                    // 38564:     switch Inst[12:11] {
  0, 21,                                     // 38567:     case 0x0: {
  OPC_SwitchField, 10, 1,                    // 38569:      switch Inst[10] {
  0, 7,                                      // 38572:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38574:       check predicate 35
  OPC_Decode, 131, 46, 198, 1,               // 38576:       decode to SABALB_ZZZ_S using decoder 198
                                             // 38576:      }
  1, 0,                                      // 38581:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38583:       check predicate 35
  OPC_Decode, 134, 46, 198, 1,               // 38585:       decode to SABALT_ZZZ_S using decoder 198
                                             // 38585:      }
                                             // 38585:      } // switch Inst[10]
                                             // 38585:     }
  1, 21,                                     // 38590:     case 0x1: {
  OPC_SwitchField, 10, 1,                    // 38592:      switch Inst[10] {
  0, 7,                                      // 38595:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38597:       check predicate 35
  OPC_Decode, 135, 62, 198, 1,               // 38599:       decode to UABALB_ZZZ_S using decoder 198
                                             // 38599:      }
  1, 0,                                      // 38604:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38606:       check predicate 35
  OPC_Decode, 138, 62, 198, 1,               // 38608:       decode to UABALT_ZZZ_S using decoder 198
                                             // 38608:      }
                                             // 38608:      } // switch Inst[10]
                                             // 38608:     }
  2, 21,                                     // 38613:     case 0x2: {
  OPC_SwitchField, 10, 1,                    // 38615:      switch Inst[10] {
  0, 7,                                      // 38618:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38620:       check predicate 35
  OPC_Decode, 228, 46, 198, 1,               // 38622:       decode to SBCLB_ZZZ_S using decoder 198
                                             // 38622:      }
  1, 0,                                      // 38627:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38629:       check predicate 35
  OPC_Decode, 230, 46, 198, 1,               // 38631:       decode to SBCLT_ZZZ_S using decoder 198
                                             // 38631:      }
                                             // 38631:      } // switch Inst[10]
                                             // 38631:     }
  3, 0,                                      // 38636:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 38638:      switch Inst[20:16] {
  0, 7,                                      // 38641:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38643:       check predicate 35
  OPC_Decode, 191, 16, 193, 2,               // 38645:       decode to CADD_ZZI_S using decoder 321
                                             // 38645:      }
  1, 0,                                      // 38650:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38652:       check predicate 35
  OPC_Decode, 164, 51, 193, 2,               // 38654:       decode to SQCADD_ZZI_S using decoder 321
                                             // 38654:      }
                                             // 38654:      } // switch Inst[20:16]
                                             // 38654:     }
                                             // 38654:     } // switch Inst[12:11]
                                             // 38654:    }
  13, 7,                                     // 38659:    case 0xd: {
  OPC_CheckPredicate, 48,                    // 38661:     check predicate 48
  OPC_Decode, 223, 33, 194, 2,               // 38663:     decode to HISTCNT_ZPzZZ_S using decoder 322
                                             // 38663:    }
  14, 95,                                    // 38668:    case 0xe: {
  OPC_SwitchField, 11, 2,                    // 38670:     switch Inst[12:11] {
  0, 21,                                     // 38673:     case 0x0: {
  OPC_SwitchField, 10, 1,                    // 38675:      switch Inst[10] {
  0, 7,                                      // 38678:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38680:       check predicate 35
  OPC_Decode, 129, 46, 198, 1,               // 38682:       decode to SABALB_ZZZ_D using decoder 198
                                             // 38682:      }
  1, 0,                                      // 38687:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38689:       check predicate 35
  OPC_Decode, 132, 46, 198, 1,               // 38691:       decode to SABALT_ZZZ_D using decoder 198
                                             // 38691:      }
                                             // 38691:      } // switch Inst[10]
                                             // 38691:     }
  1, 21,                                     // 38696:     case 0x1: {
  OPC_SwitchField, 10, 1,                    // 38698:      switch Inst[10] {
  0, 7,                                      // 38701:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38703:       check predicate 35
  OPC_Decode, 133, 62, 198, 1,               // 38705:       decode to UABALB_ZZZ_D using decoder 198
                                             // 38705:      }
  1, 0,                                      // 38710:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38712:       check predicate 35
  OPC_Decode, 136, 62, 198, 1,               // 38714:       decode to UABALT_ZZZ_D using decoder 198
                                             // 38714:      }
                                             // 38714:      } // switch Inst[10]
                                             // 38714:     }
  2, 21,                                     // 38719:     case 0x2: {
  OPC_SwitchField, 10, 1,                    // 38721:      switch Inst[10] {
  0, 7,                                      // 38724:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38726:       check predicate 35
  OPC_Decode, 227, 46, 198, 1,               // 38728:       decode to SBCLB_ZZZ_D using decoder 198
                                             // 38728:      }
  1, 0,                                      // 38733:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38735:       check predicate 35
  OPC_Decode, 229, 46, 198, 1,               // 38737:       decode to SBCLT_ZZZ_D using decoder 198
                                             // 38737:      }
                                             // 38737:      } // switch Inst[10]
                                             // 38737:     }
  3, 0,                                      // 38742:     case 0x3: {
  OPC_SwitchField, 16, 5,                    // 38744:      switch Inst[20:16] {
  0, 7,                                      // 38747:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 38749:       check predicate 35
  OPC_Decode, 189, 16, 193, 2,               // 38751:       decode to CADD_ZZI_D using decoder 321
                                             // 38751:      }
  1, 0,                                      // 38756:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38758:       check predicate 35
  OPC_Decode, 162, 51, 193, 2,               // 38760:       decode to SQCADD_ZZI_D using decoder 321
                                             // 38760:      }
                                             // 38760:      } // switch Inst[20:16]
                                             // 38760:     }
                                             // 38760:     } // switch Inst[12:11]
                                             // 38760:    }
  15, 0,                                     // 38765:    case 0xf: {
  OPC_CheckPredicate, 48,                    // 38767:     check predicate 48
  OPC_Decode, 222, 33, 194, 2,               // 38769:     decode to HISTCNT_ZPzZZ_D using decoder 322
                                             // 38769:    }
                                             // 38769:    } // switch Inst[25:21]
                                             // 38769:   }
  7, 0,                                      // 38774:   case 0x7: {
  OPC_SwitchField, 23, 3,                    // 38776:    switch Inst[25:23] {
  0, 158, 1,                                 // 38779:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 38782:     switch Inst[12:10] {
  0, 21,                                     // 38785:     case 0x0: {
  OPC_SwitchField, 21, 2,                    // 38787:      switch Inst[22:21] {
  0, 7,                                      // 38790:      case 0x0: {
  OPC_CheckPredicate, 34,                    // 38792:       check predicate 34
  OPC_Decode, 157, 71, 184, 1,               // 38794:       decode to ZIPQ1_ZZZ_B using decoder 184
                                             // 38794:      }
  2, 0,                                      // 38799:      case 0x2: {
  OPC_CheckPredicate, 34,                    // 38801:       check predicate 34
  OPC_Decode, 159, 71, 184, 1,               // 38803:       decode to ZIPQ1_ZZZ_H using decoder 184
                                             // 38803:      }
                                             // 38803:      } // switch Inst[22:21]
                                             // 38803:     }
  1, 21,                                     // 38808:     case 0x1: {
  OPC_SwitchField, 21, 2,                    // 38810:      switch Inst[22:21] {
  0, 7,                                      // 38813:      case 0x0: {
  OPC_CheckPredicate, 34,                    // 38815:       check predicate 34
  OPC_Decode, 161, 71, 184, 1,               // 38817:       decode to ZIPQ2_ZZZ_B using decoder 184
                                             // 38817:      }
  2, 0,                                      // 38822:      case 0x2: {
  OPC_CheckPredicate, 34,                    // 38824:       check predicate 34
  OPC_Decode, 163, 71, 184, 1,               // 38826:       decode to ZIPQ2_ZZZ_H using decoder 184
                                             // 38826:      }
                                             // 38826:      } // switch Inst[22:21]
                                             // 38826:     }
  2, 21,                                     // 38831:     case 0x2: {
  OPC_SwitchField, 21, 2,                    // 38833:      switch Inst[22:21] {
  0, 7,                                      // 38836:      case 0x0: {
  OPC_CheckPredicate, 34,                    // 38838:       check predicate 34
  OPC_Decode, 199, 69, 184, 1,               // 38840:       decode to UZPQ1_ZZZ_B using decoder 184
                                             // 38840:      }
  2, 0,                                      // 38845:      case 0x2: {
  OPC_CheckPredicate, 34,                    // 38847:       check predicate 34
  OPC_Decode, 201, 69, 184, 1,               // 38849:       decode to UZPQ1_ZZZ_H using decoder 184
                                             // 38849:      }
                                             // 38849:      } // switch Inst[22:21]
                                             // 38849:     }
  3, 21,                                     // 38854:     case 0x3: {
  OPC_SwitchField, 21, 2,                    // 38856:      switch Inst[22:21] {
  0, 7,                                      // 38859:      case 0x0: {
  OPC_CheckPredicate, 34,                    // 38861:       check predicate 34
  OPC_Decode, 203, 69, 184, 1,               // 38863:       decode to UZPQ2_ZZZ_B using decoder 184
                                             // 38863:      }
  2, 0,                                      // 38868:      case 0x2: {
  OPC_CheckPredicate, 34,                    // 38870:       check predicate 34
  OPC_Decode, 205, 69, 184, 1,               // 38872:       decode to UZPQ2_ZZZ_H using decoder 184
                                             // 38872:      }
                                             // 38872:      } // switch Inst[22:21]
                                             // 38872:     }
  4, 11,                                     // 38877:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 38879:      check predicate 35
  OPC_CheckField, 21, 1, 1,                  // 38881:      check Inst[21] == 0x1
  OPC_Decode, 138, 52, 195, 2,               // 38885:      decode to SQDMULH_ZZZI_H using decoder 323
                                             // 38885:     }
  5, 11,                                     // 38890:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 38892:      check predicate 35
  OPC_CheckField, 21, 1, 1,                  // 38894:      check Inst[21] == 0x1
  OPC_Decode, 136, 53, 195, 2,               // 38898:      decode to SQRDMULH_ZZZI_H using decoder 323
                                             // 38898:     }
  6, 0,                                      // 38903:     case 0x6: {
  OPC_SwitchField, 21, 1,                    // 38905:      switch Inst[21] {
  0, 21,                                     // 38908:      case 0x0: {
  OPC_SwitchField, 22, 1,                    // 38910:       switch Inst[22] {
  0, 7,                                      // 38913:       case 0x0: {
  OPC_CheckPredicate, 34,                    // 38915:        check predicate 34
  OPC_Decode, 181, 61, 184, 1,               // 38917:        decode to TBLQ_ZZZ_B using decoder 184
                                             // 38917:       }
  1, 0,                                      // 38922:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 38924:        check predicate 34
  OPC_Decode, 183, 61, 184, 1,               // 38926:        decode to TBLQ_ZZZ_H using decoder 184
                                             // 38926:       }
                                             // 38926:       } // switch Inst[22]
                                             // 38926:      }
  1, 0,                                      // 38931:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 38933:       check predicate 35
  OPC_Decode, 137, 43, 195, 2,               // 38935:       decode to MUL_ZZZI_H using decoder 323
                                             // 38935:      }
                                             // 38935:      } // switch Inst[21]
                                             // 38935:     }
                                             // 38935:     } // switch Inst[12:10]
                                             // 38935:    }
  1, 241, 1,                                 // 38940:    case 0x1: {
  OPC_SwitchField, 21, 2,                    // 38943:     switch Inst[22:21] {
  0, 48,                                     // 38946:     case 0x0: {
  OPC_SwitchField, 10, 3,                    // 38948:      switch Inst[12:10] {
  0, 7,                                      // 38951:      case 0x0: {
  OPC_CheckPredicate, 34,                    // 38953:       check predicate 34
  OPC_Decode, 160, 71, 184, 1,               // 38955:       decode to ZIPQ1_ZZZ_S using decoder 184
                                             // 38955:      }
  1, 7,                                      // 38960:      case 0x1: {
  OPC_CheckPredicate, 34,                    // 38962:       check predicate 34
  OPC_Decode, 164, 71, 184, 1,               // 38964:       decode to ZIPQ2_ZZZ_S using decoder 184
                                             // 38964:      }
  2, 7,                                      // 38969:      case 0x2: {
  OPC_CheckPredicate, 34,                    // 38971:       check predicate 34
  OPC_Decode, 202, 69, 184, 1,               // 38973:       decode to UZPQ1_ZZZ_S using decoder 184
                                             // 38973:      }
  3, 7,                                      // 38978:      case 0x3: {
  OPC_CheckPredicate, 34,                    // 38980:       check predicate 34
  OPC_Decode, 206, 69, 184, 1,               // 38982:       decode to UZPQ2_ZZZ_S using decoder 184
                                             // 38982:      }
  6, 0,                                      // 38987:      case 0x6: {
  OPC_CheckPredicate, 34,                    // 38989:       check predicate 34
  OPC_Decode, 184, 61, 184, 1,               // 38991:       decode to TBLQ_ZZZ_S using decoder 184
                                             // 38991:      }
                                             // 38991:      } // switch Inst[12:10]
                                             // 38991:     }
  1, 67,                                     // 38996:     case 0x1: {
  OPC_SwitchField, 10, 1,                    // 38998:      switch Inst[10] {
  0, 35,                                     // 39001:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 39003:       switch Inst[12] {
  0, 7,                                      // 39006:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39008:        check predicate 35
  OPC_Decode, 157, 52, 191, 2,               // 39010:        decode to SQDMULLB_ZZZI_S using decoder 319
                                             // 39010:       }
  1, 0,                                      // 39015:       case 0x1: {
  OPC_SwitchField, 11, 1,                    // 39017:        switch Inst[11] {
  0, 7,                                      // 39020:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 39022:         check predicate 35
  OPC_Decode, 139, 52, 196, 2,               // 39024:         decode to SQDMULH_ZZZI_S using decoder 324
                                             // 39024:        }
  1, 0,                                      // 39029:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 39031:         check predicate 35
  OPC_Decode, 138, 43, 196, 2,               // 39033:         decode to MUL_ZZZI_S using decoder 324
                                             // 39033:        }
                                             // 39033:        } // switch Inst[11]
                                             // 39033:       }
                                             // 39033:       } // switch Inst[12]
                                             // 39033:      }
  1, 0,                                      // 39038:      case 0x1: {
  OPC_SwitchField, 12, 1,                    // 39040:       switch Inst[12] {
  0, 7,                                      // 39043:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39045:        check predicate 35
  OPC_Decode, 162, 52, 191, 2,               // 39047:        decode to SQDMULLT_ZZZI_S using decoder 319
                                             // 39047:       }
  1, 0,                                      // 39052:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39054:        check predicate 35
  OPC_CheckField, 11, 1, 0,                  // 39056:        check Inst[11] == 0x0
  OPC_Decode, 137, 53, 196, 2,               // 39060:        decode to SQRDMULH_ZZZI_S using decoder 324
                                             // 39060:       }
                                             // 39060:       } // switch Inst[12]
                                             // 39060:      }
                                             // 39060:      } // switch Inst[10]
                                             // 39060:     }
  2, 48,                                     // 39065:     case 0x2: {
  OPC_SwitchField, 10, 3,                    // 39067:      switch Inst[12:10] {
  0, 7,                                      // 39070:      case 0x0: {
  OPC_CheckPredicate, 34,                    // 39072:       check predicate 34
  OPC_Decode, 158, 71, 184, 1,               // 39074:       decode to ZIPQ1_ZZZ_D using decoder 184
                                             // 39074:      }
  1, 7,                                      // 39079:      case 0x1: {
  OPC_CheckPredicate, 34,                    // 39081:       check predicate 34
  OPC_Decode, 162, 71, 184, 1,               // 39083:       decode to ZIPQ2_ZZZ_D using decoder 184
                                             // 39083:      }
  2, 7,                                      // 39088:      case 0x2: {
  OPC_CheckPredicate, 34,                    // 39090:       check predicate 34
  OPC_Decode, 200, 69, 184, 1,               // 39092:       decode to UZPQ1_ZZZ_D using decoder 184
                                             // 39092:      }
  3, 7,                                      // 39097:      case 0x3: {
  OPC_CheckPredicate, 34,                    // 39099:       check predicate 34
  OPC_Decode, 204, 69, 184, 1,               // 39101:       decode to UZPQ2_ZZZ_D using decoder 184
                                             // 39101:      }
  6, 0,                                      // 39106:      case 0x6: {
  OPC_CheckPredicate, 34,                    // 39108:       check predicate 34
  OPC_Decode, 182, 61, 184, 1,               // 39110:       decode to TBLQ_ZZZ_D using decoder 184
                                             // 39110:      }
                                             // 39110:      } // switch Inst[12:10]
                                             // 39110:     }
  3, 0,                                      // 39115:     case 0x3: {
  OPC_SwitchField, 10, 1,                    // 39117:      switch Inst[10] {
  0, 35,                                     // 39120:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 39122:       switch Inst[12] {
  0, 7,                                      // 39125:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39127:        check predicate 35
  OPC_Decode, 156, 52, 192, 2,               // 39129:        decode to SQDMULLB_ZZZI_D using decoder 320
                                             // 39129:       }
  1, 0,                                      // 39134:       case 0x1: {
  OPC_SwitchField, 11, 1,                    // 39136:        switch Inst[11] {
  0, 7,                                      // 39139:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 39141:         check predicate 35
  OPC_Decode, 137, 52, 197, 2,               // 39143:         decode to SQDMULH_ZZZI_D using decoder 325
                                             // 39143:        }
  1, 0,                                      // 39148:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 39150:         check predicate 35
  OPC_Decode, 136, 43, 197, 2,               // 39152:         decode to MUL_ZZZI_D using decoder 325
                                             // 39152:        }
                                             // 39152:        } // switch Inst[11]
                                             // 39152:       }
                                             // 39152:       } // switch Inst[12]
                                             // 39152:      }
  1, 0,                                      // 39157:      case 0x1: {
  OPC_SwitchField, 12, 1,                    // 39159:       switch Inst[12] {
  0, 7,                                      // 39162:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39164:        check predicate 35
  OPC_Decode, 161, 52, 192, 2,               // 39166:        decode to SQDMULLT_ZZZI_D using decoder 320
                                             // 39166:       }
  1, 0,                                      // 39171:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39173:        check predicate 35
  OPC_CheckField, 11, 1, 0,                  // 39175:        check Inst[11] == 0x0
  OPC_Decode, 135, 53, 197, 2,               // 39179:        decode to SQRDMULH_ZZZI_D using decoder 325
                                             // 39179:       }
                                             // 39179:       } // switch Inst[12]
                                             // 39179:      }
                                             // 39179:      } // switch Inst[10]
                                             // 39179:     }
                                             // 39179:     } // switch Inst[22:21]
                                             // 39179:    }
  2, 130, 4,                                 // 39184:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 39187:     switch Inst[12:10] {
  0, 75,                                     // 39190:     case 0x0: {
  OPC_SwitchField, 21, 2,                    // 39192:      switch Inst[22:21] {
  0, 25,                                     // 39195:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 39197:       switch Inst[20] {
  0, 11,                                     // 39200:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39202:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 39204:        check Inst[19] == 0x1
  OPC_Decode, 207, 55, 199, 1,               // 39208:        decode to SSRA_ZZI_B using decoder 199
                                             // 39208:       }
  1, 0,                                      // 39213:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39215:        check predicate 35
  OPC_Decode, 209, 55, 200, 1,               // 39217:        decode to SSRA_ZZI_H using decoder 200
                                             // 39217:       }
                                             // 39217:       } // switch Inst[20]
                                             // 39217:      }
  1, 34,                                     // 39222:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 39224:       switch Inst[20:16] {
  0, 11,                                     // 39227:       case 0x0: {
  OPC_CheckPredicate, 46,                    // 39229:        check predicate 46
  OPC_CheckField, 5, 5, 0,                   // 39231:        check Inst[9:5] == 0x0
  OPC_Decode, 145, 14, 198, 2,               // 39235:        decode to AESMC_ZZ_B using decoder 326
                                             // 39235:       }
  2, 7,                                      // 39240:       case 0x2: {
  OPC_CheckPredicate, 46,                    // 39242:        check predicate 46
  OPC_Decode, 141, 14, 213, 1,               // 39244:        decode to AESE_ZZZ_B using decoder 213
                                             // 39244:       }
  3, 0,                                      // 39249:       case 0x3: {
  OPC_CheckPredicate, 52,                    // 39251:        check predicate 52
  OPC_Decode, 218, 48, 213, 1,               // 39253:        decode to SM4E_ZZZ_S using decoder 213
                                             // 39253:       }
                                             // 39253:       } // switch Inst[20:16]
                                             // 39253:      }
  2, 0,                                      // 39258:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39260:       check predicate 35
  OPC_Decode, 210, 55, 201, 1,               // 39262:       decode to SSRA_ZZI_S using decoder 201
                                             // 39262:      }
                                             // 39262:      } // switch Inst[22:21]
                                             // 39262:     }
  1, 66,                                     // 39267:     case 0x1: {
  OPC_SwitchField, 21, 2,                    // 39269:      switch Inst[22:21] {
  0, 25,                                     // 39272:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 39274:       switch Inst[20] {
  0, 11,                                     // 39277:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39279:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 39281:        check Inst[19] == 0x1
  OPC_Decode, 228, 68, 199, 1,               // 39285:        decode to USRA_ZZI_B using decoder 199
                                             // 39285:       }
  1, 0,                                      // 39290:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39292:        check predicate 35
  OPC_Decode, 230, 68, 200, 1,               // 39294:        decode to USRA_ZZI_H using decoder 200
                                             // 39294:       }
                                             // 39294:       } // switch Inst[20]
                                             // 39294:      }
  1, 25,                                     // 39299:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 39301:       switch Inst[20:16] {
  0, 11,                                     // 39304:       case 0x0: {
  OPC_CheckPredicate, 46,                    // 39306:        check predicate 46
  OPC_CheckField, 5, 5, 0,                   // 39308:        check Inst[9:5] == 0x0
  OPC_Decode, 143, 14, 198, 2,               // 39312:        decode to AESIMC_ZZ_B using decoder 326
                                             // 39312:       }
  2, 0,                                      // 39317:       case 0x2: {
  OPC_CheckPredicate, 46,                    // 39319:        check predicate 46
  OPC_Decode, 135, 14, 213, 1,               // 39321:        decode to AESD_ZZZ_B using decoder 213
                                             // 39321:       }
                                             // 39321:       } // switch Inst[20:16]
                                             // 39321:      }
  2, 0,                                      // 39326:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39328:       check predicate 35
  OPC_Decode, 231, 68, 201, 1,               // 39330:       decode to USRA_ZZI_S using decoder 201
                                             // 39330:      }
                                             // 39330:      } // switch Inst[22:21]
                                             // 39330:     }
  2, 96,                                     // 39335:     case 0x2: {
  OPC_SwitchField, 21, 2,                    // 39337:      switch Inst[22:21] {
  0, 25,                                     // 39340:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 39342:       switch Inst[20] {
  0, 11,                                     // 39345:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39347:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 39349:        check Inst[19] == 0x1
  OPC_Decode, 167, 55, 199, 1,               // 39353:        decode to SRSRA_ZZI_B using decoder 199
                                             // 39353:       }
  1, 0,                                      // 39358:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39360:        check predicate 35
  OPC_Decode, 169, 55, 200, 1,               // 39362:        decode to SRSRA_ZZI_H using decoder 200
                                             // 39362:       }
                                             // 39362:       } // switch Inst[20]
                                             // 39362:      }
  1, 55,                                     // 39367:      case 0x1: {
  OPC_SwitchField, 16, 3,                    // 39369:       switch Inst[18:16] {
  2, 11,                                     // 39372:       case 0x2: {
  OPC_CheckPredicate, 53,                    // 39374:        check predicate 53
  OPC_CheckField, 0, 1, 0,                   // 39376:        check Inst[0] == 0x0
  OPC_Decode, 139, 14, 199, 2,               // 39380:        decode to AESE_2ZZI_B using decoder 327
                                             // 39380:       }
  3, 11,                                     // 39385:       case 0x3: {
  OPC_CheckPredicate, 53,                    // 39387:        check predicate 53
  OPC_CheckField, 0, 1, 0,                   // 39389:        check Inst[0] == 0x0
  OPC_Decode, 137, 14, 199, 2,               // 39393:        decode to AESEMC_2ZZI_B using decoder 327
                                             // 39393:       }
  6, 11,                                     // 39398:       case 0x6: {
  OPC_CheckPredicate, 53,                    // 39400:        check predicate 53
  OPC_CheckField, 0, 2, 0,                   // 39402:        check Inst[1:0] == 0x0
  OPC_Decode, 140, 14, 200, 2,               // 39406:        decode to AESE_4ZZI_B using decoder 328
                                             // 39406:       }
  7, 0,                                      // 39411:       case 0x7: {
  OPC_CheckPredicate, 53,                    // 39413:        check predicate 53
  OPC_CheckField, 0, 2, 0,                   // 39415:        check Inst[1:0] == 0x0
  OPC_Decode, 138, 14, 200, 2,               // 39419:        decode to AESEMC_4ZZI_B using decoder 328
                                             // 39419:       }
                                             // 39419:       } // switch Inst[18:16]
                                             // 39419:      }
  2, 0,                                      // 39424:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39426:       check predicate 35
  OPC_Decode, 170, 55, 201, 1,               // 39428:       decode to SRSRA_ZZI_S using decoder 201
                                             // 39428:      }
                                             // 39428:      } // switch Inst[22:21]
                                             // 39428:     }
  3, 96,                                     // 39433:     case 0x3: {
  OPC_SwitchField, 21, 2,                    // 39435:      switch Inst[22:21] {
  0, 25,                                     // 39438:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 39440:       switch Inst[20] {
  0, 11,                                     // 39443:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39445:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 39447:        check Inst[19] == 0x1
  OPC_Decode, 131, 68, 199, 1,               // 39451:        decode to URSRA_ZZI_B using decoder 199
                                             // 39451:       }
  1, 0,                                      // 39456:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39458:        check predicate 35
  OPC_Decode, 133, 68, 200, 1,               // 39460:        decode to URSRA_ZZI_H using decoder 200
                                             // 39460:       }
                                             // 39460:       } // switch Inst[20]
                                             // 39460:      }
  1, 55,                                     // 39465:      case 0x1: {
  OPC_SwitchField, 16, 3,                    // 39467:       switch Inst[18:16] {
  2, 11,                                     // 39470:       case 0x2: {
  OPC_CheckPredicate, 53,                    // 39472:        check predicate 53
  OPC_CheckField, 0, 1, 0,                   // 39474:        check Inst[0] == 0x0
  OPC_Decode, 133, 14, 199, 2,               // 39478:        decode to AESD_2ZZI_B using decoder 327
                                             // 39478:       }
  3, 11,                                     // 39483:       case 0x3: {
  OPC_CheckPredicate, 53,                    // 39485:        check predicate 53
  OPC_CheckField, 0, 1, 0,                   // 39487:        check Inst[0] == 0x0
  OPC_Decode, 131, 14, 199, 2,               // 39491:        decode to AESDIMC_2ZZI_B using decoder 327
                                             // 39491:       }
  6, 11,                                     // 39496:       case 0x6: {
  OPC_CheckPredicate, 53,                    // 39498:        check predicate 53
  OPC_CheckField, 0, 2, 0,                   // 39500:        check Inst[1:0] == 0x0
  OPC_Decode, 134, 14, 200, 2,               // 39504:        decode to AESD_4ZZI_B using decoder 328
                                             // 39504:       }
  7, 0,                                      // 39509:       case 0x7: {
  OPC_CheckPredicate, 53,                    // 39511:        check predicate 53
  OPC_CheckField, 0, 2, 0,                   // 39513:        check Inst[1:0] == 0x0
  OPC_Decode, 132, 14, 200, 2,               // 39517:        decode to AESDIMC_4ZZI_B using decoder 328
                                             // 39517:       }
                                             // 39517:       } // switch Inst[18:16]
                                             // 39517:      }
  2, 0,                                      // 39522:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39524:       check predicate 35
  OPC_Decode, 134, 68, 201, 1,               // 39526:       decode to URSRA_ZZI_S using decoder 201
                                             // 39526:      }
                                             // 39526:      } // switch Inst[22:21]
                                             // 39526:     }
  4, 48,                                     // 39531:     case 0x4: {
  OPC_SwitchField, 21, 2,                    // 39533:      switch Inst[22:21] {
  0, 25,                                     // 39536:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 39538:       switch Inst[20] {
  0, 11,                                     // 39541:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39543:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 39545:        check Inst[19] == 0x1
  OPC_Decode, 239, 54, 199, 1,               // 39549:        decode to SRI_ZZI_B using decoder 199
                                             // 39549:       }
  1, 0,                                      // 39554:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39556:        check predicate 35
  OPC_Decode, 241, 54, 200, 1,               // 39558:        decode to SRI_ZZI_H using decoder 200
                                             // 39558:       }
                                             // 39558:       } // switch Inst[20]
                                             // 39558:      }
  1, 7,                                      // 39563:      case 0x1: {
  OPC_CheckPredicate, 52,                    // 39565:       check predicate 52
  OPC_Decode, 216, 48, 184, 1,               // 39567:       decode to SM4EKEY_ZZZ_S using decoder 184
                                             // 39567:      }
  2, 0,                                      // 39572:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39574:       check predicate 35
  OPC_Decode, 242, 54, 201, 1,               // 39576:       decode to SRI_ZZI_S using decoder 201
                                             // 39576:      }
                                             // 39576:      } // switch Inst[22:21]
                                             // 39576:     }
  5, 48,                                     // 39581:     case 0x5: {
  OPC_SwitchField, 21, 2,                    // 39583:      switch Inst[22:21] {
  0, 25,                                     // 39586:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 39588:       switch Inst[20] {
  0, 11,                                     // 39591:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 39593:        check predicate 35
  OPC_CheckField, 19, 1, 1,                  // 39595:        check Inst[19] == 0x1
  OPC_Decode, 196, 48, 201, 2,               // 39599:        decode to SLI_ZZI_B using decoder 329
                                             // 39599:       }
  1, 0,                                      // 39604:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 39606:        check predicate 35
  OPC_Decode, 198, 48, 202, 2,               // 39608:        decode to SLI_ZZI_H using decoder 330
                                             // 39608:       }
                                             // 39608:       } // switch Inst[20]
                                             // 39608:      }
  1, 7,                                      // 39613:      case 0x1: {
  OPC_CheckPredicate, 54,                    // 39615:       check predicate 54
  OPC_Decode, 229, 44, 184, 1,               // 39617:       decode to RAX1_ZZZ_D using decoder 184
                                             // 39617:      }
  2, 0,                                      // 39622:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39624:       check predicate 35
  OPC_Decode, 199, 48, 203, 2,               // 39626:       decode to SLI_ZZI_S using decoder 331
                                             // 39626:      }
                                             // 39626:      } // switch Inst[22:21]
                                             // 39626:     }
  6, 33,                                     // 39631:     case 0x6: {
  OPC_SwitchField, 21, 2,                    // 39633:      switch Inst[22:21] {
  0, 7,                                      // 39636:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 39638:       check predicate 35
  OPC_Decode, 144, 46, 198, 1,               // 39640:       decode to SABA_ZZZ_B using decoder 198
                                             // 39640:      }
  1, 10,                                     // 39645:      case 0x1: {
  OPC_CheckPredicate, 53,                    // 39647:       check predicate 53
  OPC_CheckField, 0, 1, 0,                   // 39649:       check Inst[0] == 0x0
  OPC_Decode, 144, 44, 119,                  // 39653:       decode to PMULL_2ZZZ_Q using decoder 119
                                             // 39653:      }
  2, 0,                                      // 39657:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39659:       check predicate 35
  OPC_Decode, 146, 46, 198, 1,               // 39661:       decode to SABA_ZZZ_H using decoder 198
                                             // 39661:      }
                                             // 39661:      } // switch Inst[22:21]
                                             // 39661:     }
  7, 0,                                      // 39666:     case 0x7: {
  OPC_SwitchField, 21, 2,                    // 39668:      switch Inst[22:21] {
  0, 7,                                      // 39671:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 39673:       check predicate 35
  OPC_Decode, 148, 62, 198, 1,               // 39675:       decode to UABA_ZZZ_B using decoder 198
                                             // 39675:      }
  1, 10,                                     // 39680:      case 0x1: {
  OPC_CheckPredicate, 53,                    // 39682:       check predicate 53
  OPC_CheckField, 0, 1, 0,                   // 39684:       check Inst[0] == 0x0
  OPC_Decode, 129, 44, 117,                  // 39688:       decode to PMLAL_2ZZZ_Q using decoder 117
                                             // 39688:      }
  2, 0,                                      // 39692:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39694:       check predicate 35
  OPC_Decode, 150, 62, 198, 1,               // 39696:       decode to UABA_ZZZ_H using decoder 198
                                             // 39696:      }
                                             // 39696:      } // switch Inst[22:21]
                                             // 39696:     }
                                             // 39696:     } // switch Inst[12:10]
                                             // 39696:    }
  3, 0,                                      // 39701:    case 0x3: {
  OPC_SwitchField, 10, 3,                    // 39703:     switch Inst[12:10] {
  0, 11,                                     // 39706:     case 0x0: {
  OPC_CheckPredicate, 35,                    // 39708:      check predicate 35
  OPC_CheckField, 21, 1, 0,                  // 39710:      check Inst[21] == 0x0
  OPC_Decode, 208, 55, 202, 1,               // 39714:      decode to SSRA_ZZI_D using decoder 202
                                             // 39714:     }
  1, 11,                                     // 39719:     case 0x1: {
  OPC_CheckPredicate, 35,                    // 39721:      check predicate 35
  OPC_CheckField, 21, 1, 0,                  // 39723:      check Inst[21] == 0x0
  OPC_Decode, 229, 68, 202, 1,               // 39727:      decode to USRA_ZZI_D using decoder 202
                                             // 39727:     }
  2, 11,                                     // 39732:     case 0x2: {
  OPC_CheckPredicate, 35,                    // 39734:      check predicate 35
  OPC_CheckField, 21, 1, 0,                  // 39736:      check Inst[21] == 0x0
  OPC_Decode, 168, 55, 202, 1,               // 39740:      decode to SRSRA_ZZI_D using decoder 202
                                             // 39740:     }
  3, 11,                                     // 39745:     case 0x3: {
  OPC_CheckPredicate, 35,                    // 39747:      check predicate 35
  OPC_CheckField, 21, 1, 0,                  // 39749:      check Inst[21] == 0x0
  OPC_Decode, 132, 68, 202, 1,               // 39753:      decode to URSRA_ZZI_D using decoder 202
                                             // 39753:     }
  4, 11,                                     // 39758:     case 0x4: {
  OPC_CheckPredicate, 35,                    // 39760:      check predicate 35
  OPC_CheckField, 21, 1, 0,                  // 39762:      check Inst[21] == 0x0
  OPC_Decode, 240, 54, 202, 1,               // 39766:      decode to SRI_ZZI_D using decoder 202
                                             // 39766:     }
  5, 11,                                     // 39771:     case 0x5: {
  OPC_CheckPredicate, 35,                    // 39773:      check predicate 35
  OPC_CheckField, 21, 1, 0,                  // 39775:      check Inst[21] == 0x0
  OPC_Decode, 197, 48, 204, 2,               // 39779:      decode to SLI_ZZI_D using decoder 332
                                             // 39779:     }
  6, 21,                                     // 39784:     case 0x6: {
  OPC_SwitchField, 21, 2,                    // 39786:      switch Inst[22:21] {
  0, 7,                                      // 39789:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 39791:       check predicate 35
  OPC_Decode, 147, 46, 198, 1,               // 39793:       decode to SABA_ZZZ_S using decoder 198
                                             // 39793:      }
  2, 0,                                      // 39798:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39800:       check predicate 35
  OPC_Decode, 145, 46, 198, 1,               // 39802:       decode to SABA_ZZZ_D using decoder 198
                                             // 39802:      }
                                             // 39802:      } // switch Inst[22:21]
                                             // 39802:     }
  7, 0,                                      // 39807:     case 0x7: {
  OPC_SwitchField, 21, 2,                    // 39809:      switch Inst[22:21] {
  0, 7,                                      // 39812:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 39814:       check predicate 35
  OPC_Decode, 151, 62, 198, 1,               // 39816:       decode to UABA_ZZZ_S using decoder 198
                                             // 39816:      }
  2, 0,                                      // 39821:      case 0x2: {
  OPC_CheckPredicate, 35,                    // 39823:       check predicate 35
  OPC_Decode, 149, 62, 198, 1,               // 39825:       decode to UABA_ZZZ_D using decoder 198
                                             // 39825:      }
                                             // 39825:      } // switch Inst[22:21]
                                             // 39825:     }
                                             // 39825:     } // switch Inst[12:10]
                                             // 39825:    }
                                             // 39825:    } // switch Inst[25:23]
                                             // 39825:   }
                                             // 39825:   } // switch Inst[15:13]
                                             // 39825:  }
  3, 244, 42,                                // 39830:  case 0x3: {
  OPC_SwitchField, 23, 3,                    // 39833:   switch Inst[25:23] {
  0, 223, 6,                                 // 39836:   case 0x0: {
  OPC_SwitchField, 15, 1,                    // 39839:    switch Inst[15] {
  0, 184, 1,                                 // 39842:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 39845:     switch Inst[21] {
  0, 11,                                     // 39848:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 39850:      check predicate 33
  OPC_CheckField, 22, 1, 1,                  // 39852:      check Inst[22] == 0x1
  OPC_Decode, 132, 23, 205, 2,               // 39856:      decode to FCMLA_ZPmZZ_H using decoder 333
                                             // 39856:     }
  1, 0,                                      // 39861:     case 0x1: {
  OPC_SwitchField, 12, 3,                    // 39863:      switch Inst[14:12] {
  0, 39,                                     // 39866:      case 0x0: {
  OPC_SwitchField, 10, 2,                    // 39868:       switch Inst[11:10] {
  0, 7,                                      // 39871:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 39873:        check predicate 33
  OPC_Decode, 193, 28, 177, 2,               // 39875:        decode to FMLA_ZZZI_H using decoder 305
                                             // 39875:       }
  1, 7,                                      // 39880:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 39882:        check predicate 33
  OPC_Decode, 251, 28, 177, 2,               // 39884:        decode to FMLS_ZZZI_H using decoder 305
                                             // 39884:       }
  2, 7,                                      // 39889:       case 0x2: {
  OPC_CheckPredicate, 55,                    // 39891:        check predicate 55
  OPC_Decode, 192, 15, 177, 2,               // 39893:        decode to BFMLA_ZZZI using decoder 305
                                             // 39893:       }
  3, 0,                                      // 39898:       case 0x3: {
  OPC_CheckPredicate, 55,                    // 39900:        check predicate 55
  OPC_Decode, 212, 15, 177, 2,               // 39902:        decode to BFMLS_ZZZI using decoder 305
                                             // 39902:       }
                                             // 39902:       } // switch Inst[11:10]
                                             // 39902:      }
  2, 44,                                     // 39907:      case 0x2: {
  OPC_SwitchField, 10, 2,                    // 39909:       switch Inst[11:10] {
  0, 7,                                      // 39912:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 39914:        check predicate 33
  OPC_Decode, 137, 30, 195, 2,               // 39916:        decode to FMUL_ZZZI_H using decoder 323
                                             // 39916:       }
  1, 21,                                     // 39921:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 39923:        switch Inst[22] {
  0, 7,                                      // 39926:        case 0x0: {
  OPC_CheckPredicate, 55,                    // 39928:         check predicate 55
  OPC_Decode, 255, 14, 198, 1,               // 39930:         decode to BFCLAMP_ZZZ using decoder 198
                                             // 39930:        }
  1, 0,                                      // 39935:        case 0x1: {
  OPC_CheckPredicate, 18,                    // 39937:         check predicate 18
  OPC_Decode, 191, 22, 198, 1,               // 39939:         decode to FCLAMP_ZZZ_H using decoder 198
                                             // 39939:        }
                                             // 39939:        } // switch Inst[22]
                                             // 39939:       }
  2, 0,                                      // 39944:       case 0x2: {
  OPC_CheckPredicate, 55,                    // 39946:        check predicate 55
  OPC_Decode, 242, 15, 195, 2,               // 39948:        decode to BFMUL_ZZZI using decoder 323
                                             // 39948:       }
                                             // 39948:       } // switch Inst[11:10]
                                             // 39948:      }
  4, 61,                                     // 39953:      case 0x4: {
  OPC_SwitchField, 10, 1,                    // 39955:       switch Inst[10] {
  0, 29,                                     // 39958:       case 0x0: {
  OPC_SwitchField, 22, 1,                    // 39960:        switch Inst[22] {
  0, 11,                                     // 39963:        case 0x0: {
  OPC_CheckPredicate, 18,                    // 39965:         check predicate 18
  OPC_CheckField, 11, 1, 0,                  // 39967:         check Inst[11] == 0x0
  OPC_Decode, 144, 26, 179, 2,               // 39971:         decode to FDOT_ZZZI_S using decoder 307
                                             // 39971:        }
  1, 0,                                      // 39976:        case 0x1: {
  OPC_CheckPredicate, 56,                    // 39978:         check predicate 56
  OPC_CheckField, 11, 1, 0,                  // 39980:         check Inst[11] == 0x0
  OPC_Decode, 145, 15, 179, 2,               // 39984:         decode to BFDOT_ZZI using decoder 307
                                             // 39984:        }
                                             // 39984:        } // switch Inst[22]
                                             // 39984:       }
  1, 0,                                      // 39989:       case 0x1: {
  OPC_SwitchField, 22, 1,                    // 39991:        switch Inst[22] {
  0, 7,                                      // 39994:        case 0x0: {
  OPC_CheckPredicate, 57,                    // 39996:         check predicate 57
  OPC_Decode, 142, 26, 182, 2,               // 39998:         decode to FDOT_ZZZI_BtoH using decoder 310
                                             // 39998:        }
  1, 0,                                      // 40003:        case 0x1: {
  OPC_CheckPredicate, 58,                    // 40005:         check predicate 58
  OPC_CheckField, 11, 1, 0,                  // 40007:         check Inst[11] == 0x0
  OPC_Decode, 143, 26, 179, 2,               // 40011:         decode to FDOT_ZZZI_BtoS using decoder 307
                                             // 40011:        }
                                             // 40011:        } // switch Inst[22]
                                             // 40011:       }
                                             // 40011:       } // switch Inst[10]
                                             // 40011:      }
  5, 0,                                      // 40016:      case 0x5: {
  OPC_CheckPredicate, 59,                    // 40018:       check predicate 59
  OPC_CheckField, 22, 1, 0,                  // 40020:       check Inst[22] == 0x0
  OPC_Decode, 244, 27, 206, 2,               // 40024:       decode to FMLALB_ZZZI using decoder 334
                                             // 40024:      }
                                             // 40024:      } // switch Inst[14:12]
                                             // 40024:     }
                                             // 40024:     } // switch Inst[21]
                                             // 40024:    }
  1, 0,                                      // 40029:    case 0x1: {
  OPC_SwitchField, 13, 2,                    // 40031:     switch Inst[14:13] {
  0, 226, 1,                                 // 40034:     case 0x0: {
  OPC_SwitchField, 21, 2,                    // 40037:      switch Inst[22:21] {
  0, 21,                                     // 40040:      case 0x0: {
  OPC_SwitchField, 16, 5,                    // 40042:       switch Inst[20:16] {
  28, 7,                                     // 40045:       case 0x1c: {
  OPC_CheckPredicate, 36,                    // 40047:        check predicate 36
  OPC_Decode, 235, 30, 161, 1,               // 40049:        decode to FRINT32Z_ZPzZ_S using decoder 161
                                             // 40049:       }
  29, 0,                                     // 40054:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40056:        check predicate 36
  OPC_Decode, 253, 30, 161, 1,               // 40058:        decode to FRINT64Z_ZPzZ_S using decoder 161
                                             // 40058:       }
                                             // 40058:       } // switch Inst[20:16]
                                             // 40058:      }
  1, 39,                                     // 40063:      case 0x1: {
  OPC_SwitchField, 10, 3,                    // 40065:       switch Inst[12:10] {
  0, 7,                                      // 40068:       case 0x0: {
  OPC_CheckPredicate, 18,                    // 40070:        check predicate 18
  OPC_Decode, 147, 26, 198, 1,               // 40072:        decode to FDOT_ZZZ_S using decoder 198
                                             // 40072:       }
  1, 7,                                      // 40077:       case 0x1: {
  OPC_CheckPredicate, 57,                    // 40079:        check predicate 57
  OPC_Decode, 145, 26, 198, 1,               // 40081:        decode to FDOT_ZZZ_BtoH using decoder 198
                                             // 40081:       }
  2, 7,                                      // 40086:       case 0x2: {
  OPC_CheckPredicate, 59,                    // 40088:        check predicate 59
  OPC_Decode, 249, 27, 198, 1,               // 40090:        decode to FMLALLBB_ZZZ using decoder 198
                                             // 40090:       }
  6, 0,                                      // 40095:       case 0x6: {
  OPC_CheckPredicate, 59,                    // 40097:        check predicate 59
  OPC_Decode, 253, 27, 198, 1,               // 40099:        decode to FMLALLBT_ZZZ using decoder 198
                                             // 40099:       }
                                             // 40099:       } // switch Inst[12:10]
                                             // 40099:      }
  2, 133, 1,                                 // 40104:      case 0x2: {
  OPC_SwitchField, 17, 4,                    // 40107:       switch Inst[20:17] {
  0, 7,                                      // 40110:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 40112:        check predicate 33
  OPC_Decode, 171, 22, 207, 2,               // 40114:        decode to FCADD_ZPmZ_H using decoder 335
                                             // 40114:       }
  8, 11,                                     // 40119:       case 0x8: {
  OPC_CheckPredicate, 35,                    // 40121:        check predicate 35
  OPC_CheckField, 16, 1, 0,                  // 40123:        check Inst[16] == 0x0
  OPC_Decode, 233, 21, 160, 1,               // 40127:        decode to FADDP_ZPmZZ_H using decoder 160
                                             // 40127:       }
  10, 21,                                    // 40132:       case 0xa: {
  OPC_SwitchField, 16, 1,                    // 40134:        switch Inst[16] {
  0, 7,                                      // 40137:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 40139:         check predicate 35
  OPC_Decode, 188, 26, 160, 1,               // 40141:         decode to FMAXNMP_ZPmZZ_H using decoder 160
                                             // 40141:        }
  1, 0,                                      // 40146:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 40148:         check predicate 35
  OPC_Decode, 152, 27, 160, 1,               // 40150:         decode to FMINNMP_ZPmZZ_H using decoder 160
                                             // 40150:        }
                                             // 40150:        } // switch Inst[16]
                                             // 40150:       }
  11, 21,                                    // 40155:       case 0xb: {
  OPC_SwitchField, 16, 1,                    // 40157:        switch Inst[16] {
  0, 7,                                      // 40160:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 40162:         check predicate 35
  OPC_Decode, 232, 26, 160, 1,               // 40164:         decode to FMAXP_ZPmZZ_H using decoder 160
                                             // 40164:        }
  1, 0,                                      // 40169:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 40171:         check predicate 35
  OPC_Decode, 196, 27, 160, 1,               // 40173:         decode to FMINP_ZPmZZ_H using decoder 160
                                             // 40173:        }
                                             // 40173:        } // switch Inst[16]
                                             // 40173:       }
  12, 21,                                    // 40178:       case 0xc: {
  OPC_SwitchField, 16, 1,                    // 40180:        switch Inst[16] {
  0, 7,                                      // 40183:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 40185:         check predicate 36
  OPC_Decode, 184, 31, 161, 1,               // 40187:         decode to FRINTN_ZPzZ_H using decoder 161
                                             // 40187:        }
  1, 0,                                      // 40192:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 40194:         check predicate 36
  OPC_Decode, 138, 31, 161, 1,               // 40196:         decode to FRINTA_ZPzZ_H using decoder 161
                                             // 40196:        }
                                             // 40196:        } // switch Inst[16]
                                             // 40196:       }
  13, 11,                                    // 40201:       case 0xd: {
  OPC_CheckPredicate, 36,                    // 40203:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 40205:        check Inst[16] == 0x1
  OPC_Decode, 216, 30, 161, 1,               // 40209:        decode to FRECPX_ZPzZ_H using decoder 161
                                             // 40209:       }
  14, 11,                                    // 40214:       case 0xe: {
  OPC_CheckPredicate, 36,                    // 40216:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 40218:        check Inst[16] == 0x1
  OPC_Decode, 154, 47, 161, 1,               // 40222:        decode to SCVTF_ZPzZ_StoH using decoder 161
                                             // 40222:       }
  15, 0,                                     // 40227:       case 0xf: {
  OPC_CheckPredicate, 36,                    // 40229:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 40231:        check Inst[16] == 0x1
  OPC_Decode, 152, 25, 161, 1,               // 40235:        decode to FCVTZS_ZPzZ_HtoS using decoder 161
                                             // 40235:       }
                                             // 40235:       } // switch Inst[20:17]
                                             // 40235:      }
  3, 0,                                      // 40240:      case 0x3: {
  OPC_SwitchField, 10, 3,                    // 40242:       switch Inst[12:10] {
  0, 7,                                      // 40245:       case 0x0: {
  OPC_CheckPredicate, 56,                    // 40247:        check predicate 56
  OPC_Decode, 146, 15, 198, 1,               // 40249:        decode to BFDOT_ZZZ using decoder 198
                                             // 40249:       }
  1, 0,                                      // 40254:       case 0x1: {
  OPC_CheckPredicate, 58,                    // 40256:        check predicate 58
  OPC_Decode, 146, 26, 198, 1,               // 40258:        decode to FDOT_ZZZ_BtoS using decoder 198
                                             // 40258:       }
                                             // 40258:       } // switch Inst[12:10]
                                             // 40258:      }
                                             // 40258:      } // switch Inst[22:21]
                                             // 40258:     }
  1, 162, 1,                                 // 40263:     case 0x1: {
  OPC_SwitchField, 21, 2,                    // 40266:      switch Inst[22:21] {
  0, 48,                                     // 40269:      case 0x0: {
  OPC_SwitchField, 16, 5,                    // 40271:       switch Inst[20:16] {
  2, 7,                                      // 40274:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 40276:        check predicate 36
  OPC_Decode, 242, 24, 163, 1,               // 40278:        decode to FCVTXNT_ZPzZ_StoD using decoder 163
                                             // 40278:       }
  10, 7,                                     // 40283:       case 0xa: {
  OPC_CheckPredicate, 35,                    // 40285:        check predicate 35
  OPC_Decode, 241, 24, 163, 1,               // 40287:        decode to FCVTXNT_ZPmZ_DtoS using decoder 163
                                             // 40287:       }
  28, 7,                                     // 40292:       case 0x1c: {
  OPC_CheckPredicate, 36,                    // 40294:        check predicate 36
  OPC_Decode, 226, 30, 161, 1,               // 40296:        decode to FRINT32X_ZPzZ_S using decoder 161
                                             // 40296:       }
  29, 7,                                     // 40301:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40303:        check predicate 36
  OPC_Decode, 244, 30, 161, 1,               // 40305:        decode to FRINT64X_ZPzZ_S using decoder 161
                                             // 40305:       }
  30, 0,                                     // 40310:       case 0x1e: {
  OPC_CheckPredicate, 36,                    // 40312:        check predicate 36
  OPC_Decode, 175, 26, 161, 1,               // 40314:        decode to FLOGB_ZPzZ_H using decoder 161
                                             // 40314:       }
                                             // 40314:       } // switch Inst[20:16]
                                             // 40314:      }
  1, 21,                                     // 40319:      case 0x1: {
  OPC_SwitchField, 10, 3,                    // 40321:       switch Inst[12:10] {
  2, 7,                                      // 40324:       case 0x2: {
  OPC_CheckPredicate, 59,                    // 40326:        check predicate 59
  OPC_Decode, 129, 28, 198, 1,               // 40328:        decode to FMLALLTB_ZZZ using decoder 198
                                             // 40328:       }
  6, 0,                                      // 40333:       case 0x6: {
  OPC_CheckPredicate, 59,                    // 40335:        check predicate 59
  OPC_Decode, 133, 28, 198, 1,               // 40337:        decode to FMLALLTT_ZZZ using decoder 198
                                             // 40337:       }
                                             // 40337:       } // switch Inst[12:10]
                                             // 40337:      }
  2, 0,                                      // 40342:      case 0x2: {
  OPC_SwitchField, 16, 5,                    // 40344:       switch Inst[20:16] {
  16, 7,                                     // 40347:       case 0x10: {
  OPC_CheckPredicate, 34,                    // 40349:        check predicate 34
  OPC_Decode, 244, 21, 162, 1,               // 40351:        decode to FADDQV_H using decoder 162
                                             // 40351:       }
  20, 7,                                     // 40356:       case 0x14: {
  OPC_CheckPredicate, 34,                    // 40358:        check predicate 34
  OPC_Decode, 199, 26, 162, 1,               // 40360:        decode to FMAXNMQV_H using decoder 162
                                             // 40360:       }
  21, 7,                                     // 40365:       case 0x15: {
  OPC_CheckPredicate, 34,                    // 40367:        check predicate 34
  OPC_Decode, 163, 27, 162, 1,               // 40369:        decode to FMINNMQV_H using decoder 162
                                             // 40369:       }
  22, 7,                                     // 40374:       case 0x16: {
  OPC_CheckPredicate, 34,                    // 40376:        check predicate 34
  OPC_Decode, 243, 26, 162, 1,               // 40378:        decode to FMAXQV_H using decoder 162
                                             // 40378:       }
  23, 7,                                     // 40383:       case 0x17: {
  OPC_CheckPredicate, 34,                    // 40385:        check predicate 34
  OPC_Decode, 207, 27, 162, 1,               // 40387:        decode to FMINQV_H using decoder 162
                                             // 40387:       }
  24, 7,                                     // 40392:       case 0x18: {
  OPC_CheckPredicate, 36,                    // 40394:        check predicate 36
  OPC_Decode, 200, 31, 161, 1,               // 40396:        decode to FRINTP_ZPzZ_H using decoder 161
                                             // 40396:       }
  27, 7,                                     // 40401:       case 0x1b: {
  OPC_CheckPredicate, 36,                    // 40403:        check predicate 36
  OPC_Decode, 153, 32, 161, 1,               // 40405:        decode to FSQRT_ZPZz_H using decoder 161
                                             // 40405:       }
  29, 7,                                     // 40410:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40412:        check predicate 36
  OPC_Decode, 147, 63, 161, 1,               // 40414:        decode to UCVTF_ZPzZ_StoH using decoder 161
                                             // 40414:       }
  31, 0,                                     // 40419:       case 0x1f: {
  OPC_CheckPredicate, 36,                    // 40421:        check predicate 36
  OPC_Decode, 203, 25, 161, 1,               // 40423:        decode to FCVTZU_ZPzZ_HtoS using decoder 161
                                             // 40423:       }
                                             // 40423:       } // switch Inst[20:16]
                                             // 40423:      }
                                             // 40423:      } // switch Inst[22:21]
                                             // 40423:     }
  2, 129, 1,                                 // 40428:     case 0x2: {
  OPC_SwitchField, 21, 2,                    // 40431:      switch Inst[22:21] {
  0, 39,                                     // 40434:      case 0x0: {
  OPC_SwitchField, 16, 5,                    // 40436:       switch Inst[20:16] {
  26, 7,                                     // 40439:       case 0x1a: {
  OPC_CheckPredicate, 36,                    // 40441:        check predicate 36
  OPC_Decode, 247, 24, 161, 1,               // 40443:        decode to FCVTX_ZPzZ_DtoS using decoder 161
                                             // 40443:       }
  28, 7,                                     // 40448:       case 0x1c: {
  OPC_CheckPredicate, 36,                    // 40450:        check predicate 36
  OPC_Decode, 234, 30, 161, 1,               // 40452:        decode to FRINT32Z_ZPzZ_D using decoder 161
                                             // 40452:       }
  29, 7,                                     // 40457:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40459:        check predicate 36
  OPC_Decode, 252, 30, 161, 1,               // 40461:        decode to FRINT64Z_ZPzZ_D using decoder 161
                                             // 40461:       }
  30, 0,                                     // 40466:       case 0x1e: {
  OPC_CheckPredicate, 36,                    // 40468:        check predicate 36
  OPC_Decode, 176, 26, 161, 1,               // 40470:        decode to FLOGB_ZPzZ_S using decoder 161
                                             // 40470:       }
                                             // 40470:       } // switch Inst[20:16]
                                             // 40470:      }
  1, 11,                                     // 40475:      case 0x1: {
  OPC_CheckPredicate, 59,                    // 40477:       check predicate 59
  OPC_CheckField, 12, 1, 0,                  // 40479:       check Inst[12] == 0x0
  OPC_Decode, 250, 27, 206, 2,               // 40483:       decode to FMLALLBB_ZZZI using decoder 334
                                             // 40483:      }
  2, 57,                                     // 40488:      case 0x2: {
  OPC_SwitchField, 16, 5,                    // 40490:       switch Inst[20:16] {
  24, 7,                                     // 40493:       case 0x18: {
  OPC_CheckPredicate, 36,                    // 40495:        check predicate 36
  OPC_Decode, 168, 31, 161, 1,               // 40497:        decode to FRINTM_ZPzZ_H using decoder 161
                                             // 40497:       }
  25, 7,                                     // 40502:       case 0x19: {
  OPC_CheckPredicate, 36,                    // 40504:        check predicate 36
  OPC_Decode, 214, 31, 161, 1,               // 40506:        decode to FRINTX_ZPzZ_H using decoder 161
                                             // 40506:       }
  28, 7,                                     // 40511:       case 0x1c: {
  OPC_CheckPredicate, 36,                    // 40513:        check predicate 36
  OPC_Decode, 152, 47, 161, 1,               // 40515:        decode to SCVTF_ZPzZ_HtoH using decoder 161
                                             // 40515:       }
  29, 7,                                     // 40520:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40522:        check predicate 36
  OPC_Decode, 150, 47, 161, 1,               // 40524:        decode to SCVTF_ZPzZ_DtoH using decoder 161
                                             // 40524:       }
  30, 7,                                     // 40529:       case 0x1e: {
  OPC_CheckPredicate, 36,                    // 40531:        check predicate 36
  OPC_Decode, 151, 25, 161, 1,               // 40533:        decode to FCVTZS_ZPzZ_HtoH using decoder 161
                                             // 40533:       }
  31, 0,                                     // 40538:       case 0x1f: {
  OPC_CheckPredicate, 36,                    // 40540:        check predicate 36
  OPC_Decode, 150, 25, 161, 1,               // 40542:        decode to FCVTZS_ZPzZ_HtoD using decoder 161
                                             // 40542:       }
                                             // 40542:       } // switch Inst[20:16]
                                             // 40542:      }
  3, 0,                                      // 40547:      case 0x3: {
  OPC_CheckPredicate, 59,                    // 40549:       check predicate 59
  OPC_CheckField, 12, 1, 0,                  // 40551:       check Inst[12] == 0x0
  OPC_Decode, 254, 27, 206, 2,               // 40555:       decode to FMLALLBT_ZZZI using decoder 334
                                             // 40555:      }
                                             // 40555:      } // switch Inst[22:21]
                                             // 40555:     }
  3, 0,                                      // 40560:     case 0x3: {
  OPC_SwitchField, 21, 2,                    // 40562:      switch Inst[22:21] {
  0, 30,                                     // 40565:      case 0x0: {
  OPC_SwitchField, 16, 5,                    // 40567:       switch Inst[20:16] {
  28, 7,                                     // 40570:       case 0x1c: {
  OPC_CheckPredicate, 36,                    // 40572:        check predicate 36
  OPC_Decode, 225, 30, 161, 1,               // 40574:        decode to FRINT32X_ZPzZ_D using decoder 161
                                             // 40574:       }
  29, 7,                                     // 40579:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40581:        check predicate 36
  OPC_Decode, 243, 30, 161, 1,               // 40583:        decode to FRINT64X_ZPzZ_D using decoder 161
                                             // 40583:       }
  30, 0,                                     // 40588:       case 0x1e: {
  OPC_CheckPredicate, 36,                    // 40590:        check predicate 36
  OPC_Decode, 174, 26, 161, 1,               // 40592:        decode to FLOGB_ZPzZ_D using decoder 161
                                             // 40592:       }
                                             // 40592:       } // switch Inst[20:16]
                                             // 40592:      }
  1, 21,                                     // 40597:      case 0x1: {
  OPC_SwitchField, 10, 3,                    // 40599:       switch Inst[12:10] {
  0, 7,                                      // 40602:       case 0x0: {
  OPC_CheckPredicate, 60,                    // 40604:        check predicate 60
  OPC_Decode, 139, 29, 198, 1,               // 40606:        decode to FMMLA_ZZZ_BtoS using decoder 198
                                             // 40606:       }
  1, 0,                                      // 40611:       case 0x1: {
  OPC_CheckPredicate, 61,                    // 40613:        check predicate 61
  OPC_Decode, 208, 28, 198, 1,               // 40615:        decode to FMLLA_ZZZ_HtoS using decoder 198
                                             // 40615:       }
                                             // 40615:       } // switch Inst[12:10]
                                             // 40615:      }
  2, 57,                                     // 40620:      case 0x2: {
  OPC_SwitchField, 16, 5,                    // 40622:       switch Inst[20:16] {
  24, 7,                                     // 40625:       case 0x18: {
  OPC_CheckPredicate, 36,                    // 40627:        check predicate 36
  OPC_Decode, 228, 31, 161, 1,               // 40629:        decode to FRINTZ_ZPzZ_H using decoder 161
                                             // 40629:       }
  25, 7,                                     // 40634:       case 0x19: {
  OPC_CheckPredicate, 36,                    // 40636:        check predicate 36
  OPC_Decode, 152, 31, 161, 1,               // 40638:        decode to FRINTI_ZPzZ_H using decoder 161
                                             // 40638:       }
  28, 7,                                     // 40643:       case 0x1c: {
  OPC_CheckPredicate, 36,                    // 40645:        check predicate 36
  OPC_Decode, 145, 63, 161, 1,               // 40647:        decode to UCVTF_ZPzZ_HtoH using decoder 161
                                             // 40647:       }
  29, 7,                                     // 40652:       case 0x1d: {
  OPC_CheckPredicate, 36,                    // 40654:        check predicate 36
  OPC_Decode, 143, 63, 161, 1,               // 40656:        decode to UCVTF_ZPzZ_DtoH using decoder 161
                                             // 40656:       }
  30, 7,                                     // 40661:       case 0x1e: {
  OPC_CheckPredicate, 36,                    // 40663:        check predicate 36
  OPC_Decode, 202, 25, 161, 1,               // 40665:        decode to FCVTZU_ZPzZ_HtoH using decoder 161
                                             // 40665:       }
  31, 0,                                     // 40670:       case 0x1f: {
  OPC_CheckPredicate, 36,                    // 40672:        check predicate 36
  OPC_Decode, 201, 25, 161, 1,               // 40674:        decode to FCVTZU_ZPzZ_HtoD using decoder 161
                                             // 40674:       }
                                             // 40674:       } // switch Inst[20:16]
                                             // 40674:      }
  3, 0,                                      // 40679:      case 0x3: {
  OPC_SwitchField, 10, 3,                    // 40681:       switch Inst[12:10] {
  0, 7,                                      // 40684:       case 0x0: {
  OPC_CheckPredicate, 62,                    // 40686:        check predicate 62
  OPC_Decode, 138, 29, 198, 1,               // 40688:        decode to FMMLA_ZZZ_BtoH using decoder 198
                                             // 40688:       }
  1, 0,                                      // 40693:       case 0x1: {
  OPC_CheckPredicate, 63,                    // 40695:        check predicate 63
  OPC_Decode, 215, 15, 198, 1,               // 40697:        decode to BFMMLA_ZZZ_HtoS using decoder 198
                                             // 40697:       }
                                             // 40697:       } // switch Inst[12:10]
                                             // 40697:      }
                                             // 40697:      } // switch Inst[22:21]
                                             // 40697:     }
                                             // 40697:     } // switch Inst[14:13]
                                             // 40697:    }
                                             // 40697:    } // switch Inst[15]
                                             // 40697:   }
  1, 137, 11,                                // 40702:   case 0x1: {
  OPC_SwitchField, 21, 2,                    // 40705:    switch Inst[22:21] {
  0, 209, 3,                                 // 40708:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 40711:     switch Inst[15] {
  0, 7,                                      // 40714:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 40716:      check predicate 33
  OPC_Decode, 133, 23, 205, 2,               // 40718:      decode to FCMLA_ZPmZZ_S using decoder 333
                                             // 40718:     }
  1, 0,                                      // 40723:     case 0x1: {
  OPC_SwitchField, 17, 4,                    // 40725:      switch Inst[20:17] {
  0, 35,                                     // 40728:      case 0x0: {
  OPC_SwitchField, 13, 2,                    // 40730:       switch Inst[14:13] {
  0, 7,                                      // 40733:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 40735:        check predicate 33
  OPC_Decode, 172, 22, 207, 2,               // 40737:        decode to FCADD_ZPmZ_S using decoder 335
                                             // 40737:       }
  1, 0,                                      // 40742:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 40744:        switch Inst[16] {
  0, 7,                                      // 40747:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 40749:         check predicate 36
  OPC_Decode, 173, 24, 163, 1,               // 40751:         decode to FCVTNT_ZPzZ_StoH using decoder 163
                                             // 40751:        }
  1, 0,                                      // 40756:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 40758:         check predicate 36
  OPC_Decode, 235, 23, 161, 1,               // 40760:         decode to FCVTLT_ZPzZ_HtoS using decoder 161
                                             // 40760:        }
                                             // 40760:        } // switch Inst[16]
                                             // 40760:       }
                                             // 40760:       } // switch Inst[14:13]
                                             // 40760:      }
  1, 15,                                     // 40765:      case 0x1: {
  OPC_CheckPredicate, 36,                    // 40767:       check predicate 36
  OPC_CheckField, 16, 1, 0,                  // 40769:       check Inst[16] == 0x0
  OPC_CheckField, 13, 2, 1,                  // 40773:       check Inst[14:13] == 0x1
  OPC_Decode, 132, 15, 163, 1,               // 40777:       decode to BFCVTNT_ZPzZ_StoH using decoder 163
                                             // 40777:      }
  4, 29,                                     // 40782:      case 0x4: {
  OPC_SwitchField, 16, 1,                    // 40784:       switch Inst[16] {
  0, 11,                                     // 40787:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 40789:        check predicate 35
  OPC_CheckField, 13, 2, 1,                  // 40791:        check Inst[14:13] == 0x1
  OPC_Decode, 171, 24, 163, 1,               // 40795:        decode to FCVTNT_ZPmZ_StoH using decoder 163
                                             // 40795:       }
  1, 0,                                      // 40800:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 40802:        check predicate 35
  OPC_CheckField, 13, 2, 1,                  // 40804:        check Inst[14:13] == 0x1
  OPC_Decode, 233, 23, 163, 1,               // 40808:        decode to FCVTLT_ZPmZ_HtoS using decoder 163
                                             // 40808:       }
                                             // 40808:       } // switch Inst[16]
                                             // 40808:      }
  5, 15,                                     // 40813:      case 0x5: {
  OPC_CheckPredicate, 56,                    // 40815:       check predicate 56
  OPC_CheckField, 16, 1, 0,                  // 40817:       check Inst[16] == 0x0
  OPC_CheckField, 13, 2, 1,                  // 40821:       check Inst[14:13] == 0x1
  OPC_Decode, 131, 15, 163, 1,               // 40825:       decode to BFCVTNT_ZPmZ using decoder 163
                                             // 40825:      }
  8, 29,                                     // 40830:      case 0x8: {
  OPC_SwitchField, 13, 2,                    // 40832:       switch Inst[14:13] {
  0, 11,                                     // 40835:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 40837:        check predicate 35
  OPC_CheckField, 16, 1, 0,                  // 40839:        check Inst[16] == 0x0
  OPC_Decode, 234, 21, 160, 1,               // 40843:        decode to FADDP_ZPmZZ_S using decoder 160
                                             // 40843:       }
  1, 0,                                      // 40848:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 40850:        check predicate 34
  OPC_CheckField, 16, 1, 0,                  // 40852:        check Inst[16] == 0x0
  OPC_Decode, 245, 21, 162, 1,               // 40856:        decode to FADDQV_S using decoder 162
                                             // 40856:       }
                                             // 40856:       } // switch Inst[14:13]
                                             // 40856:      }
  10, 49,                                    // 40861:      case 0xa: {
  OPC_SwitchField, 13, 2,                    // 40863:       switch Inst[14:13] {
  0, 21,                                     // 40866:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 40868:        switch Inst[16] {
  0, 7,                                      // 40871:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 40873:         check predicate 35
  OPC_Decode, 189, 26, 160, 1,               // 40875:         decode to FMAXNMP_ZPmZZ_S using decoder 160
                                             // 40875:        }
  1, 0,                                      // 40880:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 40882:         check predicate 35
  OPC_Decode, 153, 27, 160, 1,               // 40884:         decode to FMINNMP_ZPmZZ_S using decoder 160
                                             // 40884:        }
                                             // 40884:        } // switch Inst[16]
                                             // 40884:       }
  1, 0,                                      // 40889:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 40891:        switch Inst[16] {
  0, 7,                                      // 40894:        case 0x0: {
  OPC_CheckPredicate, 34,                    // 40896:         check predicate 34
  OPC_Decode, 200, 26, 162, 1,               // 40898:         decode to FMAXNMQV_S using decoder 162
                                             // 40898:        }
  1, 0,                                      // 40903:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 40905:         check predicate 34
  OPC_Decode, 164, 27, 162, 1,               // 40907:         decode to FMINNMQV_S using decoder 162
                                             // 40907:        }
                                             // 40907:        } // switch Inst[16]
                                             // 40907:       }
                                             // 40907:       } // switch Inst[14:13]
                                             // 40907:      }
  11, 49,                                    // 40912:      case 0xb: {
  OPC_SwitchField, 13, 2,                    // 40914:       switch Inst[14:13] {
  0, 21,                                     // 40917:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 40919:        switch Inst[16] {
  0, 7,                                      // 40922:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 40924:         check predicate 35
  OPC_Decode, 233, 26, 160, 1,               // 40926:         decode to FMAXP_ZPmZZ_S using decoder 160
                                             // 40926:        }
  1, 0,                                      // 40931:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 40933:         check predicate 35
  OPC_Decode, 197, 27, 160, 1,               // 40935:         decode to FMINP_ZPmZZ_S using decoder 160
                                             // 40935:        }
                                             // 40935:        } // switch Inst[16]
                                             // 40935:       }
  1, 0,                                      // 40940:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 40942:        switch Inst[16] {
  0, 7,                                      // 40945:        case 0x0: {
  OPC_CheckPredicate, 34,                    // 40947:         check predicate 34
  OPC_Decode, 244, 26, 162, 1,               // 40949:         decode to FMAXQV_S using decoder 162
                                             // 40949:        }
  1, 0,                                      // 40954:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 40956:         check predicate 34
  OPC_Decode, 208, 27, 162, 1,               // 40958:         decode to FMINQV_S using decoder 162
                                             // 40958:        }
                                             // 40958:        } // switch Inst[16]
                                             // 40958:       }
                                             // 40958:       } // switch Inst[14:13]
                                             // 40958:      }
  12, 85,                                    // 40963:      case 0xc: {
  OPC_SwitchField, 13, 2,                    // 40965:       switch Inst[14:13] {
  0, 21,                                     // 40968:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 40970:        switch Inst[16] {
  0, 7,                                      // 40973:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 40975:         check predicate 36
  OPC_Decode, 185, 31, 161, 1,               // 40977:         decode to FRINTN_ZPzZ_S using decoder 161
                                             // 40977:        }
  1, 0,                                      // 40982:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 40984:         check predicate 36
  OPC_Decode, 139, 31, 161, 1,               // 40986:         decode to FRINTA_ZPzZ_S using decoder 161
                                             // 40986:        }
                                             // 40986:        } // switch Inst[16]
                                             // 40986:       }
  1, 11,                                     // 40991:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 40993:        check predicate 36
  OPC_CheckField, 16, 1, 0,                  // 40995:        check Inst[16] == 0x0
  OPC_Decode, 201, 31, 161, 1,               // 40999:        decode to FRINTP_ZPzZ_S using decoder 161
                                             // 40999:       }
  2, 21,                                     // 41004:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 41006:        switch Inst[16] {
  0, 7,                                      // 41009:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41011:         check predicate 36
  OPC_Decode, 169, 31, 161, 1,               // 41013:         decode to FRINTM_ZPzZ_S using decoder 161
                                             // 41013:        }
  1, 0,                                      // 41018:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41020:         check predicate 36
  OPC_Decode, 215, 31, 161, 1,               // 41022:         decode to FRINTX_ZPzZ_S using decoder 161
                                             // 41022:        }
                                             // 41022:        } // switch Inst[16]
                                             // 41022:       }
  3, 0,                                      // 41027:       case 0x3: {
  OPC_SwitchField, 16, 1,                    // 41029:        switch Inst[16] {
  0, 7,                                      // 41032:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41034:         check predicate 36
  OPC_Decode, 229, 31, 161, 1,               // 41036:         decode to FRINTZ_ZPzZ_S using decoder 161
                                             // 41036:        }
  1, 0,                                      // 41041:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41043:         check predicate 36
  OPC_Decode, 153, 31, 161, 1,               // 41045:         decode to FRINTI_ZPzZ_S using decoder 161
                                             // 41045:        }
                                             // 41045:        } // switch Inst[16]
                                             // 41045:       }
                                             // 41045:       } // switch Inst[14:13]
                                             // 41045:      }
  13, 62,                                    // 41050:      case 0xd: {
  OPC_SwitchField, 13, 2,                    // 41052:       switch Inst[14:13] {
  0, 21,                                     // 41055:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41057:        switch Inst[16] {
  0, 7,                                      // 41060:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41062:         check predicate 36
  OPC_Decode, 237, 25, 161, 1,               // 41064:         decode to FCVT_ZPzZ_StoH using decoder 161
                                             // 41064:        }
  1, 0,                                      // 41069:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41071:         check predicate 36
  OPC_Decode, 217, 30, 161, 1,               // 41073:         decode to FRECPX_ZPzZ_S using decoder 161
                                             // 41073:        }
                                             // 41073:        } // switch Inst[16]
                                             // 41073:       }
  1, 21,                                     // 41078:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41080:        switch Inst[16] {
  0, 7,                                      // 41083:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41085:         check predicate 36
  OPC_Decode, 235, 25, 161, 1,               // 41087:         decode to FCVT_ZPzZ_HtoS using decoder 161
                                             // 41087:        }
  1, 0,                                      // 41092:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41094:         check predicate 36
  OPC_Decode, 154, 32, 161, 1,               // 41096:         decode to FSQRT_ZPZz_S using decoder 161
                                             // 41096:        }
                                             // 41096:        } // switch Inst[16]
                                             // 41096:       }
  2, 0,                                      // 41101:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 41103:        check predicate 36
  OPC_CheckField, 16, 1, 0,                  // 41105:        check Inst[16] == 0x0
  OPC_Decode, 138, 15, 161, 1,               // 41109:        decode to BFCVT_ZPzZ_StoH using decoder 161
                                             // 41109:       }
                                             // 41109:       } // switch Inst[14:13]
                                             // 41109:      }
  14, 29,                                    // 41114:      case 0xe: {
  OPC_SwitchField, 13, 2,                    // 41116:       switch Inst[14:13] {
  0, 11,                                     // 41119:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 41121:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41123:        check Inst[16] == 0x1
  OPC_Decode, 155, 47, 161, 1,               // 41127:        decode to SCVTF_ZPzZ_StoS using decoder 161
                                             // 41127:       }
  1, 0,                                      // 41132:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 41134:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41136:        check Inst[16] == 0x1
  OPC_Decode, 148, 63, 161, 1,               // 41140:        decode to UCVTF_ZPzZ_StoS using decoder 161
                                             // 41140:       }
                                             // 41140:       } // switch Inst[14:13]
                                             // 41140:      }
  15, 0,                                     // 41145:      case 0xf: {
  OPC_SwitchField, 13, 2,                    // 41147:       switch Inst[14:13] {
  0, 11,                                     // 41150:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 41152:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41154:        check Inst[16] == 0x1
  OPC_Decode, 154, 25, 161, 1,               // 41158:        decode to FCVTZS_ZPzZ_StoS using decoder 161
                                             // 41158:       }
  1, 0,                                      // 41163:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 41165:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41167:        check Inst[16] == 0x1
  OPC_Decode, 205, 25, 161, 1,               // 41171:        decode to FCVTZU_ZPzZ_StoS using decoder 161
                                             // 41171:       }
                                             // 41171:       } // switch Inst[14:13]
                                             // 41171:      }
                                             // 41171:      } // switch Inst[20:17]
                                             // 41171:     }
                                             // 41171:     } // switch Inst[15]
                                             // 41171:    }
  1, 213, 1,                                 // 41176:    case 0x1: {
  OPC_SwitchField, 12, 4,                    // 41179:     switch Inst[15:12] {
  0, 21,                                     // 41182:     case 0x0: {
  OPC_SwitchField, 10, 2,                    // 41184:      switch Inst[11:10] {
  0, 7,                                      // 41187:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 41189:       check predicate 33
  OPC_Decode, 194, 28, 179, 2,               // 41191:       decode to FMLA_ZZZI_S using decoder 307
                                             // 41191:      }
  1, 0,                                      // 41196:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 41198:       check predicate 33
  OPC_Decode, 252, 28, 179, 2,               // 41200:       decode to FMLS_ZZZI_S using decoder 307
                                             // 41200:      }
                                             // 41200:      } // switch Inst[11:10]
                                             // 41200:     }
  1, 7,                                      // 41205:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 41207:      check predicate 33
  OPC_Decode, 134, 23, 184, 2,               // 41209:      decode to FCMLA_ZZZI_H using decoder 312
                                             // 41209:     }
  2, 21,                                     // 41214:     case 0x2: {
  OPC_SwitchField, 10, 2,                    // 41216:      switch Inst[11:10] {
  0, 7,                                      // 41219:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 41221:       check predicate 33
  OPC_Decode, 138, 30, 196, 2,               // 41223:       decode to FMUL_ZZZI_S using decoder 324
                                             // 41223:      }
  1, 0,                                      // 41228:      case 0x1: {
  OPC_CheckPredicate, 18,                    // 41230:       check predicate 18
  OPC_Decode, 192, 22, 198, 1,               // 41232:       decode to FCLAMP_ZZZ_S using decoder 198
                                             // 41232:      }
                                             // 41232:      } // switch Inst[11:10]
                                             // 41232:     }
  4, 21,                                     // 41237:     case 0x4: {
  OPC_SwitchField, 10, 1,                    // 41239:      switch Inst[10] {
  0, 7,                                      // 41242:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 41244:       check predicate 35
  OPC_Decode, 245, 27, 182, 2,               // 41246:       decode to FMLALB_ZZZI_SHH using decoder 310
                                             // 41246:      }
  1, 0,                                      // 41251:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 41253:       check predicate 35
  OPC_Decode, 147, 28, 182, 2,               // 41255:       decode to FMLALT_ZZZI_SHH using decoder 310
                                             // 41255:      }
                                             // 41255:      } // switch Inst[10]
                                             // 41255:     }
  5, 7,                                      // 41260:     case 0x5: {
  OPC_CheckPredicate, 59,                    // 41262:      check predicate 59
  OPC_Decode, 146, 28, 206, 2,               // 41264:      decode to FMLALT_ZZZI using decoder 334
                                             // 41264:     }
  6, 21,                                     // 41269:     case 0x6: {
  OPC_SwitchField, 10, 1,                    // 41271:      switch Inst[10] {
  0, 7,                                      // 41274:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 41276:       check predicate 35
  OPC_Decode, 213, 28, 182, 2,               // 41278:       decode to FMLSLB_ZZZI_SHH using decoder 310
                                             // 41278:      }
  1, 0,                                      // 41283:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 41285:       check predicate 35
  OPC_Decode, 215, 28, 182, 2,               // 41287:       decode to FMLSLT_ZZZI_SHH using decoder 310
                                             // 41287:      }
                                             // 41287:      } // switch Inst[10]
                                             // 41287:     }
  8, 30,                                     // 41292:     case 0x8: {
  OPC_SwitchField, 10, 2,                    // 41294:      switch Inst[11:10] {
  0, 7,                                      // 41297:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 41299:       check predicate 35
  OPC_Decode, 246, 27, 198, 1,               // 41301:       decode to FMLALB_ZZZ_SHH using decoder 198
                                             // 41301:      }
  1, 7,                                      // 41306:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 41308:       check predicate 35
  OPC_Decode, 148, 28, 198, 1,               // 41310:       decode to FMLALT_ZZZ_SHH using decoder 198
                                             // 41310:      }
  2, 0,                                      // 41315:      case 0x2: {
  OPC_CheckPredicate, 59,                    // 41317:       check predicate 59
  OPC_Decode, 243, 27, 198, 1,               // 41319:       decode to FMLALB_ZZZ using decoder 198
                                             // 41319:      }
                                             // 41319:      } // switch Inst[11:10]
                                             // 41319:     }
  9, 11,                                     // 41324:     case 0x9: {
  OPC_CheckPredicate, 59,                    // 41326:      check predicate 59
  OPC_CheckField, 10, 2, 2,                  // 41328:      check Inst[11:10] == 0x2
  OPC_Decode, 145, 28, 198, 1,               // 41332:      decode to FMLALT_ZZZ using decoder 198
                                             // 41332:     }
  10, 21,                                    // 41337:     case 0xa: {
  OPC_SwitchField, 10, 2,                    // 41339:      switch Inst[11:10] {
  0, 7,                                      // 41342:      case 0x0: {
  OPC_CheckPredicate, 35,                    // 41344:       check predicate 35
  OPC_Decode, 214, 28, 198, 1,               // 41346:       decode to FMLSLB_ZZZ_SHH using decoder 198
                                             // 41346:      }
  1, 0,                                      // 41351:      case 0x1: {
  OPC_CheckPredicate, 35,                    // 41353:       check predicate 35
  OPC_Decode, 216, 28, 198, 1,               // 41355:       decode to FMLSLT_ZZZ_SHH using decoder 198
                                             // 41355:      }
                                             // 41355:      } // switch Inst[11:10]
                                             // 41355:     }
  12, 7,                                     // 41360:     case 0xc: {
  OPC_CheckPredicate, 59,                    // 41362:      check predicate 59
  OPC_Decode, 130, 28, 206, 2,               // 41364:      decode to FMLALLTB_ZZZI using decoder 334
                                             // 41364:     }
  14, 0,                                     // 41369:     case 0xe: {
  OPC_SwitchField, 10, 2,                    // 41371:      switch Inst[11:10] {
  0, 7,                                      // 41374:      case 0x0: {
  OPC_CheckPredicate, 64,                    // 41376:       check predicate 64
  OPC_Decode, 141, 29, 198, 1,               // 41378:       decode to FMMLA_ZZZ_H using decoder 198
                                             // 41378:      }
  1, 0,                                      // 41383:      case 0x1: {
  OPC_CheckPredicate, 65,                    // 41385:       check predicate 65
  OPC_Decode, 142, 29, 198, 1,               // 41387:       decode to FMMLA_ZZZ_S using decoder 198
                                             // 41387:      }
                                             // 41387:      } // switch Inst[11:10]
                                             // 41387:     }
                                             // 41387:     } // switch Inst[15:12]
                                             // 41387:    }
  2, 159, 4,                                 // 41392:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 41395:     switch Inst[15] {
  0, 7,                                      // 41398:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 41400:      check predicate 33
  OPC_Decode, 131, 23, 205, 2,               // 41402:      decode to FCMLA_ZPmZZ_D using decoder 333
                                             // 41402:     }
  1, 0,                                      // 41407:     case 0x1: {
  OPC_SwitchField, 17, 4,                    // 41409:      switch Inst[20:17] {
  0, 11,                                     // 41412:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 41414:       check predicate 33
  OPC_CheckField, 13, 2, 0,                  // 41416:       check Inst[14:13] == 0x0
  OPC_Decode, 170, 22, 207, 2,               // 41420:       decode to FCADD_ZPmZ_D using decoder 335
                                             // 41420:      }
  1, 29,                                     // 41425:      case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41427:       switch Inst[16] {
  0, 11,                                     // 41430:       case 0x0: {
  OPC_CheckPredicate, 36,                    // 41432:        check predicate 36
  OPC_CheckField, 13, 2, 1,                  // 41434:        check Inst[14:13] == 0x1
  OPC_Decode, 172, 24, 163, 1,               // 41438:        decode to FCVTNT_ZPzZ_DtoS using decoder 163
                                             // 41438:       }
  1, 0,                                      // 41443:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 41445:        check predicate 36
  OPC_CheckField, 13, 2, 1,                  // 41447:        check Inst[14:13] == 0x1
  OPC_Decode, 236, 23, 161, 1,               // 41451:        decode to FCVTLT_ZPzZ_StoD using decoder 161
                                             // 41451:       }
                                             // 41451:       } // switch Inst[16]
                                             // 41451:      }
  5, 29,                                     // 41456:      case 0x5: {
  OPC_SwitchField, 16, 1,                    // 41458:       switch Inst[16] {
  0, 11,                                     // 41461:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 41463:        check predicate 35
  OPC_CheckField, 13, 2, 1,                  // 41465:        check Inst[14:13] == 0x1
  OPC_Decode, 170, 24, 163, 1,               // 41469:        decode to FCVTNT_ZPmZ_DtoS using decoder 163
                                             // 41469:       }
  1, 0,                                      // 41474:       case 0x1: {
  OPC_CheckPredicate, 35,                    // 41476:        check predicate 35
  OPC_CheckField, 13, 2, 1,                  // 41478:        check Inst[14:13] == 0x1
  OPC_Decode, 234, 23, 163, 1,               // 41482:        decode to FCVTLT_ZPmZ_StoD using decoder 163
                                             // 41482:       }
                                             // 41482:       } // switch Inst[16]
                                             // 41482:      }
  8, 29,                                     // 41487:      case 0x8: {
  OPC_SwitchField, 13, 2,                    // 41489:       switch Inst[14:13] {
  0, 11,                                     // 41492:       case 0x0: {
  OPC_CheckPredicate, 35,                    // 41494:        check predicate 35
  OPC_CheckField, 16, 1, 0,                  // 41496:        check Inst[16] == 0x0
  OPC_Decode, 232, 21, 160, 1,               // 41500:        decode to FADDP_ZPmZZ_D using decoder 160
                                             // 41500:       }
  1, 0,                                      // 41505:       case 0x1: {
  OPC_CheckPredicate, 34,                    // 41507:        check predicate 34
  OPC_CheckField, 16, 1, 0,                  // 41509:        check Inst[16] == 0x0
  OPC_Decode, 243, 21, 162, 1,               // 41513:        decode to FADDQV_D using decoder 162
                                             // 41513:       }
                                             // 41513:       } // switch Inst[14:13]
                                             // 41513:      }
  10, 49,                                    // 41518:      case 0xa: {
  OPC_SwitchField, 13, 2,                    // 41520:       switch Inst[14:13] {
  0, 21,                                     // 41523:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41525:        switch Inst[16] {
  0, 7,                                      // 41528:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 41530:         check predicate 35
  OPC_Decode, 187, 26, 160, 1,               // 41532:         decode to FMAXNMP_ZPmZZ_D using decoder 160
                                             // 41532:        }
  1, 0,                                      // 41537:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 41539:         check predicate 35
  OPC_Decode, 151, 27, 160, 1,               // 41541:         decode to FMINNMP_ZPmZZ_D using decoder 160
                                             // 41541:        }
                                             // 41541:        } // switch Inst[16]
                                             // 41541:       }
  1, 0,                                      // 41546:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41548:        switch Inst[16] {
  0, 7,                                      // 41551:        case 0x0: {
  OPC_CheckPredicate, 34,                    // 41553:         check predicate 34
  OPC_Decode, 198, 26, 162, 1,               // 41555:         decode to FMAXNMQV_D using decoder 162
                                             // 41555:        }
  1, 0,                                      // 41560:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 41562:         check predicate 34
  OPC_Decode, 162, 27, 162, 1,               // 41564:         decode to FMINNMQV_D using decoder 162
                                             // 41564:        }
                                             // 41564:        } // switch Inst[16]
                                             // 41564:       }
                                             // 41564:       } // switch Inst[14:13]
                                             // 41564:      }
  11, 49,                                    // 41569:      case 0xb: {
  OPC_SwitchField, 13, 2,                    // 41571:       switch Inst[14:13] {
  0, 21,                                     // 41574:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41576:        switch Inst[16] {
  0, 7,                                      // 41579:        case 0x0: {
  OPC_CheckPredicate, 35,                    // 41581:         check predicate 35
  OPC_Decode, 231, 26, 160, 1,               // 41583:         decode to FMAXP_ZPmZZ_D using decoder 160
                                             // 41583:        }
  1, 0,                                      // 41588:        case 0x1: {
  OPC_CheckPredicate, 35,                    // 41590:         check predicate 35
  OPC_Decode, 195, 27, 160, 1,               // 41592:         decode to FMINP_ZPmZZ_D using decoder 160
                                             // 41592:        }
                                             // 41592:        } // switch Inst[16]
                                             // 41592:       }
  1, 0,                                      // 41597:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41599:        switch Inst[16] {
  0, 7,                                      // 41602:        case 0x0: {
  OPC_CheckPredicate, 34,                    // 41604:         check predicate 34
  OPC_Decode, 242, 26, 162, 1,               // 41606:         decode to FMAXQV_D using decoder 162
                                             // 41606:        }
  1, 0,                                      // 41611:        case 0x1: {
  OPC_CheckPredicate, 34,                    // 41613:         check predicate 34
  OPC_Decode, 206, 27, 162, 1,               // 41615:         decode to FMINQV_D using decoder 162
                                             // 41615:        }
                                             // 41615:        } // switch Inst[16]
                                             // 41615:       }
                                             // 41615:       } // switch Inst[14:13]
                                             // 41615:      }
  12, 85,                                    // 41620:      case 0xc: {
  OPC_SwitchField, 13, 2,                    // 41622:       switch Inst[14:13] {
  0, 21,                                     // 41625:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41627:        switch Inst[16] {
  0, 7,                                      // 41630:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41632:         check predicate 36
  OPC_Decode, 183, 31, 161, 1,               // 41634:         decode to FRINTN_ZPzZ_D using decoder 161
                                             // 41634:        }
  1, 0,                                      // 41639:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41641:         check predicate 36
  OPC_Decode, 137, 31, 161, 1,               // 41643:         decode to FRINTA_ZPzZ_D using decoder 161
                                             // 41643:        }
                                             // 41643:        } // switch Inst[16]
                                             // 41643:       }
  1, 11,                                     // 41648:       case 0x1: {
  OPC_CheckPredicate, 36,                    // 41650:        check predicate 36
  OPC_CheckField, 16, 1, 0,                  // 41652:        check Inst[16] == 0x0
  OPC_Decode, 199, 31, 161, 1,               // 41656:        decode to FRINTP_ZPzZ_D using decoder 161
                                             // 41656:       }
  2, 21,                                     // 41661:       case 0x2: {
  OPC_SwitchField, 16, 1,                    // 41663:        switch Inst[16] {
  0, 7,                                      // 41666:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41668:         check predicate 36
  OPC_Decode, 167, 31, 161, 1,               // 41670:         decode to FRINTM_ZPzZ_D using decoder 161
                                             // 41670:        }
  1, 0,                                      // 41675:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41677:         check predicate 36
  OPC_Decode, 213, 31, 161, 1,               // 41679:         decode to FRINTX_ZPzZ_D using decoder 161
                                             // 41679:        }
                                             // 41679:        } // switch Inst[16]
                                             // 41679:       }
  3, 0,                                      // 41684:       case 0x3: {
  OPC_SwitchField, 16, 1,                    // 41686:        switch Inst[16] {
  0, 7,                                      // 41689:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41691:         check predicate 36
  OPC_Decode, 227, 31, 161, 1,               // 41693:         decode to FRINTZ_ZPzZ_D using decoder 161
                                             // 41693:        }
  1, 0,                                      // 41698:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41700:         check predicate 36
  OPC_Decode, 151, 31, 161, 1,               // 41702:         decode to FRINTI_ZPzZ_D using decoder 161
                                             // 41702:        }
                                             // 41702:        } // switch Inst[16]
                                             // 41702:       }
                                             // 41702:       } // switch Inst[14:13]
                                             // 41702:      }
  13, 75,                                    // 41707:      case 0xd: {
  OPC_SwitchField, 13, 2,                    // 41709:       switch Inst[14:13] {
  0, 21,                                     // 41712:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41714:        switch Inst[16] {
  0, 7,                                      // 41717:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41719:         check predicate 36
  OPC_Decode, 232, 25, 161, 1,               // 41721:         decode to FCVT_ZPzZ_DtoH using decoder 161
                                             // 41721:        }
  1, 0,                                      // 41726:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41728:         check predicate 36
  OPC_Decode, 215, 30, 161, 1,               // 41730:         decode to FRECPX_ZPzZ_D using decoder 161
                                             // 41730:        }
                                             // 41730:        } // switch Inst[16]
                                             // 41730:       }
  1, 21,                                     // 41735:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41737:        switch Inst[16] {
  0, 7,                                      // 41740:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41742:         check predicate 36
  OPC_Decode, 234, 25, 161, 1,               // 41744:         decode to FCVT_ZPzZ_HtoD using decoder 161
                                             // 41744:        }
  1, 0,                                      // 41749:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41751:         check predicate 36
  OPC_Decode, 152, 32, 161, 1,               // 41753:         decode to FSQRT_ZPZz_D using decoder 161
                                             // 41753:        }
                                             // 41753:        } // switch Inst[16]
                                             // 41753:       }
  2, 11,                                     // 41758:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 41760:        check predicate 36
  OPC_CheckField, 16, 1, 0,                  // 41762:        check Inst[16] == 0x0
  OPC_Decode, 233, 25, 161, 1,               // 41766:        decode to FCVT_ZPzZ_DtoS using decoder 161
                                             // 41766:       }
  3, 0,                                      // 41771:       case 0x3: {
  OPC_CheckPredicate, 36,                    // 41773:        check predicate 36
  OPC_CheckField, 16, 1, 0,                  // 41775:        check Inst[16] == 0x0
  OPC_Decode, 236, 25, 161, 1,               // 41779:        decode to FCVT_ZPzZ_StoD using decoder 161
                                             // 41779:       }
                                             // 41779:       } // switch Inst[14:13]
                                             // 41779:      }
  14, 75,                                    // 41784:      case 0xe: {
  OPC_SwitchField, 13, 2,                    // 41786:       switch Inst[14:13] {
  0, 21,                                     // 41789:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41791:        switch Inst[16] {
  0, 7,                                      // 41794:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41796:         check predicate 36
  OPC_Decode, 153, 47, 161, 1,               // 41798:         decode to SCVTF_ZPzZ_StoD using decoder 161
                                             // 41798:        }
  1, 0,                                      // 41803:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41805:         check predicate 36
  OPC_Decode, 151, 47, 161, 1,               // 41807:         decode to SCVTF_ZPzZ_DtoS using decoder 161
                                             // 41807:        }
                                             // 41807:        } // switch Inst[16]
                                             // 41807:       }
  1, 21,                                     // 41812:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41814:        switch Inst[16] {
  0, 7,                                      // 41817:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41819:         check predicate 36
  OPC_Decode, 146, 63, 161, 1,               // 41821:         decode to UCVTF_ZPzZ_StoD using decoder 161
                                             // 41821:        }
  1, 0,                                      // 41826:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41828:         check predicate 36
  OPC_Decode, 144, 63, 161, 1,               // 41830:         decode to UCVTF_ZPzZ_DtoS using decoder 161
                                             // 41830:        }
                                             // 41830:        } // switch Inst[16]
                                             // 41830:       }
  2, 11,                                     // 41835:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 41837:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41839:        check Inst[16] == 0x1
  OPC_Decode, 149, 47, 161, 1,               // 41843:        decode to SCVTF_ZPzZ_DtoD using decoder 161
                                             // 41843:       }
  3, 0,                                      // 41848:       case 0x3: {
  OPC_CheckPredicate, 36,                    // 41850:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41852:        check Inst[16] == 0x1
  OPC_Decode, 142, 63, 161, 1,               // 41856:        decode to UCVTF_ZPzZ_DtoD using decoder 161
                                             // 41856:       }
                                             // 41856:       } // switch Inst[14:13]
                                             // 41856:      }
  15, 0,                                     // 41861:      case 0xf: {
  OPC_SwitchField, 13, 2,                    // 41863:       switch Inst[14:13] {
  0, 21,                                     // 41866:       case 0x0: {
  OPC_SwitchField, 16, 1,                    // 41868:        switch Inst[16] {
  0, 7,                                      // 41871:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41873:         check predicate 36
  OPC_Decode, 149, 25, 161, 1,               // 41875:         decode to FCVTZS_ZPzZ_DtoS using decoder 161
                                             // 41875:        }
  1, 0,                                      // 41880:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41882:         check predicate 36
  OPC_Decode, 153, 25, 161, 1,               // 41884:         decode to FCVTZS_ZPzZ_StoD using decoder 161
                                             // 41884:        }
                                             // 41884:        } // switch Inst[16]
                                             // 41884:       }
  1, 21,                                     // 41889:       case 0x1: {
  OPC_SwitchField, 16, 1,                    // 41891:        switch Inst[16] {
  0, 7,                                      // 41894:        case 0x0: {
  OPC_CheckPredicate, 36,                    // 41896:         check predicate 36
  OPC_Decode, 200, 25, 161, 1,               // 41898:         decode to FCVTZU_ZPzZ_DtoS using decoder 161
                                             // 41898:        }
  1, 0,                                      // 41903:        case 0x1: {
  OPC_CheckPredicate, 36,                    // 41905:         check predicate 36
  OPC_Decode, 204, 25, 161, 1,               // 41907:         decode to FCVTZU_ZPzZ_StoD using decoder 161
                                             // 41907:        }
                                             // 41907:        } // switch Inst[16]
                                             // 41907:       }
  2, 11,                                     // 41912:       case 0x2: {
  OPC_CheckPredicate, 36,                    // 41914:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41916:        check Inst[16] == 0x1
  OPC_Decode, 148, 25, 161, 1,               // 41920:        decode to FCVTZS_ZPzZ_DtoD using decoder 161
                                             // 41920:       }
  3, 0,                                      // 41925:       case 0x3: {
  OPC_CheckPredicate, 36,                    // 41927:        check predicate 36
  OPC_CheckField, 16, 1, 1,                  // 41929:        check Inst[16] == 0x1
  OPC_Decode, 199, 25, 161, 1,               // 41933:        decode to FCVTZU_ZPzZ_DtoD using decoder 161
                                             // 41933:       }
                                             // 41933:       } // switch Inst[14:13]
                                             // 41933:      }
                                             // 41933:      } // switch Inst[20:17]
                                             // 41933:     }
                                             // 41933:     } // switch Inst[15]
                                             // 41933:    }
  3, 0,                                      // 41938:    case 0x3: {
  OPC_SwitchField, 12, 4,                    // 41940:     switch Inst[15:12] {
  0, 21,                                     // 41943:     case 0x0: {
  OPC_SwitchField, 10, 2,                    // 41945:      switch Inst[11:10] {
  0, 7,                                      // 41948:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 41950:       check predicate 33
  OPC_Decode, 192, 28, 180, 2,               // 41952:       decode to FMLA_ZZZI_D using decoder 308
                                             // 41952:      }
  1, 0,                                      // 41957:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 41959:       check predicate 33
  OPC_Decode, 250, 28, 180, 2,               // 41961:       decode to FMLS_ZZZI_D using decoder 308
                                             // 41961:      }
                                             // 41961:      } // switch Inst[11:10]
                                             // 41961:     }
  1, 7,                                      // 41966:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 41968:      check predicate 33
  OPC_Decode, 135, 23, 185, 2,               // 41970:      decode to FCMLA_ZZZI_S using decoder 313
                                             // 41970:     }
  2, 21,                                     // 41975:     case 0x2: {
  OPC_SwitchField, 10, 2,                    // 41977:      switch Inst[11:10] {
  0, 7,                                      // 41980:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 41982:       check predicate 33
  OPC_Decode, 136, 30, 197, 2,               // 41984:       decode to FMUL_ZZZI_D using decoder 325
                                             // 41984:      }
  1, 0,                                      // 41989:      case 0x1: {
  OPC_CheckPredicate, 18,                    // 41991:       check predicate 18
  OPC_Decode, 190, 22, 198, 1,               // 41993:       decode to FCLAMP_ZZZ_D using decoder 198
                                             // 41993:      }
                                             // 41993:      } // switch Inst[11:10]
                                             // 41993:     }
  4, 21,                                     // 41998:     case 0x4: {
  OPC_SwitchField, 10, 1,                    // 42000:      switch Inst[10] {
  0, 7,                                      // 42003:      case 0x0: {
  OPC_CheckPredicate, 56,                    // 42005:       check predicate 56
  OPC_Decode, 172, 15, 182, 2,               // 42007:       decode to BFMLALB_ZZZI using decoder 310
                                             // 42007:      }
  1, 0,                                      // 42012:      case 0x1: {
  OPC_CheckPredicate, 56,                    // 42014:       check predicate 56
  OPC_Decode, 176, 15, 182, 2,               // 42016:       decode to BFMLALT_ZZZI using decoder 310
                                             // 42016:      }
                                             // 42016:      } // switch Inst[10]
                                             // 42016:     }
  6, 21,                                     // 42021:     case 0x6: {
  OPC_SwitchField, 10, 1,                    // 42023:      switch Inst[10] {
  0, 7,                                      // 42026:      case 0x0: {
  OPC_CheckPredicate, 18,                    // 42028:       check predicate 18
  OPC_Decode, 193, 15, 182, 2,               // 42030:       decode to BFMLSLB_ZZZI_S using decoder 310
                                             // 42030:      }
  1, 0,                                      // 42035:      case 0x1: {
  OPC_CheckPredicate, 18,                    // 42037:       check predicate 18
  OPC_Decode, 195, 15, 182, 2,               // 42039:       decode to BFMLSLT_ZZZI_S using decoder 310
                                             // 42039:      }
                                             // 42039:      } // switch Inst[10]
                                             // 42039:     }
  8, 21,                                     // 42044:     case 0x8: {
  OPC_SwitchField, 10, 2,                    // 42046:      switch Inst[11:10] {
  0, 7,                                      // 42049:      case 0x0: {
  OPC_CheckPredicate, 56,                    // 42051:       check predicate 56
  OPC_Decode, 171, 15, 198, 1,               // 42053:       decode to BFMLALB_ZZZ using decoder 198
                                             // 42053:      }
  1, 0,                                      // 42058:      case 0x1: {
  OPC_CheckPredicate, 56,                    // 42060:       check predicate 56
  OPC_Decode, 175, 15, 198, 1,               // 42062:       decode to BFMLALT_ZZZ using decoder 198
                                             // 42062:      }
                                             // 42062:      } // switch Inst[11:10]
                                             // 42062:     }
  10, 21,                                    // 42067:     case 0xa: {
  OPC_SwitchField, 10, 2,                    // 42069:      switch Inst[11:10] {
  0, 7,                                      // 42072:      case 0x0: {
  OPC_CheckPredicate, 18,                    // 42074:       check predicate 18
  OPC_Decode, 194, 15, 198, 1,               // 42076:       decode to BFMLSLB_ZZZ_S using decoder 198
                                             // 42076:      }
  1, 0,                                      // 42081:      case 0x1: {
  OPC_CheckPredicate, 18,                    // 42083:       check predicate 18
  OPC_Decode, 196, 15, 198, 1,               // 42085:       decode to BFMLSLT_ZZZ_S using decoder 198
                                             // 42085:      }
                                             // 42085:      } // switch Inst[11:10]
                                             // 42085:     }
  12, 7,                                     // 42090:     case 0xc: {
  OPC_CheckPredicate, 59,                    // 42092:      check predicate 59
  OPC_Decode, 134, 28, 206, 2,               // 42094:      decode to FMLALLTT_ZZZI using decoder 334
                                             // 42094:     }
  14, 0,                                     // 42099:     case 0xe: {
  OPC_SwitchField, 10, 2,                    // 42101:      switch Inst[11:10] {
  0, 7,                                      // 42104:      case 0x0: {
  OPC_CheckPredicate, 66,                    // 42106:       check predicate 66
  OPC_Decode, 214, 15, 198, 1,               // 42108:       decode to BFMMLA_ZZZ_H using decoder 198
                                             // 42108:      }
  1, 0,                                      // 42113:      case 0x1: {
  OPC_CheckPredicate, 67,                    // 42115:       check predicate 67
  OPC_Decode, 140, 29, 198, 1,               // 42117:       decode to FMMLA_ZZZ_D using decoder 198
                                             // 42117:      }
                                             // 42117:      } // switch Inst[11:10]
                                             // 42117:     }
                                             // 42117:     } // switch Inst[15:12]
                                             // 42117:    }
                                             // 42117:    } // switch Inst[22:21]
                                             // 42117:   }
  2, 190, 10,                                // 42122:   case 0x2: {
  OPC_SwitchField, 13, 3,                    // 42125:    switch Inst[15:13] {
  0, 112,                                    // 42128:    case 0x0: {
  OPC_SwitchField, 21, 2,                    // 42130:     switch Inst[22:21] {
  0, 30,                                     // 42133:     case 0x0: {
  OPC_SwitchField, 10, 3,                    // 42135:      switch Inst[12:10] {
  0, 7,                                      // 42138:      case 0x0: {
  OPC_CheckPredicate, 55,                    // 42140:       check predicate 55
  OPC_Decode, 252, 14, 184, 1,               // 42142:       decode to BFADD_ZZZ using decoder 184
                                             // 42142:      }
  1, 7,                                      // 42147:      case 0x1: {
  OPC_CheckPredicate, 55,                    // 42149:       check predicate 55
  OPC_Decode, 253, 15, 184, 1,               // 42151:       decode to BFSUB_ZZZ using decoder 184
                                             // 42151:      }
  2, 0,                                      // 42156:      case 0x2: {
  OPC_CheckPredicate, 55,                    // 42158:       check predicate 55
  OPC_Decode, 241, 15, 184, 1,               // 42160:       decode to BFMUL_ZZZ using decoder 184
                                             // 42160:      }
                                             // 42160:      } // switch Inst[12:10]
                                             // 42160:     }
  1, 7,                                      // 42165:     case 0x1: {
  OPC_CheckPredicate, 55,                    // 42167:      check predicate 55
  OPC_Decode, 191, 15, 164, 1,               // 42169:      decode to BFMLA_ZPmZZ using decoder 164
                                             // 42169:     }
  2, 57,                                     // 42174:     case 0x2: {
  OPC_SwitchField, 10, 3,                    // 42176:      switch Inst[12:10] {
  0, 7,                                      // 42179:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 42181:       check predicate 33
  OPC_Decode, 135, 22, 184, 1,               // 42183:       decode to FADD_ZZZ_H using decoder 184
                                             // 42183:      }
  1, 7,                                      // 42188:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 42190:       check predicate 33
  OPC_Decode, 185, 32, 184, 1,               // 42192:       decode to FSUB_ZZZ_H using decoder 184
                                             // 42192:      }
  2, 7,                                      // 42197:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 42199:       check predicate 33
  OPC_Decode, 140, 30, 184, 1,               // 42201:       decode to FMUL_ZZZ_H using decoder 184
                                             // 42201:      }
  3, 7,                                      // 42206:      case 0x3: {
  OPC_CheckPredicate, 40,                    // 42208:       check predicate 40
  OPC_Decode, 201, 32, 184, 1,               // 42210:       decode to FTSMUL_ZZZ_H using decoder 184
                                             // 42210:      }
  6, 7,                                      // 42215:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 42217:       check predicate 33
  OPC_Decode, 205, 30, 184, 1,               // 42219:       decode to FRECPS_ZZZ_H using decoder 184
                                             // 42219:      }
  7, 0,                                      // 42224:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 42226:       check predicate 33
  OPC_Decode, 250, 31, 184, 1,               // 42228:       decode to FRSQRTS_ZZZ_H using decoder 184
                                             // 42228:      }
                                             // 42228:      } // switch Inst[12:10]
                                             // 42228:     }
  3, 0,                                      // 42233:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 42235:      check predicate 33
  OPC_Decode, 190, 28, 164, 1,               // 42237:      decode to FMLA_ZPmZZ_H using decoder 164
                                             // 42237:     }
                                             // 42237:     } // switch Inst[22:21]
                                             // 42237:    }
  1, 140, 3,                                 // 42242:    case 0x1: {
  OPC_SwitchField, 21, 2,                    // 42245:     switch Inst[22:21] {
  0, 144, 1,                                 // 42248:     case 0x0: {
  OPC_SwitchField, 10, 3,                    // 42251:      switch Inst[12:10] {
  4, 33,                                     // 42254:      case 0x4: {
  OPC_SwitchField, 16, 5,                    // 42256:       switch Inst[20:16] {
  8, 7,                                      // 42259:       case 0x8: {
  OPC_CheckPredicate, 68,                    // 42261:        check predicate 68
  OPC_Decode, 173, 21, 212, 1,               // 42263:        decode to F1CVT_ZZ_BtoH using decoder 212
                                             // 42263:       }
  9, 7,                                      // 42268:       case 0x9: {
  OPC_CheckPredicate, 68,                    // 42270:        check predicate 68
  OPC_Decode, 170, 21, 212, 1,               // 42272:        decode to F1CVTLT_ZZ_BtoH using decoder 212
                                             // 42272:       }
  10, 0,                                     // 42277:       case 0xa: {
  OPC_CheckPredicate, 68,                    // 42279:        check predicate 68
  OPC_CheckField, 5, 1, 0,                   // 42281:        check Inst[5] == 0x0
  OPC_Decode, 196, 24, 121,                  // 42285:        decode to FCVTN_Z2Z_HtoB using decoder 121
                                             // 42285:       }
                                             // 42285:       } // switch Inst[20:16]
                                             // 42285:      }
  5, 33,                                     // 42289:      case 0x5: {
  OPC_SwitchField, 16, 5,                    // 42291:       switch Inst[20:16] {
  8, 7,                                      // 42294:       case 0x8: {
  OPC_CheckPredicate, 68,                    // 42296:        check predicate 68
  OPC_Decode, 179, 21, 212, 1,               // 42298:        decode to F2CVT_ZZ_BtoH using decoder 212
                                             // 42298:       }
  9, 7,                                      // 42303:       case 0x9: {
  OPC_CheckPredicate, 68,                    // 42305:        check predicate 68
  OPC_Decode, 176, 21, 212, 1,               // 42307:        decode to F2CVTLT_ZZ_BtoH using decoder 212
                                             // 42307:       }
  10, 0,                                     // 42312:       case 0xa: {
  OPC_CheckPredicate, 68,                    // 42314:        check predicate 68
  OPC_CheckField, 5, 1, 0,                   // 42316:        check Inst[5] == 0x0
  OPC_Decode, 150, 24, 121,                  // 42320:        decode to FCVTNB_Z2Z_StoB using decoder 121
                                             // 42320:       }
                                             // 42320:       } // switch Inst[20:16]
                                             // 42320:      }
  6, 33,                                     // 42324:      case 0x6: {
  OPC_SwitchField, 16, 5,                    // 42326:       switch Inst[20:16] {
  8, 7,                                      // 42329:       case 0x8: {
  OPC_CheckPredicate, 68,                    // 42331:        check predicate 68
  OPC_Decode, 242, 14, 212, 1,               // 42333:        decode to BF1CVT_ZZ_BtoH using decoder 212
                                             // 42333:       }
  9, 7,                                      // 42338:       case 0x9: {
  OPC_CheckPredicate, 68,                    // 42340:        check predicate 68
  OPC_Decode, 239, 14, 212, 1,               // 42342:        decode to BF1CVTLT_ZZ_BtoH using decoder 212
                                             // 42342:       }
  10, 0,                                     // 42347:       case 0xa: {
  OPC_CheckPredicate, 68,                    // 42349:        check predicate 68
  OPC_CheckField, 5, 1, 0,                   // 42351:        check Inst[5] == 0x0
  OPC_Decode, 133, 15, 121,                  // 42355:        decode to BFCVTN_Z2Z_HtoB using decoder 121
                                             // 42355:       }
                                             // 42355:       } // switch Inst[20:16]
                                             // 42355:      }
  7, 0,                                      // 42359:      case 0x7: {
  OPC_SwitchField, 16, 5,                    // 42361:       switch Inst[20:16] {
  8, 7,                                      // 42364:       case 0x8: {
  OPC_CheckPredicate, 68,                    // 42366:        check predicate 68
  OPC_Decode, 248, 14, 212, 1,               // 42368:        decode to BF2CVT_ZZ_BtoH using decoder 212
                                             // 42368:       }
  9, 7,                                      // 42373:       case 0x9: {
  OPC_CheckPredicate, 68,                    // 42375:        check predicate 68
  OPC_Decode, 245, 14, 212, 1,               // 42377:        decode to BF2CVTLT_ZZ_BtoH using decoder 212
                                             // 42377:       }
  10, 0,                                     // 42382:       case 0xa: {
  OPC_CheckPredicate, 68,                    // 42384:        check predicate 68
  OPC_CheckField, 5, 1, 0,                   // 42386:        check Inst[5] == 0x0
  OPC_Decode, 169, 24, 208, 2,               // 42390:        decode to FCVTNT_Z2Z_StoB using decoder 336
                                             // 42390:       }
                                             // 42390:       } // switch Inst[20:16]
                                             // 42390:      }
                                             // 42390:      } // switch Inst[12:10]
                                             // 42390:     }
  1, 7,                                      // 42395:     case 0x1: {
  OPC_CheckPredicate, 55,                    // 42397:      check predicate 55
  OPC_Decode, 211, 15, 164, 1,               // 42399:      decode to BFMLS_ZPmZZ using decoder 164
                                             // 42399:     }
  2, 225, 1,                                 // 42404:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 42407:      switch Inst[20:16] {
  0, 7,                                      // 42410:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 42412:       check predicate 33
  OPC_Decode, 248, 21, 161, 1,               // 42414:       decode to FADDV_VPZ_H using decoder 161
                                             // 42414:      }
  4, 7,                                      // 42419:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 42421:       check predicate 33
  OPC_Decode, 203, 26, 161, 1,               // 42423:       decode to FMAXNMV_VPZ_H using decoder 161
                                             // 42423:      }
  5, 7,                                      // 42428:      case 0x5: {
  OPC_CheckPredicate, 33,                    // 42430:       check predicate 33
  OPC_Decode, 167, 27, 161, 1,               // 42432:       decode to FMINNMV_VPZ_H using decoder 161
                                             // 42432:      }
  6, 7,                                      // 42437:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 42439:       check predicate 33
  OPC_Decode, 247, 26, 161, 1,               // 42441:       decode to FMAXV_VPZ_H using decoder 161
                                             // 42441:      }
  7, 7,                                      // 42446:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 42448:       check predicate 33
  OPC_Decode, 211, 27, 161, 1,               // 42450:       decode to FMINV_VPZ_H using decoder 161
                                             // 42450:      }
  12, 39,                                    // 42455:      case 0xc: {
  OPC_SwitchField, 10, 3,                    // 42457:       switch Inst[12:10] {
  4, 7,                                      // 42460:       case 0x4: {
  OPC_CheckPredicate, 39,                    // 42462:        check predicate 39
  OPC_Decode, 156, 47, 212, 1,               // 42464:        decode to SCVTF_ZZ_BtoH using decoder 212
                                             // 42464:       }
  5, 7,                                      // 42469:       case 0x5: {
  OPC_CheckPredicate, 39,                    // 42471:        check predicate 39
  OPC_Decode, 149, 63, 212, 1,               // 42473:        decode to UCVTF_ZZ_BtoH using decoder 212
                                             // 42473:       }
  6, 7,                                      // 42478:       case 0x6: {
  OPC_CheckPredicate, 39,                    // 42480:        check predicate 39
  OPC_Decode, 252, 46, 212, 1,               // 42482:        decode to SCVTFLT_ZZ_BtoH using decoder 212
                                             // 42482:       }
  7, 0,                                      // 42487:       case 0x7: {
  OPC_CheckPredicate, 39,                    // 42489:        check predicate 39
  OPC_Decode, 245, 62, 212, 1,               // 42491:        decode to UCVTFLT_ZZ_BtoH using decoder 212
                                             // 42491:       }
                                             // 42491:       } // switch Inst[12:10]
                                             // 42491:      }
  13, 27,                                    // 42496:      case 0xd: {
  OPC_SwitchField, 10, 3,                    // 42498:       switch Inst[12:10] {
  4, 10,                                     // 42501:       case 0x4: {
  OPC_CheckPredicate, 39,                    // 42503:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 42505:        check Inst[5] == 0x0
  OPC_Decode, 251, 24, 121,                  // 42509:        decode to FCVTZSN_Z2Z_HtoB using decoder 121
                                             // 42509:       }
  5, 0,                                      // 42513:       case 0x5: {
  OPC_CheckPredicate, 39,                    // 42515:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 42517:        check Inst[5] == 0x0
  OPC_Decode, 174, 25, 121,                  // 42521:        decode to FCVTZUN_Z2Z_HtoB using decoder 121
                                             // 42521:       }
                                             // 42521:       } // switch Inst[12:10]
                                             // 42521:      }
  14, 11,                                    // 42525:      case 0xe: {
  OPC_CheckPredicate, 33,                    // 42527:       check predicate 33
  OPC_CheckField, 10, 3, 4,                  // 42529:       check Inst[12:10] == 0x4
  OPC_Decode, 191, 30, 212, 1,               // 42533:       decode to FRECPE_ZZ_H using decoder 212
                                             // 42533:      }
  15, 11,                                    // 42538:      case 0xf: {
  OPC_CheckPredicate, 33,                    // 42540:       check predicate 33
  OPC_CheckField, 10, 3, 4,                  // 42542:       check Inst[12:10] == 0x4
  OPC_Decode, 236, 31, 212, 1,               // 42546:       decode to FRSQRTE_ZZ_H using decoder 212
                                             // 42546:      }
  16, 21,                                    // 42551:      case 0x10: {
  OPC_SwitchField, 4, 1,                     // 42553:       switch Inst[4] {
  0, 7,                                      // 42556:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 42558:        check predicate 33
  OPC_Decode, 219, 22, 209, 2,               // 42560:        decode to FCMGE_PPzZ0_H using decoder 337
                                             // 42560:       }
  1, 0,                                      // 42565:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 42567:        check predicate 33
  OPC_Decode, 241, 22, 209, 2,               // 42569:        decode to FCMGT_PPzZ0_H using decoder 337
                                             // 42569:       }
                                             // 42569:       } // switch Inst[4]
                                             // 42569:      }
  17, 21,                                    // 42574:      case 0x11: {
  OPC_SwitchField, 4, 1,                     // 42576:       switch Inst[4] {
  0, 7,                                      // 42579:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 42581:        check predicate 33
  OPC_Decode, 156, 23, 209, 2,               // 42583:        decode to FCMLT_PPzZ0_H using decoder 337
                                             // 42583:       }
  1, 0,                                      // 42588:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 42590:        check predicate 33
  OPC_Decode, 145, 23, 209, 2,               // 42592:        decode to FCMLE_PPzZ0_H using decoder 337
                                             // 42592:       }
                                             // 42592:       } // switch Inst[4]
                                             // 42592:      }
  18, 11,                                    // 42597:      case 0x12: {
  OPC_CheckPredicate, 33,                    // 42599:       check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 42601:       check Inst[4] == 0x0
  OPC_Decode, 197, 22, 209, 2,               // 42605:       decode to FCMEQ_PPzZ0_H using decoder 337
                                             // 42605:      }
  19, 11,                                    // 42610:      case 0x13: {
  OPC_CheckPredicate, 33,                    // 42612:       check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 42614:       check Inst[4] == 0x0
  OPC_Decode, 167, 23, 209, 2,               // 42618:       decode to FCMNE_PPzZ0_H using decoder 337
                                             // 42618:      }
  24, 0,                                     // 42623:      case 0x18: {
  OPC_CheckPredicate, 40,                    // 42625:       check predicate 40
  OPC_Decode, 228, 21, 160, 1,               // 42627:       decode to FADDA_VPZ_H using decoder 160
                                             // 42627:      }
                                             // 42627:      } // switch Inst[20:16]
                                             // 42627:     }
  3, 0,                                      // 42632:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 42634:      check predicate 33
  OPC_Decode, 248, 28, 164, 1,               // 42636:      decode to FMLS_ZPmZZ_H using decoder 164
                                             // 42636:     }
                                             // 42636:     } // switch Inst[22:21]
                                             // 42636:    }
  2, 35,                                     // 42641:    case 0x2: {
  OPC_SwitchField, 21, 2,                    // 42643:     switch Inst[22:21] {
  2, 21,                                     // 42646:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 42648:      switch Inst[4] {
  0, 7,                                      // 42651:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 42653:       check predicate 33
  OPC_Decode, 222, 22, 133, 2,               // 42655:       decode to FCMGE_PPzZZ_H using decoder 261
                                             // 42655:      }
  1, 0,                                      // 42660:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 42662:       check predicate 33
  OPC_Decode, 244, 22, 133, 2,               // 42664:       decode to FCMGT_PPzZZ_H using decoder 261
                                             // 42664:      }
                                             // 42664:      } // switch Inst[4]
                                             // 42664:     }
  3, 0,                                      // 42669:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 42671:      check predicate 33
  OPC_Decode, 176, 30, 164, 1,               // 42673:      decode to FNMLA_ZPmZZ_H using decoder 164
                                             // 42673:     }
                                             // 42673:     } // switch Inst[22:21]
                                             // 42673:    }
  3, 35,                                     // 42678:    case 0x3: {
  OPC_SwitchField, 21, 2,                    // 42680:     switch Inst[22:21] {
  2, 21,                                     // 42683:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 42685:      switch Inst[4] {
  0, 7,                                      // 42688:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 42690:       check predicate 33
  OPC_Decode, 200, 22, 133, 2,               // 42692:       decode to FCMEQ_PPzZZ_H using decoder 261
                                             // 42692:      }
  1, 0,                                      // 42697:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 42699:       check predicate 33
  OPC_Decode, 170, 23, 133, 2,               // 42701:       decode to FCMNE_PPzZZ_H using decoder 261
                                             // 42701:      }
                                             // 42701:      } // switch Inst[4]
                                             // 42701:     }
  3, 0,                                      // 42706:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 42708:      check predicate 33
  OPC_Decode, 179, 30, 164, 1,               // 42710:      decode to FNMLS_ZPmZZ_H using decoder 164
                                             // 42710:     }
                                             // 42710:     } // switch Inst[22:21]
                                             // 42710:    }
  4, 234, 2,                                 // 42715:    case 0x4: {
  OPC_SwitchField, 21, 2,                    // 42718:     switch Inst[22:21] {
  0, 75,                                     // 42721:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 42723:      switch Inst[20:16] {
  0, 7,                                      // 42726:      case 0x0: {
  OPC_CheckPredicate, 55,                    // 42728:       check predicate 55
  OPC_Decode, 251, 14, 160, 1,               // 42730:       decode to BFADD_ZPmZZ using decoder 160
                                             // 42730:      }
  1, 7,                                      // 42735:      case 0x1: {
  OPC_CheckPredicate, 55,                    // 42737:       check predicate 55
  OPC_Decode, 252, 15, 160, 1,               // 42739:       decode to BFSUB_ZPmZZ using decoder 160
                                             // 42739:      }
  2, 7,                                      // 42744:      case 0x2: {
  OPC_CheckPredicate, 55,                    // 42746:       check predicate 55
  OPC_Decode, 240, 15, 160, 1,               // 42748:       decode to BFMUL_ZPmZZ using decoder 160
                                             // 42748:      }
  4, 7,                                      // 42753:      case 0x4: {
  OPC_CheckPredicate, 55,                    // 42755:       check predicate 55
  OPC_Decode, 153, 15, 160, 1,               // 42757:       decode to BFMAXNM_ZPmZZ using decoder 160
                                             // 42757:      }
  5, 7,                                      // 42762:      case 0x5: {
  OPC_CheckPredicate, 55,                    // 42764:       check predicate 55
  OPC_Decode, 163, 15, 160, 1,               // 42766:       decode to BFMINNM_ZPmZZ using decoder 160
                                             // 42766:      }
  6, 7,                                      // 42771:      case 0x6: {
  OPC_CheckPredicate, 55,                    // 42773:       check predicate 55
  OPC_Decode, 158, 15, 160, 1,               // 42775:       decode to BFMAX_ZPmZZ using decoder 160
                                             // 42775:      }
  7, 7,                                      // 42780:      case 0x7: {
  OPC_CheckPredicate, 55,                    // 42782:       check predicate 55
  OPC_Decode, 168, 15, 160, 1,               // 42784:       decode to BFMIN_ZPmZZ using decoder 160
                                             // 42784:      }
  9, 0,                                      // 42789:      case 0x9: {
  OPC_CheckPredicate, 69,                    // 42791:       check predicate 69
  OPC_Decode, 249, 15, 160, 1,               // 42793:       decode to BFSCALE_ZPZZ_H using decoder 160
                                             // 42793:      }
                                             // 42793:      } // switch Inst[20:16]
                                             // 42793:     }
  2, 142, 2,                                 // 42798:     case 0x2: {
  OPC_SwitchField, 19, 2,                    // 42801:      switch Inst[20:19] {
  0, 75,                                     // 42804:      case 0x0: {
  OPC_SwitchField, 16, 3,                    // 42806:       switch Inst[18:16] {
  0, 7,                                      // 42809:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 42811:        check predicate 33
  OPC_Decode, 132, 22, 160, 1,               // 42813:        decode to FADD_ZPmZ_H using decoder 160
                                             // 42813:       }
  1, 7,                                      // 42818:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 42820:        check predicate 33
  OPC_Decode, 182, 32, 160, 1,               // 42822:        decode to FSUB_ZPmZ_H using decoder 160
                                             // 42822:       }
  2, 7,                                      // 42827:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 42829:        check predicate 33
  OPC_Decode, 134, 30, 160, 1,               // 42831:        decode to FMUL_ZPmZ_H using decoder 160
                                             // 42831:       }
  3, 7,                                      // 42836:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 42838:        check predicate 33
  OPC_Decode, 169, 32, 160, 1,               // 42840:        decode to FSUBR_ZPmZ_H using decoder 160
                                             // 42840:       }
  4, 7,                                      // 42845:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 42847:        check predicate 33
  OPC_Decode, 224, 26, 160, 1,               // 42849:        decode to FMAXNM_ZPmZ_H using decoder 160
                                             // 42849:       }
  5, 7,                                      // 42854:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 42856:        check predicate 33
  OPC_Decode, 188, 27, 160, 1,               // 42858:        decode to FMINNM_ZPmZ_H using decoder 160
                                             // 42858:       }
  6, 7,                                      // 42863:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 42865:        check predicate 33
  OPC_Decode, 140, 27, 160, 1,               // 42867:        decode to FMAX_ZPmZ_H using decoder 160
                                             // 42867:       }
  7, 0,                                      // 42872:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 42874:        check predicate 33
  OPC_Decode, 232, 27, 160, 1,               // 42876:        decode to FMIN_ZPmZ_H using decoder 160
                                             // 42876:       }
                                             // 42876:       } // switch Inst[18:16]
                                             // 42876:      }
  1, 66,                                     // 42881:      case 0x1: {
  OPC_SwitchField, 16, 3,                    // 42883:       switch Inst[18:16] {
  0, 7,                                      // 42886:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 42888:        check predicate 33
  OPC_Decode, 184, 21, 160, 1,               // 42890:        decode to FABD_ZPmZ_H using decoder 160
                                             // 42890:       }
  1, 7,                                      // 42895:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 42897:        check predicate 33
  OPC_Decode, 142, 32, 160, 1,               // 42899:        decode to FSCALE_ZPmZ_H using decoder 160
                                             // 42899:       }
  2, 7,                                      // 42904:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 42906:        check predicate 33
  OPC_Decode, 231, 29, 160, 1,               // 42908:        decode to FMULX_ZPmZ_H using decoder 160
                                             // 42908:       }
  4, 7,                                      // 42913:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 42915:        check predicate 33
  OPC_Decode, 241, 25, 160, 1,               // 42917:        decode to FDIVR_ZPmZ_H using decoder 160
                                             // 42917:       }
  5, 7,                                      // 42922:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 42924:        check predicate 33
  OPC_Decode, 245, 25, 160, 1,               // 42926:        decode to FDIV_ZPmZ_H using decoder 160
                                             // 42926:       }
  6, 7,                                      // 42931:       case 0x6: {
  OPC_CheckPredicate, 70,                    // 42933:        check predicate 70
  OPC_Decode, 149, 22, 160, 1,               // 42935:        decode to FAMAX_ZPmZ_H using decoder 160
                                             // 42935:       }
  7, 0,                                      // 42940:       case 0x7: {
  OPC_CheckPredicate, 70,                    // 42942:        check predicate 70
  OPC_Decode, 163, 22, 160, 1,               // 42944:        decode to FAMIN_ZPmZ_H using decoder 160
                                             // 42944:       }
                                             // 42944:       } // switch Inst[18:16]
                                             // 42944:      }
  2, 11,                                     // 42949:      case 0x2: {
  OPC_CheckPredicate, 40,                    // 42951:       check predicate 40
  OPC_CheckField, 10, 3, 0,                  // 42953:       check Inst[12:10] == 0x0
  OPC_Decode, 193, 32, 210, 2,               // 42957:       decode to FTMAD_ZZI_H using decoder 338
                                             // 42957:      }
  3, 0,                                      // 42962:      case 0x3: {
  OPC_SwitchField, 16, 3,                    // 42964:       switch Inst[18:16] {
  0, 11,                                     // 42967:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 42969:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 42971:        check Inst[9:6] == 0x0
  OPC_Decode, 129, 22, 211, 2,               // 42975:        decode to FADD_ZPmI_H using decoder 339
                                             // 42975:       }
  1, 11,                                     // 42980:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 42982:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 42984:        check Inst[9:6] == 0x0
  OPC_Decode, 179, 32, 211, 2,               // 42988:        decode to FSUB_ZPmI_H using decoder 339
                                             // 42988:       }
  2, 11,                                     // 42993:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 42995:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 42997:        check Inst[9:6] == 0x0
  OPC_Decode, 131, 30, 211, 2,               // 43001:        decode to FMUL_ZPmI_H using decoder 339
                                             // 43001:       }
  3, 11,                                     // 43006:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 43008:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 43010:        check Inst[9:6] == 0x0
  OPC_Decode, 166, 32, 211, 2,               // 43014:        decode to FSUBR_ZPmI_H using decoder 339
                                             // 43014:       }
  4, 11,                                     // 43019:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 43021:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 43023:        check Inst[9:6] == 0x0
  OPC_Decode, 221, 26, 211, 2,               // 43027:        decode to FMAXNM_ZPmI_H using decoder 339
                                             // 43027:       }
  5, 11,                                     // 43032:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 43034:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 43036:        check Inst[9:6] == 0x0
  OPC_Decode, 185, 27, 211, 2,               // 43040:        decode to FMINNM_ZPmI_H using decoder 339
                                             // 43040:       }
  6, 11,                                     // 43045:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 43047:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 43049:        check Inst[9:6] == 0x0
  OPC_Decode, 137, 27, 211, 2,               // 43053:        decode to FMAX_ZPmI_H using decoder 339
                                             // 43053:       }
  7, 0,                                      // 43058:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 43060:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 43062:        check Inst[9:6] == 0x0
  OPC_Decode, 229, 27, 211, 2,               // 43066:        decode to FMIN_ZPmI_H using decoder 339
                                             // 43066:       }
                                             // 43066:       } // switch Inst[18:16]
                                             // 43066:      }
                                             // 43066:      } // switch Inst[20:19]
                                             // 43066:     }
  3, 0,                                      // 43071:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 43073:      check predicate 33
  OPC_Decode, 181, 26, 164, 1,               // 43075:      decode to FMAD_ZPmZZ_H using decoder 164
                                             // 43075:     }
                                             // 43075:     } // switch Inst[22:21]
                                             // 43075:    }
  5, 192, 2,                                 // 43080:    case 0x5: {
  OPC_SwitchField, 21, 2,                    // 43083:     switch Inst[22:21] {
  0, 111,                                    // 43086:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 43088:      switch Inst[20:16] {
  10, 7,                                     // 43091:      case 0xa: {
  OPC_CheckPredicate, 35,                    // 43093:       check predicate 35
  OPC_Decode, 246, 24, 163, 1,               // 43095:       decode to FCVTX_ZPmZ_DtoS using decoder 163
                                             // 43095:      }
  16, 7,                                     // 43100:      case 0x10: {
  OPC_CheckPredicate, 36,                    // 43102:       check predicate 36
  OPC_Decode, 233, 30, 163, 1,               // 43104:       decode to FRINT32Z_ZPmZ_S using decoder 163
                                             // 43104:      }
  17, 7,                                     // 43109:      case 0x11: {
  OPC_CheckPredicate, 36,                    // 43111:       check predicate 36
  OPC_Decode, 224, 30, 163, 1,               // 43113:       decode to FRINT32X_ZPmZ_S using decoder 163
                                             // 43113:      }
  18, 7,                                     // 43118:      case 0x12: {
  OPC_CheckPredicate, 36,                    // 43120:       check predicate 36
  OPC_Decode, 232, 30, 163, 1,               // 43122:       decode to FRINT32Z_ZPmZ_D using decoder 163
                                             // 43122:      }
  19, 7,                                     // 43127:      case 0x13: {
  OPC_CheckPredicate, 36,                    // 43129:       check predicate 36
  OPC_Decode, 223, 30, 163, 1,               // 43131:       decode to FRINT32X_ZPmZ_D using decoder 163
                                             // 43131:      }
  20, 7,                                     // 43136:      case 0x14: {
  OPC_CheckPredicate, 36,                    // 43138:       check predicate 36
  OPC_Decode, 251, 30, 163, 1,               // 43140:       decode to FRINT64Z_ZPmZ_S using decoder 163
                                             // 43140:      }
  21, 7,                                     // 43145:      case 0x15: {
  OPC_CheckPredicate, 36,                    // 43147:       check predicate 36
  OPC_Decode, 242, 30, 163, 1,               // 43149:       decode to FRINT64X_ZPmZ_S using decoder 163
                                             // 43149:      }
  22, 7,                                     // 43154:      case 0x16: {
  OPC_CheckPredicate, 36,                    // 43156:       check predicate 36
  OPC_Decode, 250, 30, 163, 1,               // 43158:       decode to FRINT64Z_ZPmZ_D using decoder 163
                                             // 43158:      }
  23, 7,                                     // 43163:      case 0x17: {
  OPC_CheckPredicate, 36,                    // 43165:       check predicate 36
  OPC_Decode, 241, 30, 163, 1,               // 43167:       decode to FRINT64X_ZPmZ_D using decoder 163
                                             // 43167:      }
  26, 7,                                     // 43172:      case 0x1a: {
  OPC_CheckPredicate, 35,                    // 43174:       check predicate 35
  OPC_Decode, 172, 26, 163, 1,               // 43176:       decode to FLOGB_ZPmZ_H using decoder 163
                                             // 43176:      }
  28, 7,                                     // 43181:      case 0x1c: {
  OPC_CheckPredicate, 35,                    // 43183:       check predicate 35
  OPC_Decode, 173, 26, 163, 1,               // 43185:       decode to FLOGB_ZPmZ_S using decoder 163
                                             // 43185:      }
  30, 0,                                     // 43190:      case 0x1e: {
  OPC_CheckPredicate, 35,                    // 43192:       check predicate 35
  OPC_Decode, 171, 26, 163, 1,               // 43194:       decode to FLOGB_ZPmZ_D using decoder 163
                                             // 43194:      }
                                             // 43194:      } // switch Inst[20:16]
                                             // 43194:     }
  2, 192, 1,                                 // 43199:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 43202:      switch Inst[20:16] {
  0, 7,                                      // 43205:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 43207:       check predicate 33
  OPC_Decode, 181, 31, 163, 1,               // 43209:       decode to FRINTN_ZPmZ_H using decoder 163
                                             // 43209:      }
  1, 7,                                      // 43214:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 43216:       check predicate 33
  OPC_Decode, 197, 31, 163, 1,               // 43218:       decode to FRINTP_ZPmZ_H using decoder 163
                                             // 43218:      }
  2, 7,                                      // 43223:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 43225:       check predicate 33
  OPC_Decode, 165, 31, 163, 1,               // 43227:       decode to FRINTM_ZPmZ_H using decoder 163
                                             // 43227:      }
  3, 7,                                      // 43232:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 43234:       check predicate 33
  OPC_Decode, 225, 31, 163, 1,               // 43236:       decode to FRINTZ_ZPmZ_H using decoder 163
                                             // 43236:      }
  4, 7,                                      // 43241:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 43243:       check predicate 33
  OPC_Decode, 135, 31, 163, 1,               // 43245:       decode to FRINTA_ZPmZ_H using decoder 163
                                             // 43245:      }
  6, 7,                                      // 43250:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 43252:       check predicate 33
  OPC_Decode, 211, 31, 163, 1,               // 43254:       decode to FRINTX_ZPmZ_H using decoder 163
                                             // 43254:      }
  7, 7,                                      // 43259:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 43261:       check predicate 33
  OPC_Decode, 149, 31, 163, 1,               // 43263:       decode to FRINTI_ZPmZ_H using decoder 163
                                             // 43263:      }
  12, 7,                                     // 43268:      case 0xc: {
  OPC_CheckPredicate, 33,                    // 43270:       check predicate 33
  OPC_Decode, 213, 30, 163, 1,               // 43272:       decode to FRECPX_ZPmZ_H using decoder 163
                                             // 43272:      }
  13, 7,                                     // 43277:      case 0xd: {
  OPC_CheckPredicate, 33,                    // 43279:       check predicate 33
  OPC_Decode, 156, 32, 163, 1,               // 43281:       decode to FSQRT_ZPmZ_H using decoder 163
                                             // 43281:      }
  18, 7,                                     // 43286:      case 0x12: {
  OPC_CheckPredicate, 33,                    // 43288:       check predicate 33
  OPC_Decode, 145, 47, 163, 1,               // 43290:       decode to SCVTF_ZPmZ_HtoH using decoder 163
                                             // 43290:      }
  19, 7,                                     // 43295:      case 0x13: {
  OPC_CheckPredicate, 33,                    // 43297:       check predicate 33
  OPC_Decode, 138, 63, 163, 1,               // 43299:       decode to UCVTF_ZPmZ_HtoH using decoder 163
                                             // 43299:      }
  20, 7,                                     // 43304:      case 0x14: {
  OPC_CheckPredicate, 33,                    // 43306:       check predicate 33
  OPC_Decode, 147, 47, 163, 1,               // 43308:       decode to SCVTF_ZPmZ_StoH using decoder 163
                                             // 43308:      }
  21, 7,                                     // 43313:      case 0x15: {
  OPC_CheckPredicate, 33,                    // 43315:       check predicate 33
  OPC_Decode, 140, 63, 163, 1,               // 43317:       decode to UCVTF_ZPmZ_StoH using decoder 163
                                             // 43317:      }
  22, 7,                                     // 43322:      case 0x16: {
  OPC_CheckPredicate, 33,                    // 43324:       check predicate 33
  OPC_Decode, 143, 47, 163, 1,               // 43326:       decode to SCVTF_ZPmZ_DtoH using decoder 163
                                             // 43326:      }
  23, 7,                                     // 43331:      case 0x17: {
  OPC_CheckPredicate, 33,                    // 43333:       check predicate 33
  OPC_Decode, 136, 63, 163, 1,               // 43335:       decode to UCVTF_ZPmZ_DtoH using decoder 163
                                             // 43335:      }
  26, 7,                                     // 43340:      case 0x1a: {
  OPC_CheckPredicate, 33,                    // 43342:       check predicate 33
  OPC_Decode, 144, 25, 163, 1,               // 43344:       decode to FCVTZS_ZPmZ_HtoH using decoder 163
                                             // 43344:      }
  27, 7,                                     // 43349:      case 0x1b: {
  OPC_CheckPredicate, 33,                    // 43351:       check predicate 33
  OPC_Decode, 195, 25, 163, 1,               // 43353:       decode to FCVTZU_ZPmZ_HtoH using decoder 163
                                             // 43353:      }
  28, 7,                                     // 43358:      case 0x1c: {
  OPC_CheckPredicate, 33,                    // 43360:       check predicate 33
  OPC_Decode, 145, 25, 163, 1,               // 43362:       decode to FCVTZS_ZPmZ_HtoS using decoder 163
                                             // 43362:      }
  29, 7,                                     // 43367:      case 0x1d: {
  OPC_CheckPredicate, 33,                    // 43369:       check predicate 33
  OPC_Decode, 196, 25, 163, 1,               // 43371:       decode to FCVTZU_ZPmZ_HtoS using decoder 163
                                             // 43371:      }
  30, 7,                                     // 43376:      case 0x1e: {
  OPC_CheckPredicate, 33,                    // 43378:       check predicate 33
  OPC_Decode, 143, 25, 163, 1,               // 43380:       decode to FCVTZS_ZPmZ_HtoD using decoder 163
                                             // 43380:      }
  31, 0,                                     // 43385:      case 0x1f: {
  OPC_CheckPredicate, 33,                    // 43387:       check predicate 33
  OPC_Decode, 194, 25, 163, 1,               // 43389:       decode to FCVTZU_ZPmZ_HtoD using decoder 163
                                             // 43389:      }
                                             // 43389:      } // switch Inst[20:16]
                                             // 43389:     }
  3, 0,                                      // 43394:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 43396:      check predicate 33
  OPC_Decode, 219, 29, 164, 1,               // 43398:      decode to FMSB_ZPmZZ_H using decoder 164
                                             // 43398:     }
                                             // 43398:     } // switch Inst[22:21]
                                             // 43398:    }
  6, 35,                                     // 43403:    case 0x6: {
  OPC_SwitchField, 21, 2,                    // 43405:     switch Inst[22:21] {
  2, 21,                                     // 43408:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 43410:      switch Inst[4] {
  0, 7,                                      // 43413:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 43415:       check predicate 33
  OPC_Decode, 185, 23, 133, 2,               // 43417:       decode to FCMUO_PPzZZ_H using decoder 261
                                             // 43417:      }
  1, 0,                                      // 43422:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 43424:       check predicate 33
  OPC_Decode, 209, 21, 133, 2,               // 43426:       decode to FACGE_PPzZZ_H using decoder 261
                                             // 43426:      }
                                             // 43426:      } // switch Inst[4]
                                             // 43426:     }
  3, 0,                                      // 43431:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 43433:      check predicate 33
  OPC_Decode, 173, 30, 164, 1,               // 43435:      decode to FNMAD_ZPmZZ_H using decoder 164
                                             // 43435:     }
                                             // 43435:     } // switch Inst[22:21]
                                             // 43435:    }
  7, 0,                                      // 43440:    case 0x7: {
  OPC_SwitchField, 21, 2,                    // 43442:     switch Inst[22:21] {
  2, 11,                                     // 43445:     case 0x2: {
  OPC_CheckPredicate, 33,                    // 43447:      check predicate 33
  OPC_CheckField, 4, 1, 1,                   // 43449:      check Inst[4] == 0x1
  OPC_Decode, 220, 21, 133, 2,               // 43453:      decode to FACGT_PPzZZ_H using decoder 261
                                             // 43453:     }
  3, 0,                                      // 43458:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 43460:      check predicate 33
  OPC_Decode, 182, 30, 164, 1,               // 43462:      decode to FNMSB_ZPmZZ_H using decoder 164
                                             // 43462:     }
                                             // 43462:     } // switch Inst[22:21]
                                             // 43462:    }
                                             // 43462:    } // switch Inst[15:13]
                                             // 43462:   }
  3, 0,                                      // 43467:   case 0x3: {
  OPC_SwitchField, 13, 3,                    // 43469:    switch Inst[15:13] {
  0, 139, 1,                                 // 43472:    case 0x0: {
  OPC_SwitchField, 21, 2,                    // 43475:     switch Inst[22:21] {
  0, 57,                                     // 43478:     case 0x0: {
  OPC_SwitchField, 10, 3,                    // 43480:      switch Inst[12:10] {
  0, 7,                                      // 43483:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 43485:       check predicate 33
  OPC_Decode, 136, 22, 184, 1,               // 43487:       decode to FADD_ZZZ_S using decoder 184
                                             // 43487:      }
  1, 7,                                      // 43492:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 43494:       check predicate 33
  OPC_Decode, 186, 32, 184, 1,               // 43496:       decode to FSUB_ZZZ_S using decoder 184
                                             // 43496:      }
  2, 7,                                      // 43501:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 43503:       check predicate 33
  OPC_Decode, 141, 30, 184, 1,               // 43505:       decode to FMUL_ZZZ_S using decoder 184
                                             // 43505:      }
  3, 7,                                      // 43510:      case 0x3: {
  OPC_CheckPredicate, 40,                    // 43512:       check predicate 40
  OPC_Decode, 202, 32, 184, 1,               // 43514:       decode to FTSMUL_ZZZ_S using decoder 184
                                             // 43514:      }
  6, 7,                                      // 43519:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 43521:       check predicate 33
  OPC_Decode, 206, 30, 184, 1,               // 43523:       decode to FRECPS_ZZZ_S using decoder 184
                                             // 43523:      }
  7, 0,                                      // 43528:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 43530:       check predicate 33
  OPC_Decode, 251, 31, 184, 1,               // 43532:       decode to FRSQRTS_ZZZ_S using decoder 184
                                             // 43532:      }
                                             // 43532:      } // switch Inst[12:10]
                                             // 43532:     }
  1, 7,                                      // 43537:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 43539:      check predicate 33
  OPC_Decode, 191, 28, 164, 1,               // 43541:      decode to FMLA_ZPmZZ_S using decoder 164
                                             // 43541:     }
  2, 57,                                     // 43546:     case 0x2: {
  OPC_SwitchField, 10, 3,                    // 43548:      switch Inst[12:10] {
  0, 7,                                      // 43551:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 43553:       check predicate 33
  OPC_Decode, 134, 22, 184, 1,               // 43555:       decode to FADD_ZZZ_D using decoder 184
                                             // 43555:      }
  1, 7,                                      // 43560:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 43562:       check predicate 33
  OPC_Decode, 184, 32, 184, 1,               // 43564:       decode to FSUB_ZZZ_D using decoder 184
                                             // 43564:      }
  2, 7,                                      // 43569:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 43571:       check predicate 33
  OPC_Decode, 139, 30, 184, 1,               // 43573:       decode to FMUL_ZZZ_D using decoder 184
                                             // 43573:      }
  3, 7,                                      // 43578:      case 0x3: {
  OPC_CheckPredicate, 40,                    // 43580:       check predicate 40
  OPC_Decode, 200, 32, 184, 1,               // 43582:       decode to FTSMUL_ZZZ_D using decoder 184
                                             // 43582:      }
  6, 7,                                      // 43587:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 43589:       check predicate 33
  OPC_Decode, 204, 30, 184, 1,               // 43591:       decode to FRECPS_ZZZ_D using decoder 184
                                             // 43591:      }
  7, 0,                                      // 43596:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 43598:       check predicate 33
  OPC_Decode, 249, 31, 184, 1,               // 43600:       decode to FRSQRTS_ZZZ_D using decoder 184
                                             // 43600:      }
                                             // 43600:      } // switch Inst[12:10]
                                             // 43600:     }
  3, 0,                                      // 43605:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 43607:      check predicate 33
  OPC_Decode, 189, 28, 164, 1,               // 43609:      decode to FMLA_ZPmZZ_D using decoder 164
                                             // 43609:     }
                                             // 43609:     } // switch Inst[22:21]
                                             // 43609:    }
  1, 221, 3,                                 // 43614:    case 0x1: {
  OPC_SwitchField, 21, 2,                    // 43617:     switch Inst[22:21] {
  0, 225, 1,                                 // 43620:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 43623:      switch Inst[20:16] {
  0, 7,                                      // 43626:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 43628:       check predicate 33
  OPC_Decode, 249, 21, 161, 1,               // 43630:       decode to FADDV_VPZ_S using decoder 161
                                             // 43630:      }
  4, 7,                                      // 43635:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 43637:       check predicate 33
  OPC_Decode, 204, 26, 161, 1,               // 43639:       decode to FMAXNMV_VPZ_S using decoder 161
                                             // 43639:      }
  5, 7,                                      // 43644:      case 0x5: {
  OPC_CheckPredicate, 33,                    // 43646:       check predicate 33
  OPC_Decode, 168, 27, 161, 1,               // 43648:       decode to FMINNMV_VPZ_S using decoder 161
                                             // 43648:      }
  6, 7,                                      // 43653:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 43655:       check predicate 33
  OPC_Decode, 248, 26, 161, 1,               // 43657:       decode to FMAXV_VPZ_S using decoder 161
                                             // 43657:      }
  7, 7,                                      // 43662:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 43664:       check predicate 33
  OPC_Decode, 212, 27, 161, 1,               // 43666:       decode to FMINV_VPZ_S using decoder 161
                                             // 43666:      }
  12, 39,                                    // 43671:      case 0xc: {
  OPC_SwitchField, 10, 3,                    // 43673:       switch Inst[12:10] {
  4, 7,                                      // 43676:       case 0x4: {
  OPC_CheckPredicate, 39,                    // 43678:        check predicate 39
  OPC_Decode, 157, 47, 212, 1,               // 43680:        decode to SCVTF_ZZ_HtoS using decoder 212
                                             // 43680:       }
  5, 7,                                      // 43685:       case 0x5: {
  OPC_CheckPredicate, 39,                    // 43687:        check predicate 39
  OPC_Decode, 150, 63, 212, 1,               // 43689:        decode to UCVTF_ZZ_HtoS using decoder 212
                                             // 43689:       }
  6, 7,                                      // 43694:       case 0x6: {
  OPC_CheckPredicate, 39,                    // 43696:        check predicate 39
  OPC_Decode, 253, 46, 212, 1,               // 43698:        decode to SCVTFLT_ZZ_HtoS using decoder 212
                                             // 43698:       }
  7, 0,                                      // 43703:       case 0x7: {
  OPC_CheckPredicate, 39,                    // 43705:        check predicate 39
  OPC_Decode, 246, 62, 212, 1,               // 43707:        decode to UCVTFLT_ZZ_HtoS using decoder 212
                                             // 43707:       }
                                             // 43707:       } // switch Inst[12:10]
                                             // 43707:      }
  13, 27,                                    // 43712:      case 0xd: {
  OPC_SwitchField, 10, 3,                    // 43714:       switch Inst[12:10] {
  4, 10,                                     // 43717:       case 0x4: {
  OPC_CheckPredicate, 39,                    // 43719:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 43721:        check Inst[5] == 0x0
  OPC_Decode, 252, 24, 121,                  // 43725:        decode to FCVTZSN_Z2Z_StoH using decoder 121
                                             // 43725:       }
  5, 0,                                      // 43729:       case 0x5: {
  OPC_CheckPredicate, 39,                    // 43731:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 43733:        check Inst[5] == 0x0
  OPC_Decode, 175, 25, 121,                  // 43737:        decode to FCVTZUN_Z2Z_StoH using decoder 121
                                             // 43737:       }
                                             // 43737:       } // switch Inst[12:10]
                                             // 43737:      }
  14, 11,                                    // 43741:      case 0xe: {
  OPC_CheckPredicate, 33,                    // 43743:       check predicate 33
  OPC_CheckField, 10, 3, 4,                  // 43745:       check Inst[12:10] == 0x4
  OPC_Decode, 192, 30, 212, 1,               // 43749:       decode to FRECPE_ZZ_S using decoder 212
                                             // 43749:      }
  15, 11,                                    // 43754:      case 0xf: {
  OPC_CheckPredicate, 33,                    // 43756:       check predicate 33
  OPC_CheckField, 10, 3, 4,                  // 43758:       check Inst[12:10] == 0x4
  OPC_Decode, 237, 31, 212, 1,               // 43762:       decode to FRSQRTE_ZZ_S using decoder 212
                                             // 43762:      }
  16, 21,                                    // 43767:      case 0x10: {
  OPC_SwitchField, 4, 1,                     // 43769:       switch Inst[4] {
  0, 7,                                      // 43772:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 43774:        check predicate 33
  OPC_Decode, 220, 22, 209, 2,               // 43776:        decode to FCMGE_PPzZ0_S using decoder 337
                                             // 43776:       }
  1, 0,                                      // 43781:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 43783:        check predicate 33
  OPC_Decode, 242, 22, 209, 2,               // 43785:        decode to FCMGT_PPzZ0_S using decoder 337
                                             // 43785:       }
                                             // 43785:       } // switch Inst[4]
                                             // 43785:      }
  17, 21,                                    // 43790:      case 0x11: {
  OPC_SwitchField, 4, 1,                     // 43792:       switch Inst[4] {
  0, 7,                                      // 43795:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 43797:        check predicate 33
  OPC_Decode, 157, 23, 209, 2,               // 43799:        decode to FCMLT_PPzZ0_S using decoder 337
                                             // 43799:       }
  1, 0,                                      // 43804:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 43806:        check predicate 33
  OPC_Decode, 146, 23, 209, 2,               // 43808:        decode to FCMLE_PPzZ0_S using decoder 337
                                             // 43808:       }
                                             // 43808:       } // switch Inst[4]
                                             // 43808:      }
  18, 11,                                    // 43813:      case 0x12: {
  OPC_CheckPredicate, 33,                    // 43815:       check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 43817:       check Inst[4] == 0x0
  OPC_Decode, 198, 22, 209, 2,               // 43821:       decode to FCMEQ_PPzZ0_S using decoder 337
                                             // 43821:      }
  19, 11,                                    // 43826:      case 0x13: {
  OPC_CheckPredicate, 33,                    // 43828:       check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 43830:       check Inst[4] == 0x0
  OPC_Decode, 168, 23, 209, 2,               // 43834:       decode to FCMNE_PPzZ0_S using decoder 337
                                             // 43834:      }
  24, 0,                                     // 43839:      case 0x18: {
  OPC_CheckPredicate, 40,                    // 43841:       check predicate 40
  OPC_Decode, 229, 21, 160, 1,               // 43843:       decode to FADDA_VPZ_S using decoder 160
                                             // 43843:      }
                                             // 43843:      } // switch Inst[20:16]
                                             // 43843:     }
  1, 7,                                      // 43848:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 43850:      check predicate 33
  OPC_Decode, 249, 28, 164, 1,               // 43852:      decode to FMLS_ZPmZZ_S using decoder 164
                                             // 43852:     }
  2, 225, 1,                                 // 43857:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 43860:      switch Inst[20:16] {
  0, 7,                                      // 43863:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 43865:       check predicate 33
  OPC_Decode, 247, 21, 161, 1,               // 43867:       decode to FADDV_VPZ_D using decoder 161
                                             // 43867:      }
  4, 7,                                      // 43872:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 43874:       check predicate 33
  OPC_Decode, 202, 26, 161, 1,               // 43876:       decode to FMAXNMV_VPZ_D using decoder 161
                                             // 43876:      }
  5, 7,                                      // 43881:      case 0x5: {
  OPC_CheckPredicate, 33,                    // 43883:       check predicate 33
  OPC_Decode, 166, 27, 161, 1,               // 43885:       decode to FMINNMV_VPZ_D using decoder 161
                                             // 43885:      }
  6, 7,                                      // 43890:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 43892:       check predicate 33
  OPC_Decode, 246, 26, 161, 1,               // 43894:       decode to FMAXV_VPZ_D using decoder 161
                                             // 43894:      }
  7, 7,                                      // 43899:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 43901:       check predicate 33
  OPC_Decode, 210, 27, 161, 1,               // 43903:       decode to FMINV_VPZ_D using decoder 161
                                             // 43903:      }
  12, 39,                                    // 43908:      case 0xc: {
  OPC_SwitchField, 10, 3,                    // 43910:       switch Inst[12:10] {
  4, 7,                                      // 43913:       case 0x4: {
  OPC_CheckPredicate, 39,                    // 43915:        check predicate 39
  OPC_Decode, 158, 47, 212, 1,               // 43917:        decode to SCVTF_ZZ_StoD using decoder 212
                                             // 43917:       }
  5, 7,                                      // 43922:       case 0x5: {
  OPC_CheckPredicate, 39,                    // 43924:        check predicate 39
  OPC_Decode, 151, 63, 212, 1,               // 43926:        decode to UCVTF_ZZ_StoD using decoder 212
                                             // 43926:       }
  6, 7,                                      // 43931:       case 0x6: {
  OPC_CheckPredicate, 39,                    // 43933:        check predicate 39
  OPC_Decode, 254, 46, 212, 1,               // 43935:        decode to SCVTFLT_ZZ_StoD using decoder 212
                                             // 43935:       }
  7, 0,                                      // 43940:       case 0x7: {
  OPC_CheckPredicate, 39,                    // 43942:        check predicate 39
  OPC_Decode, 247, 62, 212, 1,               // 43944:        decode to UCVTFLT_ZZ_StoD using decoder 212
                                             // 43944:       }
                                             // 43944:       } // switch Inst[12:10]
                                             // 43944:      }
  13, 27,                                    // 43949:      case 0xd: {
  OPC_SwitchField, 10, 3,                    // 43951:       switch Inst[12:10] {
  4, 10,                                     // 43954:       case 0x4: {
  OPC_CheckPredicate, 39,                    // 43956:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 43958:        check Inst[5] == 0x0
  OPC_Decode, 250, 24, 121,                  // 43962:        decode to FCVTZSN_Z2Z_DtoS using decoder 121
                                             // 43962:       }
  5, 0,                                      // 43966:       case 0x5: {
  OPC_CheckPredicate, 39,                    // 43968:        check predicate 39
  OPC_CheckField, 5, 1, 0,                   // 43970:        check Inst[5] == 0x0
  OPC_Decode, 173, 25, 121,                  // 43974:        decode to FCVTZUN_Z2Z_DtoS using decoder 121
                                             // 43974:       }
                                             // 43974:       } // switch Inst[12:10]
                                             // 43974:      }
  14, 11,                                    // 43978:      case 0xe: {
  OPC_CheckPredicate, 33,                    // 43980:       check predicate 33
  OPC_CheckField, 10, 3, 4,                  // 43982:       check Inst[12:10] == 0x4
  OPC_Decode, 190, 30, 212, 1,               // 43986:       decode to FRECPE_ZZ_D using decoder 212
                                             // 43986:      }
  15, 11,                                    // 43991:      case 0xf: {
  OPC_CheckPredicate, 33,                    // 43993:       check predicate 33
  OPC_CheckField, 10, 3, 4,                  // 43995:       check Inst[12:10] == 0x4
  OPC_Decode, 235, 31, 212, 1,               // 43999:       decode to FRSQRTE_ZZ_D using decoder 212
                                             // 43999:      }
  16, 21,                                    // 44004:      case 0x10: {
  OPC_SwitchField, 4, 1,                     // 44006:       switch Inst[4] {
  0, 7,                                      // 44009:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44011:        check predicate 33
  OPC_Decode, 218, 22, 209, 2,               // 44013:        decode to FCMGE_PPzZ0_D using decoder 337
                                             // 44013:       }
  1, 0,                                      // 44018:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44020:        check predicate 33
  OPC_Decode, 240, 22, 209, 2,               // 44022:        decode to FCMGT_PPzZ0_D using decoder 337
                                             // 44022:       }
                                             // 44022:       } // switch Inst[4]
                                             // 44022:      }
  17, 21,                                    // 44027:      case 0x11: {
  OPC_SwitchField, 4, 1,                     // 44029:       switch Inst[4] {
  0, 7,                                      // 44032:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44034:        check predicate 33
  OPC_Decode, 155, 23, 209, 2,               // 44036:        decode to FCMLT_PPzZ0_D using decoder 337
                                             // 44036:       }
  1, 0,                                      // 44041:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44043:        check predicate 33
  OPC_Decode, 144, 23, 209, 2,               // 44045:        decode to FCMLE_PPzZ0_D using decoder 337
                                             // 44045:       }
                                             // 44045:       } // switch Inst[4]
                                             // 44045:      }
  18, 11,                                    // 44050:      case 0x12: {
  OPC_CheckPredicate, 33,                    // 44052:       check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 44054:       check Inst[4] == 0x0
  OPC_Decode, 196, 22, 209, 2,               // 44058:       decode to FCMEQ_PPzZ0_D using decoder 337
                                             // 44058:      }
  19, 11,                                    // 44063:      case 0x13: {
  OPC_CheckPredicate, 33,                    // 44065:       check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 44067:       check Inst[4] == 0x0
  OPC_Decode, 166, 23, 209, 2,               // 44071:       decode to FCMNE_PPzZ0_D using decoder 337
                                             // 44071:      }
  24, 0,                                     // 44076:      case 0x18: {
  OPC_CheckPredicate, 40,                    // 44078:       check predicate 40
  OPC_Decode, 227, 21, 160, 1,               // 44080:       decode to FADDA_VPZ_D using decoder 160
                                             // 44080:      }
                                             // 44080:      } // switch Inst[20:16]
                                             // 44080:     }
  3, 0,                                      // 44085:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 44087:      check predicate 33
  OPC_Decode, 247, 28, 164, 1,               // 44089:      decode to FMLS_ZPmZZ_D using decoder 164
                                             // 44089:     }
                                             // 44089:     } // switch Inst[22:21]
                                             // 44089:    }
  2, 67,                                     // 44094:    case 0x2: {
  OPC_SwitchField, 21, 2,                    // 44096:     switch Inst[22:21] {
  0, 21,                                     // 44099:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 44101:      switch Inst[4] {
  0, 7,                                      // 44104:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 44106:       check predicate 33
  OPC_Decode, 223, 22, 133, 2,               // 44108:       decode to FCMGE_PPzZZ_S using decoder 261
                                             // 44108:      }
  1, 0,                                      // 44113:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 44115:       check predicate 33
  OPC_Decode, 245, 22, 133, 2,               // 44117:       decode to FCMGT_PPzZZ_S using decoder 261
                                             // 44117:      }
                                             // 44117:      } // switch Inst[4]
                                             // 44117:     }
  1, 7,                                      // 44122:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 44124:      check predicate 33
  OPC_Decode, 177, 30, 164, 1,               // 44126:      decode to FNMLA_ZPmZZ_S using decoder 164
                                             // 44126:     }
  2, 21,                                     // 44131:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 44133:      switch Inst[4] {
  0, 7,                                      // 44136:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 44138:       check predicate 33
  OPC_Decode, 221, 22, 133, 2,               // 44140:       decode to FCMGE_PPzZZ_D using decoder 261
                                             // 44140:      }
  1, 0,                                      // 44145:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 44147:       check predicate 33
  OPC_Decode, 243, 22, 133, 2,               // 44149:       decode to FCMGT_PPzZZ_D using decoder 261
                                             // 44149:      }
                                             // 44149:      } // switch Inst[4]
                                             // 44149:     }
  3, 0,                                      // 44154:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 44156:      check predicate 33
  OPC_Decode, 175, 30, 164, 1,               // 44158:      decode to FNMLA_ZPmZZ_D using decoder 164
                                             // 44158:     }
                                             // 44158:     } // switch Inst[22:21]
                                             // 44158:    }
  3, 67,                                     // 44163:    case 0x3: {
  OPC_SwitchField, 21, 2,                    // 44165:     switch Inst[22:21] {
  0, 21,                                     // 44168:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 44170:      switch Inst[4] {
  0, 7,                                      // 44173:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 44175:       check predicate 33
  OPC_Decode, 201, 22, 133, 2,               // 44177:       decode to FCMEQ_PPzZZ_S using decoder 261
                                             // 44177:      }
  1, 0,                                      // 44182:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 44184:       check predicate 33
  OPC_Decode, 171, 23, 133, 2,               // 44186:       decode to FCMNE_PPzZZ_S using decoder 261
                                             // 44186:      }
                                             // 44186:      } // switch Inst[4]
                                             // 44186:     }
  1, 7,                                      // 44191:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 44193:      check predicate 33
  OPC_Decode, 180, 30, 164, 1,               // 44195:      decode to FNMLS_ZPmZZ_S using decoder 164
                                             // 44195:     }
  2, 21,                                     // 44200:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 44202:      switch Inst[4] {
  0, 7,                                      // 44205:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 44207:       check predicate 33
  OPC_Decode, 199, 22, 133, 2,               // 44209:       decode to FCMEQ_PPzZZ_D using decoder 261
                                             // 44209:      }
  1, 0,                                      // 44214:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 44216:       check predicate 33
  OPC_Decode, 169, 23, 133, 2,               // 44218:       decode to FCMNE_PPzZZ_D using decoder 261
                                             // 44218:      }
                                             // 44218:      } // switch Inst[4]
                                             // 44218:     }
  3, 0,                                      // 44223:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 44225:      check predicate 33
  OPC_Decode, 178, 30, 164, 1,               // 44227:      decode to FNMLS_ZPmZZ_D using decoder 164
                                             // 44227:     }
                                             // 44227:     } // switch Inst[22:21]
                                             // 44227:    }
  4, 183, 4,                                 // 44232:    case 0x4: {
  OPC_SwitchField, 21, 2,                    // 44235:     switch Inst[22:21] {
  0, 142, 2,                                 // 44238:     case 0x0: {
  OPC_SwitchField, 19, 2,                    // 44241:      switch Inst[20:19] {
  0, 75,                                     // 44244:      case 0x0: {
  OPC_SwitchField, 16, 3,                    // 44246:       switch Inst[18:16] {
  0, 7,                                      // 44249:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44251:        check predicate 33
  OPC_Decode, 133, 22, 160, 1,               // 44253:        decode to FADD_ZPmZ_S using decoder 160
                                             // 44253:       }
  1, 7,                                      // 44258:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44260:        check predicate 33
  OPC_Decode, 183, 32, 160, 1,               // 44262:        decode to FSUB_ZPmZ_S using decoder 160
                                             // 44262:       }
  2, 7,                                      // 44267:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 44269:        check predicate 33
  OPC_Decode, 135, 30, 160, 1,               // 44271:        decode to FMUL_ZPmZ_S using decoder 160
                                             // 44271:       }
  3, 7,                                      // 44276:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 44278:        check predicate 33
  OPC_Decode, 170, 32, 160, 1,               // 44280:        decode to FSUBR_ZPmZ_S using decoder 160
                                             // 44280:       }
  4, 7,                                      // 44285:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 44287:        check predicate 33
  OPC_Decode, 225, 26, 160, 1,               // 44289:        decode to FMAXNM_ZPmZ_S using decoder 160
                                             // 44289:       }
  5, 7,                                      // 44294:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 44296:        check predicate 33
  OPC_Decode, 189, 27, 160, 1,               // 44298:        decode to FMINNM_ZPmZ_S using decoder 160
                                             // 44298:       }
  6, 7,                                      // 44303:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 44305:        check predicate 33
  OPC_Decode, 141, 27, 160, 1,               // 44307:        decode to FMAX_ZPmZ_S using decoder 160
                                             // 44307:       }
  7, 0,                                      // 44312:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 44314:        check predicate 33
  OPC_Decode, 233, 27, 160, 1,               // 44316:        decode to FMIN_ZPmZ_S using decoder 160
                                             // 44316:       }
                                             // 44316:       } // switch Inst[18:16]
                                             // 44316:      }
  1, 66,                                     // 44321:      case 0x1: {
  OPC_SwitchField, 16, 3,                    // 44323:       switch Inst[18:16] {
  0, 7,                                      // 44326:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44328:        check predicate 33
  OPC_Decode, 185, 21, 160, 1,               // 44330:        decode to FABD_ZPmZ_S using decoder 160
                                             // 44330:       }
  1, 7,                                      // 44335:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44337:        check predicate 33
  OPC_Decode, 143, 32, 160, 1,               // 44339:        decode to FSCALE_ZPmZ_S using decoder 160
                                             // 44339:       }
  2, 7,                                      // 44344:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 44346:        check predicate 33
  OPC_Decode, 232, 29, 160, 1,               // 44348:        decode to FMULX_ZPmZ_S using decoder 160
                                             // 44348:       }
  4, 7,                                      // 44353:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 44355:        check predicate 33
  OPC_Decode, 242, 25, 160, 1,               // 44357:        decode to FDIVR_ZPmZ_S using decoder 160
                                             // 44357:       }
  5, 7,                                      // 44362:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 44364:        check predicate 33
  OPC_Decode, 246, 25, 160, 1,               // 44366:        decode to FDIV_ZPmZ_S using decoder 160
                                             // 44366:       }
  6, 7,                                      // 44371:       case 0x6: {
  OPC_CheckPredicate, 70,                    // 44373:        check predicate 70
  OPC_Decode, 150, 22, 160, 1,               // 44375:        decode to FAMAX_ZPmZ_S using decoder 160
                                             // 44375:       }
  7, 0,                                      // 44380:       case 0x7: {
  OPC_CheckPredicate, 70,                    // 44382:        check predicate 70
  OPC_Decode, 164, 22, 160, 1,               // 44384:        decode to FAMIN_ZPmZ_S using decoder 160
                                             // 44384:       }
                                             // 44384:       } // switch Inst[18:16]
                                             // 44384:      }
  2, 11,                                     // 44389:      case 0x2: {
  OPC_CheckPredicate, 40,                    // 44391:       check predicate 40
  OPC_CheckField, 10, 3, 0,                  // 44393:       check Inst[12:10] == 0x0
  OPC_Decode, 194, 32, 210, 2,               // 44397:       decode to FTMAD_ZZI_S using decoder 338
                                             // 44397:      }
  3, 0,                                      // 44402:      case 0x3: {
  OPC_SwitchField, 16, 3,                    // 44404:       switch Inst[18:16] {
  0, 11,                                     // 44407:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44409:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44411:        check Inst[9:6] == 0x0
  OPC_Decode, 130, 22, 211, 2,               // 44415:        decode to FADD_ZPmI_S using decoder 339
                                             // 44415:       }
  1, 11,                                     // 44420:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44422:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44424:        check Inst[9:6] == 0x0
  OPC_Decode, 180, 32, 211, 2,               // 44428:        decode to FSUB_ZPmI_S using decoder 339
                                             // 44428:       }
  2, 11,                                     // 44433:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 44435:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44437:        check Inst[9:6] == 0x0
  OPC_Decode, 132, 30, 211, 2,               // 44441:        decode to FMUL_ZPmI_S using decoder 339
                                             // 44441:       }
  3, 11,                                     // 44446:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 44448:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44450:        check Inst[9:6] == 0x0
  OPC_Decode, 167, 32, 211, 2,               // 44454:        decode to FSUBR_ZPmI_S using decoder 339
                                             // 44454:       }
  4, 11,                                     // 44459:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 44461:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44463:        check Inst[9:6] == 0x0
  OPC_Decode, 222, 26, 211, 2,               // 44467:        decode to FMAXNM_ZPmI_S using decoder 339
                                             // 44467:       }
  5, 11,                                     // 44472:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 44474:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44476:        check Inst[9:6] == 0x0
  OPC_Decode, 186, 27, 211, 2,               // 44480:        decode to FMINNM_ZPmI_S using decoder 339
                                             // 44480:       }
  6, 11,                                     // 44485:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 44487:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44489:        check Inst[9:6] == 0x0
  OPC_Decode, 138, 27, 211, 2,               // 44493:        decode to FMAX_ZPmI_S using decoder 339
                                             // 44493:       }
  7, 0,                                      // 44498:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 44500:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44502:        check Inst[9:6] == 0x0
  OPC_Decode, 230, 27, 211, 2,               // 44506:        decode to FMIN_ZPmI_S using decoder 339
                                             // 44506:       }
                                             // 44506:       } // switch Inst[18:16]
                                             // 44506:      }
                                             // 44506:      } // switch Inst[20:19]
                                             // 44506:     }
  1, 7,                                      // 44511:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 44513:      check predicate 33
  OPC_Decode, 182, 26, 164, 1,               // 44515:      decode to FMAD_ZPmZZ_S using decoder 164
                                             // 44515:     }
  2, 142, 2,                                 // 44520:     case 0x2: {
  OPC_SwitchField, 19, 2,                    // 44523:      switch Inst[20:19] {
  0, 75,                                     // 44526:      case 0x0: {
  OPC_SwitchField, 16, 3,                    // 44528:       switch Inst[18:16] {
  0, 7,                                      // 44531:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44533:        check predicate 33
  OPC_Decode, 131, 22, 160, 1,               // 44535:        decode to FADD_ZPmZ_D using decoder 160
                                             // 44535:       }
  1, 7,                                      // 44540:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44542:        check predicate 33
  OPC_Decode, 181, 32, 160, 1,               // 44544:        decode to FSUB_ZPmZ_D using decoder 160
                                             // 44544:       }
  2, 7,                                      // 44549:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 44551:        check predicate 33
  OPC_Decode, 133, 30, 160, 1,               // 44553:        decode to FMUL_ZPmZ_D using decoder 160
                                             // 44553:       }
  3, 7,                                      // 44558:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 44560:        check predicate 33
  OPC_Decode, 168, 32, 160, 1,               // 44562:        decode to FSUBR_ZPmZ_D using decoder 160
                                             // 44562:       }
  4, 7,                                      // 44567:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 44569:        check predicate 33
  OPC_Decode, 223, 26, 160, 1,               // 44571:        decode to FMAXNM_ZPmZ_D using decoder 160
                                             // 44571:       }
  5, 7,                                      // 44576:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 44578:        check predicate 33
  OPC_Decode, 187, 27, 160, 1,               // 44580:        decode to FMINNM_ZPmZ_D using decoder 160
                                             // 44580:       }
  6, 7,                                      // 44585:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 44587:        check predicate 33
  OPC_Decode, 139, 27, 160, 1,               // 44589:        decode to FMAX_ZPmZ_D using decoder 160
                                             // 44589:       }
  7, 0,                                      // 44594:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 44596:        check predicate 33
  OPC_Decode, 231, 27, 160, 1,               // 44598:        decode to FMIN_ZPmZ_D using decoder 160
                                             // 44598:       }
                                             // 44598:       } // switch Inst[18:16]
                                             // 44598:      }
  1, 66,                                     // 44603:      case 0x1: {
  OPC_SwitchField, 16, 3,                    // 44605:       switch Inst[18:16] {
  0, 7,                                      // 44608:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44610:        check predicate 33
  OPC_Decode, 183, 21, 160, 1,               // 44612:        decode to FABD_ZPmZ_D using decoder 160
                                             // 44612:       }
  1, 7,                                      // 44617:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44619:        check predicate 33
  OPC_Decode, 141, 32, 160, 1,               // 44621:        decode to FSCALE_ZPmZ_D using decoder 160
                                             // 44621:       }
  2, 7,                                      // 44626:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 44628:        check predicate 33
  OPC_Decode, 230, 29, 160, 1,               // 44630:        decode to FMULX_ZPmZ_D using decoder 160
                                             // 44630:       }
  4, 7,                                      // 44635:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 44637:        check predicate 33
  OPC_Decode, 240, 25, 160, 1,               // 44639:        decode to FDIVR_ZPmZ_D using decoder 160
                                             // 44639:       }
  5, 7,                                      // 44644:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 44646:        check predicate 33
  OPC_Decode, 244, 25, 160, 1,               // 44648:        decode to FDIV_ZPmZ_D using decoder 160
                                             // 44648:       }
  6, 7,                                      // 44653:       case 0x6: {
  OPC_CheckPredicate, 70,                    // 44655:        check predicate 70
  OPC_Decode, 148, 22, 160, 1,               // 44657:        decode to FAMAX_ZPmZ_D using decoder 160
                                             // 44657:       }
  7, 0,                                      // 44662:       case 0x7: {
  OPC_CheckPredicate, 70,                    // 44664:        check predicate 70
  OPC_Decode, 162, 22, 160, 1,               // 44666:        decode to FAMIN_ZPmZ_D using decoder 160
                                             // 44666:       }
                                             // 44666:       } // switch Inst[18:16]
                                             // 44666:      }
  2, 11,                                     // 44671:      case 0x2: {
  OPC_CheckPredicate, 40,                    // 44673:       check predicate 40
  OPC_CheckField, 10, 3, 0,                  // 44675:       check Inst[12:10] == 0x0
  OPC_Decode, 192, 32, 210, 2,               // 44679:       decode to FTMAD_ZZI_D using decoder 338
                                             // 44679:      }
  3, 0,                                      // 44684:      case 0x3: {
  OPC_SwitchField, 16, 3,                    // 44686:       switch Inst[18:16] {
  0, 11,                                     // 44689:       case 0x0: {
  OPC_CheckPredicate, 33,                    // 44691:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44693:        check Inst[9:6] == 0x0
  OPC_Decode, 128, 22, 211, 2,               // 44697:        decode to FADD_ZPmI_D using decoder 339
                                             // 44697:       }
  1, 11,                                     // 44702:       case 0x1: {
  OPC_CheckPredicate, 33,                    // 44704:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44706:        check Inst[9:6] == 0x0
  OPC_Decode, 178, 32, 211, 2,               // 44710:        decode to FSUB_ZPmI_D using decoder 339
                                             // 44710:       }
  2, 11,                                     // 44715:       case 0x2: {
  OPC_CheckPredicate, 33,                    // 44717:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44719:        check Inst[9:6] == 0x0
  OPC_Decode, 130, 30, 211, 2,               // 44723:        decode to FMUL_ZPmI_D using decoder 339
                                             // 44723:       }
  3, 11,                                     // 44728:       case 0x3: {
  OPC_CheckPredicate, 33,                    // 44730:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44732:        check Inst[9:6] == 0x0
  OPC_Decode, 165, 32, 211, 2,               // 44736:        decode to FSUBR_ZPmI_D using decoder 339
                                             // 44736:       }
  4, 11,                                     // 44741:       case 0x4: {
  OPC_CheckPredicate, 33,                    // 44743:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44745:        check Inst[9:6] == 0x0
  OPC_Decode, 220, 26, 211, 2,               // 44749:        decode to FMAXNM_ZPmI_D using decoder 339
                                             // 44749:       }
  5, 11,                                     // 44754:       case 0x5: {
  OPC_CheckPredicate, 33,                    // 44756:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44758:        check Inst[9:6] == 0x0
  OPC_Decode, 184, 27, 211, 2,               // 44762:        decode to FMINNM_ZPmI_D using decoder 339
                                             // 44762:       }
  6, 11,                                     // 44767:       case 0x6: {
  OPC_CheckPredicate, 33,                    // 44769:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44771:        check Inst[9:6] == 0x0
  OPC_Decode, 136, 27, 211, 2,               // 44775:        decode to FMAX_ZPmI_D using decoder 339
                                             // 44775:       }
  7, 0,                                      // 44780:       case 0x7: {
  OPC_CheckPredicate, 33,                    // 44782:        check predicate 33
  OPC_CheckField, 6, 4, 0,                   // 44784:        check Inst[9:6] == 0x0
  OPC_Decode, 228, 27, 211, 2,               // 44788:        decode to FMIN_ZPmI_D using decoder 339
                                             // 44788:       }
                                             // 44788:       } // switch Inst[18:16]
                                             // 44788:      }
                                             // 44788:      } // switch Inst[20:19]
                                             // 44788:     }
  3, 0,                                      // 44793:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 44795:      check predicate 33
  OPC_Decode, 180, 26, 164, 1,               // 44797:      decode to FMAD_ZPmZZ_D using decoder 164
                                             // 44797:     }
                                             // 44797:     } // switch Inst[22:21]
                                             // 44797:    }
  5, 146, 3,                                 // 44802:    case 0x5: {
  OPC_SwitchField, 21, 2,                    // 44805:     switch Inst[22:21] {
  0, 147, 1,                                 // 44808:     case 0x0: {
  OPC_SwitchField, 16, 5,                    // 44811:      switch Inst[20:16] {
  0, 7,                                      // 44814:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 44816:       check predicate 33
  OPC_Decode, 182, 31, 163, 1,               // 44818:       decode to FRINTN_ZPmZ_S using decoder 163
                                             // 44818:      }
  1, 7,                                      // 44823:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 44825:       check predicate 33
  OPC_Decode, 198, 31, 163, 1,               // 44827:       decode to FRINTP_ZPmZ_S using decoder 163
                                             // 44827:      }
  2, 7,                                      // 44832:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 44834:       check predicate 33
  OPC_Decode, 166, 31, 163, 1,               // 44836:       decode to FRINTM_ZPmZ_S using decoder 163
                                             // 44836:      }
  3, 7,                                      // 44841:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 44843:       check predicate 33
  OPC_Decode, 226, 31, 163, 1,               // 44845:       decode to FRINTZ_ZPmZ_S using decoder 163
                                             // 44845:      }
  4, 7,                                      // 44850:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 44852:       check predicate 33
  OPC_Decode, 136, 31, 163, 1,               // 44854:       decode to FRINTA_ZPmZ_S using decoder 163
                                             // 44854:      }
  6, 7,                                      // 44859:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 44861:       check predicate 33
  OPC_Decode, 212, 31, 163, 1,               // 44863:       decode to FRINTX_ZPmZ_S using decoder 163
                                             // 44863:      }
  7, 7,                                      // 44868:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 44870:       check predicate 33
  OPC_Decode, 150, 31, 163, 1,               // 44872:       decode to FRINTI_ZPmZ_S using decoder 163
                                             // 44872:      }
  8, 7,                                      // 44877:      case 0x8: {
  OPC_CheckPredicate, 33,                    // 44879:       check predicate 33
  OPC_Decode, 231, 25, 163, 1,               // 44881:       decode to FCVT_ZPmZ_StoH using decoder 163
                                             // 44881:      }
  9, 7,                                      // 44886:      case 0x9: {
  OPC_CheckPredicate, 33,                    // 44888:       check predicate 33
  OPC_Decode, 229, 25, 163, 1,               // 44890:       decode to FCVT_ZPmZ_HtoS using decoder 163
                                             // 44890:      }
  10, 7,                                     // 44895:      case 0xa: {
  OPC_CheckPredicate, 56,                    // 44897:       check predicate 56
  OPC_Decode, 137, 15, 163, 1,               // 44899:       decode to BFCVT_ZPmZ using decoder 163
                                             // 44899:      }
  12, 7,                                     // 44904:      case 0xc: {
  OPC_CheckPredicate, 33,                    // 44906:       check predicate 33
  OPC_Decode, 214, 30, 163, 1,               // 44908:       decode to FRECPX_ZPmZ_S using decoder 163
                                             // 44908:      }
  13, 7,                                     // 44913:      case 0xd: {
  OPC_CheckPredicate, 33,                    // 44915:       check predicate 33
  OPC_Decode, 157, 32, 163, 1,               // 44917:       decode to FSQRT_ZPmZ_S using decoder 163
                                             // 44917:      }
  20, 7,                                     // 44922:      case 0x14: {
  OPC_CheckPredicate, 33,                    // 44924:       check predicate 33
  OPC_Decode, 148, 47, 163, 1,               // 44926:       decode to SCVTF_ZPmZ_StoS using decoder 163
                                             // 44926:      }
  21, 7,                                     // 44931:      case 0x15: {
  OPC_CheckPredicate, 33,                    // 44933:       check predicate 33
  OPC_Decode, 141, 63, 163, 1,               // 44935:       decode to UCVTF_ZPmZ_StoS using decoder 163
                                             // 44935:      }
  28, 7,                                     // 44940:      case 0x1c: {
  OPC_CheckPredicate, 33,                    // 44942:       check predicate 33
  OPC_Decode, 147, 25, 163, 1,               // 44944:       decode to FCVTZS_ZPmZ_StoS using decoder 163
                                             // 44944:      }
  29, 0,                                     // 44949:      case 0x1d: {
  OPC_CheckPredicate, 33,                    // 44951:       check predicate 33
  OPC_Decode, 198, 25, 163, 1,               // 44953:       decode to FCVTZU_ZPmZ_StoS using decoder 163
                                             // 44953:      }
                                             // 44953:      } // switch Inst[20:16]
                                             // 44953:     }
  1, 7,                                      // 44958:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 44960:      check predicate 33
  OPC_Decode, 220, 29, 164, 1,               // 44962:      decode to FMSB_ZPmZZ_S using decoder 164
                                             // 44962:     }
  2, 228, 1,                                 // 44967:     case 0x2: {
  OPC_SwitchField, 16, 5,                    // 44970:      switch Inst[20:16] {
  0, 7,                                      // 44973:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 44975:       check predicate 33
  OPC_Decode, 180, 31, 163, 1,               // 44977:       decode to FRINTN_ZPmZ_D using decoder 163
                                             // 44977:      }
  1, 7,                                      // 44982:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 44984:       check predicate 33
  OPC_Decode, 196, 31, 163, 1,               // 44986:       decode to FRINTP_ZPmZ_D using decoder 163
                                             // 44986:      }
  2, 7,                                      // 44991:      case 0x2: {
  OPC_CheckPredicate, 33,                    // 44993:       check predicate 33
  OPC_Decode, 164, 31, 163, 1,               // 44995:       decode to FRINTM_ZPmZ_D using decoder 163
                                             // 44995:      }
  3, 7,                                      // 45000:      case 0x3: {
  OPC_CheckPredicate, 33,                    // 45002:       check predicate 33
  OPC_Decode, 224, 31, 163, 1,               // 45004:       decode to FRINTZ_ZPmZ_D using decoder 163
                                             // 45004:      }
  4, 7,                                      // 45009:      case 0x4: {
  OPC_CheckPredicate, 33,                    // 45011:       check predicate 33
  OPC_Decode, 134, 31, 163, 1,               // 45013:       decode to FRINTA_ZPmZ_D using decoder 163
                                             // 45013:      }
  6, 7,                                      // 45018:      case 0x6: {
  OPC_CheckPredicate, 33,                    // 45020:       check predicate 33
  OPC_Decode, 210, 31, 163, 1,               // 45022:       decode to FRINTX_ZPmZ_D using decoder 163
                                             // 45022:      }
  7, 7,                                      // 45027:      case 0x7: {
  OPC_CheckPredicate, 33,                    // 45029:       check predicate 33
  OPC_Decode, 148, 31, 163, 1,               // 45031:       decode to FRINTI_ZPmZ_D using decoder 163
                                             // 45031:      }
  8, 7,                                      // 45036:      case 0x8: {
  OPC_CheckPredicate, 33,                    // 45038:       check predicate 33
  OPC_Decode, 226, 25, 163, 1,               // 45040:       decode to FCVT_ZPmZ_DtoH using decoder 163
                                             // 45040:      }
  9, 7,                                      // 45045:      case 0x9: {
  OPC_CheckPredicate, 33,                    // 45047:       check predicate 33
  OPC_Decode, 228, 25, 163, 1,               // 45049:       decode to FCVT_ZPmZ_HtoD using decoder 163
                                             // 45049:      }
  10, 7,                                     // 45054:      case 0xa: {
  OPC_CheckPredicate, 33,                    // 45056:       check predicate 33
  OPC_Decode, 227, 25, 163, 1,               // 45058:       decode to FCVT_ZPmZ_DtoS using decoder 163
                                             // 45058:      }
  11, 7,                                     // 45063:      case 0xb: {
  OPC_CheckPredicate, 33,                    // 45065:       check predicate 33
  OPC_Decode, 230, 25, 163, 1,               // 45067:       decode to FCVT_ZPmZ_StoD using decoder 163
                                             // 45067:      }
  12, 7,                                     // 45072:      case 0xc: {
  OPC_CheckPredicate, 33,                    // 45074:       check predicate 33
  OPC_Decode, 212, 30, 163, 1,               // 45076:       decode to FRECPX_ZPmZ_D using decoder 163
                                             // 45076:      }
  13, 7,                                     // 45081:      case 0xd: {
  OPC_CheckPredicate, 33,                    // 45083:       check predicate 33
  OPC_Decode, 155, 32, 163, 1,               // 45085:       decode to FSQRT_ZPmZ_D using decoder 163
                                             // 45085:      }
  16, 7,                                     // 45090:      case 0x10: {
  OPC_CheckPredicate, 33,                    // 45092:       check predicate 33
  OPC_Decode, 146, 47, 163, 1,               // 45094:       decode to SCVTF_ZPmZ_StoD using decoder 163
                                             // 45094:      }
  17, 7,                                     // 45099:      case 0x11: {
  OPC_CheckPredicate, 33,                    // 45101:       check predicate 33
  OPC_Decode, 139, 63, 163, 1,               // 45103:       decode to UCVTF_ZPmZ_StoD using decoder 163
                                             // 45103:      }
  20, 7,                                     // 45108:      case 0x14: {
  OPC_CheckPredicate, 33,                    // 45110:       check predicate 33
  OPC_Decode, 144, 47, 163, 1,               // 45112:       decode to SCVTF_ZPmZ_DtoS using decoder 163
                                             // 45112:      }
  21, 7,                                     // 45117:      case 0x15: {
  OPC_CheckPredicate, 33,                    // 45119:       check predicate 33
  OPC_Decode, 137, 63, 163, 1,               // 45121:       decode to UCVTF_ZPmZ_DtoS using decoder 163
                                             // 45121:      }
  22, 7,                                     // 45126:      case 0x16: {
  OPC_CheckPredicate, 33,                    // 45128:       check predicate 33
  OPC_Decode, 142, 47, 163, 1,               // 45130:       decode to SCVTF_ZPmZ_DtoD using decoder 163
                                             // 45130:      }
  23, 7,                                     // 45135:      case 0x17: {
  OPC_CheckPredicate, 33,                    // 45137:       check predicate 33
  OPC_Decode, 135, 63, 163, 1,               // 45139:       decode to UCVTF_ZPmZ_DtoD using decoder 163
                                             // 45139:      }
  24, 7,                                     // 45144:      case 0x18: {
  OPC_CheckPredicate, 33,                    // 45146:       check predicate 33
  OPC_Decode, 142, 25, 163, 1,               // 45148:       decode to FCVTZS_ZPmZ_DtoS using decoder 163
                                             // 45148:      }
  25, 7,                                     // 45153:      case 0x19: {
  OPC_CheckPredicate, 33,                    // 45155:       check predicate 33
  OPC_Decode, 193, 25, 163, 1,               // 45157:       decode to FCVTZU_ZPmZ_DtoS using decoder 163
                                             // 45157:      }
  28, 7,                                     // 45162:      case 0x1c: {
  OPC_CheckPredicate, 33,                    // 45164:       check predicate 33
  OPC_Decode, 146, 25, 163, 1,               // 45166:       decode to FCVTZS_ZPmZ_StoD using decoder 163
                                             // 45166:      }
  29, 7,                                     // 45171:      case 0x1d: {
  OPC_CheckPredicate, 33,                    // 45173:       check predicate 33
  OPC_Decode, 197, 25, 163, 1,               // 45175:       decode to FCVTZU_ZPmZ_StoD using decoder 163
                                             // 45175:      }
  30, 7,                                     // 45180:      case 0x1e: {
  OPC_CheckPredicate, 33,                    // 45182:       check predicate 33
  OPC_Decode, 141, 25, 163, 1,               // 45184:       decode to FCVTZS_ZPmZ_DtoD using decoder 163
                                             // 45184:      }
  31, 0,                                     // 45189:      case 0x1f: {
  OPC_CheckPredicate, 33,                    // 45191:       check predicate 33
  OPC_Decode, 192, 25, 163, 1,               // 45193:       decode to FCVTZU_ZPmZ_DtoD using decoder 163
                                             // 45193:      }
                                             // 45193:      } // switch Inst[20:16]
                                             // 45193:     }
  3, 0,                                      // 45198:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 45200:      check predicate 33
  OPC_Decode, 218, 29, 164, 1,               // 45202:      decode to FMSB_ZPmZZ_D using decoder 164
                                             // 45202:     }
                                             // 45202:     } // switch Inst[22:21]
                                             // 45202:    }
  6, 67,                                     // 45207:    case 0x6: {
  OPC_SwitchField, 21, 2,                    // 45209:     switch Inst[22:21] {
  0, 21,                                     // 45212:     case 0x0: {
  OPC_SwitchField, 4, 1,                     // 45214:      switch Inst[4] {
  0, 7,                                      // 45217:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 45219:       check predicate 33
  OPC_Decode, 186, 23, 133, 2,               // 45221:       decode to FCMUO_PPzZZ_S using decoder 261
                                             // 45221:      }
  1, 0,                                      // 45226:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 45228:       check predicate 33
  OPC_Decode, 210, 21, 133, 2,               // 45230:       decode to FACGE_PPzZZ_S using decoder 261
                                             // 45230:      }
                                             // 45230:      } // switch Inst[4]
                                             // 45230:     }
  1, 7,                                      // 45235:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 45237:      check predicate 33
  OPC_Decode, 174, 30, 164, 1,               // 45239:      decode to FNMAD_ZPmZZ_S using decoder 164
                                             // 45239:     }
  2, 21,                                     // 45244:     case 0x2: {
  OPC_SwitchField, 4, 1,                     // 45246:      switch Inst[4] {
  0, 7,                                      // 45249:      case 0x0: {
  OPC_CheckPredicate, 33,                    // 45251:       check predicate 33
  OPC_Decode, 184, 23, 133, 2,               // 45253:       decode to FCMUO_PPzZZ_D using decoder 261
                                             // 45253:      }
  1, 0,                                      // 45258:      case 0x1: {
  OPC_CheckPredicate, 33,                    // 45260:       check predicate 33
  OPC_Decode, 208, 21, 133, 2,               // 45262:       decode to FACGE_PPzZZ_D using decoder 261
                                             // 45262:      }
                                             // 45262:      } // switch Inst[4]
                                             // 45262:     }
  3, 0,                                      // 45267:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 45269:      check predicate 33
  OPC_Decode, 172, 30, 164, 1,               // 45271:      decode to FNMAD_ZPmZZ_D using decoder 164
                                             // 45271:     }
                                             // 45271:     } // switch Inst[22:21]
                                             // 45271:    }
  7, 0,                                      // 45276:    case 0x7: {
  OPC_SwitchField, 21, 2,                    // 45278:     switch Inst[22:21] {
  0, 11,                                     // 45281:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 45283:      check predicate 33
  OPC_CheckField, 4, 1, 1,                   // 45285:      check Inst[4] == 0x1
  OPC_Decode, 221, 21, 133, 2,               // 45289:      decode to FACGT_PPzZZ_S using decoder 261
                                             // 45289:     }
  1, 7,                                      // 45294:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 45296:      check predicate 33
  OPC_Decode, 183, 30, 164, 1,               // 45298:      decode to FNMSB_ZPmZZ_S using decoder 164
                                             // 45298:     }
  2, 11,                                     // 45303:     case 0x2: {
  OPC_CheckPredicate, 33,                    // 45305:      check predicate 33
  OPC_CheckField, 4, 1, 1,                   // 45307:      check Inst[4] == 0x1
  OPC_Decode, 219, 21, 133, 2,               // 45311:      decode to FACGT_PPzZZ_D using decoder 261
                                             // 45311:     }
  3, 0,                                      // 45316:     case 0x3: {
  OPC_CheckPredicate, 33,                    // 45318:      check predicate 33
  OPC_Decode, 181, 30, 164, 1,               // 45320:      decode to FNMSB_ZPmZZ_D using decoder 164
                                             // 45320:     }
                                             // 45320:     } // switch Inst[22:21]
                                             // 45320:    }
                                             // 45320:    } // switch Inst[15:13]
                                             // 45320:   }
                                             // 45320:   } // switch Inst[25:23]
                                             // 45320:  }
  4, 162, 8,                                 // 45325:  case 0x4: {
  OPC_SwitchField, 13, 3,                    // 45328:   switch Inst[15:13] {
  0, 129, 1,                                 // 45331:   case 0x0: {
  OPC_SwitchField, 22, 4,                    // 45334:    switch Inst[25:22] {
  0, 25,                                     // 45337:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 45339:     switch Inst[21] {
  0, 7,                                      // 45342:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45344:      check predicate 40
  OPC_Decode, 252, 32, 212, 2,               // 45346:      decode to GLD1SB_S_UXTW using decoder 340
                                             // 45346:     }
  1, 0,                                      // 45351:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45353:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45355:      check Inst[4] == 0x0
  OPC_Decode, 164, 44, 213, 2,               // 45359:      decode to PRFB_S_UXTW_SCALED using decoder 341
                                             // 45359:     }
                                             // 45359:     } // switch Inst[21]
                                             // 45359:    }
  1, 25,                                     // 45364:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 45366:     switch Inst[21] {
  0, 7,                                      // 45369:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45371:      check predicate 40
  OPC_Decode, 251, 32, 212, 2,               // 45373:      decode to GLD1SB_S_SXTW using decoder 340
                                             // 45373:     }
  1, 0,                                      // 45378:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45380:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45382:      check Inst[4] == 0x0
  OPC_Decode, 163, 44, 213, 2,               // 45386:      decode to PRFB_S_SXTW_SCALED using decoder 341
                                             // 45386:     }
                                             // 45386:     } // switch Inst[21]
                                             // 45386:    }
  2, 21,                                     // 45391:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 45393:     switch Inst[21] {
  0, 7,                                      // 45396:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45398:      check predicate 40
  OPC_Decode, 135, 33, 212, 2,               // 45400:      decode to GLD1SH_S_UXTW using decoder 340
                                             // 45400:     }
  1, 0,                                      // 45405:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45407:      check predicate 40
  OPC_Decode, 136, 33, 212, 2,               // 45409:      decode to GLD1SH_S_UXTW_SCALED using decoder 340
                                             // 45409:     }
                                             // 45409:     } // switch Inst[21]
                                             // 45409:    }
  3, 21,                                     // 45414:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 45416:     switch Inst[21] {
  0, 7,                                      // 45419:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45421:      check predicate 40
  OPC_Decode, 133, 33, 212, 2,               // 45423:      decode to GLD1SH_S_SXTW using decoder 340
                                             // 45423:     }
  1, 0,                                      // 45428:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45430:      check predicate 40
  OPC_Decode, 134, 33, 212, 2,               // 45432:      decode to GLD1SH_S_SXTW_SCALED using decoder 340
                                             // 45432:     }
                                             // 45432:     } // switch Inst[21]
                                             // 45432:    }
  6, 11,                                     // 45437:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 45439:     check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 45441:     check Inst[4] == 0x0
  OPC_Decode, 244, 39, 214, 2,               // 45445:     decode to LDR_PXI using decoder 342
                                             // 45445:    }
  7, 0,                                      // 45450:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 45452:     check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 45454:     check Inst[4] == 0x0
  OPC_Decode, 160, 44, 215, 2,               // 45458:     decode to PRFB_PRI using decoder 343
                                             // 45458:    }
                                             // 45458:    } // switch Inst[25:22]
                                             // 45458:   }
  1, 116,                                    // 45463:   case 0x1: {
  OPC_SwitchField, 22, 4,                    // 45465:    switch Inst[25:22] {
  0, 25,                                     // 45468:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 45470:     switch Inst[21] {
  0, 7,                                      // 45473:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45475:      check predicate 40
  OPC_Decode, 188, 33, 212, 2,               // 45477:      decode to GLDFF1SB_S_UXTW using decoder 340
                                             // 45477:     }
  1, 0,                                      // 45482:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45484:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45486:      check Inst[4] == 0x0
  OPC_Decode, 182, 44, 213, 2,               // 45490:      decode to PRFH_S_UXTW_SCALED using decoder 341
                                             // 45490:     }
                                             // 45490:     } // switch Inst[21]
                                             // 45490:    }
  1, 25,                                     // 45495:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 45497:     switch Inst[21] {
  0, 7,                                      // 45500:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45502:      check predicate 40
  OPC_Decode, 187, 33, 212, 2,               // 45504:      decode to GLDFF1SB_S_SXTW using decoder 340
                                             // 45504:     }
  1, 0,                                      // 45509:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45511:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45513:      check Inst[4] == 0x0
  OPC_Decode, 181, 44, 213, 2,               // 45517:      decode to PRFH_S_SXTW_SCALED using decoder 341
                                             // 45517:     }
                                             // 45517:     } // switch Inst[21]
                                             // 45517:    }
  2, 21,                                     // 45522:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 45524:     switch Inst[21] {
  0, 7,                                      // 45527:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45529:      check predicate 40
  OPC_Decode, 199, 33, 212, 2,               // 45531:      decode to GLDFF1SH_S_UXTW using decoder 340
                                             // 45531:     }
  1, 0,                                      // 45536:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45538:      check predicate 40
  OPC_Decode, 200, 33, 212, 2,               // 45540:      decode to GLDFF1SH_S_UXTW_SCALED using decoder 340
                                             // 45540:     }
                                             // 45540:     } // switch Inst[21]
                                             // 45540:    }
  3, 21,                                     // 45545:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 45547:     switch Inst[21] {
  0, 7,                                      // 45550:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45552:      check predicate 40
  OPC_Decode, 197, 33, 212, 2,               // 45554:      decode to GLDFF1SH_S_SXTW using decoder 340
                                             // 45554:     }
  1, 0,                                      // 45559:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45561:      check predicate 40
  OPC_Decode, 198, 33, 212, 2,               // 45563:      decode to GLDFF1SH_S_SXTW_SCALED using decoder 340
                                             // 45563:     }
                                             // 45563:     } // switch Inst[21]
                                             // 45563:    }
  7, 0,                                      // 45568:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 45570:     check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 45572:     check Inst[4] == 0x0
  OPC_Decode, 178, 44, 215, 2,               // 45576:     decode to PRFH_PRI using decoder 343
                                             // 45576:    }
                                             // 45576:    } // switch Inst[25:22]
                                             // 45576:   }
  2, 171, 1,                                 // 45581:   case 0x2: {
  OPC_SwitchField, 22, 4,                    // 45584:    switch Inst[25:22] {
  0, 25,                                     // 45587:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 45589:     switch Inst[21] {
  0, 7,                                      // 45592:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45594:      check predicate 40
  OPC_Decode, 225, 32, 212, 2,               // 45596:      decode to GLD1B_S_UXTW using decoder 340
                                             // 45596:     }
  1, 0,                                      // 45601:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45603:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45605:      check Inst[4] == 0x0
  OPC_Decode, 196, 44, 213, 2,               // 45609:      decode to PRFW_S_UXTW_SCALED using decoder 341
                                             // 45609:     }
                                             // 45609:     } // switch Inst[21]
                                             // 45609:    }
  1, 25,                                     // 45614:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 45616:     switch Inst[21] {
  0, 7,                                      // 45619:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45621:      check predicate 40
  OPC_Decode, 224, 32, 212, 2,               // 45623:      decode to GLD1B_S_SXTW using decoder 340
                                             // 45623:     }
  1, 0,                                      // 45628:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45630:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45632:      check Inst[4] == 0x0
  OPC_Decode, 195, 44, 213, 2,               // 45636:      decode to PRFW_S_SXTW_SCALED using decoder 341
                                             // 45636:     }
                                             // 45636:     } // switch Inst[21]
                                             // 45636:    }
  2, 21,                                     // 45641:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 45643:     switch Inst[21] {
  0, 7,                                      // 45646:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45648:      check predicate 40
  OPC_Decode, 243, 32, 212, 2,               // 45650:      decode to GLD1H_S_UXTW using decoder 340
                                             // 45650:     }
  1, 0,                                      // 45655:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45657:      check predicate 40
  OPC_Decode, 244, 32, 212, 2,               // 45659:      decode to GLD1H_S_UXTW_SCALED using decoder 340
                                             // 45659:     }
                                             // 45659:     } // switch Inst[21]
                                             // 45659:    }
  3, 21,                                     // 45664:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 45666:     switch Inst[21] {
  0, 7,                                      // 45669:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45671:      check predicate 40
  OPC_Decode, 241, 32, 212, 2,               // 45673:      decode to GLD1H_S_SXTW using decoder 340
                                             // 45673:     }
  1, 0,                                      // 45678:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45680:      check predicate 40
  OPC_Decode, 242, 32, 212, 2,               // 45682:      decode to GLD1H_S_SXTW_SCALED using decoder 340
                                             // 45682:     }
                                             // 45682:     } // switch Inst[21]
                                             // 45682:    }
  4, 21,                                     // 45687:    case 0x4: {
  OPC_SwitchField, 21, 1,                    // 45689:     switch Inst[21] {
  0, 7,                                      // 45692:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45694:      check predicate 40
  OPC_Decode, 154, 33, 212, 2,               // 45696:      decode to GLD1W_UXTW using decoder 340
                                             // 45696:     }
  1, 0,                                      // 45701:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45703:      check predicate 40
  OPC_Decode, 155, 33, 212, 2,               // 45705:      decode to GLD1W_UXTW_SCALED using decoder 340
                                             // 45705:     }
                                             // 45705:     } // switch Inst[21]
                                             // 45705:    }
  5, 21,                                     // 45710:    case 0x5: {
  OPC_SwitchField, 21, 1,                    // 45712:     switch Inst[21] {
  0, 7,                                      // 45715:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45717:      check predicate 40
  OPC_Decode, 152, 33, 212, 2,               // 45719:      decode to GLD1W_SXTW using decoder 340
                                             // 45719:     }
  1, 0,                                      // 45724:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45726:      check predicate 40
  OPC_Decode, 153, 33, 212, 2,               // 45728:      decode to GLD1W_SXTW_SCALED using decoder 340
                                             // 45728:     }
                                             // 45728:     } // switch Inst[21]
                                             // 45728:    }
  6, 7,                                      // 45733:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 45735:     check predicate 33
  OPC_Decode, 247, 39, 216, 2,               // 45737:     decode to LDR_ZXI using decoder 344
                                             // 45737:    }
  7, 0,                                      // 45742:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 45744:     check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 45746:     check Inst[4] == 0x0
  OPC_Decode, 192, 44, 215, 2,               // 45750:     decode to PRFW_PRI using decoder 343
                                             // 45750:    }
                                             // 45750:    } // switch Inst[25:22]
                                             // 45750:   }
  3, 162, 1,                                 // 45755:   case 0x3: {
  OPC_SwitchField, 22, 4,                    // 45758:    switch Inst[25:22] {
  0, 25,                                     // 45761:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 45763:     switch Inst[21] {
  0, 7,                                      // 45766:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45768:      check predicate 40
  OPC_Decode, 162, 33, 212, 2,               // 45770:      decode to GLDFF1B_S_UXTW using decoder 340
                                             // 45770:     }
  1, 0,                                      // 45775:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45777:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45779:      check Inst[4] == 0x0
  OPC_Decode, 173, 44, 213, 2,               // 45783:      decode to PRFD_S_UXTW_SCALED using decoder 341
                                             // 45783:     }
                                             // 45783:     } // switch Inst[21]
                                             // 45783:    }
  1, 25,                                     // 45788:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 45790:     switch Inst[21] {
  0, 7,                                      // 45793:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45795:      check predicate 40
  OPC_Decode, 161, 33, 212, 2,               // 45797:      decode to GLDFF1B_S_SXTW using decoder 340
                                             // 45797:     }
  1, 0,                                      // 45802:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45804:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 45806:      check Inst[4] == 0x0
  OPC_Decode, 172, 44, 213, 2,               // 45810:      decode to PRFD_S_SXTW_SCALED using decoder 341
                                             // 45810:     }
                                             // 45810:     } // switch Inst[21]
                                             // 45810:    }
  2, 21,                                     // 45815:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 45817:     switch Inst[21] {
  0, 7,                                      // 45820:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45822:      check predicate 40
  OPC_Decode, 180, 33, 212, 2,               // 45824:      decode to GLDFF1H_S_UXTW using decoder 340
                                             // 45824:     }
  1, 0,                                      // 45829:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45831:      check predicate 40
  OPC_Decode, 181, 33, 212, 2,               // 45833:      decode to GLDFF1H_S_UXTW_SCALED using decoder 340
                                             // 45833:     }
                                             // 45833:     } // switch Inst[21]
                                             // 45833:    }
  3, 21,                                     // 45838:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 45840:     switch Inst[21] {
  0, 7,                                      // 45843:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45845:      check predicate 40
  OPC_Decode, 178, 33, 212, 2,               // 45847:      decode to GLDFF1H_S_SXTW using decoder 340
                                             // 45847:     }
  1, 0,                                      // 45852:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45854:      check predicate 40
  OPC_Decode, 179, 33, 212, 2,               // 45856:      decode to GLDFF1H_S_SXTW_SCALED using decoder 340
                                             // 45856:     }
                                             // 45856:     } // switch Inst[21]
                                             // 45856:    }
  4, 21,                                     // 45861:    case 0x4: {
  OPC_SwitchField, 21, 1,                    // 45863:     switch Inst[21] {
  0, 7,                                      // 45866:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45868:      check predicate 40
  OPC_Decode, 218, 33, 212, 2,               // 45870:      decode to GLDFF1W_UXTW using decoder 340
                                             // 45870:     }
  1, 0,                                      // 45875:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45877:      check predicate 40
  OPC_Decode, 219, 33, 212, 2,               // 45879:      decode to GLDFF1W_UXTW_SCALED using decoder 340
                                             // 45879:     }
                                             // 45879:     } // switch Inst[21]
                                             // 45879:    }
  5, 21,                                     // 45884:    case 0x5: {
  OPC_SwitchField, 21, 1,                    // 45886:     switch Inst[21] {
  0, 7,                                      // 45889:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 45891:      check predicate 40
  OPC_Decode, 216, 33, 212, 2,               // 45893:      decode to GLDFF1W_SXTW using decoder 340
                                             // 45893:     }
  1, 0,                                      // 45898:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45900:      check predicate 40
  OPC_Decode, 217, 33, 212, 2,               // 45902:      decode to GLDFF1W_SXTW_SCALED using decoder 340
                                             // 45902:     }
                                             // 45902:     } // switch Inst[21]
                                             // 45902:    }
  7, 0,                                      // 45907:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 45909:     check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 45911:     check Inst[4] == 0x0
  OPC_Decode, 169, 44, 215, 2,               // 45915:     decode to PRFD_PRI using decoder 343
                                             // 45915:    }
                                             // 45915:    } // switch Inst[25:22]
                                             // 45915:   }
  4, 85,                                     // 45920:   case 0x4: {
  OPC_SwitchField, 22, 4,                    // 45922:    switch Inst[25:22] {
  0, 21,                                     // 45925:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 45927:     switch Inst[21] {
  0, 7,                                      // 45930:     case 0x0: {
  OPC_CheckPredicate, 48,                    // 45932:      check predicate 48
  OPC_Decode, 130, 39, 217, 2,               // 45934:      decode to LDNT1SB_ZZR_S using decoder 345
                                             // 45934:     }
  1, 0,                                      // 45939:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45941:      check predicate 40
  OPC_Decode, 250, 32, 218, 2,               // 45943:      decode to GLD1SB_S_IMM using decoder 346
                                             // 45943:     }
                                             // 45943:     } // switch Inst[21]
                                             // 45943:    }
  1, 7,                                      // 45948:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 45950:     check predicate 33
  OPC_Decode, 253, 34, 219, 2,               // 45952:     decode to LD1RB_IMM using decoder 347
                                             // 45952:    }
  2, 21,                                     // 45957:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 45959:     switch Inst[21] {
  0, 7,                                      // 45962:     case 0x0: {
  OPC_CheckPredicate, 48,                    // 45964:      check predicate 48
  OPC_Decode, 132, 39, 217, 2,               // 45966:      decode to LDNT1SH_ZZR_S using decoder 345
                                             // 45966:     }
  1, 0,                                      // 45971:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 45973:      check predicate 40
  OPC_Decode, 132, 33, 218, 2,               // 45975:      decode to GLD1SH_S_IMM using decoder 346
                                             // 45975:     }
                                             // 45975:     } // switch Inst[21]
                                             // 45975:    }
  3, 7,                                      // 45980:    case 0x3: {
  OPC_CheckPredicate, 33,                    // 45982:     check predicate 33
  OPC_Decode, 152, 35, 219, 2,               // 45984:     decode to LD1RSW_IMM using decoder 347
                                             // 45984:    }
  5, 7,                                      // 45989:    case 0x5: {
  OPC_CheckPredicate, 33,                    // 45991:     check predicate 33
  OPC_Decode, 150, 35, 219, 2,               // 45993:     decode to LD1RSH_D_IMM using decoder 347
                                             // 45993:    }
  7, 0,                                      // 45998:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46000:     check predicate 33
  OPC_Decode, 147, 35, 219, 2,               // 46002:     decode to LD1RSB_D_IMM using decoder 347
                                             // 46002:    }
                                             // 46002:    } // switch Inst[25:22]
                                             // 46002:   }
  5, 98,                                     // 46007:   case 0x5: {
  OPC_SwitchField, 22, 4,                    // 46009:    switch Inst[25:22] {
  0, 21,                                     // 46012:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 46014:     switch Inst[21] {
  0, 7,                                      // 46017:     case 0x0: {
  OPC_CheckPredicate, 48,                    // 46019:      check predicate 48
  OPC_Decode, 233, 38, 217, 2,               // 46021:      decode to LDNT1B_ZZR_S using decoder 345
                                             // 46021:     }
  1, 0,                                      // 46026:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46028:      check predicate 40
  OPC_Decode, 186, 33, 218, 2,               // 46030:      decode to GLDFF1SB_S_IMM using decoder 346
                                             // 46030:     }
                                             // 46030:     } // switch Inst[21]
                                             // 46030:    }
  1, 7,                                      // 46035:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 46037:     check predicate 33
  OPC_Decode, 252, 34, 219, 2,               // 46039:     decode to LD1RB_H_IMM using decoder 347
                                             // 46039:    }
  2, 21,                                     // 46044:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 46046:     switch Inst[21] {
  0, 7,                                      // 46049:     case 0x0: {
  OPC_CheckPredicate, 48,                    // 46051:      check predicate 48
  OPC_Decode, 128, 39, 217, 2,               // 46053:      decode to LDNT1H_ZZR_S using decoder 345
                                             // 46053:     }
  1, 0,                                      // 46058:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46060:      check predicate 40
  OPC_Decode, 196, 33, 218, 2,               // 46062:      decode to GLDFF1SH_S_IMM using decoder 346
                                             // 46062:     }
                                             // 46062:     } // switch Inst[21]
                                             // 46062:    }
  3, 7,                                      // 46067:    case 0x3: {
  OPC_CheckPredicate, 33,                    // 46069:     check predicate 33
  OPC_Decode, 129, 35, 219, 2,               // 46071:     decode to LD1RH_IMM using decoder 347
                                             // 46071:    }
  4, 11,                                     // 46076:    case 0x4: {
  OPC_CheckPredicate, 48,                    // 46078:     check predicate 48
  OPC_CheckField, 21, 1, 0,                  // 46080:     check Inst[21] == 0x0
  OPC_Decode, 145, 39, 217, 2,               // 46084:     decode to LDNT1W_ZZR_S using decoder 345
                                             // 46084:    }
  5, 7,                                      // 46089:    case 0x5: {
  OPC_CheckPredicate, 33,                    // 46091:     check predicate 33
  OPC_Decode, 151, 35, 219, 2,               // 46093:     decode to LD1RSH_S_IMM using decoder 347
                                             // 46093:    }
  7, 0,                                      // 46098:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46100:     check predicate 33
  OPC_Decode, 149, 35, 219, 2,               // 46102:     decode to LD1RSB_S_IMM using decoder 347
                                             // 46102:    }
                                             // 46102:    } // switch Inst[25:22]
                                             // 46102:   }
  6, 137, 1,                                 // 46107:   case 0x6: {
  OPC_SwitchField, 22, 4,                    // 46110:    switch Inst[25:22] {
  0, 25,                                     // 46113:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 46115:     switch Inst[21] {
  0, 11,                                     // 46118:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46120:      check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 46122:      check Inst[4] == 0x0
  OPC_Decode, 161, 44, 220, 2,               // 46126:      decode to PRFB_PRR using decoder 348
                                             // 46126:     }
  1, 0,                                      // 46131:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46133:      check predicate 40
  OPC_Decode, 223, 32, 218, 2,               // 46135:      decode to GLD1B_S_IMM using decoder 346
                                             // 46135:     }
                                             // 46135:     } // switch Inst[21]
                                             // 46135:    }
  1, 7,                                      // 46140:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 46142:     check predicate 33
  OPC_Decode, 254, 34, 219, 2,               // 46144:     decode to LD1RB_S_IMM using decoder 347
                                             // 46144:    }
  2, 25,                                     // 46149:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 46151:     switch Inst[21] {
  0, 11,                                     // 46154:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46156:      check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 46158:      check Inst[4] == 0x0
  OPC_Decode, 179, 44, 220, 2,               // 46162:      decode to PRFH_PRR using decoder 348
                                             // 46162:     }
  1, 0,                                      // 46167:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46169:      check predicate 40
  OPC_Decode, 240, 32, 218, 2,               // 46171:      decode to GLD1H_S_IMM using decoder 346
                                             // 46171:     }
                                             // 46171:     } // switch Inst[21]
                                             // 46171:    }
  3, 7,                                      // 46176:    case 0x3: {
  OPC_CheckPredicate, 33,                    // 46178:     check predicate 33
  OPC_Decode, 130, 35, 219, 2,               // 46180:     decode to LD1RH_S_IMM using decoder 347
                                             // 46180:    }
  4, 25,                                     // 46185:    case 0x4: {
  OPC_SwitchField, 21, 1,                    // 46187:     switch Inst[21] {
  0, 11,                                     // 46190:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46192:      check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 46194:      check Inst[4] == 0x0
  OPC_Decode, 193, 44, 220, 2,               // 46198:      decode to PRFW_PRR using decoder 348
                                             // 46198:     }
  1, 0,                                      // 46203:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46205:      check predicate 40
  OPC_Decode, 151, 33, 218, 2,               // 46207:      decode to GLD1W_IMM using decoder 346
                                             // 46207:     }
                                             // 46207:     } // switch Inst[21]
                                             // 46207:    }
  5, 7,                                      // 46212:    case 0x5: {
  OPC_CheckPredicate, 33,                    // 46214:     check predicate 33
  OPC_Decode, 154, 35, 219, 2,               // 46216:     decode to LD1RW_IMM using decoder 347
                                             // 46216:    }
  6, 15,                                     // 46221:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46223:     check predicate 33
  OPC_CheckField, 21, 1, 0,                  // 46225:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 46229:     check Inst[4] == 0x0
  OPC_Decode, 170, 44, 220, 2,               // 46233:     decode to PRFD_PRR using decoder 348
                                             // 46233:    }
  7, 0,                                      // 46238:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46240:     check predicate 33
  OPC_Decode, 148, 35, 219, 2,               // 46242:     decode to LD1RSB_H_IMM using decoder 347
                                             // 46242:    }
                                             // 46242:    } // switch Inst[25:22]
                                             // 46242:   }
  7, 0,                                      // 46247:   case 0x7: {
  OPC_SwitchField, 22, 4,                    // 46249:    switch Inst[25:22] {
  0, 25,                                     // 46252:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 46254:     switch Inst[21] {
  0, 11,                                     // 46257:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 46259:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 46261:      check Inst[4] == 0x0
  OPC_Decode, 162, 44, 221, 2,               // 46265:      decode to PRFB_S_PZI using decoder 349
                                             // 46265:     }
  1, 0,                                      // 46270:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46272:      check predicate 40
  OPC_Decode, 160, 33, 218, 2,               // 46274:      decode to GLDFF1B_S_IMM using decoder 346
                                             // 46274:     }
                                             // 46274:     } // switch Inst[21]
                                             // 46274:    }
  1, 7,                                      // 46279:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 46281:     check predicate 33
  OPC_Decode, 251, 34, 219, 2,               // 46283:     decode to LD1RB_D_IMM using decoder 347
                                             // 46283:    }
  2, 25,                                     // 46288:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 46290:     switch Inst[21] {
  0, 11,                                     // 46293:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 46295:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 46297:      check Inst[4] == 0x0
  OPC_Decode, 180, 44, 221, 2,               // 46301:      decode to PRFH_S_PZI using decoder 349
                                             // 46301:     }
  1, 0,                                      // 46306:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46308:      check predicate 40
  OPC_Decode, 177, 33, 218, 2,               // 46310:      decode to GLDFF1H_S_IMM using decoder 346
                                             // 46310:     }
                                             // 46310:     } // switch Inst[21]
                                             // 46310:    }
  3, 7,                                      // 46315:    case 0x3: {
  OPC_CheckPredicate, 33,                    // 46317:     check predicate 33
  OPC_Decode, 128, 35, 219, 2,               // 46319:     decode to LD1RH_D_IMM using decoder 347
                                             // 46319:    }
  4, 25,                                     // 46324:    case 0x4: {
  OPC_SwitchField, 21, 1,                    // 46326:     switch Inst[21] {
  0, 11,                                     // 46329:     case 0x0: {
  OPC_CheckPredicate, 40,                    // 46331:      check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 46333:      check Inst[4] == 0x0
  OPC_Decode, 194, 44, 221, 2,               // 46337:      decode to PRFW_S_PZI using decoder 349
                                             // 46337:     }
  1, 0,                                      // 46342:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46344:      check predicate 40
  OPC_Decode, 215, 33, 218, 2,               // 46346:      decode to GLDFF1W_IMM using decoder 346
                                             // 46346:     }
                                             // 46346:     } // switch Inst[21]
                                             // 46346:    }
  5, 7,                                      // 46351:    case 0x5: {
  OPC_CheckPredicate, 33,                    // 46353:     check predicate 33
  OPC_Decode, 153, 35, 219, 2,               // 46355:     decode to LD1RW_D_IMM using decoder 347
                                             // 46355:    }
  6, 15,                                     // 46360:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 46362:     check predicate 40
  OPC_CheckField, 21, 1, 0,                  // 46364:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 46368:     check Inst[4] == 0x0
  OPC_Decode, 171, 44, 221, 2,               // 46372:     decode to PRFD_S_PZI using decoder 349
                                             // 46372:    }
  7, 0,                                      // 46377:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46379:     check predicate 33
  OPC_Decode, 255, 34, 219, 2,               // 46381:     decode to LD1RD_IMM using decoder 347
                                             // 46381:    }
                                             // 46381:    } // switch Inst[25:22]
                                             // 46381:   }
                                             // 46381:   } // switch Inst[15:13]
                                             // 46381:  }
  5, 210, 10,                                // 46386:  case 0x5: {
  OPC_SwitchField, 21, 5,                    // 46389:   switch Inst[25:21] {
  0, 88,                                     // 46392:   case 0x0: {
  OPC_SwitchField, 13, 3,                    // 46394:    switch Inst[15:13] {
  0, 7,                                      // 46397:    case 0x0: {
  OPC_CheckPredicate, 33,                    // 46399:     check predicate 33
  OPC_Decode, 139, 35, 222, 2,               // 46401:     decode to LD1RQ_B using decoder 350
                                             // 46401:    }
  1, 11,                                     // 46406:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 46408:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46410:     check Inst[20] == 0x0
  OPC_Decode, 140, 35, 223, 2,               // 46414:     decode to LD1RQ_B_IMM using decoder 351
                                             // 46414:    }
  2, 7,                                      // 46419:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46421:     check predicate 33
  OPC_Decode, 177, 34, 222, 2,               // 46423:     decode to LD1B using decoder 350
                                             // 46423:    }
  3, 7,                                      // 46428:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46430:     check predicate 40
  OPC_Decode, 255, 37, 224, 2,               // 46432:     decode to LDFF1B using decoder 352
                                             // 46432:    }
  5, 21,                                     // 46437:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46439:     switch Inst[20] {
  0, 7,                                      // 46442:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46444:      check predicate 33
  OPC_Decode, 190, 34, 223, 2,               // 46446:      decode to LD1B_IMM using decoder 351
                                             // 46446:     }
  1, 0,                                      // 46451:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46453:      check predicate 40
  OPC_Decode, 203, 38, 223, 2,               // 46455:      decode to LDNF1B_IMM using decoder 351
                                             // 46455:     }
                                             // 46455:     } // switch Inst[20]
                                             // 46455:    }
  6, 7,                                      // 46460:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46462:     check predicate 33
  OPC_Decode, 231, 38, 222, 2,               // 46464:     decode to LDNT1B_ZRR using decoder 350
                                             // 46464:    }
  7, 0,                                      // 46469:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46471:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46473:     check Inst[20] == 0x0
  OPC_Decode, 230, 38, 223, 2,               // 46477:     decode to LDNT1B_ZRI using decoder 351
                                             // 46477:    }
                                             // 46477:    } // switch Inst[15:13]
                                             // 46477:   }
  1, 88,                                     // 46482:   case 0x1: {
  OPC_SwitchField, 13, 3,                    // 46484:    switch Inst[15:13] {
  0, 7,                                      // 46487:    case 0x0: {
  OPC_CheckPredicate, 67,                    // 46489:     check predicate 67
  OPC_Decode, 131, 35, 222, 2,               // 46491:     decode to LD1RO_B using decoder 350
                                             // 46491:    }
  1, 11,                                     // 46496:    case 0x1: {
  OPC_CheckPredicate, 67,                    // 46498:     check predicate 67
  OPC_CheckField, 20, 1, 0,                  // 46500:     check Inst[20] == 0x0
  OPC_Decode, 132, 35, 223, 2,               // 46504:     decode to LD1RO_B_IMM using decoder 351
                                             // 46504:    }
  2, 7,                                      // 46509:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46511:     check predicate 33
  OPC_Decode, 188, 34, 222, 2,               // 46513:     decode to LD1B_H using decoder 350
                                             // 46513:    }
  3, 7,                                      // 46518:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46520:     check predicate 40
  OPC_Decode, 129, 38, 224, 2,               // 46522:     decode to LDFF1B_H using decoder 352
                                             // 46522:    }
  5, 21,                                     // 46527:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46529:     switch Inst[20] {
  0, 7,                                      // 46532:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46534:      check predicate 33
  OPC_Decode, 189, 34, 223, 2,               // 46536:      decode to LD1B_H_IMM using decoder 351
                                             // 46536:     }
  1, 0,                                      // 46541:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46543:      check predicate 40
  OPC_Decode, 202, 38, 223, 2,               // 46545:      decode to LDNF1B_H_IMM using decoder 351
                                             // 46545:     }
                                             // 46545:     } // switch Inst[20]
                                             // 46545:    }
  6, 7,                                      // 46550:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46552:     check predicate 33
  OPC_Decode, 247, 35, 225, 2,               // 46554:     decode to LD2B using decoder 353
                                             // 46554:    }
  7, 0,                                      // 46559:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46561:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46563:     check Inst[20] == 0x0
  OPC_Decode, 248, 35, 226, 2,               // 46567:     decode to LD2B_IMM using decoder 354
                                             // 46567:    }
                                             // 46567:    } // switch Inst[15:13]
                                             // 46567:   }
  2, 66,                                     // 46572:   case 0x2: {
  OPC_SwitchField, 13, 3,                    // 46574:    switch Inst[15:13] {
  2, 7,                                      // 46577:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46579:     check predicate 33
  OPC_Decode, 191, 34, 222, 2,               // 46581:     decode to LD1B_S using decoder 350
                                             // 46581:    }
  3, 7,                                      // 46586:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46588:     check predicate 40
  OPC_Decode, 130, 38, 224, 2,               // 46590:     decode to LDFF1B_S using decoder 352
                                             // 46590:    }
  5, 21,                                     // 46595:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46597:     switch Inst[20] {
  0, 7,                                      // 46600:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46602:      check predicate 33
  OPC_Decode, 192, 34, 223, 2,               // 46604:      decode to LD1B_S_IMM using decoder 351
                                             // 46604:     }
  1, 0,                                      // 46609:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46611:      check predicate 40
  OPC_Decode, 204, 38, 223, 2,               // 46613:      decode to LDNF1B_S_IMM using decoder 351
                                             // 46613:     }
                                             // 46613:     } // switch Inst[20]
                                             // 46613:    }
  6, 7,                                      // 46618:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46620:     check predicate 33
  OPC_Decode, 167, 36, 227, 2,               // 46622:     decode to LD3B using decoder 355
                                             // 46622:    }
  7, 0,                                      // 46627:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46629:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46631:     check Inst[20] == 0x0
  OPC_Decode, 168, 36, 228, 2,               // 46635:     decode to LD3B_IMM using decoder 356
                                             // 46635:    }
                                             // 46635:    } // switch Inst[15:13]
                                             // 46635:   }
  3, 66,                                     // 46640:   case 0x3: {
  OPC_SwitchField, 13, 3,                    // 46642:    switch Inst[15:13] {
  2, 7,                                      // 46645:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46647:     check predicate 33
  OPC_Decode, 186, 34, 222, 2,               // 46649:     decode to LD1B_D using decoder 350
                                             // 46649:    }
  3, 7,                                      // 46654:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46656:     check predicate 40
  OPC_Decode, 128, 38, 224, 2,               // 46658:     decode to LDFF1B_D using decoder 352
                                             // 46658:    }
  5, 21,                                     // 46663:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46665:     switch Inst[20] {
  0, 7,                                      // 46668:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46670:      check predicate 33
  OPC_Decode, 187, 34, 223, 2,               // 46672:      decode to LD1B_D_IMM using decoder 351
                                             // 46672:     }
  1, 0,                                      // 46677:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46679:      check predicate 40
  OPC_Decode, 201, 38, 223, 2,               // 46681:      decode to LDNF1B_D_IMM using decoder 351
                                             // 46681:     }
                                             // 46681:     } // switch Inst[20]
                                             // 46681:    }
  6, 7,                                      // 46686:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46688:     check predicate 33
  OPC_Decode, 215, 36, 229, 2,               // 46690:     decode to LD4B using decoder 357
                                             // 46690:    }
  7, 0,                                      // 46695:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46697:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46699:     check Inst[20] == 0x0
  OPC_Decode, 216, 36, 230, 2,               // 46703:     decode to LD4B_IMM using decoder 358
                                             // 46703:    }
                                             // 46703:    } // switch Inst[15:13]
                                             // 46703:   }
  4, 98,                                     // 46708:   case 0x4: {
  OPC_SwitchField, 13, 3,                    // 46710:    switch Inst[15:13] {
  0, 7,                                      // 46713:    case 0x0: {
  OPC_CheckPredicate, 33,                    // 46715:     check predicate 33
  OPC_Decode, 143, 35, 222, 2,               // 46717:     decode to LD1RQ_H using decoder 350
                                             // 46717:    }
  1, 11,                                     // 46722:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 46724:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46726:     check Inst[20] == 0x0
  OPC_Decode, 144, 35, 223, 2,               // 46730:     decode to LD1RQ_H_IMM using decoder 351
                                             // 46730:    }
  2, 7,                                      // 46735:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46737:     check predicate 33
  OPC_Decode, 181, 35, 222, 2,               // 46739:     decode to LD1SW_D using decoder 350
                                             // 46739:    }
  3, 7,                                      // 46744:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46746:     check predicate 40
  OPC_Decode, 140, 38, 224, 2,               // 46748:     decode to LDFF1SW_D using decoder 352
                                             // 46748:    }
  5, 21,                                     // 46753:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46755:     switch Inst[20] {
  0, 7,                                      // 46758:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46760:      check predicate 33
  OPC_Decode, 182, 35, 223, 2,               // 46762:      decode to LD1SW_D_IMM using decoder 351
                                             // 46762:     }
  1, 0,                                      // 46767:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46769:      check predicate 40
  OPC_Decode, 214, 38, 223, 2,               // 46771:      decode to LDNF1SW_D_IMM using decoder 351
                                             // 46771:     }
                                             // 46771:     } // switch Inst[20]
                                             // 46771:    }
  6, 7,                                      // 46776:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46778:     check predicate 33
  OPC_Decode, 254, 38, 222, 2,               // 46780:     decode to LDNT1H_ZRR using decoder 350
                                             // 46780:    }
  7, 0,                                      // 46785:    case 0x7: {
  OPC_SwitchField, 20, 1,                    // 46787:     switch Inst[20] {
  0, 7,                                      // 46790:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46792:      check predicate 33
  OPC_Decode, 253, 38, 223, 2,               // 46794:      decode to LDNT1H_ZRI using decoder 351
                                             // 46794:     }
  1, 0,                                      // 46799:     case 0x1: {
  OPC_CheckPredicate, 34,                    // 46801:      check predicate 34
  OPC_Decode, 254, 35, 226, 2,               // 46803:      decode to LD2Q_IMM using decoder 354
                                             // 46803:     }
                                             // 46803:     } // switch Inst[20]
                                             // 46803:    }
                                             // 46803:    } // switch Inst[15:13]
                                             // 46803:   }
  5, 97,                                     // 46808:   case 0x5: {
  OPC_SwitchField, 13, 3,                    // 46810:    switch Inst[15:13] {
  0, 7,                                      // 46813:    case 0x0: {
  OPC_CheckPredicate, 67,                    // 46815:     check predicate 67
  OPC_Decode, 135, 35, 222, 2,               // 46817:     decode to LD1RO_H using decoder 350
                                             // 46817:    }
  1, 11,                                     // 46822:    case 0x1: {
  OPC_CheckPredicate, 67,                    // 46824:     check predicate 67
  OPC_CheckField, 20, 1, 0,                  // 46826:     check Inst[20] == 0x0
  OPC_Decode, 136, 35, 223, 2,               // 46830:     decode to LD1RO_H_IMM using decoder 351
                                             // 46830:    }
  2, 7,                                      // 46835:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46837:     check predicate 33
  OPC_Decode, 221, 34, 222, 2,               // 46839:     decode to LD1H using decoder 350
                                             // 46839:    }
  3, 7,                                      // 46844:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46846:     check predicate 40
  OPC_Decode, 132, 38, 224, 2,               // 46848:     decode to LDFF1H using decoder 352
                                             // 46848:    }
  4, 7,                                      // 46853:    case 0x4: {
  OPC_CheckPredicate, 34,                    // 46855:     check predicate 34
  OPC_Decode, 253, 35, 225, 2,               // 46857:     decode to LD2Q using decoder 353
                                             // 46857:    }
  5, 21,                                     // 46862:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46864:     switch Inst[20] {
  0, 7,                                      // 46867:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46869:      check predicate 33
  OPC_Decode, 232, 34, 223, 2,               // 46871:      decode to LD1H_IMM using decoder 351
                                             // 46871:     }
  1, 0,                                      // 46876:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46878:      check predicate 40
  OPC_Decode, 207, 38, 223, 2,               // 46880:      decode to LDNF1H_IMM using decoder 351
                                             // 46880:     }
                                             // 46880:     } // switch Inst[20]
                                             // 46880:    }
  6, 7,                                      // 46885:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46887:     check predicate 33
  OPC_Decode, 251, 35, 225, 2,               // 46889:     decode to LD2H using decoder 353
                                             // 46889:    }
  7, 0,                                      // 46894:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46896:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46898:     check Inst[20] == 0x0
  OPC_Decode, 252, 35, 226, 2,               // 46902:     decode to LD2H_IMM using decoder 354
                                             // 46902:    }
                                             // 46902:    } // switch Inst[15:13]
                                             // 46902:   }
  6, 66,                                     // 46907:   case 0x6: {
  OPC_SwitchField, 13, 3,                    // 46909:    switch Inst[15:13] {
  2, 7,                                      // 46912:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46914:     check predicate 33
  OPC_Decode, 233, 34, 222, 2,               // 46916:     decode to LD1H_S using decoder 350
                                             // 46916:    }
  3, 7,                                      // 46921:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46923:     check predicate 40
  OPC_Decode, 134, 38, 224, 2,               // 46925:     decode to LDFF1H_S using decoder 352
                                             // 46925:    }
  5, 21,                                     // 46930:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 46932:     switch Inst[20] {
  0, 7,                                      // 46935:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 46937:      check predicate 33
  OPC_Decode, 234, 34, 223, 2,               // 46939:      decode to LD1H_S_IMM using decoder 351
                                             // 46939:     }
  1, 0,                                      // 46944:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 46946:      check predicate 40
  OPC_Decode, 208, 38, 223, 2,               // 46948:      decode to LDNF1H_S_IMM using decoder 351
                                             // 46948:     }
                                             // 46948:     } // switch Inst[20]
                                             // 46948:    }
  6, 7,                                      // 46953:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 46955:     check predicate 33
  OPC_Decode, 171, 36, 227, 2,               // 46957:     decode to LD3H using decoder 355
                                             // 46957:    }
  7, 0,                                      // 46962:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 46964:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 46966:     check Inst[20] == 0x0
  OPC_Decode, 172, 36, 228, 2,               // 46970:     decode to LD3H_IMM using decoder 356
                                             // 46970:    }
                                             // 46970:    } // switch Inst[15:13]
                                             // 46970:   }
  7, 66,                                     // 46975:   case 0x7: {
  OPC_SwitchField, 13, 3,                    // 46977:    switch Inst[15:13] {
  2, 7,                                      // 46980:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 46982:     check predicate 33
  OPC_Decode, 230, 34, 222, 2,               // 46984:     decode to LD1H_D using decoder 350
                                             // 46984:    }
  3, 7,                                      // 46989:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 46991:     check predicate 40
  OPC_Decode, 133, 38, 224, 2,               // 46993:     decode to LDFF1H_D using decoder 352
                                             // 46993:    }
  5, 21,                                     // 46998:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47000:     switch Inst[20] {
  0, 7,                                      // 47003:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47005:      check predicate 33
  OPC_Decode, 231, 34, 223, 2,               // 47007:      decode to LD1H_D_IMM using decoder 351
                                             // 47007:     }
  1, 0,                                      // 47012:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47014:      check predicate 40
  OPC_Decode, 206, 38, 223, 2,               // 47016:      decode to LDNF1H_D_IMM using decoder 351
                                             // 47016:     }
                                             // 47016:     } // switch Inst[20]
                                             // 47016:    }
  6, 7,                                      // 47021:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47023:     check predicate 33
  OPC_Decode, 233, 36, 229, 2,               // 47025:     decode to LD4H using decoder 357
                                             // 47025:    }
  7, 0,                                      // 47030:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47032:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47034:     check Inst[20] == 0x0
  OPC_Decode, 234, 36, 230, 2,               // 47038:     decode to LD4H_IMM using decoder 358
                                             // 47038:    }
                                             // 47038:    } // switch Inst[15:13]
                                             // 47038:   }
  8, 117,                                    // 47043:   case 0x8: {
  OPC_SwitchField, 13, 3,                    // 47045:    switch Inst[15:13] {
  0, 7,                                      // 47048:    case 0x0: {
  OPC_CheckPredicate, 33,                    // 47050:     check predicate 33
  OPC_Decode, 145, 35, 222, 2,               // 47052:     decode to LD1RQ_W using decoder 350
                                             // 47052:    }
  1, 21,                                     // 47057:    case 0x1: {
  OPC_SwitchField, 20, 1,                    // 47059:     switch Inst[20] {
  0, 7,                                      // 47062:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47064:      check predicate 33
  OPC_Decode, 146, 35, 223, 2,               // 47066:      decode to LD1RQ_W_IMM using decoder 351
                                             // 47066:     }
  1, 0,                                      // 47071:     case 0x1: {
  OPC_CheckPredicate, 71,                    // 47073:      check predicate 71
  OPC_Decode, 228, 35, 223, 2,               // 47075:      decode to LD1W_Q_IMM using decoder 351
                                             // 47075:     }
                                             // 47075:     } // switch Inst[20]
                                             // 47075:    }
  2, 7,                                      // 47080:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47082:     check predicate 33
  OPC_Decode, 177, 35, 222, 2,               // 47084:     decode to LD1SH_D using decoder 350
                                             // 47084:    }
  3, 7,                                      // 47089:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47091:     check predicate 40
  OPC_Decode, 138, 38, 224, 2,               // 47093:     decode to LDFF1SH_D using decoder 352
                                             // 47093:    }
  4, 7,                                      // 47098:    case 0x4: {
  OPC_CheckPredicate, 71,                    // 47100:     check predicate 71
  OPC_Decode, 227, 35, 222, 2,               // 47102:     decode to LD1W_Q using decoder 350
                                             // 47102:    }
  5, 21,                                     // 47107:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47109:     switch Inst[20] {
  0, 7,                                      // 47112:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47114:      check predicate 33
  OPC_Decode, 178, 35, 223, 2,               // 47116:      decode to LD1SH_D_IMM using decoder 351
                                             // 47116:     }
  1, 0,                                      // 47121:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47123:      check predicate 40
  OPC_Decode, 212, 38, 223, 2,               // 47125:      decode to LDNF1SH_D_IMM using decoder 351
                                             // 47125:     }
                                             // 47125:     } // switch Inst[20]
                                             // 47125:    }
  6, 7,                                      // 47130:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47132:     check predicate 33
  OPC_Decode, 143, 39, 222, 2,               // 47134:     decode to LDNT1W_ZRR using decoder 350
                                             // 47134:    }
  7, 0,                                      // 47139:    case 0x7: {
  OPC_SwitchField, 20, 1,                    // 47141:     switch Inst[20] {
  0, 7,                                      // 47144:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47146:      check predicate 33
  OPC_Decode, 142, 39, 223, 2,               // 47148:      decode to LDNT1W_ZRI using decoder 351
                                             // 47148:     }
  1, 0,                                      // 47153:     case 0x1: {
  OPC_CheckPredicate, 34,                    // 47155:      check predicate 34
  OPC_Decode, 174, 36, 228, 2,               // 47157:      decode to LD3Q_IMM using decoder 356
                                             // 47157:     }
                                             // 47157:     } // switch Inst[20]
                                             // 47157:    }
                                             // 47157:    } // switch Inst[15:13]
                                             // 47157:   }
  9, 97,                                     // 47162:   case 0x9: {
  OPC_SwitchField, 13, 3,                    // 47164:    switch Inst[15:13] {
  0, 7,                                      // 47167:    case 0x0: {
  OPC_CheckPredicate, 67,                    // 47169:     check predicate 67
  OPC_Decode, 137, 35, 222, 2,               // 47171:     decode to LD1RO_W using decoder 350
                                             // 47171:    }
  1, 11,                                     // 47176:    case 0x1: {
  OPC_CheckPredicate, 67,                    // 47178:     check predicate 67
  OPC_CheckField, 20, 1, 0,                  // 47180:     check Inst[20] == 0x0
  OPC_Decode, 138, 35, 223, 2,               // 47184:     decode to LD1RO_W_IMM using decoder 351
                                             // 47184:    }
  2, 7,                                      // 47189:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47191:     check predicate 33
  OPC_Decode, 179, 35, 222, 2,               // 47193:     decode to LD1SH_S using decoder 350
                                             // 47193:    }
  3, 7,                                      // 47198:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47200:     check predicate 40
  OPC_Decode, 139, 38, 224, 2,               // 47202:     decode to LDFF1SH_S using decoder 352
                                             // 47202:    }
  4, 7,                                      // 47207:    case 0x4: {
  OPC_CheckPredicate, 34,                    // 47209:     check predicate 34
  OPC_Decode, 173, 36, 227, 2,               // 47211:     decode to LD3Q using decoder 355
                                             // 47211:    }
  5, 21,                                     // 47216:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47218:     switch Inst[20] {
  0, 7,                                      // 47221:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47223:      check predicate 33
  OPC_Decode, 180, 35, 223, 2,               // 47225:      decode to LD1SH_S_IMM using decoder 351
                                             // 47225:     }
  1, 0,                                      // 47230:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47232:      check predicate 40
  OPC_Decode, 213, 38, 223, 2,               // 47234:      decode to LDNF1SH_S_IMM using decoder 351
                                             // 47234:     }
                                             // 47234:     } // switch Inst[20]
                                             // 47234:    }
  6, 7,                                      // 47239:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47241:     check predicate 33
  OPC_Decode, 157, 36, 225, 2,               // 47243:     decode to LD2W using decoder 353
                                             // 47243:    }
  7, 0,                                      // 47248:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47250:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47252:     check Inst[20] == 0x0
  OPC_Decode, 158, 36, 226, 2,               // 47256:     decode to LD2W_IMM using decoder 354
                                             // 47256:    }
                                             // 47256:    } // switch Inst[15:13]
                                             // 47256:   }
  10, 66,                                    // 47261:   case 0xa: {
  OPC_SwitchField, 13, 3,                    // 47263:    switch Inst[15:13] {
  2, 7,                                      // 47266:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47268:     check predicate 33
  OPC_Decode, 215, 35, 222, 2,               // 47270:     decode to LD1W using decoder 350
                                             // 47270:    }
  3, 7,                                      // 47275:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47277:     check predicate 40
  OPC_Decode, 141, 38, 224, 2,               // 47279:     decode to LDFF1W using decoder 352
                                             // 47279:    }
  5, 21,                                     // 47284:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47286:     switch Inst[20] {
  0, 7,                                      // 47289:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47291:      check predicate 33
  OPC_Decode, 226, 35, 223, 2,               // 47293:      decode to LD1W_IMM using decoder 351
                                             // 47293:     }
  1, 0,                                      // 47298:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47300:      check predicate 40
  OPC_Decode, 216, 38, 223, 2,               // 47302:      decode to LDNF1W_IMM using decoder 351
                                             // 47302:     }
                                             // 47302:     } // switch Inst[20]
                                             // 47302:    }
  6, 7,                                      // 47307:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47309:     check predicate 33
  OPC_Decode, 205, 36, 227, 2,               // 47311:     decode to LD3W using decoder 355
                                             // 47311:    }
  7, 0,                                      // 47316:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47318:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47320:     check Inst[20] == 0x0
  OPC_Decode, 206, 36, 228, 2,               // 47324:     decode to LD3W_IMM using decoder 356
                                             // 47324:    }
                                             // 47324:    } // switch Inst[15:13]
                                             // 47324:   }
  11, 66,                                    // 47329:   case 0xb: {
  OPC_SwitchField, 13, 3,                    // 47331:    switch Inst[15:13] {
  2, 7,                                      // 47334:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47336:     check predicate 33
  OPC_Decode, 224, 35, 222, 2,               // 47338:     decode to LD1W_D using decoder 350
                                             // 47338:    }
  3, 7,                                      // 47343:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47345:     check predicate 40
  OPC_Decode, 142, 38, 224, 2,               // 47347:     decode to LDFF1W_D using decoder 352
                                             // 47347:    }
  5, 21,                                     // 47352:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47354:     switch Inst[20] {
  0, 7,                                      // 47357:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47359:      check predicate 33
  OPC_Decode, 225, 35, 223, 2,               // 47361:      decode to LD1W_D_IMM using decoder 351
                                             // 47361:     }
  1, 0,                                      // 47366:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47368:      check predicate 40
  OPC_Decode, 215, 38, 223, 2,               // 47370:      decode to LDNF1W_D_IMM using decoder 351
                                             // 47370:     }
                                             // 47370:     } // switch Inst[20]
                                             // 47370:    }
  6, 7,                                      // 47375:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47377:     check predicate 33
  OPC_Decode, 253, 36, 229, 2,               // 47379:     decode to LD4W using decoder 357
                                             // 47379:    }
  7, 0,                                      // 47384:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47386:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47388:     check Inst[20] == 0x0
  OPC_Decode, 254, 36, 230, 2,               // 47392:     decode to LD4W_IMM using decoder 358
                                             // 47392:    }
                                             // 47392:    } // switch Inst[15:13]
                                             // 47392:   }
  12, 117,                                   // 47397:   case 0xc: {
  OPC_SwitchField, 13, 3,                    // 47399:    switch Inst[15:13] {
  0, 7,                                      // 47402:    case 0x0: {
  OPC_CheckPredicate, 33,                    // 47404:     check predicate 33
  OPC_Decode, 141, 35, 222, 2,               // 47406:     decode to LD1RQ_D using decoder 350
                                             // 47406:    }
  1, 21,                                     // 47411:    case 0x1: {
  OPC_SwitchField, 20, 1,                    // 47413:     switch Inst[20] {
  0, 7,                                      // 47416:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47418:      check predicate 33
  OPC_Decode, 142, 35, 223, 2,               // 47420:      decode to LD1RQ_D_IMM using decoder 351
                                             // 47420:     }
  1, 0,                                      // 47425:     case 0x1: {
  OPC_CheckPredicate, 71,                    // 47427:      check predicate 71
  OPC_Decode, 204, 34, 223, 2,               // 47429:      decode to LD1D_Q_IMM using decoder 351
                                             // 47429:     }
                                             // 47429:     } // switch Inst[20]
                                             // 47429:    }
  2, 7,                                      // 47434:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47436:     check predicate 33
  OPC_Decode, 171, 35, 222, 2,               // 47438:     decode to LD1SB_D using decoder 350
                                             // 47438:    }
  3, 7,                                      // 47443:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47445:     check predicate 40
  OPC_Decode, 135, 38, 224, 2,               // 47447:     decode to LDFF1SB_D using decoder 352
                                             // 47447:    }
  4, 7,                                      // 47452:    case 0x4: {
  OPC_CheckPredicate, 71,                    // 47454:     check predicate 71
  OPC_Decode, 203, 34, 222, 2,               // 47456:     decode to LD1D_Q using decoder 350
                                             // 47456:    }
  5, 21,                                     // 47461:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47463:     switch Inst[20] {
  0, 7,                                      // 47466:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47468:      check predicate 33
  OPC_Decode, 172, 35, 223, 2,               // 47470:      decode to LD1SB_D_IMM using decoder 351
                                             // 47470:     }
  1, 0,                                      // 47475:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47477:      check predicate 40
  OPC_Decode, 209, 38, 223, 2,               // 47479:      decode to LDNF1SB_D_IMM using decoder 351
                                             // 47479:     }
                                             // 47479:     } // switch Inst[20]
                                             // 47479:    }
  6, 7,                                      // 47484:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47486:     check predicate 33
  OPC_Decode, 243, 38, 222, 2,               // 47488:     decode to LDNT1D_ZRR using decoder 350
                                             // 47488:    }
  7, 0,                                      // 47493:    case 0x7: {
  OPC_SwitchField, 20, 1,                    // 47495:     switch Inst[20] {
  0, 7,                                      // 47498:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47500:      check predicate 33
  OPC_Decode, 242, 38, 223, 2,               // 47502:      decode to LDNT1D_ZRI using decoder 351
                                             // 47502:     }
  1, 0,                                      // 47507:     case 0x1: {
  OPC_CheckPredicate, 34,                    // 47509:      check predicate 34
  OPC_Decode, 236, 36, 230, 2,               // 47511:      decode to LD4Q_IMM using decoder 358
                                             // 47511:     }
                                             // 47511:     } // switch Inst[20]
                                             // 47511:    }
                                             // 47511:    } // switch Inst[15:13]
                                             // 47511:   }
  13, 97,                                    // 47516:   case 0xd: {
  OPC_SwitchField, 13, 3,                    // 47518:    switch Inst[15:13] {
  0, 7,                                      // 47521:    case 0x0: {
  OPC_CheckPredicate, 67,                    // 47523:     check predicate 67
  OPC_Decode, 133, 35, 222, 2,               // 47525:     decode to LD1RO_D using decoder 350
                                             // 47525:    }
  1, 11,                                     // 47530:    case 0x1: {
  OPC_CheckPredicate, 67,                    // 47532:     check predicate 67
  OPC_CheckField, 20, 1, 0,                  // 47534:     check Inst[20] == 0x0
  OPC_Decode, 134, 35, 223, 2,               // 47538:     decode to LD1RO_D_IMM using decoder 351
                                             // 47538:    }
  2, 7,                                      // 47543:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47545:     check predicate 33
  OPC_Decode, 175, 35, 222, 2,               // 47547:     decode to LD1SB_S using decoder 350
                                             // 47547:    }
  3, 7,                                      // 47552:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47554:     check predicate 40
  OPC_Decode, 137, 38, 224, 2,               // 47556:     decode to LDFF1SB_S using decoder 352
                                             // 47556:    }
  4, 7,                                      // 47561:    case 0x4: {
  OPC_CheckPredicate, 34,                    // 47563:     check predicate 34
  OPC_Decode, 235, 36, 229, 2,               // 47565:     decode to LD4Q using decoder 357
                                             // 47565:    }
  5, 21,                                     // 47570:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47572:     switch Inst[20] {
  0, 7,                                      // 47575:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47577:      check predicate 33
  OPC_Decode, 176, 35, 223, 2,               // 47579:      decode to LD1SB_S_IMM using decoder 351
                                             // 47579:     }
  1, 0,                                      // 47584:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47586:      check predicate 40
  OPC_Decode, 211, 38, 223, 2,               // 47588:      decode to LDNF1SB_S_IMM using decoder 351
                                             // 47588:     }
                                             // 47588:     } // switch Inst[20]
                                             // 47588:    }
  6, 7,                                      // 47593:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47595:     check predicate 33
  OPC_Decode, 249, 35, 225, 2,               // 47597:     decode to LD2D using decoder 353
                                             // 47597:    }
  7, 0,                                      // 47602:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47604:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47606:     check Inst[20] == 0x0
  OPC_Decode, 250, 35, 226, 2,               // 47610:     decode to LD2D_IMM using decoder 354
                                             // 47610:    }
                                             // 47610:    } // switch Inst[15:13]
                                             // 47610:   }
  14, 66,                                    // 47615:   case 0xe: {
  OPC_SwitchField, 13, 3,                    // 47617:    switch Inst[15:13] {
  2, 7,                                      // 47620:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47622:     check predicate 33
  OPC_Decode, 173, 35, 222, 2,               // 47624:     decode to LD1SB_H using decoder 350
                                             // 47624:    }
  3, 7,                                      // 47629:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47631:     check predicate 40
  OPC_Decode, 136, 38, 224, 2,               // 47633:     decode to LDFF1SB_H using decoder 352
                                             // 47633:    }
  5, 21,                                     // 47638:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47640:     switch Inst[20] {
  0, 7,                                      // 47643:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47645:      check predicate 33
  OPC_Decode, 174, 35, 223, 2,               // 47647:      decode to LD1SB_H_IMM using decoder 351
                                             // 47647:     }
  1, 0,                                      // 47652:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47654:      check predicate 40
  OPC_Decode, 210, 38, 223, 2,               // 47656:      decode to LDNF1SB_H_IMM using decoder 351
                                             // 47656:     }
                                             // 47656:     } // switch Inst[20]
                                             // 47656:    }
  6, 7,                                      // 47661:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47663:     check predicate 33
  OPC_Decode, 169, 36, 227, 2,               // 47665:     decode to LD3D using decoder 355
                                             // 47665:    }
  7, 0,                                      // 47670:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47672:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47674:     check Inst[20] == 0x0
  OPC_Decode, 170, 36, 228, 2,               // 47678:     decode to LD3D_IMM using decoder 356
                                             // 47678:    }
                                             // 47678:    } // switch Inst[15:13]
                                             // 47678:   }
  15, 0,                                     // 47683:   case 0xf: {
  OPC_SwitchField, 13, 3,                    // 47685:    switch Inst[15:13] {
  2, 7,                                      // 47688:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 47690:     check predicate 33
  OPC_Decode, 193, 34, 222, 2,               // 47692:     decode to LD1D using decoder 350
                                             // 47692:    }
  3, 7,                                      // 47697:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47699:     check predicate 40
  OPC_Decode, 131, 38, 224, 2,               // 47701:     decode to LDFF1D using decoder 352
                                             // 47701:    }
  5, 21,                                     // 47706:    case 0x5: {
  OPC_SwitchField, 20, 1,                    // 47708:     switch Inst[20] {
  0, 7,                                      // 47711:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 47713:      check predicate 33
  OPC_Decode, 202, 34, 223, 2,               // 47715:      decode to LD1D_IMM using decoder 351
                                             // 47715:     }
  1, 0,                                      // 47720:     case 0x1: {
  OPC_CheckPredicate, 40,                    // 47722:      check predicate 40
  OPC_Decode, 205, 38, 223, 2,               // 47724:      decode to LDNF1D_IMM using decoder 351
                                             // 47724:     }
                                             // 47724:     } // switch Inst[20]
                                             // 47724:    }
  6, 7,                                      // 47729:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 47731:     check predicate 33
  OPC_Decode, 217, 36, 229, 2,               // 47733:     decode to LD4D using decoder 357
                                             // 47733:    }
  7, 0,                                      // 47738:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 47740:     check predicate 33
  OPC_CheckField, 20, 1, 0,                  // 47742:     check Inst[20] == 0x0
  OPC_Decode, 218, 36, 230, 2,               // 47746:     decode to LD4D_IMM using decoder 358
                                             // 47746:    }
                                             // 47746:    } // switch Inst[15:13]
                                             // 47746:   }
                                             // 47746:   } // switch Inst[25:21]
                                             // 47746:  }
  6, 241, 8,                                 // 47751:  case 0x6: {
  OPC_SwitchField, 21, 5,                    // 47754:   switch Inst[25:21] {
  0, 79,                                     // 47757:   case 0x0: {
  OPC_SwitchField, 13, 3,                    // 47759:    switch Inst[15:13] {
  0, 7,                                      // 47762:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 47764:     check predicate 40
  OPC_Decode, 249, 32, 212, 2,               // 47766:     decode to GLD1SB_D_UXTW using decoder 340
                                             // 47766:    }
  1, 7,                                      // 47771:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 47773:     check predicate 40
  OPC_Decode, 185, 33, 212, 2,               // 47775:     decode to GLDFF1SB_D_UXTW using decoder 340
                                             // 47775:    }
  2, 7,                                      // 47780:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 47782:     check predicate 40
  OPC_Decode, 222, 32, 212, 2,               // 47784:     decode to GLD1B_D_UXTW using decoder 340
                                             // 47784:    }
  3, 7,                                      // 47789:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47791:     check predicate 40
  OPC_Decode, 159, 33, 212, 2,               // 47793:     decode to GLDFF1B_D_UXTW using decoder 340
                                             // 47793:    }
  4, 7,                                      // 47798:    case 0x4: {
  OPC_CheckPredicate, 48,                    // 47800:     check predicate 48
  OPC_Decode, 129, 39, 217, 2,               // 47802:     decode to LDNT1SB_ZZR_D using decoder 345
                                             // 47802:    }
  5, 7,                                      // 47807:    case 0x5: {
  OPC_CheckPredicate, 71,                    // 47809:     check predicate 71
  OPC_Decode, 245, 32, 217, 2,               // 47811:     decode to GLD1Q using decoder 345
                                             // 47811:    }
  6, 7,                                      // 47816:    case 0x6: {
  OPC_CheckPredicate, 48,                    // 47818:     check predicate 48
  OPC_Decode, 232, 38, 217, 2,               // 47820:     decode to LDNT1B_ZZR_D using decoder 345
                                             // 47820:    }
  7, 0,                                      // 47825:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 47827:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 47829:     check Inst[4] == 0x0
  OPC_Decode, 156, 44, 221, 2,               // 47833:     decode to PRFB_D_PZI using decoder 349
                                             // 47833:    }
                                             // 47833:    } // switch Inst[15:13]
                                             // 47833:   }
  1, 91,                                     // 47838:   case 0x1: {
  OPC_SwitchField, 13, 3,                    // 47840:    switch Inst[15:13] {
  0, 11,                                     // 47843:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 47845:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 47847:     check Inst[4] == 0x0
  OPC_Decode, 159, 44, 213, 2,               // 47851:     decode to PRFB_D_UXTW_SCALED using decoder 341
                                             // 47851:    }
  1, 11,                                     // 47856:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 47858:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 47860:     check Inst[4] == 0x0
  OPC_Decode, 177, 44, 213, 2,               // 47864:     decode to PRFH_D_UXTW_SCALED using decoder 341
                                             // 47864:    }
  2, 11,                                     // 47869:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 47871:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 47873:     check Inst[4] == 0x0
  OPC_Decode, 191, 44, 213, 2,               // 47877:     decode to PRFW_D_UXTW_SCALED using decoder 341
                                             // 47877:    }
  3, 11,                                     // 47882:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47884:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 47886:     check Inst[4] == 0x0
  OPC_Decode, 168, 44, 213, 2,               // 47890:     decode to PRFD_D_UXTW_SCALED using decoder 341
                                             // 47890:    }
  4, 7,                                      // 47895:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 47897:     check predicate 40
  OPC_Decode, 247, 32, 218, 2,               // 47899:     decode to GLD1SB_D_IMM using decoder 346
                                             // 47899:    }
  5, 7,                                      // 47904:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 47906:     check predicate 40
  OPC_Decode, 183, 33, 218, 2,               // 47908:     decode to GLDFF1SB_D_IMM using decoder 346
                                             // 47908:    }
  6, 7,                                      // 47913:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 47915:     check predicate 40
  OPC_Decode, 220, 32, 218, 2,               // 47917:     decode to GLD1B_D_IMM using decoder 346
                                             // 47917:    }
  7, 0,                                      // 47922:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 47924:     check predicate 40
  OPC_Decode, 157, 33, 218, 2,               // 47926:     decode to GLDFF1B_D_IMM using decoder 346
                                             // 47926:    }
                                             // 47926:    } // switch Inst[15:13]
                                             // 47926:   }
  2, 75,                                     // 47931:   case 0x2: {
  OPC_SwitchField, 13, 3,                    // 47933:    switch Inst[15:13] {
  0, 7,                                      // 47936:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 47938:     check predicate 40
  OPC_Decode, 248, 32, 212, 2,               // 47940:     decode to GLD1SB_D_SXTW using decoder 340
                                             // 47940:    }
  1, 7,                                      // 47945:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 47947:     check predicate 40
  OPC_Decode, 184, 33, 212, 2,               // 47949:     decode to GLDFF1SB_D_SXTW using decoder 340
                                             // 47949:    }
  2, 7,                                      // 47954:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 47956:     check predicate 40
  OPC_Decode, 221, 32, 212, 2,               // 47958:     decode to GLD1B_D_SXTW using decoder 340
                                             // 47958:    }
  3, 7,                                      // 47963:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 47965:     check predicate 40
  OPC_Decode, 158, 33, 212, 2,               // 47967:     decode to GLDFF1B_D_SXTW using decoder 340
                                             // 47967:    }
  4, 7,                                      // 47972:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 47974:     check predicate 40
  OPC_Decode, 246, 32, 212, 2,               // 47976:     decode to GLD1SB_D using decoder 340
                                             // 47976:    }
  5, 7,                                      // 47981:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 47983:     check predicate 40
  OPC_Decode, 182, 33, 212, 2,               // 47985:     decode to GLDFF1SB_D using decoder 340
                                             // 47985:    }
  6, 7,                                      // 47990:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 47992:     check predicate 40
  OPC_Decode, 219, 32, 212, 2,               // 47994:     decode to GLD1B_D using decoder 340
                                             // 47994:    }
  7, 0,                                      // 47999:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48001:     check predicate 40
  OPC_Decode, 156, 33, 212, 2,               // 48003:     decode to GLDFF1B_D using decoder 340
                                             // 48003:    }
                                             // 48003:    } // switch Inst[15:13]
                                             // 48003:   }
  3, 107,                                    // 48008:   case 0x3: {
  OPC_SwitchField, 13, 3,                    // 48010:    switch Inst[15:13] {
  0, 11,                                     // 48013:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48015:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48017:     check Inst[4] == 0x0
  OPC_Decode, 158, 44, 213, 2,               // 48021:     decode to PRFB_D_SXTW_SCALED using decoder 341
                                             // 48021:    }
  1, 11,                                     // 48026:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48028:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48030:     check Inst[4] == 0x0
  OPC_Decode, 176, 44, 213, 2,               // 48034:     decode to PRFH_D_SXTW_SCALED using decoder 341
                                             // 48034:    }
  2, 11,                                     // 48039:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48041:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48043:     check Inst[4] == 0x0
  OPC_Decode, 190, 44, 213, 2,               // 48047:     decode to PRFW_D_SXTW_SCALED using decoder 341
                                             // 48047:    }
  3, 11,                                     // 48052:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48054:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48056:     check Inst[4] == 0x0
  OPC_Decode, 167, 44, 213, 2,               // 48060:     decode to PRFD_D_SXTW_SCALED using decoder 341
                                             // 48060:    }
  4, 11,                                     // 48065:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48067:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48069:     check Inst[4] == 0x0
  OPC_Decode, 157, 44, 213, 2,               // 48073:     decode to PRFB_D_SCALED using decoder 341
                                             // 48073:    }
  5, 11,                                     // 48078:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48080:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48082:     check Inst[4] == 0x0
  OPC_Decode, 175, 44, 213, 2,               // 48086:     decode to PRFH_D_SCALED using decoder 341
                                             // 48086:    }
  6, 11,                                     // 48091:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48093:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48095:     check Inst[4] == 0x0
  OPC_Decode, 189, 44, 213, 2,               // 48099:     decode to PRFW_D_SCALED using decoder 341
                                             // 48099:    }
  7, 0,                                      // 48104:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48106:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48108:     check Inst[4] == 0x0
  OPC_Decode, 166, 44, 213, 2,               // 48112:     decode to PRFD_D_SCALED using decoder 341
                                             // 48112:    }
                                             // 48112:    } // switch Inst[15:13]
                                             // 48112:   }
  4, 70,                                     // 48117:   case 0x4: {
  OPC_SwitchField, 13, 3,                    // 48119:    switch Inst[15:13] {
  0, 7,                                      // 48122:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48124:     check predicate 40
  OPC_Decode, 130, 33, 212, 2,               // 48126:     decode to GLD1SH_D_UXTW using decoder 340
                                             // 48126:    }
  1, 7,                                      // 48131:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48133:     check predicate 40
  OPC_Decode, 194, 33, 212, 2,               // 48135:     decode to GLDFF1SH_D_UXTW using decoder 340
                                             // 48135:    }
  2, 7,                                      // 48140:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48142:     check predicate 40
  OPC_Decode, 238, 32, 212, 2,               // 48144:     decode to GLD1H_D_UXTW using decoder 340
                                             // 48144:    }
  3, 7,                                      // 48149:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48151:     check predicate 40
  OPC_Decode, 175, 33, 212, 2,               // 48153:     decode to GLDFF1H_D_UXTW using decoder 340
                                             // 48153:    }
  4, 7,                                      // 48158:    case 0x4: {
  OPC_CheckPredicate, 48,                    // 48160:     check predicate 48
  OPC_Decode, 131, 39, 217, 2,               // 48162:     decode to LDNT1SH_ZZR_D using decoder 345
                                             // 48162:    }
  6, 7,                                      // 48167:    case 0x6: {
  OPC_CheckPredicate, 48,                    // 48169:     check predicate 48
  OPC_Decode, 255, 38, 217, 2,               // 48171:     decode to LDNT1H_ZZR_D using decoder 345
                                             // 48171:    }
  7, 0,                                      // 48176:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48178:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48180:     check Inst[4] == 0x0
  OPC_Decode, 174, 44, 221, 2,               // 48184:     decode to PRFH_D_PZI using decoder 349
                                             // 48184:    }
                                             // 48184:    } // switch Inst[15:13]
                                             // 48184:   }
  5, 75,                                     // 48189:   case 0x5: {
  OPC_SwitchField, 13, 3,                    // 48191:    switch Inst[15:13] {
  0, 7,                                      // 48194:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48196:     check predicate 40
  OPC_Decode, 131, 33, 212, 2,               // 48198:     decode to GLD1SH_D_UXTW_SCALED using decoder 340
                                             // 48198:    }
  1, 7,                                      // 48203:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48205:     check predicate 40
  OPC_Decode, 195, 33, 212, 2,               // 48207:     decode to GLDFF1SH_D_UXTW_SCALED using decoder 340
                                             // 48207:    }
  2, 7,                                      // 48212:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48214:     check predicate 40
  OPC_Decode, 239, 32, 212, 2,               // 48216:     decode to GLD1H_D_UXTW_SCALED using decoder 340
                                             // 48216:    }
  3, 7,                                      // 48221:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48223:     check predicate 40
  OPC_Decode, 176, 33, 212, 2,               // 48225:     decode to GLDFF1H_D_UXTW_SCALED using decoder 340
                                             // 48225:    }
  4, 7,                                      // 48230:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48232:     check predicate 40
  OPC_Decode, 254, 32, 218, 2,               // 48234:     decode to GLD1SH_D_IMM using decoder 346
                                             // 48234:    }
  5, 7,                                      // 48239:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48241:     check predicate 40
  OPC_Decode, 190, 33, 218, 2,               // 48243:     decode to GLDFF1SH_D_IMM using decoder 346
                                             // 48243:    }
  6, 7,                                      // 48248:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48250:     check predicate 40
  OPC_Decode, 234, 32, 218, 2,               // 48252:     decode to GLD1H_D_IMM using decoder 346
                                             // 48252:    }
  7, 0,                                      // 48257:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48259:     check predicate 40
  OPC_Decode, 171, 33, 218, 2,               // 48261:     decode to GLDFF1H_D_IMM using decoder 346
                                             // 48261:    }
                                             // 48261:    } // switch Inst[15:13]
                                             // 48261:   }
  6, 75,                                     // 48266:   case 0x6: {
  OPC_SwitchField, 13, 3,                    // 48268:    switch Inst[15:13] {
  0, 7,                                      // 48271:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48273:     check predicate 40
  OPC_Decode, 128, 33, 212, 2,               // 48275:     decode to GLD1SH_D_SXTW using decoder 340
                                             // 48275:    }
  1, 7,                                      // 48280:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48282:     check predicate 40
  OPC_Decode, 192, 33, 212, 2,               // 48284:     decode to GLDFF1SH_D_SXTW using decoder 340
                                             // 48284:    }
  2, 7,                                      // 48289:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48291:     check predicate 40
  OPC_Decode, 236, 32, 212, 2,               // 48293:     decode to GLD1H_D_SXTW using decoder 340
                                             // 48293:    }
  3, 7,                                      // 48298:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48300:     check predicate 40
  OPC_Decode, 173, 33, 212, 2,               // 48302:     decode to GLDFF1H_D_SXTW using decoder 340
                                             // 48302:    }
  4, 7,                                      // 48307:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48309:     check predicate 40
  OPC_Decode, 253, 32, 212, 2,               // 48311:     decode to GLD1SH_D using decoder 340
                                             // 48311:    }
  5, 7,                                      // 48316:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48318:     check predicate 40
  OPC_Decode, 189, 33, 212, 2,               // 48320:     decode to GLDFF1SH_D using decoder 340
                                             // 48320:    }
  6, 7,                                      // 48325:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48327:     check predicate 40
  OPC_Decode, 233, 32, 212, 2,               // 48329:     decode to GLD1H_D using decoder 340
                                             // 48329:    }
  7, 0,                                      // 48334:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48336:     check predicate 40
  OPC_Decode, 170, 33, 212, 2,               // 48338:     decode to GLDFF1H_D using decoder 340
                                             // 48338:    }
                                             // 48338:    } // switch Inst[15:13]
                                             // 48338:   }
  7, 75,                                     // 48343:   case 0x7: {
  OPC_SwitchField, 13, 3,                    // 48345:    switch Inst[15:13] {
  0, 7,                                      // 48348:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48350:     check predicate 40
  OPC_Decode, 129, 33, 212, 2,               // 48352:     decode to GLD1SH_D_SXTW_SCALED using decoder 340
                                             // 48352:    }
  1, 7,                                      // 48357:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48359:     check predicate 40
  OPC_Decode, 193, 33, 212, 2,               // 48361:     decode to GLDFF1SH_D_SXTW_SCALED using decoder 340
                                             // 48361:    }
  2, 7,                                      // 48366:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48368:     check predicate 40
  OPC_Decode, 237, 32, 212, 2,               // 48370:     decode to GLD1H_D_SXTW_SCALED using decoder 340
                                             // 48370:    }
  3, 7,                                      // 48375:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48377:     check predicate 40
  OPC_Decode, 174, 33, 212, 2,               // 48379:     decode to GLDFF1H_D_SXTW_SCALED using decoder 340
                                             // 48379:    }
  4, 7,                                      // 48384:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48386:     check predicate 40
  OPC_Decode, 255, 32, 212, 2,               // 48388:     decode to GLD1SH_D_SCALED using decoder 340
                                             // 48388:    }
  5, 7,                                      // 48393:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48395:     check predicate 40
  OPC_Decode, 191, 33, 212, 2,               // 48397:     decode to GLDFF1SH_D_SCALED using decoder 340
                                             // 48397:    }
  6, 7,                                      // 48402:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48404:     check predicate 40
  OPC_Decode, 235, 32, 212, 2,               // 48406:     decode to GLD1H_D_SCALED using decoder 340
                                             // 48406:    }
  7, 0,                                      // 48411:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48413:     check predicate 40
  OPC_Decode, 172, 33, 212, 2,               // 48415:     decode to GLDFF1H_D_SCALED using decoder 340
                                             // 48415:    }
                                             // 48415:    } // switch Inst[15:13]
                                             // 48415:   }
  8, 70,                                     // 48420:   case 0x8: {
  OPC_SwitchField, 13, 3,                    // 48422:    switch Inst[15:13] {
  0, 7,                                      // 48425:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48427:     check predicate 40
  OPC_Decode, 142, 33, 212, 2,               // 48429:     decode to GLD1SW_D_UXTW using decoder 340
                                             // 48429:    }
  1, 7,                                      // 48434:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48436:     check predicate 40
  OPC_Decode, 206, 33, 212, 2,               // 48438:     decode to GLDFF1SW_D_UXTW using decoder 340
                                             // 48438:    }
  2, 7,                                      // 48443:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48445:     check predicate 40
  OPC_Decode, 149, 33, 212, 2,               // 48447:     decode to GLD1W_D_UXTW using decoder 340
                                             // 48447:    }
  3, 7,                                      // 48452:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48454:     check predicate 40
  OPC_Decode, 213, 33, 212, 2,               // 48456:     decode to GLDFF1W_D_UXTW using decoder 340
                                             // 48456:    }
  4, 7,                                      // 48461:    case 0x4: {
  OPC_CheckPredicate, 48,                    // 48463:     check predicate 48
  OPC_Decode, 133, 39, 217, 2,               // 48465:     decode to LDNT1SW_ZZR_D using decoder 345
                                             // 48465:    }
  6, 7,                                      // 48470:    case 0x6: {
  OPC_CheckPredicate, 48,                    // 48472:     check predicate 48
  OPC_Decode, 144, 39, 217, 2,               // 48474:     decode to LDNT1W_ZZR_D using decoder 345
                                             // 48474:    }
  7, 0,                                      // 48479:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48481:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48483:     check Inst[4] == 0x0
  OPC_Decode, 188, 44, 221, 2,               // 48487:     decode to PRFW_D_PZI using decoder 349
                                             // 48487:    }
                                             // 48487:    } // switch Inst[15:13]
                                             // 48487:   }
  9, 75,                                     // 48492:   case 0x9: {
  OPC_SwitchField, 13, 3,                    // 48494:    switch Inst[15:13] {
  0, 7,                                      // 48497:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48499:     check predicate 40
  OPC_Decode, 143, 33, 212, 2,               // 48501:     decode to GLD1SW_D_UXTW_SCALED using decoder 340
                                             // 48501:    }
  1, 7,                                      // 48506:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48508:     check predicate 40
  OPC_Decode, 207, 33, 212, 2,               // 48510:     decode to GLDFF1SW_D_UXTW_SCALED using decoder 340
                                             // 48510:    }
  2, 7,                                      // 48515:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48517:     check predicate 40
  OPC_Decode, 150, 33, 212, 2,               // 48519:     decode to GLD1W_D_UXTW_SCALED using decoder 340
                                             // 48519:    }
  3, 7,                                      // 48524:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48526:     check predicate 40
  OPC_Decode, 214, 33, 212, 2,               // 48528:     decode to GLDFF1W_D_UXTW_SCALED using decoder 340
                                             // 48528:    }
  4, 7,                                      // 48533:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48535:     check predicate 40
  OPC_Decode, 138, 33, 218, 2,               // 48537:     decode to GLD1SW_D_IMM using decoder 346
                                             // 48537:    }
  5, 7,                                      // 48542:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48544:     check predicate 40
  OPC_Decode, 202, 33, 218, 2,               // 48546:     decode to GLDFF1SW_D_IMM using decoder 346
                                             // 48546:    }
  6, 7,                                      // 48551:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48553:     check predicate 40
  OPC_Decode, 145, 33, 218, 2,               // 48555:     decode to GLD1W_D_IMM using decoder 346
                                             // 48555:    }
  7, 0,                                      // 48560:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48562:     check predicate 40
  OPC_Decode, 209, 33, 218, 2,               // 48564:     decode to GLDFF1W_D_IMM using decoder 346
                                             // 48564:    }
                                             // 48564:    } // switch Inst[15:13]
                                             // 48564:   }
  10, 75,                                    // 48569:   case 0xa: {
  OPC_SwitchField, 13, 3,                    // 48571:    switch Inst[15:13] {
  0, 7,                                      // 48574:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48576:     check predicate 40
  OPC_Decode, 140, 33, 212, 2,               // 48578:     decode to GLD1SW_D_SXTW using decoder 340
                                             // 48578:    }
  1, 7,                                      // 48583:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48585:     check predicate 40
  OPC_Decode, 204, 33, 212, 2,               // 48587:     decode to GLDFF1SW_D_SXTW using decoder 340
                                             // 48587:    }
  2, 7,                                      // 48592:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48594:     check predicate 40
  OPC_Decode, 147, 33, 212, 2,               // 48596:     decode to GLD1W_D_SXTW using decoder 340
                                             // 48596:    }
  3, 7,                                      // 48601:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48603:     check predicate 40
  OPC_Decode, 211, 33, 212, 2,               // 48605:     decode to GLDFF1W_D_SXTW using decoder 340
                                             // 48605:    }
  4, 7,                                      // 48610:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48612:     check predicate 40
  OPC_Decode, 137, 33, 212, 2,               // 48614:     decode to GLD1SW_D using decoder 340
                                             // 48614:    }
  5, 7,                                      // 48619:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48621:     check predicate 40
  OPC_Decode, 201, 33, 212, 2,               // 48623:     decode to GLDFF1SW_D using decoder 340
                                             // 48623:    }
  6, 7,                                      // 48628:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48630:     check predicate 40
  OPC_Decode, 144, 33, 212, 2,               // 48632:     decode to GLD1W_D using decoder 340
                                             // 48632:    }
  7, 0,                                      // 48637:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48639:     check predicate 40
  OPC_Decode, 208, 33, 212, 2,               // 48641:     decode to GLDFF1W_D using decoder 340
                                             // 48641:    }
                                             // 48641:    } // switch Inst[15:13]
                                             // 48641:   }
  11, 75,                                    // 48646:   case 0xb: {
  OPC_SwitchField, 13, 3,                    // 48648:    switch Inst[15:13] {
  0, 7,                                      // 48651:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 48653:     check predicate 40
  OPC_Decode, 141, 33, 212, 2,               // 48655:     decode to GLD1SW_D_SXTW_SCALED using decoder 340
                                             // 48655:    }
  1, 7,                                      // 48660:    case 0x1: {
  OPC_CheckPredicate, 40,                    // 48662:     check predicate 40
  OPC_Decode, 205, 33, 212, 2,               // 48664:     decode to GLDFF1SW_D_SXTW_SCALED using decoder 340
                                             // 48664:    }
  2, 7,                                      // 48669:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48671:     check predicate 40
  OPC_Decode, 148, 33, 212, 2,               // 48673:     decode to GLD1W_D_SXTW_SCALED using decoder 340
                                             // 48673:    }
  3, 7,                                      // 48678:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48680:     check predicate 40
  OPC_Decode, 212, 33, 212, 2,               // 48682:     decode to GLDFF1W_D_SXTW_SCALED using decoder 340
                                             // 48682:    }
  4, 7,                                      // 48687:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 48689:     check predicate 40
  OPC_Decode, 139, 33, 212, 2,               // 48691:     decode to GLD1SW_D_SCALED using decoder 340
                                             // 48691:    }
  5, 7,                                      // 48696:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 48698:     check predicate 40
  OPC_Decode, 203, 33, 212, 2,               // 48700:     decode to GLDFF1SW_D_SCALED using decoder 340
                                             // 48700:    }
  6, 7,                                      // 48705:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48707:     check predicate 40
  OPC_Decode, 146, 33, 212, 2,               // 48709:     decode to GLD1W_D_SCALED using decoder 340
                                             // 48709:    }
  7, 0,                                      // 48714:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48716:     check predicate 40
  OPC_Decode, 210, 33, 212, 2,               // 48718:     decode to GLDFF1W_D_SCALED using decoder 340
                                             // 48718:    }
                                             // 48718:    } // switch Inst[15:13]
                                             // 48718:   }
  12, 43,                                    // 48723:   case 0xc: {
  OPC_SwitchField, 13, 3,                    // 48725:    switch Inst[15:13] {
  2, 7,                                      // 48728:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48730:     check predicate 40
  OPC_Decode, 231, 32, 212, 2,               // 48732:     decode to GLD1D_UXTW using decoder 340
                                             // 48732:    }
  3, 7,                                      // 48737:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48739:     check predicate 40
  OPC_Decode, 168, 33, 212, 2,               // 48741:     decode to GLDFF1D_UXTW using decoder 340
                                             // 48741:    }
  6, 7,                                      // 48746:    case 0x6: {
  OPC_CheckPredicate, 48,                    // 48748:     check predicate 48
  OPC_Decode, 244, 38, 217, 2,               // 48750:     decode to LDNT1D_ZZR_D using decoder 345
                                             // 48750:    }
  7, 0,                                      // 48755:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48757:     check predicate 40
  OPC_CheckField, 4, 1, 0,                   // 48759:     check Inst[4] == 0x0
  OPC_Decode, 165, 44, 221, 2,               // 48763:     decode to PRFD_D_PZI using decoder 349
                                             // 48763:    }
                                             // 48763:    } // switch Inst[15:13]
                                             // 48763:   }
  13, 39,                                    // 48768:   case 0xd: {
  OPC_SwitchField, 13, 3,                    // 48770:    switch Inst[15:13] {
  2, 7,                                      // 48773:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48775:     check predicate 40
  OPC_Decode, 232, 32, 212, 2,               // 48777:     decode to GLD1D_UXTW_SCALED using decoder 340
                                             // 48777:    }
  3, 7,                                      // 48782:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48784:     check predicate 40
  OPC_Decode, 169, 33, 212, 2,               // 48786:     decode to GLDFF1D_UXTW_SCALED using decoder 340
                                             // 48786:    }
  6, 7,                                      // 48791:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48793:     check predicate 40
  OPC_Decode, 227, 32, 218, 2,               // 48795:     decode to GLD1D_IMM using decoder 346
                                             // 48795:    }
  7, 0,                                      // 48800:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48802:     check predicate 40
  OPC_Decode, 164, 33, 218, 2,               // 48804:     decode to GLDFF1D_IMM using decoder 346
                                             // 48804:    }
                                             // 48804:    } // switch Inst[15:13]
                                             // 48804:   }
  14, 39,                                    // 48809:   case 0xe: {
  OPC_SwitchField, 13, 3,                    // 48811:    switch Inst[15:13] {
  2, 7,                                      // 48814:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48816:     check predicate 40
  OPC_Decode, 229, 32, 212, 2,               // 48818:     decode to GLD1D_SXTW using decoder 340
                                             // 48818:    }
  3, 7,                                      // 48823:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48825:     check predicate 40
  OPC_Decode, 166, 33, 212, 2,               // 48827:     decode to GLDFF1D_SXTW using decoder 340
                                             // 48827:    }
  6, 7,                                      // 48832:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48834:     check predicate 40
  OPC_Decode, 226, 32, 212, 2,               // 48836:     decode to GLD1D using decoder 340
                                             // 48836:    }
  7, 0,                                      // 48841:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48843:     check predicate 40
  OPC_Decode, 163, 33, 212, 2,               // 48845:     decode to GLDFF1D using decoder 340
                                             // 48845:    }
                                             // 48845:    } // switch Inst[15:13]
                                             // 48845:   }
  15, 0,                                     // 48850:   case 0xf: {
  OPC_SwitchField, 13, 3,                    // 48852:    switch Inst[15:13] {
  2, 7,                                      // 48855:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 48857:     check predicate 40
  OPC_Decode, 230, 32, 212, 2,               // 48859:     decode to GLD1D_SXTW_SCALED using decoder 340
                                             // 48859:    }
  3, 7,                                      // 48864:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 48866:     check predicate 40
  OPC_Decode, 167, 33, 212, 2,               // 48868:     decode to GLDFF1D_SXTW_SCALED using decoder 340
                                             // 48868:    }
  6, 7,                                      // 48873:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 48875:     check predicate 40
  OPC_Decode, 228, 32, 212, 2,               // 48877:     decode to GLD1D_SCALED using decoder 340
                                             // 48877:    }
  7, 0,                                      // 48882:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 48884:     check predicate 40
  OPC_Decode, 165, 33, 212, 2,               // 48886:     decode to GLDFF1D_SCALED using decoder 340
                                             // 48886:    }
                                             // 48886:    } // switch Inst[15:13]
                                             // 48886:   }
                                             // 48886:   } // switch Inst[25:21]
                                             // 48886:  }
  7, 0,                                      // 48891:  case 0x7: {
  OPC_SwitchField, 13, 3,                    // 48893:   switch Inst[15:13] {
  0, 97,                                     // 48896:   case 0x0: {
  OPC_SwitchField, 22, 4,                    // 48898:    switch Inst[25:22] {
  1, 25,                                     // 48901:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 48903:     switch Inst[21] {
  0, 11,                                     // 48906:     case 0x0: {
  OPC_CheckPredicate, 34,                    // 48908:      check predicate 34
  OPC_CheckField, 20, 1, 0,                  // 48910:      check Inst[20] == 0x0
  OPC_Decode, 180, 57, 226, 2,               // 48914:      decode to ST2Q_IMM using decoder 354
                                             // 48914:     }
  1, 0,                                      // 48919:     case 0x1: {
  OPC_CheckPredicate, 34,                    // 48921:      check predicate 34
  OPC_Decode, 179, 57, 225, 2,               // 48923:      decode to ST2Q using decoder 353
                                             // 48923:     }
                                             // 48923:     } // switch Inst[21]
                                             // 48923:    }
  2, 25,                                     // 48928:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 48930:     switch Inst[21] {
  0, 11,                                     // 48933:     case 0x0: {
  OPC_CheckPredicate, 34,                    // 48935:      check predicate 34
  OPC_CheckField, 20, 1, 0,                  // 48937:      check Inst[20] == 0x0
  OPC_Decode, 212, 57, 228, 2,               // 48941:      decode to ST3Q_IMM using decoder 356
                                             // 48941:     }
  1, 0,                                      // 48946:     case 0x1: {
  OPC_CheckPredicate, 34,                    // 48948:      check predicate 34
  OPC_Decode, 211, 57, 227, 2,               // 48950:      decode to ST3Q using decoder 355
                                             // 48950:     }
                                             // 48950:     } // switch Inst[21]
                                             // 48950:    }
  3, 25,                                     // 48955:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 48957:     switch Inst[21] {
  0, 11,                                     // 48960:     case 0x0: {
  OPC_CheckPredicate, 34,                    // 48962:      check predicate 34
  OPC_CheckField, 20, 1, 0,                  // 48964:      check Inst[20] == 0x0
  OPC_Decode, 130, 58, 230, 2,               // 48968:      decode to ST4Q_IMM using decoder 358
                                             // 48968:     }
  1, 0,                                      // 48973:     case 0x1: {
  OPC_CheckPredicate, 34,                    // 48975:      check predicate 34
  OPC_Decode, 129, 58, 229, 2,               // 48977:      decode to ST4Q using decoder 357
                                             // 48977:     }
                                             // 48977:     } // switch Inst[21]
                                             // 48977:    }
  6, 0,                                      // 48982:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 48984:     check predicate 33
  OPC_CheckField, 4, 1, 0,                   // 48986:     check Inst[4] == 0x0
  OPC_Decode, 211, 59, 214, 2,               // 48990:     decode to STR_PXI using decoder 342
                                             // 48990:    }
                                             // 48990:    } // switch Inst[25:22]
                                             // 48990:   }
  1, 75,                                     // 48995:   case 0x1: {
  OPC_SwitchField, 21, 5,                    // 48997:    switch Inst[25:21] {
  0, 7,                                      // 49000:    case 0x0: {
  OPC_CheckPredicate, 48,                    // 49002:     check predicate 48
  OPC_Decode, 242, 58, 217, 2,               // 49004:     decode to STNT1B_ZZR_D using decoder 345
                                             // 49004:    }
  1, 7,                                      // 49009:    case 0x1: {
  OPC_CheckPredicate, 71,                    // 49011:     check predicate 71
  OPC_Decode, 245, 55, 217, 2,               // 49013:     decode to SST1Q using decoder 345
                                             // 49013:    }
  2, 7,                                      // 49018:    case 0x2: {
  OPC_CheckPredicate, 48,                    // 49020:     check predicate 48
  OPC_Decode, 243, 58, 217, 2,               // 49022:     decode to STNT1B_ZZR_S using decoder 345
                                             // 49022:    }
  4, 7,                                      // 49027:    case 0x4: {
  OPC_CheckPredicate, 48,                    // 49029:     check predicate 48
  OPC_Decode, 137, 59, 217, 2,               // 49031:     decode to STNT1H_ZZR_D using decoder 345
                                             // 49031:    }
  6, 7,                                      // 49036:    case 0x6: {
  OPC_CheckPredicate, 48,                    // 49038:     check predicate 48
  OPC_Decode, 138, 59, 217, 2,               // 49040:     decode to STNT1H_ZZR_S using decoder 345
                                             // 49040:    }
  8, 7,                                      // 49045:    case 0x8: {
  OPC_CheckPredicate, 48,                    // 49047:     check predicate 48
  OPC_Decode, 149, 59, 217, 2,               // 49049:     decode to STNT1W_ZZR_D using decoder 345
                                             // 49049:    }
  10, 7,                                     // 49054:    case 0xa: {
  OPC_CheckPredicate, 48,                    // 49056:     check predicate 48
  OPC_Decode, 150, 59, 217, 2,               // 49058:     decode to STNT1W_ZZR_S using decoder 345
                                             // 49058:    }
  12, 0,                                     // 49063:    case 0xc: {
  OPC_CheckPredicate, 48,                    // 49065:     check predicate 48
  OPC_Decode, 254, 58, 217, 2,               // 49067:     decode to STNT1D_ZZR_D using decoder 345
                                             // 49067:    }
                                             // 49067:    } // switch Inst[25:21]
                                             // 49067:   }
  2, 153, 1,                                 // 49072:   case 0x2: {
  OPC_SwitchField, 22, 4,                    // 49075:    switch Inst[25:22] {
  0, 21,                                     // 49078:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 49080:     switch Inst[21] {
  0, 7,                                      // 49083:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 49085:      check predicate 33
  OPC_Decode, 160, 56, 222, 2,               // 49087:      decode to ST1B using decoder 350
                                             // 49087:     }
  1, 0,                                      // 49092:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 49094:      check predicate 33
  OPC_Decode, 171, 56, 222, 2,               // 49096:      decode to ST1B_H using decoder 350
                                             // 49096:     }
                                             // 49096:     } // switch Inst[21]
                                             // 49096:    }
  1, 21,                                     // 49101:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 49103:     switch Inst[21] {
  0, 7,                                      // 49106:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 49108:      check predicate 33
  OPC_Decode, 174, 56, 222, 2,               // 49110:      decode to ST1B_S using decoder 350
                                             // 49110:     }
  1, 0,                                      // 49115:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 49117:      check predicate 33
  OPC_Decode, 169, 56, 222, 2,               // 49119:      decode to ST1B_D using decoder 350
                                             // 49119:     }
                                             // 49119:     } // switch Inst[21]
                                             // 49119:    }
  2, 11,                                     // 49124:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 49126:     check predicate 33
  OPC_CheckField, 21, 1, 1,                  // 49128:     check Inst[21] == 0x1
  OPC_Decode, 204, 56, 222, 2,               // 49132:     decode to ST1H using decoder 350
                                             // 49132:    }
  3, 21,                                     // 49137:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 49139:     switch Inst[21] {
  0, 7,                                      // 49142:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 49144:      check predicate 33
  OPC_Decode, 216, 56, 222, 2,               // 49146:      decode to ST1H_S using decoder 350
                                             // 49146:     }
  1, 0,                                      // 49151:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 49153:      check predicate 33
  OPC_Decode, 213, 56, 222, 2,               // 49155:      decode to ST1H_D using decoder 350
                                             // 49155:     }
                                             // 49155:     } // switch Inst[21]
                                             // 49155:    }
  4, 11,                                     // 49160:    case 0x4: {
  OPC_CheckPredicate, 71,                    // 49162:     check predicate 71
  OPC_CheckField, 21, 1, 0,                  // 49164:     check Inst[21] == 0x0
  OPC_Decode, 150, 57, 222, 2,               // 49168:     decode to ST1W_Q using decoder 350
                                             // 49168:    }
  5, 21,                                     // 49173:    case 0x5: {
  OPC_SwitchField, 21, 1,                    // 49175:     switch Inst[21] {
  0, 7,                                      // 49178:     case 0x0: {
  OPC_CheckPredicate, 33,                    // 49180:      check predicate 33
  OPC_Decode, 138, 57, 222, 2,               // 49182:      decode to ST1W using decoder 350
                                             // 49182:     }
  1, 0,                                      // 49187:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 49189:      check predicate 33
  OPC_Decode, 147, 57, 222, 2,               // 49191:      decode to ST1W_D using decoder 350
                                             // 49191:     }
                                             // 49191:     } // switch Inst[21]
                                             // 49191:    }
  6, 7,                                      // 49196:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 49198:     check predicate 33
  OPC_Decode, 214, 59, 216, 2,               // 49200:     decode to STR_ZXI using decoder 344
                                             // 49200:    }
  7, 0,                                      // 49205:    case 0x7: {
  OPC_SwitchField, 21, 1,                    // 49207:     switch Inst[21] {
  0, 7,                                      // 49210:     case 0x0: {
  OPC_CheckPredicate, 71,                    // 49212:      check predicate 71
  OPC_Decode, 186, 56, 222, 2,               // 49214:      decode to ST1D_Q using decoder 350
                                             // 49214:     }
  1, 0,                                      // 49219:     case 0x1: {
  OPC_CheckPredicate, 33,                    // 49221:      check predicate 33
  OPC_Decode, 176, 56, 222, 2,               // 49223:      decode to ST1D using decoder 350
                                             // 49223:     }
                                             // 49223:     } // switch Inst[21]
                                             // 49223:    }
                                             // 49223:    } // switch Inst[25:22]
                                             // 49223:   }
  3, 147, 1,                                 // 49228:   case 0x3: {
  OPC_SwitchField, 21, 5,                    // 49231:    switch Inst[25:21] {
  0, 7,                                      // 49234:    case 0x0: {
  OPC_CheckPredicate, 33,                    // 49236:     check predicate 33
  OPC_Decode, 241, 58, 222, 2,               // 49238:     decode to STNT1B_ZRR using decoder 350
                                             // 49238:    }
  1, 7,                                      // 49243:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 49245:     check predicate 33
  OPC_Decode, 170, 57, 225, 2,               // 49247:     decode to ST2B using decoder 353
                                             // 49247:    }
  2, 7,                                      // 49252:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 49254:     check predicate 33
  OPC_Decode, 205, 57, 227, 2,               // 49256:     decode to ST3B using decoder 355
                                             // 49256:    }
  3, 7,                                      // 49261:    case 0x3: {
  OPC_CheckPredicate, 33,                    // 49263:     check predicate 33
  OPC_Decode, 237, 57, 229, 2,               // 49265:     decode to ST4B using decoder 357
                                             // 49265:    }
  4, 7,                                      // 49270:    case 0x4: {
  OPC_CheckPredicate, 33,                    // 49272:     check predicate 33
  OPC_Decode, 136, 59, 222, 2,               // 49274:     decode to STNT1H_ZRR using decoder 350
                                             // 49274:    }
  5, 7,                                      // 49279:    case 0x5: {
  OPC_CheckPredicate, 33,                    // 49281:     check predicate 33
  OPC_Decode, 177, 57, 225, 2,               // 49283:     decode to ST2H using decoder 353
                                             // 49283:    }
  6, 7,                                      // 49288:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 49290:     check predicate 33
  OPC_Decode, 209, 57, 227, 2,               // 49292:     decode to ST3H using decoder 355
                                             // 49292:    }
  7, 7,                                      // 49297:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 49299:     check predicate 33
  OPC_Decode, 255, 57, 229, 2,               // 49301:     decode to ST4H using decoder 357
                                             // 49301:    }
  8, 7,                                      // 49306:    case 0x8: {
  OPC_CheckPredicate, 33,                    // 49308:     check predicate 33
  OPC_Decode, 148, 59, 222, 2,               // 49310:     decode to STNT1W_ZRR using decoder 350
                                             // 49310:    }
  9, 7,                                      // 49315:    case 0x9: {
  OPC_CheckPredicate, 33,                    // 49317:     check predicate 33
  OPC_Decode, 195, 57, 225, 2,               // 49319:     decode to ST2W using decoder 353
                                             // 49319:    }
  10, 7,                                     // 49324:    case 0xa: {
  OPC_CheckPredicate, 33,                    // 49326:     check predicate 33
  OPC_Decode, 227, 57, 227, 2,               // 49328:     decode to ST3W using decoder 355
                                             // 49328:    }
  11, 7,                                     // 49333:    case 0xb: {
  OPC_CheckPredicate, 33,                    // 49335:     check predicate 33
  OPC_Decode, 131, 58, 229, 2,               // 49337:     decode to ST4W using decoder 357
                                             // 49337:    }
  12, 7,                                     // 49342:    case 0xc: {
  OPC_CheckPredicate, 33,                    // 49344:     check predicate 33
  OPC_Decode, 253, 58, 222, 2,               // 49346:     decode to STNT1D_ZRR using decoder 350
                                             // 49346:    }
  13, 7,                                     // 49351:    case 0xd: {
  OPC_CheckPredicate, 33,                    // 49353:     check predicate 33
  OPC_Decode, 172, 57, 225, 2,               // 49355:     decode to ST2D using decoder 353
                                             // 49355:    }
  14, 7,                                     // 49360:    case 0xe: {
  OPC_CheckPredicate, 33,                    // 49362:     check predicate 33
  OPC_Decode, 207, 57, 227, 2,               // 49364:     decode to ST3D using decoder 355
                                             // 49364:    }
  15, 0,                                     // 49369:    case 0xf: {
  OPC_CheckPredicate, 33,                    // 49371:     check predicate 33
  OPC_Decode, 239, 57, 229, 2,               // 49373:     decode to ST4D using decoder 357
                                             // 49373:    }
                                             // 49373:    } // switch Inst[25:21]
                                             // 49373:   }
  4, 111,                                    // 49378:   case 0x4: {
  OPC_SwitchField, 21, 5,                    // 49380:    switch Inst[25:21] {
  0, 7,                                      // 49383:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 49385:     check predicate 40
  OPC_Decode, 222, 55, 212, 2,               // 49387:     decode to SST1B_D_UXTW using decoder 340
                                             // 49387:    }
  2, 7,                                      // 49392:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 49394:     check predicate 40
  OPC_Decode, 225, 55, 212, 2,               // 49396:     decode to SST1B_S_UXTW using decoder 340
                                             // 49396:    }
  4, 7,                                      // 49401:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 49403:     check predicate 40
  OPC_Decode, 238, 55, 212, 2,               // 49405:     decode to SST1H_D_UXTW using decoder 340
                                             // 49405:    }
  5, 7,                                      // 49410:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 49412:     check predicate 40
  OPC_Decode, 239, 55, 212, 2,               // 49414:     decode to SST1H_D_UXTW_SCALED using decoder 340
                                             // 49414:    }
  6, 7,                                      // 49419:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 49421:     check predicate 40
  OPC_Decode, 243, 55, 212, 2,               // 49423:     decode to SST1H_S_UXTW using decoder 340
                                             // 49423:    }
  7, 7,                                      // 49428:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 49430:     check predicate 40
  OPC_Decode, 244, 55, 212, 2,               // 49432:     decode to SST1H_S_UXTW_SCALED using decoder 340
                                             // 49432:    }
  8, 7,                                      // 49437:    case 0x8: {
  OPC_CheckPredicate, 40,                    // 49439:     check predicate 40
  OPC_Decode, 251, 55, 212, 2,               // 49441:     decode to SST1W_D_UXTW using decoder 340
                                             // 49441:    }
  9, 7,                                      // 49446:    case 0x9: {
  OPC_CheckPredicate, 40,                    // 49448:     check predicate 40
  OPC_Decode, 252, 55, 212, 2,               // 49450:     decode to SST1W_D_UXTW_SCALED using decoder 340
                                             // 49450:    }
  10, 7,                                     // 49455:    case 0xa: {
  OPC_CheckPredicate, 40,                    // 49457:     check predicate 40
  OPC_Decode, 128, 56, 212, 2,               // 49459:     decode to SST1W_UXTW using decoder 340
                                             // 49459:    }
  11, 7,                                     // 49464:    case 0xb: {
  OPC_CheckPredicate, 40,                    // 49466:     check predicate 40
  OPC_Decode, 129, 56, 212, 2,               // 49468:     decode to SST1W_UXTW_SCALED using decoder 340
                                             // 49468:    }
  12, 7,                                     // 49473:    case 0xc: {
  OPC_CheckPredicate, 40,                    // 49475:     check predicate 40
  OPC_Decode, 231, 55, 212, 2,               // 49477:     decode to SST1D_UXTW using decoder 340
                                             // 49477:    }
  13, 0,                                     // 49482:    case 0xd: {
  OPC_CheckPredicate, 40,                    // 49484:     check predicate 40
  OPC_Decode, 232, 55, 212, 2,               // 49486:     decode to SST1D_UXTW_SCALED using decoder 340
                                             // 49486:    }
                                             // 49486:    } // switch Inst[25:21]
                                             // 49486:   }
  5, 129, 1,                                 // 49491:   case 0x5: {
  OPC_SwitchField, 21, 5,                    // 49494:    switch Inst[25:21] {
  0, 7,                                      // 49497:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 49499:     check predicate 40
  OPC_Decode, 219, 55, 212, 2,               // 49501:     decode to SST1B_D using decoder 340
                                             // 49501:    }
  2, 7,                                      // 49506:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 49508:     check predicate 40
  OPC_Decode, 220, 55, 218, 2,               // 49510:     decode to SST1B_D_IMM using decoder 346
                                             // 49510:    }
  3, 7,                                      // 49515:    case 0x3: {
  OPC_CheckPredicate, 40,                    // 49517:     check predicate 40
  OPC_Decode, 223, 55, 218, 2,               // 49519:     decode to SST1B_S_IMM using decoder 346
                                             // 49519:    }
  4, 7,                                      // 49524:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 49526:     check predicate 40
  OPC_Decode, 233, 55, 212, 2,               // 49528:     decode to SST1H_D using decoder 340
                                             // 49528:    }
  5, 7,                                      // 49533:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 49535:     check predicate 40
  OPC_Decode, 235, 55, 212, 2,               // 49537:     decode to SST1H_D_SCALED using decoder 340
                                             // 49537:    }
  6, 7,                                      // 49542:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 49544:     check predicate 40
  OPC_Decode, 234, 55, 218, 2,               // 49546:     decode to SST1H_D_IMM using decoder 346
                                             // 49546:    }
  7, 7,                                      // 49551:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 49553:     check predicate 40
  OPC_Decode, 240, 55, 218, 2,               // 49555:     decode to SST1H_S_IMM using decoder 346
                                             // 49555:    }
  8, 7,                                      // 49560:    case 0x8: {
  OPC_CheckPredicate, 40,                    // 49562:     check predicate 40
  OPC_Decode, 246, 55, 212, 2,               // 49564:     decode to SST1W_D using decoder 340
                                             // 49564:    }
  9, 7,                                      // 49569:    case 0x9: {
  OPC_CheckPredicate, 40,                    // 49571:     check predicate 40
  OPC_Decode, 248, 55, 212, 2,               // 49573:     decode to SST1W_D_SCALED using decoder 340
                                             // 49573:    }
  10, 7,                                     // 49578:    case 0xa: {
  OPC_CheckPredicate, 40,                    // 49580:     check predicate 40
  OPC_Decode, 247, 55, 218, 2,               // 49582:     decode to SST1W_D_IMM using decoder 346
                                             // 49582:    }
  11, 7,                                     // 49587:    case 0xb: {
  OPC_CheckPredicate, 40,                    // 49589:     check predicate 40
  OPC_Decode, 253, 55, 218, 2,               // 49591:     decode to SST1W_IMM using decoder 346
                                             // 49591:    }
  12, 7,                                     // 49596:    case 0xc: {
  OPC_CheckPredicate, 40,                    // 49598:     check predicate 40
  OPC_Decode, 226, 55, 212, 2,               // 49600:     decode to SST1D using decoder 340
                                             // 49600:    }
  13, 7,                                     // 49605:    case 0xd: {
  OPC_CheckPredicate, 40,                    // 49607:     check predicate 40
  OPC_Decode, 228, 55, 212, 2,               // 49609:     decode to SST1D_SCALED using decoder 340
                                             // 49609:    }
  14, 0,                                     // 49614:    case 0xe: {
  OPC_CheckPredicate, 40,                    // 49616:     check predicate 40
  OPC_Decode, 227, 55, 218, 2,               // 49618:     decode to SST1D_IMM using decoder 346
                                             // 49618:    }
                                             // 49618:    } // switch Inst[25:21]
                                             // 49618:   }
  6, 111,                                    // 49623:   case 0x6: {
  OPC_SwitchField, 21, 5,                    // 49625:    switch Inst[25:21] {
  0, 7,                                      // 49628:    case 0x0: {
  OPC_CheckPredicate, 40,                    // 49630:     check predicate 40
  OPC_Decode, 221, 55, 212, 2,               // 49632:     decode to SST1B_D_SXTW using decoder 340
                                             // 49632:    }
  2, 7,                                      // 49637:    case 0x2: {
  OPC_CheckPredicate, 40,                    // 49639:     check predicate 40
  OPC_Decode, 224, 55, 212, 2,               // 49641:     decode to SST1B_S_SXTW using decoder 340
                                             // 49641:    }
  4, 7,                                      // 49646:    case 0x4: {
  OPC_CheckPredicate, 40,                    // 49648:     check predicate 40
  OPC_Decode, 236, 55, 212, 2,               // 49650:     decode to SST1H_D_SXTW using decoder 340
                                             // 49650:    }
  5, 7,                                      // 49655:    case 0x5: {
  OPC_CheckPredicate, 40,                    // 49657:     check predicate 40
  OPC_Decode, 237, 55, 212, 2,               // 49659:     decode to SST1H_D_SXTW_SCALED using decoder 340
                                             // 49659:    }
  6, 7,                                      // 49664:    case 0x6: {
  OPC_CheckPredicate, 40,                    // 49666:     check predicate 40
  OPC_Decode, 241, 55, 212, 2,               // 49668:     decode to SST1H_S_SXTW using decoder 340
                                             // 49668:    }
  7, 7,                                      // 49673:    case 0x7: {
  OPC_CheckPredicate, 40,                    // 49675:     check predicate 40
  OPC_Decode, 242, 55, 212, 2,               // 49677:     decode to SST1H_S_SXTW_SCALED using decoder 340
                                             // 49677:    }
  8, 7,                                      // 49682:    case 0x8: {
  OPC_CheckPredicate, 40,                    // 49684:     check predicate 40
  OPC_Decode, 249, 55, 212, 2,               // 49686:     decode to SST1W_D_SXTW using decoder 340
                                             // 49686:    }
  9, 7,                                      // 49691:    case 0x9: {
  OPC_CheckPredicate, 40,                    // 49693:     check predicate 40
  OPC_Decode, 250, 55, 212, 2,               // 49695:     decode to SST1W_D_SXTW_SCALED using decoder 340
                                             // 49695:    }
  10, 7,                                     // 49700:    case 0xa: {
  OPC_CheckPredicate, 40,                    // 49702:     check predicate 40
  OPC_Decode, 254, 55, 212, 2,               // 49704:     decode to SST1W_SXTW using decoder 340
                                             // 49704:    }
  11, 7,                                     // 49709:    case 0xb: {
  OPC_CheckPredicate, 40,                    // 49711:     check predicate 40
  OPC_Decode, 255, 55, 212, 2,               // 49713:     decode to SST1W_SXTW_SCALED using decoder 340
                                             // 49713:    }
  12, 7,                                     // 49718:    case 0xc: {
  OPC_CheckPredicate, 40,                    // 49720:     check predicate 40
  OPC_Decode, 229, 55, 212, 2,               // 49722:     decode to SST1D_SXTW using decoder 340
                                             // 49722:    }
  13, 0,                                     // 49727:    case 0xd: {
  OPC_CheckPredicate, 40,                    // 49729:     check predicate 40
  OPC_Decode, 230, 55, 212, 2,               // 49731:     decode to SST1D_SXTW_SCALED using decoder 340
                                             // 49731:    }
                                             // 49731:    } // switch Inst[25:21]
                                             // 49731:   }
  7, 0,                                      // 49736:   case 0x7: {
  OPC_SwitchField, 20, 6,                    // 49738:    switch Inst[25:20] {
  0, 7,                                      // 49741:    case 0x0: {
  OPC_CheckPredicate, 33,                    // 49743:     check predicate 33
  OPC_Decode, 173, 56, 223, 2,               // 49745:     decode to ST1B_IMM using decoder 351
                                             // 49745:    }
  1, 7,                                      // 49750:    case 0x1: {
  OPC_CheckPredicate, 33,                    // 49752:     check predicate 33
  OPC_Decode, 240, 58, 223, 2,               // 49754:     decode to STNT1B_ZRI using decoder 351
                                             // 49754:    }
  2, 7,                                      // 49759:    case 0x2: {
  OPC_CheckPredicate, 33,                    // 49761:     check predicate 33
  OPC_Decode, 172, 56, 223, 2,               // 49763:     decode to ST1B_H_IMM using decoder 351
                                             // 49763:    }
  3, 7,                                      // 49768:    case 0x3: {
  OPC_CheckPredicate, 33,                    // 49770:     check predicate 33
  OPC_Decode, 171, 57, 226, 2,               // 49772:     decode to ST2B_IMM using decoder 354
                                             // 49772:    }
  4, 7,                                      // 49777:    case 0x4: {
  OPC_CheckPredicate, 33,                    // 49779:     check predicate 33
  OPC_Decode, 175, 56, 223, 2,               // 49781:     decode to ST1B_S_IMM using decoder 351
                                             // 49781:    }
  5, 7,                                      // 49786:    case 0x5: {
  OPC_CheckPredicate, 33,                    // 49788:     check predicate 33
  OPC_Decode, 206, 57, 228, 2,               // 49790:     decode to ST3B_IMM using decoder 356
                                             // 49790:    }
  6, 7,                                      // 49795:    case 0x6: {
  OPC_CheckPredicate, 33,                    // 49797:     check predicate 33
  OPC_Decode, 170, 56, 223, 2,               // 49799:     decode to ST1B_D_IMM using decoder 351
                                             // 49799:    }
  7, 7,                                      // 49804:    case 0x7: {
  OPC_CheckPredicate, 33,                    // 49806:     check predicate 33
  OPC_Decode, 238, 57, 230, 2,               // 49808:     decode to ST4B_IMM using decoder 358
                                             // 49808:    }
  9, 7,                                      // 49813:    case 0x9: {
  OPC_CheckPredicate, 33,                    // 49815:     check predicate 33
  OPC_Decode, 135, 59, 223, 2,               // 49817:     decode to STNT1H_ZRI using decoder 351
                                             // 49817:    }
  10, 7,                                     // 49822:    case 0xa: {
  OPC_CheckPredicate, 33,                    // 49824:     check predicate 33
  OPC_Decode, 215, 56, 223, 2,               // 49826:     decode to ST1H_IMM using decoder 351
                                             // 49826:    }
  11, 7,                                     // 49831:    case 0xb: {
  OPC_CheckPredicate, 33,                    // 49833:     check predicate 33
  OPC_Decode, 178, 57, 226, 2,               // 49835:     decode to ST2H_IMM using decoder 354
                                             // 49835:    }
  12, 7,                                     // 49840:    case 0xc: {
  OPC_CheckPredicate, 33,                    // 49842:     check predicate 33
  OPC_Decode, 217, 56, 223, 2,               // 49844:     decode to ST1H_S_IMM using decoder 351
                                             // 49844:    }
  13, 7,                                     // 49849:    case 0xd: {
  OPC_CheckPredicate, 33,                    // 49851:     check predicate 33
  OPC_Decode, 210, 57, 228, 2,               // 49853:     decode to ST3H_IMM using decoder 356
                                             // 49853:    }
  14, 7,                                     // 49858:    case 0xe: {
  OPC_CheckPredicate, 33,                    // 49860:     check predicate 33
  OPC_Decode, 214, 56, 223, 2,               // 49862:     decode to ST1H_D_IMM using decoder 351
                                             // 49862:    }
  15, 7,                                     // 49867:    case 0xf: {
  OPC_CheckPredicate, 33,                    // 49869:     check predicate 33
  OPC_Decode, 128, 58, 230, 2,               // 49871:     decode to ST4H_IMM using decoder 358
                                             // 49871:    }
  16, 7,                                     // 49876:    case 0x10: {
  OPC_CheckPredicate, 71,                    // 49878:     check predicate 71
  OPC_Decode, 151, 57, 223, 2,               // 49880:     decode to ST1W_Q_IMM using decoder 351
                                             // 49880:    }
  17, 7,                                     // 49885:    case 0x11: {
  OPC_CheckPredicate, 33,                    // 49887:     check predicate 33
  OPC_Decode, 147, 59, 223, 2,               // 49889:     decode to STNT1W_ZRI using decoder 351
                                             // 49889:    }
  19, 7,                                     // 49894:    case 0x13: {
  OPC_CheckPredicate, 33,                    // 49896:     check predicate 33
  OPC_Decode, 196, 57, 226, 2,               // 49898:     decode to ST2W_IMM using decoder 354
                                             // 49898:    }
  20, 7,                                     // 49903:    case 0x14: {
  OPC_CheckPredicate, 33,                    // 49905:     check predicate 33
  OPC_Decode, 149, 57, 223, 2,               // 49907:     decode to ST1W_IMM using decoder 351
                                             // 49907:    }
  21, 7,                                     // 49912:    case 0x15: {
  OPC_CheckPredicate, 33,                    // 49914:     check predicate 33
  OPC_Decode, 228, 57, 228, 2,               // 49916:     decode to ST3W_IMM using decoder 356
                                             // 49916:    }
  22, 7,                                     // 49921:    case 0x16: {
  OPC_CheckPredicate, 33,                    // 49923:     check predicate 33
  OPC_Decode, 148, 57, 223, 2,               // 49925:     decode to ST1W_D_IMM using decoder 351
                                             // 49925:    }
  23, 7,                                     // 49930:    case 0x17: {
  OPC_CheckPredicate, 33,                    // 49932:     check predicate 33
  OPC_Decode, 132, 58, 230, 2,               // 49934:     decode to ST4W_IMM using decoder 358
                                             // 49934:    }
  25, 7,                                     // 49939:    case 0x19: {
  OPC_CheckPredicate, 33,                    // 49941:     check predicate 33
  OPC_Decode, 252, 58, 223, 2,               // 49943:     decode to STNT1D_ZRI using decoder 351
                                             // 49943:    }
  27, 7,                                     // 49948:    case 0x1b: {
  OPC_CheckPredicate, 33,                    // 49950:     check predicate 33
  OPC_Decode, 173, 57, 226, 2,               // 49952:     decode to ST2D_IMM using decoder 354
                                             // 49952:    }
  28, 7,                                     // 49957:    case 0x1c: {
  OPC_CheckPredicate, 71,                    // 49959:     check predicate 71
  OPC_Decode, 187, 56, 223, 2,               // 49961:     decode to ST1D_Q_IMM using decoder 351
                                             // 49961:    }
  29, 7,                                     // 49966:    case 0x1d: {
  OPC_CheckPredicate, 33,                    // 49968:     check predicate 33
  OPC_Decode, 208, 57, 228, 2,               // 49970:     decode to ST3D_IMM using decoder 356
                                             // 49970:    }
  30, 7,                                     // 49975:    case 0x1e: {
  OPC_CheckPredicate, 33,                    // 49977:     check predicate 33
  OPC_Decode, 185, 56, 223, 2,               // 49979:     decode to ST1D_IMM using decoder 351
                                             // 49979:    }
  31, 0,                                     // 49984:    case 0x1f: {
  OPC_CheckPredicate, 33,                    // 49986:     check predicate 33
  OPC_Decode, 240, 57, 230, 2,               // 49988:     decode to ST4D_IMM using decoder 358
                                             // 49988:    }
                                             // 49988:    } // switch Inst[25:20]
                                             // 49988:   }
                                             // 49988:   } // switch Inst[15:13]
                                             // 49988:  }
                                             // 49988:  } // switch Inst[31:29]
                                             // 49988: }
  2, 151, 14,                                // 49993: case 0x2: {
  OPC_SwitchField, 29, 3,                    // 49996:  switch Inst[31:29] {
  0, 253, 1,                                 // 49999:  case 0x0: {
  OPC_SwitchField, 24, 2,                    // 50002:   switch Inst[25:24] {
  0, 205, 1,                                 // 50005:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 50008:    switch Inst[23:21] {
  0, 17,                                     // 50011:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 50013:     switch Inst[15] {
  0, 5,                                      // 50016:     case 0x0: {
  OPC_Decode, 241, 59, 231, 2,               // 50018:      decode to STXRB using decoder 359
                                             // 50018:     }
  1, 0,                                      // 50023:     case 0x1: {
  OPC_Decode, 221, 58, 231, 2,               // 50025:      decode to STLXRB using decoder 359
                                             // 50025:     }
                                             // 50025:     } // switch Inst[15]
                                             // 50025:    }
  1, 21,                                     // 50030:    case 0x1: {
  OPC_SwitchField, 10, 6,                    // 50032:     switch Inst[15:10] {
  31, 7,                                     // 50035:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50037:      check predicate 72
  OPC_Decode, 219, 16, 232, 2,               // 50039:      decode to CASPW using decoder 360
                                             // 50039:     }
  63, 0,                                     // 50044:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50046:      check predicate 72
  OPC_Decode, 216, 16, 232, 2,               // 50048:      decode to CASPLW using decoder 360
                                             // 50048:     }
                                             // 50048:     } // switch Inst[15:10]
                                             // 50048:    }
  2, 27,                                     // 50053:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 50055:     switch Inst[15] {
  0, 10,                                     // 50058:     case 0x0: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50060:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 135, 41, 231, 2,               // 50065:      decode to LDXRB using decoder 359
                                             // 50065:     }
  1, 0,                                      // 50070:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50072:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 183, 37, 231, 2,               // 50077:      decode to LDAXRB using decoder 359
                                             // 50077:     }
                                             // 50077:     } // switch Inst[15]
                                             // 50077:    }
  3, 21,                                     // 50082:    case 0x3: {
  OPC_SwitchField, 10, 6,                    // 50084:     switch Inst[15:10] {
  31, 7,                                     // 50087:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50089:      check predicate 72
  OPC_Decode, 213, 16, 232, 2,               // 50091:      decode to CASPAW using decoder 360
                                             // 50091:     }
  63, 0,                                     // 50096:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50098:      check predicate 72
  OPC_Decode, 210, 16, 232, 2,               // 50100:      decode to CASPALW using decoder 360
                                             // 50100:     }
                                             // 50100:     } // switch Inst[15:10]
                                             // 50100:    }
  4, 29,                                     // 50105:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 50107:     switch Inst[15] {
  0, 12,                                     // 50110:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 50112:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 50114:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 197, 58, 231, 2,               // 50119:      decode to STLLRB using decoder 359
                                             // 50119:     }
  1, 0,                                      // 50124:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50126:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 202, 58, 231, 2,               // 50131:      decode to STLRB using decoder 359
                                             // 50131:     }
                                             // 50131:     } // switch Inst[15]
                                             // 50131:    }
  5, 21,                                     // 50136:    case 0x5: {
  OPC_SwitchField, 10, 6,                    // 50138:     switch Inst[15:10] {
  31, 7,                                     // 50141:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50143:      check predicate 72
  OPC_Decode, 202, 16, 233, 2,               // 50145:      decode to CASB using decoder 361
                                             // 50145:     }
  63, 0,                                     // 50150:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50152:      check predicate 72
  OPC_Decode, 204, 16, 233, 2,               // 50154:      decode to CASLB using decoder 361
                                             // 50154:     }
                                             // 50154:     } // switch Inst[15:10]
                                             // 50154:    }
  6, 29,                                     // 50159:    case 0x6: {
  OPC_SwitchField, 15, 1,                    // 50161:     switch Inst[15] {
  0, 12,                                     // 50164:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 50166:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 50168:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 197, 38, 231, 2,               // 50173:      decode to LDLARB using decoder 359
                                             // 50173:     }
  1, 0,                                      // 50178:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50180:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 175, 37, 231, 2,               // 50185:      decode to LDARB using decoder 359
                                             // 50185:     }
                                             // 50185:     } // switch Inst[15]
                                             // 50185:    }
  7, 0,                                      // 50190:    case 0x7: {
  OPC_SwitchField, 10, 6,                    // 50192:     switch Inst[15:10] {
  31, 7,                                     // 50195:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50197:      check predicate 72
  OPC_Decode, 192, 16, 233, 2,               // 50199:      decode to CASAB using decoder 361
                                             // 50199:     }
  63, 0,                                     // 50204:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50206:      check predicate 72
  OPC_Decode, 194, 16, 233, 2,               // 50208:      decode to CASALB using decoder 361
                                             // 50208:     }
                                             // 50208:     } // switch Inst[15:10]
                                             // 50208:    }
                                             // 50208:    } // switch Inst[23:21]
                                             // 50208:   }
  2, 17,                                     // 50213:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 50215:    switch Inst[21] {
  0, 5,                                      // 50218:    case 0x0: {
  OPC_Decode, 161, 14, 234, 2,               // 50220:     decode to ANDWrs using decoder 362
                                             // 50220:    }
  1, 0,                                      // 50225:    case 0x1: {
  OPC_Decode, 136, 16, 234, 2,               // 50227:     decode to BICWrs using decoder 362
                                             // 50227:    }
                                             // 50227:    } // switch Inst[21]
                                             // 50227:   }
  3, 0,                                      // 50232:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 50234:    switch Inst[21] {
  0, 5,                                      // 50237:    case 0x0: {
  OPC_Decode, 193, 13, 234, 2,               // 50239:     decode to ADDWrs using decoder 362
                                             // 50239:    }
  1, 0,                                      // 50244:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 50246:     check Inst[23:22] == 0x0
  OPC_Decode, 194, 13, 235, 2,               // 50250:     decode to ADDWrx using decoder 363
                                             // 50250:    }
                                             // 50250:    } // switch Inst[21]
                                             // 50250:   }
                                             // 50250:   } // switch Inst[25:24]
                                             // 50250:  }
  1, 111,                                    // 50255:  case 0x1: {
  OPC_SwitchField, 24, 2,                    // 50257:   switch Inst[25:24] {
  0, 31,                                     // 50260:   case 0x0: {
  OPC_SwitchField, 22, 2,                    // 50262:    switch Inst[23:22] {
  0, 5,                                      // 50265:    case 0x0: {
  OPC_Decode, 230, 58, 236, 2,               // 50267:     decode to STNPWi using decoder 364
                                             // 50267:    }
  1, 5,                                      // 50272:    case 0x1: {
  OPC_Decode, 220, 38, 236, 2,               // 50274:     decode to LDNPWi using decoder 364
                                             // 50274:    }
  2, 5,                                      // 50279:    case 0x2: {
  OPC_Decode, 161, 59, 236, 2,               // 50281:     decode to STPWpost using decoder 364
                                             // 50281:    }
  3, 0,                                      // 50286:    case 0x3: {
  OPC_Decode, 159, 39, 236, 2,               // 50288:     decode to LDPWpost using decoder 364
                                             // 50288:    }
                                             // 50288:    } // switch Inst[23:22]
                                             // 50288:   }
  1, 31,                                     // 50293:   case 0x1: {
  OPC_SwitchField, 22, 2,                    // 50295:    switch Inst[23:22] {
  0, 5,                                      // 50298:    case 0x0: {
  OPC_Decode, 160, 59, 236, 2,               // 50300:     decode to STPWi using decoder 364
                                             // 50300:    }
  1, 5,                                      // 50305:    case 0x1: {
  OPC_Decode, 158, 39, 236, 2,               // 50307:     decode to LDPWi using decoder 364
                                             // 50307:    }
  2, 5,                                      // 50312:    case 0x2: {
  OPC_Decode, 162, 59, 236, 2,               // 50314:     decode to STPWpre using decoder 364
                                             // 50314:    }
  3, 0,                                      // 50319:    case 0x3: {
  OPC_Decode, 160, 39, 236, 2,               // 50321:     decode to LDPWpre using decoder 364
                                             // 50321:    }
                                             // 50321:    } // switch Inst[23:22]
                                             // 50321:   }
  2, 17,                                     // 50326:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 50328:    switch Inst[21] {
  0, 5,                                      // 50331:    case 0x0: {
  OPC_Decode, 205, 43, 234, 2,               // 50333:     decode to ORRWrs using decoder 362
                                             // 50333:    }
  1, 0,                                      // 50338:    case 0x1: {
  OPC_Decode, 194, 43, 234, 2,               // 50340:     decode to ORNWrs using decoder 362
                                             // 50340:    }
                                             // 50340:    } // switch Inst[21]
                                             // 50340:   }
  3, 0,                                      // 50345:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 50347:    switch Inst[21] {
  0, 5,                                      // 50350:    case 0x0: {
  OPC_Decode, 178, 13, 234, 2,               // 50352:     decode to ADDSWrs using decoder 362
                                             // 50352:    }
  1, 0,                                      // 50357:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 50359:     check Inst[23:22] == 0x0
  OPC_Decode, 179, 13, 235, 2,               // 50363:     decode to ADDSWrx using decoder 363
                                             // 50363:    }
                                             // 50363:    } // switch Inst[21]
                                             // 50363:   }
                                             // 50363:   } // switch Inst[25:24]
                                             // 50363:  }
  2, 196, 2,                                 // 50368:  case 0x2: {
  OPC_SwitchField, 24, 2,                    // 50371:   switch Inst[25:24] {
  0, 205, 1,                                 // 50374:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 50377:    switch Inst[23:21] {
  0, 17,                                     // 50380:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 50382:     switch Inst[15] {
  0, 5,                                      // 50385:     case 0x0: {
  OPC_Decode, 242, 59, 231, 2,               // 50387:      decode to STXRH using decoder 359
                                             // 50387:     }
  1, 0,                                      // 50392:     case 0x1: {
  OPC_Decode, 222, 58, 231, 2,               // 50394:      decode to STLXRH using decoder 359
                                             // 50394:     }
                                             // 50394:     } // switch Inst[15]
                                             // 50394:    }
  1, 21,                                     // 50399:    case 0x1: {
  OPC_SwitchField, 10, 6,                    // 50401:     switch Inst[15:10] {
  31, 7,                                     // 50404:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50406:      check predicate 72
  OPC_Decode, 220, 16, 237, 2,               // 50408:      decode to CASPX using decoder 365
                                             // 50408:     }
  63, 0,                                     // 50413:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50415:      check predicate 72
  OPC_Decode, 217, 16, 237, 2,               // 50417:      decode to CASPLX using decoder 365
                                             // 50417:     }
                                             // 50417:     } // switch Inst[15:10]
                                             // 50417:    }
  2, 27,                                     // 50422:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 50424:     switch Inst[15] {
  0, 10,                                     // 50427:     case 0x0: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50429:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 136, 41, 231, 2,               // 50434:      decode to LDXRH using decoder 359
                                             // 50434:     }
  1, 0,                                      // 50439:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50441:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 184, 37, 231, 2,               // 50446:      decode to LDAXRH using decoder 359
                                             // 50446:     }
                                             // 50446:     } // switch Inst[15]
                                             // 50446:    }
  3, 21,                                     // 50451:    case 0x3: {
  OPC_SwitchField, 10, 6,                    // 50453:     switch Inst[15:10] {
  31, 7,                                     // 50456:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50458:      check predicate 72
  OPC_Decode, 214, 16, 237, 2,               // 50460:      decode to CASPAX using decoder 365
                                             // 50460:     }
  63, 0,                                     // 50465:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50467:      check predicate 72
  OPC_Decode, 211, 16, 237, 2,               // 50469:      decode to CASPALX using decoder 365
                                             // 50469:     }
                                             // 50469:     } // switch Inst[15:10]
                                             // 50469:    }
  4, 29,                                     // 50474:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 50476:     switch Inst[15] {
  0, 12,                                     // 50479:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 50481:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 50483:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 198, 58, 231, 2,               // 50488:      decode to STLLRH using decoder 359
                                             // 50488:     }
  1, 0,                                      // 50493:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50495:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 203, 58, 231, 2,               // 50500:      decode to STLRH using decoder 359
                                             // 50500:     }
                                             // 50500:     } // switch Inst[15]
                                             // 50500:    }
  5, 21,                                     // 50505:    case 0x5: {
  OPC_SwitchField, 10, 6,                    // 50507:     switch Inst[15:10] {
  31, 7,                                     // 50510:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50512:      check predicate 72
  OPC_Decode, 203, 16, 233, 2,               // 50514:      decode to CASH using decoder 361
                                             // 50514:     }
  63, 0,                                     // 50519:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50521:      check predicate 72
  OPC_Decode, 205, 16, 233, 2,               // 50523:      decode to CASLH using decoder 361
                                             // 50523:     }
                                             // 50523:     } // switch Inst[15:10]
                                             // 50523:    }
  6, 29,                                     // 50528:    case 0x6: {
  OPC_SwitchField, 15, 1,                    // 50530:     switch Inst[15] {
  0, 12,                                     // 50533:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 50535:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 50537:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 198, 38, 231, 2,               // 50542:      decode to LDLARH using decoder 359
                                             // 50542:     }
  1, 0,                                      // 50547:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50549:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 176, 37, 231, 2,               // 50554:      decode to LDARH using decoder 359
                                             // 50554:     }
                                             // 50554:     } // switch Inst[15]
                                             // 50554:    }
  7, 0,                                      // 50559:    case 0x7: {
  OPC_SwitchField, 10, 6,                    // 50561:     switch Inst[15:10] {
  31, 7,                                     // 50564:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50566:      check predicate 72
  OPC_Decode, 193, 16, 233, 2,               // 50568:      decode to CASAH using decoder 361
                                             // 50568:     }
  63, 0,                                     // 50573:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50575:      check predicate 72
  OPC_Decode, 195, 16, 233, 2,               // 50577:      decode to CASALH using decoder 361
                                             // 50577:     }
                                             // 50577:     } // switch Inst[15:10]
                                             // 50577:    }
                                             // 50577:    } // switch Inst[23:21]
                                             // 50577:   }
  1, 69,                                     // 50582:   case 0x1: {
  OPC_SwitchField, 15, 1,                    // 50584:    switch Inst[15] {
  0, 31,                                     // 50587:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 50589:     switch Inst[23:21] {
  4, 12,                                     // 50592:     case 0x4: {
  OPC_CheckPredicate, 74,                    // 50594:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 50596:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 218, 16, 237, 2,               // 50601:      decode to CASPTX using decoder 365
                                             // 50601:     }
  6, 0,                                      // 50606:     case 0x6: {
  OPC_CheckPredicate, 74,                    // 50608:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 50610:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 212, 16, 237, 2,               // 50615:      decode to CASPATX using decoder 365
                                             // 50615:     }
                                             // 50615:     } // switch Inst[23:21]
                                             // 50615:    }
  1, 0,                                      // 50620:    case 0x1: {
  OPC_SwitchField, 21, 3,                    // 50622:     switch Inst[23:21] {
  4, 12,                                     // 50625:     case 0x4: {
  OPC_CheckPredicate, 74,                    // 50627:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 50629:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 215, 16, 237, 2,               // 50634:      decode to CASPLTX using decoder 365
                                             // 50634:     }
  6, 0,                                      // 50639:     case 0x6: {
  OPC_CheckPredicate, 74,                    // 50641:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 50643:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 209, 16, 237, 2,               // 50648:      decode to CASPALTX using decoder 365
                                             // 50648:     }
                                             // 50648:     } // switch Inst[23:21]
                                             // 50648:    }
                                             // 50648:    } // switch Inst[15]
                                             // 50648:   }
  2, 17,                                     // 50653:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 50655:    switch Inst[21] {
  0, 5,                                      // 50658:    case 0x0: {
  OPC_Decode, 132, 21, 234, 2,               // 50660:     decode to EORWrs using decoder 362
                                             // 50660:    }
  1, 0,                                      // 50665:    case 0x1: {
  OPC_Decode, 238, 20, 234, 2,               // 50667:     decode to EONWrs using decoder 362
                                             // 50667:    }
                                             // 50667:    } // switch Inst[21]
                                             // 50667:   }
  3, 0,                                      // 50672:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 50674:    switch Inst[21] {
  0, 5,                                      // 50677:    case 0x0: {
  OPC_Decode, 160, 60, 234, 2,               // 50679:     decode to SUBWrs using decoder 362
                                             // 50679:    }
  1, 0,                                      // 50684:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 50686:     check Inst[23:22] == 0x0
  OPC_Decode, 161, 60, 235, 2,               // 50690:     decode to SUBWrx using decoder 363
                                             // 50690:    }
                                             // 50690:    } // switch Inst[21]
                                             // 50690:   }
                                             // 50690:   } // switch Inst[25:24]
                                             // 50690:  }
  3, 103,                                    // 50695:  case 0x3: {
  OPC_SwitchField, 24, 2,                    // 50697:   switch Inst[25:24] {
  0, 19,                                     // 50700:   case 0x0: {
  OPC_SwitchField, 22, 2,                    // 50702:    switch Inst[23:22] {
  2, 7,                                      // 50705:    case 0x2: {
  OPC_CheckPredicate, 75,                    // 50707:     check predicate 75
  OPC_Decode, 188, 58, 236, 2,               // 50709:     decode to STGPpost using decoder 364
                                             // 50709:    }
  3, 0,                                      // 50714:    case 0x3: {
  OPC_Decode, 153, 39, 236, 2,               // 50716:     decode to LDPSWpost using decoder 364
                                             // 50716:    }
                                             // 50716:    } // switch Inst[23:22]
                                             // 50716:   }
  1, 35,                                     // 50721:   case 0x1: {
  OPC_SwitchField, 22, 2,                    // 50723:    switch Inst[23:22] {
  0, 7,                                      // 50726:    case 0x0: {
  OPC_CheckPredicate, 75,                    // 50728:     check predicate 75
  OPC_Decode, 186, 58, 236, 2,               // 50730:     decode to STGPi using decoder 364
                                             // 50730:    }
  1, 5,                                      // 50735:    case 0x1: {
  OPC_Decode, 152, 39, 236, 2,               // 50737:     decode to LDPSWi using decoder 364
                                             // 50737:    }
  2, 7,                                      // 50742:    case 0x2: {
  OPC_CheckPredicate, 75,                    // 50744:     check predicate 75
  OPC_Decode, 189, 58, 236, 2,               // 50746:     decode to STGPpre using decoder 364
                                             // 50746:    }
  3, 0,                                      // 50751:    case 0x3: {
  OPC_Decode, 154, 39, 236, 2,               // 50753:     decode to LDPSWpre using decoder 364
                                             // 50753:    }
                                             // 50753:    } // switch Inst[23:22]
                                             // 50753:   }
  2, 17,                                     // 50758:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 50760:    switch Inst[21] {
  0, 5,                                      // 50763:    case 0x0: {
  OPC_Decode, 152, 14, 234, 2,               // 50765:     decode to ANDSWrs using decoder 362
                                             // 50765:    }
  1, 0,                                      // 50770:    case 0x1: {
  OPC_Decode, 133, 16, 234, 2,               // 50772:     decode to BICSWrs using decoder 362
                                             // 50772:    }
                                             // 50772:    } // switch Inst[21]
                                             // 50772:   }
  3, 0,                                      // 50777:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 50779:    switch Inst[21] {
  0, 5,                                      // 50782:    case 0x0: {
  OPC_Decode, 153, 60, 234, 2,               // 50784:     decode to SUBSWrs using decoder 362
                                             // 50784:    }
  1, 0,                                      // 50789:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 50791:     check Inst[23:22] == 0x0
  OPC_Decode, 154, 60, 235, 2,               // 50795:     decode to SUBSWrx using decoder 363
                                             // 50795:    }
                                             // 50795:    } // switch Inst[21]
                                             // 50795:   }
                                             // 50795:   } // switch Inst[25:24]
                                             // 50795:  }
  4, 209, 2,                                 // 50800:  case 0x4: {
  OPC_SwitchField, 24, 2,                    // 50803:   switch Inst[25:24] {
  0, 207, 1,                                 // 50806:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 50809:    switch Inst[23:21] {
  0, 17,                                     // 50812:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 50814:     switch Inst[15] {
  0, 5,                                      // 50817:     case 0x0: {
  OPC_Decode, 243, 59, 231, 2,               // 50819:      decode to STXRW using decoder 359
                                             // 50819:     }
  1, 0,                                      // 50824:     case 0x1: {
  OPC_Decode, 223, 58, 231, 2,               // 50826:      decode to STLXRW using decoder 359
                                             // 50826:     }
                                             // 50826:     } // switch Inst[15]
                                             // 50826:    }
  1, 17,                                     // 50831:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 50833:     switch Inst[15] {
  0, 5,                                      // 50836:     case 0x0: {
  OPC_Decode, 239, 59, 231, 2,               // 50838:      decode to STXPW using decoder 359
                                             // 50838:     }
  1, 0,                                      // 50843:     case 0x1: {
  OPC_Decode, 219, 58, 231, 2,               // 50845:      decode to STLXPW using decoder 359
                                             // 50845:     }
                                             // 50845:     } // switch Inst[15]
                                             // 50845:    }
  2, 27,                                     // 50850:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 50852:     switch Inst[15] {
  0, 10,                                     // 50855:     case 0x0: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50857:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 137, 41, 231, 2,               // 50862:      decode to LDXRW using decoder 359
                                             // 50862:     }
  1, 0,                                      // 50867:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50869:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 185, 37, 231, 2,               // 50874:      decode to LDAXRW using decoder 359
                                             // 50874:     }
                                             // 50874:     } // switch Inst[15]
                                             // 50874:    }
  3, 27,                                     // 50879:    case 0x3: {
  OPC_SwitchField, 15, 1,                    // 50881:     switch Inst[15] {
  0, 10,                                     // 50884:     case 0x0: {
  OPC_SoftFail, 0, 128, 128, 124,            // 50886:      softfail pos=0x00000000 neg=0x001f0000
  OPC_Decode, 133, 41, 231, 2,               // 50891:      decode to LDXPW using decoder 359
                                             // 50891:     }
  1, 0,                                      // 50896:     case 0x1: {
  OPC_SoftFail, 0, 128, 128, 124,            // 50898:      softfail pos=0x00000000 neg=0x001f0000
  OPC_Decode, 181, 37, 231, 2,               // 50903:      decode to LDAXPW using decoder 359
                                             // 50903:     }
                                             // 50903:     } // switch Inst[15]
                                             // 50903:    }
  4, 29,                                     // 50908:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 50910:     switch Inst[15] {
  0, 12,                                     // 50913:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 50915:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 50917:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 199, 58, 231, 2,               // 50922:      decode to STLLRW using decoder 359
                                             // 50922:     }
  1, 0,                                      // 50927:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50929:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 204, 58, 231, 2,               // 50934:      decode to STLRW using decoder 359
                                             // 50934:     }
                                             // 50934:     } // switch Inst[15]
                                             // 50934:    }
  5, 21,                                     // 50939:    case 0x5: {
  OPC_SwitchField, 10, 6,                    // 50941:     switch Inst[15:10] {
  31, 7,                                     // 50944:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 50946:      check predicate 72
  OPC_Decode, 222, 16, 233, 2,               // 50948:      decode to CASW using decoder 361
                                             // 50948:     }
  63, 0,                                     // 50953:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 50955:      check predicate 72
  OPC_Decode, 207, 16, 233, 2,               // 50957:      decode to CASLW using decoder 361
                                             // 50957:     }
                                             // 50957:     } // switch Inst[15:10]
                                             // 50957:    }
  6, 29,                                     // 50962:    case 0x6: {
  OPC_SwitchField, 15, 1,                    // 50964:     switch Inst[15] {
  0, 12,                                     // 50967:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 50969:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 50971:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 199, 38, 231, 2,               // 50976:      decode to LDLARW using decoder 359
                                             // 50976:     }
  1, 0,                                      // 50981:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 50983:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 177, 37, 231, 2,               // 50988:      decode to LDARW using decoder 359
                                             // 50988:     }
                                             // 50988:     } // switch Inst[15]
                                             // 50988:    }
  7, 0,                                      // 50993:    case 0x7: {
  OPC_SwitchField, 10, 6,                    // 50995:     switch Inst[15:10] {
  31, 7,                                     // 50998:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 51000:      check predicate 72
  OPC_Decode, 200, 16, 233, 2,               // 51002:      decode to CASAW using decoder 361
                                             // 51002:     }
  63, 0,                                     // 51007:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 51009:      check predicate 72
  OPC_Decode, 197, 16, 233, 2,               // 51011:      decode to CASALW using decoder 361
                                             // 51011:     }
                                             // 51011:     } // switch Inst[15:10]
                                             // 51011:    }
                                             // 51011:    } // switch Inst[23:21]
                                             // 51011:   }
  1, 69,                                     // 51016:   case 0x1: {
  OPC_SwitchField, 15, 1,                    // 51018:    switch Inst[15] {
  0, 31,                                     // 51021:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 51023:     switch Inst[23:21] {
  0, 12,                                     // 51026:     case 0x0: {
  OPC_CheckPredicate, 74,                    // 51028:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51030:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 228, 59, 238, 2,               // 51035:      decode to STTXRWr using decoder 366
                                             // 51035:     }
  2, 0,                                      // 51040:     case 0x2: {
  OPC_CheckPredicate, 74,                    // 51042:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 125,            // 51044:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 213, 40, 239, 2,               // 51049:      decode to LDTXRWr using decoder 367
                                             // 51049:     }
                                             // 51049:     } // switch Inst[23:21]
                                             // 51049:    }
  1, 0,                                      // 51054:    case 0x1: {
  OPC_SwitchField, 21, 3,                    // 51056:     switch Inst[23:21] {
  0, 12,                                     // 51059:     case 0x0: {
  OPC_CheckPredicate, 74,                    // 51061:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51063:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 208, 58, 238, 2,               // 51068:      decode to STLTXRW using decoder 366
                                             // 51068:     }
  2, 0,                                      // 51073:     case 0x2: {
  OPC_CheckPredicate, 74,                    // 51075:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 125,            // 51077:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 179, 37, 239, 2,               // 51082:      decode to LDATXRW using decoder 367
                                             // 51082:     }
                                             // 51082:     } // switch Inst[23:21]
                                             // 51082:    }
                                             // 51082:    } // switch Inst[15]
                                             // 51082:   }
  2, 17,                                     // 51087:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 51089:    switch Inst[21] {
  0, 5,                                      // 51092:    case 0x0: {
  OPC_Decode, 163, 14, 234, 2,               // 51094:     decode to ANDXrs using decoder 362
                                             // 51094:    }
  1, 0,                                      // 51099:    case 0x1: {
  OPC_Decode, 137, 16, 234, 2,               // 51101:     decode to BICXrs using decoder 362
                                             // 51101:    }
                                             // 51101:    } // switch Inst[21]
                                             // 51101:   }
  3, 0,                                      // 51106:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 51108:    switch Inst[21] {
  0, 5,                                      // 51111:    case 0x0: {
  OPC_Decode, 196, 13, 234, 2,               // 51113:     decode to ADDXrs using decoder 362
                                             // 51113:    }
  1, 0,                                      // 51118:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 51120:     check Inst[23:22] == 0x0
  OPC_Scope, 9,                              // 51124:     try {
  OPC_CheckField, 13, 2, 3,                  // 51126:      check Inst[14:13] == 0x3
  OPC_Decode, 198, 13, 235, 2,               // 51130:      decode to ADDXrx64 using decoder 363
                                             // 51130:     } else try {
  OPC_Decode, 197, 13, 235, 2,               // 51135:      decode to ADDXrx using decoder 363
                                             // 51135:     }
                                             // 51135:    }
                                             // 51135:    } // switch Inst[21]
                                             // 51135:   }
                                             // 51135:   } // switch Inst[25:24]
                                             // 51135:  }
  5, 122,                                    // 51140:  case 0x5: {
  OPC_SwitchField, 24, 2,                    // 51142:   switch Inst[25:24] {
  0, 31,                                     // 51145:   case 0x0: {
  OPC_SwitchField, 22, 2,                    // 51147:    switch Inst[23:22] {
  0, 5,                                      // 51150:    case 0x0: {
  OPC_Decode, 231, 58, 236, 2,               // 51152:     decode to STNPXi using decoder 364
                                             // 51152:    }
  1, 5,                                      // 51157:    case 0x1: {
  OPC_Decode, 221, 38, 236, 2,               // 51159:     decode to LDNPXi using decoder 364
                                             // 51159:    }
  2, 5,                                      // 51164:    case 0x2: {
  OPC_Decode, 164, 59, 236, 2,               // 51166:     decode to STPXpost using decoder 364
                                             // 51166:    }
  3, 0,                                      // 51171:    case 0x3: {
  OPC_Decode, 162, 39, 236, 2,               // 51173:     decode to LDPXpost using decoder 364
                                             // 51173:    }
                                             // 51173:    } // switch Inst[23:22]
                                             // 51173:   }
  1, 31,                                     // 51178:   case 0x1: {
  OPC_SwitchField, 22, 2,                    // 51180:    switch Inst[23:22] {
  0, 5,                                      // 51183:    case 0x0: {
  OPC_Decode, 163, 59, 236, 2,               // 51185:     decode to STPXi using decoder 364
                                             // 51185:    }
  1, 5,                                      // 51190:    case 0x1: {
  OPC_Decode, 161, 39, 236, 2,               // 51192:     decode to LDPXi using decoder 364
                                             // 51192:    }
  2, 5,                                      // 51197:    case 0x2: {
  OPC_Decode, 165, 59, 236, 2,               // 51199:     decode to STPXpre using decoder 364
                                             // 51199:    }
  3, 0,                                      // 51204:    case 0x3: {
  OPC_Decode, 163, 39, 236, 2,               // 51206:     decode to LDPXpre using decoder 364
                                             // 51206:    }
                                             // 51206:    } // switch Inst[23:22]
                                             // 51206:   }
  2, 17,                                     // 51211:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 51213:    switch Inst[21] {
  0, 5,                                      // 51216:    case 0x0: {
  OPC_Decode, 207, 43, 234, 2,               // 51218:     decode to ORRXrs using decoder 362
                                             // 51218:    }
  1, 0,                                      // 51223:    case 0x1: {
  OPC_Decode, 195, 43, 234, 2,               // 51225:     decode to ORNXrs using decoder 362
                                             // 51225:    }
                                             // 51225:    } // switch Inst[21]
                                             // 51225:   }
  3, 0,                                      // 51230:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 51232:    switch Inst[21] {
  0, 5,                                      // 51235:    case 0x0: {
  OPC_Decode, 181, 13, 234, 2,               // 51237:     decode to ADDSXrs using decoder 362
                                             // 51237:    }
  1, 0,                                      // 51242:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 51244:     check Inst[23:22] == 0x0
  OPC_Scope, 9,                              // 51248:     try {
  OPC_CheckField, 13, 2, 3,                  // 51250:      check Inst[14:13] == 0x3
  OPC_Decode, 183, 13, 235, 2,               // 51254:      decode to ADDSXrx64 using decoder 363
                                             // 51254:     } else try {
  OPC_Decode, 182, 13, 235, 2,               // 51259:      decode to ADDSXrx using decoder 363
                                             // 51259:     }
                                             // 51259:    }
                                             // 51259:    } // switch Inst[21]
                                             // 51259:   }
                                             // 51259:   } // switch Inst[25:24]
                                             // 51259:  }
  6, 148, 3,                                 // 51264:  case 0x6: {
  OPC_SwitchField, 24, 2,                    // 51267:   switch Inst[25:24] {
  0, 207, 1,                                 // 51270:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 51273:    switch Inst[23:21] {
  0, 17,                                     // 51276:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 51278:     switch Inst[15] {
  0, 5,                                      // 51281:     case 0x0: {
  OPC_Decode, 244, 59, 231, 2,               // 51283:      decode to STXRX using decoder 359
                                             // 51283:     }
  1, 0,                                      // 51288:     case 0x1: {
  OPC_Decode, 224, 58, 231, 2,               // 51290:      decode to STLXRX using decoder 359
                                             // 51290:     }
                                             // 51290:     } // switch Inst[15]
                                             // 51290:    }
  1, 17,                                     // 51295:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 51297:     switch Inst[15] {
  0, 5,                                      // 51300:     case 0x0: {
  OPC_Decode, 240, 59, 231, 2,               // 51302:      decode to STXPX using decoder 359
                                             // 51302:     }
  1, 0,                                      // 51307:     case 0x1: {
  OPC_Decode, 220, 58, 231, 2,               // 51309:      decode to STLXPX using decoder 359
                                             // 51309:     }
                                             // 51309:     } // switch Inst[15]
                                             // 51309:    }
  2, 27,                                     // 51314:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 51316:     switch Inst[15] {
  0, 10,                                     // 51319:     case 0x0: {
  OPC_SoftFail, 0, 128, 248, 125,            // 51321:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 138, 41, 231, 2,               // 51326:      decode to LDXRX using decoder 359
                                             // 51326:     }
  1, 0,                                      // 51331:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 51333:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 186, 37, 231, 2,               // 51338:      decode to LDAXRX using decoder 359
                                             // 51338:     }
                                             // 51338:     } // switch Inst[15]
                                             // 51338:    }
  3, 27,                                     // 51343:    case 0x3: {
  OPC_SwitchField, 15, 1,                    // 51345:     switch Inst[15] {
  0, 10,                                     // 51348:     case 0x0: {
  OPC_SoftFail, 0, 128, 128, 124,            // 51350:      softfail pos=0x00000000 neg=0x001f0000
  OPC_Decode, 134, 41, 231, 2,               // 51355:      decode to LDXPX using decoder 359
                                             // 51355:     }
  1, 0,                                      // 51360:     case 0x1: {
  OPC_SoftFail, 0, 128, 128, 124,            // 51362:      softfail pos=0x00000000 neg=0x001f0000
  OPC_Decode, 182, 37, 231, 2,               // 51367:      decode to LDAXPX using decoder 359
                                             // 51367:     }
                                             // 51367:     } // switch Inst[15]
                                             // 51367:    }
  4, 29,                                     // 51372:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 51374:     switch Inst[15] {
  0, 12,                                     // 51377:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 51379:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 51381:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 200, 58, 231, 2,               // 51386:      decode to STLLRX using decoder 359
                                             // 51386:     }
  1, 0,                                      // 51391:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 51393:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 206, 58, 231, 2,               // 51398:      decode to STLRX using decoder 359
                                             // 51398:     }
                                             // 51398:     } // switch Inst[15]
                                             // 51398:    }
  5, 21,                                     // 51403:    case 0x5: {
  OPC_SwitchField, 10, 6,                    // 51405:     switch Inst[15:10] {
  31, 7,                                     // 51408:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 51410:      check predicate 72
  OPC_Decode, 223, 16, 240, 2,               // 51412:      decode to CASX using decoder 368
                                             // 51412:     }
  63, 0,                                     // 51417:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 51419:      check predicate 72
  OPC_Decode, 208, 16, 240, 2,               // 51421:      decode to CASLX using decoder 368
                                             // 51421:     }
                                             // 51421:     } // switch Inst[15:10]
                                             // 51421:    }
  6, 29,                                     // 51426:    case 0x6: {
  OPC_SwitchField, 15, 1,                    // 51428:     switch Inst[15] {
  0, 12,                                     // 51431:     case 0x0: {
  OPC_CheckPredicate, 73,                    // 51433:      check predicate 73
  OPC_SoftFail, 0, 128, 248, 125,            // 51435:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 200, 38, 231, 2,               // 51440:      decode to LDLARX using decoder 359
                                             // 51440:     }
  1, 0,                                      // 51445:     case 0x1: {
  OPC_SoftFail, 0, 128, 248, 125,            // 51447:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 178, 37, 231, 2,               // 51452:      decode to LDARX using decoder 359
                                             // 51452:     }
                                             // 51452:     } // switch Inst[15]
                                             // 51452:    }
  7, 0,                                      // 51457:    case 0x7: {
  OPC_SwitchField, 10, 6,                    // 51459:     switch Inst[15:10] {
  31, 7,                                     // 51462:     case 0x1f: {
  OPC_CheckPredicate, 72,                    // 51464:      check predicate 72
  OPC_Decode, 201, 16, 240, 2,               // 51466:      decode to CASAX using decoder 368
                                             // 51466:     }
  63, 0,                                     // 51471:     case 0x3f: {
  OPC_CheckPredicate, 72,                    // 51473:      check predicate 72
  OPC_Decode, 198, 16, 240, 2,               // 51475:      decode to CASALX using decoder 368
                                             // 51475:     }
                                             // 51475:     } // switch Inst[15:10]
                                             // 51475:    }
                                             // 51475:    } // switch Inst[23:21]
                                             // 51475:   }
  1, 135, 1,                                 // 51480:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 51483:    switch Inst[23:21] {
  0, 31,                                     // 51486:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 51488:     switch Inst[15] {
  0, 12,                                     // 51491:     case 0x0: {
  OPC_CheckPredicate, 74,                    // 51493:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51495:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 229, 59, 241, 2,               // 51500:      decode to STTXRXr using decoder 369
                                             // 51500:     }
  1, 0,                                      // 51505:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 51507:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51509:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 209, 58, 241, 2,               // 51514:      decode to STLTXRX using decoder 369
                                             // 51514:     }
                                             // 51514:     } // switch Inst[15]
                                             // 51514:    }
  2, 31,                                     // 51519:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 51521:     switch Inst[15] {
  0, 12,                                     // 51524:     case 0x0: {
  OPC_CheckPredicate, 74,                    // 51526:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 125,            // 51528:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 214, 40, 242, 2,               // 51533:      decode to LDTXRXr using decoder 370
                                             // 51533:     }
  1, 0,                                      // 51538:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 51540:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 125,            // 51542:      softfail pos=0x00000000 neg=0x001f7c00
  OPC_Decode, 180, 37, 242, 2,               // 51547:      decode to LDATXRX using decoder 370
                                             // 51547:     }
                                             // 51547:     } // switch Inst[15]
                                             // 51547:    }
  4, 31,                                     // 51552:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 51554:     switch Inst[15] {
  0, 12,                                     // 51557:     case 0x0: {
  OPC_CheckPredicate, 74,                    // 51559:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51561:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 221, 16, 240, 2,               // 51566:      decode to CASTX using decoder 368
                                             // 51566:     }
  1, 0,                                      // 51571:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 51573:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51575:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 206, 16, 240, 2,               // 51580:      decode to CASLTX using decoder 368
                                             // 51580:     }
                                             // 51580:     } // switch Inst[15]
                                             // 51580:    }
  6, 0,                                      // 51585:    case 0x6: {
  OPC_SwitchField, 15, 1,                    // 51587:     switch Inst[15] {
  0, 12,                                     // 51590:     case 0x0: {
  OPC_CheckPredicate, 74,                    // 51592:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51594:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 199, 16, 240, 2,               // 51599:      decode to CASATX using decoder 368
                                             // 51599:     }
  1, 0,                                      // 51604:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 51606:      check predicate 74
  OPC_SoftFail, 0, 128, 248, 1,              // 51608:      softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 196, 16, 240, 2,               // 51613:      decode to CASALTX using decoder 368
                                             // 51613:     }
                                             // 51613:     } // switch Inst[15]
                                             // 51613:    }
                                             // 51613:    } // switch Inst[23:21]
                                             // 51613:   }
  2, 17,                                     // 51618:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 51620:    switch Inst[21] {
  0, 5,                                      // 51623:    case 0x0: {
  OPC_Decode, 134, 21, 234, 2,               // 51625:     decode to EORXrs using decoder 362
                                             // 51625:    }
  1, 0,                                      // 51630:    case 0x1: {
  OPC_Decode, 239, 20, 234, 2,               // 51632:     decode to EONXrs using decoder 362
                                             // 51632:    }
                                             // 51632:    } // switch Inst[21]
                                             // 51632:   }
  3, 0,                                      // 51637:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 51639:    switch Inst[21] {
  0, 5,                                      // 51642:    case 0x0: {
  OPC_Decode, 163, 60, 234, 2,               // 51644:     decode to SUBXrs using decoder 362
                                             // 51644:    }
  1, 0,                                      // 51649:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 51651:     check Inst[23:22] == 0x0
  OPC_Scope, 9,                              // 51655:     try {
  OPC_CheckField, 13, 2, 3,                  // 51657:      check Inst[14:13] == 0x3
  OPC_Decode, 165, 60, 235, 2,               // 51661:      decode to SUBXrx64 using decoder 363
                                             // 51661:     } else try {
  OPC_Decode, 164, 60, 235, 2,               // 51666:      decode to SUBXrx using decoder 363
                                             // 51666:     }
                                             // 51666:    }
                                             // 51666:    } // switch Inst[21]
                                             // 51666:   }
                                             // 51666:   } // switch Inst[25:24]
                                             // 51666:  }
  7, 0,                                      // 51671:  case 0x7: {
  OPC_SwitchField, 24, 2,                    // 51673:   switch Inst[25:24] {
  0, 39,                                     // 51676:   case 0x0: {
  OPC_SwitchField, 22, 2,                    // 51678:    switch Inst[23:22] {
  0, 7,                                      // 51681:    case 0x0: {
  OPC_CheckPredicate, 74,                    // 51683:     check predicate 74
  OPC_Decode, 217, 59, 236, 2,               // 51685:     decode to STTNPXi using decoder 364
                                             // 51685:    }
  1, 7,                                      // 51690:    case 0x1: {
  OPC_CheckPredicate, 74,                    // 51692:     check predicate 74
  OPC_Decode, 189, 40, 236, 2,               // 51694:     decode to LDTNPXi using decoder 364
                                             // 51694:    }
  2, 7,                                      // 51699:    case 0x2: {
  OPC_CheckPredicate, 74,                    // 51701:     check predicate 74
  OPC_Decode, 222, 59, 236, 2,               // 51703:     decode to STTPpost using decoder 364
                                             // 51703:    }
  3, 0,                                      // 51708:    case 0x3: {
  OPC_CheckPredicate, 74,                    // 51710:     check predicate 74
  OPC_Decode, 194, 40, 236, 2,               // 51712:     decode to LDTPpost using decoder 364
                                             // 51712:    }
                                             // 51712:    } // switch Inst[23:22]
                                             // 51712:   }
  1, 39,                                     // 51717:   case 0x1: {
  OPC_SwitchField, 22, 2,                    // 51719:    switch Inst[23:22] {
  0, 7,                                      // 51722:    case 0x0: {
  OPC_CheckPredicate, 74,                    // 51724:     check predicate 74
  OPC_Decode, 221, 59, 236, 2,               // 51726:     decode to STTPi using decoder 364
                                             // 51726:    }
  1, 7,                                      // 51731:    case 0x1: {
  OPC_CheckPredicate, 74,                    // 51733:     check predicate 74
  OPC_Decode, 193, 40, 236, 2,               // 51735:     decode to LDTPi using decoder 364
                                             // 51735:    }
  2, 7,                                      // 51740:    case 0x2: {
  OPC_CheckPredicate, 74,                    // 51742:     check predicate 74
  OPC_Decode, 223, 59, 236, 2,               // 51744:     decode to STTPpre using decoder 364
                                             // 51744:    }
  3, 0,                                      // 51749:    case 0x3: {
  OPC_CheckPredicate, 74,                    // 51751:     check predicate 74
  OPC_Decode, 195, 40, 236, 2,               // 51753:     decode to LDTPpre using decoder 364
                                             // 51753:    }
                                             // 51753:    } // switch Inst[23:22]
                                             // 51753:   }
  2, 17,                                     // 51758:   case 0x2: {
  OPC_SwitchField, 21, 1,                    // 51760:    switch Inst[21] {
  0, 5,                                      // 51763:    case 0x0: {
  OPC_Decode, 154, 14, 234, 2,               // 51765:     decode to ANDSXrs using decoder 362
                                             // 51765:    }
  1, 0,                                      // 51770:    case 0x1: {
  OPC_Decode, 134, 16, 234, 2,               // 51772:     decode to BICSXrs using decoder 362
                                             // 51772:    }
                                             // 51772:    } // switch Inst[21]
                                             // 51772:   }
  3, 0,                                      // 51777:   case 0x3: {
  OPC_SwitchField, 21, 1,                    // 51779:    switch Inst[21] {
  0, 5,                                      // 51782:    case 0x0: {
  OPC_Decode, 156, 60, 234, 2,               // 51784:     decode to SUBSXrs using decoder 362
                                             // 51784:    }
  1, 0,                                      // 51789:    case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 51791:     check Inst[23:22] == 0x0
  OPC_Scope, 9,                              // 51795:     try {
  OPC_CheckField, 13, 2, 3,                  // 51797:      check Inst[14:13] == 0x3
  OPC_Decode, 158, 60, 235, 2,               // 51801:      decode to SUBSXrx64 using decoder 363
                                             // 51801:     } else try {
  OPC_Decode, 157, 60, 235, 2,               // 51806:      decode to SUBSXrx using decoder 363
                                             // 51806:     }
                                             // 51806:    }
                                             // 51806:    } // switch Inst[21]
                                             // 51806:   }
                                             // 51806:   } // switch Inst[25:24]
                                             // 51806:  }
                                             // 51806:  } // switch Inst[31:29]
                                             // 51806: }
  3, 206, 186, 1,                            // 51811: case 0x3: {
  OPC_SwitchField, 22, 4,                    // 51815:  switch Inst[25:22] {
  0, 144, 4,                                 // 51818:  case 0x0: {
  OPC_SwitchField, 29, 3,                    // 51821:   switch Inst[31:29] {
  0, 228, 1,                                 // 51824:   case 0x0: {
  OPC_SwitchField, 10, 12,                   // 51827:    switch Inst[21:10] {
  0, 7,                                      // 51830:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 51832:     check predicate 76
  OPC_Decode, 251, 57, 243, 2,               // 51834:     decode to ST4Fourv8b using decoder 371
                                             // 51834:    }
  1, 7,                                      // 51839:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 51841:     check predicate 76
  OPC_Decode, 247, 57, 243, 2,               // 51843:     decode to ST4Fourv4h using decoder 371
                                             // 51843:    }
  2, 7,                                      // 51848:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 51850:     check predicate 76
  OPC_Decode, 245, 57, 243, 2,               // 51852:     decode to ST4Fourv2s using decoder 371
                                             // 51852:    }
  8, 7,                                      // 51857:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 51859:     check predicate 76
  OPC_Decode, 200, 56, 243, 2,               // 51861:     decode to ST1Fourv8b using decoder 371
                                             // 51861:    }
  9, 7,                                      // 51866:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 51868:     check predicate 76
  OPC_Decode, 196, 56, 243, 2,               // 51870:     decode to ST1Fourv4h using decoder 371
                                             // 51870:    }
  10, 7,                                     // 51875:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 51877:     check predicate 76
  OPC_Decode, 194, 56, 243, 2,               // 51879:     decode to ST1Fourv2s using decoder 371
                                             // 51879:    }
  11, 7,                                     // 51884:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 51886:     check predicate 76
  OPC_Decode, 190, 56, 243, 2,               // 51888:     decode to ST1Fourv1d using decoder 371
                                             // 51888:    }
  16, 7,                                     // 51893:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 51895:     check predicate 76
  OPC_Decode, 223, 57, 244, 2,               // 51897:     decode to ST3Threev8b using decoder 372
                                             // 51897:    }
  17, 7,                                     // 51902:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 51904:     check predicate 76
  OPC_Decode, 219, 57, 244, 2,               // 51906:     decode to ST3Threev4h using decoder 372
                                             // 51906:    }
  18, 7,                                     // 51911:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 51913:     check predicate 76
  OPC_Decode, 217, 57, 244, 2,               // 51915:     decode to ST3Threev2s using decoder 372
                                             // 51915:    }
  24, 7,                                     // 51920:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 51922:     check predicate 76
  OPC_Decode, 246, 56, 244, 2,               // 51924:     decode to ST1Threev8b using decoder 372
                                             // 51924:    }
  25, 7,                                     // 51929:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 51931:     check predicate 76
  OPC_Decode, 242, 56, 244, 2,               // 51933:     decode to ST1Threev4h using decoder 372
                                             // 51933:    }
  26, 7,                                     // 51938:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 51940:     check predicate 76
  OPC_Decode, 240, 56, 244, 2,               // 51942:     decode to ST1Threev2s using decoder 372
                                             // 51942:    }
  27, 7,                                     // 51947:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 51949:     check predicate 76
  OPC_Decode, 236, 56, 244, 2,               // 51951:     decode to ST1Threev1d using decoder 372
                                             // 51951:    }
  28, 7,                                     // 51956:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 51958:     check predicate 76
  OPC_Decode, 230, 56, 245, 2,               // 51960:     decode to ST1Onev8b using decoder 373
                                             // 51960:    }
  29, 7,                                     // 51965:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 51967:     check predicate 76
  OPC_Decode, 226, 56, 245, 2,               // 51969:     decode to ST1Onev4h using decoder 373
                                             // 51969:    }
  30, 7,                                     // 51974:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 51976:     check predicate 76
  OPC_Decode, 224, 56, 245, 2,               // 51978:     decode to ST1Onev2s using decoder 373
                                             // 51978:    }
  31, 7,                                     // 51983:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 51985:     check predicate 76
  OPC_Decode, 220, 56, 245, 2,               // 51987:     decode to ST1Onev1d using decoder 373
                                             // 51987:    }
  32, 7,                                     // 51992:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 51994:     check predicate 76
  OPC_Decode, 191, 57, 246, 2,               // 51996:     decode to ST2Twov8b using decoder 374
                                             // 51996:    }
  33, 7,                                     // 52001:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 52003:     check predicate 76
  OPC_Decode, 187, 57, 246, 2,               // 52005:     decode to ST2Twov4h using decoder 374
                                             // 52005:    }
  34, 7,                                     // 52010:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 52012:     check predicate 76
  OPC_Decode, 185, 57, 246, 2,               // 52014:     decode to ST2Twov2s using decoder 374
                                             // 52014:    }
  40, 7,                                     // 52019:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 52021:     check predicate 76
  OPC_Decode, 134, 57, 246, 2,               // 52023:     decode to ST1Twov8b using decoder 374
                                             // 52023:    }
  41, 7,                                     // 52028:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 52030:     check predicate 76
  OPC_Decode, 130, 57, 246, 2,               // 52032:     decode to ST1Twov4h using decoder 374
                                             // 52032:    }
  42, 7,                                     // 52037:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 52039:     check predicate 76
  OPC_Decode, 128, 57, 246, 2,               // 52041:     decode to ST1Twov2s using decoder 374
                                             // 52041:    }
  43, 0,                                     // 52046:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 52048:     check predicate 76
  OPC_Decode, 252, 56, 246, 2,               // 52050:     decode to ST1Twov1d using decoder 374
                                             // 52050:    }
                                             // 52050:    } // switch Inst[21:10]
                                             // 52050:   }
  1, 7,                                      // 52055:   case 0x1: {
  OPC_CheckPredicate, 77,                    // 52057:    check predicate 77
  OPC_Decode, 229, 58, 236, 2,               // 52059:    decode to STNPSi using decoder 364
                                             // 52059:   }
  2, 255, 1,                                 // 52064:   case 0x2: {
  OPC_SwitchField, 10, 12,                   // 52067:    switch Inst[21:10] {
  0, 7,                                      // 52070:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 52072:     check predicate 76
  OPC_Decode, 241, 57, 247, 2,               // 52074:     decode to ST4Fourv16b using decoder 375
                                             // 52074:    }
  1, 7,                                      // 52079:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 52081:     check predicate 76
  OPC_Decode, 253, 57, 247, 2,               // 52083:     decode to ST4Fourv8h using decoder 375
                                             // 52083:    }
  2, 7,                                      // 52088:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 52090:     check predicate 76
  OPC_Decode, 249, 57, 247, 2,               // 52092:     decode to ST4Fourv4s using decoder 375
                                             // 52092:    }
  3, 7,                                      // 52097:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 52099:     check predicate 76
  OPC_Decode, 243, 57, 247, 2,               // 52101:     decode to ST4Fourv2d using decoder 375
                                             // 52101:    }
  8, 7,                                      // 52106:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 52108:     check predicate 76
  OPC_Decode, 188, 56, 247, 2,               // 52110:     decode to ST1Fourv16b using decoder 375
                                             // 52110:    }
  9, 7,                                      // 52115:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 52117:     check predicate 76
  OPC_Decode, 202, 56, 247, 2,               // 52119:     decode to ST1Fourv8h using decoder 375
                                             // 52119:    }
  10, 7,                                     // 52124:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 52126:     check predicate 76
  OPC_Decode, 198, 56, 247, 2,               // 52128:     decode to ST1Fourv4s using decoder 375
                                             // 52128:    }
  11, 7,                                     // 52133:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 52135:     check predicate 76
  OPC_Decode, 192, 56, 247, 2,               // 52137:     decode to ST1Fourv2d using decoder 375
                                             // 52137:    }
  16, 7,                                     // 52142:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 52144:     check predicate 76
  OPC_Decode, 213, 57, 248, 2,               // 52146:     decode to ST3Threev16b using decoder 376
                                             // 52146:    }
  17, 7,                                     // 52151:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 52153:     check predicate 76
  OPC_Decode, 225, 57, 248, 2,               // 52155:     decode to ST3Threev8h using decoder 376
                                             // 52155:    }
  18, 7,                                     // 52160:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 52162:     check predicate 76
  OPC_Decode, 221, 57, 248, 2,               // 52164:     decode to ST3Threev4s using decoder 376
                                             // 52164:    }
  19, 7,                                     // 52169:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 52171:     check predicate 76
  OPC_Decode, 215, 57, 248, 2,               // 52173:     decode to ST3Threev2d using decoder 376
                                             // 52173:    }
  24, 7,                                     // 52178:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 52180:     check predicate 76
  OPC_Decode, 234, 56, 248, 2,               // 52182:     decode to ST1Threev16b using decoder 376
                                             // 52182:    }
  25, 7,                                     // 52187:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 52189:     check predicate 76
  OPC_Decode, 248, 56, 248, 2,               // 52191:     decode to ST1Threev8h using decoder 376
                                             // 52191:    }
  26, 7,                                     // 52196:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 52198:     check predicate 76
  OPC_Decode, 244, 56, 248, 2,               // 52200:     decode to ST1Threev4s using decoder 376
                                             // 52200:    }
  27, 7,                                     // 52205:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 52207:     check predicate 76
  OPC_Decode, 238, 56, 248, 2,               // 52209:     decode to ST1Threev2d using decoder 376
                                             // 52209:    }
  28, 7,                                     // 52214:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 52216:     check predicate 76
  OPC_Decode, 218, 56, 249, 2,               // 52218:     decode to ST1Onev16b using decoder 377
                                             // 52218:    }
  29, 7,                                     // 52223:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 52225:     check predicate 76
  OPC_Decode, 232, 56, 249, 2,               // 52227:     decode to ST1Onev8h using decoder 377
                                             // 52227:    }
  30, 7,                                     // 52232:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 52234:     check predicate 76
  OPC_Decode, 228, 56, 249, 2,               // 52236:     decode to ST1Onev4s using decoder 377
                                             // 52236:    }
  31, 7,                                     // 52241:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 52243:     check predicate 76
  OPC_Decode, 222, 56, 249, 2,               // 52245:     decode to ST1Onev2d using decoder 377
                                             // 52245:    }
  32, 7,                                     // 52250:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 52252:     check predicate 76
  OPC_Decode, 181, 57, 250, 2,               // 52254:     decode to ST2Twov16b using decoder 378
                                             // 52254:    }
  33, 7,                                     // 52259:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 52261:     check predicate 76
  OPC_Decode, 193, 57, 250, 2,               // 52263:     decode to ST2Twov8h using decoder 378
                                             // 52263:    }
  34, 7,                                     // 52268:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 52270:     check predicate 76
  OPC_Decode, 189, 57, 250, 2,               // 52272:     decode to ST2Twov4s using decoder 378
                                             // 52272:    }
  35, 7,                                     // 52277:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 52279:     check predicate 76
  OPC_Decode, 183, 57, 250, 2,               // 52281:     decode to ST2Twov2d using decoder 378
                                             // 52281:    }
  40, 7,                                     // 52286:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 52288:     check predicate 76
  OPC_Decode, 250, 56, 250, 2,               // 52290:     decode to ST1Twov16b using decoder 378
                                             // 52290:    }
  41, 7,                                     // 52295:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 52297:     check predicate 76
  OPC_Decode, 136, 57, 250, 2,               // 52299:     decode to ST1Twov8h using decoder 378
                                             // 52299:    }
  42, 7,                                     // 52304:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 52306:     check predicate 76
  OPC_Decode, 132, 57, 250, 2,               // 52308:     decode to ST1Twov4s using decoder 378
                                             // 52308:    }
  43, 0,                                     // 52313:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 52315:     check predicate 76
  OPC_Decode, 254, 56, 250, 2,               // 52317:     decode to ST1Twov2d using decoder 378
                                             // 52317:    }
                                             // 52317:    } // switch Inst[21:10]
                                             // 52317:   }
  3, 7,                                      // 52322:   case 0x3: {
  OPC_CheckPredicate, 77,                    // 52324:    check predicate 77
  OPC_Decode, 227, 58, 236, 2,               // 52326:    decode to STNPDi using decoder 364
                                             // 52326:   }
  5, 7,                                      // 52331:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 52333:    check predicate 77
  OPC_Decode, 228, 58, 236, 2,               // 52335:    decode to STNPQi using decoder 364
                                             // 52335:   }
  7, 0,                                      // 52340:   case 0x7: {
  OPC_CheckPredicate, 78,                    // 52342:    check predicate 78
  OPC_Decode, 216, 59, 236, 2,               // 52344:    decode to STTNPQi using decoder 364
                                             // 52344:   }
                                             // 52344:   } // switch Inst[31:29]
                                             // 52344:  }
  1, 144, 4,                                 // 52349:  case 0x1: {
  OPC_SwitchField, 29, 3,                    // 52352:   switch Inst[31:29] {
  0, 228, 1,                                 // 52355:   case 0x0: {
  OPC_SwitchField, 10, 12,                   // 52358:    switch Inst[21:10] {
  0, 7,                                      // 52361:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 52363:     check predicate 76
  OPC_Decode, 229, 36, 243, 2,               // 52365:     decode to LD4Fourv8b using decoder 371
                                             // 52365:    }
  1, 7,                                      // 52370:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 52372:     check predicate 76
  OPC_Decode, 225, 36, 243, 2,               // 52374:     decode to LD4Fourv4h using decoder 371
                                             // 52374:    }
  2, 7,                                      // 52379:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 52381:     check predicate 76
  OPC_Decode, 223, 36, 243, 2,               // 52383:     decode to LD4Fourv2s using decoder 371
                                             // 52383:    }
  8, 7,                                      // 52388:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 52390:     check predicate 76
  OPC_Decode, 217, 34, 243, 2,               // 52392:     decode to LD1Fourv8b using decoder 371
                                             // 52392:    }
  9, 7,                                      // 52397:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 52399:     check predicate 76
  OPC_Decode, 213, 34, 243, 2,               // 52401:     decode to LD1Fourv4h using decoder 371
                                             // 52401:    }
  10, 7,                                     // 52406:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 52408:     check predicate 76
  OPC_Decode, 211, 34, 243, 2,               // 52410:     decode to LD1Fourv2s using decoder 371
                                             // 52410:    }
  11, 7,                                     // 52415:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 52417:     check predicate 76
  OPC_Decode, 207, 34, 243, 2,               // 52419:     decode to LD1Fourv1d using decoder 371
                                             // 52419:    }
  16, 7,                                     // 52424:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 52426:     check predicate 76
  OPC_Decode, 201, 36, 244, 2,               // 52428:     decode to LD3Threev8b using decoder 372
                                             // 52428:    }
  17, 7,                                     // 52433:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 52435:     check predicate 76
  OPC_Decode, 197, 36, 244, 2,               // 52437:     decode to LD3Threev4h using decoder 372
                                             // 52437:    }
  18, 7,                                     // 52442:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 52444:     check predicate 76
  OPC_Decode, 195, 36, 244, 2,               // 52446:     decode to LD3Threev2s using decoder 372
                                             // 52446:    }
  24, 7,                                     // 52451:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 52453:     check predicate 76
  OPC_Decode, 195, 35, 244, 2,               // 52455:     decode to LD1Threev8b using decoder 372
                                             // 52455:    }
  25, 7,                                     // 52460:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 52462:     check predicate 76
  OPC_Decode, 191, 35, 244, 2,               // 52464:     decode to LD1Threev4h using decoder 372
                                             // 52464:    }
  26, 7,                                     // 52469:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 52471:     check predicate 76
  OPC_Decode, 189, 35, 244, 2,               // 52473:     decode to LD1Threev2s using decoder 372
                                             // 52473:    }
  27, 7,                                     // 52478:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 52480:     check predicate 76
  OPC_Decode, 185, 35, 244, 2,               // 52482:     decode to LD1Threev1d using decoder 372
                                             // 52482:    }
  28, 7,                                     // 52487:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 52489:     check predicate 76
  OPC_Decode, 247, 34, 245, 2,               // 52491:     decode to LD1Onev8b using decoder 373
                                             // 52491:    }
  29, 7,                                     // 52496:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 52498:     check predicate 76
  OPC_Decode, 243, 34, 245, 2,               // 52500:     decode to LD1Onev4h using decoder 373
                                             // 52500:    }
  30, 7,                                     // 52505:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 52507:     check predicate 76
  OPC_Decode, 241, 34, 245, 2,               // 52509:     decode to LD1Onev2s using decoder 373
                                             // 52509:    }
  31, 7,                                     // 52514:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 52516:     check predicate 76
  OPC_Decode, 237, 34, 245, 2,               // 52518:     decode to LD1Onev1d using decoder 373
                                             // 52518:    }
  32, 7,                                     // 52523:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 52525:     check predicate 76
  OPC_Decode, 153, 36, 246, 2,               // 52527:     decode to LD2Twov8b using decoder 374
                                             // 52527:    }
  33, 7,                                     // 52532:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 52534:     check predicate 76
  OPC_Decode, 149, 36, 246, 2,               // 52536:     decode to LD2Twov4h using decoder 374
                                             // 52536:    }
  34, 7,                                     // 52541:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 52543:     check predicate 76
  OPC_Decode, 147, 36, 246, 2,               // 52545:     decode to LD2Twov2s using decoder 374
                                             // 52545:    }
  40, 7,                                     // 52550:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 52552:     check predicate 76
  OPC_Decode, 211, 35, 246, 2,               // 52554:     decode to LD1Twov8b using decoder 374
                                             // 52554:    }
  41, 7,                                     // 52559:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 52561:     check predicate 76
  OPC_Decode, 207, 35, 246, 2,               // 52563:     decode to LD1Twov4h using decoder 374
                                             // 52563:    }
  42, 7,                                     // 52568:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 52570:     check predicate 76
  OPC_Decode, 205, 35, 246, 2,               // 52572:     decode to LD1Twov2s using decoder 374
                                             // 52572:    }
  43, 0,                                     // 52577:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 52579:     check predicate 76
  OPC_Decode, 201, 35, 246, 2,               // 52581:     decode to LD1Twov1d using decoder 374
                                             // 52581:    }
                                             // 52581:    } // switch Inst[21:10]
                                             // 52581:   }
  1, 7,                                      // 52586:   case 0x1: {
  OPC_CheckPredicate, 77,                    // 52588:    check predicate 77
  OPC_Decode, 219, 38, 236, 2,               // 52590:    decode to LDNPSi using decoder 364
                                             // 52590:   }
  2, 255, 1,                                 // 52595:   case 0x2: {
  OPC_SwitchField, 10, 12,                   // 52598:    switch Inst[21:10] {
  0, 7,                                      // 52601:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 52603:     check predicate 76
  OPC_Decode, 219, 36, 247, 2,               // 52605:     decode to LD4Fourv16b using decoder 375
                                             // 52605:    }
  1, 7,                                      // 52610:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 52612:     check predicate 76
  OPC_Decode, 231, 36, 247, 2,               // 52614:     decode to LD4Fourv8h using decoder 375
                                             // 52614:    }
  2, 7,                                      // 52619:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 52621:     check predicate 76
  OPC_Decode, 227, 36, 247, 2,               // 52623:     decode to LD4Fourv4s using decoder 375
                                             // 52623:    }
  3, 7,                                      // 52628:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 52630:     check predicate 76
  OPC_Decode, 221, 36, 247, 2,               // 52632:     decode to LD4Fourv2d using decoder 375
                                             // 52632:    }
  8, 7,                                      // 52637:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 52639:     check predicate 76
  OPC_Decode, 205, 34, 247, 2,               // 52641:     decode to LD1Fourv16b using decoder 375
                                             // 52641:    }
  9, 7,                                      // 52646:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 52648:     check predicate 76
  OPC_Decode, 219, 34, 247, 2,               // 52650:     decode to LD1Fourv8h using decoder 375
                                             // 52650:    }
  10, 7,                                     // 52655:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 52657:     check predicate 76
  OPC_Decode, 215, 34, 247, 2,               // 52659:     decode to LD1Fourv4s using decoder 375
                                             // 52659:    }
  11, 7,                                     // 52664:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 52666:     check predicate 76
  OPC_Decode, 209, 34, 247, 2,               // 52668:     decode to LD1Fourv2d using decoder 375
                                             // 52668:    }
  16, 7,                                     // 52673:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 52675:     check predicate 76
  OPC_Decode, 191, 36, 248, 2,               // 52677:     decode to LD3Threev16b using decoder 376
                                             // 52677:    }
  17, 7,                                     // 52682:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 52684:     check predicate 76
  OPC_Decode, 203, 36, 248, 2,               // 52686:     decode to LD3Threev8h using decoder 376
                                             // 52686:    }
  18, 7,                                     // 52691:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 52693:     check predicate 76
  OPC_Decode, 199, 36, 248, 2,               // 52695:     decode to LD3Threev4s using decoder 376
                                             // 52695:    }
  19, 7,                                     // 52700:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 52702:     check predicate 76
  OPC_Decode, 193, 36, 248, 2,               // 52704:     decode to LD3Threev2d using decoder 376
                                             // 52704:    }
  24, 7,                                     // 52709:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 52711:     check predicate 76
  OPC_Decode, 183, 35, 248, 2,               // 52713:     decode to LD1Threev16b using decoder 376
                                             // 52713:    }
  25, 7,                                     // 52718:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 52720:     check predicate 76
  OPC_Decode, 197, 35, 248, 2,               // 52722:     decode to LD1Threev8h using decoder 376
                                             // 52722:    }
  26, 7,                                     // 52727:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 52729:     check predicate 76
  OPC_Decode, 193, 35, 248, 2,               // 52731:     decode to LD1Threev4s using decoder 376
                                             // 52731:    }
  27, 7,                                     // 52736:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 52738:     check predicate 76
  OPC_Decode, 187, 35, 248, 2,               // 52740:     decode to LD1Threev2d using decoder 376
                                             // 52740:    }
  28, 7,                                     // 52745:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 52747:     check predicate 76
  OPC_Decode, 235, 34, 249, 2,               // 52749:     decode to LD1Onev16b using decoder 377
                                             // 52749:    }
  29, 7,                                     // 52754:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 52756:     check predicate 76
  OPC_Decode, 249, 34, 249, 2,               // 52758:     decode to LD1Onev8h using decoder 377
                                             // 52758:    }
  30, 7,                                     // 52763:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 52765:     check predicate 76
  OPC_Decode, 245, 34, 249, 2,               // 52767:     decode to LD1Onev4s using decoder 377
                                             // 52767:    }
  31, 7,                                     // 52772:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 52774:     check predicate 76
  OPC_Decode, 239, 34, 249, 2,               // 52776:     decode to LD1Onev2d using decoder 377
                                             // 52776:    }
  32, 7,                                     // 52781:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 52783:     check predicate 76
  OPC_Decode, 143, 36, 250, 2,               // 52785:     decode to LD2Twov16b using decoder 378
                                             // 52785:    }
  33, 7,                                     // 52790:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 52792:     check predicate 76
  OPC_Decode, 155, 36, 250, 2,               // 52794:     decode to LD2Twov8h using decoder 378
                                             // 52794:    }
  34, 7,                                     // 52799:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 52801:     check predicate 76
  OPC_Decode, 151, 36, 250, 2,               // 52803:     decode to LD2Twov4s using decoder 378
                                             // 52803:    }
  35, 7,                                     // 52808:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 52810:     check predicate 76
  OPC_Decode, 145, 36, 250, 2,               // 52812:     decode to LD2Twov2d using decoder 378
                                             // 52812:    }
  40, 7,                                     // 52817:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 52819:     check predicate 76
  OPC_Decode, 199, 35, 250, 2,               // 52821:     decode to LD1Twov16b using decoder 378
                                             // 52821:    }
  41, 7,                                     // 52826:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 52828:     check predicate 76
  OPC_Decode, 213, 35, 250, 2,               // 52830:     decode to LD1Twov8h using decoder 378
                                             // 52830:    }
  42, 7,                                     // 52835:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 52837:     check predicate 76
  OPC_Decode, 209, 35, 250, 2,               // 52839:     decode to LD1Twov4s using decoder 378
                                             // 52839:    }
  43, 0,                                     // 52844:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 52846:     check predicate 76
  OPC_Decode, 203, 35, 250, 2,               // 52848:     decode to LD1Twov2d using decoder 378
                                             // 52848:    }
                                             // 52848:    } // switch Inst[21:10]
                                             // 52848:   }
  3, 7,                                      // 52853:   case 0x3: {
  OPC_CheckPredicate, 77,                    // 52855:    check predicate 77
  OPC_Decode, 217, 38, 236, 2,               // 52857:    decode to LDNPDi using decoder 364
                                             // 52857:   }
  5, 7,                                      // 52862:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 52864:    check predicate 77
  OPC_Decode, 218, 38, 236, 2,               // 52866:    decode to LDNPQi using decoder 364
                                             // 52866:   }
  7, 0,                                      // 52871:   case 0x7: {
  OPC_CheckPredicate, 78,                    // 52873:    check predicate 78
  OPC_Decode, 188, 40, 236, 2,               // 52875:    decode to LDTNPQi using decoder 364
                                             // 52875:   }
                                             // 52875:   } // switch Inst[31:29]
                                             // 52875:  }
  2, 228, 5,                                 // 52880:  case 0x2: {
  OPC_SwitchField, 29, 3,                    // 52883:   switch Inst[31:29] {
  0, 200, 2,                                 // 52886:   case 0x0: {
  OPC_SwitchField, 10, 6,                    // 52889:    switch Inst[15:10] {
  0, 11,                                     // 52892:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 52894:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52896:     check Inst[21] == 0x0
  OPC_Decode, 252, 57, 251, 2,               // 52900:     decode to ST4Fourv8b_POST using decoder 379
                                             // 52900:    }
  1, 11,                                     // 52905:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 52907:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52909:     check Inst[21] == 0x0
  OPC_Decode, 248, 57, 251, 2,               // 52913:     decode to ST4Fourv4h_POST using decoder 379
                                             // 52913:    }
  2, 11,                                     // 52918:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 52920:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52922:     check Inst[21] == 0x0
  OPC_Decode, 246, 57, 251, 2,               // 52926:     decode to ST4Fourv2s_POST using decoder 379
                                             // 52926:    }
  8, 11,                                     // 52931:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 52933:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52935:     check Inst[21] == 0x0
  OPC_Decode, 201, 56, 251, 2,               // 52939:     decode to ST1Fourv8b_POST using decoder 379
                                             // 52939:    }
  9, 11,                                     // 52944:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 52946:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52948:     check Inst[21] == 0x0
  OPC_Decode, 197, 56, 251, 2,               // 52952:     decode to ST1Fourv4h_POST using decoder 379
                                             // 52952:    }
  10, 11,                                    // 52957:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 52959:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52961:     check Inst[21] == 0x0
  OPC_Decode, 195, 56, 251, 2,               // 52965:     decode to ST1Fourv2s_POST using decoder 379
                                             // 52965:    }
  11, 11,                                    // 52970:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 52972:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52974:     check Inst[21] == 0x0
  OPC_Decode, 191, 56, 251, 2,               // 52978:     decode to ST1Fourv1d_POST using decoder 379
                                             // 52978:    }
  16, 11,                                    // 52983:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 52985:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 52987:     check Inst[21] == 0x0
  OPC_Decode, 224, 57, 252, 2,               // 52991:     decode to ST3Threev8b_POST using decoder 380
                                             // 52991:    }
  17, 11,                                    // 52996:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 52998:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53000:     check Inst[21] == 0x0
  OPC_Decode, 220, 57, 252, 2,               // 53004:     decode to ST3Threev4h_POST using decoder 380
                                             // 53004:    }
  18, 11,                                    // 53009:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 53011:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53013:     check Inst[21] == 0x0
  OPC_Decode, 218, 57, 252, 2,               // 53017:     decode to ST3Threev2s_POST using decoder 380
                                             // 53017:    }
  24, 11,                                    // 53022:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 53024:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53026:     check Inst[21] == 0x0
  OPC_Decode, 247, 56, 252, 2,               // 53030:     decode to ST1Threev8b_POST using decoder 380
                                             // 53030:    }
  25, 11,                                    // 53035:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 53037:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53039:     check Inst[21] == 0x0
  OPC_Decode, 243, 56, 252, 2,               // 53043:     decode to ST1Threev4h_POST using decoder 380
                                             // 53043:    }
  26, 11,                                    // 53048:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 53050:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53052:     check Inst[21] == 0x0
  OPC_Decode, 241, 56, 252, 2,               // 53056:     decode to ST1Threev2s_POST using decoder 380
                                             // 53056:    }
  27, 11,                                    // 53061:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 53063:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53065:     check Inst[21] == 0x0
  OPC_Decode, 237, 56, 252, 2,               // 53069:     decode to ST1Threev1d_POST using decoder 380
                                             // 53069:    }
  28, 11,                                    // 53074:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 53076:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53078:     check Inst[21] == 0x0
  OPC_Decode, 231, 56, 253, 2,               // 53082:     decode to ST1Onev8b_POST using decoder 381
                                             // 53082:    }
  29, 11,                                    // 53087:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 53089:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53091:     check Inst[21] == 0x0
  OPC_Decode, 227, 56, 253, 2,               // 53095:     decode to ST1Onev4h_POST using decoder 381
                                             // 53095:    }
  30, 11,                                    // 53100:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 53102:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53104:     check Inst[21] == 0x0
  OPC_Decode, 225, 56, 253, 2,               // 53108:     decode to ST1Onev2s_POST using decoder 381
                                             // 53108:    }
  31, 11,                                    // 53113:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 53115:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53117:     check Inst[21] == 0x0
  OPC_Decode, 221, 56, 253, 2,               // 53121:     decode to ST1Onev1d_POST using decoder 381
                                             // 53121:    }
  32, 11,                                    // 53126:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 53128:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53130:     check Inst[21] == 0x0
  OPC_Decode, 192, 57, 254, 2,               // 53134:     decode to ST2Twov8b_POST using decoder 382
                                             // 53134:    }
  33, 11,                                    // 53139:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 53141:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53143:     check Inst[21] == 0x0
  OPC_Decode, 188, 57, 254, 2,               // 53147:     decode to ST2Twov4h_POST using decoder 382
                                             // 53147:    }
  34, 11,                                    // 53152:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 53154:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53156:     check Inst[21] == 0x0
  OPC_Decode, 186, 57, 254, 2,               // 53160:     decode to ST2Twov2s_POST using decoder 382
                                             // 53160:    }
  40, 11,                                    // 53165:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 53167:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53169:     check Inst[21] == 0x0
  OPC_Decode, 135, 57, 254, 2,               // 53173:     decode to ST1Twov8b_POST using decoder 382
                                             // 53173:    }
  41, 11,                                    // 53178:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 53180:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53182:     check Inst[21] == 0x0
  OPC_Decode, 131, 57, 254, 2,               // 53186:     decode to ST1Twov4h_POST using decoder 382
                                             // 53186:    }
  42, 11,                                    // 53191:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 53193:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53195:     check Inst[21] == 0x0
  OPC_Decode, 129, 57, 254, 2,               // 53199:     decode to ST1Twov2s_POST using decoder 382
                                             // 53199:    }
  43, 0,                                     // 53204:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 53206:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53208:     check Inst[21] == 0x0
  OPC_Decode, 253, 56, 254, 2,               // 53212:     decode to ST1Twov1d_POST using decoder 382
                                             // 53212:    }
                                             // 53212:    } // switch Inst[15:10]
                                             // 53212:   }
  1, 7,                                      // 53217:   case 0x1: {
  OPC_CheckPredicate, 77,                    // 53219:    check predicate 77
  OPC_Decode, 158, 59, 236, 2,               // 53221:    decode to STPSpost using decoder 364
                                             // 53221:   }
  2, 239, 2,                                 // 53226:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 53229:    switch Inst[15:10] {
  0, 11,                                     // 53232:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 53234:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53236:     check Inst[21] == 0x0
  OPC_Decode, 242, 57, 255, 2,               // 53240:     decode to ST4Fourv16b_POST using decoder 383
                                             // 53240:    }
  1, 11,                                     // 53245:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 53247:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53249:     check Inst[21] == 0x0
  OPC_Decode, 254, 57, 255, 2,               // 53253:     decode to ST4Fourv8h_POST using decoder 383
                                             // 53253:    }
  2, 11,                                     // 53258:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 53260:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53262:     check Inst[21] == 0x0
  OPC_Decode, 250, 57, 255, 2,               // 53266:     decode to ST4Fourv4s_POST using decoder 383
                                             // 53266:    }
  3, 11,                                     // 53271:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 53273:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53275:     check Inst[21] == 0x0
  OPC_Decode, 244, 57, 255, 2,               // 53279:     decode to ST4Fourv2d_POST using decoder 383
                                             // 53279:    }
  8, 11,                                     // 53284:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 53286:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53288:     check Inst[21] == 0x0
  OPC_Decode, 189, 56, 255, 2,               // 53292:     decode to ST1Fourv16b_POST using decoder 383
                                             // 53292:    }
  9, 11,                                     // 53297:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 53299:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53301:     check Inst[21] == 0x0
  OPC_Decode, 203, 56, 255, 2,               // 53305:     decode to ST1Fourv8h_POST using decoder 383
                                             // 53305:    }
  10, 11,                                    // 53310:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 53312:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53314:     check Inst[21] == 0x0
  OPC_Decode, 199, 56, 255, 2,               // 53318:     decode to ST1Fourv4s_POST using decoder 383
                                             // 53318:    }
  11, 11,                                    // 53323:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 53325:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53327:     check Inst[21] == 0x0
  OPC_Decode, 193, 56, 255, 2,               // 53331:     decode to ST1Fourv2d_POST using decoder 383
                                             // 53331:    }
  16, 11,                                    // 53336:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 53338:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53340:     check Inst[21] == 0x0
  OPC_Decode, 214, 57, 128, 3,               // 53344:     decode to ST3Threev16b_POST using decoder 384
                                             // 53344:    }
  17, 11,                                    // 53349:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 53351:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53353:     check Inst[21] == 0x0
  OPC_Decode, 226, 57, 128, 3,               // 53357:     decode to ST3Threev8h_POST using decoder 384
                                             // 53357:    }
  18, 11,                                    // 53362:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 53364:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53366:     check Inst[21] == 0x0
  OPC_Decode, 222, 57, 128, 3,               // 53370:     decode to ST3Threev4s_POST using decoder 384
                                             // 53370:    }
  19, 11,                                    // 53375:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 53377:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53379:     check Inst[21] == 0x0
  OPC_Decode, 216, 57, 128, 3,               // 53383:     decode to ST3Threev2d_POST using decoder 384
                                             // 53383:    }
  24, 11,                                    // 53388:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 53390:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53392:     check Inst[21] == 0x0
  OPC_Decode, 235, 56, 128, 3,               // 53396:     decode to ST1Threev16b_POST using decoder 384
                                             // 53396:    }
  25, 11,                                    // 53401:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 53403:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53405:     check Inst[21] == 0x0
  OPC_Decode, 249, 56, 128, 3,               // 53409:     decode to ST1Threev8h_POST using decoder 384
                                             // 53409:    }
  26, 11,                                    // 53414:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 53416:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53418:     check Inst[21] == 0x0
  OPC_Decode, 245, 56, 128, 3,               // 53422:     decode to ST1Threev4s_POST using decoder 384
                                             // 53422:    }
  27, 11,                                    // 53427:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 53429:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53431:     check Inst[21] == 0x0
  OPC_Decode, 239, 56, 128, 3,               // 53435:     decode to ST1Threev2d_POST using decoder 384
                                             // 53435:    }
  28, 11,                                    // 53440:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 53442:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53444:     check Inst[21] == 0x0
  OPC_Decode, 219, 56, 129, 3,               // 53448:     decode to ST1Onev16b_POST using decoder 385
                                             // 53448:    }
  29, 11,                                    // 53453:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 53455:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53457:     check Inst[21] == 0x0
  OPC_Decode, 233, 56, 129, 3,               // 53461:     decode to ST1Onev8h_POST using decoder 385
                                             // 53461:    }
  30, 11,                                    // 53466:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 53468:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53470:     check Inst[21] == 0x0
  OPC_Decode, 229, 56, 129, 3,               // 53474:     decode to ST1Onev4s_POST using decoder 385
                                             // 53474:    }
  31, 11,                                    // 53479:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 53481:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53483:     check Inst[21] == 0x0
  OPC_Decode, 223, 56, 129, 3,               // 53487:     decode to ST1Onev2d_POST using decoder 385
                                             // 53487:    }
  32, 11,                                    // 53492:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 53494:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53496:     check Inst[21] == 0x0
  OPC_Decode, 182, 57, 130, 3,               // 53500:     decode to ST2Twov16b_POST using decoder 386
                                             // 53500:    }
  33, 11,                                    // 53505:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 53507:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53509:     check Inst[21] == 0x0
  OPC_Decode, 194, 57, 130, 3,               // 53513:     decode to ST2Twov8h_POST using decoder 386
                                             // 53513:    }
  34, 11,                                    // 53518:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 53520:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53522:     check Inst[21] == 0x0
  OPC_Decode, 190, 57, 130, 3,               // 53526:     decode to ST2Twov4s_POST using decoder 386
                                             // 53526:    }
  35, 11,                                    // 53531:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 53533:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53535:     check Inst[21] == 0x0
  OPC_Decode, 184, 57, 130, 3,               // 53539:     decode to ST2Twov2d_POST using decoder 386
                                             // 53539:    }
  40, 11,                                    // 53544:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 53546:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53548:     check Inst[21] == 0x0
  OPC_Decode, 251, 56, 130, 3,               // 53552:     decode to ST1Twov16b_POST using decoder 386
                                             // 53552:    }
  41, 11,                                    // 53557:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 53559:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53561:     check Inst[21] == 0x0
  OPC_Decode, 137, 57, 130, 3,               // 53565:     decode to ST1Twov8h_POST using decoder 386
                                             // 53565:    }
  42, 11,                                    // 53570:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 53572:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53574:     check Inst[21] == 0x0
  OPC_Decode, 133, 57, 130, 3,               // 53578:     decode to ST1Twov4s_POST using decoder 386
                                             // 53578:    }
  43, 0,                                     // 53583:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 53585:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53587:     check Inst[21] == 0x0
  OPC_Decode, 255, 56, 130, 3,               // 53591:     decode to ST1Twov2d_POST using decoder 386
                                             // 53591:    }
                                             // 53591:    } // switch Inst[15:10]
                                             // 53591:   }
  3, 7,                                      // 53596:   case 0x3: {
  OPC_CheckPredicate, 77,                    // 53598:    check predicate 77
  OPC_Decode, 152, 59, 236, 2,               // 53600:    decode to STPDpost using decoder 364
                                             // 53600:   }
  5, 7,                                      // 53605:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 53607:    check predicate 77
  OPC_Decode, 155, 59, 236, 2,               // 53609:    decode to STPQpost using decoder 364
                                             // 53609:   }
  7, 0,                                      // 53614:   case 0x7: {
  OPC_CheckPredicate, 78,                    // 53616:    check predicate 78
  OPC_Decode, 219, 59, 236, 2,               // 53618:    decode to STTPQpost using decoder 364
                                             // 53618:   }
                                             // 53618:   } // switch Inst[31:29]
                                             // 53618:  }
  3, 228, 5,                                 // 53623:  case 0x3: {
  OPC_SwitchField, 29, 3,                    // 53626:   switch Inst[31:29] {
  0, 200, 2,                                 // 53629:   case 0x0: {
  OPC_SwitchField, 10, 6,                    // 53632:    switch Inst[15:10] {
  0, 11,                                     // 53635:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 53637:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53639:     check Inst[21] == 0x0
  OPC_Decode, 230, 36, 251, 2,               // 53643:     decode to LD4Fourv8b_POST using decoder 379
                                             // 53643:    }
  1, 11,                                     // 53648:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 53650:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53652:     check Inst[21] == 0x0
  OPC_Decode, 226, 36, 251, 2,               // 53656:     decode to LD4Fourv4h_POST using decoder 379
                                             // 53656:    }
  2, 11,                                     // 53661:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 53663:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53665:     check Inst[21] == 0x0
  OPC_Decode, 224, 36, 251, 2,               // 53669:     decode to LD4Fourv2s_POST using decoder 379
                                             // 53669:    }
  8, 11,                                     // 53674:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 53676:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53678:     check Inst[21] == 0x0
  OPC_Decode, 218, 34, 251, 2,               // 53682:     decode to LD1Fourv8b_POST using decoder 379
                                             // 53682:    }
  9, 11,                                     // 53687:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 53689:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53691:     check Inst[21] == 0x0
  OPC_Decode, 214, 34, 251, 2,               // 53695:     decode to LD1Fourv4h_POST using decoder 379
                                             // 53695:    }
  10, 11,                                    // 53700:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 53702:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53704:     check Inst[21] == 0x0
  OPC_Decode, 212, 34, 251, 2,               // 53708:     decode to LD1Fourv2s_POST using decoder 379
                                             // 53708:    }
  11, 11,                                    // 53713:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 53715:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53717:     check Inst[21] == 0x0
  OPC_Decode, 208, 34, 251, 2,               // 53721:     decode to LD1Fourv1d_POST using decoder 379
                                             // 53721:    }
  16, 11,                                    // 53726:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 53728:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53730:     check Inst[21] == 0x0
  OPC_Decode, 202, 36, 252, 2,               // 53734:     decode to LD3Threev8b_POST using decoder 380
                                             // 53734:    }
  17, 11,                                    // 53739:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 53741:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53743:     check Inst[21] == 0x0
  OPC_Decode, 198, 36, 252, 2,               // 53747:     decode to LD3Threev4h_POST using decoder 380
                                             // 53747:    }
  18, 11,                                    // 53752:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 53754:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53756:     check Inst[21] == 0x0
  OPC_Decode, 196, 36, 252, 2,               // 53760:     decode to LD3Threev2s_POST using decoder 380
                                             // 53760:    }
  24, 11,                                    // 53765:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 53767:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53769:     check Inst[21] == 0x0
  OPC_Decode, 196, 35, 252, 2,               // 53773:     decode to LD1Threev8b_POST using decoder 380
                                             // 53773:    }
  25, 11,                                    // 53778:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 53780:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53782:     check Inst[21] == 0x0
  OPC_Decode, 192, 35, 252, 2,               // 53786:     decode to LD1Threev4h_POST using decoder 380
                                             // 53786:    }
  26, 11,                                    // 53791:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 53793:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53795:     check Inst[21] == 0x0
  OPC_Decode, 190, 35, 252, 2,               // 53799:     decode to LD1Threev2s_POST using decoder 380
                                             // 53799:    }
  27, 11,                                    // 53804:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 53806:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53808:     check Inst[21] == 0x0
  OPC_Decode, 186, 35, 252, 2,               // 53812:     decode to LD1Threev1d_POST using decoder 380
                                             // 53812:    }
  28, 11,                                    // 53817:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 53819:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53821:     check Inst[21] == 0x0
  OPC_Decode, 248, 34, 253, 2,               // 53825:     decode to LD1Onev8b_POST using decoder 381
                                             // 53825:    }
  29, 11,                                    // 53830:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 53832:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53834:     check Inst[21] == 0x0
  OPC_Decode, 244, 34, 253, 2,               // 53838:     decode to LD1Onev4h_POST using decoder 381
                                             // 53838:    }
  30, 11,                                    // 53843:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 53845:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53847:     check Inst[21] == 0x0
  OPC_Decode, 242, 34, 253, 2,               // 53851:     decode to LD1Onev2s_POST using decoder 381
                                             // 53851:    }
  31, 11,                                    // 53856:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 53858:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53860:     check Inst[21] == 0x0
  OPC_Decode, 238, 34, 253, 2,               // 53864:     decode to LD1Onev1d_POST using decoder 381
                                             // 53864:    }
  32, 11,                                    // 53869:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 53871:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53873:     check Inst[21] == 0x0
  OPC_Decode, 154, 36, 254, 2,               // 53877:     decode to LD2Twov8b_POST using decoder 382
                                             // 53877:    }
  33, 11,                                    // 53882:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 53884:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53886:     check Inst[21] == 0x0
  OPC_Decode, 150, 36, 254, 2,               // 53890:     decode to LD2Twov4h_POST using decoder 382
                                             // 53890:    }
  34, 11,                                    // 53895:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 53897:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53899:     check Inst[21] == 0x0
  OPC_Decode, 148, 36, 254, 2,               // 53903:     decode to LD2Twov2s_POST using decoder 382
                                             // 53903:    }
  40, 11,                                    // 53908:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 53910:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53912:     check Inst[21] == 0x0
  OPC_Decode, 212, 35, 254, 2,               // 53916:     decode to LD1Twov8b_POST using decoder 382
                                             // 53916:    }
  41, 11,                                    // 53921:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 53923:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53925:     check Inst[21] == 0x0
  OPC_Decode, 208, 35, 254, 2,               // 53929:     decode to LD1Twov4h_POST using decoder 382
                                             // 53929:    }
  42, 11,                                    // 53934:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 53936:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53938:     check Inst[21] == 0x0
  OPC_Decode, 206, 35, 254, 2,               // 53942:     decode to LD1Twov2s_POST using decoder 382
                                             // 53942:    }
  43, 0,                                     // 53947:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 53949:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53951:     check Inst[21] == 0x0
  OPC_Decode, 202, 35, 254, 2,               // 53955:     decode to LD1Twov1d_POST using decoder 382
                                             // 53955:    }
                                             // 53955:    } // switch Inst[15:10]
                                             // 53955:   }
  1, 7,                                      // 53960:   case 0x1: {
  OPC_CheckPredicate, 77,                    // 53962:    check predicate 77
  OPC_Decode, 156, 39, 236, 2,               // 53964:    decode to LDPSpost using decoder 364
                                             // 53964:   }
  2, 239, 2,                                 // 53969:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 53972:    switch Inst[15:10] {
  0, 11,                                     // 53975:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 53977:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53979:     check Inst[21] == 0x0
  OPC_Decode, 220, 36, 255, 2,               // 53983:     decode to LD4Fourv16b_POST using decoder 383
                                             // 53983:    }
  1, 11,                                     // 53988:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 53990:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 53992:     check Inst[21] == 0x0
  OPC_Decode, 232, 36, 255, 2,               // 53996:     decode to LD4Fourv8h_POST using decoder 383
                                             // 53996:    }
  2, 11,                                     // 54001:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 54003:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54005:     check Inst[21] == 0x0
  OPC_Decode, 228, 36, 255, 2,               // 54009:     decode to LD4Fourv4s_POST using decoder 383
                                             // 54009:    }
  3, 11,                                     // 54014:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 54016:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54018:     check Inst[21] == 0x0
  OPC_Decode, 222, 36, 255, 2,               // 54022:     decode to LD4Fourv2d_POST using decoder 383
                                             // 54022:    }
  8, 11,                                     // 54027:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 54029:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54031:     check Inst[21] == 0x0
  OPC_Decode, 206, 34, 255, 2,               // 54035:     decode to LD1Fourv16b_POST using decoder 383
                                             // 54035:    }
  9, 11,                                     // 54040:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 54042:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54044:     check Inst[21] == 0x0
  OPC_Decode, 220, 34, 255, 2,               // 54048:     decode to LD1Fourv8h_POST using decoder 383
                                             // 54048:    }
  10, 11,                                    // 54053:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 54055:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54057:     check Inst[21] == 0x0
  OPC_Decode, 216, 34, 255, 2,               // 54061:     decode to LD1Fourv4s_POST using decoder 383
                                             // 54061:    }
  11, 11,                                    // 54066:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 54068:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54070:     check Inst[21] == 0x0
  OPC_Decode, 210, 34, 255, 2,               // 54074:     decode to LD1Fourv2d_POST using decoder 383
                                             // 54074:    }
  16, 11,                                    // 54079:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 54081:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54083:     check Inst[21] == 0x0
  OPC_Decode, 192, 36, 128, 3,               // 54087:     decode to LD3Threev16b_POST using decoder 384
                                             // 54087:    }
  17, 11,                                    // 54092:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 54094:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54096:     check Inst[21] == 0x0
  OPC_Decode, 204, 36, 128, 3,               // 54100:     decode to LD3Threev8h_POST using decoder 384
                                             // 54100:    }
  18, 11,                                    // 54105:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 54107:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54109:     check Inst[21] == 0x0
  OPC_Decode, 200, 36, 128, 3,               // 54113:     decode to LD3Threev4s_POST using decoder 384
                                             // 54113:    }
  19, 11,                                    // 54118:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 54120:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54122:     check Inst[21] == 0x0
  OPC_Decode, 194, 36, 128, 3,               // 54126:     decode to LD3Threev2d_POST using decoder 384
                                             // 54126:    }
  24, 11,                                    // 54131:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 54133:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54135:     check Inst[21] == 0x0
  OPC_Decode, 184, 35, 128, 3,               // 54139:     decode to LD1Threev16b_POST using decoder 384
                                             // 54139:    }
  25, 11,                                    // 54144:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 54146:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54148:     check Inst[21] == 0x0
  OPC_Decode, 198, 35, 128, 3,               // 54152:     decode to LD1Threev8h_POST using decoder 384
                                             // 54152:    }
  26, 11,                                    // 54157:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 54159:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54161:     check Inst[21] == 0x0
  OPC_Decode, 194, 35, 128, 3,               // 54165:     decode to LD1Threev4s_POST using decoder 384
                                             // 54165:    }
  27, 11,                                    // 54170:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 54172:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54174:     check Inst[21] == 0x0
  OPC_Decode, 188, 35, 128, 3,               // 54178:     decode to LD1Threev2d_POST using decoder 384
                                             // 54178:    }
  28, 11,                                    // 54183:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 54185:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54187:     check Inst[21] == 0x0
  OPC_Decode, 236, 34, 129, 3,               // 54191:     decode to LD1Onev16b_POST using decoder 385
                                             // 54191:    }
  29, 11,                                    // 54196:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 54198:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54200:     check Inst[21] == 0x0
  OPC_Decode, 250, 34, 129, 3,               // 54204:     decode to LD1Onev8h_POST using decoder 385
                                             // 54204:    }
  30, 11,                                    // 54209:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 54211:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54213:     check Inst[21] == 0x0
  OPC_Decode, 246, 34, 129, 3,               // 54217:     decode to LD1Onev4s_POST using decoder 385
                                             // 54217:    }
  31, 11,                                    // 54222:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 54224:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54226:     check Inst[21] == 0x0
  OPC_Decode, 240, 34, 129, 3,               // 54230:     decode to LD1Onev2d_POST using decoder 385
                                             // 54230:    }
  32, 11,                                    // 54235:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 54237:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54239:     check Inst[21] == 0x0
  OPC_Decode, 144, 36, 130, 3,               // 54243:     decode to LD2Twov16b_POST using decoder 386
                                             // 54243:    }
  33, 11,                                    // 54248:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 54250:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54252:     check Inst[21] == 0x0
  OPC_Decode, 156, 36, 130, 3,               // 54256:     decode to LD2Twov8h_POST using decoder 386
                                             // 54256:    }
  34, 11,                                    // 54261:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 54263:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54265:     check Inst[21] == 0x0
  OPC_Decode, 152, 36, 130, 3,               // 54269:     decode to LD2Twov4s_POST using decoder 386
                                             // 54269:    }
  35, 11,                                    // 54274:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 54276:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54278:     check Inst[21] == 0x0
  OPC_Decode, 146, 36, 130, 3,               // 54282:     decode to LD2Twov2d_POST using decoder 386
                                             // 54282:    }
  40, 11,                                    // 54287:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 54289:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54291:     check Inst[21] == 0x0
  OPC_Decode, 200, 35, 130, 3,               // 54295:     decode to LD1Twov16b_POST using decoder 386
                                             // 54295:    }
  41, 11,                                    // 54300:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 54302:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54304:     check Inst[21] == 0x0
  OPC_Decode, 214, 35, 130, 3,               // 54308:     decode to LD1Twov8h_POST using decoder 386
                                             // 54308:    }
  42, 11,                                    // 54313:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 54315:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54317:     check Inst[21] == 0x0
  OPC_Decode, 210, 35, 130, 3,               // 54321:     decode to LD1Twov4s_POST using decoder 386
                                             // 54321:    }
  43, 0,                                     // 54326:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 54328:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 54330:     check Inst[21] == 0x0
  OPC_Decode, 204, 35, 130, 3,               // 54334:     decode to LD1Twov2d_POST using decoder 386
                                             // 54334:    }
                                             // 54334:    } // switch Inst[15:10]
                                             // 54334:   }
  3, 7,                                      // 54339:   case 0x3: {
  OPC_CheckPredicate, 77,                    // 54341:    check predicate 77
  OPC_Decode, 147, 39, 236, 2,               // 54343:    decode to LDPDpost using decoder 364
                                             // 54343:   }
  5, 7,                                      // 54348:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 54350:    check predicate 77
  OPC_Decode, 150, 39, 236, 2,               // 54352:    decode to LDPQpost using decoder 364
                                             // 54352:   }
  7, 0,                                      // 54357:   case 0x7: {
  OPC_CheckPredicate, 78,                    // 54359:    check predicate 78
  OPC_Decode, 191, 40, 236, 2,               // 54361:    decode to LDTPQpost using decoder 364
                                             // 54361:   }
                                             // 54361:   } // switch Inst[31:29]
                                             // 54361:  }
  4, 205, 2,                                 // 54366:  case 0x4: {
  OPC_SwitchField, 29, 1,                    // 54369:   switch Inst[29] {
  0, 158, 2,                                 // 54372:   case 0x0: {
  OPC_SwitchField, 13, 9,                    // 54375:    switch Inst[21:13] {
  0, 11,                                     // 54378:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 54380:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54382:     check Inst[31] == 0x0
  OPC_Decode, 168, 57, 131, 3,               // 54386:     decode to ST1i8 using decoder 387
                                             // 54386:    }
  1, 11,                                     // 54391:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 54393:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54395:     check Inst[31] == 0x0
  OPC_Decode, 235, 57, 132, 3,               // 54399:     decode to ST3i8 using decoder 388
                                             // 54399:    }
  2, 15,                                     // 54404:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 54406:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54408:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 54412:     check Inst[10] == 0x0
  OPC_Decode, 162, 57, 133, 3,               // 54416:     decode to ST1i16 using decoder 389
                                             // 54416:    }
  3, 15,                                     // 54421:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 54423:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54425:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 54429:     check Inst[10] == 0x0
  OPC_Decode, 229, 57, 134, 3,               // 54433:     decode to ST3i16 using decoder 390
                                             // 54433:    }
  4, 33,                                     // 54438:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 54440:     switch Inst[11:10] {
  0, 11,                                     // 54443:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 54445:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54447:      check Inst[31] == 0x0
  OPC_Decode, 164, 57, 135, 3,               // 54451:      decode to ST1i32 using decoder 391
                                             // 54451:     }
  1, 0,                                      // 54456:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 54458:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54460:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 54464:      check Inst[12] == 0x0
  OPC_Decode, 166, 57, 136, 3,               // 54468:      decode to ST1i64 using decoder 392
                                             // 54468:     }
                                             // 54468:     } // switch Inst[11:10]
                                             // 54468:    }
  5, 33,                                     // 54473:    case 0x5: {
  OPC_SwitchField, 10, 2,                    // 54475:     switch Inst[11:10] {
  0, 11,                                     // 54478:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 54480:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54482:      check Inst[31] == 0x0
  OPC_Decode, 231, 57, 137, 3,               // 54486:      decode to ST3i32 using decoder 393
                                             // 54486:     }
  1, 0,                                      // 54491:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 54493:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54495:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 54499:      check Inst[12] == 0x0
  OPC_Decode, 233, 57, 138, 3,               // 54503:      decode to ST3i64 using decoder 394
                                             // 54503:     }
                                             // 54503:     } // switch Inst[11:10]
                                             // 54503:    }
  12, 15,                                    // 54508:    case 0xc: {
  OPC_CheckPredicate, 79,                    // 54510:     check predicate 79
  OPC_CheckField, 31, 1, 0,                  // 54512:     check Inst[31] == 0x0
  OPC_CheckField, 10, 3, 1,                  // 54516:     check Inst[12:10] == 0x1
  OPC_Decode, 196, 58, 136, 3,               // 54520:     decode to STL1 using decoder 392
                                             // 54520:    }
  128, 2, 11,                                // 54525:    case 0x100: {
  OPC_CheckPredicate, 76,                    // 54528:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54530:     check Inst[31] == 0x0
  OPC_Decode, 203, 57, 139, 3,               // 54534:     decode to ST2i8 using decoder 395
                                             // 54534:    }
  129, 2, 11,                                // 54539:    case 0x101: {
  OPC_CheckPredicate, 76,                    // 54542:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54544:     check Inst[31] == 0x0
  OPC_Decode, 139, 58, 140, 3,               // 54548:     decode to ST4i8 using decoder 396
                                             // 54548:    }
  130, 2, 15,                                // 54553:    case 0x102: {
  OPC_CheckPredicate, 76,                    // 54556:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54558:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 54562:     check Inst[10] == 0x0
  OPC_Decode, 197, 57, 141, 3,               // 54566:     decode to ST2i16 using decoder 397
                                             // 54566:    }
  131, 2, 15,                                // 54571:    case 0x103: {
  OPC_CheckPredicate, 76,                    // 54574:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54576:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 54580:     check Inst[10] == 0x0
  OPC_Decode, 133, 58, 142, 3,               // 54584:     decode to ST4i16 using decoder 398
                                             // 54584:    }
  132, 2, 33,                                // 54589:    case 0x104: {
  OPC_SwitchField, 10, 2,                    // 54592:     switch Inst[11:10] {
  0, 11,                                     // 54595:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 54597:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54599:      check Inst[31] == 0x0
  OPC_Decode, 199, 57, 143, 3,               // 54603:      decode to ST2i32 using decoder 399
                                             // 54603:     }
  1, 0,                                      // 54608:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 54610:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54612:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 54616:      check Inst[12] == 0x0
  OPC_Decode, 201, 57, 144, 3,               // 54620:      decode to ST2i64 using decoder 400
                                             // 54620:     }
                                             // 54620:     } // switch Inst[11:10]
                                             // 54620:    }
  133, 2, 0,                                 // 54625:    case 0x105: {
  OPC_SwitchField, 10, 2,                    // 54628:     switch Inst[11:10] {
  0, 11,                                     // 54631:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 54633:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54635:      check Inst[31] == 0x0
  OPC_Decode, 135, 58, 145, 3,               // 54639:      decode to ST4i32 using decoder 401
                                             // 54639:     }
  1, 0,                                      // 54644:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 54646:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54648:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 54652:      check Inst[12] == 0x0
  OPC_Decode, 137, 58, 146, 3,               // 54656:      decode to ST4i64 using decoder 402
                                             // 54656:     }
                                             // 54656:     } // switch Inst[11:10]
                                             // 54656:    }
                                             // 54656:    } // switch Inst[21:13]
                                             // 54656:   }
  1, 0,                                      // 54661:   case 0x1: {
  OPC_SwitchField, 30, 2,                    // 54663:    switch Inst[31:30] {
  0, 7,                                      // 54666:    case 0x0: {
  OPC_CheckPredicate, 77,                    // 54668:     check predicate 77
  OPC_Decode, 157, 59, 236, 2,               // 54670:     decode to STPSi using decoder 364
                                             // 54670:    }
  1, 7,                                      // 54675:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 54677:     check predicate 77
  OPC_Decode, 151, 59, 236, 2,               // 54679:     decode to STPDi using decoder 364
                                             // 54679:    }
  2, 7,                                      // 54684:    case 0x2: {
  OPC_CheckPredicate, 77,                    // 54686:     check predicate 77
  OPC_Decode, 154, 59, 236, 2,               // 54688:     decode to STPQi using decoder 364
                                             // 54688:    }
  3, 0,                                      // 54693:    case 0x3: {
  OPC_CheckPredicate, 78,                    // 54695:     check predicate 78
  OPC_Decode, 218, 59, 236, 2,               // 54697:     decode to STTPQi using decoder 364
                                             // 54697:    }
                                             // 54697:    } // switch Inst[31:30]
                                             // 54697:   }
                                             // 54697:   } // switch Inst[29]
                                             // 54697:  }
  5, 211, 5,                                 // 54702:  case 0x5: {
  OPC_SwitchField, 29, 1,                    // 54705:   switch Inst[29] {
  0, 164, 5,                                 // 54708:   case 0x0: {
  OPC_SwitchField, 13, 9,                    // 54711:    switch Inst[21:13] {
  0, 11,                                     // 54714:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 54716:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54718:     check Inst[31] == 0x0
  OPC_Decode, 245, 35, 147, 3,               // 54722:     decode to LD1i8 using decoder 403
                                             // 54722:    }
  1, 11,                                     // 54727:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 54729:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54731:     check Inst[31] == 0x0
  OPC_Decode, 213, 36, 148, 3,               // 54735:     decode to LD3i8 using decoder 404
                                             // 54735:    }
  2, 15,                                     // 54740:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 54742:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54744:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 54748:     check Inst[10] == 0x0
  OPC_Decode, 239, 35, 149, 3,               // 54752:     decode to LD1i16 using decoder 405
                                             // 54752:    }
  3, 15,                                     // 54757:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 54759:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54761:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 54765:     check Inst[10] == 0x0
  OPC_Decode, 207, 36, 150, 3,               // 54769:     decode to LD3i16 using decoder 406
                                             // 54769:    }
  4, 33,                                     // 54774:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 54776:     switch Inst[11:10] {
  0, 11,                                     // 54779:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 54781:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54783:      check Inst[31] == 0x0
  OPC_Decode, 241, 35, 151, 3,               // 54787:      decode to LD1i32 using decoder 407
                                             // 54787:     }
  1, 0,                                      // 54792:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 54794:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54796:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 54800:      check Inst[12] == 0x0
  OPC_Decode, 243, 35, 152, 3,               // 54804:      decode to LD1i64 using decoder 408
                                             // 54804:     }
                                             // 54804:     } // switch Inst[11:10]
                                             // 54804:    }
  5, 33,                                     // 54809:    case 0x5: {
  OPC_SwitchField, 10, 2,                    // 54811:     switch Inst[11:10] {
  0, 11,                                     // 54814:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 54816:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54818:      check Inst[31] == 0x0
  OPC_Decode, 209, 36, 153, 3,               // 54822:      decode to LD3i32 using decoder 409
                                             // 54822:     }
  1, 0,                                      // 54827:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 54829:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 54831:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 54835:      check Inst[12] == 0x0
  OPC_Decode, 211, 36, 154, 3,               // 54839:      decode to LD3i64 using decoder 410
                                             // 54839:     }
                                             // 54839:     } // switch Inst[11:10]
                                             // 54839:    }
  6, 95,                                     // 54844:    case 0x6: {
  OPC_SwitchField, 10, 3,                    // 54846:     switch Inst[12:10] {
  0, 21,                                     // 54849:     case 0x0: {
  OPC_SwitchField, 30, 2,                    // 54851:      switch Inst[31:30] {
  0, 7,                                      // 54854:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54856:       check predicate 76
  OPC_Decode, 167, 35, 245, 2,               // 54858:       decode to LD1Rv8b using decoder 373
                                             // 54858:      }
  1, 0,                                      // 54863:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 54865:       check predicate 76
  OPC_Decode, 155, 35, 249, 2,               // 54867:       decode to LD1Rv16b using decoder 377
                                             // 54867:      }
                                             // 54867:      } // switch Inst[31:30]
                                             // 54867:     }
  1, 21,                                     // 54872:     case 0x1: {
  OPC_SwitchField, 30, 2,                    // 54874:      switch Inst[31:30] {
  0, 7,                                      // 54877:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54879:       check predicate 76
  OPC_Decode, 163, 35, 245, 2,               // 54881:       decode to LD1Rv4h using decoder 373
                                             // 54881:      }
  1, 0,                                      // 54886:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 54888:       check predicate 76
  OPC_Decode, 169, 35, 249, 2,               // 54890:       decode to LD1Rv8h using decoder 377
                                             // 54890:      }
                                             // 54890:      } // switch Inst[31:30]
                                             // 54890:     }
  2, 21,                                     // 54895:     case 0x2: {
  OPC_SwitchField, 30, 2,                    // 54897:      switch Inst[31:30] {
  0, 7,                                      // 54900:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54902:       check predicate 76
  OPC_Decode, 161, 35, 245, 2,               // 54904:       decode to LD1Rv2s using decoder 373
                                             // 54904:      }
  1, 0,                                      // 54909:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 54911:       check predicate 76
  OPC_Decode, 165, 35, 249, 2,               // 54913:       decode to LD1Rv4s using decoder 377
                                             // 54913:      }
                                             // 54913:      } // switch Inst[31:30]
                                             // 54913:     }
  3, 0,                                      // 54918:     case 0x3: {
  OPC_SwitchField, 30, 2,                    // 54920:      switch Inst[31:30] {
  0, 7,                                      // 54923:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54925:       check predicate 76
  OPC_Decode, 157, 35, 245, 2,               // 54927:       decode to LD1Rv1d using decoder 373
                                             // 54927:      }
  1, 0,                                      // 54932:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 54934:       check predicate 76
  OPC_Decode, 159, 35, 249, 2,               // 54936:       decode to LD1Rv2d using decoder 377
                                             // 54936:      }
                                             // 54936:      } // switch Inst[31:30]
                                             // 54936:     }
                                             // 54936:     } // switch Inst[12:10]
                                             // 54936:    }
  7, 95,                                     // 54941:    case 0x7: {
  OPC_SwitchField, 10, 3,                    // 54943:     switch Inst[12:10] {
  0, 21,                                     // 54946:     case 0x0: {
  OPC_SwitchField, 30, 2,                    // 54948:      switch Inst[31:30] {
  0, 7,                                      // 54951:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54953:       check predicate 76
  OPC_Decode, 187, 36, 244, 2,               // 54955:       decode to LD3Rv8b using decoder 372
                                             // 54955:      }
  1, 0,                                      // 54960:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 54962:       check predicate 76
  OPC_Decode, 175, 36, 248, 2,               // 54964:       decode to LD3Rv16b using decoder 376
                                             // 54964:      }
                                             // 54964:      } // switch Inst[31:30]
                                             // 54964:     }
  1, 21,                                     // 54969:     case 0x1: {
  OPC_SwitchField, 30, 2,                    // 54971:      switch Inst[31:30] {
  0, 7,                                      // 54974:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54976:       check predicate 76
  OPC_Decode, 183, 36, 244, 2,               // 54978:       decode to LD3Rv4h using decoder 372
                                             // 54978:      }
  1, 0,                                      // 54983:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 54985:       check predicate 76
  OPC_Decode, 189, 36, 248, 2,               // 54987:       decode to LD3Rv8h using decoder 376
                                             // 54987:      }
                                             // 54987:      } // switch Inst[31:30]
                                             // 54987:     }
  2, 21,                                     // 54992:     case 0x2: {
  OPC_SwitchField, 30, 2,                    // 54994:      switch Inst[31:30] {
  0, 7,                                      // 54997:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 54999:       check predicate 76
  OPC_Decode, 181, 36, 244, 2,               // 55001:       decode to LD3Rv2s using decoder 372
                                             // 55001:      }
  1, 0,                                      // 55006:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55008:       check predicate 76
  OPC_Decode, 185, 36, 248, 2,               // 55010:       decode to LD3Rv4s using decoder 376
                                             // 55010:      }
                                             // 55010:      } // switch Inst[31:30]
                                             // 55010:     }
  3, 0,                                      // 55015:     case 0x3: {
  OPC_SwitchField, 30, 2,                    // 55017:      switch Inst[31:30] {
  0, 7,                                      // 55020:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55022:       check predicate 76
  OPC_Decode, 177, 36, 244, 2,               // 55024:       decode to LD3Rv1d using decoder 372
                                             // 55024:      }
  1, 0,                                      // 55029:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55031:       check predicate 76
  OPC_Decode, 179, 36, 248, 2,               // 55033:       decode to LD3Rv2d using decoder 376
                                             // 55033:      }
                                             // 55033:      } // switch Inst[31:30]
                                             // 55033:     }
                                             // 55033:     } // switch Inst[12:10]
                                             // 55033:    }
  12, 15,                                    // 55038:    case 0xc: {
  OPC_CheckPredicate, 79,                    // 55040:     check predicate 79
  OPC_CheckField, 31, 1, 0,                  // 55042:     check Inst[31] == 0x0
  OPC_CheckField, 10, 3, 1,                  // 55046:     check Inst[12:10] == 0x1
  OPC_Decode, 152, 37, 152, 3,               // 55050:     decode to LDAP1 using decoder 408
                                             // 55050:    }
  128, 2, 11,                                // 55055:    case 0x100: {
  OPC_CheckPredicate, 76,                    // 55058:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55060:     check Inst[31] == 0x0
  OPC_Decode, 165, 36, 155, 3,               // 55064:     decode to LD2i8 using decoder 411
                                             // 55064:    }
  129, 2, 11,                                // 55069:    case 0x101: {
  OPC_CheckPredicate, 76,                    // 55072:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55074:     check Inst[31] == 0x0
  OPC_Decode, 133, 37, 156, 3,               // 55078:     decode to LD4i8 using decoder 412
                                             // 55078:    }
  130, 2, 15,                                // 55083:    case 0x102: {
  OPC_CheckPredicate, 76,                    // 55086:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55088:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55092:     check Inst[10] == 0x0
  OPC_Decode, 159, 36, 157, 3,               // 55096:     decode to LD2i16 using decoder 413
                                             // 55096:    }
  131, 2, 15,                                // 55101:    case 0x103: {
  OPC_CheckPredicate, 76,                    // 55104:     check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55106:     check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55110:     check Inst[10] == 0x0
  OPC_Decode, 255, 36, 158, 3,               // 55114:     decode to LD4i16 using decoder 414
                                             // 55114:    }
  132, 2, 33,                                // 55119:    case 0x104: {
  OPC_SwitchField, 10, 2,                    // 55122:     switch Inst[11:10] {
  0, 11,                                     // 55125:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55127:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55129:      check Inst[31] == 0x0
  OPC_Decode, 161, 36, 159, 3,               // 55133:      decode to LD2i32 using decoder 415
                                             // 55133:     }
  1, 0,                                      // 55138:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55140:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55142:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55146:      check Inst[12] == 0x0
  OPC_Decode, 163, 36, 160, 3,               // 55150:      decode to LD2i64 using decoder 416
                                             // 55150:     }
                                             // 55150:     } // switch Inst[11:10]
                                             // 55150:    }
  133, 2, 33,                                // 55155:    case 0x105: {
  OPC_SwitchField, 10, 2,                    // 55158:     switch Inst[11:10] {
  0, 11,                                     // 55161:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55163:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55165:      check Inst[31] == 0x0
  OPC_Decode, 129, 37, 161, 3,               // 55169:      decode to LD4i32 using decoder 417
                                             // 55169:     }
  1, 0,                                      // 55174:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55176:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55178:      check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55182:      check Inst[12] == 0x0
  OPC_Decode, 131, 37, 162, 3,               // 55186:      decode to LD4i64 using decoder 418
                                             // 55186:     }
                                             // 55186:     } // switch Inst[11:10]
                                             // 55186:    }
  134, 2, 95,                                // 55191:    case 0x106: {
  OPC_SwitchField, 10, 3,                    // 55194:     switch Inst[12:10] {
  0, 21,                                     // 55197:     case 0x0: {
  OPC_SwitchField, 30, 2,                    // 55199:      switch Inst[31:30] {
  0, 7,                                      // 55202:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55204:       check predicate 76
  OPC_Decode, 139, 36, 246, 2,               // 55206:       decode to LD2Rv8b using decoder 374
                                             // 55206:      }
  1, 0,                                      // 55211:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55213:       check predicate 76
  OPC_Decode, 255, 35, 250, 2,               // 55215:       decode to LD2Rv16b using decoder 378
                                             // 55215:      }
                                             // 55215:      } // switch Inst[31:30]
                                             // 55215:     }
  1, 21,                                     // 55220:     case 0x1: {
  OPC_SwitchField, 30, 2,                    // 55222:      switch Inst[31:30] {
  0, 7,                                      // 55225:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55227:       check predicate 76
  OPC_Decode, 135, 36, 246, 2,               // 55229:       decode to LD2Rv4h using decoder 374
                                             // 55229:      }
  1, 0,                                      // 55234:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55236:       check predicate 76
  OPC_Decode, 141, 36, 250, 2,               // 55238:       decode to LD2Rv8h using decoder 378
                                             // 55238:      }
                                             // 55238:      } // switch Inst[31:30]
                                             // 55238:     }
  2, 21,                                     // 55243:     case 0x2: {
  OPC_SwitchField, 30, 2,                    // 55245:      switch Inst[31:30] {
  0, 7,                                      // 55248:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55250:       check predicate 76
  OPC_Decode, 133, 36, 246, 2,               // 55252:       decode to LD2Rv2s using decoder 374
                                             // 55252:      }
  1, 0,                                      // 55257:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55259:       check predicate 76
  OPC_Decode, 137, 36, 250, 2,               // 55261:       decode to LD2Rv4s using decoder 378
                                             // 55261:      }
                                             // 55261:      } // switch Inst[31:30]
                                             // 55261:     }
  3, 0,                                      // 55266:     case 0x3: {
  OPC_SwitchField, 30, 2,                    // 55268:      switch Inst[31:30] {
  0, 7,                                      // 55271:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55273:       check predicate 76
  OPC_Decode, 129, 36, 246, 2,               // 55275:       decode to LD2Rv1d using decoder 374
                                             // 55275:      }
  1, 0,                                      // 55280:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55282:       check predicate 76
  OPC_Decode, 131, 36, 250, 2,               // 55284:       decode to LD2Rv2d using decoder 378
                                             // 55284:      }
                                             // 55284:      } // switch Inst[31:30]
                                             // 55284:     }
                                             // 55284:     } // switch Inst[12:10]
                                             // 55284:    }
  135, 2, 0,                                 // 55289:    case 0x107: {
  OPC_SwitchField, 10, 3,                    // 55292:     switch Inst[12:10] {
  0, 21,                                     // 55295:     case 0x0: {
  OPC_SwitchField, 30, 2,                    // 55297:      switch Inst[31:30] {
  0, 7,                                      // 55300:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55302:       check predicate 76
  OPC_Decode, 249, 36, 243, 2,               // 55304:       decode to LD4Rv8b using decoder 371
                                             // 55304:      }
  1, 0,                                      // 55309:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55311:       check predicate 76
  OPC_Decode, 237, 36, 247, 2,               // 55313:       decode to LD4Rv16b using decoder 375
                                             // 55313:      }
                                             // 55313:      } // switch Inst[31:30]
                                             // 55313:     }
  1, 21,                                     // 55318:     case 0x1: {
  OPC_SwitchField, 30, 2,                    // 55320:      switch Inst[31:30] {
  0, 7,                                      // 55323:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55325:       check predicate 76
  OPC_Decode, 245, 36, 243, 2,               // 55327:       decode to LD4Rv4h using decoder 371
                                             // 55327:      }
  1, 0,                                      // 55332:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55334:       check predicate 76
  OPC_Decode, 251, 36, 247, 2,               // 55336:       decode to LD4Rv8h using decoder 375
                                             // 55336:      }
                                             // 55336:      } // switch Inst[31:30]
                                             // 55336:     }
  2, 21,                                     // 55341:     case 0x2: {
  OPC_SwitchField, 30, 2,                    // 55343:      switch Inst[31:30] {
  0, 7,                                      // 55346:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55348:       check predicate 76
  OPC_Decode, 243, 36, 243, 2,               // 55350:       decode to LD4Rv2s using decoder 371
                                             // 55350:      }
  1, 0,                                      // 55355:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55357:       check predicate 76
  OPC_Decode, 247, 36, 247, 2,               // 55359:       decode to LD4Rv4s using decoder 375
                                             // 55359:      }
                                             // 55359:      } // switch Inst[31:30]
                                             // 55359:     }
  3, 0,                                      // 55364:     case 0x3: {
  OPC_SwitchField, 30, 2,                    // 55366:      switch Inst[31:30] {
  0, 7,                                      // 55369:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55371:       check predicate 76
  OPC_Decode, 239, 36, 243, 2,               // 55373:       decode to LD4Rv1d using decoder 371
                                             // 55373:      }
  1, 0,                                      // 55378:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55380:       check predicate 76
  OPC_Decode, 241, 36, 247, 2,               // 55382:       decode to LD4Rv2d using decoder 375
                                             // 55382:      }
                                             // 55382:      } // switch Inst[31:30]
                                             // 55382:     }
                                             // 55382:     } // switch Inst[12:10]
                                             // 55382:    }
                                             // 55382:    } // switch Inst[21:13]
                                             // 55382:   }
  1, 0,                                      // 55387:   case 0x1: {
  OPC_SwitchField, 30, 2,                    // 55389:    switch Inst[31:30] {
  0, 7,                                      // 55392:    case 0x0: {
  OPC_CheckPredicate, 77,                    // 55394:     check predicate 77
  OPC_Decode, 155, 39, 236, 2,               // 55396:     decode to LDPSi using decoder 364
                                             // 55396:    }
  1, 7,                                      // 55401:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 55403:     check predicate 77
  OPC_Decode, 146, 39, 236, 2,               // 55405:     decode to LDPDi using decoder 364
                                             // 55405:    }
  2, 7,                                      // 55410:    case 0x2: {
  OPC_CheckPredicate, 77,                    // 55412:     check predicate 77
  OPC_Decode, 149, 39, 236, 2,               // 55414:     decode to LDPQi using decoder 364
                                             // 55414:    }
  3, 0,                                      // 55419:    case 0x3: {
  OPC_CheckPredicate, 78,                    // 55421:     check predicate 78
  OPC_Decode, 190, 40, 236, 2,               // 55423:     decode to LDTPQi using decoder 364
                                             // 55423:    }
                                             // 55423:    } // switch Inst[31:30]
                                             // 55423:   }
                                             // 55423:   } // switch Inst[29]
                                             // 55423:  }
  6, 212, 2,                                 // 55428:  case 0x6: {
  OPC_SwitchField, 29, 1,                    // 55431:   switch Inst[29] {
  0, 165, 2,                                 // 55434:   case 0x0: {
  OPC_SwitchField, 13, 3,                    // 55437:    switch Inst[15:13] {
  0, 29,                                     // 55440:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 55442:     switch Inst[21] {
  0, 11,                                     // 55445:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55447:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55449:      check Inst[31] == 0x0
  OPC_Decode, 169, 57, 163, 3,               // 55453:      decode to ST1i8_POST using decoder 419
                                             // 55453:     }
  1, 0,                                      // 55458:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55460:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55462:      check Inst[31] == 0x0
  OPC_Decode, 204, 57, 164, 3,               // 55466:      decode to ST2i8_POST using decoder 420
                                             // 55466:     }
                                             // 55466:     } // switch Inst[21]
                                             // 55466:    }
  1, 29,                                     // 55471:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 55473:     switch Inst[21] {
  0, 11,                                     // 55476:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55478:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55480:      check Inst[31] == 0x0
  OPC_Decode, 236, 57, 165, 3,               // 55484:      decode to ST3i8_POST using decoder 421
                                             // 55484:     }
  1, 0,                                      // 55489:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55491:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55493:      check Inst[31] == 0x0
  OPC_Decode, 140, 58, 166, 3,               // 55497:      decode to ST4i8_POST using decoder 422
                                             // 55497:     }
                                             // 55497:     } // switch Inst[21]
                                             // 55497:    }
  2, 37,                                     // 55502:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 55504:     switch Inst[21] {
  0, 15,                                     // 55507:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55509:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55511:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55515:      check Inst[10] == 0x0
  OPC_Decode, 163, 57, 167, 3,               // 55519:      decode to ST1i16_POST using decoder 423
                                             // 55519:     }
  1, 0,                                      // 55524:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55526:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55528:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55532:      check Inst[10] == 0x0
  OPC_Decode, 198, 57, 168, 3,               // 55536:      decode to ST2i16_POST using decoder 424
                                             // 55536:     }
                                             // 55536:     } // switch Inst[21]
                                             // 55536:    }
  3, 37,                                     // 55541:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 55543:     switch Inst[21] {
  0, 15,                                     // 55546:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55548:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55550:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55554:      check Inst[10] == 0x0
  OPC_Decode, 230, 57, 169, 3,               // 55558:      decode to ST3i16_POST using decoder 425
                                             // 55558:     }
  1, 0,                                      // 55563:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55565:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55567:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55571:      check Inst[10] == 0x0
  OPC_Decode, 134, 58, 170, 3,               // 55575:      decode to ST4i16_POST using decoder 426
                                             // 55575:     }
                                             // 55575:     } // switch Inst[21]
                                             // 55575:    }
  4, 73,                                     // 55580:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 55582:     switch Inst[11:10] {
  0, 29,                                     // 55585:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 55587:      switch Inst[21] {
  0, 11,                                     // 55590:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55592:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55594:       check Inst[31] == 0x0
  OPC_Decode, 165, 57, 171, 3,               // 55598:       decode to ST1i32_POST using decoder 427
                                             // 55598:      }
  1, 0,                                      // 55603:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55605:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55607:       check Inst[31] == 0x0
  OPC_Decode, 200, 57, 172, 3,               // 55611:       decode to ST2i32_POST using decoder 428
                                             // 55611:      }
                                             // 55611:      } // switch Inst[21]
                                             // 55611:     }
  1, 0,                                      // 55616:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 55618:      switch Inst[21] {
  0, 15,                                     // 55621:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55623:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55625:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55629:       check Inst[12] == 0x0
  OPC_Decode, 167, 57, 173, 3,               // 55633:       decode to ST1i64_POST using decoder 429
                                             // 55633:      }
  1, 0,                                      // 55638:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55640:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55642:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55646:       check Inst[12] == 0x0
  OPC_Decode, 202, 57, 174, 3,               // 55650:       decode to ST2i64_POST using decoder 430
                                             // 55650:      }
                                             // 55650:      } // switch Inst[21]
                                             // 55650:     }
                                             // 55650:     } // switch Inst[11:10]
                                             // 55650:    }
  5, 0,                                      // 55655:    case 0x5: {
  OPC_SwitchField, 10, 2,                    // 55657:     switch Inst[11:10] {
  0, 29,                                     // 55660:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 55662:      switch Inst[21] {
  0, 11,                                     // 55665:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55667:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55669:       check Inst[31] == 0x0
  OPC_Decode, 232, 57, 175, 3,               // 55673:       decode to ST3i32_POST using decoder 431
                                             // 55673:      }
  1, 0,                                      // 55678:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55680:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55682:       check Inst[31] == 0x0
  OPC_Decode, 136, 58, 176, 3,               // 55686:       decode to ST4i32_POST using decoder 432
                                             // 55686:      }
                                             // 55686:      } // switch Inst[21]
                                             // 55686:     }
  1, 0,                                      // 55691:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 55693:      switch Inst[21] {
  0, 15,                                     // 55696:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55698:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55700:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55704:       check Inst[12] == 0x0
  OPC_Decode, 234, 57, 177, 3,               // 55708:       decode to ST3i64_POST using decoder 433
                                             // 55708:      }
  1, 0,                                      // 55713:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55715:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55717:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55721:       check Inst[12] == 0x0
  OPC_Decode, 138, 58, 178, 3,               // 55725:       decode to ST4i64_POST using decoder 434
                                             // 55725:      }
                                             // 55725:      } // switch Inst[21]
                                             // 55725:     }
                                             // 55725:     } // switch Inst[11:10]
                                             // 55725:    }
                                             // 55725:    } // switch Inst[15:13]
                                             // 55725:   }
  1, 0,                                      // 55730:   case 0x1: {
  OPC_SwitchField, 30, 2,                    // 55732:    switch Inst[31:30] {
  0, 7,                                      // 55735:    case 0x0: {
  OPC_CheckPredicate, 77,                    // 55737:     check predicate 77
  OPC_Decode, 159, 59, 236, 2,               // 55739:     decode to STPSpre using decoder 364
                                             // 55739:    }
  1, 7,                                      // 55744:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 55746:     check predicate 77
  OPC_Decode, 153, 59, 236, 2,               // 55748:     decode to STPDpre using decoder 364
                                             // 55748:    }
  2, 7,                                      // 55753:    case 0x2: {
  OPC_CheckPredicate, 77,                    // 55755:     check predicate 77
  OPC_Decode, 156, 59, 236, 2,               // 55757:     decode to STPQpre using decoder 364
                                             // 55757:    }
  3, 0,                                      // 55762:    case 0x3: {
  OPC_CheckPredicate, 78,                    // 55764:     check predicate 78
  OPC_Decode, 220, 59, 236, 2,               // 55766:     decode to STTPQpre using decoder 364
                                             // 55766:    }
                                             // 55766:    } // switch Inst[31:30]
                                             // 55766:   }
                                             // 55766:   } // switch Inst[29]
                                             // 55766:  }
  7, 247, 5,                                 // 55771:  case 0x7: {
  OPC_SwitchField, 29, 1,                    // 55774:   switch Inst[29] {
  0, 200, 5,                                 // 55777:   case 0x0: {
  OPC_SwitchField, 13, 3,                    // 55780:    switch Inst[15:13] {
  0, 29,                                     // 55783:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 55785:     switch Inst[21] {
  0, 11,                                     // 55788:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55790:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55792:      check Inst[31] == 0x0
  OPC_Decode, 246, 35, 179, 3,               // 55796:      decode to LD1i8_POST using decoder 435
                                             // 55796:     }
  1, 0,                                      // 55801:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55803:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55805:      check Inst[31] == 0x0
  OPC_Decode, 166, 36, 180, 3,               // 55809:      decode to LD2i8_POST using decoder 436
                                             // 55809:     }
                                             // 55809:     } // switch Inst[21]
                                             // 55809:    }
  1, 29,                                     // 55814:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 55816:     switch Inst[21] {
  0, 11,                                     // 55819:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55821:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55823:      check Inst[31] == 0x0
  OPC_Decode, 214, 36, 181, 3,               // 55827:      decode to LD3i8_POST using decoder 437
                                             // 55827:     }
  1, 0,                                      // 55832:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55834:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55836:      check Inst[31] == 0x0
  OPC_Decode, 134, 37, 182, 3,               // 55840:      decode to LD4i8_POST using decoder 438
                                             // 55840:     }
                                             // 55840:     } // switch Inst[21]
                                             // 55840:    }
  2, 37,                                     // 55845:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 55847:     switch Inst[21] {
  0, 15,                                     // 55850:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55852:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55854:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55858:      check Inst[10] == 0x0
  OPC_Decode, 240, 35, 183, 3,               // 55862:      decode to LD1i16_POST using decoder 439
                                             // 55862:     }
  1, 0,                                      // 55867:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55869:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55871:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55875:      check Inst[10] == 0x0
  OPC_Decode, 160, 36, 184, 3,               // 55879:      decode to LD2i16_POST using decoder 440
                                             // 55879:     }
                                             // 55879:     } // switch Inst[21]
                                             // 55879:    }
  3, 37,                                     // 55884:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 55886:     switch Inst[21] {
  0, 15,                                     // 55889:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 55891:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55893:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55897:      check Inst[10] == 0x0
  OPC_Decode, 208, 36, 185, 3,               // 55901:      decode to LD3i16_POST using decoder 441
                                             // 55901:     }
  1, 0,                                      // 55906:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 55908:      check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55910:      check Inst[31] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 55914:      check Inst[10] == 0x0
  OPC_Decode, 128, 37, 186, 3,               // 55918:      decode to LD4i16_POST using decoder 442
                                             // 55918:     }
                                             // 55918:     } // switch Inst[21]
                                             // 55918:    }
  4, 73,                                     // 55923:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 55925:     switch Inst[11:10] {
  0, 29,                                     // 55928:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 55930:      switch Inst[21] {
  0, 11,                                     // 55933:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55935:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55937:       check Inst[31] == 0x0
  OPC_Decode, 242, 35, 187, 3,               // 55941:       decode to LD1i32_POST using decoder 443
                                             // 55941:      }
  1, 0,                                      // 55946:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55948:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55950:       check Inst[31] == 0x0
  OPC_Decode, 162, 36, 188, 3,               // 55954:       decode to LD2i32_POST using decoder 444
                                             // 55954:      }
                                             // 55954:      } // switch Inst[21]
                                             // 55954:     }
  1, 0,                                      // 55959:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 55961:      switch Inst[21] {
  0, 15,                                     // 55964:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 55966:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55968:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55972:       check Inst[12] == 0x0
  OPC_Decode, 244, 35, 189, 3,               // 55976:       decode to LD1i64_POST using decoder 445
                                             // 55976:      }
  1, 0,                                      // 55981:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 55983:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 55985:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 55989:       check Inst[12] == 0x0
  OPC_Decode, 164, 36, 190, 3,               // 55993:       decode to LD2i64_POST using decoder 446
                                             // 55993:      }
                                             // 55993:      } // switch Inst[21]
                                             // 55993:     }
                                             // 55993:     } // switch Inst[11:10]
                                             // 55993:    }
  5, 73,                                     // 55998:    case 0x5: {
  OPC_SwitchField, 10, 2,                    // 56000:     switch Inst[11:10] {
  0, 29,                                     // 56003:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 56005:      switch Inst[21] {
  0, 11,                                     // 56008:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 56010:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 56012:       check Inst[31] == 0x0
  OPC_Decode, 210, 36, 191, 3,               // 56016:       decode to LD3i32_POST using decoder 447
                                             // 56016:      }
  1, 0,                                      // 56021:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 56023:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 56025:       check Inst[31] == 0x0
  OPC_Decode, 130, 37, 192, 3,               // 56029:       decode to LD4i32_POST using decoder 448
                                             // 56029:      }
                                             // 56029:      } // switch Inst[21]
                                             // 56029:     }
  1, 0,                                      // 56034:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 56036:      switch Inst[21] {
  0, 15,                                     // 56039:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 56041:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 56043:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 56047:       check Inst[12] == 0x0
  OPC_Decode, 212, 36, 193, 3,               // 56051:       decode to LD3i64_POST using decoder 449
                                             // 56051:      }
  1, 0,                                      // 56056:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 56058:       check predicate 76
  OPC_CheckField, 31, 1, 0,                  // 56060:       check Inst[31] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 56064:       check Inst[12] == 0x0
  OPC_Decode, 132, 37, 194, 3,               // 56068:       decode to LD4i64_POST using decoder 450
                                             // 56068:      }
                                             // 56068:      } // switch Inst[21]
                                             // 56068:     }
                                             // 56068:     } // switch Inst[11:10]
                                             // 56068:    }
  6, 207, 1,                                 // 56073:    case 0x6: {
  OPC_SwitchField, 10, 3,                    // 56076:     switch Inst[12:10] {
  0, 49,                                     // 56079:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 56081:      switch Inst[21] {
  0, 21,                                     // 56084:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56086:       switch Inst[31:30] {
  0, 7,                                      // 56089:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56091:        check predicate 76
  OPC_Decode, 168, 35, 253, 2,               // 56093:        decode to LD1Rv8b_POST using decoder 381
                                             // 56093:       }
  1, 0,                                      // 56098:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56100:        check predicate 76
  OPC_Decode, 156, 35, 129, 3,               // 56102:        decode to LD1Rv16b_POST using decoder 385
                                             // 56102:       }
                                             // 56102:       } // switch Inst[31:30]
                                             // 56102:      }
  1, 0,                                      // 56107:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56109:       switch Inst[31:30] {
  0, 7,                                      // 56112:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56114:        check predicate 76
  OPC_Decode, 140, 36, 254, 2,               // 56116:        decode to LD2Rv8b_POST using decoder 382
                                             // 56116:       }
  1, 0,                                      // 56121:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56123:        check predicate 76
  OPC_Decode, 128, 36, 130, 3,               // 56125:        decode to LD2Rv16b_POST using decoder 386
                                             // 56125:       }
                                             // 56125:       } // switch Inst[31:30]
                                             // 56125:      }
                                             // 56125:      } // switch Inst[21]
                                             // 56125:     }
  1, 49,                                     // 56130:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 56132:      switch Inst[21] {
  0, 21,                                     // 56135:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56137:       switch Inst[31:30] {
  0, 7,                                      // 56140:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56142:        check predicate 76
  OPC_Decode, 164, 35, 253, 2,               // 56144:        decode to LD1Rv4h_POST using decoder 381
                                             // 56144:       }
  1, 0,                                      // 56149:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56151:        check predicate 76
  OPC_Decode, 170, 35, 129, 3,               // 56153:        decode to LD1Rv8h_POST using decoder 385
                                             // 56153:       }
                                             // 56153:       } // switch Inst[31:30]
                                             // 56153:      }
  1, 0,                                      // 56158:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56160:       switch Inst[31:30] {
  0, 7,                                      // 56163:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56165:        check predicate 76
  OPC_Decode, 136, 36, 254, 2,               // 56167:        decode to LD2Rv4h_POST using decoder 382
                                             // 56167:       }
  1, 0,                                      // 56172:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56174:        check predicate 76
  OPC_Decode, 142, 36, 130, 3,               // 56176:        decode to LD2Rv8h_POST using decoder 386
                                             // 56176:       }
                                             // 56176:       } // switch Inst[31:30]
                                             // 56176:      }
                                             // 56176:      } // switch Inst[21]
                                             // 56176:     }
  2, 49,                                     // 56181:     case 0x2: {
  OPC_SwitchField, 21, 1,                    // 56183:      switch Inst[21] {
  0, 21,                                     // 56186:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56188:       switch Inst[31:30] {
  0, 7,                                      // 56191:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56193:        check predicate 76
  OPC_Decode, 162, 35, 253, 2,               // 56195:        decode to LD1Rv2s_POST using decoder 381
                                             // 56195:       }
  1, 0,                                      // 56200:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56202:        check predicate 76
  OPC_Decode, 166, 35, 129, 3,               // 56204:        decode to LD1Rv4s_POST using decoder 385
                                             // 56204:       }
                                             // 56204:       } // switch Inst[31:30]
                                             // 56204:      }
  1, 0,                                      // 56209:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56211:       switch Inst[31:30] {
  0, 7,                                      // 56214:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56216:        check predicate 76
  OPC_Decode, 134, 36, 254, 2,               // 56218:        decode to LD2Rv2s_POST using decoder 382
                                             // 56218:       }
  1, 0,                                      // 56223:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56225:        check predicate 76
  OPC_Decode, 138, 36, 130, 3,               // 56227:        decode to LD2Rv4s_POST using decoder 386
                                             // 56227:       }
                                             // 56227:       } // switch Inst[31:30]
                                             // 56227:      }
                                             // 56227:      } // switch Inst[21]
                                             // 56227:     }
  3, 0,                                      // 56232:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 56234:      switch Inst[21] {
  0, 21,                                     // 56237:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56239:       switch Inst[31:30] {
  0, 7,                                      // 56242:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56244:        check predicate 76
  OPC_Decode, 158, 35, 253, 2,               // 56246:        decode to LD1Rv1d_POST using decoder 381
                                             // 56246:       }
  1, 0,                                      // 56251:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56253:        check predicate 76
  OPC_Decode, 160, 35, 129, 3,               // 56255:        decode to LD1Rv2d_POST using decoder 385
                                             // 56255:       }
                                             // 56255:       } // switch Inst[31:30]
                                             // 56255:      }
  1, 0,                                      // 56260:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56262:       switch Inst[31:30] {
  0, 7,                                      // 56265:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56267:        check predicate 76
  OPC_Decode, 130, 36, 254, 2,               // 56269:        decode to LD2Rv1d_POST using decoder 382
                                             // 56269:       }
  1, 0,                                      // 56274:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56276:        check predicate 76
  OPC_Decode, 132, 36, 130, 3,               // 56278:        decode to LD2Rv2d_POST using decoder 386
                                             // 56278:       }
                                             // 56278:       } // switch Inst[31:30]
                                             // 56278:      }
                                             // 56278:      } // switch Inst[21]
                                             // 56278:     }
                                             // 56278:     } // switch Inst[12:10]
                                             // 56278:    }
  7, 0,                                      // 56283:    case 0x7: {
  OPC_SwitchField, 10, 3,                    // 56285:     switch Inst[12:10] {
  0, 49,                                     // 56288:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 56290:      switch Inst[21] {
  0, 21,                                     // 56293:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56295:       switch Inst[31:30] {
  0, 7,                                      // 56298:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56300:        check predicate 76
  OPC_Decode, 188, 36, 252, 2,               // 56302:        decode to LD3Rv8b_POST using decoder 380
                                             // 56302:       }
  1, 0,                                      // 56307:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56309:        check predicate 76
  OPC_Decode, 176, 36, 128, 3,               // 56311:        decode to LD3Rv16b_POST using decoder 384
                                             // 56311:       }
                                             // 56311:       } // switch Inst[31:30]
                                             // 56311:      }
  1, 0,                                      // 56316:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56318:       switch Inst[31:30] {
  0, 7,                                      // 56321:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56323:        check predicate 76
  OPC_Decode, 250, 36, 251, 2,               // 56325:        decode to LD4Rv8b_POST using decoder 379
                                             // 56325:       }
  1, 0,                                      // 56330:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56332:        check predicate 76
  OPC_Decode, 238, 36, 255, 2,               // 56334:        decode to LD4Rv16b_POST using decoder 383
                                             // 56334:       }
                                             // 56334:       } // switch Inst[31:30]
                                             // 56334:      }
                                             // 56334:      } // switch Inst[21]
                                             // 56334:     }
  1, 49,                                     // 56339:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 56341:      switch Inst[21] {
  0, 21,                                     // 56344:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56346:       switch Inst[31:30] {
  0, 7,                                      // 56349:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56351:        check predicate 76
  OPC_Decode, 184, 36, 252, 2,               // 56353:        decode to LD3Rv4h_POST using decoder 380
                                             // 56353:       }
  1, 0,                                      // 56358:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56360:        check predicate 76
  OPC_Decode, 190, 36, 128, 3,               // 56362:        decode to LD3Rv8h_POST using decoder 384
                                             // 56362:       }
                                             // 56362:       } // switch Inst[31:30]
                                             // 56362:      }
  1, 0,                                      // 56367:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56369:       switch Inst[31:30] {
  0, 7,                                      // 56372:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56374:        check predicate 76
  OPC_Decode, 246, 36, 251, 2,               // 56376:        decode to LD4Rv4h_POST using decoder 379
                                             // 56376:       }
  1, 0,                                      // 56381:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56383:        check predicate 76
  OPC_Decode, 252, 36, 255, 2,               // 56385:        decode to LD4Rv8h_POST using decoder 383
                                             // 56385:       }
                                             // 56385:       } // switch Inst[31:30]
                                             // 56385:      }
                                             // 56385:      } // switch Inst[21]
                                             // 56385:     }
  2, 49,                                     // 56390:     case 0x2: {
  OPC_SwitchField, 21, 1,                    // 56392:      switch Inst[21] {
  0, 21,                                     // 56395:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56397:       switch Inst[31:30] {
  0, 7,                                      // 56400:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56402:        check predicate 76
  OPC_Decode, 182, 36, 252, 2,               // 56404:        decode to LD3Rv2s_POST using decoder 380
                                             // 56404:       }
  1, 0,                                      // 56409:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56411:        check predicate 76
  OPC_Decode, 186, 36, 128, 3,               // 56413:        decode to LD3Rv4s_POST using decoder 384
                                             // 56413:       }
                                             // 56413:       } // switch Inst[31:30]
                                             // 56413:      }
  1, 0,                                      // 56418:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56420:       switch Inst[31:30] {
  0, 7,                                      // 56423:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56425:        check predicate 76
  OPC_Decode, 244, 36, 251, 2,               // 56427:        decode to LD4Rv2s_POST using decoder 379
                                             // 56427:       }
  1, 0,                                      // 56432:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56434:        check predicate 76
  OPC_Decode, 248, 36, 255, 2,               // 56436:        decode to LD4Rv4s_POST using decoder 383
                                             // 56436:       }
                                             // 56436:       } // switch Inst[31:30]
                                             // 56436:      }
                                             // 56436:      } // switch Inst[21]
                                             // 56436:     }
  3, 0,                                      // 56441:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 56443:      switch Inst[21] {
  0, 21,                                     // 56446:      case 0x0: {
  OPC_SwitchField, 30, 2,                    // 56448:       switch Inst[31:30] {
  0, 7,                                      // 56451:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56453:        check predicate 76
  OPC_Decode, 178, 36, 252, 2,               // 56455:        decode to LD3Rv1d_POST using decoder 380
                                             // 56455:       }
  1, 0,                                      // 56460:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56462:        check predicate 76
  OPC_Decode, 180, 36, 128, 3,               // 56464:        decode to LD3Rv2d_POST using decoder 384
                                             // 56464:       }
                                             // 56464:       } // switch Inst[31:30]
                                             // 56464:      }
  1, 0,                                      // 56469:      case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56471:       switch Inst[31:30] {
  0, 7,                                      // 56474:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56476:        check predicate 76
  OPC_Decode, 240, 36, 251, 2,               // 56478:        decode to LD4Rv1d_POST using decoder 379
                                             // 56478:       }
  1, 0,                                      // 56483:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56485:        check predicate 76
  OPC_Decode, 242, 36, 255, 2,               // 56487:        decode to LD4Rv2d_POST using decoder 383
                                             // 56487:       }
                                             // 56487:       } // switch Inst[31:30]
                                             // 56487:      }
                                             // 56487:      } // switch Inst[21]
                                             // 56487:     }
                                             // 56487:     } // switch Inst[12:10]
                                             // 56487:    }
                                             // 56487:    } // switch Inst[15:13]
                                             // 56487:   }
  1, 0,                                      // 56492:   case 0x1: {
  OPC_SwitchField, 30, 2,                    // 56494:    switch Inst[31:30] {
  0, 7,                                      // 56497:    case 0x0: {
  OPC_CheckPredicate, 77,                    // 56499:     check predicate 77
  OPC_Decode, 157, 39, 236, 2,               // 56501:     decode to LDPSpre using decoder 364
                                             // 56501:    }
  1, 7,                                      // 56506:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 56508:     check predicate 77
  OPC_Decode, 148, 39, 236, 2,               // 56510:     decode to LDPDpre using decoder 364
                                             // 56510:    }
  2, 7,                                      // 56515:    case 0x2: {
  OPC_CheckPredicate, 77,                    // 56517:     check predicate 77
  OPC_Decode, 151, 39, 236, 2,               // 56519:     decode to LDPQpre using decoder 364
                                             // 56519:    }
  3, 0,                                      // 56524:    case 0x3: {
  OPC_CheckPredicate, 78,                    // 56526:     check predicate 78
  OPC_Decode, 192, 40, 236, 2,               // 56528:     decode to LDTPQpre using decoder 364
                                             // 56528:    }
                                             // 56528:    } // switch Inst[31:30]
                                             // 56528:   }
                                             // 56528:   } // switch Inst[29]
                                             // 56528:  }
  8, 158, 35,                                // 56533:  case 0x8: {
  OPC_SwitchField, 29, 3,                    // 56536:   switch Inst[31:29] {
  0, 168, 10,                                // 56539:   case 0x0: {
  OPC_SwitchField, 10, 6,                    // 56542:    switch Inst[15:10] {
  0, 21,                                     // 56545:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 56547:     switch Inst[21] {
  0, 7,                                      // 56550:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56552:      check predicate 76
  OPC_Decode, 198, 61, 195, 3,               // 56554:      decode to TBLv8i8One using decoder 451
                                             // 56554:     }
  1, 0,                                      // 56559:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56561:      check predicate 76
  OPC_Decode, 210, 46, 196, 3,               // 56563:      decode to SADDLv8i8_v8i16 using decoder 452
                                             // 56563:     }
                                             // 56563:     } // switch Inst[21]
                                             // 56563:    }
  1, 53,                                     // 56568:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 56570:     switch Inst[21] {
  0, 39,                                     // 56573:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 56575:      switch Inst[16] {
  0, 25,                                     // 56578:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 56580:       switch Inst[17] {
  0, 11,                                     // 56583:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56585:        check predicate 76
  OPC_CheckField, 18, 1, 1,                  // 56587:        check Inst[18] == 0x1
  OPC_Decode, 227, 20, 197, 3,               // 56591:        decode to DUPv2i32lane using decoder 453
                                             // 56591:       }
  1, 0,                                      // 56596:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56598:        check predicate 76
  OPC_Decode, 231, 20, 198, 3,               // 56600:        decode to DUPv4i16lane using decoder 454
                                             // 56600:       }
                                             // 56600:       } // switch Inst[17]
                                             // 56600:      }
  1, 0,                                      // 56605:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 56607:       check predicate 76
  OPC_Decode, 237, 20, 199, 3,               // 56609:       decode to DUPv8i8lane using decoder 455
                                             // 56609:      }
                                             // 56609:      } // switch Inst[16]
                                             // 56609:     }
  1, 0,                                      // 56614:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56616:      check predicate 76
  OPC_Decode, 154, 48, 200, 3,               // 56618:      decode to SHADDv8i8 using decoder 456
                                             // 56618:     }
                                             // 56618:     } // switch Inst[21]
                                             // 56618:    }
  2, 11,                                     // 56623:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 56625:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 56627:     check Inst[21:16] == 0x20
  OPC_Decode, 204, 45, 201, 3,               // 56631:     decode to REV64v8i8 using decoder 457
                                             // 56631:    }
  3, 53,                                     // 56636:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 56638:     switch Inst[21] {
  0, 39,                                     // 56641:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 56643:      switch Inst[16] {
  0, 25,                                     // 56646:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 56648:       switch Inst[17] {
  0, 11,                                     // 56651:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 56653:        check predicate 76
  OPC_CheckField, 18, 1, 1,                  // 56655:        check Inst[18] == 0x1
  OPC_Decode, 226, 20, 202, 3,               // 56659:        decode to DUPv2i32gpr using decoder 458
                                             // 56659:       }
  1, 0,                                      // 56664:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 56666:        check predicate 76
  OPC_Decode, 230, 20, 202, 3,               // 56668:        decode to DUPv4i16gpr using decoder 458
                                             // 56668:       }
                                             // 56668:       } // switch Inst[17]
                                             // 56668:      }
  1, 0,                                      // 56673:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 56675:       check predicate 76
  OPC_Decode, 236, 20, 202, 3,               // 56677:       decode to DUPv8i8gpr using decoder 458
                                             // 56677:      }
                                             // 56677:      } // switch Inst[16]
                                             // 56677:     }
  1, 0,                                      // 56682:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56684:      check predicate 76
  OPC_Decode, 160, 51, 200, 3,               // 56686:      decode to SQADDv8i8 using decoder 456
                                             // 56686:     }
                                             // 56686:     } // switch Inst[21]
                                             // 56686:    }
  4, 21,                                     // 56691:    case 0x4: {
  OPC_SwitchField, 21, 1,                    // 56693:     switch Inst[21] {
  0, 7,                                      // 56696:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56698:      check predicate 76
  OPC_Decode, 216, 61, 203, 3,               // 56700:      decode to TBXv8i8One using decoder 459
                                             // 56700:     }
  1, 0,                                      // 56705:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56707:      check predicate 76
  OPC_Decode, 225, 46, 204, 3,               // 56709:      decode to SADDWv8i8_v8i16 using decoder 460
                                             // 56709:     }
                                             // 56709:     } // switch Inst[21]
                                             // 56709:    }
  5, 11,                                     // 56714:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 56716:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 56718:     check Inst[21] == 0x1
  OPC_Decode, 238, 54, 200, 3,               // 56722:     decode to SRHADDv8i8 using decoder 456
                                             // 56722:    }
  6, 25,                                     // 56727:    case 0x6: {
  OPC_SwitchField, 21, 1,                    // 56729:     switch Inst[21] {
  0, 7,                                      // 56732:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56734:      check predicate 76
  OPC_Decode, 182, 69, 200, 3,               // 56736:      decode to UZP1v8i8 using decoder 456
                                             // 56736:     }
  1, 0,                                      // 56741:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56743:      check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 56745:      check Inst[20:16] == 0x0
  OPC_Decode, 193, 45, 201, 3,               // 56749:      decode to REV16v8i8 using decoder 457
                                             // 56749:     }
                                             // 56749:     } // switch Inst[21]
                                             // 56749:    }
  7, 11,                                     // 56754:    case 0x7: {
  OPC_CheckPredicate, 76,                    // 56756:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 56758:     check Inst[21] == 0x1
  OPC_Decode, 172, 14, 200, 3,               // 56762:     decode to ANDv8i8 using decoder 456
                                             // 56762:    }
  8, 21,                                     // 56767:    case 0x8: {
  OPC_SwitchField, 21, 1,                    // 56769:     switch Inst[21] {
  0, 7,                                      // 56772:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56774:      check predicate 76
  OPC_Decode, 200, 61, 205, 3,               // 56776:      decode to TBLv8i8Two using decoder 461
                                             // 56776:     }
  1, 0,                                      // 56781:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56783:      check predicate 76
  OPC_Decode, 147, 56, 196, 3,               // 56785:      decode to SSUBLv8i8_v8i16 using decoder 452
                                             // 56785:     }
                                             // 56785:     } // switch Inst[21]
                                             // 56785:    }
  9, 11,                                     // 56790:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 56792:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 56794:     check Inst[21] == 0x1
  OPC_Decode, 194, 48, 200, 3,               // 56798:     decode to SHSUBv8i8 using decoder 456
                                             // 56798:    }
  10, 35,                                    // 56803:    case 0xa: {
  OPC_SwitchField, 21, 1,                    // 56805:     switch Inst[21] {
  0, 7,                                      // 56808:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56810:      check predicate 76
  OPC_Decode, 243, 61, 200, 3,               // 56812:      decode to TRN1v8i8 using decoder 456
                                             // 56812:     }
  1, 0,                                      // 56817:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 56819:      switch Inst[20:16] {
  0, 7,                                      // 56822:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 56824:       check predicate 76
  OPC_Decode, 196, 46, 201, 3,               // 56826:       decode to SADDLPv8i8_v4i16 using decoder 457
                                             // 56826:      }
  1, 0,                                      // 56831:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 56833:       check predicate 76
  OPC_Decode, 242, 70, 206, 3,               // 56835:       decode to XTNv8i8 using decoder 462
                                             // 56835:      }
                                             // 56835:      } // switch Inst[20:16]
                                             // 56835:     }
                                             // 56835:     } // switch Inst[21]
                                             // 56835:    }
  11, 65,                                    // 56840:    case 0xb: {
  OPC_SwitchField, 21, 1,                    // 56842:     switch Inst[21] {
  0, 51,                                     // 56845:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 56847:      switch Inst[16] {
  0, 24,                                     // 56850:      case 0x0: {
  OPC_CheckField, 17, 1, 1,                  // 56852:       check Inst[17] == 0x1
  OPC_Scope, 11,                             // 56856:       try {
  OPC_CheckField, 18, 3, 0,                  // 56858:        check Inst[20:18] == 0x0
  OPC_CheckPredicate, 76,                    // 56862:        check predicate 76
  OPC_Decode, 200, 50, 207, 3,               // 56864:        decode to SMOVvi16to32_idx0 using decoder 463
                                             // 56864:       } else try {
  OPC_CheckPredicate, 76,                    // 56869:        check predicate 76
  OPC_Decode, 199, 50, 208, 3,               // 56871:        decode to SMOVvi16to32 using decoder 464
                                             // 56871:       }
                                             // 56871:      }
  1, 0,                                      // 56876:      case 0x1: {
  OPC_Scope, 11,                             // 56878:       try {
  OPC_CheckField, 17, 4, 0,                  // 56880:        check Inst[20:17] == 0x0
  OPC_CheckPredicate, 76,                    // 56884:        check predicate 76
  OPC_Decode, 206, 50, 207, 3,               // 56886:        decode to SMOVvi8to32_idx0 using decoder 463
                                             // 56886:       } else try {
  OPC_CheckPredicate, 76,                    // 56891:        check predicate 76
  OPC_Decode, 205, 50, 209, 3,               // 56893:        decode to SMOVvi8to32 using decoder 465
                                             // 56893:       }
                                             // 56893:      }
                                             // 56893:      } // switch Inst[16]
                                             // 56893:     }
  1, 0,                                      // 56898:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56900:      check predicate 76
  OPC_Decode, 198, 54, 200, 3,               // 56902:      decode to SQSUBv8i8 using decoder 456
                                             // 56902:     }
                                             // 56902:     } // switch Inst[21]
                                             // 56902:    }
  12, 21,                                    // 56907:    case 0xc: {
  OPC_SwitchField, 21, 1,                    // 56909:     switch Inst[21] {
  0, 7,                                      // 56912:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56914:      check predicate 76
  OPC_Decode, 218, 61, 210, 3,               // 56916:      decode to TBXv8i8Two using decoder 466
                                             // 56916:     }
  1, 0,                                      // 56921:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 56923:      check predicate 76
  OPC_Decode, 159, 56, 204, 3,               // 56925:      decode to SSUBWv8i8_v8i16 using decoder 460
                                             // 56925:     }
                                             // 56925:     } // switch Inst[21]
                                             // 56925:    }
  13, 11,                                    // 56930:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 56932:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 56934:     check Inst[21] == 0x1
  OPC_Decode, 253, 17, 200, 3,               // 56938:     decode to CMGTv8i8 using decoder 456
                                             // 56938:    }
  14, 35,                                    // 56943:    case 0xe: {
  OPC_SwitchField, 21, 1,                    // 56945:     switch Inst[21] {
  0, 7,                                      // 56948:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 56950:      check predicate 76
  OPC_Decode, 140, 71, 200, 3,               // 56952:      decode to ZIP1v8i8 using decoder 456
                                             // 56952:     }
  1, 0,                                      // 56957:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 56959:      switch Inst[20:16] {
  0, 7,                                      // 56962:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 56964:       check predicate 76
  OPC_Decode, 130, 61, 211, 3,               // 56966:       decode to SUQADDv8i8 using decoder 467
                                             // 56966:      }
  16, 0,                                     // 56971:      case 0x10: {
  OPC_CheckPredicate, 76,                    // 56973:       check predicate 76
  OPC_Decode, 204, 46, 212, 3,               // 56975:       decode to SADDLVv8i8v using decoder 468
                                             // 56975:      }
                                             // 56975:      } // switch Inst[20:16]
                                             // 56975:     }
                                             // 56975:     } // switch Inst[21]
                                             // 56975:    }
  15, 92,                                    // 56980:    case 0xf: {
  OPC_SwitchField, 21, 1,                    // 56982:     switch Inst[21] {
  0, 78,                                     // 56985:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 56987:      switch Inst[16] {
  0, 51,                                     // 56990:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 56992:       switch Inst[17] {
  0, 24,                                     // 56995:       case 0x0: {
  OPC_CheckField, 18, 1, 1,                  // 56997:        check Inst[18] == 0x1
  OPC_Scope, 11,                             // 57001:        try {
  OPC_CheckField, 19, 2, 0,                  // 57003:         check Inst[20:19] == 0x0
  OPC_CheckPredicate, 76,                    // 57007:         check predicate 76
  OPC_Decode, 211, 65, 207, 3,               // 57009:         decode to UMOVvi32_idx0 using decoder 463
                                             // 57009:        } else try {
  OPC_CheckPredicate, 76,                    // 57014:         check predicate 76
  OPC_Decode, 210, 65, 213, 3,               // 57016:         decode to UMOVvi32 using decoder 469
                                             // 57016:        }
                                             // 57016:       }
  1, 0,                                      // 57021:       case 0x1: {
  OPC_Scope, 11,                             // 57023:        try {
  OPC_CheckField, 18, 3, 0,                  // 57025:         check Inst[20:18] == 0x0
  OPC_CheckPredicate, 76,                    // 57029:         check predicate 76
  OPC_Decode, 209, 65, 207, 3,               // 57031:         decode to UMOVvi16_idx0 using decoder 463
                                             // 57031:        } else try {
  OPC_CheckPredicate, 76,                    // 57036:         check predicate 76
  OPC_Decode, 208, 65, 208, 3,               // 57038:         decode to UMOVvi16 using decoder 464
                                             // 57038:        }
                                             // 57038:       }
                                             // 57038:       } // switch Inst[17]
                                             // 57038:      }
  1, 0,                                      // 57043:      case 0x1: {
  OPC_Scope, 11,                             // 57045:       try {
  OPC_CheckField, 17, 4, 0,                  // 57047:        check Inst[20:17] == 0x0
  OPC_CheckPredicate, 76,                    // 57051:        check predicate 76
  OPC_Decode, 215, 65, 207, 3,               // 57053:        decode to UMOVvi8_idx0 using decoder 463
                                             // 57053:       } else try {
  OPC_CheckPredicate, 76,                    // 57058:        check predicate 76
  OPC_Decode, 214, 65, 209, 3,               // 57060:        decode to UMOVvi8 using decoder 465
                                             // 57060:       }
                                             // 57060:      }
                                             // 57060:      } // switch Inst[16]
                                             // 57060:     }
  1, 0,                                      // 57065:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57067:      check predicate 76
  OPC_Decode, 237, 17, 200, 3,               // 57069:      decode to CMGEv8i8 using decoder 456
                                             // 57069:     }
                                             // 57069:     } // switch Inst[21]
                                             // 57069:    }
  16, 21,                                    // 57074:    case 0x10: {
  OPC_SwitchField, 21, 1,                    // 57076:     switch Inst[21] {
  0, 7,                                      // 57079:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57081:      check predicate 76
  OPC_Decode, 199, 61, 214, 3,               // 57083:      decode to TBLv8i8Three using decoder 470
                                             // 57083:     }
  1, 0,                                      // 57088:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57090:      check predicate 76
  OPC_Decode, 148, 13, 215, 3,               // 57092:      decode to ADDHNv8i16_v8i8 using decoder 471
                                             // 57092:     }
                                             // 57092:     } // switch Inst[21]
                                             // 57092:    }
  17, 11,                                    // 57097:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 57099:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57101:     check Inst[21] == 0x1
  OPC_Decode, 198, 55, 200, 3,               // 57105:     decode to SSHLv8i8 using decoder 456
                                             // 57105:    }
  18, 21,                                    // 57110:    case 0x12: {
  OPC_SwitchField, 16, 6,                    // 57112:     switch Inst[21:16] {
  32, 7,                                     // 57115:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 57117:      check predicate 76
  OPC_Decode, 190, 17, 201, 3,               // 57119:      decode to CLSv8i8 using decoder 457
                                             // 57119:     }
  33, 0,                                     // 57124:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 57126:      check predicate 76
  OPC_Decode, 213, 54, 206, 3,               // 57128:      decode to SQXTNv8i8 using decoder 462
                                             // 57128:     }
                                             // 57128:     } // switch Inst[21:16]
                                             // 57128:    }
  19, 11,                                    // 57133:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 57135:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57137:     check Inst[21] == 0x1
  OPC_Decode, 136, 54, 200, 3,               // 57141:     decode to SQSHLv8i8 using decoder 456
                                             // 57141:    }
  20, 21,                                    // 57146:    case 0x14: {
  OPC_SwitchField, 21, 1,                    // 57148:     switch Inst[21] {
  0, 7,                                      // 57151:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57153:      check predicate 76
  OPC_Decode, 217, 61, 216, 3,               // 57155:      decode to TBXv8i8Three using decoder 472
                                             // 57155:     }
  1, 0,                                      // 57160:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57162:      check predicate 76
  OPC_Decode, 143, 46, 217, 3,               // 57164:      decode to SABALv8i8_v8i16 using decoder 473
                                             // 57164:     }
                                             // 57164:     } // switch Inst[21]
                                             // 57164:    }
  21, 11,                                    // 57169:    case 0x15: {
  OPC_CheckPredicate, 76,                    // 57171:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57173:     check Inst[21] == 0x1
  OPC_Decode, 154, 55, 200, 3,               // 57177:     decode to SRSHLv8i8 using decoder 456
                                             // 57177:    }
  22, 25,                                    // 57182:    case 0x16: {
  OPC_SwitchField, 21, 1,                    // 57184:     switch Inst[21] {
  0, 7,                                      // 57187:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57189:      check predicate 76
  OPC_Decode, 198, 69, 200, 3,               // 57191:      decode to UZP2v8i8 using decoder 456
                                             // 57191:     }
  1, 0,                                      // 57196:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57198:      check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 57200:      check Inst[20:16] == 0x0
  OPC_Decode, 170, 19, 201, 3,               // 57204:      decode to CNTv8i8 using decoder 457
                                             // 57204:     }
                                             // 57204:     } // switch Inst[21]
                                             // 57204:    }
  23, 11,                                    // 57209:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 57211:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57213:     check Inst[21] == 0x1
  OPC_Decode, 172, 53, 200, 3,               // 57217:     decode to SQRSHLv8i8 using decoder 456
                                             // 57217:    }
  24, 21,                                    // 57222:    case 0x18: {
  OPC_SwitchField, 21, 1,                    // 57224:     switch Inst[21] {
  0, 7,                                      // 57227:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57229:      check predicate 76
  OPC_Decode, 197, 61, 218, 3,               // 57231:      decode to TBLv8i8Four using decoder 474
                                             // 57231:     }
  1, 0,                                      // 57236:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57238:      check predicate 76
  OPC_Decode, 136, 60, 215, 3,               // 57240:      decode to SUBHNv8i16_v8i8 using decoder 471
                                             // 57240:     }
                                             // 57240:     } // switch Inst[21]
                                             // 57240:    }
  25, 11,                                    // 57245:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 57247:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57249:     check Inst[21] == 0x1
  OPC_Decode, 148, 49, 200, 3,               // 57253:     decode to SMAXv8i8 using decoder 456
                                             // 57253:    }
  26, 35,                                    // 57258:    case 0x1a: {
  OPC_SwitchField, 21, 1,                    // 57260:     switch Inst[21] {
  0, 7,                                      // 57263:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57265:      check predicate 76
  OPC_Decode, 131, 62, 200, 3,               // 57267:      decode to TRN2v8i8 using decoder 456
                                             // 57267:     }
  1, 0,                                      // 57272:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 57274:      switch Inst[20:16] {
  0, 7,                                      // 57277:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 57279:       check predicate 76
  OPC_Decode, 184, 46, 211, 3,               // 57281:       decode to SADALPv8i8_v4i16 using decoder 467
                                             // 57281:      }
  1, 0,                                      // 57286:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 57288:       check predicate 76
  OPC_Decode, 200, 24, 206, 3,               // 57290:       decode to FCVTNv4i16 using decoder 462
                                             // 57290:      }
                                             // 57290:      } // switch Inst[20:16]
                                             // 57290:     }
                                             // 57290:     } // switch Inst[21]
                                             // 57290:    }
  27, 11,                                    // 57295:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 57297:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57299:     check Inst[21] == 0x1
  OPC_Decode, 206, 49, 200, 3,               // 57303:     decode to SMINv8i8 using decoder 456
                                             // 57303:    }
  28, 21,                                    // 57308:    case 0x1c: {
  OPC_SwitchField, 21, 1,                    // 57310:     switch Inst[21] {
  0, 7,                                      // 57313:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57315:      check predicate 76
  OPC_Decode, 215, 61, 219, 3,               // 57317:      decode to TBXv8i8Four using decoder 475
                                             // 57317:     }
  1, 0,                                      // 57322:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57324:      check predicate 76
  OPC_Decode, 165, 46, 196, 3,               // 57326:      decode to SABDLv8i8_v8i16 using decoder 452
                                             // 57326:     }
                                             // 57326:     } // switch Inst[21]
                                             // 57326:    }
  29, 11,                                    // 57331:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 57333:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57335:     check Inst[21] == 0x1
  OPC_Decode, 175, 46, 200, 3,               // 57339:     decode to SABDv8i8 using decoder 456
                                             // 57339:    }
  30, 35,                                    // 57344:    case 0x1e: {
  OPC_SwitchField, 21, 1,                    // 57346:     switch Inst[21] {
  0, 7,                                      // 57349:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 57351:      check predicate 76
  OPC_Decode, 156, 71, 200, 3,               // 57353:      decode to ZIP2v8i8 using decoder 456
                                             // 57353:     }
  1, 0,                                      // 57358:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 57360:      switch Inst[20:16] {
  0, 7,                                      // 57363:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 57365:       check predicate 76
  OPC_Decode, 137, 51, 201, 3,               // 57367:       decode to SQABSv8i8 using decoder 457
                                             // 57367:      }
  1, 0,                                      // 57372:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 57374:       check predicate 76
  OPC_Decode, 239, 23, 220, 3,               // 57376:       decode to FCVTLv4i16 using decoder 476
                                             // 57376:      }
                                             // 57376:      } // switch Inst[20:16]
                                             // 57376:     }
                                             // 57376:     } // switch Inst[21]
                                             // 57376:    }
  31, 11,                                    // 57381:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 57383:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57385:     check Inst[21] == 0x1
  OPC_Decode, 153, 46, 221, 3,               // 57389:     decode to SABAv8i8 using decoder 477
                                             // 57389:    }
  32, 11,                                    // 57394:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 57396:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57398:     check Inst[21] == 0x1
  OPC_Decode, 250, 49, 217, 3,               // 57402:     decode to SMLALv8i8_v8i16 using decoder 473
                                             // 57402:    }
  33, 11,                                    // 57407:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 57409:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57411:     check Inst[21] == 0x1
  OPC_Decode, 240, 13, 200, 3,               // 57415:     decode to ADDv8i8 using decoder 456
                                             // 57415:    }
  34, 21,                                    // 57420:    case 0x22: {
  OPC_SwitchField, 16, 6,                    // 57422:     switch Inst[21:16] {
  32, 7,                                     // 57425:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 57427:      check predicate 76
  OPC_Decode, 254, 17, 201, 3,               // 57429:      decode to CMGTv8i8rz using decoder 457
                                             // 57429:     }
  33, 0,                                     // 57434:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 57436:      check predicate 76
  OPC_Decode, 186, 31, 201, 3,               // 57438:      decode to FRINTNv2f32 using decoder 457
                                             // 57438:     }
                                             // 57438:     } // switch Inst[21:16]
                                             // 57438:    }
  35, 11,                                    // 57443:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 57445:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57447:     check Inst[21] == 0x1
  OPC_Decode, 138, 19, 200, 3,               // 57451:     decode to CMTSTv8i8 using decoder 456
                                             // 57451:    }
  37, 11,                                    // 57456:    case 0x25: {
  OPC_CheckPredicate, 76,                    // 57458:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57460:     check Inst[21] == 0x1
  OPC_Decode, 128, 42, 221, 3,               // 57464:     decode to MLAv8i8 using decoder 477
                                             // 57464:    }
  38, 21,                                    // 57469:    case 0x26: {
  OPC_SwitchField, 16, 6,                    // 57471:     switch Inst[21:16] {
  32, 7,                                     // 57474:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 57476:      check predicate 76
  OPC_Decode, 222, 17, 201, 3,               // 57478:      decode to CMEQv8i8rz using decoder 457
                                             // 57478:     }
  33, 0,                                     // 57483:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 57485:      check predicate 76
  OPC_Decode, 170, 31, 201, 3,               // 57487:      decode to FRINTMv2f32 using decoder 457
                                             // 57487:     }
                                             // 57487:     } // switch Inst[21:16]
                                             // 57487:    }
  39, 11,                                    // 57492:    case 0x27: {
  OPC_CheckPredicate, 76,                    // 57494:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57496:     check Inst[21] == 0x1
  OPC_Decode, 152, 43, 200, 3,               // 57500:     decode to MULv8i8 using decoder 456
                                             // 57500:    }
  40, 11,                                    // 57505:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 57507:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57509:     check Inst[21] == 0x1
  OPC_Decode, 166, 50, 217, 3,               // 57513:     decode to SMLSLv8i8_v8i16 using decoder 473
                                             // 57513:    }
  41, 11,                                    // 57518:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 57520:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57522:     check Inst[21] == 0x1
  OPC_Decode, 229, 48, 200, 3,               // 57526:     decode to SMAXPv8i8 using decoder 456
                                             // 57526:    }
  42, 39,                                    // 57531:    case 0x2a: {
  OPC_SwitchField, 16, 6,                    // 57533:     switch Inst[21:16] {
  32, 7,                                     // 57536:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 57538:      check predicate 76
  OPC_Decode, 164, 18, 201, 3,               // 57540:      decode to CMLTv8i8rz using decoder 457
                                             // 57540:     }
  33, 7,                                     // 57545:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 57547:      check predicate 76
  OPC_Decode, 164, 24, 201, 3,               // 57549:      decode to FCVTNSv2f32 using decoder 457
                                             // 57549:     }
  48, 7,                                     // 57554:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 57556:      check predicate 76
  OPC_Decode, 242, 48, 222, 3,               // 57558:      decode to SMAXVv8i8v using decoder 478
                                             // 57558:     }
  49, 0,                                     // 57563:     case 0x31: {
  OPC_CheckPredicate, 76,                    // 57565:      check predicate 76
  OPC_Decode, 172, 49, 222, 3,               // 57567:      decode to SMINVv8i8v using decoder 478
                                             // 57567:     }
                                             // 57567:     } // switch Inst[21:16]
                                             // 57567:    }
  43, 11,                                    // 57572:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 57574:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57576:     check Inst[21] == 0x1
  OPC_Decode, 159, 49, 200, 3,               // 57580:     decode to SMINPv8i8 using decoder 456
                                             // 57580:    }
  46, 30,                                    // 57585:    case 0x2e: {
  OPC_SwitchField, 16, 6,                    // 57587:     switch Inst[21:16] {
  32, 7,                                     // 57590:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 57592:      check predicate 76
  OPC_Decode, 253, 12, 201, 3,               // 57594:      decode to ABSv8i8 using decoder 457
                                             // 57594:     }
  33, 7,                                     // 57599:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 57601:      check predicate 76
  OPC_Decode, 255, 23, 201, 3,               // 57603:      decode to FCVTMSv2f32 using decoder 457
                                             // 57603:     }
  49, 0,                                     // 57608:     case 0x31: {
  OPC_CheckPredicate, 76,                    // 57610:      check predicate 76
  OPC_Decode, 191, 13, 222, 3,               // 57612:      decode to ADDVv8i8v using decoder 478
                                             // 57612:     }
                                             // 57612:     } // switch Inst[21:16]
                                             // 57612:    }
  47, 11,                                    // 57617:    case 0x2f: {
  OPC_CheckPredicate, 76,                    // 57619:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57621:     check Inst[21] == 0x1
  OPC_Decode, 162, 13, 200, 3,               // 57625:     decode to ADDPv8i8 using decoder 456
                                             // 57625:    }
  48, 11,                                    // 57630:    case 0x30: {
  OPC_CheckPredicate, 76,                    // 57632:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57634:     check Inst[21] == 0x1
  OPC_Decode, 238, 50, 196, 3,               // 57638:     decode to SMULLv8i8_v8i16 using decoder 452
                                             // 57638:    }
  49, 21,                                    // 57643:    case 0x31: {
  OPC_SwitchField, 21, 1,                    // 57645:     switch Inst[21] {
  0, 7,                                      // 57648:     case 0x0: {
  OPC_CheckPredicate, 80,                    // 57650:      check predicate 80
  OPC_Decode, 252, 27, 223, 3,               // 57652:      decode to FMLALLBBv4f32 using decoder 479
                                             // 57652:     }
  1, 0,                                      // 57657:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57659:      check predicate 76
  OPC_Decode, 226, 26, 200, 3,               // 57661:      decode to FMAXNMv2f32 using decoder 456
                                             // 57661:     }
                                             // 57661:     } // switch Inst[21]
                                             // 57661:    }
  50, 21,                                    // 57666:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 57668:     switch Inst[21:16] {
  33, 7,                                     // 57671:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 57673:      check predicate 76
  OPC_Decode, 206, 23, 201, 3,               // 57675:      decode to FCVTASv2f32 using decoder 457
                                             // 57675:     }
  48, 0,                                     // 57680:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 57682:      check predicate 81
  OPC_Decode, 205, 26, 212, 3,               // 57684:      decode to FMAXNMVv4i16v using decoder 468
                                             // 57684:     }
                                             // 57684:     } // switch Inst[21:16]
                                             // 57684:    }
  51, 11,                                    // 57689:    case 0x33: {
  OPC_CheckPredicate, 76,                    // 57691:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57693:     check Inst[21] == 0x1
  OPC_Decode, 198, 28, 221, 3,               // 57697:     decode to FMLAv2f32 using decoder 477
                                             // 57697:    }
  53, 11,                                    // 57702:    case 0x35: {
  OPC_CheckPredicate, 76,                    // 57704:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57706:     check Inst[21] == 0x1
  OPC_Decode, 137, 22, 200, 3,               // 57710:     decode to FADDv2f32 using decoder 456
                                             // 57710:    }
  54, 11,                                    // 57715:    case 0x36: {
  OPC_CheckPredicate, 76,                    // 57717:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 57719:     check Inst[21:16] == 0x21
  OPC_Decode, 165, 47, 201, 3,               // 57723:     decode to SCVTFv2f32 using decoder 457
                                             // 57723:    }
  55, 11,                                    // 57728:    case 0x37: {
  OPC_CheckPredicate, 76,                    // 57730:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57732:     check Inst[21] == 0x1
  OPC_Decode, 236, 29, 200, 3,               // 57736:     decode to FMULXv2f32 using decoder 456
                                             // 57736:    }
  56, 11,                                    // 57741:    case 0x38: {
  OPC_CheckPredicate, 76,                    // 57743:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57745:     check Inst[21] == 0x1
  OPC_Decode, 148, 44, 196, 3,               // 57749:     decode to PMULLv8i8 using decoder 452
                                             // 57749:    }
  57, 11,                                    // 57754:    case 0x39: {
  OPC_CheckPredicate, 76,                    // 57756:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 57758:     check Inst[21] == 0x1
  OPC_Decode, 205, 22, 200, 3,               // 57762:     decode to FCMEQv2f32 using decoder 456
                                             // 57762:    }
  58, 11,                                    // 57767:    case 0x3a: {
  OPC_CheckPredicate, 82,                    // 57769:     check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 57771:     check Inst[21:16] == 0x21
  OPC_Decode, 236, 30, 201, 3,               // 57775:     decode to FRINT32Zv2f32 using decoder 457
                                             // 57775:    }
  59, 11,                                    // 57780:    case 0x3b: {
  OPC_CheckPredicate, 83,                    // 57782:     check predicate 83
  OPC_CheckField, 21, 1, 1,                  // 57784:     check Inst[21] == 0x1
  OPC_Decode, 169, 28, 221, 3,               // 57788:     decode to FMLALv4f16 using decoder 477
                                             // 57788:    }
  61, 21,                                    // 57793:    case 0x3d: {
  OPC_SwitchField, 21, 1,                    // 57795:     switch Inst[21] {
  0, 7,                                      // 57798:     case 0x0: {
  OPC_CheckPredicate, 84,                    // 57800:      check predicate 84
  OPC_Decode, 195, 24, 215, 3,               // 57802:      decode to FCVTN_F32v8f8 using decoder 471
                                             // 57802:     }
  1, 0,                                      // 57807:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57809:      check predicate 76
  OPC_Decode, 142, 27, 200, 3,               // 57811:      decode to FMAXv2f32 using decoder 456
                                             // 57811:     }
                                             // 57811:     } // switch Inst[21]
                                             // 57811:    }
  62, 21,                                    // 57816:    case 0x3e: {
  OPC_SwitchField, 16, 6,                    // 57818:     switch Inst[21:16] {
  33, 7,                                     // 57821:     case 0x21: {
  OPC_CheckPredicate, 82,                    // 57823:      check predicate 82
  OPC_Decode, 254, 30, 201, 3,               // 57825:      decode to FRINT64Zv2f32 using decoder 457
                                             // 57825:     }
  48, 0,                                     // 57830:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 57832:      check predicate 81
  OPC_Decode, 249, 26, 212, 3,               // 57834:      decode to FMAXVv4i16v using decoder 468
                                             // 57834:     }
                                             // 57834:     } // switch Inst[21:16]
                                             // 57834:    }
  63, 0,                                     // 57839:    case 0x3f: {
  OPC_SwitchField, 21, 1,                    // 57841:     switch Inst[21] {
  0, 7,                                      // 57844:     case 0x0: {
  OPC_CheckPredicate, 85,                    // 57846:      check predicate 85
  OPC_Decode, 154, 26, 221, 3,               // 57848:      decode to FDOTv2f32 using decoder 477
                                             // 57848:     }
  1, 0,                                      // 57853:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 57855:      check predicate 76
  OPC_Decode, 207, 30, 200, 3,               // 57857:      decode to FRECPSv2f32 using decoder 456
                                             // 57857:     }
                                             // 57857:     } // switch Inst[21]
                                             // 57857:    }
                                             // 57857:    } // switch Inst[15:10]
                                             // 57857:   }
  1, 220, 6,                                 // 57862:   case 0x1: {
  OPC_SwitchField, 14, 2,                    // 57865:    switch Inst[15:14] {
  0, 235, 1,                                 // 57868:    case 0x0: {
  OPC_SwitchField, 10, 1,                    // 57871:     switch Inst[10] {
  0, 121,                                    // 57874:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 57876:      switch Inst[21] {
  0, 7,                                      // 57879:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 57881:       check predicate 76
  OPC_Decode, 167, 21, 224, 3,               // 57883:       decode to EXTv8i8 using decoder 480
                                             // 57883:      }
  1, 0,                                      // 57888:      case 0x1: {
  OPC_SwitchField, 11, 3,                    // 57890:       switch Inst[13:11] {
  0, 7,                                      // 57893:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 57895:        check predicate 76
  OPC_Decode, 211, 62, 196, 3,               // 57897:        decode to UADDLv8i8_v8i16 using decoder 452
                                             // 57897:       }
  1, 11,                                     // 57902:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 57904:        check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 57906:        check Inst[20:16] == 0x0
  OPC_Decode, 198, 45, 201, 3,               // 57910:        decode to REV32v8i8 using decoder 457
                                             // 57910:       }
  2, 7,                                      // 57915:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 57917:        check predicate 76
  OPC_Decode, 227, 62, 204, 3,               // 57919:        decode to UADDWv8i8_v8i16 using decoder 460
                                             // 57919:       }
  4, 7,                                      // 57924:       case 0x4: {
  OPC_CheckPredicate, 76,                    // 57926:        check predicate 76
  OPC_Decode, 252, 68, 196, 3,               // 57928:        decode to USUBLv8i8_v8i16 using decoder 452
                                             // 57928:       }
  5, 21,                                     // 57933:       case 0x5: {
  OPC_SwitchField, 16, 5,                    // 57935:        switch Inst[20:16] {
  0, 7,                                      // 57938:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 57940:         check predicate 76
  OPC_Decode, 197, 62, 201, 3,               // 57942:         decode to UADDLPv8i8_v4i16 using decoder 457
                                             // 57942:        }
  1, 0,                                      // 57947:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 57949:         check predicate 76
  OPC_Decode, 228, 54, 206, 3,               // 57951:         decode to SQXTUNv8i8 using decoder 462
                                             // 57951:        }
                                             // 57951:        } // switch Inst[20:16]
                                             // 57951:       }
  6, 7,                                      // 57956:       case 0x6: {
  OPC_CheckPredicate, 76,                    // 57958:        check predicate 76
  OPC_Decode, 136, 69, 204, 3,               // 57960:        decode to USUBWv8i8_v8i16 using decoder 460
                                             // 57960:       }
  7, 0,                                      // 57965:       case 0x7: {
  OPC_SwitchField, 16, 5,                    // 57967:        switch Inst[20:16] {
  0, 7,                                      // 57970:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 57972:         check predicate 76
  OPC_Decode, 227, 68, 211, 3,               // 57974:         decode to USQADDv8i8 using decoder 467
                                             // 57974:        }
  1, 7,                                      // 57979:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 57981:         check predicate 76
  OPC_Decode, 160, 48, 220, 3,               // 57983:         decode to SHLLv8i8 using decoder 476
                                             // 57983:        }
  16, 0,                                     // 57988:        case 0x10: {
  OPC_CheckPredicate, 76,                    // 57990:         check predicate 76
  OPC_Decode, 205, 62, 212, 3,               // 57992:         decode to UADDLVv8i8v using decoder 468
                                             // 57992:        }
                                             // 57992:        } // switch Inst[20:16]
                                             // 57992:       }
                                             // 57992:       } // switch Inst[13:11]
                                             // 57992:      }
                                             // 57992:      } // switch Inst[21]
                                             // 57992:     }
  1, 0,                                      // 57997:     case 0x1: {
  OPC_SwitchField, 11, 3,                    // 57999:      switch Inst[13:11] {
  0, 11,                                     // 58002:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 58004:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58006:       check Inst[21] == 0x1
  OPC_Decode, 214, 63, 200, 3,               // 58010:       decode to UHADDv8i8 using decoder 456
                                             // 58010:      }
  1, 11,                                     // 58015:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 58017:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58019:       check Inst[21] == 0x1
  OPC_Decode, 140, 66, 200, 3,               // 58023:       decode to UQADDv8i8 using decoder 456
                                             // 58023:      }
  2, 11,                                     // 58028:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 58030:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58032:       check Inst[21] == 0x1
  OPC_Decode, 210, 67, 200, 3,               // 58036:       decode to URHADDv8i8 using decoder 456
                                             // 58036:      }
  3, 11,                                     // 58041:      case 0x3: {
  OPC_CheckPredicate, 76,                    // 58043:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58045:       check Inst[21] == 0x1
  OPC_Decode, 143, 21, 200, 3,               // 58049:       decode to EORv8i8 using decoder 456
                                             // 58049:      }
  4, 11,                                     // 58054:      case 0x4: {
  OPC_CheckPredicate, 76,                    // 58056:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58058:       check Inst[21] == 0x1
  OPC_Decode, 228, 63, 200, 3,               // 58062:       decode to UHSUBv8i8 using decoder 456
                                             // 58062:      }
  5, 11,                                     // 58067:      case 0x5: {
  OPC_CheckPredicate, 76,                    // 58069:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58071:       check Inst[21] == 0x1
  OPC_Decode, 181, 67, 200, 3,               // 58075:       decode to UQSUBv8i8 using decoder 456
                                             // 58075:      }
  6, 11,                                     // 58080:      case 0x6: {
  OPC_CheckPredicate, 76,                    // 58082:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58084:       check Inst[21] == 0x1
  OPC_Decode, 134, 18, 200, 3,               // 58088:       decode to CMHIv8i8 using decoder 456
                                             // 58088:      }
  7, 0,                                      // 58093:      case 0x7: {
  OPC_CheckPredicate, 76,                    // 58095:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58097:       check Inst[21] == 0x1
  OPC_Decode, 142, 18, 200, 3,               // 58101:       decode to CMHSv8i8 using decoder 456
                                             // 58101:      }
                                             // 58101:      } // switch Inst[13:11]
                                             // 58101:     }
                                             // 58101:     } // switch Inst[10]
                                             // 58101:    }
  1, 231, 1,                                 // 58106:    case 0x1: {
  OPC_SwitchField, 10, 4,                    // 58109:     switch Inst[13:10] {
  0, 11,                                     // 58112:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 58114:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58116:      check Inst[21] == 0x1
  OPC_Decode, 227, 44, 215, 3,               // 58120:      decode to RADDHNv8i16_v8i8 using decoder 471
                                             // 58120:     }
  1, 11,                                     // 58125:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58127:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58129:      check Inst[21] == 0x1
  OPC_Decode, 174, 68, 200, 3,               // 58133:      decode to USHLv8i8 using decoder 456
                                             // 58133:     }
  2, 21,                                     // 58138:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 58140:      switch Inst[21:16] {
  32, 7,                                     // 58143:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 58145:       check predicate 76
  OPC_Decode, 206, 17, 201, 3,               // 58147:       decode to CLZv8i8 using decoder 457
                                             // 58147:      }
  33, 0,                                     // 58152:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 58154:       check predicate 76
  OPC_Decode, 196, 67, 206, 3,               // 58156:       decode to UQXTNv8i8 using decoder 462
                                             // 58156:      }
                                             // 58156:      } // switch Inst[21:16]
                                             // 58156:     }
  3, 11,                                     // 58161:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 58163:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58165:      check Inst[21] == 0x1
  OPC_Decode, 136, 67, 200, 3,               // 58169:      decode to UQSHLv8i8 using decoder 456
                                             // 58169:     }
  4, 11,                                     // 58174:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 58176:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58178:      check Inst[21] == 0x1
  OPC_Decode, 147, 62, 217, 3,               // 58182:      decode to UABALv8i8_v8i16 using decoder 473
                                             // 58182:     }
  5, 11,                                     // 58187:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 58189:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58191:      check Inst[21] == 0x1
  OPC_Decode, 242, 67, 200, 3,               // 58195:      decode to URSHLv8i8 using decoder 456
                                             // 58195:     }
  6, 11,                                     // 58200:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 58202:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 58204:      check Inst[21:16] == 0x20
  OPC_Decode, 192, 43, 201, 3,               // 58208:      decode to NOTv8i8 using decoder 457
                                             // 58208:     }
  7, 11,                                     // 58213:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 58215:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58217:      check Inst[21] == 0x1
  OPC_Decode, 209, 66, 200, 3,               // 58221:      decode to UQRSHLv8i8 using decoder 456
                                             // 58221:     }
  8, 11,                                     // 58226:     case 0x8: {
  OPC_CheckPredicate, 76,                    // 58228:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58230:      check Inst[21] == 0x1
  OPC_Decode, 128, 46, 215, 3,               // 58234:      decode to RSUBHNv8i16_v8i8 using decoder 471
                                             // 58234:     }
  9, 11,                                     // 58239:     case 0x9: {
  OPC_CheckPredicate, 76,                    // 58241:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58243:      check Inst[21] == 0x1
  OPC_Decode, 158, 64, 200, 3,               // 58247:      decode to UMAXv8i8 using decoder 456
                                             // 58247:     }
  10, 11,                                    // 58252:     case 0xa: {
  OPC_CheckPredicate, 76,                    // 58254:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 58256:      check Inst[21:16] == 0x20
  OPC_Decode, 188, 62, 211, 3,               // 58260:      decode to UADALPv8i8_v4i16 using decoder 467
                                             // 58260:     }
  11, 11,                                    // 58265:     case 0xb: {
  OPC_CheckPredicate, 76,                    // 58267:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58269:      check Inst[21] == 0x1
  OPC_Decode, 215, 64, 200, 3,               // 58273:      decode to UMINv8i8 using decoder 456
                                             // 58273:     }
  12, 11,                                    // 58278:     case 0xc: {
  OPC_CheckPredicate, 76,                    // 58280:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58282:      check Inst[21] == 0x1
  OPC_Decode, 169, 62, 196, 3,               // 58286:      decode to UABDLv8i8_v8i16 using decoder 452
                                             // 58286:     }
  13, 11,                                    // 58291:     case 0xd: {
  OPC_CheckPredicate, 76,                    // 58293:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58295:      check Inst[21] == 0x1
  OPC_Decode, 179, 62, 200, 3,               // 58299:      decode to UABDv8i8 using decoder 456
                                             // 58299:     }
  14, 21,                                    // 58304:     case 0xe: {
  OPC_SwitchField, 16, 6,                    // 58306:      switch Inst[21:16] {
  32, 7,                                     // 58309:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 58311:       check predicate 76
  OPC_Decode, 218, 52, 201, 3,               // 58313:       decode to SQNEGv8i8 using decoder 457
                                             // 58313:      }
  33, 0,                                     // 58318:      case 0x21: {
  OPC_CheckPredicate, 84,                    // 58320:       check predicate 84
  OPC_Decode, 168, 21, 220, 3,               // 58322:       decode to F1CVTL using decoder 476
                                             // 58322:      }
                                             // 58322:      } // switch Inst[21:16]
                                             // 58322:     }
  15, 0,                                     // 58327:     case 0xf: {
  OPC_CheckPredicate, 76,                    // 58329:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58331:      check Inst[21] == 0x1
  OPC_Decode, 157, 62, 221, 3,               // 58335:      decode to UABAv8i8 using decoder 477
                                             // 58335:     }
                                             // 58335:     } // switch Inst[13:10]
                                             // 58335:    }
  2, 208, 1,                                 // 58340:    case 0x2: {
  OPC_SwitchField, 10, 4,                    // 58343:     switch Inst[13:10] {
  0, 11,                                     // 58346:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 58348:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58350:      check Inst[21] == 0x1
  OPC_Decode, 131, 65, 217, 3,               // 58354:      decode to UMLALv8i8_v8i16 using decoder 473
                                             // 58354:     }
  1, 11,                                     // 58359:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58361:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58363:      check Inst[21] == 0x1
  OPC_Decode, 199, 60, 200, 3,               // 58367:      decode to SUBv8i8 using decoder 456
                                             // 58367:     }
  2, 21,                                     // 58372:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 58374:      switch Inst[21:16] {
  32, 7,                                     // 58377:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 58379:       check predicate 76
  OPC_Decode, 238, 17, 201, 3,               // 58381:       decode to CMGEv8i8rz using decoder 457
                                             // 58381:      }
  33, 0,                                     // 58386:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 58388:       check predicate 76
  OPC_Decode, 140, 31, 201, 3,               // 58390:       decode to FRINTAv2f32 using decoder 457
                                             // 58390:      }
                                             // 58390:      } // switch Inst[21:16]
                                             // 58390:     }
  3, 11,                                     // 58395:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 58397:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58399:      check Inst[21] == 0x1
  OPC_Decode, 221, 17, 200, 3,               // 58403:      decode to CMEQv8i8 using decoder 456
                                             // 58403:     }
  5, 11,                                     // 58408:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 58410:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58412:      check Inst[21] == 0x1
  OPC_Decode, 145, 42, 221, 3,               // 58416:      decode to MLSv8i8 using decoder 477
                                             // 58416:     }
  6, 21,                                     // 58421:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 58423:      switch Inst[21:16] {
  32, 7,                                     // 58426:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 58428:       check predicate 76
  OPC_Decode, 156, 18, 201, 3,               // 58430:       decode to CMLEv8i8rz using decoder 457
                                             // 58430:      }
  33, 0,                                     // 58435:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 58437:       check predicate 76
  OPC_Decode, 216, 31, 201, 3,               // 58439:       decode to FRINTXv2f32 using decoder 457
                                             // 58439:      }
                                             // 58439:      } // switch Inst[21:16]
                                             // 58439:     }
  7, 11,                                     // 58444:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 58446:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58448:      check Inst[21] == 0x1
  OPC_Decode, 151, 44, 200, 3,               // 58452:      decode to PMULv8i8 using decoder 456
                                             // 58452:     }
  8, 11,                                     // 58457:     case 0x8: {
  OPC_CheckPredicate, 76,                    // 58459:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58461:      check Inst[21] == 0x1
  OPC_Decode, 175, 65, 217, 3,               // 58465:      decode to UMLSLv8i8_v8i16 using decoder 473
                                             // 58465:     }
  9, 11,                                     // 58470:     case 0x9: {
  OPC_CheckPredicate, 76,                    // 58472:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58474:      check Inst[21] == 0x1
  OPC_Decode, 239, 63, 200, 3,               // 58478:      decode to UMAXPv8i8 using decoder 456
                                             // 58478:     }
  10, 30,                                    // 58483:     case 0xa: {
  OPC_SwitchField, 16, 6,                    // 58485:      switch Inst[21:16] {
  33, 7,                                     // 58488:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 58490:       check predicate 76
  OPC_Decode, 187, 24, 201, 3,               // 58492:       decode to FCVTNUv2f32 using decoder 457
                                             // 58492:      }
  48, 7,                                     // 58497:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 58499:       check predicate 76
  OPC_Decode, 252, 63, 222, 3,               // 58501:       decode to UMAXVv8i8v using decoder 478
                                             // 58501:      }
  49, 0,                                     // 58506:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 58508:       check predicate 76
  OPC_Decode, 181, 64, 222, 3,               // 58510:       decode to UMINVv8i8v using decoder 478
                                             // 58510:      }
                                             // 58510:      } // switch Inst[21:16]
                                             // 58510:     }
  11, 11,                                    // 58515:     case 0xb: {
  OPC_CheckPredicate, 76,                    // 58517:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58519:      check Inst[21] == 0x1
  OPC_Decode, 168, 64, 200, 3,               // 58523:      decode to UMINPv8i8 using decoder 456
                                             // 58523:     }
  14, 0,                                     // 58528:     case 0xe: {
  OPC_SwitchField, 16, 6,                    // 58530:      switch Inst[21:16] {
  32, 7,                                     // 58533:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 58535:       check predicate 76
  OPC_Decode, 177, 43, 201, 3,               // 58537:       decode to NEGv8i8 using decoder 457
                                             // 58537:      }
  33, 0,                                     // 58542:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 58544:       check predicate 76
  OPC_Decode, 145, 24, 201, 3,               // 58546:       decode to FCVTMUv2f32 using decoder 457
                                             // 58546:      }
                                             // 58546:      } // switch Inst[21:16]
                                             // 58546:     }
                                             // 58546:     } // switch Inst[13:10]
                                             // 58546:    }
  3, 0,                                      // 58551:    case 0x3: {
  OPC_SwitchField, 10, 4,                    // 58553:     switch Inst[13:10] {
  0, 11,                                     // 58556:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 58558:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58560:      check Inst[21] == 0x1
  OPC_Decode, 245, 65, 196, 3,               // 58564:      decode to UMULLv8i8_v8i16 using decoder 452
                                             // 58564:     }
  1, 11,                                     // 58569:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58571:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58573:      check Inst[21] == 0x1
  OPC_Decode, 190, 26, 200, 3,               // 58577:      decode to FMAXNMPv2f32 using decoder 456
                                             // 58577:     }
  2, 11,                                     // 58582:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 58584:      check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 58586:      check Inst[21:16] == 0x21
  OPC_Decode, 224, 23, 201, 3,               // 58590:      decode to FCVTAUv2f32 using decoder 457
                                             // 58590:     }
  3, 11,                                     // 58595:     case 0x3: {
  OPC_CheckPredicate, 83,                    // 58597:      check predicate 83
  OPC_CheckField, 21, 1, 1,                  // 58599:      check Inst[21] == 0x1
  OPC_Decode, 241, 27, 221, 3,               // 58603:      decode to FMLAL2v4f16 using decoder 477
                                             // 58603:     }
  5, 11,                                     // 58608:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 58610:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58612:      check Inst[21] == 0x1
  OPC_Decode, 235, 21, 200, 3,               // 58616:      decode to FADDPv2f32 using decoder 456
                                             // 58616:     }
  6, 11,                                     // 58621:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 58623:      check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 58625:      check Inst[21:16] == 0x21
  OPC_Decode, 158, 63, 201, 3,               // 58629:      decode to UCVTFv2f32 using decoder 457
                                             // 58629:     }
  7, 11,                                     // 58634:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 58636:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58638:      check Inst[21] == 0x1
  OPC_Decode, 145, 30, 200, 3,               // 58642:      decode to FMULv2f32 using decoder 456
                                             // 58642:     }
  9, 11,                                     // 58647:     case 0x9: {
  OPC_CheckPredicate, 76,                    // 58649:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58651:      check Inst[21] == 0x1
  OPC_Decode, 227, 22, 200, 3,               // 58655:      decode to FCMGEv2f32 using decoder 456
                                             // 58655:     }
  10, 11,                                    // 58660:     case 0xa: {
  OPC_CheckPredicate, 82,                    // 58662:      check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 58664:      check Inst[21:16] == 0x21
  OPC_Decode, 227, 30, 201, 3,               // 58668:      decode to FRINT32Xv2f32 using decoder 457
                                             // 58668:     }
  11, 11,                                    // 58673:     case 0xb: {
  OPC_CheckPredicate, 76,                    // 58675:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58677:      check Inst[21] == 0x1
  OPC_Decode, 211, 21, 200, 3,               // 58681:      decode to FACGEv2f32 using decoder 456
                                             // 58681:     }
  13, 11,                                    // 58686:     case 0xd: {
  OPC_CheckPredicate, 76,                    // 58688:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58690:      check Inst[21] == 0x1
  OPC_Decode, 234, 26, 200, 3,               // 58694:      decode to FMAXPv2f32 using decoder 456
                                             // 58694:     }
  14, 11,                                    // 58699:     case 0xe: {
  OPC_CheckPredicate, 82,                    // 58701:      check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 58703:      check Inst[21:16] == 0x21
  OPC_Decode, 245, 30, 201, 3,               // 58707:      decode to FRINT64Xv2f32 using decoder 457
                                             // 58707:     }
  15, 0,                                     // 58712:     case 0xf: {
  OPC_CheckPredicate, 76,                    // 58714:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58716:      check Inst[21] == 0x1
  OPC_Decode, 247, 25, 200, 3,               // 58720:      decode to FDIVv2f32 using decoder 456
                                             // 58720:     }
                                             // 58720:     } // switch Inst[13:10]
                                             // 58720:    }
                                             // 58720:    } // switch Inst[15:14]
                                             // 58720:   }
  2, 134, 11,                                // 58725:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 58728:    switch Inst[15:10] {
  0, 21,                                     // 58731:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 58733:     switch Inst[21] {
  0, 7,                                      // 58736:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 58738:      check predicate 76
  OPC_Decode, 194, 61, 225, 3,               // 58740:      decode to TBLv16i8One using decoder 481
                                             // 58740:     }
  1, 0,                                      // 58745:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58747:      check predicate 76
  OPC_Decode, 205, 46, 225, 3,               // 58749:      decode to SADDLv16i8_v8i16 using decoder 481
                                             // 58749:     }
                                             // 58749:     } // switch Inst[21]
                                             // 58749:    }
  1, 67,                                     // 58754:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 58756:     switch Inst[21] {
  0, 53,                                     // 58759:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 58761:      switch Inst[16] {
  0, 39,                                     // 58764:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 58766:       switch Inst[17] {
  0, 25,                                     // 58769:       case 0x0: {
  OPC_SwitchField, 18, 1,                    // 58771:        switch Inst[18] {
  0, 11,                                     // 58774:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 58776:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 58778:         check Inst[19] == 0x1
  OPC_Decode, 229, 20, 226, 3,               // 58782:         decode to DUPv2i64lane using decoder 482
                                             // 58782:        }
  1, 0,                                      // 58787:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 58789:         check predicate 76
  OPC_Decode, 233, 20, 227, 3,               // 58791:         decode to DUPv4i32lane using decoder 483
                                             // 58791:        }
                                             // 58791:        } // switch Inst[18]
                                             // 58791:       }
  1, 0,                                      // 58796:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 58798:        check predicate 76
  OPC_Decode, 235, 20, 228, 3,               // 58800:        decode to DUPv8i16lane using decoder 484
                                             // 58800:       }
                                             // 58800:       } // switch Inst[17]
                                             // 58800:      }
  1, 0,                                      // 58805:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 58807:       check predicate 76
  OPC_Decode, 225, 20, 229, 3,               // 58809:       decode to DUPv16i8lane using decoder 485
                                             // 58809:      }
                                             // 58809:      } // switch Inst[16]
                                             // 58809:     }
  1, 0,                                      // 58814:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58816:      check predicate 76
  OPC_Decode, 149, 48, 225, 3,               // 58818:      decode to SHADDv16i8 using decoder 481
                                             // 58818:     }
                                             // 58818:     } // switch Inst[21]
                                             // 58818:    }
  2, 11,                                     // 58823:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 58825:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 58827:     check Inst[21:16] == 0x20
  OPC_Decode, 199, 45, 230, 3,               // 58831:     decode to REV64v16i8 using decoder 486
                                             // 58831:    }
  3, 67,                                     // 58836:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 58838:     switch Inst[21] {
  0, 53,                                     // 58841:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 58843:      switch Inst[16] {
  0, 39,                                     // 58846:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 58848:       switch Inst[17] {
  0, 25,                                     // 58851:       case 0x0: {
  OPC_SwitchField, 18, 1,                    // 58853:        switch Inst[18] {
  0, 11,                                     // 58856:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 58858:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 58860:         check Inst[19] == 0x1
  OPC_Decode, 228, 20, 231, 3,               // 58864:         decode to DUPv2i64gpr using decoder 487
                                             // 58864:        }
  1, 0,                                      // 58869:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 58871:         check predicate 76
  OPC_Decode, 232, 20, 232, 3,               // 58873:         decode to DUPv4i32gpr using decoder 488
                                             // 58873:        }
                                             // 58873:        } // switch Inst[18]
                                             // 58873:       }
  1, 0,                                      // 58878:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 58880:        check predicate 76
  OPC_Decode, 234, 20, 232, 3,               // 58882:        decode to DUPv8i16gpr using decoder 488
                                             // 58882:       }
                                             // 58882:       } // switch Inst[17]
                                             // 58882:      }
  1, 0,                                      // 58887:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 58889:       check predicate 76
  OPC_Decode, 224, 20, 232, 3,               // 58891:       decode to DUPv16i8gpr using decoder 488
                                             // 58891:      }
                                             // 58891:      } // switch Inst[16]
                                             // 58891:     }
  1, 0,                                      // 58896:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58898:      check predicate 76
  OPC_Decode, 150, 51, 225, 3,               // 58900:      decode to SQADDv16i8 using decoder 481
                                             // 58900:     }
                                             // 58900:     } // switch Inst[21]
                                             // 58900:    }
  4, 21,                                     // 58905:    case 0x4: {
  OPC_SwitchField, 21, 1,                    // 58907:     switch Inst[21] {
  0, 7,                                      // 58910:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 58912:      check predicate 76
  OPC_Decode, 212, 61, 223, 3,               // 58914:      decode to TBXv16i8One using decoder 479
                                             // 58914:     }
  1, 0,                                      // 58919:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58921:      check predicate 76
  OPC_Decode, 220, 46, 225, 3,               // 58923:      decode to SADDWv16i8_v8i16 using decoder 481
                                             // 58923:     }
                                             // 58923:     } // switch Inst[21]
                                             // 58923:    }
  5, 11,                                     // 58928:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 58930:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 58932:     check Inst[21] == 0x1
  OPC_Decode, 233, 54, 225, 3,               // 58936:     decode to SRHADDv16i8 using decoder 481
                                             // 58936:    }
  6, 25,                                     // 58941:    case 0x6: {
  OPC_SwitchField, 21, 1,                    // 58943:     switch Inst[21] {
  0, 7,                                      // 58946:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 58948:      check predicate 76
  OPC_Decode, 176, 69, 225, 3,               // 58950:      decode to UZP1v16i8 using decoder 481
                                             // 58950:     }
  1, 0,                                      // 58955:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 58957:      check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 58959:      check Inst[20:16] == 0x0
  OPC_Decode, 192, 45, 230, 3,               // 58963:      decode to REV16v16i8 using decoder 486
                                             // 58963:     }
                                             // 58963:     } // switch Inst[21]
                                             // 58963:    }
  7, 67,                                     // 58968:    case 0x7: {
  OPC_SwitchField, 21, 1,                    // 58970:     switch Inst[21] {
  0, 53,                                     // 58973:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 58975:      switch Inst[16] {
  0, 39,                                     // 58978:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 58980:       switch Inst[17] {
  0, 25,                                     // 58983:       case 0x0: {
  OPC_SwitchField, 18, 1,                    // 58985:        switch Inst[18] {
  0, 11,                                     // 58988:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 58990:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 58992:         check Inst[19] == 0x1
  OPC_Decode, 151, 34, 233, 3,               // 58996:         decode to INSvi64gpr using decoder 489
                                             // 58996:        }
  1, 0,                                      // 59001:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 59003:         check predicate 76
  OPC_Decode, 149, 34, 234, 3,               // 59005:         decode to INSvi32gpr using decoder 490
                                             // 59005:        }
                                             // 59005:        } // switch Inst[18]
                                             // 59005:       }
  1, 0,                                      // 59010:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 59012:        check predicate 76
  OPC_Decode, 147, 34, 235, 3,               // 59014:        decode to INSvi16gpr using decoder 491
                                             // 59014:       }
                                             // 59014:       } // switch Inst[17]
                                             // 59014:      }
  1, 0,                                      // 59019:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 59021:       check predicate 76
  OPC_Decode, 153, 34, 236, 3,               // 59023:       decode to INSvi8gpr using decoder 492
                                             // 59023:      }
                                             // 59023:      } // switch Inst[16]
                                             // 59023:     }
  1, 0,                                      // 59028:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59030:      check predicate 76
  OPC_Decode, 171, 14, 225, 3,               // 59032:      decode to ANDv16i8 using decoder 481
                                             // 59032:     }
                                             // 59032:     } // switch Inst[21]
                                             // 59032:    }
  8, 21,                                     // 59037:    case 0x8: {
  OPC_SwitchField, 21, 1,                    // 59039:     switch Inst[21] {
  0, 7,                                      // 59042:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59044:      check predicate 76
  OPC_Decode, 196, 61, 237, 3,               // 59046:      decode to TBLv16i8Two using decoder 493
                                             // 59046:     }
  1, 0,                                      // 59051:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59053:      check predicate 76
  OPC_Decode, 142, 56, 225, 3,               // 59055:      decode to SSUBLv16i8_v8i16 using decoder 481
                                             // 59055:     }
                                             // 59055:     } // switch Inst[21]
                                             // 59055:    }
  9, 11,                                     // 59060:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 59062:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59064:     check Inst[21] == 0x1
  OPC_Decode, 189, 48, 225, 3,               // 59068:     decode to SHSUBv16i8 using decoder 481
                                             // 59068:    }
  10, 35,                                    // 59073:    case 0xa: {
  OPC_SwitchField, 21, 1,                    // 59075:     switch Inst[21] {
  0, 7,                                      // 59078:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59080:      check predicate 76
  OPC_Decode, 237, 61, 225, 3,               // 59082:      decode to TRN1v16i8 using decoder 481
                                             // 59082:     }
  1, 0,                                      // 59087:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 59089:      switch Inst[20:16] {
  0, 7,                                      // 59092:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 59094:       check predicate 76
  OPC_Decode, 191, 46, 230, 3,               // 59096:       decode to SADDLPv16i8_v8i16 using decoder 486
                                             // 59096:      }
  1, 0,                                      // 59101:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 59103:       check predicate 76
  OPC_Decode, 237, 70, 238, 3,               // 59105:       decode to XTNv16i8 using decoder 494
                                             // 59105:      }
                                             // 59105:      } // switch Inst[20:16]
                                             // 59105:     }
                                             // 59105:     } // switch Inst[21]
                                             // 59105:    }
  11, 92,                                    // 59110:    case 0xb: {
  OPC_SwitchField, 21, 1,                    // 59112:     switch Inst[21] {
  0, 78,                                     // 59115:     case 0x0: {
  OPC_SwitchField, 16, 1,                    // 59117:      switch Inst[16] {
  0, 51,                                     // 59120:      case 0x0: {
  OPC_SwitchField, 17, 1,                    // 59122:       switch Inst[17] {
  0, 24,                                     // 59125:       case 0x0: {
  OPC_CheckField, 18, 1, 1,                  // 59127:        check Inst[18] == 0x1
  OPC_Scope, 11,                             // 59131:        try {
  OPC_CheckField, 19, 2, 0,                  // 59133:         check Inst[20:19] == 0x0
  OPC_CheckPredicate, 76,                    // 59137:         check predicate 76
  OPC_Decode, 204, 50, 239, 3,               // 59139:         decode to SMOVvi32to64_idx0 using decoder 495
                                             // 59139:        } else try {
  OPC_CheckPredicate, 76,                    // 59144:         check predicate 76
  OPC_Decode, 203, 50, 240, 3,               // 59146:         decode to SMOVvi32to64 using decoder 496
                                             // 59146:        }
                                             // 59146:       }
  1, 0,                                      // 59151:       case 0x1: {
  OPC_Scope, 11,                             // 59153:        try {
  OPC_CheckField, 18, 3, 0,                  // 59155:         check Inst[20:18] == 0x0
  OPC_CheckPredicate, 76,                    // 59159:         check predicate 76
  OPC_Decode, 202, 50, 239, 3,               // 59161:         decode to SMOVvi16to64_idx0 using decoder 495
                                             // 59161:        } else try {
  OPC_CheckPredicate, 76,                    // 59166:         check predicate 76
  OPC_Decode, 201, 50, 241, 3,               // 59168:         decode to SMOVvi16to64 using decoder 497
                                             // 59168:        }
                                             // 59168:       }
                                             // 59168:       } // switch Inst[17]
                                             // 59168:      }
  1, 0,                                      // 59173:      case 0x1: {
  OPC_Scope, 11,                             // 59175:       try {
  OPC_CheckField, 17, 4, 0,                  // 59177:        check Inst[20:17] == 0x0
  OPC_CheckPredicate, 76,                    // 59181:        check predicate 76
  OPC_Decode, 208, 50, 239, 3,               // 59183:        decode to SMOVvi8to64_idx0 using decoder 495
                                             // 59183:       } else try {
  OPC_CheckPredicate, 76,                    // 59188:        check predicate 76
  OPC_Decode, 207, 50, 242, 3,               // 59190:        decode to SMOVvi8to64 using decoder 498
                                             // 59190:       }
                                             // 59190:      }
                                             // 59190:      } // switch Inst[16]
                                             // 59190:     }
  1, 0,                                      // 59195:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59197:      check predicate 76
  OPC_Decode, 188, 54, 225, 3,               // 59199:      decode to SQSUBv16i8 using decoder 481
                                             // 59199:     }
                                             // 59199:     } // switch Inst[21]
                                             // 59199:    }
  12, 21,                                    // 59204:    case 0xc: {
  OPC_SwitchField, 21, 1,                    // 59206:     switch Inst[21] {
  0, 7,                                      // 59209:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59211:      check predicate 76
  OPC_Decode, 214, 61, 243, 3,               // 59213:      decode to TBXv16i8Two using decoder 499
                                             // 59213:     }
  1, 0,                                      // 59218:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59220:      check predicate 76
  OPC_Decode, 154, 56, 225, 3,               // 59222:      decode to SSUBWv16i8_v8i16 using decoder 481
                                             // 59222:     }
                                             // 59222:     } // switch Inst[21]
                                             // 59222:    }
  13, 11,                                    // 59227:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 59229:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59231:     check Inst[21] == 0x1
  OPC_Decode, 239, 17, 225, 3,               // 59235:     decode to CMGTv16i8 using decoder 481
                                             // 59235:    }
  14, 35,                                    // 59240:    case 0xe: {
  OPC_SwitchField, 21, 1,                    // 59242:     switch Inst[21] {
  0, 7,                                      // 59245:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59247:      check predicate 76
  OPC_Decode, 134, 71, 225, 3,               // 59249:      decode to ZIP1v16i8 using decoder 481
                                             // 59249:     }
  1, 0,                                      // 59254:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 59256:      switch Inst[20:16] {
  0, 7,                                      // 59259:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 59261:       check predicate 76
  OPC_Decode, 248, 60, 238, 3,               // 59263:       decode to SUQADDv16i8 using decoder 494
                                             // 59263:      }
  16, 0,                                     // 59268:      case 0x10: {
  OPC_CheckPredicate, 76,                    // 59270:       check predicate 76
  OPC_Decode, 200, 46, 244, 3,               // 59272:       decode to SADDLVv16i8v using decoder 500
                                             // 59272:      }
                                             // 59272:      } // switch Inst[20:16]
                                             // 59272:     }
                                             // 59272:     } // switch Inst[21]
                                             // 59272:    }
  15, 38,                                    // 59277:    case 0xf: {
  OPC_SwitchField, 21, 1,                    // 59279:     switch Inst[21] {
  0, 24,                                     // 59282:     case 0x0: {
  OPC_CheckField, 16, 4, 8,                  // 59284:      check Inst[19:16] == 0x8
  OPC_Scope, 11,                             // 59288:      try {
  OPC_CheckField, 20, 1, 0,                  // 59290:       check Inst[20] == 0x0
  OPC_CheckPredicate, 76,                    // 59294:       check predicate 76
  OPC_Decode, 213, 65, 239, 3,               // 59296:       decode to UMOVvi64_idx0 using decoder 495
                                             // 59296:      } else try {
  OPC_CheckPredicate, 76,                    // 59301:       check predicate 76
  OPC_Decode, 212, 65, 245, 3,               // 59303:       decode to UMOVvi64 using decoder 501
                                             // 59303:      }
                                             // 59303:     }
  1, 0,                                      // 59308:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59310:      check predicate 76
  OPC_Decode, 223, 17, 225, 3,               // 59312:      decode to CMGEv16i8 using decoder 481
                                             // 59312:     }
                                             // 59312:     } // switch Inst[21]
                                             // 59312:    }
  16, 21,                                    // 59317:    case 0x10: {
  OPC_SwitchField, 21, 1,                    // 59319:     switch Inst[21] {
  0, 7,                                      // 59322:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59324:      check predicate 76
  OPC_Decode, 195, 61, 246, 3,               // 59326:      decode to TBLv16i8Three using decoder 502
                                             // 59326:     }
  1, 0,                                      // 59331:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59333:      check predicate 76
  OPC_Decode, 147, 13, 223, 3,               // 59335:      decode to ADDHNv8i16_v16i8 using decoder 479
                                             // 59335:     }
                                             // 59335:     } // switch Inst[21]
                                             // 59335:    }
  17, 11,                                    // 59340:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 59342:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59344:     check Inst[21] == 0x1
  OPC_Decode, 191, 55, 225, 3,               // 59348:     decode to SSHLv16i8 using decoder 481
                                             // 59348:    }
  18, 30,                                    // 59353:    case 0x12: {
  OPC_SwitchField, 16, 6,                    // 59355:     switch Inst[21:16] {
  32, 7,                                     // 59358:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 59360:      check predicate 76
  OPC_Decode, 185, 17, 230, 3,               // 59362:      decode to CLSv16i8 using decoder 486
                                             // 59362:     }
  33, 7,                                     // 59367:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 59369:      check predicate 76
  OPC_Decode, 205, 54, 238, 3,               // 59371:      decode to SQXTNv16i8 using decoder 494
                                             // 59371:     }
  40, 0,                                     // 59376:     case 0x28: {
  OPC_CheckPredicate, 86,                    // 59378:      check predicate 86
  OPC_Decode, 142, 14, 238, 3,               // 59380:      decode to AESErr using decoder 494
                                             // 59380:     }
                                             // 59380:     } // switch Inst[21:16]
                                             // 59380:    }
  19, 11,                                    // 59385:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 59387:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59389:     check Inst[21] == 0x1
  OPC_Decode, 248, 53, 225, 3,               // 59393:     decode to SQSHLv16i8 using decoder 481
                                             // 59393:    }
  20, 21,                                    // 59398:    case 0x14: {
  OPC_SwitchField, 21, 1,                    // 59400:     switch Inst[21] {
  0, 7,                                      // 59403:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59405:      check predicate 76
  OPC_Decode, 213, 61, 247, 3,               // 59407:      decode to TBXv16i8Three using decoder 503
                                             // 59407:     }
  1, 0,                                      // 59412:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59414:      check predicate 76
  OPC_Decode, 138, 46, 223, 3,               // 59416:      decode to SABALv16i8_v8i16 using decoder 479
                                             // 59416:     }
                                             // 59416:     } // switch Inst[21]
                                             // 59416:    }
  21, 11,                                    // 59421:    case 0x15: {
  OPC_CheckPredicate, 76,                    // 59423:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59425:     check Inst[21] == 0x1
  OPC_Decode, 147, 55, 225, 3,               // 59429:     decode to SRSHLv16i8 using decoder 481
                                             // 59429:    }
  22, 35,                                    // 59434:    case 0x16: {
  OPC_SwitchField, 21, 1,                    // 59436:     switch Inst[21] {
  0, 7,                                      // 59439:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59441:      check predicate 76
  OPC_Decode, 192, 69, 225, 3,               // 59443:      decode to UZP2v16i8 using decoder 481
                                             // 59443:     }
  1, 0,                                      // 59448:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 59450:      switch Inst[20:16] {
  0, 7,                                      // 59453:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 59455:       check predicate 76
  OPC_Decode, 169, 19, 230, 3,               // 59457:       decode to CNTv16i8 using decoder 486
                                             // 59457:      }
  8, 0,                                      // 59462:      case 0x8: {
  OPC_CheckPredicate, 86,                    // 59464:       check predicate 86
  OPC_Decode, 136, 14, 238, 3,               // 59466:       decode to AESDrr using decoder 494
                                             // 59466:      }
                                             // 59466:      } // switch Inst[20:16]
                                             // 59466:     }
                                             // 59466:     } // switch Inst[21]
                                             // 59466:    }
  23, 11,                                    // 59471:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 59473:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59475:     check Inst[21] == 0x1
  OPC_Decode, 162, 53, 225, 3,               // 59479:     decode to SQRSHLv16i8 using decoder 481
                                             // 59479:    }
  24, 21,                                    // 59484:    case 0x18: {
  OPC_SwitchField, 21, 1,                    // 59486:     switch Inst[21] {
  0, 7,                                      // 59489:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59491:      check predicate 76
  OPC_Decode, 193, 61, 248, 3,               // 59493:      decode to TBLv16i8Four using decoder 504
                                             // 59493:     }
  1, 0,                                      // 59498:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59500:      check predicate 76
  OPC_Decode, 135, 60, 223, 3,               // 59502:      decode to SUBHNv8i16_v16i8 using decoder 479
                                             // 59502:     }
                                             // 59502:     } // switch Inst[21]
                                             // 59502:    }
  25, 11,                                    // 59507:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 59509:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59511:     check Inst[21] == 0x1
  OPC_Decode, 143, 49, 225, 3,               // 59515:     decode to SMAXv16i8 using decoder 481
                                             // 59515:    }
  26, 44,                                    // 59520:    case 0x1a: {
  OPC_SwitchField, 21, 1,                    // 59522:     switch Inst[21] {
  0, 7,                                      // 59525:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59527:      check predicate 76
  OPC_Decode, 253, 61, 225, 3,               // 59529:      decode to TRN2v16i8 using decoder 481
                                             // 59529:     }
  1, 0,                                      // 59534:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 59536:      switch Inst[20:16] {
  0, 7,                                      // 59539:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 59541:       check predicate 76
  OPC_Decode, 179, 46, 238, 3,               // 59543:       decode to SADALPv16i8_v8i16 using decoder 494
                                             // 59543:      }
  1, 7,                                      // 59548:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 59550:       check predicate 76
  OPC_Decode, 202, 24, 238, 3,               // 59552:       decode to FCVTNv8i16 using decoder 494
                                             // 59552:      }
  8, 0,                                      // 59557:      case 0x8: {
  OPC_CheckPredicate, 86,                    // 59559:       check predicate 86
  OPC_Decode, 146, 14, 230, 3,               // 59561:       decode to AESMCrr using decoder 486
                                             // 59561:      }
                                             // 59561:      } // switch Inst[20:16]
                                             // 59561:     }
                                             // 59561:     } // switch Inst[21]
                                             // 59561:    }
  27, 11,                                    // 59566:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 59568:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59570:     check Inst[21] == 0x1
  OPC_Decode, 201, 49, 225, 3,               // 59574:     decode to SMINv16i8 using decoder 481
                                             // 59574:    }
  28, 21,                                    // 59579:    case 0x1c: {
  OPC_SwitchField, 21, 1,                    // 59581:     switch Inst[21] {
  0, 7,                                      // 59584:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59586:      check predicate 76
  OPC_Decode, 211, 61, 249, 3,               // 59588:      decode to TBXv16i8Four using decoder 505
                                             // 59588:     }
  1, 0,                                      // 59593:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59595:      check predicate 76
  OPC_Decode, 160, 46, 225, 3,               // 59597:      decode to SABDLv16i8_v8i16 using decoder 481
                                             // 59597:     }
                                             // 59597:     } // switch Inst[21]
                                             // 59597:    }
  29, 11,                                    // 59602:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 59604:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59606:     check Inst[21] == 0x1
  OPC_Decode, 170, 46, 225, 3,               // 59610:     decode to SABDv16i8 using decoder 481
                                             // 59610:    }
  30, 44,                                    // 59615:    case 0x1e: {
  OPC_SwitchField, 21, 1,                    // 59617:     switch Inst[21] {
  0, 7,                                      // 59620:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 59622:      check predicate 76
  OPC_Decode, 150, 71, 225, 3,               // 59624:      decode to ZIP2v16i8 using decoder 481
                                             // 59624:     }
  1, 0,                                      // 59629:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 59631:      switch Inst[20:16] {
  0, 7,                                      // 59634:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 59636:       check predicate 76
  OPC_Decode, 255, 50, 230, 3,               // 59638:       decode to SQABSv16i8 using decoder 486
                                             // 59638:      }
  1, 7,                                      // 59643:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 59645:       check predicate 76
  OPC_Decode, 241, 23, 230, 3,               // 59647:       decode to FCVTLv8i16 using decoder 486
                                             // 59647:      }
  8, 0,                                      // 59652:      case 0x8: {
  OPC_CheckPredicate, 86,                    // 59654:       check predicate 86
  OPC_Decode, 144, 14, 230, 3,               // 59656:       decode to AESIMCrr using decoder 486
                                             // 59656:      }
                                             // 59656:      } // switch Inst[20:16]
                                             // 59656:     }
                                             // 59656:     } // switch Inst[21]
                                             // 59656:    }
  31, 11,                                    // 59661:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 59663:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59665:     check Inst[21] == 0x1
  OPC_Decode, 148, 46, 223, 3,               // 59669:     decode to SABAv16i8 using decoder 479
                                             // 59669:    }
  32, 11,                                    // 59674:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 59676:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59678:     check Inst[21] == 0x1
  OPC_Decode, 241, 49, 223, 3,               // 59682:     decode to SMLALv16i8_v8i16 using decoder 479
                                             // 59682:    }
  33, 11,                                    // 59687:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 59689:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59691:     check Inst[21] == 0x1
  OPC_Decode, 233, 13, 225, 3,               // 59695:     decode to ADDv16i8 using decoder 481
                                             // 59695:    }
  34, 21,                                    // 59700:    case 0x22: {
  OPC_SwitchField, 16, 6,                    // 59702:     switch Inst[21:16] {
  32, 7,                                     // 59705:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 59707:      check predicate 76
  OPC_Decode, 240, 17, 230, 3,               // 59709:      decode to CMGTv16i8rz using decoder 486
                                             // 59709:     }
  33, 0,                                     // 59714:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 59716:      check predicate 76
  OPC_Decode, 189, 31, 230, 3,               // 59718:      decode to FRINTNv4f32 using decoder 486
                                             // 59718:     }
                                             // 59718:     } // switch Inst[21:16]
                                             // 59718:    }
  35, 11,                                    // 59723:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 59725:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59727:     check Inst[21] == 0x1
  OPC_Decode, 131, 19, 225, 3,               // 59731:     decode to CMTSTv16i8 using decoder 481
                                             // 59731:    }
  37, 11,                                    // 59736:    case 0x25: {
  OPC_CheckPredicate, 76,                    // 59738:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59740:     check Inst[21] == 0x1
  OPC_Decode, 247, 41, 223, 3,               // 59744:     decode to MLAv16i8 using decoder 479
                                             // 59744:    }
  38, 21,                                    // 59749:    case 0x26: {
  OPC_SwitchField, 16, 6,                    // 59751:     switch Inst[21:16] {
  32, 7,                                     // 59754:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 59756:      check predicate 76
  OPC_Decode, 208, 17, 230, 3,               // 59758:      decode to CMEQv16i8rz using decoder 486
                                             // 59758:     }
  33, 0,                                     // 59763:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 59765:      check predicate 76
  OPC_Decode, 173, 31, 230, 3,               // 59767:      decode to FRINTMv4f32 using decoder 486
                                             // 59767:     }
                                             // 59767:     } // switch Inst[21:16]
                                             // 59767:    }
  39, 11,                                    // 59772:    case 0x27: {
  OPC_CheckPredicate, 76,                    // 59774:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59776:     check Inst[21] == 0x1
  OPC_Decode, 143, 43, 225, 3,               // 59780:     decode to MULv16i8 using decoder 481
                                             // 59780:    }
  40, 11,                                    // 59785:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 59787:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59789:     check Inst[21] == 0x1
  OPC_Decode, 157, 50, 223, 3,               // 59793:     decode to SMLSLv16i8_v8i16 using decoder 479
                                             // 59793:    }
  41, 11,                                    // 59798:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 59800:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59802:     check Inst[21] == 0x1
  OPC_Decode, 224, 48, 225, 3,               // 59806:     decode to SMAXPv16i8 using decoder 481
                                             // 59806:    }
  42, 39,                                    // 59811:    case 0x2a: {
  OPC_SwitchField, 16, 6,                    // 59813:     switch Inst[21:16] {
  32, 7,                                     // 59816:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 59818:      check predicate 76
  OPC_Decode, 157, 18, 230, 3,               // 59820:      decode to CMLTv16i8rz using decoder 486
                                             // 59820:     }
  33, 7,                                     // 59825:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 59827:      check predicate 76
  OPC_Decode, 167, 24, 230, 3,               // 59829:      decode to FCVTNSv4f32 using decoder 486
                                             // 59829:     }
  48, 7,                                     // 59834:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 59836:      check predicate 76
  OPC_Decode, 238, 48, 250, 3,               // 59838:      decode to SMAXVv16i8v using decoder 506
                                             // 59838:     }
  49, 0,                                     // 59843:     case 0x31: {
  OPC_CheckPredicate, 76,                    // 59845:      check predicate 76
  OPC_Decode, 168, 49, 250, 3,               // 59847:      decode to SMINVv16i8v using decoder 506
                                             // 59847:     }
                                             // 59847:     } // switch Inst[21:16]
                                             // 59847:    }
  43, 11,                                    // 59852:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 59854:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59856:     check Inst[21] == 0x1
  OPC_Decode, 154, 49, 225, 3,               // 59860:     decode to SMINPv16i8 using decoder 481
                                             // 59860:    }
  46, 30,                                    // 59865:    case 0x2e: {
  OPC_SwitchField, 16, 6,                    // 59867:     switch Inst[21:16] {
  32, 7,                                     // 59870:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 59872:      check predicate 76
  OPC_Decode, 246, 12, 230, 3,               // 59874:      decode to ABSv16i8 using decoder 486
                                             // 59874:     }
  33, 7,                                     // 59879:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 59881:      check predicate 76
  OPC_Decode, 130, 24, 230, 3,               // 59883:      decode to FCVTMSv4f32 using decoder 486
                                             // 59883:     }
  49, 0,                                     // 59888:     case 0x31: {
  OPC_CheckPredicate, 76,                    // 59890:      check predicate 76
  OPC_Decode, 187, 13, 250, 3,               // 59892:      decode to ADDVv16i8v using decoder 506
                                             // 59892:     }
                                             // 59892:     } // switch Inst[21:16]
                                             // 59892:    }
  47, 11,                                    // 59897:    case 0x2f: {
  OPC_CheckPredicate, 76,                    // 59899:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59901:     check Inst[21] == 0x1
  OPC_Decode, 155, 13, 225, 3,               // 59905:     decode to ADDPv16i8 using decoder 481
                                             // 59905:    }
  48, 11,                                    // 59910:    case 0x30: {
  OPC_CheckPredicate, 76,                    // 59912:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59914:     check Inst[21] == 0x1
  OPC_Decode, 229, 50, 225, 3,               // 59918:     decode to SMULLv16i8_v8i16 using decoder 481
                                             // 59918:    }
  49, 21,                                    // 59923:    case 0x31: {
  OPC_SwitchField, 21, 1,                    // 59925:     switch Inst[21] {
  0, 7,                                      // 59928:     case 0x0: {
  OPC_CheckPredicate, 80,                    // 59930:      check predicate 80
  OPC_Decode, 132, 28, 223, 3,               // 59932:      decode to FMLALLTBv4f32 using decoder 479
                                             // 59932:     }
  1, 0,                                      // 59937:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 59939:      check predicate 76
  OPC_Decode, 229, 26, 225, 3,               // 59941:      decode to FMAXNMv4f32 using decoder 481
                                             // 59941:     }
                                             // 59941:     } // switch Inst[21]
                                             // 59941:    }
  50, 21,                                    // 59946:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 59948:     switch Inst[21:16] {
  33, 7,                                     // 59951:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 59953:      check predicate 76
  OPC_Decode, 209, 23, 230, 3,               // 59955:      decode to FCVTASv4f32 using decoder 486
                                             // 59955:     }
  48, 0,                                     // 59960:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 59962:      check predicate 81
  OPC_Decode, 207, 26, 244, 3,               // 59964:      decode to FMAXNMVv8i16v using decoder 500
                                             // 59964:     }
                                             // 59964:     } // switch Inst[21:16]
                                             // 59964:    }
  51, 11,                                    // 59969:    case 0x33: {
  OPC_CheckPredicate, 76,                    // 59971:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59973:     check Inst[21] == 0x1
  OPC_Decode, 203, 28, 223, 3,               // 59977:     decode to FMLAv4f32 using decoder 479
                                             // 59977:    }
  53, 11,                                    // 59982:    case 0x35: {
  OPC_CheckPredicate, 76,                    // 59984:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 59986:     check Inst[21] == 0x1
  OPC_Decode, 140, 22, 225, 3,               // 59990:     decode to FADDv4f32 using decoder 481
                                             // 59990:    }
  54, 11,                                    // 59995:    case 0x36: {
  OPC_CheckPredicate, 76,                    // 59997:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 59999:     check Inst[21:16] == 0x21
  OPC_Decode, 170, 47, 230, 3,               // 60003:     decode to SCVTFv4f32 using decoder 486
                                             // 60003:    }
  55, 11,                                    // 60008:    case 0x37: {
  OPC_CheckPredicate, 76,                    // 60010:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60012:     check Inst[21] == 0x1
  OPC_Decode, 241, 29, 225, 3,               // 60016:     decode to FMULXv4f32 using decoder 481
                                             // 60016:    }
  56, 11,                                    // 60021:    case 0x38: {
  OPC_CheckPredicate, 76,                    // 60023:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60025:     check Inst[21] == 0x1
  OPC_Decode, 145, 44, 225, 3,               // 60029:     decode to PMULLv16i8 using decoder 481
                                             // 60029:    }
  57, 11,                                    // 60034:    case 0x39: {
  OPC_CheckPredicate, 76,                    // 60036:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60038:     check Inst[21] == 0x1
  OPC_Decode, 210, 22, 225, 3,               // 60042:     decode to FCMEQv4f32 using decoder 481
                                             // 60042:    }
  58, 11,                                    // 60047:    case 0x3a: {
  OPC_CheckPredicate, 82,                    // 60049:     check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 60051:     check Inst[21:16] == 0x21
  OPC_Decode, 238, 30, 230, 3,               // 60055:     decode to FRINT32Zv4f32 using decoder 486
                                             // 60055:    }
  59, 11,                                    // 60060:    case 0x3b: {
  OPC_CheckPredicate, 83,                    // 60062:     check predicate 83
  OPC_CheckField, 21, 1, 1,                  // 60064:     check Inst[21] == 0x1
  OPC_Decode, 170, 28, 223, 3,               // 60068:     decode to FMLALv8f16 using decoder 479
                                             // 60068:    }
  61, 21,                                    // 60073:    case 0x3d: {
  OPC_SwitchField, 21, 1,                    // 60075:     switch Inst[21] {
  0, 7,                                      // 60078:     case 0x0: {
  OPC_CheckPredicate, 84,                    // 60080:      check predicate 84
  OPC_Decode, 194, 24, 223, 3,               // 60082:      decode to FCVTN_F322v16f8 using decoder 479
                                             // 60082:     }
  1, 0,                                      // 60087:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 60089:      check predicate 76
  OPC_Decode, 145, 27, 225, 3,               // 60091:      decode to FMAXv4f32 using decoder 481
                                             // 60091:     }
                                             // 60091:     } // switch Inst[21]
                                             // 60091:    }
  62, 21,                                    // 60096:    case 0x3e: {
  OPC_SwitchField, 16, 6,                    // 60098:     switch Inst[21:16] {
  33, 7,                                     // 60101:     case 0x21: {
  OPC_CheckPredicate, 82,                    // 60103:      check predicate 82
  OPC_Decode, 128, 31, 230, 3,               // 60105:      decode to FRINT64Zv4f32 using decoder 486
                                             // 60105:     }
  48, 0,                                     // 60110:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 60112:      check predicate 81
  OPC_Decode, 251, 26, 244, 3,               // 60114:      decode to FMAXVv8i16v using decoder 500
                                             // 60114:     }
                                             // 60114:     } // switch Inst[21:16]
                                             // 60114:    }
  63, 0,                                     // 60119:    case 0x3f: {
  OPC_SwitchField, 21, 1,                    // 60121:     switch Inst[21] {
  0, 7,                                      // 60124:     case 0x0: {
  OPC_CheckPredicate, 85,                    // 60126:      check predicate 85
  OPC_Decode, 157, 26, 223, 3,               // 60128:      decode to FDOTv4f32 using decoder 479
                                             // 60128:     }
  1, 0,                                      // 60133:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 60135:      check predicate 76
  OPC_Decode, 210, 30, 225, 3,               // 60137:      decode to FRECPSv4f32 using decoder 481
                                             // 60137:     }
                                             // 60137:     } // switch Inst[21]
                                             // 60137:    }
                                             // 60137:    } // switch Inst[15:10]
                                             // 60137:   }
  3, 230, 6,                                 // 60142:   case 0x3: {
  OPC_SwitchField, 10, 1,                    // 60145:    switch Inst[10] {
  0, 196, 3,                                 // 60148:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 60151:     switch Inst[15] {
  0, 229, 1,                                 // 60154:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 60157:      switch Inst[21] {
  0, 7,                                      // 60160:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 60162:       check predicate 76
  OPC_Decode, 166, 21, 251, 3,               // 60164:       decode to EXTv16i8 using decoder 507
                                             // 60164:      }
  1, 0,                                      // 60169:      case 0x1: {
  OPC_SwitchField, 11, 4,                    // 60171:       switch Inst[14:11] {
  0, 7,                                      // 60174:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 60176:        check predicate 76
  OPC_Decode, 206, 62, 225, 3,               // 60178:        decode to UADDLv16i8_v8i16 using decoder 481
                                             // 60178:       }
  1, 11,                                     // 60183:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 60185:        check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 60187:        check Inst[20:16] == 0x0
  OPC_Decode, 195, 45, 230, 3,               // 60191:        decode to REV32v16i8 using decoder 486
                                             // 60191:       }
  2, 7,                                      // 60196:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 60198:        check predicate 76
  OPC_Decode, 222, 62, 225, 3,               // 60200:        decode to UADDWv16i8_v8i16 using decoder 481
                                             // 60200:       }
  4, 7,                                      // 60205:       case 0x4: {
  OPC_CheckPredicate, 76,                    // 60207:        check predicate 76
  OPC_Decode, 247, 68, 225, 3,               // 60209:        decode to USUBLv16i8_v8i16 using decoder 481
                                             // 60209:       }
  5, 21,                                     // 60214:       case 0x5: {
  OPC_SwitchField, 16, 5,                    // 60216:        switch Inst[20:16] {
  0, 7,                                      // 60219:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 60221:         check predicate 76
  OPC_Decode, 192, 62, 230, 3,               // 60223:         decode to UADDLPv16i8_v8i16 using decoder 486
                                             // 60223:        }
  1, 0,                                      // 60228:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 60230:         check predicate 76
  OPC_Decode, 220, 54, 238, 3,               // 60232:         decode to SQXTUNv16i8 using decoder 494
                                             // 60232:        }
                                             // 60232:        } // switch Inst[20:16]
                                             // 60232:       }
  6, 7,                                      // 60237:       case 0x6: {
  OPC_CheckPredicate, 76,                    // 60239:        check predicate 76
  OPC_Decode, 131, 69, 225, 3,               // 60241:        decode to USUBWv16i8_v8i16 using decoder 481
                                             // 60241:       }
  7, 30,                                     // 60246:       case 0x7: {
  OPC_SwitchField, 16, 5,                    // 60248:        switch Inst[20:16] {
  0, 7,                                      // 60251:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 60253:         check predicate 76
  OPC_Decode, 217, 68, 238, 3,               // 60255:         decode to USQADDv16i8 using decoder 494
                                             // 60255:        }
  1, 7,                                      // 60260:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 60262:         check predicate 76
  OPC_Decode, 155, 48, 230, 3,               // 60264:         decode to SHLLv16i8 using decoder 486
                                             // 60264:        }
  16, 0,                                     // 60269:        case 0x10: {
  OPC_CheckPredicate, 76,                    // 60271:         check predicate 76
  OPC_Decode, 201, 62, 244, 3,               // 60273:         decode to UADDLVv16i8v using decoder 500
                                             // 60273:        }
                                             // 60273:        } // switch Inst[20:16]
                                             // 60273:       }
  8, 7,                                      // 60278:       case 0x8: {
  OPC_CheckPredicate, 76,                    // 60280:        check predicate 76
  OPC_Decode, 226, 44, 223, 3,               // 60282:        decode to RADDHNv8i16_v16i8 using decoder 479
                                             // 60282:       }
  9, 21,                                     // 60287:       case 0x9: {
  OPC_SwitchField, 16, 5,                    // 60289:        switch Inst[20:16] {
  0, 7,                                      // 60292:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 60294:         check predicate 76
  OPC_Decode, 201, 17, 230, 3,               // 60296:         decode to CLZv16i8 using decoder 486
                                             // 60296:        }
  1, 0,                                      // 60301:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 60303:         check predicate 76
  OPC_Decode, 188, 67, 238, 3,               // 60305:         decode to UQXTNv16i8 using decoder 494
                                             // 60305:        }
                                             // 60305:        } // switch Inst[20:16]
                                             // 60305:       }
  10, 7,                                     // 60310:       case 0xa: {
  OPC_CheckPredicate, 76,                    // 60312:        check predicate 76
  OPC_Decode, 142, 62, 223, 3,               // 60314:        decode to UABALv16i8_v8i16 using decoder 479
                                             // 60314:       }
  11, 11,                                    // 60319:       case 0xb: {
  OPC_CheckPredicate, 76,                    // 60321:        check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 60323:        check Inst[20:16] == 0x0
  OPC_Decode, 191, 43, 230, 3,               // 60327:        decode to NOTv16i8 using decoder 486
                                             // 60327:       }
  12, 7,                                     // 60332:       case 0xc: {
  OPC_CheckPredicate, 76,                    // 60334:        check predicate 76
  OPC_Decode, 255, 45, 223, 3,               // 60336:        decode to RSUBHNv8i16_v16i8 using decoder 479
                                             // 60336:       }
  13, 11,                                    // 60341:       case 0xd: {
  OPC_CheckPredicate, 76,                    // 60343:        check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 60345:        check Inst[20:16] == 0x0
  OPC_Decode, 183, 62, 238, 3,               // 60349:        decode to UADALPv16i8_v8i16 using decoder 494
                                             // 60349:       }
  14, 7,                                     // 60354:       case 0xe: {
  OPC_CheckPredicate, 76,                    // 60356:        check predicate 76
  OPC_Decode, 164, 62, 225, 3,               // 60358:        decode to UABDLv16i8_v8i16 using decoder 481
                                             // 60358:       }
  15, 0,                                     // 60363:       case 0xf: {
  OPC_SwitchField, 16, 5,                    // 60365:        switch Inst[20:16] {
  0, 7,                                      // 60368:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 60370:         check predicate 76
  OPC_Decode, 208, 52, 230, 3,               // 60372:         decode to SQNEGv16i8 using decoder 486
                                             // 60372:        }
  1, 0,                                      // 60377:        case 0x1: {
  OPC_CheckPredicate, 84,                    // 60379:         check predicate 84
  OPC_Decode, 169, 21, 230, 3,               // 60381:         decode to F1CVTL2 using decoder 486
                                             // 60381:        }
                                             // 60381:        } // switch Inst[20:16]
                                             // 60381:       }
                                             // 60381:       } // switch Inst[14:11]
                                             // 60381:      }
                                             // 60381:      } // switch Inst[21]
                                             // 60381:     }
  1, 0,                                      // 60386:     case 0x1: {
  OPC_SwitchField, 11, 4,                    // 60388:      switch Inst[14:11] {
  0, 11,                                     // 60391:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 60393:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60395:       check Inst[21] == 0x1
  OPC_Decode, 250, 64, 223, 3,               // 60399:       decode to UMLALv16i8_v8i16 using decoder 479
                                             // 60399:      }
  1, 21,                                     // 60404:      case 0x1: {
  OPC_SwitchField, 16, 6,                    // 60406:       switch Inst[21:16] {
  32, 7,                                     // 60409:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 60411:        check predicate 76
  OPC_Decode, 224, 17, 230, 3,               // 60413:        decode to CMGEv16i8rz using decoder 486
                                             // 60413:       }
  33, 0,                                     // 60418:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 60420:        check predicate 76
  OPC_Decode, 143, 31, 230, 3,               // 60422:        decode to FRINTAv4f32 using decoder 486
                                             // 60422:       }
                                             // 60422:       } // switch Inst[21:16]
                                             // 60422:      }
  3, 21,                                     // 60427:      case 0x3: {
  OPC_SwitchField, 16, 6,                    // 60429:       switch Inst[21:16] {
  32, 7,                                     // 60432:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 60434:        check predicate 76
  OPC_Decode, 149, 18, 230, 3,               // 60436:        decode to CMLEv16i8rz using decoder 486
                                             // 60436:       }
  33, 0,                                     // 60441:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 60443:        check predicate 76
  OPC_Decode, 219, 31, 230, 3,               // 60445:        decode to FRINTXv4f32 using decoder 486
                                             // 60445:       }
                                             // 60445:       } // switch Inst[21:16]
                                             // 60445:      }
  4, 11,                                     // 60450:      case 0x4: {
  OPC_CheckPredicate, 76,                    // 60452:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60454:       check Inst[21] == 0x1
  OPC_Decode, 166, 65, 223, 3,               // 60458:       decode to UMLSLv16i8_v8i16 using decoder 479
                                             // 60458:      }
  5, 30,                                     // 60463:      case 0x5: {
  OPC_SwitchField, 16, 6,                    // 60465:       switch Inst[21:16] {
  33, 7,                                     // 60468:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 60470:        check predicate 76
  OPC_Decode, 190, 24, 230, 3,               // 60472:        decode to FCVTNUv4f32 using decoder 486
                                             // 60472:       }
  48, 7,                                     // 60477:       case 0x30: {
  OPC_CheckPredicate, 76,                    // 60479:        check predicate 76
  OPC_Decode, 248, 63, 250, 3,               // 60481:        decode to UMAXVv16i8v using decoder 506
                                             // 60481:       }
  49, 0,                                     // 60486:       case 0x31: {
  OPC_CheckPredicate, 76,                    // 60488:        check predicate 76
  OPC_Decode, 177, 64, 250, 3,               // 60490:        decode to UMINVv16i8v using decoder 506
                                             // 60490:       }
                                             // 60490:       } // switch Inst[21:16]
                                             // 60490:      }
  7, 21,                                     // 60495:      case 0x7: {
  OPC_SwitchField, 16, 6,                    // 60497:       switch Inst[21:16] {
  32, 7,                                     // 60500:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 60502:        check predicate 76
  OPC_Decode, 170, 43, 230, 3,               // 60504:        decode to NEGv16i8 using decoder 486
                                             // 60504:       }
  33, 0,                                     // 60509:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 60511:        check predicate 76
  OPC_Decode, 148, 24, 230, 3,               // 60513:        decode to FCVTMUv4f32 using decoder 486
                                             // 60513:       }
                                             // 60513:       } // switch Inst[21:16]
                                             // 60513:      }
  8, 11,                                     // 60518:      case 0x8: {
  OPC_CheckPredicate, 76,                    // 60520:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60522:       check Inst[21] == 0x1
  OPC_Decode, 236, 65, 225, 3,               // 60526:       decode to UMULLv16i8_v8i16 using decoder 481
                                             // 60526:      }
  9, 21,                                     // 60531:      case 0x9: {
  OPC_SwitchField, 16, 6,                    // 60533:       switch Inst[21:16] {
  33, 7,                                     // 60536:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 60538:        check predicate 76
  OPC_Decode, 227, 23, 230, 3,               // 60540:        decode to FCVTAUv4f32 using decoder 486
                                             // 60540:       }
  48, 0,                                     // 60545:       case 0x30: {
  OPC_CheckPredicate, 76,                    // 60547:        check predicate 76
  OPC_Decode, 206, 26, 252, 3,               // 60549:        decode to FMAXNMVv4i32v using decoder 508
                                             // 60549:       }
                                             // 60549:       } // switch Inst[21:16]
                                             // 60549:      }
  11, 11,                                    // 60554:      case 0xb: {
  OPC_CheckPredicate, 76,                    // 60556:       check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 60558:       check Inst[21:16] == 0x21
  OPC_Decode, 163, 63, 230, 3,               // 60562:       decode to UCVTFv4f32 using decoder 486
                                             // 60562:      }
  13, 11,                                    // 60567:      case 0xd: {
  OPC_CheckPredicate, 82,                    // 60569:       check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 60571:       check Inst[21:16] == 0x21
  OPC_Decode, 229, 30, 230, 3,               // 60575:       decode to FRINT32Xv4f32 using decoder 486
                                             // 60575:      }
  15, 0,                                     // 60580:      case 0xf: {
  OPC_SwitchField, 16, 6,                    // 60582:       switch Inst[21:16] {
  33, 7,                                     // 60585:       case 0x21: {
  OPC_CheckPredicate, 82,                    // 60587:        check predicate 82
  OPC_Decode, 247, 30, 230, 3,               // 60589:        decode to FRINT64Xv4f32 using decoder 486
                                             // 60589:       }
  48, 0,                                     // 60594:       case 0x30: {
  OPC_CheckPredicate, 76,                    // 60596:        check predicate 76
  OPC_Decode, 250, 26, 252, 3,               // 60598:        decode to FMAXVv4i32v using decoder 508
                                             // 60598:       }
                                             // 60598:       } // switch Inst[21:16]
                                             // 60598:      }
                                             // 60598:      } // switch Inst[14:11]
                                             // 60598:     }
                                             // 60598:     } // switch Inst[15]
                                             // 60598:    }
  1, 0,                                      // 60603:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 60605:     switch Inst[15] {
  0, 207, 1,                                 // 60608:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 60611:      switch Inst[21] {
  0, 53,                                     // 60614:      case 0x0: {
  OPC_SwitchField, 16, 1,                    // 60616:       switch Inst[16] {
  0, 39,                                     // 60619:       case 0x0: {
  OPC_SwitchField, 17, 1,                    // 60621:        switch Inst[17] {
  0, 25,                                     // 60624:        case 0x0: {
  OPC_SwitchField, 18, 1,                    // 60626:         switch Inst[18] {
  0, 11,                                     // 60629:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 60631:          check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 60633:          check Inst[19] == 0x1
  OPC_Decode, 152, 34, 253, 3,               // 60637:          decode to INSvi64lane using decoder 509
                                             // 60637:         }
  1, 0,                                      // 60642:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 60644:          check predicate 76
  OPC_Decode, 150, 34, 254, 3,               // 60646:          decode to INSvi32lane using decoder 510
                                             // 60646:         }
                                             // 60646:         } // switch Inst[18]
                                             // 60646:        }
  1, 0,                                      // 60651:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 60653:         check predicate 76
  OPC_Decode, 148, 34, 255, 3,               // 60655:         decode to INSvi16lane using decoder 511
                                             // 60655:        }
                                             // 60655:        } // switch Inst[17]
                                             // 60655:       }
  1, 0,                                      // 60660:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 60662:        check predicate 76
  OPC_Decode, 154, 34, 128, 4,               // 60664:        decode to INSvi8lane using decoder 512
                                             // 60664:       }
                                             // 60664:       } // switch Inst[16]
                                             // 60664:      }
  1, 0,                                      // 60669:      case 0x1: {
  OPC_SwitchField, 11, 4,                    // 60671:       switch Inst[14:11] {
  0, 7,                                      // 60674:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 60676:        check predicate 76
  OPC_Decode, 209, 63, 225, 3,               // 60678:        decode to UHADDv16i8 using decoder 481
                                             // 60678:       }
  1, 7,                                      // 60683:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 60685:        check predicate 76
  OPC_Decode, 130, 66, 225, 3,               // 60687:        decode to UQADDv16i8 using decoder 481
                                             // 60687:       }
  2, 7,                                      // 60692:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 60694:        check predicate 76
  OPC_Decode, 205, 67, 225, 3,               // 60696:        decode to URHADDv16i8 using decoder 481
                                             // 60696:       }
  3, 7,                                      // 60701:       case 0x3: {
  OPC_CheckPredicate, 76,                    // 60703:        check predicate 76
  OPC_Decode, 142, 21, 225, 3,               // 60705:        decode to EORv16i8 using decoder 481
                                             // 60705:       }
  4, 7,                                      // 60710:       case 0x4: {
  OPC_CheckPredicate, 76,                    // 60712:        check predicate 76
  OPC_Decode, 223, 63, 225, 3,               // 60714:        decode to UHSUBv16i8 using decoder 481
                                             // 60714:       }
  5, 7,                                      // 60719:       case 0x5: {
  OPC_CheckPredicate, 76,                    // 60721:        check predicate 76
  OPC_Decode, 171, 67, 225, 3,               // 60723:        decode to UQSUBv16i8 using decoder 481
                                             // 60723:       }
  6, 7,                                      // 60728:       case 0x6: {
  OPC_CheckPredicate, 76,                    // 60730:        check predicate 76
  OPC_Decode, 255, 17, 225, 3,               // 60732:        decode to CMHIv16i8 using decoder 481
                                             // 60732:       }
  7, 7,                                      // 60737:       case 0x7: {
  OPC_CheckPredicate, 76,                    // 60739:        check predicate 76
  OPC_Decode, 135, 18, 225, 3,               // 60741:        decode to CMHSv16i8 using decoder 481
                                             // 60741:       }
  8, 7,                                      // 60746:       case 0x8: {
  OPC_CheckPredicate, 76,                    // 60748:        check predicate 76
  OPC_Decode, 167, 68, 225, 3,               // 60750:        decode to USHLv16i8 using decoder 481
                                             // 60750:       }
  9, 7,                                      // 60755:       case 0x9: {
  OPC_CheckPredicate, 76,                    // 60757:        check predicate 76
  OPC_Decode, 248, 66, 225, 3,               // 60759:        decode to UQSHLv16i8 using decoder 481
                                             // 60759:       }
  10, 7,                                     // 60764:       case 0xa: {
  OPC_CheckPredicate, 76,                    // 60766:        check predicate 76
  OPC_Decode, 235, 67, 225, 3,               // 60768:        decode to URSHLv16i8 using decoder 481
                                             // 60768:       }
  11, 7,                                     // 60773:       case 0xb: {
  OPC_CheckPredicate, 76,                    // 60775:        check predicate 76
  OPC_Decode, 199, 66, 225, 3,               // 60777:        decode to UQRSHLv16i8 using decoder 481
                                             // 60777:       }
  12, 7,                                     // 60782:       case 0xc: {
  OPC_CheckPredicate, 76,                    // 60784:        check predicate 76
  OPC_Decode, 153, 64, 225, 3,               // 60786:        decode to UMAXv16i8 using decoder 481
                                             // 60786:       }
  13, 7,                                     // 60791:       case 0xd: {
  OPC_CheckPredicate, 76,                    // 60793:        check predicate 76
  OPC_Decode, 210, 64, 225, 3,               // 60795:        decode to UMINv16i8 using decoder 481
                                             // 60795:       }
  14, 7,                                     // 60800:       case 0xe: {
  OPC_CheckPredicate, 76,                    // 60802:        check predicate 76
  OPC_Decode, 174, 62, 225, 3,               // 60804:        decode to UABDv16i8 using decoder 481
                                             // 60804:       }
  15, 0,                                     // 60809:       case 0xf: {
  OPC_CheckPredicate, 76,                    // 60811:        check predicate 76
  OPC_Decode, 152, 62, 223, 3,               // 60813:        decode to UABAv16i8 using decoder 479
                                             // 60813:       }
                                             // 60813:       } // switch Inst[14:11]
                                             // 60813:      }
                                             // 60813:      } // switch Inst[21]
                                             // 60813:     }
  1, 0,                                      // 60818:     case 0x1: {
  OPC_SwitchField, 11, 4,                    // 60820:      switch Inst[14:11] {
  0, 11,                                     // 60823:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 60825:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60827:       check Inst[21] == 0x1
  OPC_Decode, 192, 60, 225, 3,               // 60831:       decode to SUBv16i8 using decoder 481
                                             // 60831:      }
  1, 11,                                     // 60836:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 60838:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60840:       check Inst[21] == 0x1
  OPC_Decode, 207, 17, 225, 3,               // 60844:       decode to CMEQv16i8 using decoder 481
                                             // 60844:      }
  2, 11,                                     // 60849:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 60851:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60853:       check Inst[21] == 0x1
  OPC_Decode, 136, 42, 223, 3,               // 60857:       decode to MLSv16i8 using decoder 479
                                             // 60857:      }
  3, 11,                                     // 60862:      case 0x3: {
  OPC_CheckPredicate, 76,                    // 60864:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60866:       check Inst[21] == 0x1
  OPC_Decode, 150, 44, 225, 3,               // 60870:       decode to PMULv16i8 using decoder 481
                                             // 60870:      }
  4, 11,                                     // 60875:      case 0x4: {
  OPC_CheckPredicate, 76,                    // 60877:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60879:       check Inst[21] == 0x1
  OPC_Decode, 234, 63, 225, 3,               // 60883:       decode to UMAXPv16i8 using decoder 481
                                             // 60883:      }
  5, 11,                                     // 60888:      case 0x5: {
  OPC_CheckPredicate, 76,                    // 60890:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60892:       check Inst[21] == 0x1
  OPC_Decode, 163, 64, 225, 3,               // 60896:       decode to UMINPv16i8 using decoder 481
                                             // 60896:      }
  8, 11,                                     // 60901:      case 0x8: {
  OPC_CheckPredicate, 76,                    // 60903:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60905:       check Inst[21] == 0x1
  OPC_Decode, 196, 26, 225, 3,               // 60909:       decode to FMAXNMPv4f32 using decoder 481
                                             // 60909:      }
  9, 11,                                     // 60914:      case 0x9: {
  OPC_CheckPredicate, 83,                    // 60916:       check predicate 83
  OPC_CheckField, 21, 1, 1,                  // 60918:       check Inst[21] == 0x1
  OPC_Decode, 242, 27, 223, 3,               // 60922:       decode to FMLAL2v8f16 using decoder 479
                                             // 60922:      }
  10, 11,                                    // 60927:      case 0xa: {
  OPC_CheckPredicate, 76,                    // 60929:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60931:       check Inst[21] == 0x1
  OPC_Decode, 241, 21, 225, 3,               // 60935:       decode to FADDPv4f32 using decoder 481
                                             // 60935:      }
  11, 11,                                    // 60940:      case 0xb: {
  OPC_CheckPredicate, 76,                    // 60942:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60944:       check Inst[21] == 0x1
  OPC_Decode, 150, 30, 225, 3,               // 60948:       decode to FMULv4f32 using decoder 481
                                             // 60948:      }
  12, 11,                                    // 60953:      case 0xc: {
  OPC_CheckPredicate, 76,                    // 60955:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60957:       check Inst[21] == 0x1
  OPC_Decode, 232, 22, 225, 3,               // 60961:       decode to FCMGEv4f32 using decoder 481
                                             // 60961:      }
  13, 21,                                    // 60966:      case 0xd: {
  OPC_SwitchField, 21, 1,                    // 60968:       switch Inst[21] {
  0, 7,                                      // 60971:       case 0x0: {
  OPC_CheckPredicate, 87,                    // 60973:        check predicate 87
  OPC_Decode, 144, 29, 223, 3,               // 60975:        decode to FMMLAv8f16 using decoder 479
                                             // 60975:       }
  1, 0,                                      // 60980:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 60982:        check predicate 76
  OPC_Decode, 214, 21, 225, 3,               // 60984:        decode to FACGEv4f32 using decoder 481
                                             // 60984:       }
                                             // 60984:       } // switch Inst[21]
                                             // 60984:      }
  14, 11,                                    // 60989:      case 0xe: {
  OPC_CheckPredicate, 76,                    // 60991:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 60993:       check Inst[21] == 0x1
  OPC_Decode, 240, 26, 225, 3,               // 60997:       decode to FMAXPv4f32 using decoder 481
                                             // 60997:      }
  15, 0,                                     // 61002:      case 0xf: {
  OPC_CheckPredicate, 76,                    // 61004:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61006:       check Inst[21] == 0x1
  OPC_Decode, 250, 25, 225, 3,               // 61010:       decode to FDIVv4f32 using decoder 481
                                             // 61010:      }
                                             // 61010:      } // switch Inst[14:11]
                                             // 61010:     }
                                             // 61010:     } // switch Inst[15]
                                             // 61010:    }
                                             // 61010:    } // switch Inst[10]
                                             // 61010:   }
  6, 0,                                      // 61015:   case 0x6: {
  OPC_SwitchField, 21, 1,                    // 61017:    switch Inst[21] {
  0, 11,                                     // 61020:    case 0x0: {
  OPC_CheckPredicate, 88,                    // 61022:     check predicate 88
  OPC_CheckField, 15, 1, 0,                  // 61024:     check Inst[15] == 0x0
  OPC_Decode, 240, 20, 129, 4,               // 61028:     decode to EOR3 using decoder 513
                                             // 61028:    }
  1, 0,                                      // 61033:    case 0x1: {
  OPC_CheckPredicate, 88,                    // 61035:     check predicate 88
  OPC_CheckField, 15, 1, 0,                  // 61037:     check Inst[15] == 0x0
  OPC_Decode, 224, 14, 129, 4,               // 61041:     decode to BCAX using decoder 513
                                             // 61041:    }
                                             // 61041:    } // switch Inst[21]
                                             // 61041:   }
                                             // 61041:   } // switch Inst[31:29]
                                             // 61041:  }
  9, 252, 33,                                // 61046:  case 0x9: {
  OPC_SwitchField, 29, 3,                    // 61049:   switch Inst[31:29] {
  0, 193, 7,                                 // 61052:   case 0x0: {
  OPC_SwitchField, 10, 6,                    // 61055:    switch Inst[15:10] {
  0, 11,                                     // 61058:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 61060:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61062:     check Inst[21] == 0x1
  OPC_Decode, 207, 46, 196, 3,               // 61066:     decode to SADDLv4i16_v4i32 using decoder 452
                                             // 61066:    }
  1, 21,                                     // 61071:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 61073:     switch Inst[21] {
  0, 7,                                      // 61076:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61078:      check predicate 81
  OPC_Decode, 228, 26, 200, 3,               // 61080:      decode to FMAXNMv4f16 using decoder 456
                                             // 61080:     }
  1, 0,                                      // 61085:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61087:      check predicate 76
  OPC_Decode, 151, 48, 200, 3,               // 61089:      decode to SHADDv4i16 using decoder 456
                                             // 61089:     }
                                             // 61089:     } // switch Inst[21]
                                             // 61089:    }
  2, 11,                                     // 61094:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 61096:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 61098:     check Inst[21:16] == 0x20
  OPC_Decode, 201, 45, 201, 3,               // 61102:     decode to REV64v4i16 using decoder 457
                                             // 61102:    }
  3, 21,                                     // 61107:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 61109:     switch Inst[21] {
  0, 7,                                      // 61112:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61114:      check predicate 81
  OPC_Decode, 202, 28, 221, 3,               // 61116:      decode to FMLAv4f16 using decoder 477
                                             // 61116:     }
  1, 0,                                      // 61121:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61123:      check predicate 76
  OPC_Decode, 157, 51, 200, 3,               // 61125:      decode to SQADDv4i16 using decoder 456
                                             // 61125:     }
                                             // 61125:     } // switch Inst[21]
                                             // 61125:    }
  4, 11,                                     // 61130:    case 0x4: {
  OPC_CheckPredicate, 76,                    // 61132:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61134:     check Inst[21] == 0x1
  OPC_Decode, 222, 46, 204, 3,               // 61138:     decode to SADDWv4i16_v4i32 using decoder 460
                                             // 61138:    }
  5, 21,                                     // 61143:    case 0x5: {
  OPC_SwitchField, 21, 1,                    // 61145:     switch Inst[21] {
  0, 7,                                      // 61148:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61150:      check predicate 81
  OPC_Decode, 139, 22, 200, 3,               // 61152:      decode to FADDv4f16 using decoder 456
                                             // 61152:     }
  1, 0,                                      // 61157:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61159:      check predicate 76
  OPC_Decode, 235, 54, 200, 3,               // 61161:      decode to SRHADDv4i16 using decoder 456
                                             // 61161:     }
                                             // 61161:     } // switch Inst[21]
                                             // 61161:    }
  6, 11,                                     // 61166:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 61168:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 61170:     check Inst[21] == 0x0
  OPC_Decode, 179, 69, 200, 3,               // 61174:     decode to UZP1v4i16 using decoder 456
                                             // 61174:    }
  7, 21,                                     // 61179:    case 0x7: {
  OPC_SwitchField, 21, 1,                    // 61181:     switch Inst[21] {
  0, 7,                                      // 61184:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61186:      check predicate 81
  OPC_Decode, 240, 29, 200, 3,               // 61188:      decode to FMULXv4f16 using decoder 456
                                             // 61188:     }
  1, 0,                                      // 61193:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61195:      check predicate 76
  OPC_Decode, 149, 16, 200, 3,               // 61197:      decode to BICv8i8 using decoder 456
                                             // 61197:     }
                                             // 61197:     } // switch Inst[21]
                                             // 61197:    }
  8, 11,                                     // 61202:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 61204:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61206:     check Inst[21] == 0x1
  OPC_Decode, 144, 56, 196, 3,               // 61210:     decode to SSUBLv4i16_v4i32 using decoder 452
                                             // 61210:    }
  9, 21,                                     // 61215:    case 0x9: {
  OPC_SwitchField, 21, 1,                    // 61217:     switch Inst[21] {
  0, 7,                                      // 61220:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61222:      check predicate 81
  OPC_Decode, 209, 22, 200, 3,               // 61224:      decode to FCMEQv4f16 using decoder 456
                                             // 61224:     }
  1, 0,                                      // 61229:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61231:      check predicate 76
  OPC_Decode, 191, 48, 200, 3,               // 61233:      decode to SHSUBv4i16 using decoder 456
                                             // 61233:     }
                                             // 61233:     } // switch Inst[21]
                                             // 61233:    }
  10, 35,                                    // 61238:    case 0xa: {
  OPC_SwitchField, 21, 1,                    // 61240:     switch Inst[21] {
  0, 7,                                      // 61243:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 61245:      check predicate 76
  OPC_Decode, 240, 61, 200, 3,               // 61247:      decode to TRN1v4i16 using decoder 456
                                             // 61247:     }
  1, 0,                                      // 61252:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 61254:      switch Inst[20:16] {
  0, 7,                                      // 61257:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 61259:       check predicate 76
  OPC_Decode, 193, 46, 201, 3,               // 61261:       decode to SADDLPv4i16_v2i32 using decoder 457
                                             // 61261:      }
  1, 0,                                      // 61266:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 61268:       check predicate 76
  OPC_Decode, 239, 70, 206, 3,               // 61270:       decode to XTNv4i16 using decoder 462
                                             // 61270:      }
                                             // 61270:      } // switch Inst[20:16]
                                             // 61270:     }
                                             // 61270:     } // switch Inst[21]
                                             // 61270:    }
  11, 11,                                    // 61275:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 61277:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61279:     check Inst[21] == 0x1
  OPC_Decode, 195, 54, 200, 3,               // 61283:     decode to SQSUBv4i16 using decoder 456
                                             // 61283:    }
  12, 11,                                    // 61288:    case 0xc: {
  OPC_CheckPredicate, 76,                    // 61290:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61292:     check Inst[21] == 0x1
  OPC_Decode, 156, 56, 204, 3,               // 61296:     decode to SSUBWv4i16_v4i32 using decoder 460
                                             // 61296:    }
  13, 21,                                    // 61301:    case 0xd: {
  OPC_SwitchField, 21, 1,                    // 61303:     switch Inst[21] {
  0, 7,                                      // 61306:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61308:      check predicate 81
  OPC_Decode, 144, 27, 200, 3,               // 61310:      decode to FMAXv4f16 using decoder 456
                                             // 61310:     }
  1, 0,                                      // 61315:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61317:      check predicate 76
  OPC_Decode, 247, 17, 200, 3,               // 61319:      decode to CMGTv4i16 using decoder 456
                                             // 61319:     }
                                             // 61319:     } // switch Inst[21]
                                             // 61319:    }
  14, 35,                                    // 61324:    case 0xe: {
  OPC_SwitchField, 21, 1,                    // 61326:     switch Inst[21] {
  0, 7,                                      // 61329:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 61331:      check predicate 76
  OPC_Decode, 137, 71, 200, 3,               // 61333:      decode to ZIP1v4i16 using decoder 456
                                             // 61333:     }
  1, 0,                                      // 61338:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 61340:      switch Inst[20:16] {
  0, 7,                                      // 61343:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 61345:       check predicate 76
  OPC_Decode, 255, 60, 211, 3,               // 61347:       decode to SUQADDv4i16 using decoder 467
                                             // 61347:      }
  16, 0,                                     // 61352:      case 0x10: {
  OPC_CheckPredicate, 76,                    // 61354:       check predicate 76
  OPC_Decode, 201, 46, 130, 4,               // 61356:       decode to SADDLVv4i16v using decoder 514
                                             // 61356:      }
                                             // 61356:      } // switch Inst[20:16]
                                             // 61356:     }
                                             // 61356:     } // switch Inst[21]
                                             // 61356:    }
  15, 21,                                    // 61361:    case 0xf: {
  OPC_SwitchField, 21, 1,                    // 61363:     switch Inst[21] {
  0, 7,                                      // 61366:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 61368:      check predicate 81
  OPC_Decode, 209, 30, 200, 3,               // 61370:      decode to FRECPSv4f16 using decoder 456
                                             // 61370:     }
  1, 0,                                      // 61375:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 61377:      check predicate 76
  OPC_Decode, 231, 17, 200, 3,               // 61379:      decode to CMGEv4i16 using decoder 456
                                             // 61379:     }
                                             // 61379:     } // switch Inst[21]
                                             // 61379:    }
  16, 11,                                    // 61384:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 61386:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61388:     check Inst[21] == 0x1
  OPC_Decode, 145, 13, 215, 3,               // 61392:     decode to ADDHNv4i32_v4i16 using decoder 471
                                             // 61392:    }
  17, 11,                                    // 61397:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 61399:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61401:     check Inst[21] == 0x1
  OPC_Decode, 195, 55, 200, 3,               // 61405:     decode to SSHLv4i16 using decoder 456
                                             // 61405:    }
  18, 21,                                    // 61410:    case 0x12: {
  OPC_SwitchField, 16, 6,                    // 61412:     switch Inst[21:16] {
  32, 7,                                     // 61415:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 61417:      check predicate 76
  OPC_Decode, 187, 17, 201, 3,               // 61419:      decode to CLSv4i16 using decoder 457
                                             // 61419:     }
  33, 0,                                     // 61424:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 61426:      check predicate 76
  OPC_Decode, 210, 54, 206, 3,               // 61428:      decode to SQXTNv4i16 using decoder 462
                                             // 61428:     }
                                             // 61428:     } // switch Inst[21:16]
                                             // 61428:    }
  19, 11,                                    // 61433:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 61435:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61437:     check Inst[21] == 0x1
  OPC_Decode, 130, 54, 200, 3,               // 61441:     decode to SQSHLv4i16 using decoder 456
                                             // 61441:    }
  20, 11,                                    // 61446:    case 0x14: {
  OPC_CheckPredicate, 76,                    // 61448:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61450:     check Inst[21] == 0x1
  OPC_Decode, 140, 46, 217, 3,               // 61454:     decode to SABALv4i16_v4i32 using decoder 473
                                             // 61454:    }
  21, 11,                                    // 61459:    case 0x15: {
  OPC_CheckPredicate, 76,                    // 61461:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61463:     check Inst[21] == 0x1
  OPC_Decode, 151, 55, 200, 3,               // 61467:     decode to SRSHLv4i16 using decoder 456
                                             // 61467:    }
  22, 11,                                    // 61472:    case 0x16: {
  OPC_CheckPredicate, 76,                    // 61474:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 61476:     check Inst[21] == 0x0
  OPC_Decode, 195, 69, 200, 3,               // 61480:     decode to UZP2v4i16 using decoder 456
                                             // 61480:    }
  23, 11,                                    // 61485:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 61487:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61489:     check Inst[21] == 0x1
  OPC_Decode, 169, 53, 200, 3,               // 61493:     decode to SQRSHLv4i16 using decoder 456
                                             // 61493:    }
  24, 11,                                    // 61498:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 61500:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61502:     check Inst[21] == 0x1
  OPC_Decode, 133, 60, 215, 3,               // 61506:     decode to SUBHNv4i32_v4i16 using decoder 471
                                             // 61506:    }
  25, 11,                                    // 61511:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 61513:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61515:     check Inst[21] == 0x1
  OPC_Decode, 145, 49, 200, 3,               // 61519:     decode to SMAXv4i16 using decoder 456
                                             // 61519:    }
  26, 35,                                    // 61524:    case 0x1a: {
  OPC_SwitchField, 21, 1,                    // 61526:     switch Inst[21] {
  0, 7,                                      // 61529:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 61531:      check predicate 76
  OPC_Decode, 128, 62, 200, 3,               // 61533:      decode to TRN2v4i16 using decoder 456
                                             // 61533:     }
  1, 0,                                      // 61538:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 61540:      switch Inst[20:16] {
  0, 7,                                      // 61543:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 61545:       check predicate 76
  OPC_Decode, 181, 46, 211, 3,               // 61547:       decode to SADALPv4i16_v2i32 using decoder 467
                                             // 61547:      }
  1, 0,                                      // 61552:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 61554:       check predicate 76
  OPC_Decode, 199, 24, 206, 3,               // 61556:       decode to FCVTNv2i32 using decoder 462
                                             // 61556:      }
                                             // 61556:      } // switch Inst[20:16]
                                             // 61556:     }
                                             // 61556:     } // switch Inst[21]
                                             // 61556:    }
  27, 11,                                    // 61561:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 61563:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61565:     check Inst[21] == 0x1
  OPC_Decode, 203, 49, 200, 3,               // 61569:     decode to SMINv4i16 using decoder 456
                                             // 61569:    }
  28, 11,                                    // 61574:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 61576:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61578:     check Inst[21] == 0x1
  OPC_Decode, 162, 46, 196, 3,               // 61582:     decode to SABDLv4i16_v4i32 using decoder 452
                                             // 61582:    }
  29, 11,                                    // 61587:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 61589:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61591:     check Inst[21] == 0x1
  OPC_Decode, 172, 46, 200, 3,               // 61595:     decode to SABDv4i16 using decoder 456
                                             // 61595:    }
  30, 35,                                    // 61600:    case 0x1e: {
  OPC_SwitchField, 21, 1,                    // 61602:     switch Inst[21] {
  0, 7,                                      // 61605:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 61607:      check predicate 76
  OPC_Decode, 153, 71, 200, 3,               // 61609:      decode to ZIP2v4i16 using decoder 456
                                             // 61609:     }
  1, 0,                                      // 61614:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 61616:      switch Inst[20:16] {
  0, 7,                                      // 61619:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 61621:       check predicate 76
  OPC_Decode, 134, 51, 201, 3,               // 61623:       decode to SQABSv4i16 using decoder 457
                                             // 61623:      }
  1, 0,                                      // 61628:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 61630:       check predicate 76
  OPC_Decode, 238, 23, 220, 3,               // 61632:       decode to FCVTLv2i32 using decoder 476
                                             // 61632:      }
                                             // 61632:      } // switch Inst[20:16]
                                             // 61632:     }
                                             // 61632:     } // switch Inst[21]
                                             // 61632:    }
  31, 11,                                    // 61637:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 61639:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61641:     check Inst[21] == 0x1
  OPC_Decode, 150, 46, 221, 3,               // 61645:     decode to SABAv4i16 using decoder 477
                                             // 61645:    }
  32, 11,                                    // 61650:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 61652:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61654:     check Inst[21] == 0x1
  OPC_Decode, 245, 49, 217, 3,               // 61658:     decode to SMLALv4i16_v4i32 using decoder 473
                                             // 61658:    }
  33, 11,                                    // 61663:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 61665:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61667:     check Inst[21] == 0x1
  OPC_Decode, 237, 13, 200, 3,               // 61671:     decode to ADDv4i16 using decoder 456
                                             // 61671:    }
  34, 21,                                    // 61676:    case 0x22: {
  OPC_SwitchField, 16, 6,                    // 61678:     switch Inst[21:16] {
  32, 7,                                     // 61681:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 61683:      check predicate 76
  OPC_Decode, 248, 17, 201, 3,               // 61685:      decode to CMGTv4i16rz using decoder 457
                                             // 61685:     }
  57, 0,                                     // 61690:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 61692:      check predicate 81
  OPC_Decode, 188, 31, 201, 3,               // 61694:      decode to FRINTNv4f16 using decoder 457
                                             // 61694:     }
                                             // 61694:     } // switch Inst[21:16]
                                             // 61694:    }
  35, 11,                                    // 61699:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 61701:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61703:     check Inst[21] == 0x1
  OPC_Decode, 135, 19, 200, 3,               // 61707:     decode to CMTSTv4i16 using decoder 456
                                             // 61707:    }
  36, 11,                                    // 61712:    case 0x24: {
  OPC_CheckPredicate, 76,                    // 61714:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61716:     check Inst[21] == 0x1
  OPC_Decode, 219, 51, 217, 3,               // 61720:     decode to SQDMLALv4i16_v4i32 using decoder 473
                                             // 61720:    }
  37, 11,                                    // 61725:    case 0x25: {
  OPC_CheckPredicate, 76,                    // 61727:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61729:     check Inst[21] == 0x1
  OPC_Decode, 250, 41, 221, 3,               // 61733:     decode to MLAv4i16 using decoder 477
                                             // 61733:    }
  38, 21,                                    // 61738:    case 0x26: {
  OPC_SwitchField, 16, 6,                    // 61740:     switch Inst[21:16] {
  32, 7,                                     // 61743:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 61745:      check predicate 76
  OPC_Decode, 216, 17, 201, 3,               // 61747:      decode to CMEQv4i16rz using decoder 457
                                             // 61747:     }
  57, 0,                                     // 61752:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 61754:      check predicate 81
  OPC_Decode, 172, 31, 201, 3,               // 61756:      decode to FRINTMv4f16 using decoder 457
                                             // 61756:     }
                                             // 61756:     } // switch Inst[21:16]
                                             // 61756:    }
  39, 11,                                    // 61761:    case 0x27: {
  OPC_CheckPredicate, 76,                    // 61763:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61765:     check Inst[21] == 0x1
  OPC_Decode, 146, 43, 200, 3,               // 61769:     decode to MULv4i16 using decoder 456
                                             // 61769:    }
  40, 11,                                    // 61774:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 61776:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61778:     check Inst[21] == 0x1
  OPC_Decode, 161, 50, 217, 3,               // 61782:     decode to SMLSLv4i16_v4i32 using decoder 473
                                             // 61782:    }
  41, 11,                                    // 61787:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 61789:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61791:     check Inst[21] == 0x1
  OPC_Decode, 226, 48, 200, 3,               // 61795:     decode to SMAXPv4i16 using decoder 456
                                             // 61795:    }
  42, 39,                                    // 61800:    case 0x2a: {
  OPC_SwitchField, 16, 6,                    // 61802:     switch Inst[21:16] {
  32, 7,                                     // 61805:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 61807:      check predicate 76
  OPC_Decode, 161, 18, 201, 3,               // 61809:      decode to CMLTv4i16rz using decoder 457
                                             // 61809:     }
  48, 7,                                     // 61814:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 61816:      check predicate 76
  OPC_Decode, 239, 48, 212, 3,               // 61818:      decode to SMAXVv4i16v using decoder 468
                                             // 61818:     }
  49, 7,                                     // 61823:     case 0x31: {
  OPC_CheckPredicate, 76,                    // 61825:      check predicate 76
  OPC_Decode, 169, 49, 212, 3,               // 61827:      decode to SMINVv4i16v using decoder 468
                                             // 61827:     }
  57, 0,                                     // 61832:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 61834:      check predicate 81
  OPC_Decode, 166, 24, 201, 3,               // 61836:      decode to FCVTNSv4f16 using decoder 457
                                             // 61836:     }
                                             // 61836:     } // switch Inst[21:16]
                                             // 61836:    }
  43, 11,                                    // 61841:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 61843:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61845:     check Inst[21] == 0x1
  OPC_Decode, 156, 49, 200, 3,               // 61849:     decode to SMINPv4i16 using decoder 456
                                             // 61849:    }
  44, 11,                                    // 61854:    case 0x2c: {
  OPC_CheckPredicate, 76,                    // 61856:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61858:     check Inst[21] == 0x1
  OPC_Decode, 244, 51, 217, 3,               // 61862:     decode to SQDMLSLv4i16_v4i32 using decoder 473
                                             // 61862:    }
  45, 11,                                    // 61867:    case 0x2d: {
  OPC_CheckPredicate, 76,                    // 61869:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61871:     check Inst[21] == 0x1
  OPC_Decode, 150, 52, 200, 3,               // 61875:     decode to SQDMULHv4i16 using decoder 456
                                             // 61875:    }
  46, 30,                                    // 61880:    case 0x2e: {
  OPC_SwitchField, 16, 6,                    // 61882:     switch Inst[21:16] {
  32, 7,                                     // 61885:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 61887:      check predicate 76
  OPC_Decode, 250, 12, 201, 3,               // 61889:      decode to ABSv4i16 using decoder 457
                                             // 61889:     }
  49, 7,                                     // 61894:     case 0x31: {
  OPC_CheckPredicate, 76,                    // 61896:      check predicate 76
  OPC_Decode, 188, 13, 212, 3,               // 61898:      decode to ADDVv4i16v using decoder 468
                                             // 61898:     }
  57, 0,                                     // 61903:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 61905:      check predicate 81
  OPC_Decode, 129, 24, 201, 3,               // 61907:      decode to FCVTMSv4f16 using decoder 457
                                             // 61907:     }
                                             // 61907:     } // switch Inst[21:16]
                                             // 61907:    }
  47, 11,                                    // 61912:    case 0x2f: {
  OPC_CheckPredicate, 76,                    // 61914:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61916:     check Inst[21] == 0x1
  OPC_Decode, 159, 13, 200, 3,               // 61920:     decode to ADDPv4i16 using decoder 456
                                             // 61920:    }
  48, 11,                                    // 61925:    case 0x30: {
  OPC_CheckPredicate, 76,                    // 61927:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61929:     check Inst[21] == 0x1
  OPC_Decode, 233, 50, 196, 3,               // 61933:     decode to SMULLv4i16_v4i32 using decoder 452
                                             // 61933:    }
  49, 11,                                    // 61938:    case 0x31: {
  OPC_CheckPredicate, 80,                    // 61940:     check predicate 80
  OPC_CheckField, 21, 1, 0,                  // 61942:     check Inst[21] == 0x0
  OPC_Decode, 128, 28, 223, 3,               // 61946:     decode to FMLALLBTv4f32 using decoder 479
                                             // 61946:    }
  50, 11,                                    // 61951:    case 0x32: {
  OPC_CheckPredicate, 81,                    // 61953:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 61955:     check Inst[21:16] == 0x39
  OPC_Decode, 208, 23, 201, 3,               // 61959:     decode to FCVTASv4f16 using decoder 457
                                             // 61959:    }
  52, 11,                                    // 61964:    case 0x34: {
  OPC_CheckPredicate, 76,                    // 61966:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 61968:     check Inst[21] == 0x1
  OPC_Decode, 173, 52, 196, 3,               // 61972:     decode to SQDMULLv4i16_v4i32 using decoder 452
                                             // 61972:    }
  54, 11,                                    // 61977:    case 0x36: {
  OPC_CheckPredicate, 81,                    // 61979:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 61981:     check Inst[21:16] == 0x39
  OPC_Decode, 169, 47, 201, 3,               // 61985:     decode to SCVTFv4f16 using decoder 457
                                             // 61985:    }
  61, 11,                                    // 61990:    case 0x3d: {
  OPC_CheckPredicate, 84,                    // 61992:     check predicate 84
  OPC_CheckField, 21, 1, 0,                  // 61994:     check Inst[21] == 0x0
  OPC_Decode, 193, 24, 200, 3,               // 61998:     decode to FCVTN_F16v8f8 using decoder 456
                                             // 61998:    }
  63, 0,                                     // 62003:    case 0x3f: {
  OPC_CheckPredicate, 89,                    // 62005:     check predicate 89
  OPC_CheckField, 21, 1, 0,                  // 62007:     check Inst[21] == 0x0
  OPC_Decode, 155, 26, 221, 3,               // 62011:     decode to FDOTv4f16 using decoder 477
                                             // 62011:    }
                                             // 62011:    } // switch Inst[15:10]
                                             // 62011:   }
  1, 248, 6,                                 // 62016:   case 0x1: {
  OPC_SwitchField, 13, 3,                    // 62019:    switch Inst[15:13] {
  0, 124,                                    // 62022:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 62024:     switch Inst[12:10] {
  0, 11,                                     // 62027:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 62029:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62031:      check Inst[21] == 0x1
  OPC_Decode, 208, 62, 196, 3,               // 62035:      decode to UADDLv4i16_v4i32 using decoder 452
                                             // 62035:     }
  1, 21,                                     // 62040:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 62042:      switch Inst[21] {
  0, 7,                                      // 62045:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62047:       check predicate 81
  OPC_Decode, 195, 26, 200, 3,               // 62049:       decode to FMAXNMPv4f16 using decoder 456
                                             // 62049:      }
  1, 0,                                      // 62054:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62056:       check predicate 76
  OPC_Decode, 211, 63, 200, 3,               // 62058:       decode to UHADDv4i16 using decoder 456
                                             // 62058:      }
                                             // 62058:      } // switch Inst[21]
                                             // 62058:     }
  2, 11,                                     // 62063:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 62065:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 62067:      check Inst[21:16] == 0x20
  OPC_Decode, 196, 45, 201, 3,               // 62071:      decode to REV32v4i16 using decoder 457
                                             // 62071:     }
  3, 11,                                     // 62076:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 62078:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62080:      check Inst[21] == 0x1
  OPC_Decode, 137, 66, 200, 3,               // 62084:      decode to UQADDv4i16 using decoder 456
                                             // 62084:     }
  4, 11,                                     // 62089:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 62091:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62093:      check Inst[21] == 0x1
  OPC_Decode, 224, 62, 204, 3,               // 62097:      decode to UADDWv4i16_v4i32 using decoder 460
                                             // 62097:     }
  5, 21,                                     // 62102:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 62104:      switch Inst[21] {
  0, 7,                                      // 62107:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62109:       check predicate 81
  OPC_Decode, 240, 21, 200, 3,               // 62111:       decode to FADDPv4f16 using decoder 456
                                             // 62111:      }
  1, 0,                                      // 62116:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62118:       check predicate 76
  OPC_Decode, 207, 67, 200, 3,               // 62120:       decode to URHADDv4i16 using decoder 456
                                             // 62120:      }
                                             // 62120:      } // switch Inst[21]
                                             // 62120:     }
  7, 0,                                      // 62125:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 62127:      switch Inst[21] {
  0, 7,                                      // 62130:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62132:       check predicate 81
  OPC_Decode, 149, 30, 200, 3,               // 62134:       decode to FMULv4f16 using decoder 456
                                             // 62134:      }
  1, 0,                                      // 62139:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62141:       check predicate 76
  OPC_Decode, 186, 16, 221, 3,               // 62143:       decode to BSLv8i8 using decoder 477
                                             // 62143:      }
                                             // 62143:      } // switch Inst[21]
                                             // 62143:     }
                                             // 62143:     } // switch Inst[12:10]
                                             // 62143:    }
  1, 176, 1,                                 // 62148:    case 0x1: {
  OPC_SwitchField, 10, 3,                    // 62151:     switch Inst[12:10] {
  0, 11,                                     // 62154:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 62156:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62158:      check Inst[21] == 0x1
  OPC_Decode, 249, 68, 196, 3,               // 62162:      decode to USUBLv4i16_v4i32 using decoder 452
                                             // 62162:     }
  1, 21,                                     // 62167:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 62169:      switch Inst[21] {
  0, 7,                                      // 62172:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62174:       check predicate 81
  OPC_Decode, 231, 22, 200, 3,               // 62176:       decode to FCMGEv4f16 using decoder 456
                                             // 62176:      }
  1, 0,                                      // 62181:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62183:       check predicate 76
  OPC_Decode, 225, 63, 200, 3,               // 62185:       decode to UHSUBv4i16 using decoder 456
                                             // 62185:      }
                                             // 62185:      } // switch Inst[21]
                                             // 62185:     }
  2, 21,                                     // 62190:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 62192:      switch Inst[21:16] {
  32, 7,                                     // 62195:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62197:       check predicate 76
  OPC_Decode, 194, 62, 201, 3,               // 62199:       decode to UADDLPv4i16_v2i32 using decoder 457
                                             // 62199:      }
  33, 0,                                     // 62204:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 62206:       check predicate 76
  OPC_Decode, 225, 54, 206, 3,               // 62208:       decode to SQXTUNv4i16 using decoder 462
                                             // 62208:      }
                                             // 62208:      } // switch Inst[21:16]
                                             // 62208:     }
  3, 21,                                     // 62213:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 62215:      switch Inst[21] {
  0, 7,                                      // 62218:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62220:       check predicate 81
  OPC_Decode, 213, 21, 200, 3,               // 62222:       decode to FACGEv4f16 using decoder 456
                                             // 62222:      }
  1, 0,                                      // 62227:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62229:       check predicate 76
  OPC_Decode, 178, 67, 200, 3,               // 62231:       decode to UQSUBv4i16 using decoder 456
                                             // 62231:      }
                                             // 62231:      } // switch Inst[21]
                                             // 62231:     }
  4, 11,                                     // 62236:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 62238:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62240:      check Inst[21] == 0x1
  OPC_Decode, 133, 69, 204, 3,               // 62244:      decode to USUBWv4i16_v4i32 using decoder 460
                                             // 62244:     }
  5, 21,                                     // 62249:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 62251:      switch Inst[21] {
  0, 7,                                      // 62254:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62256:       check predicate 81
  OPC_Decode, 239, 26, 200, 3,               // 62258:       decode to FMAXPv4f16 using decoder 456
                                             // 62258:      }
  1, 0,                                      // 62263:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62265:       check predicate 76
  OPC_Decode, 131, 18, 200, 3,               // 62267:       decode to CMHIv4i16 using decoder 456
                                             // 62267:      }
                                             // 62267:      } // switch Inst[21]
                                             // 62267:     }
  6, 30,                                     // 62272:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 62274:      switch Inst[21:16] {
  32, 7,                                     // 62277:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62279:       check predicate 76
  OPC_Decode, 224, 68, 211, 3,               // 62281:       decode to USQADDv4i16 using decoder 467
                                             // 62281:      }
  33, 7,                                     // 62286:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 62288:       check predicate 76
  OPC_Decode, 157, 48, 220, 3,               // 62290:       decode to SHLLv4i16 using decoder 476
                                             // 62290:      }
  48, 0,                                     // 62295:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 62297:       check predicate 76
  OPC_Decode, 202, 62, 130, 4,               // 62299:       decode to UADDLVv4i16v using decoder 514
                                             // 62299:      }
                                             // 62299:      } // switch Inst[21:16]
                                             // 62299:     }
  7, 0,                                      // 62304:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 62306:      switch Inst[21] {
  0, 7,                                      // 62309:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 62311:       check predicate 81
  OPC_Decode, 249, 25, 200, 3,               // 62313:       decode to FDIVv4f16 using decoder 456
                                             // 62313:      }
  1, 0,                                      // 62318:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62320:       check predicate 76
  OPC_Decode, 139, 18, 200, 3,               // 62322:       decode to CMHSv4i16 using decoder 456
                                             // 62322:      }
                                             // 62322:      } // switch Inst[21]
                                             // 62322:     }
                                             // 62322:     } // switch Inst[12:10]
                                             // 62322:    }
  2, 117,                                    // 62327:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 62329:     switch Inst[12:10] {
  0, 11,                                     // 62332:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 62334:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62336:      check Inst[21] == 0x1
  OPC_Decode, 224, 44, 215, 3,               // 62340:      decode to RADDHNv4i32_v4i16 using decoder 471
                                             // 62340:     }
  1, 11,                                     // 62345:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 62347:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62349:      check Inst[21] == 0x1
  OPC_Decode, 171, 68, 200, 3,               // 62353:      decode to USHLv4i16 using decoder 456
                                             // 62353:     }
  2, 21,                                     // 62358:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 62360:      switch Inst[21:16] {
  32, 7,                                     // 62363:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62365:       check predicate 76
  OPC_Decode, 203, 17, 201, 3,               // 62367:       decode to CLZv4i16 using decoder 457
                                             // 62367:      }
  33, 0,                                     // 62372:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 62374:       check predicate 76
  OPC_Decode, 193, 67, 206, 3,               // 62376:       decode to UQXTNv4i16 using decoder 462
                                             // 62376:      }
                                             // 62376:      } // switch Inst[21:16]
                                             // 62376:     }
  3, 11,                                     // 62381:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 62383:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62385:      check Inst[21] == 0x1
  OPC_Decode, 130, 67, 200, 3,               // 62389:      decode to UQSHLv4i16 using decoder 456
                                             // 62389:     }
  4, 11,                                     // 62394:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 62396:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62398:      check Inst[21] == 0x1
  OPC_Decode, 144, 62, 217, 3,               // 62402:      decode to UABALv4i16_v4i32 using decoder 473
                                             // 62402:     }
  5, 11,                                     // 62407:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 62409:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62411:      check Inst[21] == 0x1
  OPC_Decode, 239, 67, 200, 3,               // 62415:      decode to URSHLv4i16 using decoder 456
                                             // 62415:     }
  6, 11,                                     // 62420:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 62422:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 62424:      check Inst[21:16] == 0x20
  OPC_Decode, 241, 44, 201, 3,               // 62428:      decode to RBITv8i8 using decoder 457
                                             // 62428:     }
  7, 0,                                      // 62433:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 62435:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62437:      check Inst[21] == 0x1
  OPC_Decode, 206, 66, 200, 3,               // 62441:      decode to UQRSHLv4i16 using decoder 456
                                             // 62441:     }
                                             // 62441:     } // switch Inst[12:10]
                                             // 62441:    }
  3, 127,                                    // 62446:    case 0x3: {
  OPC_SwitchField, 10, 3,                    // 62448:     switch Inst[12:10] {
  0, 11,                                     // 62451:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 62453:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62455:      check Inst[21] == 0x1
  OPC_Decode, 253, 45, 215, 3,               // 62459:      decode to RSUBHNv4i32_v4i16 using decoder 471
                                             // 62459:     }
  1, 11,                                     // 62464:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 62466:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62468:      check Inst[21] == 0x1
  OPC_Decode, 155, 64, 200, 3,               // 62472:      decode to UMAXv4i16 using decoder 456
                                             // 62472:     }
  2, 21,                                     // 62477:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 62479:      switch Inst[21:16] {
  32, 7,                                     // 62482:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62484:       check predicate 76
  OPC_Decode, 185, 62, 211, 3,               // 62486:       decode to UADALPv4i16_v2i32 using decoder 467
                                             // 62486:      }
  33, 0,                                     // 62491:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 62493:       check predicate 76
  OPC_Decode, 244, 24, 206, 3,               // 62495:       decode to FCVTXNv2f32 using decoder 462
                                             // 62495:      }
                                             // 62495:      } // switch Inst[21:16]
                                             // 62495:     }
  3, 11,                                     // 62500:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 62502:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62504:      check Inst[21] == 0x1
  OPC_Decode, 212, 64, 200, 3,               // 62508:      decode to UMINv4i16 using decoder 456
                                             // 62508:     }
  4, 11,                                     // 62513:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 62515:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62517:      check Inst[21] == 0x1
  OPC_Decode, 166, 62, 196, 3,               // 62521:      decode to UABDLv4i16_v4i32 using decoder 452
                                             // 62521:     }
  5, 11,                                     // 62526:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 62528:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62530:      check Inst[21] == 0x1
  OPC_Decode, 176, 62, 200, 3,               // 62534:      decode to UABDv4i16 using decoder 456
                                             // 62534:     }
  6, 21,                                     // 62539:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 62541:      switch Inst[21:16] {
  32, 7,                                     // 62544:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62546:       check predicate 76
  OPC_Decode, 215, 52, 201, 3,               // 62548:       decode to SQNEGv4i16 using decoder 457
                                             // 62548:      }
  33, 0,                                     // 62553:      case 0x21: {
  OPC_CheckPredicate, 84,                    // 62555:       check predicate 84
  OPC_Decode, 174, 21, 220, 3,               // 62557:       decode to F2CVTL using decoder 476
                                             // 62557:      }
                                             // 62557:      } // switch Inst[21:16]
                                             // 62557:     }
  7, 0,                                      // 62562:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 62564:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62566:      check Inst[21] == 0x1
  OPC_Decode, 154, 62, 221, 3,               // 62570:      decode to UABAv4i16 using decoder 477
                                             // 62570:     }
                                             // 62570:     } // switch Inst[12:10]
                                             // 62570:    }
  4, 121,                                    // 62575:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 62577:     switch Inst[12:10] {
  0, 11,                                     // 62580:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 62582:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62584:      check Inst[21] == 0x1
  OPC_Decode, 254, 64, 217, 3,               // 62588:      decode to UMLALv4i16_v4i32 using decoder 473
                                             // 62588:     }
  1, 21,                                     // 62593:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 62595:      switch Inst[21] {
  0, 7,                                      // 62598:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 62600:       check predicate 90
  OPC_Decode, 238, 52, 221, 3,               // 62602:       decode to SQRDMLAHv4i16 using decoder 477
                                             // 62602:      }
  1, 0,                                      // 62607:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62609:       check predicate 76
  OPC_Decode, 196, 60, 200, 3,               // 62611:       decode to SUBv4i16 using decoder 456
                                             // 62611:      }
                                             // 62611:      } // switch Inst[21]
                                             // 62611:     }
  2, 21,                                     // 62616:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 62618:      switch Inst[21:16] {
  32, 7,                                     // 62621:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62623:       check predicate 76
  OPC_Decode, 232, 17, 201, 3,               // 62625:       decode to CMGEv4i16rz using decoder 457
                                             // 62625:      }
  57, 0,                                     // 62630:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 62632:       check predicate 81
  OPC_Decode, 142, 31, 201, 3,               // 62634:       decode to FRINTAv4f16 using decoder 457
                                             // 62634:      }
                                             // 62634:      } // switch Inst[21:16]
                                             // 62634:     }
  3, 21,                                     // 62639:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 62641:      switch Inst[21] {
  0, 7,                                      // 62644:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 62646:       check predicate 90
  OPC_Decode, 129, 53, 221, 3,               // 62648:       decode to SQRDMLSHv4i16 using decoder 477
                                             // 62648:      }
  1, 0,                                      // 62653:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62655:       check predicate 76
  OPC_Decode, 215, 17, 200, 3,               // 62657:       decode to CMEQv4i16 using decoder 456
                                             // 62657:      }
                                             // 62657:      } // switch Inst[21]
                                             // 62657:     }
  5, 11,                                     // 62662:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 62664:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62666:      check Inst[21] == 0x1
  OPC_Decode, 139, 42, 221, 3,               // 62670:      decode to MLSv4i16 using decoder 477
                                             // 62670:     }
  6, 0,                                      // 62675:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 62677:      switch Inst[21:16] {
  32, 7,                                     // 62680:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62682:       check predicate 76
  OPC_Decode, 153, 18, 201, 3,               // 62684:       decode to CMLEv4i16rz using decoder 457
                                             // 62684:      }
  57, 0,                                     // 62689:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 62691:       check predicate 81
  OPC_Decode, 218, 31, 201, 3,               // 62693:       decode to FRINTXv4f16 using decoder 457
                                             // 62693:      }
                                             // 62693:      } // switch Inst[21:16]
                                             // 62693:     }
                                             // 62693:     } // switch Inst[12:10]
                                             // 62693:    }
  5, 110,                                    // 62698:    case 0x5: {
  OPC_SwitchField, 10, 3,                    // 62700:     switch Inst[12:10] {
  0, 11,                                     // 62703:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 62705:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62707:      check Inst[21] == 0x1
  OPC_Decode, 170, 65, 217, 3,               // 62711:      decode to UMLSLv4i16_v4i32 using decoder 473
                                             // 62711:     }
  1, 11,                                     // 62716:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 62718:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62720:      check Inst[21] == 0x1
  OPC_Decode, 236, 63, 200, 3,               // 62724:      decode to UMAXPv4i16 using decoder 456
                                             // 62724:     }
  2, 30,                                     // 62729:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 62731:      switch Inst[21:16] {
  48, 7,                                     // 62734:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 62736:       check predicate 76
  OPC_Decode, 249, 63, 212, 3,               // 62738:       decode to UMAXVv4i16v using decoder 468
                                             // 62738:      }
  49, 7,                                     // 62743:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 62745:       check predicate 76
  OPC_Decode, 178, 64, 212, 3,               // 62747:       decode to UMINVv4i16v using decoder 468
                                             // 62747:      }
  57, 0,                                     // 62752:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 62754:       check predicate 81
  OPC_Decode, 189, 24, 201, 3,               // 62756:       decode to FCVTNUv4f16 using decoder 457
                                             // 62756:      }
                                             // 62756:      } // switch Inst[21:16]
                                             // 62756:     }
  3, 11,                                     // 62761:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 62763:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62765:      check Inst[21] == 0x1
  OPC_Decode, 165, 64, 200, 3,               // 62769:      decode to UMINPv4i16 using decoder 456
                                             // 62769:     }
  5, 11,                                     // 62774:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 62776:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62778:      check Inst[21] == 0x1
  OPC_Decode, 148, 53, 200, 3,               // 62782:      decode to SQRDMULHv4i16 using decoder 456
                                             // 62782:     }
  6, 0,                                      // 62787:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 62789:      switch Inst[21:16] {
  32, 7,                                     // 62792:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 62794:       check predicate 76
  OPC_Decode, 174, 43, 201, 3,               // 62796:       decode to NEGv4i16 using decoder 457
                                             // 62796:      }
  57, 0,                                     // 62801:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 62803:       check predicate 81
  OPC_Decode, 147, 24, 201, 3,               // 62805:       decode to FCVTMUv4f16 using decoder 457
                                             // 62805:      }
                                             // 62805:      } // switch Inst[21:16]
                                             // 62805:     }
                                             // 62805:     } // switch Inst[12:10]
                                             // 62805:    }
  6, 60,                                     // 62810:    case 0x6: {
  OPC_SwitchField, 10, 1,                    // 62812:     switch Inst[10] {
  0, 42,                                     // 62815:     case 0x0: {
  OPC_SwitchField, 11, 2,                    // 62817:      switch Inst[12:11] {
  0, 11,                                     // 62820:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 62822:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62824:       check Inst[21] == 0x1
  OPC_Decode, 240, 65, 196, 3,               // 62828:       decode to UMULLv4i16_v4i32 using decoder 452
                                             // 62828:      }
  1, 11,                                     // 62833:      case 0x1: {
  OPC_CheckPredicate, 81,                    // 62835:       check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 62837:       check Inst[21:16] == 0x39
  OPC_Decode, 226, 23, 201, 3,               // 62841:       decode to FCVTAUv4f16 using decoder 457
                                             // 62841:      }
  3, 0,                                      // 62846:      case 0x3: {
  OPC_CheckPredicate, 81,                    // 62848:       check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 62850:       check Inst[21:16] == 0x39
  OPC_Decode, 162, 63, 201, 3,               // 62854:       decode to UCVTFv4f16 using decoder 457
                                             // 62854:      }
                                             // 62854:      } // switch Inst[12:11]
                                             // 62854:     }
  1, 0,                                      // 62859:     case 0x1: {
  OPC_CheckPredicate, 91,                    // 62861:      check predicate 91
  OPC_CheckField, 21, 1, 0,                  // 62863:      check Inst[21] == 0x0
  OPC_Decode, 138, 23, 131, 4,               // 62867:      decode to FCMLAv4f16 using decoder 515
                                             // 62867:     }
                                             // 62867:     } // switch Inst[10]
                                             // 62867:    }
  7, 0,                                      // 62872:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 62874:     switch Inst[11:10] {
  1, 11,                                     // 62877:     case 0x1: {
  OPC_CheckPredicate, 91,                    // 62879:      check predicate 91
  OPC_CheckField, 21, 1, 0,                  // 62881:      check Inst[21] == 0x0
  OPC_Decode, 175, 22, 132, 4,               // 62885:      decode to FCADDv4f16 using decoder 516
                                             // 62885:     }
  3, 0,                                      // 62890:     case 0x3: {
  OPC_CheckPredicate, 92,                    // 62892:      check predicate 92
  OPC_CheckField, 21, 1, 0,                  // 62894:      check Inst[21] == 0x0
  OPC_CheckField, 12, 1, 1,                  // 62898:      check Inst[12] == 0x1
  OPC_Decode, 147, 15, 221, 3,               // 62902:      decode to BFDOTv4bf16 using decoder 477
                                             // 62902:     }
                                             // 62902:     } // switch Inst[11:10]
                                             // 62902:    }
                                             // 62902:    } // switch Inst[15:13]
                                             // 62902:   }
  2, 191, 9,                                 // 62907:   case 0x2: {
  OPC_SwitchField, 10, 3,                    // 62910:    switch Inst[12:10] {
  0, 119,                                    // 62913:    case 0x0: {
  OPC_SwitchField, 13, 1,                    // 62915:     switch Inst[13] {
  0, 55,                                     // 62918:     case 0x0: {
  OPC_SwitchField, 14, 2,                    // 62920:      switch Inst[15:14] {
  0, 11,                                     // 62923:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 62925:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62927:       check Inst[21] == 0x1
  OPC_Decode, 209, 46, 225, 3,               // 62931:       decode to SADDLv8i16_v4i32 using decoder 481
                                             // 62931:      }
  1, 11,                                     // 62936:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 62938:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62940:       check Inst[21] == 0x1
  OPC_Decode, 146, 13, 223, 3,               // 62944:       decode to ADDHNv4i32_v8i16 using decoder 479
                                             // 62944:      }
  2, 11,                                     // 62949:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 62951:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62953:       check Inst[21] == 0x1
  OPC_Decode, 249, 49, 223, 3,               // 62957:       decode to SMLALv8i16_v4i32 using decoder 479
                                             // 62957:      }
  3, 0,                                      // 62962:      case 0x3: {
  OPC_CheckPredicate, 76,                    // 62964:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 62966:       check Inst[21] == 0x1
  OPC_Decode, 237, 50, 225, 3,               // 62970:       decode to SMULLv8i16_v4i32 using decoder 481
                                             // 62970:      }
                                             // 62970:      } // switch Inst[15:14]
                                             // 62970:     }
  1, 0,                                      // 62975:     case 0x1: {
  OPC_SwitchField, 15, 1,                    // 62977:      switch Inst[15] {
  0, 35,                                     // 62980:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 62982:       switch Inst[21] {
  0, 7,                                      // 62985:       case 0x0: {
  OPC_CheckPredicate, 93,                    // 62987:        check predicate 93
  OPC_Decode, 189, 41, 133, 4,               // 62989:        decode to LUT4_B using decoder 517
                                             // 62989:       }
  1, 0,                                      // 62994:       case 0x1: {
  OPC_SwitchField, 14, 1,                    // 62996:        switch Inst[14] {
  0, 7,                                      // 62999:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 63001:         check predicate 76
  OPC_Decode, 146, 56, 225, 3,               // 63003:         decode to SSUBLv8i16_v4i32 using decoder 481
                                             // 63003:        }
  1, 0,                                      // 63008:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 63010:         check predicate 76
  OPC_Decode, 134, 60, 223, 3,               // 63012:         decode to SUBHNv4i32_v8i16 using decoder 479
                                             // 63012:        }
                                             // 63012:        } // switch Inst[14]
                                             // 63012:       }
                                             // 63012:       } // switch Inst[21]
                                             // 63012:      }
  1, 0,                                      // 63017:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63019:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63021:       check Inst[21] == 0x1
  OPC_CheckField, 14, 1, 0,                  // 63025:       check Inst[14] == 0x0
  OPC_Decode, 165, 50, 223, 3,               // 63029:       decode to SMLSLv8i16_v4i32 using decoder 479
                                             // 63029:      }
                                             // 63029:      } // switch Inst[15]
                                             // 63029:     }
                                             // 63029:     } // switch Inst[13]
                                             // 63029:    }
  1, 137, 1,                                 // 63034:    case 0x1: {
  OPC_SwitchField, 13, 3,                    // 63037:     switch Inst[15:13] {
  0, 21,                                     // 63040:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 63042:      switch Inst[21] {
  0, 7,                                      // 63045:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 63047:       check predicate 81
  OPC_Decode, 230, 26, 225, 3,               // 63049:       decode to FMAXNMv8f16 using decoder 481
                                             // 63049:      }
  1, 0,                                      // 63054:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63056:       check predicate 76
  OPC_Decode, 153, 48, 225, 3,               // 63058:       decode to SHADDv8i16 using decoder 481
                                             // 63058:      }
                                             // 63058:      } // switch Inst[21]
                                             // 63058:     }
  1, 21,                                     // 63063:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 63065:      switch Inst[21] {
  0, 7,                                      // 63068:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 63070:       check predicate 81
  OPC_Decode, 213, 22, 225, 3,               // 63072:       decode to FCMEQv8f16 using decoder 481
                                             // 63072:      }
  1, 0,                                      // 63077:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63079:       check predicate 76
  OPC_Decode, 193, 48, 225, 3,               // 63081:       decode to SHSUBv8i16 using decoder 481
                                             // 63081:      }
                                             // 63081:      } // switch Inst[21]
                                             // 63081:     }
  2, 11,                                     // 63086:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 63088:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63090:      check Inst[21] == 0x1
  OPC_Decode, 197, 55, 225, 3,               // 63094:      decode to SSHLv8i16 using decoder 481
                                             // 63094:     }
  3, 11,                                     // 63099:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 63101:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63103:      check Inst[21] == 0x1
  OPC_Decode, 147, 49, 225, 3,               // 63107:      decode to SMAXv8i16 using decoder 481
                                             // 63107:     }
  4, 11,                                     // 63112:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 63114:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63116:      check Inst[21] == 0x1
  OPC_Decode, 239, 13, 225, 3,               // 63120:      decode to ADDv8i16 using decoder 481
                                             // 63120:     }
  5, 11,                                     // 63125:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 63127:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63129:      check Inst[21] == 0x1
  OPC_Decode, 228, 48, 225, 3,               // 63133:      decode to SMAXPv8i16 using decoder 481
                                             // 63133:     }
  6, 21,                                     // 63138:     case 0x6: {
  OPC_SwitchField, 21, 1,                    // 63140:      switch Inst[21] {
  0, 7,                                      // 63143:      case 0x0: {
  OPC_CheckPredicate, 80,                    // 63145:       check predicate 80
  OPC_Decode, 136, 28, 223, 3,               // 63147:       decode to FMLALLTTv4f32 using decoder 479
                                             // 63147:      }
  1, 0,                                      // 63152:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63154:       check predicate 76
  OPC_Decode, 227, 26, 225, 3,               // 63156:       decode to FMAXNMv2f64 using decoder 481
                                             // 63156:      }
                                             // 63156:      } // switch Inst[21]
                                             // 63156:     }
  7, 0,                                      // 63161:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 63163:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63165:      check Inst[21] == 0x1
  OPC_Decode, 206, 22, 225, 3,               // 63169:      decode to FCMEQv2f64 using decoder 481
                                             // 63169:     }
                                             // 63169:     } // switch Inst[15:13]
                                             // 63169:    }
  2, 231, 1,                                 // 63174:    case 0x2: {
  OPC_SwitchField, 13, 3,                    // 63177:     switch Inst[15:13] {
  0, 11,                                     // 63180:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 63182:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 63184:      check Inst[21:16] == 0x20
  OPC_Decode, 203, 45, 230, 3,               // 63188:      decode to REV64v8i16 using decoder 486
                                             // 63188:     }
  1, 35,                                     // 63193:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 63195:      switch Inst[21] {
  0, 7,                                      // 63198:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 63200:       check predicate 76
  OPC_Decode, 242, 61, 225, 3,               // 63202:       decode to TRN1v8i16 using decoder 481
                                             // 63202:      }
  1, 0,                                      // 63207:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 63209:       switch Inst[20:16] {
  0, 7,                                      // 63212:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 63214:        check predicate 76
  OPC_Decode, 195, 46, 230, 3,               // 63216:        decode to SADDLPv8i16_v4i32 using decoder 486
                                             // 63216:       }
  1, 0,                                      // 63221:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 63223:        check predicate 76
  OPC_Decode, 241, 70, 238, 3,               // 63225:        decode to XTNv8i16 using decoder 494
                                             // 63225:       }
                                             // 63225:       } // switch Inst[20:16]
                                             // 63225:      }
                                             // 63225:      } // switch Inst[21]
                                             // 63225:     }
  2, 21,                                     // 63230:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 63232:      switch Inst[21:16] {
  32, 7,                                     // 63235:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 63237:       check predicate 76
  OPC_Decode, 189, 17, 230, 3,               // 63239:       decode to CLSv8i16 using decoder 486
                                             // 63239:      }
  33, 0,                                     // 63244:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63246:       check predicate 76
  OPC_Decode, 212, 54, 238, 3,               // 63248:       decode to SQXTNv8i16 using decoder 494
                                             // 63248:      }
                                             // 63248:      } // switch Inst[21:16]
                                             // 63248:     }
  3, 35,                                     // 63253:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 63255:      switch Inst[21] {
  0, 7,                                      // 63258:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 63260:       check predicate 76
  OPC_Decode, 130, 62, 225, 3,               // 63262:       decode to TRN2v8i16 using decoder 481
                                             // 63262:      }
  1, 0,                                      // 63267:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 63269:       switch Inst[20:16] {
  0, 7,                                      // 63272:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 63274:        check predicate 76
  OPC_Decode, 183, 46, 238, 3,               // 63276:        decode to SADALPv8i16_v4i32 using decoder 494
                                             // 63276:       }
  1, 0,                                      // 63281:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 63283:        check predicate 76
  OPC_Decode, 201, 24, 238, 3,               // 63285:        decode to FCVTNv4i32 using decoder 494
                                             // 63285:       }
                                             // 63285:       } // switch Inst[20:16]
                                             // 63285:      }
                                             // 63285:      } // switch Inst[21]
                                             // 63285:     }
  4, 30,                                     // 63290:     case 0x4: {
  OPC_SwitchField, 16, 6,                    // 63292:      switch Inst[21:16] {
  32, 7,                                     // 63295:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 63297:       check predicate 76
  OPC_Decode, 252, 17, 230, 3,               // 63299:       decode to CMGTv8i16rz using decoder 486
                                             // 63299:      }
  33, 7,                                     // 63304:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63306:       check predicate 76
  OPC_Decode, 187, 31, 230, 3,               // 63308:       decode to FRINTNv2f64 using decoder 486
                                             // 63308:      }
  57, 0,                                     // 63313:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 63315:       check predicate 81
  OPC_Decode, 190, 31, 230, 3,               // 63317:       decode to FRINTNv8f16 using decoder 486
                                             // 63317:      }
                                             // 63317:      } // switch Inst[21:16]
                                             // 63317:     }
  5, 48,                                     // 63322:     case 0x5: {
  OPC_SwitchField, 16, 6,                    // 63324:      switch Inst[21:16] {
  32, 7,                                     // 63327:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 63329:       check predicate 76
  OPC_Decode, 163, 18, 230, 3,               // 63331:       decode to CMLTv8i16rz using decoder 486
                                             // 63331:      }
  33, 7,                                     // 63336:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63338:       check predicate 76
  OPC_Decode, 165, 24, 230, 3,               // 63340:       decode to FCVTNSv2f64 using decoder 486
                                             // 63340:      }
  48, 7,                                     // 63345:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 63347:       check predicate 76
  OPC_Decode, 241, 48, 244, 3,               // 63349:       decode to SMAXVv8i16v using decoder 500
                                             // 63349:      }
  49, 7,                                     // 63354:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 63356:       check predicate 76
  OPC_Decode, 171, 49, 244, 3,               // 63358:       decode to SMINVv8i16v using decoder 500
                                             // 63358:      }
  57, 0,                                     // 63363:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 63365:       check predicate 81
  OPC_Decode, 168, 24, 230, 3,               // 63367:       decode to FCVTNSv8f16 using decoder 486
                                             // 63367:      }
                                             // 63367:      } // switch Inst[21:16]
                                             // 63367:     }
  6, 21,                                     // 63372:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 63374:      switch Inst[21:16] {
  33, 7,                                     // 63377:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63379:       check predicate 76
  OPC_Decode, 207, 23, 230, 3,               // 63381:       decode to FCVTASv2f64 using decoder 486
                                             // 63381:      }
  57, 0,                                     // 63386:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 63388:       check predicate 81
  OPC_Decode, 210, 23, 230, 3,               // 63390:       decode to FCVTASv8f16 using decoder 486
                                             // 63390:      }
                                             // 63390:      } // switch Inst[21:16]
                                             // 63390:     }
  7, 0,                                      // 63395:     case 0x7: {
  OPC_CheckPredicate, 82,                    // 63397:      check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 63399:      check Inst[21:16] == 0x21
  OPC_Decode, 237, 30, 230, 3,               // 63403:      decode to FRINT32Zv2f64 using decoder 486
                                             // 63403:     }
                                             // 63403:     } // switch Inst[15:13]
                                             // 63403:    }
  3, 117,                                    // 63408:    case 0x3: {
  OPC_SwitchField, 13, 3,                    // 63410:     switch Inst[15:13] {
  0, 21,                                     // 63413:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 63415:      switch Inst[21] {
  0, 7,                                      // 63418:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 63420:       check predicate 81
  OPC_Decode, 206, 28, 223, 3,               // 63422:       decode to FMLAv8f16 using decoder 479
                                             // 63422:      }
  1, 0,                                      // 63427:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63429:       check predicate 76
  OPC_Decode, 159, 51, 225, 3,               // 63431:       decode to SQADDv8i16 using decoder 481
                                             // 63431:      }
                                             // 63431:      } // switch Inst[21]
                                             // 63431:     }
  1, 11,                                     // 63436:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 63438:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63440:      check Inst[21] == 0x1
  OPC_Decode, 197, 54, 225, 3,               // 63444:      decode to SQSUBv8i16 using decoder 481
                                             // 63444:     }
  2, 11,                                     // 63449:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 63451:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63453:      check Inst[21] == 0x1
  OPC_Decode, 134, 54, 225, 3,               // 63457:      decode to SQSHLv8i16 using decoder 481
                                             // 63457:     }
  3, 11,                                     // 63462:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 63464:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63466:      check Inst[21] == 0x1
  OPC_Decode, 205, 49, 225, 3,               // 63470:      decode to SMINv8i16 using decoder 481
                                             // 63470:     }
  4, 11,                                     // 63475:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 63477:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63479:      check Inst[21] == 0x1
  OPC_Decode, 137, 19, 225, 3,               // 63483:      decode to CMTSTv8i16 using decoder 481
                                             // 63483:     }
  5, 11,                                     // 63488:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 63490:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63492:      check Inst[21] == 0x1
  OPC_Decode, 158, 49, 225, 3,               // 63496:      decode to SMINPv8i16 using decoder 481
                                             // 63496:     }
  6, 11,                                     // 63501:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 63503:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63505:      check Inst[21] == 0x1
  OPC_Decode, 199, 28, 223, 3,               // 63509:      decode to FMLAv2f64 using decoder 479
                                             // 63509:     }
  7, 0,                                      // 63514:     case 0x7: {
  OPC_CheckPredicate, 94,                    // 63516:      check predicate 94
  OPC_CheckField, 21, 1, 0,                  // 63518:      check Inst[21] == 0x0
  OPC_Decode, 145, 29, 223, 3,               // 63522:      decode to FMMLAv8f16_v4f32 using decoder 479
                                             // 63522:     }
                                             // 63522:     } // switch Inst[15:13]
                                             // 63522:    }
  4, 102,                                    // 63527:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 63529:     switch Inst[15] {
  0, 53,                                     // 63532:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 63534:      switch Inst[21] {
  0, 7,                                      // 63537:      case 0x0: {
  OPC_CheckPredicate, 93,                    // 63539:       check predicate 93
  OPC_Decode, 190, 41, 134, 4,               // 63541:       decode to LUT4_H using decoder 518
                                             // 63541:      }
  1, 0,                                      // 63546:      case 0x1: {
  OPC_SwitchField, 13, 2,                    // 63548:       switch Inst[14:13] {
  0, 7,                                      // 63551:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 63553:        check predicate 76
  OPC_Decode, 224, 46, 225, 3,               // 63555:        decode to SADDWv8i16_v4i32 using decoder 481
                                             // 63555:       }
  1, 7,                                      // 63560:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 63562:        check predicate 76
  OPC_Decode, 158, 56, 225, 3,               // 63564:        decode to SSUBWv8i16_v4i32 using decoder 481
                                             // 63564:       }
  2, 7,                                      // 63569:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 63571:        check predicate 76
  OPC_Decode, 142, 46, 223, 3,               // 63573:        decode to SABALv8i16_v4i32 using decoder 479
                                             // 63573:       }
  3, 0,                                      // 63578:       case 0x3: {
  OPC_CheckPredicate, 76,                    // 63580:        check predicate 76
  OPC_Decode, 164, 46, 225, 3,               // 63582:        decode to SABDLv8i16_v4i32 using decoder 481
                                             // 63582:       }
                                             // 63582:       } // switch Inst[14:13]
                                             // 63582:      }
                                             // 63582:      } // switch Inst[21]
                                             // 63582:     }
  1, 0,                                      // 63587:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 63589:      switch Inst[14:13] {
  0, 11,                                     // 63592:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 63594:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63596:       check Inst[21] == 0x1
  OPC_Decode, 223, 51, 223, 3,               // 63600:       decode to SQDMLALv8i16_v4i32 using decoder 479
                                             // 63600:      }
  1, 11,                                     // 63605:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63607:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63609:       check Inst[21] == 0x1
  OPC_Decode, 248, 51, 223, 3,               // 63613:       decode to SQDMLSLv8i16_v4i32 using decoder 479
                                             // 63613:      }
  2, 0,                                      // 63618:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 63620:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63622:       check Inst[21] == 0x1
  OPC_Decode, 177, 52, 225, 3,               // 63626:       decode to SQDMULLv8i16_v4i32 using decoder 481
                                             // 63626:      }
                                             // 63626:      } // switch Inst[14:13]
                                             // 63626:     }
                                             // 63626:     } // switch Inst[15]
                                             // 63626:    }
  5, 137, 1,                                 // 63631:    case 0x5: {
  OPC_SwitchField, 13, 3,                    // 63634:     switch Inst[15:13] {
  0, 21,                                     // 63637:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 63639:      switch Inst[21] {
  0, 7,                                      // 63642:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 63644:       check predicate 81
  OPC_Decode, 141, 22, 225, 3,               // 63646:       decode to FADDv8f16 using decoder 481
                                             // 63646:      }
  1, 0,                                      // 63651:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63653:       check predicate 76
  OPC_Decode, 237, 54, 225, 3,               // 63655:       decode to SRHADDv8i16 using decoder 481
                                             // 63655:      }
                                             // 63655:      } // switch Inst[21]
                                             // 63655:     }
  1, 21,                                     // 63660:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 63662:      switch Inst[21] {
  0, 7,                                      // 63665:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 63667:       check predicate 81
  OPC_Decode, 146, 27, 225, 3,               // 63669:       decode to FMAXv8f16 using decoder 481
                                             // 63669:      }
  1, 0,                                      // 63674:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63676:       check predicate 76
  OPC_Decode, 251, 17, 225, 3,               // 63678:       decode to CMGTv8i16 using decoder 481
                                             // 63678:      }
                                             // 63678:      } // switch Inst[21]
                                             // 63678:     }
  2, 11,                                     // 63683:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 63685:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63687:      check Inst[21] == 0x1
  OPC_Decode, 153, 55, 225, 3,               // 63691:      decode to SRSHLv8i16 using decoder 481
                                             // 63691:     }
  3, 11,                                     // 63696:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 63698:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63700:      check Inst[21] == 0x1
  OPC_Decode, 174, 46, 225, 3,               // 63704:      decode to SABDv8i16 using decoder 481
                                             // 63704:     }
  4, 11,                                     // 63709:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 63711:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63713:      check Inst[21] == 0x1
  OPC_Decode, 254, 41, 223, 3,               // 63717:      decode to MLAv8i16 using decoder 479
                                             // 63717:     }
  5, 11,                                     // 63722:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 63724:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63726:      check Inst[21] == 0x1
  OPC_Decode, 154, 52, 225, 3,               // 63730:      decode to SQDMULHv8i16 using decoder 481
                                             // 63730:     }
  6, 11,                                     // 63735:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 63737:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 63739:      check Inst[21] == 0x1
  OPC_Decode, 138, 22, 225, 3,               // 63743:      decode to FADDv2f64 using decoder 481
                                             // 63743:     }
  7, 0,                                      // 63748:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 63750:      switch Inst[21] {
  0, 7,                                      // 63753:      case 0x0: {
  OPC_CheckPredicate, 84,                    // 63755:       check predicate 84
  OPC_Decode, 192, 24, 225, 3,               // 63757:       decode to FCVTN_F16v16f8 using decoder 481
                                             // 63757:      }
  1, 0,                                      // 63762:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 63764:       check predicate 76
  OPC_Decode, 143, 27, 225, 3,               // 63766:       decode to FMAXv2f64 using decoder 481
                                             // 63766:      }
                                             // 63766:      } // switch Inst[21]
                                             // 63766:     }
                                             // 63766:     } // switch Inst[15:13]
                                             // 63766:    }
  6, 212, 1,                                 // 63771:    case 0x6: {
  OPC_SwitchField, 13, 3,                    // 63774:     switch Inst[15:13] {
  0, 11,                                     // 63777:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 63779:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 63781:      check Inst[21] == 0x0
  OPC_Decode, 181, 69, 225, 3,               // 63785:      decode to UZP1v8i16 using decoder 481
                                             // 63785:     }
  1, 35,                                     // 63790:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 63792:      switch Inst[21] {
  0, 7,                                      // 63795:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 63797:       check predicate 76
  OPC_Decode, 139, 71, 225, 3,               // 63799:       decode to ZIP1v8i16 using decoder 481
                                             // 63799:      }
  1, 0,                                      // 63804:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 63806:       switch Inst[20:16] {
  0, 7,                                      // 63809:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 63811:        check predicate 76
  OPC_Decode, 129, 61, 238, 3,               // 63813:        decode to SUQADDv8i16 using decoder 494
                                             // 63813:       }
  16, 0,                                     // 63818:       case 0x10: {
  OPC_CheckPredicate, 76,                    // 63820:        check predicate 76
  OPC_Decode, 203, 46, 252, 3,               // 63822:        decode to SADDLVv8i16v using decoder 508
                                             // 63822:       }
                                             // 63822:       } // switch Inst[20:16]
                                             // 63822:      }
                                             // 63822:      } // switch Inst[21]
                                             // 63822:     }
  2, 11,                                     // 63827:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 63829:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 63831:      check Inst[21] == 0x0
  OPC_Decode, 197, 69, 225, 3,               // 63835:      decode to UZP2v8i16 using decoder 481
                                             // 63835:     }
  3, 35,                                     // 63840:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 63842:      switch Inst[21] {
  0, 7,                                      // 63845:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 63847:       check predicate 76
  OPC_Decode, 155, 71, 225, 3,               // 63849:       decode to ZIP2v8i16 using decoder 481
                                             // 63849:      }
  1, 0,                                      // 63854:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 63856:       switch Inst[20:16] {
  0, 7,                                      // 63859:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 63861:        check predicate 76
  OPC_Decode, 136, 51, 230, 3,               // 63863:        decode to SQABSv8i16 using decoder 486
                                             // 63863:       }
  1, 0,                                      // 63868:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 63870:        check predicate 76
  OPC_Decode, 240, 23, 230, 3,               // 63872:        decode to FCVTLv4i32 using decoder 486
                                             // 63872:       }
                                             // 63872:       } // switch Inst[20:16]
                                             // 63872:      }
                                             // 63872:      } // switch Inst[21]
                                             // 63872:     }
  4, 30,                                     // 63877:     case 0x4: {
  OPC_SwitchField, 16, 6,                    // 63879:      switch Inst[21:16] {
  32, 7,                                     // 63882:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 63884:       check predicate 76
  OPC_Decode, 220, 17, 230, 3,               // 63886:       decode to CMEQv8i16rz using decoder 486
                                             // 63886:      }
  33, 7,                                     // 63891:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63893:       check predicate 76
  OPC_Decode, 171, 31, 230, 3,               // 63895:       decode to FRINTMv2f64 using decoder 486
                                             // 63895:      }
  57, 0,                                     // 63900:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 63902:       check predicate 81
  OPC_Decode, 174, 31, 230, 3,               // 63904:       decode to FRINTMv8f16 using decoder 486
                                             // 63904:      }
                                             // 63904:      } // switch Inst[21:16]
                                             // 63904:     }
  5, 39,                                     // 63909:     case 0x5: {
  OPC_SwitchField, 16, 6,                    // 63911:      switch Inst[21:16] {
  32, 7,                                     // 63914:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 63916:       check predicate 76
  OPC_Decode, 252, 12, 230, 3,               // 63918:       decode to ABSv8i16 using decoder 486
                                             // 63918:      }
  33, 7,                                     // 63923:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63925:       check predicate 76
  OPC_Decode, 128, 24, 230, 3,               // 63927:       decode to FCVTMSv2f64 using decoder 486
                                             // 63927:      }
  49, 7,                                     // 63932:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 63934:       check predicate 76
  OPC_Decode, 190, 13, 244, 3,               // 63936:       decode to ADDVv8i16v using decoder 500
                                             // 63936:      }
  57, 0,                                     // 63941:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 63943:       check predicate 81
  OPC_Decode, 131, 24, 230, 3,               // 63945:       decode to FCVTMSv8f16 using decoder 486
                                             // 63945:      }
                                             // 63945:      } // switch Inst[21:16]
                                             // 63945:     }
  6, 21,                                     // 63950:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 63952:      switch Inst[21:16] {
  33, 7,                                     // 63955:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 63957:       check predicate 76
  OPC_Decode, 166, 47, 230, 3,               // 63959:       decode to SCVTFv2f64 using decoder 486
                                             // 63959:      }
  57, 0,                                     // 63964:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 63966:       check predicate 81
  OPC_Decode, 173, 47, 230, 3,               // 63968:       decode to SCVTFv8f16 using decoder 486
                                             // 63968:      }
                                             // 63968:      } // switch Inst[21:16]
                                             // 63968:     }
  7, 0,                                      // 63973:     case 0x7: {
  OPC_CheckPredicate, 82,                    // 63975:      check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 63977:      check Inst[21:16] == 0x21
  OPC_Decode, 255, 30, 230, 3,               // 63981:      decode to FRINT64Zv2f64 using decoder 486
                                             // 63981:     }
                                             // 63981:     } // switch Inst[15:13]
                                             // 63981:    }
  7, 0,                                      // 63986:    case 0x7: {
  OPC_SwitchField, 13, 3,                    // 63988:     switch Inst[15:13] {
  0, 21,                                     // 63991:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 63993:      switch Inst[21] {
  0, 7,                                      // 63996:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 63998:       check predicate 81
  OPC_Decode, 244, 29, 225, 3,               // 64000:       decode to FMULXv8f16 using decoder 481
                                             // 64000:      }
  1, 0,                                      // 64005:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64007:       check predicate 76
  OPC_Decode, 144, 16, 225, 3,               // 64009:       decode to BICv16i8 using decoder 481
                                             // 64009:      }
                                             // 64009:      } // switch Inst[21]
                                             // 64009:     }
  1, 21,                                     // 64014:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 64016:      switch Inst[21] {
  0, 7,                                      // 64019:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64021:       check predicate 81
  OPC_Decode, 211, 30, 225, 3,               // 64023:       decode to FRECPSv8f16 using decoder 481
                                             // 64023:      }
  1, 0,                                      // 64028:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64030:       check predicate 76
  OPC_Decode, 235, 17, 225, 3,               // 64032:       decode to CMGEv8i16 using decoder 481
                                             // 64032:      }
                                             // 64032:      } // switch Inst[21]
                                             // 64032:     }
  2, 11,                                     // 64037:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 64039:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64041:      check Inst[21] == 0x1
  OPC_Decode, 171, 53, 225, 3,               // 64045:      decode to SQRSHLv8i16 using decoder 481
                                             // 64045:     }
  3, 11,                                     // 64050:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 64052:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64054:      check Inst[21] == 0x1
  OPC_Decode, 152, 46, 223, 3,               // 64058:      decode to SABAv8i16 using decoder 479
                                             // 64058:     }
  4, 11,                                     // 64063:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 64065:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64067:      check Inst[21] == 0x1
  OPC_Decode, 150, 43, 225, 3,               // 64071:      decode to MULv8i16 using decoder 481
                                             // 64071:     }
  5, 11,                                     // 64076:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 64078:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64080:      check Inst[21] == 0x1
  OPC_Decode, 161, 13, 225, 3,               // 64084:      decode to ADDPv8i16 using decoder 481
                                             // 64084:     }
  6, 11,                                     // 64089:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 64091:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64093:      check Inst[21] == 0x1
  OPC_Decode, 237, 29, 225, 3,               // 64097:      decode to FMULXv2f64 using decoder 481
                                             // 64097:     }
  7, 0,                                      // 64102:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 64104:      switch Inst[21] {
  0, 7,                                      // 64107:      case 0x0: {
  OPC_CheckPredicate, 89,                    // 64109:       check predicate 89
  OPC_Decode, 158, 26, 223, 3,               // 64111:       decode to FDOTv8f16 using decoder 479
                                             // 64111:      }
  1, 0,                                      // 64116:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64118:       check predicate 76
  OPC_Decode, 208, 30, 225, 3,               // 64120:       decode to FRECPSv2f64 using decoder 481
                                             // 64120:      }
                                             // 64120:      } // switch Inst[21]
                                             // 64120:     }
                                             // 64120:     } // switch Inst[15:13]
                                             // 64120:    }
                                             // 64120:    } // switch Inst[12:10]
                                             // 64120:   }
  3, 172, 8,                                 // 64125:   case 0x3: {
  OPC_SwitchField, 13, 3,                    // 64128:    switch Inst[15:13] {
  0, 124,                                    // 64131:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 64133:     switch Inst[12:10] {
  0, 11,                                     // 64136:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 64138:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64140:      check Inst[21] == 0x1
  OPC_Decode, 210, 62, 225, 3,               // 64144:      decode to UADDLv8i16_v4i32 using decoder 481
                                             // 64144:     }
  1, 21,                                     // 64149:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 64151:      switch Inst[21] {
  0, 7,                                      // 64154:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64156:       check predicate 81
  OPC_Decode, 197, 26, 225, 3,               // 64158:       decode to FMAXNMPv8f16 using decoder 481
                                             // 64158:      }
  1, 0,                                      // 64163:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64165:       check predicate 76
  OPC_Decode, 213, 63, 225, 3,               // 64167:       decode to UHADDv8i16 using decoder 481
                                             // 64167:      }
                                             // 64167:      } // switch Inst[21]
                                             // 64167:     }
  2, 11,                                     // 64172:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 64174:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 64176:      check Inst[21:16] == 0x20
  OPC_Decode, 197, 45, 230, 3,               // 64180:      decode to REV32v8i16 using decoder 486
                                             // 64180:     }
  3, 11,                                     // 64185:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 64187:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64189:      check Inst[21] == 0x1
  OPC_Decode, 139, 66, 225, 3,               // 64193:      decode to UQADDv8i16 using decoder 481
                                             // 64193:     }
  4, 11,                                     // 64198:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 64200:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64202:      check Inst[21] == 0x1
  OPC_Decode, 226, 62, 225, 3,               // 64206:      decode to UADDWv8i16_v4i32 using decoder 481
                                             // 64206:     }
  5, 21,                                     // 64211:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 64213:      switch Inst[21] {
  0, 7,                                      // 64216:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64218:       check predicate 81
  OPC_Decode, 242, 21, 225, 3,               // 64220:       decode to FADDPv8f16 using decoder 481
                                             // 64220:      }
  1, 0,                                      // 64225:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64227:       check predicate 76
  OPC_Decode, 209, 67, 225, 3,               // 64229:       decode to URHADDv8i16 using decoder 481
                                             // 64229:      }
                                             // 64229:      } // switch Inst[21]
                                             // 64229:     }
  7, 0,                                      // 64234:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 64236:      switch Inst[21] {
  0, 7,                                      // 64239:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64241:       check predicate 81
  OPC_Decode, 153, 30, 225, 3,               // 64243:       decode to FMULv8f16 using decoder 481
                                             // 64243:      }
  1, 0,                                      // 64248:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64250:       check predicate 76
  OPC_Decode, 185, 16, 223, 3,               // 64252:       decode to BSLv16i8 using decoder 479
                                             // 64252:      }
                                             // 64252:      } // switch Inst[21]
                                             // 64252:     }
                                             // 64252:     } // switch Inst[12:10]
                                             // 64252:    }
  1, 176, 1,                                 // 64257:    case 0x1: {
  OPC_SwitchField, 10, 3,                    // 64260:     switch Inst[12:10] {
  0, 11,                                     // 64263:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 64265:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64267:      check Inst[21] == 0x1
  OPC_Decode, 251, 68, 225, 3,               // 64271:      decode to USUBLv8i16_v4i32 using decoder 481
                                             // 64271:     }
  1, 21,                                     // 64276:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 64278:      switch Inst[21] {
  0, 7,                                      // 64281:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64283:       check predicate 81
  OPC_Decode, 235, 22, 225, 3,               // 64285:       decode to FCMGEv8f16 using decoder 481
                                             // 64285:      }
  1, 0,                                      // 64290:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64292:       check predicate 76
  OPC_Decode, 227, 63, 225, 3,               // 64294:       decode to UHSUBv8i16 using decoder 481
                                             // 64294:      }
                                             // 64294:      } // switch Inst[21]
                                             // 64294:     }
  2, 21,                                     // 64299:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 64301:      switch Inst[21:16] {
  32, 7,                                     // 64304:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64306:       check predicate 76
  OPC_Decode, 196, 62, 230, 3,               // 64308:       decode to UADDLPv8i16_v4i32 using decoder 486
                                             // 64308:      }
  33, 0,                                     // 64313:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64315:       check predicate 76
  OPC_Decode, 227, 54, 238, 3,               // 64317:       decode to SQXTUNv8i16 using decoder 494
                                             // 64317:      }
                                             // 64317:      } // switch Inst[21:16]
                                             // 64317:     }
  3, 21,                                     // 64322:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 64324:      switch Inst[21] {
  0, 7,                                      // 64327:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64329:       check predicate 81
  OPC_Decode, 215, 21, 225, 3,               // 64331:       decode to FACGEv8f16 using decoder 481
                                             // 64331:      }
  1, 0,                                      // 64336:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64338:       check predicate 76
  OPC_Decode, 180, 67, 225, 3,               // 64340:       decode to UQSUBv8i16 using decoder 481
                                             // 64340:      }
                                             // 64340:      } // switch Inst[21]
                                             // 64340:     }
  4, 11,                                     // 64345:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 64347:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64349:      check Inst[21] == 0x1
  OPC_Decode, 135, 69, 225, 3,               // 64353:      decode to USUBWv8i16_v4i32 using decoder 481
                                             // 64353:     }
  5, 21,                                     // 64358:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 64360:      switch Inst[21] {
  0, 7,                                      // 64363:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64365:       check predicate 81
  OPC_Decode, 241, 26, 225, 3,               // 64367:       decode to FMAXPv8f16 using decoder 481
                                             // 64367:      }
  1, 0,                                      // 64372:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64374:       check predicate 76
  OPC_Decode, 133, 18, 225, 3,               // 64376:       decode to CMHIv8i16 using decoder 481
                                             // 64376:      }
                                             // 64376:      } // switch Inst[21]
                                             // 64376:     }
  6, 30,                                     // 64381:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 64383:      switch Inst[21:16] {
  32, 7,                                     // 64386:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64388:       check predicate 76
  OPC_Decode, 226, 68, 238, 3,               // 64390:       decode to USQADDv8i16 using decoder 494
                                             // 64390:      }
  33, 7,                                     // 64395:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64397:       check predicate 76
  OPC_Decode, 159, 48, 230, 3,               // 64399:       decode to SHLLv8i16 using decoder 486
                                             // 64399:      }
  48, 0,                                     // 64404:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 64406:       check predicate 76
  OPC_Decode, 204, 62, 252, 3,               // 64408:       decode to UADDLVv8i16v using decoder 508
                                             // 64408:      }
                                             // 64408:      } // switch Inst[21:16]
                                             // 64408:     }
  7, 0,                                      // 64413:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 64415:      switch Inst[21] {
  0, 7,                                      // 64418:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 64420:       check predicate 81
  OPC_Decode, 251, 25, 225, 3,               // 64422:       decode to FDIVv8f16 using decoder 481
                                             // 64422:      }
  1, 0,                                      // 64427:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64429:       check predicate 76
  OPC_Decode, 141, 18, 225, 3,               // 64431:       decode to CMHSv8i16 using decoder 481
                                             // 64431:      }
                                             // 64431:      } // switch Inst[21]
                                             // 64431:     }
                                             // 64431:     } // switch Inst[12:10]
                                             // 64431:    }
  2, 117,                                    // 64436:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 64438:     switch Inst[12:10] {
  0, 11,                                     // 64441:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 64443:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64445:      check Inst[21] == 0x1
  OPC_Decode, 225, 44, 223, 3,               // 64449:      decode to RADDHNv4i32_v8i16 using decoder 479
                                             // 64449:     }
  1, 11,                                     // 64454:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 64456:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64458:      check Inst[21] == 0x1
  OPC_Decode, 173, 68, 225, 3,               // 64462:      decode to USHLv8i16 using decoder 481
                                             // 64462:     }
  2, 21,                                     // 64467:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 64469:      switch Inst[21:16] {
  32, 7,                                     // 64472:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64474:       check predicate 76
  OPC_Decode, 205, 17, 230, 3,               // 64476:       decode to CLZv8i16 using decoder 486
                                             // 64476:      }
  33, 0,                                     // 64481:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64483:       check predicate 76
  OPC_Decode, 195, 67, 238, 3,               // 64485:       decode to UQXTNv8i16 using decoder 494
                                             // 64485:      }
                                             // 64485:      } // switch Inst[21:16]
                                             // 64485:     }
  3, 11,                                     // 64490:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 64492:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64494:      check Inst[21] == 0x1
  OPC_Decode, 134, 67, 225, 3,               // 64498:      decode to UQSHLv8i16 using decoder 481
                                             // 64498:     }
  4, 11,                                     // 64503:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 64505:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64507:      check Inst[21] == 0x1
  OPC_Decode, 146, 62, 223, 3,               // 64511:      decode to UABALv8i16_v4i32 using decoder 479
                                             // 64511:     }
  5, 11,                                     // 64516:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 64518:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64520:      check Inst[21] == 0x1
  OPC_Decode, 241, 67, 225, 3,               // 64524:      decode to URSHLv8i16 using decoder 481
                                             // 64524:     }
  6, 11,                                     // 64529:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 64531:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 64533:      check Inst[21:16] == 0x20
  OPC_Decode, 240, 44, 230, 3,               // 64537:      decode to RBITv16i8 using decoder 486
                                             // 64537:     }
  7, 0,                                      // 64542:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 64544:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64546:      check Inst[21] == 0x1
  OPC_Decode, 208, 66, 225, 3,               // 64550:      decode to UQRSHLv8i16 using decoder 481
                                             // 64550:     }
                                             // 64550:     } // switch Inst[12:10]
                                             // 64550:    }
  3, 127,                                    // 64555:    case 0x3: {
  OPC_SwitchField, 10, 3,                    // 64557:     switch Inst[12:10] {
  0, 11,                                     // 64560:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 64562:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64564:      check Inst[21] == 0x1
  OPC_Decode, 254, 45, 223, 3,               // 64568:      decode to RSUBHNv4i32_v8i16 using decoder 479
                                             // 64568:     }
  1, 11,                                     // 64573:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 64575:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64577:      check Inst[21] == 0x1
  OPC_Decode, 157, 64, 225, 3,               // 64581:      decode to UMAXv8i16 using decoder 481
                                             // 64581:     }
  2, 21,                                     // 64586:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 64588:      switch Inst[21:16] {
  32, 7,                                     // 64591:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64593:       check predicate 76
  OPC_Decode, 187, 62, 238, 3,               // 64595:       decode to UADALPv8i16_v4i32 using decoder 494
                                             // 64595:      }
  33, 0,                                     // 64600:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64602:       check predicate 76
  OPC_Decode, 245, 24, 238, 3,               // 64604:       decode to FCVTXNv4f32 using decoder 494
                                             // 64604:      }
                                             // 64604:      } // switch Inst[21:16]
                                             // 64604:     }
  3, 11,                                     // 64609:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 64611:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64613:      check Inst[21] == 0x1
  OPC_Decode, 214, 64, 225, 3,               // 64617:      decode to UMINv8i16 using decoder 481
                                             // 64617:     }
  4, 11,                                     // 64622:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 64624:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64626:      check Inst[21] == 0x1
  OPC_Decode, 168, 62, 225, 3,               // 64630:      decode to UABDLv8i16_v4i32 using decoder 481
                                             // 64630:     }
  5, 11,                                     // 64635:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 64637:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64639:      check Inst[21] == 0x1
  OPC_Decode, 178, 62, 225, 3,               // 64643:      decode to UABDv8i16 using decoder 481
                                             // 64643:     }
  6, 21,                                     // 64648:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 64650:      switch Inst[21:16] {
  32, 7,                                     // 64653:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64655:       check predicate 76
  OPC_Decode, 217, 52, 230, 3,               // 64657:       decode to SQNEGv8i16 using decoder 486
                                             // 64657:      }
  33, 0,                                     // 64662:      case 0x21: {
  OPC_CheckPredicate, 84,                    // 64664:       check predicate 84
  OPC_Decode, 175, 21, 230, 3,               // 64666:       decode to F2CVTL2 using decoder 486
                                             // 64666:      }
                                             // 64666:      } // switch Inst[21:16]
                                             // 64666:     }
  7, 0,                                      // 64671:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 64673:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64675:      check Inst[21] == 0x1
  OPC_Decode, 156, 62, 223, 3,               // 64679:      decode to UABAv8i16 using decoder 479
                                             // 64679:     }
                                             // 64679:     } // switch Inst[12:10]
                                             // 64679:    }
  4, 139, 1,                                 // 64684:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 64687:     switch Inst[12:10] {
  0, 11,                                     // 64690:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 64692:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64694:      check Inst[21] == 0x1
  OPC_Decode, 130, 65, 223, 3,               // 64698:      decode to UMLALv8i16_v4i32 using decoder 479
                                             // 64698:     }
  1, 21,                                     // 64703:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 64705:      switch Inst[21] {
  0, 7,                                      // 64708:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 64710:       check predicate 90
  OPC_Decode, 242, 52, 223, 3,               // 64712:       decode to SQRDMLAHv8i16 using decoder 479
                                             // 64712:      }
  1, 0,                                      // 64717:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64719:       check predicate 76
  OPC_Decode, 198, 60, 225, 3,               // 64721:       decode to SUBv8i16 using decoder 481
                                             // 64721:      }
                                             // 64721:      } // switch Inst[21]
                                             // 64721:     }
  2, 30,                                     // 64726:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 64728:      switch Inst[21:16] {
  32, 7,                                     // 64731:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64733:       check predicate 76
  OPC_Decode, 236, 17, 230, 3,               // 64735:       decode to CMGEv8i16rz using decoder 486
                                             // 64735:      }
  33, 7,                                     // 64740:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64742:       check predicate 76
  OPC_Decode, 141, 31, 230, 3,               // 64744:       decode to FRINTAv2f64 using decoder 486
                                             // 64744:      }
  57, 0,                                     // 64749:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 64751:       check predicate 81
  OPC_Decode, 144, 31, 230, 3,               // 64753:       decode to FRINTAv8f16 using decoder 486
                                             // 64753:      }
                                             // 64753:      } // switch Inst[21:16]
                                             // 64753:     }
  3, 21,                                     // 64758:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 64760:      switch Inst[21] {
  0, 7,                                      // 64763:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 64765:       check predicate 90
  OPC_Decode, 133, 53, 223, 3,               // 64767:       decode to SQRDMLSHv8i16 using decoder 479
                                             // 64767:      }
  1, 0,                                      // 64772:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 64774:       check predicate 76
  OPC_Decode, 219, 17, 225, 3,               // 64776:       decode to CMEQv8i16 using decoder 481
                                             // 64776:      }
                                             // 64776:      } // switch Inst[21]
                                             // 64776:     }
  5, 11,                                     // 64781:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 64783:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64785:      check Inst[21] == 0x1
  OPC_Decode, 143, 42, 223, 3,               // 64789:      decode to MLSv8i16 using decoder 479
                                             // 64789:     }
  6, 0,                                      // 64794:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 64796:      switch Inst[21:16] {
  32, 7,                                     // 64799:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64801:       check predicate 76
  OPC_Decode, 155, 18, 230, 3,               // 64803:       decode to CMLEv8i16rz using decoder 486
                                             // 64803:      }
  33, 7,                                     // 64808:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64810:       check predicate 76
  OPC_Decode, 217, 31, 230, 3,               // 64812:       decode to FRINTXv2f64 using decoder 486
                                             // 64812:      }
  57, 0,                                     // 64817:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 64819:       check predicate 81
  OPC_Decode, 220, 31, 230, 3,               // 64821:       decode to FRINTXv8f16 using decoder 486
                                             // 64821:      }
                                             // 64821:      } // switch Inst[21:16]
                                             // 64821:     }
                                             // 64821:     } // switch Inst[12:10]
                                             // 64821:    }
  5, 128, 1,                                 // 64826:    case 0x5: {
  OPC_SwitchField, 10, 3,                    // 64829:     switch Inst[12:10] {
  0, 11,                                     // 64832:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 64834:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64836:      check Inst[21] == 0x1
  OPC_Decode, 174, 65, 223, 3,               // 64840:      decode to UMLSLv8i16_v4i32 using decoder 479
                                             // 64840:     }
  1, 11,                                     // 64845:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 64847:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64849:      check Inst[21] == 0x1
  OPC_Decode, 238, 63, 225, 3,               // 64853:      decode to UMAXPv8i16 using decoder 481
                                             // 64853:     }
  2, 39,                                     // 64858:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 64860:      switch Inst[21:16] {
  33, 7,                                     // 64863:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64865:       check predicate 76
  OPC_Decode, 188, 24, 230, 3,               // 64867:       decode to FCVTNUv2f64 using decoder 486
                                             // 64867:      }
  48, 7,                                     // 64872:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 64874:       check predicate 76
  OPC_Decode, 251, 63, 244, 3,               // 64876:       decode to UMAXVv8i16v using decoder 500
                                             // 64876:      }
  49, 7,                                     // 64881:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 64883:       check predicate 76
  OPC_Decode, 180, 64, 244, 3,               // 64885:       decode to UMINVv8i16v using decoder 500
                                             // 64885:      }
  57, 0,                                     // 64890:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 64892:       check predicate 81
  OPC_Decode, 191, 24, 230, 3,               // 64894:       decode to FCVTNUv8f16 using decoder 486
                                             // 64894:      }
                                             // 64894:      } // switch Inst[21:16]
                                             // 64894:     }
  3, 11,                                     // 64899:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 64901:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64903:      check Inst[21] == 0x1
  OPC_Decode, 167, 64, 225, 3,               // 64907:      decode to UMINPv8i16 using decoder 481
                                             // 64907:     }
  5, 11,                                     // 64912:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 64914:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64916:      check Inst[21] == 0x1
  OPC_Decode, 152, 53, 225, 3,               // 64920:      decode to SQRDMULHv8i16 using decoder 481
                                             // 64920:     }
  6, 0,                                      // 64925:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 64927:      switch Inst[21:16] {
  32, 7,                                     // 64930:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 64932:       check predicate 76
  OPC_Decode, 176, 43, 230, 3,               // 64934:       decode to NEGv8i16 using decoder 486
                                             // 64934:      }
  33, 7,                                     // 64939:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 64941:       check predicate 76
  OPC_Decode, 146, 24, 230, 3,               // 64943:       decode to FCVTMUv2f64 using decoder 486
                                             // 64943:      }
  57, 0,                                     // 64948:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 64950:       check predicate 81
  OPC_Decode, 149, 24, 230, 3,               // 64952:       decode to FCVTMUv8f16 using decoder 486
                                             // 64952:      }
                                             // 64952:      } // switch Inst[21:16]
                                             // 64952:     }
                                             // 64952:     } // switch Inst[12:10]
                                             // 64952:    }
  6, 113,                                    // 64957:    case 0x6: {
  OPC_SwitchField, 10, 1,                    // 64959:     switch Inst[10] {
  0, 62,                                     // 64962:     case 0x0: {
  OPC_SwitchField, 11, 2,                    // 64964:      switch Inst[12:11] {
  0, 11,                                     // 64967:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 64969:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 64971:       check Inst[21] == 0x1
  OPC_Decode, 244, 65, 225, 3,               // 64975:       decode to UMULLv8i16_v4i32 using decoder 481
                                             // 64975:      }
  1, 21,                                     // 64980:      case 0x1: {
  OPC_SwitchField, 16, 6,                    // 64982:       switch Inst[21:16] {
  33, 7,                                     // 64985:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 64987:        check predicate 76
  OPC_Decode, 225, 23, 230, 3,               // 64989:        decode to FCVTAUv2f64 using decoder 486
                                             // 64989:       }
  57, 0,                                     // 64994:       case 0x39: {
  OPC_CheckPredicate, 81,                    // 64996:        check predicate 81
  OPC_Decode, 228, 23, 230, 3,               // 64998:        decode to FCVTAUv8f16 using decoder 486
                                             // 64998:       }
                                             // 64998:       } // switch Inst[21:16]
                                             // 64998:      }
  3, 0,                                      // 65003:      case 0x3: {
  OPC_SwitchField, 16, 6,                    // 65005:       switch Inst[21:16] {
  33, 7,                                     // 65008:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 65010:        check predicate 76
  OPC_Decode, 159, 63, 230, 3,               // 65012:        decode to UCVTFv2f64 using decoder 486
                                             // 65012:       }
  57, 0,                                     // 65017:       case 0x39: {
  OPC_CheckPredicate, 81,                    // 65019:        check predicate 81
  OPC_Decode, 166, 63, 230, 3,               // 65021:        decode to UCVTFv8f16 using decoder 486
                                             // 65021:       }
                                             // 65021:       } // switch Inst[21:16]
                                             // 65021:      }
                                             // 65021:      } // switch Inst[12:11]
                                             // 65021:     }
  1, 0,                                      // 65026:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 65028:      switch Inst[21] {
  0, 7,                                      // 65031:      case 0x0: {
  OPC_CheckPredicate, 91,                    // 65033:       check predicate 91
  OPC_Decode, 142, 23, 135, 4,               // 65035:       decode to FCMLAv8f16 using decoder 519
                                             // 65035:      }
  1, 0,                                      // 65040:      case 0x1: {
  OPC_SwitchField, 11, 2,                    // 65042:       switch Inst[12:11] {
  0, 7,                                      // 65045:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 65047:        check predicate 76
  OPC_Decode, 191, 26, 225, 3,               // 65049:        decode to FMAXNMPv2f64 using decoder 481
                                             // 65049:       }
  2, 7,                                      // 65054:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 65056:        check predicate 76
  OPC_Decode, 236, 21, 225, 3,               // 65058:        decode to FADDPv2f64 using decoder 481
                                             // 65058:       }
  3, 0,                                      // 65063:       case 0x3: {
  OPC_CheckPredicate, 76,                    // 65065:        check predicate 76
  OPC_Decode, 146, 30, 225, 3,               // 65067:        decode to FMULv2f64 using decoder 481
                                             // 65067:       }
                                             // 65067:       } // switch Inst[12:11]
                                             // 65067:      }
                                             // 65067:      } // switch Inst[21]
                                             // 65067:     }
                                             // 65067:     } // switch Inst[10]
                                             // 65067:    }
  7, 0,                                      // 65072:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 65074:     switch Inst[11:10] {
  1, 35,                                     // 65077:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 65079:      switch Inst[21] {
  0, 7,                                      // 65082:      case 0x0: {
  OPC_CheckPredicate, 91,                    // 65084:       check predicate 91
  OPC_Decode, 177, 22, 136, 4,               // 65086:       decode to FCADDv8f16 using decoder 520
                                             // 65086:      }
  1, 0,                                      // 65091:      case 0x1: {
  OPC_SwitchField, 12, 1,                    // 65093:       switch Inst[12] {
  0, 7,                                      // 65096:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 65098:        check predicate 76
  OPC_Decode, 228, 22, 225, 3,               // 65100:        decode to FCMGEv2f64 using decoder 481
                                             // 65100:       }
  1, 0,                                      // 65105:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 65107:        check predicate 76
  OPC_Decode, 235, 26, 225, 3,               // 65109:        decode to FMAXPv2f64 using decoder 481
                                             // 65109:       }
                                             // 65109:       } // switch Inst[12]
                                             // 65109:      }
                                             // 65109:      } // switch Inst[21]
                                             // 65109:     }
  2, 29,                                     // 65114:     case 0x2: {
  OPC_SwitchField, 12, 1,                    // 65116:      switch Inst[12] {
  0, 11,                                     // 65119:      case 0x0: {
  OPC_CheckPredicate, 82,                    // 65121:       check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 65123:       check Inst[21:16] == 0x21
  OPC_Decode, 228, 30, 230, 3,               // 65127:       decode to FRINT32Xv2f64 using decoder 486
                                             // 65127:      }
  1, 0,                                      // 65132:      case 0x1: {
  OPC_CheckPredicate, 82,                    // 65134:       check predicate 82
  OPC_CheckField, 16, 6, 33,                 // 65136:       check Inst[21:16] == 0x21
  OPC_Decode, 246, 30, 230, 3,               // 65140:       decode to FRINT64Xv2f64 using decoder 486
                                             // 65140:      }
                                             // 65140:      } // switch Inst[12]
                                             // 65140:     }
  3, 0,                                      // 65145:     case 0x3: {
  OPC_SwitchField, 12, 1,                    // 65147:      switch Inst[12] {
  0, 21,                                     // 65150:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 65152:       switch Inst[21] {
  0, 7,                                      // 65155:       case 0x0: {
  OPC_CheckPredicate, 92,                    // 65157:        check predicate 92
  OPC_Decode, 213, 15, 223, 3,               // 65159:        decode to BFMMLA using decoder 479
                                             // 65159:       }
  1, 0,                                      // 65164:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 65166:        check predicate 76
  OPC_Decode, 212, 21, 225, 3,               // 65168:        decode to FACGEv2f64 using decoder 481
                                             // 65168:       }
                                             // 65168:       } // switch Inst[21]
                                             // 65168:      }
  1, 0,                                      // 65173:      case 0x1: {
  OPC_SwitchField, 21, 1,                    // 65175:       switch Inst[21] {
  0, 7,                                      // 65178:       case 0x0: {
  OPC_CheckPredicate, 92,                    // 65180:        check predicate 92
  OPC_Decode, 148, 15, 223, 3,               // 65182:        decode to BFDOTv8bf16 using decoder 479
                                             // 65182:       }
  1, 0,                                      // 65187:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 65189:        check predicate 76
  OPC_Decode, 248, 25, 225, 3,               // 65191:        decode to FDIVv2f64 using decoder 481
                                             // 65191:       }
                                             // 65191:       } // switch Inst[21]
                                             // 65191:      }
                                             // 65191:      } // switch Inst[12]
                                             // 65191:     }
                                             // 65191:     } // switch Inst[11:10]
                                             // 65191:    }
                                             // 65191:    } // switch Inst[15:13]
                                             // 65191:   }
  6, 0,                                      // 65196:   case 0x6: {
  OPC_SwitchField, 15, 1,                    // 65198:    switch Inst[15] {
  0, 11,                                     // 65201:    case 0x0: {
  OPC_CheckPredicate, 95,                    // 65203:     check predicate 95
  OPC_CheckField, 21, 1, 0,                  // 65205:     check Inst[21] == 0x0
  OPC_Decode, 210, 48, 129, 4,               // 65209:     decode to SM3SS1 using decoder 513
                                             // 65209:    }
  1, 0,                                      // 65214:    case 0x1: {
  OPC_SwitchField, 10, 2,                    // 65216:     switch Inst[11:10] {
  0, 47,                                     // 65219:     case 0x0: {
  OPC_SwitchField, 14, 1,                    // 65221:      switch Inst[14] {
  0, 25,                                     // 65224:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 65226:       switch Inst[21] {
  0, 7,                                      // 65229:       case 0x0: {
  OPC_CheckPredicate, 95,                    // 65231:        check predicate 95
  OPC_Decode, 211, 48, 137, 4,               // 65233:        decode to SM3TT1A using decoder 521
                                             // 65233:       }
  1, 0,                                      // 65238:       case 0x1: {
  OPC_CheckPredicate, 88,                    // 65240:        check predicate 88
  OPC_CheckField, 12, 2, 0,                  // 65242:        check Inst[13:12] == 0x0
  OPC_Decode, 141, 48, 223, 3,               // 65246:        decode to SHA512H using decoder 479
                                             // 65246:       }
                                             // 65246:       } // switch Inst[21]
                                             // 65246:      }
  1, 0,                                      // 65251:      case 0x1: {
  OPC_CheckPredicate, 95,                    // 65253:       check predicate 95
  OPC_CheckField, 21, 1, 1,                  // 65255:       check Inst[21] == 0x1
  OPC_CheckField, 12, 2, 0,                  // 65259:       check Inst[13:12] == 0x0
  OPC_Decode, 208, 48, 223, 3,               // 65263:       decode to SM3PARTW1 using decoder 479
                                             // 65263:      }
                                             // 65263:      } // switch Inst[14]
                                             // 65263:     }
  1, 47,                                     // 65268:     case 0x1: {
  OPC_SwitchField, 14, 1,                    // 65270:      switch Inst[14] {
  0, 25,                                     // 65273:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 65275:       switch Inst[21] {
  0, 7,                                      // 65278:       case 0x0: {
  OPC_CheckPredicate, 95,                    // 65280:        check predicate 95
  OPC_Decode, 212, 48, 137, 4,               // 65282:        decode to SM3TT1B using decoder 521
                                             // 65282:       }
  1, 0,                                      // 65287:       case 0x1: {
  OPC_CheckPredicate, 88,                    // 65289:        check predicate 88
  OPC_CheckField, 12, 2, 0,                  // 65291:        check Inst[13:12] == 0x0
  OPC_Decode, 142, 48, 223, 3,               // 65295:        decode to SHA512H2 using decoder 479
                                             // 65295:       }
                                             // 65295:       } // switch Inst[21]
                                             // 65295:      }
  1, 0,                                      // 65300:      case 0x1: {
  OPC_CheckPredicate, 95,                    // 65302:       check predicate 95
  OPC_CheckField, 21, 1, 1,                  // 65304:       check Inst[21] == 0x1
  OPC_CheckField, 12, 2, 0,                  // 65308:       check Inst[13:12] == 0x0
  OPC_Decode, 209, 48, 223, 3,               // 65312:       decode to SM3PARTW2 using decoder 479
                                             // 65312:      }
                                             // 65312:      } // switch Inst[14]
                                             // 65312:     }
  2, 47,                                     // 65317:     case 0x2: {
  OPC_SwitchField, 14, 1,                    // 65319:      switch Inst[14] {
  0, 25,                                     // 65322:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 65324:       switch Inst[21] {
  0, 7,                                      // 65327:       case 0x0: {
  OPC_CheckPredicate, 95,                    // 65329:        check predicate 95
  OPC_Decode, 213, 48, 137, 4,               // 65331:        decode to SM3TT2A using decoder 521
                                             // 65331:       }
  1, 0,                                      // 65336:       case 0x1: {
  OPC_CheckPredicate, 88,                    // 65338:        check predicate 88
  OPC_CheckField, 12, 2, 0,                  // 65340:        check Inst[13:12] == 0x0
  OPC_Decode, 144, 48, 223, 3,               // 65344:        decode to SHA512SU1 using decoder 479
                                             // 65344:       }
                                             // 65344:       } // switch Inst[21]
                                             // 65344:      }
  1, 0,                                      // 65349:      case 0x1: {
  OPC_CheckPredicate, 95,                    // 65351:       check predicate 95
  OPC_CheckField, 21, 1, 1,                  // 65353:       check Inst[21] == 0x1
  OPC_CheckField, 12, 2, 0,                  // 65357:       check Inst[13:12] == 0x0
  OPC_Decode, 217, 48, 225, 3,               // 65361:       decode to SM4ENCKEY using decoder 481
                                             // 65361:      }
                                             // 65361:      } // switch Inst[14]
                                             // 65361:     }
  3, 0,                                      // 65366:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 65368:      switch Inst[21] {
  0, 11,                                     // 65371:      case 0x0: {
  OPC_CheckPredicate, 95,                    // 65373:       check predicate 95
  OPC_CheckField, 14, 1, 0,                  // 65375:       check Inst[14] == 0x0
  OPC_Decode, 214, 48, 137, 4,               // 65379:       decode to SM3TT2B using decoder 521
                                             // 65379:      }
  1, 0,                                      // 65384:      case 0x1: {
  OPC_CheckPredicate, 88,                    // 65386:       check predicate 88
  OPC_CheckField, 12, 3, 0,                  // 65388:       check Inst[14:12] == 0x0
  OPC_Decode, 228, 44, 225, 3,               // 65392:       decode to RAX1 using decoder 481
                                             // 65392:      }
                                             // 65392:      } // switch Inst[21]
                                             // 65392:     }
                                             // 65392:     } // switch Inst[11:10]
                                             // 65392:    }
                                             // 65392:    } // switch Inst[15]
                                             // 65392:   }
                                             // 65392:   } // switch Inst[31:29]
                                             // 65392:  }
  10, 245, 30,                               // 65397:  case 0xa: {
  OPC_SwitchField, 29, 3,                    // 65400:   switch Inst[31:29] {
  0, 223, 7,                                 // 65403:   case 0x0: {
  OPC_SwitchField, 10, 6,                    // 65406:    switch Inst[15:10] {
  0, 11,                                     // 65409:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 65411:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65413:     check Inst[21] == 0x1
  OPC_Decode, 206, 46, 196, 3,               // 65417:     decode to SADDLv2i32_v2i64 using decoder 452
                                             // 65417:    }
  1, 11,                                     // 65422:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 65424:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65426:     check Inst[21] == 0x1
  OPC_Decode, 150, 48, 200, 3,               // 65430:     decode to SHADDv2i32 using decoder 456
                                             // 65430:    }
  2, 11,                                     // 65435:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 65437:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 65439:     check Inst[21:16] == 0x20
  OPC_Decode, 200, 45, 201, 3,               // 65443:     decode to REV64v2i32 using decoder 457
                                             // 65443:    }
  3, 11,                                     // 65448:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 65450:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65452:     check Inst[21] == 0x1
  OPC_Decode, 155, 51, 200, 3,               // 65456:     decode to SQADDv2i32 using decoder 456
                                             // 65456:    }
  4, 11,                                     // 65461:    case 0x4: {
  OPC_CheckPredicate, 76,                    // 65463:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65465:     check Inst[21] == 0x1
  OPC_Decode, 221, 46, 204, 3,               // 65469:     decode to SADDWv2i32_v2i64 using decoder 460
                                             // 65469:    }
  5, 11,                                     // 65474:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 65476:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65478:     check Inst[21] == 0x1
  OPC_Decode, 234, 54, 200, 3,               // 65482:     decode to SRHADDv2i32 using decoder 456
                                             // 65482:    }
  6, 11,                                     // 65487:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 65489:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 65491:     check Inst[21] == 0x0
  OPC_Decode, 177, 69, 200, 3,               // 65495:     decode to UZP1v2i32 using decoder 456
                                             // 65495:    }
  7, 11,                                     // 65500:    case 0x7: {
  OPC_CheckPredicate, 76,                    // 65502:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65504:     check Inst[21] == 0x1
  OPC_Decode, 220, 43, 200, 3,               // 65508:     decode to ORRv8i8 using decoder 456
                                             // 65508:    }
  8, 11,                                     // 65513:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 65515:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65517:     check Inst[21] == 0x1
  OPC_Decode, 143, 56, 196, 3,               // 65521:     decode to SSUBLv2i32_v2i64 using decoder 452
                                             // 65521:    }
  9, 11,                                     // 65526:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 65528:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65530:     check Inst[21] == 0x1
  OPC_Decode, 190, 48, 200, 3,               // 65534:     decode to SHSUBv2i32 using decoder 456
                                             // 65534:    }
  10, 35,                                    // 65539:    case 0xa: {
  OPC_SwitchField, 21, 1,                    // 65541:     switch Inst[21] {
  0, 7,                                      // 65544:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 65546:      check predicate 76
  OPC_Decode, 238, 61, 200, 3,               // 65548:      decode to TRN1v2i32 using decoder 456
                                             // 65548:     }
  1, 0,                                      // 65553:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 65555:      switch Inst[20:16] {
  0, 7,                                      // 65558:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 65560:       check predicate 76
  OPC_Decode, 192, 46, 201, 3,               // 65562:       decode to SADDLPv2i32_v1i64 using decoder 457
                                             // 65562:      }
  1, 0,                                      // 65567:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 65569:       check predicate 76
  OPC_Decode, 238, 70, 206, 3,               // 65571:       decode to XTNv2i32 using decoder 462
                                             // 65571:      }
                                             // 65571:      } // switch Inst[20:16]
                                             // 65571:     }
                                             // 65571:     } // switch Inst[21]
                                             // 65571:    }
  11, 11,                                    // 65576:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 65578:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65580:     check Inst[21] == 0x1
  OPC_Decode, 193, 54, 200, 3,               // 65584:     decode to SQSUBv2i32 using decoder 456
                                             // 65584:    }
  12, 11,                                    // 65589:    case 0xc: {
  OPC_CheckPredicate, 76,                    // 65591:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65593:     check Inst[21] == 0x1
  OPC_Decode, 155, 56, 204, 3,               // 65597:     decode to SSUBWv2i32_v2i64 using decoder 460
                                             // 65597:    }
  13, 11,                                    // 65602:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 65604:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65606:     check Inst[21] == 0x1
  OPC_Decode, 243, 17, 200, 3,               // 65610:     decode to CMGTv2i32 using decoder 456
                                             // 65610:    }
  14, 25,                                    // 65615:    case 0xe: {
  OPC_SwitchField, 21, 1,                    // 65617:     switch Inst[21] {
  0, 7,                                      // 65620:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 65622:      check predicate 76
  OPC_Decode, 135, 71, 200, 3,               // 65624:      decode to ZIP1v2i32 using decoder 456
                                             // 65624:     }
  1, 0,                                      // 65629:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 65631:      check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 65633:      check Inst[20:16] == 0x0
  OPC_Decode, 253, 60, 211, 3,               // 65637:      decode to SUQADDv2i32 using decoder 467
                                             // 65637:     }
                                             // 65637:     } // switch Inst[21]
                                             // 65637:    }
  15, 11,                                    // 65642:    case 0xf: {
  OPC_CheckPredicate, 76,                    // 65644:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65646:     check Inst[21] == 0x1
  OPC_Decode, 227, 17, 200, 3,               // 65650:     decode to CMGEv2i32 using decoder 456
                                             // 65650:    }
  16, 11,                                    // 65655:    case 0x10: {
  OPC_CheckPredicate, 76,                    // 65657:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65659:     check Inst[21] == 0x1
  OPC_Decode, 143, 13, 215, 3,               // 65663:     decode to ADDHNv2i64_v2i32 using decoder 471
                                             // 65663:    }
  17, 11,                                    // 65668:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 65670:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65672:     check Inst[21] == 0x1
  OPC_Decode, 193, 55, 200, 3,               // 65676:     decode to SSHLv2i32 using decoder 456
                                             // 65676:    }
  18, 21,                                    // 65681:    case 0x12: {
  OPC_SwitchField, 16, 6,                    // 65683:     switch Inst[21:16] {
  32, 7,                                     // 65686:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 65688:      check predicate 76
  OPC_Decode, 186, 17, 201, 3,               // 65690:      decode to CLSv2i32 using decoder 457
                                             // 65690:     }
  33, 0,                                     // 65695:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 65697:      check predicate 76
  OPC_Decode, 209, 54, 206, 3,               // 65699:      decode to SQXTNv2i32 using decoder 462
                                             // 65699:     }
                                             // 65699:     } // switch Inst[21:16]
                                             // 65699:    }
  19, 11,                                    // 65704:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 65706:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65708:     check Inst[21] == 0x1
  OPC_Decode, 254, 53, 200, 3,               // 65712:     decode to SQSHLv2i32 using decoder 456
                                             // 65712:    }
  20, 11,                                    // 65717:    case 0x14: {
  OPC_CheckPredicate, 76,                    // 65719:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65721:     check Inst[21] == 0x1
  OPC_Decode, 139, 46, 217, 3,               // 65725:     decode to SABALv2i32_v2i64 using decoder 473
                                             // 65725:    }
  21, 11,                                    // 65730:    case 0x15: {
  OPC_CheckPredicate, 76,                    // 65732:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65734:     check Inst[21] == 0x1
  OPC_Decode, 149, 55, 200, 3,               // 65738:     decode to SRSHLv2i32 using decoder 456
                                             // 65738:    }
  22, 11,                                    // 65743:    case 0x16: {
  OPC_CheckPredicate, 76,                    // 65745:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 65747:     check Inst[21] == 0x0
  OPC_Decode, 193, 69, 200, 3,               // 65751:     decode to UZP2v2i32 using decoder 456
                                             // 65751:    }
  23, 11,                                    // 65756:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 65758:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65760:     check Inst[21] == 0x1
  OPC_Decode, 167, 53, 200, 3,               // 65764:     decode to SQRSHLv2i32 using decoder 456
                                             // 65764:    }
  24, 11,                                    // 65769:    case 0x18: {
  OPC_CheckPredicate, 76,                    // 65771:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65773:     check Inst[21] == 0x1
  OPC_Decode, 131, 60, 215, 3,               // 65777:     decode to SUBHNv2i64_v2i32 using decoder 471
                                             // 65777:    }
  25, 11,                                    // 65782:    case 0x19: {
  OPC_CheckPredicate, 76,                    // 65784:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65786:     check Inst[21] == 0x1
  OPC_Decode, 144, 49, 200, 3,               // 65790:     decode to SMAXv2i32 using decoder 456
                                             // 65790:    }
  26, 35,                                    // 65795:    case 0x1a: {
  OPC_SwitchField, 21, 1,                    // 65797:     switch Inst[21] {
  0, 7,                                      // 65800:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 65802:      check predicate 76
  OPC_Decode, 254, 61, 200, 3,               // 65804:      decode to TRN2v2i32 using decoder 456
                                             // 65804:     }
  1, 0,                                      // 65809:     case 0x1: {
  OPC_SwitchField, 16, 5,                    // 65811:      switch Inst[20:16] {
  0, 7,                                      // 65814:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 65816:       check predicate 76
  OPC_Decode, 180, 46, 211, 3,               // 65818:       decode to SADALPv2i32_v1i64 using decoder 467
                                             // 65818:      }
  1, 0,                                      // 65823:      case 0x1: {
  OPC_CheckPredicate, 92,                    // 65825:       check predicate 92
  OPC_Decode, 129, 15, 206, 3,               // 65827:       decode to BFCVTN using decoder 462
                                             // 65827:      }
                                             // 65827:      } // switch Inst[20:16]
                                             // 65827:     }
                                             // 65827:     } // switch Inst[21]
                                             // 65827:    }
  27, 11,                                    // 65832:    case 0x1b: {
  OPC_CheckPredicate, 76,                    // 65834:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65836:     check Inst[21] == 0x1
  OPC_Decode, 202, 49, 200, 3,               // 65840:     decode to SMINv2i32 using decoder 456
                                             // 65840:    }
  28, 11,                                    // 65845:    case 0x1c: {
  OPC_CheckPredicate, 76,                    // 65847:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65849:     check Inst[21] == 0x1
  OPC_Decode, 161, 46, 196, 3,               // 65853:     decode to SABDLv2i32_v2i64 using decoder 452
                                             // 65853:    }
  29, 11,                                    // 65858:    case 0x1d: {
  OPC_CheckPredicate, 76,                    // 65860:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65862:     check Inst[21] == 0x1
  OPC_Decode, 171, 46, 200, 3,               // 65866:     decode to SABDv2i32 using decoder 456
                                             // 65866:    }
  30, 25,                                    // 65871:    case 0x1e: {
  OPC_SwitchField, 21, 1,                    // 65873:     switch Inst[21] {
  0, 7,                                      // 65876:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 65878:      check predicate 76
  OPC_Decode, 151, 71, 200, 3,               // 65880:      decode to ZIP2v2i32 using decoder 456
                                             // 65880:     }
  1, 0,                                      // 65885:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 65887:      check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 65889:      check Inst[20:16] == 0x0
  OPC_Decode, 132, 51, 201, 3,               // 65893:      decode to SQABSv2i32 using decoder 457
                                             // 65893:     }
                                             // 65893:     } // switch Inst[21]
                                             // 65893:    }
  31, 11,                                    // 65898:    case 0x1f: {
  OPC_CheckPredicate, 76,                    // 65900:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65902:     check Inst[21] == 0x1
  OPC_Decode, 149, 46, 221, 3,               // 65906:     decode to SABAv2i32 using decoder 477
                                             // 65906:    }
  32, 11,                                    // 65911:    case 0x20: {
  OPC_CheckPredicate, 76,                    // 65913:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65915:     check Inst[21] == 0x1
  OPC_Decode, 243, 49, 217, 3,               // 65919:     decode to SMLALv2i32_v2i64 using decoder 473
                                             // 65919:    }
  33, 11,                                    // 65924:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 65926:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65928:     check Inst[21] == 0x1
  OPC_Decode, 235, 13, 200, 3,               // 65932:     decode to ADDv2i32 using decoder 456
                                             // 65932:    }
  34, 21,                                    // 65937:    case 0x22: {
  OPC_SwitchField, 16, 6,                    // 65939:     switch Inst[21:16] {
  32, 7,                                     // 65942:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 65944:      check predicate 76
  OPC_Decode, 244, 17, 201, 3,               // 65946:      decode to CMGTv2i32rz using decoder 457
                                             // 65946:     }
  33, 0,                                     // 65951:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 65953:      check predicate 76
  OPC_Decode, 202, 31, 201, 3,               // 65955:      decode to FRINTPv2f32 using decoder 457
                                             // 65955:     }
                                             // 65955:     } // switch Inst[21:16]
                                             // 65955:    }
  35, 11,                                    // 65960:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 65962:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65964:     check Inst[21] == 0x1
  OPC_Decode, 133, 19, 200, 3,               // 65968:     decode to CMTSTv2i32 using decoder 456
                                             // 65968:    }
  36, 11,                                    // 65973:    case 0x24: {
  OPC_CheckPredicate, 76,                    // 65975:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 65977:     check Inst[21] == 0x1
  OPC_Decode, 217, 51, 217, 3,               // 65981:     decode to SQDMLALv2i32_v2i64 using decoder 473
                                             // 65981:    }
  37, 21,                                    // 65986:    case 0x25: {
  OPC_SwitchField, 21, 1,                    // 65988:     switch Inst[21] {
  0, 7,                                      // 65991:     case 0x0: {
  OPC_CheckPredicate, 96,                    // 65993:      check predicate 96
  OPC_Decode, 210, 47, 221, 3,               // 65995:      decode to SDOTv8i8 using decoder 477
                                             // 65995:     }
  1, 0,                                      // 66000:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66002:      check predicate 76
  OPC_Decode, 248, 41, 221, 3,               // 66004:      decode to MLAv2i32 using decoder 477
                                             // 66004:     }
                                             // 66004:     } // switch Inst[21]
                                             // 66004:    }
  38, 21,                                    // 66009:    case 0x26: {
  OPC_SwitchField, 16, 6,                    // 66011:     switch Inst[21:16] {
  32, 7,                                     // 66014:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 66016:      check predicate 76
  OPC_Decode, 212, 17, 201, 3,               // 66018:      decode to CMEQv2i32rz using decoder 457
                                             // 66018:     }
  33, 0,                                     // 66023:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 66025:      check predicate 76
  OPC_Decode, 230, 31, 201, 3,               // 66027:      decode to FRINTZv2f32 using decoder 457
                                             // 66027:     }
                                             // 66027:     } // switch Inst[21:16]
                                             // 66027:    }
  39, 21,                                    // 66032:    case 0x27: {
  OPC_SwitchField, 21, 1,                    // 66034:     switch Inst[21] {
  0, 7,                                      // 66037:     case 0x0: {
  OPC_CheckPredicate, 97,                    // 66039:      check predicate 97
  OPC_Decode, 154, 68, 221, 3,               // 66041:      decode to USDOTv8i8 using decoder 477
                                             // 66041:     }
  1, 0,                                      // 66046:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66048:      check predicate 76
  OPC_Decode, 144, 43, 200, 3,               // 66050:      decode to MULv2i32 using decoder 456
                                             // 66050:     }
                                             // 66050:     } // switch Inst[21]
                                             // 66050:    }
  40, 11,                                    // 66055:    case 0x28: {
  OPC_CheckPredicate, 76,                    // 66057:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66059:     check Inst[21] == 0x1
  OPC_Decode, 159, 50, 217, 3,               // 66063:     decode to SMLSLv2i32_v2i64 using decoder 473
                                             // 66063:    }
  41, 11,                                    // 66068:    case 0x29: {
  OPC_CheckPredicate, 76,                    // 66070:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66072:     check Inst[21] == 0x1
  OPC_Decode, 225, 48, 200, 3,               // 66076:     decode to SMAXPv2i32 using decoder 456
                                             // 66076:    }
  42, 21,                                    // 66081:    case 0x2a: {
  OPC_SwitchField, 16, 6,                    // 66083:     switch Inst[21:16] {
  32, 7,                                     // 66086:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 66088:      check predicate 76
  OPC_Decode, 159, 18, 201, 3,               // 66090:      decode to CMLTv2i32rz using decoder 457
                                             // 66090:     }
  33, 0,                                     // 66095:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 66097:      check predicate 76
  OPC_Decode, 216, 24, 201, 3,               // 66099:      decode to FCVTPSv2f32 using decoder 457
                                             // 66099:     }
                                             // 66099:     } // switch Inst[21:16]
                                             // 66099:    }
  43, 11,                                    // 66104:    case 0x2b: {
  OPC_CheckPredicate, 76,                    // 66106:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66108:     check Inst[21] == 0x1
  OPC_Decode, 155, 49, 200, 3,               // 66112:     decode to SMINPv2i32 using decoder 456
                                             // 66112:    }
  44, 11,                                    // 66117:    case 0x2c: {
  OPC_CheckPredicate, 76,                    // 66119:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66121:     check Inst[21] == 0x1
  OPC_Decode, 242, 51, 217, 3,               // 66125:     decode to SQDMLSLv2i32_v2i64 using decoder 473
                                             // 66125:    }
  45, 11,                                    // 66130:    case 0x2d: {
  OPC_CheckPredicate, 76,                    // 66132:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66134:     check Inst[21] == 0x1
  OPC_Decode, 148, 52, 200, 3,               // 66138:     decode to SQDMULHv2i32 using decoder 456
                                             // 66138:    }
  46, 21,                                    // 66143:    case 0x2e: {
  OPC_SwitchField, 16, 6,                    // 66145:     switch Inst[21:16] {
  32, 7,                                     // 66148:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 66150:      check predicate 76
  OPC_Decode, 248, 12, 201, 3,               // 66152:      decode to ABSv2i32 using decoder 457
                                             // 66152:     }
  33, 0,                                     // 66157:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 66159:      check predicate 76
  OPC_Decode, 161, 25, 201, 3,               // 66161:      decode to FCVTZSv2f32 using decoder 457
                                             // 66161:     }
                                             // 66161:     } // switch Inst[21:16]
                                             // 66161:    }
  47, 11,                                    // 66166:    case 0x2f: {
  OPC_CheckPredicate, 76,                    // 66168:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66170:     check Inst[21] == 0x1
  OPC_Decode, 156, 13, 200, 3,               // 66174:     decode to ADDPv2i32 using decoder 456
                                             // 66174:    }
  48, 11,                                    // 66179:    case 0x30: {
  OPC_CheckPredicate, 76,                    // 66181:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66183:     check Inst[21] == 0x1
  OPC_Decode, 231, 50, 196, 3,               // 66187:     decode to SMULLv2i32_v2i64 using decoder 452
                                             // 66187:    }
  49, 11,                                    // 66192:    case 0x31: {
  OPC_CheckPredicate, 76,                    // 66194:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66196:     check Inst[21] == 0x1
  OPC_Decode, 190, 27, 200, 3,               // 66200:     decode to FMINNMv2f32 using decoder 456
                                             // 66200:    }
  50, 30,                                    // 66205:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 66207:     switch Inst[21:16] {
  32, 7,                                     // 66210:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 66212:      check predicate 76
  OPC_Decode, 251, 22, 201, 3,               // 66214:      decode to FCMGTv2i32rz using decoder 457
                                             // 66214:     }
  33, 7,                                     // 66219:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 66221:      check predicate 76
  OPC_Decode, 199, 67, 201, 3,               // 66223:      decode to URECPEv2i32 using decoder 457
                                             // 66223:     }
  48, 0,                                     // 66228:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 66230:      check predicate 81
  OPC_Decode, 169, 27, 212, 3,               // 66232:      decode to FMINNMVv4i16v using decoder 468
                                             // 66232:     }
                                             // 66232:     } // switch Inst[21:16]
                                             // 66232:    }
  51, 11,                                    // 66237:    case 0x33: {
  OPC_CheckPredicate, 76,                    // 66239:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66241:     check Inst[21] == 0x1
  OPC_Decode, 128, 29, 221, 3,               // 66245:     decode to FMLSv2f32 using decoder 477
                                             // 66245:    }
  52, 11,                                    // 66250:    case 0x34: {
  OPC_CheckPredicate, 76,                    // 66252:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66254:     check Inst[21] == 0x1
  OPC_Decode, 171, 52, 196, 3,               // 66258:     decode to SQDMULLv2i32_v2i64 using decoder 452
                                             // 66258:    }
  53, 11,                                    // 66263:    case 0x35: {
  OPC_CheckPredicate, 76,                    // 66265:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66267:     check Inst[21] == 0x1
  OPC_Decode, 187, 32, 200, 3,               // 66271:     decode to FSUBv2f32 using decoder 456
                                             // 66271:    }
  54, 21,                                    // 66276:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 66278:     switch Inst[21:16] {
  32, 7,                                     // 66281:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 66283:      check predicate 76
  OPC_Decode, 207, 22, 201, 3,               // 66285:      decode to FCMEQv2i32rz using decoder 457
                                             // 66285:     }
  33, 0,                                     // 66290:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 66292:      check predicate 76
  OPC_Decode, 196, 30, 201, 3,               // 66294:      decode to FRECPEv2f32 using decoder 457
                                             // 66294:     }
                                             // 66294:     } // switch Inst[21:16]
                                             // 66294:    }
  55, 11,                                    // 66299:    case 0x37: {
  OPC_CheckPredicate, 98,                    // 66301:     check predicate 98
  OPC_CheckField, 21, 1, 1,                  // 66303:     check Inst[21] == 0x1
  OPC_Decode, 151, 22, 200, 3,               // 66307:     decode to FAMAXv2f32 using decoder 456
                                             // 66307:    }
  58, 11,                                    // 66312:    case 0x3a: {
  OPC_CheckPredicate, 76,                    // 66314:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 66316:     check Inst[21:16] == 0x20
  OPC_Decode, 161, 23, 201, 3,               // 66320:     decode to FCMLTv2i32rz using decoder 457
                                             // 66320:    }
  59, 11,                                    // 66325:    case 0x3b: {
  OPC_CheckPredicate, 83,                    // 66327:     check predicate 83
  OPC_CheckField, 21, 1, 1,                  // 66329:     check Inst[21] == 0x1
  OPC_Decode, 227, 28, 221, 3,               // 66333:     decode to FMLSLv4f16 using decoder 477
                                             // 66333:    }
  61, 11,                                    // 66338:    case 0x3d: {
  OPC_CheckPredicate, 76,                    // 66340:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66342:     check Inst[21] == 0x1
  OPC_Decode, 234, 27, 200, 3,               // 66346:     decode to FMINv2f32 using decoder 456
                                             // 66346:    }
  62, 21,                                    // 66351:    case 0x3e: {
  OPC_SwitchField, 16, 6,                    // 66353:     switch Inst[21:16] {
  32, 7,                                     // 66356:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 66358:      check predicate 76
  OPC_Decode, 200, 21, 201, 3,               // 66360:      decode to FABSv2f32 using decoder 457
                                             // 66360:     }
  48, 0,                                     // 66365:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 66367:      check predicate 81
  OPC_Decode, 213, 27, 212, 3,               // 66369:      decode to FMINVv4i16v using decoder 468
                                             // 66369:     }
                                             // 66369:     } // switch Inst[21:16]
                                             // 66369:    }
  63, 0,                                     // 66374:    case 0x3f: {
  OPC_SwitchField, 21, 1,                    // 66376:     switch Inst[21] {
  0, 7,                                      // 66379:     case 0x0: {
  OPC_CheckPredicate, 99,                    // 66381:      check predicate 99
  OPC_Decode, 156, 26, 221, 3,               // 66383:      decode to FDOTv4f16_v2f32 using decoder 477
                                             // 66383:     }
  1, 0,                                      // 66388:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66390:      check predicate 76
  OPC_Decode, 252, 31, 200, 3,               // 66392:      decode to FRSQRTSv2f32 using decoder 456
                                             // 66392:     }
                                             // 66392:     } // switch Inst[21]
                                             // 66392:    }
                                             // 66392:    } // switch Inst[15:10]
                                             // 66392:   }
  1, 244, 6,                                 // 66397:   case 0x1: {
  OPC_SwitchField, 13, 3,                    // 66400:    switch Inst[15:13] {
  0, 81,                                     // 66403:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 66405:     switch Inst[12:10] {
  0, 11,                                     // 66408:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 66410:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66412:      check Inst[21] == 0x1
  OPC_Decode, 207, 62, 196, 3,               // 66416:      decode to UADDLv2i32_v2i64 using decoder 452
                                             // 66416:     }
  1, 11,                                     // 66421:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66423:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66425:      check Inst[21] == 0x1
  OPC_Decode, 210, 63, 200, 3,               // 66429:      decode to UHADDv2i32 using decoder 456
                                             // 66429:     }
  3, 11,                                     // 66434:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 66436:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66438:      check Inst[21] == 0x1
  OPC_Decode, 135, 66, 200, 3,               // 66442:      decode to UQADDv2i32 using decoder 456
                                             // 66442:     }
  4, 11,                                     // 66447:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 66449:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66451:      check Inst[21] == 0x1
  OPC_Decode, 223, 62, 204, 3,               // 66455:      decode to UADDWv2i32_v2i64 using decoder 460
                                             // 66455:     }
  5, 11,                                     // 66460:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 66462:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66464:      check Inst[21] == 0x1
  OPC_Decode, 206, 67, 200, 3,               // 66468:      decode to URHADDv2i32 using decoder 456
                                             // 66468:     }
  7, 0,                                      // 66473:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 66475:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66477:      check Inst[21] == 0x1
  OPC_Decode, 153, 16, 221, 3,               // 66481:      decode to BITv8i8 using decoder 477
                                             // 66481:     }
                                             // 66481:     } // switch Inst[12:10]
                                             // 66481:    }
  1, 127,                                    // 66486:    case 0x1: {
  OPC_SwitchField, 10, 3,                    // 66488:     switch Inst[12:10] {
  0, 11,                                     // 66491:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 66493:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66495:      check Inst[21] == 0x1
  OPC_Decode, 248, 68, 196, 3,               // 66499:      decode to USUBLv2i32_v2i64 using decoder 452
                                             // 66499:     }
  1, 11,                                     // 66504:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66506:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66508:      check Inst[21] == 0x1
  OPC_Decode, 224, 63, 200, 3,               // 66512:      decode to UHSUBv2i32 using decoder 456
                                             // 66512:     }
  2, 21,                                     // 66517:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 66519:      switch Inst[21:16] {
  32, 7,                                     // 66522:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 66524:       check predicate 76
  OPC_Decode, 193, 62, 201, 3,               // 66526:       decode to UADDLPv2i32_v1i64 using decoder 457
                                             // 66526:      }
  33, 0,                                     // 66531:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 66533:       check predicate 76
  OPC_Decode, 224, 54, 206, 3,               // 66535:       decode to SQXTUNv2i32 using decoder 462
                                             // 66535:      }
                                             // 66535:      } // switch Inst[21:16]
                                             // 66535:     }
  3, 11,                                     // 66540:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 66542:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66544:      check Inst[21] == 0x1
  OPC_Decode, 176, 67, 200, 3,               // 66548:      decode to UQSUBv2i32 using decoder 456
                                             // 66548:     }
  4, 11,                                     // 66553:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 66555:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66557:      check Inst[21] == 0x1
  OPC_Decode, 132, 69, 204, 3,               // 66561:      decode to USUBWv2i32_v2i64 using decoder 460
                                             // 66561:     }
  5, 11,                                     // 66566:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 66568:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66570:      check Inst[21] == 0x1
  OPC_Decode, 129, 18, 200, 3,               // 66574:      decode to CMHIv2i32 using decoder 456
                                             // 66574:     }
  6, 21,                                     // 66579:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 66581:      switch Inst[21:16] {
  32, 7,                                     // 66584:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 66586:       check predicate 76
  OPC_Decode, 222, 68, 211, 3,               // 66588:       decode to USQADDv2i32 using decoder 467
                                             // 66588:      }
  33, 0,                                     // 66593:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 66595:       check predicate 76
  OPC_Decode, 156, 48, 220, 3,               // 66597:       decode to SHLLv2i32 using decoder 476
                                             // 66597:      }
                                             // 66597:      } // switch Inst[21:16]
                                             // 66597:     }
  7, 0,                                      // 66602:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 66604:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66606:      check Inst[21] == 0x1
  OPC_Decode, 137, 18, 200, 3,               // 66610:      decode to CMHSv2i32 using decoder 456
                                             // 66610:     }
                                             // 66610:     } // switch Inst[12:10]
                                             // 66610:    }
  2, 104,                                    // 66615:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 66617:     switch Inst[12:10] {
  0, 11,                                     // 66620:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 66622:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66624:      check Inst[21] == 0x1
  OPC_Decode, 222, 44, 215, 3,               // 66628:      decode to RADDHNv2i64_v2i32 using decoder 471
                                             // 66628:     }
  1, 11,                                     // 66633:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66635:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66637:      check Inst[21] == 0x1
  OPC_Decode, 169, 68, 200, 3,               // 66641:      decode to USHLv2i32 using decoder 456
                                             // 66641:     }
  2, 21,                                     // 66646:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 66648:      switch Inst[21:16] {
  32, 7,                                     // 66651:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 66653:       check predicate 76
  OPC_Decode, 202, 17, 201, 3,               // 66655:       decode to CLZv2i32 using decoder 457
                                             // 66655:      }
  33, 0,                                     // 66660:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 66662:       check predicate 76
  OPC_Decode, 192, 67, 206, 3,               // 66664:       decode to UQXTNv2i32 using decoder 462
                                             // 66664:      }
                                             // 66664:      } // switch Inst[21:16]
                                             // 66664:     }
  3, 11,                                     // 66669:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 66671:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66673:      check Inst[21] == 0x1
  OPC_Decode, 254, 66, 200, 3,               // 66677:      decode to UQSHLv2i32 using decoder 456
                                             // 66677:     }
  4, 11,                                     // 66682:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 66684:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66686:      check Inst[21] == 0x1
  OPC_Decode, 143, 62, 217, 3,               // 66690:      decode to UABALv2i32_v2i64 using decoder 473
                                             // 66690:     }
  5, 11,                                     // 66695:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 66697:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66699:      check Inst[21] == 0x1
  OPC_Decode, 237, 67, 200, 3,               // 66703:      decode to URSHLv2i32 using decoder 456
                                             // 66703:     }
  7, 0,                                      // 66708:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 66710:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66712:      check Inst[21] == 0x1
  OPC_Decode, 204, 66, 200, 3,               // 66716:      decode to UQRSHLv2i32 using decoder 456
                                             // 66716:     }
                                             // 66716:     } // switch Inst[12:10]
                                             // 66716:    }
  3, 117,                                    // 66721:    case 0x3: {
  OPC_SwitchField, 10, 3,                    // 66723:     switch Inst[12:10] {
  0, 11,                                     // 66726:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 66728:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66730:      check Inst[21] == 0x1
  OPC_Decode, 251, 45, 215, 3,               // 66734:      decode to RSUBHNv2i64_v2i32 using decoder 471
                                             // 66734:     }
  1, 11,                                     // 66739:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66741:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66743:      check Inst[21] == 0x1
  OPC_Decode, 154, 64, 200, 3,               // 66747:      decode to UMAXv2i32 using decoder 456
                                             // 66747:     }
  2, 11,                                     // 66752:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 66754:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 66756:      check Inst[21:16] == 0x20
  OPC_Decode, 184, 62, 211, 3,               // 66760:      decode to UADALPv2i32_v1i64 using decoder 467
                                             // 66760:     }
  3, 11,                                     // 66765:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 66767:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66769:      check Inst[21] == 0x1
  OPC_Decode, 211, 64, 200, 3,               // 66773:      decode to UMINv2i32 using decoder 456
                                             // 66773:     }
  4, 11,                                     // 66778:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 66780:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66782:      check Inst[21] == 0x1
  OPC_Decode, 165, 62, 196, 3,               // 66786:      decode to UABDLv2i32_v2i64 using decoder 452
                                             // 66786:     }
  5, 11,                                     // 66791:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 66793:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66795:      check Inst[21] == 0x1
  OPC_Decode, 175, 62, 200, 3,               // 66799:      decode to UABDv2i32 using decoder 456
                                             // 66799:     }
  6, 21,                                     // 66804:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 66806:      switch Inst[21:16] {
  32, 7,                                     // 66809:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 66811:       check predicate 76
  OPC_Decode, 213, 52, 201, 3,               // 66813:       decode to SQNEGv2i32 using decoder 457
                                             // 66813:      }
  33, 0,                                     // 66818:      case 0x21: {
  OPC_CheckPredicate, 84,                    // 66820:       check predicate 84
  OPC_Decode, 237, 14, 220, 3,               // 66822:       decode to BF1CVTL using decoder 476
                                             // 66822:      }
                                             // 66822:      } // switch Inst[21:16]
                                             // 66822:     }
  7, 0,                                      // 66827:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 66829:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66831:      check Inst[21] == 0x1
  OPC_Decode, 153, 62, 221, 3,               // 66835:      decode to UABAv2i32 using decoder 477
                                             // 66835:     }
                                             // 66835:     } // switch Inst[12:10]
                                             // 66835:    }
  4, 121,                                    // 66840:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 66842:     switch Inst[12:10] {
  0, 11,                                     // 66845:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 66847:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66849:      check Inst[21] == 0x1
  OPC_Decode, 252, 64, 217, 3,               // 66853:      decode to UMLALv2i32_v2i64 using decoder 473
                                             // 66853:     }
  1, 21,                                     // 66858:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 66860:      switch Inst[21] {
  0, 7,                                      // 66863:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 66865:       check predicate 90
  OPC_Decode, 236, 52, 221, 3,               // 66867:       decode to SQRDMLAHv2i32 using decoder 477
                                             // 66867:      }
  1, 0,                                      // 66872:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 66874:       check predicate 76
  OPC_Decode, 194, 60, 200, 3,               // 66876:       decode to SUBv2i32 using decoder 456
                                             // 66876:      }
                                             // 66876:      } // switch Inst[21]
                                             // 66876:     }
  2, 11,                                     // 66881:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 66883:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 66885:      check Inst[21:16] == 0x20
  OPC_Decode, 228, 17, 201, 3,               // 66889:      decode to CMGEv2i32rz using decoder 457
                                             // 66889:     }
  3, 21,                                     // 66894:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 66896:      switch Inst[21] {
  0, 7,                                      // 66899:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 66901:       check predicate 90
  OPC_Decode, 255, 52, 221, 3,               // 66903:       decode to SQRDMLSHv2i32 using decoder 477
                                             // 66903:      }
  1, 0,                                      // 66908:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 66910:       check predicate 76
  OPC_Decode, 211, 17, 200, 3,               // 66912:       decode to CMEQv2i32 using decoder 456
                                             // 66912:      }
                                             // 66912:      } // switch Inst[21]
                                             // 66912:     }
  5, 21,                                     // 66917:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 66919:      switch Inst[21] {
  0, 7,                                      // 66922:      case 0x0: {
  OPC_CheckPredicate, 96,                    // 66924:       check predicate 96
  OPC_Decode, 204, 63, 221, 3,               // 66926:       decode to UDOTv8i8 using decoder 477
                                             // 66926:      }
  1, 0,                                      // 66931:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 66933:       check predicate 76
  OPC_Decode, 137, 42, 221, 3,               // 66935:       decode to MLSv2i32 using decoder 477
                                             // 66935:      }
                                             // 66935:      } // switch Inst[21]
                                             // 66935:     }
  6, 0,                                      // 66940:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 66942:      switch Inst[21:16] {
  32, 7,                                     // 66945:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 66947:       check predicate 76
  OPC_Decode, 151, 18, 201, 3,               // 66949:       decode to CMLEv2i32rz using decoder 457
                                             // 66949:      }
  33, 0,                                     // 66954:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 66956:       check predicate 76
  OPC_Decode, 154, 31, 201, 3,               // 66958:       decode to FRINTIv2f32 using decoder 457
                                             // 66958:      }
                                             // 66958:      } // switch Inst[21:16]
                                             // 66958:     }
                                             // 66958:     } // switch Inst[12:10]
                                             // 66958:    }
  5, 91,                                     // 66963:    case 0x5: {
  OPC_SwitchField, 10, 3,                    // 66965:     switch Inst[12:10] {
  0, 11,                                     // 66968:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 66970:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66972:      check Inst[21] == 0x1
  OPC_Decode, 168, 65, 217, 3,               // 66976:      decode to UMLSLv2i32_v2i64 using decoder 473
                                             // 66976:     }
  1, 11,                                     // 66981:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 66983:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 66985:      check Inst[21] == 0x1
  OPC_Decode, 235, 63, 200, 3,               // 66989:      decode to UMAXPv2i32 using decoder 456
                                             // 66989:     }
  2, 11,                                     // 66994:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 66996:      check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 66998:      check Inst[21:16] == 0x21
  OPC_Decode, 234, 24, 201, 3,               // 67002:      decode to FCVTPUv2f32 using decoder 457
                                             // 67002:     }
  3, 11,                                     // 67007:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 67009:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67011:      check Inst[21] == 0x1
  OPC_Decode, 164, 64, 200, 3,               // 67015:      decode to UMINPv2i32 using decoder 456
                                             // 67015:     }
  5, 11,                                     // 67020:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 67022:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67024:      check Inst[21] == 0x1
  OPC_Decode, 146, 53, 200, 3,               // 67028:      decode to SQRDMULHv2i32 using decoder 456
                                             // 67028:     }
  6, 0,                                      // 67033:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 67035:      switch Inst[21:16] {
  32, 7,                                     // 67038:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 67040:       check predicate 76
  OPC_Decode, 172, 43, 201, 3,               // 67042:       decode to NEGv2i32 using decoder 457
                                             // 67042:      }
  33, 0,                                     // 67047:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 67049:       check predicate 76
  OPC_Decode, 212, 25, 201, 3,               // 67051:       decode to FCVTZUv2f32 using decoder 457
                                             // 67051:      }
                                             // 67051:      } // switch Inst[21:16]
                                             // 67051:     }
                                             // 67051:     } // switch Inst[12:10]
                                             // 67051:    }
  6, 122,                                    // 67056:    case 0x6: {
  OPC_SwitchField, 10, 1,                    // 67058:     switch Inst[10] {
  0, 62,                                     // 67061:     case 0x0: {
  OPC_SwitchField, 11, 2,                    // 67063:      switch Inst[12:11] {
  0, 11,                                     // 67066:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 67068:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67070:       check Inst[21] == 0x1
  OPC_Decode, 238, 65, 196, 3,               // 67074:       decode to UMULLv2i32_v2i64 using decoder 452
                                             // 67074:      }
  1, 21,                                     // 67079:      case 0x1: {
  OPC_SwitchField, 16, 6,                    // 67081:       switch Inst[21:16] {
  32, 7,                                     // 67084:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 67086:        check predicate 76
  OPC_Decode, 229, 22, 201, 3,               // 67088:        decode to FCMGEv2i32rz using decoder 457
                                             // 67088:       }
  33, 0,                                     // 67093:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 67095:        check predicate 76
  OPC_Decode, 129, 68, 201, 3,               // 67097:        decode to URSQRTEv2i32 using decoder 457
                                             // 67097:       }
                                             // 67097:       } // switch Inst[21:16]
                                             // 67097:      }
  3, 0,                                      // 67102:      case 0x3: {
  OPC_SwitchField, 16, 6,                    // 67104:       switch Inst[21:16] {
  32, 7,                                     // 67107:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 67109:        check predicate 76
  OPC_Decode, 150, 23, 201, 3,               // 67111:        decode to FCMLEv2i32rz using decoder 457
                                             // 67111:       }
  33, 0,                                     // 67116:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 67118:        check predicate 76
  OPC_Decode, 241, 31, 201, 3,               // 67120:        decode to FRSQRTEv2f32 using decoder 457
                                             // 67120:       }
                                             // 67120:       } // switch Inst[21:16]
                                             // 67120:      }
                                             // 67120:      } // switch Inst[12:11]
                                             // 67120:     }
  1, 0,                                      // 67125:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 67127:      switch Inst[21] {
  0, 7,                                      // 67130:      case 0x0: {
  OPC_CheckPredicate, 100,                   // 67132:       check predicate 100
  OPC_Decode, 136, 23, 131, 4,               // 67134:       decode to FCMLAv2f32 using decoder 515
                                             // 67134:      }
  1, 0,                                      // 67139:      case 0x1: {
  OPC_SwitchField, 11, 2,                    // 67141:       switch Inst[12:11] {
  0, 7,                                      // 67144:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 67146:        check predicate 76
  OPC_Decode, 154, 27, 200, 3,               // 67148:        decode to FMINNMPv2f32 using decoder 456
                                             // 67148:       }
  1, 7,                                      // 67153:       case 0x1: {
  OPC_CheckPredicate, 83,                    // 67155:        check predicate 83
  OPC_Decode, 211, 28, 221, 3,               // 67157:        decode to FMLSL2v4f16 using decoder 477
                                             // 67157:       }
  2, 7,                                      // 67162:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 67164:        check predicate 76
  OPC_Decode, 186, 21, 200, 3,               // 67166:        decode to FABDv2f32 using decoder 456
                                             // 67166:       }
  3, 0,                                      // 67171:       case 0x3: {
  OPC_CheckPredicate, 98,                    // 67173:        check predicate 98
  OPC_Decode, 165, 22, 200, 3,               // 67175:        decode to FAMINv2f32 using decoder 456
                                             // 67175:       }
                                             // 67175:       } // switch Inst[12:11]
                                             // 67175:      }
                                             // 67175:      } // switch Inst[21]
                                             // 67175:     }
                                             // 67175:     } // switch Inst[10]
                                             // 67175:    }
  7, 0,                                      // 67180:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 67182:     switch Inst[11:10] {
  1, 35,                                     // 67185:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 67187:      switch Inst[21] {
  0, 7,                                      // 67190:      case 0x0: {
  OPC_CheckPredicate, 100,                   // 67192:       check predicate 100
  OPC_Decode, 173, 22, 132, 4,               // 67194:       decode to FCADDv2f32 using decoder 516
                                             // 67194:      }
  1, 0,                                      // 67199:      case 0x1: {
  OPC_SwitchField, 12, 1,                    // 67201:       switch Inst[12] {
  0, 7,                                      // 67204:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 67206:        check predicate 76
  OPC_Decode, 249, 22, 200, 3,               // 67208:        decode to FCMGTv2f32 using decoder 456
                                             // 67208:       }
  1, 0,                                      // 67213:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 67215:        check predicate 76
  OPC_Decode, 198, 27, 200, 3,               // 67217:        decode to FMINPv2f32 using decoder 456
                                             // 67217:       }
                                             // 67217:       } // switch Inst[12]
                                             // 67217:      }
                                             // 67217:      } // switch Inst[21]
                                             // 67217:     }
  2, 29,                                     // 67222:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 67224:      switch Inst[21:16] {
  32, 11,                                    // 67227:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 67229:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 67231:       check Inst[12] == 0x1
  OPC_Decode, 164, 30, 201, 3,               // 67235:       decode to FNEGv2f32 using decoder 457
                                             // 67235:      }
  33, 0,                                     // 67240:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 67242:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 67244:       check Inst[12] == 0x1
  OPC_Decode, 158, 32, 201, 3,               // 67248:       decode to FSQRTv2f32 using decoder 457
                                             // 67248:      }
                                             // 67248:      } // switch Inst[21:16]
                                             // 67248:     }
  3, 0,                                      // 67253:     case 0x3: {
  OPC_SwitchField, 12, 1,                    // 67255:      switch Inst[12] {
  0, 11,                                     // 67258:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 67260:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67262:       check Inst[21] == 0x1
  OPC_Decode, 222, 21, 200, 3,               // 67266:       decode to FACGTv2f32 using decoder 456
                                             // 67266:      }
  1, 0,                                      // 67271:      case 0x1: {
  OPC_CheckPredicate, 84,                    // 67273:       check predicate 84
  OPC_CheckField, 21, 1, 1,                  // 67275:       check Inst[21] == 0x1
  OPC_Decode, 144, 32, 200, 3,               // 67279:       decode to FSCALEv2f32 using decoder 456
                                             // 67279:      }
                                             // 67279:      } // switch Inst[12]
                                             // 67279:     }
                                             // 67279:     } // switch Inst[11:10]
                                             // 67279:    }
                                             // 67279:    } // switch Inst[15:13]
                                             // 67279:   }
  2, 202, 8,                                 // 67284:   case 0x2: {
  OPC_SwitchField, 10, 3,                    // 67287:    switch Inst[12:10] {
  0, 94,                                     // 67290:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 67292:     switch Inst[15:13] {
  0, 11,                                     // 67295:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 67297:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67299:      check Inst[21] == 0x1
  OPC_Decode, 208, 46, 225, 3,               // 67303:      decode to SADDLv4i32_v2i64 using decoder 481
                                             // 67303:     }
  1, 11,                                     // 67308:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 67310:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67312:      check Inst[21] == 0x1
  OPC_Decode, 145, 56, 225, 3,               // 67316:      decode to SSUBLv4i32_v2i64 using decoder 481
                                             // 67316:     }
  2, 11,                                     // 67321:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 67323:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67325:      check Inst[21] == 0x1
  OPC_Decode, 144, 13, 223, 3,               // 67329:      decode to ADDHNv2i64_v4i32 using decoder 479
                                             // 67329:     }
  3, 11,                                     // 67334:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 67336:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67338:      check Inst[21] == 0x1
  OPC_Decode, 132, 60, 223, 3,               // 67342:      decode to SUBHNv2i64_v4i32 using decoder 479
                                             // 67342:     }
  4, 11,                                     // 67347:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 67349:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67351:      check Inst[21] == 0x1
  OPC_Decode, 247, 49, 223, 3,               // 67355:      decode to SMLALv4i32_v2i64 using decoder 479
                                             // 67355:     }
  5, 11,                                     // 67360:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 67362:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67364:      check Inst[21] == 0x1
  OPC_Decode, 163, 50, 223, 3,               // 67368:      decode to SMLSLv4i32_v2i64 using decoder 479
                                             // 67368:     }
  6, 0,                                      // 67373:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 67375:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67377:      check Inst[21] == 0x1
  OPC_Decode, 235, 50, 225, 3,               // 67381:      decode to SMULLv4i32_v2i64 using decoder 481
                                             // 67381:     }
                                             // 67381:     } // switch Inst[15:13]
                                             // 67381:    }
  1, 104,                                    // 67386:    case 0x1: {
  OPC_SwitchField, 13, 3,                    // 67388:     switch Inst[15:13] {
  0, 11,                                     // 67391:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 67393:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67395:      check Inst[21] == 0x1
  OPC_Decode, 152, 48, 225, 3,               // 67399:      decode to SHADDv4i32 using decoder 481
                                             // 67399:     }
  1, 11,                                     // 67404:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 67406:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67408:      check Inst[21] == 0x1
  OPC_Decode, 192, 48, 225, 3,               // 67412:      decode to SHSUBv4i32 using decoder 481
                                             // 67412:     }
  2, 11,                                     // 67417:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 67419:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67421:      check Inst[21] == 0x1
  OPC_Decode, 196, 55, 225, 3,               // 67425:      decode to SSHLv4i32 using decoder 481
                                             // 67425:     }
  3, 11,                                     // 67430:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 67432:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67434:      check Inst[21] == 0x1
  OPC_Decode, 146, 49, 225, 3,               // 67438:      decode to SMAXv4i32 using decoder 481
                                             // 67438:     }
  4, 11,                                     // 67443:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 67445:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67447:      check Inst[21] == 0x1
  OPC_Decode, 238, 13, 225, 3,               // 67451:      decode to ADDv4i32 using decoder 481
                                             // 67451:     }
  5, 21,                                     // 67456:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 67458:      switch Inst[21] {
  0, 7,                                      // 67461:      case 0x0: {
  OPC_CheckPredicate, 97,                    // 67463:       check predicate 97
  OPC_Decode, 167, 50, 223, 3,               // 67465:       decode to SMMLA using decoder 479
                                             // 67465:      }
  1, 0,                                      // 67470:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 67472:       check predicate 76
  OPC_Decode, 227, 48, 225, 3,               // 67474:       decode to SMAXPv4i32 using decoder 481
                                             // 67474:      }
                                             // 67474:      } // switch Inst[21]
                                             // 67474:     }
  6, 0,                                      // 67479:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 67481:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67483:      check Inst[21] == 0x1
  OPC_Decode, 193, 27, 225, 3,               // 67487:      decode to FMINNMv4f32 using decoder 481
                                             // 67487:     }
                                             // 67487:     } // switch Inst[15:13]
                                             // 67487:    }
  2, 222, 1,                                 // 67492:    case 0x2: {
  OPC_SwitchField, 13, 3,                    // 67495:     switch Inst[15:13] {
  0, 11,                                     // 67498:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 67500:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 67502:      check Inst[21:16] == 0x20
  OPC_Decode, 202, 45, 230, 3,               // 67506:      decode to REV64v4i32 using decoder 486
                                             // 67506:     }
  1, 35,                                     // 67511:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 67513:      switch Inst[21] {
  0, 7,                                      // 67516:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 67518:       check predicate 76
  OPC_Decode, 241, 61, 225, 3,               // 67520:       decode to TRN1v4i32 using decoder 481
                                             // 67520:      }
  1, 0,                                      // 67525:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 67527:       switch Inst[20:16] {
  0, 7,                                      // 67530:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 67532:        check predicate 76
  OPC_Decode, 194, 46, 230, 3,               // 67534:        decode to SADDLPv4i32_v2i64 using decoder 486
                                             // 67534:       }
  1, 0,                                      // 67539:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 67541:        check predicate 76
  OPC_Decode, 240, 70, 238, 3,               // 67543:        decode to XTNv4i32 using decoder 494
                                             // 67543:       }
                                             // 67543:       } // switch Inst[20:16]
                                             // 67543:      }
                                             // 67543:      } // switch Inst[21]
                                             // 67543:     }
  2, 21,                                     // 67548:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 67550:      switch Inst[21:16] {
  32, 7,                                     // 67553:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 67555:       check predicate 76
  OPC_Decode, 188, 17, 230, 3,               // 67557:       decode to CLSv4i32 using decoder 486
                                             // 67557:      }
  33, 0,                                     // 67562:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 67564:       check predicate 76
  OPC_Decode, 211, 54, 238, 3,               // 67566:       decode to SQXTNv4i32 using decoder 494
                                             // 67566:      }
                                             // 67566:      } // switch Inst[21:16]
                                             // 67566:     }
  3, 35,                                     // 67571:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 67573:      switch Inst[21] {
  0, 7,                                      // 67576:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 67578:       check predicate 76
  OPC_Decode, 129, 62, 225, 3,               // 67580:       decode to TRN2v4i32 using decoder 481
                                             // 67580:      }
  1, 0,                                      // 67585:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 67587:       switch Inst[20:16] {
  0, 7,                                      // 67590:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 67592:        check predicate 76
  OPC_Decode, 182, 46, 238, 3,               // 67594:        decode to SADALPv4i32_v2i64 using decoder 494
                                             // 67594:       }
  1, 0,                                      // 67599:       case 0x1: {
  OPC_CheckPredicate, 92,                    // 67601:        check predicate 92
  OPC_Decode, 130, 15, 238, 3,               // 67603:        decode to BFCVTN2 using decoder 494
                                             // 67603:       }
                                             // 67603:       } // switch Inst[20:16]
                                             // 67603:      }
                                             // 67603:      } // switch Inst[21]
                                             // 67603:     }
  4, 21,                                     // 67608:     case 0x4: {
  OPC_SwitchField, 16, 6,                    // 67610:      switch Inst[21:16] {
  32, 7,                                     // 67613:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 67615:       check predicate 76
  OPC_Decode, 250, 17, 230, 3,               // 67617:       decode to CMGTv4i32rz using decoder 486
                                             // 67617:      }
  33, 0,                                     // 67622:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 67624:       check predicate 76
  OPC_Decode, 205, 31, 230, 3,               // 67626:       decode to FRINTPv4f32 using decoder 486
                                             // 67626:      }
                                             // 67626:      } // switch Inst[21:16]
                                             // 67626:     }
  5, 39,                                     // 67631:     case 0x5: {
  OPC_SwitchField, 16, 6,                    // 67633:      switch Inst[21:16] {
  32, 7,                                     // 67636:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 67638:       check predicate 76
  OPC_Decode, 162, 18, 230, 3,               // 67640:       decode to CMLTv4i32rz using decoder 486
                                             // 67640:      }
  33, 7,                                     // 67645:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 67647:       check predicate 76
  OPC_Decode, 219, 24, 230, 3,               // 67649:       decode to FCVTPSv4f32 using decoder 486
                                             // 67649:      }
  48, 7,                                     // 67654:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 67656:       check predicate 76
  OPC_Decode, 240, 48, 252, 3,               // 67658:       decode to SMAXVv4i32v using decoder 508
                                             // 67658:      }
  49, 0,                                     // 67663:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 67665:       check predicate 76
  OPC_Decode, 170, 49, 252, 3,               // 67667:       decode to SMINVv4i32v using decoder 508
                                             // 67667:      }
                                             // 67667:      } // switch Inst[21:16]
                                             // 67667:     }
  6, 30,                                     // 67672:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 67674:      switch Inst[21:16] {
  32, 7,                                     // 67677:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 67679:       check predicate 76
  OPC_Decode, 128, 23, 230, 3,               // 67681:       decode to FCMGTv4i32rz using decoder 486
                                             // 67681:      }
  33, 7,                                     // 67686:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 67688:       check predicate 76
  OPC_Decode, 200, 67, 230, 3,               // 67690:       decode to URECPEv4i32 using decoder 486
                                             // 67690:      }
  48, 0,                                     // 67695:      case 0x30: {
  OPC_CheckPredicate, 81,                    // 67697:       check predicate 81
  OPC_Decode, 171, 27, 244, 3,               // 67699:       decode to FMINNMVv8i16v using decoder 500
                                             // 67699:      }
                                             // 67699:      } // switch Inst[21:16]
                                             // 67699:     }
  7, 0,                                      // 67704:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 67706:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 67708:      check Inst[21:16] == 0x20
  OPC_Decode, 164, 23, 230, 3,               // 67712:      decode to FCMLTv4i32rz using decoder 486
                                             // 67712:     }
                                             // 67712:     } // switch Inst[15:13]
                                             // 67712:    }
  3, 117,                                    // 67717:    case 0x3: {
  OPC_SwitchField, 13, 3,                    // 67719:     switch Inst[15:13] {
  0, 11,                                     // 67722:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 67724:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67726:      check Inst[21] == 0x1
  OPC_Decode, 158, 51, 225, 3,               // 67730:      decode to SQADDv4i32 using decoder 481
                                             // 67730:     }
  1, 11,                                     // 67735:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 67737:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67739:      check Inst[21] == 0x1
  OPC_Decode, 196, 54, 225, 3,               // 67743:      decode to SQSUBv4i32 using decoder 481
                                             // 67743:     }
  2, 11,                                     // 67748:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 67750:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67752:      check Inst[21] == 0x1
  OPC_Decode, 132, 54, 225, 3,               // 67756:      decode to SQSHLv4i32 using decoder 481
                                             // 67756:     }
  3, 11,                                     // 67761:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 67763:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67765:      check Inst[21] == 0x1
  OPC_Decode, 204, 49, 225, 3,               // 67769:      decode to SMINv4i32 using decoder 481
                                             // 67769:     }
  4, 11,                                     // 67774:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 67776:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67778:      check Inst[21] == 0x1
  OPC_Decode, 136, 19, 225, 3,               // 67782:      decode to CMTSTv4i32 using decoder 481
                                             // 67782:     }
  5, 21,                                     // 67787:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 67789:      switch Inst[21] {
  0, 7,                                      // 67792:      case 0x0: {
  OPC_CheckPredicate, 97,                    // 67794:       check predicate 97
  OPC_Decode, 191, 68, 223, 3,               // 67796:       decode to USMMLA using decoder 479
                                             // 67796:      }
  1, 0,                                      // 67801:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 67803:       check predicate 76
  OPC_Decode, 157, 49, 225, 3,               // 67805:       decode to SMINPv4i32 using decoder 481
                                             // 67805:      }
                                             // 67805:      } // switch Inst[21]
                                             // 67805:     }
  6, 11,                                     // 67810:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 67812:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67814:      check Inst[21] == 0x1
  OPC_Decode, 133, 29, 223, 3,               // 67818:      decode to FMLSv4f32 using decoder 479
                                             // 67818:     }
  7, 0,                                      // 67823:     case 0x7: {
  OPC_CheckPredicate, 83,                    // 67825:      check predicate 83
  OPC_CheckField, 21, 1, 1,                  // 67827:      check Inst[21] == 0x1
  OPC_Decode, 228, 28, 223, 3,               // 67831:      decode to FMLSLv8f16 using decoder 479
                                             // 67831:     }
                                             // 67831:     } // switch Inst[15:13]
                                             // 67831:    }
  4, 102,                                    // 67836:    case 0x4: {
  OPC_SwitchField, 15, 1,                    // 67838:     switch Inst[15] {
  0, 53,                                     // 67841:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 67843:      switch Inst[21] {
  0, 7,                                      // 67846:      case 0x0: {
  OPC_CheckPredicate, 93,                    // 67848:       check predicate 93
  OPC_Decode, 187, 41, 138, 4,               // 67850:       decode to LUT2_B using decoder 522
                                             // 67850:      }
  1, 0,                                      // 67855:      case 0x1: {
  OPC_SwitchField, 13, 2,                    // 67857:       switch Inst[14:13] {
  0, 7,                                      // 67860:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 67862:        check predicate 76
  OPC_Decode, 223, 46, 225, 3,               // 67864:        decode to SADDWv4i32_v2i64 using decoder 481
                                             // 67864:       }
  1, 7,                                      // 67869:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 67871:        check predicate 76
  OPC_Decode, 157, 56, 225, 3,               // 67873:        decode to SSUBWv4i32_v2i64 using decoder 481
                                             // 67873:       }
  2, 7,                                      // 67878:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 67880:        check predicate 76
  OPC_Decode, 141, 46, 223, 3,               // 67882:        decode to SABALv4i32_v2i64 using decoder 479
                                             // 67882:       }
  3, 0,                                      // 67887:       case 0x3: {
  OPC_CheckPredicate, 76,                    // 67889:        check predicate 76
  OPC_Decode, 163, 46, 225, 3,               // 67891:        decode to SABDLv4i32_v2i64 using decoder 481
                                             // 67891:       }
                                             // 67891:       } // switch Inst[14:13]
                                             // 67891:      }
                                             // 67891:      } // switch Inst[21]
                                             // 67891:     }
  1, 0,                                      // 67896:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 67898:      switch Inst[14:13] {
  0, 11,                                     // 67901:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 67903:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67905:       check Inst[21] == 0x1
  OPC_Decode, 221, 51, 223, 3,               // 67909:       decode to SQDMLALv4i32_v2i64 using decoder 479
                                             // 67909:      }
  1, 11,                                     // 67914:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 67916:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67918:       check Inst[21] == 0x1
  OPC_Decode, 246, 51, 223, 3,               // 67922:       decode to SQDMLSLv4i32_v2i64 using decoder 479
                                             // 67922:      }
  2, 0,                                      // 67927:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 67929:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67931:       check Inst[21] == 0x1
  OPC_Decode, 175, 52, 225, 3,               // 67935:       decode to SQDMULLv4i32_v2i64 using decoder 481
                                             // 67935:      }
                                             // 67935:      } // switch Inst[14:13]
                                             // 67935:     }
                                             // 67935:     } // switch Inst[15]
                                             // 67935:    }
  5, 117,                                    // 67940:    case 0x5: {
  OPC_SwitchField, 13, 3,                    // 67942:     switch Inst[15:13] {
  0, 11,                                     // 67945:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 67947:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67949:      check Inst[21] == 0x1
  OPC_Decode, 236, 54, 225, 3,               // 67953:      decode to SRHADDv4i32 using decoder 481
                                             // 67953:     }
  1, 11,                                     // 67958:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 67960:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67962:      check Inst[21] == 0x1
  OPC_Decode, 249, 17, 225, 3,               // 67966:      decode to CMGTv4i32 using decoder 481
                                             // 67966:     }
  2, 11,                                     // 67971:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 67973:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67975:      check Inst[21] == 0x1
  OPC_Decode, 152, 55, 225, 3,               // 67979:      decode to SRSHLv4i32 using decoder 481
                                             // 67979:     }
  3, 11,                                     // 67984:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 67986:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 67988:      check Inst[21] == 0x1
  OPC_Decode, 173, 46, 225, 3,               // 67992:      decode to SABDv4i32 using decoder 481
                                             // 67992:     }
  4, 21,                                     // 67997:     case 0x4: {
  OPC_SwitchField, 21, 1,                    // 67999:      switch Inst[21] {
  0, 7,                                      // 68002:      case 0x0: {
  OPC_CheckPredicate, 96,                    // 68004:       check predicate 96
  OPC_Decode, 209, 47, 223, 3,               // 68006:       decode to SDOTv16i8 using decoder 479
                                             // 68006:      }
  1, 0,                                      // 68011:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68013:       check predicate 76
  OPC_Decode, 252, 41, 223, 3,               // 68015:       decode to MLAv4i32 using decoder 479
                                             // 68015:      }
                                             // 68015:      } // switch Inst[21]
                                             // 68015:     }
  5, 11,                                     // 68020:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 68022:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68024:      check Inst[21] == 0x1
  OPC_Decode, 152, 52, 225, 3,               // 68028:      decode to SQDMULHv4i32 using decoder 481
                                             // 68028:     }
  6, 11,                                     // 68033:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 68035:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68037:      check Inst[21] == 0x1
  OPC_Decode, 190, 32, 225, 3,               // 68041:      decode to FSUBv4f32 using decoder 481
                                             // 68041:     }
  7, 0,                                      // 68046:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 68048:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68050:      check Inst[21] == 0x1
  OPC_Decode, 237, 27, 225, 3,               // 68054:      decode to FMINv4f32 using decoder 481
                                             // 68054:     }
                                             // 68054:     } // switch Inst[15:13]
                                             // 68054:    }
  6, 194, 1,                                 // 68059:    case 0x6: {
  OPC_SwitchField, 13, 3,                    // 68062:     switch Inst[15:13] {
  0, 11,                                     // 68065:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68067:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 68069:      check Inst[21] == 0x0
  OPC_Decode, 180, 69, 225, 3,               // 68073:      decode to UZP1v4i32 using decoder 481
                                             // 68073:     }
  1, 35,                                     // 68078:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 68080:      switch Inst[21] {
  0, 7,                                      // 68083:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 68085:       check predicate 76
  OPC_Decode, 138, 71, 225, 3,               // 68087:       decode to ZIP1v4i32 using decoder 481
                                             // 68087:      }
  1, 0,                                      // 68092:      case 0x1: {
  OPC_SwitchField, 16, 5,                    // 68094:       switch Inst[20:16] {
  0, 7,                                      // 68097:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 68099:        check predicate 76
  OPC_Decode, 128, 61, 238, 3,               // 68101:        decode to SUQADDv4i32 using decoder 494
                                             // 68101:       }
  16, 0,                                     // 68106:       case 0x10: {
  OPC_CheckPredicate, 76,                    // 68108:        check predicate 76
  OPC_Decode, 202, 46, 206, 3,               // 68110:        decode to SADDLVv4i32v using decoder 462
                                             // 68110:       }
                                             // 68110:       } // switch Inst[20:16]
                                             // 68110:      }
                                             // 68110:      } // switch Inst[21]
                                             // 68110:     }
  2, 11,                                     // 68115:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 68117:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 68119:      check Inst[21] == 0x0
  OPC_Decode, 196, 69, 225, 3,               // 68123:      decode to UZP2v4i32 using decoder 481
                                             // 68123:     }
  3, 25,                                     // 68128:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 68130:      switch Inst[21] {
  0, 7,                                      // 68133:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 68135:       check predicate 76
  OPC_Decode, 154, 71, 225, 3,               // 68137:       decode to ZIP2v4i32 using decoder 481
                                             // 68137:      }
  1, 0,                                      // 68142:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68144:       check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 68146:       check Inst[20:16] == 0x0
  OPC_Decode, 135, 51, 230, 3,               // 68150:       decode to SQABSv4i32 using decoder 486
                                             // 68150:      }
                                             // 68150:      } // switch Inst[21]
                                             // 68150:     }
  4, 21,                                     // 68155:     case 0x4: {
  OPC_SwitchField, 16, 6,                    // 68157:      switch Inst[21:16] {
  32, 7,                                     // 68160:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68162:       check predicate 76
  OPC_Decode, 218, 17, 230, 3,               // 68164:       decode to CMEQv4i32rz using decoder 486
                                             // 68164:      }
  33, 0,                                     // 68169:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68171:       check predicate 76
  OPC_Decode, 233, 31, 230, 3,               // 68173:       decode to FRINTZv4f32 using decoder 486
                                             // 68173:      }
                                             // 68173:      } // switch Inst[21:16]
                                             // 68173:     }
  5, 30,                                     // 68178:     case 0x5: {
  OPC_SwitchField, 16, 6,                    // 68180:      switch Inst[21:16] {
  32, 7,                                     // 68183:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68185:       check predicate 76
  OPC_Decode, 251, 12, 230, 3,               // 68187:       decode to ABSv4i32 using decoder 486
                                             // 68187:      }
  33, 7,                                     // 68192:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68194:       check predicate 76
  OPC_Decode, 166, 25, 230, 3,               // 68196:       decode to FCVTZSv4f32 using decoder 486
                                             // 68196:      }
  49, 0,                                     // 68201:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 68203:       check predicate 76
  OPC_Decode, 189, 13, 252, 3,               // 68205:       decode to ADDVv4i32v using decoder 508
                                             // 68205:      }
                                             // 68205:      } // switch Inst[21:16]
                                             // 68205:     }
  6, 21,                                     // 68210:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 68212:      switch Inst[21:16] {
  32, 7,                                     // 68215:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68217:       check predicate 76
  OPC_Decode, 212, 22, 230, 3,               // 68219:       decode to FCMEQv4i32rz using decoder 486
                                             // 68219:      }
  33, 0,                                     // 68224:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68226:       check predicate 76
  OPC_Decode, 199, 30, 230, 3,               // 68228:       decode to FRECPEv4f32 using decoder 486
                                             // 68228:      }
                                             // 68228:      } // switch Inst[21:16]
                                             // 68228:     }
  7, 0,                                      // 68233:     case 0x7: {
  OPC_SwitchField, 16, 6,                    // 68235:      switch Inst[21:16] {
  32, 7,                                     // 68238:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68240:       check predicate 76
  OPC_Decode, 203, 21, 230, 3,               // 68242:       decode to FABSv4f32 using decoder 486
                                             // 68242:      }
  48, 0,                                     // 68247:      case 0x30: {
  OPC_CheckPredicate, 81,                    // 68249:       check predicate 81
  OPC_Decode, 215, 27, 244, 3,               // 68251:       decode to FMINVv8i16v using decoder 500
                                             // 68251:      }
                                             // 68251:      } // switch Inst[21:16]
                                             // 68251:     }
                                             // 68251:     } // switch Inst[15:13]
                                             // 68251:    }
  7, 0,                                      // 68256:    case 0x7: {
  OPC_SwitchField, 13, 3,                    // 68258:     switch Inst[15:13] {
  0, 11,                                     // 68261:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68263:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68265:      check Inst[21] == 0x1
  OPC_Decode, 215, 43, 225, 3,               // 68269:      decode to ORRv16i8 using decoder 481
                                             // 68269:     }
  1, 11,                                     // 68274:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 68276:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68278:      check Inst[21] == 0x1
  OPC_Decode, 233, 17, 225, 3,               // 68282:      decode to CMGEv4i32 using decoder 481
                                             // 68282:     }
  2, 11,                                     // 68287:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 68289:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68291:      check Inst[21] == 0x1
  OPC_Decode, 170, 53, 225, 3,               // 68295:      decode to SQRSHLv4i32 using decoder 481
                                             // 68295:     }
  3, 11,                                     // 68300:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 68302:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68304:      check Inst[21] == 0x1
  OPC_Decode, 151, 46, 223, 3,               // 68308:      decode to SABAv4i32 using decoder 479
                                             // 68308:     }
  4, 21,                                     // 68313:     case 0x4: {
  OPC_SwitchField, 21, 1,                    // 68315:      switch Inst[21] {
  0, 7,                                      // 68318:      case 0x0: {
  OPC_CheckPredicate, 97,                    // 68320:       check predicate 97
  OPC_Decode, 153, 68, 223, 3,               // 68322:       decode to USDOTv16i8 using decoder 479
                                             // 68322:      }
  1, 0,                                      // 68327:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68329:       check predicate 76
  OPC_Decode, 148, 43, 225, 3,               // 68331:       decode to MULv4i32 using decoder 481
                                             // 68331:      }
                                             // 68331:      } // switch Inst[21]
                                             // 68331:     }
  5, 11,                                     // 68336:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 68338:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68340:      check Inst[21] == 0x1
  OPC_Decode, 160, 13, 225, 3,               // 68344:      decode to ADDPv4i32 using decoder 481
                                             // 68344:     }
  6, 11,                                     // 68349:     case 0x6: {
  OPC_CheckPredicate, 98,                    // 68351:      check predicate 98
  OPC_CheckField, 21, 1, 1,                  // 68353:      check Inst[21] == 0x1
  OPC_Decode, 154, 22, 225, 3,               // 68357:      decode to FAMAXv4f32 using decoder 481
                                             // 68357:     }
  7, 0,                                      // 68362:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 68364:      switch Inst[21] {
  0, 7,                                      // 68367:      case 0x0: {
  OPC_CheckPredicate, 99,                    // 68369:       check predicate 99
  OPC_Decode, 159, 26, 223, 3,               // 68371:       decode to FDOTv8f16_v4f32 using decoder 479
                                             // 68371:      }
  1, 0,                                      // 68376:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68378:       check predicate 76
  OPC_Decode, 255, 31, 225, 3,               // 68380:       decode to FRSQRTSv4f32 using decoder 481
                                             // 68380:      }
                                             // 68380:      } // switch Inst[21]
                                             // 68380:     }
                                             // 68380:     } // switch Inst[15:13]
                                             // 68380:    }
                                             // 68380:    } // switch Inst[12:10]
                                             // 68380:   }
  3, 188, 7,                                 // 68385:   case 0x3: {
  OPC_SwitchField, 13, 3,                    // 68388:    switch Inst[15:13] {
  0, 81,                                     // 68391:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 68393:     switch Inst[12:10] {
  0, 11,                                     // 68396:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68398:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68400:      check Inst[21] == 0x1
  OPC_Decode, 209, 62, 225, 3,               // 68404:      decode to UADDLv4i32_v2i64 using decoder 481
                                             // 68404:     }
  1, 11,                                     // 68409:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 68411:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68413:      check Inst[21] == 0x1
  OPC_Decode, 212, 63, 225, 3,               // 68417:      decode to UHADDv4i32 using decoder 481
                                             // 68417:     }
  3, 11,                                     // 68422:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 68424:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68426:      check Inst[21] == 0x1
  OPC_Decode, 138, 66, 225, 3,               // 68430:      decode to UQADDv4i32 using decoder 481
                                             // 68430:     }
  4, 11,                                     // 68435:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 68437:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68439:      check Inst[21] == 0x1
  OPC_Decode, 225, 62, 225, 3,               // 68443:      decode to UADDWv4i32_v2i64 using decoder 481
                                             // 68443:     }
  5, 11,                                     // 68448:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 68450:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68452:      check Inst[21] == 0x1
  OPC_Decode, 208, 67, 225, 3,               // 68456:      decode to URHADDv4i32 using decoder 481
                                             // 68456:     }
  7, 0,                                      // 68461:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 68463:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68465:      check Inst[21] == 0x1
  OPC_Decode, 152, 16, 223, 3,               // 68469:      decode to BITv16i8 using decoder 479
                                             // 68469:     }
                                             // 68469:     } // switch Inst[12:10]
                                             // 68469:    }
  1, 136, 1,                                 // 68474:    case 0x1: {
  OPC_SwitchField, 10, 3,                    // 68477:     switch Inst[12:10] {
  0, 11,                                     // 68480:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68482:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68484:      check Inst[21] == 0x1
  OPC_Decode, 250, 68, 225, 3,               // 68488:      decode to USUBLv4i32_v2i64 using decoder 481
                                             // 68488:     }
  1, 11,                                     // 68493:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 68495:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68497:      check Inst[21] == 0x1
  OPC_Decode, 226, 63, 225, 3,               // 68501:      decode to UHSUBv4i32 using decoder 481
                                             // 68501:     }
  2, 21,                                     // 68506:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 68508:      switch Inst[21:16] {
  32, 7,                                     // 68511:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68513:       check predicate 76
  OPC_Decode, 195, 62, 230, 3,               // 68515:       decode to UADDLPv4i32_v2i64 using decoder 486
                                             // 68515:      }
  33, 0,                                     // 68520:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68522:       check predicate 76
  OPC_Decode, 226, 54, 238, 3,               // 68524:       decode to SQXTUNv4i32 using decoder 494
                                             // 68524:      }
                                             // 68524:      } // switch Inst[21:16]
                                             // 68524:     }
  3, 11,                                     // 68529:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 68531:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68533:      check Inst[21] == 0x1
  OPC_Decode, 179, 67, 225, 3,               // 68537:      decode to UQSUBv4i32 using decoder 481
                                             // 68537:     }
  4, 11,                                     // 68542:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 68544:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68546:      check Inst[21] == 0x1
  OPC_Decode, 134, 69, 225, 3,               // 68550:      decode to USUBWv4i32_v2i64 using decoder 481
                                             // 68550:     }
  5, 11,                                     // 68555:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 68557:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68559:      check Inst[21] == 0x1
  OPC_Decode, 132, 18, 225, 3,               // 68563:      decode to CMHIv4i32 using decoder 481
                                             // 68563:     }
  6, 30,                                     // 68568:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 68570:      switch Inst[21:16] {
  32, 7,                                     // 68573:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68575:       check predicate 76
  OPC_Decode, 225, 68, 238, 3,               // 68577:       decode to USQADDv4i32 using decoder 494
                                             // 68577:      }
  33, 7,                                     // 68582:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68584:       check predicate 76
  OPC_Decode, 158, 48, 230, 3,               // 68586:       decode to SHLLv4i32 using decoder 486
                                             // 68586:      }
  48, 0,                                     // 68591:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 68593:       check predicate 76
  OPC_Decode, 203, 62, 206, 3,               // 68595:       decode to UADDLVv4i32v using decoder 462
                                             // 68595:      }
                                             // 68595:      } // switch Inst[21:16]
                                             // 68595:     }
  7, 0,                                      // 68600:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 68602:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68604:      check Inst[21] == 0x1
  OPC_Decode, 140, 18, 225, 3,               // 68608:      decode to CMHSv4i32 using decoder 481
                                             // 68608:     }
                                             // 68608:     } // switch Inst[12:10]
                                             // 68608:    }
  2, 104,                                    // 68613:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 68615:     switch Inst[12:10] {
  0, 11,                                     // 68618:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68620:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68622:      check Inst[21] == 0x1
  OPC_Decode, 223, 44, 223, 3,               // 68626:      decode to RADDHNv2i64_v4i32 using decoder 479
                                             // 68626:     }
  1, 11,                                     // 68631:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 68633:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68635:      check Inst[21] == 0x1
  OPC_Decode, 172, 68, 225, 3,               // 68639:      decode to USHLv4i32 using decoder 481
                                             // 68639:     }
  2, 21,                                     // 68644:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 68646:      switch Inst[21:16] {
  32, 7,                                     // 68649:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68651:       check predicate 76
  OPC_Decode, 204, 17, 230, 3,               // 68653:       decode to CLZv4i32 using decoder 486
                                             // 68653:      }
  33, 0,                                     // 68658:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68660:       check predicate 76
  OPC_Decode, 194, 67, 238, 3,               // 68662:       decode to UQXTNv4i32 using decoder 494
                                             // 68662:      }
                                             // 68662:      } // switch Inst[21:16]
                                             // 68662:     }
  3, 11,                                     // 68667:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 68669:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68671:      check Inst[21] == 0x1
  OPC_Decode, 132, 67, 225, 3,               // 68675:      decode to UQSHLv4i32 using decoder 481
                                             // 68675:     }
  4, 11,                                     // 68680:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 68682:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68684:      check Inst[21] == 0x1
  OPC_Decode, 145, 62, 223, 3,               // 68688:      decode to UABALv4i32_v2i64 using decoder 479
                                             // 68688:     }
  5, 11,                                     // 68693:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 68695:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68697:      check Inst[21] == 0x1
  OPC_Decode, 240, 67, 225, 3,               // 68701:      decode to URSHLv4i32 using decoder 481
                                             // 68701:     }
  7, 0,                                      // 68706:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 68708:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68710:      check Inst[21] == 0x1
  OPC_Decode, 207, 66, 225, 3,               // 68714:      decode to UQRSHLv4i32 using decoder 481
                                             // 68714:     }
                                             // 68714:     } // switch Inst[12:10]
                                             // 68714:    }
  3, 117,                                    // 68719:    case 0x3: {
  OPC_SwitchField, 10, 3,                    // 68721:     switch Inst[12:10] {
  0, 11,                                     // 68724:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68726:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68728:      check Inst[21] == 0x1
  OPC_Decode, 252, 45, 223, 3,               // 68732:      decode to RSUBHNv2i64_v4i32 using decoder 479
                                             // 68732:     }
  1, 11,                                     // 68737:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 68739:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68741:      check Inst[21] == 0x1
  OPC_Decode, 156, 64, 225, 3,               // 68745:      decode to UMAXv4i32 using decoder 481
                                             // 68745:     }
  2, 11,                                     // 68750:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 68752:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 68754:      check Inst[21:16] == 0x20
  OPC_Decode, 186, 62, 238, 3,               // 68758:      decode to UADALPv4i32_v2i64 using decoder 494
                                             // 68758:     }
  3, 11,                                     // 68763:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 68765:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68767:      check Inst[21] == 0x1
  OPC_Decode, 213, 64, 225, 3,               // 68771:      decode to UMINv4i32 using decoder 481
                                             // 68771:     }
  4, 11,                                     // 68776:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 68778:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68780:      check Inst[21] == 0x1
  OPC_Decode, 167, 62, 225, 3,               // 68784:      decode to UABDLv4i32_v2i64 using decoder 481
                                             // 68784:     }
  5, 11,                                     // 68789:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 68791:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68793:      check Inst[21] == 0x1
  OPC_Decode, 177, 62, 225, 3,               // 68797:      decode to UABDv4i32 using decoder 481
                                             // 68797:     }
  6, 21,                                     // 68802:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 68804:      switch Inst[21:16] {
  32, 7,                                     // 68807:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68809:       check predicate 76
  OPC_Decode, 216, 52, 230, 3,               // 68811:       decode to SQNEGv4i32 using decoder 486
                                             // 68811:      }
  33, 0,                                     // 68816:      case 0x21: {
  OPC_CheckPredicate, 84,                    // 68818:       check predicate 84
  OPC_Decode, 238, 14, 230, 3,               // 68820:       decode to BF1CVTL2 using decoder 486
                                             // 68820:      }
                                             // 68820:      } // switch Inst[21:16]
                                             // 68820:     }
  7, 0,                                      // 68825:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 68827:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68829:      check Inst[21] == 0x1
  OPC_Decode, 155, 62, 223, 3,               // 68833:      decode to UABAv4i32 using decoder 479
                                             // 68833:     }
                                             // 68833:     } // switch Inst[12:10]
                                             // 68833:    }
  4, 121,                                    // 68838:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 68840:     switch Inst[12:10] {
  0, 11,                                     // 68843:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68845:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68847:      check Inst[21] == 0x1
  OPC_Decode, 128, 65, 223, 3,               // 68851:      decode to UMLALv4i32_v2i64 using decoder 479
                                             // 68851:     }
  1, 21,                                     // 68856:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 68858:      switch Inst[21] {
  0, 7,                                      // 68861:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 68863:       check predicate 90
  OPC_Decode, 240, 52, 223, 3,               // 68865:       decode to SQRDMLAHv4i32 using decoder 479
                                             // 68865:      }
  1, 0,                                      // 68870:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68872:       check predicate 76
  OPC_Decode, 197, 60, 225, 3,               // 68874:       decode to SUBv4i32 using decoder 481
                                             // 68874:      }
                                             // 68874:      } // switch Inst[21]
                                             // 68874:     }
  2, 11,                                     // 68879:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 68881:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 68883:      check Inst[21:16] == 0x20
  OPC_Decode, 234, 17, 230, 3,               // 68887:      decode to CMGEv4i32rz using decoder 486
                                             // 68887:     }
  3, 21,                                     // 68892:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 68894:      switch Inst[21] {
  0, 7,                                      // 68897:      case 0x0: {
  OPC_CheckPredicate, 90,                    // 68899:       check predicate 90
  OPC_Decode, 131, 53, 223, 3,               // 68901:       decode to SQRDMLSHv4i32 using decoder 479
                                             // 68901:      }
  1, 0,                                      // 68906:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68908:       check predicate 76
  OPC_Decode, 217, 17, 225, 3,               // 68910:       decode to CMEQv4i32 using decoder 481
                                             // 68910:      }
                                             // 68910:      } // switch Inst[21]
                                             // 68910:     }
  5, 21,                                     // 68915:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 68917:      switch Inst[21] {
  0, 7,                                      // 68920:      case 0x0: {
  OPC_CheckPredicate, 96,                    // 68922:       check predicate 96
  OPC_Decode, 203, 63, 223, 3,               // 68924:       decode to UDOTv16i8 using decoder 479
                                             // 68924:      }
  1, 0,                                      // 68929:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68931:       check predicate 76
  OPC_Decode, 141, 42, 223, 3,               // 68933:       decode to MLSv4i32 using decoder 479
                                             // 68933:      }
                                             // 68933:      } // switch Inst[21]
                                             // 68933:     }
  6, 0,                                      // 68938:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 68940:      switch Inst[21:16] {
  32, 7,                                     // 68943:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 68945:       check predicate 76
  OPC_Decode, 154, 18, 230, 3,               // 68947:       decode to CMLEv4i32rz using decoder 486
                                             // 68947:      }
  33, 0,                                     // 68952:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 68954:       check predicate 76
  OPC_Decode, 157, 31, 230, 3,               // 68956:       decode to FRINTIv4f32 using decoder 486
                                             // 68956:      }
                                             // 68956:      } // switch Inst[21:16]
                                             // 68956:     }
                                             // 68956:     } // switch Inst[12:10]
                                             // 68956:    }
  5, 120,                                    // 68961:    case 0x5: {
  OPC_SwitchField, 10, 3,                    // 68963:     switch Inst[12:10] {
  0, 11,                                     // 68966:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 68968:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 68970:      check Inst[21] == 0x1
  OPC_Decode, 172, 65, 223, 3,               // 68974:      decode to UMLSLv4i32_v2i64 using decoder 479
                                             // 68974:     }
  1, 21,                                     // 68979:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 68981:      switch Inst[21] {
  0, 7,                                      // 68984:      case 0x0: {
  OPC_CheckPredicate, 97,                    // 68986:       check predicate 97
  OPC_Decode, 176, 65, 223, 3,               // 68988:       decode to UMMLA using decoder 479
                                             // 68988:      }
  1, 0,                                      // 68993:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 68995:       check predicate 76
  OPC_Decode, 237, 63, 225, 3,               // 68997:       decode to UMAXPv4i32 using decoder 481
                                             // 68997:      }
                                             // 68997:      } // switch Inst[21]
                                             // 68997:     }
  2, 30,                                     // 69002:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 69004:      switch Inst[21:16] {
  33, 7,                                     // 69007:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 69009:       check predicate 76
  OPC_Decode, 237, 24, 230, 3,               // 69011:       decode to FCVTPUv4f32 using decoder 486
                                             // 69011:      }
  48, 7,                                     // 69016:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 69018:       check predicate 76
  OPC_Decode, 250, 63, 252, 3,               // 69020:       decode to UMAXVv4i32v using decoder 508
                                             // 69020:      }
  49, 0,                                     // 69025:      case 0x31: {
  OPC_CheckPredicate, 76,                    // 69027:       check predicate 76
  OPC_Decode, 179, 64, 252, 3,               // 69029:       decode to UMINVv4i32v using decoder 508
                                             // 69029:      }
                                             // 69029:      } // switch Inst[21:16]
                                             // 69029:     }
  3, 11,                                     // 69034:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 69036:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69038:      check Inst[21] == 0x1
  OPC_Decode, 166, 64, 225, 3,               // 69042:      decode to UMINPv4i32 using decoder 481
                                             // 69042:     }
  5, 11,                                     // 69047:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 69049:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69051:      check Inst[21] == 0x1
  OPC_Decode, 150, 53, 225, 3,               // 69055:      decode to SQRDMULHv4i32 using decoder 481
                                             // 69055:     }
  6, 0,                                      // 69060:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 69062:      switch Inst[21:16] {
  32, 7,                                     // 69065:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 69067:       check predicate 76
  OPC_Decode, 175, 43, 230, 3,               // 69069:       decode to NEGv4i32 using decoder 486
                                             // 69069:      }
  33, 0,                                     // 69074:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 69076:       check predicate 76
  OPC_Decode, 217, 25, 230, 3,               // 69078:       decode to FCVTZUv4f32 using decoder 486
                                             // 69078:      }
                                             // 69078:      } // switch Inst[21:16]
                                             // 69078:     }
                                             // 69078:     } // switch Inst[12:10]
                                             // 69078:    }
  6, 131, 1,                                 // 69083:    case 0x6: {
  OPC_SwitchField, 10, 1,                    // 69086:     switch Inst[10] {
  0, 71,                                     // 69089:     case 0x0: {
  OPC_SwitchField, 11, 2,                    // 69091:      switch Inst[12:11] {
  0, 11,                                     // 69094:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 69096:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69098:       check Inst[21] == 0x1
  OPC_Decode, 242, 65, 225, 3,               // 69102:       decode to UMULLv4i32_v2i64 using decoder 481
                                             // 69102:      }
  1, 30,                                     // 69107:      case 0x1: {
  OPC_SwitchField, 16, 6,                    // 69109:       switch Inst[21:16] {
  32, 7,                                     // 69112:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 69114:        check predicate 76
  OPC_Decode, 234, 22, 230, 3,               // 69116:        decode to FCMGEv4i32rz using decoder 486
                                             // 69116:       }
  33, 7,                                     // 69121:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 69123:        check predicate 76
  OPC_Decode, 130, 68, 230, 3,               // 69125:        decode to URSQRTEv4i32 using decoder 486
                                             // 69125:       }
  48, 0,                                     // 69130:       case 0x30: {
  OPC_CheckPredicate, 76,                    // 69132:        check predicate 76
  OPC_Decode, 170, 27, 252, 3,               // 69134:        decode to FMINNMVv4i32v using decoder 508
                                             // 69134:       }
                                             // 69134:       } // switch Inst[21:16]
                                             // 69134:      }
  3, 0,                                      // 69139:      case 0x3: {
  OPC_SwitchField, 16, 6,                    // 69141:       switch Inst[21:16] {
  32, 7,                                     // 69144:       case 0x20: {
  OPC_CheckPredicate, 76,                    // 69146:        check predicate 76
  OPC_Decode, 153, 23, 230, 3,               // 69148:        decode to FCMLEv4i32rz using decoder 486
                                             // 69148:       }
  33, 0,                                     // 69153:       case 0x21: {
  OPC_CheckPredicate, 76,                    // 69155:        check predicate 76
  OPC_Decode, 244, 31, 230, 3,               // 69157:        decode to FRSQRTEv4f32 using decoder 486
                                             // 69157:       }
                                             // 69157:       } // switch Inst[21:16]
                                             // 69157:      }
                                             // 69157:      } // switch Inst[12:11]
                                             // 69157:     }
  1, 0,                                      // 69162:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 69164:      switch Inst[21] {
  0, 7,                                      // 69167:      case 0x0: {
  OPC_CheckPredicate, 100,                   // 69169:       check predicate 100
  OPC_Decode, 140, 23, 135, 4,               // 69171:       decode to FCMLAv4f32 using decoder 519
                                             // 69171:      }
  1, 0,                                      // 69176:      case 0x1: {
  OPC_SwitchField, 11, 2,                    // 69178:       switch Inst[12:11] {
  0, 7,                                      // 69181:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 69183:        check predicate 76
  OPC_Decode, 160, 27, 225, 3,               // 69185:        decode to FMINNMPv4f32 using decoder 481
                                             // 69185:       }
  1, 7,                                      // 69190:       case 0x1: {
  OPC_CheckPredicate, 83,                    // 69192:        check predicate 83
  OPC_Decode, 212, 28, 223, 3,               // 69194:        decode to FMLSL2v8f16 using decoder 479
                                             // 69194:       }
  2, 7,                                      // 69199:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 69201:        check predicate 76
  OPC_Decode, 189, 21, 225, 3,               // 69203:        decode to FABDv4f32 using decoder 481
                                             // 69203:       }
  3, 0,                                      // 69208:       case 0x3: {
  OPC_CheckPredicate, 98,                    // 69210:        check predicate 98
  OPC_Decode, 168, 22, 225, 3,               // 69212:        decode to FAMINv4f32 using decoder 481
                                             // 69212:       }
                                             // 69212:       } // switch Inst[12:11]
                                             // 69212:      }
                                             // 69212:      } // switch Inst[21]
                                             // 69212:     }
                                             // 69212:     } // switch Inst[10]
                                             // 69212:    }
  7, 0,                                      // 69217:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 69219:     switch Inst[11:10] {
  1, 35,                                     // 69222:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 69224:      switch Inst[21] {
  0, 7,                                      // 69227:      case 0x0: {
  OPC_CheckPredicate, 100,                   // 69229:       check predicate 100
  OPC_Decode, 176, 22, 136, 4,               // 69231:       decode to FCADDv4f32 using decoder 520
                                             // 69231:      }
  1, 0,                                      // 69236:      case 0x1: {
  OPC_SwitchField, 12, 1,                    // 69238:       switch Inst[12] {
  0, 7,                                      // 69241:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 69243:        check predicate 76
  OPC_Decode, 254, 22, 225, 3,               // 69245:        decode to FCMGTv4f32 using decoder 481
                                             // 69245:       }
  1, 0,                                      // 69250:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 69252:        check predicate 76
  OPC_Decode, 204, 27, 225, 3,               // 69254:        decode to FMINPv4f32 using decoder 481
                                             // 69254:       }
                                             // 69254:       } // switch Inst[12]
                                             // 69254:      }
                                             // 69254:      } // switch Inst[21]
                                             // 69254:     }
  2, 42,                                     // 69259:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 69261:      switch Inst[21:16] {
  32, 11,                                    // 69264:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 69266:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 69268:       check Inst[12] == 0x1
  OPC_Decode, 167, 30, 230, 3,               // 69272:       decode to FNEGv4f32 using decoder 486
                                             // 69272:      }
  33, 11,                                    // 69277:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 69279:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 69281:       check Inst[12] == 0x1
  OPC_Decode, 161, 32, 230, 3,               // 69285:       decode to FSQRTv4f32 using decoder 486
                                             // 69285:      }
  48, 0,                                     // 69290:      case 0x30: {
  OPC_CheckPredicate, 76,                    // 69292:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 69294:       check Inst[12] == 0x1
  OPC_Decode, 214, 27, 252, 3,               // 69298:       decode to FMINVv4i32v using decoder 508
                                             // 69298:      }
                                             // 69298:      } // switch Inst[21:16]
                                             // 69298:     }
  3, 0,                                      // 69303:     case 0x3: {
  OPC_SwitchField, 12, 1,                    // 69305:      switch Inst[12] {
  0, 21,                                     // 69308:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 69310:       switch Inst[21] {
  0, 7,                                      // 69313:       case 0x0: {
  OPC_CheckPredicate, 101,                   // 69315:        check predicate 101
  OPC_Decode, 143, 29, 223, 3,               // 69317:        decode to FMMLAv4f32 using decoder 479
                                             // 69317:       }
  1, 0,                                      // 69322:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 69324:        check predicate 76
  OPC_Decode, 225, 21, 225, 3,               // 69326:        decode to FACGTv4f32 using decoder 481
                                             // 69326:       }
                                             // 69326:       } // switch Inst[21]
                                             // 69326:      }
  1, 0,                                      // 69331:      case 0x1: {
  OPC_CheckPredicate, 84,                    // 69333:       check predicate 84
  OPC_CheckField, 21, 1, 1,                  // 69335:       check Inst[21] == 0x1
  OPC_Decode, 147, 32, 225, 3,               // 69339:       decode to FSCALEv4f32 using decoder 481
                                             // 69339:      }
                                             // 69339:      } // switch Inst[12]
                                             // 69339:     }
                                             // 69339:     } // switch Inst[11:10]
                                             // 69339:    }
                                             // 69339:    } // switch Inst[15:13]
                                             // 69339:   }
  6, 0,                                      // 69344:   case 0x6: {
  OPC_CheckPredicate, 88,                    // 69346:    check predicate 88
  OPC_CheckField, 21, 1, 0,                  // 69348:    check Inst[21] == 0x0
  OPC_Decode, 229, 70, 139, 4,               // 69352:    decode to XAR using decoder 523
                                             // 69352:   }
                                             // 69352:   } // switch Inst[31:29]
                                             // 69352:  }
  11, 221, 14,                               // 69357:  case 0xb: {
  OPC_SwitchField, 29, 3,                    // 69360:   switch Inst[31:29] {
  0, 231, 1,                                 // 69363:   case 0x0: {
  OPC_SwitchField, 10, 6,                    // 69366:    switch Inst[15:10] {
  1, 11,                                     // 69369:    case 0x1: {
  OPC_CheckPredicate, 81,                    // 69371:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69373:     check Inst[21] == 0x0
  OPC_Decode, 192, 27, 200, 3,               // 69377:     decode to FMINNMv4f16 using decoder 456
                                             // 69377:    }
  3, 11,                                     // 69382:    case 0x3: {
  OPC_CheckPredicate, 81,                    // 69384:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69386:     check Inst[21] == 0x0
  OPC_Decode, 132, 29, 221, 3,               // 69390:     decode to FMLSv4f16 using decoder 477
                                             // 69390:    }
  5, 11,                                     // 69395:    case 0x5: {
  OPC_CheckPredicate, 81,                    // 69397:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69399:     check Inst[21] == 0x0
  OPC_Decode, 189, 32, 200, 3,               // 69403:     decode to FSUBv4f16 using decoder 456
                                             // 69403:    }
  7, 21,                                     // 69408:    case 0x7: {
  OPC_SwitchField, 21, 1,                    // 69410:     switch Inst[21] {
  0, 7,                                      // 69413:     case 0x0: {
  OPC_CheckPredicate, 98,                    // 69415:      check predicate 98
  OPC_Decode, 153, 22, 200, 3,               // 69417:      decode to FAMAXv4f16 using decoder 456
                                             // 69417:     }
  1, 0,                                      // 69422:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 69424:      check predicate 76
  OPC_Decode, 198, 43, 200, 3,               // 69426:      decode to ORNv8i8 using decoder 456
                                             // 69426:     }
                                             // 69426:     } // switch Inst[21]
                                             // 69426:    }
  13, 11,                                    // 69431:    case 0xd: {
  OPC_CheckPredicate, 81,                    // 69433:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69435:     check Inst[21] == 0x0
  OPC_Decode, 236, 27, 200, 3,               // 69439:     decode to FMINv4f16 using decoder 456
                                             // 69439:    }
  15, 11,                                    // 69444:    case 0xf: {
  OPC_CheckPredicate, 81,                    // 69446:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69448:     check Inst[21] == 0x0
  OPC_Decode, 254, 31, 200, 3,               // 69452:     decode to FRSQRTSv4f16 using decoder 456
                                             // 69452:    }
  34, 11,                                    // 69457:    case 0x22: {
  OPC_CheckPredicate, 81,                    // 69459:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69461:     check Inst[21:16] == 0x39
  OPC_Decode, 204, 31, 201, 3,               // 69465:     decode to FRINTPv4f16 using decoder 457
                                             // 69465:    }
  38, 11,                                    // 69470:    case 0x26: {
  OPC_CheckPredicate, 81,                    // 69472:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69474:     check Inst[21:16] == 0x39
  OPC_Decode, 232, 31, 201, 3,               // 69478:     decode to FRINTZv4f16 using decoder 457
                                             // 69478:    }
  42, 11,                                    // 69483:    case 0x2a: {
  OPC_CheckPredicate, 81,                    // 69485:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69487:     check Inst[21:16] == 0x39
  OPC_Decode, 218, 24, 201, 3,               // 69491:     decode to FCVTPSv4f16 using decoder 457
                                             // 69491:    }
  46, 11,                                    // 69496:    case 0x2e: {
  OPC_CheckPredicate, 81,                    // 69498:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69500:     check Inst[21:16] == 0x39
  OPC_Decode, 165, 25, 201, 3,               // 69504:     decode to FCVTZSv4f16 using decoder 457
                                             // 69504:    }
  50, 11,                                    // 69509:    case 0x32: {
  OPC_CheckPredicate, 81,                    // 69511:     check predicate 81
  OPC_CheckField, 16, 6, 56,                 // 69513:     check Inst[21:16] == 0x38
  OPC_Decode, 255, 22, 201, 3,               // 69517:     decode to FCMGTv4i16rz using decoder 457
                                             // 69517:    }
  54, 21,                                    // 69522:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 69524:     switch Inst[21:16] {
  56, 7,                                     // 69527:     case 0x38: {
  OPC_CheckPredicate, 81,                    // 69529:      check predicate 81
  OPC_Decode, 211, 22, 201, 3,               // 69531:      decode to FCMEQv4i16rz using decoder 457
                                             // 69531:     }
  57, 0,                                     // 69536:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 69538:      check predicate 81
  OPC_Decode, 198, 30, 201, 3,               // 69540:      decode to FRECPEv4f16 using decoder 457
                                             // 69540:     }
                                             // 69540:     } // switch Inst[21:16]
                                             // 69540:    }
  56, 11,                                    // 69545:    case 0x38: {
  OPC_CheckPredicate, 86,                    // 69547:     check predicate 86
  OPC_CheckField, 21, 1, 1,                  // 69549:     check Inst[21] == 0x1
  OPC_Decode, 146, 44, 196, 3,               // 69553:     decode to PMULLv1i64 using decoder 452
                                             // 69553:    }
  58, 11,                                    // 69558:    case 0x3a: {
  OPC_CheckPredicate, 81,                    // 69560:     check predicate 81
  OPC_CheckField, 16, 6, 56,                 // 69562:     check Inst[21:16] == 0x38
  OPC_Decode, 163, 23, 201, 3,               // 69566:     decode to FCMLTv4i16rz using decoder 457
                                             // 69566:    }
  62, 11,                                    // 69571:    case 0x3e: {
  OPC_CheckPredicate, 81,                    // 69573:     check predicate 81
  OPC_CheckField, 16, 6, 56,                 // 69575:     check Inst[21:16] == 0x38
  OPC_Decode, 202, 21, 201, 3,               // 69579:     decode to FABSv4f16 using decoder 457
                                             // 69579:    }
  63, 0,                                     // 69584:    case 0x3f: {
  OPC_CheckPredicate, 80,                    // 69586:     check predicate 80
  OPC_CheckField, 21, 1, 0,                  // 69588:     check Inst[21] == 0x0
  OPC_Decode, 248, 27, 223, 3,               // 69592:     decode to FMLALBv16i8_v8f16 using decoder 479
                                             // 69592:    }
                                             // 69592:    } // switch Inst[15:10]
                                             // 69592:   }
  1, 228, 1,                                 // 69597:   case 0x1: {
  OPC_SwitchField, 10, 6,                    // 69600:    switch Inst[15:10] {
  1, 11,                                     // 69603:    case 0x1: {
  OPC_CheckPredicate, 81,                    // 69605:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69607:     check Inst[21] == 0x0
  OPC_Decode, 159, 27, 200, 3,               // 69611:     decode to FMINNMPv4f16 using decoder 456
                                             // 69611:    }
  5, 11,                                     // 69616:    case 0x5: {
  OPC_CheckPredicate, 81,                    // 69618:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69620:     check Inst[21] == 0x0
  OPC_Decode, 188, 21, 200, 3,               // 69624:     decode to FABDv4f16 using decoder 456
                                             // 69624:    }
  7, 21,                                     // 69629:    case 0x7: {
  OPC_SwitchField, 21, 1,                    // 69631:     switch Inst[21] {
  0, 7,                                      // 69634:     case 0x0: {
  OPC_CheckPredicate, 98,                    // 69636:      check predicate 98
  OPC_Decode, 167, 22, 200, 3,               // 69638:      decode to FAMINv4f16 using decoder 456
                                             // 69638:     }
  1, 0,                                      // 69643:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 69645:      check predicate 76
  OPC_Decode, 151, 16, 221, 3,               // 69647:      decode to BIFv8i8 using decoder 477
                                             // 69647:     }
                                             // 69647:     } // switch Inst[21]
                                             // 69647:    }
  9, 11,                                     // 69652:    case 0x9: {
  OPC_CheckPredicate, 81,                    // 69654:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69656:     check Inst[21] == 0x0
  OPC_Decode, 253, 22, 200, 3,               // 69660:     decode to FCMGTv4f16 using decoder 456
                                             // 69660:    }
  11, 11,                                    // 69665:    case 0xb: {
  OPC_CheckPredicate, 81,                    // 69667:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69669:     check Inst[21] == 0x0
  OPC_Decode, 224, 21, 200, 3,               // 69673:     decode to FACGTv4f16 using decoder 456
                                             // 69673:    }
  13, 11,                                    // 69678:    case 0xd: {
  OPC_CheckPredicate, 81,                    // 69680:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69682:     check Inst[21] == 0x0
  OPC_Decode, 203, 27, 200, 3,               // 69686:     decode to FMINPv4f16 using decoder 456
                                             // 69686:    }
  15, 11,                                    // 69691:    case 0xf: {
  OPC_CheckPredicate, 84,                    // 69693:     check predicate 84
  OPC_CheckField, 21, 1, 0,                  // 69695:     check Inst[21] == 0x0
  OPC_Decode, 146, 32, 200, 3,               // 69699:     decode to FSCALEv4f16 using decoder 456
                                             // 69699:    }
  30, 11,                                    // 69704:    case 0x1e: {
  OPC_CheckPredicate, 84,                    // 69706:     check predicate 84
  OPC_CheckField, 16, 6, 33,                 // 69708:     check Inst[21:16] == 0x21
  OPC_Decode, 243, 14, 220, 3,               // 69712:     decode to BF2CVTL using decoder 476
                                             // 69712:    }
  38, 11,                                    // 69717:    case 0x26: {
  OPC_CheckPredicate, 81,                    // 69719:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69721:     check Inst[21:16] == 0x39
  OPC_Decode, 156, 31, 201, 3,               // 69725:     decode to FRINTIv4f16 using decoder 457
                                             // 69725:    }
  42, 11,                                    // 69730:    case 0x2a: {
  OPC_CheckPredicate, 81,                    // 69732:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69734:     check Inst[21:16] == 0x39
  OPC_Decode, 236, 24, 201, 3,               // 69738:     decode to FCVTPUv4f16 using decoder 457
                                             // 69738:    }
  46, 11,                                    // 69743:    case 0x2e: {
  OPC_CheckPredicate, 81,                    // 69745:     check predicate 81
  OPC_CheckField, 16, 6, 57,                 // 69747:     check Inst[21:16] == 0x39
  OPC_Decode, 216, 25, 201, 3,               // 69751:     decode to FCVTZUv4f16 using decoder 457
                                             // 69751:    }
  50, 11,                                    // 69756:    case 0x32: {
  OPC_CheckPredicate, 81,                    // 69758:     check predicate 81
  OPC_CheckField, 16, 6, 56,                 // 69760:     check Inst[21:16] == 0x38
  OPC_Decode, 233, 22, 201, 3,               // 69764:     decode to FCMGEv4i16rz using decoder 457
                                             // 69764:    }
  54, 21,                                    // 69769:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 69771:     switch Inst[21:16] {
  56, 7,                                     // 69774:     case 0x38: {
  OPC_CheckPredicate, 81,                    // 69776:      check predicate 81
  OPC_Decode, 152, 23, 201, 3,               // 69778:      decode to FCMLEv4i16rz using decoder 457
                                             // 69778:     }
  57, 0,                                     // 69783:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 69785:      check predicate 81
  OPC_Decode, 243, 31, 201, 3,               // 69787:      decode to FRSQRTEv4f16 using decoder 457
                                             // 69787:     }
                                             // 69787:     } // switch Inst[21:16]
                                             // 69787:    }
  62, 21,                                    // 69792:    case 0x3e: {
  OPC_SwitchField, 16, 6,                    // 69794:     switch Inst[21:16] {
  56, 7,                                     // 69797:     case 0x38: {
  OPC_CheckPredicate, 81,                    // 69799:      check predicate 81
  OPC_Decode, 166, 30, 201, 3,               // 69801:      decode to FNEGv4f16 using decoder 457
                                             // 69801:     }
  57, 0,                                     // 69806:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 69808:      check predicate 81
  OPC_Decode, 160, 32, 201, 3,               // 69810:      decode to FSQRTv4f16 using decoder 457
                                             // 69810:     }
                                             // 69810:     } // switch Inst[21:16]
                                             // 69810:    }
  63, 0,                                     // 69815:    case 0x3f: {
  OPC_CheckPredicate, 92,                    // 69817:     check predicate 92
  OPC_CheckField, 21, 1, 0,                  // 69819:     check Inst[21] == 0x0
  OPC_Decode, 169, 15, 223, 3,               // 69823:     decode to BFMLALB using decoder 479
                                             // 69823:    }
                                             // 69823:    } // switch Inst[15:10]
                                             // 69823:   }
  2, 210, 5,                                 // 69828:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 69831:    switch Inst[11:10] {
  0, 33,                                     // 69834:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 69836:     switch Inst[15] {
  0, 11,                                     // 69839:     case 0x0: {
  OPC_CheckPredicate, 93,                    // 69841:      check predicate 93
  OPC_CheckField, 21, 1, 0,                  // 69843:      check Inst[21] == 0x0
  OPC_Decode, 188, 41, 140, 4,               // 69847:      decode to LUT2_H using decoder 524
                                             // 69847:     }
  1, 0,                                      // 69852:     case 0x1: {
  OPC_CheckPredicate, 86,                    // 69854:      check predicate 86
  OPC_CheckField, 21, 1, 1,                  // 69856:      check Inst[21] == 0x1
  OPC_CheckField, 12, 3, 6,                  // 69860:      check Inst[14:12] == 0x6
  OPC_Decode, 147, 44, 225, 3,               // 69864:      decode to PMULLv2i64 using decoder 481
                                             // 69864:     }
                                             // 69864:     } // switch Inst[15]
                                             // 69864:    }
  1, 130, 1,                                 // 69869:    case 0x1: {
  OPC_SwitchField, 12, 4,                    // 69872:     switch Inst[15:12] {
  0, 11,                                     // 69875:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 69877:      check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69879:      check Inst[21] == 0x0
  OPC_Decode, 194, 27, 225, 3,               // 69883:      decode to FMINNMv8f16 using decoder 481
                                             // 69883:     }
  1, 11,                                     // 69888:     case 0x1: {
  OPC_CheckPredicate, 81,                    // 69890:      check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 69892:      check Inst[21] == 0x0
  OPC_Decode, 191, 32, 225, 3,               // 69896:      decode to FSUBv8f16 using decoder 481
                                             // 69896:     }
  3, 21,                                     // 69901:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 69903:      switch Inst[21] {
  0, 7,                                      // 69906:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 69908:       check predicate 81
  OPC_Decode, 238, 27, 225, 3,               // 69910:       decode to FMINv8f16 using decoder 481
                                             // 69910:      }
  1, 0,                                      // 69915:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 69917:       check predicate 76
  OPC_Decode, 245, 17, 225, 3,               // 69919:       decode to CMGTv2i64 using decoder 481
                                             // 69919:      }
                                             // 69919:      } // switch Inst[21]
                                             // 69919:     }
  4, 11,                                     // 69924:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 69926:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69928:      check Inst[21] == 0x1
  OPC_Decode, 194, 55, 225, 3,               // 69932:      decode to SSHLv2i64 using decoder 481
                                             // 69932:     }
  5, 11,                                     // 69937:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 69939:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69941:      check Inst[21] == 0x1
  OPC_Decode, 150, 55, 225, 3,               // 69945:      decode to SRSHLv2i64 using decoder 481
                                             // 69945:     }
  8, 11,                                     // 69950:     case 0x8: {
  OPC_CheckPredicate, 76,                    // 69952:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69954:      check Inst[21] == 0x1
  OPC_Decode, 236, 13, 225, 3,               // 69958:      decode to ADDv2i64 using decoder 481
                                             // 69958:     }
  12, 11,                                    // 69963:     case 0xc: {
  OPC_CheckPredicate, 76,                    // 69965:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69967:      check Inst[21] == 0x1
  OPC_Decode, 191, 27, 225, 3,               // 69971:      decode to FMINNMv2f64 using decoder 481
                                             // 69971:     }
  13, 11,                                    // 69976:     case 0xd: {
  OPC_CheckPredicate, 76,                    // 69978:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69980:      check Inst[21] == 0x1
  OPC_Decode, 188, 32, 225, 3,               // 69984:      decode to FSUBv2f64 using decoder 481
                                             // 69984:     }
  15, 0,                                     // 69989:     case 0xf: {
  OPC_CheckPredicate, 76,                    // 69991:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 69993:      check Inst[21] == 0x1
  OPC_Decode, 235, 27, 225, 3,               // 69997:      decode to FMINv2f64 using decoder 481
                                             // 69997:     }
                                             // 69997:     } // switch Inst[15:12]
                                             // 69997:    }
  2, 219, 2,                                 // 70002:    case 0x2: {
  OPC_SwitchField, 12, 4,                    // 70005:     switch Inst[15:12] {
  1, 11,                                     // 70008:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 70010:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 70012:      check Inst[21] == 0x0
  OPC_Decode, 178, 69, 225, 3,               // 70016:      decode to UZP1v2i64 using decoder 481
                                             // 70016:     }
  2, 11,                                     // 70021:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 70023:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 70025:      check Inst[21] == 0x0
  OPC_Decode, 239, 61, 225, 3,               // 70029:      decode to TRN1v2i64 using decoder 481
                                             // 70029:     }
  3, 25,                                     // 70034:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 70036:      switch Inst[21] {
  0, 7,                                      // 70039:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 70041:       check predicate 76
  OPC_Decode, 136, 71, 225, 3,               // 70043:       decode to ZIP1v2i64 using decoder 481
                                             // 70043:      }
  1, 0,                                      // 70048:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70050:       check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 70052:       check Inst[20:16] == 0x0
  OPC_Decode, 254, 60, 238, 3,               // 70056:       decode to SUQADDv2i64 using decoder 494
                                             // 70056:      }
                                             // 70056:      } // switch Inst[21]
                                             // 70056:     }
  5, 11,                                     // 70061:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 70063:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 70065:      check Inst[21] == 0x0
  OPC_Decode, 194, 69, 225, 3,               // 70069:      decode to UZP2v2i64 using decoder 481
                                             // 70069:     }
  6, 11,                                     // 70074:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 70076:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 70078:      check Inst[21] == 0x0
  OPC_Decode, 255, 61, 225, 3,               // 70082:      decode to TRN2v2i64 using decoder 481
                                             // 70082:     }
  7, 25,                                     // 70087:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 70089:      switch Inst[21] {
  0, 7,                                      // 70092:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 70094:       check predicate 76
  OPC_Decode, 152, 71, 225, 3,               // 70096:       decode to ZIP2v2i64 using decoder 481
                                             // 70096:      }
  1, 0,                                      // 70101:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70103:       check predicate 76
  OPC_CheckField, 16, 5, 0,                  // 70105:       check Inst[20:16] == 0x0
  OPC_Decode, 133, 51, 230, 3,               // 70109:       decode to SQABSv2i64 using decoder 486
                                             // 70109:      }
                                             // 70109:      } // switch Inst[21]
                                             // 70109:     }
  8, 30,                                     // 70114:     case 0x8: {
  OPC_SwitchField, 16, 6,                    // 70116:      switch Inst[21:16] {
  32, 7,                                     // 70119:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70121:       check predicate 76
  OPC_Decode, 246, 17, 230, 3,               // 70123:       decode to CMGTv2i64rz using decoder 486
                                             // 70123:      }
  33, 7,                                     // 70128:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70130:       check predicate 76
  OPC_Decode, 203, 31, 230, 3,               // 70132:       decode to FRINTPv2f64 using decoder 486
                                             // 70132:      }
  57, 0,                                     // 70137:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 70139:       check predicate 81
  OPC_Decode, 206, 31, 230, 3,               // 70141:       decode to FRINTPv8f16 using decoder 486
                                             // 70141:      }
                                             // 70141:      } // switch Inst[21:16]
                                             // 70141:     }
  9, 30,                                     // 70146:     case 0x9: {
  OPC_SwitchField, 16, 6,                    // 70148:      switch Inst[21:16] {
  32, 7,                                     // 70151:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70153:       check predicate 76
  OPC_Decode, 214, 17, 230, 3,               // 70155:       decode to CMEQv2i64rz using decoder 486
                                             // 70155:      }
  33, 7,                                     // 70160:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70162:       check predicate 76
  OPC_Decode, 231, 31, 230, 3,               // 70164:       decode to FRINTZv2f64 using decoder 486
                                             // 70164:      }
  57, 0,                                     // 70169:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 70171:       check predicate 81
  OPC_Decode, 234, 31, 230, 3,               // 70173:       decode to FRINTZv8f16 using decoder 486
                                             // 70173:      }
                                             // 70173:      } // switch Inst[21:16]
                                             // 70173:     }
  10, 30,                                    // 70178:     case 0xa: {
  OPC_SwitchField, 16, 6,                    // 70180:      switch Inst[21:16] {
  32, 7,                                     // 70183:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70185:       check predicate 76
  OPC_Decode, 160, 18, 230, 3,               // 70187:       decode to CMLTv2i64rz using decoder 486
                                             // 70187:      }
  33, 7,                                     // 70192:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70194:       check predicate 76
  OPC_Decode, 217, 24, 230, 3,               // 70196:       decode to FCVTPSv2f64 using decoder 486
                                             // 70196:      }
  57, 0,                                     // 70201:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 70203:       check predicate 81
  OPC_Decode, 220, 24, 230, 3,               // 70205:       decode to FCVTPSv8f16 using decoder 486
                                             // 70205:      }
                                             // 70205:      } // switch Inst[21:16]
                                             // 70205:     }
  11, 30,                                    // 70210:     case 0xb: {
  OPC_SwitchField, 16, 6,                    // 70212:      switch Inst[21:16] {
  32, 7,                                     // 70215:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70217:       check predicate 76
  OPC_Decode, 249, 12, 230, 3,               // 70219:       decode to ABSv2i64 using decoder 486
                                             // 70219:      }
  33, 7,                                     // 70224:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70226:       check predicate 76
  OPC_Decode, 162, 25, 230, 3,               // 70228:       decode to FCVTZSv2f64 using decoder 486
                                             // 70228:      }
  57, 0,                                     // 70233:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 70235:       check predicate 81
  OPC_Decode, 169, 25, 230, 3,               // 70237:       decode to FCVTZSv8f16 using decoder 486
                                             // 70237:      }
                                             // 70237:      } // switch Inst[21:16]
                                             // 70237:     }
  12, 21,                                    // 70242:     case 0xc: {
  OPC_SwitchField, 16, 6,                    // 70244:      switch Inst[21:16] {
  32, 7,                                     // 70247:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70249:       check predicate 76
  OPC_Decode, 252, 22, 230, 3,               // 70251:       decode to FCMGTv2i64rz using decoder 486
                                             // 70251:      }
  56, 0,                                     // 70256:      case 0x38: {
  OPC_CheckPredicate, 81,                    // 70258:       check predicate 81
  OPC_Decode, 130, 23, 230, 3,               // 70260:       decode to FCMGTv8i16rz using decoder 486
                                             // 70260:      }
                                             // 70260:      } // switch Inst[21:16]
                                             // 70260:     }
  13, 39,                                    // 70265:     case 0xd: {
  OPC_SwitchField, 16, 6,                    // 70267:      switch Inst[21:16] {
  32, 7,                                     // 70270:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70272:       check predicate 76
  OPC_Decode, 208, 22, 230, 3,               // 70274:       decode to FCMEQv2i64rz using decoder 486
                                             // 70274:      }
  33, 7,                                     // 70279:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70281:       check predicate 76
  OPC_Decode, 197, 30, 230, 3,               // 70283:       decode to FRECPEv2f64 using decoder 486
                                             // 70283:      }
  56, 7,                                     // 70288:      case 0x38: {
  OPC_CheckPredicate, 81,                    // 70290:       check predicate 81
  OPC_Decode, 214, 22, 230, 3,               // 70292:       decode to FCMEQv8i16rz using decoder 486
                                             // 70292:      }
  57, 0,                                     // 70297:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 70299:       check predicate 81
  OPC_Decode, 200, 30, 230, 3,               // 70301:       decode to FRECPEv8f16 using decoder 486
                                             // 70301:      }
                                             // 70301:      } // switch Inst[21:16]
                                             // 70301:     }
  14, 21,                                    // 70306:     case 0xe: {
  OPC_SwitchField, 16, 6,                    // 70308:      switch Inst[21:16] {
  32, 7,                                     // 70311:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70313:       check predicate 76
  OPC_Decode, 162, 23, 230, 3,               // 70315:       decode to FCMLTv2i64rz using decoder 486
                                             // 70315:      }
  56, 0,                                     // 70320:      case 0x38: {
  OPC_CheckPredicate, 81,                    // 70322:       check predicate 81
  OPC_Decode, 165, 23, 230, 3,               // 70324:       decode to FCMLTv8i16rz using decoder 486
                                             // 70324:      }
                                             // 70324:      } // switch Inst[21:16]
                                             // 70324:     }
  15, 0,                                     // 70329:     case 0xf: {
  OPC_SwitchField, 16, 6,                    // 70331:      switch Inst[21:16] {
  32, 7,                                     // 70334:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70336:       check predicate 76
  OPC_Decode, 201, 21, 230, 3,               // 70338:       decode to FABSv2f64 using decoder 486
                                             // 70338:      }
  56, 0,                                     // 70343:      case 0x38: {
  OPC_CheckPredicate, 81,                    // 70345:       check predicate 81
  OPC_Decode, 204, 21, 230, 3,               // 70347:       decode to FABSv8f16 using decoder 486
                                             // 70347:      }
                                             // 70347:      } // switch Inst[21:16]
                                             // 70347:     }
                                             // 70347:     } // switch Inst[15:12]
                                             // 70347:    }
  3, 0,                                      // 70352:    case 0x3: {
  OPC_SwitchField, 12, 4,                    // 70354:     switch Inst[15:12] {
  0, 21,                                     // 70357:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 70359:      switch Inst[21] {
  0, 7,                                      // 70362:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 70364:       check predicate 81
  OPC_Decode, 136, 29, 223, 3,               // 70366:       decode to FMLSv8f16 using decoder 479
                                             // 70366:      }
  1, 0,                                      // 70371:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70373:       check predicate 76
  OPC_Decode, 156, 51, 225, 3,               // 70375:       decode to SQADDv2i64 using decoder 481
                                             // 70375:      }
                                             // 70375:      } // switch Inst[21]
                                             // 70375:     }
  1, 21,                                     // 70380:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 70382:      switch Inst[21] {
  0, 7,                                      // 70385:      case 0x0: {
  OPC_CheckPredicate, 98,                    // 70387:       check predicate 98
  OPC_Decode, 155, 22, 225, 3,               // 70389:       decode to FAMAXv8f16 using decoder 481
                                             // 70389:      }
  1, 0,                                      // 70394:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70396:       check predicate 76
  OPC_Decode, 197, 43, 225, 3,               // 70398:       decode to ORNv16i8 using decoder 481
                                             // 70398:      }
                                             // 70398:      } // switch Inst[21]
                                             // 70398:     }
  2, 11,                                     // 70403:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 70405:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70407:      check Inst[21] == 0x1
  OPC_Decode, 194, 54, 225, 3,               // 70411:      decode to SQSUBv2i64 using decoder 481
                                             // 70411:     }
  3, 21,                                     // 70416:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 70418:      switch Inst[21] {
  0, 7,                                      // 70421:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 70423:       check predicate 81
  OPC_Decode, 128, 32, 225, 3,               // 70425:       decode to FRSQRTSv8f16 using decoder 481
                                             // 70425:      }
  1, 0,                                      // 70430:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70432:       check predicate 76
  OPC_Decode, 229, 17, 225, 3,               // 70434:       decode to CMGEv2i64 using decoder 481
                                             // 70434:      }
                                             // 70434:      } // switch Inst[21]
                                             // 70434:     }
  4, 11,                                     // 70439:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 70441:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70443:      check Inst[21] == 0x1
  OPC_Decode, 128, 54, 225, 3,               // 70447:      decode to SQSHLv2i64 using decoder 481
                                             // 70447:     }
  5, 11,                                     // 70452:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 70454:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70456:      check Inst[21] == 0x1
  OPC_Decode, 168, 53, 225, 3,               // 70460:      decode to SQRSHLv2i64 using decoder 481
                                             // 70460:     }
  8, 11,                                     // 70465:     case 0x8: {
  OPC_CheckPredicate, 76,                    // 70467:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70469:      check Inst[21] == 0x1
  OPC_Decode, 134, 19, 225, 3,               // 70473:      decode to CMTSTv2i64 using decoder 481
                                             // 70473:     }
  11, 11,                                    // 70478:     case 0xb: {
  OPC_CheckPredicate, 76,                    // 70480:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70482:      check Inst[21] == 0x1
  OPC_Decode, 157, 13, 225, 3,               // 70486:      decode to ADDPv2i64 using decoder 481
                                             // 70486:     }
  12, 11,                                    // 70491:     case 0xc: {
  OPC_CheckPredicate, 76,                    // 70493:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70495:      check Inst[21] == 0x1
  OPC_Decode, 129, 29, 223, 3,               // 70499:      decode to FMLSv2f64 using decoder 479
                                             // 70499:     }
  13, 11,                                    // 70504:     case 0xd: {
  OPC_CheckPredicate, 98,                    // 70506:      check predicate 98
  OPC_CheckField, 21, 1, 1,                  // 70508:      check Inst[21] == 0x1
  OPC_Decode, 152, 22, 225, 3,               // 70512:      decode to FAMAXv2f64 using decoder 481
                                             // 70512:     }
  14, 11,                                    // 70517:     case 0xe: {
  OPC_CheckPredicate, 102,                   // 70519:      check predicate 102
  OPC_CheckField, 21, 1, 0,                  // 70521:      check Inst[21] == 0x0
  OPC_Decode, 146, 29, 223, 3,               // 70525:      decode to FMMLAv8f16_v8f16 using decoder 479
                                             // 70525:     }
  15, 0,                                     // 70530:     case 0xf: {
  OPC_SwitchField, 21, 1,                    // 70532:      switch Inst[21] {
  0, 7,                                      // 70535:      case 0x0: {
  OPC_CheckPredicate, 80,                    // 70537:       check predicate 80
  OPC_Decode, 150, 28, 223, 3,               // 70539:       decode to FMLALTv16i8_v8f16 using decoder 479
                                             // 70539:      }
  1, 0,                                      // 70544:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70546:       check predicate 76
  OPC_Decode, 253, 31, 225, 3,               // 70548:       decode to FRSQRTSv2f64 using decoder 481
                                             // 70548:      }
                                             // 70548:      } // switch Inst[21]
                                             // 70548:     }
                                             // 70548:     } // switch Inst[15:12]
                                             // 70548:    }
                                             // 70548:    } // switch Inst[11:10]
                                             // 70548:   }
  3, 154, 5,                                 // 70553:   case 0x3: {
  OPC_SwitchField, 13, 3,                    // 70556:    switch Inst[15:13] {
  0, 65,                                     // 70559:    case 0x0: {
  OPC_SwitchField, 10, 3,                    // 70561:     switch Inst[12:10] {
  1, 11,                                     // 70564:     case 0x1: {
  OPC_CheckPredicate, 81,                    // 70566:      check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 70568:      check Inst[21] == 0x0
  OPC_Decode, 161, 27, 225, 3,               // 70572:      decode to FMINNMPv8f16 using decoder 481
                                             // 70572:     }
  3, 11,                                     // 70577:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 70579:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70581:      check Inst[21] == 0x1
  OPC_Decode, 136, 66, 225, 3,               // 70585:      decode to UQADDv2i64 using decoder 481
                                             // 70585:     }
  5, 11,                                     // 70590:     case 0x5: {
  OPC_CheckPredicate, 81,                    // 70592:      check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 70594:      check Inst[21] == 0x0
  OPC_Decode, 190, 21, 225, 3,               // 70598:      decode to FABDv8f16 using decoder 481
                                             // 70598:     }
  7, 0,                                      // 70603:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 70605:      switch Inst[21] {
  0, 7,                                      // 70608:      case 0x0: {
  OPC_CheckPredicate, 98,                    // 70610:       check predicate 98
  OPC_Decode, 169, 22, 225, 3,               // 70612:       decode to FAMINv8f16 using decoder 481
                                             // 70612:      }
  1, 0,                                      // 70617:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70619:       check predicate 76
  OPC_Decode, 150, 16, 223, 3,               // 70621:       decode to BIFv16i8 using decoder 479
                                             // 70621:      }
                                             // 70621:      } // switch Inst[21]
                                             // 70621:     }
                                             // 70621:     } // switch Inst[12:10]
                                             // 70621:    }
  1, 98,                                     // 70626:    case 0x1: {
  OPC_SwitchField, 10, 3,                    // 70628:     switch Inst[12:10] {
  1, 11,                                     // 70631:     case 0x1: {
  OPC_CheckPredicate, 81,                    // 70633:      check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 70635:      check Inst[21] == 0x0
  OPC_Decode, 129, 23, 225, 3,               // 70639:      decode to FCMGTv8f16 using decoder 481
                                             // 70639:     }
  3, 21,                                     // 70644:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 70646:      switch Inst[21] {
  0, 7,                                      // 70649:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 70651:       check predicate 81
  OPC_Decode, 226, 21, 225, 3,               // 70653:       decode to FACGTv8f16 using decoder 481
                                             // 70653:      }
  1, 0,                                      // 70658:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70660:       check predicate 76
  OPC_Decode, 177, 67, 225, 3,               // 70662:       decode to UQSUBv2i64 using decoder 481
                                             // 70662:      }
                                             // 70662:      } // switch Inst[21]
                                             // 70662:     }
  5, 21,                                     // 70667:     case 0x5: {
  OPC_SwitchField, 21, 1,                    // 70669:      switch Inst[21] {
  0, 7,                                      // 70672:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 70674:       check predicate 81
  OPC_Decode, 205, 27, 225, 3,               // 70676:       decode to FMINPv8f16 using decoder 481
                                             // 70676:      }
  1, 0,                                      // 70681:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70683:       check predicate 76
  OPC_Decode, 130, 18, 225, 3,               // 70685:       decode to CMHIv2i64 using decoder 481
                                             // 70685:      }
                                             // 70685:      } // switch Inst[21]
                                             // 70685:     }
  6, 11,                                     // 70690:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 70692:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 70694:      check Inst[21:16] == 0x20
  OPC_Decode, 223, 68, 238, 3,               // 70698:      decode to USQADDv2i64 using decoder 494
                                             // 70698:     }
  7, 0,                                      // 70703:     case 0x7: {
  OPC_SwitchField, 21, 1,                    // 70705:      switch Inst[21] {
  0, 7,                                      // 70708:      case 0x0: {
  OPC_CheckPredicate, 84,                    // 70710:       check predicate 84
  OPC_Decode, 148, 32, 225, 3,               // 70712:       decode to FSCALEv8f16 using decoder 481
                                             // 70712:      }
  1, 0,                                      // 70717:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 70719:       check predicate 76
  OPC_Decode, 138, 18, 225, 3,               // 70721:       decode to CMHSv2i64 using decoder 481
                                             // 70721:      }
                                             // 70721:      } // switch Inst[21]
                                             // 70721:     }
                                             // 70721:     } // switch Inst[12:10]
                                             // 70721:    }
  2, 55,                                     // 70726:    case 0x2: {
  OPC_SwitchField, 10, 3,                    // 70728:     switch Inst[12:10] {
  1, 11,                                     // 70731:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 70733:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70735:      check Inst[21] == 0x1
  OPC_Decode, 170, 68, 225, 3,               // 70739:      decode to USHLv2i64 using decoder 481
                                             // 70739:     }
  3, 11,                                     // 70744:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 70746:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70748:      check Inst[21] == 0x1
  OPC_Decode, 128, 67, 225, 3,               // 70752:      decode to UQSHLv2i64 using decoder 481
                                             // 70752:     }
  5, 11,                                     // 70757:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 70759:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70761:      check Inst[21] == 0x1
  OPC_Decode, 238, 67, 225, 3,               // 70765:      decode to URSHLv2i64 using decoder 481
                                             // 70765:     }
  7, 0,                                      // 70770:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 70772:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70774:      check Inst[21] == 0x1
  OPC_Decode, 205, 66, 225, 3,               // 70778:      decode to UQRSHLv2i64 using decoder 481
                                             // 70778:     }
                                             // 70778:     } // switch Inst[12:10]
                                             // 70778:    }
  3, 29,                                     // 70783:    case 0x3: {
  OPC_SwitchField, 16, 6,                    // 70785:     switch Inst[21:16] {
  32, 11,                                    // 70788:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 70790:      check predicate 76
  OPC_CheckField, 10, 3, 6,                  // 70792:      check Inst[12:10] == 0x6
  OPC_Decode, 214, 52, 230, 3,               // 70796:      decode to SQNEGv2i64 using decoder 486
                                             // 70796:     }
  33, 0,                                     // 70801:     case 0x21: {
  OPC_CheckPredicate, 84,                    // 70803:      check predicate 84
  OPC_CheckField, 10, 3, 6,                  // 70805:      check Inst[12:10] == 0x6
  OPC_Decode, 244, 14, 230, 3,               // 70809:      decode to BF2CVTL2 using decoder 486
                                             // 70809:     }
                                             // 70809:     } // switch Inst[21:16]
                                             // 70809:    }
  4, 74,                                     // 70814:    case 0x4: {
  OPC_SwitchField, 10, 3,                    // 70816:     switch Inst[12:10] {
  1, 11,                                     // 70819:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 70821:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70823:      check Inst[21] == 0x1
  OPC_Decode, 195, 60, 225, 3,               // 70827:      decode to SUBv2i64 using decoder 481
                                             // 70827:     }
  2, 11,                                     // 70832:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 70834:      check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 70836:      check Inst[21:16] == 0x20
  OPC_Decode, 230, 17, 230, 3,               // 70840:      decode to CMGEv2i64rz using decoder 486
                                             // 70840:     }
  3, 11,                                     // 70845:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 70847:      check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 70849:      check Inst[21] == 0x1
  OPC_Decode, 213, 17, 225, 3,               // 70853:      decode to CMEQv2i64 using decoder 481
                                             // 70853:     }
  6, 0,                                      // 70858:     case 0x6: {
  OPC_SwitchField, 16, 6,                    // 70860:      switch Inst[21:16] {
  32, 7,                                     // 70863:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 70865:       check predicate 76
  OPC_Decode, 152, 18, 230, 3,               // 70867:       decode to CMLEv2i64rz using decoder 486
                                             // 70867:      }
  33, 7,                                     // 70872:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70874:       check predicate 76
  OPC_Decode, 155, 31, 230, 3,               // 70876:       decode to FRINTIv2f64 using decoder 486
                                             // 70876:      }
  57, 0,                                     // 70881:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 70883:       check predicate 81
  OPC_Decode, 158, 31, 230, 3,               // 70885:       decode to FRINTIv8f16 using decoder 486
                                             // 70885:      }
                                             // 70885:      } // switch Inst[21:16]
                                             // 70885:     }
                                             // 70885:     } // switch Inst[12:10]
                                             // 70885:    }
  5, 62,                                     // 70890:    case 0x5: {
  OPC_SwitchField, 16, 6,                    // 70892:     switch Inst[21:16] {
  32, 11,                                    // 70895:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 70897:      check predicate 76
  OPC_CheckField, 10, 3, 6,                  // 70899:      check Inst[12:10] == 0x6
  OPC_Decode, 173, 43, 230, 3,               // 70903:      decode to NEGv2i64 using decoder 486
                                             // 70903:     }
  33, 21,                                    // 70908:     case 0x21: {
  OPC_SwitchField, 10, 3,                    // 70910:      switch Inst[12:10] {
  2, 7,                                      // 70913:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 70915:       check predicate 76
  OPC_Decode, 235, 24, 230, 3,               // 70917:       decode to FCVTPUv2f64 using decoder 486
                                             // 70917:      }
  6, 0,                                      // 70922:      case 0x6: {
  OPC_CheckPredicate, 76,                    // 70924:       check predicate 76
  OPC_Decode, 213, 25, 230, 3,               // 70926:       decode to FCVTZUv2f64 using decoder 486
                                             // 70926:      }
                                             // 70926:      } // switch Inst[12:10]
                                             // 70926:     }
  57, 0,                                     // 70931:     case 0x39: {
  OPC_SwitchField, 10, 3,                    // 70933:      switch Inst[12:10] {
  2, 7,                                      // 70936:      case 0x2: {
  OPC_CheckPredicate, 81,                    // 70938:       check predicate 81
  OPC_Decode, 238, 24, 230, 3,               // 70940:       decode to FCVTPUv8f16 using decoder 486
                                             // 70940:      }
  6, 0,                                      // 70945:      case 0x6: {
  OPC_CheckPredicate, 81,                    // 70947:       check predicate 81
  OPC_Decode, 220, 25, 230, 3,               // 70949:       decode to FCVTZUv8f16 using decoder 486
                                             // 70949:      }
                                             // 70949:      } // switch Inst[12:10]
                                             // 70949:     }
                                             // 70949:     } // switch Inst[21:16]
                                             // 70949:    }
  6, 126,                                    // 70954:    case 0x6: {
  OPC_SwitchField, 10, 1,                    // 70956:     switch Inst[10] {
  0, 75,                                     // 70959:     case 0x0: {
  OPC_SwitchField, 16, 6,                    // 70961:      switch Inst[21:16] {
  32, 21,                                    // 70964:      case 0x20: {
  OPC_SwitchField, 11, 2,                    // 70966:       switch Inst[12:11] {
  1, 7,                                      // 70969:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 70971:        check predicate 76
  OPC_Decode, 230, 22, 230, 3,               // 70973:        decode to FCMGEv2i64rz using decoder 486
                                             // 70973:       }
  3, 0,                                      // 70978:       case 0x3: {
  OPC_CheckPredicate, 76,                    // 70980:        check predicate 76
  OPC_Decode, 151, 23, 230, 3,               // 70982:        decode to FCMLEv2i64rz using decoder 486
                                             // 70982:       }
                                             // 70982:       } // switch Inst[12:11]
                                             // 70982:      }
  33, 11,                                    // 70987:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 70989:       check predicate 76
  OPC_CheckField, 11, 2, 3,                  // 70991:       check Inst[12:11] == 0x3
  OPC_Decode, 242, 31, 230, 3,               // 70995:       decode to FRSQRTEv2f64 using decoder 486
                                             // 70995:      }
  56, 21,                                    // 71000:      case 0x38: {
  OPC_SwitchField, 11, 2,                    // 71002:       switch Inst[12:11] {
  1, 7,                                      // 71005:       case 0x1: {
  OPC_CheckPredicate, 81,                    // 71007:        check predicate 81
  OPC_Decode, 236, 22, 230, 3,               // 71009:        decode to FCMGEv8i16rz using decoder 486
                                             // 71009:       }
  3, 0,                                      // 71014:       case 0x3: {
  OPC_CheckPredicate, 81,                    // 71016:        check predicate 81
  OPC_Decode, 154, 23, 230, 3,               // 71018:        decode to FCMLEv8i16rz using decoder 486
                                             // 71018:       }
                                             // 71018:       } // switch Inst[12:11]
                                             // 71018:      }
  57, 0,                                     // 71023:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 71025:       check predicate 81
  OPC_CheckField, 11, 2, 3,                  // 71027:       check Inst[12:11] == 0x3
  OPC_Decode, 245, 31, 230, 3,               // 71031:       decode to FRSQRTEv8f16 using decoder 486
                                             // 71031:      }
                                             // 71031:      } // switch Inst[21:16]
                                             // 71031:     }
  1, 0,                                      // 71036:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 71038:      switch Inst[21] {
  0, 7,                                      // 71041:      case 0x0: {
  OPC_CheckPredicate, 100,                   // 71043:       check predicate 100
  OPC_Decode, 137, 23, 135, 4,               // 71045:       decode to FCMLAv2f64 using decoder 519
                                             // 71045:      }
  1, 0,                                      // 71050:      case 0x1: {
  OPC_SwitchField, 11, 2,                    // 71052:       switch Inst[12:11] {
  0, 7,                                      // 71055:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 71057:        check predicate 76
  OPC_Decode, 155, 27, 225, 3,               // 71059:        decode to FMINNMPv2f64 using decoder 481
                                             // 71059:       }
  2, 7,                                      // 71064:       case 0x2: {
  OPC_CheckPredicate, 76,                    // 71066:        check predicate 76
  OPC_Decode, 187, 21, 225, 3,               // 71068:        decode to FABDv2f64 using decoder 481
                                             // 71068:       }
  3, 0,                                      // 71073:       case 0x3: {
  OPC_CheckPredicate, 98,                    // 71075:        check predicate 98
  OPC_Decode, 166, 22, 225, 3,               // 71077:        decode to FAMINv2f64 using decoder 481
                                             // 71077:       }
                                             // 71077:       } // switch Inst[12:11]
                                             // 71077:      }
                                             // 71077:      } // switch Inst[21]
                                             // 71077:     }
                                             // 71077:     } // switch Inst[10]
                                             // 71077:    }
  7, 0,                                      // 71082:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 71084:     switch Inst[11:10] {
  1, 35,                                     // 71087:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 71089:      switch Inst[21] {
  0, 7,                                      // 71092:      case 0x0: {
  OPC_CheckPredicate, 100,                   // 71094:       check predicate 100
  OPC_Decode, 174, 22, 136, 4,               // 71096:       decode to FCADDv2f64 using decoder 520
                                             // 71096:      }
  1, 0,                                      // 71101:      case 0x1: {
  OPC_SwitchField, 12, 1,                    // 71103:       switch Inst[12] {
  0, 7,                                      // 71106:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 71108:        check predicate 76
  OPC_Decode, 250, 22, 225, 3,               // 71110:        decode to FCMGTv2f64 using decoder 481
                                             // 71110:       }
  1, 0,                                      // 71115:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 71117:        check predicate 76
  OPC_Decode, 199, 27, 225, 3,               // 71119:        decode to FMINPv2f64 using decoder 481
                                             // 71119:       }
                                             // 71119:       } // switch Inst[12]
                                             // 71119:      }
                                             // 71119:      } // switch Inst[21]
                                             // 71119:     }
  2, 55,                                     // 71124:     case 0x2: {
  OPC_SwitchField, 16, 6,                    // 71126:      switch Inst[21:16] {
  32, 11,                                    // 71129:      case 0x20: {
  OPC_CheckPredicate, 76,                    // 71131:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 71133:       check Inst[12] == 0x1
  OPC_Decode, 165, 30, 230, 3,               // 71137:       decode to FNEGv2f64 using decoder 486
                                             // 71137:      }
  33, 11,                                    // 71142:      case 0x21: {
  OPC_CheckPredicate, 76,                    // 71144:       check predicate 76
  OPC_CheckField, 12, 1, 1,                  // 71146:       check Inst[12] == 0x1
  OPC_Decode, 159, 32, 230, 3,               // 71150:       decode to FSQRTv2f64 using decoder 486
                                             // 71150:      }
  56, 11,                                    // 71155:      case 0x38: {
  OPC_CheckPredicate, 81,                    // 71157:       check predicate 81
  OPC_CheckField, 12, 1, 1,                  // 71159:       check Inst[12] == 0x1
  OPC_Decode, 168, 30, 230, 3,               // 71163:       decode to FNEGv8f16 using decoder 486
                                             // 71163:      }
  57, 0,                                     // 71168:      case 0x39: {
  OPC_CheckPredicate, 81,                    // 71170:       check predicate 81
  OPC_CheckField, 12, 1, 1,                  // 71172:       check Inst[12] == 0x1
  OPC_Decode, 162, 32, 230, 3,               // 71176:       decode to FSQRTv8f16 using decoder 486
                                             // 71176:      }
                                             // 71176:      } // switch Inst[21:16]
                                             // 71176:     }
  3, 0,                                      // 71181:     case 0x3: {
  OPC_SwitchField, 12, 1,                    // 71183:      switch Inst[12] {
  0, 11,                                     // 71186:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 71188:       check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 71190:       check Inst[21] == 0x1
  OPC_Decode, 223, 21, 225, 3,               // 71194:       decode to FACGTv2f64 using decoder 481
                                             // 71194:      }
  1, 0,                                      // 71199:      case 0x1: {
  OPC_SwitchField, 21, 1,                    // 71201:       switch Inst[21] {
  0, 7,                                      // 71204:       case 0x0: {
  OPC_CheckPredicate, 92,                    // 71206:        check predicate 92
  OPC_Decode, 173, 15, 223, 3,               // 71208:        decode to BFMLALT using decoder 479
                                             // 71208:       }
  1, 0,                                      // 71213:       case 0x1: {
  OPC_CheckPredicate, 84,                    // 71215:        check predicate 84
  OPC_Decode, 145, 32, 225, 3,               // 71217:        decode to FSCALEv2f64 using decoder 481
                                             // 71217:       }
                                             // 71217:       } // switch Inst[21]
                                             // 71217:      }
                                             // 71217:      } // switch Inst[12]
                                             // 71217:     }
                                             // 71217:     } // switch Inst[11:10]
                                             // 71217:    }
                                             // 71217:    } // switch Inst[15:13]
                                             // 71217:   }
  6, 0,                                      // 71222:   case 0x6: {
  OPC_SwitchField, 10, 12,                   // 71224:    switch Inst[21:10] {
  32, 7,                                     // 71227:    case 0x20: {
  OPC_CheckPredicate, 88,                    // 71229:     check predicate 88
  OPC_Decode, 143, 48, 238, 3,               // 71231:     decode to SHA512SU0 using decoder 494
                                             // 71231:    }
  33, 0,                                     // 71236:    case 0x21: {
  OPC_CheckPredicate, 95,                    // 71238:     check predicate 95
  OPC_Decode, 215, 48, 238, 3,               // 71240:     decode to SM4E using decoder 494
                                             // 71240:    }
                                             // 71240:    } // switch Inst[21:10]
                                             // 71240:   }
                                             // 71240:   } // switch Inst[31:29]
                                             // 71240:  }
  12, 204, 21,                               // 71245:  case 0xc: {
  OPC_SwitchField, 29, 3,                    // 71248:   switch Inst[31:29] {
  0, 170, 5,                                 // 71251:   case 0x0: {
  OPC_SwitchField, 10, 1,                    // 71254:    switch Inst[10] {
  0, 48,                                     // 71257:    case 0x0: {
  OPC_SwitchField, 12, 4,                    // 71259:     switch Inst[15:12] {
  0, 7,                                      // 71262:     case 0x0: {
  OPC_CheckPredicate, 85,                    // 71264:      check predicate 85
  OPC_Decode, 148, 26, 141, 4,               // 71266:      decode to FDOTlanev2f32 using decoder 525
                                             // 71266:     }
  1, 7,                                      // 71271:     case 0x1: {
  OPC_CheckPredicate, 81,                    // 71273:      check predicate 81
  OPC_Decode, 204, 28, 142, 4,               // 71275:      decode to FMLAv4i16_indexed using decoder 526
                                             // 71275:     }
  5, 7,                                      // 71280:     case 0x5: {
  OPC_CheckPredicate, 81,                    // 71282:      check predicate 81
  OPC_Decode, 134, 29, 142, 4,               // 71284:      decode to FMLSv4i16_indexed using decoder 526
                                             // 71284:     }
  9, 7,                                      // 71289:     case 0x9: {
  OPC_CheckPredicate, 81,                    // 71291:      check predicate 81
  OPC_Decode, 151, 30, 143, 4,               // 71293:      decode to FMULv4i16_indexed using decoder 527
                                             // 71293:     }
  15, 0,                                     // 71298:     case 0xf: {
  OPC_CheckPredicate, 97,                    // 71300:      check predicate 97
  OPC_Decode, 206, 60, 141, 4,               // 71302:      decode to SUDOTlanev8i8 using decoder 525
                                             // 71302:     }
                                             // 71302:     } // switch Inst[15:12]
                                             // 71302:    }
  1, 0,                                      // 71307:    case 0x1: {
  OPC_SwitchField, 11, 1,                    // 71309:     switch Inst[11] {
  0, 237, 3,                                 // 71312:     case 0x0: {
  OPC_SwitchField, 15, 1,                    // 71315:      switch Inst[15] {
  0, 243, 1,                                 // 71318:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 71321:       switch Inst[12] {
  0, 91,                                     // 71324:       case 0x0: {
  OPC_SwitchField, 21, 1,                    // 71326:        switch Inst[21] {
  0, 63,                                     // 71329:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71331:         switch Inst[20] {
  0, 35,                                     // 71334:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 71336:          switch Inst[19] {
  0, 7,                                      // 71339:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 71341:           check predicate 76
  OPC_Decode, 217, 42, 144, 4,               // 71343:           decode to MOVIv2i32 using decoder 528
                                             // 71343:          }
  1, 0,                                      // 71348:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 71350:           switch Inst[14:13] {
  0, 7,                                      // 71353:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 71355:            check predicate 76
  OPC_Decode, 206, 55, 145, 4,               // 71357:            decode to SSHRv8i8_shift using decoder 529
                                             // 71357:           }
  1, 0,                                      // 71362:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 71364:            check predicate 76
  OPC_Decode, 166, 55, 145, 4,               // 71366:            decode to SRSHRv8i8_shift using decoder 529
                                             // 71366:           }
                                             // 71366:           } // switch Inst[14:13]
                                             // 71366:          }
                                             // 71366:          } // switch Inst[19]
                                             // 71366:         }
  1, 0,                                      // 71371:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 71373:          switch Inst[14:13] {
  0, 7,                                      // 71376:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 71378:           check predicate 76
  OPC_Decode, 203, 55, 146, 4,               // 71380:           decode to SSHRv4i16_shift using decoder 530
                                             // 71380:          }
  1, 0,                                      // 71385:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 71387:           check predicate 76
  OPC_Decode, 163, 55, 146, 4,               // 71389:           decode to SRSHRv4i16_shift using decoder 530
                                             // 71389:          }
                                             // 71389:          } // switch Inst[14:13]
                                             // 71389:         }
                                             // 71389:         } // switch Inst[20]
                                             // 71389:        }
  1, 0,                                      // 71394:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 71396:         switch Inst[14:13] {
  0, 7,                                      // 71399:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 71401:          check predicate 76
  OPC_Decode, 201, 55, 147, 4,               // 71403:          decode to SSHRv2i32_shift using decoder 531
                                             // 71403:         }
  1, 0,                                      // 71408:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 71410:          check predicate 76
  OPC_Decode, 161, 55, 147, 4,               // 71412:          decode to SRSHRv2i32_shift using decoder 531
                                             // 71412:         }
                                             // 71412:         } // switch Inst[14:13]
                                             // 71412:        }
                                             // 71412:        } // switch Inst[21]
                                             // 71412:       }
  1, 0,                                      // 71417:       case 0x1: {
  OPC_SwitchField, 21, 1,                    // 71419:        switch Inst[21] {
  0, 99,                                     // 71422:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71424:         switch Inst[20] {
  0, 53,                                     // 71427:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 71429:          switch Inst[19] {
  0, 7,                                      // 71432:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 71434:           check predicate 76
  OPC_Decode, 216, 43, 148, 4,               // 71436:           decode to ORRv2i32 using decoder 532
                                             // 71436:          }
  1, 0,                                      // 71441:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 71443:           switch Inst[14:13] {
  0, 7,                                      // 71446:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 71448:            check predicate 76
  OPC_Decode, 218, 55, 149, 4,               // 71450:            decode to SSRAv8i8_shift using decoder 533
                                             // 71450:           }
  1, 7,                                      // 71455:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 71457:            check predicate 76
  OPC_Decode, 178, 55, 149, 4,               // 71459:            decode to SRSRAv8i8_shift using decoder 533
                                             // 71459:           }
  2, 7,                                      // 71464:           case 0x2: {
  OPC_CheckPredicate, 76,                    // 71466:            check predicate 76
  OPC_Decode, 168, 48, 150, 4,               // 71468:            decode to SHLv8i8_shift using decoder 534
                                             // 71468:           }
  3, 0,                                      // 71473:           case 0x3: {
  OPC_CheckPredicate, 76,                    // 71475:            check predicate 76
  OPC_Decode, 137, 54, 150, 4,               // 71477:            decode to SQSHLv8i8_shift using decoder 534
                                             // 71477:           }
                                             // 71477:           } // switch Inst[14:13]
                                             // 71477:          }
                                             // 71477:          } // switch Inst[19]
                                             // 71477:         }
  1, 0,                                      // 71482:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 71484:          switch Inst[14:13] {
  0, 7,                                      // 71487:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 71489:           check predicate 76
  OPC_Decode, 215, 55, 151, 4,               // 71491:           decode to SSRAv4i16_shift using decoder 535
                                             // 71491:          }
  1, 7,                                      // 71496:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 71498:           check predicate 76
  OPC_Decode, 175, 55, 151, 4,               // 71500:           decode to SRSRAv4i16_shift using decoder 535
                                             // 71500:          }
  2, 7,                                      // 71505:          case 0x2: {
  OPC_CheckPredicate, 76,                    // 71507:           check predicate 76
  OPC_Decode, 165, 48, 152, 4,               // 71509:           decode to SHLv4i16_shift using decoder 536
                                             // 71509:          }
  3, 0,                                      // 71514:          case 0x3: {
  OPC_CheckPredicate, 76,                    // 71516:           check predicate 76
  OPC_Decode, 131, 54, 152, 4,               // 71518:           decode to SQSHLv4i16_shift using decoder 536
                                             // 71518:          }
                                             // 71518:          } // switch Inst[14:13]
                                             // 71518:         }
                                             // 71518:         } // switch Inst[20]
                                             // 71518:        }
  1, 0,                                      // 71523:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 71525:         switch Inst[14:13] {
  0, 7,                                      // 71528:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 71530:          check predicate 76
  OPC_Decode, 213, 55, 153, 4,               // 71532:          decode to SSRAv2i32_shift using decoder 537
                                             // 71532:         }
  1, 7,                                      // 71537:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 71539:          check predicate 76
  OPC_Decode, 173, 55, 153, 4,               // 71541:          decode to SRSRAv2i32_shift using decoder 537
                                             // 71541:         }
  2, 7,                                      // 71546:         case 0x2: {
  OPC_CheckPredicate, 76,                    // 71548:          check predicate 76
  OPC_Decode, 163, 48, 154, 4,               // 71550:          decode to SHLv2i32_shift using decoder 538
                                             // 71550:         }
  3, 0,                                      // 71555:         case 0x3: {
  OPC_CheckPredicate, 76,                    // 71557:          check predicate 76
  OPC_Decode, 255, 53, 154, 4,               // 71559:          decode to SQSHLv2i32_shift using decoder 538
                                             // 71559:         }
                                             // 71559:         } // switch Inst[14:13]
                                             // 71559:        }
                                             // 71559:        } // switch Inst[21]
                                             // 71559:       }
                                             // 71559:       } // switch Inst[12]
                                             // 71559:      }
  1, 0,                                      // 71564:      case 0x1: {
  OPC_SwitchField, 14, 1,                    // 71566:       switch Inst[14] {
  0, 159, 1,                                 // 71569:       case 0x0: {
  OPC_SwitchField, 12, 1,                    // 71572:        switch Inst[12] {
  0, 91,                                     // 71575:        case 0x0: {
  OPC_SwitchField, 21, 1,                    // 71577:         switch Inst[21] {
  0, 63,                                     // 71580:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71582:          switch Inst[20] {
  0, 35,                                     // 71585:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 71587:           switch Inst[19] {
  0, 7,                                      // 71590:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 71592:            check predicate 76
  OPC_Decode, 219, 42, 144, 4,               // 71594:            decode to MOVIv4i16 using decoder 528
                                             // 71594:           }
  1, 0,                                      // 71599:           case 0x1: {
  OPC_SwitchField, 13, 1,                    // 71601:            switch Inst[13] {
  0, 7,                                      // 71604:            case 0x0: {
  OPC_CheckPredicate, 76,                    // 71606:             check predicate 76
  OPC_Decode, 180, 48, 155, 4,               // 71608:             decode to SHRNv8i8_shift using decoder 539
                                             // 71608:            }
  1, 0,                                      // 71613:            case 0x1: {
  OPC_CheckPredicate, 76,                    // 71615:             check predicate 76
  OPC_Decode, 190, 55, 156, 4,               // 71617:             decode to SSHLLv8i8_shift using decoder 540
                                             // 71617:            }
                                             // 71617:            } // switch Inst[13]
                                             // 71617:           }
                                             // 71617:           } // switch Inst[19]
                                             // 71617:          }
  1, 0,                                      // 71622:          case 0x1: {
  OPC_SwitchField, 13, 1,                    // 71624:           switch Inst[13] {
  0, 7,                                      // 71627:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 71629:            check predicate 76
  OPC_Decode, 177, 48, 157, 4,               // 71631:            decode to SHRNv4i16_shift using decoder 541
                                             // 71631:           }
  1, 0,                                      // 71636:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 71638:            check predicate 76
  OPC_Decode, 187, 55, 158, 4,               // 71640:            decode to SSHLLv4i16_shift using decoder 542
                                             // 71640:           }
                                             // 71640:           } // switch Inst[13]
                                             // 71640:          }
                                             // 71640:          } // switch Inst[20]
                                             // 71640:         }
  1, 0,                                      // 71645:         case 0x1: {
  OPC_SwitchField, 13, 1,                    // 71647:          switch Inst[13] {
  0, 7,                                      // 71650:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 71652:           check predicate 76
  OPC_Decode, 176, 48, 159, 4,               // 71654:           decode to SHRNv2i32_shift using decoder 543
                                             // 71654:          }
  1, 0,                                      // 71659:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 71661:           check predicate 76
  OPC_Decode, 186, 55, 160, 4,               // 71663:           decode to SSHLLv2i32_shift using decoder 544
                                             // 71663:          }
                                             // 71663:          } // switch Inst[13]
                                             // 71663:         }
                                             // 71663:         } // switch Inst[21]
                                             // 71663:        }
  1, 0,                                      // 71668:        case 0x1: {
  OPC_SwitchField, 21, 1,                    // 71670:         switch Inst[21] {
  0, 43,                                     // 71673:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71675:          switch Inst[20] {
  0, 25,                                     // 71678:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 71680:           switch Inst[19] {
  0, 7,                                      // 71683:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 71685:            check predicate 76
  OPC_Decode, 217, 43, 148, 4,               // 71687:            decode to ORRv4i16 using decoder 532
                                             // 71687:           }
  1, 0,                                      // 71692:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 71694:            check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 71696:            check Inst[13] == 0x0
  OPC_Decode, 154, 54, 155, 4,               // 71700:            decode to SQSHRNv8i8_shift using decoder 539
                                             // 71700:           }
                                             // 71700:           } // switch Inst[19]
                                             // 71700:          }
  1, 0,                                      // 71705:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 71707:           check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 71709:           check Inst[13] == 0x0
  OPC_Decode, 151, 54, 157, 4,               // 71713:           decode to SQSHRNv4i16_shift using decoder 541
                                             // 71713:          }
                                             // 71713:          } // switch Inst[20]
                                             // 71713:         }
  1, 0,                                      // 71718:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 71720:          check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 71722:          check Inst[13] == 0x0
  OPC_Decode, 150, 54, 159, 4,               // 71726:          decode to SQSHRNv2i32_shift using decoder 543
                                             // 71726:         }
                                             // 71726:         } // switch Inst[21]
                                             // 71726:        }
                                             // 71726:        } // switch Inst[12]
                                             // 71726:       }
  1, 0,                                      // 71731:       case 0x1: {
  OPC_SwitchField, 13, 1,                    // 71733:        switch Inst[13] {
  0, 11,                                     // 71736:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 71738:         check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 71740:         check Inst[21:19] == 0x0
  OPC_Decode, 218, 42, 144, 4,               // 71744:         decode to MOVIv2s_msl using decoder 528
                                             // 71744:        }
  1, 0,                                      // 71749:        case 0x1: {
  OPC_SwitchField, 12, 1,                    // 71751:         switch Inst[12] {
  0, 39,                                     // 71754:         case 0x0: {
  OPC_SwitchField, 21, 1,                    // 71756:          switch Inst[21] {
  0, 25,                                     // 71759:          case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71761:           switch Inst[20] {
  0, 11,                                     // 71764:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 71766:            check predicate 76
  OPC_CheckField, 19, 1, 0,                  // 71768:            check Inst[19] == 0x0
  OPC_Decode, 222, 42, 144, 4,               // 71772:            decode to MOVIv8b_ns using decoder 528
                                             // 71772:           }
  1, 0,                                      // 71777:           case 0x1: {
  OPC_CheckPredicate, 81,                    // 71779:            check predicate 81
  OPC_Decode, 171, 47, 146, 4,               // 71781:            decode to SCVTFv4i16_shift using decoder 530
                                             // 71781:           }
                                             // 71781:           } // switch Inst[20]
                                             // 71781:          }
  1, 0,                                      // 71786:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 71788:           check predicate 76
  OPC_Decode, 167, 47, 147, 4,               // 71790:           decode to SCVTFv2i32_shift using decoder 531
                                             // 71790:          }
                                             // 71790:          } // switch Inst[21]
                                             // 71790:         }
  1, 0,                                      // 71795:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 71797:          check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 71799:          check Inst[21:19] == 0x0
  OPC_Decode, 213, 29, 144, 4,               // 71803:          decode to FMOVv2f32_ns using decoder 528
                                             // 71803:         }
                                             // 71803:         } // switch Inst[12]
                                             // 71803:        }
                                             // 71803:        } // switch Inst[13]
                                             // 71803:       }
                                             // 71803:       } // switch Inst[14]
                                             // 71803:      }
                                             // 71803:      } // switch Inst[15]
                                             // 71803:     }
  1, 0,                                      // 71808:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 71810:      switch Inst[15:12] {
  8, 39,                                     // 71813:      case 0x8: {
  OPC_SwitchField, 21, 1,                    // 71815:       switch Inst[21] {
  0, 25,                                     // 71818:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71820:        switch Inst[20] {
  0, 11,                                     // 71823:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 71825:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 71827:         check Inst[19] == 0x1
  OPC_Decode, 244, 45, 155, 4,               // 71831:         decode to RSHRNv8i8_shift using decoder 539
                                             // 71831:        }
  1, 0,                                      // 71836:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 71838:         check predicate 76
  OPC_Decode, 241, 45, 157, 4,               // 71840:         decode to RSHRNv4i16_shift using decoder 541
                                             // 71840:        }
                                             // 71840:        } // switch Inst[20]
                                             // 71840:       }
  1, 0,                                      // 71845:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 71847:        check predicate 76
  OPC_Decode, 240, 45, 159, 4,               // 71849:        decode to RSHRNv2i32_shift using decoder 543
                                             // 71849:       }
                                             // 71849:       } // switch Inst[21]
                                             // 71849:      }
  9, 39,                                     // 71854:      case 0x9: {
  OPC_SwitchField, 21, 1,                    // 71856:       switch Inst[21] {
  0, 25,                                     // 71859:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71861:        switch Inst[20] {
  0, 11,                                     // 71864:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 71866:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 71868:         check Inst[19] == 0x1
  OPC_Decode, 191, 53, 155, 4,               // 71872:         decode to SQRSHRNv8i8_shift using decoder 539
                                             // 71872:        }
  1, 0,                                      // 71877:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 71879:         check predicate 76
  OPC_Decode, 188, 53, 157, 4,               // 71881:         decode to SQRSHRNv4i16_shift using decoder 541
                                             // 71881:        }
                                             // 71881:        } // switch Inst[20]
                                             // 71881:       }
  1, 0,                                      // 71886:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 71888:        check predicate 76
  OPC_Decode, 187, 53, 159, 4,               // 71890:        decode to SQRSHRNv2i32_shift using decoder 543
                                             // 71890:       }
                                             // 71890:       } // switch Inst[21]
                                             // 71890:      }
  15, 0,                                     // 71895:      case 0xf: {
  OPC_SwitchField, 21, 1,                    // 71897:       switch Inst[21] {
  0, 25,                                     // 71900:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71902:        switch Inst[20] {
  0, 11,                                     // 71905:        case 0x0: {
  OPC_CheckPredicate, 81,                    // 71907:         check predicate 81
  OPC_CheckField, 19, 1, 0,                  // 71909:         check Inst[19] == 0x0
  OPC_Decode, 215, 29, 144, 4,               // 71913:         decode to FMOVv4f16_ns using decoder 528
                                             // 71913:        }
  1, 0,                                      // 71918:        case 0x1: {
  OPC_CheckPredicate, 81,                    // 71920:         check predicate 81
  OPC_Decode, 167, 25, 146, 4,               // 71922:         decode to FCVTZSv4i16_shift using decoder 530
                                             // 71922:        }
                                             // 71922:        } // switch Inst[20]
                                             // 71922:       }
  1, 0,                                      // 71927:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 71929:        check predicate 76
  OPC_Decode, 163, 25, 147, 4,               // 71931:        decode to FCVTZSv2i32_shift using decoder 531
                                             // 71931:       }
                                             // 71931:       } // switch Inst[21]
                                             // 71931:      }
                                             // 71931:      } // switch Inst[15:12]
                                             // 71931:     }
                                             // 71931:     } // switch Inst[11]
                                             // 71931:    }
                                             // 71931:    } // switch Inst[10]
                                             // 71931:   }
  1, 178, 5,                                 // 71936:   case 0x1: {
  OPC_SwitchField, 10, 1,                    // 71939:    switch Inst[10] {
  0, 21,                                     // 71942:    case 0x0: {
  OPC_SwitchField, 12, 4,                    // 71944:     switch Inst[15:12] {
  8, 7,                                      // 71947:     case 0x8: {
  OPC_CheckPredicate, 80,                    // 71949:      check predicate 80
  OPC_Decode, 251, 27, 161, 4,               // 71951:      decode to FMLALLBBlanev4f32 using decoder 545
                                             // 71951:     }
  9, 0,                                      // 71956:     case 0x9: {
  OPC_CheckPredicate, 81,                    // 71958:      check predicate 81
  OPC_Decode, 242, 29, 143, 4,               // 71960:      decode to FMULXv4i16_indexed using decoder 527
                                             // 71960:     }
                                             // 71960:     } // switch Inst[15:12]
                                             // 71960:    }
  1, 0,                                      // 71965:    case 0x1: {
  OPC_SwitchField, 11, 1,                    // 71967:     switch Inst[11] {
  0, 158, 4,                                 // 71970:     case 0x0: {
  OPC_SwitchField, 15, 1,                    // 71973:      switch Inst[15] {
  0, 170, 2,                                 // 71976:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 71979:       switch Inst[12] {
  0, 145, 1,                                 // 71982:       case 0x0: {
  OPC_SwitchField, 21, 1,                    // 71985:        switch Inst[21] {
  0, 99,                                     // 71988:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 71990:         switch Inst[20] {
  0, 53,                                     // 71993:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 71995:          switch Inst[19] {
  0, 7,                                      // 71998:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72000:           check predicate 76
  OPC_Decode, 153, 43, 144, 4,               // 72002:           decode to MVNIv2i32 using decoder 528
                                             // 72002:          }
  1, 0,                                      // 72007:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72009:           switch Inst[14:13] {
  0, 7,                                      // 72012:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72014:            check predicate 76
  OPC_Decode, 182, 68, 145, 4,               // 72016:            decode to USHRv8i8_shift using decoder 529
                                             // 72016:           }
  1, 7,                                      // 72021:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 72023:            check predicate 76
  OPC_Decode, 254, 67, 145, 4,               // 72025:            decode to URSHRv8i8_shift using decoder 529
                                             // 72025:           }
  2, 7,                                      // 72030:           case 0x2: {
  OPC_CheckPredicate, 76,                    // 72032:            check predicate 76
  OPC_Decode, 250, 54, 149, 4,               // 72034:            decode to SRIv8i8_shift using decoder 533
                                             // 72034:           }
  3, 0,                                      // 72039:           case 0x3: {
  OPC_CheckPredicate, 76,                    // 72041:            check predicate 76
  OPC_Decode, 235, 53, 150, 4,               // 72043:            decode to SQSHLUv8i8_shift using decoder 534
                                             // 72043:           }
                                             // 72043:           } // switch Inst[14:13]
                                             // 72043:          }
                                             // 72043:          } // switch Inst[19]
                                             // 72043:         }
  1, 0,                                      // 72048:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72050:          switch Inst[14:13] {
  0, 7,                                      // 72053:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72055:           check predicate 76
  OPC_Decode, 179, 68, 146, 4,               // 72057:           decode to USHRv4i16_shift using decoder 530
                                             // 72057:          }
  1, 7,                                      // 72062:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 72064:           check predicate 76
  OPC_Decode, 251, 67, 146, 4,               // 72066:           decode to URSHRv4i16_shift using decoder 530
                                             // 72066:          }
  2, 7,                                      // 72071:          case 0x2: {
  OPC_CheckPredicate, 76,                    // 72073:           check predicate 76
  OPC_Decode, 247, 54, 151, 4,               // 72075:           decode to SRIv4i16_shift using decoder 535
                                             // 72075:          }
  3, 0,                                      // 72080:          case 0x3: {
  OPC_CheckPredicate, 76,                    // 72082:           check predicate 76
  OPC_Decode, 232, 53, 152, 4,               // 72084:           decode to SQSHLUv4i16_shift using decoder 536
                                             // 72084:          }
                                             // 72084:          } // switch Inst[14:13]
                                             // 72084:         }
                                             // 72084:         } // switch Inst[20]
                                             // 72084:        }
  1, 0,                                      // 72089:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72091:         switch Inst[14:13] {
  0, 7,                                      // 72094:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 72096:          check predicate 76
  OPC_Decode, 177, 68, 147, 4,               // 72098:          decode to USHRv2i32_shift using decoder 531
                                             // 72098:         }
  1, 7,                                      // 72103:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 72105:          check predicate 76
  OPC_Decode, 249, 67, 147, 4,               // 72107:          decode to URSHRv2i32_shift using decoder 531
                                             // 72107:         }
  2, 7,                                      // 72112:         case 0x2: {
  OPC_CheckPredicate, 76,                    // 72114:          check predicate 76
  OPC_Decode, 245, 54, 153, 4,               // 72116:          decode to SRIv2i32_shift using decoder 537
                                             // 72116:         }
  3, 0,                                      // 72121:         case 0x3: {
  OPC_CheckPredicate, 76,                    // 72123:          check predicate 76
  OPC_Decode, 230, 53, 154, 4,               // 72125:          decode to SQSHLUv2i32_shift using decoder 538
                                             // 72125:         }
                                             // 72125:         } // switch Inst[14:13]
                                             // 72125:        }
                                             // 72125:        } // switch Inst[21]
                                             // 72125:       }
  1, 0,                                      // 72130:       case 0x1: {
  OPC_SwitchField, 21, 1,                    // 72132:        switch Inst[21] {
  0, 99,                                     // 72135:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72137:         switch Inst[20] {
  0, 53,                                     // 72140:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 72142:          switch Inst[19] {
  0, 7,                                      // 72145:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72147:           check predicate 76
  OPC_Decode, 145, 16, 148, 4,               // 72149:           decode to BICv2i32 using decoder 532
                                             // 72149:          }
  1, 0,                                      // 72154:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72156:           switch Inst[14:13] {
  0, 7,                                      // 72159:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72161:            check predicate 76
  OPC_Decode, 239, 68, 149, 4,               // 72163:            decode to USRAv8i8_shift using decoder 533
                                             // 72163:           }
  1, 7,                                      // 72168:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 72170:            check predicate 76
  OPC_Decode, 142, 68, 149, 4,               // 72172:            decode to URSRAv8i8_shift using decoder 533
                                             // 72172:           }
  2, 7,                                      // 72177:           case 0x2: {
  OPC_CheckPredicate, 76,                    // 72179:            check predicate 76
  OPC_Decode, 207, 48, 162, 4,               // 72181:            decode to SLIv8i8_shift using decoder 546
                                             // 72181:           }
  3, 0,                                      // 72186:           case 0x3: {
  OPC_CheckPredicate, 76,                    // 72188:            check predicate 76
  OPC_Decode, 137, 67, 150, 4,               // 72190:            decode to UQSHLv8i8_shift using decoder 534
                                             // 72190:           }
                                             // 72190:           } // switch Inst[14:13]
                                             // 72190:          }
                                             // 72190:          } // switch Inst[19]
                                             // 72190:         }
  1, 0,                                      // 72195:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72197:          switch Inst[14:13] {
  0, 7,                                      // 72200:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72202:           check predicate 76
  OPC_Decode, 236, 68, 151, 4,               // 72204:           decode to USRAv4i16_shift using decoder 535
                                             // 72204:          }
  1, 7,                                      // 72209:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 72211:           check predicate 76
  OPC_Decode, 139, 68, 151, 4,               // 72213:           decode to URSRAv4i16_shift using decoder 535
                                             // 72213:          }
  2, 7,                                      // 72218:          case 0x2: {
  OPC_CheckPredicate, 76,                    // 72220:           check predicate 76
  OPC_Decode, 204, 48, 163, 4,               // 72222:           decode to SLIv4i16_shift using decoder 547
                                             // 72222:          }
  3, 0,                                      // 72227:          case 0x3: {
  OPC_CheckPredicate, 76,                    // 72229:           check predicate 76
  OPC_Decode, 131, 67, 152, 4,               // 72231:           decode to UQSHLv4i16_shift using decoder 536
                                             // 72231:          }
                                             // 72231:          } // switch Inst[14:13]
                                             // 72231:         }
                                             // 72231:         } // switch Inst[20]
                                             // 72231:        }
  1, 0,                                      // 72236:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72238:         switch Inst[14:13] {
  0, 7,                                      // 72241:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 72243:          check predicate 76
  OPC_Decode, 234, 68, 153, 4,               // 72245:          decode to USRAv2i32_shift using decoder 537
                                             // 72245:         }
  1, 7,                                      // 72250:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 72252:          check predicate 76
  OPC_Decode, 137, 68, 153, 4,               // 72254:          decode to URSRAv2i32_shift using decoder 537
                                             // 72254:         }
  2, 7,                                      // 72259:         case 0x2: {
  OPC_CheckPredicate, 76,                    // 72261:          check predicate 76
  OPC_Decode, 202, 48, 164, 4,               // 72263:          decode to SLIv2i32_shift using decoder 548
                                             // 72263:         }
  3, 0,                                      // 72268:         case 0x3: {
  OPC_CheckPredicate, 76,                    // 72270:          check predicate 76
  OPC_Decode, 255, 66, 154, 4,               // 72272:          decode to UQSHLv2i32_shift using decoder 538
                                             // 72272:         }
                                             // 72272:         } // switch Inst[14:13]
                                             // 72272:        }
                                             // 72272:        } // switch Inst[21]
                                             // 72272:       }
                                             // 72272:       } // switch Inst[12]
                                             // 72272:      }
  1, 0,                                      // 72277:      case 0x1: {
  OPC_SwitchField, 14, 1,                    // 72279:       switch Inst[14] {
  0, 159, 1,                                 // 72282:       case 0x0: {
  OPC_SwitchField, 12, 1,                    // 72285:        switch Inst[12] {
  0, 91,                                     // 72288:        case 0x0: {
  OPC_SwitchField, 21, 1,                    // 72290:         switch Inst[21] {
  0, 63,                                     // 72293:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72295:          switch Inst[20] {
  0, 35,                                     // 72298:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 72300:           switch Inst[19] {
  0, 7,                                      // 72303:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72305:            check predicate 76
  OPC_Decode, 155, 43, 144, 4,               // 72307:            decode to MVNIv4i16 using decoder 528
                                             // 72307:           }
  1, 0,                                      // 72312:           case 0x1: {
  OPC_SwitchField, 13, 1,                    // 72314:            switch Inst[13] {
  0, 7,                                      // 72317:            case 0x0: {
  OPC_CheckPredicate, 76,                    // 72319:             check predicate 76
  OPC_Decode, 171, 54, 155, 4,               // 72321:             decode to SQSHRUNv8i8_shift using decoder 539
                                             // 72321:            }
  1, 0,                                      // 72326:            case 0x1: {
  OPC_CheckPredicate, 76,                    // 72328:             check predicate 76
  OPC_Decode, 166, 68, 156, 4,               // 72330:             decode to USHLLv8i8_shift using decoder 540
                                             // 72330:            }
                                             // 72330:            } // switch Inst[13]
                                             // 72330:           }
                                             // 72330:           } // switch Inst[19]
                                             // 72330:          }
  1, 0,                                      // 72335:          case 0x1: {
  OPC_SwitchField, 13, 1,                    // 72337:           switch Inst[13] {
  0, 7,                                      // 72340:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72342:            check predicate 76
  OPC_Decode, 168, 54, 157, 4,               // 72344:            decode to SQSHRUNv4i16_shift using decoder 541
                                             // 72344:           }
  1, 0,                                      // 72349:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 72351:            check predicate 76
  OPC_Decode, 163, 68, 158, 4,               // 72353:            decode to USHLLv4i16_shift using decoder 542
                                             // 72353:           }
                                             // 72353:           } // switch Inst[13]
                                             // 72353:          }
                                             // 72353:          } // switch Inst[20]
                                             // 72353:         }
  1, 0,                                      // 72358:         case 0x1: {
  OPC_SwitchField, 13, 1,                    // 72360:          switch Inst[13] {
  0, 7,                                      // 72363:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72365:           check predicate 76
  OPC_Decode, 167, 54, 159, 4,               // 72367:           decode to SQSHRUNv2i32_shift using decoder 543
                                             // 72367:          }
  1, 0,                                      // 72372:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 72374:           check predicate 76
  OPC_Decode, 162, 68, 160, 4,               // 72376:           decode to USHLLv2i32_shift using decoder 544
                                             // 72376:          }
                                             // 72376:          } // switch Inst[13]
                                             // 72376:         }
                                             // 72376:         } // switch Inst[21]
                                             // 72376:        }
  1, 0,                                      // 72381:        case 0x1: {
  OPC_SwitchField, 21, 1,                    // 72383:         switch Inst[21] {
  0, 43,                                     // 72386:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72388:          switch Inst[20] {
  0, 25,                                     // 72391:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 72393:           switch Inst[19] {
  0, 7,                                      // 72396:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72398:            check predicate 76
  OPC_Decode, 146, 16, 148, 4,               // 72400:            decode to BICv4i16 using decoder 532
                                             // 72400:           }
  1, 0,                                      // 72405:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 72407:            check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 72409:            check Inst[13] == 0x0
  OPC_Decode, 154, 67, 155, 4,               // 72413:            decode to UQSHRNv8i8_shift using decoder 539
                                             // 72413:           }
                                             // 72413:           } // switch Inst[19]
                                             // 72413:          }
  1, 0,                                      // 72418:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 72420:           check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 72422:           check Inst[13] == 0x0
  OPC_Decode, 151, 67, 157, 4,               // 72426:           decode to UQSHRNv4i16_shift using decoder 541
                                             // 72426:          }
                                             // 72426:          } // switch Inst[20]
                                             // 72426:         }
  1, 0,                                      // 72431:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 72433:          check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 72435:          check Inst[13] == 0x0
  OPC_Decode, 150, 67, 159, 4,               // 72439:          decode to UQSHRNv2i32_shift using decoder 543
                                             // 72439:         }
                                             // 72439:         } // switch Inst[21]
                                             // 72439:        }
                                             // 72439:        } // switch Inst[12]
                                             // 72439:       }
  1, 0,                                      // 72444:       case 0x1: {
  OPC_SwitchField, 13, 1,                    // 72446:        switch Inst[13] {
  0, 11,                                     // 72449:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 72451:         check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 72453:         check Inst[21:19] == 0x0
  OPC_Decode, 154, 43, 144, 4,               // 72457:         decode to MVNIv2s_msl using decoder 528
                                             // 72457:        }
  1, 0,                                      // 72462:        case 0x1: {
  OPC_SwitchField, 21, 1,                    // 72464:         switch Inst[21] {
  0, 33,                                     // 72467:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72469:          switch Inst[20] {
  0, 15,                                     // 72472:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72474:           check predicate 76
  OPC_CheckField, 19, 1, 0,                  // 72476:           check Inst[19] == 0x0
  OPC_CheckField, 12, 1, 0,                  // 72480:           check Inst[12] == 0x0
  OPC_Decode, 214, 42, 144, 4,               // 72484:           decode to MOVID using decoder 528
                                             // 72484:          }
  1, 0,                                      // 72489:          case 0x1: {
  OPC_CheckPredicate, 81,                    // 72491:           check predicate 81
  OPC_CheckField, 12, 1, 0,                  // 72493:           check Inst[12] == 0x0
  OPC_Decode, 164, 63, 146, 4,               // 72497:           decode to UCVTFv4i16_shift using decoder 530
                                             // 72497:          }
                                             // 72497:          } // switch Inst[20]
                                             // 72497:         }
  1, 0,                                      // 72502:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 72504:          check predicate 76
  OPC_CheckField, 12, 1, 0,                  // 72506:          check Inst[12] == 0x0
  OPC_Decode, 160, 63, 147, 4,               // 72510:          decode to UCVTFv2i32_shift using decoder 531
                                             // 72510:         }
                                             // 72510:         } // switch Inst[21]
                                             // 72510:        }
                                             // 72510:        } // switch Inst[13]
                                             // 72510:       }
                                             // 72510:       } // switch Inst[14]
                                             // 72510:      }
                                             // 72510:      } // switch Inst[15]
                                             // 72510:     }
  1, 0,                                      // 72515:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 72517:      switch Inst[15:12] {
  8, 39,                                     // 72520:      case 0x8: {
  OPC_SwitchField, 21, 1,                    // 72522:       switch Inst[21] {
  0, 25,                                     // 72525:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72527:        switch Inst[20] {
  0, 11,                                     // 72530:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 72532:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 72534:         check Inst[19] == 0x1
  OPC_Decode, 210, 53, 155, 4,               // 72538:         decode to SQRSHRUNv8i8_shift using decoder 539
                                             // 72538:        }
  1, 0,                                      // 72543:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 72545:         check predicate 76
  OPC_Decode, 207, 53, 157, 4,               // 72547:         decode to SQRSHRUNv4i16_shift using decoder 541
                                             // 72547:        }
                                             // 72547:        } // switch Inst[20]
                                             // 72547:       }
  1, 0,                                      // 72552:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 72554:        check predicate 76
  OPC_Decode, 206, 53, 159, 4,               // 72556:        decode to SQRSHRUNv2i32_shift using decoder 543
                                             // 72556:       }
                                             // 72556:       } // switch Inst[21]
                                             // 72556:      }
  9, 39,                                     // 72561:      case 0x9: {
  OPC_SwitchField, 21, 1,                    // 72563:       switch Inst[21] {
  0, 25,                                     // 72566:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72568:        switch Inst[20] {
  0, 11,                                     // 72571:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 72573:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 72575:         check Inst[19] == 0x1
  OPC_Decode, 228, 66, 155, 4,               // 72579:         decode to UQRSHRNv8i8_shift using decoder 539
                                             // 72579:        }
  1, 0,                                      // 72584:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 72586:         check predicate 76
  OPC_Decode, 225, 66, 157, 4,               // 72588:         decode to UQRSHRNv4i16_shift using decoder 541
                                             // 72588:        }
                                             // 72588:        } // switch Inst[20]
                                             // 72588:       }
  1, 0,                                      // 72593:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 72595:        check predicate 76
  OPC_Decode, 224, 66, 159, 4,               // 72597:        decode to UQRSHRNv2i32_shift using decoder 543
                                             // 72597:       }
                                             // 72597:       } // switch Inst[21]
                                             // 72597:      }
  15, 0,                                     // 72602:      case 0xf: {
  OPC_SwitchField, 21, 1,                    // 72604:       switch Inst[21] {
  0, 11,                                     // 72607:       case 0x0: {
  OPC_CheckPredicate, 81,                    // 72609:        check predicate 81
  OPC_CheckField, 20, 1, 1,                  // 72611:        check Inst[20] == 0x1
  OPC_Decode, 218, 25, 146, 4,               // 72615:        decode to FCVTZUv4i16_shift using decoder 530
                                             // 72615:       }
  1, 0,                                      // 72620:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 72622:        check predicate 76
  OPC_Decode, 214, 25, 147, 4,               // 72624:        decode to FCVTZUv2i32_shift using decoder 531
                                             // 72624:       }
                                             // 72624:       } // switch Inst[21]
                                             // 72624:      }
                                             // 72624:      } // switch Inst[15:12]
                                             // 72624:     }
                                             // 72624:     } // switch Inst[11]
                                             // 72624:    }
                                             // 72624:    } // switch Inst[10]
                                             // 72624:   }
  2, 170, 5,                                 // 72629:   case 0x2: {
  OPC_SwitchField, 10, 1,                    // 72632:    switch Inst[10] {
  0, 48,                                     // 72635:    case 0x0: {
  OPC_SwitchField, 12, 4,                    // 72637:     switch Inst[15:12] {
  0, 7,                                      // 72640:     case 0x0: {
  OPC_CheckPredicate, 85,                    // 72642:      check predicate 85
  OPC_Decode, 151, 26, 165, 4,               // 72644:      decode to FDOTlanev4f32 using decoder 549
                                             // 72644:     }
  1, 7,                                      // 72649:     case 0x1: {
  OPC_CheckPredicate, 81,                    // 72651:      check predicate 81
  OPC_Decode, 207, 28, 166, 4,               // 72653:      decode to FMLAv8i16_indexed using decoder 550
                                             // 72653:     }
  5, 7,                                      // 72658:     case 0x5: {
  OPC_CheckPredicate, 81,                    // 72660:      check predicate 81
  OPC_Decode, 137, 29, 166, 4,               // 72662:      decode to FMLSv8i16_indexed using decoder 550
                                             // 72662:     }
  9, 7,                                      // 72667:     case 0x9: {
  OPC_CheckPredicate, 81,                    // 72669:      check predicate 81
  OPC_Decode, 154, 30, 167, 4,               // 72671:      decode to FMULv8i16_indexed using decoder 551
                                             // 72671:     }
  15, 0,                                     // 72676:     case 0xf: {
  OPC_CheckPredicate, 97,                    // 72678:      check predicate 97
  OPC_Decode, 205, 60, 165, 4,               // 72680:      decode to SUDOTlanev16i8 using decoder 549
                                             // 72680:     }
                                             // 72680:     } // switch Inst[15:12]
                                             // 72680:    }
  1, 0,                                      // 72685:    case 0x1: {
  OPC_SwitchField, 11, 1,                    // 72687:     switch Inst[11] {
  0, 237, 3,                                 // 72690:     case 0x0: {
  OPC_SwitchField, 15, 1,                    // 72693:      switch Inst[15] {
  0, 243, 1,                                 // 72696:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 72699:       switch Inst[12] {
  0, 91,                                     // 72702:       case 0x0: {
  OPC_SwitchField, 21, 1,                    // 72704:        switch Inst[21] {
  0, 63,                                     // 72707:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72709:         switch Inst[20] {
  0, 35,                                     // 72712:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 72714:          switch Inst[19] {
  0, 7,                                      // 72717:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72719:           check predicate 76
  OPC_Decode, 220, 42, 144, 4,               // 72721:           decode to MOVIv4i32 using decoder 528
                                             // 72721:          }
  1, 0,                                      // 72726:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72728:           switch Inst[14:13] {
  0, 7,                                      // 72731:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72733:            check predicate 76
  OPC_Decode, 200, 55, 168, 4,               // 72735:            decode to SSHRv16i8_shift using decoder 552
                                             // 72735:           }
  1, 0,                                      // 72740:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 72742:            check predicate 76
  OPC_Decode, 160, 55, 168, 4,               // 72744:            decode to SRSHRv16i8_shift using decoder 552
                                             // 72744:           }
                                             // 72744:           } // switch Inst[14:13]
                                             // 72744:          }
                                             // 72744:          } // switch Inst[19]
                                             // 72744:         }
  1, 0,                                      // 72749:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72751:          switch Inst[14:13] {
  0, 7,                                      // 72754:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72756:           check predicate 76
  OPC_Decode, 205, 55, 169, 4,               // 72758:           decode to SSHRv8i16_shift using decoder 553
                                             // 72758:          }
  1, 0,                                      // 72763:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 72765:           check predicate 76
  OPC_Decode, 165, 55, 169, 4,               // 72767:           decode to SRSHRv8i16_shift using decoder 553
                                             // 72767:          }
                                             // 72767:          } // switch Inst[14:13]
                                             // 72767:         }
                                             // 72767:         } // switch Inst[20]
                                             // 72767:        }
  1, 0,                                      // 72772:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72774:         switch Inst[14:13] {
  0, 7,                                      // 72777:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 72779:          check predicate 76
  OPC_Decode, 204, 55, 170, 4,               // 72781:          decode to SSHRv4i32_shift using decoder 554
                                             // 72781:         }
  1, 0,                                      // 72786:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 72788:          check predicate 76
  OPC_Decode, 164, 55, 170, 4,               // 72790:          decode to SRSHRv4i32_shift using decoder 554
                                             // 72790:         }
                                             // 72790:         } // switch Inst[14:13]
                                             // 72790:        }
                                             // 72790:        } // switch Inst[21]
                                             // 72790:       }
  1, 0,                                      // 72795:       case 0x1: {
  OPC_SwitchField, 21, 1,                    // 72797:        switch Inst[21] {
  0, 99,                                     // 72800:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72802:         switch Inst[20] {
  0, 53,                                     // 72805:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 72807:          switch Inst[19] {
  0, 7,                                      // 72810:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72812:           check predicate 76
  OPC_Decode, 218, 43, 148, 4,               // 72814:           decode to ORRv4i32 using decoder 532
                                             // 72814:          }
  1, 0,                                      // 72819:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72821:           switch Inst[14:13] {
  0, 7,                                      // 72824:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72826:            check predicate 76
  OPC_Decode, 212, 55, 171, 4,               // 72828:            decode to SSRAv16i8_shift using decoder 555
                                             // 72828:           }
  1, 7,                                      // 72833:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 72835:            check predicate 76
  OPC_Decode, 172, 55, 171, 4,               // 72837:            decode to SRSRAv16i8_shift using decoder 555
                                             // 72837:           }
  2, 7,                                      // 72842:           case 0x2: {
  OPC_CheckPredicate, 76,                    // 72844:            check predicate 76
  OPC_Decode, 162, 48, 172, 4,               // 72846:            decode to SHLv16i8_shift using decoder 556
                                             // 72846:           }
  3, 0,                                      // 72851:           case 0x3: {
  OPC_CheckPredicate, 76,                    // 72853:            check predicate 76
  OPC_Decode, 249, 53, 172, 4,               // 72855:            decode to SQSHLv16i8_shift using decoder 556
                                             // 72855:           }
                                             // 72855:           } // switch Inst[14:13]
                                             // 72855:          }
                                             // 72855:          } // switch Inst[19]
                                             // 72855:         }
  1, 0,                                      // 72860:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72862:          switch Inst[14:13] {
  0, 7,                                      // 72865:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 72867:           check predicate 76
  OPC_Decode, 217, 55, 173, 4,               // 72869:           decode to SSRAv8i16_shift using decoder 557
                                             // 72869:          }
  1, 7,                                      // 72874:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 72876:           check predicate 76
  OPC_Decode, 177, 55, 173, 4,               // 72878:           decode to SRSRAv8i16_shift using decoder 557
                                             // 72878:          }
  2, 7,                                      // 72883:          case 0x2: {
  OPC_CheckPredicate, 76,                    // 72885:           check predicate 76
  OPC_Decode, 167, 48, 174, 4,               // 72887:           decode to SHLv8i16_shift using decoder 558
                                             // 72887:          }
  3, 0,                                      // 72892:          case 0x3: {
  OPC_CheckPredicate, 76,                    // 72894:           check predicate 76
  OPC_Decode, 135, 54, 174, 4,               // 72896:           decode to SQSHLv8i16_shift using decoder 558
                                             // 72896:          }
                                             // 72896:          } // switch Inst[14:13]
                                             // 72896:         }
                                             // 72896:         } // switch Inst[20]
                                             // 72896:        }
  1, 0,                                      // 72901:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 72903:         switch Inst[14:13] {
  0, 7,                                      // 72906:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 72908:          check predicate 76
  OPC_Decode, 216, 55, 175, 4,               // 72910:          decode to SSRAv4i32_shift using decoder 559
                                             // 72910:         }
  1, 7,                                      // 72915:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 72917:          check predicate 76
  OPC_Decode, 176, 55, 175, 4,               // 72919:          decode to SRSRAv4i32_shift using decoder 559
                                             // 72919:         }
  2, 7,                                      // 72924:         case 0x2: {
  OPC_CheckPredicate, 76,                    // 72926:          check predicate 76
  OPC_Decode, 166, 48, 176, 4,               // 72928:          decode to SHLv4i32_shift using decoder 560
                                             // 72928:         }
  3, 0,                                      // 72933:         case 0x3: {
  OPC_CheckPredicate, 76,                    // 72935:          check predicate 76
  OPC_Decode, 133, 54, 176, 4,               // 72937:          decode to SQSHLv4i32_shift using decoder 560
                                             // 72937:         }
                                             // 72937:         } // switch Inst[14:13]
                                             // 72937:        }
                                             // 72937:        } // switch Inst[21]
                                             // 72937:       }
                                             // 72937:       } // switch Inst[12]
                                             // 72937:      }
  1, 0,                                      // 72942:      case 0x1: {
  OPC_SwitchField, 14, 1,                    // 72944:       switch Inst[14] {
  0, 159, 1,                                 // 72947:       case 0x0: {
  OPC_SwitchField, 12, 1,                    // 72950:        switch Inst[12] {
  0, 91,                                     // 72953:        case 0x0: {
  OPC_SwitchField, 21, 1,                    // 72955:         switch Inst[21] {
  0, 63,                                     // 72958:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 72960:          switch Inst[20] {
  0, 35,                                     // 72963:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 72965:           switch Inst[19] {
  0, 7,                                      // 72968:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 72970:            check predicate 76
  OPC_Decode, 223, 42, 144, 4,               // 72972:            decode to MOVIv8i16 using decoder 528
                                             // 72972:           }
  1, 0,                                      // 72977:           case 0x1: {
  OPC_SwitchField, 13, 1,                    // 72979:            switch Inst[13] {
  0, 7,                                      // 72982:            case 0x0: {
  OPC_CheckPredicate, 76,                    // 72984:             check predicate 76
  OPC_Decode, 175, 48, 177, 4,               // 72986:             decode to SHRNv16i8_shift using decoder 561
                                             // 72986:            }
  1, 0,                                      // 72991:            case 0x1: {
  OPC_CheckPredicate, 76,                    // 72993:             check predicate 76
  OPC_Decode, 185, 55, 172, 4,               // 72995:             decode to SSHLLv16i8_shift using decoder 556
                                             // 72995:            }
                                             // 72995:            } // switch Inst[13]
                                             // 72995:           }
                                             // 72995:           } // switch Inst[19]
                                             // 72995:          }
  1, 0,                                      // 73000:          case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73002:           switch Inst[13] {
  0, 7,                                      // 73005:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73007:            check predicate 76
  OPC_Decode, 179, 48, 178, 4,               // 73009:            decode to SHRNv8i16_shift using decoder 562
                                             // 73009:           }
  1, 0,                                      // 73014:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 73016:            check predicate 76
  OPC_Decode, 189, 55, 174, 4,               // 73018:            decode to SSHLLv8i16_shift using decoder 558
                                             // 73018:           }
                                             // 73018:           } // switch Inst[13]
                                             // 73018:          }
                                             // 73018:          } // switch Inst[20]
                                             // 73018:         }
  1, 0,                                      // 73023:         case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73025:          switch Inst[13] {
  0, 7,                                      // 73028:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 73030:           check predicate 76
  OPC_Decode, 178, 48, 179, 4,               // 73032:           decode to SHRNv4i32_shift using decoder 563
                                             // 73032:          }
  1, 0,                                      // 73037:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73039:           check predicate 76
  OPC_Decode, 188, 55, 176, 4,               // 73041:           decode to SSHLLv4i32_shift using decoder 560
                                             // 73041:          }
                                             // 73041:          } // switch Inst[13]
                                             // 73041:         }
                                             // 73041:         } // switch Inst[21]
                                             // 73041:        }
  1, 0,                                      // 73046:        case 0x1: {
  OPC_SwitchField, 21, 1,                    // 73048:         switch Inst[21] {
  0, 43,                                     // 73051:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73053:          switch Inst[20] {
  0, 25,                                     // 73056:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 73058:           switch Inst[19] {
  0, 7,                                      // 73061:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73063:            check predicate 76
  OPC_Decode, 219, 43, 148, 4,               // 73065:            decode to ORRv8i16 using decoder 532
                                             // 73065:           }
  1, 0,                                      // 73070:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 73072:            check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 73074:            check Inst[13] == 0x0
  OPC_Decode, 149, 54, 177, 4,               // 73078:            decode to SQSHRNv16i8_shift using decoder 561
                                             // 73078:           }
                                             // 73078:           } // switch Inst[19]
                                             // 73078:          }
  1, 0,                                      // 73083:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73085:           check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 73087:           check Inst[13] == 0x0
  OPC_Decode, 153, 54, 178, 4,               // 73091:           decode to SQSHRNv8i16_shift using decoder 562
                                             // 73091:          }
                                             // 73091:          } // switch Inst[20]
                                             // 73091:         }
  1, 0,                                      // 73096:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 73098:          check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 73100:          check Inst[13] == 0x0
  OPC_Decode, 152, 54, 179, 4,               // 73104:          decode to SQSHRNv4i32_shift using decoder 563
                                             // 73104:         }
                                             // 73104:         } // switch Inst[21]
                                             // 73104:        }
                                             // 73104:        } // switch Inst[12]
                                             // 73104:       }
  1, 0,                                      // 73109:       case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73111:        switch Inst[13] {
  0, 11,                                     // 73114:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 73116:         check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 73118:         check Inst[21:19] == 0x0
  OPC_Decode, 221, 42, 144, 4,               // 73122:         decode to MOVIv4s_msl using decoder 528
                                             // 73122:        }
  1, 0,                                      // 73127:        case 0x1: {
  OPC_SwitchField, 12, 1,                    // 73129:         switch Inst[12] {
  0, 39,                                     // 73132:         case 0x0: {
  OPC_SwitchField, 21, 1,                    // 73134:          switch Inst[21] {
  0, 25,                                     // 73137:          case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73139:           switch Inst[20] {
  0, 11,                                     // 73142:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73144:            check predicate 76
  OPC_CheckField, 19, 1, 0,                  // 73146:            check Inst[19] == 0x0
  OPC_Decode, 215, 42, 144, 4,               // 73150:            decode to MOVIv16b_ns using decoder 528
                                             // 73150:           }
  1, 0,                                      // 73155:           case 0x1: {
  OPC_CheckPredicate, 81,                    // 73157:            check predicate 81
  OPC_Decode, 174, 47, 169, 4,               // 73159:            decode to SCVTFv8i16_shift using decoder 553
                                             // 73159:           }
                                             // 73159:           } // switch Inst[20]
                                             // 73159:          }
  1, 0,                                      // 73164:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73166:           check predicate 76
  OPC_Decode, 172, 47, 170, 4,               // 73168:           decode to SCVTFv4i32_shift using decoder 554
                                             // 73168:          }
                                             // 73168:          } // switch Inst[21]
                                             // 73168:         }
  1, 0,                                      // 73173:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 73175:          check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 73177:          check Inst[21:19] == 0x0
  OPC_Decode, 216, 29, 144, 4,               // 73181:          decode to FMOVv4f32_ns using decoder 528
                                             // 73181:         }
                                             // 73181:         } // switch Inst[12]
                                             // 73181:        }
                                             // 73181:        } // switch Inst[13]
                                             // 73181:       }
                                             // 73181:       } // switch Inst[14]
                                             // 73181:      }
                                             // 73181:      } // switch Inst[15]
                                             // 73181:     }
  1, 0,                                      // 73186:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 73188:      switch Inst[15:12] {
  8, 39,                                     // 73191:      case 0x8: {
  OPC_SwitchField, 21, 1,                    // 73193:       switch Inst[21] {
  0, 25,                                     // 73196:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73198:        switch Inst[20] {
  0, 11,                                     // 73201:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 73203:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 73205:         check Inst[19] == 0x1
  OPC_Decode, 239, 45, 177, 4,               // 73209:         decode to RSHRNv16i8_shift using decoder 561
                                             // 73209:        }
  1, 0,                                      // 73214:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 73216:         check predicate 76
  OPC_Decode, 243, 45, 178, 4,               // 73218:         decode to RSHRNv8i16_shift using decoder 562
                                             // 73218:        }
                                             // 73218:        } // switch Inst[20]
                                             // 73218:       }
  1, 0,                                      // 73223:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 73225:        check predicate 76
  OPC_Decode, 242, 45, 179, 4,               // 73227:        decode to RSHRNv4i32_shift using decoder 563
                                             // 73227:       }
                                             // 73227:       } // switch Inst[21]
                                             // 73227:      }
  9, 39,                                     // 73232:      case 0x9: {
  OPC_SwitchField, 21, 1,                    // 73234:       switch Inst[21] {
  0, 25,                                     // 73237:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73239:        switch Inst[20] {
  0, 11,                                     // 73242:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 73244:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 73246:         check Inst[19] == 0x1
  OPC_Decode, 186, 53, 177, 4,               // 73250:         decode to SQRSHRNv16i8_shift using decoder 561
                                             // 73250:        }
  1, 0,                                      // 73255:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 73257:         check predicate 76
  OPC_Decode, 190, 53, 178, 4,               // 73259:         decode to SQRSHRNv8i16_shift using decoder 562
                                             // 73259:        }
                                             // 73259:        } // switch Inst[20]
                                             // 73259:       }
  1, 0,                                      // 73264:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 73266:        check predicate 76
  OPC_Decode, 189, 53, 179, 4,               // 73268:        decode to SQRSHRNv4i32_shift using decoder 563
                                             // 73268:       }
                                             // 73268:       } // switch Inst[21]
                                             // 73268:      }
  15, 0,                                     // 73273:      case 0xf: {
  OPC_SwitchField, 21, 1,                    // 73275:       switch Inst[21] {
  0, 25,                                     // 73278:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73280:        switch Inst[20] {
  0, 11,                                     // 73283:        case 0x0: {
  OPC_CheckPredicate, 81,                    // 73285:         check predicate 81
  OPC_CheckField, 19, 1, 0,                  // 73287:         check Inst[19] == 0x0
  OPC_Decode, 217, 29, 144, 4,               // 73291:         decode to FMOVv8f16_ns using decoder 528
                                             // 73291:        }
  1, 0,                                      // 73296:        case 0x1: {
  OPC_CheckPredicate, 81,                    // 73298:         check predicate 81
  OPC_Decode, 170, 25, 169, 4,               // 73300:         decode to FCVTZSv8i16_shift using decoder 553
                                             // 73300:        }
                                             // 73300:        } // switch Inst[20]
                                             // 73300:       }
  1, 0,                                      // 73305:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 73307:        check predicate 76
  OPC_Decode, 168, 25, 170, 4,               // 73309:        decode to FCVTZSv4i32_shift using decoder 554
                                             // 73309:       }
                                             // 73309:       } // switch Inst[21]
                                             // 73309:      }
                                             // 73309:      } // switch Inst[15:12]
                                             // 73309:     }
                                             // 73309:     } // switch Inst[11]
                                             // 73309:    }
                                             // 73309:    } // switch Inst[10]
                                             // 73309:   }
  3, 0,                                      // 73314:   case 0x3: {
  OPC_SwitchField, 10, 1,                    // 73316:    switch Inst[10] {
  0, 21,                                     // 73319:    case 0x0: {
  OPC_SwitchField, 12, 4,                    // 73321:     switch Inst[15:12] {
  8, 7,                                      // 73324:     case 0x8: {
  OPC_CheckPredicate, 80,                    // 73326:      check predicate 80
  OPC_Decode, 131, 28, 161, 4,               // 73328:      decode to FMLALLTBlanev4f32 using decoder 545
                                             // 73328:     }
  9, 0,                                      // 73333:     case 0x9: {
  OPC_CheckPredicate, 81,                    // 73335:      check predicate 81
  OPC_Decode, 245, 29, 167, 4,               // 73337:      decode to FMULXv8i16_indexed using decoder 551
                                             // 73337:     }
                                             // 73337:     } // switch Inst[15:12]
                                             // 73337:    }
  1, 0,                                      // 73342:    case 0x1: {
  OPC_SwitchField, 11, 1,                    // 73344:     switch Inst[11] {
  0, 164, 4,                                 // 73347:     case 0x0: {
  OPC_SwitchField, 15, 1,                    // 73350:      switch Inst[15] {
  0, 170, 2,                                 // 73353:      case 0x0: {
  OPC_SwitchField, 12, 1,                    // 73356:       switch Inst[12] {
  0, 145, 1,                                 // 73359:       case 0x0: {
  OPC_SwitchField, 21, 1,                    // 73362:        switch Inst[21] {
  0, 99,                                     // 73365:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73367:         switch Inst[20] {
  0, 53,                                     // 73370:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 73372:          switch Inst[19] {
  0, 7,                                      // 73375:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 73377:           check predicate 76
  OPC_Decode, 156, 43, 144, 4,               // 73379:           decode to MVNIv4i32 using decoder 528
                                             // 73379:          }
  1, 0,                                      // 73384:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 73386:           switch Inst[14:13] {
  0, 7,                                      // 73389:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73391:            check predicate 76
  OPC_Decode, 176, 68, 168, 4,               // 73393:            decode to USHRv16i8_shift using decoder 552
                                             // 73393:           }
  1, 7,                                      // 73398:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 73400:            check predicate 76
  OPC_Decode, 248, 67, 168, 4,               // 73402:            decode to URSHRv16i8_shift using decoder 552
                                             // 73402:           }
  2, 7,                                      // 73407:           case 0x2: {
  OPC_CheckPredicate, 76,                    // 73409:            check predicate 76
  OPC_Decode, 244, 54, 171, 4,               // 73411:            decode to SRIv16i8_shift using decoder 555
                                             // 73411:           }
  3, 0,                                      // 73416:           case 0x3: {
  OPC_CheckPredicate, 76,                    // 73418:            check predicate 76
  OPC_Decode, 229, 53, 172, 4,               // 73420:            decode to SQSHLUv16i8_shift using decoder 556
                                             // 73420:           }
                                             // 73420:           } // switch Inst[14:13]
                                             // 73420:          }
                                             // 73420:          } // switch Inst[19]
                                             // 73420:         }
  1, 0,                                      // 73425:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 73427:          switch Inst[14:13] {
  0, 7,                                      // 73430:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 73432:           check predicate 76
  OPC_Decode, 181, 68, 169, 4,               // 73434:           decode to USHRv8i16_shift using decoder 553
                                             // 73434:          }
  1, 7,                                      // 73439:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73441:           check predicate 76
  OPC_Decode, 253, 67, 169, 4,               // 73443:           decode to URSHRv8i16_shift using decoder 553
                                             // 73443:          }
  2, 7,                                      // 73448:          case 0x2: {
  OPC_CheckPredicate, 76,                    // 73450:           check predicate 76
  OPC_Decode, 249, 54, 173, 4,               // 73452:           decode to SRIv8i16_shift using decoder 557
                                             // 73452:          }
  3, 0,                                      // 73457:          case 0x3: {
  OPC_CheckPredicate, 76,                    // 73459:           check predicate 76
  OPC_Decode, 234, 53, 174, 4,               // 73461:           decode to SQSHLUv8i16_shift using decoder 558
                                             // 73461:          }
                                             // 73461:          } // switch Inst[14:13]
                                             // 73461:         }
                                             // 73461:         } // switch Inst[20]
                                             // 73461:        }
  1, 0,                                      // 73466:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 73468:         switch Inst[14:13] {
  0, 7,                                      // 73471:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 73473:          check predicate 76
  OPC_Decode, 180, 68, 170, 4,               // 73475:          decode to USHRv4i32_shift using decoder 554
                                             // 73475:         }
  1, 7,                                      // 73480:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 73482:          check predicate 76
  OPC_Decode, 252, 67, 170, 4,               // 73484:          decode to URSHRv4i32_shift using decoder 554
                                             // 73484:         }
  2, 7,                                      // 73489:         case 0x2: {
  OPC_CheckPredicate, 76,                    // 73491:          check predicate 76
  OPC_Decode, 248, 54, 175, 4,               // 73493:          decode to SRIv4i32_shift using decoder 559
                                             // 73493:         }
  3, 0,                                      // 73498:         case 0x3: {
  OPC_CheckPredicate, 76,                    // 73500:          check predicate 76
  OPC_Decode, 233, 53, 176, 4,               // 73502:          decode to SQSHLUv4i32_shift using decoder 560
                                             // 73502:         }
                                             // 73502:         } // switch Inst[14:13]
                                             // 73502:        }
                                             // 73502:        } // switch Inst[21]
                                             // 73502:       }
  1, 0,                                      // 73507:       case 0x1: {
  OPC_SwitchField, 21, 1,                    // 73509:        switch Inst[21] {
  0, 99,                                     // 73512:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73514:         switch Inst[20] {
  0, 53,                                     // 73517:         case 0x0: {
  OPC_SwitchField, 19, 1,                    // 73519:          switch Inst[19] {
  0, 7,                                      // 73522:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 73524:           check predicate 76
  OPC_Decode, 147, 16, 148, 4,               // 73526:           decode to BICv4i32 using decoder 532
                                             // 73526:          }
  1, 0,                                      // 73531:          case 0x1: {
  OPC_SwitchField, 13, 2,                    // 73533:           switch Inst[14:13] {
  0, 7,                                      // 73536:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73538:            check predicate 76
  OPC_Decode, 233, 68, 171, 4,               // 73540:            decode to USRAv16i8_shift using decoder 555
                                             // 73540:           }
  1, 7,                                      // 73545:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 73547:            check predicate 76
  OPC_Decode, 136, 68, 171, 4,               // 73549:            decode to URSRAv16i8_shift using decoder 555
                                             // 73549:           }
  2, 7,                                      // 73554:           case 0x2: {
  OPC_CheckPredicate, 76,                    // 73556:            check predicate 76
  OPC_Decode, 201, 48, 180, 4,               // 73558:            decode to SLIv16i8_shift using decoder 564
                                             // 73558:           }
  3, 0,                                      // 73563:           case 0x3: {
  OPC_CheckPredicate, 76,                    // 73565:            check predicate 76
  OPC_Decode, 249, 66, 172, 4,               // 73567:            decode to UQSHLv16i8_shift using decoder 556
                                             // 73567:           }
                                             // 73567:           } // switch Inst[14:13]
                                             // 73567:          }
                                             // 73567:          } // switch Inst[19]
                                             // 73567:         }
  1, 0,                                      // 73572:         case 0x1: {
  OPC_SwitchField, 13, 2,                    // 73574:          switch Inst[14:13] {
  0, 7,                                      // 73577:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 73579:           check predicate 76
  OPC_Decode, 238, 68, 173, 4,               // 73581:           decode to USRAv8i16_shift using decoder 557
                                             // 73581:          }
  1, 7,                                      // 73586:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73588:           check predicate 76
  OPC_Decode, 141, 68, 173, 4,               // 73590:           decode to URSRAv8i16_shift using decoder 557
                                             // 73590:          }
  2, 7,                                      // 73595:          case 0x2: {
  OPC_CheckPredicate, 76,                    // 73597:           check predicate 76
  OPC_Decode, 206, 48, 181, 4,               // 73599:           decode to SLIv8i16_shift using decoder 565
                                             // 73599:          }
  3, 0,                                      // 73604:          case 0x3: {
  OPC_CheckPredicate, 76,                    // 73606:           check predicate 76
  OPC_Decode, 135, 67, 174, 4,               // 73608:           decode to UQSHLv8i16_shift using decoder 558
                                             // 73608:          }
                                             // 73608:          } // switch Inst[14:13]
                                             // 73608:         }
                                             // 73608:         } // switch Inst[20]
                                             // 73608:        }
  1, 0,                                      // 73613:        case 0x1: {
  OPC_SwitchField, 13, 2,                    // 73615:         switch Inst[14:13] {
  0, 7,                                      // 73618:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 73620:          check predicate 76
  OPC_Decode, 237, 68, 175, 4,               // 73622:          decode to USRAv4i32_shift using decoder 559
                                             // 73622:         }
  1, 7,                                      // 73627:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 73629:          check predicate 76
  OPC_Decode, 140, 68, 175, 4,               // 73631:          decode to URSRAv4i32_shift using decoder 559
                                             // 73631:         }
  2, 7,                                      // 73636:         case 0x2: {
  OPC_CheckPredicate, 76,                    // 73638:          check predicate 76
  OPC_Decode, 205, 48, 182, 4,               // 73640:          decode to SLIv4i32_shift using decoder 566
                                             // 73640:         }
  3, 0,                                      // 73645:         case 0x3: {
  OPC_CheckPredicate, 76,                    // 73647:          check predicate 76
  OPC_Decode, 133, 67, 176, 4,               // 73649:          decode to UQSHLv4i32_shift using decoder 560
                                             // 73649:         }
                                             // 73649:         } // switch Inst[14:13]
                                             // 73649:        }
                                             // 73649:        } // switch Inst[21]
                                             // 73649:       }
                                             // 73649:       } // switch Inst[12]
                                             // 73649:      }
  1, 0,                                      // 73654:      case 0x1: {
  OPC_SwitchField, 14, 1,                    // 73656:       switch Inst[14] {
  0, 159, 1,                                 // 73659:       case 0x0: {
  OPC_SwitchField, 12, 1,                    // 73662:        switch Inst[12] {
  0, 91,                                     // 73665:        case 0x0: {
  OPC_SwitchField, 21, 1,                    // 73667:         switch Inst[21] {
  0, 63,                                     // 73670:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73672:          switch Inst[20] {
  0, 35,                                     // 73675:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 73677:           switch Inst[19] {
  0, 7,                                      // 73680:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73682:            check predicate 76
  OPC_Decode, 158, 43, 144, 4,               // 73684:            decode to MVNIv8i16 using decoder 528
                                             // 73684:           }
  1, 0,                                      // 73689:           case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73691:            switch Inst[13] {
  0, 7,                                      // 73694:            case 0x0: {
  OPC_CheckPredicate, 76,                    // 73696:             check predicate 76
  OPC_Decode, 166, 54, 177, 4,               // 73698:             decode to SQSHRUNv16i8_shift using decoder 561
                                             // 73698:            }
  1, 0,                                      // 73703:            case 0x1: {
  OPC_CheckPredicate, 76,                    // 73705:             check predicate 76
  OPC_Decode, 161, 68, 172, 4,               // 73707:             decode to USHLLv16i8_shift using decoder 556
                                             // 73707:            }
                                             // 73707:            } // switch Inst[13]
                                             // 73707:           }
                                             // 73707:           } // switch Inst[19]
                                             // 73707:          }
  1, 0,                                      // 73712:          case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73714:           switch Inst[13] {
  0, 7,                                      // 73717:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73719:            check predicate 76
  OPC_Decode, 170, 54, 178, 4,               // 73721:            decode to SQSHRUNv8i16_shift using decoder 562
                                             // 73721:           }
  1, 0,                                      // 73726:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 73728:            check predicate 76
  OPC_Decode, 165, 68, 174, 4,               // 73730:            decode to USHLLv8i16_shift using decoder 558
                                             // 73730:           }
                                             // 73730:           } // switch Inst[13]
                                             // 73730:          }
                                             // 73730:          } // switch Inst[20]
                                             // 73730:         }
  1, 0,                                      // 73735:         case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73737:          switch Inst[13] {
  0, 7,                                      // 73740:          case 0x0: {
  OPC_CheckPredicate, 76,                    // 73742:           check predicate 76
  OPC_Decode, 169, 54, 179, 4,               // 73744:           decode to SQSHRUNv4i32_shift using decoder 563
                                             // 73744:          }
  1, 0,                                      // 73749:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73751:           check predicate 76
  OPC_Decode, 164, 68, 176, 4,               // 73753:           decode to USHLLv4i32_shift using decoder 560
                                             // 73753:          }
                                             // 73753:          } // switch Inst[13]
                                             // 73753:         }
                                             // 73753:         } // switch Inst[21]
                                             // 73753:        }
  1, 0,                                      // 73758:        case 0x1: {
  OPC_SwitchField, 21, 1,                    // 73760:         switch Inst[21] {
  0, 43,                                     // 73763:         case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73765:          switch Inst[20] {
  0, 25,                                     // 73768:          case 0x0: {
  OPC_SwitchField, 19, 1,                    // 73770:           switch Inst[19] {
  0, 7,                                      // 73773:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73775:            check predicate 76
  OPC_Decode, 148, 16, 148, 4,               // 73777:            decode to BICv8i16 using decoder 532
                                             // 73777:           }
  1, 0,                                      // 73782:           case 0x1: {
  OPC_CheckPredicate, 76,                    // 73784:            check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 73786:            check Inst[13] == 0x0
  OPC_Decode, 149, 67, 177, 4,               // 73790:            decode to UQSHRNv16i8_shift using decoder 561
                                             // 73790:           }
                                             // 73790:           } // switch Inst[19]
                                             // 73790:          }
  1, 0,                                      // 73795:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73797:           check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 73799:           check Inst[13] == 0x0
  OPC_Decode, 153, 67, 178, 4,               // 73803:           decode to UQSHRNv8i16_shift using decoder 562
                                             // 73803:          }
                                             // 73803:          } // switch Inst[20]
                                             // 73803:         }
  1, 0,                                      // 73808:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 73810:          check predicate 76
  OPC_CheckField, 13, 1, 0,                  // 73812:          check Inst[13] == 0x0
  OPC_Decode, 152, 67, 179, 4,               // 73816:          decode to UQSHRNv4i32_shift using decoder 563
                                             // 73816:         }
                                             // 73816:         } // switch Inst[21]
                                             // 73816:        }
                                             // 73816:        } // switch Inst[12]
                                             // 73816:       }
  1, 0,                                      // 73821:       case 0x1: {
  OPC_SwitchField, 13, 1,                    // 73823:        switch Inst[13] {
  0, 11,                                     // 73826:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 73828:         check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 73830:         check Inst[21:19] == 0x0
  OPC_Decode, 157, 43, 144, 4,               // 73834:         decode to MVNIv4s_msl using decoder 528
                                             // 73834:        }
  1, 0,                                      // 73839:        case 0x1: {
  OPC_SwitchField, 12, 1,                    // 73841:         switch Inst[12] {
  0, 39,                                     // 73844:         case 0x0: {
  OPC_SwitchField, 21, 1,                    // 73846:          switch Inst[21] {
  0, 25,                                     // 73849:          case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73851:           switch Inst[20] {
  0, 11,                                     // 73854:           case 0x0: {
  OPC_CheckPredicate, 76,                    // 73856:            check predicate 76
  OPC_CheckField, 19, 1, 0,                  // 73858:            check Inst[19] == 0x0
  OPC_Decode, 216, 42, 144, 4,               // 73862:            decode to MOVIv2d_ns using decoder 528
                                             // 73862:           }
  1, 0,                                      // 73867:           case 0x1: {
  OPC_CheckPredicate, 81,                    // 73869:            check predicate 81
  OPC_Decode, 167, 63, 169, 4,               // 73871:            decode to UCVTFv8i16_shift using decoder 553
                                             // 73871:           }
                                             // 73871:           } // switch Inst[20]
                                             // 73871:          }
  1, 0,                                      // 73876:          case 0x1: {
  OPC_CheckPredicate, 76,                    // 73878:           check predicate 76
  OPC_Decode, 165, 63, 170, 4,               // 73880:           decode to UCVTFv4i32_shift using decoder 554
                                             // 73880:          }
                                             // 73880:          } // switch Inst[21]
                                             // 73880:         }
  1, 0,                                      // 73885:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 73887:          check predicate 76
  OPC_CheckField, 19, 3, 0,                  // 73889:          check Inst[21:19] == 0x0
  OPC_Decode, 214, 29, 144, 4,               // 73893:          decode to FMOVv2f64_ns using decoder 528
                                             // 73893:         }
                                             // 73893:         } // switch Inst[12]
                                             // 73893:        }
                                             // 73893:        } // switch Inst[13]
                                             // 73893:       }
                                             // 73893:       } // switch Inst[14]
                                             // 73893:      }
                                             // 73893:      } // switch Inst[15]
                                             // 73893:     }
  1, 0,                                      // 73898:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 73900:      switch Inst[15:12] {
  8, 39,                                     // 73903:      case 0x8: {
  OPC_SwitchField, 21, 1,                    // 73905:       switch Inst[21] {
  0, 25,                                     // 73908:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73910:        switch Inst[20] {
  0, 11,                                     // 73913:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 73915:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 73917:         check Inst[19] == 0x1
  OPC_Decode, 205, 53, 177, 4,               // 73921:         decode to SQRSHRUNv16i8_shift using decoder 561
                                             // 73921:        }
  1, 0,                                      // 73926:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 73928:         check predicate 76
  OPC_Decode, 209, 53, 178, 4,               // 73930:         decode to SQRSHRUNv8i16_shift using decoder 562
                                             // 73930:        }
                                             // 73930:        } // switch Inst[20]
                                             // 73930:       }
  1, 0,                                      // 73935:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 73937:        check predicate 76
  OPC_Decode, 208, 53, 179, 4,               // 73939:        decode to SQRSHRUNv4i32_shift using decoder 563
                                             // 73939:       }
                                             // 73939:       } // switch Inst[21]
                                             // 73939:      }
  9, 39,                                     // 73944:      case 0x9: {
  OPC_SwitchField, 21, 1,                    // 73946:       switch Inst[21] {
  0, 25,                                     // 73949:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 73951:        switch Inst[20] {
  0, 11,                                     // 73954:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 73956:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 73958:         check Inst[19] == 0x1
  OPC_Decode, 223, 66, 177, 4,               // 73962:         decode to UQRSHRNv16i8_shift using decoder 561
                                             // 73962:        }
  1, 0,                                      // 73967:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 73969:         check predicate 76
  OPC_Decode, 227, 66, 178, 4,               // 73971:         decode to UQRSHRNv8i16_shift using decoder 562
                                             // 73971:        }
                                             // 73971:        } // switch Inst[20]
                                             // 73971:       }
  1, 0,                                      // 73976:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 73978:        check predicate 76
  OPC_Decode, 226, 66, 179, 4,               // 73980:        decode to UQRSHRNv4i32_shift using decoder 563
                                             // 73980:       }
                                             // 73980:       } // switch Inst[21]
                                             // 73980:      }
  15, 0,                                     // 73985:      case 0xf: {
  OPC_SwitchField, 21, 1,                    // 73987:       switch Inst[21] {
  0, 11,                                     // 73990:       case 0x0: {
  OPC_CheckPredicate, 81,                    // 73992:        check predicate 81
  OPC_CheckField, 20, 1, 1,                  // 73994:        check Inst[20] == 0x1
  OPC_Decode, 221, 25, 169, 4,               // 73998:        decode to FCVTZUv8i16_shift using decoder 553
                                             // 73998:       }
  1, 0,                                      // 74003:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 74005:        check predicate 76
  OPC_Decode, 219, 25, 170, 4,               // 74007:        decode to FCVTZUv4i32_shift using decoder 554
                                             // 74007:       }
                                             // 74007:       } // switch Inst[21]
                                             // 74007:      }
                                             // 74007:      } // switch Inst[15:12]
                                             // 74007:     }
                                             // 74007:     } // switch Inst[11]
                                             // 74007:    }
                                             // 74007:    } // switch Inst[10]
                                             // 74007:   }
                                             // 74007:   } // switch Inst[31:29]
                                             // 74007:  }
  13, 135, 6,                                // 74012:  case 0xd: {
  OPC_SwitchField, 29, 3,                    // 74015:   switch Inst[31:29] {
  0, 159, 1,                                 // 74018:   case 0x0: {
  OPC_SwitchField, 12, 4,                    // 74021:    switch Inst[15:12] {
  0, 11,                                     // 74024:    case 0x0: {
  OPC_CheckPredicate, 89,                    // 74026:     check predicate 89
  OPC_CheckField, 10, 1, 0,                  // 74028:     check Inst[10] == 0x0
  OPC_Decode, 149, 26, 142, 4,               // 74032:     decode to FDOTlanev4f16 using decoder 526
                                             // 74032:    }
  2, 11,                                     // 74037:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 74039:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74041:     check Inst[10] == 0x0
  OPC_Decode, 244, 49, 183, 4,               // 74045:     decode to SMLALv4i16_indexed using decoder 567
                                             // 74045:    }
  3, 11,                                     // 74050:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 74052:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74054:     check Inst[10] == 0x0
  OPC_Decode, 218, 51, 183, 4,               // 74058:     decode to SQDMLALv4i16_indexed using decoder 567
                                             // 74058:    }
  6, 11,                                     // 74063:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 74065:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74067:     check Inst[10] == 0x0
  OPC_Decode, 160, 50, 183, 4,               // 74071:     decode to SMLSLv4i16_indexed using decoder 567
                                             // 74071:    }
  7, 11,                                     // 74076:    case 0x7: {
  OPC_CheckPredicate, 76,                    // 74078:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74080:     check Inst[10] == 0x0
  OPC_Decode, 243, 51, 183, 4,               // 74084:     decode to SQDMLSLv4i16_indexed using decoder 567
                                             // 74084:    }
  8, 11,                                     // 74089:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 74091:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74093:     check Inst[10] == 0x0
  OPC_Decode, 147, 43, 143, 4,               // 74097:     decode to MULv4i16_indexed using decoder 527
                                             // 74097:    }
  9, 11,                                     // 74102:    case 0x9: {
  OPC_CheckPredicate, 99,                    // 74104:     check predicate 99
  OPC_CheckField, 10, 1, 0,                  // 74106:     check Inst[10] == 0x0
  OPC_Decode, 150, 26, 141, 4,               // 74110:     decode to FDOTlanev4f16_v2f32 using decoder 525
                                             // 74110:    }
  10, 11,                                    // 74115:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 74117:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74119:     check Inst[10] == 0x0
  OPC_Decode, 232, 50, 184, 4,               // 74123:     decode to SMULLv4i16_indexed using decoder 568
                                             // 74123:    }
  11, 11,                                    // 74128:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 74130:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74132:     check Inst[10] == 0x0
  OPC_Decode, 172, 52, 184, 4,               // 74136:     decode to SQDMULLv4i16_indexed using decoder 568
                                             // 74136:    }
  12, 11,                                    // 74141:    case 0xc: {
  OPC_CheckPredicate, 76,                    // 74143:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74145:     check Inst[10] == 0x0
  OPC_Decode, 151, 52, 143, 4,               // 74149:     decode to SQDMULHv4i16_indexed using decoder 527
                                             // 74149:    }
  13, 11,                                    // 74154:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 74156:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74158:     check Inst[10] == 0x0
  OPC_Decode, 149, 53, 143, 4,               // 74162:     decode to SQRDMULHv4i16_indexed using decoder 527
                                             // 74162:    }
  15, 0,                                     // 74167:    case 0xf: {
  OPC_CheckPredicate, 92,                    // 74169:     check predicate 92
  OPC_CheckField, 10, 1, 0,                  // 74171:     check Inst[10] == 0x0
  OPC_Decode, 235, 14, 141, 4,               // 74175:     decode to BF16DOTlanev4bf16 using decoder 525
                                             // 74175:    }
                                             // 74175:    } // switch Inst[15:12]
                                             // 74175:   }
  1, 135, 1,                                 // 74180:   case 0x1: {
  OPC_SwitchField, 12, 1,                    // 74183:    switch Inst[12] {
  0, 81,                                     // 74186:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 74188:     switch Inst[15:13] {
  0, 11,                                     // 74191:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 74193:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74195:      check Inst[10] == 0x0
  OPC_Decode, 251, 41, 142, 4,               // 74199:      decode to MLAv4i16_indexed using decoder 526
                                             // 74199:     }
  1, 11,                                     // 74204:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 74206:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74208:      check Inst[10] == 0x0
  OPC_Decode, 253, 64, 183, 4,               // 74212:      decode to UMLALv4i16_indexed using decoder 567
                                             // 74212:     }
  2, 11,                                     // 74217:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 74219:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74221:      check Inst[10] == 0x0
  OPC_Decode, 140, 42, 142, 4,               // 74225:      decode to MLSv4i16_indexed using decoder 526
                                             // 74225:     }
  3, 11,                                     // 74230:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 74232:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74234:      check Inst[10] == 0x0
  OPC_Decode, 169, 65, 183, 4,               // 74238:      decode to UMLSLv4i16_indexed using decoder 567
                                             // 74238:     }
  4, 11,                                     // 74243:     case 0x4: {
  OPC_CheckPredicate, 80,                    // 74245:      check predicate 80
  OPC_CheckField, 10, 1, 0,                  // 74247:      check Inst[10] == 0x0
  OPC_Decode, 255, 27, 161, 4,               // 74251:      decode to FMLALLBTlanev4f32 using decoder 545
                                             // 74251:     }
  5, 0,                                      // 74256:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 74258:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74260:      check Inst[10] == 0x0
  OPC_Decode, 239, 65, 184, 4,               // 74264:      decode to UMULLv4i16_indexed using decoder 568
                                             // 74264:     }
                                             // 74264:     } // switch Inst[15:13]
                                             // 74264:    }
  1, 0,                                      // 74269:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 74271:     switch Inst[15] {
  0, 11,                                     // 74274:     case 0x0: {
  OPC_CheckPredicate, 91,                    // 74276:      check predicate 91
  OPC_CheckField, 10, 2, 0,                  // 74278:      check Inst[11:10] == 0x0
  OPC_Decode, 139, 23, 185, 4,               // 74282:      decode to FCMLAv4f16_indexed using decoder 569
                                             // 74282:     }
  1, 0,                                      // 74287:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 74289:      switch Inst[14:13] {
  2, 11,                                     // 74292:      case 0x2: {
  OPC_CheckPredicate, 90,                    // 74294:       check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 74296:       check Inst[10] == 0x0
  OPC_Decode, 239, 52, 142, 4,               // 74300:       decode to SQRDMLAHv4i16_indexed using decoder 526
                                             // 74300:      }
  3, 0,                                      // 74305:      case 0x3: {
  OPC_CheckPredicate, 90,                    // 74307:       check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 74309:       check Inst[10] == 0x0
  OPC_Decode, 130, 53, 142, 4,               // 74313:       decode to SQRDMLSHv4i16_indexed using decoder 526
                                             // 74313:      }
                                             // 74313:      } // switch Inst[14:13]
                                             // 74313:     }
                                             // 74313:     } // switch Inst[15]
                                             // 74313:    }
                                             // 74313:    } // switch Inst[12]
                                             // 74313:   }
  2, 140, 2,                                 // 74318:   case 0x2: {
  OPC_SwitchField, 12, 4,                    // 74321:    switch Inst[15:12] {
  0, 25,                                     // 74324:    case 0x0: {
  OPC_SwitchField, 10, 1,                    // 74326:     switch Inst[10] {
  0, 7,                                      // 74329:     case 0x0: {
  OPC_CheckPredicate, 89,                    // 74331:      check predicate 89
  OPC_Decode, 152, 26, 166, 4,               // 74333:      decode to FDOTlanev8f16 using decoder 550
                                             // 74333:     }
  1, 0,                                      // 74338:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 74340:      check predicate 76
  OPC_CheckField, 11, 1, 0,                  // 74342:      check Inst[11] == 0x0
  OPC_Decode, 202, 55, 186, 4,               // 74346:      decode to SSHRv2i64_shift using decoder 570
                                             // 74346:     }
                                             // 74346:     } // switch Inst[10]
                                             // 74346:    }
  1, 11,                                     // 74351:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 74353:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 74355:     check Inst[11:10] == 0x1
  OPC_Decode, 214, 55, 187, 4,               // 74359:     decode to SSRAv2i64_shift using decoder 571
                                             // 74359:    }
  2, 25,                                     // 74364:    case 0x2: {
  OPC_SwitchField, 10, 1,                    // 74366:     switch Inst[10] {
  0, 7,                                      // 74369:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 74371:      check predicate 76
  OPC_Decode, 248, 49, 166, 4,               // 74373:      decode to SMLALv8i16_indexed using decoder 550
                                             // 74373:     }
  1, 0,                                      // 74378:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 74380:      check predicate 76
  OPC_CheckField, 11, 1, 0,                  // 74382:      check Inst[11] == 0x0
  OPC_Decode, 162, 55, 186, 4,               // 74386:      decode to SRSHRv2i64_shift using decoder 570
                                             // 74386:     }
                                             // 74386:     } // switch Inst[10]
                                             // 74386:    }
  3, 25,                                     // 74391:    case 0x3: {
  OPC_SwitchField, 10, 1,                    // 74393:     switch Inst[10] {
  0, 7,                                      // 74396:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 74398:      check predicate 76
  OPC_Decode, 222, 51, 166, 4,               // 74400:      decode to SQDMLALv8i16_indexed using decoder 550
                                             // 74400:     }
  1, 0,                                      // 74405:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 74407:      check predicate 76
  OPC_CheckField, 11, 1, 0,                  // 74409:      check Inst[11] == 0x0
  OPC_Decode, 174, 55, 187, 4,               // 74413:      decode to SRSRAv2i64_shift using decoder 571
                                             // 74413:     }
                                             // 74413:     } // switch Inst[10]
                                             // 74413:    }
  5, 11,                                     // 74418:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 74420:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 74422:     check Inst[11:10] == 0x1
  OPC_Decode, 164, 48, 188, 4,               // 74426:     decode to SHLv2i64_shift using decoder 572
                                             // 74426:    }
  6, 11,                                     // 74431:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 74433:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74435:     check Inst[10] == 0x0
  OPC_Decode, 164, 50, 166, 4,               // 74439:     decode to SMLSLv8i16_indexed using decoder 550
                                             // 74439:    }
  7, 25,                                     // 74444:    case 0x7: {
  OPC_SwitchField, 10, 1,                    // 74446:     switch Inst[10] {
  0, 7,                                      // 74449:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 74451:      check predicate 76
  OPC_Decode, 247, 51, 166, 4,               // 74453:      decode to SQDMLSLv8i16_indexed using decoder 550
                                             // 74453:     }
  1, 0,                                      // 74458:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 74460:      check predicate 76
  OPC_CheckField, 11, 1, 0,                  // 74462:      check Inst[11] == 0x0
  OPC_Decode, 129, 54, 188, 4,               // 74466:      decode to SQSHLv2i64_shift using decoder 572
                                             // 74466:     }
                                             // 74466:     } // switch Inst[10]
                                             // 74466:    }
  8, 11,                                     // 74471:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 74473:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74475:     check Inst[10] == 0x0
  OPC_Decode, 151, 43, 167, 4,               // 74479:     decode to MULv8i16_indexed using decoder 551
                                             // 74479:    }
  9, 11,                                     // 74484:    case 0x9: {
  OPC_CheckPredicate, 99,                    // 74486:     check predicate 99
  OPC_CheckField, 10, 1, 0,                  // 74488:     check Inst[10] == 0x0
  OPC_Decode, 153, 26, 165, 4,               // 74492:     decode to FDOTlanev8f16_v4f32 using decoder 549
                                             // 74492:    }
  10, 11,                                    // 74497:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 74499:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74501:     check Inst[10] == 0x0
  OPC_Decode, 236, 50, 167, 4,               // 74505:     decode to SMULLv8i16_indexed using decoder 551
                                             // 74505:    }
  11, 11,                                    // 74510:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 74512:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74514:     check Inst[10] == 0x0
  OPC_Decode, 176, 52, 167, 4,               // 74518:     decode to SQDMULLv8i16_indexed using decoder 551
                                             // 74518:    }
  12, 11,                                    // 74523:    case 0xc: {
  OPC_CheckPredicate, 76,                    // 74525:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74527:     check Inst[10] == 0x0
  OPC_Decode, 155, 52, 167, 4,               // 74531:     decode to SQDMULHv8i16_indexed using decoder 551
                                             // 74531:    }
  13, 11,                                    // 74536:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 74538:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74540:     check Inst[10] == 0x0
  OPC_Decode, 153, 53, 167, 4,               // 74544:     decode to SQRDMULHv8i16_indexed using decoder 551
                                             // 74544:    }
  14, 11,                                    // 74549:    case 0xe: {
  OPC_CheckPredicate, 76,                    // 74551:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 74553:     check Inst[11:10] == 0x1
  OPC_Decode, 168, 47, 186, 4,               // 74557:     decode to SCVTFv2i64_shift using decoder 570
                                             // 74557:    }
  15, 0,                                     // 74562:    case 0xf: {
  OPC_SwitchField, 10, 1,                    // 74564:     switch Inst[10] {
  0, 7,                                      // 74567:     case 0x0: {
  OPC_CheckPredicate, 92,                    // 74569:      check predicate 92
  OPC_Decode, 236, 14, 165, 4,               // 74571:      decode to BF16DOTlanev8bf16 using decoder 549
                                             // 74571:     }
  1, 0,                                      // 74576:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 74578:      check predicate 76
  OPC_CheckField, 11, 1, 1,                  // 74580:      check Inst[11] == 0x1
  OPC_Decode, 164, 25, 186, 4,               // 74584:      decode to FCVTZSv2i64_shift using decoder 570
                                             // 74584:     }
                                             // 74584:     } // switch Inst[10]
                                             // 74584:    }
                                             // 74584:    } // switch Inst[15:12]
                                             // 74584:   }
  3, 0,                                      // 74589:   case 0x3: {
  OPC_SwitchField, 10, 1,                    // 74591:    switch Inst[10] {
  0, 99,                                     // 74594:    case 0x0: {
  OPC_SwitchField, 12, 1,                    // 74596:     switch Inst[12] {
  0, 57,                                     // 74599:     case 0x0: {
  OPC_SwitchField, 13, 3,                    // 74601:      switch Inst[15:13] {
  0, 7,                                      // 74604:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 74606:       check predicate 76
  OPC_Decode, 255, 41, 166, 4,               // 74608:       decode to MLAv8i16_indexed using decoder 550
                                             // 74608:      }
  1, 7,                                      // 74613:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 74615:       check predicate 76
  OPC_Decode, 129, 65, 166, 4,               // 74617:       decode to UMLALv8i16_indexed using decoder 550
                                             // 74617:      }
  2, 7,                                      // 74622:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 74624:       check predicate 76
  OPC_Decode, 144, 42, 166, 4,               // 74626:       decode to MLSv8i16_indexed using decoder 550
                                             // 74626:      }
  3, 7,                                      // 74631:      case 0x3: {
  OPC_CheckPredicate, 76,                    // 74633:       check predicate 76
  OPC_Decode, 173, 65, 166, 4,               // 74635:       decode to UMLSLv8i16_indexed using decoder 550
                                             // 74635:      }
  4, 7,                                      // 74640:      case 0x4: {
  OPC_CheckPredicate, 80,                    // 74642:       check predicate 80
  OPC_Decode, 135, 28, 161, 4,               // 74644:       decode to FMLALLTTlanev4f32 using decoder 545
                                             // 74644:      }
  5, 0,                                      // 74649:      case 0x5: {
  OPC_CheckPredicate, 76,                    // 74651:       check predicate 76
  OPC_Decode, 243, 65, 167, 4,               // 74653:       decode to UMULLv8i16_indexed using decoder 551
                                             // 74653:      }
                                             // 74653:      } // switch Inst[15:13]
                                             // 74653:     }
  1, 0,                                      // 74658:     case 0x1: {
  OPC_SwitchField, 15, 1,                    // 74660:      switch Inst[15] {
  0, 7,                                      // 74663:      case 0x0: {
  OPC_CheckPredicate, 91,                    // 74665:       check predicate 91
  OPC_Decode, 143, 23, 189, 4,               // 74667:       decode to FCMLAv8f16_indexed using decoder 573
                                             // 74667:      }
  1, 0,                                      // 74672:      case 0x1: {
  OPC_SwitchField, 13, 2,                    // 74674:       switch Inst[14:13] {
  2, 7,                                      // 74677:       case 0x2: {
  OPC_CheckPredicate, 90,                    // 74679:        check predicate 90
  OPC_Decode, 243, 52, 166, 4,               // 74681:        decode to SQRDMLAHv8i16_indexed using decoder 550
                                             // 74681:       }
  3, 0,                                      // 74686:       case 0x3: {
  OPC_CheckPredicate, 90,                    // 74688:        check predicate 90
  OPC_Decode, 134, 53, 166, 4,               // 74690:        decode to SQRDMLSHv8i16_indexed using decoder 550
                                             // 74690:       }
                                             // 74690:       } // switch Inst[14:13]
                                             // 74690:      }
                                             // 74690:      } // switch Inst[15]
                                             // 74690:     }
                                             // 74690:     } // switch Inst[12]
                                             // 74690:    }
  1, 0,                                      // 74695:    case 0x1: {
  OPC_SwitchField, 11, 5,                    // 74697:     switch Inst[15:11] {
  0, 7,                                      // 74700:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 74702:      check predicate 76
  OPC_Decode, 178, 68, 186, 4,               // 74704:      decode to USHRv2i64_shift using decoder 570
                                             // 74704:     }
  2, 7,                                      // 74709:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 74711:      check predicate 76
  OPC_Decode, 235, 68, 187, 4,               // 74713:      decode to USRAv2i64_shift using decoder 571
                                             // 74713:     }
  4, 7,                                      // 74718:     case 0x4: {
  OPC_CheckPredicate, 76,                    // 74720:      check predicate 76
  OPC_Decode, 250, 67, 186, 4,               // 74722:      decode to URSHRv2i64_shift using decoder 570
                                             // 74722:     }
  6, 7,                                      // 74727:     case 0x6: {
  OPC_CheckPredicate, 76,                    // 74729:      check predicate 76
  OPC_Decode, 138, 68, 187, 4,               // 74731:      decode to URSRAv2i64_shift using decoder 571
                                             // 74731:     }
  8, 7,                                      // 74736:     case 0x8: {
  OPC_CheckPredicate, 76,                    // 74738:      check predicate 76
  OPC_Decode, 246, 54, 187, 4,               // 74740:      decode to SRIv2i64_shift using decoder 571
                                             // 74740:     }
  10, 7,                                     // 74745:     case 0xa: {
  OPC_CheckPredicate, 76,                    // 74747:      check predicate 76
  OPC_Decode, 203, 48, 190, 4,               // 74749:      decode to SLIv2i64_shift using decoder 574
                                             // 74749:     }
  12, 7,                                     // 74754:     case 0xc: {
  OPC_CheckPredicate, 76,                    // 74756:      check predicate 76
  OPC_Decode, 231, 53, 188, 4,               // 74758:      decode to SQSHLUv2i64_shift using decoder 572
                                             // 74758:     }
  14, 7,                                     // 74763:     case 0xe: {
  OPC_CheckPredicate, 76,                    // 74765:      check predicate 76
  OPC_Decode, 129, 67, 188, 4,               // 74767:      decode to UQSHLv2i64_shift using decoder 572
                                             // 74767:     }
  28, 7,                                     // 74772:     case 0x1c: {
  OPC_CheckPredicate, 76,                    // 74774:      check predicate 76
  OPC_Decode, 161, 63, 186, 4,               // 74776:      decode to UCVTFv2i64_shift using decoder 570
                                             // 74776:     }
  31, 0,                                     // 74781:     case 0x1f: {
  OPC_CheckPredicate, 76,                    // 74783:      check predicate 76
  OPC_Decode, 215, 25, 186, 4,               // 74785:      decode to FCVTZUv2i64_shift using decoder 570
                                             // 74785:     }
                                             // 74785:     } // switch Inst[15:11]
                                             // 74785:    }
                                             // 74785:    } // switch Inst[10]
                                             // 74785:   }
                                             // 74785:   } // switch Inst[31:29]
                                             // 74785:  }
  14, 248, 5,                                // 74790:  case 0xe: {
  OPC_SwitchField, 29, 3,                    // 74793:   switch Inst[31:29] {
  0, 211, 1,                                 // 74796:   case 0x0: {
  OPC_SwitchField, 12, 4,                    // 74799:    switch Inst[15:12] {
  0, 11,                                     // 74802:    case 0x0: {
  OPC_CheckPredicate, 83,                    // 74804:     check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 74806:     check Inst[10] == 0x0
  OPC_Decode, 167, 28, 142, 4,               // 74810:     decode to FMLALlanev4f16 using decoder 526
                                             // 74810:    }
  1, 11,                                     // 74815:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 74817:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74819:     check Inst[10] == 0x0
  OPC_Decode, 200, 28, 141, 4,               // 74823:     decode to FMLAv2i32_indexed using decoder 525
                                             // 74823:    }
  2, 11,                                     // 74828:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 74830:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74832:     check Inst[10] == 0x0
  OPC_Decode, 242, 49, 191, 4,               // 74836:     decode to SMLALv2i32_indexed using decoder 575
                                             // 74836:    }
  3, 11,                                     // 74841:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 74843:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74845:     check Inst[10] == 0x0
  OPC_Decode, 216, 51, 191, 4,               // 74849:     decode to SQDMLALv2i32_indexed using decoder 575
                                             // 74849:    }
  4, 11,                                     // 74854:    case 0x4: {
  OPC_CheckPredicate, 83,                    // 74856:     check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 74858:     check Inst[10] == 0x0
  OPC_Decode, 225, 28, 142, 4,               // 74862:     decode to FMLSLlanev4f16 using decoder 526
                                             // 74862:    }
  5, 11,                                     // 74867:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 74869:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74871:     check Inst[10] == 0x0
  OPC_Decode, 130, 29, 141, 4,               // 74875:     decode to FMLSv2i32_indexed using decoder 525
                                             // 74875:    }
  6, 11,                                     // 74880:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 74882:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74884:     check Inst[10] == 0x0
  OPC_Decode, 158, 50, 191, 4,               // 74888:     decode to SMLSLv2i32_indexed using decoder 575
                                             // 74888:    }
  7, 11,                                     // 74893:    case 0x7: {
  OPC_CheckPredicate, 76,                    // 74895:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74897:     check Inst[10] == 0x0
  OPC_Decode, 241, 51, 191, 4,               // 74901:     decode to SQDMLSLv2i32_indexed using decoder 575
                                             // 74901:    }
  8, 11,                                     // 74906:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 74908:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74910:     check Inst[10] == 0x0
  OPC_Decode, 145, 43, 192, 4,               // 74914:     decode to MULv2i32_indexed using decoder 576
                                             // 74914:    }
  9, 11,                                     // 74919:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 74921:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74923:     check Inst[10] == 0x0
  OPC_Decode, 147, 30, 192, 4,               // 74927:     decode to FMULv2i32_indexed using decoder 576
                                             // 74927:    }
  10, 11,                                    // 74932:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 74934:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74936:     check Inst[10] == 0x0
  OPC_Decode, 230, 50, 193, 4,               // 74940:     decode to SMULLv2i32_indexed using decoder 577
                                             // 74940:    }
  11, 11,                                    // 74945:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 74947:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74949:     check Inst[10] == 0x0
  OPC_Decode, 170, 52, 193, 4,               // 74953:     decode to SQDMULLv2i32_indexed using decoder 577
                                             // 74953:    }
  12, 11,                                    // 74958:    case 0xc: {
  OPC_CheckPredicate, 76,                    // 74960:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74962:     check Inst[10] == 0x0
  OPC_Decode, 149, 52, 192, 4,               // 74966:     decode to SQDMULHv2i32_indexed using decoder 576
                                             // 74966:    }
  13, 11,                                    // 74971:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 74973:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 74975:     check Inst[10] == 0x0
  OPC_Decode, 147, 53, 192, 4,               // 74979:     decode to SQRDMULHv2i32_indexed using decoder 576
                                             // 74979:    }
  14, 11,                                    // 74984:    case 0xe: {
  OPC_CheckPredicate, 96,                    // 74986:     check predicate 96
  OPC_CheckField, 10, 1, 0,                  // 74988:     check Inst[10] == 0x0
  OPC_Decode, 208, 47, 141, 4,               // 74992:     decode to SDOTlanev8i8 using decoder 525
                                             // 74992:    }
  15, 0,                                     // 74997:    case 0xf: {
  OPC_CheckPredicate, 97,                    // 74999:     check predicate 97
  OPC_CheckField, 10, 1, 0,                  // 75001:     check Inst[10] == 0x0
  OPC_Decode, 152, 68, 141, 4,               // 75005:     decode to USDOTlanev8i8 using decoder 525
                                             // 75005:    }
                                             // 75005:    } // switch Inst[15:12]
                                             // 75005:   }
  1, 146, 1,                                 // 75010:   case 0x1: {
  OPC_SwitchField, 12, 4,                    // 75013:    switch Inst[15:12] {
  0, 11,                                     // 75016:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 75018:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75020:     check Inst[10] == 0x0
  OPC_Decode, 249, 41, 141, 4,               // 75024:     decode to MLAv2i32_indexed using decoder 525
                                             // 75024:    }
  2, 11,                                     // 75029:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 75031:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75033:     check Inst[10] == 0x0
  OPC_Decode, 251, 64, 191, 4,               // 75037:     decode to UMLALv2i32_indexed using decoder 575
                                             // 75037:    }
  4, 11,                                     // 75042:    case 0x4: {
  OPC_CheckPredicate, 76,                    // 75044:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75046:     check Inst[10] == 0x0
  OPC_Decode, 138, 42, 141, 4,               // 75050:     decode to MLSv2i32_indexed using decoder 525
                                             // 75050:    }
  6, 11,                                     // 75055:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 75057:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75059:     check Inst[10] == 0x0
  OPC_Decode, 167, 65, 191, 4,               // 75063:     decode to UMLSLv2i32_indexed using decoder 575
                                             // 75063:    }
  8, 11,                                     // 75068:    case 0x8: {
  OPC_CheckPredicate, 83,                    // 75070:     check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 75072:     check Inst[10] == 0x0
  OPC_Decode, 239, 27, 142, 4,               // 75076:     decode to FMLAL2lanev4f16 using decoder 526
                                             // 75076:    }
  9, 11,                                     // 75081:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 75083:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75085:     check Inst[10] == 0x0
  OPC_Decode, 238, 29, 192, 4,               // 75089:     decode to FMULXv2i32_indexed using decoder 576
                                             // 75089:    }
  10, 11,                                    // 75094:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 75096:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75098:     check Inst[10] == 0x0
  OPC_Decode, 237, 65, 193, 4,               // 75102:     decode to UMULLv2i32_indexed using decoder 577
                                             // 75102:    }
  12, 11,                                    // 75107:    case 0xc: {
  OPC_CheckPredicate, 83,                    // 75109:     check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 75111:     check Inst[10] == 0x0
  OPC_Decode, 209, 28, 142, 4,               // 75115:     decode to FMLSL2lanev4f16 using decoder 526
                                             // 75115:    }
  13, 11,                                    // 75120:    case 0xd: {
  OPC_CheckPredicate, 90,                    // 75122:     check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 75124:     check Inst[10] == 0x0
  OPC_Decode, 237, 52, 141, 4,               // 75128:     decode to SQRDMLAHv2i32_indexed using decoder 525
                                             // 75128:    }
  14, 11,                                    // 75133:    case 0xe: {
  OPC_CheckPredicate, 96,                    // 75135:     check predicate 96
  OPC_CheckField, 10, 1, 0,                  // 75137:     check Inst[10] == 0x0
  OPC_Decode, 202, 63, 141, 4,               // 75141:     decode to UDOTlanev8i8 using decoder 525
                                             // 75141:    }
  15, 0,                                     // 75146:    case 0xf: {
  OPC_CheckPredicate, 90,                    // 75148:     check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 75150:     check Inst[10] == 0x0
  OPC_Decode, 128, 53, 141, 4,               // 75154:     decode to SQRDMLSHv2i32_indexed using decoder 525
                                             // 75154:    }
                                             // 75154:    } // switch Inst[15:12]
                                             // 75154:   }
  2, 211, 1,                                 // 75159:   case 0x2: {
  OPC_SwitchField, 12, 4,                    // 75162:    switch Inst[15:12] {
  0, 11,                                     // 75165:    case 0x0: {
  OPC_CheckPredicate, 83,                    // 75167:     check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 75169:     check Inst[10] == 0x0
  OPC_Decode, 168, 28, 166, 4,               // 75173:     decode to FMLALlanev8f16 using decoder 550
                                             // 75173:    }
  1, 11,                                     // 75178:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 75180:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75182:     check Inst[10] == 0x0
  OPC_Decode, 205, 28, 165, 4,               // 75186:     decode to FMLAv4i32_indexed using decoder 549
                                             // 75186:    }
  2, 11,                                     // 75191:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 75193:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75195:     check Inst[10] == 0x0
  OPC_Decode, 246, 49, 165, 4,               // 75199:     decode to SMLALv4i32_indexed using decoder 549
                                             // 75199:    }
  3, 11,                                     // 75204:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 75206:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75208:     check Inst[10] == 0x0
  OPC_Decode, 220, 51, 165, 4,               // 75212:     decode to SQDMLALv4i32_indexed using decoder 549
                                             // 75212:    }
  4, 11,                                     // 75217:    case 0x4: {
  OPC_CheckPredicate, 83,                    // 75219:     check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 75221:     check Inst[10] == 0x0
  OPC_Decode, 226, 28, 166, 4,               // 75225:     decode to FMLSLlanev8f16 using decoder 550
                                             // 75225:    }
  5, 11,                                     // 75230:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 75232:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75234:     check Inst[10] == 0x0
  OPC_Decode, 135, 29, 165, 4,               // 75238:     decode to FMLSv4i32_indexed using decoder 549
                                             // 75238:    }
  6, 11,                                     // 75243:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 75245:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75247:     check Inst[10] == 0x0
  OPC_Decode, 162, 50, 165, 4,               // 75251:     decode to SMLSLv4i32_indexed using decoder 549
                                             // 75251:    }
  7, 11,                                     // 75256:    case 0x7: {
  OPC_CheckPredicate, 76,                    // 75258:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75260:     check Inst[10] == 0x0
  OPC_Decode, 245, 51, 165, 4,               // 75264:     decode to SQDMLSLv4i32_indexed using decoder 549
                                             // 75264:    }
  8, 11,                                     // 75269:    case 0x8: {
  OPC_CheckPredicate, 76,                    // 75271:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75273:     check Inst[10] == 0x0
  OPC_Decode, 149, 43, 194, 4,               // 75277:     decode to MULv4i32_indexed using decoder 578
                                             // 75277:    }
  9, 11,                                     // 75282:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 75284:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75286:     check Inst[10] == 0x0
  OPC_Decode, 152, 30, 194, 4,               // 75290:     decode to FMULv4i32_indexed using decoder 578
                                             // 75290:    }
  10, 11,                                    // 75295:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 75297:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75299:     check Inst[10] == 0x0
  OPC_Decode, 234, 50, 194, 4,               // 75303:     decode to SMULLv4i32_indexed using decoder 578
                                             // 75303:    }
  11, 11,                                    // 75308:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 75310:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75312:     check Inst[10] == 0x0
  OPC_Decode, 174, 52, 194, 4,               // 75316:     decode to SQDMULLv4i32_indexed using decoder 578
                                             // 75316:    }
  12, 11,                                    // 75321:    case 0xc: {
  OPC_CheckPredicate, 76,                    // 75323:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75325:     check Inst[10] == 0x0
  OPC_Decode, 153, 52, 194, 4,               // 75329:     decode to SQDMULHv4i32_indexed using decoder 578
                                             // 75329:    }
  13, 11,                                    // 75334:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 75336:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75338:     check Inst[10] == 0x0
  OPC_Decode, 151, 53, 194, 4,               // 75342:     decode to SQRDMULHv4i32_indexed using decoder 578
                                             // 75342:    }
  14, 11,                                    // 75347:    case 0xe: {
  OPC_CheckPredicate, 96,                    // 75349:     check predicate 96
  OPC_CheckField, 10, 1, 0,                  // 75351:     check Inst[10] == 0x0
  OPC_Decode, 207, 47, 165, 4,               // 75355:     decode to SDOTlanev16i8 using decoder 549
                                             // 75355:    }
  15, 0,                                     // 75360:    case 0xf: {
  OPC_CheckPredicate, 97,                    // 75362:     check predicate 97
  OPC_CheckField, 10, 1, 0,                  // 75364:     check Inst[10] == 0x0
  OPC_Decode, 151, 68, 165, 4,               // 75368:     decode to USDOTlanev16i8 using decoder 549
                                             // 75368:    }
                                             // 75368:    } // switch Inst[15:12]
                                             // 75368:   }
  3, 0,                                      // 75373:   case 0x3: {
  OPC_SwitchField, 12, 1,                    // 75375:    switch Inst[12] {
  0, 107,                                    // 75378:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 75380:     switch Inst[15:13] {
  0, 11,                                     // 75383:     case 0x0: {
  OPC_CheckPredicate, 76,                    // 75385:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75387:      check Inst[10] == 0x0
  OPC_Decode, 253, 41, 165, 4,               // 75391:      decode to MLAv4i32_indexed using decoder 549
                                             // 75391:     }
  1, 11,                                     // 75396:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 75398:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75400:      check Inst[10] == 0x0
  OPC_Decode, 255, 64, 165, 4,               // 75404:      decode to UMLALv4i32_indexed using decoder 549
                                             // 75404:     }
  2, 11,                                     // 75409:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 75411:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75413:      check Inst[10] == 0x0
  OPC_Decode, 142, 42, 165, 4,               // 75417:      decode to MLSv4i32_indexed using decoder 549
                                             // 75417:     }
  3, 11,                                     // 75422:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 75424:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75426:      check Inst[10] == 0x0
  OPC_Decode, 171, 65, 165, 4,               // 75430:      decode to UMLSLv4i32_indexed using decoder 549
                                             // 75430:     }
  4, 11,                                     // 75435:     case 0x4: {
  OPC_CheckPredicate, 83,                    // 75437:      check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 75439:      check Inst[10] == 0x0
  OPC_Decode, 240, 27, 166, 4,               // 75443:      decode to FMLAL2lanev8f16 using decoder 550
                                             // 75443:     }
  5, 11,                                     // 75448:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 75450:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75452:      check Inst[10] == 0x0
  OPC_Decode, 241, 65, 194, 4,               // 75456:      decode to UMULLv4i32_indexed using decoder 578
                                             // 75456:     }
  6, 11,                                     // 75461:     case 0x6: {
  OPC_CheckPredicate, 83,                    // 75463:      check predicate 83
  OPC_CheckField, 10, 1, 0,                  // 75465:      check Inst[10] == 0x0
  OPC_Decode, 210, 28, 166, 4,               // 75469:      decode to FMLSL2lanev8f16 using decoder 550
                                             // 75469:     }
  7, 0,                                      // 75474:     case 0x7: {
  OPC_CheckPredicate, 96,                    // 75476:      check predicate 96
  OPC_CheckField, 10, 1, 0,                  // 75478:      check Inst[10] == 0x0
  OPC_Decode, 201, 63, 165, 4,               // 75482:      decode to UDOTlanev16i8 using decoder 549
                                             // 75482:     }
                                             // 75482:     } // switch Inst[15:13]
                                             // 75482:    }
  1, 0,                                      // 75487:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 75489:     switch Inst[15] {
  0, 15,                                     // 75492:     case 0x0: {
  OPC_CheckPredicate, 100,                   // 75494:      check predicate 100
  OPC_CheckField, 21, 1, 0,                  // 75496:      check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 75500:      check Inst[10] == 0x0
  OPC_Decode, 141, 23, 195, 4,               // 75504:      decode to FCMLAv4f32_indexed using decoder 579
                                             // 75504:     }
  1, 0,                                      // 75509:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 75511:      switch Inst[14:13] {
  0, 11,                                     // 75514:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 75516:       check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 75518:       check Inst[10] == 0x0
  OPC_Decode, 243, 29, 194, 4,               // 75522:       decode to FMULXv4i32_indexed using decoder 578
                                             // 75522:      }
  2, 11,                                     // 75527:      case 0x2: {
  OPC_CheckPredicate, 90,                    // 75529:       check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 75531:       check Inst[10] == 0x0
  OPC_Decode, 241, 52, 165, 4,               // 75535:       decode to SQRDMLAHv4i32_indexed using decoder 549
                                             // 75535:      }
  3, 0,                                      // 75540:      case 0x3: {
  OPC_CheckPredicate, 90,                    // 75542:       check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 75544:       check Inst[10] == 0x0
  OPC_Decode, 132, 53, 165, 4,               // 75548:       decode to SQRDMLSHv4i32_indexed using decoder 549
                                             // 75548:      }
                                             // 75548:      } // switch Inst[14:13]
                                             // 75548:     }
                                             // 75548:     } // switch Inst[15]
                                             // 75548:    }
                                             // 75548:    } // switch Inst[12]
                                             // 75548:   }
                                             // 75548:   } // switch Inst[31:29]
                                             // 75548:  }
  15, 0,                                     // 75553:  case 0xf: {
  OPC_SwitchField, 12, 4,                    // 75555:   switch Inst[15:12] {
  0, 29,                                     // 75558:   case 0x0: {
  OPC_SwitchField, 29, 3,                    // 75560:    switch Inst[31:29] {
  0, 11,                                     // 75563:    case 0x0: {
  OPC_CheckPredicate, 80,                    // 75565:     check predicate 80
  OPC_CheckField, 10, 1, 0,                  // 75567:     check Inst[10] == 0x0
  OPC_Decode, 247, 27, 161, 4,               // 75571:     decode to FMLALBlanev8f16 using decoder 545
                                             // 75571:    }
  2, 0,                                      // 75576:    case 0x2: {
  OPC_CheckPredicate, 80,                    // 75578:     check predicate 80
  OPC_CheckField, 10, 1, 0,                  // 75580:     check Inst[10] == 0x0
  OPC_Decode, 149, 28, 161, 4,               // 75584:     decode to FMLALTlanev8f16 using decoder 545
                                             // 75584:    }
                                             // 75584:    } // switch Inst[31:29]
                                             // 75584:   }
  1, 19,                                     // 75589:   case 0x1: {
  OPC_CheckPredicate, 76,                    // 75591:    check predicate 76
  OPC_CheckField, 29, 3, 2,                  // 75593:    check Inst[31:29] == 0x2
  OPC_CheckField, 21, 1, 0,                  // 75597:    check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 75601:    check Inst[10] == 0x0
  OPC_Decode, 201, 28, 196, 4,               // 75605:    decode to FMLAv2i64_indexed using decoder 580
                                             // 75605:   }
  5, 19,                                     // 75610:   case 0x5: {
  OPC_CheckPredicate, 76,                    // 75612:    check predicate 76
  OPC_CheckField, 29, 3, 2,                  // 75614:    check Inst[31:29] == 0x2
  OPC_CheckField, 21, 1, 0,                  // 75618:    check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 75622:    check Inst[10] == 0x0
  OPC_Decode, 131, 29, 196, 4,               // 75626:    decode to FMLSv2i64_indexed using decoder 580
                                             // 75626:   }
  9, 37,                                     // 75631:   case 0x9: {
  OPC_SwitchField, 29, 3,                    // 75633:    switch Inst[31:29] {
  2, 15,                                     // 75636:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 75638:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 75640:     check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 75644:     check Inst[10] == 0x0
  OPC_Decode, 148, 30, 197, 4,               // 75648:     decode to FMULv2i64_indexed using decoder 581
                                             // 75648:    }
  3, 0,                                      // 75653:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 75655:     check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 75657:     check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 75661:     check Inst[10] == 0x0
  OPC_Decode, 239, 29, 197, 4,               // 75665:     decode to FMULXv2i64_indexed using decoder 581
                                             // 75665:    }
                                             // 75665:    } // switch Inst[31:29]
                                             // 75665:   }
  15, 0,                                     // 75670:   case 0xf: {
  OPC_SwitchField, 29, 3,                    // 75672:    switch Inst[31:29] {
  0, 11,                                     // 75675:    case 0x0: {
  OPC_CheckPredicate, 92,                    // 75677:     check predicate 92
  OPC_CheckField, 10, 1, 0,                  // 75679:     check Inst[10] == 0x0
  OPC_Decode, 170, 15, 166, 4,               // 75683:     decode to BFMLALBIdx using decoder 550
                                             // 75683:    }
  2, 0,                                      // 75688:    case 0x2: {
  OPC_CheckPredicate, 92,                    // 75690:     check predicate 92
  OPC_CheckField, 10, 1, 0,                  // 75692:     check Inst[10] == 0x0
  OPC_Decode, 174, 15, 166, 4,               // 75696:     decode to BFMLALTIdx using decoder 550
                                             // 75696:    }
                                             // 75696:    } // switch Inst[31:29]
                                             // 75696:   }
                                             // 75696:   } // switch Inst[15:12]
                                             // 75696:  }
                                             // 75696:  } // switch Inst[25:22]
                                             // 75696: }
  4, 238, 3,                                 // 75701: case 0x4: {
  OPC_SwitchField, 24, 2,                    // 75704:  switch Inst[25:24] {
  0, 17,                                     // 75707:  case 0x0: {
  OPC_SwitchField, 31, 1,                    // 75709:   switch Inst[31] {
  0, 5,                                      // 75712:   case 0x0: {
  OPC_Decode, 241, 13, 198, 4,               // 75714:    decode to ADR using decoder 582
                                             // 75714:   }
  1, 0,                                      // 75719:   case 0x1: {
  OPC_Decode, 242, 13, 198, 4,               // 75721:    decode to ADRP using decoder 582
                                             // 75721:   }
                                             // 75721:   } // switch Inst[31]
                                             // 75721:  }
  1, 177, 1,                                 // 75726:  case 0x1: {
  OPC_SwitchField, 29, 3,                    // 75729:   switch Inst[31:29] {
  0, 46,                                     // 75732:   case 0x0: {
  OPC_Scope, 39,                             // 75734:    try {
  OPC_SwitchField, 18, 6,                    // 75736:     switch Inst[23:18] {
  48, 7,                                     // 75739:     case 0x30: {
  OPC_CheckPredicate, 103,                   // 75741:      check predicate 103
  OPC_Decode, 243, 48, 199, 4,               // 75743:      decode to SMAXWri using decoder 583
                                             // 75743:     }
  49, 7,                                     // 75748:     case 0x31: {
  OPC_CheckPredicate, 103,                   // 75750:      check predicate 103
  OPC_Decode, 253, 63, 200, 4,               // 75752:      decode to UMAXWri using decoder 584
                                             // 75752:     }
  50, 7,                                     // 75757:     case 0x32: {
  OPC_CheckPredicate, 103,                   // 75759:      check predicate 103
  OPC_Decode, 173, 49, 199, 4,               // 75761:      decode to SMINWri using decoder 583
                                             // 75761:     }
  51, 0,                                     // 75766:     case 0x33: {
  OPC_CheckPredicate, 103,                   // 75768:      check predicate 103
  OPC_Decode, 182, 64, 200, 4,               // 75770:      decode to UMINWri using decoder 584
                                             // 75770:     }
                                             // 75770:     } // switch Inst[23:18]
                                             // 75770:    } else try {
  OPC_Decode, 192, 13, 201, 4,               // 75775:     decode to ADDWri using decoder 585
                                             // 75775:    }
                                             // 75775:   }
  1, 5,                                      // 75780:   case 0x1: {
  OPC_Decode, 177, 13, 201, 4,               // 75782:    decode to ADDSWri using decoder 585
                                             // 75782:   }
  2, 5,                                      // 75787:   case 0x2: {
  OPC_Decode, 159, 60, 201, 4,               // 75789:    decode to SUBWri using decoder 585
                                             // 75789:   }
  3, 5,                                      // 75794:   case 0x3: {
  OPC_Decode, 152, 60, 201, 4,               // 75796:    decode to SUBSWri using decoder 585
                                             // 75796:   }
  4, 64,                                     // 75801:   case 0x4: {
  OPC_Scope, 39,                             // 75803:    try {
  OPC_SwitchField, 18, 6,                    // 75805:     switch Inst[23:18] {
  48, 7,                                     // 75808:     case 0x30: {
  OPC_CheckPredicate, 103,                   // 75810:      check predicate 103
  OPC_Decode, 245, 48, 202, 4,               // 75812:      decode to SMAXXri using decoder 586
                                             // 75812:     }
  49, 7,                                     // 75817:     case 0x31: {
  OPC_CheckPredicate, 103,                   // 75819:      check predicate 103
  OPC_Decode, 255, 63, 203, 4,               // 75821:      decode to UMAXXri using decoder 587
                                             // 75821:     }
  50, 7,                                     // 75826:     case 0x32: {
  OPC_CheckPredicate, 103,                   // 75828:      check predicate 103
  OPC_Decode, 175, 49, 202, 4,               // 75830:      decode to SMINXri using decoder 586
                                             // 75830:     }
  51, 0,                                     // 75835:     case 0x33: {
  OPC_CheckPredicate, 103,                   // 75837:      check predicate 103
  OPC_Decode, 184, 64, 203, 4,               // 75839:      decode to UMINXri using decoder 587
                                             // 75839:     }
                                             // 75839:     } // switch Inst[23:18]
  OPC_Scope, 16,                             // 75844:    } else try {
  OPC_CheckField, 22, 2, 2,                  // 75846:     check Inst[23:22] == 0x2
  OPC_CheckPredicate, 75,                    // 75850:     check predicate 75
  OPC_SoftFail, 128, 128, 3, 0,              // 75852:     softfail pos=0x0000c000 neg=0x00000000
  OPC_Decode, 134, 13, 204, 4,               // 75857:     decode to ADDG using decoder 588
                                             // 75857:    } else try {
  OPC_Decode, 195, 13, 201, 4,               // 75862:     decode to ADDXri using decoder 585
                                             // 75862:    }
                                             // 75862:   }
  5, 5,                                      // 75867:   case 0x5: {
  OPC_Decode, 180, 13, 201, 4,               // 75869:    decode to ADDSXri using decoder 585
                                             // 75869:   }
  6, 23,                                     // 75874:   case 0x6: {
  OPC_Scope, 16,                             // 75876:    try {
  OPC_CheckField, 22, 2, 2,                  // 75878:     check Inst[23:22] == 0x2
  OPC_CheckPredicate, 75,                    // 75882:     check predicate 75
  OPC_SoftFail, 128, 128, 3, 0,              // 75884:     softfail pos=0x0000c000 neg=0x00000000
  OPC_Decode, 252, 59, 204, 4,               // 75889:     decode to SUBG using decoder 588
                                             // 75889:    } else try {
  OPC_Decode, 162, 60, 201, 4,               // 75894:     decode to SUBXri using decoder 585
                                             // 75894:    }
                                             // 75894:   }
  7, 0,                                      // 75899:   case 0x7: {
  OPC_Decode, 155, 60, 201, 4,               // 75901:    decode to SUBSXri using decoder 585
                                             // 75901:   }
                                             // 75901:   } // switch Inst[31:29]
                                             // 75901:  }
  2, 151, 1,                                 // 75906:  case 0x2: {
  OPC_SwitchField, 29, 3,                    // 75909:   switch Inst[31:29] {
  0, 21,                                     // 75912:   case 0x0: {
  OPC_SwitchField, 23, 1,                    // 75914:    switch Inst[23] {
  0, 9,                                      // 75917:    case 0x0: {
  OPC_CheckField, 22, 1, 0,                  // 75919:     check Inst[22] == 0x0
  OPC_Decode, 160, 14, 205, 4,               // 75923:     decode to ANDWri using decoder 589
                                             // 75923:    }
  1, 0,                                      // 75928:    case 0x1: {
  OPC_Decode, 226, 42, 206, 4,               // 75930:     decode to MOVNWi using decoder 590
                                             // 75930:    }
                                             // 75930:    } // switch Inst[23]
                                             // 75930:   }
  1, 9,                                      // 75935:   case 0x1: {
  OPC_CheckField, 22, 2, 0,                  // 75937:    check Inst[23:22] == 0x0
  OPC_Decode, 204, 43, 205, 4,               // 75941:    decode to ORRWri using decoder 589
                                             // 75941:   }
  2, 21,                                     // 75946:   case 0x2: {
  OPC_SwitchField, 23, 1,                    // 75948:    switch Inst[23] {
  0, 9,                                      // 75951:    case 0x0: {
  OPC_CheckField, 22, 1, 0,                  // 75953:     check Inst[22] == 0x0
  OPC_Decode, 131, 21, 205, 4,               // 75957:     decode to EORWri using decoder 589
                                             // 75957:    }
  1, 0,                                      // 75962:    case 0x1: {
  OPC_Decode, 240, 42, 206, 4,               // 75964:     decode to MOVZWi using decoder 590
                                             // 75964:    }
                                             // 75964:    } // switch Inst[23]
                                             // 75964:   }
  3, 21,                                     // 75969:   case 0x3: {
  OPC_SwitchField, 23, 1,                    // 75971:    switch Inst[23] {
  0, 9,                                      // 75974:    case 0x0: {
  OPC_CheckField, 22, 1, 0,                  // 75976:     check Inst[22] == 0x0
  OPC_Decode, 151, 14, 205, 4,               // 75980:     decode to ANDSWri using decoder 589
                                             // 75980:    }
  1, 0,                                      // 75985:    case 0x1: {
  OPC_Decode, 224, 42, 206, 4,               // 75987:     decode to MOVKWi using decoder 590
                                             // 75987:    }
                                             // 75987:    } // switch Inst[23]
                                             // 75987:   }
  4, 17,                                     // 75992:   case 0x4: {
  OPC_SwitchField, 23, 1,                    // 75994:    switch Inst[23] {
  0, 5,                                      // 75997:    case 0x0: {
  OPC_Decode, 162, 14, 205, 4,               // 75999:     decode to ANDXri using decoder 589
                                             // 75999:    }
  1, 0,                                      // 76004:    case 0x1: {
  OPC_Decode, 227, 42, 206, 4,               // 76006:     decode to MOVNXi using decoder 590
                                             // 76006:    }
                                             // 76006:    } // switch Inst[23]
                                             // 76006:   }
  5, 9,                                      // 76011:   case 0x5: {
  OPC_CheckField, 23, 1, 0,                  // 76013:    check Inst[23] == 0x0
  OPC_Decode, 206, 43, 205, 4,               // 76017:    decode to ORRXri using decoder 589
                                             // 76017:   }
  6, 17,                                     // 76022:   case 0x6: {
  OPC_SwitchField, 23, 1,                    // 76024:    switch Inst[23] {
  0, 5,                                      // 76027:    case 0x0: {
  OPC_Decode, 133, 21, 205, 4,               // 76029:     decode to EORXri using decoder 589
                                             // 76029:    }
  1, 0,                                      // 76034:    case 0x1: {
  OPC_Decode, 241, 42, 206, 4,               // 76036:     decode to MOVZXi using decoder 590
                                             // 76036:    }
                                             // 76036:    } // switch Inst[23]
                                             // 76036:   }
  7, 0,                                      // 76041:   case 0x7: {
  OPC_SwitchField, 23, 1,                    // 76043:    switch Inst[23] {
  0, 5,                                      // 76046:    case 0x0: {
  OPC_Decode, 153, 14, 205, 4,               // 76048:     decode to ANDSXri using decoder 589
                                             // 76048:    }
  1, 0,                                      // 76053:    case 0x1: {
  OPC_Decode, 225, 42, 206, 4,               // 76055:     decode to MOVKXi using decoder 590
                                             // 76055:    }
                                             // 76055:    } // switch Inst[23]
                                             // 76055:   }
                                             // 76055:   } // switch Inst[31:29]
                                             // 76055:  }
  3, 0,                                      // 76060:  case 0x3: {
  OPC_SwitchField, 29, 3,                    // 76062:   switch Inst[31:29] {
  0, 25,                                     // 76065:   case 0x0: {
  OPC_SwitchField, 21, 3,                    // 76067:    switch Inst[23:21] {
  0, 9,                                      // 76070:    case 0x0: {
  OPC_CheckField, 15, 1, 0,                  // 76072:     check Inst[15] == 0x0
  OPC_Decode, 235, 46, 207, 4,               // 76076:     decode to SBFMWri using decoder 591
                                             // 76076:    }
  4, 0,                                      // 76081:    case 0x4: {
  OPC_CheckField, 15, 1, 0,                  // 76083:     check Inst[15] == 0x0
  OPC_Decode, 162, 21, 208, 4,               // 76087:     decode to EXTRWrri using decoder 592
                                             // 76087:    }
                                             // 76087:    } // switch Inst[23:21]
                                             // 76087:   }
  1, 13,                                     // 76092:   case 0x1: {
  OPC_CheckField, 21, 3, 0,                  // 76094:    check Inst[23:21] == 0x0
  OPC_CheckField, 15, 1, 0,                  // 76098:    check Inst[15] == 0x0
  OPC_Decode, 243, 15, 209, 4,               // 76102:    decode to BFMWri using decoder 593
                                             // 76102:   }
  2, 13,                                     // 76107:   case 0x2: {
  OPC_CheckField, 21, 3, 0,                  // 76109:    check Inst[23:21] == 0x0
  OPC_CheckField, 15, 1, 0,                  // 76113:    check Inst[15] == 0x0
  OPC_Decode, 228, 62, 207, 4,               // 76117:    decode to UBFMWri using decoder 591
                                             // 76117:   }
  4, 21,                                     // 76122:   case 0x4: {
  OPC_SwitchField, 22, 2,                    // 76124:    switch Inst[23:22] {
  1, 5,                                      // 76127:    case 0x1: {
  OPC_Decode, 236, 46, 210, 4,               // 76129:     decode to SBFMXri using decoder 594
                                             // 76129:    }
  3, 0,                                      // 76134:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 76136:     check Inst[21] == 0x0
  OPC_Decode, 163, 21, 211, 4,               // 76140:     decode to EXTRXrri using decoder 595
                                             // 76140:    }
                                             // 76140:    } // switch Inst[23:22]
                                             // 76140:   }
  5, 9,                                      // 76145:   case 0x5: {
  OPC_CheckField, 22, 2, 1,                  // 76147:    check Inst[23:22] == 0x1
  OPC_Decode, 244, 15, 212, 4,               // 76151:    decode to BFMXri using decoder 596
                                             // 76151:   }
  6, 9,                                      // 76156:   case 0x6: {
  OPC_CheckField, 22, 2, 1,                  // 76158:    check Inst[23:22] == 0x1
  OPC_Decode, 229, 62, 210, 4,               // 76162:    decode to UBFMXri using decoder 594
                                             // 76162:   }
  7, 0,                                      // 76167:   case 0x7: {
  OPC_SwitchField, 21, 3,                    // 76169:    switch Inst[23:21] {
  4, 11,                                     // 76172:    case 0x4: {
  OPC_CheckPredicate, 104,                   // 76174:     check predicate 104
  OPC_CheckField, 0, 5, 31,                  // 76176:     check Inst[4:0] == 0x1f
  OPC_Decode, 210, 14, 213, 4,               // 76180:     decode to AUTIASPPCi using decoder 597
                                             // 76180:    }
  5, 0,                                      // 76185:    case 0x5: {
  OPC_CheckPredicate, 104,                   // 76187:     check predicate 104
  OPC_CheckField, 0, 5, 31,                  // 76189:     check Inst[4:0] == 0x1f
  OPC_Decode, 217, 14, 213, 4,               // 76193:     decode to AUTIBSPPCi using decoder 597
                                             // 76193:    }
                                             // 76193:    } // switch Inst[23:21]
                                             // 76193:   }
                                             // 76193:   } // switch Inst[31:29]
                                             // 76193:  }
                                             // 76193:  } // switch Inst[25:24]
                                             // 76193: }
  5, 194, 12,                                // 76198: case 0x5: {
  OPC_SwitchField, 29, 3,                    // 76201:  switch Inst[31:29] {
  0, 5,                                      // 76204:  case 0x0: {
  OPC_Decode, 223, 14, 214, 4,               // 76206:   decode to B using decoder 598
                                             // 76206:  }
  1, 31,                                     // 76211:  case 0x1: {
  OPC_SwitchField, 24, 2,                    // 76213:   switch Inst[25:24] {
  0, 5,                                      // 76216:   case 0x0: {
  OPC_Decode, 134, 17, 215, 4,               // 76218:    decode to CBZW using decoder 599
                                             // 76218:   }
  1, 5,                                      // 76223:   case 0x1: {
  OPC_Decode, 132, 17, 215, 4,               // 76225:    decode to CBNZW using decoder 599
                                             // 76225:   }
  2, 5,                                      // 76230:   case 0x2: {
  OPC_Decode, 219, 61, 216, 4,               // 76232:    decode to TBZW using decoder 600
                                             // 76232:   }
  3, 0,                                      // 76237:   case 0x3: {
  OPC_Decode, 201, 61, 216, 4,               // 76239:    decode to TBNZW using decoder 600
                                             // 76239:   }
                                             // 76239:   } // switch Inst[25:24]
                                             // 76239:  }
  2, 59,                                     // 76244:  case 0x2: {
  OPC_SwitchField, 4, 1,                     // 76246:   switch Inst[4] {
  0, 9,                                      // 76249:   case 0x0: {
  OPC_CheckField, 24, 2, 0,                  // 76251:    check Inst[25:24] == 0x0
  OPC_Decode, 187, 16, 217, 4,               // 76255:    decode to Bcc using decoder 601
                                             // 76255:   }
  1, 0,                                      // 76260:   case 0x1: {
  OPC_SwitchField, 24, 2,                    // 76262:    switch Inst[25:24] {
  0, 7,                                      // 76265:    case 0x0: {
  OPC_CheckPredicate, 105,                   // 76267:     check predicate 105
  OPC_Decode, 226, 14, 217, 4,               // 76269:     decode to BCcc using decoder 601
                                             // 76269:    }
  1, 0,                                      // 76274:    case 0x1: {
  OPC_SwitchField, 21, 3,                    // 76276:     switch Inst[23:21] {
  0, 11,                                     // 76279:     case 0x0: {
  OPC_CheckPredicate, 104,                   // 76281:      check predicate 104
  OPC_CheckField, 0, 4, 15,                  // 76283:      check Inst[3:0] == 0xf
  OPC_Decode, 185, 45, 213, 4,               // 76287:      decode to RETAASPPCi using decoder 597
                                             // 76287:     }
  1, 0,                                      // 76292:     case 0x1: {
  OPC_CheckPredicate, 104,                   // 76294:      check predicate 104
  OPC_CheckField, 0, 4, 15,                  // 76296:      check Inst[3:0] == 0xf
  OPC_Decode, 188, 45, 213, 4,               // 76300:      decode to RETABSPPCi using decoder 597
                                             // 76300:     }
                                             // 76300:     } // switch Inst[23:21]
                                             // 76300:    }
                                             // 76300:    } // switch Inst[25:24]
                                             // 76300:   }
                                             // 76300:   } // switch Inst[4]
                                             // 76300:  }
  3, 145, 2,                                 // 76305:  case 0x3: {
  OPC_SwitchField, 21, 5,                    // 76308:   switch Inst[25:21] {
  0, 30,                                     // 76311:   case 0x0: {
  OPC_SwitchField, 14, 2,                    // 76313:    switch Inst[15:14] {
  0, 7,                                      // 76316:    case 0x0: {
  OPC_CheckPredicate, 106,                   // 76318:     check predicate 106
  OPC_Decode, 237, 16, 218, 4,               // 76320:     decode to CBGTWrr using decoder 602
                                             // 76320:    }
  2, 7,                                      // 76325:    case 0x2: {
  OPC_CheckPredicate, 106,                   // 76327:     check predicate 106
  OPC_Decode, 226, 16, 218, 4,               // 76329:     decode to CBBGTWrr using decoder 602
                                             // 76329:    }
  3, 0,                                      // 76334:    case 0x3: {
  OPC_CheckPredicate, 106,                   // 76336:     check predicate 106
  OPC_Decode, 242, 16, 218, 4,               // 76338:     decode to CBHGTWrr using decoder 602
                                             // 76338:    }
                                             // 76338:    } // switch Inst[15:14]
                                             // 76338:   }
  1, 30,                                     // 76343:   case 0x1: {
  OPC_SwitchField, 14, 2,                    // 76345:    switch Inst[15:14] {
  0, 7,                                      // 76348:    case 0x0: {
  OPC_CheckPredicate, 106,                   // 76350:     check predicate 106
  OPC_Decode, 234, 16, 218, 4,               // 76352:     decode to CBGEWrr using decoder 602
                                             // 76352:    }
  2, 7,                                      // 76357:    case 0x2: {
  OPC_CheckPredicate, 106,                   // 76359:     check predicate 106
  OPC_Decode, 225, 16, 218, 4,               // 76361:     decode to CBBGEWrr using decoder 602
                                             // 76361:    }
  3, 0,                                      // 76366:    case 0x3: {
  OPC_CheckPredicate, 106,                   // 76368:     check predicate 106
  OPC_Decode, 241, 16, 218, 4,               // 76370:     decode to CBHGEWrr using decoder 602
                                             // 76370:    }
                                             // 76370:    } // switch Inst[15:14]
                                             // 76370:   }
  2, 30,                                     // 76375:   case 0x2: {
  OPC_SwitchField, 14, 2,                    // 76377:    switch Inst[15:14] {
  0, 7,                                      // 76380:    case 0x0: {
  OPC_CheckPredicate, 106,                   // 76382:     check predicate 106
  OPC_Decode, 246, 16, 218, 4,               // 76384:     decode to CBHIWrr using decoder 602
                                             // 76384:    }
  2, 7,                                      // 76389:    case 0x2: {
  OPC_CheckPredicate, 106,                   // 76391:     check predicate 106
  OPC_Decode, 227, 16, 218, 4,               // 76393:     decode to CBBHIWrr using decoder 602
                                             // 76393:    }
  3, 0,                                      // 76398:    case 0x3: {
  OPC_CheckPredicate, 106,                   // 76400:     check predicate 106
  OPC_Decode, 243, 16, 218, 4,               // 76402:     decode to CBHHIWrr using decoder 602
                                             // 76402:    }
                                             // 76402:    } // switch Inst[15:14]
                                             // 76402:   }
  3, 30,                                     // 76407:   case 0x3: {
  OPC_SwitchField, 14, 2,                    // 76409:    switch Inst[15:14] {
  0, 7,                                      // 76412:    case 0x0: {
  OPC_CheckPredicate, 106,                   // 76414:     check predicate 106
  OPC_Decode, 250, 16, 218, 4,               // 76416:     decode to CBHSWrr using decoder 602
                                             // 76416:    }
  2, 7,                                      // 76421:    case 0x2: {
  OPC_CheckPredicate, 106,                   // 76423:     check predicate 106
  OPC_Decode, 228, 16, 218, 4,               // 76425:     decode to CBBHSWrr using decoder 602
                                             // 76425:    }
  3, 0,                                      // 76430:    case 0x3: {
  OPC_CheckPredicate, 106,                   // 76432:     check predicate 106
  OPC_Decode, 244, 16, 218, 4,               // 76434:     decode to CBHHSWrr using decoder 602
                                             // 76434:    }
                                             // 76434:    } // switch Inst[15:14]
                                             // 76434:   }
  6, 30,                                     // 76439:   case 0x6: {
  OPC_SwitchField, 14, 2,                    // 76441:    switch Inst[15:14] {
  0, 7,                                      // 76444:    case 0x0: {
  OPC_CheckPredicate, 106,                   // 76446:     check predicate 106
  OPC_Decode, 231, 16, 218, 4,               // 76448:     decode to CBEQWrr using decoder 602
                                             // 76448:    }
  2, 7,                                      // 76453:    case 0x2: {
  OPC_CheckPredicate, 106,                   // 76455:     check predicate 106
  OPC_Decode, 224, 16, 218, 4,               // 76457:     decode to CBBEQWrr using decoder 602
                                             // 76457:    }
  3, 0,                                      // 76462:    case 0x3: {
  OPC_CheckPredicate, 106,                   // 76464:     check predicate 106
  OPC_Decode, 240, 16, 218, 4,               // 76466:     decode to CBHEQWrr using decoder 602
                                             // 76466:    }
                                             // 76466:    } // switch Inst[15:14]
                                             // 76466:   }
  7, 30,                                     // 76471:   case 0x7: {
  OPC_SwitchField, 14, 2,                    // 76473:    switch Inst[15:14] {
  0, 7,                                      // 76476:    case 0x0: {
  OPC_CheckPredicate, 106,                   // 76478:     check predicate 106
  OPC_Decode, 129, 17, 218, 4,               // 76480:     decode to CBNEWrr using decoder 602
                                             // 76480:    }
  2, 7,                                      // 76485:    case 0x2: {
  OPC_CheckPredicate, 106,                   // 76487:     check predicate 106
  OPC_Decode, 229, 16, 218, 4,               // 76489:     decode to CBBNEWrr using decoder 602
                                             // 76489:    }
  3, 0,                                      // 76494:    case 0x3: {
  OPC_CheckPredicate, 106,                   // 76496:     check predicate 106
  OPC_Decode, 249, 16, 218, 4,               // 76498:     decode to CBHNEWrr using decoder 602
                                             // 76498:    }
                                             // 76498:    } // switch Inst[15:14]
                                             // 76498:   }
  8, 11,                                     // 76503:   case 0x8: {
  OPC_CheckPredicate, 106,                   // 76505:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 76507:    check Inst[14] == 0x0
  OPC_Decode, 236, 16, 219, 4,               // 76511:    decode to CBGTWri using decoder 603
                                             // 76511:   }
  9, 11,                                     // 76516:   case 0x9: {
  OPC_CheckPredicate, 106,                   // 76518:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 76520:    check Inst[14] == 0x0
  OPC_Decode, 254, 16, 219, 4,               // 76524:    decode to CBLTWri using decoder 603
                                             // 76524:   }
  10, 11,                                    // 76529:   case 0xa: {
  OPC_CheckPredicate, 106,                   // 76531:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 76533:    check Inst[14] == 0x0
  OPC_Decode, 245, 16, 219, 4,               // 76537:    decode to CBHIWri using decoder 603
                                             // 76537:   }
  11, 11,                                    // 76542:   case 0xb: {
  OPC_CheckPredicate, 106,                   // 76544:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 76546:    check Inst[14] == 0x0
  OPC_Decode, 252, 16, 219, 4,               // 76550:    decode to CBLOWri using decoder 603
                                             // 76550:   }
  14, 11,                                    // 76555:   case 0xe: {
  OPC_CheckPredicate, 106,                   // 76557:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 76559:    check Inst[14] == 0x0
  OPC_Decode, 230, 16, 219, 4,               // 76563:    decode to CBEQWri using decoder 603
                                             // 76563:   }
  15, 0,                                     // 76568:   case 0xf: {
  OPC_CheckPredicate, 106,                   // 76570:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 76572:    check Inst[14] == 0x0
  OPC_Decode, 128, 17, 219, 4,               // 76576:    decode to CBNEWri using decoder 603
                                             // 76576:   }
                                             // 76576:   } // switch Inst[25:21]
                                             // 76576:  }
  4, 5,                                      // 76581:  case 0x4: {
  OPC_Decode, 154, 16, 214, 4,               // 76583:   decode to BL using decoder 598
                                             // 76583:  }
  5, 31,                                     // 76588:  case 0x5: {
  OPC_SwitchField, 24, 2,                    // 76590:   switch Inst[25:24] {
  0, 5,                                      // 76593:   case 0x0: {
  OPC_Decode, 135, 17, 220, 4,               // 76595:    decode to CBZX using decoder 604
                                             // 76595:   }
  1, 5,                                      // 76600:   case 0x1: {
  OPC_Decode, 133, 17, 220, 4,               // 76602:    decode to CBNZX using decoder 604
                                             // 76602:   }
  2, 5,                                      // 76607:   case 0x2: {
  OPC_Decode, 220, 61, 216, 4,               // 76609:    decode to TBZX using decoder 600
                                             // 76609:   }
  3, 0,                                      // 76614:   case 0x3: {
  OPC_Decode, 202, 61, 216, 4,               // 76616:    decode to TBNZX using decoder 600
                                             // 76616:   }
                                             // 76616:   } // switch Inst[25:24]
                                             // 76616:  }
  6, 250, 7,                                 // 76621:  case 0x6: {
  OPC_SwitchField, 21, 5,                    // 76624:   switch Inst[25:21] {
  0, 26,                                     // 76627:   case 0x0: {
  OPC_SwitchField, 0, 5,                     // 76629:    switch Inst[4:0] {
  1, 5,                                      // 76632:    case 0x1: {
  OPC_Decode, 133, 61, 221, 4,               // 76634:     decode to SVC using decoder 605
                                             // 76634:    }
  2, 5,                                      // 76639:    case 0x2: {
  OPC_Decode, 226, 33, 221, 4,               // 76641:     decode to HVC using decoder 605
                                             // 76641:    }
  3, 0,                                      // 76646:    case 0x3: {
  OPC_CheckPredicate, 107,                   // 76648:     check predicate 107
  OPC_Decode, 149, 49, 221, 4,               // 76650:     decode to SMC using decoder 605
                                             // 76650:    }
                                             // 76650:    } // switch Inst[4:0]
                                             // 76650:   }
  1, 9,                                      // 76655:   case 0x1: {
  OPC_CheckField, 0, 5, 0,                   // 76657:    check Inst[4:0] == 0x0
  OPC_Decode, 169, 16, 221, 4,               // 76661:    decode to BRK using decoder 605
                                             // 76661:   }
  2, 9,                                      // 76666:   case 0x2: {
  OPC_CheckField, 0, 5, 0,                   // 76668:    check Inst[4:0] == 0x0
  OPC_Decode, 225, 33, 221, 4,               // 76672:    decode to HLT using decoder 605
                                             // 76672:   }
  5, 26,                                     // 76677:   case 0x5: {
  OPC_SwitchField, 0, 5,                     // 76679:    switch Inst[4:0] {
  1, 5,                                      // 76682:    case 0x1: {
  OPC_Decode, 181, 20, 221, 4,               // 76684:     decode to DCPS1 using decoder 605
                                             // 76684:    }
  2, 5,                                      // 76689:    case 0x2: {
  OPC_Decode, 182, 20, 221, 4,               // 76691:     decode to DCPS2 using decoder 605
                                             // 76691:    }
  3, 0,                                      // 76696:    case 0x3: {
  OPC_CheckPredicate, 107,                   // 76698:     check predicate 107
  OPC_Decode, 183, 20, 221, 4,               // 76700:     decode to DCPS3 using decoder 605
                                             // 76700:    }
                                             // 76700:    } // switch Inst[4:0]
                                             // 76700:   }
  7, 19,                                     // 76705:   case 0x7: {
  OPC_CheckPredicate, 108,                   // 76707:    check predicate 108
  OPC_CheckField, 18, 3, 0,                  // 76709:    check Inst[20:18] == 0x0
  OPC_CheckField, 12, 5, 0,                  // 76713:    check Inst[16:12] == 0x0
  OPC_CheckField, 0, 5, 0,                   // 76717:    check Inst[4:0] == 0x0
  OPC_Decode, 225, 61, 222, 4,               // 76721:    decode to TENTER using decoder 606
                                             // 76721:   }
  8, 237, 3,                                 // 76726:   case 0x8: {
  OPC_SwitchField, 19, 2,                    // 76729:    switch Inst[20:19] {
  0, 249, 2,                                 // 76732:    case 0x0: {
  OPC_SwitchField, 12, 4,                    // 76735:     switch Inst[15:12] {
  1, 29,                                     // 76738:     case 0x1: {
  OPC_SwitchField, 5, 7,                     // 76740:      switch Inst[11:5] {
  0, 11,                                     // 76743:      case 0x0: {
  OPC_CheckPredicate, 109,                   // 76745:       check predicate 109
  OPC_CheckField, 16, 3, 3,                  // 76747:       check Inst[18:16] == 0x3
  OPC_Decode, 217, 69, 223, 4,               // 76751:       decode to WFET using decoder 607
                                             // 76751:      }
  1, 0,                                      // 76756:      case 0x1: {
  OPC_CheckPredicate, 109,                   // 76758:       check predicate 109
  OPC_CheckField, 16, 3, 3,                  // 76760:       check Inst[18:16] == 0x3
  OPC_Decode, 218, 69, 223, 4,               // 76764:       decode to WFIT using decoder 607
                                             // 76764:      }
                                             // 76764:      } // switch Inst[11:5]
                                             // 76764:     }
  2, 175, 1,                                 // 76769:     case 0x2: {
  OPC_CheckField, 0, 5, 31,                  // 76772:      check Inst[4:0] == 0x1f
  OPC_CheckField, 16, 3, 3,                  // 76776:      check Inst[18:16] == 0x3
  OPC_Scope, 133, 1,                         // 76780:      try {
  OPC_SwitchField, 5, 7,                     // 76783:       switch Inst[11:5] {
  0, 5,                                      // 76786:       case 0x0: {
  OPC_Decode, 180, 43, 165, 2,               // 76788:        decode to NOP using decoder 293
                                             // 76788:       }
  7, 5,                                      // 76793:       case 0x7: {
  OPC_Decode, 236, 70, 165, 2,               // 76795:        decode to XPACLRI using decoder 293
                                             // 76795:       }
  8, 5,                                      // 76800:       case 0x8: {
  OPC_Decode, 231, 43, 165, 2,               // 76802:        decode to PACIA1716 using decoder 293
                                             // 76802:       }
  10, 5,                                     // 76807:       case 0xa: {
  OPC_Decode, 237, 43, 165, 2,               // 76809:        decode to PACIB1716 using decoder 293
                                             // 76809:       }
  12, 5,                                     // 76814:       case 0xc: {
  OPC_Decode, 207, 14, 165, 2,               // 76816:        decode to AUTIA1716 using decoder 293
                                             // 76816:       }
  14, 5,                                     // 76821:       case 0xe: {
  OPC_Decode, 214, 14, 165, 2,               // 76823:        decode to AUTIB1716 using decoder 293
                                             // 76823:       }
  18, 7,                                     // 76828:       case 0x12: {
  OPC_CheckPredicate, 110,                   // 76830:        check predicate 110
  OPC_Decode, 132, 62, 224, 4,               // 76832:        decode to TSB using decoder 608
                                             // 76832:       }
  24, 5,                                     // 76837:       case 0x18: {
  OPC_Decode, 235, 43, 165, 2,               // 76839:        decode to PACIAZ using decoder 293
                                             // 76839:       }
  25, 5,                                     // 76844:       case 0x19: {
  OPC_Decode, 233, 43, 165, 2,               // 76846:        decode to PACIASP using decoder 293
                                             // 76846:       }
  26, 5,                                     // 76851:       case 0x1a: {
  OPC_Decode, 241, 43, 165, 2,               // 76853:        decode to PACIBZ using decoder 293
                                             // 76853:       }
  27, 5,                                     // 76858:       case 0x1b: {
  OPC_Decode, 239, 43, 165, 2,               // 76860:        decode to PACIBSP using decoder 293
                                             // 76860:       }
  28, 5,                                     // 76865:       case 0x1c: {
  OPC_Decode, 212, 14, 165, 2,               // 76867:        decode to AUTIAZ using decoder 293
                                             // 76867:       }
  29, 5,                                     // 76872:       case 0x1d: {
  OPC_Decode, 209, 14, 165, 2,               // 76874:        decode to AUTIASP using decoder 293
                                             // 76874:       }
  30, 5,                                     // 76879:       case 0x1e: {
  OPC_Decode, 219, 14, 165, 2,               // 76881:        decode to AUTIBZ using decoder 293
                                             // 76881:       }
  31, 5,                                     // 76886:       case 0x1f: {
  OPC_Decode, 216, 14, 165, 2,               // 76888:        decode to AUTIBSP using decoder 293
                                             // 76888:       }
  39, 5,                                     // 76893:       case 0x27: {
  OPC_Decode, 244, 43, 165, 2,               // 76895:        decode to PACM using decoder 293
                                             // 76895:       }
  40, 5,                                     // 76900:       case 0x28: {
  OPC_Decode, 149, 17, 165, 2,               // 76902:        decode to CHKFEAT using decoder 293
                                             // 76902:       }
  52, 0,                                     // 76907:       case 0x34: {
  OPC_CheckPredicate, 111,                   // 76909:        check predicate 111
  OPC_Decode, 154, 58, 165, 2,               // 76911:        decode to STCPH using decoder 293
                                             // 76911:       }
                                             // 76911:       } // switch Inst[11:5]
  OPC_Scope, 11,                             // 76916:      } else try {
  OPC_CheckField, 6, 6, 25,                  // 76918:       check Inst[11:6] == 0x19
  OPC_CheckPredicate, 111,                   // 76922:       check predicate 111
  OPC_Decode, 195, 48, 225, 4,               // 76924:       decode to SHUH using decoder 609
  OPC_Scope, 11,                             // 76929:      } else try {
  OPC_CheckField, 8, 4, 6,                   // 76931:       check Inst[11:8] == 0x6
  OPC_CheckPredicate, 112,                   // 76935:       check predicate 112
  OPC_Decode, 215, 59, 226, 4,               // 76937:       decode to STSHH using decoder 610
                                             // 76937:      } else try {
  OPC_Decode, 221, 33, 227, 4,               // 76942:       decode to HINT using decoder 611
                                             // 76942:      }
                                             // 76942:     }
  3, 85,                                     // 76947:     case 0x3: {
  OPC_SwitchField, 0, 8,                     // 76949:      switch Inst[7:0] {
  63, 15,                                    // 76952:      case 0x3f: {
  OPC_CheckPredicate, 113,                   // 76954:       check predicate 113
  OPC_CheckField, 16, 3, 3,                  // 76956:       check Inst[18:16] == 0x3
  OPC_CheckField, 8, 2, 2,                   // 76960:       check Inst[9:8] == 0x2
  OPC_Decode, 201, 20, 228, 4,               // 76964:       decode to DSBnXS using decoder 612
                                             // 76964:      }
  95, 9,                                     // 76969:      case 0x5f: {
  OPC_CheckField, 16, 3, 3,                  // 76971:       check Inst[18:16] == 0x3
  OPC_Decode, 174, 17, 229, 4,               // 76975:       decode to CLREX using decoder 613
                                             // 76975:      }
  159, 1, 9,                                 // 76980:      case 0x9f: {
  OPC_CheckField, 16, 3, 3,                  // 76983:       check Inst[18:16] == 0x3
  OPC_Decode, 200, 20, 229, 4,               // 76987:       decode to DSB using decoder 613
                                             // 76987:      }
  191, 1, 9,                                 // 76992:      case 0xbf: {
  OPC_CheckField, 16, 3, 3,                  // 76995:       check Inst[18:16] == 0x3
  OPC_Decode, 198, 20, 229, 4,               // 76999:       decode to DMB using decoder 613
                                             // 76999:      }
  223, 1, 9,                                 // 77004:      case 0xdf: {
  OPC_CheckField, 16, 3, 3,                  // 77007:       check Inst[18:16] == 0x3
  OPC_Decode, 156, 34, 229, 4,               // 77011:       decode to ISB using decoder 613
                                             // 77011:      }
  255, 1, 0,                                 // 77016:      case 0xff: {
  OPC_CheckPredicate, 114,                   // 77019:       check predicate 114
  OPC_CheckField, 16, 3, 3,                  // 77021:       check Inst[18:16] == 0x3
  OPC_SoftFail, 128, 30, 0,                  // 77025:       softfail pos=0x00000f00 neg=0x00000000
  OPC_Decode, 226, 46, 165, 2,               // 77029:       decode to SB using decoder 293
                                             // 77029:      }
                                             // 77029:      } // switch Inst[7:0]
                                             // 77029:     }
  4, 0,                                      // 77034:     case 0x4: {
  OPC_CheckField, 0, 5, 31,                  // 77036:      check Inst[4:0] == 0x1f
  OPC_Scope, 65,                             // 77040:      try {
  OPC_SwitchField, 5, 3,                     // 77042:       switch Inst[7:5] {
  0, 15,                                     // 77045:       case 0x0: {
  OPC_CheckPredicate, 115,                   // 77047:        check predicate 115
  OPC_CheckField, 16, 3, 0,                  // 77049:        check Inst[18:16] == 0x0
  OPC_SoftFail, 128, 30, 0,                  // 77053:        softfail pos=0x00000f00 neg=0x00000000
  OPC_Decode, 148, 17, 165, 2,               // 77057:        decode to CFINV using decoder 293
                                             // 77057:       }
  1, 15,                                     // 77062:       case 0x1: {
  OPC_CheckPredicate, 116,                   // 77064:        check predicate 116
  OPC_CheckField, 16, 3, 0,                  // 77066:        check Inst[18:16] == 0x0
  OPC_SoftFail, 128, 30, 0,                  // 77070:        softfail pos=0x00000f00 neg=0x00000000
  OPC_Decode, 228, 70, 165, 2,               // 77074:        decode to XAFLAG using decoder 293
                                             // 77074:       }
  2, 15,                                     // 77079:       case 0x2: {
  OPC_CheckPredicate, 116,                   // 77081:        check predicate 116
  OPC_CheckField, 16, 3, 0,                  // 77083:        check Inst[18:16] == 0x0
  OPC_SoftFail, 128, 30, 0,                  // 77087:        softfail pos=0x00000f00 neg=0x00000000
  OPC_Decode, 222, 14, 165, 2,               // 77091:        decode to AXFLAG using decoder 293
                                             // 77091:       }
  3, 0,                                      // 77096:       case 0x3: {
  OPC_CheckField, 16, 3, 3,                  // 77098:        check Inst[18:16] == 0x3
  OPC_Decode, 252, 42, 230, 4,               // 77102:        decode to MSRpstatesvcrImm1 using decoder 614
                                             // 77102:       }
                                             // 77102:       } // switch Inst[7:5]
                                             // 77102:      } else try {
  OPC_Decode, 251, 42, 231, 4,               // 77107:       decode to MSRpstateImm4 using decoder 615
                                             // 77107:      }
                                             // 77107:     }
                                             // 77107:     } // switch Inst[15:12]
                                             // 77107:    }
  1, 0,                                      // 77112:    case 0x1: {
  OPC_Scope, 58,                             // 77114:     try {
  OPC_SwitchField, 0, 19,                    // 77116:      switch Inst[18:0] {
  159, 239, 1, 7,                            // 77119:      case 0x779f: {
  OPC_CheckPredicate, 117,                   // 77123:       check predicate 117
  OPC_Decode, 214, 32, 165, 2,               // 77125:       decode to GCSPUSHX using decoder 293
                                             // 77125:      }
  191, 239, 1, 7,                            // 77130:      case 0x77bf: {
  OPC_CheckPredicate, 117,                   // 77134:       check predicate 117
  OPC_Decode, 210, 32, 165, 2,               // 77136:       decode to GCSPOPCX using decoder 293
                                             // 77136:      }
  223, 239, 1, 7,                            // 77141:      case 0x77df: {
  OPC_CheckPredicate, 117,                   // 77145:       check predicate 117
  OPC_Decode, 212, 32, 165, 2,               // 77147:       decode to GCSPOPX using decoder 293
                                             // 77147:      }
  159, 229, 5, 7,                            // 77152:      case 0x1729f: {
  OPC_CheckPredicate, 118,                   // 77156:       check predicate 118
  OPC_Decode, 167, 16, 165, 2,               // 77158:       decode to BRB_IALL using decoder 293
                                             // 77158:      }
  191, 229, 5, 0,                            // 77163:      case 0x172bf: {
  OPC_CheckPredicate, 118,                   // 77167:       check predicate 118
  OPC_Decode, 168, 16, 165, 2,               // 77169:       decode to BRB_INJ using decoder 293
                                             // 77169:      }
                                             // 77169:      } // switch Inst[18:0]
  OPC_Scope, 41,                             // 77174:     } else try {
  OPC_SwitchField, 5, 14,                    // 77176:      switch Inst[18:5] {
  151, 55, 7,                                // 77179:      case 0x1b97: {
  OPC_CheckPredicate, 119,                   // 77182:       check predicate 119
  OPC_Decode, 227, 61, 223, 4,               // 77184:       decode to TRCIT using decoder 607
                                             // 77184:      }
  184, 55, 7,                                // 77189:      case 0x1bb8: {
  OPC_CheckPredicate, 117,                   // 77192:       check predicate 117
  OPC_Decode, 213, 32, 223, 4,               // 77194:       decode to GCSPUSHM using decoder 607
                                             // 77194:      }
  186, 55, 7,                                // 77199:      case 0x1bba: {
  OPC_CheckPredicate, 117,                   // 77202:       check predicate 117
  OPC_Decode, 215, 32, 223, 4,               // 77204:       decode to GCSSS1 using decoder 607
                                             // 77204:      }
  128, 103, 0,                               // 77209:      case 0x3380: {
  OPC_Decode, 173, 14, 223, 4,               // 77212:       decode to APAS using decoder 607
                                             // 77212:      }
                                             // 77212:      } // switch Inst[18:5]
                                             // 77212:     } else try {
  OPC_Decode, 180, 61, 232, 4,               // 77217:      decode to SYSxt using decoder 616
                                             // 77217:     }
                                             // 77217:    }
                                             // 77217:    } // switch Inst[20:19]
                                             // 77217:   }
  9, 34,                                     // 77222:   case 0x9: {
  OPC_CheckField, 19, 2, 1,                  // 77224:    check Inst[20:19] == 0x1
  OPC_Scope, 23,                             // 77228:    try {
  OPC_SwitchField, 5, 14,                    // 77230:     switch Inst[18:5] {
  185, 55, 7,                                // 77233:     case 0x1bb9: {
  OPC_CheckPredicate, 117,                   // 77236:      check predicate 117
  OPC_Decode, 211, 32, 233, 4,               // 77238:      decode to GCSPOPM using decoder 617
                                             // 77238:     }
  187, 55, 0,                                // 77243:     case 0x1bbb: {
  OPC_CheckPredicate, 117,                   // 77246:      check predicate 117
  OPC_Decode, 216, 32, 233, 4,               // 77248:      decode to GCSSS2 using decoder 617
                                             // 77248:     }
                                             // 77248:     } // switch Inst[18:5]
                                             // 77248:    } else try {
  OPC_Decode, 177, 61, 234, 4,               // 77253:     decode to SYSLxt using decoder 618
                                             // 77253:    }
                                             // 77253:   }
  10, 37,                                    // 77258:   case 0xa: {
  OPC_Scope, 15,                             // 77260:    try {
  OPC_CheckField, 0, 5, 31,                  // 77262:     check Inst[4:0] == 0x1f
  OPC_CheckPredicate, 120,                   // 77266:     check predicate 120
  OPC_CheckField, 19, 2, 1,                  // 77268:     check Inst[20:19] == 0x1
  OPC_Decode, 179, 61, 235, 4,               // 77272:     decode to SYSPxt_XZR using decoder 619
  OPC_Scope, 11,                             // 77277:    } else try {
  OPC_CheckField, 19, 2, 1,                  // 77279:     check Inst[20:19] == 0x1
  OPC_CheckPredicate, 120,                   // 77283:     check predicate 120
  OPC_Decode, 178, 61, 236, 4,               // 77285:     decode to SYSPxt using decoder 620
                                             // 77285:    } else try {
  OPC_CheckPredicate, 120,                   // 77290:     check predicate 120
  OPC_Decode, 249, 42, 237, 4,               // 77292:     decode to MSRR using decoder 621
                                             // 77292:    }
                                             // 77292:   }
  11, 7,                                     // 77297:   case 0xb: {
  OPC_CheckPredicate, 120,                   // 77299:    check predicate 120
  OPC_Decode, 242, 42, 238, 4,               // 77301:    decode to MRRS using decoder 622
                                             // 77301:   }
  12, 55,                                    // 77306:   case 0xc: {
  OPC_SwitchField, 18, 3,                    // 77308:    switch Inst[20:18] {
  0, 11,                                     // 77311:    case 0x0: {
  OPC_CheckPredicate, 121,                   // 77313:     check predicate 121
  OPC_CheckField, 10, 7, 0,                  // 77315:     check Inst[16:10] == 0x0
  OPC_Decode, 224, 61, 239, 4,               // 77319:     decode to TCHANGEFrr using decoder 623
                                             // 77319:    }
  1, 11,                                     // 77324:    case 0x1: {
  OPC_CheckPredicate, 121,                   // 77326:     check predicate 121
  OPC_CheckField, 10, 7, 0,                  // 77328:     check Inst[16:10] == 0x0
  OPC_Decode, 222, 61, 239, 4,               // 77332:     decode to TCHANGEBrr using decoder 623
                                             // 77332:    }
  4, 11,                                     // 77337:    case 0x4: {
  OPC_CheckPredicate, 121,                   // 77339:     check predicate 121
  OPC_CheckField, 12, 5, 0,                  // 77341:     check Inst[16:12] == 0x0
  OPC_Decode, 223, 61, 240, 4,               // 77345:     decode to TCHANGEFri using decoder 624
                                             // 77345:    }
  5, 0,                                      // 77350:    case 0x5: {
  OPC_CheckPredicate, 121,                   // 77352:     check predicate 121
  OPC_CheckField, 12, 5, 0,                  // 77354:     check Inst[16:12] == 0x0
  OPC_Decode, 221, 61, 240, 4,               // 77358:     decode to TCHANGEBri using decoder 624
                                             // 77358:    }
                                             // 77358:    } // switch Inst[20:18]
                                             // 77358:   }
  16, 43,                                    // 77363:   case 0x10: {
  OPC_SwitchField, 10, 11,                   // 77365:    switch Inst[20:10] {
  192, 15, 9,                                // 77368:    case 0x7c0: {
  OPC_CheckField, 0, 5, 0,                   // 77371:     check Inst[4:0] == 0x0
  OPC_Decode, 162, 16, 241, 4,               // 77375:     decode to BR using decoder 625
                                             // 77375:    }
  194, 15, 11,                               // 77380:    case 0x7c2: {
  OPC_CheckPredicate, 122,                   // 77383:     check predicate 122
  OPC_CheckField, 0, 5, 31,                  // 77385:     check Inst[4:0] == 0x1f
  OPC_Decode, 164, 16, 241, 4,               // 77389:     decode to BRAAZ using decoder 625
                                             // 77389:    }
  195, 15, 0,                                // 77394:    case 0x7c3: {
  OPC_CheckPredicate, 122,                   // 77397:     check predicate 122
  OPC_CheckField, 0, 5, 31,                  // 77399:     check Inst[4:0] == 0x1f
  OPC_Decode, 166, 16, 241, 4,               // 77403:     decode to BRABZ using decoder 625
                                             // 77403:    }
                                             // 77403:    } // switch Inst[20:10]
                                             // 77403:   }
  17, 43,                                    // 77408:   case 0x11: {
  OPC_SwitchField, 10, 11,                   // 77410:    switch Inst[20:10] {
  192, 15, 9,                                // 77413:    case 0x7c0: {
  OPC_CheckField, 0, 5, 0,                   // 77416:     check Inst[4:0] == 0x0
  OPC_Decode, 155, 16, 241, 4,               // 77420:     decode to BLR using decoder 625
                                             // 77420:    }
  194, 15, 11,                               // 77425:    case 0x7c2: {
  OPC_CheckPredicate, 122,                   // 77428:     check predicate 122
  OPC_CheckField, 0, 5, 31,                  // 77430:     check Inst[4:0] == 0x1f
  OPC_Decode, 157, 16, 241, 4,               // 77434:     decode to BLRAAZ using decoder 625
                                             // 77434:    }
  195, 15, 0,                                // 77439:    case 0x7c3: {
  OPC_CheckPredicate, 122,                   // 77442:     check predicate 122
  OPC_CheckField, 0, 5, 31,                  // 77444:     check Inst[4:0] == 0x1f
  OPC_Decode, 159, 16, 241, 4,               // 77448:     decode to BLRABZ using decoder 625
                                             // 77448:    }
                                             // 77448:    } // switch Inst[20:10]
                                             // 77448:   }
  18, 69,                                    // 77453:   case 0x12: {
  OPC_SwitchField, 10, 11,                   // 77455:    switch Inst[20:10] {
  192, 15, 9,                                // 77458:    case 0x7c0: {
  OPC_CheckField, 0, 5, 0,                   // 77461:     check Inst[4:0] == 0x0
  OPC_Decode, 183, 45, 241, 4,               // 77465:     decode to RET using decoder 625
                                             // 77465:    }
  194, 15, 24,                               // 77470:    case 0x7c2: {
  OPC_CheckField, 5, 5, 31,                  // 77473:     check Inst[9:5] == 0x1f
  OPC_Scope, 11,                             // 77477:     try {
  OPC_CheckField, 0, 5, 31,                  // 77479:      check Inst[4:0] == 0x1f
  OPC_CheckPredicate, 122,                   // 77483:      check predicate 122
  OPC_Decode, 184, 45, 165, 2,               // 77485:      decode to RETAA using decoder 293
                                             // 77485:     } else try {
  OPC_CheckPredicate, 104,                   // 77490:      check predicate 104
  OPC_Decode, 186, 45, 242, 4,               // 77492:      decode to RETAASPPCr using decoder 626
                                             // 77492:     }
                                             // 77492:    }
  195, 15, 0,                                // 77497:    case 0x7c3: {
  OPC_CheckField, 5, 5, 31,                  // 77500:     check Inst[9:5] == 0x1f
  OPC_Scope, 11,                             // 77504:     try {
  OPC_CheckField, 0, 5, 31,                  // 77506:      check Inst[4:0] == 0x1f
  OPC_CheckPredicate, 122,                   // 77510:      check predicate 122
  OPC_Decode, 187, 45, 165, 2,               // 77512:      decode to RETAB using decoder 293
                                             // 77512:     } else try {
  OPC_CheckPredicate, 104,                   // 77517:      check predicate 104
  OPC_Decode, 189, 45, 242, 4,               // 77519:      decode to RETABSPPCr using decoder 626
                                             // 77519:     }
                                             // 77519:    }
                                             // 77519:    } // switch Inst[20:10]
                                             // 77519:   }
  20, 34,                                    // 77524:   case 0x14: {
  OPC_SwitchField, 0, 21,                    // 77526:    switch Inst[20:0] {
  224, 135, 124, 5,                          // 77529:    case 0x1f03e0: {
  OPC_Decode, 144, 21, 165, 2,               // 77533:     decode to ERET using decoder 293
                                             // 77533:    }
  255, 151, 124, 7,                          // 77538:    case 0x1f0bff: {
  OPC_CheckPredicate, 122,                   // 77542:     check predicate 122
  OPC_Decode, 145, 21, 165, 2,               // 77544:     decode to ERETAA using decoder 293
                                             // 77544:    }
  255, 159, 124, 0,                          // 77549:    case 0x1f0fff: {
  OPC_CheckPredicate, 122,                   // 77553:     check predicate 122
  OPC_Decode, 146, 21, 165, 2,               // 77555:     decode to ERETAB using decoder 293
                                             // 77555:    }
                                             // 77555:    } // switch Inst[20:0]
                                             // 77555:   }
  21, 11,                                    // 77560:   case 0x15: {
  OPC_CheckField, 0, 21, 224, 135, 124,      // 77562:    check Inst[20:0] == 0x1f03e0
  OPC_Decode, 199, 20, 165, 2,               // 77568:    decode to DRPS using decoder 293
                                             // 77568:   }
  23, 17,                                    // 77573:   case 0x17: {
  OPC_CheckPredicate, 108,                   // 77575:    check predicate 108
  OPC_CheckField, 11, 10, 224, 7,            // 77577:    check Inst[20:11] == 0x3e0
  OPC_CheckField, 0, 10, 224, 7,             // 77582:    check Inst[9:0] == 0x3e0
  OPC_Decode, 226, 61, 243, 4,               // 77587:    decode to TEXIT using decoder 627
                                             // 77587:   }
  24, 23,                                    // 77592:   case 0x18: {
  OPC_SwitchField, 10, 11,                   // 77594:    switch Inst[20:10] {
  194, 15, 7,                                // 77597:    case 0x7c2: {
  OPC_CheckPredicate, 122,                   // 77600:     check predicate 122
  OPC_Decode, 163, 16, 244, 4,               // 77602:     decode to BRAA using decoder 628
                                             // 77602:    }
  195, 15, 0,                                // 77607:    case 0x7c3: {
  OPC_CheckPredicate, 122,                   // 77610:     check predicate 122
  OPC_Decode, 165, 16, 244, 4,               // 77612:     decode to BRAB using decoder 628
                                             // 77612:    }
                                             // 77612:    } // switch Inst[20:10]
                                             // 77612:   }
  25, 0,                                     // 77617:   case 0x19: {
  OPC_SwitchField, 10, 11,                   // 77619:    switch Inst[20:10] {
  194, 15, 7,                                // 77622:    case 0x7c2: {
  OPC_CheckPredicate, 122,                   // 77625:     check predicate 122
  OPC_Decode, 156, 16, 244, 4,               // 77627:     decode to BLRAA using decoder 628
                                             // 77627:    }
  195, 15, 0,                                // 77632:    case 0x7c3: {
  OPC_CheckPredicate, 122,                   // 77635:     check predicate 122
  OPC_Decode, 158, 16, 244, 4,               // 77637:     decode to BLRAB using decoder 628
                                             // 77637:    }
                                             // 77637:    } // switch Inst[20:10]
                                             // 77637:   }
                                             // 77637:   } // switch Inst[25:21]
                                             // 77637:  }
  7, 0,                                      // 77642:  case 0x7: {
  OPC_SwitchField, 21, 5,                    // 77644:   switch Inst[25:21] {
  0, 11,                                     // 77647:   case 0x0: {
  OPC_CheckPredicate, 106,                   // 77649:    check predicate 106
  OPC_CheckField, 14, 2, 0,                  // 77651:    check Inst[15:14] == 0x0
  OPC_Decode, 239, 16, 245, 4,               // 77655:    decode to CBGTXrr using decoder 629
                                             // 77655:   }
  1, 11,                                     // 77660:   case 0x1: {
  OPC_CheckPredicate, 106,                   // 77662:    check predicate 106
  OPC_CheckField, 14, 2, 0,                  // 77664:    check Inst[15:14] == 0x0
  OPC_Decode, 235, 16, 245, 4,               // 77668:    decode to CBGEXrr using decoder 629
                                             // 77668:   }
  2, 11,                                     // 77673:   case 0x2: {
  OPC_CheckPredicate, 106,                   // 77675:    check predicate 106
  OPC_CheckField, 14, 2, 0,                  // 77677:    check Inst[15:14] == 0x0
  OPC_Decode, 248, 16, 245, 4,               // 77681:    decode to CBHIXrr using decoder 629
                                             // 77681:   }
  3, 11,                                     // 77686:   case 0x3: {
  OPC_CheckPredicate, 106,                   // 77688:    check predicate 106
  OPC_CheckField, 14, 2, 0,                  // 77690:    check Inst[15:14] == 0x0
  OPC_Decode, 251, 16, 245, 4,               // 77694:    decode to CBHSXrr using decoder 629
                                             // 77694:   }
  6, 11,                                     // 77699:   case 0x6: {
  OPC_CheckPredicate, 106,                   // 77701:    check predicate 106
  OPC_CheckField, 14, 2, 0,                  // 77703:    check Inst[15:14] == 0x0
  OPC_Decode, 233, 16, 245, 4,               // 77707:    decode to CBEQXrr using decoder 629
                                             // 77707:   }
  7, 11,                                     // 77712:   case 0x7: {
  OPC_CheckPredicate, 106,                   // 77714:    check predicate 106
  OPC_CheckField, 14, 2, 0,                  // 77716:    check Inst[15:14] == 0x0
  OPC_Decode, 131, 17, 245, 4,               // 77720:    decode to CBNEXrr using decoder 629
                                             // 77720:   }
  8, 11,                                     // 77725:   case 0x8: {
  OPC_CheckPredicate, 106,                   // 77727:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 77729:    check Inst[14] == 0x0
  OPC_Decode, 238, 16, 246, 4,               // 77733:    decode to CBGTXri using decoder 630
                                             // 77733:   }
  9, 11,                                     // 77738:   case 0x9: {
  OPC_CheckPredicate, 106,                   // 77740:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 77742:    check Inst[14] == 0x0
  OPC_Decode, 255, 16, 246, 4,               // 77746:    decode to CBLTXri using decoder 630
                                             // 77746:   }
  10, 11,                                    // 77751:   case 0xa: {
  OPC_CheckPredicate, 106,                   // 77753:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 77755:    check Inst[14] == 0x0
  OPC_Decode, 247, 16, 246, 4,               // 77759:    decode to CBHIXri using decoder 630
                                             // 77759:   }
  11, 11,                                    // 77764:   case 0xb: {
  OPC_CheckPredicate, 106,                   // 77766:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 77768:    check Inst[14] == 0x0
  OPC_Decode, 253, 16, 246, 4,               // 77772:    decode to CBLOXri using decoder 630
                                             // 77772:   }
  14, 11,                                    // 77777:   case 0xe: {
  OPC_CheckPredicate, 106,                   // 77779:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 77781:    check Inst[14] == 0x0
  OPC_Decode, 232, 16, 246, 4,               // 77785:    decode to CBEQXri using decoder 630
                                             // 77785:   }
  15, 0,                                     // 77790:   case 0xf: {
  OPC_CheckPredicate, 106,                   // 77792:    check predicate 106
  OPC_CheckField, 14, 1, 0,                  // 77794:    check Inst[14] == 0x0
  OPC_Decode, 130, 17, 246, 4,               // 77798:    decode to CBNEXri using decoder 630
                                             // 77798:   }
                                             // 77798:   } // switch Inst[25:21]
                                             // 77798:  }
                                             // 77798:  } // switch Inst[31:29]
                                             // 77798: }
  6, 151, 49,                                // 77803: case 0x6: {
  OPC_SwitchField, 29, 3,                    // 77806:  switch Inst[31:29] {
  0, 193, 10,                                // 77809:  case 0x0: {
  OPC_SwitchField, 24, 2,                    // 77812:   switch Inst[25:24] {
  0, 5,                                      // 77815:   case 0x0: {
  OPC_Decode, 232, 39, 215, 4,               // 77817:    decode to LDRWl using decoder 599
                                             // 77817:   }
  1, 178, 8,                                 // 77822:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 77825:    switch Inst[23:21] {
  0, 161, 1,                                 // 77828:    case 0x0: {
  OPC_SwitchField, 10, 2,                    // 77831:     switch Inst[11:10] {
  0, 7,                                      // 77834:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 77836:      check predicate 123
  OPC_Decode, 210, 58, 247, 4,               // 77838:      decode to STLURBi using decoder 631
                                             // 77838:     }
  1, 0,                                      // 77843:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 77845:      switch Inst[15:12] {
  0, 7,                                      // 77848:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 77850:       check predicate 124
  OPC_Decode, 223, 19, 248, 4,               // 77852:       decode to CPYFP using decoder 632
                                             // 77852:      }
  1, 7,                                      // 77857:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 77859:       check predicate 124
  OPC_Decode, 235, 19, 248, 4,               // 77861:       decode to CPYFPWT using decoder 632
                                             // 77861:      }
  2, 7,                                      // 77866:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 77868:       check predicate 124
  OPC_Decode, 226, 19, 248, 4,               // 77870:       decode to CPYFPRT using decoder 632
                                             // 77870:      }
  3, 7,                                      // 77875:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 77877:       check predicate 124
  OPC_Decode, 230, 19, 248, 4,               // 77879:       decode to CPYFPT using decoder 632
                                             // 77879:      }
  4, 7,                                      // 77884:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 77886:       check predicate 124
  OPC_Decode, 234, 19, 248, 4,               // 77888:       decode to CPYFPWN using decoder 632
                                             // 77888:      }
  5, 7,                                      // 77893:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 77895:       check predicate 124
  OPC_Decode, 238, 19, 248, 4,               // 77897:       decode to CPYFPWTWN using decoder 632
                                             // 77897:      }
  6, 7,                                      // 77902:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 77904:       check predicate 124
  OPC_Decode, 229, 19, 248, 4,               // 77906:       decode to CPYFPRTWN using decoder 632
                                             // 77906:      }
  7, 7,                                      // 77911:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 77913:       check predicate 124
  OPC_Decode, 233, 19, 248, 4,               // 77915:       decode to CPYFPTWN using decoder 632
                                             // 77915:      }
  8, 7,                                      // 77920:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 77922:       check predicate 124
  OPC_Decode, 225, 19, 248, 4,               // 77924:       decode to CPYFPRN using decoder 632
                                             // 77924:      }
  9, 7,                                      // 77929:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 77931:       check predicate 124
  OPC_Decode, 237, 19, 248, 4,               // 77933:       decode to CPYFPWTRN using decoder 632
                                             // 77933:      }
  10, 7,                                     // 77938:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 77940:       check predicate 124
  OPC_Decode, 228, 19, 248, 4,               // 77942:       decode to CPYFPRTRN using decoder 632
                                             // 77942:      }
  11, 7,                                     // 77947:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 77949:       check predicate 124
  OPC_Decode, 232, 19, 248, 4,               // 77951:       decode to CPYFPTRN using decoder 632
                                             // 77951:      }
  12, 7,                                     // 77956:      case 0xc: {
  OPC_CheckPredicate, 124,                   // 77958:       check predicate 124
  OPC_Decode, 224, 19, 248, 4,               // 77960:       decode to CPYFPN using decoder 632
                                             // 77960:      }
  13, 7,                                     // 77965:      case 0xd: {
  OPC_CheckPredicate, 124,                   // 77967:       check predicate 124
  OPC_Decode, 236, 19, 248, 4,               // 77969:       decode to CPYFPWTN using decoder 632
                                             // 77969:      }
  14, 7,                                     // 77974:      case 0xe: {
  OPC_CheckPredicate, 124,                   // 77976:       check predicate 124
  OPC_Decode, 227, 19, 248, 4,               // 77978:       decode to CPYFPRTN using decoder 632
                                             // 77978:      }
  15, 0,                                     // 77983:      case 0xf: {
  OPC_CheckPredicate, 124,                   // 77985:       check predicate 124
  OPC_Decode, 231, 19, 248, 4,               // 77987:       decode to CPYFPTN using decoder 632
                                             // 77987:      }
                                             // 77987:      } // switch Inst[15:12]
                                             // 77987:     }
                                             // 77987:     } // switch Inst[11:10]
                                             // 77987:    }
  1, 111,                                    // 77992:    case 0x1: {
  OPC_SwitchField, 10, 6,                    // 77994:     switch Inst[15:10] {
  1, 7,                                      // 77997:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 77999:      check predicate 74
  OPC_Decode, 178, 40, 249, 4,               // 78001:      decode to LDTADDW using decoder 633
                                             // 78001:     }
  2, 7,                                      // 78006:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 78008:      check predicate 125
  OPC_Decode, 242, 44, 240, 2,               // 78010:      decode to RCWCAS using decoder 368
                                             // 78010:     }
  3, 7,                                      // 78015:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 78017:      check predicate 126
  OPC_Decode, 246, 44, 237, 2,               // 78019:      decode to RCWCASP using decoder 365
                                             // 78019:     }
  4, 7,                                      // 78024:     case 0x4: {
  OPC_CheckPredicate, 127,                   // 78026:      check predicate 127
  OPC_Decode, 221, 37, 250, 4,               // 78028:      decode to LDCLRP using decoder 634
                                             // 78028:     }
  5, 7,                                      // 78033:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 78035:      check predicate 74
  OPC_Decode, 186, 40, 249, 4,               // 78037:      decode to LDTCLRW using decoder 633
                                             // 78037:     }
  12, 7,                                     // 78042:     case 0xc: {
  OPC_CheckPredicate, 127,                   // 78044:      check predicate 127
  OPC_Decode, 134, 40, 250, 4,               // 78046:      decode to LDSETP using decoder 634
                                             // 78046:     }
  13, 7,                                     // 78051:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 78053:      check predicate 74
  OPC_Decode, 211, 40, 249, 4,               // 78055:      decode to LDTSETW using decoder 633
                                             // 78055:     }
  32, 7,                                     // 78060:     case 0x20: {
  OPC_CheckPredicate, 127,                   // 78062:      check predicate 127
  OPC_Decode, 151, 61, 250, 4,               // 78064:      decode to SWPP using decoder 634
                                             // 78064:     }
  33, 7,                                     // 78069:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 78071:      check predicate 74
  OPC_Decode, 161, 61, 249, 4,               // 78073:      decode to SWPTW using decoder 633
                                             // 78073:     }
  36, 7,                                     // 78078:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 78080:      check predicate 126
  OPC_Decode, 254, 44, 250, 4,               // 78082:      decode to RCWCLRP using decoder 634
                                             // 78082:     }
  40, 7,                                     // 78087:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 78089:      check predicate 126
  OPC_Decode, 166, 45, 250, 4,               // 78091:      decode to RCWSWPP using decoder 634
                                             // 78091:     }
  44, 0,                                     // 78096:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 78098:      check predicate 126
  OPC_Decode, 150, 45, 250, 4,               // 78100:      decode to RCWSETP using decoder 634
                                             // 78100:     }
                                             // 78100:     } // switch Inst[15:10]
                                             // 78100:    }
  2, 161, 1,                                 // 78105:    case 0x2: {
  OPC_SwitchField, 10, 2,                    // 78108:     switch Inst[11:10] {
  0, 7,                                      // 78111:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 78113:      check predicate 123
  OPC_Decode, 160, 37, 247, 4,               // 78115:      decode to LDAPURBi using decoder 631
                                             // 78115:     }
  1, 0,                                      // 78120:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 78122:      switch Inst[15:12] {
  0, 7,                                      // 78125:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 78127:       check predicate 124
  OPC_Decode, 207, 19, 248, 4,               // 78129:       decode to CPYFM using decoder 632
                                             // 78129:      }
  1, 7,                                      // 78134:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 78136:       check predicate 124
  OPC_Decode, 219, 19, 248, 4,               // 78138:       decode to CPYFMWT using decoder 632
                                             // 78138:      }
  2, 7,                                      // 78143:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 78145:       check predicate 124
  OPC_Decode, 210, 19, 248, 4,               // 78147:       decode to CPYFMRT using decoder 632
                                             // 78147:      }
  3, 7,                                      // 78152:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 78154:       check predicate 124
  OPC_Decode, 214, 19, 248, 4,               // 78156:       decode to CPYFMT using decoder 632
                                             // 78156:      }
  4, 7,                                      // 78161:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 78163:       check predicate 124
  OPC_Decode, 218, 19, 248, 4,               // 78165:       decode to CPYFMWN using decoder 632
                                             // 78165:      }
  5, 7,                                      // 78170:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 78172:       check predicate 124
  OPC_Decode, 222, 19, 248, 4,               // 78174:       decode to CPYFMWTWN using decoder 632
                                             // 78174:      }
  6, 7,                                      // 78179:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 78181:       check predicate 124
  OPC_Decode, 213, 19, 248, 4,               // 78183:       decode to CPYFMRTWN using decoder 632
                                             // 78183:      }
  7, 7,                                      // 78188:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 78190:       check predicate 124
  OPC_Decode, 217, 19, 248, 4,               // 78192:       decode to CPYFMTWN using decoder 632
                                             // 78192:      }
  8, 7,                                      // 78197:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 78199:       check predicate 124
  OPC_Decode, 209, 19, 248, 4,               // 78201:       decode to CPYFMRN using decoder 632
                                             // 78201:      }
  9, 7,                                      // 78206:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 78208:       check predicate 124
  OPC_Decode, 221, 19, 248, 4,               // 78210:       decode to CPYFMWTRN using decoder 632
                                             // 78210:      }
  10, 7,                                     // 78215:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 78217:       check predicate 124
  OPC_Decode, 212, 19, 248, 4,               // 78219:       decode to CPYFMRTRN using decoder 632
                                             // 78219:      }
  11, 7,                                     // 78224:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 78226:       check predicate 124
  OPC_Decode, 216, 19, 248, 4,               // 78228:       decode to CPYFMTRN using decoder 632
                                             // 78228:      }
  12, 7,                                     // 78233:      case 0xc: {
  OPC_CheckPredicate, 124,                   // 78235:       check predicate 124
  OPC_Decode, 208, 19, 248, 4,               // 78237:       decode to CPYFMN using decoder 632
                                             // 78237:      }
  13, 7,                                     // 78242:      case 0xd: {
  OPC_CheckPredicate, 124,                   // 78244:       check predicate 124
  OPC_Decode, 220, 19, 248, 4,               // 78246:       decode to CPYFMWTN using decoder 632
                                             // 78246:      }
  14, 7,                                     // 78251:      case 0xe: {
  OPC_CheckPredicate, 124,                   // 78253:       check predicate 124
  OPC_Decode, 211, 19, 248, 4,               // 78255:       decode to CPYFMRTN using decoder 632
                                             // 78255:      }
  15, 0,                                     // 78260:      case 0xf: {
  OPC_CheckPredicate, 124,                   // 78262:       check predicate 124
  OPC_Decode, 215, 19, 248, 4,               // 78264:       decode to CPYFMTN using decoder 632
                                             // 78264:      }
                                             // 78264:      } // switch Inst[15:12]
                                             // 78264:     }
                                             // 78264:     } // switch Inst[11:10]
                                             // 78264:    }
  3, 111,                                    // 78269:    case 0x3: {
  OPC_SwitchField, 10, 6,                    // 78271:     switch Inst[15:10] {
  1, 7,                                      // 78274:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 78276:      check predicate 74
  OPC_Decode, 176, 40, 249, 4,               // 78278:      decode to LDTADDLW using decoder 633
                                             // 78278:     }
  2, 7,                                      // 78283:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 78285:      check predicate 125
  OPC_Decode, 245, 44, 240, 2,               // 78287:      decode to RCWCASL using decoder 368
                                             // 78287:     }
  3, 7,                                      // 78292:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 78294:      check predicate 126
  OPC_Decode, 249, 44, 237, 2,               // 78296:      decode to RCWCASPL using decoder 365
                                             // 78296:     }
  4, 7,                                      // 78301:     case 0x4: {
  OPC_CheckPredicate, 127,                   // 78303:      check predicate 127
  OPC_Decode, 224, 37, 250, 4,               // 78305:      decode to LDCLRPL using decoder 634
                                             // 78305:     }
  5, 7,                                      // 78310:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 78312:      check predicate 74
  OPC_Decode, 184, 40, 249, 4,               // 78314:      decode to LDTCLRLW using decoder 633
                                             // 78314:     }
  12, 7,                                     // 78319:     case 0xc: {
  OPC_CheckPredicate, 127,                   // 78321:      check predicate 127
  OPC_Decode, 137, 40, 250, 4,               // 78323:      decode to LDSETPL using decoder 634
                                             // 78323:     }
  13, 7,                                     // 78328:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 78330:      check predicate 74
  OPC_Decode, 209, 40, 249, 4,               // 78332:      decode to LDTSETLW using decoder 633
                                             // 78332:     }
  32, 7,                                     // 78337:     case 0x20: {
  OPC_CheckPredicate, 127,                   // 78339:      check predicate 127
  OPC_Decode, 154, 61, 250, 4,               // 78341:      decode to SWPPL using decoder 634
                                             // 78341:     }
  33, 7,                                     // 78346:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 78348:      check predicate 74
  OPC_Decode, 159, 61, 249, 4,               // 78350:      decode to SWPTLW using decoder 633
                                             // 78350:     }
  36, 7,                                     // 78355:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 78357:      check predicate 126
  OPC_Decode, 129, 45, 250, 4,               // 78359:      decode to RCWCLRPL using decoder 634
                                             // 78359:     }
  40, 7,                                     // 78364:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 78366:      check predicate 126
  OPC_Decode, 169, 45, 250, 4,               // 78368:      decode to RCWSWPPL using decoder 634
                                             // 78368:     }
  44, 0,                                     // 78373:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 78375:      check predicate 126
  OPC_Decode, 153, 45, 250, 4,               // 78377:      decode to RCWSETPL using decoder 634
                                             // 78377:     }
                                             // 78377:     } // switch Inst[15:10]
                                             // 78377:    }
  4, 161, 1,                                 // 78382:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 78385:     switch Inst[11:10] {
  0, 7,                                      // 78388:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 78390:      check predicate 123
  OPC_Decode, 163, 37, 247, 4,               // 78392:      decode to LDAPURSBXi using decoder 631
                                             // 78392:     }
  1, 0,                                      // 78397:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 78399:      switch Inst[15:12] {
  0, 7,                                      // 78402:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 78404:       check predicate 124
  OPC_Decode, 191, 19, 248, 4,               // 78406:       decode to CPYFE using decoder 632
                                             // 78406:      }
  1, 7,                                      // 78411:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 78413:       check predicate 124
  OPC_Decode, 203, 19, 248, 4,               // 78415:       decode to CPYFEWT using decoder 632
                                             // 78415:      }
  2, 7,                                      // 78420:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 78422:       check predicate 124
  OPC_Decode, 194, 19, 248, 4,               // 78424:       decode to CPYFERT using decoder 632
                                             // 78424:      }
  3, 7,                                      // 78429:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 78431:       check predicate 124
  OPC_Decode, 198, 19, 248, 4,               // 78433:       decode to CPYFET using decoder 632
                                             // 78433:      }
  4, 7,                                      // 78438:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 78440:       check predicate 124
  OPC_Decode, 202, 19, 248, 4,               // 78442:       decode to CPYFEWN using decoder 632
                                             // 78442:      }
  5, 7,                                      // 78447:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 78449:       check predicate 124
  OPC_Decode, 206, 19, 248, 4,               // 78451:       decode to CPYFEWTWN using decoder 632
                                             // 78451:      }
  6, 7,                                      // 78456:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 78458:       check predicate 124
  OPC_Decode, 197, 19, 248, 4,               // 78460:       decode to CPYFERTWN using decoder 632
                                             // 78460:      }
  7, 7,                                      // 78465:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 78467:       check predicate 124
  OPC_Decode, 201, 19, 248, 4,               // 78469:       decode to CPYFETWN using decoder 632
                                             // 78469:      }
  8, 7,                                      // 78474:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 78476:       check predicate 124
  OPC_Decode, 193, 19, 248, 4,               // 78478:       decode to CPYFERN using decoder 632
                                             // 78478:      }
  9, 7,                                      // 78483:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 78485:       check predicate 124
  OPC_Decode, 205, 19, 248, 4,               // 78487:       decode to CPYFEWTRN using decoder 632
                                             // 78487:      }
  10, 7,                                     // 78492:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 78494:       check predicate 124
  OPC_Decode, 196, 19, 248, 4,               // 78496:       decode to CPYFERTRN using decoder 632
                                             // 78496:      }
  11, 7,                                     // 78501:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 78503:       check predicate 124
  OPC_Decode, 200, 19, 248, 4,               // 78505:       decode to CPYFETRN using decoder 632
                                             // 78505:      }
  12, 7,                                     // 78510:      case 0xc: {
  OPC_CheckPredicate, 124,                   // 78512:       check predicate 124
  OPC_Decode, 192, 19, 248, 4,               // 78514:       decode to CPYFEN using decoder 632
                                             // 78514:      }
  13, 7,                                     // 78519:      case 0xd: {
  OPC_CheckPredicate, 124,                   // 78521:       check predicate 124
  OPC_Decode, 204, 19, 248, 4,               // 78523:       decode to CPYFEWTN using decoder 632
                                             // 78523:      }
  14, 7,                                     // 78528:      case 0xe: {
  OPC_CheckPredicate, 124,                   // 78530:       check predicate 124
  OPC_Decode, 195, 19, 248, 4,               // 78532:       decode to CPYFERTN using decoder 632
                                             // 78532:      }
  15, 0,                                     // 78537:      case 0xf: {
  OPC_CheckPredicate, 124,                   // 78539:       check predicate 124
  OPC_Decode, 199, 19, 248, 4,               // 78541:       decode to CPYFETN using decoder 632
                                             // 78541:      }
                                             // 78541:      } // switch Inst[15:12]
                                             // 78541:     }
                                             // 78541:     } // switch Inst[11:10]
                                             // 78541:    }
  5, 111,                                    // 78546:    case 0x5: {
  OPC_SwitchField, 10, 6,                    // 78548:     switch Inst[15:10] {
  1, 7,                                      // 78551:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 78553:      check predicate 74
  OPC_Decode, 174, 40, 249, 4,               // 78555:      decode to LDTADDAW using decoder 633
                                             // 78555:     }
  2, 7,                                      // 78560:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 78562:      check predicate 125
  OPC_Decode, 243, 44, 240, 2,               // 78564:      decode to RCWCASA using decoder 368
                                             // 78564:     }
  3, 7,                                      // 78569:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 78571:      check predicate 126
  OPC_Decode, 247, 44, 237, 2,               // 78573:      decode to RCWCASPA using decoder 365
                                             // 78573:     }
  4, 7,                                      // 78578:     case 0x4: {
  OPC_CheckPredicate, 127,                   // 78580:      check predicate 127
  OPC_Decode, 222, 37, 250, 4,               // 78582:      decode to LDCLRPA using decoder 634
                                             // 78582:     }
  5, 7,                                      // 78587:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 78589:      check predicate 74
  OPC_Decode, 182, 40, 249, 4,               // 78591:      decode to LDTCLRAW using decoder 633
                                             // 78591:     }
  12, 7,                                     // 78596:     case 0xc: {
  OPC_CheckPredicate, 127,                   // 78598:      check predicate 127
  OPC_Decode, 135, 40, 250, 4,               // 78600:      decode to LDSETPA using decoder 634
                                             // 78600:     }
  13, 7,                                     // 78605:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 78607:      check predicate 74
  OPC_Decode, 207, 40, 249, 4,               // 78609:      decode to LDTSETAW using decoder 633
                                             // 78609:     }
  32, 7,                                     // 78614:     case 0x20: {
  OPC_CheckPredicate, 127,                   // 78616:      check predicate 127
  OPC_Decode, 152, 61, 250, 4,               // 78618:      decode to SWPPA using decoder 634
                                             // 78618:     }
  33, 7,                                     // 78623:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 78625:      check predicate 74
  OPC_Decode, 157, 61, 249, 4,               // 78627:      decode to SWPTAW using decoder 633
                                             // 78627:     }
  36, 7,                                     // 78632:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 78634:      check predicate 126
  OPC_Decode, 255, 44, 250, 4,               // 78636:      decode to RCWCLRPA using decoder 634
                                             // 78636:     }
  40, 7,                                     // 78641:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 78643:      check predicate 126
  OPC_Decode, 167, 45, 250, 4,               // 78645:      decode to RCWSWPPA using decoder 634
                                             // 78645:     }
  44, 0,                                     // 78650:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 78652:      check predicate 126
  OPC_Decode, 151, 45, 250, 4,               // 78654:      decode to RCWSETPA using decoder 634
                                             // 78654:     }
                                             // 78654:     } // switch Inst[15:10]
                                             // 78654:    }
  6, 125,                                    // 78659:    case 0x6: {
  OPC_SwitchField, 10, 2,                    // 78661:     switch Inst[11:10] {
  0, 7,                                      // 78664:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 78666:      check predicate 123
  OPC_Decode, 162, 37, 247, 4,               // 78668:      decode to LDAPURSBWi using decoder 631
                                             // 78668:     }
  1, 0,                                      // 78673:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 78675:      switch Inst[15:12] {
  0, 7,                                      // 78678:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 78680:       check predicate 124
  OPC_Decode, 255, 47, 251, 4,               // 78682:       decode to SETP using decoder 635
                                             // 78682:      }
  1, 7,                                      // 78687:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 78689:       check predicate 124
  OPC_Decode, 129, 48, 251, 4,               // 78691:       decode to SETPT using decoder 635
                                             // 78691:      }
  2, 7,                                      // 78696:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 78698:       check predicate 124
  OPC_Decode, 128, 48, 251, 4,               // 78700:       decode to SETPN using decoder 635
                                             // 78700:      }
  3, 7,                                      // 78705:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 78707:       check predicate 124
  OPC_Decode, 130, 48, 251, 4,               // 78709:       decode to SETPTN using decoder 635
                                             // 78709:      }
  4, 7,                                      // 78714:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 78716:       check predicate 124
  OPC_Decode, 251, 47, 251, 4,               // 78718:       decode to SETM using decoder 635
                                             // 78718:      }
  5, 7,                                      // 78723:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 78725:       check predicate 124
  OPC_Decode, 253, 47, 251, 4,               // 78727:       decode to SETMT using decoder 635
                                             // 78727:      }
  6, 7,                                      // 78732:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 78734:       check predicate 124
  OPC_Decode, 252, 47, 251, 4,               // 78736:       decode to SETMN using decoder 635
                                             // 78736:      }
  7, 7,                                      // 78741:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 78743:       check predicate 124
  OPC_Decode, 254, 47, 251, 4,               // 78745:       decode to SETMTN using decoder 635
                                             // 78745:      }
  8, 7,                                      // 78750:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 78752:       check predicate 124
  OPC_Decode, 224, 47, 251, 4,               // 78754:       decode to SETE using decoder 635
                                             // 78754:      }
  9, 7,                                      // 78759:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 78761:       check predicate 124
  OPC_Decode, 226, 47, 251, 4,               // 78763:       decode to SETET using decoder 635
                                             // 78763:      }
  10, 7,                                     // 78768:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 78770:       check predicate 124
  OPC_Decode, 225, 47, 251, 4,               // 78772:       decode to SETEN using decoder 635
                                             // 78772:      }
  11, 0,                                     // 78777:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 78779:       check predicate 124
  OPC_Decode, 227, 47, 251, 4,               // 78781:       decode to SETETN using decoder 635
                                             // 78781:      }
                                             // 78781:      } // switch Inst[15:12]
                                             // 78781:     }
                                             // 78781:     } // switch Inst[11:10]
                                             // 78781:    }
  7, 0,                                      // 78786:    case 0x7: {
  OPC_SwitchField, 10, 6,                    // 78788:     switch Inst[15:10] {
  1, 7,                                      // 78791:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 78793:      check predicate 74
  OPC_Decode, 172, 40, 249, 4,               // 78795:      decode to LDTADDALW using decoder 633
                                             // 78795:     }
  2, 7,                                      // 78800:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 78802:      check predicate 125
  OPC_Decode, 244, 44, 240, 2,               // 78804:      decode to RCWCASAL using decoder 368
                                             // 78804:     }
  3, 7,                                      // 78809:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 78811:      check predicate 126
  OPC_Decode, 248, 44, 237, 2,               // 78813:      decode to RCWCASPAL using decoder 365
                                             // 78813:     }
  4, 7,                                      // 78818:     case 0x4: {
  OPC_CheckPredicate, 127,                   // 78820:      check predicate 127
  OPC_Decode, 223, 37, 250, 4,               // 78822:      decode to LDCLRPAL using decoder 634
                                             // 78822:     }
  5, 7,                                      // 78827:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 78829:      check predicate 74
  OPC_Decode, 180, 40, 249, 4,               // 78831:      decode to LDTCLRALW using decoder 633
                                             // 78831:     }
  12, 7,                                     // 78836:     case 0xc: {
  OPC_CheckPredicate, 127,                   // 78838:      check predicate 127
  OPC_Decode, 136, 40, 250, 4,               // 78840:      decode to LDSETPAL using decoder 634
                                             // 78840:     }
  13, 7,                                     // 78845:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 78847:      check predicate 74
  OPC_Decode, 205, 40, 249, 4,               // 78849:      decode to LDTSETALW using decoder 633
                                             // 78849:     }
  32, 7,                                     // 78854:     case 0x20: {
  OPC_CheckPredicate, 127,                   // 78856:      check predicate 127
  OPC_Decode, 153, 61, 250, 4,               // 78858:      decode to SWPPAL using decoder 634
                                             // 78858:     }
  33, 7,                                     // 78863:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 78865:      check predicate 74
  OPC_Decode, 155, 61, 249, 4,               // 78867:      decode to SWPTALW using decoder 633
                                             // 78867:     }
  36, 7,                                     // 78872:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 78874:      check predicate 126
  OPC_Decode, 128, 45, 250, 4,               // 78876:      decode to RCWCLRPAL using decoder 634
                                             // 78876:     }
  40, 7,                                     // 78881:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 78883:      check predicate 126
  OPC_Decode, 168, 45, 250, 4,               // 78885:      decode to RCWSWPPAL using decoder 634
                                             // 78885:     }
  44, 0,                                     // 78890:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 78892:      check predicate 126
  OPC_Decode, 152, 45, 250, 4,               // 78894:      decode to RCWSETPAL using decoder 634
                                             // 78894:     }
                                             // 78894:     } // switch Inst[15:10]
                                             // 78894:    }
                                             // 78894:    } // switch Inst[23:21]
                                             // 78894:   }
  2, 228, 1,                                 // 78899:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 78902:    switch Inst[11:10] {
  0, 62,                                     // 78905:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 78907:     switch Inst[23:21] {
  0, 9,                                      // 78910:     case 0x0: {
  OPC_CheckField, 12, 4, 0,                  // 78912:      check Inst[15:12] == 0x0
  OPC_Decode, 132, 13, 252, 4,               // 78916:      decode to ADCWr using decoder 636
                                             // 78916:     }
  4, 5,                                      // 78921:     case 0x4: {
  OPC_Decode, 167, 20, 253, 4,               // 78923:      decode to CSELWr using decoder 637
                                             // 78923:     }
  6, 0,                                      // 78928:     case 0x6: {
  OPC_SwitchField, 12, 4,                    // 78930:      switch Inst[15:12] {
  2, 5,                                      // 78933:      case 0x2: {
  OPC_Decode, 143, 41, 252, 4,               // 78935:       decode to LSLVWr using decoder 636
                                             // 78935:      }
  4, 8,                                      // 78940:      case 0x4: {
  OPC_CheckPredicate, 128, 1,                // 78942:       check predicate 128
  OPC_Decode, 159, 20, 252, 4,               // 78945:       decode to CRC32Brr using decoder 636
                                             // 78945:      }
  5, 8,                                      // 78950:      case 0x5: {
  OPC_CheckPredicate, 128, 1,                // 78952:       check predicate 128
  OPC_Decode, 160, 20, 252, 4,               // 78955:       decode to CRC32CBrr using decoder 636
                                             // 78955:      }
  6, 0,                                      // 78960:      case 0x6: {
  OPC_CheckPredicate, 103,                   // 78962:       check predicate 103
  OPC_Decode, 244, 48, 252, 4,               // 78964:       decode to SMAXWrr using decoder 636
                                             // 78964:      }
                                             // 78964:      } // switch Inst[15:12]
                                             // 78964:     }
                                             // 78964:     } // switch Inst[23:21]
                                             // 78964:    }
  1, 51,                                     // 78969:    case 0x1: {
  OPC_SwitchField, 21, 3,                    // 78971:     switch Inst[23:21] {
  4, 5,                                      // 78974:     case 0x4: {
  OPC_Decode, 169, 20, 253, 4,               // 78976:      decode to CSINCWr using decoder 637
                                             // 78976:     }
  6, 0,                                      // 78981:     case 0x6: {
  OPC_SwitchField, 12, 4,                    // 78983:      switch Inst[15:12] {
  2, 5,                                      // 78986:      case 0x2: {
  OPC_Decode, 167, 41, 252, 4,               // 78988:       decode to LSRVWr using decoder 636
                                             // 78988:      }
  4, 8,                                      // 78993:      case 0x4: {
  OPC_CheckPredicate, 128, 1,                // 78995:       check predicate 128
  OPC_Decode, 164, 20, 252, 4,               // 78998:       decode to CRC32Hrr using decoder 636
                                             // 78998:      }
  5, 8,                                      // 79003:      case 0x5: {
  OPC_CheckPredicate, 128, 1,                // 79005:       check predicate 128
  OPC_Decode, 161, 20, 252, 4,               // 79008:       decode to CRC32CHrr using decoder 636
                                             // 79008:      }
  6, 0,                                      // 79013:      case 0x6: {
  OPC_CheckPredicate, 103,                   // 79015:       check predicate 103
  OPC_Decode, 254, 63, 252, 4,               // 79017:       decode to UMAXWrr using decoder 636
                                             // 79017:      }
                                             // 79017:      } // switch Inst[15:12]
                                             // 79017:     }
                                             // 79017:     } // switch Inst[23:21]
                                             // 79017:    }
  2, 66,                                     // 79022:    case 0x2: {
  OPC_SwitchField, 12, 4,                    // 79024:     switch Inst[15:12] {
  0, 9,                                      // 79027:     case 0x0: {
  OPC_CheckField, 21, 3, 6,                  // 79029:      check Inst[23:21] == 0x6
  OPC_Decode, 171, 63, 252, 4,               // 79033:      decode to UDIVWr using decoder 636
                                             // 79033:     }
  2, 9,                                      // 79038:     case 0x2: {
  OPC_CheckField, 21, 3, 6,                  // 79040:      check Inst[23:21] == 0x6
  OPC_Decode, 182, 14, 252, 4,               // 79044:      decode to ASRVWr using decoder 636
                                             // 79044:     }
  4, 12,                                     // 79049:     case 0x4: {
  OPC_CheckPredicate, 128, 1,                // 79051:      check predicate 128
  OPC_CheckField, 21, 3, 6,                  // 79054:      check Inst[23:21] == 0x6
  OPC_Decode, 165, 20, 252, 4,               // 79058:      decode to CRC32Wrr using decoder 636
                                             // 79058:     }
  5, 12,                                     // 79063:     case 0x5: {
  OPC_CheckPredicate, 128, 1,                // 79065:      check predicate 128
  OPC_CheckField, 21, 3, 6,                  // 79068:      check Inst[23:21] == 0x6
  OPC_Decode, 162, 20, 252, 4,               // 79072:      decode to CRC32CWrr using decoder 636
                                             // 79072:     }
  6, 0,                                      // 79077:     case 0x6: {
  OPC_CheckPredicate, 103,                   // 79079:      check predicate 103
  OPC_CheckField, 21, 3, 6,                  // 79081:      check Inst[23:21] == 0x6
  OPC_Decode, 174, 49, 252, 4,               // 79085:      decode to SMINWrr using decoder 636
                                             // 79085:     }
                                             // 79085:     } // switch Inst[15:12]
                                             // 79085:    }
  3, 0,                                      // 79090:    case 0x3: {
  OPC_SwitchField, 12, 4,                    // 79092:     switch Inst[15:12] {
  0, 9,                                      // 79095:     case 0x0: {
  OPC_CheckField, 21, 3, 6,                  // 79097:      check Inst[23:21] == 0x6
  OPC_Decode, 177, 47, 252, 4,               // 79101:      decode to SDIVWr using decoder 636
                                             // 79101:     }
  2, 9,                                      // 79106:     case 0x2: {
  OPC_CheckField, 21, 3, 6,                  // 79108:      check Inst[23:21] == 0x6
  OPC_Decode, 230, 45, 252, 4,               // 79112:      decode to RORVWr using decoder 636
                                             // 79112:     }
  6, 0,                                      // 79117:     case 0x6: {
  OPC_CheckPredicate, 103,                   // 79119:      check predicate 103
  OPC_CheckField, 21, 3, 6,                  // 79121:      check Inst[23:21] == 0x6
  OPC_Decode, 183, 64, 252, 4,               // 79125:      decode to UMINWrr using decoder 636
                                             // 79125:     }
                                             // 79125:     } // switch Inst[15:12]
                                             // 79125:    }
                                             // 79125:    } // switch Inst[11:10]
                                             // 79125:   }
  3, 0,                                      // 79130:   case 0x3: {
  OPC_SwitchField, 15, 1,                    // 79132:    switch Inst[15] {
  0, 9,                                      // 79135:    case 0x0: {
  OPC_CheckField, 21, 3, 0,                  // 79137:     check Inst[23:21] == 0x0
  OPC_Decode, 230, 41, 254, 4,               // 79141:     decode to MADDWrrr using decoder 638
                                             // 79141:    }
  1, 0,                                      // 79146:    case 0x1: {
  OPC_CheckField, 21, 3, 0,                  // 79148:     check Inst[23:21] == 0x0
  OPC_Decode, 254, 42, 254, 4,               // 79152:     decode to MSUBWrrr using decoder 638
                                             // 79152:    }
                                             // 79152:    } // switch Inst[15]
                                             // 79152:   }
                                             // 79152:   } // switch Inst[25:24]
                                             // 79152:  }
  1, 227, 6,                                 // 79157:  case 0x1: {
  OPC_SwitchField, 22, 4,                    // 79160:   switch Inst[25:22] {
  0, 181, 1,                                 // 79163:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 79166:    switch Inst[11:10] {
  0, 123,                                    // 79169:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 79171:     switch Inst[21] {
  0, 5,                                      // 79174:     case 0x0: {
  OPC_Decode, 230, 59, 247, 4,               // 79176:      decode to STURBBi using decoder 631
                                             // 79176:     }
  1, 0,                                      // 79181:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 79183:      switch Inst[15:12] {
  0, 7,                                      // 79186:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 79188:       check predicate 72
  OPC_Decode, 144, 37, 249, 4,               // 79190:       decode to LDADDB using decoder 633
                                             // 79190:      }
  1, 7,                                      // 79195:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 79197:       check predicate 72
  OPC_Decode, 215, 37, 249, 4,               // 79199:       decode to LDCLRB using decoder 633
                                             // 79199:      }
  2, 7,                                      // 79204:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 79206:       check predicate 72
  OPC_Decode, 235, 37, 249, 4,               // 79208:       decode to LDEORB using decoder 633
                                             // 79208:      }
  3, 7,                                      // 79213:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 79215:       check predicate 72
  OPC_Decode, 128, 40, 249, 4,               // 79217:       decode to LDSETB using decoder 633
                                             // 79217:      }
  4, 7,                                      // 79222:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 79224:       check predicate 72
  OPC_Decode, 148, 40, 249, 4,               // 79226:       decode to LDSMAXB using decoder 633
                                             // 79226:      }
  5, 7,                                      // 79231:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 79233:       check predicate 72
  OPC_Decode, 164, 40, 249, 4,               // 79235:       decode to LDSMINB using decoder 633
                                             // 79235:      }
  6, 7,                                      // 79240:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 79242:       check predicate 72
  OPC_Decode, 223, 40, 249, 4,               // 79244:       decode to LDUMAXB using decoder 633
                                             // 79244:      }
  7, 7,                                      // 79249:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 79251:       check predicate 72
  OPC_Decode, 239, 40, 249, 4,               // 79253:       decode to LDUMINB using decoder 633
                                             // 79253:      }
  8, 7,                                      // 79258:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 79260:       check predicate 72
  OPC_Decode, 145, 61, 249, 4,               // 79262:       decode to SWPB using decoder 633
                                             // 79262:      }
  9, 7,                                      // 79267:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 79269:       check predicate 125
  OPC_Decode, 250, 44, 255, 4,               // 79271:       decode to RCWCLR using decoder 639
                                             // 79271:      }
  10, 7,                                     // 79276:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 79278:       check predicate 125
  OPC_Decode, 162, 45, 255, 4,               // 79280:       decode to RCWSWP using decoder 639
                                             // 79280:      }
  11, 0,                                     // 79285:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 79287:       check predicate 125
  OPC_Decode, 146, 45, 255, 4,               // 79289:       decode to RCWSET using decoder 639
                                             // 79289:      }
                                             // 79289:      } // switch Inst[15:12]
                                             // 79289:     }
                                             // 79289:     } // switch Inst[21]
                                             // 79289:    }
  1, 9,                                      // 79294:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 79296:     check Inst[21] == 0x0
  OPC_Decode, 166, 59, 247, 4,               // 79300:     decode to STRBBpost using decoder 631
                                             // 79300:    }
  2, 29,                                     // 79305:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 79307:     switch Inst[21] {
  0, 5,                                      // 79310:     case 0x0: {
  OPC_Decode, 224, 59, 247, 4,               // 79312:      decode to STTRBi using decoder 631
                                             // 79312:     }
  1, 0,                                      // 79317:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 79319:      switch Inst[14:13] {
  2, 5,                                      // 79322:      case 0x2: {
  OPC_Decode, 168, 59, 128, 5,               // 79324:       decode to STRBBroW using decoder 640
                                             // 79324:      }
  3, 0,                                      // 79329:      case 0x3: {
  OPC_Decode, 169, 59, 129, 5,               // 79331:       decode to STRBBroX using decoder 641
                                             // 79331:      }
                                             // 79331:      } // switch Inst[14:13]
                                             // 79331:     }
                                             // 79331:     } // switch Inst[21]
                                             // 79331:    }
  3, 0,                                      // 79336:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 79338:     check Inst[21] == 0x0
  OPC_Decode, 167, 59, 247, 4,               // 79342:     decode to STRBBpre using decoder 631
                                             // 79342:    }
                                             // 79342:    } // switch Inst[11:10]
                                             // 79342:   }
  1, 181, 1,                                 // 79347:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 79350:    switch Inst[11:10] {
  0, 123,                                    // 79353:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 79355:     switch Inst[21] {
  0, 5,                                      // 79358:     case 0x0: {
  OPC_Decode, 247, 40, 247, 4,               // 79360:      decode to LDURBBi using decoder 631
                                             // 79360:     }
  1, 0,                                      // 79365:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 79367:      switch Inst[15:12] {
  0, 7,                                      // 79370:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 79372:       check predicate 72
  OPC_Decode, 146, 37, 249, 4,               // 79374:       decode to LDADDLB using decoder 633
                                             // 79374:      }
  1, 7,                                      // 79379:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 79381:       check predicate 72
  OPC_Decode, 217, 37, 249, 4,               // 79383:       decode to LDCLRLB using decoder 633
                                             // 79383:      }
  2, 7,                                      // 79388:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 79390:       check predicate 72
  OPC_Decode, 237, 37, 249, 4,               // 79392:       decode to LDEORLB using decoder 633
                                             // 79392:      }
  3, 7,                                      // 79397:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 79399:       check predicate 72
  OPC_Decode, 130, 40, 249, 4,               // 79401:       decode to LDSETLB using decoder 633
                                             // 79401:      }
  4, 7,                                      // 79406:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 79408:       check predicate 72
  OPC_Decode, 150, 40, 249, 4,               // 79410:       decode to LDSMAXLB using decoder 633
                                             // 79410:      }
  5, 7,                                      // 79415:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 79417:       check predicate 72
  OPC_Decode, 166, 40, 249, 4,               // 79419:       decode to LDSMINLB using decoder 633
                                             // 79419:      }
  6, 7,                                      // 79424:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 79426:       check predicate 72
  OPC_Decode, 225, 40, 249, 4,               // 79428:       decode to LDUMAXLB using decoder 633
                                             // 79428:      }
  7, 7,                                      // 79433:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 79435:       check predicate 72
  OPC_Decode, 241, 40, 249, 4,               // 79437:       decode to LDUMINLB using decoder 633
                                             // 79437:      }
  8, 7,                                      // 79442:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 79444:       check predicate 72
  OPC_Decode, 147, 61, 249, 4,               // 79446:       decode to SWPLB using decoder 633
                                             // 79446:      }
  9, 7,                                      // 79451:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 79453:       check predicate 125
  OPC_Decode, 253, 44, 255, 4,               // 79455:       decode to RCWCLRL using decoder 639
                                             // 79455:      }
  10, 7,                                     // 79460:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 79462:       check predicate 125
  OPC_Decode, 165, 45, 255, 4,               // 79464:       decode to RCWSWPL using decoder 639
                                             // 79464:      }
  11, 0,                                     // 79469:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 79471:       check predicate 125
  OPC_Decode, 149, 45, 255, 4,               // 79473:       decode to RCWSETL using decoder 639
                                             // 79473:      }
                                             // 79473:      } // switch Inst[15:12]
                                             // 79473:     }
                                             // 79473:     } // switch Inst[21]
                                             // 79473:    }
  1, 9,                                      // 79478:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 79480:     check Inst[21] == 0x0
  OPC_Decode, 168, 39, 247, 4,               // 79484:     decode to LDRBBpost using decoder 631
                                             // 79484:    }
  2, 29,                                     // 79489:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 79491:     switch Inst[21] {
  0, 5,                                      // 79494:     case 0x0: {
  OPC_Decode, 196, 40, 247, 4,               // 79496:      decode to LDTRBi using decoder 631
                                             // 79496:     }
  1, 0,                                      // 79501:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 79503:      switch Inst[14:13] {
  2, 5,                                      // 79506:      case 0x2: {
  OPC_Decode, 170, 39, 128, 5,               // 79508:       decode to LDRBBroW using decoder 640
                                             // 79508:      }
  3, 0,                                      // 79513:      case 0x3: {
  OPC_Decode, 171, 39, 129, 5,               // 79515:       decode to LDRBBroX using decoder 641
                                             // 79515:      }
                                             // 79515:      } // switch Inst[14:13]
                                             // 79515:     }
                                             // 79515:     } // switch Inst[21]
                                             // 79515:    }
  3, 0,                                      // 79520:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 79522:     check Inst[21] == 0x0
  OPC_Decode, 169, 39, 247, 4,               // 79526:     decode to LDRBBpre using decoder 631
                                             // 79526:    }
                                             // 79526:    } // switch Inst[11:10]
                                             // 79526:   }
  2, 196, 1,                                 // 79531:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 79534:    switch Inst[11:10] {
  0, 137, 1,                                 // 79537:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 79540:     switch Inst[21] {
  0, 5,                                      // 79543:     case 0x0: {
  OPC_Decode, 254, 40, 247, 4,               // 79545:      decode to LDURSBXi using decoder 631
                                             // 79545:     }
  1, 0,                                      // 79550:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 79552:      switch Inst[15:12] {
  0, 7,                                      // 79555:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 79557:       check predicate 72
  OPC_Decode, 136, 37, 249, 4,               // 79559:       decode to LDADDAB using decoder 633
                                             // 79559:      }
  1, 7,                                      // 79564:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 79566:       check predicate 72
  OPC_Decode, 207, 37, 249, 4,               // 79568:       decode to LDCLRAB using decoder 633
                                             // 79568:      }
  2, 7,                                      // 79573:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 79575:       check predicate 72
  OPC_Decode, 227, 37, 249, 4,               // 79577:       decode to LDEORAB using decoder 633
                                             // 79577:      }
  3, 7,                                      // 79582:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 79584:       check predicate 72
  OPC_Decode, 248, 39, 249, 4,               // 79586:       decode to LDSETAB using decoder 633
                                             // 79586:      }
  4, 7,                                      // 79591:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 79593:       check predicate 72
  OPC_Decode, 140, 40, 249, 4,               // 79595:       decode to LDSMAXAB using decoder 633
                                             // 79595:      }
  5, 7,                                      // 79600:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 79602:       check predicate 72
  OPC_Decode, 156, 40, 249, 4,               // 79604:       decode to LDSMINAB using decoder 633
                                             // 79604:      }
  6, 7,                                      // 79609:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 79611:       check predicate 72
  OPC_Decode, 215, 40, 249, 4,               // 79613:       decode to LDUMAXAB using decoder 633
                                             // 79613:      }
  7, 7,                                      // 79618:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 79620:       check predicate 72
  OPC_Decode, 231, 40, 249, 4,               // 79622:       decode to LDUMINAB using decoder 633
                                             // 79622:      }
  8, 7,                                      // 79627:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 79629:       check predicate 72
  OPC_Decode, 137, 61, 249, 4,               // 79631:       decode to SWPAB using decoder 633
                                             // 79631:      }
  9, 7,                                      // 79636:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 79638:       check predicate 125
  OPC_Decode, 251, 44, 255, 4,               // 79640:       decode to RCWCLRA using decoder 639
                                             // 79640:      }
  10, 7,                                     // 79645:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 79647:       check predicate 125
  OPC_Decode, 163, 45, 255, 4,               // 79649:       decode to RCWSWPA using decoder 639
                                             // 79649:      }
  11, 7,                                     // 79654:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 79656:       check predicate 125
  OPC_Decode, 147, 45, 255, 4,               // 79658:       decode to RCWSETA using decoder 639
                                             // 79658:      }
  12, 0,                                     // 79663:      case 0xc: {
  OPC_CheckPredicate, 129, 1,                // 79665:       check predicate 129
  OPC_CheckField, 16, 5, 31,                 // 79668:       check Inst[20:16] == 0x1f
  OPC_Decode, 154, 37, 239, 2,               // 79672:       decode to LDAPRB using decoder 367
                                             // 79672:      }
                                             // 79672:      } // switch Inst[15:12]
                                             // 79672:     }
                                             // 79672:     } // switch Inst[21]
                                             // 79672:    }
  1, 9,                                      // 79677:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 79679:     check Inst[21] == 0x0
  OPC_Decode, 205, 39, 247, 4,               // 79683:     decode to LDRSBXpost using decoder 631
                                             // 79683:    }
  2, 29,                                     // 79688:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 79690:     switch Inst[21] {
  0, 5,                                      // 79693:     case 0x0: {
  OPC_Decode, 199, 40, 247, 4,               // 79695:      decode to LDTRSBXi using decoder 631
                                             // 79695:     }
  1, 0,                                      // 79700:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 79702:      switch Inst[14:13] {
  2, 5,                                      // 79705:      case 0x2: {
  OPC_Decode, 207, 39, 130, 5,               // 79707:       decode to LDRSBXroW using decoder 642
                                             // 79707:      }
  3, 0,                                      // 79712:      case 0x3: {
  OPC_Decode, 208, 39, 131, 5,               // 79714:       decode to LDRSBXroX using decoder 643
                                             // 79714:      }
                                             // 79714:      } // switch Inst[14:13]
                                             // 79714:     }
                                             // 79714:     } // switch Inst[21]
                                             // 79714:    }
  3, 0,                                      // 79719:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 79721:     check Inst[21] == 0x0
  OPC_Decode, 206, 39, 247, 4,               // 79725:     decode to LDRSBXpre using decoder 631
                                             // 79725:    }
                                             // 79725:    } // switch Inst[11:10]
                                             // 79725:   }
  3, 181, 1,                                 // 79730:   case 0x3: {
  OPC_SwitchField, 10, 2,                    // 79733:    switch Inst[11:10] {
  0, 123,                                    // 79736:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 79738:     switch Inst[21] {
  0, 5,                                      // 79741:     case 0x0: {
  OPC_Decode, 253, 40, 247, 4,               // 79743:      decode to LDURSBWi using decoder 631
                                             // 79743:     }
  1, 0,                                      // 79748:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 79750:      switch Inst[15:12] {
  0, 7,                                      // 79753:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 79755:       check predicate 72
  OPC_Decode, 138, 37, 249, 4,               // 79757:       decode to LDADDALB using decoder 633
                                             // 79757:      }
  1, 7,                                      // 79762:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 79764:       check predicate 72
  OPC_Decode, 209, 37, 249, 4,               // 79766:       decode to LDCLRALB using decoder 633
                                             // 79766:      }
  2, 7,                                      // 79771:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 79773:       check predicate 72
  OPC_Decode, 229, 37, 249, 4,               // 79775:       decode to LDEORALB using decoder 633
                                             // 79775:      }
  3, 7,                                      // 79780:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 79782:       check predicate 72
  OPC_Decode, 250, 39, 249, 4,               // 79784:       decode to LDSETALB using decoder 633
                                             // 79784:      }
  4, 7,                                      // 79789:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 79791:       check predicate 72
  OPC_Decode, 142, 40, 249, 4,               // 79793:       decode to LDSMAXALB using decoder 633
                                             // 79793:      }
  5, 7,                                      // 79798:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 79800:       check predicate 72
  OPC_Decode, 158, 40, 249, 4,               // 79802:       decode to LDSMINALB using decoder 633
                                             // 79802:      }
  6, 7,                                      // 79807:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 79809:       check predicate 72
  OPC_Decode, 217, 40, 249, 4,               // 79811:       decode to LDUMAXALB using decoder 633
                                             // 79811:      }
  7, 7,                                      // 79816:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 79818:       check predicate 72
  OPC_Decode, 233, 40, 249, 4,               // 79820:       decode to LDUMINALB using decoder 633
                                             // 79820:      }
  8, 7,                                      // 79825:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 79827:       check predicate 72
  OPC_Decode, 139, 61, 249, 4,               // 79829:       decode to SWPALB using decoder 633
                                             // 79829:      }
  9, 7,                                      // 79834:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 79836:       check predicate 125
  OPC_Decode, 252, 44, 255, 4,               // 79838:       decode to RCWCLRAL using decoder 639
                                             // 79838:      }
  10, 7,                                     // 79843:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 79845:       check predicate 125
  OPC_Decode, 164, 45, 255, 4,               // 79847:       decode to RCWSWPAL using decoder 639
                                             // 79847:      }
  11, 0,                                     // 79852:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 79854:       check predicate 125
  OPC_Decode, 148, 45, 255, 4,               // 79856:       decode to RCWSETAL using decoder 639
                                             // 79856:      }
                                             // 79856:      } // switch Inst[15:12]
                                             // 79856:     }
                                             // 79856:     } // switch Inst[21]
                                             // 79856:    }
  1, 9,                                      // 79861:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 79863:     check Inst[21] == 0x0
  OPC_Decode, 200, 39, 247, 4,               // 79867:     decode to LDRSBWpost using decoder 631
                                             // 79867:    }
  2, 29,                                     // 79872:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 79874:     switch Inst[21] {
  0, 5,                                      // 79877:     case 0x0: {
  OPC_Decode, 198, 40, 247, 4,               // 79879:      decode to LDTRSBWi using decoder 631
                                             // 79879:     }
  1, 0,                                      // 79884:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 79886:      switch Inst[14:13] {
  2, 5,                                      // 79889:      case 0x2: {
  OPC_Decode, 202, 39, 128, 5,               // 79891:       decode to LDRSBWroW using decoder 640
                                             // 79891:      }
  3, 0,                                      // 79896:      case 0x3: {
  OPC_Decode, 203, 39, 129, 5,               // 79898:       decode to LDRSBWroX using decoder 641
                                             // 79898:      }
                                             // 79898:      } // switch Inst[14:13]
                                             // 79898:     }
                                             // 79898:     } // switch Inst[21]
                                             // 79898:    }
  3, 0,                                      // 79903:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 79905:     check Inst[21] == 0x0
  OPC_Decode, 201, 39, 247, 4,               // 79909:     decode to LDRSBWpre using decoder 631
                                             // 79909:    }
                                             // 79909:    } // switch Inst[11:10]
                                             // 79909:   }
  4, 5,                                      // 79914:   case 0x4: {
  OPC_Decode, 170, 59, 132, 5,               // 79916:    decode to STRBBui using decoder 644
                                             // 79916:   }
  5, 5,                                      // 79921:   case 0x5: {
  OPC_Decode, 172, 39, 132, 5,               // 79923:    decode to LDRBBui using decoder 644
                                             // 79923:   }
  6, 5,                                      // 79928:   case 0x6: {
  OPC_Decode, 209, 39, 132, 5,               // 79930:    decode to LDRSBXui using decoder 644
                                             // 79930:   }
  7, 5,                                      // 79935:   case 0x7: {
  OPC_Decode, 204, 39, 132, 5,               // 79937:    decode to LDRSBWui using decoder 644
                                             // 79937:   }
  8, 48,                                     // 79942:   case 0x8: {
  OPC_SwitchField, 10, 6,                    // 79944:    switch Inst[15:10] {
  0, 9,                                      // 79947:    case 0x0: {
  OPC_CheckField, 21, 1, 0,                  // 79949:     check Inst[21] == 0x0
  OPC_Decode, 130, 13, 252, 4,               // 79953:     decode to ADCSWr using decoder 636
                                             // 79953:    }
  2, 15,                                     // 79958:    case 0x2: {
  OPC_CheckPredicate, 115,                   // 79960:     check predicate 115
  OPC_CheckField, 16, 6, 0,                  // 79962:     check Inst[21:16] == 0x0
  OPC_CheckField, 0, 5, 13,                  // 79966:     check Inst[4:0] == 0xd
  OPC_Decode, 229, 47, 133, 5,               // 79970:     decode to SETF8 using decoder 645
                                             // 79970:    }
  18, 0,                                     // 79975:    case 0x12: {
  OPC_CheckPredicate, 115,                   // 79977:     check predicate 115
  OPC_CheckField, 16, 6, 0,                  // 79979:     check Inst[21:16] == 0x0
  OPC_CheckField, 0, 5, 13,                  // 79983:     check Inst[4:0] == 0xd
  OPC_Decode, 228, 47, 133, 5,               // 79987:     decode to SETF16 using decoder 645
                                             // 79987:    }
                                             // 79987:    } // switch Inst[15:10]
                                             // 79987:   }
  9, 0,                                      // 79992:   case 0x9: {
  OPC_SwitchField, 10, 2,                    // 79994:    switch Inst[11:10] {
  0, 13,                                     // 79997:    case 0x0: {
  OPC_CheckField, 21, 1, 0,                  // 79999:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 80003:     check Inst[4] == 0x0
  OPC_Decode, 137, 17, 134, 5,               // 80007:     decode to CCMNWr using decoder 646
                                             // 80007:    }
  2, 0,                                      // 80012:    case 0x2: {
  OPC_CheckField, 21, 1, 0,                  // 80014:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 80018:     check Inst[4] == 0x0
  OPC_Decode, 136, 17, 135, 5,               // 80022:     decode to CCMNWi using decoder 647
                                             // 80022:    }
                                             // 80022:    } // switch Inst[11:10]
                                             // 80022:   }
                                             // 80022:   } // switch Inst[25:22]
                                             // 80022:  }
  2, 152, 4,                                 // 80027:  case 0x2: {
  OPC_SwitchField, 24, 2,                    // 80030:   switch Inst[25:24] {
  0, 5,                                      // 80033:   case 0x0: {
  OPC_Decode, 238, 39, 220, 4,               // 80035:    decode to LDRXl using decoder 604
                                             // 80035:   }
  1, 143, 3,                                 // 80040:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 80043:    switch Inst[23:21] {
  0, 11,                                     // 80046:    case 0x0: {
  OPC_CheckPredicate, 123,                   // 80048:     check predicate 123
  OPC_CheckField, 10, 2, 0,                  // 80050:     check Inst[11:10] == 0x0
  OPC_Decode, 211, 58, 247, 4,               // 80054:     decode to STLURHi using decoder 631
                                             // 80054:    }
  1, 84,                                     // 80059:    case 0x1: {
  OPC_SwitchField, 10, 6,                    // 80061:     switch Inst[15:10] {
  1, 7,                                      // 80064:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 80066:      check predicate 74
  OPC_Decode, 179, 40, 255, 4,               // 80068:      decode to LDTADDX using decoder 639
                                             // 80068:     }
  2, 7,                                      // 80073:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 80075:      check predicate 125
  OPC_Decode, 138, 45, 240, 2,               // 80077:      decode to RCWSCAS using decoder 368
                                             // 80077:     }
  3, 7,                                      // 80082:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 80084:      check predicate 126
  OPC_Decode, 142, 45, 237, 2,               // 80086:      decode to RCWSCASP using decoder 365
                                             // 80086:     }
  5, 7,                                      // 80091:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 80093:      check predicate 74
  OPC_Decode, 187, 40, 255, 4,               // 80095:      decode to LDTCLRX using decoder 639
                                             // 80095:     }
  13, 7,                                     // 80100:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 80102:      check predicate 74
  OPC_Decode, 212, 40, 255, 4,               // 80104:      decode to LDTSETX using decoder 639
                                             // 80104:     }
  33, 7,                                     // 80109:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 80111:      check predicate 74
  OPC_Decode, 162, 61, 255, 4,               // 80113:      decode to SWPTX using decoder 639
                                             // 80113:     }
  36, 7,                                     // 80118:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 80120:      check predicate 126
  OPC_Decode, 134, 45, 250, 4,               // 80122:      decode to RCWCLRSP using decoder 634
                                             // 80122:     }
  40, 7,                                     // 80127:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 80129:      check predicate 126
  OPC_Decode, 174, 45, 250, 4,               // 80131:      decode to RCWSWPSP using decoder 634
                                             // 80131:     }
  44, 0,                                     // 80136:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 80138:      check predicate 126
  OPC_Decode, 158, 45, 250, 4,               // 80140:      decode to RCWSETSP using decoder 634
                                             // 80140:     }
                                             // 80140:     } // switch Inst[15:10]
                                             // 80140:    }
  2, 11,                                     // 80145:    case 0x2: {
  OPC_CheckPredicate, 123,                   // 80147:     check predicate 123
  OPC_CheckField, 10, 2, 0,                  // 80149:     check Inst[11:10] == 0x0
  OPC_Decode, 161, 37, 247, 4,               // 80153:     decode to LDAPURHi using decoder 631
                                             // 80153:    }
  3, 84,                                     // 80158:    case 0x3: {
  OPC_SwitchField, 10, 6,                    // 80160:     switch Inst[15:10] {
  1, 7,                                      // 80163:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 80165:      check predicate 74
  OPC_Decode, 177, 40, 255, 4,               // 80167:      decode to LDTADDLX using decoder 639
                                             // 80167:     }
  2, 7,                                      // 80172:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 80174:      check predicate 125
  OPC_Decode, 141, 45, 240, 2,               // 80176:      decode to RCWSCASL using decoder 368
                                             // 80176:     }
  3, 7,                                      // 80181:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 80183:      check predicate 126
  OPC_Decode, 145, 45, 237, 2,               // 80185:      decode to RCWSCASPL using decoder 365
                                             // 80185:     }
  5, 7,                                      // 80190:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 80192:      check predicate 74
  OPC_Decode, 185, 40, 255, 4,               // 80194:      decode to LDTCLRLX using decoder 639
                                             // 80194:     }
  13, 7,                                     // 80199:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 80201:      check predicate 74
  OPC_Decode, 210, 40, 255, 4,               // 80203:      decode to LDTSETLX using decoder 639
                                             // 80203:     }
  33, 7,                                     // 80208:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 80210:      check predicate 74
  OPC_Decode, 160, 61, 255, 4,               // 80212:      decode to SWPTLX using decoder 639
                                             // 80212:     }
  36, 7,                                     // 80217:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 80219:      check predicate 126
  OPC_Decode, 137, 45, 250, 4,               // 80221:      decode to RCWCLRSPL using decoder 634
                                             // 80221:     }
  40, 7,                                     // 80226:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 80228:      check predicate 126
  OPC_Decode, 177, 45, 250, 4,               // 80230:      decode to RCWSWPSPL using decoder 634
                                             // 80230:     }
  44, 0,                                     // 80235:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 80237:      check predicate 126
  OPC_Decode, 161, 45, 250, 4,               // 80239:      decode to RCWSETSPL using decoder 634
                                             // 80239:     }
                                             // 80239:     } // switch Inst[15:10]
                                             // 80239:    }
  4, 11,                                     // 80244:    case 0x4: {
  OPC_CheckPredicate, 123,                   // 80246:     check predicate 123
  OPC_CheckField, 10, 2, 0,                  // 80248:     check Inst[11:10] == 0x0
  OPC_Decode, 165, 37, 247, 4,               // 80252:     decode to LDAPURSHXi using decoder 631
                                             // 80252:    }
  5, 84,                                     // 80257:    case 0x5: {
  OPC_SwitchField, 10, 6,                    // 80259:     switch Inst[15:10] {
  1, 7,                                      // 80262:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 80264:      check predicate 74
  OPC_Decode, 175, 40, 255, 4,               // 80266:      decode to LDTADDAX using decoder 639
                                             // 80266:     }
  2, 7,                                      // 80271:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 80273:      check predicate 125
  OPC_Decode, 139, 45, 240, 2,               // 80275:      decode to RCWSCASA using decoder 368
                                             // 80275:     }
  3, 7,                                      // 80280:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 80282:      check predicate 126
  OPC_Decode, 143, 45, 237, 2,               // 80284:      decode to RCWSCASPA using decoder 365
                                             // 80284:     }
  5, 7,                                      // 80289:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 80291:      check predicate 74
  OPC_Decode, 183, 40, 255, 4,               // 80293:      decode to LDTCLRAX using decoder 639
                                             // 80293:     }
  13, 7,                                     // 80298:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 80300:      check predicate 74
  OPC_Decode, 208, 40, 255, 4,               // 80302:      decode to LDTSETAX using decoder 639
                                             // 80302:     }
  33, 7,                                     // 80307:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 80309:      check predicate 74
  OPC_Decode, 158, 61, 255, 4,               // 80311:      decode to SWPTAX using decoder 639
                                             // 80311:     }
  36, 7,                                     // 80316:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 80318:      check predicate 126
  OPC_Decode, 135, 45, 250, 4,               // 80320:      decode to RCWCLRSPA using decoder 634
                                             // 80320:     }
  40, 7,                                     // 80325:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 80327:      check predicate 126
  OPC_Decode, 175, 45, 250, 4,               // 80329:      decode to RCWSWPSPA using decoder 634
                                             // 80329:     }
  44, 0,                                     // 80334:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 80336:      check predicate 126
  OPC_Decode, 159, 45, 250, 4,               // 80338:      decode to RCWSETSPA using decoder 634
                                             // 80338:     }
                                             // 80338:     } // switch Inst[15:10]
                                             // 80338:    }
  6, 11,                                     // 80343:    case 0x6: {
  OPC_CheckPredicate, 123,                   // 80345:     check predicate 123
  OPC_CheckField, 10, 2, 0,                  // 80347:     check Inst[11:10] == 0x0
  OPC_Decode, 164, 37, 247, 4,               // 80351:     decode to LDAPURSHWi using decoder 631
                                             // 80351:    }
  7, 0,                                      // 80356:    case 0x7: {
  OPC_SwitchField, 10, 6,                    // 80358:     switch Inst[15:10] {
  1, 7,                                      // 80361:     case 0x1: {
  OPC_CheckPredicate, 74,                    // 80363:      check predicate 74
  OPC_Decode, 173, 40, 255, 4,               // 80365:      decode to LDTADDALX using decoder 639
                                             // 80365:     }
  2, 7,                                      // 80370:     case 0x2: {
  OPC_CheckPredicate, 125,                   // 80372:      check predicate 125
  OPC_Decode, 140, 45, 240, 2,               // 80374:      decode to RCWSCASAL using decoder 368
                                             // 80374:     }
  3, 7,                                      // 80379:     case 0x3: {
  OPC_CheckPredicate, 126,                   // 80381:      check predicate 126
  OPC_Decode, 144, 45, 237, 2,               // 80383:      decode to RCWSCASPAL using decoder 365
                                             // 80383:     }
  5, 7,                                      // 80388:     case 0x5: {
  OPC_CheckPredicate, 74,                    // 80390:      check predicate 74
  OPC_Decode, 181, 40, 255, 4,               // 80392:      decode to LDTCLRALX using decoder 639
                                             // 80392:     }
  13, 7,                                     // 80397:     case 0xd: {
  OPC_CheckPredicate, 74,                    // 80399:      check predicate 74
  OPC_Decode, 206, 40, 255, 4,               // 80401:      decode to LDTSETALX using decoder 639
                                             // 80401:     }
  33, 7,                                     // 80406:     case 0x21: {
  OPC_CheckPredicate, 74,                    // 80408:      check predicate 74
  OPC_Decode, 156, 61, 255, 4,               // 80410:      decode to SWPTALX using decoder 639
                                             // 80410:     }
  36, 7,                                     // 80415:     case 0x24: {
  OPC_CheckPredicate, 126,                   // 80417:      check predicate 126
  OPC_Decode, 136, 45, 250, 4,               // 80419:      decode to RCWCLRSPAL using decoder 634
                                             // 80419:     }
  40, 7,                                     // 80424:     case 0x28: {
  OPC_CheckPredicate, 126,                   // 80426:      check predicate 126
  OPC_Decode, 176, 45, 250, 4,               // 80428:      decode to RCWSWPSPAL using decoder 634
                                             // 80428:     }
  44, 0,                                     // 80433:     case 0x2c: {
  OPC_CheckPredicate, 126,                   // 80435:      check predicate 126
  OPC_Decode, 160, 45, 250, 4,               // 80437:      decode to RCWSETSPAL using decoder 634
                                             // 80437:     }
                                             // 80437:     } // switch Inst[15:10]
                                             // 80437:    }
                                             // 80437:    } // switch Inst[23:21]
                                             // 80437:   }
  2, 0,                                      // 80442:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 80444:    switch Inst[11:10] {
  0, 49,                                     // 80447:    case 0x0: {
  OPC_SwitchField, 21, 3,                    // 80449:     switch Inst[23:21] {
  0, 9,                                      // 80452:     case 0x0: {
  OPC_CheckField, 12, 4, 0,                  // 80454:      check Inst[15:12] == 0x0
  OPC_Decode, 233, 46, 252, 4,               // 80458:      decode to SBCWr using decoder 636
                                             // 80458:     }
  4, 5,                                      // 80463:     case 0x4: {
  OPC_Decode, 171, 20, 253, 4,               // 80465:      decode to CSINVWr using decoder 637
                                             // 80465:     }
  6, 0,                                      // 80470:     case 0x6: {
  OPC_SwitchField, 12, 9,                    // 80472:      switch Inst[20:12] {
  0, 5,                                      // 80475:      case 0x0: {
  OPC_Decode, 230, 44, 136, 5,               // 80477:       decode to RBITWr using decoder 648
                                             // 80477:      }
  1, 5,                                      // 80482:      case 0x1: {
  OPC_Decode, 191, 17, 136, 5,               // 80484:       decode to CLZWr using decoder 648
                                             // 80484:      }
  2, 0,                                      // 80489:      case 0x2: {
  OPC_CheckPredicate, 103,                   // 80491:       check predicate 103
  OPC_Decode, 236, 12, 136, 5,               // 80493:       decode to ABSWr using decoder 648
                                             // 80493:      }
                                             // 80493:      } // switch Inst[20:12]
                                             // 80493:     }
                                             // 80493:     } // switch Inst[23:21]
                                             // 80493:    }
  1, 29,                                     // 80498:    case 0x1: {
  OPC_SwitchField, 21, 3,                    // 80500:     switch Inst[23:21] {
  4, 5,                                      // 80503:     case 0x4: {
  OPC_Decode, 173, 20, 253, 4,               // 80505:      decode to CSNEGWr using decoder 637
                                             // 80505:     }
  6, 0,                                      // 80510:     case 0x6: {
  OPC_SwitchField, 12, 9,                    // 80512:      switch Inst[20:12] {
  0, 5,                                      // 80515:      case 0x0: {
  OPC_Decode, 190, 45, 136, 5,               // 80517:       decode to REV16Wr using decoder 648
                                             // 80517:      }
  1, 0,                                      // 80522:      case 0x1: {
  OPC_Decode, 175, 17, 136, 5,               // 80524:       decode to CLSWr using decoder 648
                                             // 80524:      }
                                             // 80524:      } // switch Inst[20:12]
                                             // 80524:     }
                                             // 80524:     } // switch Inst[23:21]
                                             // 80524:    }
  2, 21,                                     // 80529:    case 0x2: {
  OPC_SwitchField, 12, 12,                   // 80531:     switch Inst[23:12] {
  128, 24, 5,                                // 80534:     case 0xc00: {
  OPC_Decode, 219, 45, 136, 5,               // 80537:      decode to REVWr using decoder 648
                                             // 80537:     }
  129, 24, 0,                                // 80542:     case 0xc01: {
  OPC_CheckPredicate, 103,                   // 80545:      check predicate 103
  OPC_Decode, 179, 20, 136, 5,               // 80547:      decode to CTZWr using decoder 648
                                             // 80547:     }
                                             // 80547:     } // switch Inst[23:12]
                                             // 80547:    }
  3, 0,                                      // 80552:    case 0x3: {
  OPC_CheckPredicate, 103,                   // 80554:     check predicate 103
  OPC_CheckField, 12, 12, 129, 24,           // 80556:     check Inst[23:12] == 0xc01
  OPC_Decode, 159, 19, 136, 5,               // 80561:     decode to CNTWr using decoder 648
                                             // 80561:    }
                                             // 80561:    } // switch Inst[11:10]
                                             // 80561:   }
                                             // 80561:   } // switch Inst[25:24]
                                             // 80561:  }
  3, 192, 6,                                 // 80566:  case 0x3: {
  OPC_SwitchField, 22, 4,                    // 80569:   switch Inst[25:22] {
  0, 181, 1,                                 // 80572:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 80575:    switch Inst[11:10] {
  0, 123,                                    // 80578:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 80580:     switch Inst[21] {
  0, 5,                                      // 80583:     case 0x0: {
  OPC_Decode, 233, 59, 247, 4,               // 80585:      decode to STURHHi using decoder 631
                                             // 80585:     }
  1, 0,                                      // 80590:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 80592:      switch Inst[15:12] {
  0, 7,                                      // 80595:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 80597:       check predicate 72
  OPC_Decode, 145, 37, 249, 4,               // 80599:       decode to LDADDH using decoder 633
                                             // 80599:      }
  1, 7,                                      // 80604:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 80606:       check predicate 72
  OPC_Decode, 216, 37, 249, 4,               // 80608:       decode to LDCLRH using decoder 633
                                             // 80608:      }
  2, 7,                                      // 80613:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 80615:       check predicate 72
  OPC_Decode, 236, 37, 249, 4,               // 80617:       decode to LDEORH using decoder 633
                                             // 80617:      }
  3, 7,                                      // 80622:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 80624:       check predicate 72
  OPC_Decode, 129, 40, 249, 4,               // 80626:       decode to LDSETH using decoder 633
                                             // 80626:      }
  4, 7,                                      // 80631:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 80633:       check predicate 72
  OPC_Decode, 149, 40, 249, 4,               // 80635:       decode to LDSMAXH using decoder 633
                                             // 80635:      }
  5, 7,                                      // 80640:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 80642:       check predicate 72
  OPC_Decode, 165, 40, 249, 4,               // 80644:       decode to LDSMINH using decoder 633
                                             // 80644:      }
  6, 7,                                      // 80649:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 80651:       check predicate 72
  OPC_Decode, 224, 40, 249, 4,               // 80653:       decode to LDUMAXH using decoder 633
                                             // 80653:      }
  7, 7,                                      // 80658:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 80660:       check predicate 72
  OPC_Decode, 240, 40, 249, 4,               // 80662:       decode to LDUMINH using decoder 633
                                             // 80662:      }
  8, 7,                                      // 80667:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 80669:       check predicate 72
  OPC_Decode, 146, 61, 249, 4,               // 80671:       decode to SWPH using decoder 633
                                             // 80671:      }
  9, 7,                                      // 80676:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 80678:       check predicate 125
  OPC_Decode, 130, 45, 255, 4,               // 80680:       decode to RCWCLRS using decoder 639
                                             // 80680:      }
  10, 7,                                     // 80685:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 80687:       check predicate 125
  OPC_Decode, 170, 45, 255, 4,               // 80689:       decode to RCWSWPS using decoder 639
                                             // 80689:      }
  11, 0,                                     // 80694:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 80696:       check predicate 125
  OPC_Decode, 154, 45, 255, 4,               // 80698:       decode to RCWSETS using decoder 639
                                             // 80698:      }
                                             // 80698:      } // switch Inst[15:12]
                                             // 80698:     }
                                             // 80698:     } // switch Inst[21]
                                             // 80698:    }
  1, 9,                                      // 80703:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 80705:     check Inst[21] == 0x0
  OPC_Decode, 181, 59, 247, 4,               // 80709:     decode to STRHHpost using decoder 631
                                             // 80709:    }
  2, 29,                                     // 80714:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 80716:     switch Inst[21] {
  0, 5,                                      // 80719:     case 0x0: {
  OPC_Decode, 225, 59, 247, 4,               // 80721:      decode to STTRHi using decoder 631
                                             // 80721:     }
  1, 0,                                      // 80726:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 80728:      switch Inst[14:13] {
  2, 5,                                      // 80731:      case 0x2: {
  OPC_Decode, 183, 59, 128, 5,               // 80733:       decode to STRHHroW using decoder 640
                                             // 80733:      }
  3, 0,                                      // 80738:      case 0x3: {
  OPC_Decode, 184, 59, 129, 5,               // 80740:       decode to STRHHroX using decoder 641
                                             // 80740:      }
                                             // 80740:      } // switch Inst[14:13]
                                             // 80740:     }
                                             // 80740:     } // switch Inst[21]
                                             // 80740:    }
  3, 0,                                      // 80745:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 80747:     check Inst[21] == 0x0
  OPC_Decode, 182, 59, 247, 4,               // 80751:     decode to STRHHpre using decoder 631
                                             // 80751:    }
                                             // 80751:    } // switch Inst[11:10]
                                             // 80751:   }
  1, 181, 1,                                 // 80756:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 80759:    switch Inst[11:10] {
  0, 123,                                    // 80762:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 80764:     switch Inst[21] {
  0, 5,                                      // 80767:     case 0x0: {
  OPC_Decode, 250, 40, 247, 4,               // 80769:      decode to LDURHHi using decoder 631
                                             // 80769:     }
  1, 0,                                      // 80774:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 80776:      switch Inst[15:12] {
  0, 7,                                      // 80779:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 80781:       check predicate 72
  OPC_Decode, 147, 37, 249, 4,               // 80783:       decode to LDADDLH using decoder 633
                                             // 80783:      }
  1, 7,                                      // 80788:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 80790:       check predicate 72
  OPC_Decode, 218, 37, 249, 4,               // 80792:       decode to LDCLRLH using decoder 633
                                             // 80792:      }
  2, 7,                                      // 80797:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 80799:       check predicate 72
  OPC_Decode, 238, 37, 249, 4,               // 80801:       decode to LDEORLH using decoder 633
                                             // 80801:      }
  3, 7,                                      // 80806:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 80808:       check predicate 72
  OPC_Decode, 131, 40, 249, 4,               // 80810:       decode to LDSETLH using decoder 633
                                             // 80810:      }
  4, 7,                                      // 80815:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 80817:       check predicate 72
  OPC_Decode, 151, 40, 249, 4,               // 80819:       decode to LDSMAXLH using decoder 633
                                             // 80819:      }
  5, 7,                                      // 80824:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 80826:       check predicate 72
  OPC_Decode, 167, 40, 249, 4,               // 80828:       decode to LDSMINLH using decoder 633
                                             // 80828:      }
  6, 7,                                      // 80833:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 80835:       check predicate 72
  OPC_Decode, 226, 40, 249, 4,               // 80837:       decode to LDUMAXLH using decoder 633
                                             // 80837:      }
  7, 7,                                      // 80842:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 80844:       check predicate 72
  OPC_Decode, 242, 40, 249, 4,               // 80846:       decode to LDUMINLH using decoder 633
                                             // 80846:      }
  8, 7,                                      // 80851:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 80853:       check predicate 72
  OPC_Decode, 148, 61, 249, 4,               // 80855:       decode to SWPLH using decoder 633
                                             // 80855:      }
  9, 7,                                      // 80860:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 80862:       check predicate 125
  OPC_Decode, 133, 45, 255, 4,               // 80864:       decode to RCWCLRSL using decoder 639
                                             // 80864:      }
  10, 7,                                     // 80869:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 80871:       check predicate 125
  OPC_Decode, 173, 45, 255, 4,               // 80873:       decode to RCWSWPSL using decoder 639
                                             // 80873:      }
  11, 0,                                     // 80878:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 80880:       check predicate 125
  OPC_Decode, 157, 45, 255, 4,               // 80882:       decode to RCWSETSL using decoder 639
                                             // 80882:      }
                                             // 80882:      } // switch Inst[15:12]
                                             // 80882:     }
                                             // 80882:     } // switch Inst[21]
                                             // 80882:    }
  1, 9,                                      // 80887:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 80889:     check Inst[21] == 0x0
  OPC_Decode, 184, 39, 247, 4,               // 80893:     decode to LDRHHpost using decoder 631
                                             // 80893:    }
  2, 29,                                     // 80898:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 80900:     switch Inst[21] {
  0, 5,                                      // 80903:     case 0x0: {
  OPC_Decode, 197, 40, 247, 4,               // 80905:      decode to LDTRHi using decoder 631
                                             // 80905:     }
  1, 0,                                      // 80910:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 80912:      switch Inst[14:13] {
  2, 5,                                      // 80915:      case 0x2: {
  OPC_Decode, 186, 39, 128, 5,               // 80917:       decode to LDRHHroW using decoder 640
                                             // 80917:      }
  3, 0,                                      // 80922:      case 0x3: {
  OPC_Decode, 187, 39, 129, 5,               // 80924:       decode to LDRHHroX using decoder 641
                                             // 80924:      }
                                             // 80924:      } // switch Inst[14:13]
                                             // 80924:     }
                                             // 80924:     } // switch Inst[21]
                                             // 80924:    }
  3, 0,                                      // 80929:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 80931:     check Inst[21] == 0x0
  OPC_Decode, 185, 39, 247, 4,               // 80935:     decode to LDRHHpre using decoder 631
                                             // 80935:    }
                                             // 80935:    } // switch Inst[11:10]
                                             // 80935:   }
  2, 196, 1,                                 // 80940:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 80943:    switch Inst[11:10] {
  0, 137, 1,                                 // 80946:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 80949:     switch Inst[21] {
  0, 5,                                      // 80952:     case 0x0: {
  OPC_Decode, 128, 41, 247, 4,               // 80954:      decode to LDURSHXi using decoder 631
                                             // 80954:     }
  1, 0,                                      // 80959:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 80961:      switch Inst[15:12] {
  0, 7,                                      // 80964:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 80966:       check predicate 72
  OPC_Decode, 137, 37, 249, 4,               // 80968:       decode to LDADDAH using decoder 633
                                             // 80968:      }
  1, 7,                                      // 80973:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 80975:       check predicate 72
  OPC_Decode, 208, 37, 249, 4,               // 80977:       decode to LDCLRAH using decoder 633
                                             // 80977:      }
  2, 7,                                      // 80982:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 80984:       check predicate 72
  OPC_Decode, 228, 37, 249, 4,               // 80986:       decode to LDEORAH using decoder 633
                                             // 80986:      }
  3, 7,                                      // 80991:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 80993:       check predicate 72
  OPC_Decode, 249, 39, 249, 4,               // 80995:       decode to LDSETAH using decoder 633
                                             // 80995:      }
  4, 7,                                      // 81000:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 81002:       check predicate 72
  OPC_Decode, 141, 40, 249, 4,               // 81004:       decode to LDSMAXAH using decoder 633
                                             // 81004:      }
  5, 7,                                      // 81009:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 81011:       check predicate 72
  OPC_Decode, 157, 40, 249, 4,               // 81013:       decode to LDSMINAH using decoder 633
                                             // 81013:      }
  6, 7,                                      // 81018:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 81020:       check predicate 72
  OPC_Decode, 216, 40, 249, 4,               // 81022:       decode to LDUMAXAH using decoder 633
                                             // 81022:      }
  7, 7,                                      // 81027:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 81029:       check predicate 72
  OPC_Decode, 232, 40, 249, 4,               // 81031:       decode to LDUMINAH using decoder 633
                                             // 81031:      }
  8, 7,                                      // 81036:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 81038:       check predicate 72
  OPC_Decode, 138, 61, 249, 4,               // 81040:       decode to SWPAH using decoder 633
                                             // 81040:      }
  9, 7,                                      // 81045:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 81047:       check predicate 125
  OPC_Decode, 131, 45, 255, 4,               // 81049:       decode to RCWCLRSA using decoder 639
                                             // 81049:      }
  10, 7,                                     // 81054:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 81056:       check predicate 125
  OPC_Decode, 171, 45, 255, 4,               // 81058:       decode to RCWSWPSA using decoder 639
                                             // 81058:      }
  11, 7,                                     // 81063:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 81065:       check predicate 125
  OPC_Decode, 155, 45, 255, 4,               // 81067:       decode to RCWSETSA using decoder 639
                                             // 81067:      }
  12, 0,                                     // 81072:      case 0xc: {
  OPC_CheckPredicate, 129, 1,                // 81074:       check predicate 129
  OPC_CheckField, 16, 5, 31,                 // 81077:       check Inst[20:16] == 0x1f
  OPC_Decode, 155, 37, 239, 2,               // 81081:       decode to LDAPRH using decoder 367
                                             // 81081:      }
                                             // 81081:      } // switch Inst[15:12]
                                             // 81081:     }
                                             // 81081:     } // switch Inst[21]
                                             // 81081:    }
  1, 9,                                      // 81086:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 81088:     check Inst[21] == 0x0
  OPC_Decode, 215, 39, 247, 4,               // 81092:     decode to LDRSHXpost using decoder 631
                                             // 81092:    }
  2, 29,                                     // 81097:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 81099:     switch Inst[21] {
  0, 5,                                      // 81102:     case 0x0: {
  OPC_Decode, 201, 40, 247, 4,               // 81104:      decode to LDTRSHXi using decoder 631
                                             // 81104:     }
  1, 0,                                      // 81109:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 81111:      switch Inst[14:13] {
  2, 5,                                      // 81114:      case 0x2: {
  OPC_Decode, 217, 39, 130, 5,               // 81116:       decode to LDRSHXroW using decoder 642
                                             // 81116:      }
  3, 0,                                      // 81121:      case 0x3: {
  OPC_Decode, 218, 39, 131, 5,               // 81123:       decode to LDRSHXroX using decoder 643
                                             // 81123:      }
                                             // 81123:      } // switch Inst[14:13]
                                             // 81123:     }
                                             // 81123:     } // switch Inst[21]
                                             // 81123:    }
  3, 0,                                      // 81128:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 81130:     check Inst[21] == 0x0
  OPC_Decode, 216, 39, 247, 4,               // 81134:     decode to LDRSHXpre using decoder 631
                                             // 81134:    }
                                             // 81134:    } // switch Inst[11:10]
                                             // 81134:   }
  3, 181, 1,                                 // 81139:   case 0x3: {
  OPC_SwitchField, 10, 2,                    // 81142:    switch Inst[11:10] {
  0, 123,                                    // 81145:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 81147:     switch Inst[21] {
  0, 5,                                      // 81150:     case 0x0: {
  OPC_Decode, 255, 40, 247, 4,               // 81152:      decode to LDURSHWi using decoder 631
                                             // 81152:     }
  1, 0,                                      // 81157:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 81159:      switch Inst[15:12] {
  0, 7,                                      // 81162:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 81164:       check predicate 72
  OPC_Decode, 139, 37, 249, 4,               // 81166:       decode to LDADDALH using decoder 633
                                             // 81166:      }
  1, 7,                                      // 81171:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 81173:       check predicate 72
  OPC_Decode, 210, 37, 249, 4,               // 81175:       decode to LDCLRALH using decoder 633
                                             // 81175:      }
  2, 7,                                      // 81180:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 81182:       check predicate 72
  OPC_Decode, 230, 37, 249, 4,               // 81184:       decode to LDEORALH using decoder 633
                                             // 81184:      }
  3, 7,                                      // 81189:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 81191:       check predicate 72
  OPC_Decode, 251, 39, 249, 4,               // 81193:       decode to LDSETALH using decoder 633
                                             // 81193:      }
  4, 7,                                      // 81198:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 81200:       check predicate 72
  OPC_Decode, 143, 40, 249, 4,               // 81202:       decode to LDSMAXALH using decoder 633
                                             // 81202:      }
  5, 7,                                      // 81207:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 81209:       check predicate 72
  OPC_Decode, 159, 40, 249, 4,               // 81211:       decode to LDSMINALH using decoder 633
                                             // 81211:      }
  6, 7,                                      // 81216:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 81218:       check predicate 72
  OPC_Decode, 218, 40, 249, 4,               // 81220:       decode to LDUMAXALH using decoder 633
                                             // 81220:      }
  7, 7,                                      // 81225:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 81227:       check predicate 72
  OPC_Decode, 234, 40, 249, 4,               // 81229:       decode to LDUMINALH using decoder 633
                                             // 81229:      }
  8, 7,                                      // 81234:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 81236:       check predicate 72
  OPC_Decode, 140, 61, 249, 4,               // 81238:       decode to SWPALH using decoder 633
                                             // 81238:      }
  9, 7,                                      // 81243:      case 0x9: {
  OPC_CheckPredicate, 125,                   // 81245:       check predicate 125
  OPC_Decode, 132, 45, 255, 4,               // 81247:       decode to RCWCLRSAL using decoder 639
                                             // 81247:      }
  10, 7,                                     // 81252:      case 0xa: {
  OPC_CheckPredicate, 125,                   // 81254:       check predicate 125
  OPC_Decode, 172, 45, 255, 4,               // 81256:       decode to RCWSWPSAL using decoder 639
                                             // 81256:      }
  11, 0,                                     // 81261:      case 0xb: {
  OPC_CheckPredicate, 125,                   // 81263:       check predicate 125
  OPC_Decode, 156, 45, 255, 4,               // 81265:       decode to RCWSETSAL using decoder 639
                                             // 81265:      }
                                             // 81265:      } // switch Inst[15:12]
                                             // 81265:     }
                                             // 81265:     } // switch Inst[21]
                                             // 81265:    }
  1, 9,                                      // 81270:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 81272:     check Inst[21] == 0x0
  OPC_Decode, 210, 39, 247, 4,               // 81276:     decode to LDRSHWpost using decoder 631
                                             // 81276:    }
  2, 29,                                     // 81281:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 81283:     switch Inst[21] {
  0, 5,                                      // 81286:     case 0x0: {
  OPC_Decode, 200, 40, 247, 4,               // 81288:      decode to LDTRSHWi using decoder 631
                                             // 81288:     }
  1, 0,                                      // 81293:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 81295:      switch Inst[14:13] {
  2, 5,                                      // 81298:      case 0x2: {
  OPC_Decode, 212, 39, 128, 5,               // 81300:       decode to LDRSHWroW using decoder 640
                                             // 81300:      }
  3, 0,                                      // 81305:      case 0x3: {
  OPC_Decode, 213, 39, 129, 5,               // 81307:       decode to LDRSHWroX using decoder 641
                                             // 81307:      }
                                             // 81307:      } // switch Inst[14:13]
                                             // 81307:     }
                                             // 81307:     } // switch Inst[21]
                                             // 81307:    }
  3, 0,                                      // 81312:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 81314:     check Inst[21] == 0x0
  OPC_Decode, 211, 39, 247, 4,               // 81318:     decode to LDRSHWpre using decoder 631
                                             // 81318:    }
                                             // 81318:    } // switch Inst[11:10]
                                             // 81318:   }
  4, 5,                                      // 81323:   case 0x4: {
  OPC_Decode, 185, 59, 132, 5,               // 81325:    decode to STRHHui using decoder 644
                                             // 81325:   }
  5, 5,                                      // 81330:   case 0x5: {
  OPC_Decode, 188, 39, 132, 5,               // 81332:    decode to LDRHHui using decoder 644
                                             // 81332:   }
  6, 5,                                      // 81337:   case 0x6: {
  OPC_Decode, 219, 39, 132, 5,               // 81339:    decode to LDRSHXui using decoder 644
                                             // 81339:   }
  7, 5,                                      // 81344:   case 0x7: {
  OPC_Decode, 214, 39, 132, 5,               // 81346:    decode to LDRSHWui using decoder 644
                                             // 81346:   }
  8, 13,                                     // 81351:   case 0x8: {
  OPC_CheckField, 21, 1, 0,                  // 81353:    check Inst[21] == 0x0
  OPC_CheckField, 10, 6, 0,                  // 81357:    check Inst[15:10] == 0x0
  OPC_Decode, 231, 46, 252, 4,               // 81361:    decode to SBCSWr using decoder 636
                                             // 81361:   }
  9, 0,                                      // 81366:   case 0x9: {
  OPC_SwitchField, 10, 2,                    // 81368:    switch Inst[11:10] {
  0, 13,                                     // 81371:    case 0x0: {
  OPC_CheckField, 21, 1, 0,                  // 81373:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 81377:     check Inst[4] == 0x0
  OPC_Decode, 141, 17, 134, 5,               // 81381:     decode to CCMPWr using decoder 646
                                             // 81381:    }
  2, 0,                                      // 81386:    case 0x2: {
  OPC_CheckField, 21, 1, 0,                  // 81388:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 81392:     check Inst[4] == 0x0
  OPC_Decode, 140, 17, 135, 5,               // 81396:     decode to CCMPWi using decoder 647
                                             // 81396:    }
                                             // 81396:    } // switch Inst[11:10]
                                             // 81396:   }
                                             // 81396:   } // switch Inst[25:22]
                                             // 81396:  }
  4, 188, 3,                                 // 81401:  case 0x4: {
  OPC_SwitchField, 24, 2,                    // 81404:   switch Inst[25:24] {
  0, 5,                                      // 81407:   case 0x0: {
  OPC_Decode, 220, 39, 220, 4,               // 81409:    decode to LDRSWl using decoder 604
                                             // 81409:   }
  1, 123,                                    // 81414:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 81416:    switch Inst[23:21] {
  0, 37,                                     // 81419:    case 0x0: {
  OPC_SwitchField, 10, 2,                    // 81421:     switch Inst[11:10] {
  0, 7,                                      // 81424:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 81426:      check predicate 123
  OPC_Decode, 212, 58, 247, 4,               // 81428:      decode to STLURWi using decoder 631
                                             // 81428:     }
  2, 0,                                      // 81433:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 81435:      switch Inst[15:12] {
  0, 8,                                      // 81438:      case 0x0: {
  OPC_CheckPredicate, 130, 1,                // 81440:       check predicate 130
  OPC_Decode, 193, 58, 137, 5,               // 81443:       decode to STILPWpre using decoder 649
                                             // 81443:      }
  1, 0,                                      // 81448:      case 0x1: {
  OPC_CheckPredicate, 130, 1,                // 81450:       check predicate 130
  OPC_Decode, 192, 58, 249, 4,               // 81453:       decode to STILPW using decoder 633
                                             // 81453:      }
                                             // 81453:      } // switch Inst[15:12]
                                             // 81453:     }
                                             // 81453:     } // switch Inst[11:10]
                                             // 81453:    }
  2, 37,                                     // 81458:    case 0x2: {
  OPC_SwitchField, 10, 2,                    // 81460:     switch Inst[11:10] {
  0, 7,                                      // 81463:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 81465:      check predicate 123
  OPC_Decode, 171, 37, 247, 4,               // 81467:      decode to LDAPURi using decoder 631
                                             // 81467:     }
  2, 0,                                      // 81472:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 81474:      switch Inst[15:12] {
  0, 8,                                      // 81477:      case 0x0: {
  OPC_CheckPredicate, 130, 1,                // 81479:       check predicate 130
  OPC_Decode, 194, 38, 137, 5,               // 81482:       decode to LDIAPPWpost using decoder 649
                                             // 81482:      }
  1, 0,                                      // 81487:      case 0x1: {
  OPC_CheckPredicate, 130, 1,                // 81489:       check predicate 130
  OPC_Decode, 193, 38, 249, 4,               // 81492:       decode to LDIAPPW using decoder 633
                                             // 81492:      }
                                             // 81492:      } // switch Inst[15:12]
                                             // 81492:     }
                                             // 81492:     } // switch Inst[11:10]
                                             // 81492:    }
  4, 26,                                     // 81497:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 81499:     switch Inst[11:10] {
  0, 7,                                      // 81502:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 81504:      check predicate 123
  OPC_Decode, 166, 37, 247, 4,               // 81506:      decode to LDAPURSWi using decoder 631
                                             // 81506:     }
  2, 0,                                      // 81511:     case 0x2: {
  OPC_CheckPredicate, 130, 1,                // 81513:      check predicate 130
  OPC_CheckField, 12, 9, 0,                  // 81516:      check Inst[20:12] == 0x0
  OPC_Decode, 205, 58, 138, 5,               // 81520:      decode to STLRWpre using decoder 650
                                             // 81520:     }
                                             // 81520:     } // switch Inst[11:10]
                                             // 81520:    }
  6, 0,                                      // 81525:    case 0x6: {
  OPC_CheckPredicate, 130, 1,                // 81527:     check predicate 130
  OPC_CheckField, 10, 11, 2,                 // 81530:     check Inst[20:10] == 0x2
  OPC_Decode, 157, 37, 138, 5,               // 81534:     decode to LDAPRWpost using decoder 650
                                             // 81534:    }
                                             // 81534:    } // switch Inst[23:21]
                                             // 81534:   }
  2, 187, 1,                                 // 81539:   case 0x2: {
  OPC_SwitchField, 21, 3,                    // 81542:    switch Inst[23:21] {
  0, 24,                                     // 81545:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 81547:     switch Inst[15:13] {
  0, 9,                                      // 81550:     case 0x0: {
  OPC_CheckField, 10, 3, 0,                  // 81552:      check Inst[12:10] == 0x0
  OPC_Decode, 133, 13, 139, 5,               // 81556:      decode to ADCXr using decoder 651
                                             // 81556:     }
  1, 0,                                      // 81561:     case 0x1: {
  OPC_CheckPredicate, 131, 1,                // 81563:      check predicate 131
  OPC_Decode, 150, 13, 140, 5,               // 81566:      decode to ADDPT_shift using decoder 652
                                             // 81566:     }
                                             // 81566:     } // switch Inst[15:13]
                                             // 81566:    }
  4, 17,                                     // 81571:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 81573:     switch Inst[11:10] {
  0, 5,                                      // 81576:     case 0x0: {
  OPC_Decode, 168, 20, 141, 5,               // 81578:      decode to CSELXr using decoder 653
                                             // 81578:     }
  1, 0,                                      // 81583:     case 0x1: {
  OPC_Decode, 170, 20, 141, 5,               // 81585:      decode to CSINCXr using decoder 653
                                             // 81585:     }
                                             // 81585:     } // switch Inst[11:10]
                                             // 81585:    }
  6, 0,                                      // 81590:    case 0x6: {
  OPC_SwitchField, 10, 6,                    // 81592:     switch Inst[15:10] {
  0, 7,                                      // 81595:     case 0x0: {
  OPC_CheckPredicate, 75,                    // 81597:      check predicate 75
  OPC_Decode, 137, 60, 142, 5,               // 81599:      decode to SUBP using decoder 654
                                             // 81599:     }
  2, 5,                                      // 81604:     case 0x2: {
  OPC_Decode, 172, 63, 139, 5,               // 81606:      decode to UDIVXr using decoder 651
                                             // 81606:     }
  3, 5,                                      // 81611:     case 0x3: {
  OPC_Decode, 178, 47, 139, 5,               // 81613:      decode to SDIVXr using decoder 651
                                             // 81613:     }
  4, 7,                                      // 81618:     case 0x4: {
  OPC_CheckPredicate, 75,                    // 81620:      check predicate 75
  OPC_Decode, 155, 34, 143, 5,               // 81622:      decode to IRG using decoder 655
                                             // 81622:     }
  5, 7,                                      // 81627:     case 0x5: {
  OPC_CheckPredicate, 75,                    // 81629:      check predicate 75
  OPC_Decode, 220, 33, 144, 5,               // 81631:      decode to GMI using decoder 656
                                             // 81631:     }
  8, 5,                                      // 81636:     case 0x8: {
  OPC_Decode, 144, 41, 139, 5,               // 81638:      decode to LSLVXr using decoder 651
                                             // 81638:     }
  9, 5,                                      // 81643:     case 0x9: {
  OPC_Decode, 168, 41, 139, 5,               // 81645:      decode to LSRVXr using decoder 651
                                             // 81645:     }
  10, 5,                                     // 81650:     case 0xa: {
  OPC_Decode, 183, 14, 139, 5,               // 81652:      decode to ASRVXr using decoder 651
                                             // 81652:     }
  11, 5,                                     // 81657:     case 0xb: {
  OPC_Decode, 231, 45, 139, 5,               // 81659:      decode to RORVXr using decoder 651
                                             // 81659:     }
  12, 7,                                     // 81664:     case 0xc: {
  OPC_CheckPredicate, 122,                   // 81666:      check predicate 122
  OPC_Decode, 229, 43, 145, 5,               // 81668:      decode to PACGA using decoder 657
                                             // 81668:     }
  19, 8,                                     // 81673:     case 0x13: {
  OPC_CheckPredicate, 128, 1,                // 81675:      check predicate 128
  OPC_Decode, 166, 20, 146, 5,               // 81678:      decode to CRC32Xrr using decoder 658
                                             // 81678:     }
  23, 8,                                     // 81683:     case 0x17: {
  OPC_CheckPredicate, 128, 1,                // 81685:      check predicate 128
  OPC_Decode, 163, 20, 146, 5,               // 81688:      decode to CRC32CXrr using decoder 658
                                             // 81688:     }
  24, 7,                                     // 81693:     case 0x18: {
  OPC_CheckPredicate, 103,                   // 81695:      check predicate 103
  OPC_Decode, 246, 48, 139, 5,               // 81697:      decode to SMAXXrr using decoder 651
                                             // 81697:     }
  25, 7,                                     // 81702:     case 0x19: {
  OPC_CheckPredicate, 103,                   // 81704:      check predicate 103
  OPC_Decode, 128, 64, 139, 5,               // 81706:      decode to UMAXXrr using decoder 651
                                             // 81706:     }
  26, 7,                                     // 81711:     case 0x1a: {
  OPC_CheckPredicate, 103,                   // 81713:      check predicate 103
  OPC_Decode, 176, 49, 139, 5,               // 81715:      decode to SMINXrr using decoder 651
                                             // 81715:     }
  27, 0,                                     // 81720:     case 0x1b: {
  OPC_CheckPredicate, 103,                   // 81722:      check predicate 103
  OPC_Decode, 185, 64, 139, 5,               // 81724:      decode to UMINXrr using decoder 651
                                             // 81724:     }
                                             // 81724:     } // switch Inst[15:10]
                                             // 81724:    }
                                             // 81724:    } // switch Inst[23:21]
                                             // 81724:   }
  3, 0,                                      // 81729:   case 0x3: {
  OPC_SwitchField, 21, 3,                    // 81731:    switch Inst[23:21] {
  0, 17,                                     // 81734:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 81736:     switch Inst[15] {
  0, 5,                                      // 81739:     case 0x0: {
  OPC_Decode, 231, 41, 147, 5,               // 81741:      decode to MADDXrrr using decoder 659
                                             // 81741:     }
  1, 0,                                      // 81746:     case 0x1: {
  OPC_Decode, 255, 42, 147, 5,               // 81748:      decode to MSUBXrrr using decoder 659
                                             // 81748:     }
                                             // 81748:     } // switch Inst[15]
                                             // 81748:    }
  1, 17,                                     // 81753:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 81755:     switch Inst[15] {
  0, 5,                                      // 81758:     case 0x0: {
  OPC_Decode, 219, 48, 148, 5,               // 81760:      decode to SMADDLrrr using decoder 660
                                             // 81760:     }
  1, 0,                                      // 81765:     case 0x1: {
  OPC_Decode, 209, 50, 148, 5,               // 81767:      decode to SMSUBLrrr using decoder 660
                                             // 81767:     }
                                             // 81767:     } // switch Inst[15]
                                             // 81767:    }
  2, 14,                                     // 81772:    case 0x2: {
  OPC_CheckField, 15, 1, 0,                  // 81774:     check Inst[15] == 0x0
  OPC_SoftFail, 0, 128, 248, 1,              // 81778:     softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 218, 50, 139, 5,               // 81783:     decode to SMULHrr using decoder 651
                                             // 81783:    }
  3, 23,                                     // 81788:    case 0x3: {
  OPC_SwitchField, 15, 1,                    // 81790:     switch Inst[15] {
  0, 8,                                      // 81793:     case 0x0: {
  OPC_CheckPredicate, 131, 1,                // 81795:      check predicate 131
  OPC_Decode, 229, 41, 147, 5,               // 81798:      decode to MADDPT using decoder 659
                                             // 81798:     }
  1, 0,                                      // 81803:     case 0x1: {
  OPC_CheckPredicate, 131, 1,                // 81805:      check predicate 131
  OPC_Decode, 253, 42, 147, 5,               // 81808:      decode to MSUBPT using decoder 659
                                             // 81808:     }
                                             // 81808:     } // switch Inst[15]
                                             // 81808:    }
  5, 17,                                     // 81813:    case 0x5: {
  OPC_SwitchField, 15, 1,                    // 81815:     switch Inst[15] {
  0, 5,                                      // 81818:     case 0x0: {
  OPC_Decode, 229, 63, 148, 5,               // 81820:      decode to UMADDLrrr using decoder 660
                                             // 81820:     }
  1, 0,                                      // 81825:     case 0x1: {
  OPC_Decode, 216, 65, 148, 5,               // 81827:      decode to UMSUBLrrr using decoder 660
                                             // 81827:     }
                                             // 81827:     } // switch Inst[15]
                                             // 81827:    }
  6, 0,                                      // 81832:    case 0x6: {
  OPC_CheckField, 15, 1, 0,                  // 81834:     check Inst[15] == 0x0
  OPC_SoftFail, 0, 128, 248, 1,              // 81838:     softfail pos=0x00000000 neg=0x00007c00
  OPC_Decode, 225, 65, 139, 5,               // 81843:     decode to UMULHrr using decoder 651
                                             // 81843:    }
                                             // 81843:    } // switch Inst[23:21]
                                             // 81843:   }
                                             // 81843:   } // switch Inst[25:24]
                                             // 81843:  }
  5, 208, 5,                                 // 81848:  case 0x5: {
  OPC_SwitchField, 22, 4,                    // 81851:   switch Inst[25:22] {
  0, 154, 1,                                 // 81854:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 81857:    switch Inst[11:10] {
  0, 96,                                     // 81860:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 81862:     switch Inst[21] {
  0, 5,                                      // 81865:     case 0x0: {
  OPC_Decode, 237, 59, 247, 4,               // 81867:      decode to STURWi using decoder 631
                                             // 81867:     }
  1, 0,                                      // 81872:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 81874:      switch Inst[15:12] {
  0, 7,                                      // 81877:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 81879:       check predicate 72
  OPC_Decode, 150, 37, 249, 4,               // 81881:       decode to LDADDW using decoder 633
                                             // 81881:      }
  1, 7,                                      // 81886:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 81888:       check predicate 72
  OPC_Decode, 225, 37, 249, 4,               // 81890:       decode to LDCLRW using decoder 633
                                             // 81890:      }
  2, 7,                                      // 81895:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 81897:       check predicate 72
  OPC_Decode, 241, 37, 249, 4,               // 81899:       decode to LDEORW using decoder 633
                                             // 81899:      }
  3, 7,                                      // 81904:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 81906:       check predicate 72
  OPC_Decode, 138, 40, 249, 4,               // 81908:       decode to LDSETW using decoder 633
                                             // 81908:      }
  4, 7,                                      // 81913:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 81915:       check predicate 72
  OPC_Decode, 154, 40, 249, 4,               // 81917:       decode to LDSMAXW using decoder 633
                                             // 81917:      }
  5, 7,                                      // 81922:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 81924:       check predicate 72
  OPC_Decode, 170, 40, 249, 4,               // 81926:       decode to LDSMINW using decoder 633
                                             // 81926:      }
  6, 7,                                      // 81931:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 81933:       check predicate 72
  OPC_Decode, 229, 40, 249, 4,               // 81935:       decode to LDUMAXW using decoder 633
                                             // 81935:      }
  7, 7,                                      // 81940:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 81942:       check predicate 72
  OPC_Decode, 245, 40, 249, 4,               // 81944:       decode to LDUMINW using decoder 633
                                             // 81944:      }
  8, 0,                                      // 81949:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 81951:       check predicate 72
  OPC_Decode, 163, 61, 249, 4,               // 81953:       decode to SWPW using decoder 633
                                             // 81953:      }
                                             // 81953:      } // switch Inst[15:12]
                                             // 81953:     }
                                             // 81953:     } // switch Inst[21]
                                             // 81953:    }
  1, 9,                                      // 81958:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 81960:     check Inst[21] == 0x0
  OPC_Decode, 201, 59, 247, 4,               // 81964:     decode to STRWpost using decoder 631
                                             // 81964:    }
  2, 29,                                     // 81969:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 81971:     switch Inst[21] {
  0, 5,                                      // 81974:     case 0x0: {
  OPC_Decode, 226, 59, 247, 4,               // 81976:      decode to STTRWi using decoder 631
                                             // 81976:     }
  1, 0,                                      // 81981:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 81983:      switch Inst[14:13] {
  2, 5,                                      // 81986:      case 0x2: {
  OPC_Decode, 203, 59, 128, 5,               // 81988:       decode to STRWroW using decoder 640
                                             // 81988:      }
  3, 0,                                      // 81993:      case 0x3: {
  OPC_Decode, 204, 59, 129, 5,               // 81995:       decode to STRWroX using decoder 641
                                             // 81995:      }
                                             // 81995:      } // switch Inst[14:13]
                                             // 81995:     }
                                             // 81995:     } // switch Inst[21]
                                             // 81995:    }
  3, 0,                                      // 82000:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 82002:     check Inst[21] == 0x0
  OPC_Decode, 202, 59, 247, 4,               // 82006:     decode to STRWpre using decoder 631
                                             // 82006:    }
                                             // 82006:    } // switch Inst[11:10]
                                             // 82006:   }
  1, 154, 1,                                 // 82011:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 82014:    switch Inst[11:10] {
  0, 96,                                     // 82017:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 82019:     switch Inst[21] {
  0, 5,                                      // 82022:     case 0x0: {
  OPC_Decode, 131, 41, 247, 4,               // 82024:      decode to LDURWi using decoder 631
                                             // 82024:     }
  1, 0,                                      // 82029:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 82031:      switch Inst[15:12] {
  0, 7,                                      // 82034:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 82036:       check predicate 72
  OPC_Decode, 148, 37, 249, 4,               // 82038:       decode to LDADDLW using decoder 633
                                             // 82038:      }
  1, 7,                                      // 82043:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 82045:       check predicate 72
  OPC_Decode, 219, 37, 249, 4,               // 82047:       decode to LDCLRLW using decoder 633
                                             // 82047:      }
  2, 7,                                      // 82052:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 82054:       check predicate 72
  OPC_Decode, 239, 37, 249, 4,               // 82056:       decode to LDEORLW using decoder 633
                                             // 82056:      }
  3, 7,                                      // 82061:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 82063:       check predicate 72
  OPC_Decode, 132, 40, 249, 4,               // 82065:       decode to LDSETLW using decoder 633
                                             // 82065:      }
  4, 7,                                      // 82070:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 82072:       check predicate 72
  OPC_Decode, 152, 40, 249, 4,               // 82074:       decode to LDSMAXLW using decoder 633
                                             // 82074:      }
  5, 7,                                      // 82079:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 82081:       check predicate 72
  OPC_Decode, 168, 40, 249, 4,               // 82083:       decode to LDSMINLW using decoder 633
                                             // 82083:      }
  6, 7,                                      // 82088:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 82090:       check predicate 72
  OPC_Decode, 227, 40, 249, 4,               // 82092:       decode to LDUMAXLW using decoder 633
                                             // 82092:      }
  7, 7,                                      // 82097:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 82099:       check predicate 72
  OPC_Decode, 243, 40, 249, 4,               // 82101:       decode to LDUMINLW using decoder 633
                                             // 82101:      }
  8, 0,                                      // 82106:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 82108:       check predicate 72
  OPC_Decode, 149, 61, 249, 4,               // 82110:       decode to SWPLW using decoder 633
                                             // 82110:      }
                                             // 82110:      } // switch Inst[15:12]
                                             // 82110:     }
                                             // 82110:     } // switch Inst[21]
                                             // 82110:    }
  1, 9,                                      // 82115:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 82117:     check Inst[21] == 0x0
  OPC_Decode, 233, 39, 247, 4,               // 82121:     decode to LDRWpost using decoder 631
                                             // 82121:    }
  2, 29,                                     // 82126:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 82128:     switch Inst[21] {
  0, 5,                                      // 82131:     case 0x0: {
  OPC_Decode, 203, 40, 247, 4,               // 82133:      decode to LDTRWi using decoder 631
                                             // 82133:     }
  1, 0,                                      // 82138:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 82140:      switch Inst[14:13] {
  2, 5,                                      // 82143:      case 0x2: {
  OPC_Decode, 235, 39, 128, 5,               // 82145:       decode to LDRWroW using decoder 640
                                             // 82145:      }
  3, 0,                                      // 82150:      case 0x3: {
  OPC_Decode, 236, 39, 129, 5,               // 82152:       decode to LDRWroX using decoder 641
                                             // 82152:      }
                                             // 82152:      } // switch Inst[14:13]
                                             // 82152:     }
                                             // 82152:     } // switch Inst[21]
                                             // 82152:    }
  3, 0,                                      // 82157:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 82159:     check Inst[21] == 0x0
  OPC_Decode, 234, 39, 247, 4,               // 82163:     decode to LDRWpre using decoder 631
                                             // 82163:    }
                                             // 82163:    } // switch Inst[11:10]
                                             // 82163:   }
  2, 168, 1,                                 // 82168:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 82171:    switch Inst[11:10] {
  0, 110,                                    // 82174:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 82176:     switch Inst[21] {
  0, 5,                                      // 82179:     case 0x0: {
  OPC_Decode, 129, 41, 247, 4,               // 82181:      decode to LDURSWi using decoder 631
                                             // 82181:     }
  1, 0,                                      // 82186:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 82188:      switch Inst[15:12] {
  0, 7,                                      // 82191:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 82193:       check predicate 72
  OPC_Decode, 142, 37, 249, 4,               // 82195:       decode to LDADDAW using decoder 633
                                             // 82195:      }
  1, 7,                                      // 82200:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 82202:       check predicate 72
  OPC_Decode, 213, 37, 249, 4,               // 82204:       decode to LDCLRAW using decoder 633
                                             // 82204:      }
  2, 7,                                      // 82209:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 82211:       check predicate 72
  OPC_Decode, 233, 37, 249, 4,               // 82213:       decode to LDEORAW using decoder 633
                                             // 82213:      }
  3, 7,                                      // 82218:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 82220:       check predicate 72
  OPC_Decode, 254, 39, 249, 4,               // 82222:       decode to LDSETAW using decoder 633
                                             // 82222:      }
  4, 7,                                      // 82227:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 82229:       check predicate 72
  OPC_Decode, 146, 40, 249, 4,               // 82231:       decode to LDSMAXAW using decoder 633
                                             // 82231:      }
  5, 7,                                      // 82236:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 82238:       check predicate 72
  OPC_Decode, 162, 40, 249, 4,               // 82240:       decode to LDSMINAW using decoder 633
                                             // 82240:      }
  6, 7,                                      // 82245:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 82247:       check predicate 72
  OPC_Decode, 221, 40, 249, 4,               // 82249:       decode to LDUMAXAW using decoder 633
                                             // 82249:      }
  7, 7,                                      // 82254:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 82256:       check predicate 72
  OPC_Decode, 237, 40, 249, 4,               // 82258:       decode to LDUMINAW using decoder 633
                                             // 82258:      }
  8, 7,                                      // 82263:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 82265:       check predicate 72
  OPC_Decode, 143, 61, 249, 4,               // 82267:       decode to SWPAW using decoder 633
                                             // 82267:      }
  12, 0,                                     // 82272:      case 0xc: {
  OPC_CheckPredicate, 129, 1,                // 82274:       check predicate 129
  OPC_CheckField, 16, 5, 31,                 // 82277:       check Inst[20:16] == 0x1f
  OPC_Decode, 156, 37, 239, 2,               // 82281:       decode to LDAPRW using decoder 367
                                             // 82281:      }
                                             // 82281:      } // switch Inst[15:12]
                                             // 82281:     }
                                             // 82281:     } // switch Inst[21]
                                             // 82281:    }
  1, 9,                                      // 82286:    case 0x1: {
  OPC_CheckField, 21, 1, 0,                  // 82288:     check Inst[21] == 0x0
  OPC_Decode, 221, 39, 247, 4,               // 82292:     decode to LDRSWpost using decoder 631
                                             // 82292:    }
  2, 29,                                     // 82297:    case 0x2: {
  OPC_SwitchField, 21, 1,                    // 82299:     switch Inst[21] {
  0, 5,                                      // 82302:     case 0x0: {
  OPC_Decode, 202, 40, 247, 4,               // 82304:      decode to LDTRSWi using decoder 631
                                             // 82304:     }
  1, 0,                                      // 82309:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 82311:      switch Inst[14:13] {
  2, 5,                                      // 82314:      case 0x2: {
  OPC_Decode, 223, 39, 130, 5,               // 82316:       decode to LDRSWroW using decoder 642
                                             // 82316:      }
  3, 0,                                      // 82321:      case 0x3: {
  OPC_Decode, 224, 39, 131, 5,               // 82323:       decode to LDRSWroX using decoder 643
                                             // 82323:      }
                                             // 82323:      } // switch Inst[14:13]
                                             // 82323:     }
                                             // 82323:     } // switch Inst[21]
                                             // 82323:    }
  3, 0,                                      // 82328:    case 0x3: {
  OPC_CheckField, 21, 1, 0,                  // 82330:     check Inst[21] == 0x0
  OPC_Decode, 222, 39, 247, 4,               // 82334:     decode to LDRSWpre using decoder 631
                                             // 82334:    }
                                             // 82334:    } // switch Inst[11:10]
                                             // 82334:   }
  3, 120,                                    // 82339:   case 0x3: {
  OPC_SwitchField, 10, 6,                    // 82341:    switch Inst[15:10] {
  0, 11,                                     // 82344:    case 0x0: {
  OPC_CheckPredicate, 72,                    // 82346:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82348:     check Inst[21] == 0x1
  OPC_Decode, 140, 37, 249, 4,               // 82352:     decode to LDADDALW using decoder 633
                                             // 82352:    }
  4, 11,                                     // 82357:    case 0x4: {
  OPC_CheckPredicate, 72,                    // 82359:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82361:     check Inst[21] == 0x1
  OPC_Decode, 211, 37, 249, 4,               // 82365:     decode to LDCLRALW using decoder 633
                                             // 82365:    }
  8, 11,                                     // 82370:    case 0x8: {
  OPC_CheckPredicate, 72,                    // 82372:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82374:     check Inst[21] == 0x1
  OPC_Decode, 231, 37, 249, 4,               // 82378:     decode to LDEORALW using decoder 633
                                             // 82378:    }
  12, 11,                                    // 82383:    case 0xc: {
  OPC_CheckPredicate, 72,                    // 82385:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82387:     check Inst[21] == 0x1
  OPC_Decode, 252, 39, 249, 4,               // 82391:     decode to LDSETALW using decoder 633
                                             // 82391:    }
  16, 11,                                    // 82396:    case 0x10: {
  OPC_CheckPredicate, 72,                    // 82398:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82400:     check Inst[21] == 0x1
  OPC_Decode, 144, 40, 249, 4,               // 82404:     decode to LDSMAXALW using decoder 633
                                             // 82404:    }
  20, 11,                                    // 82409:    case 0x14: {
  OPC_CheckPredicate, 72,                    // 82411:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82413:     check Inst[21] == 0x1
  OPC_Decode, 160, 40, 249, 4,               // 82417:     decode to LDSMINALW using decoder 633
                                             // 82417:    }
  24, 11,                                    // 82422:    case 0x18: {
  OPC_CheckPredicate, 72,                    // 82424:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82426:     check Inst[21] == 0x1
  OPC_Decode, 219, 40, 249, 4,               // 82430:     decode to LDUMAXALW using decoder 633
                                             // 82430:    }
  28, 11,                                    // 82435:    case 0x1c: {
  OPC_CheckPredicate, 72,                    // 82437:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82439:     check Inst[21] == 0x1
  OPC_Decode, 235, 40, 249, 4,               // 82443:     decode to LDUMINALW using decoder 633
                                             // 82443:    }
  32, 0,                                     // 82448:    case 0x20: {
  OPC_CheckPredicate, 72,                    // 82450:     check predicate 72
  OPC_CheckField, 21, 1, 1,                  // 82452:     check Inst[21] == 0x1
  OPC_Decode, 141, 61, 249, 4,               // 82456:     decode to SWPALW using decoder 633
                                             // 82456:    }
                                             // 82456:    } // switch Inst[15:10]
                                             // 82456:   }
  4, 5,                                      // 82461:   case 0x4: {
  OPC_Decode, 205, 59, 132, 5,               // 82463:    decode to STRWui using decoder 644
                                             // 82463:   }
  5, 5,                                      // 82468:   case 0x5: {
  OPC_Decode, 237, 39, 132, 5,               // 82470:    decode to LDRWui using decoder 644
                                             // 82470:   }
  6, 5,                                      // 82475:   case 0x6: {
  OPC_Decode, 225, 39, 132, 5,               // 82477:    decode to LDRSWui using decoder 644
                                             // 82477:   }
  8, 35,                                     // 82482:   case 0x8: {
  OPC_SwitchField, 10, 5,                    // 82484:    switch Inst[14:10] {
  0, 13,                                     // 82487:    case 0x0: {
  OPC_CheckField, 21, 1, 0,                  // 82489:     check Inst[21] == 0x0
  OPC_CheckField, 15, 1, 0,                  // 82493:     check Inst[15] == 0x0
  OPC_Decode, 131, 13, 139, 5,               // 82497:     decode to ADCSXr using decoder 651
                                             // 82497:    }
  1, 0,                                      // 82502:    case 0x1: {
  OPC_CheckPredicate, 115,                   // 82504:     check predicate 115
  OPC_CheckField, 21, 1, 0,                  // 82506:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 82510:     check Inst[4] == 0x0
  OPC_Decode, 229, 45, 149, 5,               // 82514:     decode to RMIF using decoder 661
                                             // 82514:    }
                                             // 82514:    } // switch Inst[14:10]
                                             // 82514:   }
  9, 33,                                     // 82519:   case 0x9: {
  OPC_SwitchField, 10, 2,                    // 82521:    switch Inst[11:10] {
  0, 13,                                     // 82524:    case 0x0: {
  OPC_CheckField, 21, 1, 0,                  // 82526:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 82530:     check Inst[4] == 0x0
  OPC_Decode, 139, 17, 150, 5,               // 82534:     decode to CCMNXr using decoder 662
                                             // 82534:    }
  2, 0,                                      // 82539:    case 0x2: {
  OPC_CheckField, 21, 1, 0,                  // 82541:     check Inst[21] == 0x0
  OPC_CheckField, 4, 1, 0,                   // 82545:     check Inst[4] == 0x0
  OPC_Decode, 138, 17, 151, 5,               // 82549:     decode to CCMNXi using decoder 663
                                             // 82549:    }
                                             // 82549:    } // switch Inst[11:10]
                                             // 82549:   }
  11, 0,                                     // 82554:   case 0xb: {
  OPC_CheckPredicate, 75,                    // 82556:    check predicate 75
  OPC_CheckField, 21, 1, 0,                  // 82558:    check Inst[21] == 0x0
  OPC_CheckField, 10, 6, 0,                  // 82562:    check Inst[15:10] == 0x0
  OPC_Decode, 138, 60, 142, 5,               // 82566:    decode to SUBPS using decoder 654
                                             // 82566:   }
                                             // 82566:   } // switch Inst[25:22]
                                             // 82566:  }
  6, 177, 6,                                 // 82571:  case 0x6: {
  OPC_SwitchField, 24, 2,                    // 82574:   switch Inst[25:24] {
  0, 5,                                      // 82577:   case 0x0: {
  OPC_Decode, 183, 44, 152, 5,               // 82579:    decode to PRFMl using decoder 664
                                             // 82579:   }
  1, 212, 2,                                 // 82584:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 82587:    switch Inst[23:21] {
  0, 72,                                     // 82590:    case 0x0: {
  OPC_SwitchField, 10, 2,                    // 82592:     switch Inst[11:10] {
  0, 7,                                      // 82595:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 82597:      check predicate 123
  OPC_Decode, 213, 58, 247, 4,               // 82599:      decode to STLURXi using decoder 631
                                             // 82599:     }
  2, 33,                                     // 82604:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 82606:      switch Inst[15:12] {
  0, 8,                                      // 82609:      case 0x0: {
  OPC_CheckPredicate, 130, 1,                // 82611:       check predicate 130
  OPC_Decode, 195, 58, 153, 5,               // 82614:       decode to STILPXpre using decoder 665
                                             // 82614:      }
  1, 8,                                      // 82619:      case 0x1: {
  OPC_CheckPredicate, 130, 1,                // 82621:       check predicate 130
  OPC_Decode, 194, 58, 255, 4,               // 82624:       decode to STILPX using decoder 639
                                             // 82624:      }
  5, 0,                                      // 82629:      case 0x5: {
  OPC_CheckPredicate, 132, 1,                // 82631:       check predicate 132
  OPC_Decode, 201, 58, 255, 4,               // 82634:       decode to STLPi using decoder 639
                                             // 82634:      }
                                             // 82634:      } // switch Inst[15:12]
                                             // 82634:     }
  3, 0,                                      // 82639:     case 0x3: {
  OPC_SwitchField, 12, 9,                    // 82641:      switch Inst[20:12] {
  240, 3, 7,                                 // 82644:      case 0x1f0: {
  OPC_CheckPredicate, 117,                   // 82647:       check predicate 117
  OPC_Decode, 217, 32, 242, 2,               // 82649:       decode to GCSSTR using decoder 370
                                             // 82649:      }
  241, 3, 0,                                 // 82654:      case 0x1f1: {
  OPC_CheckPredicate, 117,                   // 82657:       check predicate 117
  OPC_Decode, 218, 32, 242, 2,               // 82659:       decode to GCSSTTR using decoder 370
                                             // 82659:      }
                                             // 82659:      } // switch Inst[20:12]
                                             // 82659:     }
                                             // 82659:     } // switch Inst[11:10]
                                             // 82659:    }
  1, 43,                                     // 82664:    case 0x1: {
  OPC_SwitchField, 10, 2,                    // 82666:     switch Inst[11:10] {
  0, 11,                                     // 82669:     case 0x0: {
  OPC_CheckPredicate, 75,                    // 82671:      check predicate 75
  OPC_CheckField, 12, 9, 0,                  // 82673:      check Inst[20:12] == 0x0
  OPC_Decode, 248, 59, 242, 2,               // 82677:      decode to STZGM using decoder 370
                                             // 82677:     }
  1, 7,                                      // 82682:     case 0x1: {
  OPC_CheckPredicate, 75,                    // 82684:      check predicate 75
  OPC_Decode, 187, 58, 154, 5,               // 82686:      decode to STGPostIndex using decoder 666
                                             // 82686:     }
  2, 7,                                      // 82691:     case 0x2: {
  OPC_CheckPredicate, 75,                    // 82693:      check predicate 75
  OPC_Decode, 191, 58, 155, 5,               // 82695:      decode to STGi using decoder 667
                                             // 82695:     }
  3, 0,                                      // 82700:     case 0x3: {
  OPC_CheckPredicate, 75,                    // 82702:      check predicate 75
  OPC_Decode, 190, 58, 154, 5,               // 82704:      decode to STGPreIndex using decoder 666
                                             // 82704:     }
                                             // 82704:     } // switch Inst[11:10]
                                             // 82704:    }
  2, 57,                                     // 82709:    case 0x2: {
  OPC_SwitchField, 10, 2,                    // 82711:     switch Inst[11:10] {
  0, 7,                                      // 82714:     case 0x0: {
  OPC_CheckPredicate, 123,                   // 82716:      check predicate 123
  OPC_Decode, 167, 37, 247, 4,               // 82718:      decode to LDAPURXi using decoder 631
                                             // 82718:     }
  2, 0,                                      // 82723:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 82725:      switch Inst[15:12] {
  0, 8,                                      // 82728:      case 0x0: {
  OPC_CheckPredicate, 130, 1,                // 82730:       check predicate 130
  OPC_Decode, 196, 38, 153, 5,               // 82733:       decode to LDIAPPXpost using decoder 665
                                             // 82733:      }
  1, 8,                                      // 82738:      case 0x1: {
  OPC_CheckPredicate, 130, 1,                // 82740:       check predicate 130
  OPC_Decode, 195, 38, 255, 4,               // 82743:       decode to LDIAPPX using decoder 639
                                             // 82743:      }
  5, 8,                                      // 82748:      case 0x5: {
  OPC_CheckPredicate, 132, 1,                // 82750:       check predicate 132
  OPC_Decode, 174, 37, 255, 4,               // 82753:       decode to LDAPi using decoder 639
                                             // 82753:      }
  7, 0,                                      // 82758:      case 0x7: {
  OPC_CheckPredicate, 132, 1,                // 82760:       check predicate 132
  OPC_Decode, 153, 37, 255, 4,               // 82763:       decode to LDAPPi using decoder 639
                                             // 82763:      }
                                             // 82763:      } // switch Inst[15:12]
                                             // 82763:     }
                                             // 82763:     } // switch Inst[11:10]
                                             // 82763:    }
  3, 39,                                     // 82768:    case 0x3: {
  OPC_SwitchField, 10, 2,                    // 82770:     switch Inst[11:10] {
  0, 7,                                      // 82773:     case 0x0: {
  OPC_CheckPredicate, 75,                    // 82775:      check predicate 75
  OPC_Decode, 191, 38, 156, 5,               // 82777:      decode to LDG using decoder 668
                                             // 82777:     }
  1, 7,                                      // 82782:     case 0x1: {
  OPC_CheckPredicate, 75,                    // 82784:      check predicate 75
  OPC_Decode, 249, 59, 154, 5,               // 82786:      decode to STZGPostIndex using decoder 666
                                             // 82786:     }
  2, 7,                                      // 82791:     case 0x2: {
  OPC_CheckPredicate, 75,                    // 82793:      check predicate 75
  OPC_Decode, 251, 59, 155, 5,               // 82795:      decode to STZGi using decoder 667
                                             // 82795:     }
  3, 0,                                      // 82800:     case 0x3: {
  OPC_CheckPredicate, 75,                    // 82802:      check predicate 75
  OPC_Decode, 250, 59, 154, 5,               // 82804:      decode to STZGPreIndex using decoder 666
                                             // 82804:     }
                                             // 82804:     } // switch Inst[11:10]
                                             // 82804:    }
  4, 12,                                     // 82809:    case 0x4: {
  OPC_CheckPredicate, 130, 1,                // 82811:     check predicate 130
  OPC_CheckField, 10, 11, 2,                 // 82814:     check Inst[20:10] == 0x2
  OPC_Decode, 207, 58, 157, 5,               // 82818:     decode to STLRXpre using decoder 669
                                             // 82818:    }
  5, 43,                                     // 82823:    case 0x5: {
  OPC_SwitchField, 10, 2,                    // 82825:     switch Inst[11:10] {
  0, 11,                                     // 82828:     case 0x0: {
  OPC_CheckPredicate, 75,                    // 82830:      check predicate 75
  OPC_CheckField, 12, 9, 0,                  // 82832:      check Inst[20:12] == 0x0
  OPC_Decode, 185, 58, 242, 2,               // 82836:      decode to STGM using decoder 370
                                             // 82836:     }
  1, 7,                                      // 82841:     case 0x1: {
  OPC_CheckPredicate, 75,                    // 82843:      check predicate 75
  OPC_Decode, 174, 57, 154, 5,               // 82845:      decode to ST2GPostIndex using decoder 666
                                             // 82845:     }
  2, 7,                                      // 82850:     case 0x2: {
  OPC_CheckPredicate, 75,                    // 82852:      check predicate 75
  OPC_Decode, 176, 57, 155, 5,               // 82854:      decode to ST2Gi using decoder 667
                                             // 82854:     }
  3, 0,                                      // 82859:     case 0x3: {
  OPC_CheckPredicate, 75,                    // 82861:      check predicate 75
  OPC_Decode, 175, 57, 154, 5,               // 82863:      decode to ST2GPreIndex using decoder 666
                                             // 82863:     }
                                             // 82863:     } // switch Inst[11:10]
                                             // 82863:    }
  6, 12,                                     // 82868:    case 0x6: {
  OPC_CheckPredicate, 130, 1,                // 82870:     check predicate 130
  OPC_CheckField, 10, 11, 2,                 // 82873:     check Inst[20:10] == 0x2
  OPC_Decode, 159, 37, 157, 5,               // 82877:     decode to LDAPRXpost using decoder 669
                                             // 82877:    }
  7, 0,                                      // 82882:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 82884:     switch Inst[11:10] {
  0, 11,                                     // 82887:     case 0x0: {
  OPC_CheckPredicate, 75,                    // 82889:      check predicate 75
  OPC_CheckField, 12, 9, 0,                  // 82891:      check Inst[20:12] == 0x0
  OPC_Decode, 192, 38, 242, 2,               // 82895:      decode to LDGM using decoder 370
                                             // 82895:     }
  1, 7,                                      // 82900:     case 0x1: {
  OPC_CheckPredicate, 75,                    // 82902:      check predicate 75
  OPC_Decode, 245, 59, 154, 5,               // 82904:      decode to STZ2GPostIndex using decoder 666
                                             // 82904:     }
  2, 7,                                      // 82909:     case 0x2: {
  OPC_CheckPredicate, 75,                    // 82911:      check predicate 75
  OPC_Decode, 247, 59, 155, 5,               // 82913:      decode to STZ2Gi using decoder 667
                                             // 82913:     }
  3, 0,                                      // 82918:     case 0x3: {
  OPC_CheckPredicate, 75,                    // 82920:      check predicate 75
  OPC_Decode, 246, 59, 154, 5,               // 82922:      decode to STZ2GPreIndex using decoder 666
                                             // 82922:     }
                                             // 82922:     } // switch Inst[11:10]
                                             // 82922:    }
                                             // 82922:    } // switch Inst[23:21]
                                             // 82922:   }
  2, 0,                                      // 82927:   case 0x2: {
  OPC_SwitchField, 21, 3,                    // 82929:    switch Inst[23:21] {
  0, 24,                                     // 82932:    case 0x0: {
  OPC_SwitchField, 13, 3,                    // 82934:     switch Inst[15:13] {
  0, 9,                                      // 82937:     case 0x0: {
  OPC_CheckField, 10, 3, 0,                  // 82939:      check Inst[12:10] == 0x0
  OPC_Decode, 234, 46, 139, 5,               // 82943:      decode to SBCXr using decoder 651
                                             // 82943:     }
  1, 0,                                      // 82948:     case 0x1: {
  OPC_CheckPredicate, 131, 1,                // 82950:      check predicate 131
  OPC_Decode, 139, 60, 140, 5,               // 82953:      decode to SUBPT_shift using decoder 652
                                             // 82953:     }
                                             // 82953:     } // switch Inst[15:13]
                                             // 82953:    }
  4, 17,                                     // 82958:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 82960:     switch Inst[11:10] {
  0, 5,                                      // 82963:     case 0x0: {
  OPC_Decode, 172, 20, 141, 5,               // 82965:      decode to CSINVXr using decoder 653
                                             // 82965:     }
  1, 0,                                      // 82970:     case 0x1: {
  OPC_Decode, 174, 20, 141, 5,               // 82972:      decode to CSNEGXr using decoder 653
                                             // 82972:     }
                                             // 82972:     } // switch Inst[11:10]
                                             // 82972:    }
  6, 0,                                      // 82977:    case 0x6: {
  OPC_SwitchField, 10, 11,                   // 82979:     switch Inst[20:10] {
  0, 5,                                      // 82982:     case 0x0: {
  OPC_Decode, 231, 44, 158, 5,               // 82984:      decode to RBITXr using decoder 670
                                             // 82984:     }
  1, 5,                                      // 82989:     case 0x1: {
  OPC_Decode, 191, 45, 158, 5,               // 82991:      decode to REV16Xr using decoder 670
                                             // 82991:     }
  2, 5,                                      // 82996:     case 0x2: {
  OPC_Decode, 194, 45, 158, 5,               // 82998:      decode to REV32Xr using decoder 670
                                             // 82998:     }
  3, 5,                                      // 83003:     case 0x3: {
  OPC_Decode, 220, 45, 158, 5,               // 83005:      decode to REVXr using decoder 670
                                             // 83005:     }
  4, 5,                                      // 83010:     case 0x4: {
  OPC_Decode, 192, 17, 158, 5,               // 83012:      decode to CLZXr using decoder 670
                                             // 83012:     }
  5, 5,                                      // 83017:     case 0x5: {
  OPC_Decode, 176, 17, 158, 5,               // 83019:      decode to CLSXr using decoder 670
                                             // 83019:     }
  6, 7,                                      // 83024:     case 0x6: {
  OPC_CheckPredicate, 103,                   // 83026:      check predicate 103
  OPC_Decode, 180, 20, 158, 5,               // 83028:      decode to CTZXr using decoder 670
                                             // 83028:     }
  7, 7,                                      // 83033:     case 0x7: {
  OPC_CheckPredicate, 103,                   // 83035:      check predicate 103
  OPC_Decode, 160, 19, 158, 5,               // 83037:      decode to CNTXr using decoder 670
                                             // 83037:     }
  8, 7,                                      // 83042:     case 0x8: {
  OPC_CheckPredicate, 103,                   // 83044:      check predicate 103
  OPC_Decode, 237, 12, 158, 5,               // 83046:      decode to ABSXr using decoder 670
                                             // 83046:     }
  64, 7,                                     // 83051:     case 0x40: {
  OPC_CheckPredicate, 122,                   // 83053:      check predicate 122
  OPC_Decode, 230, 43, 159, 5,               // 83055:      decode to PACIA using decoder 671
                                             // 83055:     }
  65, 7,                                     // 83060:     case 0x41: {
  OPC_CheckPredicate, 122,                   // 83062:      check predicate 122
  OPC_Decode, 236, 43, 159, 5,               // 83064:      decode to PACIB using decoder 671
                                             // 83064:     }
  66, 7,                                     // 83069:     case 0x42: {
  OPC_CheckPredicate, 122,                   // 83071:      check predicate 122
  OPC_Decode, 225, 43, 159, 5,               // 83073:      decode to PACDA using decoder 671
                                             // 83073:     }
  67, 7,                                     // 83078:     case 0x43: {
  OPC_CheckPredicate, 122,                   // 83080:      check predicate 122
  OPC_Decode, 226, 43, 159, 5,               // 83082:      decode to PACDB using decoder 671
                                             // 83082:     }
  68, 7,                                     // 83087:     case 0x44: {
  OPC_CheckPredicate, 122,                   // 83089:      check predicate 122
  OPC_Decode, 206, 14, 159, 5,               // 83091:      decode to AUTIA using decoder 671
                                             // 83091:     }
  69, 7,                                     // 83096:     case 0x45: {
  OPC_CheckPredicate, 122,                   // 83098:      check predicate 122
  OPC_Decode, 213, 14, 159, 5,               // 83100:      decode to AUTIB using decoder 671
                                             // 83100:     }
  70, 7,                                     // 83105:     case 0x46: {
  OPC_CheckPredicate, 122,                   // 83107:      check predicate 122
  OPC_Decode, 202, 14, 159, 5,               // 83109:      decode to AUTDA using decoder 671
                                             // 83109:     }
  71, 7,                                     // 83114:     case 0x47: {
  OPC_CheckPredicate, 122,                   // 83116:      check predicate 122
  OPC_Decode, 203, 14, 159, 5,               // 83118:      decode to AUTDB using decoder 671
                                             // 83118:     }
  72, 11,                                    // 83123:     case 0x48: {
  OPC_CheckPredicate, 122,                   // 83125:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83127:      check Inst[9:5] == 0x1f
  OPC_Decode, 242, 43, 233, 4,               // 83131:      decode to PACIZA using decoder 617
                                             // 83131:     }
  73, 11,                                    // 83136:     case 0x49: {
  OPC_CheckPredicate, 122,                   // 83138:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83140:      check Inst[9:5] == 0x1f
  OPC_Decode, 243, 43, 233, 4,               // 83144:      decode to PACIZB using decoder 617
                                             // 83144:     }
  74, 11,                                    // 83149:     case 0x4a: {
  OPC_CheckPredicate, 122,                   // 83151:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83153:      check Inst[9:5] == 0x1f
  OPC_Decode, 227, 43, 233, 4,               // 83157:      decode to PACDZA using decoder 617
                                             // 83157:     }
  75, 11,                                    // 83162:     case 0x4b: {
  OPC_CheckPredicate, 122,                   // 83164:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83166:      check Inst[9:5] == 0x1f
  OPC_Decode, 228, 43, 233, 4,               // 83170:      decode to PACDZB using decoder 617
                                             // 83170:     }
  76, 11,                                    // 83175:     case 0x4c: {
  OPC_CheckPredicate, 122,                   // 83177:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83179:      check Inst[9:5] == 0x1f
  OPC_Decode, 220, 14, 233, 4,               // 83183:      decode to AUTIZA using decoder 617
                                             // 83183:     }
  77, 11,                                    // 83188:     case 0x4d: {
  OPC_CheckPredicate, 122,                   // 83190:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83192:      check Inst[9:5] == 0x1f
  OPC_Decode, 221, 14, 233, 4,               // 83196:      decode to AUTIZB using decoder 617
                                             // 83196:     }
  78, 11,                                    // 83201:     case 0x4e: {
  OPC_CheckPredicate, 122,                   // 83203:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83205:      check Inst[9:5] == 0x1f
  OPC_Decode, 204, 14, 233, 4,               // 83209:      decode to AUTDZA using decoder 617
                                             // 83209:     }
  79, 11,                                    // 83214:     case 0x4f: {
  OPC_CheckPredicate, 122,                   // 83216:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83218:      check Inst[9:5] == 0x1f
  OPC_Decode, 205, 14, 233, 4,               // 83222:      decode to AUTDZB using decoder 617
                                             // 83222:     }
  80, 11,                                    // 83227:     case 0x50: {
  OPC_CheckPredicate, 122,                   // 83229:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83231:      check Inst[9:5] == 0x1f
  OPC_Decode, 235, 70, 233, 4,               // 83235:      decode to XPACI using decoder 617
                                             // 83235:     }
  81, 11,                                    // 83240:     case 0x51: {
  OPC_CheckPredicate, 122,                   // 83242:      check predicate 122
  OPC_CheckField, 5, 5, 31,                  // 83244:      check Inst[9:5] == 0x1f
  OPC_Decode, 234, 70, 233, 4,               // 83248:      decode to XPACD using decoder 617
                                             // 83248:     }
  96, 12,                                    // 83253:     case 0x60: {
  OPC_CheckPredicate, 104,                   // 83255:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83257:      check Inst[9:0] == 0x3fe
  OPC_Decode, 245, 43, 165, 2,               // 83262:      decode to PACNBIASPPC using decoder 293
                                             // 83262:     }
  97, 12,                                    // 83267:     case 0x61: {
  OPC_CheckPredicate, 104,                   // 83269:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83271:      check Inst[9:0] == 0x3fe
  OPC_Decode, 246, 43, 165, 2,               // 83276:      decode to PACNBIBSPPC using decoder 293
                                             // 83276:     }
  98, 12,                                    // 83281:     case 0x62: {
  OPC_CheckPredicate, 104,                   // 83283:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83285:      check Inst[9:0] == 0x3fe
  OPC_Decode, 232, 43, 165, 2,               // 83290:      decode to PACIA171615 using decoder 293
                                             // 83290:     }
  99, 12,                                    // 83295:     case 0x63: {
  OPC_CheckPredicate, 104,                   // 83297:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83299:      check Inst[9:0] == 0x3fe
  OPC_Decode, 238, 43, 165, 2,               // 83304:      decode to PACIB171615 using decoder 293
                                             // 83304:     }
  100, 11,                                   // 83309:     case 0x64: {
  OPC_CheckPredicate, 104,                   // 83311:      check predicate 104
  OPC_CheckField, 0, 5, 30,                  // 83313:      check Inst[4:0] == 0x1e
  OPC_Decode, 211, 14, 241, 4,               // 83317:      decode to AUTIASPPCr using decoder 625
                                             // 83317:     }
  101, 11,                                   // 83322:     case 0x65: {
  OPC_CheckPredicate, 104,                   // 83324:      check predicate 104
  OPC_CheckField, 0, 5, 30,                  // 83326:      check Inst[4:0] == 0x1e
  OPC_Decode, 218, 14, 241, 4,               // 83330:      decode to AUTIBSPPCr using decoder 625
                                             // 83330:     }
  104, 12,                                   // 83335:     case 0x68: {
  OPC_CheckPredicate, 104,                   // 83337:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83339:      check Inst[9:0] == 0x3fe
  OPC_Decode, 234, 43, 165, 2,               // 83344:      decode to PACIASPPC using decoder 293
                                             // 83344:     }
  105, 12,                                   // 83349:     case 0x69: {
  OPC_CheckPredicate, 104,                   // 83351:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83353:      check Inst[9:0] == 0x3fe
  OPC_Decode, 240, 43, 165, 2,               // 83358:      decode to PACIBSPPC using decoder 293
                                             // 83358:     }
  110, 12,                                   // 83363:     case 0x6e: {
  OPC_CheckPredicate, 104,                   // 83365:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83367:      check Inst[9:0] == 0x3fe
  OPC_Decode, 208, 14, 165, 2,               // 83372:      decode to AUTIA171615 using decoder 293
                                             // 83372:     }
  111, 0,                                    // 83377:     case 0x6f: {
  OPC_CheckPredicate, 104,                   // 83379:      check predicate 104
  OPC_CheckField, 0, 10, 254, 7,             // 83381:      check Inst[9:0] == 0x3fe
  OPC_Decode, 215, 14, 165, 2,               // 83386:      decode to AUTIB171615 using decoder 293
                                             // 83386:     }
                                             // 83386:     } // switch Inst[20:10]
                                             // 83386:    }
                                             // 83386:    } // switch Inst[23:21]
                                             // 83386:   }
                                             // 83386:   } // switch Inst[25:24]
                                             // 83386:  }
  7, 0,                                      // 83391:  case 0x7: {
  OPC_SwitchField, 23, 3,                    // 83393:   switch Inst[25:23] {
  0, 239, 2,                                 // 83396:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 83399:    switch Inst[11:10] {
  0, 238, 1,                                 // 83402:    case 0x0: {
  OPC_SwitchField, 21, 2,                    // 83405:     switch Inst[22:21] {
  0, 5,                                      // 83408:     case 0x0: {
  OPC_Decode, 238, 59, 247, 4,               // 83410:      decode to STURXi using decoder 631
                                             // 83410:     }
  1, 132, 1,                                 // 83415:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 83418:      switch Inst[15:12] {
  0, 7,                                      // 83421:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 83423:       check predicate 72
  OPC_Decode, 151, 37, 255, 4,               // 83425:       decode to LDADDX using decoder 639
                                             // 83425:      }
  1, 7,                                      // 83430:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 83432:       check predicate 72
  OPC_Decode, 226, 37, 255, 4,               // 83434:       decode to LDCLRX using decoder 639
                                             // 83434:      }
  2, 7,                                      // 83439:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 83441:       check predicate 72
  OPC_Decode, 242, 37, 255, 4,               // 83443:       decode to LDEORX using decoder 639
                                             // 83443:      }
  3, 7,                                      // 83448:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 83450:       check predicate 72
  OPC_Decode, 139, 40, 255, 4,               // 83452:       decode to LDSETX using decoder 639
                                             // 83452:      }
  4, 7,                                      // 83457:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 83459:       check predicate 72
  OPC_Decode, 155, 40, 255, 4,               // 83461:       decode to LDSMAXX using decoder 639
                                             // 83461:      }
  5, 7,                                      // 83466:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 83468:       check predicate 72
  OPC_Decode, 171, 40, 255, 4,               // 83470:       decode to LDSMINX using decoder 639
                                             // 83470:      }
  6, 7,                                      // 83475:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 83477:       check predicate 72
  OPC_Decode, 230, 40, 255, 4,               // 83479:       decode to LDUMAXX using decoder 639
                                             // 83479:      }
  7, 7,                                      // 83484:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 83486:       check predicate 72
  OPC_Decode, 246, 40, 255, 4,               // 83488:       decode to LDUMINX using decoder 639
                                             // 83488:      }
  8, 7,                                      // 83493:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 83495:       check predicate 72
  OPC_Decode, 164, 61, 255, 4,               // 83497:       decode to SWPX using decoder 639
                                             // 83497:      }
  9, 12,                                     // 83502:      case 0x9: {
  OPC_CheckPredicate, 133, 1,                // 83504:       check predicate 133
  OPC_CheckField, 16, 5, 31,                 // 83507:       check Inst[20:16] == 0x1f
  OPC_Decode, 141, 58, 160, 5,               // 83511:       decode to ST64B using decoder 672
                                             // 83511:      }
  10, 8,                                     // 83516:      case 0xa: {
  OPC_CheckPredicate, 133, 1,                // 83518:       check predicate 133
  OPC_Decode, 143, 58, 161, 5,               // 83521:       decode to ST64BV0 using decoder 673
                                             // 83521:      }
  11, 8,                                     // 83526:      case 0xb: {
  OPC_CheckPredicate, 133, 1,                // 83528:       check predicate 133
  OPC_Decode, 142, 58, 161, 5,               // 83531:       decode to ST64BV using decoder 673
                                             // 83531:      }
  13, 0,                                     // 83536:      case 0xd: {
  OPC_CheckPredicate, 133, 1,                // 83538:       check predicate 133
  OPC_CheckField, 16, 5, 31,                 // 83541:       check Inst[20:16] == 0x1f
  OPC_Decode, 135, 37, 160, 5,               // 83545:       decode to LD64B using decoder 672
                                             // 83545:      }
                                             // 83545:      } // switch Inst[15:12]
                                             // 83545:     }
  2, 5,                                      // 83550:     case 0x2: {
  OPC_Decode, 132, 41, 247, 4,               // 83552:      decode to LDURXi using decoder 631
                                             // 83552:     }
  3, 0,                                      // 83557:     case 0x3: {
  OPC_SwitchField, 12, 4,                    // 83559:      switch Inst[15:12] {
  0, 7,                                      // 83562:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 83564:       check predicate 72
  OPC_Decode, 149, 37, 255, 4,               // 83566:       decode to LDADDLX using decoder 639
                                             // 83566:      }
  1, 7,                                      // 83571:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 83573:       check predicate 72
  OPC_Decode, 220, 37, 255, 4,               // 83575:       decode to LDCLRLX using decoder 639
                                             // 83575:      }
  2, 7,                                      // 83580:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 83582:       check predicate 72
  OPC_Decode, 240, 37, 255, 4,               // 83584:       decode to LDEORLX using decoder 639
                                             // 83584:      }
  3, 7,                                      // 83589:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 83591:       check predicate 72
  OPC_Decode, 133, 40, 255, 4,               // 83593:       decode to LDSETLX using decoder 639
                                             // 83593:      }
  4, 7,                                      // 83598:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 83600:       check predicate 72
  OPC_Decode, 153, 40, 255, 4,               // 83602:       decode to LDSMAXLX using decoder 639
                                             // 83602:      }
  5, 7,                                      // 83607:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 83609:       check predicate 72
  OPC_Decode, 169, 40, 255, 4,               // 83611:       decode to LDSMINLX using decoder 639
                                             // 83611:      }
  6, 7,                                      // 83616:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 83618:       check predicate 72
  OPC_Decode, 228, 40, 255, 4,               // 83620:       decode to LDUMAXLX using decoder 639
                                             // 83620:      }
  7, 7,                                      // 83625:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 83627:       check predicate 72
  OPC_Decode, 244, 40, 255, 4,               // 83629:       decode to LDUMINLX using decoder 639
                                             // 83629:      }
  8, 0,                                      // 83634:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 83636:       check predicate 72
  OPC_Decode, 150, 61, 255, 4,               // 83638:       decode to SWPLX using decoder 639
                                             // 83638:      }
                                             // 83638:      } // switch Inst[15:12]
                                             // 83638:     }
                                             // 83638:     } // switch Inst[22:21]
                                             // 83638:    }
  1, 31,                                     // 83643:    case 0x1: {
  OPC_SwitchField, 21, 1,                    // 83645:     switch Inst[21] {
  0, 17,                                     // 83648:     case 0x0: {
  OPC_SwitchField, 22, 1,                    // 83650:      switch Inst[22] {
  0, 5,                                      // 83653:      case 0x0: {
  OPC_Decode, 206, 59, 247, 4,               // 83655:       decode to STRXpost using decoder 631
                                             // 83655:      }
  1, 0,                                      // 83660:      case 0x1: {
  OPC_Decode, 239, 39, 247, 4,               // 83662:       decode to LDRXpost using decoder 631
                                             // 83662:      }
                                             // 83662:      } // switch Inst[22]
                                             // 83662:     }
  1, 0,                                      // 83667:     case 0x1: {
  OPC_CheckPredicate, 122,                   // 83669:      check predicate 122
  OPC_Decode, 164, 39, 162, 5,               // 83671:      decode to LDRAAindexed using decoder 674
                                             // 83671:     }
                                             // 83671:     } // switch Inst[21]
                                             // 83671:    }
  2, 55,                                     // 83676:    case 0x2: {
  OPC_SwitchField, 21, 2,                    // 83678:     switch Inst[22:21] {
  0, 5,                                      // 83681:     case 0x0: {
  OPC_Decode, 227, 59, 247, 4,               // 83683:      decode to STTRXi using decoder 631
                                             // 83683:     }
  1, 17,                                     // 83688:     case 0x1: {
  OPC_SwitchField, 13, 2,                    // 83690:      switch Inst[14:13] {
  2, 5,                                      // 83693:      case 0x2: {
  OPC_Decode, 208, 59, 130, 5,               // 83695:       decode to STRXroW using decoder 642
                                             // 83695:      }
  3, 0,                                      // 83700:      case 0x3: {
  OPC_Decode, 209, 59, 131, 5,               // 83702:       decode to STRXroX using decoder 643
                                             // 83702:      }
                                             // 83702:      } // switch Inst[14:13]
                                             // 83702:     }
  2, 5,                                      // 83707:     case 0x2: {
  OPC_Decode, 204, 40, 247, 4,               // 83709:      decode to LDTRXi using decoder 631
                                             // 83709:     }
  3, 0,                                      // 83714:     case 0x3: {
  OPC_SwitchField, 13, 2,                    // 83716:      switch Inst[14:13] {
  2, 5,                                      // 83719:      case 0x2: {
  OPC_Decode, 241, 39, 130, 5,               // 83721:       decode to LDRXroW using decoder 642
                                             // 83721:      }
  3, 0,                                      // 83726:      case 0x3: {
  OPC_Decode, 242, 39, 131, 5,               // 83728:       decode to LDRXroX using decoder 643
                                             // 83728:      }
                                             // 83728:      } // switch Inst[14:13]
                                             // 83728:     }
                                             // 83728:     } // switch Inst[22:21]
                                             // 83728:    }
  3, 0,                                      // 83733:    case 0x3: {
  OPC_SwitchField, 21, 1,                    // 83735:     switch Inst[21] {
  0, 17,                                     // 83738:     case 0x0: {
  OPC_SwitchField, 22, 1,                    // 83740:      switch Inst[22] {
  0, 5,                                      // 83743:      case 0x0: {
  OPC_Decode, 207, 59, 247, 4,               // 83745:       decode to STRXpre using decoder 631
                                             // 83745:      }
  1, 0,                                      // 83750:      case 0x1: {
  OPC_Decode, 240, 39, 247, 4,               // 83752:       decode to LDRXpre using decoder 631
                                             // 83752:      }
                                             // 83752:      } // switch Inst[22]
                                             // 83752:     }
  1, 0,                                      // 83757:     case 0x1: {
  OPC_CheckPredicate, 122,                   // 83759:      check predicate 122
  OPC_Decode, 165, 39, 162, 5,               // 83761:      decode to LDRAAwriteback using decoder 674
                                             // 83761:     }
                                             // 83761:     } // switch Inst[21]
                                             // 83761:    }
                                             // 83761:    } // switch Inst[11:10]
                                             // 83761:   }
  1, 255, 1,                                 // 83766:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 83769:    switch Inst[11:10] {
  0, 196, 1,                                 // 83772:    case 0x0: {
  OPC_SwitchField, 21, 2,                    // 83775:     switch Inst[22:21] {
  0, 5,                                      // 83778:     case 0x0: {
  OPC_Decode, 187, 44, 247, 4,               // 83780:      decode to PRFUMi using decoder 631
                                             // 83780:     }
  1, 98,                                     // 83785:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 83787:      switch Inst[15:12] {
  0, 7,                                      // 83790:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 83792:       check predicate 72
  OPC_Decode, 143, 37, 255, 4,               // 83794:       decode to LDADDAX using decoder 639
                                             // 83794:      }
  1, 7,                                      // 83799:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 83801:       check predicate 72
  OPC_Decode, 214, 37, 255, 4,               // 83803:       decode to LDCLRAX using decoder 639
                                             // 83803:      }
  2, 7,                                      // 83808:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 83810:       check predicate 72
  OPC_Decode, 234, 37, 255, 4,               // 83812:       decode to LDEORAX using decoder 639
                                             // 83812:      }
  3, 7,                                      // 83817:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 83819:       check predicate 72
  OPC_Decode, 255, 39, 255, 4,               // 83821:       decode to LDSETAX using decoder 639
                                             // 83821:      }
  4, 7,                                      // 83826:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 83828:       check predicate 72
  OPC_Decode, 147, 40, 255, 4,               // 83830:       decode to LDSMAXAX using decoder 639
                                             // 83830:      }
  5, 7,                                      // 83835:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 83837:       check predicate 72
  OPC_Decode, 163, 40, 255, 4,               // 83839:       decode to LDSMINAX using decoder 639
                                             // 83839:      }
  6, 7,                                      // 83844:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 83846:       check predicate 72
  OPC_Decode, 222, 40, 255, 4,               // 83848:       decode to LDUMAXAX using decoder 639
                                             // 83848:      }
  7, 7,                                      // 83853:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 83855:       check predicate 72
  OPC_Decode, 238, 40, 255, 4,               // 83857:       decode to LDUMINAX using decoder 639
                                             // 83857:      }
  8, 7,                                      // 83862:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 83864:       check predicate 72
  OPC_Decode, 144, 61, 255, 4,               // 83866:       decode to SWPAX using decoder 639
                                             // 83866:      }
  12, 0,                                     // 83871:      case 0xc: {
  OPC_CheckPredicate, 129, 1,                // 83873:       check predicate 129
  OPC_CheckField, 16, 5, 31,                 // 83876:       check Inst[20:16] == 0x1f
  OPC_Decode, 158, 37, 242, 2,               // 83880:       decode to LDAPRX using decoder 370
                                             // 83880:      }
                                             // 83880:      } // switch Inst[15:12]
                                             // 83880:     }
  3, 0,                                      // 83885:     case 0x3: {
  OPC_SwitchField, 12, 4,                    // 83887:      switch Inst[15:12] {
  0, 7,                                      // 83890:      case 0x0: {
  OPC_CheckPredicate, 72,                    // 83892:       check predicate 72
  OPC_Decode, 141, 37, 255, 4,               // 83894:       decode to LDADDALX using decoder 639
                                             // 83894:      }
  1, 7,                                      // 83899:      case 0x1: {
  OPC_CheckPredicate, 72,                    // 83901:       check predicate 72
  OPC_Decode, 212, 37, 255, 4,               // 83903:       decode to LDCLRALX using decoder 639
                                             // 83903:      }
  2, 7,                                      // 83908:      case 0x2: {
  OPC_CheckPredicate, 72,                    // 83910:       check predicate 72
  OPC_Decode, 232, 37, 255, 4,               // 83912:       decode to LDEORALX using decoder 639
                                             // 83912:      }
  3, 7,                                      // 83917:      case 0x3: {
  OPC_CheckPredicate, 72,                    // 83919:       check predicate 72
  OPC_Decode, 253, 39, 255, 4,               // 83921:       decode to LDSETALX using decoder 639
                                             // 83921:      }
  4, 7,                                      // 83926:      case 0x4: {
  OPC_CheckPredicate, 72,                    // 83928:       check predicate 72
  OPC_Decode, 145, 40, 255, 4,               // 83930:       decode to LDSMAXALX using decoder 639
                                             // 83930:      }
  5, 7,                                      // 83935:      case 0x5: {
  OPC_CheckPredicate, 72,                    // 83937:       check predicate 72
  OPC_Decode, 161, 40, 255, 4,               // 83939:       decode to LDSMINALX using decoder 639
                                             // 83939:      }
  6, 7,                                      // 83944:      case 0x6: {
  OPC_CheckPredicate, 72,                    // 83946:       check predicate 72
  OPC_Decode, 220, 40, 255, 4,               // 83948:       decode to LDUMAXALX using decoder 639
                                             // 83948:      }
  7, 7,                                      // 83953:      case 0x7: {
  OPC_CheckPredicate, 72,                    // 83955:       check predicate 72
  OPC_Decode, 236, 40, 255, 4,               // 83957:       decode to LDUMINALX using decoder 639
                                             // 83957:      }
  8, 0,                                      // 83962:      case 0x8: {
  OPC_CheckPredicate, 72,                    // 83964:       check predicate 72
  OPC_Decode, 142, 61, 255, 4,               // 83966:       decode to SWPALX using decoder 639
                                             // 83966:      }
                                             // 83966:      } // switch Inst[15:12]
                                             // 83966:     }
                                             // 83966:     } // switch Inst[22:21]
                                             // 83966:    }
  1, 11,                                     // 83971:    case 0x1: {
  OPC_CheckPredicate, 122,                   // 83973:     check predicate 122
  OPC_CheckField, 21, 1, 1,                  // 83975:     check Inst[21] == 0x1
  OPC_Decode, 166, 39, 162, 5,               // 83979:     decode to LDRABindexed using decoder 674
                                             // 83979:    }
  2, 25,                                     // 83984:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 83986:     switch Inst[14:13] {
  2, 9,                                      // 83989:     case 0x2: {
  OPC_CheckField, 21, 2, 1,                  // 83991:      check Inst[22:21] == 0x1
  OPC_Decode, 184, 44, 163, 5,               // 83995:      decode to PRFMroW using decoder 675
                                             // 83995:     }
  3, 0,                                      // 84000:     case 0x3: {
  OPC_CheckField, 21, 2, 1,                  // 84002:      check Inst[22:21] == 0x1
  OPC_Decode, 185, 44, 163, 5,               // 84006:      decode to PRFMroX using decoder 675
                                             // 84006:     }
                                             // 84006:     } // switch Inst[14:13]
                                             // 84006:    }
  3, 0,                                      // 84011:    case 0x3: {
  OPC_CheckPredicate, 122,                   // 84013:     check predicate 122
  OPC_CheckField, 21, 1, 1,                  // 84015:     check Inst[21] == 0x1
  OPC_Decode, 167, 39, 162, 5,               // 84019:     decode to LDRABwriteback using decoder 674
                                             // 84019:    }
                                             // 84019:    } // switch Inst[11:10]
                                             // 84019:   }
  2, 17,                                     // 84024:   case 0x2: {
  OPC_SwitchField, 22, 1,                    // 84026:    switch Inst[22] {
  0, 5,                                      // 84029:    case 0x0: {
  OPC_Decode, 210, 59, 132, 5,               // 84031:     decode to STRXui using decoder 644
                                             // 84031:    }
  1, 0,                                      // 84036:    case 0x1: {
  OPC_Decode, 243, 39, 132, 5,               // 84038:     decode to LDRXui using decoder 644
                                             // 84038:    }
                                             // 84038:    } // switch Inst[22]
                                             // 84038:   }
  3, 9,                                      // 84043:   case 0x3: {
  OPC_CheckField, 22, 1, 0,                  // 84045:    check Inst[22] == 0x0
  OPC_Decode, 186, 44, 132, 5,               // 84049:    decode to PRFMui using decoder 644
                                             // 84049:   }
  4, 0,                                      // 84054:   case 0x4: {
  OPC_SwitchField, 10, 2,                    // 84056:    switch Inst[11:10] {
  0, 25,                                     // 84059:    case 0x0: {
  OPC_SwitchField, 21, 2,                    // 84061:     switch Inst[22:21] {
  0, 9,                                      // 84064:     case 0x0: {
  OPC_CheckField, 12, 4, 0,                  // 84066:      check Inst[15:12] == 0x0
  OPC_Decode, 232, 46, 139, 5,               // 84070:      decode to SBCSXr using decoder 651
                                             // 84070:     }
  2, 0,                                      // 84075:     case 0x2: {
  OPC_CheckField, 4, 1, 0,                   // 84077:      check Inst[4] == 0x0
  OPC_Decode, 143, 17, 150, 5,               // 84081:      decode to CCMPXr using decoder 662
                                             // 84081:     }
                                             // 84081:     } // switch Inst[22:21]
                                             // 84081:    }
  2, 0,                                      // 84086:    case 0x2: {
  OPC_CheckField, 21, 2, 2,                  // 84088:     check Inst[22:21] == 0x2
  OPC_CheckField, 4, 1, 0,                   // 84092:     check Inst[4] == 0x0
  OPC_Decode, 142, 17, 151, 5,               // 84096:     decode to CCMPXi using decoder 663
                                             // 84096:    }
                                             // 84096:    } // switch Inst[11:10]
                                             // 84096:   }
                                             // 84096:   } // switch Inst[25:23]
                                             // 84096:  }
                                             // 84096:  } // switch Inst[31:29]
                                             // 84096: }
  7, 0,                                      // 84101: case 0x7: {
  OPC_SwitchField, 29, 3,                    // 84103:  switch Inst[31:29] {
  0, 211, 21,                                // 84106:  case 0x0: {
  OPC_SwitchField, 24, 2,                    // 84109:   switch Inst[25:24] {
  0, 7,                                      // 84112:   case 0x0: {
  OPC_CheckPredicate, 77,                    // 84114:    check predicate 77
  OPC_Decode, 226, 39, 164, 5,               // 84116:    decode to LDRSl using decoder 676
                                             // 84116:   }
  1, 135, 6,                                 // 84121:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 84124:    switch Inst[23:21] {
  0, 162, 1,                                 // 84127:    case 0x0: {
  OPC_SwitchField, 10, 2,                    // 84130:     switch Inst[11:10] {
  1, 147, 1,                                 // 84133:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 84136:      switch Inst[15:12] {
  0, 7,                                      // 84139:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 84141:       check predicate 124
  OPC_Decode, 255, 19, 248, 4,               // 84143:       decode to CPYP using decoder 632
                                             // 84143:      }
  1, 7,                                      // 84148:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 84150:       check predicate 124
  OPC_Decode, 139, 20, 248, 4,               // 84152:       decode to CPYPWT using decoder 632
                                             // 84152:      }
  2, 7,                                      // 84157:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 84159:       check predicate 124
  OPC_Decode, 130, 20, 248, 4,               // 84161:       decode to CPYPRT using decoder 632
                                             // 84161:      }
  3, 7,                                      // 84166:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 84168:       check predicate 124
  OPC_Decode, 134, 20, 248, 4,               // 84170:       decode to CPYPT using decoder 632
                                             // 84170:      }
  4, 7,                                      // 84175:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 84177:       check predicate 124
  OPC_Decode, 138, 20, 248, 4,               // 84179:       decode to CPYPWN using decoder 632
                                             // 84179:      }
  5, 7,                                      // 84184:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 84186:       check predicate 124
  OPC_Decode, 142, 20, 248, 4,               // 84188:       decode to CPYPWTWN using decoder 632
                                             // 84188:      }
  6, 7,                                      // 84193:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 84195:       check predicate 124
  OPC_Decode, 133, 20, 248, 4,               // 84197:       decode to CPYPRTWN using decoder 632
                                             // 84197:      }
  7, 7,                                      // 84202:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 84204:       check predicate 124
  OPC_Decode, 137, 20, 248, 4,               // 84206:       decode to CPYPTWN using decoder 632
                                             // 84206:      }
  8, 7,                                      // 84211:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 84213:       check predicate 124
  OPC_Decode, 129, 20, 248, 4,               // 84215:       decode to CPYPRN using decoder 632
                                             // 84215:      }
  9, 7,                                      // 84220:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 84222:       check predicate 124
  OPC_Decode, 141, 20, 248, 4,               // 84224:       decode to CPYPWTRN using decoder 632
                                             // 84224:      }
  10, 7,                                     // 84229:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 84231:       check predicate 124
  OPC_Decode, 132, 20, 248, 4,               // 84233:       decode to CPYPRTRN using decoder 632
                                             // 84233:      }
  11, 7,                                     // 84238:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 84240:       check predicate 124
  OPC_Decode, 136, 20, 248, 4,               // 84242:       decode to CPYPTRN using decoder 632
                                             // 84242:      }
  12, 7,                                     // 84247:      case 0xc: {
  OPC_CheckPredicate, 124,                   // 84249:       check predicate 124
  OPC_Decode, 128, 20, 248, 4,               // 84251:       decode to CPYPN using decoder 632
                                             // 84251:      }
  13, 7,                                     // 84256:      case 0xd: {
  OPC_CheckPredicate, 124,                   // 84258:       check predicate 124
  OPC_Decode, 140, 20, 248, 4,               // 84260:       decode to CPYPWTN using decoder 632
                                             // 84260:      }
  14, 7,                                     // 84265:      case 0xe: {
  OPC_CheckPredicate, 124,                   // 84267:       check predicate 124
  OPC_Decode, 131, 20, 248, 4,               // 84269:       decode to CPYPRTN using decoder 632
                                             // 84269:      }
  15, 0,                                     // 84274:      case 0xf: {
  OPC_CheckPredicate, 124,                   // 84276:       check predicate 124
  OPC_Decode, 135, 20, 248, 4,               // 84278:       decode to CPYPTN using decoder 632
                                             // 84278:      }
                                             // 84278:      } // switch Inst[15:12]
                                             // 84278:     }
  2, 0,                                      // 84283:     case 0x2: {
  OPC_CheckPredicate, 79,                    // 84285:      check predicate 79
  OPC_Decode, 214, 58, 165, 5,               // 84287:      decode to STLURbi using decoder 677
                                             // 84287:     }
                                             // 84287:     } // switch Inst[11:10]
                                             // 84287:    }
  2, 162, 1,                                 // 84292:    case 0x2: {
  OPC_SwitchField, 10, 2,                    // 84295:     switch Inst[11:10] {
  1, 147, 1,                                 // 84298:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 84301:      switch Inst[15:12] {
  0, 7,                                      // 84304:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 84306:       check predicate 124
  OPC_Decode, 239, 19, 248, 4,               // 84308:       decode to CPYM using decoder 632
                                             // 84308:      }
  1, 7,                                      // 84313:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 84315:       check predicate 124
  OPC_Decode, 251, 19, 248, 4,               // 84317:       decode to CPYMWT using decoder 632
                                             // 84317:      }
  2, 7,                                      // 84322:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 84324:       check predicate 124
  OPC_Decode, 242, 19, 248, 4,               // 84326:       decode to CPYMRT using decoder 632
                                             // 84326:      }
  3, 7,                                      // 84331:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 84333:       check predicate 124
  OPC_Decode, 246, 19, 248, 4,               // 84335:       decode to CPYMT using decoder 632
                                             // 84335:      }
  4, 7,                                      // 84340:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 84342:       check predicate 124
  OPC_Decode, 250, 19, 248, 4,               // 84344:       decode to CPYMWN using decoder 632
                                             // 84344:      }
  5, 7,                                      // 84349:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 84351:       check predicate 124
  OPC_Decode, 254, 19, 248, 4,               // 84353:       decode to CPYMWTWN using decoder 632
                                             // 84353:      }
  6, 7,                                      // 84358:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 84360:       check predicate 124
  OPC_Decode, 245, 19, 248, 4,               // 84362:       decode to CPYMRTWN using decoder 632
                                             // 84362:      }
  7, 7,                                      // 84367:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 84369:       check predicate 124
  OPC_Decode, 249, 19, 248, 4,               // 84371:       decode to CPYMTWN using decoder 632
                                             // 84371:      }
  8, 7,                                      // 84376:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 84378:       check predicate 124
  OPC_Decode, 241, 19, 248, 4,               // 84380:       decode to CPYMRN using decoder 632
                                             // 84380:      }
  9, 7,                                      // 84385:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 84387:       check predicate 124
  OPC_Decode, 253, 19, 248, 4,               // 84389:       decode to CPYMWTRN using decoder 632
                                             // 84389:      }
  10, 7,                                     // 84394:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 84396:       check predicate 124
  OPC_Decode, 244, 19, 248, 4,               // 84398:       decode to CPYMRTRN using decoder 632
                                             // 84398:      }
  11, 7,                                     // 84403:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 84405:       check predicate 124
  OPC_Decode, 248, 19, 248, 4,               // 84407:       decode to CPYMTRN using decoder 632
                                             // 84407:      }
  12, 7,                                     // 84412:      case 0xc: {
  OPC_CheckPredicate, 124,                   // 84414:       check predicate 124
  OPC_Decode, 240, 19, 248, 4,               // 84416:       decode to CPYMN using decoder 632
                                             // 84416:      }
  13, 7,                                     // 84421:      case 0xd: {
  OPC_CheckPredicate, 124,                   // 84423:       check predicate 124
  OPC_Decode, 252, 19, 248, 4,               // 84425:       decode to CPYMWTN using decoder 632
                                             // 84425:      }
  14, 7,                                     // 84430:      case 0xe: {
  OPC_CheckPredicate, 124,                   // 84432:       check predicate 124
  OPC_Decode, 243, 19, 248, 4,               // 84434:       decode to CPYMRTN using decoder 632
                                             // 84434:      }
  15, 0,                                     // 84439:      case 0xf: {
  OPC_CheckPredicate, 124,                   // 84441:       check predicate 124
  OPC_Decode, 247, 19, 248, 4,               // 84443:       decode to CPYMTN using decoder 632
                                             // 84443:      }
                                             // 84443:      } // switch Inst[15:12]
                                             // 84443:     }
  2, 0,                                      // 84448:     case 0x2: {
  OPC_CheckPredicate, 79,                    // 84450:      check predicate 79
  OPC_Decode, 168, 37, 165, 5,               // 84452:      decode to LDAPURbi using decoder 677
                                             // 84452:     }
                                             // 84452:     } // switch Inst[11:10]
                                             // 84452:    }
  4, 162, 1,                                 // 84457:    case 0x4: {
  OPC_SwitchField, 10, 2,                    // 84460:     switch Inst[11:10] {
  1, 147, 1,                                 // 84463:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 84466:      switch Inst[15:12] {
  0, 7,                                      // 84469:      case 0x0: {
  OPC_CheckPredicate, 124,                   // 84471:       check predicate 124
  OPC_Decode, 175, 19, 248, 4,               // 84473:       decode to CPYE using decoder 632
                                             // 84473:      }
  1, 7,                                      // 84478:      case 0x1: {
  OPC_CheckPredicate, 124,                   // 84480:       check predicate 124
  OPC_Decode, 187, 19, 248, 4,               // 84482:       decode to CPYEWT using decoder 632
                                             // 84482:      }
  2, 7,                                      // 84487:      case 0x2: {
  OPC_CheckPredicate, 124,                   // 84489:       check predicate 124
  OPC_Decode, 178, 19, 248, 4,               // 84491:       decode to CPYERT using decoder 632
                                             // 84491:      }
  3, 7,                                      // 84496:      case 0x3: {
  OPC_CheckPredicate, 124,                   // 84498:       check predicate 124
  OPC_Decode, 182, 19, 248, 4,               // 84500:       decode to CPYET using decoder 632
                                             // 84500:      }
  4, 7,                                      // 84505:      case 0x4: {
  OPC_CheckPredicate, 124,                   // 84507:       check predicate 124
  OPC_Decode, 186, 19, 248, 4,               // 84509:       decode to CPYEWN using decoder 632
                                             // 84509:      }
  5, 7,                                      // 84514:      case 0x5: {
  OPC_CheckPredicate, 124,                   // 84516:       check predicate 124
  OPC_Decode, 190, 19, 248, 4,               // 84518:       decode to CPYEWTWN using decoder 632
                                             // 84518:      }
  6, 7,                                      // 84523:      case 0x6: {
  OPC_CheckPredicate, 124,                   // 84525:       check predicate 124
  OPC_Decode, 181, 19, 248, 4,               // 84527:       decode to CPYERTWN using decoder 632
                                             // 84527:      }
  7, 7,                                      // 84532:      case 0x7: {
  OPC_CheckPredicate, 124,                   // 84534:       check predicate 124
  OPC_Decode, 185, 19, 248, 4,               // 84536:       decode to CPYETWN using decoder 632
                                             // 84536:      }
  8, 7,                                      // 84541:      case 0x8: {
  OPC_CheckPredicate, 124,                   // 84543:       check predicate 124
  OPC_Decode, 177, 19, 248, 4,               // 84545:       decode to CPYERN using decoder 632
                                             // 84545:      }
  9, 7,                                      // 84550:      case 0x9: {
  OPC_CheckPredicate, 124,                   // 84552:       check predicate 124
  OPC_Decode, 189, 19, 248, 4,               // 84554:       decode to CPYEWTRN using decoder 632
                                             // 84554:      }
  10, 7,                                     // 84559:      case 0xa: {
  OPC_CheckPredicate, 124,                   // 84561:       check predicate 124
  OPC_Decode, 180, 19, 248, 4,               // 84563:       decode to CPYERTRN using decoder 632
                                             // 84563:      }
  11, 7,                                     // 84568:      case 0xb: {
  OPC_CheckPredicate, 124,                   // 84570:       check predicate 124
  OPC_Decode, 184, 19, 248, 4,               // 84572:       decode to CPYETRN using decoder 632
                                             // 84572:      }
  12, 7,                                     // 84577:      case 0xc: {
  OPC_CheckPredicate, 124,                   // 84579:       check predicate 124
  OPC_Decode, 176, 19, 248, 4,               // 84581:       decode to CPYEN using decoder 632
                                             // 84581:      }
  13, 7,                                     // 84586:      case 0xd: {
  OPC_CheckPredicate, 124,                   // 84588:       check predicate 124
  OPC_Decode, 188, 19, 248, 4,               // 84590:       decode to CPYEWTN using decoder 632
                                             // 84590:      }
  14, 7,                                     // 84595:      case 0xe: {
  OPC_CheckPredicate, 124,                   // 84597:       check predicate 124
  OPC_Decode, 179, 19, 248, 4,               // 84599:       decode to CPYERTN using decoder 632
                                             // 84599:      }
  15, 0,                                     // 84604:      case 0xf: {
  OPC_CheckPredicate, 124,                   // 84606:       check predicate 124
  OPC_Decode, 183, 19, 248, 4,               // 84608:       decode to CPYETN using decoder 632
                                             // 84608:      }
                                             // 84608:      } // switch Inst[15:12]
                                             // 84608:     }
  2, 0,                                      // 84613:     case 0x2: {
  OPC_CheckPredicate, 79,                    // 84615:      check predicate 79
  OPC_Decode, 217, 58, 166, 5,               // 84617:      decode to STLURqi using decoder 678
                                             // 84617:     }
                                             // 84617:     } // switch Inst[11:10]
                                             // 84617:    }
  6, 0,                                      // 84622:    case 0x6: {
  OPC_SwitchField, 10, 2,                    // 84624:     switch Inst[11:10] {
  0, 135, 1,                                 // 84627:     case 0x0: {
  OPC_SwitchField, 12, 9,                    // 84630:      switch Inst[20:12] {
  240, 3, 8,                                 // 84633:      case 0x1f0: {
  OPC_CheckPredicate, 134, 1,                // 84636:       check predicate 134
  OPC_Decode, 243, 47, 167, 5,               // 84639:       decode to SETGOP using decoder 679
                                             // 84639:      }
  241, 3, 8,                                 // 84644:      case 0x1f1: {
  OPC_CheckPredicate, 134, 1,                // 84647:       check predicate 134
  OPC_Decode, 245, 47, 167, 5,               // 84650:       decode to SETGOPT using decoder 679
                                             // 84650:      }
  242, 3, 8,                                 // 84655:      case 0x1f2: {
  OPC_CheckPredicate, 134, 1,                // 84658:       check predicate 134
  OPC_Decode, 244, 47, 167, 5,               // 84661:       decode to SETGOPN using decoder 679
                                             // 84661:      }
  243, 3, 8,                                 // 84666:      case 0x1f3: {
  OPC_CheckPredicate, 134, 1,                // 84669:       check predicate 134
  OPC_Decode, 246, 47, 167, 5,               // 84672:       decode to SETGOPTN using decoder 679
                                             // 84672:      }
  244, 3, 8,                                 // 84677:      case 0x1f4: {
  OPC_CheckPredicate, 134, 1,                // 84680:       check predicate 134
  OPC_Decode, 239, 47, 167, 5,               // 84683:       decode to SETGOM using decoder 679
                                             // 84683:      }
  245, 3, 8,                                 // 84688:      case 0x1f5: {
  OPC_CheckPredicate, 134, 1,                // 84691:       check predicate 134
  OPC_Decode, 241, 47, 167, 5,               // 84694:       decode to SETGOMT using decoder 679
                                             // 84694:      }
  246, 3, 8,                                 // 84699:      case 0x1f6: {
  OPC_CheckPredicate, 134, 1,                // 84702:       check predicate 134
  OPC_Decode, 240, 47, 167, 5,               // 84705:       decode to SETGOMN using decoder 679
                                             // 84705:      }
  247, 3, 8,                                 // 84710:      case 0x1f7: {
  OPC_CheckPredicate, 134, 1,                // 84713:       check predicate 134
  OPC_Decode, 242, 47, 167, 5,               // 84716:       decode to SETGOMTN using decoder 679
                                             // 84716:      }
  248, 3, 8,                                 // 84721:      case 0x1f8: {
  OPC_CheckPredicate, 134, 1,                // 84724:       check predicate 134
  OPC_Decode, 235, 47, 167, 5,               // 84727:       decode to SETGOE using decoder 679
                                             // 84727:      }
  249, 3, 8,                                 // 84732:      case 0x1f9: {
  OPC_CheckPredicate, 134, 1,                // 84735:       check predicate 134
  OPC_Decode, 237, 47, 167, 5,               // 84738:       decode to SETGOET using decoder 679
                                             // 84738:      }
  250, 3, 8,                                 // 84743:      case 0x1fa: {
  OPC_CheckPredicate, 134, 1,                // 84746:       check predicate 134
  OPC_Decode, 236, 47, 167, 5,               // 84749:       decode to SETGOEN using decoder 679
                                             // 84749:      }
  251, 3, 0,                                 // 84754:      case 0x1fb: {
  OPC_CheckPredicate, 134, 1,                // 84757:       check predicate 134
  OPC_Decode, 238, 47, 167, 5,               // 84760:       decode to SETGOETN using decoder 679
                                             // 84760:      }
                                             // 84760:      } // switch Inst[20:12]
                                             // 84760:     }
  1, 123,                                    // 84765:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 84767:      switch Inst[15:12] {
  0, 8,                                      // 84770:      case 0x0: {
  OPC_CheckPredicate, 135, 1,                // 84772:       check predicate 135
  OPC_Decode, 247, 47, 251, 4,               // 84775:       decode to SETGP using decoder 635
                                             // 84775:      }
  1, 8,                                      // 84780:      case 0x1: {
  OPC_CheckPredicate, 135, 1,                // 84782:       check predicate 135
  OPC_Decode, 249, 47, 251, 4,               // 84785:       decode to SETGPT using decoder 635
                                             // 84785:      }
  2, 8,                                      // 84790:      case 0x2: {
  OPC_CheckPredicate, 135, 1,                // 84792:       check predicate 135
  OPC_Decode, 248, 47, 251, 4,               // 84795:       decode to SETGPN using decoder 635
                                             // 84795:      }
  3, 8,                                      // 84800:      case 0x3: {
  OPC_CheckPredicate, 135, 1,                // 84802:       check predicate 135
  OPC_Decode, 250, 47, 251, 4,               // 84805:       decode to SETGPTN using decoder 635
                                             // 84805:      }
  4, 8,                                      // 84810:      case 0x4: {
  OPC_CheckPredicate, 135, 1,                // 84812:       check predicate 135
  OPC_Decode, 231, 47, 251, 4,               // 84815:       decode to SETGM using decoder 635
                                             // 84815:      }
  5, 8,                                      // 84820:      case 0x5: {
  OPC_CheckPredicate, 135, 1,                // 84822:       check predicate 135
  OPC_Decode, 233, 47, 251, 4,               // 84825:       decode to SETGMT using decoder 635
                                             // 84825:      }
  6, 8,                                      // 84830:      case 0x6: {
  OPC_CheckPredicate, 135, 1,                // 84832:       check predicate 135
  OPC_Decode, 232, 47, 251, 4,               // 84835:       decode to SETGMN using decoder 635
                                             // 84835:      }
  7, 8,                                      // 84840:      case 0x7: {
  OPC_CheckPredicate, 135, 1,                // 84842:       check predicate 135
  OPC_Decode, 234, 47, 251, 4,               // 84845:       decode to SETGMTN using decoder 635
                                             // 84845:      }
  8, 8,                                      // 84850:      case 0x8: {
  OPC_CheckPredicate, 135, 1,                // 84852:       check predicate 135
  OPC_Decode, 146, 42, 251, 4,               // 84855:       decode to MOPSSETGE using decoder 635
                                             // 84855:      }
  9, 8,                                      // 84860:      case 0x9: {
  OPC_CheckPredicate, 135, 1,                // 84862:       check predicate 135
  OPC_Decode, 148, 42, 251, 4,               // 84865:       decode to MOPSSETGET using decoder 635
                                             // 84865:      }
  10, 8,                                     // 84870:      case 0xa: {
  OPC_CheckPredicate, 135, 1,                // 84872:       check predicate 135
  OPC_Decode, 147, 42, 251, 4,               // 84875:       decode to MOPSSETGEN using decoder 635
                                             // 84875:      }
  11, 0,                                     // 84880:      case 0xb: {
  OPC_CheckPredicate, 135, 1,                // 84882:       check predicate 135
  OPC_Decode, 149, 42, 251, 4,               // 84885:       decode to MOPSSETGETN using decoder 635
                                             // 84885:      }
                                             // 84885:      } // switch Inst[15:12]
                                             // 84885:     }
  2, 0,                                      // 84890:     case 0x2: {
  OPC_CheckPredicate, 79,                    // 84892:      check predicate 79
  OPC_Decode, 172, 37, 166, 5,               // 84894:      decode to LDAPURqi using decoder 678
                                             // 84894:     }
                                             // 84894:     } // switch Inst[11:10]
                                             // 84894:    }
                                             // 84894:    } // switch Inst[23:21]
                                             // 84894:   }
  2, 167, 14,                                // 84899:   case 0x2: {
  OPC_SwitchField, 21, 3,                    // 84902:    switch Inst[23:21] {
  0, 39,                                     // 84905:    case 0x0: {
  OPC_SwitchField, 15, 6,                    // 84907:     switch Inst[20:15] {
  5, 7,                                      // 84910:     case 0x5: {
  OPC_CheckPredicate, 77,                    // 84912:      check predicate 77
  OPC_Decode, 130, 47, 168, 5,               // 84914:      decode to SCVTFSWSri using decoder 680
                                             // 84914:     }
  7, 7,                                      // 84919:     case 0x7: {
  OPC_CheckPredicate, 77,                    // 84921:      check predicate 77
  OPC_Decode, 251, 62, 168, 5,               // 84923:      decode to UCVTFSWSri using decoder 680
                                             // 84923:     }
  49, 7,                                     // 84928:     case 0x31: {
  OPC_CheckPredicate, 77,                    // 84930:      check predicate 77
  OPC_Decode, 129, 25, 169, 5,               // 84932:      decode to FCVTZSSWSri using decoder 681
                                             // 84932:     }
  51, 0,                                     // 84937:     case 0x33: {
  OPC_CheckPredicate, 77,                    // 84939:      check predicate 77
  OPC_Decode, 180, 25, 169, 5,               // 84941:      decode to FCVTZUSWSri using decoder 681
                                             // 84941:     }
                                             // 84941:     } // switch Inst[20:15]
                                             // 84941:    }
  1, 226, 3,                                 // 84946:    case 0x1: {
  OPC_SwitchField, 10, 2,                    // 84949:     switch Inst[11:10] {
  0, 230, 2,                                 // 84952:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 84955:      switch Inst[12] {
  0, 211, 2,                                 // 84958:      case 0x0: {
  OPC_SwitchField, 13, 3,                    // 84961:       switch Inst[15:13] {
  0, 129, 1,                                 // 84964:       case 0x0: {
  OPC_SwitchField, 16, 5,                    // 84967:        switch Inst[20:16] {
  0, 7,                                      // 84970:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 84972:         check predicate 77
  OPC_Decode, 157, 24, 170, 5,               // 84974:         decode to FCVTNSUWSr using decoder 682
                                             // 84974:        }
  1, 7,                                      // 84979:        case 0x1: {
  OPC_CheckPredicate, 77,                    // 84981:         check predicate 77
  OPC_Decode, 180, 24, 170, 5,               // 84983:         decode to FCVTNUUWSr using decoder 682
                                             // 84983:        }
  2, 7,                                      // 84988:        case 0x2: {
  OPC_CheckPredicate, 77,                    // 84990:         check predicate 77
  OPC_Decode, 136, 47, 171, 5,               // 84992:         decode to SCVTFUWSri using decoder 683
                                             // 84992:        }
  3, 7,                                      // 84997:        case 0x3: {
  OPC_CheckPredicate, 77,                    // 84999:         check predicate 77
  OPC_Decode, 129, 63, 171, 5,               // 85001:         decode to UCVTFUWSri using decoder 683
                                             // 85001:        }
  4, 7,                                      // 85006:        case 0x4: {
  OPC_CheckPredicate, 77,                    // 85008:         check predicate 77
  OPC_Decode, 199, 23, 170, 5,               // 85010:         decode to FCVTASUWSr using decoder 682
                                             // 85010:        }
  5, 7,                                      // 85015:        case 0x5: {
  OPC_CheckPredicate, 77,                    // 85017:         check predicate 77
  OPC_Decode, 217, 23, 170, 5,               // 85019:         decode to FCVTAUUWSr using decoder 682
                                             // 85019:        }
  6, 7,                                      // 85024:        case 0x6: {
  OPC_CheckPredicate, 77,                    // 85026:         check predicate 77
  OPC_Decode, 205, 29, 170, 5,               // 85028:         decode to FMOVSWr using decoder 682
                                             // 85028:        }
  7, 7,                                      // 85033:        case 0x7: {
  OPC_CheckPredicate, 77,                    // 85035:         check predicate 77
  OPC_Decode, 209, 29, 171, 5,               // 85037:         decode to FMOVWSr using decoder 683
                                             // 85037:        }
  8, 7,                                      // 85042:        case 0x8: {
  OPC_CheckPredicate, 77,                    // 85044:         check predicate 77
  OPC_Decode, 209, 24, 170, 5,               // 85046:         decode to FCVTPSUWSr using decoder 682
                                             // 85046:        }
  9, 7,                                      // 85051:        case 0x9: {
  OPC_CheckPredicate, 77,                    // 85053:         check predicate 77
  OPC_Decode, 227, 24, 170, 5,               // 85055:         decode to FCVTPUUWSr using decoder 682
                                             // 85055:        }
  16, 7,                                     // 85060:        case 0x10: {
  OPC_CheckPredicate, 77,                    // 85062:         check predicate 77
  OPC_Decode, 248, 23, 170, 5,               // 85064:         decode to FCVTMSUWSr using decoder 682
                                             // 85064:        }
  17, 7,                                     // 85069:        case 0x11: {
  OPC_CheckPredicate, 77,                    // 85071:         check predicate 77
  OPC_Decode, 138, 24, 170, 5,               // 85073:         decode to FCVTMUUWSr using decoder 682
                                             // 85073:        }
  24, 7,                                     // 85078:        case 0x18: {
  OPC_CheckPredicate, 77,                    // 85080:         check predicate 77
  OPC_Decode, 135, 25, 170, 5,               // 85082:         decode to FCVTZSUWSr using decoder 682
                                             // 85082:        }
  25, 0,                                     // 85087:        case 0x19: {
  OPC_CheckPredicate, 77,                    // 85089:         check predicate 77
  OPC_Decode, 186, 25, 170, 5,               // 85091:         decode to FCVTZUUWSr using decoder 682
                                             // 85091:        }
                                             // 85091:        } // switch Inst[20:16]
                                             // 85091:       }
  1, 39,                                     // 85096:       case 0x1: {
  OPC_SwitchField, 0, 5,                     // 85098:        switch Inst[4:0] {
  0, 7,                                      // 85101:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85103:         check predicate 77
  OPC_Decode, 183, 23, 172, 5,               // 85105:         decode to FCMPSrr using decoder 684
                                             // 85105:        }
  8, 7,                                      // 85110:        case 0x8: {
  OPC_CheckPredicate, 77,                    // 85112:         check predicate 77
  OPC_Decode, 182, 23, 173, 5,               // 85114:         decode to FCMPSri using decoder 685
                                             // 85114:        }
  16, 7,                                     // 85119:        case 0x10: {
  OPC_CheckPredicate, 77,                    // 85121:         check predicate 77
  OPC_Decode, 179, 23, 172, 5,               // 85123:         decode to FCMPESrr using decoder 684
                                             // 85123:        }
  24, 0,                                     // 85128:        case 0x18: {
  OPC_CheckPredicate, 77,                    // 85130:         check predicate 77
  OPC_Decode, 178, 23, 173, 5,               // 85132:         decode to FCMPESri using decoder 685
                                             // 85132:        }
                                             // 85132:        } // switch Inst[4:0]
                                             // 85132:       }
  2, 75,                                     // 85137:       case 0x2: {
  OPC_SwitchField, 16, 5,                    // 85139:        switch Inst[20:16] {
  0, 7,                                      // 85142:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85144:         check predicate 77
  OPC_Decode, 207, 29, 174, 5,               // 85146:         decode to FMOVSr using decoder 686
                                             // 85146:        }
  1, 7,                                      // 85151:        case 0x1: {
  OPC_CheckPredicate, 77,                    // 85153:         check predicate 77
  OPC_Decode, 157, 30, 174, 5,               // 85155:         decode to FNEGSr using decoder 686
                                             // 85155:        }
  4, 7,                                      // 85160:        case 0x4: {
  OPC_CheckPredicate, 77,                    // 85162:         check predicate 77
  OPC_Decode, 177, 31, 174, 5,               // 85164:         decode to FRINTNSr using decoder 686
                                             // 85164:        }
  5, 7,                                      // 85169:        case 0x5: {
  OPC_CheckPredicate, 77,                    // 85171:         check predicate 77
  OPC_Decode, 161, 31, 174, 5,               // 85173:         decode to FRINTMSr using decoder 686
                                             // 85173:        }
  6, 7,                                      // 85178:        case 0x6: {
  OPC_CheckPredicate, 77,                    // 85180:         check predicate 77
  OPC_Decode, 131, 31, 174, 5,               // 85182:         decode to FRINTASr using decoder 686
                                             // 85182:        }
  7, 7,                                      // 85187:        case 0x7: {
  OPC_CheckPredicate, 77,                    // 85189:         check predicate 77
  OPC_Decode, 209, 31, 174, 5,               // 85191:         decode to FRINTXSr using decoder 686
                                             // 85191:        }
  8, 7,                                      // 85196:        case 0x8: {
  OPC_CheckPredicate, 82,                    // 85198:         check predicate 82
  OPC_Decode, 231, 30, 174, 5,               // 85200:         decode to FRINT32ZSr using decoder 686
                                             // 85200:        }
  9, 0,                                      // 85205:        case 0x9: {
  OPC_CheckPredicate, 82,                    // 85207:         check predicate 82
  OPC_Decode, 249, 30, 174, 5,               // 85209:         decode to FRINT64ZSr using decoder 686
                                             // 85209:        }
                                             // 85209:        } // switch Inst[20:16]
                                             // 85209:       }
  6, 0,                                      // 85214:       case 0x6: {
  OPC_SwitchField, 16, 5,                    // 85216:        switch Inst[20:16] {
  0, 7,                                      // 85219:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85221:         check predicate 77
  OPC_Decode, 193, 21, 174, 5,               // 85223:         decode to FABSSr using decoder 686
                                             // 85223:        }
  1, 7,                                      // 85228:        case 0x1: {
  OPC_CheckPredicate, 77,                    // 85230:         check predicate 77
  OPC_Decode, 151, 32, 174, 5,               // 85232:         decode to FSQRTSr using decoder 686
                                             // 85232:        }
  2, 7,                                      // 85237:        case 0x2: {
  OPC_CheckPredicate, 77,                    // 85239:         check predicate 77
  OPC_Decode, 230, 23, 175, 5,               // 85241:         decode to FCVTDSr using decoder 687
                                             // 85241:        }
  3, 7,                                      // 85246:        case 0x3: {
  OPC_CheckPredicate, 77,                    // 85248:         check predicate 77
  OPC_Decode, 232, 23, 176, 5,               // 85250:         decode to FCVTHSr using decoder 688
                                             // 85250:        }
  4, 7,                                      // 85255:        case 0x4: {
  OPC_CheckPredicate, 77,                    // 85257:         check predicate 77
  OPC_Decode, 193, 31, 174, 5,               // 85259:         decode to FRINTPSr using decoder 686
                                             // 85259:        }
  5, 7,                                      // 85264:        case 0x5: {
  OPC_CheckPredicate, 77,                    // 85266:         check predicate 77
  OPC_Decode, 223, 31, 174, 5,               // 85268:         decode to FRINTZSr using decoder 686
                                             // 85268:        }
  7, 7,                                      // 85273:        case 0x7: {
  OPC_CheckPredicate, 77,                    // 85275:         check predicate 77
  OPC_Decode, 147, 31, 174, 5,               // 85277:         decode to FRINTISr using decoder 686
                                             // 85277:        }
  8, 7,                                      // 85282:        case 0x8: {
  OPC_CheckPredicate, 82,                    // 85284:         check predicate 82
  OPC_Decode, 222, 30, 174, 5,               // 85286:         decode to FRINT32XSr using decoder 686
                                             // 85286:        }
  9, 0,                                      // 85291:        case 0x9: {
  OPC_CheckPredicate, 82,                    // 85293:         check predicate 82
  OPC_Decode, 240, 30, 174, 5,               // 85295:         decode to FRINT64XSr using decoder 686
                                             // 85295:        }
                                             // 85295:        } // switch Inst[20:16]
                                             // 85295:       }
                                             // 85295:       } // switch Inst[15:13]
                                             // 85295:      }
  1, 0,                                      // 85300:      case 0x1: {
  OPC_CheckPredicate, 77,                    // 85302:       check predicate 77
  OPC_CheckField, 5, 5, 0,                   // 85304:       check Inst[9:5] == 0x0
  OPC_Decode, 206, 29, 177, 5,               // 85308:       decode to FMOVSi using decoder 689
                                             // 85308:      }
                                             // 85308:      } // switch Inst[12]
                                             // 85308:     }
  1, 21,                                     // 85313:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 85315:      switch Inst[4] {
  0, 7,                                      // 85318:      case 0x0: {
  OPC_CheckPredicate, 77,                    // 85320:       check predicate 77
  OPC_Decode, 183, 22, 178, 5,               // 85322:       decode to FCCMPSrr using decoder 690
                                             // 85322:      }
  1, 0,                                      // 85327:      case 0x1: {
  OPC_CheckPredicate, 77,                    // 85329:       check predicate 77
  OPC_Decode, 181, 22, 178, 5,               // 85331:       decode to FCCMPESrr using decoder 690
                                             // 85331:      }
                                             // 85331:      } // switch Inst[4]
                                             // 85331:     }
  2, 84,                                     // 85336:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 85338:      switch Inst[15:12] {
  0, 7,                                      // 85341:      case 0x0: {
  OPC_CheckPredicate, 77,                    // 85343:       check predicate 77
  OPC_Decode, 226, 29, 179, 5,               // 85345:       decode to FMULSrr using decoder 691
                                             // 85345:      }
  1, 7,                                      // 85350:      case 0x1: {
  OPC_CheckPredicate, 77,                    // 85352:       check predicate 77
  OPC_Decode, 243, 25, 179, 5,               // 85354:       decode to FDIVSrr using decoder 691
                                             // 85354:      }
  2, 7,                                      // 85359:      case 0x2: {
  OPC_CheckPredicate, 77,                    // 85361:       check predicate 77
  OPC_Decode, 246, 21, 179, 5,               // 85363:       decode to FADDSrr using decoder 691
                                             // 85363:      }
  3, 7,                                      // 85368:      case 0x3: {
  OPC_CheckPredicate, 77,                    // 85370:       check predicate 77
  OPC_Decode, 171, 32, 179, 5,               // 85372:       decode to FSUBSrr using decoder 691
                                             // 85372:      }
  4, 7,                                      // 85377:      case 0x4: {
  OPC_CheckPredicate, 77,                    // 85379:       check predicate 77
  OPC_Decode, 245, 26, 179, 5,               // 85381:       decode to FMAXSrr using decoder 691
                                             // 85381:      }
  5, 7,                                      // 85386:      case 0x5: {
  OPC_CheckPredicate, 77,                    // 85388:       check predicate 77
  OPC_Decode, 209, 27, 179, 5,               // 85390:       decode to FMINSrr using decoder 691
                                             // 85390:      }
  6, 7,                                      // 85395:      case 0x6: {
  OPC_CheckPredicate, 77,                    // 85397:       check predicate 77
  OPC_Decode, 201, 26, 179, 5,               // 85399:       decode to FMAXNMSrr using decoder 691
                                             // 85399:      }
  7, 7,                                      // 85404:      case 0x7: {
  OPC_CheckPredicate, 77,                    // 85406:       check predicate 77
  OPC_Decode, 165, 27, 179, 5,               // 85408:       decode to FMINNMSrr using decoder 691
                                             // 85408:      }
  8, 0,                                      // 85413:      case 0x8: {
  OPC_CheckPredicate, 77,                    // 85415:       check predicate 77
  OPC_Decode, 189, 30, 179, 5,               // 85417:       decode to FNMULSrr using decoder 691
                                             // 85417:      }
                                             // 85417:      } // switch Inst[15:12]
                                             // 85417:     }
  3, 0,                                      // 85422:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 85424:      check predicate 77
  OPC_Decode, 192, 23, 180, 5,               // 85426:      decode to FCSELSrrr using decoder 692
                                             // 85426:     }
                                             // 85426:     } // switch Inst[11:10]
                                             // 85426:    }
  2, 39,                                     // 85431:    case 0x2: {
  OPC_SwitchField, 15, 6,                    // 85433:     switch Inst[20:15] {
  5, 7,                                      // 85436:     case 0x5: {
  OPC_CheckPredicate, 77,                    // 85438:      check predicate 77
  OPC_Decode, 128, 47, 181, 5,               // 85440:      decode to SCVTFSWDri using decoder 693
                                             // 85440:     }
  7, 7,                                      // 85445:     case 0x7: {
  OPC_CheckPredicate, 77,                    // 85447:      check predicate 77
  OPC_Decode, 249, 62, 181, 5,               // 85449:      decode to UCVTFSWDri using decoder 693
                                             // 85449:     }
  49, 7,                                     // 85454:     case 0x31: {
  OPC_CheckPredicate, 77,                    // 85456:      check predicate 77
  OPC_Decode, 255, 24, 182, 5,               // 85458:      decode to FCVTZSSWDri using decoder 694
                                             // 85458:     }
  51, 0,                                     // 85463:     case 0x33: {
  OPC_CheckPredicate, 77,                    // 85465:      check predicate 77
  OPC_Decode, 178, 25, 182, 5,               // 85467:      decode to FCVTZUSWDri using decoder 694
                                             // 85467:     }
                                             // 85467:     } // switch Inst[20:15]
                                             // 85467:    }
  3, 219, 4,                                 // 85472:    case 0x3: {
  OPC_SwitchField, 10, 2,                    // 85475:     switch Inst[11:10] {
  0, 223, 3,                                 // 85478:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 85481:      switch Inst[12] {
  0, 204, 3,                                 // 85484:      case 0x0: {
  OPC_SwitchField, 13, 3,                    // 85487:       switch Inst[15:13] {
  0, 241, 1,                                 // 85490:       case 0x0: {
  OPC_SwitchField, 16, 5,                    // 85493:        switch Inst[20:16] {
  0, 7,                                      // 85496:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85498:         check predicate 77
  OPC_Decode, 155, 24, 183, 5,               // 85500:         decode to FCVTNSUWDr using decoder 695
                                             // 85500:        }
  1, 7,                                      // 85505:        case 0x1: {
  OPC_CheckPredicate, 77,                    // 85507:         check predicate 77
  OPC_Decode, 178, 24, 183, 5,               // 85509:         decode to FCVTNUUWDr using decoder 695
                                             // 85509:        }
  2, 7,                                      // 85514:        case 0x2: {
  OPC_CheckPredicate, 77,                    // 85516:         check predicate 77
  OPC_Decode, 134, 47, 202, 3,               // 85518:         decode to SCVTFUWDri using decoder 458
                                             // 85518:        }
  3, 7,                                      // 85523:        case 0x3: {
  OPC_CheckPredicate, 77,                    // 85525:         check predicate 77
  OPC_Decode, 255, 62, 202, 3,               // 85527:         decode to UCVTFUWDri using decoder 458
                                             // 85527:        }
  4, 7,                                      // 85532:        case 0x4: {
  OPC_CheckPredicate, 77,                    // 85534:         check predicate 77
  OPC_Decode, 197, 23, 183, 5,               // 85536:         decode to FCVTASUWDr using decoder 695
                                             // 85536:        }
  5, 7,                                      // 85541:        case 0x5: {
  OPC_CheckPredicate, 77,                    // 85543:         check predicate 77
  OPC_Decode, 215, 23, 183, 5,               // 85545:         decode to FCVTAUUWDr using decoder 695
                                             // 85545:        }
  8, 7,                                      // 85550:        case 0x8: {
  OPC_CheckPredicate, 77,                    // 85552:         check predicate 77
  OPC_Decode, 207, 24, 183, 5,               // 85554:         decode to FCVTPSUWDr using decoder 695
                                             // 85554:        }
  9, 7,                                      // 85559:        case 0x9: {
  OPC_CheckPredicate, 77,                    // 85561:         check predicate 77
  OPC_Decode, 225, 24, 183, 5,               // 85563:         decode to FCVTPUUWDr using decoder 695
                                             // 85563:        }
  10, 8,                                     // 85568:        case 0xa: {
  OPC_CheckPredicate, 136, 1,                // 85570:         check predicate 136
  OPC_Decode, 153, 24, 130, 4,               // 85573:         decode to FCVTNSSDr using decoder 514
                                             // 85573:        }
  11, 8,                                     // 85578:        case 0xb: {
  OPC_CheckPredicate, 136, 1,                // 85580:         check predicate 136
  OPC_Decode, 176, 24, 130, 4,               // 85583:         decode to FCVTNUSDr using decoder 514
                                             // 85583:        }
  16, 7,                                     // 85588:        case 0x10: {
  OPC_CheckPredicate, 77,                    // 85590:         check predicate 77
  OPC_Decode, 246, 23, 183, 5,               // 85592:         decode to FCVTMSUWDr using decoder 695
                                             // 85592:        }
  17, 7,                                     // 85597:        case 0x11: {
  OPC_CheckPredicate, 77,                    // 85599:         check predicate 77
  OPC_Decode, 136, 24, 183, 5,               // 85601:         decode to FCVTMUUWDr using decoder 695
                                             // 85601:        }
  18, 8,                                     // 85606:        case 0x12: {
  OPC_CheckPredicate, 136, 1,                // 85608:         check predicate 136
  OPC_Decode, 205, 24, 130, 4,               // 85611:         decode to FCVTPSSDr using decoder 514
                                             // 85611:        }
  19, 8,                                     // 85616:        case 0x13: {
  OPC_CheckPredicate, 136, 1,                // 85618:         check predicate 136
  OPC_Decode, 223, 24, 130, 4,               // 85621:         decode to FCVTPUSDr using decoder 514
                                             // 85621:        }
  20, 8,                                     // 85626:        case 0x14: {
  OPC_CheckPredicate, 136, 1,                // 85628:         check predicate 136
  OPC_Decode, 244, 23, 130, 4,               // 85631:         decode to FCVTMSSDr using decoder 514
                                             // 85631:        }
  21, 8,                                     // 85636:        case 0x15: {
  OPC_CheckPredicate, 136, 1,                // 85638:         check predicate 136
  OPC_Decode, 134, 24, 130, 4,               // 85641:         decode to FCVTMUSDr using decoder 514
                                             // 85641:        }
  22, 8,                                     // 85646:        case 0x16: {
  OPC_CheckPredicate, 136, 1,                // 85648:         check predicate 136
  OPC_Decode, 253, 24, 130, 4,               // 85651:         decode to FCVTZSSDr using decoder 514
                                             // 85651:        }
  23, 8,                                     // 85656:        case 0x17: {
  OPC_CheckPredicate, 136, 1,                // 85658:         check predicate 136
  OPC_Decode, 176, 25, 130, 4,               // 85661:         decode to FCVTZUSDr using decoder 514
                                             // 85661:        }
  24, 7,                                     // 85666:        case 0x18: {
  OPC_CheckPredicate, 77,                    // 85668:         check predicate 77
  OPC_Decode, 133, 25, 183, 5,               // 85670:         decode to FCVTZSUWDr using decoder 695
                                             // 85670:        }
  25, 7,                                     // 85675:        case 0x19: {
  OPC_CheckPredicate, 77,                    // 85677:         check predicate 77
  OPC_Decode, 184, 25, 183, 5,               // 85679:         decode to FCVTZUUWDr using decoder 695
                                             // 85679:        }
  26, 8,                                     // 85684:        case 0x1a: {
  OPC_CheckPredicate, 136, 1,                // 85686:         check predicate 136
  OPC_Decode, 195, 23, 130, 4,               // 85689:         decode to FCVTASSDr using decoder 514
                                             // 85689:        }
  27, 8,                                     // 85694:        case 0x1b: {
  OPC_CheckPredicate, 136, 1,                // 85696:         check predicate 136
  OPC_Decode, 213, 23, 130, 4,               // 85699:         decode to FCVTAUSDr using decoder 514
                                             // 85699:        }
  28, 8,                                     // 85704:        case 0x1c: {
  OPC_CheckPredicate, 136, 1,                // 85706:         check predicate 136
  OPC_Decode, 249, 46, 175, 5,               // 85709:         decode to SCVTFDSr using decoder 687
                                             // 85709:        }
  29, 8,                                     // 85714:        case 0x1d: {
  OPC_CheckPredicate, 136, 1,                // 85716:         check predicate 136
  OPC_Decode, 242, 62, 175, 5,               // 85719:         decode to UCVTFDSr using decoder 687
                                             // 85719:        }
  30, 0,                                     // 85724:        case 0x1e: {
  OPC_CheckPredicate, 137, 1,                // 85726:         check predicate 137
  OPC_Decode, 170, 26, 183, 5,               // 85729:         decode to FJCVTZS using decoder 695
                                             // 85729:        }
                                             // 85729:        } // switch Inst[20:16]
                                             // 85729:       }
  1, 39,                                     // 85734:       case 0x1: {
  OPC_SwitchField, 0, 5,                     // 85736:        switch Inst[4:0] {
  0, 7,                                      // 85739:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85741:         check predicate 77
  OPC_Decode, 173, 23, 184, 5,               // 85743:         decode to FCMPDrr using decoder 696
                                             // 85743:        }
  8, 7,                                      // 85748:        case 0x8: {
  OPC_CheckPredicate, 77,                    // 85750:         check predicate 77
  OPC_Decode, 172, 23, 185, 5,               // 85752:         decode to FCMPDri using decoder 697
                                             // 85752:        }
  16, 7,                                     // 85757:        case 0x10: {
  OPC_CheckPredicate, 77,                    // 85759:         check predicate 77
  OPC_Decode, 175, 23, 184, 5,               // 85761:         decode to FCMPEDrr using decoder 696
                                             // 85761:        }
  24, 0,                                     // 85766:        case 0x18: {
  OPC_CheckPredicate, 77,                    // 85768:         check predicate 77
  OPC_Decode, 174, 23, 185, 5,               // 85770:         decode to FCMPEDri using decoder 697
                                             // 85770:        }
                                             // 85770:        } // switch Inst[4:0]
                                             // 85770:       }
  2, 93,                                     // 85775:       case 0x2: {
  OPC_SwitchField, 16, 5,                    // 85777:        switch Inst[20:16] {
  0, 7,                                      // 85780:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85782:         check predicate 77
  OPC_Decode, 200, 29, 201, 3,               // 85784:         decode to FMOVDr using decoder 457
                                             // 85784:        }
  1, 7,                                      // 85789:        case 0x1: {
  OPC_CheckPredicate, 77,                    // 85791:         check predicate 77
  OPC_Decode, 155, 30, 201, 3,               // 85793:         decode to FNEGDr using decoder 457
                                             // 85793:        }
  2, 7,                                      // 85798:        case 0x2: {
  OPC_CheckPredicate, 77,                    // 85800:         check predicate 77
  OPC_Decode, 239, 24, 130, 4,               // 85802:         decode to FCVTSDr using decoder 514
                                             // 85802:        }
  3, 7,                                      // 85807:        case 0x3: {
  OPC_CheckPredicate, 92,                    // 85809:         check predicate 92
  OPC_Decode, 128, 15, 176, 5,               // 85811:         decode to BFCVT using decoder 688
                                             // 85811:        }
  4, 7,                                      // 85816:        case 0x4: {
  OPC_CheckPredicate, 77,                    // 85818:         check predicate 77
  OPC_Decode, 175, 31, 201, 3,               // 85820:         decode to FRINTNDr using decoder 457
                                             // 85820:        }
  5, 7,                                      // 85825:        case 0x5: {
  OPC_CheckPredicate, 77,                    // 85827:         check predicate 77
  OPC_Decode, 159, 31, 201, 3,               // 85829:         decode to FRINTMDr using decoder 457
                                             // 85829:        }
  6, 7,                                      // 85834:        case 0x6: {
  OPC_CheckPredicate, 77,                    // 85836:         check predicate 77
  OPC_Decode, 129, 31, 201, 3,               // 85838:         decode to FRINTADr using decoder 457
                                             // 85838:        }
  7, 7,                                      // 85843:        case 0x7: {
  OPC_CheckPredicate, 77,                    // 85845:         check predicate 77
  OPC_Decode, 207, 31, 201, 3,               // 85847:         decode to FRINTXDr using decoder 457
                                             // 85847:        }
  8, 7,                                      // 85852:        case 0x8: {
  OPC_CheckPredicate, 82,                    // 85854:         check predicate 82
  OPC_Decode, 230, 30, 201, 3,               // 85856:         decode to FRINT32ZDr using decoder 457
                                             // 85856:        }
  9, 0,                                      // 85861:        case 0x9: {
  OPC_CheckPredicate, 82,                    // 85863:         check predicate 82
  OPC_Decode, 248, 30, 201, 3,               // 85865:         decode to FRINT64ZDr using decoder 457
                                             // 85865:        }
                                             // 85865:        } // switch Inst[20:16]
                                             // 85865:       }
  6, 0,                                      // 85870:       case 0x6: {
  OPC_SwitchField, 16, 5,                    // 85872:        switch Inst[20:16] {
  0, 7,                                      // 85875:        case 0x0: {
  OPC_CheckPredicate, 77,                    // 85877:         check predicate 77
  OPC_Decode, 191, 21, 201, 3,               // 85879:         decode to FABSDr using decoder 457
                                             // 85879:        }
  1, 7,                                      // 85884:        case 0x1: {
  OPC_CheckPredicate, 77,                    // 85886:         check predicate 77
  OPC_Decode, 149, 32, 201, 3,               // 85888:         decode to FSQRTDr using decoder 457
                                             // 85888:        }
  3, 7,                                      // 85893:        case 0x3: {
  OPC_CheckPredicate, 77,                    // 85895:         check predicate 77
  OPC_Decode, 231, 23, 212, 3,               // 85897:         decode to FCVTHDr using decoder 468
                                             // 85897:        }
  4, 7,                                      // 85902:        case 0x4: {
  OPC_CheckPredicate, 77,                    // 85904:         check predicate 77
  OPC_Decode, 191, 31, 201, 3,               // 85906:         decode to FRINTPDr using decoder 457
                                             // 85906:        }
  5, 7,                                      // 85911:        case 0x5: {
  OPC_CheckPredicate, 77,                    // 85913:         check predicate 77
  OPC_Decode, 221, 31, 201, 3,               // 85915:         decode to FRINTZDr using decoder 457
                                             // 85915:        }
  7, 7,                                      // 85920:        case 0x7: {
  OPC_CheckPredicate, 77,                    // 85922:         check predicate 77
  OPC_Decode, 145, 31, 201, 3,               // 85924:         decode to FRINTIDr using decoder 457
                                             // 85924:        }
  8, 7,                                      // 85929:        case 0x8: {
  OPC_CheckPredicate, 82,                    // 85931:         check predicate 82
  OPC_Decode, 221, 30, 201, 3,               // 85933:         decode to FRINT32XDr using decoder 457
                                             // 85933:        }
  9, 0,                                      // 85938:        case 0x9: {
  OPC_CheckPredicate, 82,                    // 85940:         check predicate 82
  OPC_Decode, 239, 30, 201, 3,               // 85942:         decode to FRINT64XDr using decoder 457
                                             // 85942:        }
                                             // 85942:        } // switch Inst[20:16]
                                             // 85942:       }
                                             // 85942:       } // switch Inst[15:13]
                                             // 85942:      }
  1, 0,                                      // 85947:      case 0x1: {
  OPC_CheckPredicate, 77,                    // 85949:       check predicate 77
  OPC_CheckField, 5, 5, 0,                   // 85951:       check Inst[9:5] == 0x0
  OPC_Decode, 199, 29, 186, 5,               // 85955:       decode to FMOVDi using decoder 698
                                             // 85955:      }
                                             // 85955:      } // switch Inst[12]
                                             // 85955:     }
  1, 21,                                     // 85960:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 85962:      switch Inst[4] {
  0, 7,                                      // 85965:      case 0x0: {
  OPC_CheckPredicate, 77,                    // 85967:       check predicate 77
  OPC_Decode, 178, 22, 187, 5,               // 85969:       decode to FCCMPDrr using decoder 699
                                             // 85969:      }
  1, 0,                                      // 85974:      case 0x1: {
  OPC_CheckPredicate, 77,                    // 85976:       check predicate 77
  OPC_Decode, 179, 22, 187, 5,               // 85978:       decode to FCCMPEDrr using decoder 699
                                             // 85978:      }
                                             // 85978:      } // switch Inst[4]
                                             // 85978:     }
  2, 84,                                     // 85983:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 85985:      switch Inst[15:12] {
  0, 7,                                      // 85988:      case 0x0: {
  OPC_CheckPredicate, 77,                    // 85990:       check predicate 77
  OPC_Decode, 224, 29, 200, 3,               // 85992:       decode to FMULDrr using decoder 456
                                             // 85992:      }
  1, 7,                                      // 85997:      case 0x1: {
  OPC_CheckPredicate, 77,                    // 85999:       check predicate 77
  OPC_Decode, 238, 25, 200, 3,               // 86001:       decode to FDIVDrr using decoder 456
                                             // 86001:      }
  2, 7,                                      // 86006:      case 0x2: {
  OPC_CheckPredicate, 77,                    // 86008:       check predicate 77
  OPC_Decode, 230, 21, 200, 3,               // 86010:       decode to FADDDrr using decoder 456
                                             // 86010:      }
  3, 7,                                      // 86015:      case 0x3: {
  OPC_CheckPredicate, 77,                    // 86017:       check predicate 77
  OPC_Decode, 163, 32, 200, 3,               // 86019:       decode to FSUBDrr using decoder 456
                                             // 86019:      }
  4, 7,                                      // 86024:      case 0x4: {
  OPC_CheckPredicate, 77,                    // 86026:       check predicate 77
  OPC_Decode, 183, 26, 200, 3,               // 86028:       decode to FMAXDrr using decoder 456
                                             // 86028:      }
  5, 7,                                      // 86033:      case 0x5: {
  OPC_CheckPredicate, 77,                    // 86035:       check predicate 77
  OPC_Decode, 147, 27, 200, 3,               // 86037:       decode to FMINDrr using decoder 456
                                             // 86037:      }
  6, 7,                                      // 86042:      case 0x6: {
  OPC_CheckPredicate, 77,                    // 86044:       check predicate 77
  OPC_Decode, 185, 26, 200, 3,               // 86046:       decode to FMAXNMDrr using decoder 456
                                             // 86046:      }
  7, 7,                                      // 86051:      case 0x7: {
  OPC_CheckPredicate, 77,                    // 86053:       check predicate 77
  OPC_Decode, 149, 27, 200, 3,               // 86055:       decode to FMINNMDrr using decoder 456
                                             // 86055:      }
  8, 0,                                      // 86060:      case 0x8: {
  OPC_CheckPredicate, 77,                    // 86062:       check predicate 77
  OPC_Decode, 187, 30, 200, 3,               // 86064:       decode to FNMULDrr using decoder 456
                                             // 86064:      }
                                             // 86064:      } // switch Inst[15:12]
                                             // 86064:     }
  3, 0,                                      // 86069:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 86071:      check predicate 77
  OPC_Decode, 190, 23, 188, 5,               // 86073:      decode to FCSELDrrr using decoder 700
                                             // 86073:     }
                                             // 86073:     } // switch Inst[11:10]
                                             // 86073:    }
  6, 43,                                     // 86078:    case 0x6: {
  OPC_SwitchField, 15, 6,                    // 86080:     switch Inst[20:15] {
  5, 8,                                      // 86083:     case 0x5: {
  OPC_CheckPredicate, 138, 1,                // 86085:      check predicate 138
  OPC_Decode, 129, 47, 189, 5,               // 86088:      decode to SCVTFSWHri using decoder 701
                                             // 86088:     }
  7, 8,                                      // 86093:     case 0x7: {
  OPC_CheckPredicate, 138, 1,                // 86095:      check predicate 138
  OPC_Decode, 250, 62, 189, 5,               // 86098:      decode to UCVTFSWHri using decoder 701
                                             // 86098:     }
  49, 8,                                     // 86103:     case 0x31: {
  OPC_CheckPredicate, 138, 1,                // 86105:      check predicate 138
  OPC_Decode, 128, 25, 190, 5,               // 86108:      decode to FCVTZSSWHri using decoder 702
                                             // 86108:     }
  51, 0,                                     // 86113:     case 0x33: {
  OPC_CheckPredicate, 138, 1,                // 86115:      check predicate 138
  OPC_Decode, 179, 25, 190, 5,               // 86118:      decode to FCVTZUSWHri using decoder 702
                                             // 86118:     }
                                             // 86118:     } // switch Inst[20:15]
                                             // 86118:    }
  7, 0,                                      // 86123:    case 0x7: {
  OPC_SwitchField, 10, 2,                    // 86125:     switch Inst[11:10] {
  0, 216, 3,                                 // 86128:     case 0x0: {
  OPC_SwitchField, 12, 1,                    // 86131:      switch Inst[12] {
  0, 196, 3,                                 // 86134:      case 0x0: {
  OPC_SwitchField, 13, 3,                    // 86137:       switch Inst[15:13] {
  0, 135, 2,                                 // 86140:       case 0x0: {
  OPC_SwitchField, 16, 5,                    // 86143:        switch Inst[20:16] {
  0, 8,                                      // 86146:        case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86148:         check predicate 138
  OPC_Decode, 156, 24, 191, 5,               // 86151:         decode to FCVTNSUWHr using decoder 703
                                             // 86151:        }
  1, 8,                                      // 86156:        case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86158:         check predicate 138
  OPC_Decode, 179, 24, 191, 5,               // 86161:         decode to FCVTNUUWHr using decoder 703
                                             // 86161:        }
  2, 8,                                      // 86166:        case 0x2: {
  OPC_CheckPredicate, 138, 1,                // 86168:         check predicate 138
  OPC_Decode, 135, 47, 192, 5,               // 86171:         decode to SCVTFUWHri using decoder 704
                                             // 86171:        }
  3, 8,                                      // 86176:        case 0x3: {
  OPC_CheckPredicate, 138, 1,                // 86178:         check predicate 138
  OPC_Decode, 128, 63, 192, 5,               // 86181:         decode to UCVTFUWHri using decoder 704
                                             // 86181:        }
  4, 8,                                      // 86186:        case 0x4: {
  OPC_CheckPredicate, 138, 1,                // 86188:         check predicate 138
  OPC_Decode, 198, 23, 191, 5,               // 86191:         decode to FCVTASUWHr using decoder 703
                                             // 86191:        }
  5, 8,                                      // 86196:        case 0x5: {
  OPC_CheckPredicate, 138, 1,                // 86198:         check predicate 138
  OPC_Decode, 216, 23, 191, 5,               // 86201:         decode to FCVTAUUWHr using decoder 703
                                             // 86201:        }
  6, 8,                                      // 86206:        case 0x6: {
  OPC_CheckPredicate, 138, 1,                // 86208:         check predicate 138
  OPC_Decode, 201, 29, 191, 5,               // 86211:         decode to FMOVHWr using decoder 703
                                             // 86211:        }
  7, 8,                                      // 86216:        case 0x7: {
  OPC_CheckPredicate, 138, 1,                // 86218:         check predicate 138
  OPC_Decode, 208, 29, 192, 5,               // 86221:         decode to FMOVWHr using decoder 704
                                             // 86221:        }
  8, 8,                                      // 86226:        case 0x8: {
  OPC_CheckPredicate, 138, 1,                // 86228:         check predicate 138
  OPC_Decode, 208, 24, 191, 5,               // 86231:         decode to FCVTPSUWHr using decoder 703
                                             // 86231:        }
  9, 8,                                      // 86236:        case 0x9: {
  OPC_CheckPredicate, 138, 1,                // 86238:         check predicate 138
  OPC_Decode, 226, 24, 191, 5,               // 86241:         decode to FCVTPUUWHr using decoder 703
                                             // 86241:        }
  10, 8,                                     // 86246:        case 0xa: {
  OPC_CheckPredicate, 136, 1,                // 86248:         check predicate 136
  OPC_Decode, 154, 24, 193, 5,               // 86251:         decode to FCVTNSSHr using decoder 705
                                             // 86251:        }
  11, 8,                                     // 86256:        case 0xb: {
  OPC_CheckPredicate, 136, 1,                // 86258:         check predicate 136
  OPC_Decode, 177, 24, 193, 5,               // 86261:         decode to FCVTNUSHr using decoder 705
                                             // 86261:        }
  16, 8,                                     // 86266:        case 0x10: {
  OPC_CheckPredicate, 138, 1,                // 86268:         check predicate 138
  OPC_Decode, 247, 23, 191, 5,               // 86271:         decode to FCVTMSUWHr using decoder 703
                                             // 86271:        }
  17, 8,                                     // 86276:        case 0x11: {
  OPC_CheckPredicate, 138, 1,                // 86278:         check predicate 138
  OPC_Decode, 137, 24, 191, 5,               // 86281:         decode to FCVTMUUWHr using decoder 703
                                             // 86281:        }
  18, 8,                                     // 86286:        case 0x12: {
  OPC_CheckPredicate, 136, 1,                // 86288:         check predicate 136
  OPC_Decode, 206, 24, 193, 5,               // 86291:         decode to FCVTPSSHr using decoder 705
                                             // 86291:        }
  19, 8,                                     // 86296:        case 0x13: {
  OPC_CheckPredicate, 136, 1,                // 86298:         check predicate 136
  OPC_Decode, 224, 24, 193, 5,               // 86301:         decode to FCVTPUSHr using decoder 705
                                             // 86301:        }
  20, 8,                                     // 86306:        case 0x14: {
  OPC_CheckPredicate, 136, 1,                // 86308:         check predicate 136
  OPC_Decode, 245, 23, 193, 5,               // 86311:         decode to FCVTMSSHr using decoder 705
                                             // 86311:        }
  21, 8,                                     // 86316:        case 0x15: {
  OPC_CheckPredicate, 136, 1,                // 86318:         check predicate 136
  OPC_Decode, 135, 24, 193, 5,               // 86321:         decode to FCVTMUSHr using decoder 705
                                             // 86321:        }
  22, 8,                                     // 86326:        case 0x16: {
  OPC_CheckPredicate, 136, 1,                // 86328:         check predicate 136
  OPC_Decode, 254, 24, 193, 5,               // 86331:         decode to FCVTZSSHr using decoder 705
                                             // 86331:        }
  23, 8,                                     // 86336:        case 0x17: {
  OPC_CheckPredicate, 136, 1,                // 86338:         check predicate 136
  OPC_Decode, 177, 25, 193, 5,               // 86341:         decode to FCVTZUSHr using decoder 705
                                             // 86341:        }
  24, 8,                                     // 86346:        case 0x18: {
  OPC_CheckPredicate, 138, 1,                // 86348:         check predicate 138
  OPC_Decode, 134, 25, 191, 5,               // 86351:         decode to FCVTZSUWHr using decoder 703
                                             // 86351:        }
  25, 8,                                     // 86356:        case 0x19: {
  OPC_CheckPredicate, 138, 1,                // 86358:         check predicate 138
  OPC_Decode, 185, 25, 191, 5,               // 86361:         decode to FCVTZUUWHr using decoder 703
                                             // 86361:        }
  26, 8,                                     // 86366:        case 0x1a: {
  OPC_CheckPredicate, 136, 1,                // 86368:         check predicate 136
  OPC_Decode, 196, 23, 193, 5,               // 86371:         decode to FCVTASSHr using decoder 705
                                             // 86371:        }
  27, 8,                                     // 86376:        case 0x1b: {
  OPC_CheckPredicate, 136, 1,                // 86378:         check predicate 136
  OPC_Decode, 214, 23, 193, 5,               // 86381:         decode to FCVTAUSHr using decoder 705
                                             // 86381:        }
  28, 8,                                     // 86386:        case 0x1c: {
  OPC_CheckPredicate, 136, 1,                // 86388:         check predicate 136
  OPC_Decode, 251, 46, 176, 5,               // 86391:         decode to SCVTFHSr using decoder 688
                                             // 86391:        }
  29, 0,                                     // 86396:        case 0x1d: {
  OPC_CheckPredicate, 136, 1,                // 86398:         check predicate 136
  OPC_Decode, 244, 62, 176, 5,               // 86401:         decode to UCVTFHSr using decoder 688
                                             // 86401:        }
                                             // 86401:        } // switch Inst[20:16]
                                             // 86401:       }
  1, 43,                                     // 86406:       case 0x1: {
  OPC_SwitchField, 0, 5,                     // 86408:        switch Inst[4:0] {
  0, 8,                                      // 86411:        case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86413:         check predicate 138
  OPC_Decode, 181, 23, 194, 5,               // 86416:         decode to FCMPHrr using decoder 706
                                             // 86416:        }
  8, 8,                                      // 86421:        case 0x8: {
  OPC_CheckPredicate, 138, 1,                // 86423:         check predicate 138
  OPC_Decode, 180, 23, 195, 5,               // 86426:         decode to FCMPHri using decoder 707
                                             // 86426:        }
  16, 8,                                     // 86431:        case 0x10: {
  OPC_CheckPredicate, 138, 1,                // 86433:         check predicate 138
  OPC_Decode, 177, 23, 194, 5,               // 86436:         decode to FCMPEHrr using decoder 706
                                             // 86436:        }
  24, 0,                                     // 86441:        case 0x18: {
  OPC_CheckPredicate, 138, 1,                // 86443:         check predicate 138
  OPC_Decode, 176, 23, 195, 5,               // 86446:         decode to FCMPEHri using decoder 707
                                             // 86446:        }
                                             // 86446:        } // switch Inst[4:0]
                                             // 86446:       }
  2, 72,                                     // 86451:       case 0x2: {
  OPC_SwitchField, 16, 5,                    // 86453:        switch Inst[20:16] {
  0, 8,                                      // 86456:        case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86458:         check predicate 138
  OPC_Decode, 204, 29, 196, 5,               // 86461:         decode to FMOVHr using decoder 708
                                             // 86461:        }
  1, 8,                                      // 86466:        case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86468:         check predicate 138
  OPC_Decode, 156, 30, 196, 5,               // 86471:         decode to FNEGHr using decoder 708
                                             // 86471:        }
  2, 7,                                      // 86476:        case 0x2: {
  OPC_CheckPredicate, 77,                    // 86478:         check predicate 77
  OPC_Decode, 240, 24, 193, 5,               // 86480:         decode to FCVTSHr using decoder 705
                                             // 86480:        }
  4, 8,                                      // 86485:        case 0x4: {
  OPC_CheckPredicate, 138, 1,                // 86487:         check predicate 138
  OPC_Decode, 176, 31, 196, 5,               // 86490:         decode to FRINTNHr using decoder 708
                                             // 86490:        }
  5, 8,                                      // 86495:        case 0x5: {
  OPC_CheckPredicate, 138, 1,                // 86497:         check predicate 138
  OPC_Decode, 160, 31, 196, 5,               // 86500:         decode to FRINTMHr using decoder 708
                                             // 86500:        }
  6, 8,                                      // 86505:        case 0x6: {
  OPC_CheckPredicate, 138, 1,                // 86507:         check predicate 138
  OPC_Decode, 130, 31, 196, 5,               // 86510:         decode to FRINTAHr using decoder 708
                                             // 86510:        }
  7, 0,                                      // 86515:        case 0x7: {
  OPC_CheckPredicate, 138, 1,                // 86517:         check predicate 138
  OPC_Decode, 208, 31, 196, 5,               // 86520:         decode to FRINTXHr using decoder 708
                                             // 86520:        }
                                             // 86520:        } // switch Inst[20:16]
                                             // 86520:       }
  6, 0,                                      // 86525:       case 0x6: {
  OPC_SwitchField, 16, 5,                    // 86527:        switch Inst[20:16] {
  0, 8,                                      // 86530:        case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86532:         check predicate 138
  OPC_Decode, 192, 21, 196, 5,               // 86535:         decode to FABSHr using decoder 708
                                             // 86535:        }
  1, 8,                                      // 86540:        case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86542:         check predicate 138
  OPC_Decode, 150, 32, 196, 5,               // 86545:         decode to FSQRTHr using decoder 708
                                             // 86545:        }
  2, 7,                                      // 86550:        case 0x2: {
  OPC_CheckPredicate, 77,                    // 86552:         check predicate 77
  OPC_Decode, 229, 23, 197, 5,               // 86554:         decode to FCVTDHr using decoder 709
                                             // 86554:        }
  4, 8,                                      // 86559:        case 0x4: {
  OPC_CheckPredicate, 138, 1,                // 86561:         check predicate 138
  OPC_Decode, 192, 31, 196, 5,               // 86564:         decode to FRINTPHr using decoder 708
                                             // 86564:        }
  5, 8,                                      // 86569:        case 0x5: {
  OPC_CheckPredicate, 138, 1,                // 86571:         check predicate 138
  OPC_Decode, 222, 31, 196, 5,               // 86574:         decode to FRINTZHr using decoder 708
                                             // 86574:        }
  7, 0,                                      // 86579:        case 0x7: {
  OPC_CheckPredicate, 138, 1,                // 86581:         check predicate 138
  OPC_Decode, 146, 31, 196, 5,               // 86584:         decode to FRINTIHr using decoder 708
                                             // 86584:        }
                                             // 86584:        } // switch Inst[20:16]
                                             // 86584:       }
                                             // 86584:       } // switch Inst[15:13]
                                             // 86584:      }
  1, 0,                                      // 86589:      case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86591:       check predicate 138
  OPC_CheckField, 5, 5, 0,                   // 86594:       check Inst[9:5] == 0x0
  OPC_Decode, 203, 29, 198, 5,               // 86598:       decode to FMOVHi using decoder 710
                                             // 86598:      }
                                             // 86598:      } // switch Inst[12]
                                             // 86598:     }
  1, 23,                                     // 86603:     case 0x1: {
  OPC_SwitchField, 4, 1,                     // 86605:      switch Inst[4] {
  0, 8,                                      // 86608:      case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86610:       check predicate 138
  OPC_Decode, 182, 22, 199, 5,               // 86613:       decode to FCCMPHrr using decoder 711
                                             // 86613:      }
  1, 0,                                      // 86618:      case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86620:       check predicate 138
  OPC_Decode, 180, 22, 199, 5,               // 86623:       decode to FCCMPEHrr using decoder 711
                                             // 86623:      }
                                             // 86623:      } // switch Inst[4]
                                             // 86623:     }
  2, 93,                                     // 86628:     case 0x2: {
  OPC_SwitchField, 12, 4,                    // 86630:      switch Inst[15:12] {
  0, 8,                                      // 86633:      case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86635:       check predicate 138
  OPC_Decode, 225, 29, 200, 5,               // 86638:       decode to FMULHrr using decoder 712
                                             // 86638:      }
  1, 8,                                      // 86643:      case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86645:       check predicate 138
  OPC_Decode, 239, 25, 200, 5,               // 86648:       decode to FDIVHrr using decoder 712
                                             // 86648:      }
  2, 8,                                      // 86653:      case 0x2: {
  OPC_CheckPredicate, 138, 1,                // 86655:       check predicate 138
  OPC_Decode, 231, 21, 200, 5,               // 86658:       decode to FADDHrr using decoder 712
                                             // 86658:      }
  3, 8,                                      // 86663:      case 0x3: {
  OPC_CheckPredicate, 138, 1,                // 86665:       check predicate 138
  OPC_Decode, 164, 32, 200, 5,               // 86668:       decode to FSUBHrr using decoder 712
                                             // 86668:      }
  4, 8,                                      // 86673:      case 0x4: {
  OPC_CheckPredicate, 138, 1,                // 86675:       check predicate 138
  OPC_Decode, 184, 26, 200, 5,               // 86678:       decode to FMAXHrr using decoder 712
                                             // 86678:      }
  5, 8,                                      // 86683:      case 0x5: {
  OPC_CheckPredicate, 138, 1,                // 86685:       check predicate 138
  OPC_Decode, 148, 27, 200, 5,               // 86688:       decode to FMINHrr using decoder 712
                                             // 86688:      }
  6, 8,                                      // 86693:      case 0x6: {
  OPC_CheckPredicate, 138, 1,                // 86695:       check predicate 138
  OPC_Decode, 186, 26, 200, 5,               // 86698:       decode to FMAXNMHrr using decoder 712
                                             // 86698:      }
  7, 8,                                      // 86703:      case 0x7: {
  OPC_CheckPredicate, 138, 1,                // 86705:       check predicate 138
  OPC_Decode, 150, 27, 200, 5,               // 86708:       decode to FMINNMHrr using decoder 712
                                             // 86708:      }
  8, 0,                                      // 86713:      case 0x8: {
  OPC_CheckPredicate, 138, 1,                // 86715:       check predicate 138
  OPC_Decode, 188, 30, 200, 5,               // 86718:       decode to FNMULHrr using decoder 712
                                             // 86718:      }
                                             // 86718:      } // switch Inst[15:12]
                                             // 86718:     }
  3, 0,                                      // 86723:     case 0x3: {
  OPC_CheckPredicate, 138, 1,                // 86725:      check predicate 138
  OPC_Decode, 191, 23, 201, 5,               // 86728:      decode to FCSELHrrr using decoder 713
                                             // 86728:     }
                                             // 86728:     } // switch Inst[11:10]
                                             // 86728:    }
                                             // 86728:    } // switch Inst[23:21]
                                             // 86728:   }
  3, 0,                                      // 86733:   case 0x3: {
  OPC_SwitchField, 21, 3,                    // 86735:    switch Inst[23:21] {
  0, 21,                                     // 86738:    case 0x0: {
  OPC_SwitchField, 15, 1,                    // 86740:     switch Inst[15] {
  0, 7,                                      // 86743:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 86745:      check predicate 77
  OPC_Decode, 179, 26, 202, 5,               // 86747:      decode to FMADDSrrr using decoder 714
                                             // 86747:     }
  1, 0,                                      // 86752:     case 0x1: {
  OPC_CheckPredicate, 77,                    // 86754:      check predicate 77
  OPC_Decode, 223, 29, 202, 5,               // 86756:      decode to FMSUBSrrr using decoder 714
                                             // 86756:     }
                                             // 86756:     } // switch Inst[15]
                                             // 86756:    }
  1, 21,                                     // 86761:    case 0x1: {
  OPC_SwitchField, 15, 1,                    // 86763:     switch Inst[15] {
  0, 7,                                      // 86766:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 86768:      check predicate 77
  OPC_Decode, 171, 30, 202, 5,               // 86770:      decode to FNMADDSrrr using decoder 714
                                             // 86770:     }
  1, 0,                                      // 86775:     case 0x1: {
  OPC_CheckPredicate, 77,                    // 86777:      check predicate 77
  OPC_Decode, 186, 30, 202, 5,               // 86779:      decode to FNMSUBSrrr using decoder 714
                                             // 86779:     }
                                             // 86779:     } // switch Inst[15]
                                             // 86779:    }
  2, 21,                                     // 86784:    case 0x2: {
  OPC_SwitchField, 15, 1,                    // 86786:     switch Inst[15] {
  0, 7,                                      // 86789:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 86791:      check predicate 77
  OPC_Decode, 177, 26, 203, 5,               // 86793:      decode to FMADDDrrr using decoder 715
                                             // 86793:     }
  1, 0,                                      // 86798:     case 0x1: {
  OPC_CheckPredicate, 77,                    // 86800:      check predicate 77
  OPC_Decode, 221, 29, 203, 5,               // 86802:      decode to FMSUBDrrr using decoder 715
                                             // 86802:     }
                                             // 86802:     } // switch Inst[15]
                                             // 86802:    }
  3, 21,                                     // 86807:    case 0x3: {
  OPC_SwitchField, 15, 1,                    // 86809:     switch Inst[15] {
  0, 7,                                      // 86812:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 86814:      check predicate 77
  OPC_Decode, 169, 30, 203, 5,               // 86816:      decode to FNMADDDrrr using decoder 715
                                             // 86816:     }
  1, 0,                                      // 86821:     case 0x1: {
  OPC_CheckPredicate, 77,                    // 86823:      check predicate 77
  OPC_Decode, 184, 30, 203, 5,               // 86825:      decode to FNMSUBDrrr using decoder 715
                                             // 86825:     }
                                             // 86825:     } // switch Inst[15]
                                             // 86825:    }
  6, 23,                                     // 86830:    case 0x6: {
  OPC_SwitchField, 15, 1,                    // 86832:     switch Inst[15] {
  0, 8,                                      // 86835:     case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86837:      check predicate 138
  OPC_Decode, 178, 26, 204, 5,               // 86840:      decode to FMADDHrrr using decoder 716
                                             // 86840:     }
  1, 0,                                      // 86845:     case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86847:      check predicate 138
  OPC_Decode, 222, 29, 204, 5,               // 86850:      decode to FMSUBHrrr using decoder 716
                                             // 86850:     }
                                             // 86850:     } // switch Inst[15]
                                             // 86850:    }
  7, 0,                                      // 86855:    case 0x7: {
  OPC_SwitchField, 15, 1,                    // 86857:     switch Inst[15] {
  0, 8,                                      // 86860:     case 0x0: {
  OPC_CheckPredicate, 138, 1,                // 86862:      check predicate 138
  OPC_Decode, 170, 30, 204, 5,               // 86865:      decode to FNMADDHrrr using decoder 716
                                             // 86865:     }
  1, 0,                                      // 86870:     case 0x1: {
  OPC_CheckPredicate, 138, 1,                // 86872:      check predicate 138
  OPC_Decode, 185, 30, 204, 5,               // 86875:      decode to FNMSUBHrrr using decoder 716
                                             // 86875:     }
                                             // 86875:     } // switch Inst[15]
                                             // 86875:    }
                                             // 86875:    } // switch Inst[23:21]
                                             // 86875:   }
                                             // 86875:   } // switch Inst[25:24]
                                             // 86875:  }
  1, 197, 5,                                 // 86880:  case 0x1: {
  OPC_SwitchField, 22, 4,                    // 86883:   switch Inst[25:22] {
  0, 200, 1,                                 // 86886:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 86889:    switch Inst[11:10] {
  0, 137, 1,                                 // 86892:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 86895:     switch Inst[21] {
  0, 7,                                      // 86898:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 86900:      check predicate 77
  OPC_Decode, 231, 59, 247, 4,               // 86902:      decode to STURBi using decoder 631
                                             // 86902:     }
  1, 0,                                      // 86907:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 86909:      switch Inst[15:12] {
  0, 8,                                      // 86912:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 86914:       check predicate 139
  OPC_Decode, 187, 37, 205, 5,               // 86917:       decode to LDBFADD using decoder 717
                                             // 86917:      }
  4, 8,                                      // 86922:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 86924:       check predicate 139
  OPC_Decode, 191, 37, 205, 5,               // 86927:       decode to LDBFMAX using decoder 717
                                             // 86927:      }
  5, 8,                                      // 86932:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 86934:       check predicate 139
  OPC_Decode, 199, 37, 205, 5,               // 86937:       decode to LDBFMIN using decoder 717
                                             // 86937:      }
  6, 8,                                      // 86942:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 86944:       check predicate 139
  OPC_Decode, 195, 37, 205, 5,               // 86947:       decode to LDBFMAXNM using decoder 717
                                             // 86947:      }
  7, 8,                                      // 86952:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 86954:       check predicate 139
  OPC_Decode, 203, 37, 205, 5,               // 86957:       decode to LDBFMINNM using decoder 717
                                             // 86957:      }
  8, 12,                                     // 86962:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 86964:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 86967:       check Inst[4:0] == 0x1f
  OPC_Decode, 144, 58, 206, 5,               // 86971:       decode to STBFADD using decoder 718
                                             // 86971:      }
  12, 12,                                    // 86976:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 86978:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 86981:       check Inst[4:0] == 0x1f
  OPC_Decode, 146, 58, 206, 5,               // 86985:       decode to STBFMAX using decoder 718
                                             // 86985:      }
  13, 12,                                    // 86990:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 86992:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 86995:       check Inst[4:0] == 0x1f
  OPC_Decode, 150, 58, 206, 5,               // 86999:       decode to STBFMIN using decoder 718
                                             // 86999:      }
  14, 12,                                    // 87004:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 87006:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87009:       check Inst[4:0] == 0x1f
  OPC_Decode, 148, 58, 206, 5,               // 87013:       decode to STBFMAXNM using decoder 718
                                             // 87013:      }
  15, 0,                                     // 87018:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 87020:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87023:       check Inst[4:0] == 0x1f
  OPC_Decode, 152, 58, 206, 5,               // 87027:       decode to STBFMINNM using decoder 718
                                             // 87027:      }
                                             // 87027:      } // switch Inst[15:12]
                                             // 87027:     }
                                             // 87027:     } // switch Inst[21]
                                             // 87027:    }
  1, 11,                                     // 87032:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 87034:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87036:     check Inst[21] == 0x0
  OPC_Decode, 171, 59, 247, 4,               // 87040:     decode to STRBpost using decoder 631
                                             // 87040:    }
  2, 29,                                     // 87045:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 87047:     switch Inst[14:13] {
  2, 11,                                     // 87050:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 87052:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87054:      check Inst[21] == 0x1
  OPC_Decode, 173, 59, 207, 5,               // 87058:      decode to STRBroW using decoder 719
                                             // 87058:     }
  3, 0,                                      // 87063:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 87065:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87067:      check Inst[21] == 0x1
  OPC_Decode, 174, 59, 208, 5,               // 87071:      decode to STRBroX using decoder 720
                                             // 87071:     }
                                             // 87071:     } // switch Inst[14:13]
                                             // 87071:    }
  3, 0,                                      // 87076:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 87078:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87080:     check Inst[21] == 0x0
  OPC_Decode, 172, 59, 247, 4,               // 87084:     decode to STRBpre using decoder 631
                                             // 87084:    }
                                             // 87084:    } // switch Inst[11:10]
                                             // 87084:   }
  1, 200, 1,                                 // 87089:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 87092:    switch Inst[11:10] {
  0, 137, 1,                                 // 87095:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 87098:     switch Inst[21] {
  0, 7,                                      // 87101:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 87103:      check predicate 77
  OPC_Decode, 248, 40, 247, 4,               // 87105:      decode to LDURBi using decoder 631
                                             // 87105:     }
  1, 0,                                      // 87110:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 87112:      switch Inst[15:12] {
  0, 8,                                      // 87115:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 87117:       check predicate 139
  OPC_Decode, 190, 37, 205, 5,               // 87120:       decode to LDBFADDL using decoder 717
                                             // 87120:      }
  4, 8,                                      // 87125:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 87127:       check predicate 139
  OPC_Decode, 194, 37, 205, 5,               // 87130:       decode to LDBFMAXL using decoder 717
                                             // 87130:      }
  5, 8,                                      // 87135:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 87137:       check predicate 139
  OPC_Decode, 202, 37, 205, 5,               // 87140:       decode to LDBFMINL using decoder 717
                                             // 87140:      }
  6, 8,                                      // 87145:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 87147:       check predicate 139
  OPC_Decode, 198, 37, 205, 5,               // 87150:       decode to LDBFMAXNML using decoder 717
                                             // 87150:      }
  7, 8,                                      // 87155:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 87157:       check predicate 139
  OPC_Decode, 206, 37, 205, 5,               // 87160:       decode to LDBFMINNML using decoder 717
                                             // 87160:      }
  8, 12,                                     // 87165:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 87167:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87170:       check Inst[4:0] == 0x1f
  OPC_Decode, 145, 58, 206, 5,               // 87174:       decode to STBFADDL using decoder 718
                                             // 87174:      }
  12, 12,                                    // 87179:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 87181:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87184:       check Inst[4:0] == 0x1f
  OPC_Decode, 147, 58, 206, 5,               // 87188:       decode to STBFMAXL using decoder 718
                                             // 87188:      }
  13, 12,                                    // 87193:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 87195:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87198:       check Inst[4:0] == 0x1f
  OPC_Decode, 151, 58, 206, 5,               // 87202:       decode to STBFMINL using decoder 718
                                             // 87202:      }
  14, 12,                                    // 87207:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 87209:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87212:       check Inst[4:0] == 0x1f
  OPC_Decode, 149, 58, 206, 5,               // 87216:       decode to STBFMAXNML using decoder 718
                                             // 87216:      }
  15, 0,                                     // 87221:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 87223:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 87226:       check Inst[4:0] == 0x1f
  OPC_Decode, 153, 58, 206, 5,               // 87230:       decode to STBFMINNML using decoder 718
                                             // 87230:      }
                                             // 87230:      } // switch Inst[15:12]
                                             // 87230:     }
                                             // 87230:     } // switch Inst[21]
                                             // 87230:    }
  1, 11,                                     // 87235:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 87237:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87239:     check Inst[21] == 0x0
  OPC_Decode, 173, 39, 247, 4,               // 87243:     decode to LDRBpost using decoder 631
                                             // 87243:    }
  2, 29,                                     // 87248:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 87250:     switch Inst[14:13] {
  2, 11,                                     // 87253:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 87255:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87257:      check Inst[21] == 0x1
  OPC_Decode, 175, 39, 207, 5,               // 87261:      decode to LDRBroW using decoder 719
                                             // 87261:     }
  3, 0,                                      // 87266:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 87268:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87270:      check Inst[21] == 0x1
  OPC_Decode, 176, 39, 208, 5,               // 87274:      decode to LDRBroX using decoder 720
                                             // 87274:     }
                                             // 87274:     } // switch Inst[14:13]
                                             // 87274:    }
  3, 0,                                      // 87279:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 87281:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87283:     check Inst[21] == 0x0
  OPC_Decode, 174, 39, 247, 4,               // 87287:     decode to LDRBpre using decoder 631
                                             // 87287:    }
                                             // 87287:    } // switch Inst[11:10]
                                             // 87287:   }
  2, 129, 1,                                 // 87292:   case 0x2: {
  OPC_SwitchField, 10, 2,                    // 87295:    switch Inst[11:10] {
  0, 67,                                     // 87298:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 87300:     switch Inst[21] {
  0, 7,                                      // 87303:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 87305:      check predicate 77
  OPC_Decode, 235, 59, 247, 4,               // 87307:      decode to STURQi using decoder 631
                                             // 87307:     }
  1, 0,                                      // 87312:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 87314:      switch Inst[15:12] {
  0, 8,                                      // 87317:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 87319:       check predicate 139
  OPC_Decode, 188, 37, 205, 5,               // 87322:       decode to LDBFADDA using decoder 717
                                             // 87322:      }
  4, 8,                                      // 87327:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 87329:       check predicate 139
  OPC_Decode, 192, 37, 205, 5,               // 87332:       decode to LDBFMAXA using decoder 717
                                             // 87332:      }
  5, 8,                                      // 87337:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 87339:       check predicate 139
  OPC_Decode, 200, 37, 205, 5,               // 87342:       decode to LDBFMINA using decoder 717
                                             // 87342:      }
  6, 8,                                      // 87347:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 87349:       check predicate 139
  OPC_Decode, 196, 37, 205, 5,               // 87352:       decode to LDBFMAXNMA using decoder 717
                                             // 87352:      }
  7, 0,                                      // 87357:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 87359:       check predicate 139
  OPC_Decode, 204, 37, 205, 5,               // 87362:       decode to LDBFMINNMA using decoder 717
                                             // 87362:      }
                                             // 87362:      } // switch Inst[15:12]
                                             // 87362:     }
                                             // 87362:     } // switch Inst[21]
                                             // 87362:    }
  1, 11,                                     // 87367:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 87369:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87371:     check Inst[21] == 0x0
  OPC_Decode, 191, 59, 247, 4,               // 87375:     decode to STRQpost using decoder 631
                                             // 87375:    }
  2, 29,                                     // 87380:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 87382:     switch Inst[14:13] {
  2, 11,                                     // 87385:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 87387:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87389:      check Inst[21] == 0x1
  OPC_Decode, 193, 59, 209, 5,               // 87393:      decode to STRQroW using decoder 721
                                             // 87393:     }
  3, 0,                                      // 87398:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 87400:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87402:      check Inst[21] == 0x1
  OPC_Decode, 194, 59, 210, 5,               // 87406:      decode to STRQroX using decoder 722
                                             // 87406:     }
                                             // 87406:     } // switch Inst[14:13]
                                             // 87406:    }
  3, 0,                                      // 87411:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 87413:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87415:     check Inst[21] == 0x0
  OPC_Decode, 192, 59, 247, 4,               // 87419:     decode to STRQpre using decoder 631
                                             // 87419:    }
                                             // 87419:    } // switch Inst[11:10]
                                             // 87419:   }
  3, 129, 1,                                 // 87424:   case 0x3: {
  OPC_SwitchField, 10, 2,                    // 87427:    switch Inst[11:10] {
  0, 67,                                     // 87430:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 87432:     switch Inst[21] {
  0, 7,                                      // 87435:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 87437:      check predicate 77
  OPC_Decode, 252, 40, 247, 4,               // 87439:      decode to LDURQi using decoder 631
                                             // 87439:     }
  1, 0,                                      // 87444:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 87446:      switch Inst[15:12] {
  0, 8,                                      // 87449:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 87451:       check predicate 139
  OPC_Decode, 189, 37, 205, 5,               // 87454:       decode to LDBFADDAL using decoder 717
                                             // 87454:      }
  4, 8,                                      // 87459:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 87461:       check predicate 139
  OPC_Decode, 193, 37, 205, 5,               // 87464:       decode to LDBFMAXAL using decoder 717
                                             // 87464:      }
  5, 8,                                      // 87469:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 87471:       check predicate 139
  OPC_Decode, 201, 37, 205, 5,               // 87474:       decode to LDBFMINAL using decoder 717
                                             // 87474:      }
  6, 8,                                      // 87479:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 87481:       check predicate 139
  OPC_Decode, 197, 37, 205, 5,               // 87484:       decode to LDBFMAXNMAL using decoder 717
                                             // 87484:      }
  7, 0,                                      // 87489:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 87491:       check predicate 139
  OPC_Decode, 205, 37, 205, 5,               // 87494:       decode to LDBFMINNMAL using decoder 717
                                             // 87494:      }
                                             // 87494:      } // switch Inst[15:12]
                                             // 87494:     }
                                             // 87494:     } // switch Inst[21]
                                             // 87494:    }
  1, 11,                                     // 87499:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 87501:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87503:     check Inst[21] == 0x0
  OPC_Decode, 195, 39, 247, 4,               // 87507:     decode to LDRQpost using decoder 631
                                             // 87507:    }
  2, 29,                                     // 87512:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 87514:     switch Inst[14:13] {
  2, 11,                                     // 87517:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 87519:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87521:      check Inst[21] == 0x1
  OPC_Decode, 197, 39, 209, 5,               // 87525:      decode to LDRQroW using decoder 721
                                             // 87525:     }
  3, 0,                                      // 87530:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 87532:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 87534:      check Inst[21] == 0x1
  OPC_Decode, 198, 39, 210, 5,               // 87538:      decode to LDRQroX using decoder 722
                                             // 87538:     }
                                             // 87538:     } // switch Inst[14:13]
                                             // 87538:    }
  3, 0,                                      // 87543:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 87545:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 87547:     check Inst[21] == 0x0
  OPC_Decode, 196, 39, 247, 4,               // 87551:     decode to LDRQpre using decoder 631
                                             // 87551:    }
                                             // 87551:    } // switch Inst[11:10]
                                             // 87551:   }
  4, 7,                                      // 87556:   case 0x4: {
  OPC_CheckPredicate, 77,                    // 87558:    check predicate 77
  OPC_Decode, 175, 59, 132, 5,               // 87560:    decode to STRBui using decoder 644
                                             // 87560:   }
  5, 7,                                      // 87565:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 87567:    check predicate 77
  OPC_Decode, 177, 39, 132, 5,               // 87569:    decode to LDRBui using decoder 644
                                             // 87569:   }
  6, 7,                                      // 87574:   case 0x6: {
  OPC_CheckPredicate, 77,                    // 87576:    check predicate 77
  OPC_Decode, 195, 59, 132, 5,               // 87578:    decode to STRQui using decoder 644
                                             // 87578:   }
  7, 0,                                      // 87583:   case 0x7: {
  OPC_CheckPredicate, 77,                    // 87585:    check predicate 77
  OPC_Decode, 199, 39, 132, 5,               // 87587:    decode to LDRQui using decoder 644
                                             // 87587:   }
                                             // 87587:   } // switch Inst[25:22]
                                             // 87587:  }
  2, 146, 15,                                // 87592:  case 0x2: {
  OPC_SwitchField, 24, 2,                    // 87595:   switch Inst[25:24] {
  0, 7,                                      // 87598:   case 0x0: {
  OPC_CheckPredicate, 77,                    // 87600:    check predicate 77
  OPC_Decode, 178, 39, 211, 5,               // 87602:    decode to LDRDl using decoder 723
                                             // 87602:   }
  1, 29,                                     // 87607:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 87609:    switch Inst[23:21] {
  0, 11,                                     // 87612:    case 0x0: {
  OPC_CheckPredicate, 79,                    // 87614:     check predicate 79
  OPC_CheckField, 10, 2, 2,                  // 87616:     check Inst[11:10] == 0x2
  OPC_Decode, 216, 58, 212, 5,               // 87620:     decode to STLURhi using decoder 724
                                             // 87620:    }
  2, 0,                                      // 87625:    case 0x2: {
  OPC_CheckPredicate, 79,                    // 87627:     check predicate 79
  OPC_CheckField, 10, 2, 2,                  // 87629:     check Inst[11:10] == 0x2
  OPC_Decode, 170, 37, 212, 5,               // 87633:     decode to LDAPURhi using decoder 724
                                             // 87633:    }
                                             // 87633:    } // switch Inst[23:21]
                                             // 87633:   }
  2, 228, 9,                                 // 87638:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 87641:    switch Inst[15:10] {
  0, 12,                                     // 87644:    case 0x0: {
  OPC_CheckPredicate, 140, 1,                // 87646:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 87649:     check Inst[23:21] == 0x0
  OPC_Decode, 131, 48, 213, 5,               // 87653:     decode to SHA1Crrr using decoder 725
                                             // 87653:    }
  1, 69,                                     // 87658:    case 0x1: {
  OPC_SwitchField, 16, 1,                    // 87660:     switch Inst[16] {
  0, 51,                                     // 87663:     case 0x0: {
  OPC_SwitchField, 17, 1,                    // 87665:      switch Inst[17] {
  0, 33,                                     // 87668:      case 0x0: {
  OPC_SwitchField, 18, 1,                    // 87670:       switch Inst[18] {
  0, 15,                                     // 87673:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 87675:        check predicate 76
  OPC_CheckField, 21, 3, 0,                  // 87677:        check Inst[23:21] == 0x0
  OPC_CheckField, 19, 1, 1,                  // 87681:        check Inst[19] == 0x1
  OPC_Decode, 222, 20, 214, 5,               // 87685:        decode to DUPi64 using decoder 726
                                             // 87685:       }
  1, 0,                                      // 87690:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 87692:        check predicate 76
  OPC_CheckField, 21, 3, 0,                  // 87694:        check Inst[23:21] == 0x0
  OPC_Decode, 221, 20, 215, 5,               // 87698:        decode to DUPi32 using decoder 727
                                             // 87698:       }
                                             // 87698:       } // switch Inst[18]
                                             // 87698:      }
  1, 0,                                      // 87703:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 87705:       check predicate 76
  OPC_CheckField, 21, 3, 0,                  // 87707:       check Inst[23:21] == 0x0
  OPC_Decode, 220, 20, 216, 5,               // 87711:       decode to DUPi16 using decoder 728
                                             // 87711:      }
                                             // 87711:      } // switch Inst[17]
                                             // 87711:     }
  1, 0,                                      // 87716:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 87718:      check predicate 76
  OPC_CheckField, 21, 3, 0,                  // 87720:      check Inst[23:21] == 0x0
  OPC_Decode, 223, 20, 217, 5,               // 87724:      decode to DUPi8 using decoder 729
                                             // 87724:     }
                                             // 87724:     } // switch Inst[16]
                                             // 87724:    }
  2, 12,                                     // 87729:    case 0x2: {
  OPC_CheckPredicate, 140, 1,                // 87731:     check predicate 140
  OPC_CheckField, 16, 8, 40,                 // 87734:     check Inst[23:16] == 0x28
  OPC_Decode, 132, 48, 174, 5,               // 87738:     decode to SHA1Hrr using decoder 686
                                             // 87738:    }
  3, 39,                                     // 87743:    case 0x3: {
  OPC_SwitchField, 21, 3,                    // 87745:     switch Inst[23:21] {
  1, 7,                                      // 87748:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 87750:      check predicate 76
  OPC_Decode, 154, 51, 218, 5,               // 87752:      decode to SQADDv1i8 using decoder 730
                                             // 87752:     }
  3, 7,                                      // 87757:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 87759:      check predicate 76
  OPC_Decode, 151, 51, 200, 5,               // 87761:      decode to SQADDv1i16 using decoder 712
                                             // 87761:     }
  5, 7,                                      // 87766:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 87768:      check predicate 76
  OPC_Decode, 152, 51, 179, 5,               // 87770:      decode to SQADDv1i32 using decoder 691
                                             // 87770:     }
  7, 0,                                      // 87775:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 87777:      check predicate 76
  OPC_Decode, 153, 51, 200, 3,               // 87779:      decode to SQADDv1i64 using decoder 456
                                             // 87779:     }
                                             // 87779:     } // switch Inst[23:21]
                                             // 87779:    }
  4, 12,                                     // 87784:    case 0x4: {
  OPC_CheckPredicate, 140, 1,                // 87786:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 87789:     check Inst[23:21] == 0x0
  OPC_Decode, 134, 48, 213, 5,               // 87793:     decode to SHA1Prrr using decoder 725
                                             // 87793:    }
  6, 12,                                     // 87798:    case 0x6: {
  OPC_CheckPredicate, 140, 1,                // 87800:     check predicate 140
  OPC_CheckField, 16, 8, 40,                 // 87803:     check Inst[23:16] == 0x28
  OPC_Decode, 136, 48, 238, 3,               // 87807:     decode to SHA1SU1rr using decoder 494
                                             // 87807:    }
  7, 11,                                     // 87812:    case 0x7: {
  OPC_CheckPredicate, 81,                    // 87814:     check predicate 81
  OPC_CheckField, 21, 3, 2,                  // 87816:     check Inst[23:21] == 0x2
  OPC_Decode, 227, 29, 200, 5,               // 87820:     decode to FMULX16 using decoder 712
                                             // 87820:    }
  8, 12,                                     // 87825:    case 0x8: {
  OPC_CheckPredicate, 140, 1,                // 87827:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 87830:     check Inst[23:21] == 0x0
  OPC_Decode, 133, 48, 213, 5,               // 87834:     decode to SHA1Mrrr using decoder 725
                                             // 87834:    }
  9, 11,                                     // 87839:    case 0x9: {
  OPC_CheckPredicate, 81,                    // 87841:     check predicate 81
  OPC_CheckField, 21, 3, 2,                  // 87843:     check Inst[23:21] == 0x2
  OPC_Decode, 193, 22, 200, 5,               // 87847:     decode to FCMEQ16 using decoder 712
                                             // 87847:    }
  10, 12,                                    // 87852:    case 0xa: {
  OPC_CheckPredicate, 140, 1,                // 87854:     check predicate 140
  OPC_CheckField, 16, 8, 40,                 // 87857:     check Inst[23:16] == 0x28
  OPC_Decode, 139, 48, 238, 3,               // 87861:     decode to SHA256SU0rr using decoder 494
                                             // 87861:    }
  11, 39,                                    // 87866:    case 0xb: {
  OPC_SwitchField, 21, 3,                    // 87868:     switch Inst[23:21] {
  1, 7,                                      // 87871:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 87873:      check predicate 76
  OPC_Decode, 192, 54, 218, 5,               // 87875:      decode to SQSUBv1i8 using decoder 730
                                             // 87875:     }
  3, 7,                                      // 87880:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 87882:      check predicate 76
  OPC_Decode, 189, 54, 200, 5,               // 87884:      decode to SQSUBv1i16 using decoder 712
                                             // 87884:     }
  5, 7,                                      // 87889:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 87891:      check predicate 76
  OPC_Decode, 190, 54, 179, 5,               // 87893:      decode to SQSUBv1i32 using decoder 691
                                             // 87893:     }
  7, 0,                                      // 87898:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 87900:      check predicate 76
  OPC_Decode, 191, 54, 200, 3,               // 87902:      decode to SQSUBv1i64 using decoder 456
                                             // 87902:     }
                                             // 87902:     } // switch Inst[23:21]
                                             // 87902:    }
  12, 12,                                    // 87907:    case 0xc: {
  OPC_CheckPredicate, 140, 1,                // 87909:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 87912:     check Inst[23:21] == 0x0
  OPC_Decode, 135, 48, 223, 3,               // 87916:     decode to SHA1SU0rrr using decoder 479
                                             // 87916:    }
  13, 11,                                    // 87921:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 87923:     check predicate 76
  OPC_CheckField, 21, 3, 7,                  // 87925:     check Inst[23:21] == 0x7
  OPC_Decode, 241, 17, 200, 3,               // 87929:     decode to CMGTv1i64 using decoder 456
                                             // 87929:    }
  14, 41,                                    // 87934:    case 0xe: {
  OPC_SwitchField, 16, 8,                    // 87936:     switch Inst[23:16] {
  32, 7,                                     // 87939:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 87941:      check predicate 76
  OPC_Decode, 252, 60, 219, 5,               // 87943:      decode to SUQADDv1i8 using decoder 731
                                             // 87943:     }
  96, 7,                                     // 87948:     case 0x60: {
  OPC_CheckPredicate, 76,                    // 87950:      check predicate 76
  OPC_Decode, 249, 60, 220, 5,               // 87952:      decode to SUQADDv1i16 using decoder 732
                                             // 87952:     }
  160, 1, 7,                                 // 87957:     case 0xa0: {
  OPC_CheckPredicate, 76,                    // 87960:      check predicate 76
  OPC_Decode, 250, 60, 221, 5,               // 87962:      decode to SUQADDv1i32 using decoder 733
                                             // 87962:     }
  224, 1, 0,                                 // 87967:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 87970:      check predicate 76
  OPC_Decode, 251, 60, 211, 3,               // 87972:      decode to SUQADDv1i64 using decoder 467
                                             // 87972:     }
                                             // 87972:     } // switch Inst[23:16]
                                             // 87972:    }
  15, 30,                                    // 87977:    case 0xf: {
  OPC_SwitchField, 21, 3,                    // 87979:     switch Inst[23:21] {
  2, 7,                                      // 87982:     case 0x2: {
  OPC_CheckPredicate, 81,                    // 87984:      check predicate 81
  OPC_Decode, 201, 30, 200, 5,               // 87986:      decode to FRECPS16 using decoder 712
                                             // 87986:     }
  6, 7,                                      // 87991:     case 0x6: {
  OPC_CheckPredicate, 81,                    // 87993:      check predicate 81
  OPC_Decode, 246, 31, 200, 5,               // 87995:      decode to FRSQRTS16 using decoder 712
                                             // 87995:     }
  7, 0,                                      // 88000:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 88002:      check predicate 76
  OPC_Decode, 225, 17, 200, 3,               // 88004:      decode to CMGEv1i64 using decoder 456
                                             // 88004:     }
                                             // 88004:     } // switch Inst[23:21]
                                             // 88004:    }
  16, 12,                                    // 88009:    case 0x10: {
  OPC_CheckPredicate, 140, 1,                // 88011:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 88014:     check Inst[23:21] == 0x0
  OPC_Decode, 138, 48, 223, 3,               // 88018:     decode to SHA256Hrrr using decoder 479
                                             // 88018:    }
  17, 11,                                    // 88023:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 88025:     check predicate 76
  OPC_CheckField, 21, 3, 7,                  // 88027:     check Inst[23:21] == 0x7
  OPC_Decode, 192, 55, 200, 3,               // 88031:     decode to SSHLv1i64 using decoder 456
                                             // 88031:    }
  18, 31,                                    // 88036:    case 0x12: {
  OPC_SwitchField, 16, 8,                    // 88038:     switch Inst[23:16] {
  33, 7,                                     // 88041:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 88043:      check predicate 76
  OPC_Decode, 208, 54, 222, 5,               // 88045:      decode to SQXTNv1i8 using decoder 734
                                             // 88045:     }
  97, 7,                                     // 88050:     case 0x61: {
  OPC_CheckPredicate, 76,                    // 88052:      check predicate 76
  OPC_Decode, 206, 54, 176, 5,               // 88054:      decode to SQXTNv1i16 using decoder 688
                                             // 88054:     }
  161, 1, 0,                                 // 88059:     case 0xa1: {
  OPC_CheckPredicate, 76,                    // 88062:      check predicate 76
  OPC_Decode, 207, 54, 130, 4,               // 88064:      decode to SQXTNv1i32 using decoder 514
                                             // 88064:     }
                                             // 88064:     } // switch Inst[23:16]
                                             // 88064:    }
  19, 39,                                    // 88069:    case 0x13: {
  OPC_SwitchField, 21, 3,                    // 88071:     switch Inst[23:21] {
  1, 7,                                      // 88074:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 88076:      check predicate 76
  OPC_Decode, 253, 53, 218, 5,               // 88078:      decode to SQSHLv1i8 using decoder 730
                                             // 88078:     }
  3, 7,                                      // 88083:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88085:      check predicate 76
  OPC_Decode, 250, 53, 200, 5,               // 88087:      decode to SQSHLv1i16 using decoder 712
                                             // 88087:     }
  5, 7,                                      // 88092:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88094:      check predicate 76
  OPC_Decode, 251, 53, 179, 5,               // 88096:      decode to SQSHLv1i32 using decoder 691
                                             // 88096:     }
  7, 0,                                      // 88101:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 88103:      check predicate 76
  OPC_Decode, 252, 53, 200, 3,               // 88105:      decode to SQSHLv1i64 using decoder 456
                                             // 88105:     }
                                             // 88105:     } // switch Inst[23:21]
                                             // 88105:    }
  20, 12,                                    // 88110:    case 0x14: {
  OPC_CheckPredicate, 140, 1,                // 88112:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 88115:     check Inst[23:21] == 0x0
  OPC_Decode, 137, 48, 223, 3,               // 88119:     decode to SHA256H2rrr using decoder 479
                                             // 88119:    }
  21, 11,                                    // 88124:    case 0x15: {
  OPC_CheckPredicate, 76,                    // 88126:     check predicate 76
  OPC_CheckField, 21, 3, 7,                  // 88128:     check Inst[23:21] == 0x7
  OPC_Decode, 148, 55, 200, 3,               // 88132:     decode to SRSHLv1i64 using decoder 456
                                             // 88132:    }
  23, 39,                                    // 88137:    case 0x17: {
  OPC_SwitchField, 21, 3,                    // 88139:     switch Inst[23:21] {
  1, 7,                                      // 88142:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 88144:      check predicate 76
  OPC_Decode, 166, 53, 218, 5,               // 88146:      decode to SQRSHLv1i8 using decoder 730
                                             // 88146:     }
  3, 7,                                      // 88151:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88153:      check predicate 76
  OPC_Decode, 163, 53, 200, 5,               // 88155:      decode to SQRSHLv1i16 using decoder 712
                                             // 88155:     }
  5, 7,                                      // 88160:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88162:      check predicate 76
  OPC_Decode, 164, 53, 179, 5,               // 88164:      decode to SQRSHLv1i32 using decoder 691
                                             // 88164:     }
  7, 0,                                      // 88169:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 88171:      check predicate 76
  OPC_Decode, 165, 53, 200, 3,               // 88173:      decode to SQRSHLv1i64 using decoder 456
                                             // 88173:     }
                                             // 88173:     } // switch Inst[23:21]
                                             // 88173:    }
  24, 12,                                    // 88178:    case 0x18: {
  OPC_CheckPredicate, 140, 1,                // 88180:     check predicate 140
  OPC_CheckField, 21, 3, 0,                  // 88183:     check Inst[23:21] == 0x0
  OPC_Decode, 140, 48, 223, 3,               // 88187:     decode to SHA256SU1rrr using decoder 479
                                             // 88187:    }
  30, 41,                                    // 88192:    case 0x1e: {
  OPC_SwitchField, 16, 8,                    // 88194:     switch Inst[23:16] {
  32, 7,                                     // 88197:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 88199:      check predicate 76
  OPC_Decode, 131, 51, 223, 5,               // 88201:      decode to SQABSv1i8 using decoder 735
                                             // 88201:     }
  96, 7,                                     // 88206:     case 0x60: {
  OPC_CheckPredicate, 76,                    // 88208:      check predicate 76
  OPC_Decode, 128, 51, 196, 5,               // 88210:      decode to SQABSv1i16 using decoder 708
                                             // 88210:     }
  160, 1, 7,                                 // 88215:     case 0xa0: {
  OPC_CheckPredicate, 76,                    // 88218:      check predicate 76
  OPC_Decode, 129, 51, 174, 5,               // 88220:      decode to SQABSv1i32 using decoder 686
                                             // 88220:     }
  224, 1, 0,                                 // 88225:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 88228:      check predicate 76
  OPC_Decode, 130, 51, 201, 3,               // 88230:      decode to SQABSv1i64 using decoder 457
                                             // 88230:     }
                                             // 88230:     } // switch Inst[23:16]
                                             // 88230:    }
  33, 11,                                    // 88235:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 88237:     check predicate 76
  OPC_CheckField, 21, 3, 7,                  // 88239:     check Inst[23:21] == 0x7
  OPC_Decode, 234, 13, 200, 3,               // 88243:     decode to ADDv1i64 using decoder 456
                                             // 88243:    }
  34, 12,                                    // 88248:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 88250:     check predicate 76
  OPC_CheckField, 16, 8, 224, 1,             // 88252:     check Inst[23:16] == 0xe0
  OPC_Decode, 242, 17, 201, 3,               // 88257:     decode to CMGTv1i64rz using decoder 457
                                             // 88257:    }
  35, 11,                                    // 88262:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 88264:     check predicate 76
  OPC_CheckField, 21, 3, 7,                  // 88266:     check Inst[23:21] == 0x7
  OPC_Decode, 132, 19, 200, 3,               // 88270:     decode to CMTSTv1i64 using decoder 456
                                             // 88270:    }
  36, 21,                                    // 88275:    case 0x24: {
  OPC_SwitchField, 21, 3,                    // 88277:     switch Inst[23:21] {
  3, 7,                                      // 88280:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88282:      check predicate 76
  OPC_Decode, 212, 51, 224, 5,               // 88284:      decode to SQDMLALi16 using decoder 736
                                             // 88284:     }
  5, 0,                                      // 88289:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88291:      check predicate 76
  OPC_Decode, 213, 51, 225, 5,               // 88293:      decode to SQDMLALi32 using decoder 737
                                             // 88293:     }
                                             // 88293:     } // switch Inst[23:21]
                                             // 88293:    }
  38, 12,                                    // 88298:    case 0x26: {
  OPC_CheckPredicate, 76,                    // 88300:     check predicate 76
  OPC_CheckField, 16, 8, 224, 1,             // 88302:     check Inst[23:16] == 0xe0
  OPC_Decode, 210, 17, 201, 3,               // 88307:     decode to CMEQv1i64rz using decoder 457
                                             // 88307:    }
  42, 70,                                    // 88312:    case 0x2a: {
  OPC_SwitchField, 16, 8,                    // 88314:     switch Inst[23:16] {
  33, 7,                                     // 88317:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 88319:      check predicate 76
  OPC_Decode, 162, 24, 174, 5,               // 88321:      decode to FCVTNSv1i32 using decoder 686
                                             // 88321:     }
  97, 7,                                     // 88326:     case 0x61: {
  OPC_CheckPredicate, 76,                    // 88328:      check predicate 76
  OPC_Decode, 163, 24, 201, 3,               // 88330:      decode to FCVTNSv1i64 using decoder 457
                                             // 88330:     }
  121, 7,                                    // 88335:     case 0x79: {
  OPC_CheckPredicate, 81,                    // 88337:      check predicate 81
  OPC_Decode, 161, 24, 196, 5,               // 88339:      decode to FCVTNSv1f16 using decoder 708
                                             // 88339:     }
  161, 1, 7,                                 // 88344:     case 0xa1: {
  OPC_CheckPredicate, 76,                    // 88347:      check predicate 76
  OPC_Decode, 214, 24, 174, 5,               // 88349:      decode to FCVTPSv1i32 using decoder 686
                                             // 88349:     }
  224, 1, 7,                                 // 88354:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 88357:      check predicate 76
  OPC_Decode, 158, 18, 201, 3,               // 88359:      decode to CMLTv1i64rz using decoder 457
                                             // 88359:     }
  225, 1, 7,                                 // 88364:     case 0xe1: {
  OPC_CheckPredicate, 76,                    // 88367:      check predicate 76
  OPC_Decode, 215, 24, 201, 3,               // 88369:      decode to FCVTPSv1i64 using decoder 457
                                             // 88369:     }
  249, 1, 0,                                 // 88374:     case 0xf9: {
  OPC_CheckPredicate, 81,                    // 88377:      check predicate 81
  OPC_Decode, 213, 24, 196, 5,               // 88379:      decode to FCVTPSv1f16 using decoder 708
                                             // 88379:     }
                                             // 88379:     } // switch Inst[23:16]
                                             // 88379:    }
  44, 21,                                    // 88384:    case 0x2c: {
  OPC_SwitchField, 21, 3,                    // 88386:     switch Inst[23:21] {
  3, 7,                                      // 88389:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88391:      check predicate 76
  OPC_Decode, 237, 51, 224, 5,               // 88393:      decode to SQDMLSLi16 using decoder 736
                                             // 88393:     }
  5, 0,                                      // 88398:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88400:      check predicate 76
  OPC_Decode, 238, 51, 225, 5,               // 88402:      decode to SQDMLSLi32 using decoder 737
                                             // 88402:     }
                                             // 88402:     } // switch Inst[23:21]
                                             // 88402:    }
  45, 21,                                    // 88407:    case 0x2d: {
  OPC_SwitchField, 21, 3,                    // 88409:     switch Inst[23:21] {
  3, 7,                                      // 88412:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88414:      check predicate 76
  OPC_Decode, 144, 52, 200, 5,               // 88416:      decode to SQDMULHv1i16 using decoder 712
                                             // 88416:     }
  5, 0,                                      // 88421:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88423:      check predicate 76
  OPC_Decode, 146, 52, 179, 5,               // 88425:      decode to SQDMULHv1i32 using decoder 691
                                             // 88425:     }
                                             // 88425:     } // switch Inst[23:21]
                                             // 88425:    }
  46, 80,                                    // 88430:    case 0x2e: {
  OPC_SwitchField, 16, 8,                    // 88432:     switch Inst[23:16] {
  33, 7,                                     // 88435:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 88437:      check predicate 76
  OPC_Decode, 253, 23, 174, 5,               // 88439:      decode to FCVTMSv1i32 using decoder 686
                                             // 88439:     }
  97, 7,                                     // 88444:     case 0x61: {
  OPC_CheckPredicate, 76,                    // 88446:      check predicate 76
  OPC_Decode, 254, 23, 201, 3,               // 88448:      decode to FCVTMSv1i64 using decoder 457
                                             // 88448:     }
  121, 7,                                    // 88453:     case 0x79: {
  OPC_CheckPredicate, 81,                    // 88455:      check predicate 81
  OPC_Decode, 252, 23, 196, 5,               // 88457:      decode to FCVTMSv1f16 using decoder 708
                                             // 88457:     }
  161, 1, 7,                                 // 88462:     case 0xa1: {
  OPC_CheckPredicate, 76,                    // 88465:      check predicate 76
  OPC_Decode, 159, 25, 174, 5,               // 88467:      decode to FCVTZSv1i32 using decoder 686
                                             // 88467:     }
  224, 1, 7,                                 // 88472:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 88475:      check predicate 76
  OPC_Decode, 247, 12, 201, 3,               // 88477:      decode to ABSv1i64 using decoder 457
                                             // 88477:     }
  225, 1, 7,                                 // 88482:     case 0xe1: {
  OPC_CheckPredicate, 76,                    // 88485:      check predicate 76
  OPC_Decode, 160, 25, 201, 3,               // 88487:      decode to FCVTZSv1i64 using decoder 457
                                             // 88487:     }
  241, 1, 7,                                 // 88492:     case 0xf1: {
  OPC_CheckPredicate, 76,                    // 88495:      check predicate 76
  OPC_Decode, 158, 13, 206, 3,               // 88497:      decode to ADDPv2i64p using decoder 462
                                             // 88497:     }
  249, 1, 0,                                 // 88502:     case 0xf9: {
  OPC_CheckPredicate, 81,                    // 88505:      check predicate 81
  OPC_Decode, 158, 25, 196, 5,               // 88507:      decode to FCVTZSv1f16 using decoder 708
                                             // 88507:     }
                                             // 88507:     } // switch Inst[23:16]
                                             // 88507:    }
  50, 79,                                    // 88512:    case 0x32: {
  OPC_SwitchField, 16, 8,                    // 88514:     switch Inst[23:16] {
  33, 7,                                     // 88517:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 88519:      check predicate 76
  OPC_Decode, 204, 23, 174, 5,               // 88521:      decode to FCVTASv1i32 using decoder 686
                                             // 88521:     }
  48, 7,                                     // 88526:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 88528:      check predicate 81
  OPC_Decode, 192, 26, 212, 3,               // 88530:      decode to FMAXNMPv2i16p using decoder 468
                                             // 88530:     }
  97, 7,                                     // 88535:     case 0x61: {
  OPC_CheckPredicate, 76,                    // 88537:      check predicate 76
  OPC_Decode, 205, 23, 201, 3,               // 88539:      decode to FCVTASv1i64 using decoder 457
                                             // 88539:     }
  121, 7,                                    // 88544:     case 0x79: {
  OPC_CheckPredicate, 81,                    // 88546:      check predicate 81
  OPC_Decode, 203, 23, 196, 5,               // 88548:      decode to FCVTASv1f16 using decoder 708
                                             // 88548:     }
  160, 1, 7,                                 // 88553:     case 0xa0: {
  OPC_CheckPredicate, 76,                    // 88556:      check predicate 76
  OPC_Decode, 247, 22, 174, 5,               // 88558:      decode to FCMGTv1i32rz using decoder 686
                                             // 88558:     }
  176, 1, 7,                                 // 88563:     case 0xb0: {
  OPC_CheckPredicate, 81,                    // 88566:      check predicate 81
  OPC_Decode, 156, 27, 212, 3,               // 88568:      decode to FMINNMPv2i16p using decoder 468
                                             // 88568:     }
  224, 1, 7,                                 // 88573:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 88576:      check predicate 76
  OPC_Decode, 248, 22, 201, 3,               // 88578:      decode to FCMGTv1i64rz using decoder 457
                                             // 88578:     }
  248, 1, 0,                                 // 88583:     case 0xf8: {
  OPC_CheckPredicate, 81,                    // 88586:      check predicate 81
  OPC_Decode, 246, 22, 196, 5,               // 88588:      decode to FCMGTv1i16rz using decoder 708
                                             // 88588:     }
                                             // 88588:     } // switch Inst[23:16]
                                             // 88588:    }
  52, 21,                                    // 88593:    case 0x34: {
  OPC_SwitchField, 21, 3,                    // 88595:     switch Inst[23:21] {
  3, 7,                                      // 88598:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88600:      check predicate 76
  OPC_Decode, 166, 52, 226, 5,               // 88602:      decode to SQDMULLi16 using decoder 738
                                             // 88602:     }
  5, 0,                                      // 88607:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88609:      check predicate 76
  OPC_Decode, 167, 52, 227, 5,               // 88611:      decode to SQDMULLi32 using decoder 739
                                             // 88611:     }
                                             // 88611:     } // switch Inst[23:21]
                                             // 88611:    }
  54, 99,                                    // 88616:    case 0x36: {
  OPC_SwitchField, 16, 8,                    // 88618:     switch Inst[23:16] {
  33, 7,                                     // 88621:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 88623:      check predicate 76
  OPC_Decode, 163, 47, 174, 5,               // 88625:      decode to SCVTFv1i32 using decoder 686
                                             // 88625:     }
  48, 7,                                     // 88630:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 88632:      check predicate 81
  OPC_Decode, 237, 21, 212, 3,               // 88634:      decode to FADDPv2i16p using decoder 468
                                             // 88634:     }
  97, 7,                                     // 88639:     case 0x61: {
  OPC_CheckPredicate, 76,                    // 88641:      check predicate 76
  OPC_Decode, 164, 47, 201, 3,               // 88643:      decode to SCVTFv1i64 using decoder 457
                                             // 88643:     }
  121, 7,                                    // 88648:     case 0x79: {
  OPC_CheckPredicate, 81,                    // 88650:      check predicate 81
  OPC_Decode, 162, 47, 196, 5,               // 88652:      decode to SCVTFv1i16 using decoder 708
                                             // 88652:     }
  160, 1, 7,                                 // 88657:     case 0xa0: {
  OPC_CheckPredicate, 76,                    // 88660:      check predicate 76
  OPC_Decode, 203, 22, 174, 5,               // 88662:      decode to FCMEQv1i32rz using decoder 686
                                             // 88662:     }
  161, 1, 7,                                 // 88667:     case 0xa1: {
  OPC_CheckPredicate, 76,                    // 88670:      check predicate 76
  OPC_Decode, 194, 30, 174, 5,               // 88672:      decode to FRECPEv1i32 using decoder 686
                                             // 88672:     }
  224, 1, 7,                                 // 88677:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 88680:      check predicate 76
  OPC_Decode, 204, 22, 201, 3,               // 88682:      decode to FCMEQv1i64rz using decoder 457
                                             // 88682:     }
  225, 1, 7,                                 // 88687:     case 0xe1: {
  OPC_CheckPredicate, 76,                    // 88690:      check predicate 76
  OPC_Decode, 195, 30, 201, 3,               // 88692:      decode to FRECPEv1i64 using decoder 457
                                             // 88692:     }
  248, 1, 7,                                 // 88697:     case 0xf8: {
  OPC_CheckPredicate, 81,                    // 88700:      check predicate 81
  OPC_Decode, 202, 22, 196, 5,               // 88702:      decode to FCMEQv1i16rz using decoder 708
                                             // 88702:     }
  249, 1, 0,                                 // 88707:     case 0xf9: {
  OPC_CheckPredicate, 81,                    // 88710:      check predicate 81
  OPC_Decode, 193, 30, 196, 5,               // 88712:      decode to FRECPEv1f16 using decoder 708
                                             // 88712:     }
                                             // 88712:     } // switch Inst[23:16]
                                             // 88712:    }
  55, 21,                                    // 88717:    case 0x37: {
  OPC_SwitchField, 21, 3,                    // 88719:     switch Inst[23:21] {
  1, 7,                                      // 88722:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 88724:      check predicate 76
  OPC_Decode, 228, 29, 179, 5,               // 88726:      decode to FMULX32 using decoder 691
                                             // 88726:     }
  3, 0,                                      // 88731:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88733:      check predicate 76
  OPC_Decode, 229, 29, 200, 3,               // 88735:      decode to FMULX64 using decoder 456
                                             // 88735:     }
                                             // 88735:     } // switch Inst[23:21]
                                             // 88735:    }
  57, 21,                                    // 88740:    case 0x39: {
  OPC_SwitchField, 21, 3,                    // 88742:     switch Inst[23:21] {
  1, 7,                                      // 88745:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 88747:      check predicate 76
  OPC_Decode, 194, 22, 179, 5,               // 88749:      decode to FCMEQ32 using decoder 691
                                             // 88749:     }
  3, 0,                                      // 88754:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88756:      check predicate 76
  OPC_Decode, 195, 22, 200, 3,               // 88758:      decode to FCMEQ64 using decoder 456
                                             // 88758:     }
                                             // 88758:     } // switch Inst[23:21]
                                             // 88758:    }
  58, 33,                                    // 88763:    case 0x3a: {
  OPC_SwitchField, 16, 8,                    // 88765:     switch Inst[23:16] {
  160, 1, 7,                                 // 88768:     case 0xa0: {
  OPC_CheckPredicate, 76,                    // 88771:      check predicate 76
  OPC_Decode, 159, 23, 174, 5,               // 88773:      decode to FCMLTv1i32rz using decoder 686
                                             // 88773:     }
  224, 1, 7,                                 // 88778:     case 0xe0: {
  OPC_CheckPredicate, 76,                    // 88781:      check predicate 76
  OPC_Decode, 160, 23, 201, 3,               // 88783:      decode to FCMLTv1i64rz using decoder 457
                                             // 88783:     }
  248, 1, 0,                                 // 88788:     case 0xf8: {
  OPC_CheckPredicate, 81,                    // 88791:      check predicate 81
  OPC_Decode, 158, 23, 196, 5,               // 88793:      decode to FCMLTv1i16rz using decoder 708
                                             // 88793:     }
                                             // 88793:     } // switch Inst[23:16]
                                             // 88793:    }
  62, 52,                                    // 88798:    case 0x3e: {
  OPC_SwitchField, 16, 8,                    // 88800:     switch Inst[23:16] {
  48, 7,                                     // 88803:     case 0x30: {
  OPC_CheckPredicate, 81,                    // 88805:      check predicate 81
  OPC_Decode, 236, 26, 212, 3,               // 88807:      decode to FMAXPv2i16p using decoder 468
                                             // 88807:     }
  161, 1, 7,                                 // 88812:     case 0xa1: {
  OPC_CheckPredicate, 76,                    // 88815:      check predicate 76
  OPC_Decode, 219, 30, 174, 5,               // 88817:      decode to FRECPXv1i32 using decoder 686
                                             // 88817:     }
  176, 1, 7,                                 // 88822:     case 0xb0: {
  OPC_CheckPredicate, 81,                    // 88825:      check predicate 81
  OPC_Decode, 200, 27, 212, 3,               // 88827:      decode to FMINPv2i16p using decoder 468
                                             // 88827:     }
  225, 1, 7,                                 // 88832:     case 0xe1: {
  OPC_CheckPredicate, 76,                    // 88835:      check predicate 76
  OPC_Decode, 220, 30, 201, 3,               // 88837:      decode to FRECPXv1i64 using decoder 457
                                             // 88837:     }
  249, 1, 0,                                 // 88842:     case 0xf9: {
  OPC_CheckPredicate, 81,                    // 88845:      check predicate 81
  OPC_Decode, 218, 30, 196, 5,               // 88847:      decode to FRECPXv1f16 using decoder 708
                                             // 88847:     }
                                             // 88847:     } // switch Inst[23:16]
                                             // 88847:    }
  63, 0,                                     // 88852:    case 0x3f: {
  OPC_SwitchField, 21, 3,                    // 88854:     switch Inst[23:21] {
  1, 7,                                      // 88857:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 88859:      check predicate 76
  OPC_Decode, 202, 30, 179, 5,               // 88861:      decode to FRECPS32 using decoder 691
                                             // 88861:     }
  3, 7,                                      // 88866:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88868:      check predicate 76
  OPC_Decode, 203, 30, 200, 3,               // 88870:      decode to FRECPS64 using decoder 456
                                             // 88870:     }
  5, 7,                                      // 88875:     case 0x5: {
  OPC_CheckPredicate, 76,                    // 88877:      check predicate 76
  OPC_Decode, 247, 31, 179, 5,               // 88879:      decode to FRSQRTS32 using decoder 691
                                             // 88879:     }
  7, 0,                                      // 88884:     case 0x7: {
  OPC_CheckPredicate, 76,                    // 88886:      check predicate 76
  OPC_Decode, 248, 31, 200, 3,               // 88888:      decode to FRSQRTS64 using decoder 456
                                             // 88888:     }
                                             // 88888:     } // switch Inst[23:21]
                                             // 88888:    }
                                             // 88888:    } // switch Inst[15:10]
                                             // 88888:   }
  3, 0,                                      // 88893:   case 0x3: {
  OPC_SwitchField, 12, 4,                    // 88895:    switch Inst[15:12] {
  0, 15,                                     // 88898:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 88900:     check predicate 76
  OPC_CheckField, 22, 2, 1,                  // 88902:     check Inst[23:22] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 88906:     check Inst[11:10] == 0x1
  OPC_Decode, 199, 55, 228, 5,               // 88910:     decode to SSHRd using decoder 740
                                             // 88910:    }
  1, 59,                                     // 88915:    case 0x1: {
  OPC_SwitchField, 22, 2,                    // 88917:     switch Inst[23:22] {
  0, 11,                                     // 88920:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 88922:      check predicate 81
  OPC_CheckField, 10, 1, 0,                  // 88924:      check Inst[10] == 0x0
  OPC_Decode, 195, 28, 229, 5,               // 88928:      decode to FMLAv1i16_indexed using decoder 741
                                             // 88928:     }
  1, 11,                                     // 88933:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 88935:      check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 88937:      check Inst[11:10] == 0x1
  OPC_Decode, 211, 55, 230, 5,               // 88941:      decode to SSRAd using decoder 742
                                             // 88941:     }
  2, 11,                                     // 88946:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 88948:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 88950:      check Inst[10] == 0x0
  OPC_Decode, 196, 28, 231, 5,               // 88954:      decode to FMLAv1i32_indexed using decoder 743
                                             // 88954:     }
  3, 0,                                      // 88959:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 88961:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 88963:      check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 88967:      check Inst[10] == 0x0
  OPC_Decode, 197, 28, 232, 5,               // 88971:      decode to FMLAv1i64_indexed using decoder 744
                                             // 88971:     }
                                             // 88971:     } // switch Inst[23:22]
                                             // 88971:    }
  2, 15,                                     // 88976:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 88978:     check predicate 76
  OPC_CheckField, 22, 2, 1,                  // 88980:     check Inst[23:22] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 88984:     check Inst[11:10] == 0x1
  OPC_Decode, 159, 55, 228, 5,               // 88988:     decode to SRSHRd using decoder 740
                                             // 88988:    }
  3, 43,                                     // 88993:    case 0x3: {
  OPC_SwitchField, 10, 1,                    // 88995:     switch Inst[10] {
  0, 21,                                     // 88998:     case 0x0: {
  OPC_SwitchField, 22, 2,                    // 89000:      switch Inst[23:22] {
  1, 7,                                      // 89003:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 89005:       check predicate 76
  OPC_Decode, 214, 51, 233, 5,               // 89007:       decode to SQDMLALv1i32_indexed using decoder 745
                                             // 89007:      }
  2, 0,                                      // 89012:      case 0x2: {
  OPC_CheckPredicate, 76,                    // 89014:       check predicate 76
  OPC_Decode, 215, 51, 234, 5,               // 89016:       decode to SQDMLALv1i64_indexed using decoder 746
                                             // 89016:      }
                                             // 89016:      } // switch Inst[23:22]
                                             // 89016:     }
  1, 0,                                      // 89021:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89023:      check predicate 76
  OPC_CheckField, 22, 2, 1,                  // 89025:      check Inst[23:22] == 0x1
  OPC_CheckField, 11, 1, 0,                  // 89029:      check Inst[11] == 0x0
  OPC_Decode, 171, 55, 230, 5,               // 89033:      decode to SRSRAd using decoder 742
                                             // 89033:     }
                                             // 89033:     } // switch Inst[10]
                                             // 89033:    }
  5, 59,                                     // 89038:    case 0x5: {
  OPC_SwitchField, 22, 2,                    // 89040:     switch Inst[23:22] {
  0, 11,                                     // 89043:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 89045:      check predicate 81
  OPC_CheckField, 10, 1, 0,                  // 89047:      check Inst[10] == 0x0
  OPC_Decode, 253, 28, 229, 5,               // 89051:      decode to FMLSv1i16_indexed using decoder 741
                                             // 89051:     }
  1, 11,                                     // 89056:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89058:      check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 89060:      check Inst[11:10] == 0x1
  OPC_Decode, 161, 48, 235, 5,               // 89064:      decode to SHLd using decoder 747
                                             // 89064:     }
  2, 11,                                     // 89069:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 89071:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89073:      check Inst[10] == 0x0
  OPC_Decode, 254, 28, 231, 5,               // 89077:      decode to FMLSv1i32_indexed using decoder 743
                                             // 89077:     }
  3, 0,                                      // 89082:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 89084:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 89086:      check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 89090:      check Inst[10] == 0x0
  OPC_Decode, 255, 28, 232, 5,               // 89094:      decode to FMLSv1i64_indexed using decoder 744
                                             // 89094:     }
                                             // 89094:     } // switch Inst[23:22]
                                             // 89094:    }
  7, 96,                                     // 89099:    case 0x7: {
  OPC_SwitchField, 22, 2,                    // 89101:     switch Inst[23:22] {
  0, 51,                                     // 89104:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 89106:      switch Inst[21] {
  0, 33,                                     // 89109:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 89111:       switch Inst[20] {
  0, 15,                                     // 89114:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 89116:        check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 89118:        check Inst[19] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 89122:        check Inst[11:10] == 0x1
  OPC_Decode, 244, 53, 236, 5,               // 89126:        decode to SQSHLb using decoder 748
                                             // 89126:       }
  1, 0,                                      // 89131:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 89133:        check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 89135:        check Inst[11:10] == 0x1
  OPC_Decode, 246, 53, 237, 5,               // 89139:        decode to SQSHLh using decoder 749
                                             // 89139:       }
                                             // 89139:       } // switch Inst[20]
                                             // 89139:      }
  1, 0,                                      // 89144:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 89146:       check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 89148:       check Inst[11:10] == 0x1
  OPC_Decode, 247, 53, 238, 5,               // 89152:       decode to SQSHLs using decoder 750
                                             // 89152:      }
                                             // 89152:      } // switch Inst[21]
                                             // 89152:     }
  1, 25,                                     // 89157:     case 0x1: {
  OPC_SwitchField, 10, 1,                    // 89159:      switch Inst[10] {
  0, 7,                                      // 89162:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 89164:       check predicate 76
  OPC_Decode, 239, 51, 233, 5,               // 89166:       decode to SQDMLSLv1i32_indexed using decoder 745
                                             // 89166:      }
  1, 0,                                      // 89171:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 89173:       check predicate 76
  OPC_CheckField, 11, 1, 0,                  // 89175:       check Inst[11] == 0x0
  OPC_Decode, 245, 53, 235, 5,               // 89179:       decode to SQSHLd using decoder 747
                                             // 89179:      }
                                             // 89179:      } // switch Inst[10]
                                             // 89179:     }
  2, 0,                                      // 89184:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 89186:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89188:      check Inst[10] == 0x0
  OPC_Decode, 240, 51, 234, 5,               // 89192:      decode to SQDMLSLv1i64_indexed using decoder 746
                                             // 89192:     }
                                             // 89192:     } // switch Inst[23:22]
                                             // 89192:    }
  9, 134, 1,                                 // 89197:    case 0x9: {
  OPC_SwitchField, 22, 2,                    // 89200:     switch Inst[23:22] {
  0, 99,                                     // 89203:     case 0x0: {
  OPC_SwitchField, 10, 1,                    // 89205:      switch Inst[10] {
  0, 7,                                      // 89208:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 89210:       check predicate 81
  OPC_Decode, 142, 30, 239, 5,               // 89212:       decode to FMULv1i16_indexed using decoder 751
                                             // 89212:      }
  1, 0,                                      // 89217:      case 0x1: {
  OPC_SwitchField, 11, 1,                    // 89219:       switch Inst[11] {
  0, 39,                                     // 89222:       case 0x0: {
  OPC_SwitchField, 21, 1,                    // 89224:        switch Inst[21] {
  0, 25,                                     // 89227:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 89229:         switch Inst[20] {
  0, 11,                                     // 89232:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 89234:          check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 89236:          check Inst[19] == 0x1
  OPC_Decode, 146, 54, 240, 5,               // 89240:          decode to SQSHRNb using decoder 752
                                             // 89240:         }
  1, 0,                                      // 89245:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 89247:          check predicate 76
  OPC_Decode, 147, 54, 241, 5,               // 89249:          decode to SQSHRNh using decoder 753
                                             // 89249:         }
                                             // 89249:         } // switch Inst[20]
                                             // 89249:        }
  1, 0,                                      // 89254:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 89256:         check predicate 76
  OPC_Decode, 148, 54, 242, 5,               // 89258:         decode to SQSHRNs using decoder 754
                                             // 89258:        }
                                             // 89258:        } // switch Inst[21]
                                             // 89258:       }
  1, 0,                                      // 89263:       case 0x1: {
  OPC_SwitchField, 21, 1,                    // 89265:        switch Inst[21] {
  0, 25,                                     // 89268:        case 0x0: {
  OPC_SwitchField, 20, 1,                    // 89270:         switch Inst[20] {
  0, 11,                                     // 89273:         case 0x0: {
  OPC_CheckPredicate, 76,                    // 89275:          check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 89277:          check Inst[19] == 0x1
  OPC_Decode, 183, 53, 240, 5,               // 89281:          decode to SQRSHRNb using decoder 752
                                             // 89281:         }
  1, 0,                                      // 89286:         case 0x1: {
  OPC_CheckPredicate, 76,                    // 89288:          check predicate 76
  OPC_Decode, 184, 53, 241, 5,               // 89290:          decode to SQRSHRNh using decoder 753
                                             // 89290:         }
                                             // 89290:         } // switch Inst[20]
                                             // 89290:        }
  1, 0,                                      // 89295:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 89297:         check predicate 76
  OPC_Decode, 185, 53, 242, 5,               // 89299:         decode to SQRSHRNs using decoder 754
                                             // 89299:        }
                                             // 89299:        } // switch Inst[21]
                                             // 89299:       }
                                             // 89299:       } // switch Inst[11]
                                             // 89299:      }
                                             // 89299:      } // switch Inst[10]
                                             // 89299:     }
  2, 11,                                     // 89304:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 89306:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89308:      check Inst[10] == 0x0
  OPC_Decode, 143, 30, 243, 5,               // 89312:      decode to FMULv1i32_indexed using decoder 755
                                             // 89312:     }
  3, 0,                                      // 89317:     case 0x3: {
  OPC_CheckPredicate, 76,                    // 89319:      check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 89321:      check Inst[21] == 0x0
  OPC_CheckField, 10, 1, 0,                  // 89325:      check Inst[10] == 0x0
  OPC_Decode, 144, 30, 244, 5,               // 89329:      decode to FMULv1i64_indexed using decoder 756
                                             // 89329:     }
                                             // 89329:     } // switch Inst[23:22]
                                             // 89329:    }
  11, 29,                                    // 89334:    case 0xb: {
  OPC_SwitchField, 22, 2,                    // 89336:     switch Inst[23:22] {
  1, 11,                                     // 89339:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89341:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89343:      check Inst[10] == 0x0
  OPC_Decode, 168, 52, 245, 5,               // 89347:      decode to SQDMULLv1i32_indexed using decoder 757
                                             // 89347:     }
  2, 0,                                      // 89352:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 89354:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89356:      check Inst[10] == 0x0
  OPC_Decode, 169, 52, 246, 5,               // 89360:      decode to SQDMULLv1i64_indexed using decoder 758
                                             // 89360:     }
                                             // 89360:     } // switch Inst[23:22]
                                             // 89360:    }
  12, 29,                                    // 89365:    case 0xc: {
  OPC_SwitchField, 22, 2,                    // 89367:     switch Inst[23:22] {
  1, 11,                                     // 89370:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89372:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89374:      check Inst[10] == 0x0
  OPC_Decode, 145, 52, 239, 5,               // 89378:      decode to SQDMULHv1i16_indexed using decoder 751
                                             // 89378:     }
  2, 0,                                      // 89383:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 89385:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89387:      check Inst[10] == 0x0
  OPC_Decode, 147, 52, 243, 5,               // 89391:      decode to SQDMULHv1i32_indexed using decoder 755
                                             // 89391:     }
                                             // 89391:     } // switch Inst[23:22]
                                             // 89391:    }
  13, 29,                                    // 89396:    case 0xd: {
  OPC_SwitchField, 22, 2,                    // 89398:     switch Inst[23:22] {
  1, 11,                                     // 89401:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89403:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89405:      check Inst[10] == 0x0
  OPC_Decode, 143, 53, 239, 5,               // 89409:      decode to SQRDMULHv1i16_indexed using decoder 751
                                             // 89409:     }
  2, 0,                                      // 89414:     case 0x2: {
  OPC_CheckPredicate, 76,                    // 89416:      check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 89418:      check Inst[10] == 0x0
  OPC_Decode, 145, 53, 243, 5,               // 89422:      decode to SQRDMULHv1i32_indexed using decoder 755
                                             // 89422:     }
                                             // 89422:     } // switch Inst[23:22]
                                             // 89422:    }
  14, 51,                                    // 89427:    case 0xe: {
  OPC_SwitchField, 22, 2,                    // 89429:     switch Inst[23:22] {
  0, 33,                                     // 89432:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 89434:      switch Inst[21] {
  0, 15,                                     // 89437:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 89439:       check predicate 81
  OPC_CheckField, 20, 1, 1,                  // 89441:       check Inst[20] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 89445:       check Inst[11:10] == 0x1
  OPC_Decode, 160, 47, 247, 5,               // 89449:       decode to SCVTFh using decoder 759
                                             // 89449:      }
  1, 0,                                      // 89454:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 89456:       check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 89458:       check Inst[11:10] == 0x1
  OPC_Decode, 161, 47, 248, 5,               // 89462:       decode to SCVTFs using decoder 760
                                             // 89462:      }
                                             // 89462:      } // switch Inst[21]
                                             // 89462:     }
  1, 0,                                      // 89467:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89469:      check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 89471:      check Inst[11:10] == 0x1
  OPC_Decode, 159, 47, 228, 5,               // 89475:      decode to SCVTFd using decoder 740
                                             // 89475:     }
                                             // 89475:     } // switch Inst[23:22]
                                             // 89475:    }
  15, 0,                                     // 89480:    case 0xf: {
  OPC_SwitchField, 22, 2,                    // 89482:     switch Inst[23:22] {
  0, 33,                                     // 89485:     case 0x0: {
  OPC_SwitchField, 21, 1,                    // 89487:      switch Inst[21] {
  0, 15,                                     // 89490:      case 0x0: {
  OPC_CheckPredicate, 81,                    // 89492:       check predicate 81
  OPC_CheckField, 20, 1, 1,                  // 89494:       check Inst[20] == 0x1
  OPC_CheckField, 10, 2, 3,                  // 89498:       check Inst[11:10] == 0x3
  OPC_Decode, 156, 25, 247, 5,               // 89502:       decode to FCVTZSh using decoder 759
                                             // 89502:      }
  1, 0,                                      // 89507:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 89509:       check predicate 76
  OPC_CheckField, 10, 2, 3,                  // 89511:       check Inst[11:10] == 0x3
  OPC_Decode, 157, 25, 248, 5,               // 89515:       decode to FCVTZSs using decoder 760
                                             // 89515:      }
                                             // 89515:      } // switch Inst[21]
                                             // 89515:     }
  1, 0,                                      // 89520:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 89522:      check predicate 76
  OPC_CheckField, 10, 2, 3,                  // 89524:      check Inst[11:10] == 0x3
  OPC_Decode, 155, 25, 228, 5,               // 89528:      decode to FCVTZSd using decoder 740
                                             // 89528:     }
                                             // 89528:     } // switch Inst[23:22]
                                             // 89528:    }
                                             // 89528:    } // switch Inst[15:12]
                                             // 89528:   }
                                             // 89528:   } // switch Inst[25:24]
                                             // 89528:  }
  3, 235, 18,                                // 89533:  case 0x3: {
  OPC_SwitchField, 22, 4,                    // 89536:   switch Inst[25:22] {
  0, 200, 1,                                 // 89539:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 89542:    switch Inst[11:10] {
  0, 137, 1,                                 // 89545:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 89548:     switch Inst[21] {
  0, 7,                                      // 89551:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 89553:      check predicate 77
  OPC_Decode, 234, 59, 247, 4,               // 89555:      decode to STURHi using decoder 631
                                             // 89555:     }
  1, 0,                                      // 89560:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 89562:      switch Inst[15:12] {
  0, 8,                                      // 89565:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 89567:       check predicate 139
  OPC_Decode, 250, 37, 205, 5,               // 89570:       decode to LDFADDH using decoder 717
                                             // 89570:      }
  4, 8,                                      // 89575:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 89577:       check predicate 139
  OPC_Decode, 150, 38, 205, 5,               // 89580:       decode to LDFMAXH using decoder 717
                                             // 89580:      }
  5, 8,                                      // 89585:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 89587:       check predicate 139
  OPC_Decode, 174, 38, 205, 5,               // 89590:       decode to LDFMINH using decoder 717
                                             // 89590:      }
  6, 8,                                      // 89595:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 89597:       check predicate 139
  OPC_Decode, 161, 38, 205, 5,               // 89600:       decode to LDFMAXNMH using decoder 717
                                             // 89600:      }
  7, 8,                                      // 89605:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 89607:       check predicate 139
  OPC_Decode, 185, 38, 205, 5,               // 89610:       decode to LDFMINNMH using decoder 717
                                             // 89610:      }
  8, 12,                                     // 89615:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 89617:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89620:       check Inst[4:0] == 0x1f
  OPC_Decode, 156, 58, 206, 5,               // 89624:       decode to STFADDH using decoder 718
                                             // 89624:      }
  12, 12,                                    // 89629:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 89631:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89634:       check Inst[4:0] == 0x1f
  OPC_Decode, 162, 58, 206, 5,               // 89638:       decode to STFMAXH using decoder 718
                                             // 89638:      }
  13, 12,                                    // 89643:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 89645:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89648:       check Inst[4:0] == 0x1f
  OPC_Decode, 174, 58, 206, 5,               // 89652:       decode to STFMINH using decoder 718
                                             // 89652:      }
  14, 12,                                    // 89657:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 89659:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89662:       check Inst[4:0] == 0x1f
  OPC_Decode, 167, 58, 206, 5,               // 89666:       decode to STFMAXNMH using decoder 718
                                             // 89666:      }
  15, 0,                                     // 89671:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 89673:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89676:       check Inst[4:0] == 0x1f
  OPC_Decode, 179, 58, 206, 5,               // 89680:       decode to STFMINNMH using decoder 718
                                             // 89680:      }
                                             // 89680:      } // switch Inst[15:12]
                                             // 89680:     }
                                             // 89680:     } // switch Inst[21]
                                             // 89680:    }
  1, 11,                                     // 89685:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 89687:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 89689:     check Inst[21] == 0x0
  OPC_Decode, 186, 59, 247, 4,               // 89693:     decode to STRHpost using decoder 631
                                             // 89693:    }
  2, 29,                                     // 89698:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 89700:     switch Inst[14:13] {
  2, 11,                                     // 89703:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 89705:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 89707:      check Inst[21] == 0x1
  OPC_Decode, 188, 59, 249, 5,               // 89711:      decode to STRHroW using decoder 761
                                             // 89711:     }
  3, 0,                                      // 89716:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 89718:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 89720:      check Inst[21] == 0x1
  OPC_Decode, 189, 59, 250, 5,               // 89724:      decode to STRHroX using decoder 762
                                             // 89724:     }
                                             // 89724:     } // switch Inst[14:13]
                                             // 89724:    }
  3, 0,                                      // 89729:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 89731:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 89733:     check Inst[21] == 0x0
  OPC_Decode, 187, 59, 247, 4,               // 89737:     decode to STRHpre using decoder 631
                                             // 89737:    }
                                             // 89737:    } // switch Inst[11:10]
                                             // 89737:   }
  1, 200, 1,                                 // 89742:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 89745:    switch Inst[11:10] {
  0, 137, 1,                                 // 89748:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 89751:     switch Inst[21] {
  0, 7,                                      // 89754:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 89756:      check predicate 77
  OPC_Decode, 251, 40, 247, 4,               // 89758:      decode to LDURHi using decoder 631
                                             // 89758:     }
  1, 0,                                      // 89763:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 89765:      switch Inst[15:12] {
  0, 8,                                      // 89768:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 89770:       check predicate 139
  OPC_Decode, 252, 37, 205, 5,               // 89773:       decode to LDFADDLH using decoder 717
                                             // 89773:      }
  4, 8,                                      // 89778:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 89780:       check predicate 139
  OPC_Decode, 152, 38, 205, 5,               // 89783:       decode to LDFMAXLH using decoder 717
                                             // 89783:      }
  5, 8,                                      // 89788:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 89790:       check predicate 139
  OPC_Decode, 176, 38, 205, 5,               // 89793:       decode to LDFMINLH using decoder 717
                                             // 89793:      }
  6, 8,                                      // 89798:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 89800:       check predicate 139
  OPC_Decode, 163, 38, 205, 5,               // 89803:       decode to LDFMAXNMLH using decoder 717
                                             // 89803:      }
  7, 8,                                      // 89808:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 89810:       check predicate 139
  OPC_Decode, 187, 38, 205, 5,               // 89813:       decode to LDFMINNMLH using decoder 717
                                             // 89813:      }
  8, 12,                                     // 89818:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 89820:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89823:       check Inst[4:0] == 0x1f
  OPC_Decode, 158, 58, 206, 5,               // 89827:       decode to STFADDLH using decoder 718
                                             // 89827:      }
  12, 12,                                    // 89832:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 89834:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89837:       check Inst[4:0] == 0x1f
  OPC_Decode, 164, 58, 206, 5,               // 89841:       decode to STFMAXLH using decoder 718
                                             // 89841:      }
  13, 12,                                    // 89846:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 89848:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89851:       check Inst[4:0] == 0x1f
  OPC_Decode, 176, 58, 206, 5,               // 89855:       decode to STFMINLH using decoder 718
                                             // 89855:      }
  14, 12,                                    // 89860:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 89862:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89865:       check Inst[4:0] == 0x1f
  OPC_Decode, 169, 58, 206, 5,               // 89869:       decode to STFMAXNMLH using decoder 718
                                             // 89869:      }
  15, 0,                                     // 89874:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 89876:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 89879:       check Inst[4:0] == 0x1f
  OPC_Decode, 181, 58, 206, 5,               // 89883:       decode to STFMINNMLH using decoder 718
                                             // 89883:      }
                                             // 89883:      } // switch Inst[15:12]
                                             // 89883:     }
                                             // 89883:     } // switch Inst[21]
                                             // 89883:    }
  1, 11,                                     // 89888:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 89890:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 89892:     check Inst[21] == 0x0
  OPC_Decode, 189, 39, 247, 4,               // 89896:     decode to LDRHpost using decoder 631
                                             // 89896:    }
  2, 29,                                     // 89901:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 89903:     switch Inst[14:13] {
  2, 11,                                     // 89906:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 89908:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 89910:      check Inst[21] == 0x1
  OPC_Decode, 191, 39, 249, 5,               // 89914:      decode to LDRHroW using decoder 761
                                             // 89914:     }
  3, 0,                                      // 89919:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 89921:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 89923:      check Inst[21] == 0x1
  OPC_Decode, 192, 39, 250, 5,               // 89927:      decode to LDRHroX using decoder 762
                                             // 89927:     }
                                             // 89927:     } // switch Inst[14:13]
                                             // 89927:    }
  3, 0,                                      // 89932:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 89934:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 89936:     check Inst[21] == 0x0
  OPC_Decode, 190, 39, 247, 4,               // 89940:     decode to LDRHpre using decoder 631
                                             // 89940:    }
                                             // 89940:    } // switch Inst[11:10]
                                             // 89940:   }
  2, 73,                                     // 89945:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 89947:    switch Inst[15:10] {
  0, 12,                                     // 89950:    case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 89952:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 89955:     check Inst[21] == 0x1
  OPC_Decode, 244, 37, 205, 5,               // 89959:     decode to LDFADDAH using decoder 717
                                             // 89959:    }
  16, 12,                                    // 89964:    case 0x10: {
  OPC_CheckPredicate, 139, 1,                // 89966:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 89969:     check Inst[21] == 0x1
  OPC_Decode, 144, 38, 205, 5,               // 89973:     decode to LDFMAXAH using decoder 717
                                             // 89973:    }
  20, 12,                                    // 89978:    case 0x14: {
  OPC_CheckPredicate, 139, 1,                // 89980:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 89983:     check Inst[21] == 0x1
  OPC_Decode, 168, 38, 205, 5,               // 89987:     decode to LDFMINAH using decoder 717
                                             // 89987:    }
  24, 12,                                    // 89992:    case 0x18: {
  OPC_CheckPredicate, 139, 1,                // 89994:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 89997:     check Inst[21] == 0x1
  OPC_Decode, 155, 38, 205, 5,               // 90001:     decode to LDFMAXNMAH using decoder 717
                                             // 90001:    }
  28, 0,                                     // 90006:    case 0x1c: {
  OPC_CheckPredicate, 139, 1,                // 90008:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 90011:     check Inst[21] == 0x1
  OPC_Decode, 179, 38, 205, 5,               // 90015:     decode to LDFMINNMAH using decoder 717
                                             // 90015:    }
                                             // 90015:    } // switch Inst[15:10]
                                             // 90015:   }
  3, 73,                                     // 90020:   case 0x3: {
  OPC_SwitchField, 10, 6,                    // 90022:    switch Inst[15:10] {
  0, 12,                                     // 90025:    case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 90027:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 90030:     check Inst[21] == 0x1
  OPC_Decode, 246, 37, 205, 5,               // 90034:     decode to LDFADDALH using decoder 717
                                             // 90034:    }
  16, 12,                                    // 90039:    case 0x10: {
  OPC_CheckPredicate, 139, 1,                // 90041:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 90044:     check Inst[21] == 0x1
  OPC_Decode, 146, 38, 205, 5,               // 90048:     decode to LDFMAXALH using decoder 717
                                             // 90048:    }
  20, 12,                                    // 90053:    case 0x14: {
  OPC_CheckPredicate, 139, 1,                // 90055:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 90058:     check Inst[21] == 0x1
  OPC_Decode, 170, 38, 205, 5,               // 90062:     decode to LDFMINALH using decoder 717
                                             // 90062:    }
  24, 12,                                    // 90067:    case 0x18: {
  OPC_CheckPredicate, 139, 1,                // 90069:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 90072:     check Inst[21] == 0x1
  OPC_Decode, 157, 38, 205, 5,               // 90076:     decode to LDFMAXNMALH using decoder 717
                                             // 90076:    }
  28, 0,                                     // 90081:    case 0x1c: {
  OPC_CheckPredicate, 139, 1,                // 90083:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 90086:     check Inst[21] == 0x1
  OPC_Decode, 181, 38, 205, 5,               // 90090:     decode to LDFMINNMALH using decoder 717
                                             // 90090:    }
                                             // 90090:    } // switch Inst[15:10]
                                             // 90090:   }
  4, 7,                                      // 90095:   case 0x4: {
  OPC_CheckPredicate, 77,                    // 90097:    check predicate 77
  OPC_Decode, 190, 59, 132, 5,               // 90099:    decode to STRHui using decoder 644
                                             // 90099:   }
  5, 7,                                      // 90104:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 90106:    check predicate 77
  OPC_Decode, 193, 39, 132, 5,               // 90108:    decode to LDRHui using decoder 644
                                             // 90108:   }
  8, 218, 1,                                 // 90113:   case 0x8: {
  OPC_SwitchField, 10, 6,                    // 90116:    switch Inst[15:10] {
  3, 11,                                     // 90119:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 90121:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90123:     check Inst[21] == 0x1
  OPC_Decode, 134, 66, 218, 5,               // 90127:     decode to UQADDv1i8 using decoder 730
                                             // 90127:    }
  10, 11,                                    // 90132:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 90134:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90136:     check Inst[21:16] == 0x21
  OPC_Decode, 223, 54, 222, 5,               // 90140:     decode to SQXTUNv1i8 using decoder 734
                                             // 90140:    }
  11, 11,                                    // 90145:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 90147:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90149:     check Inst[21] == 0x1
  OPC_Decode, 175, 67, 218, 5,               // 90153:     decode to UQSUBv1i8 using decoder 730
                                             // 90153:    }
  14, 11,                                    // 90158:    case 0xe: {
  OPC_CheckPredicate, 76,                    // 90160:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 90162:     check Inst[21:16] == 0x20
  OPC_Decode, 221, 68, 219, 5,               // 90166:     decode to USQADDv1i8 using decoder 731
                                             // 90166:    }
  18, 11,                                    // 90171:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 90173:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90175:     check Inst[21:16] == 0x21
  OPC_Decode, 191, 67, 222, 5,               // 90179:     decode to UQXTNv1i8 using decoder 734
                                             // 90179:    }
  19, 11,                                    // 90184:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 90186:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90188:     check Inst[21] == 0x1
  OPC_Decode, 253, 66, 218, 5,               // 90192:     decode to UQSHLv1i8 using decoder 730
                                             // 90192:    }
  23, 11,                                    // 90197:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 90199:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90201:     check Inst[21] == 0x1
  OPC_Decode, 203, 66, 218, 5,               // 90205:     decode to UQRSHLv1i8 using decoder 730
                                             // 90205:    }
  30, 11,                                    // 90210:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 90212:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 90214:     check Inst[21:16] == 0x20
  OPC_Decode, 212, 52, 223, 5,               // 90218:     decode to SQNEGv1i8 using decoder 735
                                             // 90218:    }
  42, 11,                                    // 90223:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 90225:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90227:     check Inst[21:16] == 0x21
  OPC_Decode, 185, 24, 174, 5,               // 90231:     decode to FCVTNUv1i32 using decoder 686
                                             // 90231:    }
  46, 11,                                    // 90236:    case 0x2e: {
  OPC_CheckPredicate, 76,                    // 90238:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90240:     check Inst[21:16] == 0x21
  OPC_Decode, 143, 24, 174, 5,               // 90244:     decode to FCVTMUv1i32 using decoder 686
                                             // 90244:    }
  50, 21,                                    // 90249:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 90251:     switch Inst[21:16] {
  33, 7,                                     // 90254:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90256:      check predicate 76
  OPC_Decode, 222, 23, 174, 5,               // 90258:      decode to FCVTAUv1i32 using decoder 686
                                             // 90258:     }
  48, 0,                                     // 90263:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 90265:      check predicate 76
  OPC_Decode, 193, 26, 130, 4,               // 90267:      decode to FMAXNMPv2i32p using decoder 514
                                             // 90267:     }
                                             // 90267:     } // switch Inst[21:16]
                                             // 90267:    }
  54, 21,                                    // 90272:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 90274:     switch Inst[21:16] {
  33, 7,                                     // 90277:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90279:      check predicate 76
  OPC_Decode, 156, 63, 174, 5,               // 90281:      decode to UCVTFv1i32 using decoder 686
                                             // 90281:     }
  48, 0,                                     // 90286:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 90288:      check predicate 76
  OPC_Decode, 238, 21, 130, 4,               // 90290:      decode to FADDPv2i32p using decoder 514
                                             // 90290:     }
                                             // 90290:     } // switch Inst[21:16]
                                             // 90290:    }
  57, 11,                                    // 90295:    case 0x39: {
  OPC_CheckPredicate, 76,                    // 90297:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90299:     check Inst[21] == 0x1
  OPC_Decode, 216, 22, 179, 5,               // 90303:     decode to FCMGE32 using decoder 691
                                             // 90303:    }
  59, 11,                                    // 90308:    case 0x3b: {
  OPC_CheckPredicate, 76,                    // 90310:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90312:     check Inst[21] == 0x1
  OPC_Decode, 206, 21, 179, 5,               // 90316:     decode to FACGE32 using decoder 691
                                             // 90316:    }
  62, 0,                                     // 90321:    case 0x3e: {
  OPC_CheckPredicate, 76,                    // 90323:     check predicate 76
  OPC_CheckField, 16, 6, 48,                 // 90325:     check Inst[21:16] == 0x30
  OPC_Decode, 237, 26, 130, 4,               // 90329:     decode to FMAXPv2i32p using decoder 514
                                             // 90329:    }
                                             // 90329:    } // switch Inst[15:10]
                                             // 90329:   }
  9, 205, 2,                                 // 90334:   case 0x9: {
  OPC_SwitchField, 10, 6,                    // 90337:    switch Inst[15:10] {
  3, 11,                                     // 90340:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 90342:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90344:     check Inst[21] == 0x1
  OPC_Decode, 131, 66, 200, 5,               // 90348:     decode to UQADDv1i16 using decoder 712
                                             // 90348:    }
  9, 11,                                     // 90353:    case 0x9: {
  OPC_CheckPredicate, 81,                    // 90355:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 90357:     check Inst[21] == 0x0
  OPC_Decode, 215, 22, 200, 5,               // 90361:     decode to FCMGE16 using decoder 712
                                             // 90361:    }
  10, 11,                                    // 90366:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 90368:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90370:     check Inst[21:16] == 0x21
  OPC_Decode, 221, 54, 176, 5,               // 90374:     decode to SQXTUNv1i16 using decoder 688
                                             // 90374:    }
  11, 21,                                    // 90379:    case 0xb: {
  OPC_SwitchField, 21, 1,                    // 90381:     switch Inst[21] {
  0, 7,                                      // 90384:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 90386:      check predicate 81
  OPC_Decode, 205, 21, 200, 5,               // 90388:      decode to FACGE16 using decoder 712
                                             // 90388:     }
  1, 0,                                      // 90393:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 90395:      check predicate 76
  OPC_Decode, 172, 67, 200, 5,               // 90397:      decode to UQSUBv1i16 using decoder 712
                                             // 90397:     }
                                             // 90397:     } // switch Inst[21]
                                             // 90397:    }
  14, 11,                                    // 90402:    case 0xe: {
  OPC_CheckPredicate, 76,                    // 90404:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 90406:     check Inst[21:16] == 0x20
  OPC_Decode, 218, 68, 220, 5,               // 90410:     decode to USQADDv1i16 using decoder 732
                                             // 90410:    }
  18, 11,                                    // 90415:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 90417:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90419:     check Inst[21:16] == 0x21
  OPC_Decode, 189, 67, 176, 5,               // 90423:     decode to UQXTNv1i16 using decoder 688
                                             // 90423:    }
  19, 11,                                    // 90428:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 90430:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90432:     check Inst[21] == 0x1
  OPC_Decode, 250, 66, 200, 5,               // 90436:     decode to UQSHLv1i16 using decoder 712
                                             // 90436:    }
  23, 11,                                    // 90441:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 90443:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90445:     check Inst[21] == 0x1
  OPC_Decode, 200, 66, 200, 5,               // 90449:     decode to UQRSHLv1i16 using decoder 712
                                             // 90449:    }
  26, 11,                                    // 90454:    case 0x1a: {
  OPC_CheckPredicate, 76,                    // 90456:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90458:     check Inst[21:16] == 0x21
  OPC_Decode, 243, 24, 130, 4,               // 90462:     decode to FCVTXNv1i64 using decoder 514
                                             // 90462:    }
  30, 11,                                    // 90467:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 90469:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 90471:     check Inst[21:16] == 0x20
  OPC_Decode, 209, 52, 196, 5,               // 90475:     decode to SQNEGv1i16 using decoder 708
                                             // 90475:    }
  33, 12,                                    // 90480:    case 0x21: {
  OPC_CheckPredicate, 141, 1,                // 90482:     check predicate 141
  OPC_CheckField, 21, 1, 0,                  // 90485:     check Inst[21] == 0x0
  OPC_Decode, 232, 52, 251, 5,               // 90489:     decode to SQRDMLAHv1i16 using decoder 763
                                             // 90489:    }
  35, 12,                                    // 90494:    case 0x23: {
  OPC_CheckPredicate, 141, 1,                // 90496:     check predicate 141
  OPC_CheckField, 21, 1, 0,                  // 90499:     check Inst[21] == 0x0
  OPC_Decode, 251, 52, 251, 5,               // 90503:     decode to SQRDMLSHv1i16 using decoder 763
                                             // 90503:    }
  42, 21,                                    // 90508:    case 0x2a: {
  OPC_SwitchField, 16, 6,                    // 90510:     switch Inst[21:16] {
  33, 7,                                     // 90513:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90515:      check predicate 76
  OPC_Decode, 186, 24, 201, 3,               // 90517:      decode to FCVTNUv1i64 using decoder 457
                                             // 90517:     }
  57, 0,                                     // 90522:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 90524:      check predicate 81
  OPC_Decode, 184, 24, 196, 5,               // 90526:      decode to FCVTNUv1f16 using decoder 708
                                             // 90526:     }
                                             // 90526:     } // switch Inst[21:16]
                                             // 90526:    }
  45, 11,                                    // 90531:    case 0x2d: {
  OPC_CheckPredicate, 76,                    // 90533:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90535:     check Inst[21] == 0x1
  OPC_Decode, 142, 53, 200, 5,               // 90539:     decode to SQRDMULHv1i16 using decoder 712
                                             // 90539:    }
  46, 21,                                    // 90544:    case 0x2e: {
  OPC_SwitchField, 16, 6,                    // 90546:     switch Inst[21:16] {
  33, 7,                                     // 90549:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90551:      check predicate 76
  OPC_Decode, 144, 24, 201, 3,               // 90553:      decode to FCVTMUv1i64 using decoder 457
                                             // 90553:     }
  57, 0,                                     // 90558:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 90560:      check predicate 81
  OPC_Decode, 142, 24, 196, 5,               // 90562:      decode to FCVTMUv1f16 using decoder 708
                                             // 90562:     }
                                             // 90562:     } // switch Inst[21:16]
                                             // 90562:    }
  50, 30,                                    // 90567:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 90569:     switch Inst[21:16] {
  33, 7,                                     // 90572:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90574:      check predicate 76
  OPC_Decode, 223, 23, 201, 3,               // 90576:      decode to FCVTAUv1i64 using decoder 457
                                             // 90576:     }
  48, 7,                                     // 90581:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 90583:      check predicate 76
  OPC_Decode, 194, 26, 206, 3,               // 90585:      decode to FMAXNMPv2i64p using decoder 462
                                             // 90585:     }
  57, 0,                                     // 90590:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 90592:      check predicate 81
  OPC_Decode, 221, 23, 196, 5,               // 90594:      decode to FCVTAUv1f16 using decoder 708
                                             // 90594:     }
                                             // 90594:     } // switch Inst[21:16]
                                             // 90594:    }
  54, 30,                                    // 90599:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 90601:     switch Inst[21:16] {
  33, 7,                                     // 90604:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90606:      check predicate 76
  OPC_Decode, 157, 63, 201, 3,               // 90608:      decode to UCVTFv1i64 using decoder 457
                                             // 90608:     }
  48, 7,                                     // 90613:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 90615:      check predicate 76
  OPC_Decode, 239, 21, 206, 3,               // 90617:      decode to FADDPv2i64p using decoder 462
                                             // 90617:     }
  57, 0,                                     // 90622:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 90624:      check predicate 81
  OPC_Decode, 155, 63, 196, 5,               // 90626:      decode to UCVTFv1i16 using decoder 708
                                             // 90626:     }
                                             // 90626:     } // switch Inst[21:16]
                                             // 90626:    }
  57, 11,                                    // 90631:    case 0x39: {
  OPC_CheckPredicate, 76,                    // 90633:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90635:     check Inst[21] == 0x1
  OPC_Decode, 217, 22, 200, 3,               // 90639:     decode to FCMGE64 using decoder 456
                                             // 90639:    }
  59, 11,                                    // 90644:    case 0x3b: {
  OPC_CheckPredicate, 76,                    // 90646:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90648:     check Inst[21] == 0x1
  OPC_Decode, 207, 21, 200, 3,               // 90652:     decode to FACGE64 using decoder 456
                                             // 90652:    }
  62, 0,                                     // 90657:    case 0x3e: {
  OPC_CheckPredicate, 76,                    // 90659:     check predicate 76
  OPC_CheckField, 16, 6, 48,                 // 90661:     check Inst[21:16] == 0x30
  OPC_Decode, 238, 26, 206, 3,               // 90665:     decode to FMAXPv2i64p using decoder 462
                                             // 90665:    }
                                             // 90665:    } // switch Inst[15:10]
                                             // 90665:   }
  10, 144, 2,                                // 90670:   case 0xa: {
  OPC_SwitchField, 10, 6,                    // 90673:    switch Inst[15:10] {
  3, 11,                                     // 90676:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 90678:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90680:     check Inst[21] == 0x1
  OPC_Decode, 132, 66, 179, 5,               // 90684:     decode to UQADDv1i32 using decoder 691
                                             // 90684:    }
  10, 11,                                    // 90689:    case 0xa: {
  OPC_CheckPredicate, 76,                    // 90691:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90693:     check Inst[21:16] == 0x21
  OPC_Decode, 222, 54, 130, 4,               // 90697:     decode to SQXTUNv1i32 using decoder 514
                                             // 90697:    }
  11, 11,                                    // 90702:    case 0xb: {
  OPC_CheckPredicate, 76,                    // 90704:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90706:     check Inst[21] == 0x1
  OPC_Decode, 173, 67, 179, 5,               // 90710:     decode to UQSUBv1i32 using decoder 691
                                             // 90710:    }
  14, 11,                                    // 90715:    case 0xe: {
  OPC_CheckPredicate, 76,                    // 90717:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 90719:     check Inst[21:16] == 0x20
  OPC_Decode, 219, 68, 221, 5,               // 90723:     decode to USQADDv1i32 using decoder 733
                                             // 90723:    }
  18, 11,                                    // 90728:    case 0x12: {
  OPC_CheckPredicate, 76,                    // 90730:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90732:     check Inst[21:16] == 0x21
  OPC_Decode, 190, 67, 130, 4,               // 90736:     decode to UQXTNv1i32 using decoder 514
                                             // 90736:    }
  19, 11,                                    // 90741:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 90743:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90745:     check Inst[21] == 0x1
  OPC_Decode, 251, 66, 179, 5,               // 90749:     decode to UQSHLv1i32 using decoder 691
                                             // 90749:    }
  23, 11,                                    // 90754:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 90756:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90758:     check Inst[21] == 0x1
  OPC_Decode, 201, 66, 179, 5,               // 90762:     decode to UQRSHLv1i32 using decoder 691
                                             // 90762:    }
  30, 11,                                    // 90767:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 90769:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 90771:     check Inst[21:16] == 0x20
  OPC_Decode, 210, 52, 174, 5,               // 90775:     decode to SQNEGv1i32 using decoder 686
                                             // 90775:    }
  33, 12,                                    // 90780:    case 0x21: {
  OPC_CheckPredicate, 141, 1,                // 90782:     check predicate 141
  OPC_CheckField, 21, 1, 0,                  // 90785:     check Inst[21] == 0x0
  OPC_Decode, 234, 52, 252, 5,               // 90789:     decode to SQRDMLAHv1i32 using decoder 764
                                             // 90789:    }
  35, 12,                                    // 90794:    case 0x23: {
  OPC_CheckPredicate, 141, 1,                // 90796:     check predicate 141
  OPC_CheckField, 21, 1, 0,                  // 90799:     check Inst[21] == 0x0
  OPC_Decode, 253, 52, 252, 5,               // 90803:     decode to SQRDMLSHv1i32 using decoder 764
                                             // 90803:    }
  42, 11,                                    // 90808:    case 0x2a: {
  OPC_CheckPredicate, 76,                    // 90810:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90812:     check Inst[21:16] == 0x21
  OPC_Decode, 232, 24, 174, 5,               // 90816:     decode to FCVTPUv1i32 using decoder 686
                                             // 90816:    }
  45, 11,                                    // 90821:    case 0x2d: {
  OPC_CheckPredicate, 76,                    // 90823:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90825:     check Inst[21] == 0x1
  OPC_Decode, 144, 53, 179, 5,               // 90829:     decode to SQRDMULHv1i32 using decoder 691
                                             // 90829:    }
  46, 11,                                    // 90834:    case 0x2e: {
  OPC_CheckPredicate, 76,                    // 90836:     check predicate 76
  OPC_CheckField, 16, 6, 33,                 // 90838:     check Inst[21:16] == 0x21
  OPC_Decode, 210, 25, 174, 5,               // 90842:     decode to FCVTZUv1i32 using decoder 686
                                             // 90842:    }
  50, 21,                                    // 90847:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 90849:     switch Inst[21:16] {
  32, 7,                                     // 90852:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 90854:      check predicate 76
  OPC_Decode, 225, 22, 174, 5,               // 90856:      decode to FCMGEv1i32rz using decoder 686
                                             // 90856:     }
  48, 0,                                     // 90861:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 90863:      check predicate 76
  OPC_Decode, 157, 27, 130, 4,               // 90865:      decode to FMINNMPv2i32p using decoder 514
                                             // 90865:     }
                                             // 90865:     } // switch Inst[21:16]
                                             // 90865:    }
  53, 11,                                    // 90870:    case 0x35: {
  OPC_CheckPredicate, 76,                    // 90872:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90874:     check Inst[21] == 0x1
  OPC_Decode, 181, 21, 179, 5,               // 90878:     decode to FABD32 using decoder 691
                                             // 90878:    }
  54, 21,                                    // 90883:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 90885:     switch Inst[21:16] {
  32, 7,                                     // 90888:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 90890:      check predicate 76
  OPC_Decode, 148, 23, 174, 5,               // 90892:      decode to FCMLEv1i32rz using decoder 686
                                             // 90892:     }
  33, 0,                                     // 90897:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 90899:      check predicate 76
  OPC_Decode, 239, 31, 174, 5,               // 90901:      decode to FRSQRTEv1i32 using decoder 686
                                             // 90901:     }
                                             // 90901:     } // switch Inst[21:16]
                                             // 90901:    }
  57, 11,                                    // 90906:    case 0x39: {
  OPC_CheckPredicate, 76,                    // 90908:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90910:     check Inst[21] == 0x1
  OPC_Decode, 238, 22, 179, 5,               // 90914:     decode to FCMGT32 using decoder 691
                                             // 90914:    }
  59, 11,                                    // 90919:    case 0x3b: {
  OPC_CheckPredicate, 76,                    // 90921:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90923:     check Inst[21] == 0x1
  OPC_Decode, 217, 21, 179, 5,               // 90927:     decode to FACGT32 using decoder 691
                                             // 90927:    }
  62, 0,                                     // 90932:    case 0x3e: {
  OPC_CheckPredicate, 76,                    // 90934:     check predicate 76
  OPC_CheckField, 16, 6, 48,                 // 90936:     check Inst[21:16] == 0x30
  OPC_Decode, 201, 27, 130, 4,               // 90940:     decode to FMINPv2i32p using decoder 514
                                             // 90940:    }
                                             // 90940:    } // switch Inst[15:10]
                                             // 90940:   }
  11, 145, 3,                                // 90945:   case 0xb: {
  OPC_SwitchField, 10, 6,                    // 90948:    switch Inst[15:10] {
  3, 11,                                     // 90951:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 90953:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 90955:     check Inst[21] == 0x1
  OPC_Decode, 133, 66, 200, 3,               // 90959:     decode to UQADDv1i64 using decoder 456
                                             // 90959:    }
  5, 11,                                     // 90964:    case 0x5: {
  OPC_CheckPredicate, 81,                    // 90966:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 90968:     check Inst[21] == 0x0
  OPC_Decode, 180, 21, 200, 5,               // 90972:     decode to FABD16 using decoder 712
                                             // 90972:    }
  9, 11,                                     // 90977:    case 0x9: {
  OPC_CheckPredicate, 81,                    // 90979:     check predicate 81
  OPC_CheckField, 21, 1, 0,                  // 90981:     check Inst[21] == 0x0
  OPC_Decode, 237, 22, 200, 5,               // 90985:     decode to FCMGT16 using decoder 712
                                             // 90985:    }
  11, 21,                                    // 90990:    case 0xb: {
  OPC_SwitchField, 21, 1,                    // 90992:     switch Inst[21] {
  0, 7,                                      // 90995:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 90997:      check predicate 81
  OPC_Decode, 216, 21, 200, 5,               // 90999:      decode to FACGT16 using decoder 712
                                             // 90999:     }
  1, 0,                                      // 91004:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 91006:      check predicate 76
  OPC_Decode, 174, 67, 200, 3,               // 91008:      decode to UQSUBv1i64 using decoder 456
                                             // 91008:     }
                                             // 91008:     } // switch Inst[21]
                                             // 91008:    }
  13, 11,                                    // 91013:    case 0xd: {
  OPC_CheckPredicate, 76,                    // 91015:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91017:     check Inst[21] == 0x1
  OPC_Decode, 128, 18, 200, 3,               // 91021:     decode to CMHIv1i64 using decoder 456
                                             // 91021:    }
  14, 11,                                    // 91026:    case 0xe: {
  OPC_CheckPredicate, 76,                    // 91028:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 91030:     check Inst[21:16] == 0x20
  OPC_Decode, 220, 68, 211, 3,               // 91034:     decode to USQADDv1i64 using decoder 467
                                             // 91034:    }
  15, 11,                                    // 91039:    case 0xf: {
  OPC_CheckPredicate, 76,                    // 91041:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91043:     check Inst[21] == 0x1
  OPC_Decode, 136, 18, 200, 3,               // 91047:     decode to CMHSv1i64 using decoder 456
                                             // 91047:    }
  17, 11,                                    // 91052:    case 0x11: {
  OPC_CheckPredicate, 76,                    // 91054:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91056:     check Inst[21] == 0x1
  OPC_Decode, 168, 68, 200, 3,               // 91060:     decode to USHLv1i64 using decoder 456
                                             // 91060:    }
  19, 11,                                    // 91065:    case 0x13: {
  OPC_CheckPredicate, 76,                    // 91067:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91069:     check Inst[21] == 0x1
  OPC_Decode, 252, 66, 200, 3,               // 91073:     decode to UQSHLv1i64 using decoder 456
                                             // 91073:    }
  21, 11,                                    // 91078:    case 0x15: {
  OPC_CheckPredicate, 76,                    // 91080:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91082:     check Inst[21] == 0x1
  OPC_Decode, 236, 67, 200, 3,               // 91086:     decode to URSHLv1i64 using decoder 456
                                             // 91086:    }
  23, 11,                                    // 91091:    case 0x17: {
  OPC_CheckPredicate, 76,                    // 91093:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91095:     check Inst[21] == 0x1
  OPC_Decode, 202, 66, 200, 3,               // 91099:     decode to UQRSHLv1i64 using decoder 456
                                             // 91099:    }
  30, 11,                                    // 91104:    case 0x1e: {
  OPC_CheckPredicate, 76,                    // 91106:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 91108:     check Inst[21:16] == 0x20
  OPC_Decode, 211, 52, 201, 3,               // 91112:     decode to SQNEGv1i64 using decoder 457
                                             // 91112:    }
  33, 11,                                    // 91117:    case 0x21: {
  OPC_CheckPredicate, 76,                    // 91119:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91121:     check Inst[21] == 0x1
  OPC_Decode, 193, 60, 200, 3,               // 91125:     decode to SUBv1i64 using decoder 456
                                             // 91125:    }
  34, 11,                                    // 91130:    case 0x22: {
  OPC_CheckPredicate, 76,                    // 91132:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 91134:     check Inst[21:16] == 0x20
  OPC_Decode, 226, 17, 201, 3,               // 91138:     decode to CMGEv1i64rz using decoder 457
                                             // 91138:    }
  35, 11,                                    // 91143:    case 0x23: {
  OPC_CheckPredicate, 76,                    // 91145:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91147:     check Inst[21] == 0x1
  OPC_Decode, 209, 17, 200, 3,               // 91151:     decode to CMEQv1i64 using decoder 456
                                             // 91151:    }
  38, 11,                                    // 91156:    case 0x26: {
  OPC_CheckPredicate, 76,                    // 91158:     check predicate 76
  OPC_CheckField, 16, 6, 32,                 // 91160:     check Inst[21:16] == 0x20
  OPC_Decode, 150, 18, 201, 3,               // 91164:     decode to CMLEv1i64rz using decoder 457
                                             // 91164:    }
  42, 21,                                    // 91169:    case 0x2a: {
  OPC_SwitchField, 16, 6,                    // 91171:     switch Inst[21:16] {
  33, 7,                                     // 91174:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 91176:      check predicate 76
  OPC_Decode, 233, 24, 201, 3,               // 91178:      decode to FCVTPUv1i64 using decoder 457
                                             // 91178:     }
  57, 0,                                     // 91183:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 91185:      check predicate 81
  OPC_Decode, 231, 24, 196, 5,               // 91187:      decode to FCVTPUv1f16 using decoder 708
                                             // 91187:     }
                                             // 91187:     } // switch Inst[21:16]
                                             // 91187:    }
  46, 30,                                    // 91192:    case 0x2e: {
  OPC_SwitchField, 16, 6,                    // 91194:     switch Inst[21:16] {
  32, 7,                                     // 91197:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 91199:      check predicate 76
  OPC_Decode, 171, 43, 201, 3,               // 91201:      decode to NEGv1i64 using decoder 457
                                             // 91201:     }
  33, 7,                                     // 91206:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 91208:      check predicate 76
  OPC_Decode, 211, 25, 201, 3,               // 91210:      decode to FCVTZUv1i64 using decoder 457
                                             // 91210:     }
  57, 0,                                     // 91215:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 91217:      check predicate 81
  OPC_Decode, 209, 25, 196, 5,               // 91219:      decode to FCVTZUv1f16 using decoder 708
                                             // 91219:     }
                                             // 91219:     } // switch Inst[21:16]
                                             // 91219:    }
  50, 30,                                    // 91224:    case 0x32: {
  OPC_SwitchField, 16, 6,                    // 91226:     switch Inst[21:16] {
  32, 7,                                     // 91229:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 91231:      check predicate 76
  OPC_Decode, 226, 22, 201, 3,               // 91233:      decode to FCMGEv1i64rz using decoder 457
                                             // 91233:     }
  48, 7,                                     // 91238:     case 0x30: {
  OPC_CheckPredicate, 76,                    // 91240:      check predicate 76
  OPC_Decode, 158, 27, 206, 3,               // 91242:      decode to FMINNMPv2i64p using decoder 462
                                             // 91242:     }
  56, 0,                                     // 91247:     case 0x38: {
  OPC_CheckPredicate, 81,                    // 91249:      check predicate 81
  OPC_Decode, 224, 22, 196, 5,               // 91251:      decode to FCMGEv1i16rz using decoder 708
                                             // 91251:     }
                                             // 91251:     } // switch Inst[21:16]
                                             // 91251:    }
  53, 11,                                    // 91256:    case 0x35: {
  OPC_CheckPredicate, 76,                    // 91258:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91260:     check Inst[21] == 0x1
  OPC_Decode, 182, 21, 200, 3,               // 91264:     decode to FABD64 using decoder 456
                                             // 91264:    }
  54, 39,                                    // 91269:    case 0x36: {
  OPC_SwitchField, 16, 6,                    // 91271:     switch Inst[21:16] {
  32, 7,                                     // 91274:     case 0x20: {
  OPC_CheckPredicate, 76,                    // 91276:      check predicate 76
  OPC_Decode, 149, 23, 201, 3,               // 91278:      decode to FCMLEv1i64rz using decoder 457
                                             // 91278:     }
  33, 7,                                     // 91283:     case 0x21: {
  OPC_CheckPredicate, 76,                    // 91285:      check predicate 76
  OPC_Decode, 240, 31, 201, 3,               // 91287:      decode to FRSQRTEv1i64 using decoder 457
                                             // 91287:     }
  56, 7,                                     // 91292:     case 0x38: {
  OPC_CheckPredicate, 81,                    // 91294:      check predicate 81
  OPC_Decode, 147, 23, 196, 5,               // 91296:      decode to FCMLEv1i16rz using decoder 708
                                             // 91296:     }
  57, 0,                                     // 91301:     case 0x39: {
  OPC_CheckPredicate, 81,                    // 91303:      check predicate 81
  OPC_Decode, 238, 31, 196, 5,               // 91305:      decode to FRSQRTEv1f16 using decoder 708
                                             // 91305:     }
                                             // 91305:     } // switch Inst[21:16]
                                             // 91305:    }
  57, 11,                                    // 91310:    case 0x39: {
  OPC_CheckPredicate, 76,                    // 91312:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91314:     check Inst[21] == 0x1
  OPC_Decode, 239, 22, 200, 3,               // 91318:     decode to FCMGT64 using decoder 456
                                             // 91318:    }
  59, 11,                                    // 91323:    case 0x3b: {
  OPC_CheckPredicate, 76,                    // 91325:     check predicate 76
  OPC_CheckField, 21, 1, 1,                  // 91327:     check Inst[21] == 0x1
  OPC_Decode, 218, 21, 200, 3,               // 91331:     decode to FACGT64 using decoder 456
                                             // 91331:    }
  62, 0,                                     // 91336:    case 0x3e: {
  OPC_CheckPredicate, 76,                    // 91338:     check predicate 76
  OPC_CheckField, 16, 6, 48,                 // 91340:     check Inst[21:16] == 0x30
  OPC_Decode, 202, 27, 206, 3,               // 91344:     decode to FMINPv2i64p using decoder 462
                                             // 91344:    }
                                             // 91344:    } // switch Inst[15:10]
                                             // 91344:   }
  12, 239, 2,                                // 91349:   case 0xc: {
  OPC_SwitchField, 12, 4,                    // 91352:    switch Inst[15:12] {
  6, 51,                                     // 91355:    case 0x6: {
  OPC_SwitchField, 21, 1,                    // 91357:     switch Inst[21] {
  0, 33,                                     // 91360:     case 0x0: {
  OPC_SwitchField, 20, 1,                    // 91362:      switch Inst[20] {
  0, 15,                                     // 91365:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 91367:       check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 91369:       check Inst[19] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 91373:       check Inst[11:10] == 0x1
  OPC_Decode, 225, 53, 236, 5,               // 91377:       decode to SQSHLUb using decoder 748
                                             // 91377:      }
  1, 0,                                      // 91382:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 91384:       check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91386:       check Inst[11:10] == 0x1
  OPC_Decode, 227, 53, 237, 5,               // 91390:       decode to SQSHLUh using decoder 749
                                             // 91390:      }
                                             // 91390:      } // switch Inst[20]
                                             // 91390:     }
  1, 0,                                      // 91395:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 91397:      check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91399:      check Inst[11:10] == 0x1
  OPC_Decode, 228, 53, 238, 5,               // 91403:      decode to SQSHLUs using decoder 750
                                             // 91403:     }
                                             // 91403:     } // switch Inst[21]
                                             // 91403:    }
  7, 51,                                     // 91408:    case 0x7: {
  OPC_SwitchField, 21, 1,                    // 91410:     switch Inst[21] {
  0, 33,                                     // 91413:     case 0x0: {
  OPC_SwitchField, 20, 1,                    // 91415:      switch Inst[20] {
  0, 15,                                     // 91418:      case 0x0: {
  OPC_CheckPredicate, 76,                    // 91420:       check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 91422:       check Inst[19] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 91426:       check Inst[11:10] == 0x1
  OPC_Decode, 244, 66, 236, 5,               // 91430:       decode to UQSHLb using decoder 748
                                             // 91430:      }
  1, 0,                                      // 91435:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 91437:       check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91439:       check Inst[11:10] == 0x1
  OPC_Decode, 246, 66, 237, 5,               // 91443:       decode to UQSHLh using decoder 749
                                             // 91443:      }
                                             // 91443:      } // switch Inst[20]
                                             // 91443:     }
  1, 0,                                      // 91448:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 91450:      check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91452:      check Inst[11:10] == 0x1
  OPC_Decode, 247, 66, 238, 5,               // 91456:      decode to UQSHLs using decoder 750
                                             // 91456:     }
                                             // 91456:     } // switch Inst[21]
                                             // 91456:    }
  8, 85,                                     // 91461:    case 0x8: {
  OPC_SwitchField, 10, 2,                    // 91463:     switch Inst[11:10] {
  1, 39,                                     // 91466:     case 0x1: {
  OPC_SwitchField, 21, 1,                    // 91468:      switch Inst[21] {
  0, 25,                                     // 91471:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 91473:       switch Inst[20] {
  0, 11,                                     // 91476:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 91478:        check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 91480:        check Inst[19] == 0x1
  OPC_Decode, 163, 54, 240, 5,               // 91484:        decode to SQSHRUNb using decoder 752
                                             // 91484:       }
  1, 0,                                      // 91489:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 91491:        check predicate 76
  OPC_Decode, 164, 54, 241, 5,               // 91493:        decode to SQSHRUNh using decoder 753
                                             // 91493:       }
                                             // 91493:       } // switch Inst[20]
                                             // 91493:      }
  1, 0,                                      // 91498:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 91500:       check predicate 76
  OPC_Decode, 165, 54, 242, 5,               // 91502:       decode to SQSHRUNs using decoder 754
                                             // 91502:      }
                                             // 91502:      } // switch Inst[21]
                                             // 91502:     }
  3, 0,                                      // 91507:     case 0x3: {
  OPC_SwitchField, 21, 1,                    // 91509:      switch Inst[21] {
  0, 25,                                     // 91512:      case 0x0: {
  OPC_SwitchField, 20, 1,                    // 91514:       switch Inst[20] {
  0, 11,                                     // 91517:       case 0x0: {
  OPC_CheckPredicate, 76,                    // 91519:        check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 91521:        check Inst[19] == 0x1
  OPC_Decode, 202, 53, 240, 5,               // 91525:        decode to SQRSHRUNb using decoder 752
                                             // 91525:       }
  1, 0,                                      // 91530:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 91532:        check predicate 76
  OPC_Decode, 203, 53, 241, 5,               // 91534:        decode to SQRSHRUNh using decoder 753
                                             // 91534:       }
                                             // 91534:       } // switch Inst[20]
                                             // 91534:      }
  1, 0,                                      // 91539:      case 0x1: {
  OPC_CheckPredicate, 76,                    // 91541:       check predicate 76
  OPC_Decode, 204, 53, 242, 5,               // 91543:       decode to SQRSHRUNs using decoder 754
                                             // 91543:      }
                                             // 91543:      } // switch Inst[21]
                                             // 91543:     }
                                             // 91543:     } // switch Inst[11:10]
                                             // 91543:    }
  9, 99,                                     // 91548:    case 0x9: {
  OPC_SwitchField, 10, 1,                    // 91550:     switch Inst[10] {
  0, 7,                                      // 91553:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 91555:      check predicate 81
  OPC_Decode, 233, 29, 239, 5,               // 91557:      decode to FMULXv1i16_indexed using decoder 751
                                             // 91557:     }
  1, 0,                                      // 91562:     case 0x1: {
  OPC_SwitchField, 11, 1,                    // 91564:      switch Inst[11] {
  0, 39,                                     // 91567:      case 0x0: {
  OPC_SwitchField, 21, 1,                    // 91569:       switch Inst[21] {
  0, 25,                                     // 91572:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 91574:        switch Inst[20] {
  0, 11,                                     // 91577:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 91579:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 91581:         check Inst[19] == 0x1
  OPC_Decode, 146, 67, 240, 5,               // 91585:         decode to UQSHRNb using decoder 752
                                             // 91585:        }
  1, 0,                                      // 91590:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 91592:         check predicate 76
  OPC_Decode, 147, 67, 241, 5,               // 91594:         decode to UQSHRNh using decoder 753
                                             // 91594:        }
                                             // 91594:        } // switch Inst[20]
                                             // 91594:       }
  1, 0,                                      // 91599:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 91601:        check predicate 76
  OPC_Decode, 148, 67, 242, 5,               // 91603:        decode to UQSHRNs using decoder 754
                                             // 91603:       }
                                             // 91603:       } // switch Inst[21]
                                             // 91603:      }
  1, 0,                                      // 91608:      case 0x1: {
  OPC_SwitchField, 21, 1,                    // 91610:       switch Inst[21] {
  0, 25,                                     // 91613:       case 0x0: {
  OPC_SwitchField, 20, 1,                    // 91615:        switch Inst[20] {
  0, 11,                                     // 91618:        case 0x0: {
  OPC_CheckPredicate, 76,                    // 91620:         check predicate 76
  OPC_CheckField, 19, 1, 1,                  // 91622:         check Inst[19] == 0x1
  OPC_Decode, 220, 66, 240, 5,               // 91626:         decode to UQRSHRNb using decoder 752
                                             // 91626:        }
  1, 0,                                      // 91631:        case 0x1: {
  OPC_CheckPredicate, 76,                    // 91633:         check predicate 76
  OPC_Decode, 221, 66, 241, 5,               // 91635:         decode to UQRSHRNh using decoder 753
                                             // 91635:        }
                                             // 91635:        } // switch Inst[20]
                                             // 91635:       }
  1, 0,                                      // 91640:       case 0x1: {
  OPC_CheckPredicate, 76,                    // 91642:        check predicate 76
  OPC_Decode, 222, 66, 242, 5,               // 91644:        decode to UQRSHRNs using decoder 754
                                             // 91644:       }
                                             // 91644:       } // switch Inst[21]
                                             // 91644:      }
                                             // 91644:      } // switch Inst[11]
                                             // 91644:     }
                                             // 91644:     } // switch Inst[10]
                                             // 91644:    }
  14, 33,                                    // 91649:    case 0xe: {
  OPC_SwitchField, 21, 1,                    // 91651:     switch Inst[21] {
  0, 15,                                     // 91654:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 91656:      check predicate 81
  OPC_CheckField, 20, 1, 1,                  // 91658:      check Inst[20] == 0x1
  OPC_CheckField, 10, 2, 1,                  // 91662:      check Inst[11:10] == 0x1
  OPC_Decode, 153, 63, 247, 5,               // 91666:      decode to UCVTFh using decoder 759
                                             // 91666:     }
  1, 0,                                      // 91671:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 91673:      check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91675:      check Inst[11:10] == 0x1
  OPC_Decode, 154, 63, 248, 5,               // 91679:      decode to UCVTFs using decoder 760
                                             // 91679:     }
                                             // 91679:     } // switch Inst[21]
                                             // 91679:    }
  15, 0,                                     // 91684:    case 0xf: {
  OPC_SwitchField, 21, 1,                    // 91686:     switch Inst[21] {
  0, 15,                                     // 91689:     case 0x0: {
  OPC_CheckPredicate, 81,                    // 91691:      check predicate 81
  OPC_CheckField, 20, 1, 1,                  // 91693:      check Inst[20] == 0x1
  OPC_CheckField, 10, 2, 3,                  // 91697:      check Inst[11:10] == 0x3
  OPC_Decode, 207, 25, 247, 5,               // 91701:      decode to FCVTZUh using decoder 759
                                             // 91701:     }
  1, 0,                                      // 91706:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 91708:      check predicate 76
  OPC_CheckField, 10, 2, 3,                  // 91710:      check Inst[11:10] == 0x3
  OPC_Decode, 208, 25, 248, 5,               // 91714:      decode to FCVTZUs using decoder 760
                                             // 91714:     }
                                             // 91714:     } // switch Inst[21]
                                             // 91714:    }
                                             // 91714:    } // switch Inst[15:12]
                                             // 91714:   }
  13, 160, 1,                                // 91719:   case 0xd: {
  OPC_SwitchField, 12, 4,                    // 91722:    switch Inst[15:12] {
  0, 11,                                     // 91725:    case 0x0: {
  OPC_CheckPredicate, 76,                    // 91727:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91729:     check Inst[11:10] == 0x1
  OPC_Decode, 175, 68, 228, 5,               // 91733:     decode to USHRd using decoder 740
                                             // 91733:    }
  1, 11,                                     // 91738:    case 0x1: {
  OPC_CheckPredicate, 76,                    // 91740:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91742:     check Inst[11:10] == 0x1
  OPC_Decode, 232, 68, 230, 5,               // 91746:     decode to USRAd using decoder 742
                                             // 91746:    }
  2, 11,                                     // 91751:    case 0x2: {
  OPC_CheckPredicate, 76,                    // 91753:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91755:     check Inst[11:10] == 0x1
  OPC_Decode, 247, 67, 228, 5,               // 91759:     decode to URSHRd using decoder 740
                                             // 91759:    }
  3, 11,                                     // 91764:    case 0x3: {
  OPC_CheckPredicate, 76,                    // 91766:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91768:     check Inst[11:10] == 0x1
  OPC_Decode, 135, 68, 230, 5,               // 91772:     decode to URSRAd using decoder 742
                                             // 91772:    }
  4, 11,                                     // 91777:    case 0x4: {
  OPC_CheckPredicate, 76,                    // 91779:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91781:     check Inst[11:10] == 0x1
  OPC_Decode, 243, 54, 230, 5,               // 91785:     decode to SRId using decoder 742
                                             // 91785:    }
  5, 11,                                     // 91790:    case 0x5: {
  OPC_CheckPredicate, 76,                    // 91792:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91794:     check Inst[11:10] == 0x1
  OPC_Decode, 200, 48, 253, 5,               // 91798:     decode to SLId using decoder 765
                                             // 91798:    }
  6, 11,                                     // 91803:    case 0x6: {
  OPC_CheckPredicate, 76,                    // 91805:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91807:     check Inst[11:10] == 0x1
  OPC_Decode, 226, 53, 235, 5,               // 91811:     decode to SQSHLUd using decoder 747
                                             // 91811:    }
  7, 11,                                     // 91816:    case 0x7: {
  OPC_CheckPredicate, 76,                    // 91818:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91820:     check Inst[11:10] == 0x1
  OPC_Decode, 245, 66, 235, 5,               // 91824:     decode to UQSHLd using decoder 747
                                             // 91824:    }
  13, 11,                                    // 91829:    case 0xd: {
  OPC_CheckPredicate, 90,                    // 91831:     check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 91833:     check Inst[10] == 0x0
  OPC_Decode, 233, 52, 229, 5,               // 91837:     decode to SQRDMLAHv1i16_indexed using decoder 741
                                             // 91837:    }
  14, 11,                                    // 91842:    case 0xe: {
  OPC_CheckPredicate, 76,                    // 91844:     check predicate 76
  OPC_CheckField, 10, 2, 1,                  // 91846:     check Inst[11:10] == 0x1
  OPC_Decode, 152, 63, 228, 5,               // 91850:     decode to UCVTFd using decoder 740
                                             // 91850:    }
  15, 0,                                     // 91855:    case 0xf: {
  OPC_SwitchField, 10, 1,                    // 91857:     switch Inst[10] {
  0, 7,                                      // 91860:     case 0x0: {
  OPC_CheckPredicate, 90,                    // 91862:      check predicate 90
  OPC_Decode, 252, 52, 229, 5,               // 91864:      decode to SQRDMLSHv1i16_indexed using decoder 741
                                             // 91864:     }
  1, 0,                                      // 91869:     case 0x1: {
  OPC_CheckPredicate, 76,                    // 91871:      check predicate 76
  OPC_CheckField, 11, 1, 1,                  // 91873:      check Inst[11] == 0x1
  OPC_Decode, 206, 25, 228, 5,               // 91877:      decode to FCVTZUd using decoder 740
                                             // 91877:     }
                                             // 91877:     } // switch Inst[10]
                                             // 91877:    }
                                             // 91877:    } // switch Inst[15:12]
                                             // 91877:   }
  14, 42,                                    // 91882:   case 0xe: {
  OPC_SwitchField, 12, 4,                    // 91884:    switch Inst[15:12] {
  9, 11,                                     // 91887:    case 0x9: {
  OPC_CheckPredicate, 76,                    // 91889:     check predicate 76
  OPC_CheckField, 10, 1, 0,                  // 91891:     check Inst[10] == 0x0
  OPC_Decode, 234, 29, 243, 5,               // 91895:     decode to FMULXv1i32_indexed using decoder 755
                                             // 91895:    }
  13, 11,                                    // 91900:    case 0xd: {
  OPC_CheckPredicate, 90,                    // 91902:     check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 91904:     check Inst[10] == 0x0
  OPC_Decode, 235, 52, 231, 5,               // 91908:     decode to SQRDMLAHv1i32_indexed using decoder 743
                                             // 91908:    }
  15, 0,                                     // 91913:    case 0xf: {
  OPC_CheckPredicate, 90,                    // 91915:     check predicate 90
  OPC_CheckField, 10, 1, 0,                  // 91917:     check Inst[10] == 0x0
  OPC_Decode, 254, 52, 231, 5,               // 91921:     decode to SQRDMLSHv1i32_indexed using decoder 743
                                             // 91921:    }
                                             // 91921:    } // switch Inst[15:12]
                                             // 91921:   }
  15, 0,                                     // 91926:   case 0xf: {
  OPC_CheckPredicate, 76,                    // 91928:    check predicate 76
  OPC_CheckField, 21, 1, 0,                  // 91930:    check Inst[21] == 0x0
  OPC_CheckField, 12, 4, 9,                  // 91934:    check Inst[15:12] == 0x9
  OPC_CheckField, 10, 1, 0,                  // 91938:    check Inst[10] == 0x0
  OPC_Decode, 235, 29, 244, 5,               // 91942:    decode to FMULXv1i64_indexed using decoder 756
                                             // 91942:   }
                                             // 91942:   } // switch Inst[25:22]
                                             // 91942:  }
  4, 192, 8,                                 // 91947:  case 0x4: {
  OPC_SwitchField, 24, 2,                    // 91950:   switch Inst[25:24] {
  0, 7,                                      // 91953:   case 0x0: {
  OPC_CheckPredicate, 77,                    // 91955:    check predicate 77
  OPC_Decode, 194, 39, 254, 5,               // 91957:    decode to LDRQl using decoder 766
                                             // 91957:   }
  1, 29,                                     // 91962:   case 0x1: {
  OPC_SwitchField, 21, 3,                    // 91964:    switch Inst[23:21] {
  0, 11,                                     // 91967:    case 0x0: {
  OPC_CheckPredicate, 79,                    // 91969:     check predicate 79
  OPC_CheckField, 10, 2, 2,                  // 91971:     check Inst[11:10] == 0x2
  OPC_Decode, 218, 58, 255, 5,               // 91975:     decode to STLURsi using decoder 767
                                             // 91975:    }
  2, 0,                                      // 91980:    case 0x2: {
  OPC_CheckPredicate, 79,                    // 91982:     check predicate 79
  OPC_CheckField, 10, 2, 2,                  // 91984:     check Inst[11:10] == 0x2
  OPC_Decode, 173, 37, 255, 5,               // 91988:     decode to LDAPURsi using decoder 767
                                             // 91988:    }
                                             // 91988:    } // switch Inst[23:21]
                                             // 91988:   }
  2, 0,                                      // 91993:   case 0x2: {
  OPC_SwitchField, 16, 8,                    // 91995:    switch Inst[23:16] {
  2, 7,                                      // 91998:    case 0x2: {
  OPC_CheckPredicate, 77,                    // 92000:     check predicate 77
  OPC_Decode, 133, 47, 128, 6,               // 92002:     decode to SCVTFSXSri using decoder 768
                                             // 92002:    }
  3, 7,                                      // 92007:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 92009:     check predicate 77
  OPC_Decode, 254, 62, 128, 6,               // 92011:     decode to UCVTFSXSri using decoder 768
                                             // 92011:    }
  24, 7,                                     // 92016:    case 0x18: {
  OPC_CheckPredicate, 77,                    // 92018:     check predicate 77
  OPC_Decode, 132, 25, 129, 6,               // 92020:     decode to FCVTZSSXSri using decoder 769
                                             // 92020:    }
  25, 7,                                     // 92025:    case 0x19: {
  OPC_CheckPredicate, 77,                    // 92027:     check predicate 77
  OPC_Decode, 183, 25, 129, 6,               // 92029:     decode to FCVTZUSXSri using decoder 769
                                             // 92029:    }
  32, 11,                                    // 92034:    case 0x20: {
  OPC_CheckPredicate, 77,                    // 92036:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92038:     check Inst[15:10] == 0x0
  OPC_Decode, 160, 24, 130, 6,               // 92042:     decode to FCVTNSUXSr using decoder 770
                                             // 92042:    }
  33, 11,                                    // 92047:    case 0x21: {
  OPC_CheckPredicate, 77,                    // 92049:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92051:     check Inst[15:10] == 0x0
  OPC_Decode, 183, 24, 130, 6,               // 92055:     decode to FCVTNUUXSr using decoder 770
                                             // 92055:    }
  34, 11,                                    // 92060:    case 0x22: {
  OPC_CheckPredicate, 77,                    // 92062:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92064:     check Inst[15:10] == 0x0
  OPC_Decode, 139, 47, 131, 6,               // 92068:     decode to SCVTFUXSri using decoder 771
                                             // 92068:    }
  35, 11,                                    // 92073:    case 0x23: {
  OPC_CheckPredicate, 77,                    // 92075:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92077:     check Inst[15:10] == 0x0
  OPC_Decode, 132, 63, 131, 6,               // 92081:     decode to UCVTFUXSri using decoder 771
                                             // 92081:    }
  36, 11,                                    // 92086:    case 0x24: {
  OPC_CheckPredicate, 77,                    // 92088:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92090:     check Inst[15:10] == 0x0
  OPC_Decode, 202, 23, 130, 6,               // 92094:     decode to FCVTASUXSr using decoder 770
                                             // 92094:    }
  37, 11,                                    // 92099:    case 0x25: {
  OPC_CheckPredicate, 77,                    // 92101:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92103:     check Inst[15:10] == 0x0
  OPC_Decode, 220, 23, 130, 6,               // 92107:     decode to FCVTAUUXSr using decoder 770
                                             // 92107:    }
  40, 11,                                    // 92112:    case 0x28: {
  OPC_CheckPredicate, 77,                    // 92114:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92116:     check Inst[15:10] == 0x0
  OPC_Decode, 212, 24, 130, 6,               // 92120:     decode to FCVTPSUXSr using decoder 770
                                             // 92120:    }
  41, 11,                                    // 92125:    case 0x29: {
  OPC_CheckPredicate, 77,                    // 92127:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92129:     check Inst[15:10] == 0x0
  OPC_Decode, 230, 24, 130, 6,               // 92133:     decode to FCVTPUUXSr using decoder 770
                                             // 92133:    }
  42, 12,                                    // 92138:    case 0x2a: {
  OPC_CheckPredicate, 136, 1,                // 92140:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92143:     check Inst[15:10] == 0x0
  OPC_Decode, 152, 24, 175, 5,               // 92147:     decode to FCVTNSDSr using decoder 687
                                             // 92147:    }
  43, 12,                                    // 92152:    case 0x2b: {
  OPC_CheckPredicate, 136, 1,                // 92154:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92157:     check Inst[15:10] == 0x0
  OPC_Decode, 175, 24, 175, 5,               // 92161:     decode to FCVTNUDSr using decoder 687
                                             // 92161:    }
  48, 11,                                    // 92166:    case 0x30: {
  OPC_CheckPredicate, 77,                    // 92168:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92170:     check Inst[15:10] == 0x0
  OPC_Decode, 251, 23, 130, 6,               // 92174:     decode to FCVTMSUXSr using decoder 770
                                             // 92174:    }
  49, 11,                                    // 92179:    case 0x31: {
  OPC_CheckPredicate, 77,                    // 92181:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92183:     check Inst[15:10] == 0x0
  OPC_Decode, 141, 24, 130, 6,               // 92187:     decode to FCVTMUUXSr using decoder 770
                                             // 92187:    }
  50, 12,                                    // 92192:    case 0x32: {
  OPC_CheckPredicate, 136, 1,                // 92194:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92197:     check Inst[15:10] == 0x0
  OPC_Decode, 204, 24, 175, 5,               // 92201:     decode to FCVTPSDSr using decoder 687
                                             // 92201:    }
  51, 12,                                    // 92206:    case 0x33: {
  OPC_CheckPredicate, 136, 1,                // 92208:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92211:     check Inst[15:10] == 0x0
  OPC_Decode, 222, 24, 175, 5,               // 92215:     decode to FCVTPUDSr using decoder 687
                                             // 92215:    }
  52, 12,                                    // 92220:    case 0x34: {
  OPC_CheckPredicate, 136, 1,                // 92222:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92225:     check Inst[15:10] == 0x0
  OPC_Decode, 243, 23, 175, 5,               // 92229:     decode to FCVTMSDSr using decoder 687
                                             // 92229:    }
  53, 12,                                    // 92234:    case 0x35: {
  OPC_CheckPredicate, 136, 1,                // 92236:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92239:     check Inst[15:10] == 0x0
  OPC_Decode, 133, 24, 175, 5,               // 92243:     decode to FCVTMUDSr using decoder 687
                                             // 92243:    }
  54, 12,                                    // 92248:    case 0x36: {
  OPC_CheckPredicate, 136, 1,                // 92250:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92253:     check Inst[15:10] == 0x0
  OPC_Decode, 249, 24, 175, 5,               // 92257:     decode to FCVTZSDSr using decoder 687
                                             // 92257:    }
  55, 12,                                    // 92262:    case 0x37: {
  OPC_CheckPredicate, 136, 1,                // 92264:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92267:     check Inst[15:10] == 0x0
  OPC_Decode, 172, 25, 175, 5,               // 92271:     decode to FCVTZUDSr using decoder 687
                                             // 92271:    }
  56, 11,                                    // 92276:    case 0x38: {
  OPC_CheckPredicate, 77,                    // 92278:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92280:     check Inst[15:10] == 0x0
  OPC_Decode, 138, 25, 130, 6,               // 92284:     decode to FCVTZSUXSr using decoder 770
                                             // 92284:    }
  57, 11,                                    // 92289:    case 0x39: {
  OPC_CheckPredicate, 77,                    // 92291:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92293:     check Inst[15:10] == 0x0
  OPC_Decode, 189, 25, 130, 6,               // 92297:     decode to FCVTZUUXSr using decoder 770
                                             // 92297:    }
  58, 12,                                    // 92302:    case 0x3a: {
  OPC_CheckPredicate, 136, 1,                // 92304:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92307:     check Inst[15:10] == 0x0
  OPC_Decode, 194, 23, 175, 5,               // 92311:     decode to FCVTASDSr using decoder 687
                                             // 92311:    }
  59, 12,                                    // 92316:    case 0x3b: {
  OPC_CheckPredicate, 136, 1,                // 92318:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92321:     check Inst[15:10] == 0x0
  OPC_Decode, 212, 23, 175, 5,               // 92325:     decode to FCVTAUDSr using decoder 687
                                             // 92325:    }
  60, 12,                                    // 92330:    case 0x3c: {
  OPC_CheckPredicate, 136, 1,                // 92332:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92335:     check Inst[15:10] == 0x0
  OPC_Decode, 255, 46, 130, 4,               // 92339:     decode to SCVTFSDr using decoder 514
                                             // 92339:    }
  61, 12,                                    // 92344:    case 0x3d: {
  OPC_CheckPredicate, 136, 1,                // 92346:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92349:     check Inst[15:10] == 0x0
  OPC_Decode, 248, 62, 130, 4,               // 92353:     decode to UCVTFSDr using decoder 514
                                             // 92353:    }
  66, 7,                                     // 92358:    case 0x42: {
  OPC_CheckPredicate, 77,                    // 92360:     check predicate 77
  OPC_Decode, 131, 47, 132, 6,               // 92362:     decode to SCVTFSXDri using decoder 772
                                             // 92362:    }
  67, 7,                                     // 92367:    case 0x43: {
  OPC_CheckPredicate, 77,                    // 92369:     check predicate 77
  OPC_Decode, 252, 62, 132, 6,               // 92371:     decode to UCVTFSXDri using decoder 772
                                             // 92371:    }
  88, 7,                                     // 92376:    case 0x58: {
  OPC_CheckPredicate, 77,                    // 92378:     check predicate 77
  OPC_Decode, 130, 25, 133, 6,               // 92380:     decode to FCVTZSSXDri using decoder 773
                                             // 92380:    }
  89, 7,                                     // 92385:    case 0x59: {
  OPC_CheckPredicate, 77,                    // 92387:     check predicate 77
  OPC_Decode, 181, 25, 133, 6,               // 92389:     decode to FCVTZUSXDri using decoder 773
                                             // 92389:    }
  96, 11,                                    // 92394:    case 0x60: {
  OPC_CheckPredicate, 77,                    // 92396:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92398:     check Inst[15:10] == 0x0
  OPC_Decode, 158, 24, 134, 6,               // 92402:     decode to FCVTNSUXDr using decoder 774
                                             // 92402:    }
  97, 11,                                    // 92407:    case 0x61: {
  OPC_CheckPredicate, 77,                    // 92409:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92411:     check Inst[15:10] == 0x0
  OPC_Decode, 181, 24, 134, 6,               // 92415:     decode to FCVTNUUXDr using decoder 774
                                             // 92415:    }
  98, 11,                                    // 92420:    case 0x62: {
  OPC_CheckPredicate, 77,                    // 92422:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92424:     check Inst[15:10] == 0x0
  OPC_Decode, 137, 47, 135, 6,               // 92428:     decode to SCVTFUXDri using decoder 775
                                             // 92428:    }
  99, 11,                                    // 92433:    case 0x63: {
  OPC_CheckPredicate, 77,                    // 92435:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92437:     check Inst[15:10] == 0x0
  OPC_Decode, 130, 63, 135, 6,               // 92441:     decode to UCVTFUXDri using decoder 775
                                             // 92441:    }
  100, 11,                                   // 92446:    case 0x64: {
  OPC_CheckPredicate, 77,                    // 92448:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92450:     check Inst[15:10] == 0x0
  OPC_Decode, 200, 23, 134, 6,               // 92454:     decode to FCVTASUXDr using decoder 774
                                             // 92454:    }
  101, 11,                                   // 92459:    case 0x65: {
  OPC_CheckPredicate, 77,                    // 92461:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92463:     check Inst[15:10] == 0x0
  OPC_Decode, 218, 23, 134, 6,               // 92467:     decode to FCVTAUUXDr using decoder 774
                                             // 92467:    }
  102, 11,                                   // 92472:    case 0x66: {
  OPC_CheckPredicate, 77,                    // 92474:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92476:     check Inst[15:10] == 0x0
  OPC_Decode, 198, 29, 134, 6,               // 92480:     decode to FMOVDXr using decoder 774
                                             // 92480:    }
  103, 11,                                   // 92485:    case 0x67: {
  OPC_CheckPredicate, 77,                    // 92487:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92489:     check Inst[15:10] == 0x0
  OPC_Decode, 211, 29, 135, 6,               // 92493:     decode to FMOVXDr using decoder 775
                                             // 92493:    }
  104, 11,                                   // 92498:    case 0x68: {
  OPC_CheckPredicate, 77,                    // 92500:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92502:     check Inst[15:10] == 0x0
  OPC_Decode, 210, 24, 134, 6,               // 92506:     decode to FCVTPSUXDr using decoder 774
                                             // 92506:    }
  105, 11,                                   // 92511:    case 0x69: {
  OPC_CheckPredicate, 77,                    // 92513:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92515:     check Inst[15:10] == 0x0
  OPC_Decode, 228, 24, 134, 6,               // 92519:     decode to FCVTPUUXDr using decoder 774
                                             // 92519:    }
  112, 11,                                   // 92524:    case 0x70: {
  OPC_CheckPredicate, 77,                    // 92526:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92528:     check Inst[15:10] == 0x0
  OPC_Decode, 249, 23, 134, 6,               // 92532:     decode to FCVTMSUXDr using decoder 774
                                             // 92532:    }
  113, 11,                                   // 92537:    case 0x71: {
  OPC_CheckPredicate, 77,                    // 92539:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92541:     check Inst[15:10] == 0x0
  OPC_Decode, 139, 24, 134, 6,               // 92545:     decode to FCVTMUUXDr using decoder 774
                                             // 92545:    }
  120, 11,                                   // 92550:    case 0x78: {
  OPC_CheckPredicate, 77,                    // 92552:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92554:     check Inst[15:10] == 0x0
  OPC_Decode, 136, 25, 134, 6,               // 92558:     decode to FCVTZSUXDr using decoder 774
                                             // 92558:    }
  121, 11,                                   // 92563:    case 0x79: {
  OPC_CheckPredicate, 77,                    // 92565:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92567:     check Inst[15:10] == 0x0
  OPC_Decode, 187, 25, 134, 6,               // 92571:     decode to FCVTZUUXDr using decoder 774
                                             // 92571:    }
  174, 1, 11,                                // 92576:    case 0xae: {
  OPC_CheckPredicate, 77,                    // 92579:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92581:     check Inst[15:10] == 0x0
  OPC_Decode, 197, 29, 136, 6,               // 92585:     decode to FMOVDXHighr using decoder 776
                                             // 92585:    }
  175, 1, 11,                                // 92590:    case 0xaf: {
  OPC_CheckPredicate, 77,                    // 92593:     check predicate 77
  OPC_CheckField, 10, 6, 0,                  // 92595:     check Inst[15:10] == 0x0
  OPC_Decode, 210, 29, 136, 6,               // 92599:     decode to FMOVXDHighr using decoder 776
                                             // 92599:    }
  194, 1, 8,                                 // 92604:    case 0xc2: {
  OPC_CheckPredicate, 138, 1,                // 92607:     check predicate 138
  OPC_Decode, 132, 47, 137, 6,               // 92610:     decode to SCVTFSXHri using decoder 777
                                             // 92610:    }
  195, 1, 8,                                 // 92615:    case 0xc3: {
  OPC_CheckPredicate, 138, 1,                // 92618:     check predicate 138
  OPC_Decode, 253, 62, 137, 6,               // 92621:     decode to UCVTFSXHri using decoder 777
                                             // 92621:    }
  216, 1, 8,                                 // 92626:    case 0xd8: {
  OPC_CheckPredicate, 138, 1,                // 92629:     check predicate 138
  OPC_Decode, 131, 25, 138, 6,               // 92632:     decode to FCVTZSSXHri using decoder 778
                                             // 92632:    }
  217, 1, 8,                                 // 92637:    case 0xd9: {
  OPC_CheckPredicate, 138, 1,                // 92640:     check predicate 138
  OPC_Decode, 182, 25, 138, 6,               // 92643:     decode to FCVTZUSXHri using decoder 778
                                             // 92643:    }
  224, 1, 12,                                // 92648:    case 0xe0: {
  OPC_CheckPredicate, 138, 1,                // 92651:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92654:     check Inst[15:10] == 0x0
  OPC_Decode, 159, 24, 139, 6,               // 92658:     decode to FCVTNSUXHr using decoder 779
                                             // 92658:    }
  225, 1, 12,                                // 92663:    case 0xe1: {
  OPC_CheckPredicate, 138, 1,                // 92666:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92669:     check Inst[15:10] == 0x0
  OPC_Decode, 182, 24, 139, 6,               // 92673:     decode to FCVTNUUXHr using decoder 779
                                             // 92673:    }
  226, 1, 12,                                // 92678:    case 0xe2: {
  OPC_CheckPredicate, 138, 1,                // 92681:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92684:     check Inst[15:10] == 0x0
  OPC_Decode, 138, 47, 140, 6,               // 92688:     decode to SCVTFUXHri using decoder 780
                                             // 92688:    }
  227, 1, 12,                                // 92693:    case 0xe3: {
  OPC_CheckPredicate, 138, 1,                // 92696:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92699:     check Inst[15:10] == 0x0
  OPC_Decode, 131, 63, 140, 6,               // 92703:     decode to UCVTFUXHri using decoder 780
                                             // 92703:    }
  228, 1, 12,                                // 92708:    case 0xe4: {
  OPC_CheckPredicate, 138, 1,                // 92711:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92714:     check Inst[15:10] == 0x0
  OPC_Decode, 201, 23, 139, 6,               // 92718:     decode to FCVTASUXHr using decoder 779
                                             // 92718:    }
  229, 1, 12,                                // 92723:    case 0xe5: {
  OPC_CheckPredicate, 138, 1,                // 92726:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92729:     check Inst[15:10] == 0x0
  OPC_Decode, 219, 23, 139, 6,               // 92733:     decode to FCVTAUUXHr using decoder 779
                                             // 92733:    }
  230, 1, 12,                                // 92738:    case 0xe6: {
  OPC_CheckPredicate, 138, 1,                // 92741:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92744:     check Inst[15:10] == 0x0
  OPC_Decode, 202, 29, 139, 6,               // 92748:     decode to FMOVHXr using decoder 779
                                             // 92748:    }
  231, 1, 12,                                // 92753:    case 0xe7: {
  OPC_CheckPredicate, 138, 1,                // 92756:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92759:     check Inst[15:10] == 0x0
  OPC_Decode, 212, 29, 140, 6,               // 92763:     decode to FMOVXHr using decoder 780
                                             // 92763:    }
  232, 1, 12,                                // 92768:    case 0xe8: {
  OPC_CheckPredicate, 138, 1,                // 92771:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92774:     check Inst[15:10] == 0x0
  OPC_Decode, 211, 24, 139, 6,               // 92778:     decode to FCVTPSUXHr using decoder 779
                                             // 92778:    }
  233, 1, 12,                                // 92783:    case 0xe9: {
  OPC_CheckPredicate, 138, 1,                // 92786:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92789:     check Inst[15:10] == 0x0
  OPC_Decode, 229, 24, 139, 6,               // 92793:     decode to FCVTPUUXHr using decoder 779
                                             // 92793:    }
  234, 1, 12,                                // 92798:    case 0xea: {
  OPC_CheckPredicate, 136, 1,                // 92801:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92804:     check Inst[15:10] == 0x0
  OPC_Decode, 151, 24, 197, 5,               // 92808:     decode to FCVTNSDHr using decoder 709
                                             // 92808:    }
  235, 1, 12,                                // 92813:    case 0xeb: {
  OPC_CheckPredicate, 136, 1,                // 92816:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92819:     check Inst[15:10] == 0x0
  OPC_Decode, 174, 24, 197, 5,               // 92823:     decode to FCVTNUDHr using decoder 709
                                             // 92823:    }
  240, 1, 12,                                // 92828:    case 0xf0: {
  OPC_CheckPredicate, 138, 1,                // 92831:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92834:     check Inst[15:10] == 0x0
  OPC_Decode, 250, 23, 139, 6,               // 92838:     decode to FCVTMSUXHr using decoder 779
                                             // 92838:    }
  241, 1, 12,                                // 92843:    case 0xf1: {
  OPC_CheckPredicate, 138, 1,                // 92846:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92849:     check Inst[15:10] == 0x0
  OPC_Decode, 140, 24, 139, 6,               // 92853:     decode to FCVTMUUXHr using decoder 779
                                             // 92853:    }
  242, 1, 12,                                // 92858:    case 0xf2: {
  OPC_CheckPredicate, 136, 1,                // 92861:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92864:     check Inst[15:10] == 0x0
  OPC_Decode, 203, 24, 197, 5,               // 92868:     decode to FCVTPSDHr using decoder 709
                                             // 92868:    }
  243, 1, 12,                                // 92873:    case 0xf3: {
  OPC_CheckPredicate, 136, 1,                // 92876:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92879:     check Inst[15:10] == 0x0
  OPC_Decode, 221, 24, 197, 5,               // 92883:     decode to FCVTPUDHr using decoder 709
                                             // 92883:    }
  244, 1, 12,                                // 92888:    case 0xf4: {
  OPC_CheckPredicate, 136, 1,                // 92891:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92894:     check Inst[15:10] == 0x0
  OPC_Decode, 242, 23, 197, 5,               // 92898:     decode to FCVTMSDHr using decoder 709
                                             // 92898:    }
  245, 1, 12,                                // 92903:    case 0xf5: {
  OPC_CheckPredicate, 136, 1,                // 92906:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92909:     check Inst[15:10] == 0x0
  OPC_Decode, 132, 24, 197, 5,               // 92913:     decode to FCVTMUDHr using decoder 709
                                             // 92913:    }
  246, 1, 12,                                // 92918:    case 0xf6: {
  OPC_CheckPredicate, 136, 1,                // 92921:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92924:     check Inst[15:10] == 0x0
  OPC_Decode, 248, 24, 197, 5,               // 92928:     decode to FCVTZSDHr using decoder 709
                                             // 92928:    }
  247, 1, 12,                                // 92933:    case 0xf7: {
  OPC_CheckPredicate, 136, 1,                // 92936:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92939:     check Inst[15:10] == 0x0
  OPC_Decode, 171, 25, 197, 5,               // 92943:     decode to FCVTZUDHr using decoder 709
                                             // 92943:    }
  248, 1, 12,                                // 92948:    case 0xf8: {
  OPC_CheckPredicate, 138, 1,                // 92951:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92954:     check Inst[15:10] == 0x0
  OPC_Decode, 137, 25, 139, 6,               // 92958:     decode to FCVTZSUXHr using decoder 779
                                             // 92958:    }
  249, 1, 12,                                // 92963:    case 0xf9: {
  OPC_CheckPredicate, 138, 1,                // 92966:     check predicate 138
  OPC_CheckField, 10, 6, 0,                  // 92969:     check Inst[15:10] == 0x0
  OPC_Decode, 188, 25, 139, 6,               // 92973:     decode to FCVTZUUXHr using decoder 779
                                             // 92973:    }
  250, 1, 12,                                // 92978:    case 0xfa: {
  OPC_CheckPredicate, 136, 1,                // 92981:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92984:     check Inst[15:10] == 0x0
  OPC_Decode, 193, 23, 197, 5,               // 92988:     decode to FCVTASDHr using decoder 709
                                             // 92988:    }
  251, 1, 12,                                // 92993:    case 0xfb: {
  OPC_CheckPredicate, 136, 1,                // 92996:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 92999:     check Inst[15:10] == 0x0
  OPC_Decode, 211, 23, 197, 5,               // 93003:     decode to FCVTAUDHr using decoder 709
                                             // 93003:    }
  252, 1, 12,                                // 93008:    case 0xfc: {
  OPC_CheckPredicate, 136, 1,                // 93011:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 93014:     check Inst[15:10] == 0x0
  OPC_Decode, 250, 46, 212, 3,               // 93018:     decode to SCVTFHDr using decoder 468
                                             // 93018:    }
  253, 1, 0,                                 // 93023:    case 0xfd: {
  OPC_CheckPredicate, 136, 1,                // 93026:     check predicate 136
  OPC_CheckField, 10, 6, 0,                  // 93029:     check Inst[15:10] == 0x0
  OPC_Decode, 243, 62, 212, 3,               // 93033:     decode to UCVTFHDr using decoder 468
                                             // 93033:    }
                                             // 93033:    } // switch Inst[23:16]
                                             // 93033:   }
                                             // 93033:   } // switch Inst[25:24]
                                             // 93033:  }
  5, 193, 4,                                 // 93038:  case 0x5: {
  OPC_SwitchField, 22, 4,                    // 93041:   switch Inst[25:22] {
  0, 200, 1,                                 // 93044:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 93047:    switch Inst[11:10] {
  0, 137, 1,                                 // 93050:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 93053:     switch Inst[21] {
  0, 7,                                      // 93056:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 93058:      check predicate 77
  OPC_Decode, 236, 59, 247, 4,               // 93060:      decode to STURSi using decoder 631
                                             // 93060:     }
  1, 0,                                      // 93065:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 93067:      switch Inst[15:12] {
  0, 8,                                      // 93070:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 93072:       check predicate 139
  OPC_Decode, 254, 37, 141, 6,               // 93075:       decode to LDFADDS using decoder 781
                                             // 93075:      }
  4, 8,                                      // 93080:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 93082:       check predicate 139
  OPC_Decode, 166, 38, 141, 6,               // 93085:       decode to LDFMAXS using decoder 781
                                             // 93085:      }
  5, 8,                                      // 93090:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 93092:       check predicate 139
  OPC_Decode, 190, 38, 141, 6,               // 93095:       decode to LDFMINS using decoder 781
                                             // 93095:      }
  6, 8,                                      // 93100:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 93102:       check predicate 139
  OPC_Decode, 165, 38, 141, 6,               // 93105:       decode to LDFMAXNMS using decoder 781
                                             // 93105:      }
  7, 8,                                      // 93110:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 93112:       check predicate 139
  OPC_Decode, 189, 38, 141, 6,               // 93115:       decode to LDFMINNMS using decoder 781
                                             // 93115:      }
  8, 12,                                     // 93120:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 93122:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93125:       check Inst[4:0] == 0x1f
  OPC_Decode, 160, 58, 142, 6,               // 93129:       decode to STFADDS using decoder 782
                                             // 93129:      }
  12, 12,                                    // 93134:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 93136:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93139:       check Inst[4:0] == 0x1f
  OPC_Decode, 172, 58, 142, 6,               // 93143:       decode to STFMAXS using decoder 782
                                             // 93143:      }
  13, 12,                                    // 93148:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 93150:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93153:       check Inst[4:0] == 0x1f
  OPC_Decode, 184, 58, 142, 6,               // 93157:       decode to STFMINS using decoder 782
                                             // 93157:      }
  14, 12,                                    // 93162:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 93164:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93167:       check Inst[4:0] == 0x1f
  OPC_Decode, 171, 58, 142, 6,               // 93171:       decode to STFMAXNMS using decoder 782
                                             // 93171:      }
  15, 0,                                     // 93176:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 93178:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93181:       check Inst[4:0] == 0x1f
  OPC_Decode, 183, 58, 142, 6,               // 93185:       decode to STFMINNMS using decoder 782
                                             // 93185:      }
                                             // 93185:      } // switch Inst[15:12]
                                             // 93185:     }
                                             // 93185:     } // switch Inst[21]
                                             // 93185:    }
  1, 11,                                     // 93190:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 93192:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 93194:     check Inst[21] == 0x0
  OPC_Decode, 196, 59, 247, 4,               // 93198:     decode to STRSpost using decoder 631
                                             // 93198:    }
  2, 29,                                     // 93203:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 93205:     switch Inst[14:13] {
  2, 11,                                     // 93208:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 93210:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 93212:      check Inst[21] == 0x1
  OPC_Decode, 198, 59, 143, 6,               // 93216:      decode to STRSroW using decoder 783
                                             // 93216:     }
  3, 0,                                      // 93221:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 93223:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 93225:      check Inst[21] == 0x1
  OPC_Decode, 199, 59, 144, 6,               // 93229:      decode to STRSroX using decoder 784
                                             // 93229:     }
                                             // 93229:     } // switch Inst[14:13]
                                             // 93229:    }
  3, 0,                                      // 93234:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 93236:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 93238:     check Inst[21] == 0x0
  OPC_Decode, 197, 59, 247, 4,               // 93242:     decode to STRSpre using decoder 631
                                             // 93242:    }
                                             // 93242:    } // switch Inst[11:10]
                                             // 93242:   }
  1, 200, 1,                                 // 93247:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 93250:    switch Inst[11:10] {
  0, 137, 1,                                 // 93253:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 93256:     switch Inst[21] {
  0, 7,                                      // 93259:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 93261:      check predicate 77
  OPC_Decode, 130, 41, 247, 4,               // 93263:      decode to LDURSi using decoder 631
                                             // 93263:     }
  1, 0,                                      // 93268:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 93270:      switch Inst[15:12] {
  0, 8,                                      // 93273:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 93275:       check predicate 139
  OPC_Decode, 253, 37, 141, 6,               // 93278:       decode to LDFADDLS using decoder 781
                                             // 93278:      }
  4, 8,                                      // 93283:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 93285:       check predicate 139
  OPC_Decode, 153, 38, 141, 6,               // 93288:       decode to LDFMAXLS using decoder 781
                                             // 93288:      }
  5, 8,                                      // 93293:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 93295:       check predicate 139
  OPC_Decode, 177, 38, 141, 6,               // 93298:       decode to LDFMINLS using decoder 781
                                             // 93298:      }
  6, 8,                                      // 93303:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 93305:       check predicate 139
  OPC_Decode, 164, 38, 141, 6,               // 93308:       decode to LDFMAXNMLS using decoder 781
                                             // 93308:      }
  7, 8,                                      // 93313:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 93315:       check predicate 139
  OPC_Decode, 188, 38, 141, 6,               // 93318:       decode to LDFMINNMLS using decoder 781
                                             // 93318:      }
  8, 12,                                     // 93323:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 93325:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93328:       check Inst[4:0] == 0x1f
  OPC_Decode, 159, 58, 142, 6,               // 93332:       decode to STFADDLS using decoder 782
                                             // 93332:      }
  12, 12,                                    // 93337:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 93339:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93342:       check Inst[4:0] == 0x1f
  OPC_Decode, 165, 58, 142, 6,               // 93346:       decode to STFMAXLS using decoder 782
                                             // 93346:      }
  13, 12,                                    // 93351:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 93353:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93356:       check Inst[4:0] == 0x1f
  OPC_Decode, 177, 58, 142, 6,               // 93360:       decode to STFMINLS using decoder 782
                                             // 93360:      }
  14, 12,                                    // 93365:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 93367:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93370:       check Inst[4:0] == 0x1f
  OPC_Decode, 170, 58, 142, 6,               // 93374:       decode to STFMAXNMLS using decoder 782
                                             // 93374:      }
  15, 0,                                     // 93379:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 93381:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93384:       check Inst[4:0] == 0x1f
  OPC_Decode, 182, 58, 142, 6,               // 93388:       decode to STFMINNMLS using decoder 782
                                             // 93388:      }
                                             // 93388:      } // switch Inst[15:12]
                                             // 93388:     }
                                             // 93388:     } // switch Inst[21]
                                             // 93388:    }
  1, 11,                                     // 93393:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 93395:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 93397:     check Inst[21] == 0x0
  OPC_Decode, 227, 39, 247, 4,               // 93401:     decode to LDRSpost using decoder 631
                                             // 93401:    }
  2, 29,                                     // 93406:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 93408:     switch Inst[14:13] {
  2, 11,                                     // 93411:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 93413:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 93415:      check Inst[21] == 0x1
  OPC_Decode, 229, 39, 143, 6,               // 93419:      decode to LDRSroW using decoder 783
                                             // 93419:     }
  3, 0,                                      // 93424:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 93426:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 93428:      check Inst[21] == 0x1
  OPC_Decode, 230, 39, 144, 6,               // 93432:      decode to LDRSroX using decoder 784
                                             // 93432:     }
                                             // 93432:     } // switch Inst[14:13]
                                             // 93432:    }
  3, 0,                                      // 93437:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 93439:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 93441:     check Inst[21] == 0x0
  OPC_Decode, 228, 39, 247, 4,               // 93445:     decode to LDRSpre using decoder 631
                                             // 93445:    }
                                             // 93445:    } // switch Inst[11:10]
                                             // 93445:   }
  2, 73,                                     // 93450:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 93452:    switch Inst[15:10] {
  0, 12,                                     // 93455:    case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 93457:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93460:     check Inst[21] == 0x1
  OPC_Decode, 248, 37, 141, 6,               // 93464:     decode to LDFADDAS using decoder 781
                                             // 93464:    }
  16, 12,                                    // 93469:    case 0x10: {
  OPC_CheckPredicate, 139, 1,                // 93471:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93474:     check Inst[21] == 0x1
  OPC_Decode, 148, 38, 141, 6,               // 93478:     decode to LDFMAXAS using decoder 781
                                             // 93478:    }
  20, 12,                                    // 93483:    case 0x14: {
  OPC_CheckPredicate, 139, 1,                // 93485:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93488:     check Inst[21] == 0x1
  OPC_Decode, 172, 38, 141, 6,               // 93492:     decode to LDFMINAS using decoder 781
                                             // 93492:    }
  24, 12,                                    // 93497:    case 0x18: {
  OPC_CheckPredicate, 139, 1,                // 93499:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93502:     check Inst[21] == 0x1
  OPC_Decode, 159, 38, 141, 6,               // 93506:     decode to LDFMAXNMAS using decoder 781
                                             // 93506:    }
  28, 0,                                     // 93511:    case 0x1c: {
  OPC_CheckPredicate, 139, 1,                // 93513:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93516:     check Inst[21] == 0x1
  OPC_Decode, 183, 38, 141, 6,               // 93520:     decode to LDFMINNMAS using decoder 781
                                             // 93520:    }
                                             // 93520:    } // switch Inst[15:10]
                                             // 93520:   }
  3, 73,                                     // 93525:   case 0x3: {
  OPC_SwitchField, 10, 6,                    // 93527:    switch Inst[15:10] {
  0, 12,                                     // 93530:    case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 93532:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93535:     check Inst[21] == 0x1
  OPC_Decode, 247, 37, 141, 6,               // 93539:     decode to LDFADDALS using decoder 781
                                             // 93539:    }
  16, 12,                                    // 93544:    case 0x10: {
  OPC_CheckPredicate, 139, 1,                // 93546:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93549:     check Inst[21] == 0x1
  OPC_Decode, 147, 38, 141, 6,               // 93553:     decode to LDFMAXALS using decoder 781
                                             // 93553:    }
  20, 12,                                    // 93558:    case 0x14: {
  OPC_CheckPredicate, 139, 1,                // 93560:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93563:     check Inst[21] == 0x1
  OPC_Decode, 171, 38, 141, 6,               // 93567:     decode to LDFMINALS using decoder 781
                                             // 93567:    }
  24, 12,                                    // 93572:    case 0x18: {
  OPC_CheckPredicate, 139, 1,                // 93574:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93577:     check Inst[21] == 0x1
  OPC_Decode, 158, 38, 141, 6,               // 93581:     decode to LDFMAXNMALS using decoder 781
                                             // 93581:    }
  28, 0,                                     // 93586:    case 0x1c: {
  OPC_CheckPredicate, 139, 1,                // 93588:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 93591:     check Inst[21] == 0x1
  OPC_Decode, 182, 38, 141, 6,               // 93595:     decode to LDFMINNMALS using decoder 781
                                             // 93595:    }
                                             // 93595:    } // switch Inst[15:10]
                                             // 93595:   }
  4, 7,                                      // 93600:   case 0x4: {
  OPC_CheckPredicate, 77,                    // 93602:    check predicate 77
  OPC_Decode, 200, 59, 132, 5,               // 93604:    decode to STRSui using decoder 644
                                             // 93604:   }
  5, 0,                                      // 93609:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 93611:    check predicate 77
  OPC_Decode, 231, 39, 132, 5,               // 93613:    decode to LDRSui using decoder 644
                                             // 93613:   }
                                             // 93613:   } // switch Inst[25:22]
                                             // 93613:  }
  6, 29,                                     // 93618:  case 0x6: {
  OPC_SwitchField, 21, 5,                    // 93620:   switch Inst[25:21] {
  8, 11,                                     // 93623:   case 0x8: {
  OPC_CheckPredicate, 79,                    // 93625:    check predicate 79
  OPC_CheckField, 10, 2, 2,                  // 93627:    check Inst[11:10] == 0x2
  OPC_Decode, 215, 58, 145, 6,               // 93631:    decode to STLURdi using decoder 785
                                             // 93631:   }
  10, 0,                                     // 93636:   case 0xa: {
  OPC_CheckPredicate, 79,                    // 93638:    check predicate 79
  OPC_CheckField, 10, 2, 2,                  // 93640:    check Inst[11:10] == 0x2
  OPC_Decode, 169, 37, 145, 6,               // 93644:    decode to LDAPURdi using decoder 785
                                             // 93644:   }
                                             // 93644:   } // switch Inst[25:21]
                                             // 93644:  }
  7, 0,                                      // 93649:  case 0x7: {
  OPC_SwitchField, 22, 4,                    // 93651:   switch Inst[25:22] {
  0, 200, 1,                                 // 93654:   case 0x0: {
  OPC_SwitchField, 10, 2,                    // 93657:    switch Inst[11:10] {
  0, 137, 1,                                 // 93660:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 93663:     switch Inst[21] {
  0, 7,                                      // 93666:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 93668:      check predicate 77
  OPC_Decode, 232, 59, 247, 4,               // 93670:      decode to STURDi using decoder 631
                                             // 93670:     }
  1, 0,                                      // 93675:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 93677:      switch Inst[15:12] {
  0, 8,                                      // 93680:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 93682:       check predicate 139
  OPC_Decode, 249, 37, 146, 6,               // 93685:       decode to LDFADDD using decoder 786
                                             // 93685:      }
  4, 8,                                      // 93690:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 93692:       check predicate 139
  OPC_Decode, 149, 38, 146, 6,               // 93695:       decode to LDFMAXD using decoder 786
                                             // 93695:      }
  5, 8,                                      // 93700:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 93702:       check predicate 139
  OPC_Decode, 173, 38, 146, 6,               // 93705:       decode to LDFMIND using decoder 786
                                             // 93705:      }
  6, 8,                                      // 93710:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 93712:       check predicate 139
  OPC_Decode, 160, 38, 146, 6,               // 93715:       decode to LDFMAXNMD using decoder 786
                                             // 93715:      }
  7, 8,                                      // 93720:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 93722:       check predicate 139
  OPC_Decode, 184, 38, 146, 6,               // 93725:       decode to LDFMINNMD using decoder 786
                                             // 93725:      }
  8, 12,                                     // 93730:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 93732:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93735:       check Inst[4:0] == 0x1f
  OPC_Decode, 155, 58, 147, 6,               // 93739:       decode to STFADDD using decoder 787
                                             // 93739:      }
  12, 12,                                    // 93744:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 93746:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93749:       check Inst[4:0] == 0x1f
  OPC_Decode, 161, 58, 147, 6,               // 93753:       decode to STFMAXD using decoder 787
                                             // 93753:      }
  13, 12,                                    // 93758:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 93760:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93763:       check Inst[4:0] == 0x1f
  OPC_Decode, 173, 58, 147, 6,               // 93767:       decode to STFMIND using decoder 787
                                             // 93767:      }
  14, 12,                                    // 93772:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 93774:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93777:       check Inst[4:0] == 0x1f
  OPC_Decode, 166, 58, 147, 6,               // 93781:       decode to STFMAXNMD using decoder 787
                                             // 93781:      }
  15, 0,                                     // 93786:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 93788:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93791:       check Inst[4:0] == 0x1f
  OPC_Decode, 178, 58, 147, 6,               // 93795:       decode to STFMINNMD using decoder 787
                                             // 93795:      }
                                             // 93795:      } // switch Inst[15:12]
                                             // 93795:     }
                                             // 93795:     } // switch Inst[21]
                                             // 93795:    }
  1, 11,                                     // 93800:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 93802:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 93804:     check Inst[21] == 0x0
  OPC_Decode, 176, 59, 247, 4,               // 93808:     decode to STRDpost using decoder 631
                                             // 93808:    }
  2, 29,                                     // 93813:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 93815:     switch Inst[14:13] {
  2, 11,                                     // 93818:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 93820:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 93822:      check Inst[21] == 0x1
  OPC_Decode, 178, 59, 148, 6,               // 93826:      decode to STRDroW using decoder 788
                                             // 93826:     }
  3, 0,                                      // 93831:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 93833:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 93835:      check Inst[21] == 0x1
  OPC_Decode, 179, 59, 149, 6,               // 93839:      decode to STRDroX using decoder 789
                                             // 93839:     }
                                             // 93839:     } // switch Inst[14:13]
                                             // 93839:    }
  3, 0,                                      // 93844:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 93846:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 93848:     check Inst[21] == 0x0
  OPC_Decode, 177, 59, 247, 4,               // 93852:     decode to STRDpre using decoder 631
                                             // 93852:    }
                                             // 93852:    } // switch Inst[11:10]
                                             // 93852:   }
  1, 200, 1,                                 // 93857:   case 0x1: {
  OPC_SwitchField, 10, 2,                    // 93860:    switch Inst[11:10] {
  0, 137, 1,                                 // 93863:    case 0x0: {
  OPC_SwitchField, 21, 1,                    // 93866:     switch Inst[21] {
  0, 7,                                      // 93869:     case 0x0: {
  OPC_CheckPredicate, 77,                    // 93871:      check predicate 77
  OPC_Decode, 249, 40, 247, 4,               // 93873:      decode to LDURDi using decoder 631
                                             // 93873:     }
  1, 0,                                      // 93878:     case 0x1: {
  OPC_SwitchField, 12, 4,                    // 93880:      switch Inst[15:12] {
  0, 8,                                      // 93883:      case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 93885:       check predicate 139
  OPC_Decode, 251, 37, 146, 6,               // 93888:       decode to LDFADDLD using decoder 786
                                             // 93888:      }
  4, 8,                                      // 93893:      case 0x4: {
  OPC_CheckPredicate, 139, 1,                // 93895:       check predicate 139
  OPC_Decode, 151, 38, 146, 6,               // 93898:       decode to LDFMAXLD using decoder 786
                                             // 93898:      }
  5, 8,                                      // 93903:      case 0x5: {
  OPC_CheckPredicate, 139, 1,                // 93905:       check predicate 139
  OPC_Decode, 175, 38, 146, 6,               // 93908:       decode to LDFMINLD using decoder 786
                                             // 93908:      }
  6, 8,                                      // 93913:      case 0x6: {
  OPC_CheckPredicate, 139, 1,                // 93915:       check predicate 139
  OPC_Decode, 162, 38, 146, 6,               // 93918:       decode to LDFMAXNMLD using decoder 786
                                             // 93918:      }
  7, 8,                                      // 93923:      case 0x7: {
  OPC_CheckPredicate, 139, 1,                // 93925:       check predicate 139
  OPC_Decode, 186, 38, 146, 6,               // 93928:       decode to LDFMINNMLD using decoder 786
                                             // 93928:      }
  8, 12,                                     // 93933:      case 0x8: {
  OPC_CheckPredicate, 139, 1,                // 93935:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93938:       check Inst[4:0] == 0x1f
  OPC_Decode, 157, 58, 147, 6,               // 93942:       decode to STFADDLD using decoder 787
                                             // 93942:      }
  12, 12,                                    // 93947:      case 0xc: {
  OPC_CheckPredicate, 139, 1,                // 93949:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93952:       check Inst[4:0] == 0x1f
  OPC_Decode, 163, 58, 147, 6,               // 93956:       decode to STFMAXLD using decoder 787
                                             // 93956:      }
  13, 12,                                    // 93961:      case 0xd: {
  OPC_CheckPredicate, 139, 1,                // 93963:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93966:       check Inst[4:0] == 0x1f
  OPC_Decode, 175, 58, 147, 6,               // 93970:       decode to STFMINLD using decoder 787
                                             // 93970:      }
  14, 12,                                    // 93975:      case 0xe: {
  OPC_CheckPredicate, 139, 1,                // 93977:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93980:       check Inst[4:0] == 0x1f
  OPC_Decode, 168, 58, 147, 6,               // 93984:       decode to STFMAXNMLD using decoder 787
                                             // 93984:      }
  15, 0,                                     // 93989:      case 0xf: {
  OPC_CheckPredicate, 139, 1,                // 93991:       check predicate 139
  OPC_CheckField, 0, 5, 31,                  // 93994:       check Inst[4:0] == 0x1f
  OPC_Decode, 180, 58, 147, 6,               // 93998:       decode to STFMINNMLD using decoder 787
                                             // 93998:      }
                                             // 93998:      } // switch Inst[15:12]
                                             // 93998:     }
                                             // 93998:     } // switch Inst[21]
                                             // 93998:    }
  1, 11,                                     // 94003:    case 0x1: {
  OPC_CheckPredicate, 77,                    // 94005:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 94007:     check Inst[21] == 0x0
  OPC_Decode, 179, 39, 247, 4,               // 94011:     decode to LDRDpost using decoder 631
                                             // 94011:    }
  2, 29,                                     // 94016:    case 0x2: {
  OPC_SwitchField, 13, 2,                    // 94018:     switch Inst[14:13] {
  2, 11,                                     // 94021:     case 0x2: {
  OPC_CheckPredicate, 77,                    // 94023:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 94025:      check Inst[21] == 0x1
  OPC_Decode, 181, 39, 148, 6,               // 94029:      decode to LDRDroW using decoder 788
                                             // 94029:     }
  3, 0,                                      // 94034:     case 0x3: {
  OPC_CheckPredicate, 77,                    // 94036:      check predicate 77
  OPC_CheckField, 21, 1, 1,                  // 94038:      check Inst[21] == 0x1
  OPC_Decode, 182, 39, 149, 6,               // 94042:      decode to LDRDroX using decoder 789
                                             // 94042:     }
                                             // 94042:     } // switch Inst[14:13]
                                             // 94042:    }
  3, 0,                                      // 94047:    case 0x3: {
  OPC_CheckPredicate, 77,                    // 94049:     check predicate 77
  OPC_CheckField, 21, 1, 0,                  // 94051:     check Inst[21] == 0x0
  OPC_Decode, 180, 39, 247, 4,               // 94055:     decode to LDRDpre using decoder 631
                                             // 94055:    }
                                             // 94055:    } // switch Inst[11:10]
                                             // 94055:   }
  2, 73,                                     // 94060:   case 0x2: {
  OPC_SwitchField, 10, 6,                    // 94062:    switch Inst[15:10] {
  0, 12,                                     // 94065:    case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 94067:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94070:     check Inst[21] == 0x1
  OPC_Decode, 243, 37, 146, 6,               // 94074:     decode to LDFADDAD using decoder 786
                                             // 94074:    }
  16, 12,                                    // 94079:    case 0x10: {
  OPC_CheckPredicate, 139, 1,                // 94081:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94084:     check Inst[21] == 0x1
  OPC_Decode, 143, 38, 146, 6,               // 94088:     decode to LDFMAXAD using decoder 786
                                             // 94088:    }
  20, 12,                                    // 94093:    case 0x14: {
  OPC_CheckPredicate, 139, 1,                // 94095:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94098:     check Inst[21] == 0x1
  OPC_Decode, 167, 38, 146, 6,               // 94102:     decode to LDFMINAD using decoder 786
                                             // 94102:    }
  24, 12,                                    // 94107:    case 0x18: {
  OPC_CheckPredicate, 139, 1,                // 94109:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94112:     check Inst[21] == 0x1
  OPC_Decode, 154, 38, 146, 6,               // 94116:     decode to LDFMAXNMAD using decoder 786
                                             // 94116:    }
  28, 0,                                     // 94121:    case 0x1c: {
  OPC_CheckPredicate, 139, 1,                // 94123:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94126:     check Inst[21] == 0x1
  OPC_Decode, 178, 38, 146, 6,               // 94130:     decode to LDFMINNMAD using decoder 786
                                             // 94130:    }
                                             // 94130:    } // switch Inst[15:10]
                                             // 94130:   }
  3, 73,                                     // 94135:   case 0x3: {
  OPC_SwitchField, 10, 6,                    // 94137:    switch Inst[15:10] {
  0, 12,                                     // 94140:    case 0x0: {
  OPC_CheckPredicate, 139, 1,                // 94142:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94145:     check Inst[21] == 0x1
  OPC_Decode, 245, 37, 146, 6,               // 94149:     decode to LDFADDALD using decoder 786
                                             // 94149:    }
  16, 12,                                    // 94154:    case 0x10: {
  OPC_CheckPredicate, 139, 1,                // 94156:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94159:     check Inst[21] == 0x1
  OPC_Decode, 145, 38, 146, 6,               // 94163:     decode to LDFMAXALD using decoder 786
                                             // 94163:    }
  20, 12,                                    // 94168:    case 0x14: {
  OPC_CheckPredicate, 139, 1,                // 94170:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94173:     check Inst[21] == 0x1
  OPC_Decode, 169, 38, 146, 6,               // 94177:     decode to LDFMINALD using decoder 786
                                             // 94177:    }
  24, 12,                                    // 94182:    case 0x18: {
  OPC_CheckPredicate, 139, 1,                // 94184:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94187:     check Inst[21] == 0x1
  OPC_Decode, 156, 38, 146, 6,               // 94191:     decode to LDFMAXNMALD using decoder 786
                                             // 94191:    }
  28, 0,                                     // 94196:    case 0x1c: {
  OPC_CheckPredicate, 139, 1,                // 94198:     check predicate 139
  OPC_CheckField, 21, 1, 1,                  // 94201:     check Inst[21] == 0x1
  OPC_Decode, 180, 38, 146, 6,               // 94205:     decode to LDFMINNMALD using decoder 786
                                             // 94205:    }
                                             // 94205:    } // switch Inst[15:10]
                                             // 94205:   }
  4, 7,                                      // 94210:   case 0x4: {
  OPC_CheckPredicate, 77,                    // 94212:    check predicate 77
  OPC_Decode, 180, 59, 132, 5,               // 94214:    decode to STRDui using decoder 644
                                             // 94214:   }
  5, 0,                                      // 94219:   case 0x5: {
  OPC_CheckPredicate, 77,                    // 94221:    check predicate 77
  OPC_Decode, 183, 39, 132, 5,               // 94223:    decode to LDRDui using decoder 644
                                             // 94223:   }
                                             // 94223:   } // switch Inst[25:22]
                                             // 94223:  }
                                             // 94223:  } // switch Inst[31:29]
                                             // 94223: }
                                             // 94223: } // switch Inst[28:26]
};
static const uint8_t DecoderTableFallback32[58] = {
  OPC_SwitchField, 21, 11,                   //  0: switch Inst[31:21] {
  168, 13, 24,                               //  3: case 0x6a8: {
  OPC_Scope, 17,                             //  6:  try {
  OPC_CheckField, 0, 5, 31,                  //  8:   check Inst[4:0] == 0x1f
  OPC_CheckField, 19, 2, 0,                  // 12:   check Inst[20:19] == 0x0
  OPC_CheckField, 12, 4, 4,                  // 16:   check Inst[15:12] == 0x4
  OPC_Decode, 250, 42, 150, 6,               // 20:   decode to MSRpstateImm1 using decoder 790
                                             // 20:  } else try {
  OPC_Decode, 248, 42, 151, 6,               // 25:   decode to MSR using decoder 791
                                             // 25:  }
                                             // 25: }
  169, 13, 5,                                // 30: case 0x6a9: {
  OPC_Decode, 243, 42, 152, 6,               // 33:  decode to MRS using decoder 792
                                             // 33: }
  197, 15, 0,                                // 38: case 0x7c5: {
  OPC_CheckField, 14, 1, 1,                  // 41:  check Inst[14] == 0x1
  OPC_CheckField, 10, 2, 2,                  // 45:  check Inst[11:10] == 0x2
  OPC_CheckField, 3, 2, 3,                   // 49:  check Inst[4:3] == 0x3
  OPC_Decode, 232, 45, 153, 6,               // 53:  decode to RPRFM using decoder 793
                                             // 53: }
                                             // 53: } // switch Inst[31:21]
};
// Handling 794 cases.
template <typename InsnType>
static DecodeStatus decodeToMCInst(unsigned Idx, DecodeStatus S, InsnType insn, MCInst &MI, uint64_t Address, const MCDisassembler *Decoder, bool &DecodeComplete) {
  DecodeComplete = true;
  using TmpType = std::conditional_t<std::is_integral<InsnType>::value, InsnType, uint64_t>;
  TmpType tmp;
  switch (Idx) {
  default: llvm_unreachable("Invalid decoder index!");
  case 0:
    tmp = fieldFromInstruction(insn, 0, 16);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 1:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<0, 14>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<16, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 2:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<0, 14>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<16, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 3:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<16, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 4:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<16, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 5:
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<0, 14>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<16, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 6:
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<0, 14>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<16, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 7:
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<16, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 8:
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<16, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 9:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeZK(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 10:
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeZK(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 4, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 11:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 12:
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 13:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 14:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<0, 14>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<16, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 15:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<0, 14>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<16, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 16:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPRMul2_MinMax<16, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 17:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 3);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<16, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 18:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 19:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 20:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 21:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 22:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 3);
    tmp |= fieldFromInstruction(insn, 4, 1) << 3;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 23:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 24:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 3);
    tmp |= fieldFromInstruction(insn, 4, 1) << 3;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 25:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 26:
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 27:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 28:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 29:
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 30:
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 31:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 32:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 33:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 34:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 35:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 36:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 37:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 38:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 39:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 40:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 41:
    tmp = fieldFromInstruction(insn, 0, 8);
    if (!Check(S, DecodeMatrixTileListRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 42:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 43:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 44:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 45:
    tmp = fieldFromInstruction(insn, 3, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 3, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 46:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 47:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 48:
    tmp = fieldFromInstruction(insn, 2, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 2, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 49:
    tmp = fieldFromInstruction(insn, 1, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 50:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 51:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 52:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 53:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 54:
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 55:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 12, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 56:
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 12, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 57:
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 12, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 58:
    tmp = fieldFromInstruction(insn, 2, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 2, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 59:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 60:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 61:
    tmp = fieldFromInstruction(insn, 1, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 62:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 63:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 64:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 65:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 66:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 67:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR3RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 68:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 69:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 70:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 71:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 72:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 73:
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 74:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR3RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 75:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 76:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 77:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 3);
    tmp |= fieldFromInstruction(insn, 4, 1) << 3;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 78:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 79:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 3);
    tmp |= fieldFromInstruction(insn, 4, 1) << 3;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 80:
    tmp = fieldFromInstruction(insn, 1, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 81:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeMPR128RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeMPR128RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 82:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 83:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 84:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeMPR128RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 85:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeMPR128RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeMPR128RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 86:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 87:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 88:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 89:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 90:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 91:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 92:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 93:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 14, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 94:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 14, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 95:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 96:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 15, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 97:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 2);
    tmp |= fieldFromInstruction(insn, 10, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 98:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 99:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 2);
    tmp |= fieldFromInstruction(insn, 10, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 100:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 101:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 1);
    tmp |= fieldFromInstruction(insn, 10, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 102:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 1);
    tmp |= fieldFromInstruction(insn, 10, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 103:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 104:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 105:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 106:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 107:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 108:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 109:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 110:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 111:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 112:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 113:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 114:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 115:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 116:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 117:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 118:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 119:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 120:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 121:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 122:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 123:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 124:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 125:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 126:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 127:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 128:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 129:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 130:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 131:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 132:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 2);
    tmp |= fieldFromInstruction(insn, 4, 1) << 2;
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4StridedRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 133:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 2);
    tmp |= fieldFromInstruction(insn, 15, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 134:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 1);
    tmp |= fieldFromInstruction(insn, 10, 2) << 1;
    tmp |= fieldFromInstruction(insn, 15, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 135:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 2);
    tmp |= fieldFromInstruction(insn, 10, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 136:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 137:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 1, 2);
    tmp |= fieldFromInstruction(insn, 10, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 138:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 139:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 3, 1);
    tmp |= fieldFromInstruction(insn, 10, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 140:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 2);
    tmp |= fieldFromInstruction(insn, 15, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 141:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 1);
    tmp |= fieldFromInstruction(insn, 10, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 142:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 1);
    tmp |= fieldFromInstruction(insn, 10, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 143:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 2);
    tmp |= fieldFromInstruction(insn, 10, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 144:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 2, 2);
    tmp |= fieldFromInstruction(insn, 10, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 145:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 146:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 147:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 148:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 149:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 150:
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    if (!Check(S, DecodeMPRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_8_11RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 151:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 152:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 7, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 16, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 153:
    if (!Check(S, DecodeMPR8RegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 154:
    tmp = fieldFromInstruction(insn, 3, 1);
    if (!Check(S, DecodeMPR16RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 155:
    tmp = fieldFromInstruction(insn, 2, 2);
    if (!Check(S, DecodeMPR32RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 156:
    tmp = fieldFromInstruction(insn, 1, 3);
    if (!Check(S, DecodeMPR64RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 157:
    if (!Check(S, DecodeSMESpillFillInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 158:
    if (!Check(S, DecodeZTRRegisterClass(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 159:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeMPR128RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 160:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 161:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 162:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 163:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 164:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 165:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 166:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 167:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 168:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 169:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 170:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 171:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 172:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 173:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 5, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 174:
    if (!Check(S, DecodeSVELogicalImmInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 175:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 176:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 177:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 178:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 179:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 180:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 181:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<64>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 182:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 183:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<64>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 184:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 185:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 16, 5) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 186:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 16, 5) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 187:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 188:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 20, 1);
    tmp |= fieldFromInstruction(insn, 22, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 189:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 19, 2);
    tmp |= fieldFromInstruction(insn, 22, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 190:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 18, 3);
    tmp |= fieldFromInstruction(insn, 22, 2) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 191:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 17, 4);
    tmp |= fieldFromInstruction(insn, 22, 2) << 4;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 192:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 193:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 194:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 195:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 196:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 197:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 198:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 199:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 200:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 201:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 202:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 16, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 203:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 204:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32spRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 205:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 206:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 207:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 208:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 209:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 210:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 211:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 212:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 213:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 214:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 215:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 216:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 17, 2);
    tmp |= fieldFromInstruction(insn, 22, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 217:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 17, 2);
    tmp |= fieldFromInstruction(insn, 22, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 218:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 219:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 220:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 221:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 222:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 223:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 224:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 225:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 226:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 227:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 6);
    if (!Check(S, DecodeSImm<6>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 228:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 6);
    if (!Check(S, DecodeSImm<6>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 229:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 230:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 231:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 232:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 233:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 234:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 235:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 16, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 236:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 16, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 237:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 238:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 239:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 240:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 241:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 242:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32spRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 243:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 244:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 245:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 246:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 247:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 248:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 249:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 250:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 251:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 252:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 253:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 254:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 255:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 256:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSVEIncDecImm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 257:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSVEIncDecImm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 258:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSVEIncDecImm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 259:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSVEIncDecImm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 260:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 261:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 262:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSImm<5>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 263:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 264:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 265:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 266:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 267:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 268:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 269:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 270:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 271:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 272:
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 273:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 14, 7);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 274:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 275:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 276:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 277:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 278:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 23, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 279:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 280:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 20, 1);
    tmp |= fieldFromInstruction(insn, 22, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 281:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 2);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::MatrixIndexGPR32_12_15RegClassID, 0, 4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 19, 2);
    tmp |= fieldFromInstruction(insn, 22, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 282:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 283:
    tmp = fieldFromInstruction(insn, 1, 3);
    if (!Check(S, DecodePPR2Mul2RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 284:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 285:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPR2RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 286:
    tmp = fieldFromInstruction(insn, 0, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 8, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 287:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 288:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 289:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 290:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 291:
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 292:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 293:
    return S;
  case 294:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 295:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 296:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 297:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<64>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 298:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    if (!Check(S, DecodeSImm<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 299:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 300:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 301:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 302:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 303:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodeImm8OptLsl<64>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 304:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 305:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 19, 2);
    tmp |= fieldFromInstruction(insn, 22, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 306:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 307:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 308:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 309:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 310:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1);
    tmp |= fieldFromInstruction(insn, 19, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 311:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1);
    tmp |= fieldFromInstruction(insn, 20, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 312:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 313:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 314:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 315:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 22, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 316:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 23, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 317:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 23, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 318:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 22, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 319:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1);
    tmp |= fieldFromInstruction(insn, 19, 2) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 320:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1);
    tmp |= fieldFromInstruction(insn, 20, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 321:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 322:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 323:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 19, 2);
    tmp |= fieldFromInstruction(insn, 22, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 324:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 325:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 326:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 327:
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 1, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 328:
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 2, 3);
    if (!Check(S, DecodeZPR4Mul4RegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 329:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 330:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 331:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 332:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 16, 5);
    tmp |= fieldFromInstruction(insn, 22, 1) << 5;
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 333:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 334:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 2);
    tmp |= fieldFromInstruction(insn, 19, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 335:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 336:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 6, 4);
    if (!Check(S, DecodeZPR2Mul2RegisterClass<0, 30>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 337:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 338:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 339:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 340:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 341:
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 342:
    tmp = fieldFromInstruction(insn, 0, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRorPNRRegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 16, 6) << 3;
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 343:
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeSImm<6>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 344:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 16, 6) << 3;
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 345:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 346:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 347:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 348:
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 349:
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 350:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 351:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 352:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPRRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 353:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 354:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR2RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 355:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR3RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 356:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR3RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 357:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 358:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::ZPR4RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::PPRRegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSImm<4>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 359:
    if (!Check(S, DecodeExclusiveLdStInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 360:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeWSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeWSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeWSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 361:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 362:
    if (!Check(S, DecodeThreeAddrSRegInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 363:
    if (!Check(S, DecodeAddSubERegInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 364:
    if (!Check(S, DecodePairLdStInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 365:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeXSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeXSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeXSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 366:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 367:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 368:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 369:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 370:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 371:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::DDDDRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 372:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::DDDRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 373:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 374:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::DDRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 375:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 376:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 377:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 378:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 379:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::DDDDRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 380:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::DDDRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 381:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 382:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::DDRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 383:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 384:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 385:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 386:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 387:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 388:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 389:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 390:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 391:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 392:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 393:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 394:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 395:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 396:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 397:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 398:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 399:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 400:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 401:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 402:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 403:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 404:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 405:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 406:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 407:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 408:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 409:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 410:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 411:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 412:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 413:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 414:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 415:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 416:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 417:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 418:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 419:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 420:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 421:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 422:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 423:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 424:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 425:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 426:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 427:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 428:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 429:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 430:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 431:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 432:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 433:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 434:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 435:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 436:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 437:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 438:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 10, 3);
    tmp |= fieldFromInstruction(insn, 30, 1) << 3;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 439:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 440:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 441:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 442:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 2);
    tmp |= fieldFromInstruction(insn, 30, 1) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 443:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 444:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 445:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 446:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 447:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 448:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 30, 1) << 1;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 449:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 450:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 30, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 451:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 452:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 453:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 454:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 455:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 456:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 457:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 458:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 459:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 460:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 461:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 462:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 463:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 464:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 465:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 466:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 467:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 468:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 469:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 470:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 471:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 472:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 473:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 474:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 475:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 476:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 477:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 478:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 479:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 480:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 481:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 482:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 483:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 484:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 485:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 486:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 487:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 488:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 489:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 490:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 491:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 492:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 493:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 494:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 495:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    if (!Check(S, DecodeZeroImm(MI, Decoder)))
      return MCDisassembler::Fail;
    return S;
  case 496:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 497:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 498:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 499:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 500:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 501:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 502:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 503:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 504:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 505:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQQQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 506:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 507:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 508:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 509:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 14, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 510:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 511:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 512:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 513:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 514:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 515:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 516:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 517:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 14, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 518:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::QQRegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 519:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 520:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 521:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 522:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 523:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 524:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 525:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 526:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 527:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 528:
    if (!Check(S, DecodeModImmInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 529:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 530:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 531:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 532:
    if (!Check(S, DecodeModImmTiedInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 533:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 534:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 535:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 536:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 537:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 538:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 539:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR16ImmNarrow(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 540:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 541:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR32ImmNarrow(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 542:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 543:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR64ImmNarrow(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 544:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 545:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 3;
    tmp |= fieldFromInstruction(insn, 19, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 546:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 547:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 548:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 549:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 550:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 551:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 552:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 553:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 554:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 555:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 556:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 557:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 558:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 559:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 560:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 561:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR16ImmNarrow(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 562:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR32ImmNarrow(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 563:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR64ImmNarrow(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 564:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 565:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 566:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 567:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 568:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 569:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 570:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 571:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 572:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 573:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 574:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 575:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 576:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 577:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 578:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 579:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 13, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 580:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 581:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 582:
    if (!Check(S, DecodeAdrInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 583:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 8);
    if (!Check(S, DecodeSImm<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 584:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 585:
    if (!Check(S, DecodeAddSubImmShift(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 586:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 8);
    if (!Check(S, DecodeSImm<8>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 587:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 588:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 589:
    if (!Check(S, DecodeLogicalImmInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 590:
    if (!Check(S, DecodeMoveImmInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 591:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 592:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 593:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 594:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 595:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 596:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 10, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 597:
    tmp = fieldFromInstruction(insn, 5, 16);
    if (!Check(S, DecodePCRelLabel16(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 598:
    if (!Check(S, DecodeUnconditionalBranch(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 599:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 600:
    if (!Check(S, DecodeTestAndBranch(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 601:
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 602:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodePCRelLabel9(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 603:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodePCRelLabel9(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 604:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 605:
    tmp = fieldFromInstruction(insn, 5, 16);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 606:
    tmp = fieldFromInstruction(insn, 5, 7);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 17, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 607:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 608:
    tmp = 0x2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 609:
    tmp = fieldFromInstruction(insn, 5, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 610:
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 611:
    tmp = fieldFromInstruction(insn, 5, 7);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 612:
    tmp = 0x3;
    tmp |= fieldFromInstruction(insn, 10, 2) << 2;
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 613:
    tmp = fieldFromInstruction(insn, 8, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 614:
    tmp = fieldFromInstruction(insn, 9, 3);
    if (!Check(S, DecodeSVCROp(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 8, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 615:
    if (!Check(S, DecodeSystemPStateImm0_15Instruction(MI, insn, Address, Decoder))) { DecodeComplete = false; return MCDisassembler::Fail; }
    return S;
  case 616:
    tmp = fieldFromInstruction(insn, 16, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 8, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 617:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 618:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 8, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 619:
    if (!Check(S, DecodeSyspXzrInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 620:
    tmp = fieldFromInstruction(insn, 16, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 8, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeXSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 621:
    tmp = fieldFromInstruction(insn, 5, 16);
    if (!Check(S, DecodeMSRSystemRegister(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeXSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 622:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeXSeqPairsClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 16);
    if (!Check(S, DecodeMRSSystemRegister(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 623:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 624:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 7);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 17, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 625:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 626:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 627:
    tmp = fieldFromInstruction(insn, 10, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 628:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 629:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodePCRelLabel9(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 630:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 9);
    if (!Check(S, DecodePCRelLabel9(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 631:
    if (!Check(S, DecodeSignedLdStInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 632:
    if (!Check(S, DecodeCPYMemOpInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 633:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 634:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64commonRegClassID, 0, 31>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 635:
    if (!Check(S, DecodeSETMemOpInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 636:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 637:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 638:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 639:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 640:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 641:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 642:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 643:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 644:
    if (!Check(S, DecodeUnsignedLdStInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 645:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 646:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 647:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 648:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 649:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 650:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 651:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 652:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 653:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 654:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 655:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 656:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 657:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 658:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 659:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 660:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 661:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 15, 6);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 662:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 663:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 664:
    tmp = fieldFromInstruction(insn, 0, 5);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 665:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 666:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 667:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 668:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 669:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 670:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 671:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 672:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeGPR64x8ClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 673:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeGPR64x8ClassRegisterClass(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 674:
    if (!Check(S, DecodeAuthLoadInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 675:
    if (!Check(S, DecodePRFMRegInstruction(MI, insn, Address, Decoder))) { DecodeComplete = false; return MCDisassembler::Fail; }
    return S;
  case 676:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 677:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 678:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 679:
    if (!Check(S, DecodeSETMemGoOpInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 680:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x20;
    tmp |= fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeFixedPointScaleImm32(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 681:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x20;
    tmp |= fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeFixedPointScaleImm32(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 682:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 683:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 684:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 685:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 686:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 687:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 688:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 689:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 690:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 691:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 692:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 693:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x20;
    tmp |= fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeFixedPointScaleImm32(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 694:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x20;
    tmp |= fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeFixedPointScaleImm32(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 695:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 696:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 697:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 698:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 699:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 700:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 701:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x20;
    tmp |= fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeFixedPointScaleImm32(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 702:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x20;
    tmp |= fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeFixedPointScaleImm32(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 703:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 704:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 705:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 706:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 707:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 708:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 709:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 710:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 13, 8);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 711:
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 712:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 713:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 714:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 715:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 716:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 717:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 718:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 719:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 720:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 721:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 722:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 723:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 724:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 725:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 726:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 20, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 727:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 19, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 728:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 18, 3);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 729:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 17, 4);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 730:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 731:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 732:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 733:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 734:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 735:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 736:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 737:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 738:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 739:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 740:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 741:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 742:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftR64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 743:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 744:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 745:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 746:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 747:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 748:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftL8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 749:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftL16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 750:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftL32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 751:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 752:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR8RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 3);
    if (!Check(S, DecodeVecShiftR8Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 753:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 754:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 755:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 756:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 11, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 757:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 16>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 2;
    tmp |= fieldFromInstruction(insn, 20, 2);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 758:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 11, 1) << 1;
    tmp |= fieldFromInstruction(insn, 21, 1);
    MI.addOperand(MCOperand::createImm(tmp));
    return S;
  case 759:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 4);
    if (!Check(S, DecodeVecShiftR16Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 760:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeVecShiftR32Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 761:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 762:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 763:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 764:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 765:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 6);
    if (!Check(S, DecodeVecShiftL64Imm(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 766:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR128RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 19);
    if (!Check(S, DecodePCRelLabel19(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 767:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 768:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    if (!Check(S, DecodeFixedPointScaleImm64(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 769:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    if (!Check(S, DecodeFixedPointScaleImm64(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 770:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 771:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 772:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    if (!Check(S, DecodeFixedPointScaleImm64(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 773:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    if (!Check(S, DecodeFixedPointScaleImm64(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 774:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 775:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 776:
    if (!Check(S, DecodeFMOVLaneInstruction(MI, insn, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 777:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    if (!Check(S, DecodeFixedPointScaleImm64(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 778:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 10, 6);
    if (!Check(S, DecodeFixedPointScaleImm64(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 779:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 780:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR16RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 781:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 782:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 783:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 784:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 785:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 12, 9);
    if (!Check(S, DecodeSImm<9>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 786:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 787:
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 788:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR32RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 789:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::FPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 12, 1);
    tmp |= fieldFromInstruction(insn, 15, 1) << 1;
    if (!Check(S, DecodeMemExtend(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 790:
    if (!Check(S, DecodeSystemPStateImm0_1Instruction(MI, insn, Address, Decoder))) { DecodeComplete = false; return MCDisassembler::Fail; }
    return S;
  case 791:
    tmp = fieldFromInstruction(insn, 5, 16);
    if (!Check(S, DecodeMSRSystemRegister(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 792:
    tmp = fieldFromInstruction(insn, 0, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 16);
    if (!Check(S, DecodeMRSSystemRegister(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  case 793:
    tmp = 0x0;
    tmp |= fieldFromInstruction(insn, 0, 3);
    tmp |= fieldFromInstruction(insn, 12, 2) << 3;
    tmp |= fieldFromInstruction(insn, 15, 1) << 5;
    MI.addOperand(MCOperand::createImm(tmp));
    tmp = fieldFromInstruction(insn, 16, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64RegClassID, 0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    tmp = fieldFromInstruction(insn, 5, 5);
    if (!Check(S, DecodeSimpleRegisterClass<AArch64::GPR64spRegClassID,0, 32>(MI, tmp, Address, Decoder))) { return MCDisassembler::Fail; }
    return S;
  }
}
static bool checkDecoderPredicate(unsigned Idx, const FeatureBitset &FB) {
  switch (Idx) {
  default: llvm_unreachable("Invalid index!");
  case 0:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4];
  case 1:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF8F32]);
  case 2:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF8F16]);
  case 3:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_TMOP];
  case 4:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_TMOP]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF8F32]);
  case 5:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_TMOP]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF8F16]);
  case 6:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME];
  case 7:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2];
  case 8:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF8F32];
  case 9:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF8F16];
  case 10:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF64F64];
  case 11:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF64F64]);
  case 12:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF16F16]);
  case 13:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEB16B16]);
  case 14:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_TMOP]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF16F16]);
  case 15:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_TMOP]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEB16B16]);
  case 16:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF16F16];
  case 17:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEB16B16];
  case 18:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSME2] || FB[AArch64::FeatureSVE2p1]);
  case 19:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEI16I64];
  case 20:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_MOP4]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEI16I64]);
  case 21:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2p1];
  case 22:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_LUTv2];
  case 23:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2p3];
  case 24:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2p1]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME_LUTv2]);
  case 25:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEI16I64]);
  case 26:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSMEF64F64]);
  case 27:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEB16B16]);
  case 28:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEBFSCALE]);
  case 29:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP8]);
  case 30:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFAMINMAX]);
  case 31:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSME2p2];
  case 32:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSMEF16F16] || FB[AArch64::FeatureSMEF8F16]);
  case 33:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME]);
  case 34:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSME2p1] || FB[AArch64::FeatureSVE2p1]);
  case 35:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE2] || FB[AArch64::FeatureSME]);
  case 36:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSME2p2] || FB[AArch64::FeatureSVE2p2]);
  case 37:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureCPA]);
  case 38:
    return (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME])) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMatMulFP64]);
  case 39:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSME2p3] || FB[AArch64::FeatureSVE2p3]);
  case 40:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE];
  case 41:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSSVE_FEXPA]);
  case 42:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE2p2] || FB[AArch64::FeatureSME2p2]);
  case 43:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSME] || FB[AArch64::FeatureSVE2p1]);
  case 44:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME2p2]);
  case 45:
    return (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME])) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMatMulInt8]);
  case 46:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEAES]) && (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSSVE_AES]));
  case 47:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMatMulInt8]);
  case 48:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE2];
  case 49:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEBitPerm]) && (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSSVE_BitPerm]));
  case 50:
    return (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE2] || FB[AArch64::FeatureSME2])) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureLUT]);
  case 51:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE2p3];
  case 52:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVESM4];
  case 53:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEAES2]) && (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSSVE_AES]));
  case 54:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVESHA3]) && (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME2p1]));
  case 55:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEB16B16]) && (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME2]));
  case 56:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureBF16]) && (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE] || FB[AArch64::FeatureSME]));
  case 57:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSSVE_FP8DOT2] || (FB[AArch64::FeatureSVE2] && FB[AArch64::FeatureFP8DOT2]));
  case 58:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSSVE_FP8DOT4] || (FB[AArch64::FeatureSVE2] && FB[AArch64::FeatureFP8DOT4]));
  case 59:
    return FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSSVE_FP8FMA] || (FB[AArch64::FeatureSVE2] && FB[AArch64::FeatureFP8FMA]));
  case 60:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureF8F32MM]);
  case 61:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE_F16F32MM];
  case 62:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureF8F16MM]);
  case 63:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureBF16]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE]);
  case 64:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE2p2]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureF16MM]);
  case 65:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMatMulFP32]);
  case 66:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE_B16MM];
  case 67:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMatMulFP64]);
  case 68:
    return (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE2] || FB[AArch64::FeatureSME2])) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP8]);
  case 69:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVEBFSCALE];
  case 70:
    return (FB[AArch64::FeatureAll] || (FB[AArch64::FeatureSVE2] || FB[AArch64::FeatureSME2])) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFAMINMAX]);
  case 71:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSVE2p1];
  case 72:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLSE];
  case 73:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLOR];
  case 74:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLSUI];
  case 75:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureMTE];
  case 76:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON];
  case 77:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFPARMv8];
  case 78:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureLSUI]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]);
  case 79:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureRCPC3]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]);
  case 80:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP8FMA];
  case 81:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFullFP16]);
  case 82:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFRInt3264];
  case 83:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP16FML]);
  case 84:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP8];
  case 85:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP8DOT4];
  case 86:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureAES];
  case 87:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureF8F16MM]);
  case 88:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSHA3];
  case 89:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFP8DOT2];
  case 90:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureRDM]);
  case 91:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureComplxNum]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFullFP16]);
  case 92:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureBF16]);
  case 93:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLUT];
  case 94:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureF16F32MM];
  case 95:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSM4];
  case 96:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureDotProd];
  case 97:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureMatMulInt8];
  case 98:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFAMINMAX]);
  case 99:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureF16F32DOT];
  case 100:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureComplxNum]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]);
  case 101:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureF8F32MM]);
  case 102:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureF16MM];
  case 103:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureCSSC];
  case 104:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeaturePAuthLR];
  case 105:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureHBC];
  case 106:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureCMPBR];
  case 107:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureEL3];
  case 108:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureTEV];
  case 109:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureWFxT];
  case 110:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureTRACEV8_4];
  case 111:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureCMH];
  case 112:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeaturePCDPHINT];
  case 113:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureXS];
  case 114:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSB];
  case 115:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFlagM];
  case 116:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureAltFPCmp];
  case 117:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureGCS];
  case 118:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureBRBE];
  case 119:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureITE];
  case 120:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureD128];
  case 121:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureS1POE2];
  case 122:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeaturePAuth];
  case 123:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureRCPC_IMMO];
  case 124:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureMOPS];
  case 125:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureTHE];
  case 126:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureTHE]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureD128]);
  case 127:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLSE128];
  case 128:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureCRC];
  case 129:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureRCPC];
  case 130:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureRCPC3];
  case 131:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureCPA];
  case 132:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLSCP];
  case 133:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLS64];
  case 134:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMOPS_GO]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMTE]);
  case 135:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMOPS]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureMTE]);
  case 136:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureNEON]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFPRCVT]);
  case 137:
    return (FB[AArch64::FeatureAll] || FB[AArch64::FeatureJS]) && (FB[AArch64::FeatureAll] || FB[AArch64::FeatureFPARMv8]);
  case 138:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureFullFP16];
  case 139:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureLSFE];
  case 140:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureSHA2];
  case 141:
    return FB[AArch64::FeatureAll] || FB[AArch64::FeatureRDM];
  }
}


template <typename InsnType>
static DecodeStatus decodeInstruction(const uint8_t DecodeTable[], MCInst &MI,
                                      InsnType insn, uint64_t Address,
                                      const MCDisassembler *DisAsm,
                                      const MCSubtargetInfo &STI) {
  const FeatureBitset &Bits = STI.getFeatureBits();
  const uint8_t *Ptr = DecodeTable;

  SmallVector<const uint8_t *, 8> ScopeStack;
  DecodeStatus S = MCDisassembler::Success;
  while (true) {
    ptrdiff_t Loc = Ptr - DecodeTable;
    const uint8_t DecoderOp = *Ptr++;
    switch (DecoderOp) {
    default:
      errs() << Loc << ": Unexpected decode table opcode: "
             << (int)DecoderOp << '\n';
      return MCDisassembler::Fail;
    case OPC_Scope: {
      unsigned NumToSkip = decodeULEB128AndIncUnsafe(Ptr);
      const uint8_t *SkipTo = Ptr + NumToSkip;
      ScopeStack.push_back(SkipTo);
      LLVM_DEBUG(dbgs() << Loc << ": OPC_Scope(" << SkipTo - DecodeTable
                        << ")\n");
      continue;
    }
    case OPC_SwitchField: {
      // Decode the start value.
      unsigned Start = decodeULEB128AndIncUnsafe(Ptr);
      unsigned Len = *Ptr++;
      uint64_t FieldValue = fieldFromInstruction(insn, Start, Len);
      uint64_t CaseValue;
      unsigned CaseSize;
      while (true) {
        CaseValue = decodeULEB128AndIncUnsafe(Ptr);
        CaseSize = decodeULEB128AndIncUnsafe(Ptr);
        if (FieldValue == CaseValue || !CaseSize)
          break;
        Ptr += CaseSize;
      }
      if (FieldValue == CaseValue) {
        LLVM_DEBUG(dbgs() << Loc << ": OPC_SwitchField(" << Start << ", " << Len
                          << "): " << FieldValue << '\n');
        continue;
      }
      break;
    }
    case OPC_CheckField: {
      // Decode the start value.
      unsigned Start = decodeULEB128AndIncUnsafe(Ptr);
      unsigned Len = *Ptr;
      uint64_t FieldValue = fieldFromInstruction(insn, Start, Len);
      // Decode the field value.
      unsigned PtrLen = 0;
      uint64_t ExpectedValue = decodeULEB128(++Ptr, &PtrLen);
      Ptr += PtrLen;
      bool Failed = ExpectedValue != FieldValue;

      LLVM_DEBUG(dbgs() << Loc << ": OPC_CheckField(" << Start << ", " << Len
                        << ", " << ExpectedValue << "): FieldValue = "
                        << FieldValue << ", ExpectedValue = " << ExpectedValue
                        << ": " << (Failed ? "FAIL, " : "PASS\n"););
      if (!Failed)
        continue;
      break;
    }
    case OPC_CheckPredicate: {
      // Decode the Predicate Index value.
      unsigned PIdx = decodeULEB128AndIncUnsafe(Ptr);
      // Check the predicate.
      bool Failed = !checkDecoderPredicate(PIdx, Bits);

      LLVM_DEBUG(dbgs() << Loc << ": OPC_CheckPredicate(" << PIdx << "): "
                        << (Failed ? "FAIL, " : "PASS\n"););
      if (!Failed)
        continue;
      break;
    }
    case OPC_Decode: {
      // Decode the Opcode value.
      unsigned Opc = decodeULEB128AndIncUnsafe(Ptr);
      unsigned DecodeIdx = decodeULEB128AndIncUnsafe(Ptr);

      MI.clear();
      MI.setOpcode(Opc);
      bool DecodeComplete;
      S = decodeToMCInst(DecodeIdx, S, insn, MI, Address, DisAsm,
                         DecodeComplete);
      LLVM_DEBUG(dbgs() << Loc << ": OPC_Decode: opcode " << Opc
                        << ", using decoder " << DecodeIdx << ": "
                        << (S ? "PASS, " : "FAIL, "));

      if (DecodeComplete) {
        LLVM_DEBUG(dbgs() << "decoding complete\n");
        return S;
      }
      assert(S == MCDisassembler::Fail);
      // Reset decode status. This also drops a SoftFail status that could be
      // set before the decode attempt.
      S = MCDisassembler::Success;
      break;
    }
    case OPC_SoftFail: {
      // Decode the mask values.
      uint64_t PositiveMask = decodeULEB128AndIncUnsafe(Ptr);
      uint64_t NegativeMask = decodeULEB128AndIncUnsafe(Ptr);
      bool Failed = (insn & PositiveMask) != 0 || (~insn & NegativeMask) != 0;
      if (Failed)
        S = MCDisassembler::SoftFail;
      LLVM_DEBUG(dbgs() << Loc << ": OPC_SoftFail: " << (Failed ? "FAIL\n" : "PASS\n"));
      continue;
    }
    }
    if (ScopeStack.empty()) {
      LLVM_DEBUG(dbgs() << "returning Fail\n");
      return MCDisassembler::Fail;
    }
    Ptr = ScopeStack.pop_back_val();
    LLVM_DEBUG(dbgs() << "continuing at " << Ptr - DecodeTable << '\n');
  }
  llvm_unreachable("bogosity detected in disassembler state machine!");
}


} // namespace
